/*!***************************************************************************

  module:		 Container_DoubleNode.hpp

  ------------------------------------------------------------------------------

  responsible:  Till Luhmann

  special area: Standard container in SAPDB

  description:  Doubly linked list node template

  see also:     

  copyright:    (C) 2000 SAP AG

  ------------------------------------------------------------------------------

  ========== licence begin LGPL
  Copyright (C) 2000 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*******************************************************************************/


#ifndef CONTAINER_DOUBLENODE_HPP
#define CONTAINER_DOUBLENODE_HPP


/*!*****************************************************************************

   class: Container_DoubleNode

   description: 
   
   Node of a doubly linked list. 
   
   Contains element of type T and two pointers pointing to successor and 
   predecessor nodes, respectively.

   template arguments:

   <T>            node content type

*******************************************************************************/
template <class T>
class Container_DoubleNode
{
private:
    typedef Container_DoubleNode<T>  NodeType;

public:
    NodeType* m_Prev;
    NodeType* m_Next;
    T         m_Content;

    /*!-------------------------------------------------------------------------
    function:     Container_DoubleNode()
    description:  Constructor

                  - Assigns content

    arguments:    content [in]    content to be assigned
    return value: none
    --------------------------------------------------------------------------*/
    Container_DoubleNode
        (const T&                       content)
        : 
         m_Content(content),
         m_Prev(0),
         m_Next(0) 
    {}

    static NodeType* InvalidNode;

};
/*!*****************************************************************************

   endclass: Container_DoubleNode

*******************************************************************************/

template <class T>
typename Container_DoubleNode<T>::NodeType*
Container_DoubleNode<T>::InvalidNode = (NodeType*)((SAPDB_Long)0xC0DEC0DE);


#endif // CONTAINER_DOUBLENODE_HPP


