/*!***************************************************************************

  module:		 Container_Node.hpp

  ------------------------------------------------------------------------------

  author:       Stefan Baier, Till Luhmann

  responsible:  Till Luhmann

  special area: Standard container in SAPDB

  description:  Singly linked list node template

  see also:     Container_DoubleNode.html

  copyright:    (C) 2000 SAP AG

  ------------------------------------------------------------------------------

  ========== licence begin LGPL
  Copyright (C) 2000 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*******************************************************************************/


#ifndef CONTAINER_NODE_HPP
#define CONTAINER_NODE_HPP


/*!*****************************************************************************

   class: Container_Node

   description: 
   
   Node of a singly linked list. 
   
   Contains element of type T and a pointer pointing to successor node. 

   template arguments:

   <T>            node content type

*******************************************************************************/
template <class T>
class Container_Node
{
private:
    typedef Container_Node<T>  NodeType;

public:
    NodeType* m_Next;
    T         m_Content;

    /*!-------------------------------------------------------------------------
    function:     Container_Node()
    description:  Constructor

                  - Assigns content

    arguments:    content [in]    content to be assigned
    return value: none
    --------------------------------------------------------------------------*/
    Container_Node
        (const T&                       content)
        : 
         m_Content(content),
         m_Next(0) 
    {}

    static NodeType* InvalidNode;

};
/*!*****************************************************************************

   endclass: Container_Node

*******************************************************************************/

template <class T>
typename Container_Node<T>::NodeType*
Container_Node<T>::InvalidNode = (NodeType*)0xC0DEC0DE;


#endif // CONTAINER_NODE_HPP


