/*!******************************************************************************

  module:       Container_SortableVector.hpp

  ------------------------------------------------------------------------------

  author:       Henrik

  responsible:  Henrik

  special area: sortable version of the standard container in SAPDB

  description:  Class declaration and definition for a template vector class

  see also:     

  copyright:    (C) 2002 SAP AG

  ------------------------------------------------------------------------------

  ========== licence begin LGPL
  Copyright (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*******************************************************************************/

#ifndef CONTAINER_SORTABLEVECTOR_HPP
#define CONTAINER_SORTABLEVECTOR_HPP

#include "SAPDBCommon/Algorithms/SAPDBAlgo_QuickSort.hpp"  // QuickSort
#include "Container/Container_Vector.hpp"


/*!****************************************************************************
  class:        Container_SortableVector

  description: This class implements a sortable vector class.

               Simple subclass of Container_Vector

******************************************************************************/


template <class T>
class Container_SortableVector : public Container_Vector<T>
{

public:
    // typedef copied from Container_Vector.hpp to please various C++ compilers
    typedef SAPDBMem_IRawAllocator Allocator; // Allocator type used for vectors

    /*!----------------------------------------------------------------------
    function: Container_SortableVector()
    description: Constructs a vector

                 A vector can only be constructed, if a raw allocator is given.
                 Thus, a default constructor is not offered.
    arguments:   RawAlloc [in]
                 The raw allocator providing the vector with memory.
    -----------------------------------------------------------------------*/
    Container_SortableVector(Allocator& RawAlloc);

    /*!----------------------------------------------------------------------
    function: QuickSort()
    description: sorts the vector
    -----------------------------------------------------------------------*/
    SAPDB_Bool QuickSort();
};

template <class T>
Container_SortableVector<T>::Container_SortableVector(Allocator& RawAlloc)
        : Container_Vector<T>(RawAlloc)
{ }

template <class T>
SAPDB_Bool Container_SortableVector<T>::QuickSort()
{
    return SAPDBAlgo_QuickSort(this->m_ArrayPtr, this->m_Size);
}

// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------

#endif // CONTAINER_SORTABLEVECTOR_HPP
