/*!**************************************************************************

  module      : Converter_IDump.hpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area: converter
  description : abstract dump interface class to the converter 


  last changed: 2001-06-12  17:43
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/


#ifndef CONVERTER_IDUMP_HPP
#define CONVERTER_IDUMP_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gsp00.h"

#include "KernelCommon/Kernel_Common.hpp"
#include "KernelCommon/Kernel_Dump.hpp"


 /*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!*****************************************************************************

   class: Converter_IDump

*******************************************************************************/

class Converter_IDump
{
public:

    /*--------------------------------------------------------------------------
    function:     GetInstance
    description:  This method returns the access to the Converter_IDump interface
                  of component Converter. It is assumed that the Converter singleton 
                  is available! No CreateInstance call will be done.
    arguments:    none
    return value: Access to the Converter_IDump interface
    --------------------------------------------------------------------------*/

    static Converter_IDump& Instance();

    /*!-------------------------------------------------------------------------
    function:     Dump()
    description:  This method is used to dump the main memory structures of the
                  component converter.
    arguments:    TaskId       [in]     identification of the calling task
                  Dump         [in/out] kernel dump file
                  bEnterRegion [in]     use the converter regions
    return value: none
    --------------------------------------------------------------------------*/
    
    virtual void Dump(
        const tsp00_TaskId  TaskId,
        Kernel_Dump         &Dump,
        const SAPDB_Bool    bEnterRegion ) = 0;
};

/*!*****************************************************************************

   endclass: Converter_IDump

*******************************************************************************/


#endif  /* CONVERTER_IDUMP_HPP */
