/*!**************************************************************************

  module      : Converter_IPageIO.hpp

  -------------------------------------------------------------------------

  responsible : Henrik

  special area: converter
  description : abstract restart interface class to the converter 


  last changed: 2002-04-09
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin  GPL
  Copyright (C) 2002 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/


#ifndef CONVERTER_IPAGEIO_HPP
#define CONVERTER_IPAGEIO_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/


/*===========================================================================*
*  DEFINES                                                                  *
*===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!*****************************************************************************

   class: Converter_IPageIO

*******************************************************************************/

class Converter_IPageIO
{
public:

    /*--------------------------------------------------------------------------
    function:     Instance
    description:  This method returns the access to the Converter_IPageIO interface
                  of component Converter. It is assumed that the Converter singleton 
                  is available! No CreateInstance call will be done.
    arguments:    none
    return value: Access to the Converter_IPageIO interface
    --------------------------------------------------------------------------*/

    static Converter_IPageIO& Instance();

    /*!-------------------------------------------------------------------------
    function:     ReadPages()
    description:  This method is used to read a converter leaf page from a device
    arguments:    TaskId     [in] own Task ID
                  pagerIndex [in] logical id of the calling pager task
                  totalPages [out] number of read pages
                  totalIO    [out] number of io's made to read the pages
    return value: none
    --------------------------------------------------------------------------*/

    virtual void ReadPages(
        const tsp00_TaskId  taskId,
        const SAPDB_UInt    pagerIndex,
        SAPDB_UInt          &totalPages,
        SAPDB_UInt          &totalIO ) = 0;

    /*!-------------------------------------------------------------------------
     function:     WritePages
     description:  Flush static and dynamic changed converter leaf pages of one 
                   section to devspace
     arguments:    TaskId     [in]  task id
                   iSection   [in]  converter section to flush
                   totalPages [out] number of changed pages
                   totalIO    [out] number of io's made to flush the pages
     return value: none
     --------------------------------------------------------------------------*/

    virtual void WritePages(
        const tsp00_TaskId  taskId,
        const SAPDB_UInt    iSection,
        SAPDB_UInt          &totalPages,
        SAPDB_UInt          &totalIO ) = 0;
};

/*!*****************************************************************************

   endclass: Converter_IPageIO

*******************************************************************************/


#endif  /* CONVERTER_IPAGEIO */
