/*!**************************************************************************

  module      : Converter_LeafReference.hpp

  -------------------------------------------------------------------------

  responsible : Henrik

  special area:
  description : This paragraph appears in overviews.

    The rest of the description appears only in MyHeader.html.

  last changed: 2002-04-08
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/


#ifndef CONVERTER_LEAFREFERENCE_HPP
#define CONVERTER_LEAFREFERENCE_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "IOManager/IOMan_BlockAddress.hpp"
#include "Converter/Converter_ParentReference.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!*****************************************************************************

   class: Converter_LeafReference

*******************************************************************************/
class Converter_LeafReference
{
public:

    Converter_LeafReference
    (const IOMan_BlockAddress & blockPos,
     const Converter_ParentReference & parentRef)
    :
    m_Block(blockPos),
    m_ParentReference(parentRef)
    {}

    Converter_LeafReference()
    {}

    Converter_LeafReference(const Converter_LeafReference& leaf)
    :
    m_Block(leaf.m_Block),
    m_ParentReference(leaf.m_ParentReference)
    {}

    /*!-------------------------------------------------------------------------
    function:     operator!=()
    description:  Compares with other LeafIndex
    arguments:    index [in]    other
    return value: (SAPDB_Bool) true if unequal
    --------------------------------------------------------------------------*/
    SAPDB_Bool operator!=(const Converter_LeafReference &idx) const
    {
        return (m_Block.GetBlockNo() != idx.m_Block.GetBlockNo());
    }

    /*!-------------------------------------------------------------------------
    function:     operator==()
    description:  Compares with other LeafIndex
    arguments:    index [in]    other
    return value: (SAPDB_Bool) true if equal
    --------------------------------------------------------------------------*/
    SAPDB_Bool operator==(const Converter_LeafReference &idx) const
    {
        return (m_Block.GetBlockNo() == idx.m_Block.GetBlockNo());
    }

    /*!-------------------------------------------------------------------------
    function:     operator<=()
    description:  Compares with other LeafIndex
    arguments:    index [in]    other
    return value: (SAPDB_Bool) true if less or equal
    --------------------------------------------------------------------------*/
    SAPDB_Bool operator<=(const Converter_LeafReference &idx) const
    {
        return (m_Block.GetBlockNo() <= idx.m_Block.GetBlockNo());
    }

    /*!-------------------------------------------------------------------------
    function:     operator<()
    description:  Compares with other LeafIndex
    arguments:    index [in]    other
    return value: (SAPDB_Bool) true if less
    --------------------------------------------------------------------------*/
    SAPDB_Bool operator<(const Converter_LeafReference &idx) const
    {
        return (m_Block.GetBlockNo() < idx.m_Block.GetBlockNo());
    }

    /*!-------------------------------------------------------------------------
    function:     operator>=()
    description:  Compares with other LeafIndex
    arguments:    index [in]    other
    return value: (SAPDB_Bool) true if greater or equal
    --------------------------------------------------------------------------*/
    SAPDB_Bool operator>=(const Converter_LeafReference &idx) const
    {
        return (m_Block.GetBlockNo() >= idx.m_Block.GetBlockNo());
    }

    /*!-------------------------------------------------------------------------
    function:     operator>()
    description:  Compares with other LeafIndex
    arguments:    index [in]    other
    return value: (SAPDB_Bool) true if greater
    --------------------------------------------------------------------------*/
    SAPDB_Bool operator>(const Converter_LeafReference &idx) const
    {
        return (m_Block.GetBlockNo() > idx.m_Block.GetBlockNo());
    }

    IOMan_BlockAddress  GetBlockAddress() const {
        return m_Block;
    }

    Converter_ParentReference GetParentReference() const {
        return m_ParentReference;
    }

private:

    IOMan_BlockAddress	m_Block;
    Converter_ParentReference m_ParentReference;

};

#endif  /* MYHEADER_HPP */
