/*!**************************************************************************

  module      : Converter_MapError.hpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  author:       AlexanderK

  special area: 
  description : 


  last changed: 2001-01-29  13:39
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/

#ifndef CONVERTER_MAPERROR_HPP
#define CONVERTER_MAPERROR_HPP

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "Converter/Converter_Exception.hpp"
#include "Converter/Converter_Messages.hpp"    /* nocheck */ 
#include "SAPDBCommon/SAPDB_ToString.hpp"
#include "Converter/Converter_Types.hpp"
#include "DataAccess/Data_Types.hpp"

/*!*****************************************************************************

   class: Converter_MapError

*******************************************************************************/
class Converter_MapError : public Converter_Exception
{
public:

    /*!-------------------------------------------------------------------------
    function:     Converter_MapError()
    description:  Constructor
    arguments:    filename   [in] filename (to be generated)
                  linenumber [in] linenumber (to be generated) 
                  recMode    [in] recovery mode of given page
                  pageNo     [in] current page number
                  retCode    [in] error state
    return value: none
    --------------------------------------------------------------------------*/
    Converter_MapError
    (const SAPDB_Char*              filename,
     const SAPDB_UInt4              linenumber,
     const Data_PageRecoveryMode    &recMode,
     const Data_PageNo              PageNo,
     const Converter_ReturnCode     retCode)
    :
    Converter_Exception(
        filename,
        linenumber,
        CONVERTER_MAP_ERROR,
    ( recMode.PageIsRecoverable() ? ( recMode.UseDynamicPageAddressing() ? "dynamic " : "static ") : "temp " ),
        SAPDB_ToString( PageNo, _T_d ),
        GetErrorText( retCode ))
    {}

private:

    SAPDB_Char* GetErrorText( Converter_ReturnCode  retCode ) const
    {
        switch( retCode )
        {
        case Converter_Ok:
                return(( SAPDB_Char* )"Okay" );
        case Converter_PageNoAlreadyInUse:
            return(( SAPDB_Char* )"PageNoAlreadyInUse" );
        case Converter_FrameAllocationFailed:
            return(( SAPDB_Char* )"FrameAllocationFailed" );
        case Converter_ConverterPageIOPending:
            return(( SAPDB_Char* )"ConverterPageIOPending" );
        case Converter_NoBlockAddressFound:
            return(( SAPDB_Char* )"NoBlockAddressFound" );
        case Converter_InvalidBlockAddressFound:
            return(( SAPDB_Char* )"InvalidBlockAddressFound" );
        case Converter_PageNoNotReserved:
            return(( SAPDB_Char* )"PageNoNotReserved" );
        case Converter_PageNoAlreadyFree:
            return(( SAPDB_Char* )"PageNoAlreadyFree" );
        case Converter_DuplicatePageNo:
            return(( SAPDB_Char* )"DuplicatePageNo" );
        case Converter_MemoryAllocationFailed:
            return(( SAPDB_Char* )"MemoryAllocationFailed" );
        case Converter_SavePointActive:
            return(( SAPDB_Char* )"SavePointActive" );
        case Converter_NoPageNotAvailable:
            return(( SAPDB_Char* )"NoPageNotAvailable" );
        default:
            return(( SAPDB_Char* )"Unknown" );
        }
    }
};

/*!*****************************************************************************

   endclass: Converter_MapError

*******************************************************************************/


#endif  /* CONVERTER_MAPERROR_HPP */
