  /*!***************************************************************************

  module:       Converter_ParentReference.hpp

  ------------------------------------------------------------------------------

  responsible:  TorstenS

  author:       TillL

  special area: Converter

  description:  Reference to parent page in the converter parent index
                
  see also:     

  ------------------------------------------------------------------------------

  copyright:            Copyright by SAP AG, 2000



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

  *****************************************************************************/


#ifndef CONVERTER_PARENTREFERENCE_HPP
#define CONVERTER_PARENTREFERENCE_HPP


#include "Converter/Converter_Types.hpp"


/*!*****************************************************************************

   class: Converter_ParentReference

   description: Handles a reference to a converter parent page within converter
                parent index

*******************************************************************************/
class Converter_ParentReference
{
public:

    typedef Converter_IndexEntryNo              IndexEntryNo;
    typedef Converter_PageEntryNo               PageEntryNo;

    /*!-------------------------------------------------------------------------
    function:     Converter_ParentReference()
    description:  Default constructor
    arguments:    none
    return value: none
    --------------------------------------------------------------------------*/
    Converter_ParentReference() 
    {}

    /*!-------------------------------------------------------------------------
    function:     Converter_ParentReference()
    description:  Constructor, assigns reference (using standard implementaion)
    arguments:    indexeno [in]  reference to parent page in converter index
                  pageeno  [in]  reference to entry within parent page
    return value: none
    --------------------------------------------------------------------------*/
    Converter_ParentReference
        (const IndexEntryNo&            indexeno, 
         const PageEntryNo&             pageeno)
        : 
         m_IndexEntryNo(indexeno), 
         m_PageEntryNo(pageeno) 
    {}

    /*!-------------------------------------------------------------------------
    function:     SetReference()
    description:  Sets parent reference
    arguments:    indexeno [in]  reference to parent page in converter index
                  pageeno  [in]  reference to entry within parent page
    return value: none
    --------------------------------------------------------------------------*/
    void SetReference
        (const IndexEntryNo&            indexeno, 
         const PageEntryNo&             pageeno)
    {
        m_IndexEntryNo = indexeno;
        m_PageEntryNo  = pageeno;
    }

    /*!-------------------------------------------------------------------------
    function:     SetIndexEntryNo()
    description:  Sets reference to parent page in converter index
    arguments:    indexeno [in]  reference to parent page in parent index
    return value: none
    --------------------------------------------------------------------------*/
    void SetIndexEntryNo
        (const IndexEntryNo&            indexeno)
    {
        m_IndexEntryNo = indexeno;
    }

    /*!-------------------------------------------------------------------------
    function:     SetPageEntryNo()
    description:  Sets reference to entry within parent page
    arguments:    pageeno [in]   reference to entry within parent page
    return value: none
    --------------------------------------------------------------------------*/
    void SetPageEntryNo
        (const PageEntryNo&             pageeno)
    {
        m_PageEntryNo  = pageeno;
    }

    /*!-------------------------------------------------------------------------
    function:     GetIndexEntryNo()
    description:  Returns reference to parent page in parent index
    arguments:    none
    return value: (const IndexEntryNo&)
    --------------------------------------------------------------------------*/
    const IndexEntryNo& GetIndexEntryNo() const
    {
        return m_IndexEntryNo;
    }

    /*!-------------------------------------------------------------------------
    function:     GetPageEntryNo()
    description:  Returns reference to entry within parent page
    arguments:    none
    return value: (const PageEntryNo&)
    --------------------------------------------------------------------------*/
    const PageEntryNo& GetPageEntryNo() const
    {
        return m_PageEntryNo;
    }

private:

    /*--------------------------------------------------------------------------
    declaration: m_IndexEntryNo
    description: reference to parent page in converter index
    --------------------------------------------------------------------------*/
    IndexEntryNo m_IndexEntryNo;

    /*--------------------------------------------------------------------------
    declaration: m_PageEntryNo
    description: reference to entry within parent page
    --------------------------------------------------------------------------*/
    PageEntryNo  m_PageEntryNo;

};
/*!*****************************************************************************

   endclass: Converter_ParentReference

*******************************************************************************/




#endif // CONVERTER_PARENTREFERENCE_HPP



