/****************************************************************************

  module      : Converter_PascalInterface.cpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  author:       AlexanderK

  special area: PascalInterface of the Converter
  description : 


  last changed: 2001-02-14  11:14
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "gbd00.h"

#include "hbd10.h"
#include "hgg01.h"
#include "hgg01_3.h"
#include "hgg08.h"

#include "Converter/Converter_Converter.hpp"

#include "IOManager/IOMan_BlockAddress.hpp"
#include "IOManager/IOMan_IDataInfo.hpp"
#include "DataAccess/Data_Types.hpp"


/*===========================================================================*
 *  LOCAL FUNCTIONS                                                          *
 *===========================================================================*/

/*===========================================================================*
 *  GLOBAL FUNCTIONS                                                         *
 *===========================================================================*/
externC void
bd10CreateConverterSingleton ()
{
    Converter_Converter::CreateInstance();
}



/*---------------------------------------------------------------------------*/

externC void
b10rc_restart_converter (
    tgg00_TransContext        &Trans,
    tsp00_Int4                 ConverterVersion,
    tsp00_Int4                 NumAllPages,
    tsp00_Bool                 bIncConverterVersion,
    tsp00_Bool                 bLastSaveDataSuccessful,
    tgg00_ConfigRestartParam  &RestartParam)
{
    IOMan_PackedBlockAddress PackedConvRootBlockAddress (RestartParam.crConvRootBlockAddr_gg00);

    Converter_IManager::Instance().Restart(Trans.trTaskId_gg00,
                                           ConverterVersion,
                                           Unpack(PackedConvRootBlockAddress, g01DevNoBitCount()),
                                           bIncConverterVersion,
                                           bLastSaveDataSuccessful,
                                           RestartParam.crMaxDynamicPno_gg00,
                                           RestartParam.crMaxStaticPno_gg00,
                                           NumAllPages,
                                           g08convert0,
                                           g01region_cnt (rgnConvert_egg00),
                                           g01page_size,
                                           g01DevNoBitCount ());
}

/*---------------------------------------------------------------------------*/

externC void
b10restart_converter_for_restore (
    tgg00_TransContext        &Trans,
    tsp00_Int4                 ConverterVersion,
    tsp00_Int4                 NumAllPages,
    tsp00_Bool                 bLastSaveDataSuccessful,
    tgg00_ConfigRestartParam  &RestartParam,
    tgg00_ConfigRestartParam  &OldRestartParam)
{
    IOMan_PackedBlockAddress PackedConvRootBlockAddress (OldRestartParam.crConvRootBlockAddr_gg00);

    Converter_IManager::Instance().RestartForRestore(
        Trans.trTaskId_gg00,
        ConverterVersion,
        Unpack(PackedConvRootBlockAddress, g01DevNoBitCount()),
        bLastSaveDataSuccessful,
        SAPDB_MAX (RestartParam.crMaxDynamicPno_gg00, OldRestartParam.crMaxDynamicPno_gg00),
        SAPDB_MAX (RestartParam.crMaxStaticPno_gg00,  OldRestartParam.crMaxStaticPno_gg00),
        NumAllPages,
        g08convert0,
        g01region_cnt (rgnConvert_egg00),
        g01page_size,
        g01DevNoBitCount ());

    Converter_IBackUp::Instance().BeginRestoreUsedPageNos();
}

/*---------------------------------------------------------------------------*/

externC void
b10create_converter (
    tsp00_TaskId   TaskId,
    tsp00_Int4     NumAllPages )
{
    Converter_IManager::Instance().Create (
        TaskId,
        NumAllPages,
        g08convert0,
        g01region_cnt (rgnConvert_egg00),
        g01page_size,
        g01DevNoBitCount ());
}

/*---------------------------------------------------------------------------*/

externC void
b10create_converter_for_restore (
    tsp00_TaskId               TaskId,
    tsp00_Int4                 ConverterVersion,
    tsp00_Int4                 NumAllPages,
    tsp00_Bool                 bLastSaveDataSuccessful,
    tgg00_ConfigRestartParam  &RestartParam)
{
    Converter_IManager::Instance().CreateForRestore (
        TaskId,
        ConverterVersion,
        bLastSaveDataSuccessful,
        RestartParam.crMaxDynamicPno_gg00,
        RestartParam.crMaxStaticPno_gg00,
        NumAllPages,
        g08convert0,
        g01region_cnt (rgnConvert_egg00),
        g01page_size,
        g01DevNoBitCount());
}

/*---------------------------------------------------------------------------*/

externC tsp00_PageNo
bd10MaxTempPno()
{
    return Converter_ICommon::Instance().MaxTempPageNo();
}

/*---------------------------------------------------------------------------*/

externC tsp00_PageNo
bd10MaxPermDynamicPno()
{
    return Converter_ICommon::Instance().MaxPermDynamicPageNo();
}

/*---------------------------------------------------------------------------*/

externC tsp00_PageNo
bd10MaxStaticStaticPno()
{
    return Converter_ICommon::Instance().MaxPermStaticPageNo();
}

/*---------------------------------------------------------------------------*/

externC tsp00_Bool
b10backup_active ()
{
    return Converter_ICommon::Instance().BackUpIsActive();
}

/*---------------------------------------------------------------------------*/

externC void
bd999CheckSpace (
    tgg00_TransContext  &trans,
    tsp00_Int4          numDataPagesRequested)
{
    Converter_IPageNoManager::Instance().HandleDBFull (trans,
            numDataPagesRequested, IOMan_IDataInfo::GetInstance().AddDataDevSpacePossible());
}

/*---------------------------------------------------------------------------*/

externC void
bd10EndColdVerify (tsp00_TaskId taskId,
                   tsp00_Int4   &NumPermPages,
                   tsp00_Int4   &NumReleasesBlocks)
{
    Data_PageCount       NumStaticPermDataPages  = 0;
    Data_PageCount       NumDynamicPermDataPages = 0;
    Data_PageCount       NumPermConvPages        = 0;
    Data_PageCount       ReleasedBlocks          = 0;

    Converter_IVerify::Instance().EndColdVerify (
        taskId,
        NumStaticPermDataPages,
        NumDynamicPermDataPages,
        NumPermConvPages,
        ReleasedBlocks);

    NumPermPages = NumStaticPermDataPages + NumDynamicPermDataPages + NumPermConvPages;
    NumReleasesBlocks = ReleasedBlocks;
}

/*---------------------------------------------------------------------------*/

externC void
bd10EndFailedColdVerify (tsp00_TaskId  TaskId)
{
    Converter_IVerify::Instance().EndFailedColdVerify (TaskId);
}

/*---------------------------------------------------------------------------*/

externC void
bd10EndSave (tsp00_TaskId    TaskId,
             tsp00_Bool      bBackUpSuccessfullyFinished)
{
    Converter_IBackUp::Instance().EndSave (TaskId, bBackUpSuccessfullyFinished);
}

/*---------------------------------------------------------------------------*/

externC tsp00_Int4
b10bup_data_page_cnt (tsp00_TaskId TaskId)
{
    return Converter_IBackUp::Instance().GetNumPagesForBackUp();
}


/*---------------------------------------------------------------------------*/

externC tsp00_Int4
bd10conv_page_cnt ()
{
    return Converter_IBackUp::Instance().GetNumConvPagesForBackUp();
}

/*---------------------------------------------------------------------------*/

externPascal tsp00_Int4
bd10GetConverterVersion ()
{
    return Converter_ICommon::Instance().Version ();
}

/*---------------------------------------------------------------------------*/

externC void
b10use_pno (tgg00_TransContext    &Trans,
            tsp00_PageNo           PageNo)
{
    Converter_IVerify::Instance().MarkPageNoAsUsed (Trans.trTaskId_gg00, PageNo, Data_Dynamic);
}


/*---------------------------------------------------------------------------*/

externC void
bd10conv_restore_entry (tsp00_TaskId TaskId,
                        tsp00_PageNo PageNo,
                        tsp00_Int2 dev_no,
                        tsp00_Int4 dev_offs,
                        tsp00_Bool bAddressingModeIsStatic)
{
    IOMan_BlockAddress BlockAddress (dev_no, dev_offs);
    const Data_AddressingMode  AddressingMode = (bAddressingModeIsStatic? Data_Static:Data_Dynamic);

    Converter_IBackUp::Instance().SetBlockAddressOfRestoredPage (TaskId, BlockAddress, PageNo, AddressingMode);
}


/*---------------------------------------------------------------------------*/

externC void
bd10bup_pno_ready (tgg00_TransContext &Trans,
                   tsp00_PageNo PageNo ,
                   tsp00_Int2 dev_no ,
                   tsp00_Int4 dev_offs,
                   tsp00_Bool bAddressingModeIsStatic)
{
    IOMan_BlockAddress BlockAddress (dev_no, dev_offs);
    const Data_AddressingMode  AddressingMode = (bAddressingModeIsStatic? Data_Static:Data_Dynamic);

    if (! Converter_IBackUp::Instance().ResetDataPageAfterBackUp (Trans.trTaskId_gg00,
            BlockAddress, PageNo, AddressingMode))
    {
        Trans.trError_gg00 = e_invalid_fbm_mark;
    }
}


/*---------------------------------------------------------------------------*/

externC void
bd10BeginSavePages (tsp00_TaskId TaskId)
{
    Converter_IBackUp::Instance().BeginSavePages (TaskId);
}

/*---------------------------------------------------------------------------*/

externC void
bd10BeginSaveData(tsp00_TaskId TaskId)
{
    Converter_IBackUp::Instance().BeginSaveData (TaskId);
}

/*---------------------------------------------------------------------------*/

externC void
bd10FlushConverterForColdSave (tgg00_TransContext  &Trans,
                               tsp00_Int4          &PackedConvRootBlockAddress)
{
    Converter_Version   DummyConverterVersion;
    Data_PageNo         DummyMaxStaticPageNo;
    Data_PageNo         DummyMaxDynamicPageNo;
    IOMan_BlockAddress  ConvRootBlockAddress;

    if( ! Converter_IManager::Instance().FlushSerial (Trans.trTaskId_gg00,
            DummyConverterVersion,
            DummyMaxStaticPageNo,
            DummyMaxDynamicPageNo,
            ConvRootBlockAddress))
    {
        Trans.trError_gg00 = e_shutdown;
    }

    PackedConvRootBlockAddress = Pack(ConvRootBlockAddress, g01DevNoBitCount ());
}

/*---------------------------------------------------------------------------*/

externC void
bd10FlushConverterSerial (tgg00_TransContext  &Trans,
                          tsp00_Int4          &ConverterVersion,
                          tsp00_PageNo        &MaxStaticPageNo,
                          tsp00_PageNo        &MaxDynamicPageNo,
                          tsp00_Int4          &PackedConvRootBlockAddress)
{
    Converter_Version _ConverterVersion;
    Data_PageNo       _MaxStaticPageNo;
    Data_PageNo       _MaxDynamicPageNo;
    IOMan_BlockAddress _ConvRootBlockAddress;

    if( ! Converter_IManager::Instance().FlushSerial (Trans.trTaskId_gg00,
            _ConverterVersion,
            _MaxStaticPageNo,
            _MaxDynamicPageNo,
            _ConvRootBlockAddress))
    {
        Trans.trError_gg00 = e_shutdown;
    }

    ConverterVersion = _ConverterVersion;
    MaxStaticPageNo  = _MaxStaticPageNo;
    MaxDynamicPageNo = _MaxDynamicPageNo;

    PackedConvRootBlockAddress = Pack(_ConvRootBlockAddress, g01DevNoBitCount ());
}

/*---------------------------------------------------------------------------*/

externC void
bd10BeginColdVerify( tsp00_TaskId taskId )
{
    Converter_IVerify::Instance().BeginColdVerify( taskId );
}


/*---------------------------------------------------------------------------*/

externC void
bd10BeginSavepoint (tsp00_TaskId     TaskId)
{
    Converter_ISavePoint::Instance().BeginSavepoint (TaskId);
}

/*---------------------------------------------------------------------------*/

externC void
bd10EndSavepoint (tsp00_TaskId     TaskId)
{
    Converter_ISavePoint::Instance().EndSavepoint(TaskId );
}

/*---------------------------------------------------------------------------*/

externC void
bd10ShutdownConverter (tsp00_TaskId  TaskId)
{
    Converter_IManager::Instance().Shutdown(TaskId);
}

/*---------------------------------------------------------------------------*/

externC void
bd10dbspace_statistic (tgg00_TransContext &Trans,
                       tgg00_DbSpaceInfo  &DbSpaceInfo)
{
    Converter_ICommon::Instance().GetDbSpaceStatistics (Trans.trTaskId_gg00, DbSpaceInfo);
}

/*---------------------------------------------------------------------------*/

externC void
bd10AddDBFillingEvent (tsp31_event_ident_Param   EventId,
                       tsp00_Int4                EventThreshold,
                       tsp31_event_prio_Param    EventPrio)
{
    SAPDBERR_ASSERT_STATE ((sp31ei_db_filling_above_limit==EventId) || (sp31ei_db_filling_below_limit==EventId));
    SAPDB_Bool bOverflow = (sp31ei_db_filling_above_limit==EventId);
    Converter_IEvent::Instance().AddEvent(EventThreshold, bOverflow, EventPrio);
}

/*---------------------------------------------------------------------------*/

externC void
bd10DelDBFillingEvent (tsp31_event_ident_Param   EventId,
                       tsp00_Int4                EventThreshold)
{
    SAPDBERR_ASSERT_STATE ((sp31ei_db_filling_above_limit==EventId) || (sp31ei_db_filling_below_limit==EventId));
    SAPDB_Bool bOverflow = (sp31ei_db_filling_above_limit==EventId);
    Converter_IEvent::Instance().DelEvent(EventThreshold, bOverflow);
}

/*---------------------------------------------------------------------------*/

externC void
bd10GetDBFillingEventSet (tsp31_short_event_desc  &ShortEventDesc,
                          tsp00_Bool               bGetOverflowEvents)
{
    Converter_IEvent::Instance().GetEventSet(ShortEventDesc, bGetOverflowEvents);
}

/*---------------------------------------------------------------------------*/

externC void
bd10GetConverterBitMapPage (tsp00_TaskId    TaskId,
                            tsp00_Page     &CopyPage,
                            tsp00_Bool     &bNoMorePages)
{
    bNoMorePages = false;

    Converter_BitMapPage  BitMapPage;
    Converter_UsedPageNoContainer::Iterator iter;
    Converter_IBackUp & c = Converter_IBackUp::Instance();

    c.LockUsedPageNoContainers();

    if (!c.GetUsedStaticPageNoContainer().IsEmpty())
    {
        SAPDB_UInt Size = c.GetUsedStaticPageNoContainer().GetSize();
        iter = c.GetUsedStaticPageNoContainer().Position(Size-1);
        BitMapPage = (*iter);
        c.GetUsedStaticPageNoContainer().Resize(Size-1);
    }
    else
    {
        if (!c.GetUsedDynamicPageNoContainer().IsEmpty())
        {
            SAPDB_UInt Size = c.GetUsedDynamicPageNoContainer().GetSize();
            iter = c.GetUsedDynamicPageNoContainer().Position(Size - 1);
            BitMapPage = (*iter);
            c.GetUsedDynamicPageNoContainer().Resize(Size-1);
        }
        else
        {
            c.UnlockLockUsedPageNoContainers();
            bNoMorePages = true;
            return;
        }
    }

    c.UnlockLockUsedPageNoContainers();

    SAPDBERR_ASSERT_STATE (sizeof(CopyPage) == BitMapPage.GetFrame().Capacity());

    BitMapPage.PrepareForWrite();
    SAPDBERR_ASSERT_STATE (BitMapPage.CheckAfterRead());

    memcpy (&CopyPage, BitMapPage.GetFramePtr(), sizeof(CopyPage));

    Converter_PageFrameAllocator PageFrameAllocator (TaskId);
    PageFrameAllocator.Free(BitMapPage.GetFrame(), false );

}

/*---------------------------------------------------------------------------*/

externC void
bd10RemoveConverterBitMapPages (tsp00_TaskId TaskId)
{
    Converter_IBackUp::Instance().ClearUsedPageNoContainers(TaskId);
}

/*---------------------------------------------------------------------------*/

externC void
bd10RestoreUsedPageNos (tsp00_TaskId   TaskId,
                        tsp00_PageAddr pPage)
{
    Kernel_Page::PageFrame frame (pPage, sizeof(*pPage));
    Converter_BitMapPage BitMapPage (frame);
    Converter_IBackUp::Instance().RestoreUsedPageNos (TaskId, BitMapPage);
}

/*---------------------------------------------------------------------------*/

externC void
bd10FinishRestoreUsedPageNos (tsp00_TaskId TaskId)
{
    Converter_IBackUp::Instance().FinishRestoreUsedPageNos (TaskId);
}

/*---------------------------------------------------------------------------*/

externC void
bd10CkeckConverterBitMapPage(tsp00_Page     &Page,
                             tsp00_Bool     &bPageIsDamaged)
{
    Kernel_Page::PageFrame frame (&Page, sizeof(Page));
    Converter_BitMapPage BitMapPage (frame);
    bPageIsDamaged = (! BitMapPage.CheckAfterRead());
}

/*---------------------------------------------------------------------------*/

externC tsp00_Int4
bd10ConverterSize()
{
    return( Converter_ICommon::Instance().UsedConverterPages());
}


/*---------------------------------------------------------------------------*/

externC tsp00_Bool
bd10IsDbFull( tsp00_TaskId taskId )
{
    return( Converter_ICommon::Instance().IsDBFull( taskId ));
}

/*---------------------------------------------------------------------------*/

externC tsp00_Bool
bd10IsConnectAllowed()
{
    return( ! Converter_ICommon::Instance().SecuritySpaceReached() );
}

/*---------------------------------------------------------------------------*/

externC void
bd10CheckAndStartSavepoint(tgg00_TransContext & Trans )
{
    Converter_ICommon::Instance().StartSavepoint( Trans );
}

/*---------------------------------------------------------------------------*/

externC void
bd10GetBlockAddress (
    tgg00_TransContext & Trans,
    tsp00_PageNo         PageNo,
    pasbool              bIsDynamic,
    tsp00_Int2         &  DevNo,
    tsp00_Int4         &  BlockNo)
{
    const Data_AddressingMode  AddressingMode = ( bIsDynamic ? Data_Dynamic : Data_Static );

    IOMan_BlockAddress  Block =
        Converter_IPageNoMapping::Instance().GetDevicePositionForReading(
            Trans.trTaskId_gg00, PageNo, AddressingMode);

    if( Block.IsValid() )
    {
        DevNo   = Block.GetDeviceNo();
        BlockNo = Block.GetBlockNo();
    }
    else
        Trans.trError_gg00 = e_no_converter_entry;
}

/*---------------------------------------------------------------------------*/

externC tsp00_Bool
bd10CheckBackup( tsp00_TaskId taskId )
{
    return Converter_IBackUp::Instance().CheckBackUp( taskId );
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
