/*!***************************************************************************

module:       Converter_Types.hpp

------------------------------------------------------------------------------

responsible:  TorstenS

author:       TillL

special area: Converter

description:  Common definitions of this component
              
see also:     

------------------------------------------------------------------------------

copyright:            Copyright by SAP AG, 2000



  ========== licence begin  GPL
  Copyright (C) 2000 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/


#ifndef CONVERTER_TYPES_HPP
#define CONVERTER_TYPES_HPP


#include "KernelCommon/Kernel_Common.hpp"
#include "SAPDBCommon/SAPDB_EnhancedTypes.hpp"

#include "SAPDBCommon/Diagnostics/SAPDBDiag_CheckTopic.hpp"
#include "SAPDBCommon/Tracing/SAPDBTrace_Topic.hpp"
#include "SAPDBCommon/Tracing/SAPDBTrace_Stream.hpp"



/*===========================================================================*
*  EXTERNAL VARIABLES                                                       *
*===========================================================================*/

extern SAPDBDiag_CheckTopic Converter_Check;
extern SAPDBTrace_Topic     Converter_Trace;

/*!-----------------------------------------------------------------------------
declaration: Converter_Version
description: Converter version (also: savepoint version)
------------------------------------------------------------------------------*/
class  Converter_Version : public SAPDB_Invalidatable<SAPDB_UInt4, SAPDB_MAX_UINT4>
{
public:
    static Converter_Version GetNilVersion(){ return SAPDB_MAX_UINT4;   }

    void SetToInitialVersion(){m_Value=1;};

    Converter_Version (){};

    Converter_Version (const SAPDB_UInt4& Value)
    : SAPDB_Invalidatable<SAPDB_UInt4, SAPDB_MAX_UINT4>(Value){};
};

/*!-----------------------------------------------------------------------------
declaration : Converter_IndexEntryCount
description:  number of entries of converter parent index
------------------------------------------------------------------------------*/
typedef SAPDB_UInt2
Converter_IndexEntryCount;
/*!-----------------------------------------------------------------------------
declaration : Converter_IndexEntryNo
description:  number which identifies a specific converter parent index entry
------------------------------------------------------------------------------*/
typedef SAPDB_Invalidatable<SAPDB_UInt2, SAPDB_MAX_UINT2>
Converter_IndexEntryNo;

/*!-----------------------------------------------------------------------------
declaration : Converter_PageEntryCount
description:  number of entries on a converter page
------------------------------------------------------------------------------*/
typedef SAPDB_UInt2
Converter_PageEntryCount;
/*!-----------------------------------------------------------------------------
declaration : Converter_PageEntryCount
description:  number which identifies a specific converter page entry
------------------------------------------------------------------------------*/
typedef SAPDB_Invalidatable<SAPDB_UInt2, SAPDB_MAX_UINT2>
Converter_PageEntryNo;

/*!-----------------------------------------------------------------------------
declaration : Converter_MapEntryCount
description:  number of entries of converter map
------------------------------------------------------------------------------*/
typedef SAPDB_UInt4
Converter_MapEntryCount;
/*!-----------------------------------------------------------------------------
declaration : Converter_MapEntryNo
description:  number which identifies a specific converter map entry
------------------------------------------------------------------------------*/
typedef SAPDB_Invalidatable<SAPDB_UInt4, SAPDB_MAX_UINT4>
Converter_MapEntryNo;

/*!-----------------------------------------------------------------------------
declaration : Converter_PageType
description:  constants for different converter page types
------------------------------------------------------------------------------*/
typedef enum {
    Converter_IndexPageType,
    Converter_LeafPageType
}
Converter_PageType;

/*!-----------------------------------------------------------------------------
declaration : Converter_ReturnCode
description:  constants for different converter error reasons
------------------------------------------------------------------------------*/
typedef enum {
    Converter_Ok = 0,
    Converter_PageNoAlreadyInUse,
    Converter_FrameAllocationFailed,
    Converter_ConverterPageIOPending,
    Converter_NoBlockAddressFound,
    Converter_InvalidBlockAddressFound,
    Converter_PageNoNotReserved,
    Converter_PageNoAlreadyFree,
    Converter_DuplicatePageNo,
    Converter_MemoryAllocationFailed,
    Converter_SavePointActive,
    Converter_NoPageNotAvailable
}
Converter_ReturnCode;

/*!-----------------------------------------------------------------------------
declaration : Converter_GarbageCollectionReturnCode
description:  constants for garbage collection needed requests
------------------------------------------------------------------------------*/
typedef enum {
    Converter_GCNotWanted = 0,
    Converter_GCNotNeeded,
    Converter_GCNeeded
}
Converter_GarbageCollectionReturnCode;


#endif // CONVERTER_TYPES_HPP



