/*!
  @file           DBMCli_EventListener.hpp
  @author         Bernd Vorsprach - bernd.vorsprach@sap.com
  @brief          Listener for database kernel events

\if EMIT_LICENCE
    ========== licence begin LGPL
    Copyright (C) 2003 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end
\endif
*/

#ifndef _DBMCli_EventListener_HPP_
#define _DBMCli_EventListener_HPP_

/*
  -----------------------------------------------------------------------------
  includes
  -----------------------------------------------------------------------------
 */
#include "SAPDB/DBM/Cli/DBMCli_Database.hpp"
#include "SAPDB/DBM/Cli/DBMCli_DateTime.hpp"

/*!
  @brief     Event
 */
class DBMCli_Event
{
public:  
  /*! @brief constructor */
  DBMCli_Event (DBMCli_Result & oResult);

  /*! @brief priority type */
  typedef enum {
    prio_unknown,
    prio_low,
    prio_medium,
    prio_high } Priority;

  /*! @brief get a event property */
  const DBMCli_String & GetName        () const { return m_sName;       }
  /*! @brief get a event property */
        Priority        GetPriority    () const  { return m_oPriority;   }
  /*! @brief get a event property */
  const DBMCli_Date   & GetDate        () const  { return m_oDate;       }
  /*! @brief get a event property */
  const DBMCli_Time   & GetTime        () const  { return m_oTime;       }
  /*! @brief get a event property */
        SAPDB_Int4      GetValue1      () const  { return m_nValue1;     }
  /*! @brief get a event property */
        SAPDB_Int4      GetValue2      () const  { return m_nValue2;     }
  /*! @brief get a event property */
        SAPDB_Int4      GetCount       () const  { return m_nCount;      }
  /*! @brief get a event property */
  const DBMCli_String & GetText        () const  { return m_sText;       }
  /*! @brief get a event property */
  const DBMCli_String & GetDescription () const  { return m_sDescription;}

private:
  /*! @breif set a property */
  void SetProperty (const DBMCli_String & sProperty);
  /*! @brief property member */
  DBMCli_String m_sName;
  /*! @brief property member */
  Priority      m_oPriority;
  /*! @brief property member */
  DBMCli_Date   m_oDate;
  /*! @brief property member */
  DBMCli_Time   m_oTime;
  /*! @brief property member */
  SAPDB_Int4    m_nValue1;
  /*! @brief property member */
  SAPDB_Int4    m_nValue2;
  /*! @brief property member */
  SAPDB_Int4    m_nCount;
  /*! @brief property member */
  DBMCli_String m_sText;
  /*! @brief property member */
  DBMCli_String m_sDescription;

}; // end class DBMCli_Event

/*!
  @brief     Event Listener
 */
class DBMCli_EventListener : public DBMCli_Database
{
public:

  /*!
    @brief   Constructor 
    Constructor to create a the object
    @param sServer   [IN] name of server node
    @param sDatabase [IN] name of database instance
    @param sUserPwd  [IN] user,pwd
  */
  DBMCli_EventListener
    ( const DBMCli_String  & sServer,
      const DBMCli_String  & sDatabase,
      const DBMCli_String  & sUserPwd,
      SAPDBErr_MessageList & oMessageList);

  /*! @brief Returns the last message */
  const SAPDBErr_MessageList & LastMessage () {return m_oMessageList;}

  /*! @brief Runs the event listen loop */
  bool Run ();
 
  /*! @brief Handles one event */
  virtual bool EventProc (const DBMCli_Event & oEvent);

protected:
  /*! @brief Returns the message list object */
  SAPDBErr_MessageList & GetMsgObj () {return m_oMessageList;}

private:
  /*! @brief Messagelist object */
  SAPDBErr_MessageList   m_oMessageList;

}; // end DBMCli_EventListener

#endif /* _DBMCli_EventListener_HPP_ */
