/*!***************************************************************************

  module      : Data_ChainPage.hpp
  responsible : UweH
  special area: DataAccess
  created     : 2000-11-22
  last changed: 2000-11-22 12:00
  copyright   : Copyright by SAP AG, 2000
  description : defines class Data_ChainPage



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/

#ifndef Data_ChainPage_HPP
#define Data_ChainPage_HPP

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SAPDBCommon/ErrorsAndMessages/SAPDBErr_Assertions.hpp"
#include "DataAccess/Data_Types.hpp"
#include "DataAccess/Data_Page.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/
/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/
/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/
/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*---------------------------------------------------------------------------*/
/*! class:       Data_ChainPage
    description: The page which is used by Data_Chain.
 */
class Data_ChainPage : public Data_Page
{
public:

    /*! ---------------------------------------------------------------------------
    function:    InitializeFrame
    */
    virtual void InitializeFrame ()
    {
        Data_Page::InitializeFrame();
        SetNextPageNo (SAPDB_MAX_INT4);
        SetLastPageNo (SAPDB_MAX_INT4);
    }

    /*! -----------------------------------------------------------------------
    function:    NextPageNo
    returns:     PageNo
    description: The page id of the next page is returned.
    
                 If no next page is available, PageId.IsValid() returns false.
    */
    Data_PageNo NextPageNo ()
    {
        SAPDBERR_ASSERT_STATE(IsAssigned());
        return STATIC_CAST(Data_PageNo,NodePtr()->nd_right());
    }

    /*! -----------------------------------------------------------------------
    function:    SetNextPageNo
    arguments:   PageNo [in]
    */
    void SetNextPageNo (const Data_PageNo& PageNo)
    {
        SAPDBERR_ASSERT_STATE(IsAssigned());
        NodePtr()->nd_right() = (tsp00_PageNo)PageNo;
    }

    /*! -----------------------------------------------------------------------
    function:    LastPageNo
    returns:     PageNo
    description: The page id of the last page is returned.
    */
    Data_PageNo LastPageNo ()
    {
        SAPDBERR_ASSERT_STATE(IsAssigned());
        return STATIC_CAST(Data_PageNo,NodePtr()->nd_last());
    }

    /*! -----------------------------------------------------------------------
    function:    SetLastPageNo
    arguments:   PageNo [in]
    */
    void SetLastPageNo (const Data_PageNo& PageNo)
    {
        SAPDBERR_ASSERT_STATE(IsAssigned());
        NodePtr()->nd_last() = (tsp00_PageNo)PageNo;
    }

    /*! -----------------------------------------------------------------------
    function:    Append
    description: NewPage [out]
    */
    void Append (Data_ChainPage& NewPage,
                 Data_PageNo&    LastPageNo)
    {
        SAPDBERR_ASSERT_STATE( IsAssigned()           );
        SAPDBERR_ASSERT_STATE( ! NewPage.IsAssigned() );

        (void) m_pAccessManager->NewPage (NewPage);

        NewPage.SetNextPageNo ( NextPageNo()     );
        NewPage.SetRootPageNo ( RootPageNo()     );
        SetNextPageNo         ( NewPage.PageNo() );

        if ( ! NewPage.NextPageNo().IsValid() )
            LastPageNo = NewPage.PageNo();
    }

};
/*! endclass: Data_ChainPage */

#endif // Data_ChainPage_HPP
