/*!***************************************************************************

  module      : File_IFile.hpp
  responsible : UweH
  special area: File
  created     : 2000-11-20
  last changed: 2000-11-20 12:00
  copyright   : Copyright by SAP AG, 2000
  description : Interface to general file methods



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/

#ifndef File_IFile_HPP
#define File_IFile_HPP

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"

#include "DataAccess/Data_Types.hpp"
#include "DataAccess/Data_PageAccessManager.hpp"
#include "File/File_Types.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/
/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/
/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/
/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*! ---------------------------------------------------------------------------
class:       File_IFile
description: Interface for operations on all files.
*/
class File_IFile
{
public:

    /*! ---------------------------------------------------------------------------
    function:    File_IFile
    description: default constructor
    */
    File_IFile (File_Type           Type,
                tgg00_TransContext &Trans,
                const Data_PageId  &RootId)
        : m_Type              (Type),
          m_PageAccessManager (Trans, Type, RootId.PageNo()),
          m_RootId            (RootId)
    {}

    /*! ---------------------------------------------------------------------------
    function:    Drop
    description: All pages referring to the file are deleted.
    */
    virtual void Drop() = 0;

    /*! ---------------------------------------------------------------------------
    function:    Verify
    arguments:   trans  [in]
                 isCold [in]
    returns:     true, if the file is consistent
    */
    virtual bool Verify(tgg00_TransContext &trans,
                        bool                isCold) = 0;

    /*! ---------------------------------------------------------------------------
    function:    WriteToTrace
    description: This traces some basic information about the file to the trace.
    */
    virtual void WriteToTrace () const = 0;

    /*! ---------------------------------------------------------------------------
    function:    GetRootId
    returns:     Data_PageId - of the root
    description: This returns the Id of the root page of this file.
                 More over this specifies what kind of recovery mode pages of the file are.
    */
    const Data_PageId& GetRootId() const
    {
        return m_RootId;
    }

    /*! ---------------------------------------------------------------------------
    function:    GetType
    returns:     File_Type
    description: This returns the Id of the root page of this file.
                 More over this specifies what kind of recovery mode pages of the file are.
    */
    File_Type GetType() const
    {
        return m_Type;
    }

    /*! ---------------------------------------------------------------------------
    function:    GetPageAccessManager
    returns:     Data_PageId - of the root
    description: This returns the Id of the root page of this file.
                 More over this specifies what kind of recovery mode pages of the file are.
    */
    Data_IPageAccessManager& GetPageAccessManager()
    {
        return m_PageAccessManager;
    }

protected:

    /*! ---------------------------------------------------------------------------
    variable:    m_Type
    description: Containes the type of the file.
    */
    File_Type m_Type;

    /*! ---------------------------------------------------------------------------
    variable:    m_PAM
    description: This is the handle to access the pages of the file.
    */
    Data_PageAccessManager m_PageAccessManager;

    /*! ---------------------------------------------------------------------------
    variable:    m_RootId
    description: This identifes the first page in the container of this file.
    */
    Data_PageId  m_RootId;
};
/*! endclass: File_IFile */

#endif // File_IFile_HPP
