/*!**************************************************************************

  module      : FrameCtrl_ICommon.hpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area: FrameCtrl_ICommon.hpp
  description : Interface to get common information about the component
                FrameControl.

  last changed: 2001-03-15  19:43
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/


#ifndef FRAMECTRL_ICOMMON_HPP
#define FRAMECTRL_ICOMMON_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "KernelCommon/Kernel_Common.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!*****************************************************************************

   class: FrameCtrl_ICommon

   description: This interface is used to get common information about
                the component FrameControl.
              
*******************************************************************************/

class FrameCtrl_ICommon
{
public:

    /*--------------------------------------------------------------------------
    function:     GetInstance
    description:  This method returns the access to the FrameCtrl_ICommon 
                  interface of component FrameControl. It is assumed that the 
                  FrameControl singleton is available! No CreateInstance call 
                  will be done.
    arguments:    none
    return value: Access to the FrameCtrl_ICommon interface
    --------------------------------------------------------------------------*/

    static FrameCtrl_ICommon& GetInstance();

    /*!-------------------------------------------------------------------------
    function:     GetFreeFrameCount()
    description:  This method is used to get the number of frames currently 
                  available and therefore requestable by the new methods.
    arguments:    none       
    return value: SAPDB_UInt Number of unused frames
    --------------------------------------------------------------------------*/

    virtual SAPDB_UInt GetFreeFrameCount() const = 0;

};

/*!*****************************************************************************

   endclass: FrameCtrl_ICommon

*******************************************************************************/



#endif  /* FRAMECTRL_ICOMMON_HPP */
