/*!***************************************************************************

  module      : GC_IGarbageCollector.hpp
  special area: GarbageCollector
  last changed: 2002-02-11
  last changed: 2002-02-11  12:00
  copyright   : Copyright by SAP AG, 2000
  responsible : FerdiF
  description : defines an interface to the GarbageCollector



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#ifndef GC_IGARBAGECOLLECTOR_HPP
#define GC_IGARBAGECOLLECTOR_HPP

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/
/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/
/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/
/*===========================================================================*
 *  FORWARD DECLARATION *
 *===========================================================================*/

/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*! ---------------------------------------------------------------------------
class:       GC_IGarbageCollector
description: this is an Interface to garbage collector 
*/
class GC_IGarbageCollector
{
public:
  
  /*! -----------------------------------------------------------------------
  function:    GetHistOption
  description: This method returns the active history remove option  
               ( see: Log_History::RemoveOptions )
  arguments:   -none-
  returns:     active history remove option ( 0=normal, 1=yellow, 2=red )
  */
  virtual SAPDB_UInt1 GetHistOption() const = 0;

  /*! -----------------------------------------------------------------------
  function:    IncHistCreateObjFileCount
  description: This method increments the statistic counter for create object
               entries handled by garbage collector
  arguments:   -none-
  */
  virtual void IncHistCreateObjFileCount() = 0;

  /*! -----------------------------------------------------------------------
  function:    IncHistDeleteObjCount
  description: This method increments the statistic counter for delete object
               entries handled by garbage collector
  arguments:   -none-
  */
  virtual void IncHistDeleteObjCount() = 0;

  /*! -----------------------------------------------------------------------
  function:    IncHistDropObjFileCount
  description: This method increments the statistic counter for drop object
               entries handled by garbage collector
  arguments:   -none-
  */
  virtual void IncHistDropObjFileCount() = 0;

  /*! -----------------------------------------------------------------------
  function:    IncHistEntryReleaseCount
  description: This method increments the statistic counter for history file
               entries released by the garbage collector
  arguments:   -none-
  returns:     
  */
  virtual void IncHistEntryReleaseCount() = 0;

  /*! -----------------------------------------------------------------------
  function:    IncHistLockObjCount
  description: This method increments the statistic counter for lock object
               entries handled by garbage collector
  arguments:   -none-
  */
  virtual void IncHistLockObjCount() = 0;

  /*! -----------------------------------------------------------------------
  function:    IncHistLockObjCount
  description: This method increments the statistic counter for insert object
               entries handled by garbage collector
  arguments:   -none-
  */
  virtual void IncHistInsertObjCount() = 0;

  /*! -----------------------------------------------------------------------
  function:    IncHistNewObjCount
  description: This method increments the statistic counter for new object
               entries handled by garbage collector
  arguments:   -none-
  */
  virtual void IncHistNewObjCount() = 0;

  /*! -----------------------------------------------------------------------
  function:    IncHistUpdateObjCount
  description: This method increments the statistic counter for update object
               entries handled by garbage collector
  arguments:   -none-
  */
  virtual void IncHistUpdateObjCount() = 0;

  /*! -----------------------------------------------------------------------
  function:    IncObjReleaseCount
  description: This method increments the statistic counter for objects 
               released ( deleted ) by the garbage collector
  arguments:   -none-
  returns:     
  */
  virtual void IncObjReleaseCount() = 0;

  /*! -----------------------------------------------------------------------
  function:    IncReleaseEmptyPageCount
  description: This method increments the statistic counter for pages  
               released ( deleted ) by the garbage collector, because they 
               became empty
  arguments:   -none-
  returns:     
  */
  virtual void IncReleaseEmptyPageCount() = 0;

  /*! -------------------------------------------------------------------------
  function:    IsInterrrupted
  description: This method returns if the garbage collector is to be 
               interrupted because of a savepoint request.
  arguments:   -none-
  returns:     true, if GarbagaCollection is to be interrupted, else false
  */
  virtual bool IsInterrrupted () const = 0;

  /*! -----------------------------------------------------------------------
  function:    SetHistOption
  description: This method sets the history remove option used  
               ( see: Log_History::RemoveOptions )
  arguments:   -none-
  returns:     
  */
  virtual void SetHistOption(SAPDB_UInt1 option) = 0;

};
/*! endclass: GC_IGarbageCollector */


#endif // GC_IGARBAGECOLLECTOR_HPP
