/****************************************************************************

  module      : IOMan_IDataVolumeInfo.cpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area: I/O Management

  description : Interface for data volume informations 

    
  last changed: 2002-08-16
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "IOManager/IOMan_IDataVolumeInfo.hpp"
#include "IOManager/IOMan_Manager.hpp"
#include "FreeBlockManagement/FBM_Manager.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

SAPDB_Bool 
IOMan_DataVolumeInfo::IsVolumeOnline() const
{
    return( IOMan_Manager::GetInstance().IsDataDevSpaceOnline( m_VolumeNo ));
}

/*---------------------------------------------------------------------------*/

SAPDB_Bool
IOMan_DataVolumeInfo::GetVolumeName( tsp00_VFilename   &volumeName ) const
{
    return( IOMan_Manager::GetInstance().GetDataDevName( m_VolumeNo, volumeName ));
}

/*---------------------------------------------------------------------------*/

IOMan_BlockCount
IOMan_DataVolumeInfo::GetVolumeSize() const
{
    return( IOMan_Manager::GetInstance().GetDataDevSpaceSize( m_VolumeNo ));
}

/*---------------------------------------------------------------------------*/

IOMan_BlockCount
IOMan_DataVolumeInfo::GetUsableVolumeSize() const
{
    return( IOMan_Manager::GetInstance().GetDataDevSpaceUsableSize( m_VolumeNo ));
}

/*---------------------------------------------------------------------------*/

IOMan_BlockCount 
IOMan_DataVolumeInfo::GetUsedVolumeSize() const
{
    if( IsVolumeOnline() )
        return( FBM_Manager::Instance().NumBlocksUsed( m_VolumeNo ));
    else
        return( 0 );
}

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

IOMan_DataVolumeInfoIterator::IOMan_DataVolumeInfoIterator()
:m_VolumeInfo( IOMan_Manager::GetInstance().GetFirstDataDevSpaceNo() )
{
}

/*---------------------------------------------------------------------------*/

void
IOMan_DataVolumeInfoIterator::Begin()
{
    m_VolumeInfo.SetVolumeNo( IOMan_Manager::GetInstance().GetFirstDataDevSpaceNo());  
}

/*---------------------------------------------------------------------------*/

SAPDB_Bool
IOMan_DataVolumeInfoIterator::End() const
{
    return( ! IOMan_Manager::GetInstance().IsDataDevSpaceNoValid( m_VolumeInfo.GetVolumeNo() ));
}

/*---------------------------------------------------------------------------*/

void
IOMan_DataVolumeInfoIterator::operator++()
{
    const IOMan_DeviceNo nextDevNo = 
        IOMan_Manager::GetInstance().GetNextDataDevSpaceNo( m_VolumeInfo.GetVolumeNo() +1 );

    m_VolumeInfo.SetVolumeNo( nextDevNo );
}

/*---------------------------------------------------------------------------*/

const IOMan_DataVolumeInfo&
IOMan_DataVolumeInfoIterator::operator*() const
{
    return( m_VolumeInfo );
}


/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
