/*!**************************************************************************

  module      : IOMan_IDiagnose.hpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area: I/O Management
  description : Interface for diagnose/repair 


  last changed: 2001-07-18  11:50
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/


#ifndef IOMAN_IDIAGNOSE_HPP
#define IOMAN_IDIAGNOSE_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gsp00.h"   // PASCAL: SP_basic_constants_and_types

#include "IOManager/IOMan_BlockAddress.hpp"
#include "IOManager/IOMan_Types.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!*****************************************************************************

   class: IOMan_IDiagnose

   description: This interface is used for diagnose and repair purposes.
              
*******************************************************************************/

class IOMan_IDiagnose
{
public:

    /*--------------------------------------------------------------------------
    function:     GetInstance
    description:  This method returns the access to the IOMan_IDiagnose interface
                  of component IOManager. It is assumed that the IOManager singleton 
                  is available! No CreateInstance call will be done.
    arguments:    none
    return value: Access to the IOMan_IDiagnose interface
    --------------------------------------------------------------------------*/

    static IOMan_IDiagnose& GetInstance();

    /*!-------------------------------------------------------------------------
    function:     EnableBadDevSpace()
    description:  This method is used to set a devspace from bad to okay.
    arguments:    taskId  [in] Identification of the calling task.
                  devName [in] Name of the devspace to be enabled
    return value: true means devspace was successful enabled
    --------------------------------------------------------------------------*/

    virtual bool EnableBadDevSpace(
        const tsp00_TaskId  taskId,
        tsp00_VFilename     &devName ) = 0;

    /*!-------------------------------------------------------------------------
    function:     ReadPageDirectFromDataDevSpace()
    description:  This method is used to read a page from the given block address.
                  from a data devspace. Note that NO consistent check is made.
    arguments:    taskId  [in] Identification of the calling task.
                  block   [in] Destination position for the given page frame
                  pPage   [in] Pointer to the page frame to be written
    return value: true means write was successful; else false
    --------------------------------------------------------------------------*/

    virtual bool ReadPageDirectFromDataDevSpace(
        const tsp00_TaskId          taskId,
        const IOMan_BlockAddress    &block,
        const tsp00_PageAddr        pPage ) = 0;

    /*!-------------------------------------------------------------------------
    function:     WritePageDirectToDataDevSpace()
    description:  This method is used to write a given page into the given
                  data devspace. Note that NO consistent check is made and NO
                  checksum is calculated!
    arguments:    taskId  [in] Identification of the calling task.
                  block   [in] Destination position for the given page frame
                  pPage   [in] Pointer to the page frame to be written
    return value: true means write was successful; else false
    --------------------------------------------------------------------------*/

    virtual bool WritePageDirectToDataDevSpace(
        const tsp00_TaskId          taskId,
        const IOMan_BlockAddress    &block,
        const tsp00_PageAddr        pPage ) = 0;

    /*!-------------------------------------------------------------------------
    function:     ReadPageDirectFromArchiveLog()
    description:  This method is used to read a page from the given block address.
                  from the archive log.Note that NO consistent check is made.
    arguments:    taskId  [in] Identification of the calling task.
                  block   [in] Destination position for the given page frame
                  pPage   [in] Pointer to the page frame to be written
    return value: true means write was successful; else false
    --------------------------------------------------------------------------*/

    virtual bool ReadPageDirectFromArchiveLog(
        const tsp00_TaskId          taskId,
        const IOMan_BlockAddress    &block,
        const tsp00_PageAddr        pPage ) = 0;

    /*!-------------------------------------------------------------------------
    function:     WritePageDirectToArchiveLog()
    description:  This method is used to write a given page into the given
                  log devspace. Note that NO consistent check is made and NO
                  checksum is calculated!
    arguments:    taskId  [in] Identification of the calling task.
                  block   [in] Destination position for the given page frame
                  pPage   [in] Pointer to the page frame to be written
    return value: true means write was successful; else false
    --------------------------------------------------------------------------*/

    virtual bool WritePageDirectToArchiveLog(
        const tsp00_TaskId          taskId,
        const IOMan_BlockAddress    &block,
        const tsp00_PageAddr        pPage ) = 0;
};

/*!*****************************************************************************

   endclass: IOMan_IDiagnose

*******************************************************************************/


#endif  /* IOMAN_IDIAGNOSE_HPP */
