/****************************************************************************

  module      : IOMan_ILogVolumeInfo.cpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area: I/O Management

  description : Interface for log volume informations 

    
  last changed: 2002-08-16
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "IOManager/IOMan_ILogVolumeInfo.hpp"
#include "IOManager/IOMan_Manager.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

SAPDB_Bool 
IOMan_LogVolumeInfo::IsVolumeOnline() const
{
    return( IOMan_Manager::GetInstance().IsLogDevSpaceOnline( m_VolumeNo ));
}

/*---------------------------------------------------------------------------*/

SAPDB_Bool
IOMan_LogVolumeInfo::GetVolumeName( tsp00_VFilename   &volumeName ) const
{
    return( IOMan_Manager::GetInstance().GetLogDevName( m_VolumeNo, volumeName ));
}

/*---------------------------------------------------------------------------*/

IOMan_BlockCount
IOMan_LogVolumeInfo::GetVolumeSize() const
{
    return( IOMan_Manager::GetInstance().GetLogDevSpaceSize( m_VolumeNo ));
}

/*---------------------------------------------------------------------------*/

IOMan_BlockCount
IOMan_LogVolumeInfo::GetUsableVolumeSize() const
{
    return( IOMan_Manager::GetInstance().GetLogDevSpaceUsableSize( m_VolumeNo ));
}

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

IOMan_LogVolumeInfoIterator::IOMan_LogVolumeInfoIterator()
:m_VolumeInfo( IOMan_Manager::GetInstance().GetFirstLogDevSpaceNo() )
{
}

/*---------------------------------------------------------------------------*/

void
IOMan_LogVolumeInfoIterator::Begin()
{
    m_VolumeInfo.SetVolumeNo( IOMan_Manager::GetInstance().GetFirstLogDevSpaceNo());  
}

/*---------------------------------------------------------------------------*/

SAPDB_Bool
IOMan_LogVolumeInfoIterator::End() const
{
    return( ! IOMan_Manager::GetInstance().IsLogDevSpaceNoValid( m_VolumeInfo.GetVolumeNo() ));
}

/*---------------------------------------------------------------------------*/

void
IOMan_LogVolumeInfoIterator::operator++()
{
    const IOMan_DeviceNo nextDevNo = 
        IOMan_Manager::GetInstance().GetNextLogDevSpaceNo( m_VolumeInfo.GetVolumeNo() +1 );

    m_VolumeInfo.SetVolumeNo( nextDevNo );
}

/*---------------------------------------------------------------------------*/

const IOMan_LogVolumeInfo&
IOMan_LogVolumeInfoIterator::operator*() const
{
    return( m_VolumeInfo );
}


/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
