/****************************************************************************

  module      : IOMan_IMigration.hpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area: I/O Management
  description : Interface for for data base inplace migration from
                7.2/7.3 to 7.4

  last changed: 2001-03-15  19:43
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/


#ifndef IOMAN_IMIGRATION_HPP
#define IOMAN_IMIGRATION_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gsp00.h"   // PASCAL: SP_basic_constants_and_types
#include "ggg00.h"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!*****************************************************************************

   class: IOMan_IMigration

   description: This interface is used for data base inplace migration from
                7.2/7.3 to 7.4
              
*******************************************************************************/

class IOMan_IMigration
{
public:

    /*--------------------------------------------------------------------------
    function:     GetInstance
    description:  This method returns the access to the IOMan_IMigration interface
                  of component IOManager. It is assumed that the IOManager singleton 
                  is available! No CreateInstance call will be done.
    arguments:    none
    return value: Access to the IOMan_ILogIO interface
    --------------------------------------------------------------------------*/

    static IOMan_IMigration& GetInstance();

    /*!-------------------------------------------------------------------------
    function:     MigrateDataBase()
    description:  This method is used to migrate a non liveCache data base
                  from release 7.2/7.3 to release 7.4 inplace. This includes
                  the migration of the converter and the scattering over all
                  data devspaces, the creation of the IOManInfoPages and the
                  initialization of the log devspaces. In case of a fatal error
                  the system will execute and emergency shutdown. In case of a
                  softer error a base error ist set.
    arguments:    taskId  [in]  Identification of the calling task.
                  trError [out] Current error state
    return value: none
    --------------------------------------------------------------------------*/

    virtual void MigrateDataBase(
        const tsp00_TaskId  taskId,
        tgg00_BasisError    &trError ) = 0;
};

/*!*****************************************************************************

   endclass: IOMan_IMigration

*******************************************************************************/



#endif  /* IOMAN_IMIGRATION_HPP */
