/****************************************************************************

  module      : IOMan_LogDevices.cpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area: I/O Management

  description : This module is used to handle a collecton of log devices
                in particular single and dual logs.

  last changed: 2002-09-02

  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "IOManager/IOMan_LogDevices.hpp"
#include "IOManager/IOMan_SingleLogDevice.hpp"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_NewDestroy.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

SAPDB_Bool
IOMan_LogDevices::Initialize(
    const tsp00_TaskId  taskId,
    const SAPDB_Int     maxLogDevSpaces )
{
    // Create anchor list for single or dual log devices
    if(! m_LogDevices.Resize( maxLogDevSpaces ))
        return( false );

    // Link proper devices with anchor list
    if( m_bIsLogMirrored )
    {
        const SAPDB_UInt4 wantedSpace = maxLogDevSpaces * sizeof( IOMan_DualLogDevice );

        IOMan_DualLogDevice *pDualDevices = reinterpret_cast< IOMan_DualLogDevice* >
                ( m_LogDevices.GetAllocator().Allocate( wantedSpace ));

        if( NULL == pDualDevices )
            return( false );

        newarray( pDualDevices, maxLogDevSpaces, pDualDevices );

        for( Iterator iter = m_LogDevices.Begin(); iter != m_LogDevices.End();
                ++iter, ++pDualDevices )
        {
            *iter = pDualDevices;
        }
    }
    else
    {
        const SAPDB_UInt4 wantedSpace = maxLogDevSpaces * sizeof( IOMan_SingleLogDevice );

        IOMan_SingleLogDevice *pSingleDevices = reinterpret_cast< IOMan_SingleLogDevice* >
                ( m_LogDevices.GetAllocator().Allocate( wantedSpace ));

        if( NULL == pSingleDevices )
            return( false );

        newarray( pSingleDevices, maxLogDevSpaces, pSingleDevices );

        for( Iterator iter = m_LogDevices.Begin(); iter != m_LogDevices.End();
                ++iter, ++pSingleDevices )
        {
            *iter = pSingleDevices;
        }
    }
    return( true );
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
