/*!*****************************************************************************

  module:       IOMan_Types.hpp

  ------------------------------------------------------------------------------

  responsible:  TorstenS

  special area: IOManager

  description:  Basic types of this component
                
  see also:     

  ------------------------------------------------------------------------------

  copyright:            Copyright by SAP AG, 2000



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*******************************************************************************/

#ifndef IOMAN_TYPES_HPP
#define IOMAN_TYPES_HPP

#include "KernelCommon/Kernel_Common.hpp"
#include "SAPDBCommon/SAPDB_EnhancedTypes.hpp"

#include "SAPDBCommon/Tracing/SAPDBTrace_Topic.hpp"
#include "SAPDBCommon/Diagnostics/SAPDBDiag_CheckTopic.hpp"


extern SAPDBTrace_Topic     IOMan_Trace;
extern SAPDBDiag_CheckTopic IOMan_Check;


/*!-----------------------------------------------------------------------------
declaration: IOMan_DeviceNo
description: Logical device number
------------------------------------------------------------------------------*/

typedef SAPDB_Invalidatable<SAPDB_Int2, SAPDB_MAX_INT2>
IOMan_DeviceNo;

/*!-----------------------------------------------------------------------------
declaration: IOMan_BlockNo
description: Logical block offset on device
------------------------------------------------------------------------------*/

typedef SAPDB_Invalidatable<SAPDB_Int4, SAPDB_MAX_INT4>
IOMan_BlockNo;

/*!-----------------------------------------------------------------------------
declaration: IOMan_BlockCount
description: Count of blocks
------------------------------------------------------------------------------*/

typedef SAPDB_Int4 IOMan_BlockCount;

/*!-----------------------------------------------------------------------------
declaration: IOMan_ReturnCode
description: ReturnCodes of component I/O Manager
------------------------------------------------------------------------------*/

typedef enum
{
    IOMan_Okay = 0,
    IOMan_NotConfigured,
    IOMan_AttachFailed,
    IOMan_ConfigurationCorrupted,
    IOMan_FormatFailed,
    IOMan_BadDevSpace,
    IOMan_VolumeCapacityMismatch,
    IOMan_ReadError,
    IOMan_WriteError,
    IOMan_BadInfoPage
} IOMan_ReturnCode;

#endif // IOMAN_TYPES_HPP

