/****************************************************************************

  module      : IOMan_Wrapper.cpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area: IOMan_Wrapper
  description : 


  last changed: 2001-02-06  12:36
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gsp00.h"
#include "ggg00.h"   // PASCAL: GG_kernel_constants_and_types
#include "gbd00.h"
#include "gsp03.h"
#include "hgg01.h"   // g01io_block_count
#include "hgg01_1.h"
#include "hgg04.h"

#include "KernelCommon/Kernel_IPage.hpp"
#include "DataAccess/Data_BasePage.hpp"
#include "DataAccess/Data_Types.hpp"
#include "FrameControl/FrameCtrl_ILog.hpp"
#include "IOManager/IOMan_IDataInfo.hpp"
#include "IOManager/IOMan_IDataIO.hpp"
#include "IOManager/IOMan_IDataManager.hpp"
#include "IOManager/IOMan_IDiagnose.hpp"
#include "IOManager/IOMan_ILogInfo.hpp"
#include "IOManager/IOMan_ILogManager.hpp"
#include "IOManager/IOMan_IMigration.hpp"
#include "IOManager/IOMan_KernelPages.hpp"
#include "IOManager/IOMan_Manager.hpp"
#include "Logging/Log_Page.hpp"
#include "Logging/Log_Types.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define ALLOCATE_FAILED_MSG1 "IOManager: bad alloc    "


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTION PROTOTYPES                                        *
 *===========================================================================*/


/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

externC void bd999CheckPageAfterRead(
    tgg00_BasisError   &TrError,
    tbd_nodeptr        &pNode)
{
    if( pNode->nd_pt2() == pt2Checkpt_egg00)
    {
        tkb00_PagePtr   pRestart = REINTERPRET_CAST( tkb00_PagePtr, pNode );

        Kernel_RestartPage  RestartPage( pRestart );
        if( RestartPage.CheckAfterRead() ) return;
    }
    else
    {
        Data_BasePage   DataPage( pNode );
        if( DataPage.CheckAfterRead() ) return;
    }
    TrError = e_bad_datapage;
}

/*---------------------------------------------------------------------------*/

externC void bd999CreateIOManager( const tsp00_TaskId  taskId )
{
    if ( true != IOMan_Manager::CreateInstance( taskId ))
        g01abort (csp3_allocat, csp3_n_memory, ALLOCATE_FAILED_MSG1, 0);

};

/*---------------------------------------------------------------------------*/

externC void bd999CreateAllLogDevices(
    const tsp00_TaskId  taskId,
    const tsp00_Byte    *pDBIdentifier,
    tgg00_BasisError    &trError )
{
    if(! IOMan_ILogManager::GetInstance().CreateAllLogDevSpaces( taskId, pDBIdentifier ))
        trError = e_disk_not_accessible;
};

/*---------------------------------------------------------------------------*/

externC void bd999CreateAllDataDevices(
    const tsp00_TaskId  taskId,
    const tsp00_Byte    *pDBIdentifier,
    tgg00_BasisError    &trError )
{

    if(! IOMan_IDataManager::GetInstance().CreateAllDataDevSpaces(  taskId, pDBIdentifier ))
        trError = e_disk_not_accessible;
};

/*---------------------------------------------------------------------------*/

externC void bd999AttachOneDataDevice(
    tsp00_TaskId      TaskId,
    tsp00_Int2        DevNo,
    tgg00_BasisError  &TrError )
{
    if(! IOMan_IDataManager::GetInstance().OpenOneDataDevSpace( TaskId, DevNo ))
        TrError = e_disk_not_accessible;
};

/*---------------------------------------------------------------------------*/


externC void bd999AttachOneLogDevice(
    tsp00_TaskId      TaskId,
    tsp00_Int2        DevNo,
    tgg00_BasisError  &TrError )
{
    if(! IOMan_ILogManager::GetInstance().OpenOneLogDevSpace( TaskId, DevNo ))
        TrError = e_disk_not_accessible;
};
/*---------------------------------------------------------------------------*/

externC void bd999AttachAllDevices( tgg00_TransContext &Trans )
{
    if(! IOMan_IDataManager::GetInstance().OpenAllDataDevSpaces( Trans.trTaskId_gg00 ))
    {
        Trans.trError_gg00 = e_disk_not_accessible;
        return;
    }
    if(! IOMan_ILogManager::GetInstance().OpenAllArchiveLogs( Trans.trTaskId_gg00 ))
        Trans.trError_gg00 = e_disk_not_accessible;
};

/*---------------------------------------------------------------------------*/

externC pasbool bd999AttachAllDataDevices( tsp00_TaskId taskId )
{
    return( IOMan_IDataManager::GetInstance().OpenAllDataDevSpaces( taskId ));
}

/*---------------------------------------------------------------------------*/

externC pasbool bd999AttachAllArchiveLogs( tsp00_TaskId taskId )
{
    return( IOMan_ILogManager::GetInstance().OpenAllArchiveLogs( taskId ));
}

/*---------------------------------------------------------------------------*/

externC tsp00_Int4 bd999DataDevSpaceSize( tsp00_Int2 devNo )
{
    return( IOMan_IDataInfo::GetInstance().GetDataDevSpaceSize( devNo ));
};

/*---------------------------------------------------------------------------*/

externC tsp00_Int4 bd999LogDevSpaceSize( tsp00_Int2 devNo )
{
    return( IOMan_ILogInfo::GetInstance().GetLogDevSpaceSize( devNo ));
};

/*---------------------------------------------------------------------------*/

externC void bd999DetachAllDevices( tsp00_TaskId  taskId )
{
    IOMan_IDataManager::GetInstance().CloseAllDataDevSpaces( taskId );
    IOMan_ILogManager::GetInstance().CloseAllLogDevSpaces( taskId );
};

/*---------------------------------------------------------------------------*/

externC void bd999DetachAllDataDevices( tsp00_TaskId  taskId )
{
    IOMan_IDataManager::GetInstance().CloseAllDataDevSpaces( taskId );
}

/*---------------------------------------------------------------------------*/

externC void bd999DetachAllArchiveLogs( tsp00_TaskId  taskId )
{
    IOMan_ILogManager::GetInstance().CloseAllLogDevSpaces( taskId );
}

/*---------------------------------------------------------------------------*/

externC tsp00_Int4 bd999LogIsMirrored()
{
    return( IOMan_ILogInfo::GetInstance().IsLogMirrored() );
}

/*---------------------------------------------------------------------------*/

externC tsp00_Int4 bd999LogSize()
{
    return( IOMan_ILogInfo::GetInstance().TotalUsableLogPages() );
}

/*---------------------------------------------------------------------------*/

externC tsp00_Int4 bd999NoOfDataDevices()
{
    tsp00_Int4 NumOfDevSpaces = 0;

    NumOfDevSpaces = IOMan_IDataInfo::GetInstance().UsedDataDevSpaces();

    if( 0 == NumOfDevSpaces )
        return( IOMan_IDataInfo::GetInstance().ConfiguredDataDevSpaces());
    else
        return( NumOfDevSpaces );
};

/*---------------------------------------------------------------------------*/

externC tsp00_Int4  bd999NoOfLogDevices()
{
    return( IOMan_ILogInfo::GetInstance().ConfiguredLogDevSpaces() );
}

/*---------------------------------------------------------------------------*/

externC tsp00_Int4 bd999TotalDataPages()
{
    return( IOMan_IDataInfo::GetInstance().TotalUsableDataPages());
};

/*---------------------------------------------------------------------------*/

externC void bd999ReadRestartPage(
    const tsp00_TaskId  taskId,
    const tkb00_PagePtr &pRestart )
{
    Kernel_RestartPage  RestartPage( pRestart );

    IOMan_IDataIO::GetInstance().ReadRestartPage( taskId, RestartPage );
};

/*---------------------------------------------------------------------------*/

externC void bd999WriteRestartPage(
    tsp00_TaskId    TaskId,
    tkb00_PagePtr   &pRestart )
{
    Kernel_RestartPage  RestartPage( pRestart );

    IOMan_IDataIO::GetInstance().WriteRestartPage( TaskId, RestartPage );
};

/*---------------------------------------------------------------------------*/

externC void bd999WriteDataPage(
    tgg00_TransContext  &Trans,
    tbd_nodeptr         &pNode
)
{
    Data_BasePage   Page( pNode,  !gg04IsStaticPage( pNode->nd_pt2() ),
                          pNode->nd_pmode().includes( pmTemp_egg00 ) );

    IOMan_IDataIO::GetInstance().WriteDataPage( Trans.trTaskId_gg00,
            Trans.trError_gg00, Page );
};

/*---------------------------------------------------------------------------*/

externC void bd999AsynReadDataPages(
    tgg00_TransContext  &Trans,
    tsp00_PageAddr      pPageList,
    SAPDB_Int4          HostId,
    SAPDB_Int2          DevNo,
    SAPDB_Int4          BlockNo,
    SAPDB_Int2          BlockCount
)
{
    tbd_nodeptr             pNode;
    IOMan_BlockAddress      Block( DevNo, BlockNo );
    IOMan_ClusterAddress    Cluster( Block, BlockCount );
    IOMan_DataPages         Pages( *(REINTERPRET_CAST( SAPDBMem_IRawAllocator*,
                                     Trans.trAllocator_gg00 )), UTF8( "Pager:ReadDataPages" ));
    if(! Pages.Initialize( BlockCount ))
    {
        Trans.trError_gg00 = e_sysbuf_storage_exceeded;
        return;
    }
    for(SAPDB_Int2 Index = 0; Index < BlockCount; ++Index )
    {
        pNode = reinterpret_cast<tbd_nodeptr>( pPageList + Index );

        //Data_BasePage Page( pNode, !gg04IsStaticPage( pNode->nd_pt2() ),
        //    pNode->nd_pmode().includes( pmTemp_egg00 ));

        // PTS 1116852 TS 2002-07-24
        Data_BasePage Page( pNode, true, ! true );

        Pages.Add( Page );
    }
    if( ! IOMan_IDataIO::GetInstance().ReadDataPagesForSave( Trans.trTaskId_gg00, HostId, Pages, Cluster ))
        Trans.trError_gg00 = e_disk_not_accessible;
}

/*---------------------------------------------------------------------------*/

externC void bd999AsynWriteDataPages(
    tgg00_TransContext  &Trans,
    tsp00_PageAddr      pPageList,
    SAPDB_Int4          HostId,
    SAPDB_Int2          DevNo,
    SAPDB_Int4          BlockNo,
    SAPDB_Int2          BlockCount
)
{
    tbd_nodeptr             pNode;
    IOMan_BlockAddress      Block( DevNo, BlockNo );
    IOMan_ClusterAddress    Cluster( Block, BlockCount );
    IOMan_DataPages         Pages( *(REINTERPRET_CAST( SAPDBMem_IRawAllocator*,
                                     Trans.trAllocator_gg00 )), UTF8( "Pager:WriteDataPages" ));
    if(! Pages.Initialize( BlockCount ))
    {
        Trans.trError_gg00 = e_sysbuf_storage_exceeded;
        return;
    }
    for(SAPDB_Int2 Index = 0; Index < BlockCount; ++Index )
    {
        pNode = reinterpret_cast<tbd_nodeptr> ( pPageList + Index );

        Data_BasePage Page( pNode, !gg04IsStaticPage( pNode->nd_pt2() ),
                            pNode->nd_pmode().includes( pmTemp_egg00 ));

        Pages.Add( Page );
    }
    if( ! IOMan_IDataIO::GetInstance().WriteDataPagesForRestore( Trans.trTaskId_gg00, HostId, Pages, Cluster ))
        Trans.trError_gg00 = e_disk_not_accessible;
}

/*---------------------------------------------------------------------------*/

externC void bd999EnableBadDevSpace(
    tsp00_TaskId        taskId,
    tsp00_VFilename     &devName,
    tgg00_BasisError    &trError )
{
    if( ! IOMan_IDiagnose::GetInstance().EnableBadDevSpace( taskId, devName ))
        trError = e_disk_not_accessible;
}

/*---------------------------------------------------------------------------*/

externC void bd999GetDataDevName(
    SAPDB_Int2      devNo,
    tsp00_VFilename &devName )
{
    IOMan_IDataInfo::GetInstance().GetDataDevName( devNo, devName );
}


/*---------------------------------------------------------------------------*/

externC void bd999GetLogDevName(
    SAPDB_Int2      devNo,
    tsp00_VFilename &devName )
{
    IOMan_ILogInfo::GetInstance().GetLogDevName( devNo, devName );
}

/*---------------------------------------------------------------------------*/

externC pasbool bd999GetDataDevNo(
    tsp00_VFilename &devName,
    IOMan_DeviceNo  &devNo)
{
    return( IOMan_IDataInfo::GetInstance().GetDataDevSpaceNo( devName, devNo ));
}

/*---------------------------------------------------------------------------*/

externC pasbool bd999GetLogDevNo(
    tsp00_VFilename &devName,
    IOMan_DeviceNo  &devNo)
{
    return( IOMan_ILogInfo::GetInstance().GetLogDevSpaceNo( devName, devNo ));
}

/*---------------------------------------------------------------------------*/


externC void bd999ReadDataPageDirect(
    tsp00_TaskId        taskId,
    tsp00_Int2          devNo,
    tsp00_Int4          blockNo,
    tsp00_PageAddr      &pPage,
    tgg00_BasisError    &trError)
{
    IOMan_BlockAddress  block( devNo, blockNo );

    if( ! IOMan_IDiagnose::GetInstance().ReadPageDirectFromDataDevSpace( taskId, block, pPage ))
        trError = e_disk_not_accessible;
}

/*---------------------------------------------------------------------------*/

externC void bd999WriteDataPageDirect(
    tsp00_TaskId        taskId,
    tsp00_Int2          devNo,
    tsp00_Int4          blockNo,
    tsp00_PageAddr      &pPage,
    tgg00_BasisError    &trError)
{
    IOMan_BlockAddress  block( devNo, blockNo );

    if( ! IOMan_IDiagnose::GetInstance().WritePageDirectToDataDevSpace( taskId, block, pPage ))
        trError = e_disk_not_accessible;
}

/*---------------------------------------------------------------------------*/

externC void bd999ReadLogPageDirect(
    tsp00_TaskId        taskId,
    tsp00_Int2          devNo,
    tsp00_Int4          blockNo,
    tsp00_PageAddr      &pPage,
    tgg00_BasisError    &trError)
{
    IOMan_BlockAddress  block( devNo, blockNo );

    if( ! IOMan_IDiagnose::GetInstance().ReadPageDirectFromArchiveLog( taskId, block, pPage ))
        trError = e_disk_not_accessible;
}

/*---------------------------------------------------------------------------*/

externC void bd999WriteLogPageDirect(
    tsp00_TaskId        taskId,
    tsp00_Int2          devNo,
    tsp00_Int4          blockNo,
    tsp00_PageAddr      &pPage,
    tgg00_BasisError    &trError)
{
    IOMan_BlockAddress  block( devNo, blockNo );

    if( ! IOMan_IDiagnose::GetInstance().WritePageDirectToArchiveLog( taskId, block, pPage ))
        trError = e_disk_not_accessible;
}

/*---------------------------------------------------------------------------*/

externC pasbool bd999AddDataDevice(
    tsp00_TaskId    taskId,
    pasbool         bIsCold,
    tsp00_VFilename &devName,
    tsp00_Int4      devSize,
    tsp00_Int2      devNo )
{
    return( IOMan_IDataManager::GetInstance().AddDataDevSpace( taskId, bIsCold,
            devName, devSize, devNo ));
}

/*---------------------------------------------------------------------------*/

externC pasbool bd999AddLogDevice(
    tgg00_TransContext  &trans,
    pasbool             bIsCold,
    tsp00_VFilename     &devName,
    tsp00_VFilename     &mirrDevName,
    tsp00_Int4          devSize,
    tsp00_Int2          devNo,
    tsp00_Int4          &newLogSize )
{
    tsp00_TaskId                &taskId   = trans.trTaskId_gg00;
    Kernel_IPage::PageFrame     pageFrame = FrameCtrl_ILog::GetInstance().NewLogFrame( taskId );

    Log_Page page( pageFrame, Log_QueueID(), Log_IOSequenceNo() );

    IOMan_LogPages  pages( *(REINTERPRET_CAST( SAPDBMem_IRawAllocator*,
                             trans.trAllocator_gg00 )), UTF8( "Pager:WriteLogPages" ));
    if(! pages.Initialize( g01io_block_count() ))
    {
        trans.trError_gg00 = e_sysbuf_storage_exceeded;
        return false;
    }

    for( SAPDB_Int i = 0; i < 4; ++i )
        pages.Add( page );

    SAPDB_Bool retCode = IOMan_ILogManager::GetInstance().AddLogDevSpace( trans.trTaskId_gg00, bIsCold,
                         devName, mirrDevName, devSize, devNo, pages, newLogSize );

    FrameCtrl_ILog::GetInstance().FreeLogFrame( taskId, pageFrame );

    return( retCode );
}

/*---------------------------------------------------------------------------*/

externC void bd999MigrateDataBase(
    tsp00_TaskId       taskId,
    tgg00_BasisError   &trError )
{
    IOMan_IMigration::GetInstance().MigrateDataBase( taskId, trError );
}

/*---------------------------------------------------------------------------*/

externC void bd999ReIntegrateBadLogVolume(
    tsp00_TaskId        taskId,
    tsp00_VFilename     &badDevName,
    tgg00_BasisError    &trError )
{
    if( ! IOMan_ILogManager::GetInstance().ReIntegrateBadLogVolume( taskId, badDevName ))
        trError = e_disk_not_accessible;
}
/*---------------------------------------------------------------------------*/

externC void bd999GetBadLogVolume(
    tsp00_TaskId        taskId,
    tsp00_VFilename     &badDevName,
    tgg00_BasisError    &trError )
{
    if( ! IOMan_ILogInfo::GetInstance().GetBadLogVolume( taskId, badDevName ))
        trError = e_row_not_found; // no bad volume found
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
