
/*!**************************************************************************

    module      : Kernel_Test.cpp

    -------------------------------------------------------------------------

    responsible : N.N.

    special area: KernelCommon
    description : entrypoint for KernelCommon tests

    last changed: 2001-01-17

    -------------------------------------------------------------------------

    copyright:    Copyright by SAP AG, 2001

    ========== licence begin LGPL
    Copyright (C) 2001 SAP AG
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "KernelCommon/Kernel_Test.hpp"
#include "KernelCommon/Kernel_WaitQueue.hpp"
#include "SAPDBCommon/ErrorsAndMessages/SAPDBErr_Assertions.hpp"

#include "gsp00_0.h" // ROUTINE_DBG_MSP00

//#include "SAPDBCommon/MemoryManagement/SAPDBMem_IncrementalRawAllocator.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/
/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/
/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/
/*===========================================================================*
 *  LOCAL FUNCTIONS                                                          *
 *===========================================================================*/
/*===========================================================================*
 *  EXTERNAL FUNCTIONS                                                       *
 *===========================================================================*/

static inline void Kernel_WaitQueueTest (tsp00_TaskId CallingTaskId)
{
    ROUTINE_DBG_MSP00 ("Kernel_WaitQueueTest");
    
    Kernel_WaitStack WaitStack;
    tsp00_TaskId     PopTaskId;
    SAPDB_Int4       MaxItems;
    SAPDB_Int4       ItemCount = 0;

    Kernel_TaskStack::Kernel_PopItemState PopItemState;
    

    SAPDBERR_ASSERT_STATE (! Kernel_WaitStackGlob::Instance().IsEmptyFreeList (CallingTaskId));

    SAPDBERR_ASSERT_STATE (WaitStack.IsEmpty(CallingTaskId));

    WaitStack.PushTaskId (CallingTaskId);
    SAPDBERR_ASSERT_STATE (! WaitStack.IsEmpty(CallingTaskId));
    
    WaitStack.PushTaskId (CallingTaskId);
    SAPDBERR_ASSERT_STATE (! WaitStack.IsEmpty(CallingTaskId));
    
    WaitStack.PopTaskId (CallingTaskId, PopTaskId, PopItemState);
    SAPDBERR_ASSERT_STATE (! WaitStack.IsEmpty(CallingTaskId));
    SAPDBERR_ASSERT_STATE ( Kernel_TaskStack::pstAnyItemLeft == PopItemState);
    
    WaitStack.PopTaskId (CallingTaskId, PopTaskId, PopItemState);
    SAPDBERR_ASSERT_STATE (WaitStack.IsEmpty(CallingTaskId));
    SAPDBERR_ASSERT_STATE (Kernel_TaskStack::pstLastItem == PopItemState);

    WaitStack.InsertTaskIdAtBottom (CallingTaskId);
    SAPDBERR_ASSERT_STATE (! WaitStack.IsEmpty(CallingTaskId));
    
    WaitStack.InsertTaskIdAtBottom (CallingTaskId);
    SAPDBERR_ASSERT_STATE (! WaitStack.IsEmpty(CallingTaskId));
    
    WaitStack.PopTaskId (CallingTaskId, PopTaskId, PopItemState);
    SAPDBERR_ASSERT_STATE ( Kernel_TaskStack::pstAnyItemLeft == PopItemState);
    SAPDBERR_ASSERT_STATE (! WaitStack.IsEmpty(CallingTaskId));
    
    WaitStack.PopTaskId (CallingTaskId, PopTaskId, PopItemState);
    SAPDBERR_ASSERT_STATE (Kernel_TaskStack::pstLastItem == PopItemState);
    SAPDBERR_ASSERT_STATE (WaitStack.IsEmpty(CallingTaskId));

    while ( ! Kernel_WaitStackGlob::Instance().IsEmptyFreeList (CallingTaskId) )
    {
        WaitStack.InsertTaskIdAtBottom (CallingTaskId);
        ItemCount += 1;
        
        SAPDBERR_ASSERT_STATE( ! WaitStack.IsEmpty(CallingTaskId) );
    }
    
    MaxItems = ItemCount;
    
    while ( 0 < ItemCount )
    {
        PopTaskId = 0;
        WaitStack.PopTaskId (CallingTaskId, PopTaskId, PopItemState);
        ItemCount -= 1;
        SAPDBERR_ASSERT_STATE( PopTaskId == CallingTaskId);
        
        if ( 0 < ItemCount )
        {
            SAPDBERR_ASSERT_STATE( ! WaitStack.IsEmpty(CallingTaskId) );
            SAPDBERR_ASSERT_STATE( Kernel_TaskStack::pstAnyItemLeft == PopItemState);
        }
        else
        {
            SAPDBERR_ASSERT_STATE( WaitStack.IsEmpty(CallingTaskId) );
            SAPDBERR_ASSERT_STATE( Kernel_TaskStack::pstLastItem == PopItemState);
        }
    }
    
    ItemCount = 0;

    while (ItemCount < MaxItems)
    {
        WaitStack.PushTaskId (CallingTaskId);
        ItemCount += 1;
        
        SAPDBERR_ASSERT_STATE( ! WaitStack.IsEmpty(CallingTaskId) );
    }

    do
    {
        PopTaskId = 0;
        WaitStack.PopTaskId (CallingTaskId, PopTaskId, PopItemState);
        ItemCount -= 1;
        SAPDBERR_ASSERT_STATE( PopTaskId == CallingTaskId);
    }
    while (Kernel_TaskStack::pstAnyItemLeft == PopItemState);
    
    SAPDBERR_ASSERT_STATE( WaitStack.IsEmpty(CallingTaskId) );
    SAPDBERR_ASSERT_STATE( 0 == ItemCount);
}

/* --------------------------------------------------------------------------- */

externCpp void Kernel_Test (tgg00_TransContext& Trans)
{
    ROUTINE_DBG_MSP00 ("Kernel_Test");
    
    SAPDBErr_MessageList AllExceptions;
    
    // not used but tested
    
    //SAPDBMem_IncrementalRawAllocator& Allocator =
    //    *( REINTERPRET_CAST(SAPDBMem_IncrementalRawAllocator*, Trans.trAllocator_gg00) );
    
    Kernel_WaitQueueTest (Trans.trTaskId_gg00);
    
    return; // ++++ TEST was successfull ++++
}
