/*!*********************************************************************

  module:       Kernel_TraceTopics.hpp

  ----------------------------------------------------------------------

  responsible:  DanielD

  special area: Kernel Tracing

  description:  exports array with all kernel trace topics

  ----------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000

    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

  *********************************************************************/

#ifndef KERNEL_TRACETOPICS_HPP
#define KERNEL_TRACETOPICS_HPP

class SAPDBTrace_Topic;

/*!
    declaration: Kernel_AllTraceTopics
    description: variable to be used as parameter for
        <Method>SAPDBTrace_Topic::MultiSwitch</>
 */
extern SAPDBTrace_Topic * const Kernel_AllTraceTopics [];

/* enddeclaration: */

/*! --------------------------------------------------------------------
    function:    Kernel_CheckSwitch
    description: parse and executes a line with check instructions

        The line is of the form "<id> [<int>] [<id> [<int>] ] ...

        An id '*' serves as a wildcard for all topics.

        interface in vgg941.
*/
externPascal void Kernel_CheckSwitch (const char*  TopicStr, /* c20 */
                                      int          TopicStrLen);

/*! --------------------------------------------------------------------
    function:    Kernel_TraceSwitch
    description: parse and executes a line with trace instructions

        The line is of the form "<id> [<int>] [<id> [<int>] ] ...

        An id '*' serves as a wildcard for all topics.

        interface in vgg941.
*/
externPascal void Kernel_TraceSwitch (const char*  TopicStr, /* c20 */
                                      int          TopicStrLen);

#endif

