/*!***************************************************************************

  module      : LVCSim_ObjectVerHandle.hpp

  -------------------------------------------------------------------------

  responsible : IvanS

  special area: liveCache Simulator
  description : Internal object version handle 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002



    ========== licence begin LGPL
    Copyright (C) 2002 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef LVCSIM_OBJECTVERHANDLE_HPP
#define LVCSIM_OBJECTVERHANDLE_HPP

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

//#include "SAPDBCommon/SAPDB_Types.hpp"
#include "LVCSimulator/LVCSim_Internal.hpp"
#include "LVCSimulator/LVCSim_OIDAllocator.hpp"
#include "LVCSimulator/LVCSim_CheckpointIO.hpp"
#include <memory.h>

/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/



/*! 
  Class: LVCSim_ObjectVerHandle
  
  Description: Defines the control block of one version of the persistent
    object in memory. This handle is allocated for each version of the
	persistent object. It also contains the object itself.
*/


class LVCSim_ObjectVerHandle
{
protected:
	friend class LVCSim_ObjectHandle;
	friend class LVCSim_liveCacheSink;

	LVCSim_TID				m_tid_min;
	LVCSim_ObjectVerHandle	*m_next;

	union {
		char				fix[1];
		struct {
			size_t			size;
			char			data[1];
		} var;
	} m_data;

public:
	LVCSim_ObjectVerHandle(LVCSim_CheckpointReader &i, size_t objsize);
	LVCSim_ObjectVerHandle(
		LVCSim_TID tmin,
		const void *bodydata,
		size_t bodylen, 
		bool varsize,
		LVCSim_ObjectVerHandle *nextobj)
	:
		m_tid_min(tmin),
		m_next(nextobj)
	{
		if (varsize) {
			m_data.var.size = bodylen;
			memcpy(m_data.var.data, bodydata, bodylen);
		} else {
			memcpy(m_data.fix, bodydata, bodylen);
		}
	}

	~LVCSim_ObjectVerHandle();


	static void *operator new(size_t sz, size_t datasize);
	static void operator delete(void *ptr, size_t datasize);
	static void operator delete(void *ptr);

	void writeToStream(LVCSim_CheckpointWriter &o, size_t objsize) const;
};

#endif
