/*!**************************************************************************

  module      : Log_ActionPartialRollback.hpp
  special area: Logging
  responsible : UweH
  last changed: 2000-09-25  12:00
  copyright:    Copyright by SAP AG, 2000
  description : defines class Log_ActionPartialRollback



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef Log_ActionPartialRollback_H
#define Log_ActionPartialRollback_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SAPDBCommon/SAPDB_Types.hpp"
#include "DataAccess/Data_ISplitSpace.hpp"
#include "Logging/Log_Types.hpp"
#include "Logging/Log_IAction.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/
/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/
/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/
/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*! --------------------------------------------------------------------------------
class:       Log_ActionPartialRollback
description: Handle for manipulating OMS objects.
*/

class Log_ActionPartialRollback : public Log_IAction
{

public:

    /*! --------------------------------------------------------------------------------
    function:    Initialize
    description: assigns the sequences for truncation
    arguments:   UndoStopSequence [in]
                 RedoStopSequence [in]
    */
    void Initialize (Log_EntrySequence UndoStopSequence,
                     Log_EntrySequence RedoStopSequence)
    {
        m_Head.UndoStopSequence = UndoStopSequence;
        m_Head.RedoStopSequence = RedoStopSequence;
    }
    
    /*! --------------------------------------------------------------------------------
    function:    GetMinimalLengthToCopyEntry
    */
    static SAPDB_UInt GetMinimalLengthToCopyEntry()
    {
        return sizeof (Head);
    }

    /*! --------------------------------------------------------------------------------
    function:    GetMinimalPersistentLength
    description: 
    */
    virtual SAPDB_UInt GetMinimalPersistentLength() const
    {
        return GetMinimalLengthToCopyEntry();
    }
    
    /*! --------------------------------------------------------------------------------
    function:    GetPersistentLength
    description: 
    */
    virtual SAPDB_UInt GetPersistentLength() const
    {
        return GetMinimalPersistentLength();
    }
    
    /*! --------------------------------------------------------------------------------
    function:    WritePersistentFormat
    arguments:   Writer    [in]
                 isOK      [out]
    description: The Space must be initialized with the value given by GetPersistentLength().
    */
    virtual void WritePersistentFormat (Data_SplitSpaceWriter &Writer,
                                        bool                  &isOK) const
    {
        Head                          *pHead;
        Data_SplitSpaceWriter::Result  result;
        Writer.Reserve (sizeof(Head), REINTERPRET_CAST(SAPDB_Byte*&, pHead), result);
        isOK = Data_SplitSpaceWriter::ok == result;
        if ( ! isOK ) return;
        *pHead = m_Head;
    }

    /*! --------------------------------------------------------------------------------
    function:    ReadPersistentFormat
    arguments:   Reader    [in]
                 Allocator [in/out]
                 isOK      [out]
    description: all internal components are initialized with the reader
    */
    virtual void ReadPersistentFormat (Data_SplitSpaceReader  &Reader,
                                       SAPDBMem_IRawAllocator &Allocator,
                                       bool                   &isOK)
    {
        Head                          *pHead;
        Data_SplitSpaceReader::Result  result;
        Reader.Reserve (sizeof(Head), REINTERPRET_CAST(SAPDB_Byte*&, pHead), result);
        isOK = Data_SplitSpaceReader::ok == result;
        if ( ! isOK ) return;
        m_Head = *pHead;
    }

    /*  --------------------------------------------------------------------------------
    function:    WriteToTrace
    arguments:   Title [in]
    description: The action is written to the trace file
    */    
    virtual void WriteToTrace (const char * Title) const
    {}
    /*  --------------------------------------------------------------------------------
    function:    GetRedoStopSequence
    */    
    Log_EntrySequence GetRedoStopSequence () const
    {
        return m_Head.RedoStopSequence;
    }
    /*  --------------------------------------------------------------------------------
    function:    GetUndoStopSequence
    */    
    Log_EntrySequence GetUndoStopSequence () const
    {
        return m_Head.UndoStopSequence;
    }

private:

    /*! --------------------------------------------------------------------------------
    declaration:    Header
    */
    typedef struct
    {
        Log_EntrySequence UndoStopSequence;
        Log_EntrySequence RedoStopSequence;
    } Head;
    /*! enddeclaration: */

    /*! --------------------------------------------------------------------------------
    variable:       m_Head
    */
    Head m_Head;
};
/*! endclass: Log_ActionPartialRollback */

#endif  /* Log_ActionPartialRollback_H */
