/*!**************************************************************************

  module      : Log_ActionSavepoint.hpp
  special area: Logging
  responsible : UweH
  last changed: 2000-09-25  12:00
  copyright:    Copyright by SAP AG, 2000
  description : defines class Log_ActionSavepoint



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef Log_ActionSavepoint_H
#define Log_ActionSavepoint_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SAPDBCommon/SAPDB_Types.hpp"
#include "DataAccess/Data_ISplitSpace.hpp"
#include "Logging/Log_IAction.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/
/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/
/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/
/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*! --------------------------------------------------------------------------------
class:       Log_ActionSavepoint
description: Handle for manipulating OMS objects.
*/

class Log_ActionSavepoint : public Log_IAction
{

public:

    /*! --------------------------------------------------------------------------------
    function:    Log_ActionSavepoint
    */
    Log_ActionSavepoint (Log_SavepointReason Reason)
    {
        m_Head.Reason = Reason;
    }
    
    /*  --------------------------------------------------------------------------------
    function:    ForceLogWriting
    returns:     true
    */                         
	// PTS 1117320 UH 2002-10-28 added
    virtual bool ForceLogWriting () const
    {
        return true;
    }

    /*  --------------------------------------------------------------------------------
    function:    WaitForLogWriting
    returns:     true
    */                         
	// PTS 1117320 UH 2002-10-28 added
    virtual bool WaitForLogWriting () const
    {
        return true;
    }
    
    /*! --------------------------------------------------------------------------------
    function:    GetMinimalLengthToCopyEntry
    */
    static SAPDB_UInt GetMinimalLengthToCopyEntry()
    {
        return sizeof (Head);
    }

    /*! --------------------------------------------------------------------------------
    function:    GetMinimalPersistentLength
    description: 
    */
    virtual SAPDB_UInt GetMinimalPersistentLength() const
    {
        return GetMinimalLengthToCopyEntry();
    }
    
    /*! --------------------------------------------------------------------------------
    function:    GetPersistentLength
    description: 
    */
    virtual SAPDB_UInt GetPersistentLength() const
    {
        return GetMinimalPersistentLength();
    }
    
    /*! --------------------------------------------------------------------------------
    function:    WritePersistentFormat
    arguments:   Writer    [in]
                 isOK      [out]
    description: The Space must be initialized with the value given by GetPersistentLength().
    */
    virtual void WritePersistentFormat (Data_SplitSpaceWriter &Writer,
                                        bool                  &isOK) const
    {
        Head                          *pHead;
        Data_SplitSpaceWriter::Result  result;
        Writer.Reserve (sizeof(Head), REINTERPRET_CAST(SAPDB_Byte*&, pHead), result);
        isOK = Data_SplitSpaceWriter::ok == result;
        if ( ! isOK ) return;
        *pHead = m_Head;
    }
    

    /*! --------------------------------------------------------------------------------
    function:    ReadPersistentFormat
    arguments:   Reader    [in]
                 Allocator [in/out]
                 isOK      [out]
    description: all internal components are initialized with the reader
    */
    virtual void ReadPersistentFormat (Data_SplitSpaceReader  &Reader,
                                       SAPDBMem_IRawAllocator &Allocator,
                                       bool                   &isOK)
    {
        Head                          *pHead;
        Data_SplitSpaceReader::Result  result;
        Reader.Reserve (sizeof(Head), REINTERPRET_CAST(SAPDB_Byte*&, pHead), result);
        isOK = Data_SplitSpaceReader::ok == result;
        if ( ! isOK ) return;
        m_Head = *pHead;
    }
    
    /*  --------------------------------------------------------------------------------
    function:    WriteToTrace
    arguments:   Title [in]
    description: The action is written to the trace file
    */    
    virtual void WriteToTrace (const char * Title) const
    {}

private:

    /*! --------------------------------------------------------------------------------
    declaration:    Header
    */
    typedef struct Head
    {
        SAPDB_UInt1 Reason;  // 1 byte
        SAPDB_UInt1 Filler1; // 1 byte
        SAPDB_UInt2 Filler2; // 2 byte
        Head()
        {
            Reason  = Log_SVPReasonUnknown;
            Filler1 = 0;
            Filler2 = 0;
        }
    } Head;
    /*! enddeclaration: */

    /*! --------------------------------------------------------------------------------
    variable:       m_Head
    */
    Head m_Head;
};
/*! endclass: Log_ActionSavepoint */

#endif  /* Log_ActionSavepoint_H */
