/*!**************************************************************************

  module      : Log_IEventing.hpp
  special area: Logging
  responsible : UweH
  created     : 2000-09-25  12:00
  last changed: 2000-10-27  12:00
  copyright:    Copyright by SAP AG, 2000
  description : defines the classes Log_IEventing



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef Log_IEventing_H
#define Log_IEventing_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SAPDBCommon/SAPDB_Types.hpp"
#include "vsp0031.h" // events
#include "gsp00.h"   // tsp00_TaskId

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/
/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/
/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/
/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*---------------------------------------------------------------------------*/
/*! class:       Log_IEventing
    description: This is the Interface for all components,
                 which want to know what is going on.
*/
class Log_IEventing
{
public:

    /*! --------------------------------------------------------------------------------
    function:    DelEventAboveLimit
    arguments:   TaskId  [in]
                 Percent [in]
    */
    virtual void DelEventAboveLimit (tsp00_TaskId TaskId,
                                     SAPDB_Int4   Percent);

    /*! --------------------------------------------------------------------------------
    function:    SetEventAboveLimit
    arguments:   TaskId  [in]
                 Percent [in]
                 Prio    [in]
    */
    virtual void SetEventAboveLimit (tsp00_TaskId     TaskId,
                                     SAPDB_Int4       Percent,
                                     tsp31_event_prio Prio);

    /*! --------------------------------------------------------------------------------
    function:    GetEvents
    arguments:   EventInfo [out]
    */
    virtual void GetEvents (tsp31_short_event_desc &EventInfo);

};
/*! endclass: Log_IEventing */



/*===========================================================================*
 *  DEFINITION OF INLINE METHODS                                              *
 *===========================================================================*/
                                         
#endif  /* Log_IEventing_H */
