
/*!**************************************************************************

    module      : Log_IAction.hpp

    -------------------------------------------------------------------------

    responsible : UweH

    special area: Logging
    description : defines the class Log_IImage

    last changed: 2002-04-05

    -------------------------------------------------------------------------

    copyright:    Copyright by SAP AG, 2001

    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef Log_IImage_H
#define Log_IImage_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SAPDBCommon/SAPDB_Types.hpp"
#include "DataAccess/Data_ISplitSpace.hpp"
#include "Logging/Log_Types.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/
/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/
/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/
/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*! --------------------------------------------------------------------------------
class:        Log_IImage
description : common image interface
----------------------------------------------------------------------------------*/
class Log_IImage
{

public:
    /*! --------------------------------------------------------------------------------
    function:    GetActionType
    returns:     action type
    ----------------------------------------------------------------------------------*/
    virtual Log_ActionType GetActionType() const = 0;
    /*! --------------------------------------------------------------------------------
    function:    GetSequence
    returns:     local entry sequence
    ----------------------------------------------------------------------------------*/
    virtual Log_EntrySequence GetSequence() const = 0;
    /*! --------------------------------------------------------------------------------
    function:    WritePersistentFormat
    arguments:   Writer [in/out]
                 isOK   [out]
    description: the image is written as persistent layout
    ----------------------------------------------------------------------------------*/
    virtual void WritePersistentFormat (Data_SplitSpaceWriter &Writer,
                                        bool                  &isOK) const = 0;

    /*! --------------------------------------------------------------------------------
    function:    ReadPersistentFormat
    arguments:   Reader [in/out]
                 isOK   [out]
    description: the image is read from persistent layout
    ----------------------------------------------------------------------------------*/
    virtual void ReadPersistentFormat (Data_SplitSpaceReader &Reader,
                                       bool                  &isOK) = 0;
    /*  --------------------------------------------------------------------------------
    function:    WriteToTrace
    arguments:   Title [in]
    description: The action is written to the trace file
    ----------------------------------------------------------------------------------*/ 
    virtual void WriteToTrace (const char * title) const = 0;
    /*  --------------------------------------------------------------------------------
    function:    IsAfterImage
    description: returns true, if image is an after image.
    ----------------------------------------------------------------------------------*/ 
    virtual bool IsAfterImage () const = 0;
};
/*! endclass: Log_IImage */

#endif  /* Log_IImage_H */
