/*!**************************************************************************

  module      : Log_IMonitoring.hpp
  special area: Logging
  responsible : UweH
  created     : 2000-09-25  12:00
  last changed: 2000-10-27  12:00
  copyright:    Copyright by SAP AG, 2000
  description : defines the classes Log_IMonitoring



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef Log_IMONITORING_H
#define Log_IMONITORING_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SAPDBCommon/SAPDB_Types.hpp"
#include "gsp00.h" // tsp00_TaskId

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/
/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/
/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/
/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*---------------------------------------------------------------------------*/
/*! class:       Log_MonitorValues
    description: This base class describes a general interface for exchanging
                 values, used for monitoring.
                 
                 These values are read and interpreted by a human beings.
                 Therefor a little table is defined. The columns are 
*/
class Log_MonitorValues
{
public:
    /*! --------------------------------------------------------------------------------
    function:    PutValue
    arguments:   Description [in]
                 Value       [in]
    description: A new value is inserted to be read out by the questionaire.
    */
    void PutValue (char*        Description,
                   SAPDB_ULong &Value);

    /*! --------------------------------------------------------------------------------
    function:    GetValue
    arguments:   Description [out]
                 Value       [out]
                 ThereIsMore [out]
    description: Read out a value, if it is the last one ThereIsMore is false.
    */
    void GetValue (char*        Description,
                   SAPDB_ULong &Value,
                   bool        &ThereIsMore);
};
/*! endclass: Log_MonitorValues */

/*---------------------------------------------------------------------------*/
/*! class:       Log_IMonitoring
    description: This is the Interface for all components,
                 which want to know what is going on.
*/
class Log_IMonitoring
{
public:
    /*!
    function:    ResetMonitorValues
    arguments:   TaskId [in]
    description: The "counter are resetted".
    */
    virtual void ResetMonitorValues (tsp00_TaskId TaskId) = 0;
    /*!
    function:    GetMonitorValues
    arguments:   TaskId    [in]
                 Statistic [out]
    */
    virtual void GetMonitorValues (tsp00_TaskId          TaskId,
                                   Log_MonitorValues &Values) = 0;
};
/*! endclass: Log_IMonitoring */



/*===========================================================================*
 *  DEFINITION OF INLINE METHODS                                              *
 *===========================================================================*/
                                         
#endif  /* Log_IMONITORING_H */
