/*!**************************************************************************

  module      : Log_IReadingRedoEntries.hpp
  special area: Logging
  responsible : UweH
  created     : 2000-09-25  12:00
  last changed: 2000-10-27  12:00
  copyright:    Copyright by SAP AG, 2000
  description : defines the classes Log_IReadingRedoEntries and Log_RedoLogentry



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef Log_IReadingRedoEntries_H
#define Log_IReadingRedoEntries_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h" // tgg00_TransContext, tgg00_BasisError

#include "SAPDBCommon/SAPDB_Types.hpp"
#include "Logging/Log_AfterImage.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/

/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/

/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*---------------------------------------------------------------------------*/
/*! class:       Log_IReadingRedoEntries
    description: This is the Interface is for reading from the archive log.
*/
class Log_IReadingRedoEntries
{
public:
    /*---------------------------------------------------------------------------*/
    /*! class:       Iterator
        description: This allows read access to archive log entries.
    */
    class Iterator
    {
    public:
        /*!
        function:    Iterator
        */
        Iterator() {}
        /*!
        function:    deref
        arguments:   AfterImage[out]
        description: The Iterator is dereferenced to the logentry on the current position.
        */
        void deref (Log_AfterImage & AfterImage);
        /*!
        function:    ++
        description: The Iterator is set to next logentry on the archive log.
        */
        Iterator & operator ++ ();
        /*!
        function:    End
        returns:     true, if the iterator is at the end of the log.
        */
        bool End();
    protected:
        /*!
        function:    Set
        */
        void Set ();
    };
    /*! endclass: Iterator */

    /*!
    function:    Begin
    arguments:   TaskId   [in]
                 Iterator [out]
    description: The iterator is set to the first logentry which can be read.
    */
    virtual void Begin (tsp00_TaskId &TaskId,
                        Iterator     &Iterator) = 0;
};
/*! endclass: Log_IReadingRedoEntries */


/*===========================================================================*
 *  DEFINITION OF INLINE METHODS                                              *
 *===========================================================================*/
                                         
#endif  /* Log_IReadingRedoEntries_H */
