
/*!**************************************************************************
    module      : Log_IRedoReadTask.hpp
    -------------------------------------------------------------------------
    responsible : UweH
    special area: Logging
    description : defines an interface class of a singleton to handle log reader tasks
    last changed: 2002-01-16
    -------------------------------------------------------------------------

    copyright:    Copyright by SAP AG, 2001

    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef LOG_IREDO_READ_TASK_H
#define LOG_IREDO_READ_TASK_H


/*===========================================================================*
*  INCLUDES                                                                  *
*============================================================================*/

#include "gsp00.h"
#include "ggg00.h"
#include "KernelCommon/Kernel_DateTime.hpp"
#include "Logging/Log_Types.hpp"

/*===========================================================================*
*  CLASSES                                                                   *
*============================================================================*/

//=======================================================================================
/*! class:       Log_IRedoReadTask

    description: interface class of a singleton to handle log reader tasks
*/
class Log_IRedoReadTask
{
protected:
    
    static Log_IRedoReadTask *m_pInstance;
    
public:

    //----------------------------------------------------------------------------------
    /*! function:     ReadLogAndCreateRedoFiles
        description:  Reads log entries from the archive log or the log save and creates redo files
        argument:     TransContext [in out]
    */
    virtual void ReadLogAndCreateRedoFiles (tgg00_TransContext &TransContext) = 0;
    //----------------------------------------------------------------------------------
    /*! function:     GetReadPosition
        description:  The actual read position is determined. e.g. for the savepoint.
        argument:     iosequence  [out]
                      pageoffset  [out]
                      entryoffset [out]
                      entrytype   [out]
                      entrydate   [out]
                      entrytime   [out]
    */
    virtual void GetReadPosition (Log_IOSequenceNo    &iosequence,
                                  Log_RawDeviceOffset &pageoffset,
                                  SAPDB_UInt2         &entryoffset,
                                  Log_ActionType      &entrytype,
                                  Kernel_Date         &entrydate,
                                  Kernel_Time         &entrytime) const = 0;
    //----------------------------------------------------------------------------------
    /*! function:     Instance
        return value: the reference to the global instance of the singleton
    */
    static Log_IRedoReadTask &Instance ()
    { 
        return (*m_pInstance);
    }
};
/*! endclass: Log_IRedoReadTask */

#endif  /* LOG_IREDO_READ_TASK_H */
