
/*!**************************************************************************

    module      : Log_RedoManager.hpp

    -------------------------------------------------------------------------

    responsible : UweH

    special area: Logging
    description : parallel undo/redo of transactions in order to restart the database

    last changed: 2001-03-22

    -------------------------------------------------------------------------

    copyright:    Copyright by SAP AG, 2001

    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef LOG_REDO_MANAGER_H
#define LOG_REDO_MANAGER_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "KernelCommon/Kernel_DateTime.hpp"
#include "Logging/Log_Types.hpp"


/*===========================================================================*
 *  CLASSES                                                                  *
 *===========================================================================*/

//===================================================================================
/*! class:       Log_RedoManager

    description: parallel undo/redo of transactions in order to restart the database
*/
class Log_RedoManager
{

public:
    
    //--------------------------------------------------------------Log_RedoManager---
    /*! function:    Log_RedoManager ctor
    */
    Log_RedoManager ()
    : m_redoIsAborted(false),
      m_abortReason(e_ok),
      m_transread(0),
      m_transredone(0)
    { }

    //--------------------------------------------------------------Log_RedoManager---
    /*! function:    RedoLog
        description: parallel undo/redo of transactions
        argument:    TransContext [in out]
                     restoreLog   [in]
    */
    void RedoLog (      tgg00_TransContext &TransContext,
				  const Kernel_Date         untilDate,     // PTS 1111525 mb 2001-01-15
                  const Kernel_Time         untilTime,     // PTS 1111525 mb 2001-01-15
                  const Log_IOSequenceNo    untilIOSeq,    // PTS 1121539 mb 2003-06-02
                        bool                restoreLog);

    //--------------------------------------------------------------Log_RedoManager---
    /*! function:    AbortRedo
        description: Sets the abort flag.
    */
    void AbortRedo (tgg00_BasisError abortReason);

    //--------------------------------------------------------------Log_RedoManager---
    /*! function:    GetProgressInfo
        arguments:   transread   [out]
                     transredone [out]
        description: Sets the abort flag.
    */
    void GetProgressInfo( SAPDB_UInt &transread,
                          SAPDB_UInt &transredone );

private:
    volatile bool               m_redoIsAborted; // changed by user outside
    volatile tgg00_BasisError   m_abortReason;   // changed when the redoTask is aborted
    SAPDB_UInt                  m_transread;     // changed by redo reader task
    SAPDB_UInt                  m_transredone;   // changed by redo task
};
/*! endclass: Log_RedoManager */

#endif  /* LOG_REDO_MANAGER_H */
