
/*!**************************************************************************

    module      : Log_RedoReadTask.hpp

    -------------------------------------------------------------------------

    responsible :    UweH
    
    original author: JuergenA

    special area:    Logging
    description :    defines a class to allocate the RedoReadTask singleton

    last changed:    2001-04-24

    -------------------------------------------------------------------------

    copyright:    Copyright by SAP AG, 2001

    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef LOG_REDO_READ_TASK_H
#define LOG_REDO_READ_TASK_H


/*===========================================================================*
*  INCLUDES                                                                  *
*============================================================================*/

#include "SAPDBCommon/MemoryManagement/SAPDBMem_IRawAllocator.hpp"
#include "KernelCommon/Kernel_DateTime.hpp"
#include "Logging/Log_Types.hpp"

/*===========================================================================*
*  CLASSES                                                                   *
*============================================================================*/

//=======================================================================================
/*! class:       Log_RedoAllocateReadTask

    description: a class to allocate the RedoReadTask singleton
*/
class Log_RedoAllocateReadTask
{
public:
    
    //----------------------------------------------------------------------------
    /*! function:    Log_RedoAllocateReadTask ctor
        description: constructor without any allocation of the singleton RedoReadTask
    */
    Log_RedoAllocateReadTask ()
    {}
    
    //----------------------------------------------------------------------------
    /*! function:    Log_RedoAllocateReadTask dtor
        description: deallocates the singleton RedoReadTask
    */
    ~Log_RedoAllocateReadTask ();
    
    //----------------------------------------------------------------------------
    /*! function:    Allocate
        description: allocates the singleton RedoReadTask
        arguments:   RawAllocator [in out]
                     restoreLog   [in]
                     IsOk            [out]
    */
    void Allocate (
              SAPDBMem_IRawAllocator &RawAllocator,
        const Kernel_Date             untilDate,        // PTS 1111525 mb 2001-01-15
        const Kernel_Time             untilTime,        // PTS 1111525 mb 2001-01-15
        const Log_IOSequenceNo        untilIOSeq,       // PTS 1121539 mb 2003-05-30
              bool                    restoreLog,
              volatile bool          &redoIsAborted,
              bool                   &IsOk);
};
/*! endclass: Log_RedoAllocateReadTask */

#endif  /* LOG_REDO_READ_TASK_H */
