
/*!**************************************************************************

    module      : Log_RedoTaskLifeCycle.hpp

    -------------------------------------------------------------------------

    author      : JuergenA
    responsible : UweH

    special area: Logging
    description : defines classes to start and stop redo tasks

    last changed: 2001-03-21

    -------------------------------------------------------------------------

    copyright:    Copyright by SAP AG, 2001

    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef LOG_REDO_TASK_LIFE_CYCLE_H
#define LOG_REDO_TASK_LIFE_CYCLE_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SAPDBCommon/SAPDB_Types.hpp"
#include "ggg00.h"


/*===========================================================================*
 *  CLASSES                                                                  *
 *===========================================================================*/

//===================================================================================
/*! class:       Log_RedoExecuteTaskLifeCycle

    description: starting and stopping of redo execute tasks
*/
class Log_RedoExecuteTaskLifeCycle
{

private:

    tgg00_TransContext &m_TransContext;
    SAPDB_Int           m_RunningTasks;

public:
    
    //--------------------------------------------------------------Log_RedoExecuteTaskLifeCycle---
    /*! function:    Log_RedoExecuteTaskLifeCycle ctor

        description: starts all redo execute tasks

                     If an error occurs the basis error of the TransContext is set.
    */
    Log_RedoExecuteTaskLifeCycle (tgg00_TransContext &TransContext);

    //--------------------------------------------------------------Log_RedoExecuteTaskLifeCycle---
    /*! function:    Log_RedoExecuteTaskLifeCycle dtor

        description: stops all redo execute tasks

                     If an error occurs the basis error of the TransContext is set.
    */
    ~Log_RedoExecuteTaskLifeCycle ();
};
/*! endclass: Log_RedoExecuteTaskLifeCycle */


//===================================================================================
/*! class:       Log_RedoReadTaskLifeCycle

    description: starting and stopping of the log reader task
*/
class Log_RedoReadTaskLifeCycle
{

private:

    tgg00_TransContext &m_TransContext;
    SAPDB_Int           m_RunningTasks;

public:
    
    //--------------------------------------------------------------Log_RedoReadTaskLifeCycle---
    /*! function:    Log_RedoReadTaskLifeCycle ctor

        description: starts the log reader task

                     If an error occurs the basis error of the TransContext is set.
    */
    Log_RedoReadTaskLifeCycle (tgg00_TransContext &TransContext);

    //--------------------------------------------------------------Log_RedoReadTaskLifeCycle---
    /*! function:    Log_RedoReadTaskLifeCycle dtor

        description: stops the log reader task

                     If an error occurs the basis error of the TransContext is set.
    */
    ~Log_RedoReadTaskLifeCycle ();
};
/*! endclass: Log_RedoReadTaskLifeCycle */

#endif  /* LOG_REDO_TASK_LIFE_CYCLE_H */
