
/*!**************************************************************************

    module      : Log_RedoTrafficControl.hpp

    -------------------------------------------------------------------------

    author      : JuergenA,UweH
    responsible : UweH

    special area: Logging
    description : defines a class to allocate the RedoTrafficControl singleton

    last changed: 2001-03-22

    -------------------------------------------------------------------------

    copyright:    Copyright by SAP AG, 2001

    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef LOG_REDO_TRAFFIC_CONTROL_H
#define LOG_REDO_TRAFFIC_CONTROL_H


/*===========================================================================*
*  INCLUDES                                                                  *
*============================================================================*/

#include "SAPDBCommon/MemoryManagement/SAPDBMem_IRawAllocator.hpp"

/*===========================================================================*
*  CLASSES                                                                   *
*============================================================================*/

//=======================================================================================
/*! class:       Log_RedoAllocateTrafficControl

    description: a class to allocate the RedoTrafficControl singleton
*/
class Log_RedoAllocateTrafficControl
{
public:
    
    //----------------------------------------------------------------------------
    /*! function:    Log_RedoAllocateTrafficControl ctor
    
        description: constructor without any allocation of the singleton RedoTrafficControl
    */
    Log_RedoAllocateTrafficControl ()
    { }
    
    //----------------------------------------------------------------------------
    /*! function:    Log_RedoAllocateTrafficControl dtor
    
        description: deallocates the singleton RedoTrafficControl
    */
    ~Log_RedoAllocateTrafficControl ();
    
    //----------------------------------------------------------------------------
    /*! function:    Allocate
    
        description: allocates the singleton RedoTrafficControl
      
        arguments:   RawAllocator [in out]
        IsOk         [out]
    */
    void Allocate (
        SAPDBMem_IRawAllocator &RawAllocator,
        volatile bool          &redoIsAborted,
        SAPDB_UInt             &inserted,
        SAPDB_UInt             &removed,
        bool                   &IsOk);
};
/*! endclass: Log_RedoAllocateTrafficControl */

#endif  /* LOG_REDO_TRAFFIC_CONTROL_H */
