/*!**************************************************************************

  module      : Log_SeqTapeReader.hpp
  special area: Logging
  responsible : MartinB
  last changed: 2001-07-24  12:00
  copyright:    Copyright by SAP AG, 2000
  description : defines class Log_SeqTapeReader



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef Log_SeqTapeReader_H
#define Log_SeqTapeReader_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "Logging/Log_ISequentialReader.hpp"
#include "hkb39.h"
#include "hgg01_3.h"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/

/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/

/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*! --------------------------------------------------------------------------------
class:       Log_SeqTapeReader
description: Impelements a reader for reading from tape. Used by the ReDoReader.
*/

class Log_SeqTapeReader : public Log_ISequentialReader
{
public:
    /*!-------------------------------------------------------------------------
    function:     Log_SeqTapeReader 
    description:  constructor 
    arguments:    none
    return value: none
    --------------------------------------------------------------------------*/
    Log_SeqTapeReader(Log_IOSequenceNo ioSeq,
                      tgg00_TransContext         &readerTrans)
    : m_currentIOSeq(ioSeq),
      m_readerTrans(readerTrans)
    {}
     
    /*!-------------------------------------------------------------------------
    function:     SetIOSeq 
    description:  sets the actual actual read position according to the given IOSequence
    arguments:    ioSeq          ioSeq, which shall be read next 
    return value: ReaderStatus
    --------------------------------------------------------------------------*/
    virtual ReaderStatus  SetIOSeq(   Log_IOSequenceNo         ioSeq)
    {
        m_currentIOSeq = ioSeq;
        if (ioSeq.IsValid())
        {
            return reader_ok;
        }
        else
        {
            return invalid_IOSeq;
        }
    }
protected:

    /*!-------------------------------------------------------------------------
    function:     ReadNextPage 
    description:  reads the actual page and then increments the actual position 
    arguments:    taskId        identification of the running task
                  logPage       location where the read page will be stored
    return value: ReaderStatus
    --------------------------------------------------------------------------*/
    virtual ReaderStatus  ReadNextPage(  tsp00_TaskId             taskId,
                                         Log_Page                &logPage)
    {
        if (m_currentIOSeq.IsValid())
        {
            tkb00_PagePtr framePtr = NULL;
            tsp00_MoveObjPtr   dummyLogbody = NULL;
            tsp00_Int4         dummyLogbody_len = 0;

            k39locate_logentries_in_page(   m_readerTrans,
                                            m_currentIOSeq,
                                            framePtr,
                                            dummyLogbody,
                                            dummyLogbody_len);
            if (e_ok == m_readerTrans.trError_gg00)
            {
                Log_Page::PageFrame frame(framePtr,g01page_size);
                logPage.SetFrame(frame);
                return reader_ok;
            }
            else if (e_end_of_backup == m_readerTrans.trError_gg00
                     ||
                     // PTS1111525 mb 2002-02-19 write_task_crash is set when the restore is cancelled / ignored 
                     e_write_task_crash == m_readerTrans.trError_gg00) 
            {
                m_readerTrans.trError_gg00 = e_ok;
                return end_of_file;
            }
        }
        else
        {
            return invalid_IOSeq;
        }

        // this statement should never be reached
        return not_implemented;
    }
    
    
private:
    /*--------------------------------------------------------------------------
    declaration : m_currentIOSeq
    description:  next IOSequence to be read
    --------------------------------------------------------------------------*/
    Log_IOSequenceNo             m_currentIOSeq;
    
    /*--------------------------------------------------------------------------
    declaration : m_readerTrans
    description:  transaction of the redo
    --------------------------------------------------------------------------*/
    tgg00_TransContext         &m_readerTrans;


};
/*! endclass: Log_SeqTapeReader */

#endif  /* Log_SeqTapeReader_H */
