/*!================================================================
 module:        OMS_DummyKernelCallback.hpp

 responsible:   ThomasA
 
 special area:  OMS
 
 description:   Dummy kernel sink
 
 see:
 
 copyright:     SAP AG 1998 


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License; or (at your option) any later version.

    This library is distributed in the hope that it will be useful;
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not; write to the Free Software
    Foundation; Inc.; 59 Temple Place; Suite 330; Boston; MA  02111-1307  USA
    ========== licence end

 ===================================================================*/



#ifndef OMS_DUMMYKERNELCALLBACK_HPP
#define OMS_DUMMYKERNELCALLBACK_HPP

#include "gco001.h"
#include "ggg00.h"

/*! 
  Class: OMS_DummyKernelCallback

  Description: this class yields a dummy implementation of the kernel sink.
               All interface functions simply return ok.
*/

class OMS_DummyKernelCallback : public IliveCacheSink 
{
    virtual HRESULT STDMETHODCALLTYPE QueryInterface( 
    /* [in] */ REFIID riid,
    /* [iid_is][out] */ void  **ppvObject) { return S_OK; }
    virtual ULONG STDMETHODCALLTYPE AddRef( void) { return S_OK; }
    virtual ULONG STDMETHODCALLTYPE Release( void) { return S_OK; }

    virtual HRESULT STDMETHODCALLTYPE ABAPRead( 
            /* [out][in] */ OmsTypeInt4   &tabHandle,
            /* [out]     */ OmsTypeInt4   &count,
            /* [out]     */ OmsTypeInt4   &bufSize,
            /* [out]     */ OmsTypeByte*  &buf,
            /* [out]     */ short         &error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE ABAPWrite( 
    /* [in] */ OmsTypeInt4 *TabHandle,
    /* [in] */ tsp00_Int4 row_size,
    /* [in] */ tsp00_Int4 row_count,
    /* [in] */ unsigned char  *lpv,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE AllocPacket( 
    /* [out] */ unsigned char  * *p,
    /* [out] */ tsp00_Int4  *sz) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE BasisError( 
    /* [in] */ short b_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE CloseVersion( 
    /* [in] */ OmsVersionId  *lpVersionId,
    /* [in] */ tsp00_Int4 lastKey,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE ConsoleMsg( 
    /* [in] */ tsp00_Int4 buflen,
    /* [size_is][in] */ unsigned char  *lpv) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE CreateOpenVersion( 
    /* [in] */ unsigned char  *lpConsistentView,
    /* [in] */ OmsVersionId  *lpVersionId,
    /* [in] */ boolean createVersion,
    /* [out] */ unsigned char  *lpVersionContext,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE DeleteAllObj(
    /* [out][in] */ tgg91_TransNo&  consistentView,
    /* [in]      */ unsigned char  *lpContainerId,
    /* [in]      */ unsigned char  *lpVersionFile,
    /* [out]     */ int            &lpDeleted,
    /* [out]     */ short          &lpb_error,
    /* [out] */     OmsTypeOid     &errorOid) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE DeleteObj( 
    /* [in] */ unsigned char  *lpConsistentView,
    /* [in] */ unsigned char  *lpContainerId,
    /* [in] */ unsigned char  *lpVersionContext,
    /* [size_is][in] */ unsigned char  *lpoid,
    /* [size_is][in] */ unsigned char  *lpobjseq,
    /* [in] */ unsigned char  *lpcoldesc,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE EndConsistentView( 
    /* [out][in] */ unsigned char  *lpConsistentView,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE EnterCriticalSection(/*in */ short regionId) { return S_OK; };
  
  virtual HRESULT STDMETHODCALLTYPE GetErrorMsg( 
    /* [in] */ short errorno,
    /* [out][in] */ tsp00_Int4  *lpbuflen,
    /* [size_is][out][in] */ unsigned char  *lpv) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE GetObj(
    /* [out][in] */ tgg91_TransNo&                 consistentView,
    /* [out][in] */ tgg00_ContainerId&             containerId, 
    /*      [in] */ const tgg00_OmsVersionContext* pVersionContext,
    /*      [in] */ const OmsTypeOid&              oid,
    /*      [in] */ bool                           doLock,
    /* [out]     */ tgg91_PageRef&                 objSeq,
    /*      [in] */ tsp00_Int4                     bodySize,
    /* [out]     */ unsigned char*                 pBody,
    /* [out]     */ tsp00_Int4&                    bodyLen,
    /* [out]     */ tsp00_Int4&                    histLogCount,
    /* [out]     */ tgg00_BasisError&              e) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE GetObjWithKey( 
    /* [out][in] */ unsigned char  *lpConsistentView,
    /* [in] */ unsigned char  *lptree,
    /* [in] */ unsigned char  *lpVersionContext,
    /* [in] */ tsp00_Int4 keylen,
    /* [in] */ unsigned char  *lpkey,
    /* [in] */ boolean doLock,
    /* [in] */ tsp00_Int4 bodysize,
    /* [size_is][out] */ unsigned char  *lpbody,
    /* [out] */ OmsTypeOid  *lpoid,
    /* [size_is][out] */ unsigned char  *lpobjseq,
    /* [out] */ int  *hist_log_count,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE HResultError( 
    /* [in] */ HRESULT h_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE IsDebugEnvironment( void) { return S_OK; };
  
    virtual HRESULT STDMETHODCALLTYPE IsLocked( 
    /* [in] */ unsigned char  *lpContainerId,
    /* [in] */ OmsTypeOid  *lpoid,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE LeaveCriticalSection(/*in */ short regionId) { return S_OK; };
  
    virtual HRESULT STDMETHODCALLTYPE LockObj( 
    /* [in] */ unsigned char  *lpConsistentView,
    /* [in] */ unsigned char  *lpContainerId,
    /* [in] */ OmsTypeOid  *lpoid,
    /* [size_is][in] */ unsigned char  *lpobjseq,
    /* [out] */ short  *lpb_error) { return S_OK; };

    virtual HRESULT STDMETHODCALLTYPE MultiPurpose( 
    /* [in] */ unsigned char MessType,
    /* [in] */ unsigned char MessType2,
    /* [out][in] */ tsp00_Int4  *lpbuflen,
    /* [out][in] */ unsigned char  *lpv,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE NewConsistentView( 
    /* [out][in] */ unsigned char  *lpConsistentView,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE NewObj( 
    /* [in] */ unsigned char  *lpConsistentView,
    /* [in] */ unsigned char  *lpContainerId,
    /* [in] */ unsigned char  *lpVersionContext,
    /* [in] */ tsp00_Int4 KeyPos,
    /* [in] */ tsp00_Int4 KeyColSize,
    /* [in] */ unsigned char  *KeyColumn,
    /* [out] */ OmsTypeOid  *lpoid,
    /* [size_is][out] */ unsigned char  *lpobjseq,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE NextObj( 
    /* [out][in] */ unsigned char  *lpConsistentView,
    /* [in] */ unsigned char  *lpContainerId,
    /* [in] */ unsigned char  *lpVersionContext,
    /* [out][in] */ OmsTypeOid  *lpoid,
    /* [size_is][out] */ unsigned char  *lpobjseq,
    /* [in] */ tsp00_Int4 bodysize,
    /* [size_is][out] */ unsigned char  *lpbody,
    /* [out] */ tsp00_Int4& objHistReadCount, // PTS 1107819 TA 13/09/2000
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE RegisterClass( 
    /* [in] */ tsp00_Int4          SchemaId,
    /* [in] */ GUID                *lpGUID,
    /* [in] */ tsp00_KnlIdentifier *lpClassName,
    /* [in] */ tsp00_Int4          size,
    /* [in] */ OmsContainerNo      ContainerNo,
    /* [out][in] */ unsigned char  *lpcoldesc,
    /* [out] */ unsigned char      *lpContainerId,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE ResetVersion( 
             /* [in ] */ unsigned char *lpVersionContext,
             /* [out] */ short  &error) { return S_OK; };
 
    virtual HRESULT STDMETHODCALLTYPE Rollback( 
    /* [in] */ tsp00_Int4 buflen,
    /* [size_is][in] */ unsigned char  *lpv,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE SetError( 
    /* [in] */ short errorno,
    /* [in] */ tsp00_Int4 buflen,
    /* [size_is][in] */ const unsigned char  *lpv,
    /* [in] */ tsp00_CodeType codeType) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE SQLReceive( 
    /* [out] */ unsigned char  * *lpv) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE SQLRequest( 
    /* [in] */ tsp00_Int4 buflen,
    /* [size_is][in] */ unsigned char  *lpv,
    /* [out] */ short  *lp_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE SubtransCommit( 
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE SubtransRollback( 
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE SubtransStart( 
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE UnlockObj( 
    /* [in] */ unsigned char  *lpContainerId,
    /* [in] */ OmsTypeOid  *lpoid,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE UpdateObj( 
    /* [in] */ unsigned char  *lpConsistentView,
    /* [in] */ unsigned char  *lpContainerId,
    /* [in] */ unsigned char  *lpVersionContext,
    /* [size_is][in] */ unsigned char  *lpoid,
    /* [size_is][out][in] */ unsigned char  *lpobjseq,
    /* [in] */ tsp00_Int4 bodysize,
    /* [size_is][in] */ unsigned char  *lpbody,
    /* [in] */ unsigned char  *lpcoldesc,
    /* [out] */ short  *lpb_error) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE Vtrace( 
    /* [in] */ tsp00_Int4 buflen,
    /* [size_is][in] */ unsigned char  *lpv) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE GetDefaultContext(/* [out] */ void** p,
       pasbool** pCancelByte,
       tsp00_TaskId& TaskId) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE SetDefaultContext(/* [in] */ void* p) { return S_OK; };

    /* new method since PTS 1120804 */
    virtual HRESULT STDMETHODCALLTYPE GetSqlSessionContext(
    /* [out] */ void** p) { return S_OK; };

    /* new method since PTS 1120804 */
    virtual HRESULT STDMETHODCALLTYPE SetSqlSessionContext(
    /* [in] */ void* p) { return S_OK; };
    
    virtual HRESULT STDMETHODCALLTYPE OpenFile(/* [in] */ char  *lpFileName,
    /* [in]  */ tsp00_Uint4 FileSize,
    /* [in]  */ boolean     doAppend,
    /* [out] */ tsp00_Int4  *FileHandle,
    /* [out] */ short  *lpb_error) { return S_OK; }
    virtual HRESULT STDMETHODCALLTYPE WriteFile(/* [in] */ tsp00_Int4 FileHandle,
    /* [in]  */ char   *msg,
    /* [out] */ short  *lpb_error) { return S_OK; } 
    virtual HRESULT STDMETHODCALLTYPE CloseFile(/* [in] */ tsp00_Int4 FileHandle,
    /* [out] */ short  *lpb_error) { return S_OK; }
    virtual HRESULT STDMETHODCALLTYPE SyncFile(/* [in] */ char  *lpFileName,
    /* [in] */ tsp00_Int4 FileHandle,
    /* [out] */ short  *lpb_error) { return S_OK; } 
    virtual HRESULT STDMETHODCALLTYPE GetDateTime(/* [out] */ tsp00_Date *Date,
    /* [out] */ tsp00_Time *Time) { return S_OK; } 
    virtual HRESULT STDMETHODCALLTYPE GetClock(/* [out] */ tsp00_Int4 *Sec,
    /* [out] */ tsp00_Int4 *MSec) { return S_OK; } 
    virtual HRESULT STDMETHODCALLTYPE DropVersion( /* in */ unsigned char  *lpVersionContext,
    /* [out] */ short  *lpb_error) { return S_OK; }
    virtual HRESULT STDMETHODCALLTYPE CreateSchema(/* [in] */ const OmsTypeWyde *SchemaName,
    /* [out] */ tsp00_Int4           *SchemaId,
    /* [out] */ short                *lpb_error) { return S_OK; }
    virtual HRESULT STDMETHODCALLTYPE ExistsSchema(/* [in] */const OmsTypeWyde *lpSchemaName,
    /* [out] */ tsp00_Int4           *SchemaId,
    /* [out] */ short                *lpb_error) { return S_OK; }
    virtual HRESULT STDMETHODCALLTYPE DropSchema(/* [in] */ tsp00_Int4 SchemaId,
            /* [out] */ short      *lpb_error) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE GetSchemaName(
            /* [in]  */ tsp00_Int4   SchemaId,
            /* [out] */ OmsTypeWyde *lpSchemaName,
            /* [out] */ short       *lpb_error) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE ExistsFile(/* [in] */ char  *lpFileName,
          /* [out] */ bool     &existsFile,
          /* [out] */ bool     &isDirectory,
          /* [out] */ short    &error)  { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE DropFile(/* [in] */ char  *lpFileName,
          /* [out] */ short  &error)  { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE GetWorkingDirectory(/* [out] */ tsp00_Pathc& pathName,
          /* [out] */ short  &error) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE GetDbrootSapPath(/* [out] */ tsp00_Pathc& pathName,
          /* [out] */ short  &error) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE LoadLibraryEx( /* in */ const char* path, 
          /* in  */ bool        loadGlobal,
          /* out */ HANDLE      *hd,
          /* out */ tsp_errtext &errtext,
          /* in  */ int         errtext_size) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE FreeLibrary( /* in */ HANDLE hd,
          /* out */ bool        &ok,
          /* out */ tsp_errtext &errtext,
          /* in  */ int         errtext_size) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE DropContainer(/* [in] */ OmsSchemaHandle schema,
          /* [in] */ const GUID&        guid,
          /* [in] */ OmsContainerNo     cno,
          /* [out] */ tgg00_BasisError& error) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE GetContainerInfo(/* [in] */ tsp00_Uint4 containerHandle,
          /* [out] */ GUID& guid,
          /* [out] */ OmsSchemaHandle& schema,
          /* [out] */ OmsContainerNo&  cno,
          /* [out] */ short& error) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE IsObjHistoryUsed( 
            /* [out][in] */ tgg00_ContainerId& containerId, 
            /*      [in] */ const OmsTypeOid&  oid,
            /* [out]     */ bool&              isUsed,
            /* [out]     */ tgg00_BasisError&  e) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE Reschedule() { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE LockUpdObj( 
    /* [in] */ unsigned char  *lpConsistentView,
    /* [in] */ unsigned char  *lpContainerId,
    /* [in] */ OmsTypeOid  *lpoid,
    /* [size_is][in] */ unsigned char  *lpobjseq,
    /* [out] */ short  *lpb_error) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE Signal(tsp00_TaskId TaskId, tsp00_Bool ok) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE Wait(/*[out]*/ tsp00_Bool& ok) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE ExistsContainer(
    /* [in]  */ tsp00_Int4     SchemaId,
    /* [in]  */ const GUID&    Guid,
    /* [in]  */ OmsContainerNo ContainerNo,
    /* [out] */ short&         error) { return S_OK; };
    virtual HRESULT STDMETHODCALLTYPE CreateKeyRangeIterator(
           tgg91_TransNo&          consistentView,
           tgg00_OmsVersionContext *lpVersionContext,
           tgg00_FileId&           objFile,
           tsp00_Int4              keyLen,
           const void*             pStartKey,
           const void*             pStopKey,
           void**                  pKeyRangeIterator,
           short&                  e) { return S_OK; }
     virtual HRESULT STDMETHODCALLTYPE DestroyKeyRangeIterator(
     /* [in] */ void* pKeyRangeIterator) { return S_OK; };
     virtual HRESULT STDMETHODCALLTYPE NextObjFromKeyIterator (
            void                    *pKeyIterator,    
            tgg00_OmsVersionContext *VersionContext,
            tgg00_ContainerId&       ObjFile,
            tsp00_Int4               ObjBodySize,
            void                    *pObjBody,
            OmsTypeOid              &ResultOId, 
            tgg91_PageRef           &ResultObjVers,
            tsp00_Int4              &LogReadAccesses,
            tsp00_Int2              &e) { return S_OK; }
     virtual HRESULT STDMETHODCALLTYPE VersionNextKey(
            tgg00_OmsVersionContext* VersionContext,
            const tgg00_ContainerId& ObjFile,
            void*                    pKey,
            tsp00_Int4               KeyLen,
            bool                     Inclusive,
            const void*              pStopKey,
            OmsTypeOid&              Oid,
            tgg00_BasisError&        e) { return S_OK; }
     virtual HRESULT STDMETHODCALLTYPE GetSessionHeapUsage(
            /* [out] */ tsp00_8ByteCounter& heapUsage) { return S_OK; }  

	   virtual HRESULT STDMETHODCALLTYPE EvalCallStatistics(
            bool            AllocatorOnly,
            OmsMonitorData& MonitorData) { return S_OK; }

	virtual HRESULT STDMETHODCALLTYPE SimCtlDumpToHtml(
			class OmsHandle				&handle,
			const struct OmsObjectId	&obj, 
			class OmsSimDumpCollector	&str)
	{ return S_OK; } 

	virtual HRESULT STDMETHODCALLTYPE SimCtlSetObjectName(
			class OmsHandle				&handle,
			const struct OmsObjectId	&obj,
			const char					*name)
	{ return S_OK; }

	virtual HRESULT STDMETHODCALLTYPE SimCtlGetObjectName(
			class OmsHandle				&handle,
			const struct OmsObjectId	&obj,
			const char					**name)
	{
		*name = NULL;
		return S_OK;
	}

	virtual HRESULT STDMETHODCALLTYPE SimCtlDestroyHandleCB(
			class OmsHandle				&handle,
			struct IliveCacheSink		**session)
	{
		return S_OK;
	}

    virtual HRESULT STDMETHODCALLTYPE GetSessionAllocator(SAPDBMem_RawAllocator* &allocator) 
    {
        return S_OK;
    }

    virtual HRESULT STDMETHODCALLTYPE GetOmsSessionAllocator(SAPDBMem_RawAllocator* &allocator)
    {
        return S_OK;
    }

    virtual HRESULT STDMETHODCALLTYPE SimCtlGetHTMLRef(
			class OmsHandle				&handle,
			const struct OmsObjectId	&obj,
			const char					**data,
			int							flags)
    {
        *data = "";
        return S_OK;
    }

    virtual HRESULT STDMETHODCALLTYPE IsInRegion(
            /* [in]  */ int   regionId,
            /* [out] */ bool& inRegion)
    {
        inRegion = false;
        return S_OK;
    }

    virtual HRESULT STDMETHODCALLTYPE GetSqlClsAllocator(SAPDBMem_RawAllocator* &allocator) 
    {
        return S_OK;
    }

    virtual HRESULT STDMETHODCALLTYPE GetGlobalUserAllocator(
        OmsAllocatorInterface* &allocator, const char *name)
    {
        allocator = NULL;
        return S_OK;
    } 

    virtual HRESULT STDMETHODCALLTYPE GetSessionUserAllocator(
        OmsAllocatorInterface* &allocator)
    {
        allocator = NULL;
        return S_OK;
    }

    virtual HRESULT STDMETHODCALLTYPE GetTransactionUserAllocator(
        OmsAllocatorInterface* &allocator)
    {
        allocator = NULL;
        return S_OK;
    }

    virtual HRESULT STDMETHODCALLTYPE GetCOMRoutineUserAllocator(
        OmsAllocatorInterface* &allocator)
    {
        allocator = NULL;
        return S_OK;
    }

    virtual HRESULT STDMETHODCALLTYPE UserAllocatorCtl(
        int msg, void *param)
    {
        return S_OK;
    }
     
    virtual HRESULT STDMETHODCALLTYPE IsUnicodeInstance(
         bool& isUnicodeInstance,
         short& sqlError)
    {
        return S_OK;
    }
};

/*! endclass: OMS_DummyKernelCallback */


#endif
