/*!**************************************************************************

  module      : Pager_IDump.hpp

  -------------------------------------------------------------------------

  responsible : Henrik

  special area: pager
  description : abstract dump interface class to the pager 


  last changed: 2002-03-11
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/


#ifndef PAGER_IDUMP_HPP
#define PAGER_IDUMP_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gsp00.h"

#include "KernelCommon/Kernel_Common.hpp"
#include "KernelCommon/Kernel_Dump.hpp"


 /*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!*****************************************************************************

   class: Pager_IDump

*******************************************************************************/

class Pager_IDump
{
public:

    /*--------------------------------------------------------------------------
    function:     GetInstance
    description:  This method returns the access to the Pager_IDump interface
                  of component Pager. 
    arguments:    none
    return value: Access to the Pager_IDump interface
    --------------------------------------------------------------------------*/

    static Pager_IDump& GetInstance();

    /*!-------------------------------------------------------------------------
    function:     Dump()
    description:  This method is used to dump the main memory structures of the
                  component Pager.
    arguments:    TaskId [in]     identification of the calling task
                  Dump   [in/out] kernel dump file
    return value: none
    --------------------------------------------------------------------------*/
    
    virtual void Dump(
        const tsp00_TaskId    TaskId,
        Kernel_Dump     &Dump ) = 0;
};

/*!*****************************************************************************

   endclass: Pager_IDump

*******************************************************************************/


#endif  /* PAGER_IDUMP_HPP */
