/****************************************************************************

  module      : Pager_Writer.cpp

  -------------------------------------------------------------------------

  responsible : Henrik

  special area:
  description : This description is currently not used, as
    documentation is only generated from header files
-
  last changed: 2002-03-07
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "Pager/Pager_Writer.hpp"
#include "Pager/Pager_Types.hpp"
#include "Pager/Pager_Controller.hpp"
#include "Pager/Pager_Exception.hpp"
#include "Pager/Pager_Messages.hpp"
#include "RunTime/RTE_Message.hpp"
#include "IOManager/IOMan_IDataIO.hpp"
#include "FreeBlockManagement/FBM_Manager.hpp"
#include "Converter/Converter_IPageIO.hpp"


#include "heo56.h"          // vsuspend
#include "hkb28.h"          // k28emergency_shutdown 
#include "hbd20_5.h"        // bd20*
#include "hgg04.h"          // gg04isStaticPage

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/


Pager_Writer::Pager_Writer(tgg00_TransContext &t)
        : m_reqFlush(RTEMem_Allocator::Instance()),
        m_trans(t),
        m_err(t.trError_gg00),
        m_pagerController(Pager_Controller::GetInstance()),
        m_blockCount(g01io_block_count()),
        m_pages(*(REINTERPRET_CAST( SAPDBMem_IRawAllocator*,                // PTS 1117889
        t.trAllocator_gg00)) , UTF8( "Pager:WriteDataPages" ))
{ 
};




/*---------------------------------------------------------------------------*/


void Pager_Writer::Initialize(){
    SAPDBTRACE_ROUTINE_DEBUG("Pager_Writer::Initialize", Pager_Trace, 5 );

    const SAPDB_Int numPagerWriter = g01maxdatawriter();
    const SAPDB_Int numCaches      = g01region_cnt (rgnData_egg00);

    m_pages.Initialize(m_blockCount);

    m_reqFlush.Initialize(numCaches,false);

    CleanupActiveCache();

    m_myIndex = m_pagerController.RegisterNextPagerWriter(m_trans.trTaskId_gg00, this);

    PagerWriterIterator current = Begin();
    const PagerWriterIterator& iterEnd = End();

    Pager_Exception StartTaskID(__CONTEXT__, PAGER_WRITER_START_TASKID,
                                SAPDB_ToString(m_trans.trTaskId_gg00, _T_d));
    RTE_Message (StartTaskID);
    Pager_Exception FirstCache(__CONTEXT__, PAGER_WRITER_FIRST_CACHE,
                               SAPDB_ToString(current.value(), _T_d));
    RTE_Message (FirstCache);

    current++;

    while (current != iterEnd) {
        Pager_Exception NextCache(__CONTEXT__, PAGER_WRITER_NEXT_CACHE,
                                  SAPDB_ToString(current.value(), _T_d));
        RTE_Message (NextCache);

        current++;
    }
    Run();
}

/*---------------------------------------------------------------------------*/

void Pager_Writer::CleanupActiveCache(){
    SAPDBTRACE_ROUTINE_DEBUG("Pager_Writer::CleanupActiveCache", Pager_Trace, 5 );

    const SAPDB_Int numCaches = g01region_cnt (rgnData_egg00);

    m_reqFlush.Initialize(numCaches,false);
}

/*---------------------------------------------------------------------------*/

void Pager_Writer::ReqFlushForDataCache(SAPDB_Int regOffset){

    SAPDBTRACE_ROUTINE_DEBUG("Pager_Writer::ReqFlushForDataCache", Pager_Trace, 5 );

    m_reqFlush[regOffset] = true;
}

/*---------------------------------------------------------------------------*/

tsp00_TaskId Pager_Writer::GetTaskId() const{

    SAPDBTRACE_ROUTINE_DEBUG("Pager_Writer::GetTaskId", Pager_Trace, 5 );

    return m_trans.trTaskId_gg00;
}

/*---------------------------------------------------------------------------*/

void Pager_Writer::Run() {

    SAPDBTRACE_ROUTINE_DEBUG("Pager_Writer::Run", Pager_Trace, 5 );

    SAPDB_Bool found;

    while(true) {
        if ((e_ok != m_err ) &&
                (e_shutdown != m_err)) {        // PTS 1107037 2000-06-26
            k28emergency_shutdown (m_trans, m_savepoint);
            break;
        }

        // scan for preDisplace Requests ...
        PagerWriterIterator iter = Begin();
        const PagerWriterIterator& iterEnd = End();
        do {
            found = m_reqFlush[iter.value()];
            if  (!found){
                iter++;
            }
        } while ((iter != iterEnd) && !found);

        if ((! found)  ||
                (m_pagerController.IsPreparePagerWriter()) ||
                (e_shutdown == m_err)) {          // PTS 1107037 TS 2000-06-26
            m_err  = e_ok; // PTS 1107037 TS 2000-06-26 *)

            m_pagerController.SuspendPagerWriter(m_myIndex, m_trans.trTaskId_gg00);  // sleep

        }
        switch(m_pagerController.GetPagerOrder()) {
        case poFlushDataCacheBeforePrepare :
            FlushDataCacheBeforePrepare();
            break;
        case poFlushDataCacheAfterPrepare :
            FlushDataCacheAfterPrepare();
            break;
        case poReadConverterCache:
            ReadConverter();
            break;
        case poFlushConverterCache:
            FlushConverter();
            break;
        default:
            PreDisplaceIO ();
            break;
        }
    }
}

/*---------------------------------------------------------------------------*/

void Pager_Writer::PreDisplaceIO() {

    SAPDBTRACE_ROUTINE_DEBUG("Pager_Writer::PreDisplaceIO", Pager_Trace, 5 );

    SAPDB_Bool              found;
    SAPDB_Bool1             useIOChain;
    SAPDB_Int2              curPageCnt;
    SAPDB_Int               pageCnt;
    tbd00_NodePtrList       nodePtrList;
    tbd2_data_cb_flush_list cbptr_list;

    IOMan_BlockAddress startBlock;
    IOMan_ClusterAddress cluster;

    found      = false;
    useIOChain = SAPDB_FALSE;


    PagerWriterIterator iter = Begin();
    const PagerWriterIterator& iterEnd = End();

    do {
        found = m_reqFlush[iter.value()];
        if  (!found){
            iter++;
        }
    } while ((iter != iterEnd) && !found);


    while ((e_ok == m_err) &&
            (iter != iterEnd) &&
            !m_pagerController.IsPreparePagerWriter()){

        pageCnt = m_blockCount;
        cluster = FBM_Manager::Instance().GetMultFreeBlocks(
                      m_trans.trTaskId_gg00, pageCnt, m_err);

        startBlock = cluster.GetBlockAddress(0);
        pageCnt    = cluster.GetBlockCount();

        if (e_ok == m_err){
            curPageCnt = pageCnt;
            bd20PreDisplaceIO (m_trans.trTaskId_gg00, iter.value(), m_err, useIOChain,
                               curPageCnt, nodePtrList, cbptr_list);
        }

        if (e_ok == m_err) {
            if (curPageCnt > 0) {
                cluster.SetCluster(startBlock, curPageCnt);
                WriteDataPages (m_trans, nodePtrList, cluster);
                if (e_ok == m_err){
                    bd20ResetIOState(m_trans.trTaskId_gg00, iter.value(), curPageCnt, cbptr_list);
                    pageCnt  -= curPageCnt;
                    startBlock.SetBlockNo(startBlock.GetBlockNo() + curPageCnt);
                    cluster.SetCluster(startBlock, pageCnt);
                }
            }
            if  ((pageCnt > 0) && (e_ok == m_err)) {
                useIOChain = SAPDB_FALSE;

                m_reqFlush[iter.value()] = false;

                do {
                    found = m_reqFlush[iter.value()];
                    if  (!found){
                        iter++;
                    }
                } while ((iter != iterEnd) && !found);

                FBM_Manager::Instance().SetClusterStateToFree(m_trans.trTaskId_gg00, cluster);

            }
        }
    }
}

/*---------------------------------------------------------------------------*/

void Pager_Writer::FlushDataCacheBeforePrepare () {

    SAPDBTRACE_ROUTINE_DEBUG("Pager_Writer::FlushDataCacheBeforePrepare", Pager_Trace, 5 );

    SAPDB_Bool1             dummy;
    IOMan_BlockCount        pageCnt;
    SAPDB_Int2              curPageCnt;
    SAPDB_Int               totalIO;
    SAPDB_Int               totalPages;
    SAPDB_Int               totalIOElems;
    tbd00_NodePtrList       nodePtrList;
    tbd2_data_cb_flush_list cbptrList;

    PagerWriterIterator current = Begin();
    const PagerWriterIterator& iterEnd = End();

    IOMan_BlockAddress  startBlock;
    IOMan_ClusterAddress cluster;

    m_err      = e_ok;
    totalPages = 0;
    totalIO    = 0;

    do {
        totalIOElems = bd20IOElemCount (m_trans.trTaskId_gg00, current.value() );
        do {
            pageCnt = m_blockCount;
            cluster = FBM_Manager::Instance().GetMultFreeBlocks(
                          m_trans.trTaskId_gg00, pageCnt, m_err);
            startBlock = cluster.GetBlockAddress(0);
            pageCnt    = cluster.GetBlockCount();

            while ((e_ok == m_err) && (pageCnt > 0) && (totalIOElems > 0)){
                curPageCnt = pageCnt;
                bd20GetPageForIO (m_trans.trTaskId_gg00, current.value(), m_err,
                                  curPageCnt, nodePtrList, cbptrList, dummy);
                if  (0 == curPageCnt){
                    totalIOElems = 0;
                }
                else {
                    totalIOElems -= curPageCnt;
                }

                if ((e_ok == m_err) && (curPageCnt >0))
                {
                    totalIO++;
                    totalPages += curPageCnt;

                    cluster.SetCluster(startBlock, curPageCnt);
                    WriteDataPages (m_trans, nodePtrList, cluster);

                    if (e_ok == m_err){
                        bd20ResetIOState (m_trans.trTaskId_gg00, current.value(), curPageCnt, cbptrList);
                        pageCnt   -= curPageCnt;
                        startBlock.SetBlockNo(startBlock.GetBlockNo() + curPageCnt);
                        cluster.SetCluster(startBlock, pageCnt);
                    }
                }
            }
        } while ((e_ok == m_err) && (totalIOElems > 0 ));

        FBM_Manager::Instance().SetClusterStateToFree(m_trans.trTaskId_gg00, cluster);

        current++;
    } while ((e_ok == m_err) && (current != iterEnd));

    m_pagerController.AddDataIOTotal(totalIO);
    m_pagerController.AddDataPagesTotal(totalPages);
}

/*---------------------------------------------------------------------------*/

void Pager_Writer::FlushDataCacheAfterPrepare() {

    SAPDBTRACE_ROUTINE_DEBUG("Pager_Writer::FlushDataCacheAfterPrepare", Pager_Trace, 5 );

    SAPDB_Bool1             svpCompleted;
    SAPDB_Bool              stopLoop;
    IOMan_BlockCount        pageCnt;
    SAPDB_Int2              curPageCnt;
    SAPDB_Int               loopCnt;
    SAPDB_Int               totalIO;
    SAPDB_Int               totalPages;
    tbd00_NodePtrList       nodePtrList;
    tbd2_data_cb_flush_list cbptrList;

    PagerWriterIterator current = Begin();
    const PagerWriterIterator& iterEnd = End();

    IOMan_BlockAddress  startBlock;
    IOMan_ClusterAddress cluster;

    m_err      = e_ok;
    totalPages = 0;
    totalIO    = 0;

    do {
        svpCompleted = SAPDB_FALSE;
        loopCnt      = 0;
        while ((e_ok == m_err) && !svpCompleted){
            pageCnt = m_blockCount;
            IOMan_ClusterAddress cluster = FBM_Manager::Instance().GetMultFreeBlocks(
                                               m_trans.trTaskId_gg00, pageCnt, m_err);
            startBlock = cluster.GetBlockAddress(0);
            pageCnt   = cluster.GetBlockCount();

            stopLoop = false;
            while ((e_ok == m_err) && (pageCnt > 0) && (!stopLoop)){
                curPageCnt = pageCnt;
                bd20GetPageForIO (m_trans.trTaskId_gg00, current.value(), m_err, curPageCnt,
                                  nodePtrList, cbptrList, svpCompleted);
                if (e_ok == m_err){
                    if (curPageCnt > 0){
                        totalIO++;
                        totalPages += curPageCnt;

                        cluster.SetCluster(startBlock, curPageCnt);
                        WriteDataPages (m_trans, nodePtrList, cluster);

                        if (e_ok == m_err){
                            bd20ResetIOState (m_trans.trTaskId_gg00, current.value(), curPageCnt, cbptrList);
                            pageCnt -= curPageCnt;
                            startBlock.SetBlockNo(startBlock.GetBlockNo() + curPageCnt);
                            cluster.SetCluster(startBlock, pageCnt);
                        }
                    }
                    else {
                        stopLoop = true;
                    }
                }
            }

            FBM_Manager::Instance().SetClusterStateToFree(m_trans.trTaskId_gg00, cluster);

            if ((e_ok == m_err) && (!svpCompleted) && stopLoop) {
                if (loopCnt >= 5){
                    vsleep (m_trans.trTaskId_gg00, 1);
                    loopCnt = 0;
                }
                else {
                    loopCnt++;
                    vsleep (m_trans.trTaskId_gg00, 0);
                }
            }
        }
        current++;
    }
    while ((e_ok == m_err) && (current != iterEnd));

    m_pagerController.AddDataIOTotal(totalIO);
    m_pagerController.AddDataPagesTotal(totalPages);
}

/*---------------------------------------------------------------------------*/

void Pager_Writer::WriteDataPages(tgg00_TransContext  &Trans,
                                  tbd00_NodePtrList   &pNodes,
                                  IOMan_ClusterAddress Cluster) {

    SAPDBTRACE_ROUTINE_DEBUG("Pager_Writer::WriteDataPages", Pager_Trace, 5 );

    const IOMan_BlockCount BlockCount = Cluster.GetBlockCount();

    m_pages.Clear();

    Data_BasePage Page;
    for(SAPDB_Int Index = 0; Index < BlockCount; ++Index )
    {
        Page.Assign( pNodes[ Index ], !gg04IsStaticPage( pNodes[ Index ]->nd_pt2() ),
                     pNodes[ Index ]->nd_pmode().includes( pmTemp_egg00 ));

        m_pages.Add( Page );
    }
    IOMan_IDataIO::GetInstance().WriteDataPages( Trans.trTaskId_gg00,
            Trans.trError_gg00, m_pages, Cluster );

};

/*---------------------------------------------------------------------------*/

void Pager_Writer::FlushConverter()
{
    SAPDBTRACE_ROUTINE_DEBUG("Pager_Writer::FlushConverter", Pager_Trace, 5 );

    SAPDB_UInt  totalIO    = 0;
    SAPDB_UInt  totalPages = 0;

    Converter_IPageIO::Instance().WritePages( m_trans.trTaskId_gg00, m_myIndex, totalPages, totalIO );

    m_pagerController.AddConverterIOTotal( totalIO );
    m_pagerController.AddConverterPagesTotal( totalPages );


}

/*---------------------------------------------------------------------------*/

void Pager_Writer::ReadConverter()
{
    SAPDBTRACE_ROUTINE_DEBUG("Pager_Writer::ReadConverter", Pager_Trace, 5 );

    SAPDB_UInt  totalIO    = 0;
    SAPDB_UInt  totalPages = 0;

    Converter_IPageIO::Instance().ReadPages( m_trans.trTaskId_gg00, m_myIndex, totalPages, totalIO );

    m_pagerController.AddConverterIOTotal( totalIO );
    m_pagerController.AddConverterPagesTotal( totalPages );
}

/*===========================================================================*
*  END OF CODE                                                              *
*===========================================================================*/
