/*!***************************************************************************

  module      : RTEConf_ParameterAccessKernelInterface.h

  -------------------------------------------------------------------------

  responsible : JoergM, RobinW

  description : C/Pascal interface for class RTEConf_ParameterAccess (class for kernel configurable parameter access)


  last changed: 2002-04-29  16:18

  first created:2000-03-22  17:49

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/

#ifndef CONF_PARAMETER_ACCESS_H
#define CONF_PARAMETER_ACCESS_H

#include "vsp0011.h"
#include "geo002.h"

/*!----------------------------------------------
  Function: vconf_param_get_int4
  Description: 

  Arguments: conf_param_name [in] parameter name reference
             conf_param_value [out] value reference as 4-byte integer
             errtext [out] error text if unsuccessful
             conf_param_ret [out] return value as defined in vsp0011.h
 */
externC void vconf_param_get_int4      ( tsp11_ConfParamName        VAR_ARRAY_REF conf_param_name,
                                         tsp00_Int4                 VAR_VALUE_REF conf_param_value,
                                         tsp00_ErrText              VAR_ARRAY_REF errtext,
                                         tsp11_ConfParamReturnValue VAR_VALUE_REF conf_param_ret );

/*!----------------------------------------------
  Function: vconf_param_get_integer
  Description: 

  Arguments: conf_param_name [in] parameter name reference
             conf_param_value [out] value reference as 8-byte integer
             errtext [out] error text if unsuccessful
             conf_param_ret [out] return value as defined in vsp0011.h
 */
externC void vconf_param_get_integer   ( tsp11_ConfParamName        VAR_ARRAY_REF conf_param_name,
                                         tsp00_Int8                 VAR_VALUE_REF conf_param_value,
                                         tsp00_ErrText              VAR_ARRAY_REF errtext,
                                         tsp11_ConfParamReturnValue VAR_VALUE_REF conf_param_ret );

/*!----------------------------------------------
  Function: vconf_param_get_real
  Description: 

  Arguments: conf_param_name [in] parameter name reference
             conf_param_value [out] value reference as longreal
             errtext [out] error text if unsuccessful
             conf_param_ret [out] return value as defined in vsp0011.h
 */
externC void vconf_param_get_real      ( tsp11_ConfParamName        VAR_ARRAY_REF conf_param_name,
                                         tsp00_Longreal             VAR_VALUE_REF conf_param_value,
                                         tsp00_ErrText              VAR_ARRAY_REF errtext,
                                         tsp11_ConfParamReturnValue VAR_VALUE_REF conf_param_ret );

/*!----------------------------------------------
  Function: vconf_param_get_string
  Description: 

  Arguments: conf_param_name [in] parameter name reference
             conf_param_value [out] value reference as string
             conf_param_length [out] length of the string
             errtext [out] error text if unsuccessful
             conf_param_ret [out] return value as defined in vsp0011.h
 */
externC void vconf_param_get_string    ( tsp11_ConfParamName        VAR_ARRAY_REF conf_param_name,
                                         tsp11_ConfParamValue       VAR_ARRAY_REF conf_param_value,
                                         tsp00_Int4                 VAR_VALUE_REF conf_param_length,
                                         tsp00_ErrText              VAR_ARRAY_REF errtext,
                                         tsp11_ConfParamReturnValue VAR_VALUE_REF conf_param_ret );

/*!----------------------------------------------
  Function: vconf_param_get_cryptinfo
  Description: 

  Arguments: conf_param_name [in] parameter name reference
             conf_param_value [out] value reference as crypt info
             errtext [out] error text if unsuccessful
             conf_param_ret [out] return value as defined in vsp0011.h
 */
externC void vconf_param_get_cryptinfo ( tsp11_ConfParamName        VAR_ARRAY_REF conf_param_name,
                                         tsp00_CryptName            VAR_ARRAY_REF conf_param_value,
                                         tsp00_ErrText              VAR_ARRAY_REF errtext,
                                         tsp11_ConfParamReturnValue VAR_VALUE_REF conf_param_ret );

/*!----------------------------------------------
  Function: vconf_param_put
  Description: 

  Arguments: conf_param_name [in] parameter name reference
             conf_param_value [in] value reference as string,numeric
                                    parameters are given in their ASCII representation,
             is_numeric [in] flag to indicate a numeric parameter   
             errtext [out] error text if unsuccessful
             conf_param_ret [out] return value as defined in vsp0011.h
 */
externC void vconf_param_put           ( tsp11_ConfParamName        VAR_ARRAY_REF conf_param_name,
                                         tsp11_ConfParamValue       VAR_ARRAY_REF conf_param_value,
                                         boolean                                  is_numeric,
                                         tsp00_ErrText              VAR_ARRAY_REF errtext,
                                         tsp11_ConfParamReturnValue VAR_VALUE_REF conf_param_ret );
 
/*!
  Function: RTEConf_CheckDuplicateDevSpaceName
  Description: Check if given names has duplicate in devspacesArray
  Arguments: devspacesArray [in] Array of devspace description arrays
             szDevspaceName [in] Devspacename to check
  Return value: true is no duplicate, false if duplicate
 */
externC SAPDB_Bool RTEConf_CheckDuplicateDevSpaceName( DEVSPACES_DESCR *devspacesArray,
                                                       SAPDB_Char *szDevspaceName );

                                           
externC SAPDB_Bool RTEConf_ReadAndAnalyzeParam ( tsp00_DbNamec VAR_ARRAY_REF    ServerDB,
                                                 RTE_XPARAM_REC   *XParam ,
                                                 SAPDB_Bool *fGotControlPw,
                                                 SAPDB_Char **errText);

externC void RTEConf_FreeTaskClusterArrayInXParamStructure( RTE_XPARAM_REC *XParam);

externC void RTEConf_FreeDevDescrInXParamStructure( RTE_XPARAM_REC *XParam,int idx );


externC SAPDB_Bool RTEConf_ParameterAccessGetInteger   ( SAPDB_UTF8 *conf_param_name,
									                     SAPDB_Int8  *conf_param_value,
									                     SAPDB_Char ** errtext);


externC SAPDB_Bool RTEConf_ParameterAccessGetReal      ( SAPDB_UTF8 *conf_param_name,
									                     SAPDB_Real8 *conf_param_value,
									                     SAPDB_Char ** errtext);


externC SAPDB_Bool RTEConf_ParameterAccessGetString    ( SAPDB_UTF8 *conf_param_name,
									                     SAPDB_Char **conf_param_value,
									                     SAPDB_Char ** errtext);


externC SAPDB_Bool RTEConf_ParameterAccessGetCryptInfo ( SAPDB_UTF8 *conf_param_name,
									                     SAPDB_Byte *conf_param_value,
									                     SAPDB_Char ** errtext);


externC SAPDB_Bool RTEConf_ParameterAccessSetCrashSem ( SAPDB_Int8 conf_param_value,
                                                        SAPDB_Char ** errtext);

externC SAPDB_Bool RTEConf_ParameterAccessResetParamChangedFlag ( SAPDB_Char ** errtext);

externC SAPDB_Bool RTEConf_ParameterAccessIsVerified            ( );

externC SAPDB_Bool RTEConf_ParameterAccessGetCrashSem           ( );

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/

#endif /* CONF_PARAMETER_ACCESS_H */
