/*!**************************************************************************

  module       : RTEMem_PageAllocator.hpp

  -------------------------------------------------------------------------

  responsible : J. Mensing (jrg)

  special area: RTE I/O Page Allocator
  description : Singleton for I/O Page allocation
  
  first created:2000-11-07  10:01
  last changed: 2000-11-07  17:48

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2002 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef RTEMEM_PAGEALLOCATOR_HPP
#define RTEMEM_PAGEALLOCATOR_HPP

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SAPDBCommon/MemoryManagement/SAPDBMem_PageAllocator.hpp"
#include "SAPDBCommon/SAPDB_Singleton.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

class RTEMem_PageAllocator : public SAPDBMem_PageAllocator
{
public:
  /*!
    Function: Initialize

    description: Initialize the RTEMem_PageAllocator instance

    Arguments: dbPageSize [in] database page size in bytes
   */
    static void Initialize( SAPDB_Int4 dbPageSize );

  /*!
    Function: Instance

    description: Return reference to single instance of RTEMem_PageAllocator

    The Initialize() function must have been called before.
    This is necessary since the database page size is only known by the caller
    of the Initialize() function
    
    Return value: Reference to SAPDBMem_PageAllocator instance
   */
    static SAPDBMem_PageAllocator & Instance( );

#ifdef AIX
    /*!----------------------------------------------------------------------
    @brief Resets the m_Instance 
    -----------------------------------------------------------------------*/
    static void Reset()
    {
        m_Instance = 0;
    }
#endif

private:
    RTEMem_PageAllocator(SAPDB_ULong pageSize):SAPDBMem_PageAllocator(pageSize) {}

    SAPDB_Singleton m_ThisIsASingleton; // prevents copy and assignment

    static RTEMem_PageAllocator *m_Instance;
  // Using a static reference would mean, that initialization function is called before main
  // This we have to prevent for the same reason we prevent static class instances
  // -> dynamic linked RunTime would not get initialized correctly on some OS...
};

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/



#endif  /* RTEMEM_PAGEALLOCATOR_HPP */
