/*!***************************************************************************

  module      : 

  -------------------------------------------------------------------------

  responsible : RobinW, RaymondR

  special area: DBM Support
  description : SAPDB Instance and Database Registration and Management Interfaces

  last changed: 2002-11-11  11:11
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SAPDB/SAPDBCommon/SAPDB_Types.h"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#ifndef externC
# if defined(__cplusplus)
#define externC extern "C" 
#  else
#define externC extern
# endif
#endif

/*===========================================================================*
 *  PROTOTYPES                                                               *
 *===========================================================================*/

#ifdef _WIN32

/*!---------------------------------------------
  Function: RTE_getDBRootFromRegistry (Windows only)
  Description: 
  Arguments: pszServerDB [in] Full(!) name of database. 
                Full means: If the database is not registered for speed fast, 
                one of the speeds it is registered for has to follow the database names 
                in brackets. For example: DB74 (slow) 
             pszNodeName [in] name of the node the database is on. NULL if the database is local.
             ulServerType [in] type of the database. Values are:
                SERVER_TYPE_SERVERDB             (0) 
                SERVER_TYPE_GATEWAY              (1)
                SERVER_TYPE_ADABAS_SERVERDB      (2)
                SERVER_TYPE_ADABAS_GATEWAY       (3)

  Return value: the DBRoot of the database. NULL if none was found.
*/
externC SAPDB_Char *RTE_getDBRootFromRegistry ( SAPDB_Char  *pszNodeName,
                                        SAPDB_ULong ulServerType,
                                        SAPDB_Char  *pszServerDB );

#endif

/*!---------------------------------------------
  Function: RTE_GetDBRootOfDB
  Description: get the DBRoot of the database given by DBName
  Arguments: DBName [in] Name of database
             DBRootOfDB [out] Pointer to a buffer that is filled with DBRoot of database
             MaxDBRootLen [in] size of the buffer
  Return value: true if call suceeded, false if not 
*/
externC SAPDB_Bool RTE_GetDBRootOfDB          ( SAPDB_Char *DBName,
                                        SAPDB_Char *DBRootOfDB,
                                        SAPDB_Int  MaxDBRootLen);

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
