/*!***************************************************************************

  module      : RTE_KernelMain.h

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Kernel Runtime Initialization
  description : First routine called

  Responsible for initializing resources from scratch

  last changed: 2000-07-17  11:48
  see also    : example.html ...
  first created:2000-06-22  9:29

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef RTE_KERNELMAIN_H
#define RTE_KERNELMAIN_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/
/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/
#ifndef externC
# if defined(__cplusplus)
#define externC extern "C" 
#  else
#define externC extern
# endif
#endif
/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/

/*!---------------------------------------------
  function: RTE_KernelMain

  description: First function called
   This function is called before other action is done. It is responsible for
   initializing RTE_IGlobal, RTE_ISystem and the allocators RTEMem_Allocator and
   RTEMem_BlockAllocator. 

  arguments: argc [in] Number of arguments passed
             argv [in] Argument vector

  return value: 
                != 0 if initialization failed (to be used as exit code) <br>
                0 if initialization succeeded
 */
externC int RTE_KernelMain(int argc, char *argv[]);

/*!---------------------------------------------
  function: RTE_SetKGS

  description: Setup pointer to kernel global section
   This function is called once after KGS is setup. It enable new code to use RTE_KGS::Instance()
   to kernel global section.

  arguments: pKGS [in] Pointer to kernel global section
 */
externC void RTE_SetKGS(void *pKGS);

 /*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/


#endif  /* RTE_KERNELMAIN_H */
