/*!
  @file           RTE_SystemPA.h
  @author         JoergM
  @ingroup        Runtime
  @brief          System Interface

  Needed for Precision architecture specialties.
   aCC (C++ compiler on HPUX) does not support inline assembler

\if EMIT_LICENCE
  ========== licence begin LGPL
  Copyright (C) 2001-2003 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end
\endif
*/
#ifndef RTE_SYSTEMPA_H
#define RTE_SYSTEMPA_H

#if defined(__cplusplus)
extern "C" 
#endif

/*!
  @brief Implementation of own context setup interface

   struct RTE_OwnCoroutineContext
   {
       void *sp ;          Stack pointer
       void (*addr)() ;	Return pointer
   };

   void RTE_OwnMakeContext(struct RTE_OwnCoroutineContext *pContext)

   pContext must point to an already filled context
     pContext->sp   set to first byte of stack array,
     pContext->addr point to initial function

  @param pContext [in] context to setup
 */
void RTE_OwnMakeContext(struct RTE_OwnCoroutineContext *pContext);

/*
   @brief Implementation of own context switch interface

   struct RTE_OwnCoroutineContext
   {
       void *sp ;          Stack pointer
       void (*addr)() ;	Return pointer
   };

   SAPDB_Bool RTE_OwnSwapContext( struct RTE_OwnCoroutineContext *pCallr,
                                  struct RTE_OwnCoroutineContext *pNew )
 
   This call replaces swapcontext() and will always succeed. Stack over must be check outside if needed

   @param pCallerContext [out] context of caller (for update)
   @param pNewContext [in] context to resume into
 */
void RTE_OwnSwapContext( struct RTE_OwnCoroutineContext *pCallerContext
                        ,struct RTE_OwnCoroutineContext *pNewContext );

#endif  /* RTE_SYSTEMPA_H */
