/*!***************************************************************************

  module      : RTESys_Time.h

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: System calls
  description : Time system call wrapper


  last changed: 2001-04-04  9:46
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/


#ifndef RTESYS_TIME_H
#define RTESYS_TIME_H



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "RunTime/RTE_Types.h"
#include "RunTime/RTE_CompilerFeatures.h" /* for externC and VAR_ARRAY_REF ... */

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define SAPDB_TIMESTAMP_FORMAT "%04d-%2d-%2d %2d:%2d:%2d"

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*
 * SAPDB_TIMESTAMP_FORMAT          "YYYY-MM-DD HH:MM:SS\0"
 */
typedef SAPDB_Char RTESys_Timestamp[  5 +3 +3 +3 +3 +3   ];

/*!
  Function: RTESys_FillTimeStamp
  Description: Fill the given timestamp buffer with local time information
   Format SAPDB_TIMESTAMP_FORMAT YYYY-MM-DD HH:MM:SS
  Argument: timeBuffer [out] the time buffer to fill
  Return value: Pointer to first character
 */
externC SAPDB_Char *RTESys_FillProtocolTimeStamp(RTESys_Timestamp VAR_ARRAY_REF timeBuffer);

/*
  Function: RTESys_FillGMTTimeStamp
  Description: Fill the given timestamp buffer with GMT time information
   Format SAPDB_TIMESTAMP_FORMAT YYYY-MM-DD HH:MM:SS
  Argument: timeBuffer [out] the time buffer to fill
  Return value: Pointer to first character
 */
externC SAPDB_Char *RTESys_FillProtocolGMTTimeStamp(RTESys_Timestamp VAR_ARRAY_REF timeBuffer);

/*!
  Function: RTESys_Time
  Description: Returns seconds since 1.1.1990
  Argument: none
  Return value: Seconds since 1.1.1990
 */
externC SAPDB_UInt8 RTESys_Time();

/*!
  Function: RTESys_SetInCrashSignalHandler
  Description: Set flag to indicate we are running inside a crash signal handler

  Some of the function we use are not async signal save. To prevent calling them the signal
  handler must call this flag before issuing any of the system calls
 */
externC void RTESys_SetInCrashSignalHandler();

/*!
  Function: RTESys_ResetInCrashSignalHandler
  Description: Reset flag to indicate we are running outside a crash signal handler

  Some of the function we use are not async signal save. To prevent calling them the signal
  handler must call this global flag before issuing any of the system calls
 */
externC void RTESys_ResetInCrashSignalHandler();

/*!
  Function: RTESys_IsInCrashSignalHandler
  Description: Check flag to indicate we are running inside a crash signal handler
  Return value: true if flag set, false if flag is reset
 */
externC SAPDB_Bool RTESys_IsInCrashSignalHandler();

/*!
  Function: RTESys_Timeout
  Description: Check flag until true or timeout exceeded

  The flag is normally modified by another thread or process.
  The waiting interval time is fixed to 100 milliseconds. If no waiting time
  is given, the call only tests the flag and returns 1 if flag is true, 0 if not!

  Arguments: pFlag [in] pointer to flag (i.e. in shared memory)
             waitingIntervalsOf100msec [in] number of waiting intervals (minimum 1)
  Return value: 0 if timeout wa exceeded, else number of waiting intervals left!
 */
externC SAPDB_UInt4 RTESys_Timeout(SAPDB_Bool *pFlag, SAPDB_UInt4 waitingIntervalsOf100msec);

#endif  /* RTESYS_TIME_H */
