#include "SAPDBCommon/SAPDB_Types.h"
#include "RunTime/Tasking/RTETask_TCB.hpp"
#include "RunTime/Tasking/RTETask_Yield.h"
#include "geo007_1.h"
#include "geo50_0.h"
#include "RunTime/System/RTESys_Spinlock.h"

#ifndef WIN32
#include "hen50.h" /* nocheck */
#endif

void RTETask_getTCBandUKTCB( RTE_TCB &TCB, RTE_UKT &UKTCB, RTE_TaskId &pid )
{
    if(RTE_UnknownTaskId != pid)
    {
        TASK_CTRL pTaskCtrl;
        PID_TCB(pid,pTaskCtrl);
        TCB     = pTaskCtrl;
        UKTCB   = TCB.ukt();
    }
    else
    {
        UKTCB   = THIS_UKT_CTRL;
        TCB     = UKTCB.activeTask();
        pid     = TCB.taskIndex();
    }
}


#ifdef WIN32
SAPDB_Long RTE_TCB::m_dummy;
#endif

/*!
  @brief Yield call inside kernel
  If kernel code runs outside of task group a thread rescheduling is executed.
  If kernel code runs inside a task group, it uses the dispatcher to find a new
  task, and a thread rescheduling is only executed, if a new tasks exists. Otherwise
  a switch to the new task is done.
 */
extern "C" void vgetpid( RTE_TaskId &ownId );
extern "C" void vsleep( RTE_TaskId ownId, SAPDB_Int2 interval );
extern "C" void vmovelock( RTE_TaskId ownId );
extern "C" void vmoveunlock( RTE_TaskId ownId );

extern "C" void RTETask_Yield(SAPDB_Bool allowToMove)
{
    RTE_TaskId ownId;
    vgetpid( ownId );

    if ( RTE_UnknownTaskId != ownId )
    {
        TASK_CTRL ownTCB;
        PID_TCB(ownId, ownTCB);

        // Prevent task from being moved...
        if ( !allowToMove ) vmovelock( ownId );
          tsp00_Uint4 taskSwitches = UKT(ownTCB)->taskSwitches;
          vsleep(ownId, 0);
        if ( !allowToMove ) vmoveunlock( ownId );

        /* if number of task switches did, not change, we can savely reschedule the thread... */
        if ( taskSwitches == UKT(ownTCB)->taskSwitches )
        {
            RTESys_GiveUpTimeSlice();
        }
    }
    else
    {
        RTESys_GiveUpTimeSlice();
    }
}
