/*!*********************************************************************

  module:       SAPDBTrace_CheckTopic.hpp

  ----------------------------------------------------------------------

  responsible:  DanielD

  special area: SAPDB diagnostics

  description:  allows to switch groups of conditional statements

  ----------------------------------------------------------------------

  copyright:            Copyright by SAP AG, 2001



    ========== licence begin LGPL
    Copyright (C) 2001 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

  *********************************************************************/

#ifndef SAPDBDIAG_CHECKTOPIC_HPP
#define SAPDBDIAG_CHECKTOPIC_HPP

#include "SAPDBCommon/Diagnostics/SAPDBDiag_Topic.hpp"

/*!*********************************************************************

  Class: SAPDBDiag_CheckTopic

 **********************************************************************/
class SAPDBDiag_CheckTopic : public SAPDBDiag_Topic
{
public:
    /*! function: constructor

        description: creates a new SAPDBDiag_CheckTopic.
        arguments:   topicKey [in] the string to be used by
                        <Method>Matches</>
     */
    SAPDBDiag_CheckTopic (const SAPDB_Char * topicKey);

    /*! Function: ChecksLevel

        returns: true if the local check level is smaller or equal
            <Param>requestedLevel</>
     */
    bool ChecksLevel (int requestedLevel) const;

    /*! Function: MultiSwitch

        description: parses a line with switch instructions

            The line has the format "topic level [topic level ...]",
            a topic key followed by a numeric trace level.
     */
    static void MultiSwitch (
        SAPDBDiag_CheckTopic * const topics [],
        const SAPDB_Char * line,
        int                lineLen = -1);

protected:
    /* Function: GetKind

        returns: the 'class' of the topic
     */
    virtual const SAPDB_Char * GetTopicKind () const;

};

/*! EndClass: SAPDBDiag_CheckTopic */

#endif

