/*!******************************************************************************

  module:		 SAPDBFields_FieldRAMBuffer.hpp

  ------------------------------------------------------------------------------

  responsible:  TillL

  special area: SAPDBCommon/Fields

  description:  Template class for field based random access buffers

                - This is a template class for common usage.
                - Template argument T is the basis type of the buffer
                  (e.g., T = SAPDB_Byte, SAPDB_UInt2, etc.)

  see also:     SAPDBFields_BasicRAMBuffer.hmtl,
                SAPDBFields_FieldResource.html

  ------------------------------------------------------------------------------

                         Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*******************************************************************************/


#ifndef SAPDBFIELDS_FIELDRAMBUFFER_HPP
#define SAPDBFIELDS_FIELDRAMBUFFER_HPP

#include "SAPDBCommon/Fields/SAPDBFields_BasicRAMBuffer.hpp"
#include "SAPDBCommon/Fields/SAPDBFields_FieldResource.hpp"


/*!*****************************************************************************

   class: SAPDBFields_FieldRAMBuffer

*******************************************************************************/
template <class T>
class SAPDBFields_FieldRAMBuffer :
public SAPDBFields_BasicRAMBuffer < SAPDBFields_FieldResource < T > >
{
public:
    /* -----------------------------------------------------------------------*/
    /*! chapter: Data Types                                                   */
    /*!-------------------------------------------------------------------------
    declaration : BasicBuffer
    description:  Parent class
    --------------------------------------------------------------------------*/
    typedef SAPDBFields_BasicRAMBuffer < SAPDBFields_FieldResource < T > >
                                                           BasicBuffer;

    typedef typename BasicBuffer::ResourceType             ResourceType;
    typedef typename BasicBuffer::SizeType                 SizeType;
    typedef typename BasicBuffer::IndexType                IndexType;
    typedef typename BasicBuffer::BasisType                BasisType;      
    typedef typename BasicBuffer::Pointer                  Pointer;
    typedef typename BasicBuffer::Iterator                 Iterator;
    typedef typename BasicBuffer::ConstPointer             ConstPointer;
    typedef typename BasicBuffer::ConstIterator            ConstIterator;
    typedef typename BasicBuffer::BasisFieldType           BasisFieldType;
    typedef typename BasicBuffer::CommonFieldType          CommonFieldType;
    typedef typename BasicBuffer::AllocatorType            AllocatorType;

    /*!-------------------------------------------------------------------------
    function:     SAPDBFields_FieldRAMBuffer()
    description:  Constructors
    
                  - 'src' and 'len' specify the source for the buffer.

    arguments:    src   [in]  pointer to memory block
                  len   [in]  length of memory block
    return value: none
    prototypes:   2
    --------------------------------------------------------------------------*/
    SAPDBFields_FieldRAMBuffer
        ()
    {}

    SAPDBFields_FieldRAMBuffer
        (void*                          src, 
         SizeType                       len)
        :
        BasicBuffer(src, len)
    {}
};

/*!*****************************************************************************

   endclass: SAPDBFields_FieldRAMBuffer

*******************************************************************************/




#endif // SAPDBFIELDS_FIELDRAMBUFFER_HPP
