/*!******************************************************************************

  module:		 SAPDBFields_IResource.hpp

  ------------------------------------------------------------------------------

  responsible:  TillL

  special area: SAPDBCommon

  description:  Interface template class for basic resources of elements of T

  see also:     

  ------------------------------------------------------------------------------

                         Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*******************************************************************************/


#ifndef SAPDBFIELDS_IRESOURCE_HPP
#define SAPDBFIELDS_IRESOURCE_HPP

#include "SAPDBCommon/SAPDB_Types.hpp"
#include "SAPDBCommon/SAPDB_ToString.hpp"
#include "SAPDBCommon/Fields/SAPDBFields_Exception.hpp"
#include "SAPDBCommon/Fields/SAPDBFields_Resource_Messages.hpp" /* nocheck */


// *****************************************************************************
//
// Exceptions thrown by resource classes
//
// -----------------------------------------------------------------------------
/*! class:       SAPDBFields_ResourceExhausted
    description: Exception class                                              */
// -----------------------------------------------------------------------------
class SAPDBFields_ResourceExhausted : public SAPDBFields_Exception
{
public:
    SAPDBFields_ResourceExhausted() {}
    SAPDBFields_ResourceExhausted
        (const SAPDB_Char*              Filename, 
         SAPDB_UInt4                    LineNumber, 
         SAPDB_Int                      maxelem, 
         SAPDB_Int                      requested)
    : SAPDBFields_Exception
        (Filename, LineNumber, SAPDBFIELDS_RESOURCE_EXHAUSTED, 
         SAPDB_ToString(maxelem, _T_d),
         SAPDB_ToString(requested, _T_d)) {}
};
// -----------------------------------------------------------------------------
/*! endclass:    SAPDBFields_ResourceExhausted                                */
// -----------------------------------------------------------------------------




/*!*****************************************************************************

    class: SAPDBFields_IResource

*******************************************************************************/
template <class T>
class SAPDBFields_IResource
{
public:
    /*!-------------------------------------------------------------------------
    declaration : SizeType
    description:  The range for buffer lengths (in sizeof(T))
    --------------------------------------------------------------------------*/
    typedef SAPDB_UInt                      SizeType;
    /*!-------------------------------------------------------------------------
    declaration : IndexType
    description:  The range for positions within buffers
    --------------------------------------------------------------------------*/
    typedef SizeType                        IndexType;
    /*!-------------------------------------------------------------------------
    declaration : BasisType
    description:  Type of data contained in resource
    --------------------------------------------------------------------------*/
    typedef T                               BasisType;      
    /*!-------------------------------------------------------------------------
    declaration : Reference
    description:  Reference to data contained in resource
    --------------------------------------------------------------------------*/
    typedef BasisType&                      Reference;      
    /*!-------------------------------------------------------------------------
    declaration : ConstReference
    description:  Reference to constant data contained in resource
    --------------------------------------------------------------------------*/
    typedef const BasisType&                ConstReference;
    /*!-------------------------------------------------------------------------
    declaration : Pointer
    description:  Pointer to resource data
    --------------------------------------------------------------------------*/
    typedef BasisType*                      Pointer;
    /*!-------------------------------------------------------------------------
    declaration : ConstPointer
    description:  Pointer to constant resource data
    --------------------------------------------------------------------------*/
    typedef const BasisType*                ConstPointer;

    /*!-------------------------------------------------------------------------
    declaration : AllocatorType
    description:  Dummy definition for compiler needs
    --------------------------------------------------------------------------*/
    typedef T                               AllocatorType;

};
/*!*****************************************************************************

    endclass: SAPDBFields_IResource

*******************************************************************************/



#endif // SAPDBFIELDS_IRESOURCE_HPP

