/*!******************************************************************************

  module:        SAPDBMem_Exceptions.hpp

  ------------------------------------------------------------------------------

  responsible:  Stefan Baier

  special area: Memory management exceptions

  description:  Declaration of the exception classes for component SAPDBMem

  see also:      

  ------------------------------------------------------------------------------

  copyright:                     Copyright by SAP AG, 2000



     ========== licence begin LGPL
     Copyright (C) 2000 SAP AG

     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
     version 2.1 of the License, or (at your option) any later version.

     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.

     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     ========== licence end

*******************************************************************************/


#ifndef SAPDBMem_Exceptions_HPP
#define SAPDBMem_Exceptions_HPP

#include "SAPDBCommon/SAPDB_ToString.hpp"
#include "SAPDBCommon/ErrorsAndMessages/SAPDBErr_MessageList.hpp"

// Constants for error messages, generated from "SAPDBMem_Messages.msg"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_Messages.hpp"     /* nocheck */

/*!-----------------------------------------------------------------------------
declaration: SAPDBERR_DEFINE_COMPONENT_EXCEPTION_CLASS( SAPDBMem )
description: defines an exception class for the component SAPDBMem
------------------------------------------------------------------------------*/
SAPDBERR_DEFINE_COMPONENT_EXCEPTION_CLASS (SAPDBMem);


class SAPDBMem_BadAllocException : public SAPDBMem_Exception
{
public:
    SAPDBMem_BadAllocException() {}
    SAPDBMem_BadAllocException(const SAPDB_Char* Filename, SAPDB_UInt4 LineNumber, int Size)
        : SAPDBMem_Exception(Filename, LineNumber, SAPDBMEM_ERR_BAD_ALLOC, SAPDB_ToString(Size)) {}
    virtual ~SAPDBMem_BadAllocException() {}
};

class SAPDBMem_BadDeallocException : public SAPDBMem_Exception
{
public:
    SAPDBMem_BadDeallocException() {}
    SAPDBMem_BadDeallocException(const SAPDB_Char* Filename, SAPDB_UInt4 LineNumber)
        : SAPDBMem_Exception(Filename, LineNumber, SAPDBMEM_ERR_BAD_ALLOC) {}
    virtual ~SAPDBMem_BadDeallocException() {}
};

class SAPDBMem_ConsistencyException : public SAPDBMem_Exception
{
public:
    SAPDBMem_ConsistencyException() {}
    SAPDBMem_ConsistencyException (const SAPDB_Char* Filename,
                                   SAPDB_UInt4 LineNumber,
                                   const SAPDB_Char* Message)
        : SAPDBMem_Exception(Filename, LineNumber, SAPDBMEM_ERR_CONSISTENCY, Message) {}
    virtual ~SAPDBMem_ConsistencyException () {}
};


#endif // SAPDBMem_Exceptions_HPP

