/*!*****************************************************************************

  module:       SAPDB_Singleton.hpp

  ------------------------------------------------------------------------------

  responsible:  JoergM

  special area: SAPDB Common

  description:  Common singleton base class
  
  After several bad problems with unwanted copy actions on singletons, this
  base class is introduced. It has protected copy constructor and assignment
  operator. Therefore with a singleton that is derived from this class, the
  compiler will detect all attempts to copy a singleton...

  ------------------------------------------------------------------------------

  ========== licence begin  GPL
  Copyright (C) 2000 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*******************************************************************************/


#ifndef SAPDB_SINGLETON_HPP
#define SAPDB_SINGLETON_HPP


/*!*****************************************************************************

   class: SAPDB_Singleton

   description: Base class for singletons

    Usage of this class as base class prevents copy or assignment of singletons

*******************************************************************************/
class SAPDB_Singleton
{
public:
    /*!-------------------------------------------------------------------------
    declaration : ctor
    description:  Default constructor 
    
    Needed if copy constructor is defined for a class
    --------------------------------------------------------------------------*/
    SAPDB_Singleton() {}
protected:
    /*!-------------------------------------------------------------------------
    declaration : Copy ctor
    description:  prevents unwanted copy of singleton
    --------------------------------------------------------------------------*/
    SAPDB_Singleton(const SAPDB_Singleton &) {}
    /*!-------------------------------------------------------------------------
    declaration : Assignment operator
    description:  prevents unwanted assignments of singleton
    --------------------------------------------------------------------------*/
    SAPDB_Singleton & operator=(const SAPDB_Singleton &) { return *this; }
}; /*! endclass: SAPDB_Singleton */

#endif /* SAPDB_SINGLETON_HPP */

