/*!*********************************************************************

  module:       SAPDBTrace_Brace.hpp

  ----------------------------------------------------------------------

  responsible:  DanielD

  special area: SAPDB Tracing

  description:  classes to trace method entry and exit

  ----------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

  *********************************************************************/

#ifndef SAPDBTRACE_METHODBRACE_HPP
#define SAPDBTRACE_METHODBRACE_HPP

#include "SAPDBCommon/SAPDB_Types.h"

class SAPDBTrace_Topic;

/*!*********************************************************************

  Class: SAPDBTrace_MethodBrace

  description: traces method entry and exit

        Objects of this class shouldn't be instantiated directly.
        Instead, the macros SAPDBTRACE_ROUTINE_DEBUG and
        SAPDBTRACE_METHOD_DEBUG should be used.

        Method enty and exit will always appear in pairs, changing
        the trace level after method entry will have no efect
        on the exit trace.

 **********************************************************************/
class SAPDBTrace_MethodBrace
{
public:
    /*! function: constructor
        description: traces method entry

        arguments:
            nameP [in] the name to be written
            thisPtr [in] the value of the this pointer in object methods
            traceTopic [in] the <Class>SAPDBTrace_Topic</> to test against
            traceLevel [in] the lvel to test against

        prototypes: 2

     */
    SAPDBTrace_MethodBrace (
        const char * nameP,
        SAPDBTrace_Topic & traceTopic,
        int traceLevel);
    SAPDBTrace_MethodBrace (
        const char * nameP,
        const void * thisPtr,
        SAPDBTrace_Topic & traceTopic,
        int traceLevel);

    /*! function: destructor
        description: traces method exit
     */
    ~SAPDBTrace_MethodBrace ();

protected:
    const SAPDB_Char * methodName;
    SAPDBTrace_Topic * traceTopic;
};

/*! EndClass: SAPDBTrace_MethodBrace */

#endif

