/*!*********************************************************************

  module:       SAPDBTrace_Topic.hpp

  ----------------------------------------------------------------------

  responsible:  DanielD

  special area: SAPDB Tracing

  description:  allows to switch groups of trace statements

  ----------------------------------------------------------------------

  copyright:            Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

  *********************************************************************/

#ifndef SAPDBTRACE_TOPIC_HPP
#define SAPDBTRACE_TOPIC_HPP

#include "SAPDBCommon/SAPDB_Types.hpp"
#include "SAPDBCommon/Diagnostics/SAPDBDiag_Topic.hpp"

class SAPDBTrace_LinePrinter;

/*!*********************************************************************

  Class: SAPDBTrace_Topic

 **********************************************************************/
class SAPDBTrace_Topic : public SAPDBDiag_Topic
{
public:
    /*! function: constructor

        description: creates a new SAPDBTrace_Topic.
        arguments:   switchKey [in] the string to be used by
                        <Method>SwitchConditional</>
     */
    SAPDBTrace_Topic (const SAPDB_Char * switchKey);

    /*! Function: TracesLevel

        returns: true if the local trace level is smaller or equal
            <Param>requestedLevel</>
     */
    bool TracesLevel (int requestedLevel) const;

    /*! function: SetTracePrinter
        description: changes the global trace printer for this topic
        arguments: printer [in] the new trace printer
     */
    void SetTracePrinter (SAPDBTrace_LinePrinter * printer);

    /*! Function: GetTracePrinter

        returns: the shared trace printer
     */
    SAPDBTrace_LinePrinter & GetTracePrinter () const;

    /*! Function: InitGroup

        description: sets the trace printer for a group of topics.
     */
    static void InitGroup (
        SAPDBTrace_Topic * const topics [],
        SAPDBTrace_LinePrinter * printer);

    /*! Function: MultiSwitch

        description: parses a line with trace instructions

            The line has the format "topic level [topic level ...]",
            a topic key followed by a numeric trace level.
     */
    static void MultiSwitch (
        SAPDBTrace_Topic * const topics [],
        const SAPDB_Char * line,
        int                lineLen = -1);

protected:
    /* Function: GetKind

        returns: the 'class' of the topic
     */
    virtual const SAPDB_Char * GetTopicKind () const;

private:
    SAPDBTrace_LinePrinter * linePrinter;
};

/*! EndClass: SAPDBTrace_Topic */

#endif

