/*
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/


/*!**********************************************************************

  module: StudioOAL_AParameters.cpp

  -----------------------------------------------------------------------

  responsible: WolfgangA, Hans-GeorgB

  special area: WebQuery, RepMan, DBAnalyzer

  description:  ASCII Version of OAL

  version     : 7.3


************************************************************************/


#include "SAPDB/SQLStudio/OAL/StudioOAL_AParameters.hpp"


/*===========================================================================*
 *    StudioOAL_AParameters::StudioOAL_AParameters
 *===========================================================================*/
StudioOAL_AParameters::StudioOAL_AParameters(SQLHANDLE   hStmt,
										     SAPDB_Int2  nParameterNo)
{

	m_hStmt = hStmt;
	m_nParameterNo = nParameterNo;

	m_emParameterType = ptUnknown;
	m_lValueLength = SQL_DATA_AT_EXEC;

	m_pValue = new SAPDB_Byte[MX_BYTEVALUE];
	memset( m_pValue,0,MX_BYTEVALUE );
}
/*===========================================================================*
 *    
 *===========================================================================*/
StudioOAL_AParameters::~StudioOAL_AParameters()
{
	if ( m_pValue != NULL )
	{
		 delete[] m_pValue;
	     m_pValue = NULL;
	}
}
/*===========================================================================*
 *    
 *===========================================================================*/
SAPDB_Bool StudioOAL_AParameters::bindParameter()
{
	//#define SQL_NTS                   (-3)
	SAPDB_Int2 lRC = 0;

	if(m_emParameterType == ptInOut || m_emParameterType == ptOut )
	{

		switch (m_nColumnType)
		{

			case SQL_BINARY:
			case SQL_VARBINARY:
			case SQL_LONGVARBINARY:
				  lRC = SQLBindParameter(m_hStmt,
										 m_nParameterNo,
										 m_emParameterType, 
										 SQL_BINARY,
										 m_nColumnType,
										 0,
										 0,
										 m_pValue, 
										 MX_BYTEVALUE,
										 &m_lValueLength);//!!IA64
			break;
			default:
				  m_lValueLength = SQL_NTS;
				  lRC = SQLBindParameter(m_hStmt,
										 m_nParameterNo,
										 m_emParameterType, 
										 SQL_CHAR,
										 m_nColumnType, 
										 0,
										 0,
										 m_pValue, 
										 MX_BYTEVALUE, 
										 &m_lValueLength);//!!IA64
		}
	}			  
    else
	{
		switch (m_nColumnType)
		{

			case SQL_BINARY:
			case SQL_VARBINARY:
			case SQL_LONGVARBINARY:
			  lRC = SQLBindParameter(m_hStmt,
									 m_nParameterNo,
									 SQL_PARAM_INPUT, 
									 SQL_BINARY, 
									 m_nColumnType, 
									 0,
									 0,
									 &m_nParameterNo,
									 0, 
									 &m_lValueLength); //!!IA64
			break;
			default:
			  lRC = SQLBindParameter(m_hStmt,
									 m_nParameterNo,
									 SQL_PARAM_INPUT, 
									 SQL_CHAR,
									 m_nColumnType, 
									 0,
									 0,
									 &m_nParameterNo,
									 0, 
									 &m_lValueLength); //!!IA64
		  }
	}
   
    if( lRC != SQL_SUCCESS ) 
		return SAPDB_FALSE;

	return SAPDB_TRUE;

}
/*===========================================================================*
 *  StudioOAL_AParameters::bindSQLParameter
 *===========================================================================*/
SAPDB_Bool StudioOAL_AParameters::bindSQLParameter()
{
	SAPDB_Int2 lRC = 0;

	m_lValueLength = SQL_NTS;

	if(m_emParameterType == ptIn  )
	{
		switch (m_nColumnType)
		{

			case SQL_BINARY:
			case SQL_VARBINARY:
			case SQL_LONGVARBINARY:
				  lRC = SQLBindParameter(m_hStmt,
										 m_nParameterNo,
										 SQL_PARAM_INPUT, 
										 SQL_C_BINARY,
										 m_nColumnType,
										 0,
										 0,
										 m_pValue, 
										 0,
										 &m_lValueLength); //!!IA64
			break;
			default:
				  lRC = SQLBindParameter(m_hStmt,
										 m_nParameterNo,
										 SQL_PARAM_INPUT, 
										 SQL_C_CHAR,
										 m_nColumnType, 
										 0,
										 0,
										 m_pValue, 
										 0, 
										 &m_lValueLength); //!!IA64
		}
	}
	
    if( lRC != SQL_SUCCESS ) return SAPDB_FALSE;

	return SAPDB_TRUE;

}
/*===========================================================================*
 *   StudioOAL_AParameters::setValue 
 *===========================================================================*/
void StudioOAL_AParameters::setValue (StudioOAL_String sValue) 
{
	const SAPDB_Char* valueEnd = ( const SAPDB_Char* ) m_pValue + MX_BYTEVALUE;
	SAPDB_Char*       destAt;

	sValue.ConvertToASCII_Latin1((SAPDB_Char*)m_pValue, valueEnd, destAt);

}
/*===========================================================================*
 *   StudioOAL_AParameters::getValue 
 *===========================================================================*/
void StudioOAL_AParameters::getValue (StudioOAL_String &sValue)
{
	sValue.Erase();	
	if(m_lValueLength > 0){

		sValue.ConvertFromASCII_Latin1(
					(StudioOAL_String::ASCIIConstPointer)m_pValue,
					(StudioOAL_String::ASCIIConstPointer)m_pValue+m_lValueLength);
	}

	//SQL_NTS. The parameter value is a null-terminated string. 
	if(m_lValueLength == SQL_NTS){

	
		sValue.ConvertFromASCII_Latin1(
					(StudioOAL_String::ASCIIConstPointer)m_pValue,
					(StudioOAL_String::ASCIIConstPointer)m_pValue+strlen((const SAPDB_Char *)m_pValue));
	}

}

/*===========================================================================*
 *    End
 *===========================================================================*/