/*
    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/


/*!**********************************************************************

  module: StudioOAL_AResult.cpp

  -----------------------------------------------------------------------

  responsible: WolfgangA and d033893

  special area: WebQuery, SAP DB LOADER, DBAnalyzer

  description:  ASCII Version of OAL

  version     : 7.4


************************************************************************/

#include "SAPDB/SQLStudio/OAL/StudioOAL_AResult.hpp"

/*===========================================================================*
 *  StudioOAL_AResult::StudioOAL_AResult
 *===========================================================================*/
StudioOAL_AResult::StudioOAL_AResult(SQLHANDLE hDbc,SQLHENV hEnv)
{

	SAPDB_ULong lRun;
	
	m_hDbc = hDbc;
	m_hEnv = hEnv;
	m_enumResultCursorType = stoalCursorForwardOnly;

    //init statement handle
	m_hStmt = 0;
	m_StmtSqlMode        = cpr_kind_internal;
    m_bDBProcedureResult = SAPDB_FALSE;
	m_lLimitForLong = 32768;
	m_bNoRows = SAPDB_TRUE;	
		
	m_pError = new StudioOAL_AError(m_hDbc,m_hEnv);

	for( lRun = 0; lRun <= 1023;lRun++ ){

		m_pColumn[lRun] = NULL;
	}
	
}
/*===========================================================================*
 *  StudioOAL_AResult::~StudioOAL_AResult
 *===========================================================================*/
StudioOAL_AResult::~StudioOAL_AResult()
{
	SAPDB_ULong lRun;

	clearParameters();

	if( m_pError != NULL) {

		delete	m_pError;
		m_pError = NULL;
	}

	for( lRun = 0; lRun <= 1023;lRun++ )
	{
		if (m_pColumn[lRun] != NULL ){

			delete m_pColumn[lRun];
		}
		m_pColumn[lRun] = NULL;
	}

	closeResult();
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::init (const StudioOAL_String &sCmdStr)
{
	clearParameters();

	//if there is a open result the close it
    if(m_hStmt != SQL_NULL_HSTMT)
	   closeResult();

	StudioOAL_AParameters* poParameterDefinitions  = NULL;    
    
    //inits
	m_nColCount = 0;
    m_sStatement = sCmdStr;    

	SQLRETURN		nRc = SQL_SUCCESS;
    
    m_lFetchedRows = 0;
    m_bNoRows = SAPDB_TRUE;	
 	
    // SQLAllocHandle
	nRc = SQLAllocHandle(SQL_HANDLE_STMT,m_hDbc,&m_hStmt);
	if(nRc != SQL_SUCCESS)
	{	
		SQLFreeStmt(m_hStmt, SQL_DROP);
		m_hStmt = SQL_NULL_HSTMT;
		return SAPDB_FALSE;	
	}	
	
    // SQLPrepare SQLNumParams SQLDescribeParams
    if (!prepareForParameters()) {
    
        SQLFreeStmt(m_hStmt, SQL_DROP);
		m_hStmt = SQL_NULL_HSTMT;
        return SAPDB_FALSE;
	}
	// SQLBindParameter
    if(!bindParameters()){

	    SQLFreeStmt(m_hStmt, SQL_DROP);
		m_hStmt = SQL_NULL_HSTMT;
        return SAPDB_FALSE;
	}

	// SQLNumResultCols SQLDescribeCol
	if (!getColumnDescription() ) {
		
		SQLFreeStmt(m_hStmt, SQL_DROP);
		m_hStmt = SQL_NULL_HSTMT;
        return SAPDB_FALSE;
	}

	// SQLBindCol
	if (!bindColumnDataAsString()){

		SQLFreeStmt(m_hStmt, SQL_DROP);
		m_hStmt = SQL_NULL_HSTMT;
        return SAPDB_FALSE;
	}


	return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::bindParameters() 
{
    
    StudioOAL_AParameters  *poCurrentParam;
    
	poCurrentParam = (StudioOAL_AParameters*)m_ParamList.first();
    while (poCurrentParam != NULL){

		if(!poCurrentParam->bindSQLParameter()) return SAPDB_FALSE;
		poCurrentParam = (StudioOAL_AParameters*)m_ParamList.next();
	}    
    
    return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::setParamValues( StudioOAL_String *record )
{
	// assert (record.size() == m_ParamList.size() )
	SAPDB_Int4 nParamNo = 0;

	StudioOAL_AParameters  *poCurrentParam;
    
	poCurrentParam = (StudioOAL_AParameters*)m_ParamList.first();
	while (poCurrentParam != NULL)
	{
		poCurrentParam->setValueLength( record[nParamNo].Length() );
		poCurrentParam->setValue      ( record[nParamNo] );
		poCurrentParam = (StudioOAL_AParameters*)m_ParamList.next();
		nParamNo++;
	}    
    
    return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::bindColumnDataAsString()
{
    SQLRETURN			nRc = 0;

	SAPDB_Char         *pValue = NULL;
	SAPDB_Long			lGetDataLen;
	SAPDB_Long		    lValueLen = 0;  
    
	//SQLRETURN SQLBindCol( SQLHSTMT StatementHandle, SQLUSMALLINT ColumnNumber, SQLSMALLINT TargetType, SQLPOINTER TargetValuePtr, SQLINTEGER BufferLength, SQLLEN     * StrLen_or_Ind);
	//SQLRETURN SQLGetData( SQLHSTMT StatementHandle, SQLUSMALLINT ColumnNumber, SQLSMALLINT TargetType, SQLPOINTER TargetValuePtr, SQLINTEGER BufferLength, SQLINTEGER * StrLen_or_IndPtr);

    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV || m_hStmt == SQL_NULL_HSTMT ) 
		return SAPDB_FALSE;
	if( m_pError == NULL)	
		return SAPDB_FALSE;	

	SAPDB_UInt2 nCol;

   	for (nCol = 1; nCol <= getColCount(); nCol++)
	{
	
		if (m_pColumn[nCol-1] == NULL)	
			return SAPDB_FALSE;	
		
		//get the data for the current col
		switch (m_pColumn[nCol-1]->getODBCColType())
		{    
			case SQL_DATE:
						
				pValue = new char [MX_COLUMN_GENERIC_NUMERIC];
				memset( pValue, 0, MX_COLUMN_GENERIC_NUMERIC );
				if (pValue == NULL) return SAPDB_FALSE;	
				
				m_pColumn[nCol-1]->setColValue(pValue);
				m_pColumn[nCol-1]->setColLength((SAPDB_Long)MX_COLUMN_GENERIC_NUMERIC);
				
				nRc = SQLBindCol(m_hStmt, nCol, SQL_C_CHAR, 
					            (SQLPOINTER ) m_pColumn[nCol-1]->m_pValue, MX_COLUMN_GENERIC_NUMERIC, 
								 &m_pColumn[nCol-1]->m_ValueLength); //!!IA64
				//nRc = SQLBindCol(m_hStmt, nCol, SQL_C_CHAR, &m_pColumn[nCol-1]->m_pValue, MX_COLUMN_GENERIC_NUMERIC, &m_pColumn[nCol-1]->m_ValueLength);
			
				if( nRc != SQL_SUCCESS) {
    
					if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

						delete[] pValue;	
						return SAPDB_FALSE;
					};
				};    
                            
				break;

			case SQL_TIME:
						
				pValue = new char [MX_COLUMN_GENERIC_NUMERIC];
				memset( pValue, 0, MX_COLUMN_GENERIC_NUMERIC );
				if (pValue == NULL) return SAPDB_FALSE;

				m_pColumn[nCol-1]->setColValue(pValue);
				m_pColumn[nCol-1]->setColLength((SAPDB_Long)MX_COLUMN_GENERIC_NUMERIC);
				
				nRc = SQLBindCol(m_hStmt, nCol, SQL_C_CHAR, 
					            (SQLPOINTER)  m_pColumn[nCol-1]->m_pValue, MX_COLUMN_GENERIC_NUMERIC, 
								&m_pColumn[nCol-1]->m_ValueLength);  //!!IA64
			
				if( nRc != SQL_SUCCESS) {

    				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    
						delete[] pValue;
						return SAPDB_FALSE;
					};
				};    
            
				break;

		   case SQL_TIMESTAMP:
													
				pValue = new char [MX_COLUMN_GENERIC_NUMERIC];
				memset( pValue, 0, MX_COLUMN_GENERIC_NUMERIC );
				if (pValue == NULL) return SAPDB_FALSE;

				m_pColumn[nCol-1]->setColValue(pValue);
				m_pColumn[nCol-1]->setColLength((SAPDB_Long)MX_COLUMN_GENERIC_NUMERIC);
				
				nRc = SQLBindCol(m_hStmt, nCol, SQL_C_CHAR, 
					             (SQLPOINTER)  m_pColumn[nCol-1]->m_pValue, MX_COLUMN_GENERIC_NUMERIC , 
								 &m_pColumn[nCol-1]->m_ValueLength);  //!!IA64

				if( nRc != SQL_SUCCESS) {
    			
				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

						delete[] pValue;	
						return SAPDB_FALSE;
					};
				};    
				
				break;
                                   
			case SQL_LONGVARBINARY:
			case SQL_LONGVARCHAR: 
							
				if( m_lLimitForLong >= 0 )
				{

					 lGetDataLen = m_lLimitForLong;                 
					 //Limit for long column can overwrite max value for long column
					 if( m_lLimitForLong > MX_COLUMN_LONG_VALUE ) 
						 lGetDataLen = MX_COLUMN_LONG_VALUE;   
					 
				}
				else
					lGetDataLen = MX_COLUMN_LONG_VALUE;
            
					pValue = new char [lGetDataLen];
					memset( pValue, 0, lGetDataLen );
					if (pValue == NULL) return SAPDB_FALSE;	
					
					m_pColumn[nCol-1]->setColValue (pValue);
					m_pColumn[nCol-1]->setColLength(lGetDataLen);

					m_pError->setNoSQLState(SQLSTATE_DATA_TRUNCATED);
				

					nRc = SQLBindCol(m_hStmt, nCol, SQL_C_CHAR, 
						             (SQLPOINTER )m_pColumn[nCol-1]->m_pValue, 
									 (SQLINTEGER )lGetDataLen , 
									 &m_pColumn[nCol-1]->m_ValueLength);  //!!IA64
					
					if( nRc != SQL_SUCCESS ) {            
						
						if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

							m_pError->setNoSQLState (SQLSTATE_SUCCESS);

							delete[] pValue;
							return SAPDB_FALSE;
						}
						m_pError->setNoSQLState (SQLSTATE_SUCCESS);                
					 };

				break;

			case SQL_DECIMAL:
			case SQL_NUMERIC:
			case SQL_REAL:
			case SQL_FLOAT:
			case SQL_DOUBLE:
			case SQL_INTEGER:
        
				//get the current value			
				pValue = new char [MX_COLUMN_GENERIC_NUMERIC];
				memset( pValue, 0, MX_COLUMN_GENERIC_NUMERIC );
				if (pValue == NULL) return SAPDB_FALSE;

				m_pColumn[nCol-1]->setColValue(pValue);
				m_pColumn[nCol-1]->setColLength((SAPDB_Long)MX_COLUMN_GENERIC_NUMERIC);
				
				nRc = SQLBindCol(m_hStmt, nCol, SQL_C_CHAR, 
					             (SQLPOINTER ) m_pColumn[nCol-1]->m_pValue, MX_COLUMN_GENERIC_NUMERIC ,
								 &m_pColumn[nCol-1]->m_ValueLength); //!!IA64

				if( nRc != SQL_SUCCESS) {
    
					if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt)))
					{    
						delete[] pValue;	
						return SAPDB_FALSE;
					}
				}
				
				break;

			case SQL_BIT:

				//get the current value	
				
				lGetDataLen = 6;

				pValue = new char [lGetDataLen];
				memset( pValue, 0, lGetDataLen );
				if (pValue == NULL) return SAPDB_FALSE;	
				
				m_pColumn[nCol-1]->setColValue(pValue);
				m_pColumn[nCol-1]->setColLength((SAPDB_Long)6);
            
				nRc = SQLBindCol(m_hStmt, nCol, SQL_C_CHAR, 
					            (SQLPOINTER )m_pColumn[nCol-1]->m_pValue, 
								(SQLINTEGER )lGetDataLen , 
								&m_pColumn[nCol-1]->m_ValueLength); //!!IA64
				 			
				if( nRc != SQL_SUCCESS) 
					if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) 
						return SAPDB_FALSE;				
				
				break;

			default:
				//SQL_VARBINARY

				if( m_pColumn[nCol-1]->getODBCColType() == SQL_VARBINARY ||
					m_pColumn[nCol-1]->getODBCColType() == SQL_BINARY )
					//will be transformed from hex to ascii 
					lGetDataLen = m_pColumn[nCol-1]->getColLength() * 2 + 1;
				else
					lGetDataLen = m_pColumn[nCol-1]->getColLength() + 1 ;

				pValue = new char [lGetDataLen];
				memset( pValue, 0, lGetDataLen );
				if (pValue == NULL) return SAPDB_FALSE;

				m_pColumn[nCol-1]->setColValue(pValue);
				m_pColumn[nCol-1]->setColLength(lGetDataLen);
				
				nRc = SQLBindCol(m_hStmt, 
					             nCol, 
								 SQL_C_CHAR, 
								 (SQLPOINTER) m_pColumn[nCol-1]->m_pValue , 
								 (SQLINTEGER) lGetDataLen , 
								 &m_pColumn[nCol-1]->m_ValueLength); //!!IA64

				if( nRc != SQL_SUCCESS) 
				{
    				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt)))
					{    
						delete[] pValue;
						return SAPDB_FALSE;
					}
				}

		};//switch
	}// for (nCol = 1; nCol <= getColCount(); nCol++)

	//if data was truncated then get the possible size lGetDataLen
	//if(m_pColumn[nCol-1]->getColValueLength() == SQL_NO_TOTAL) 
	  // m_pColumn[nCol-1]->setColValueLength(lGetDataLen);
	
	return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::bindColumnDataAsString (SAPDB_UInt2 nCol,StudioOAL_String &Value)
{
	Value.Erase();

    if( m_hDbc == SQL_NULL_HDBC || 
		m_hEnv == SQL_NULL_HENV ||
		m_hStmt == SQL_NULL_HSTMT) 	return SAPDB_FALSE;	
        
    if (m_pColumn[nCol-1] == NULL)	return SAPDB_FALSE;	
	if( m_pError == NULL)	        return SAPDB_FALSE;	

	//get the data for the current col
	switch (m_pColumn[nCol-1]->getODBCColType())
	{    
			case SQL_DATE:
		
				if( m_pColumn[nCol-1]->getColValueLength() >= 0 )
            		m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);
				else
				{
					strcpy((SAPDB_Char*)m_pColumn[nCol-1]->getColValue(),REPOFNULLVALUE);
					m_pColumn[nCol-1]->setColValueLength(strlen(REPOFNULLVALUE));
					m_pColumn[nCol-1]->setColIsNull(SAPDB_TRUE);
				}
				
				break;

			case SQL_TIME:

				if( m_pColumn[nCol-1]->getColValueLength() >= 0 )
					m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);
				else
				{
					strcpy((SAPDB_Char*)m_pColumn[nCol-1]->getColValue(),REPOFNULLVALUE);
					m_pColumn[nCol-1]->setColValueLength(strlen(REPOFNULLVALUE));
					m_pColumn[nCol-1]->setColIsNull(SAPDB_TRUE);
				};

				break;

			case SQL_TIMESTAMP:

				if( m_pColumn[nCol-1]->getColValueLength() >= 0 )
					m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);
				else
				{
					strcpy((SAPDB_Char*)m_pColumn[nCol-1]->getColValue(),REPOFNULLVALUE);
					m_pColumn[nCol-1]->setColValueLength(strlen(REPOFNULLVALUE));
					m_pColumn[nCol-1]->setColIsNull(SAPDB_TRUE);
				};

				break;


		   case SQL_LONGVARBINARY:
		   case SQL_LONGVARCHAR  : 

			   if( m_pColumn[nCol-1]->getColValueLength() == SQL_NO_TOTAL )            
			       m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);                				
			   else
			   {
					if( m_pColumn[nCol-1]->getColValueLength() >= 0 )                    
						m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);					
					else
					{
						strcpy((SAPDB_Char*)m_pColumn[nCol-1]->getColValue(),REPOFNULLVALUE);
						m_pColumn[nCol-1]->setColValueLength(strlen(REPOFNULLVALUE));
						m_pColumn[nCol-1]->setColIsNull(SAPDB_TRUE);
					};
				};

			   break;

		   case SQL_DECIMAL:
		   case SQL_NUMERIC:
		   case SQL_REAL:
		   case SQL_FLOAT:
		   case SQL_DOUBLE:
		   case SQL_INTEGER:

			   	if( m_pColumn[nCol-1]->getColValueLength() >= 0 )            
					m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);            
				else
				{
					strcpy((SAPDB_Char*)m_pColumn[nCol-1]->getColValue(),REPOFNULLVALUE);
					m_pColumn[nCol-1]->setColValueLength( strlen(REPOFNULLVALUE) );
					m_pColumn[nCol-1]->setColIsNull(SAPDB_TRUE);
				}

			    break;

			   
		   case SQL_BIT:

			   if(m_pColumn[nCol-1]->getColValueLength() > 0)
			      m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);
			   else
			   {
					strcpy((SAPDB_Char*)m_pColumn[nCol-1]->getColValue(),REPOFNULLVALUE);
					m_pColumn[nCol-1]->setColValueLength(strlen(REPOFNULLVALUE));
					m_pColumn[nCol-1]->setColIsNull(SAPDB_TRUE);
			   }

			   break;

		   default:

			   if( m_pColumn[nCol-1]->getColValueLength() > 0 )
				   m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);
			   else
			   {
				    if( m_pColumn[nCol-1]->getColValueLength() != SQL_NULL_DATA )
					{
                		//delete[] pValue;
						//pValue  = 0;
						m_pColumn[nCol-1]->setColValueLength(0);
						m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);
					}
					else 
					{
						strcpy((SAPDB_Char*)m_pColumn[nCol-1]->getColValue(),REPOFNULLVALUE);
						m_pColumn[nCol-1]->setColValueLength(strlen(REPOFNULLVALUE));
						m_pColumn[nCol-1]->setColIsNull(SAPDB_TRUE);
					}
			   }	

			   break;

	};

	//if data was truncated then get the possible size lGetDataLen
	//if(m_pColumn[nCol-1]->getColValueLength() == SQL_NO_TOTAL) 
	  // m_pColumn[nCol-1]->setColValueLength(lGetDataLen); //!! ToDo

    if (m_pColumn[nCol-1]->getColValue() != NULL && m_pColumn[nCol-1]->getColValueLength() > 0)
	{
 		if(Value.ConvertFromASCII_Latin1(
						(StudioOAL_String::ASCIIConstPointer)m_pColumn[nCol-1]->getColValue(),
						(StudioOAL_String::ASCIIConstPointer)m_pColumn[nCol-1]->getColValue() + m_pColumn[nCol-1]->getColValueLength() )
						!= Tools_UTF8Basis::Success)

		   return SAPDB_FALSE;
	}
	else
	 	 return SAPDB_FALSE;


	return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::getDataSourceTypeInfo()
{ 
 
    //check it
	if( m_pError == NULL) return SAPDB_FALSE;
    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV)  return SAPDB_FALSE;
	

    //init	variables
	SQLRETURN		nRc = SQL_SUCCESS;    
    m_sStatement   =  "";
    m_lFetchedRows = 0;
    m_bNoRows = SAPDB_TRUE;	

    //if there is a open result the close it
    if(m_hStmt != SQL_NULL_HSTMT) 
	   closeResult();

    //allocate handle for statement    
	nRc = SQLAllocHandle(SQL_HANDLE_STMT,m_hDbc,&m_hStmt);
	if( nRc != SQL_SUCCESS)
	{
		if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {

			SQLFreeStmt(m_hStmt, SQL_DROP);
			m_hStmt = SQL_NULL_HSTMT;
			return SAPDB_FALSE;
		};
	}		
	
    
    nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_ROW_BIND_TYPE, SQL_BIND_BY_COLUMN);
    if( nRc != SQL_SUCCESS ) {
    
		if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
            SQLFreeStmt(m_hStmt, SQL_DROP);
			m_hStmt = SQL_NULL_HSTMT;
            return SAPDB_FALSE;
        };
    };
    
    switch (m_enumResultCursorType)
    {
        case stoalCursorForwardOnly:
    
            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CONCURRENCY, SQL_CONCUR_READ_ONLY);
            if( nRc != SQL_SUCCESS ) {
                
                if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
        
                    SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
                    return SAPDB_FALSE;
                };
            };
            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CURSOR_TYPE, SQL_CURSOR_FORWARD_ONLY);
            if( nRc != SQL_SUCCESS ) {
              if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
                    SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
                    return SAPDB_FALSE;
                };
            };

			break;
        
        case stoalCursorStatic:

            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CONCURRENCY, SQL_CONCUR_ROWVER);
            if( nRc != SQL_SUCCESS ) {
                if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
        
                    SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
                    return SAPDB_FALSE;
                };
            };
            
            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CURSOR_TYPE, SQL_CURSOR_STATIC);
            if( nRc != SQL_SUCCESS ) {
                
                if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
                
                    SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
                    return SAPDB_FALSE;
                };
            };
			break;
            
		case stoalOptimizeCursor:
    
			nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CONCURRENCY, SQL_CONCUR_READ_ONLY);
			if( nRc != SQL_SUCCESS ) {
            
				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
					SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
					return SAPDB_FALSE;
				};
			};
        
			nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_OPTIMIZE_CURSOR, 1);
			if( nRc != SQL_SUCCESS ) {
        
				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {

					//Die Option ExtendedFetch ohne for reuse gibts erst ab 6.2.10
					if(m_pError->getSQLStateErrorNumber() == "S1092") {  // Out of Option range
							
                
						SQLFreeStmt(m_hStmt, SQL_DROP);
						m_hStmt = SQL_NULL_HSTMT;
						return SAPDB_FALSE;
					};
				};
			};
			nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CURSOR_TYPE, SQL_CURSOR_KEYSET_DRIVEN);
			if( nRc != SQL_SUCCESS ) {
        
				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {

					SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
					return SAPDB_FALSE;
				};
			};
			break;
         
		default:
			return SAPDB_FALSE;
			break;   
   }

   nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_USE_BOOKMARKS, SQL_UB_ON);
   if( nRc != SQL_SUCCESS ) {

        if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
            SQLFreeStmt(m_hStmt, SQL_DROP);
			m_hStmt = SQL_NULL_HSTMT;
            return SAPDB_FALSE;
        }
    }
    
    nRc = SQLSetStmtOption(m_hStmt, SQL_ROWSET_SIZE, 1);
    if( nRc != SQL_SUCCESS ) {

        if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
            SQLFreeStmt(m_hStmt, SQL_DROP);
			m_hStmt = SQL_NULL_HSTMT;
            return SAPDB_FALSE;
        }
    }
    
    //open the result set
    nRc = SQLGetTypeInfo(m_hStmt,SQL_ALL_TYPES);
	if( nRc != SQL_SUCCESS ) {

        if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
            SQLFreeStmt(m_hStmt, SQL_DROP);
			m_hStmt = SQL_NULL_HSTMT;
            return SAPDB_FALSE;
        }
    }

	if(!getColDescriptions()) 	
		return SAPDB_FALSE;
    
    //is there a result
	m_lRowCount = rowCount(&m_hStmt);
    if( m_lRowCount == 0 ) {
        
        return SAPDB_TRUE;
	}
	else{
        m_bNoRows = SAPDB_FALSE;
    }    
    return SAPDB_TRUE;

}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
void StudioOAL_AResult::closeCursor()
{       
    
    //check it
    if( m_hStmt == SQL_NULL_HSTMT) return;	    
	SQLFreeStmt(m_hStmt, SQL_CLOSE);    	

};
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
void StudioOAL_AResult::reset()
{
	m_enumResultCursorType = stoalCursorForwardOnly;
    m_bDBProcedureResult   = SAPDB_FALSE;
	m_lLimitForLong        = 32768;	
	m_lFetchedRows         = 0;
	m_bNoRows              = SAPDB_TRUE;	
	m_lRowCount            = 0;

}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::fullExecStmt(const StudioOAL_String& sCmdStr)
{

	//check it
	if( m_pError == NULL) return SAPDB_FALSE;
    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV)  return SAPDB_FALSE;
	

    //Init	
	SQLRETURN		nRc = SQL_SUCCESS;
    
    m_sStatement = sCmdStr;
    m_lFetchedRows = 0;
    m_bNoRows = SAPDB_TRUE;	

    //if there is a open result the close it
    if(m_hStmt != SQL_NULL_HSTMT )
	   closeResult();

    //allocate handle for statement    
	nRc = SQLAllocHandle(SQL_HANDLE_STMT,m_hDbc,&m_hStmt);
	if( nRc != SQL_SUCCESS)
	{
		if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {

			SQLFreeStmt(m_hStmt, SQL_DROP);
			m_hStmt = SQL_NULL_HSTMT;
			return SAPDB_FALSE;
		};
	}		
    
    //open the result set
    if(!execStmt()) {

        //free statement
        SQLFreeStmt(m_hStmt, SQL_DROP);
		m_hStmt = SQL_NULL_HSTMT;
        return SAPDB_FALSE;
    };
    

    m_bNoRows = SAPDB_TRUE;
    
    return SAPDB_TRUE;
	
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::openResult(const StudioOAL_String &sCmdStr)
{ 
 

    //check it
	if( m_pError == NULL) return SAPDB_FALSE;
    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV)  return SAPDB_FALSE;
	

    //init	variables
	SQLRETURN		nRc = SQL_SUCCESS;    
    m_sStatement = sCmdStr;
    m_lFetchedRows = 0;
    m_bNoRows = SAPDB_TRUE;	

    //if there is a open result the close it
    if(m_hStmt != SQL_NULL_HSTMT) 
	   closeResult();

    //allocate handle for statement    
	nRc = SQLAllocHandle(SQL_HANDLE_STMT,m_hDbc,&m_hStmt);
	if( nRc != SQL_SUCCESS)
	{
		if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {

			SQLFreeStmt(m_hStmt, SQL_DROP);
			m_hStmt = SQL_NULL_HSTMT;
			return SAPDB_FALSE;
		};
	}		
	
    nRc = SQLSetStmtOption(m_hStmt,SQL_SQLMODE,getSqlMode()); 
    if( nRc != SQL_SUCCESS ) {
    
		if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
            SQLFreeStmt(m_hStmt, SQL_DROP);
			m_hStmt = SQL_NULL_HSTMT;
            return SAPDB_FALSE;
        };
    };

    nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_ROW_BIND_TYPE, SQL_BIND_BY_COLUMN);
    if( nRc != SQL_SUCCESS ) {
    
		if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
            SQLFreeStmt(m_hStmt, SQL_DROP);
			m_hStmt = SQL_NULL_HSTMT;
            return SAPDB_FALSE;
        };
    };
    
    switch (m_enumResultCursorType)
    {
        case stoalCursorForwardOnly:
    
            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CONCURRENCY, SQL_CONCUR_READ_ONLY);
            if( nRc != SQL_SUCCESS ) {
                
                if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
        
                    SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
                    return SAPDB_FALSE;
                };
            };
            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CURSOR_TYPE, SQL_CURSOR_FORWARD_ONLY);
            if( nRc != SQL_SUCCESS ) {
              if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
                    SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
                    return SAPDB_FALSE;
                };
            };

			break;
        
        case stoalCursorStatic:

            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CONCURRENCY, SQL_CONCUR_ROWVER);
            if( nRc != SQL_SUCCESS ) {
                if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
        
                    SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
                    return SAPDB_FALSE;
                };
            };
            
            nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CURSOR_TYPE, SQL_CURSOR_STATIC);
            if( nRc != SQL_SUCCESS ) {
                
                if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
                
                    SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
                    return SAPDB_FALSE;
                };
            };
			break;
            
		case stoalOptimizeCursor:
    
			nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CONCURRENCY, SQL_CONCUR_READ_ONLY);
			if( nRc != SQL_SUCCESS ) {
            
				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
					SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
					return SAPDB_FALSE;
				};
			};
        
			nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_OPTIMIZE_CURSOR, 1);
			if( nRc != SQL_SUCCESS ) {
        
				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {

					//Die Option ExtendedFetch ohne for reuse gibts erst ab 6.2.10
					if(m_pError->getSQLStateErrorNumber() == "S1092") {  // Out of Option range
							
                
						SQLFreeStmt(m_hStmt, SQL_DROP);
						m_hStmt = SQL_NULL_HSTMT;
						return SAPDB_FALSE;
					};
				};
			};
			nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_CURSOR_TYPE, SQL_CURSOR_KEYSET_DRIVEN);
			if( nRc != SQL_SUCCESS ) {
        
				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {

					SQLFreeStmt(m_hStmt, SQL_DROP);
					m_hStmt = SQL_NULL_HSTMT;
					return SAPDB_FALSE;
				};
			};
			break;
         
		default:
			return SAPDB_FALSE;
			break;   
   }

   nRc = SQLSetStmtOption(m_hStmt, SQL_ATTR_USE_BOOKMARKS, SQL_UB_ON);
   if( nRc != SQL_SUCCESS ) {

        if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
            SQLFreeStmt(m_hStmt, SQL_DROP);
			m_hStmt = SQL_NULL_HSTMT;
            return SAPDB_FALSE;
        }
    }
    
    nRc = SQLSetStmtOption(m_hStmt, SQL_ROWSET_SIZE, 1);
    if( nRc != SQL_SUCCESS ) {

        if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
    
            SQLFreeStmt(m_hStmt, SQL_DROP);
			m_hStmt = SQL_NULL_HSTMT;
            return SAPDB_FALSE;
        }
    }
    
    //open the result set
    if(!execStmt()){

        //free statement
        SQLFreeStmt(m_hStmt, SQL_DROP);
		m_hStmt = SQL_NULL_HSTMT;
        return SAPDB_FALSE;
    };
    
    if(!getColDescriptions()) 	return SAPDB_FALSE;
	

    //is there a result
	m_lRowCount = rowCount(&m_hStmt);
    if( m_lRowCount == 0 ) {
        
        return SAPDB_TRUE;
	}
	else{
        m_bNoRows = SAPDB_FALSE;
    }    
    return SAPDB_TRUE;

}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Long StudioOAL_AResult::rowCount(SQLHANDLE *hStmt)
{

	SQLRETURN	 nRc = 0;
    SAPDB_Long	 lRowCount = 0;

     //check it
    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV || m_pError == NULL) return 0;
	

    // Try to get the number of rows
    nRc = SQLRowCount(*hStmt,  &lRowCount); //!!IA64
	if( nRc != SQL_SUCCESS){
    
        if(!(m_pError->checkSQLReturnCode(nRc,*hStmt)))  return 0;        
    };
    
	return lRowCount;

}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
void StudioOAL_AResult::closeResult()
{       
    
    //check it
    if( m_hStmt == SQL_NULL_HSTMT) return;	    
	SQLFreeStmt(m_hStmt, SQL_DROP);    	
    m_hStmt = SQL_NULL_HSTMT;

}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::getColDescriptions() 
{

    SQLUSMALLINT		lRun;    
    SQLSMALLINT			nColNameLen;
    SQLSMALLINT			nColType;
	  SQLULEN         lColLength;
    SQLRETURN			nRc;
    SQLSMALLINT			nColDecimal;
    SQLSMALLINT			nColNullable;

	char				sColName[MX_COLUMN_NAME];    
    
    //check it
	if( m_pError == NULL) return SAPDB_FALSE;
    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV) return SAPDB_FALSE;	
	

    //get number of columns
    nRc = SQLNumResultCols(m_hStmt, &m_nColCount);
    if( nRc != SQL_SUCCESS) {
    
        if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) return SAPDB_FALSE;		
	};    
    
    //get the column descriptions
    for( lRun = 1; lRun <= m_nColCount;lRun++ )
	{
    
        m_pColumn[lRun-1] = new StudioOAL_AColumn;

 		//get the individual column description
        nRc = SQLDescribeCol(m_hStmt,
							 lRun,
							 (unsigned char *) sColName, 
							 MX_COLUMN_NAME,
							 &nColNameLen, 
							 &nColType,
							 &lColLength,
							 &nColDecimal,
							 &nColNullable);
        if( nRc != SQL_SUCCESS ){        

               if( !(m_pError->checkSQLReturnCode(nRc,m_hStmt)) ) {    

				   return SAPDB_FALSE;
			   };
        };
        
        //set column description
		StudioOAL_String sBuffer;
		if (nColNameLen > 0){
				
			char* szpConvert = NULL;	
			sBuffer.ConvertFromASCII_Latin1(
							(StudioOAL_String::ASCIIConstPointer)sColName,
							(StudioOAL_String::ASCIIConstPointer)sColName+nColNameLen) ;
            m_pColumn[lRun-1]->setColName (sBuffer); 		
		}
		else
            m_pColumn[lRun-1]->setColName (sBuffer);
                
        m_pColumn[lRun-1]->setODBCColType(nColType);
        m_pColumn[lRun-1]->setColLength  (lColLength);
        m_pColumn[lRun-1]->setColDecimal (nColDecimal);
        
        if (nColNullable != 0)        
            m_pColumn[lRun-1]->setColNullable(SAPDB_TRUE);		        
		else
            m_pColumn[lRun-1]->setColNullable(SAPDB_FALSE);
    };

	for (lRun = 1023; lRun > m_nColCount;lRun-- ){

		if(m_pColumn[lRun] != NULL )
			delete m_pColumn[lRun];
		
		m_pColumn[lRun] = NULL;
	}
    
    
	return SAPDB_TRUE;

}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
StudioOAL_AColumn  * StudioOAL_AResult::getColumn( SAPDB_UInt2 nCol )
{ 
	assert (nCol <= m_nColCount && nCol+1 >= 1) ;
	return m_pColumn[nCol-1] ;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::getColumnName(SAPDB_UInt2 nCol,StudioOAL_String &sColName)
{
	
	//init
	sColName.Erase();
	if (nCol > m_nColCount || nCol < 1)	return SAPDB_FALSE;	
	m_pColumn[nCol-1]->getColName (sColName); 

	return SAPDB_TRUE;
	
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::execStmt() 
{    

	//check it
	if (m_pError == NULL)	return SAPDB_FALSE;		
    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV) return SAPDB_FALSE;	
	

	//Size()+1 for 0 termination
	char*	pAsciiStatement = new char[m_sStatement.Size()+1];
	char* szpConvert = NULL;
	memset(pAsciiStatement,0,m_sStatement.Size()+1);
	if(m_sStatement.ConvertToASCII_Latin1((char*)pAsciiStatement,pAsciiStatement+m_sStatement.Size(),szpConvert)
		!= Tools_UTF8Basis::Success){
		return SAPDB_FALSE;
	}	

	SQLRETURN nRc = SQLExecDirect(m_hStmt, (unsigned char*)(char*)pAsciiStatement,(SQLINTEGER) strlen(pAsciiStatement));
	if (nRc != SQL_SUCCESS) {
    
        if (!(m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

			delete[] pAsciiStatement;	

			return SAPDB_FALSE;
		};
	};    

	delete[] pAsciiStatement;	

	return SAPDB_TRUE;	
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::fetch() 
{

	SQLRETURN	nRc;
    
    //inits
    m_bNoRows = SAPDB_FALSE;

	//check it
	if (m_pError == NULL)	return SAPDB_FALSE;
		
    if (m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV || m_hStmt == SQL_NULL_HSTMT) return SAPDB_FALSE;	
    
 
    //fetch
	nRc = SQLFetch(m_hStmt);
    
    if (!(m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

		m_bNoRows = SAPDB_TRUE;			
		return SAPDB_FALSE;
	};
	    
        
	if (m_pError->getReturnCode() == SQL_NO_DATA_FOUND)    
        m_bNoRows = SAPDB_TRUE;				
	else
        m_lFetchedRows ++;


	return SAPDB_TRUE;

}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::extendedFetch(SAPDB_UInt2 lFetchType,SAPDB_ULong lRow,SAPDB_ULong *NoRowsFetched,SAPDB_UInt2 *nRowStatus) 
{

	SQLRETURN	nRc;
    
    //inits
    m_bNoRows = SAPDB_FALSE;
	*NoRowsFetched = 0;

	//check it
	if( m_pError == NULL) return SAPDB_FALSE;	
	
    if( m_hDbc == SQL_NULL_HDBC || 
		m_hEnv == SQL_NULL_HENV || 
		m_hStmt == SQL_NULL_HSTMT)  return SAPDB_FALSE;
	

	if (lFetchType == SQL_FETCH_LAST  || 
		lFetchType == SQL_FETCH_FIRST ||
		lFetchType == SQL_FETCH_ABSOLUTE )	m_lFetchedRows = 0;

	SQLULEN * pNoRowsFetched = NoRowsFetched;
		    
 
    //fetch
	nRc = SQLExtendedFetch(m_hStmt, lFetchType, lRow, pNoRowsFetched, nRowStatus); //!!IA64
    
    if (!(m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

		m_bNoRows = SAPDB_TRUE;			
		return SAPDB_FALSE;
	}
        
	if( m_pError->getReturnCode() == SQL_NO_DATA_FOUND)
    
        m_bNoRows = SAPDB_TRUE;			
	else
        m_lFetchedRows ++;	

	return SAPDB_TRUE;

}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::getLongData(SAPDB_UInt2 nCol,
										   SAPDB_Byte *pValue,
										   SAPDB_Long lBufferSize,
										   SAPDB_Long* lDataSize,
										   SAPDB_Bool* bDataTrunc)
{

	SQLRETURN			nRc = 0;
	SAPDB_Long			lValueLen = 0;	    
	
    if( m_hDbc == SQL_NULL_HDBC || 
		m_hEnv == SQL_NULL_HENV ||
		m_hStmt == SQL_NULL_HSTMT) 	return SAPDB_FALSE;	
        
    if (m_pColumn[nCol-1] == NULL)	return SAPDB_FALSE;	
	if( m_pError == NULL)	return SAPDB_FALSE;

	//get the data for the current col
    if(m_pColumn[nCol-1]->getODBCColType() != SQL_LONGVARBINARY &&
       m_pColumn[nCol-1]->getODBCColType() != SQL_LONGVARCHAR) 	return SAPDB_FALSE;
	
    
    m_pError->setNoSQLState(SQLSTATE_DATA_TRUNCATED);

    nRc = SQLGetData(m_hStmt, nCol, SQL_BINARY ,pValue, 
		             ( SQLINTEGER ) lBufferSize, 
					 &lValueLen); //!!IA64
	
	if( nRc != SQL_SUCCESS ){
    
		
        if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

			m_pError->setNoSQLState (SQLSTATE_SUCCESS);
			return SAPDB_FALSE;
		}
		m_pError->setNoSQLState (SQLSTATE_SUCCESS);                
    }
    

	//set value
    if (lValueLen > lBufferSize){
		
        m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);   
		*lDataSize  = lBufferSize;
		*bDataTrunc = SAPDB_TRUE;
    }
	else if(lValueLen <= lBufferSize  && lValueLen > 0){                        
        m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);
		*lDataSize = lValueLen;
		*bDataTrunc = SAPDB_FALSE;

	}else if (lValueLen == 0){
            m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);
			*lDataSize = 0;
			*bDataTrunc = SAPDB_FALSE;

    }else if (lValueLen == -1){
            strcpy((char*)pValue,REPOFNULLVALUE);
			m_pColumn[nCol-1]->setColIsNull(SAPDB_TRUE);
			*bDataTrunc = SAPDB_FALSE;
			*lDataSize = 0;
    }	

	return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::getColumnDataAsString(SAPDB_UInt2 nCol,StudioOAL_String &Value)
{
 
    SAPDB_Bool			bBoolValue;
	SQLRETURN			nRc = 0;

	char 				*pValue = NULL;
	SAPDB_Long			lGetDataLen = 0;
	SAPDB_Long          lValueLen = 0;
    
	Value.Erase();

    if( m_hDbc == SQL_NULL_HDBC || 
		m_hEnv == SQL_NULL_HENV ||
		m_hStmt == SQL_NULL_HSTMT) 	return SAPDB_FALSE;	
        
    if (m_pColumn[nCol-1] == NULL)	return SAPDB_FALSE;	
	if( m_pError == NULL)	return SAPDB_FALSE;	

    //get the data for the current col
    switch (m_pColumn[nCol-1]->getODBCColType())
	{    
        case SQL_DATE:
		case SQL_TIME:
        case SQL_TIMESTAMP:
		case SQL_DECIMAL:
		case SQL_NUMERIC:
        case SQL_REAL:
		case SQL_FLOAT:
        case SQL_DOUBLE:
		case SQL_INTEGER:
			        
			pValue = new char [MX_COLUMN_GENERIC_NUMERIC];
			if (pValue == NULL) return SAPDB_FALSE;			
			
            //get the current value
            nRc = SQLGetData(m_hStmt, nCol, SQL_CHAR, pValue, MX_COLUMN_GENERIC_NUMERIC,&lValueLen); //!!IA64
            if( nRc != SQL_SUCCESS) {
    
				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

					delete[] pValue;	
					return SAPDB_FALSE;
				};
			};                                
            lGetDataLen = lValueLen; 
			break;
                                   
	    case SQL_LONGVARBINARY:
        case SQL_LONGVARCHAR: 
			            
            if( m_lLimitForLong >= 0 ){

                 lGetDataLen = m_lLimitForLong;                 

				 //Limit for long column can overwrite max value for long column
                 if( m_lLimitForLong > MX_COLUMN_LONG_VALUE ) lGetDataLen = MX_COLUMN_LONG_VALUE;   
				 
            }
			else
                lGetDataLen = MX_COLUMN_LONG_VALUE;
            
				pValue = new char [lGetDataLen];
				if (pValue == NULL) return SAPDB_FALSE;			

				m_pError->setNoSQLState(SQLSTATE_DATA_TRUNCATED);
				nRc = SQLGetData(m_hStmt, nCol, SQL_CHAR,pValue, ( SQLINTEGER  ) lGetDataLen,  &lValueLen); //!!IA64
				if( nRc != SQL_SUCCESS ){            
					
					if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

						m_pError->setNoSQLState (SQLSTATE_SUCCESS);

						delete[] pValue;
						return SAPDB_FALSE;
					}
					m_pError->setNoSQLState (SQLSTATE_SUCCESS);                
				};

			break;

        case SQL_BIT:

			lGetDataLen = 6;
			//get the current value			
			pValue = new char [6];
			if (pValue == NULL) return SAPDB_FALSE;			
            
            //get the current value
            nRc = SQLGetData(m_hStmt, nCol, SQL_BIT, &bBoolValue,sizeof(SAPDB_Bool),  &lValueLen); //!!IA64
            if( nRc != SQL_SUCCESS) {
    
				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

					delete[] pValue;
					return SAPDB_FALSE;
				}
			}
            //set value
            if(lValueLen > 0){
                if(bBoolValue){
                
                    strcpy(pValue,"TRUE");
					lValueLen = 4;
                }
				else{
                    strcpy(pValue,"FALSE");
					lValueLen = 5;
                }                
            }
			

			break;

        default:
            //SQL_VARBINARY

            if( m_pColumn[nCol-1]->getODBCColType() == SQL_VARBINARY ||
				m_pColumn[nCol-1]->getODBCColType() == SQL_BINARY ){

                //will be transformed from hex to ascii 
                lGetDataLen = m_pColumn[nCol-1]->getColLength() * 2 + 1;
            }
			else
                lGetDataLen = m_pColumn[nCol-1]->getColLength() + 1;  

			pValue = new char [lGetDataLen];
			if (pValue == NULL) return SAPDB_FALSE;
			
            
            //get the current value
            nRc = SQLGetData(m_hStmt, nCol, SQL_CHAR, pValue, (SQLINTEGER)lGetDataLen,  &lValueLen); //!!IA64
            if( nRc != SQL_SUCCESS) {
    
				if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

					delete[] pValue;
					return SAPDB_FALSE;
				}
			}
            
	};//switch

	//set value
	if (lValueLen > lGetDataLen){         
		m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);                				

	}else if(lValueLen <= lGetDataLen && lValueLen > 0){   				
		m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);					

	}else if (lValueLen == 0){
		m_pColumn[nCol-1]->setColIsNull(SAPDB_FALSE);			 

	}else if (lValueLen == -1){
  		strcpy(pValue,REPOFNULLVALUE);
  		lValueLen = strlen(REPOFNULLVALUE);
  		m_pColumn[nCol-1]->setColIsNull(SAPDB_TRUE);
	}		
	
    //if data was truncated then get the possible size lGetDataLen
	if(lValueLen > lGetDataLen) 
	   lValueLen = lGetDataLen; 


	//return value
	if (pValue != NULL && lValueLen > 0){

		if(Value.ConvertFromASCII_Latin1(
					(StudioOAL_String::ASCIIConstPointer)pValue,
					(StudioOAL_String::ASCIIConstPointer)pValue+lValueLen) 
					!= Tools_UTF8Basis::Success){

			delete[] pValue;
			return SAPDB_FALSE;
		}
		
	}

	        
	delete[] pValue;		
	return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
const StudioOAL_String& StudioOAL_AResult::getErrorText()
{

	if( m_pError == NULL) return m_strEmty;	
	return m_pError->getErrorText();
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
const StudioOAL_String& StudioOAL_AResult::getSQLState()
{

	if( m_pError == NULL) return m_strEmty;	
	return m_pError->getSQLState();
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
const StudioOAL_String& StudioOAL_AResult::getSQLStateErrorNumber()
{

	if( m_pError == NULL) return m_strEmty;	
	return m_pError->getSQLStateErrorNumber();
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Long StudioOAL_AResult::getNativeError()
{

	if( m_pError == NULL) return 0;	
	return m_pError->getNativeError();
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
const StudioOAL_String& StudioOAL_AResult::getNativeErrorAsString()
{

	if( m_pError == NULL) return m_strEmty;	
	return m_pError->getNativeErrorAsString();
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::execute()
{
	if (m_pError == NULL)	
		return SAPDB_FALSE;		
    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV || m_hStmt == SQL_NULL_HSTMT ) 
		return SAPDB_FALSE;	
	
	// SQLExecute
	SQLRETURN nRc = SQLExecute(m_hStmt);

	if (nRc != SQL_SUCCESS) 
	{
    
        if (!(m_pError->checkSQLReturnCode(nRc,m_hStmt))) 
			return SAPDB_FALSE;
	};    

	return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::getParamData(SAPDB_Int2 *&nParamData)
{
	//check it
	if (m_pError == NULL)	
		return SAPDB_FALSE;		
    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV || m_hStmt == SQL_NULL_HSTMT ) 
		return SAPDB_FALSE;	

	SQLRETURN nRc =  SQLParamData(m_hStmt,(SQLPOINTER*)&nParamData);

	if (nRc != SQL_SUCCESS) 
	{
        if (!(m_pError->checkSQLReturnCode(nRc,m_hStmt))) 
			return SAPDB_FALSE;
	};  
	
	return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::putParamData(SQLPOINTER pValue,SQLINTEGER nValueLength)
{

	//check it
	if (m_pError == NULL)	
		return SAPDB_FALSE;		
    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV || m_hStmt == SQL_NULL_HSTMT  ) 
		return SAPDB_FALSE;	

	SQLRETURN nRc = SQLPutData(m_hStmt, pValue, nValueLength);
	
	if (nRc != SQL_SUCCESS) 
	{
        if (!(m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

			return SAPDB_FALSE;
		}
	} 

	return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool  StudioOAL_AResult::getColumnDescription()
{
	SQLUSMALLINT		lRun;    
    SQLSMALLINT			nColNameLen;
    SQLSMALLINT			nColType;
	SQLULEN		        lColLength;
    SQLRETURN			nRc;
    SQLSMALLINT			nColDecimal;
    SQLSMALLINT			nColNullable;
	char				sColName[MX_COLUMN_NAME];    
    
    //check it
	if( m_pError == NULL) return SAPDB_FALSE;
    if( m_hDbc == SQL_NULL_HDBC || m_hEnv == SQL_NULL_HENV) return SAPDB_FALSE;	
	

    //get number of columns
    nRc = SQLNumResultCols(m_hStmt, &m_nColCount);
    if( nRc != SQL_SUCCESS) {
    
        if(! (m_pError->checkSQLReturnCode(nRc,m_hStmt))) return SAPDB_FALSE;		
	};    
    
    //get the column descriptions
    for( lRun = 1; lRun <= m_nColCount;lRun++ )
	{
    
        m_pColumn[lRun-1] = new StudioOAL_AColumn;

 		//get the individual column description
        nRc = SQLDescribeCol(m_hStmt,
							 lRun,
							 (unsigned char *) sColName, 
							 MX_COLUMN_NAME,
							 &nColNameLen, 
							 &nColType,
							 &lColLength,
							 &nColDecimal,
							 &nColNullable);
        if( nRc != SQL_SUCCESS ){        

               if( !(m_pError->checkSQLReturnCode(nRc,m_hStmt)) ) {    

				   return SAPDB_FALSE;
			   };
        };
        
        //set column description
		StudioOAL_String sBuffer;
		if (nColNameLen > 0){
				
			char* szpConvert = NULL;	
			sBuffer.ConvertFromASCII_Latin1(
							(StudioOAL_String::ASCIIConstPointer)sColName,
							(StudioOAL_String::ASCIIConstPointer)sColName+nColNameLen) ;
            m_pColumn[lRun-1]->setColName (sBuffer); 		
		}
		else
            m_pColumn[lRun-1]->setColName (sBuffer);
                
        m_pColumn[lRun-1]->setODBCColType(nColType);
        m_pColumn[lRun-1]->setColLength(lColLength);
        m_pColumn[lRun-1]->setColDecimal(nColDecimal);
        
        if (nColNullable != 0)        
            m_pColumn[lRun-1]->setColNullable(SAPDB_TRUE);		        
		else
            m_pColumn[lRun-1]->setColNullable(SAPDB_FALSE);
    };

	for (lRun = 1023; lRun > m_nColCount;lRun-- ){

		if(m_pColumn[lRun] != NULL )
			delete m_pColumn[lRun];
		
		m_pColumn[lRun] = NULL;
	}
    
    
	return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::openParamResult(const StudioOAL_String &sCmdStr)
{
	
	//if there is a open result the close it
    if(m_hStmt != SQL_NULL_HSTMT)
	   closeResult();

	m_bDBProcedureResult = SAPDB_TRUE;
	StudioOAL_AParameters* poParameterDefinitions  = NULL;    
    
    //inits
	m_nColCount = 0;
    m_sStatement = sCmdStr;    

    if(fullParameterExecute()){                

        if(m_ParamList.size() < 1){
        
            m_lRowCount = 0;
            m_nColCount = 0;
            m_bNoRows = SAPDB_TRUE;
            return SAPDB_FALSE;
		}
	}
    else
        return SAPDB_FALSE;                        
                    
    m_nColCount = 0;
    m_bNoRows = SAPDB_FALSE;
    m_lRowCount = 1;
	StudioOAL_String sColName;
	char sParamNo[8];
   
	poParameterDefinitions = (StudioOAL_AParameters*)m_ParamList.first();
    while (poParameterDefinitions != NULL){
    
        m_pColumn[m_nColCount] = new StudioOAL_AColumn;
		
        //build colname
		 switch (poParameterDefinitions->getParameterType())
		 {        
		 case StudioOAL_AParameters::ptUnknown:
                sColName = PARAM_UNKNOWN;
				break;
            case StudioOAL_AParameters::ptIn:
                sColName = PARAM_IN;
				break;
            case StudioOAL_AParameters::ptInOut:
                sColName = PARAM_INOUT;
				break;
            case StudioOAL_AParameters::ptResultCol:
                sColName = PARAM_RESULTCOL;
				break;
            case StudioOAL_AParameters::ptOut:
                sColName = PARAM_OUT;
				break;
            case StudioOAL_AParameters::ptReturnValue:
                sColName = PARAM_RETURNVALUE;
				break;
			default:
				break;
        }
        
        sColName +=  " (";
		memset(sParamNo,0,8);	
		sprintf(sParamNo,"%d",poParameterDefinitions->getParameterNo());				
		sColName += sParamNo;
		sColName += ")";
		
        m_pColumn[m_nColCount]->setColName (sColName); 
		m_pColumn[m_nColCount]->setColNullable(SAPDB_FALSE);		                

		m_nColCount = m_nColCount + 1;        

		poParameterDefinitions = (StudioOAL_AParameters*)m_ParamList.next();
    }

	return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::fullParameterExecute()
{    
    
    SQLRETURN		nRc = SQL_SUCCESS;
    
    m_lFetchedRows = 0;
    m_bNoRows = SAPDB_TRUE;	

    //if there is a open result the close it
    if(m_hStmt != SQL_NULL_HSTMT)
	   closeResult();

	clearParameters();

    //allocate handle for statement    
	nRc = SQLAllocHandle(SQL_HANDLE_STMT,m_hDbc,&m_hStmt);
	if(nRc != SQL_SUCCESS)
	{	
		SQLFreeStmt(m_hStmt, SQL_DROP);
		m_hStmt = SQL_NULL_HSTMT;
		return SAPDB_FALSE;	
	}		
    if (!prepareForParameters()) {
    
        //free statement
        SQLFreeStmt(m_hStmt, SQL_DROP);
		m_hStmt = SQL_NULL_HSTMT;
        return SAPDB_FALSE;
	}

    //open the result set
    if(!execStmt() ) {

        //free statement
        SQLFreeStmt(m_hStmt, SQL_DROP);
		m_hStmt = SQL_NULL_HSTMT;
        return SAPDB_FALSE;
    };

    //free statement
    SQLFreeStmt(m_hStmt, SQL_DROP);
    m_hStmt = SQL_NULL_HSTMT;
    
    return SAPDB_TRUE;

}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::prepareForParameters()
{
    
    SAPDB_Int2 nRc= 0;    
        
    if( m_hStmt == SQL_NULL_HSTMT) return SAPDB_FALSE;
    
    //prepare statement

    char* szpConvert = NULL;
	char*	pAsciiStatement = new char[m_sStatement.Size()+1];
	memset(pAsciiStatement,0,m_sStatement.Size()+1);
	if(m_sStatement.ConvertToASCII_Latin1((char*)pAsciiStatement,pAsciiStatement+m_sStatement.Size(),szpConvert)
		!= Tools_UTF8Basis::Success){
		return SAPDB_FALSE;
	}	

    // SQLPrepare
	nRc = SQLPrepare(m_hStmt, (SQLCHAR*)(char*)pAsciiStatement,( SQLINTEGER )strlen(pAsciiStatement)); //!!IA64
    if( nRc != SQL_SUCCESS) {

		if (!(m_pError->checkSQLReturnCode(nRc,m_hStmt))) {    

			delete[] pAsciiStatement;
			return SAPDB_FALSE;    
		}
	}
    
    // SQLNumParams SQLDescribeParam
    if(!getParameterDescription()){

		delete[] pAsciiStatement;
		return SAPDB_FALSE;
	}
    
   	delete[] pAsciiStatement;
    return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::getParameterDescription() 
{
    
    SAPDB_Int2  nRc;
    SAPDB_Int2  nParameterCount;
    SAPDB_Int2  nParam;
    SAPDB_Int2  nColType;
    SAPDB_ULong lColLength; 
	SAPDB_Int2  nColDecimal;
    SAPDB_Int2  nColNullable;
    
	StudioOAL_AParameters  *poNewParamDefinition = NULL;
        
    nRc = SQLNumParams(m_hStmt, &nParameterCount);
    if( nRc != SQL_SUCCESS) return SAPDB_FALSE;
    
    for (nParam = 1 ;nParam <= nParameterCount; nParam ++)
	{
        nRc = SQLDescribeParam(m_hStmt, nParam, &nColType, &lColLength, &nColDecimal, &nColNullable);
        if( nRc != SQL_SUCCESS) return SAPDB_FALSE;
        
        poNewParamDefinition = new StudioOAL_AParameters(m_hStmt,nParam);                
        
        // -99 <= nColType >= +99) then parameter type is input
        // -9900 <= nColType >= -100 Or 100 <= nColType >= 9900) then parameter type is output
        // -10099 <= nColType >= -10001 Or 10001 <= nColType >= 10099) then parameter type is input/output
        poNewParamDefinition->setParameterType(StudioOAL_AParameters::ptUnknown);
        nColType *= (nColType > 0) ? 1 :-1;
        if( nColType > 99){

          if( nColType > 9900){

            poNewParamDefinition->setParameterType(StudioOAL_AParameters::ptInOut);
            if( nColType < 0)
              poNewParamDefinition->setColumnType(nColType + 10000);
            else
              poNewParamDefinition->setColumnType (nColType - 10000);
		  }
		  else{

			poNewParamDefinition->setParameterType(StudioOAL_AParameters::ptOut);
            poNewParamDefinition->setColumnType( nColType / 100);
          }
		}
		else{

          poNewParamDefinition->setParameterType(StudioOAL_AParameters::ptIn);
          poNewParamDefinition->setColumnType(nColType);
        }
		m_ParamList.append(poNewParamDefinition);
	}    	
    
    return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
void StudioOAL_AResult::clearParameters() 
{
    
    StudioOAL_AParameters  *poCurrentParam;
    
	poCurrentParam = (StudioOAL_AParameters*)m_ParamList.first();
    while (poCurrentParam != NULL){
		
		delete poCurrentParam;
		poCurrentParam = (StudioOAL_AParameters*)m_ParamList.next();
	} 

	m_ParamList.clear();
		
    return;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::getParamValue(SAPDB_Int2 nParamNo,StudioOAL_String &sValue)
{

	StudioOAL_AParameters  *poCurrentParam;
    
	poCurrentParam = (StudioOAL_AParameters*)m_ParamList.at(nParamNo);
    if (poCurrentParam != NULL)

		poCurrentParam->getValue(sValue);
	else		
		return SAPDB_FALSE;

	return SAPDB_TRUE;

}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::getCursorPos(SAPDB_Int4 &nCursorPos)
{
	if( m_hDbc == SQL_NULL_HDBC || 
		m_hEnv == SQL_NULL_HENV ||
		m_hStmt == SQL_NULL_HSTMT) 	return SAPDB_FALSE;	

    
	nCursorPos = 0;
	SQLINTEGER StringLengthPtr;
    SQLRETURN nRc = SQLGetStmtAttr(m_hStmt, SQL_ROW_NUMBER,&nCursorPos,SQL_IS_INTEGER,&StringLengthPtr);
    
    if (nRc != SQL_SUCCESS)	{
    
        if(!(m_pError->checkSQLReturnCode(nRc,m_hStmt))) {
        
            //invalid cursor state => cursor is behind last or before first row
            if( m_pError->getSQLStateErrorNumber() == (SAPDB_UTF8*)"24000") 
                return SAPDB_FALSE;
            else
                nCursorPos = 0;
            
        }
	}

	return SAPDB_TRUE;
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::isColNull(SAPDB_UInt2 nCol,SAPDB_Bool &bIsNull)
{
	if (nCol > m_nColCount || nCol < 1)	return SAPDB_FALSE;	
	bIsNull = m_pColumn[nCol-1]->getColIsNull();

	return SAPDB_TRUE;	
}
/*===========================================================================*
 *  StudioOAL_AResult::
 *===========================================================================*/
SAPDB_Bool StudioOAL_AResult::getODBCColType (SAPDB_UInt2 nCol,SQLSMALLINT &nColType)	
{
	if (nCol > m_nColCount || nCol < 1)	return SAPDB_FALSE;	
	nColType = m_pColumn[nCol-1]->getODBCColType();
	return SAPDB_TRUE;	
}

/*===========================================================================*
 *  END
 *===========================================================================*/