/*
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/
/*!**********************************************************************

  module: StudioWeb_StoredQueryTree.cpp

  -----------------------------------------------------------------------

  special area: WebQuery

  mail to:  sapdb-general@sap.com

  description:  class for stored webqueries from sqlstudio

  see also:		/

  -----------------------------------------------------------------------

  copyright:                Copyright by SAP AG, 2000


************************************************************************/

//html code fragments
#include "SAPDB/SQLStudio/WebQuery/StudioWeb_HtmlFragments.h"

#include "SAPDB/SQLStudio/WebQuery/StudioWeb_StoredQueryTree.hpp"
//error messages
#include "SAPDB/SQLStudio/WebQuery/StudioWeb_Error.h"



#define SQL1	     "SELECT TABLENAME FROM DOMAIN.TABLES WHERE OWNER ="
#define SQL2		 " AND TABLENAME ='"
#define SYSDBA		 "SYSDBA"
#define SIMPLEQUOTE	 "'"


StudioWeb_StoredQueryTree::StudioWeb_StoredQueryTree(Studio_DBFSObject			*poDBFS)
:m_poDBFS(poDBFS)
{	
	m_sRefreshPath.Erase(); 
}

StudioWeb_StoredQueryTree::~StudioWeb_StoredQueryTree()
{
	
}

SAPDB_Bool StudioWeb_StoredQueryTree::doService (sapdbwa_WebAgent     & wa,
												 sapdbwa_HttpRequest  & req,
	   											 sapdbwa_HttpReply    & rep)
{
	
	//init refresh path
	m_sRefreshPath.Erase();

	sendPageBegin(rep,WQ_MAINWINDOW_TAG);

	if(m_poDBFS == NULL){

		sendErrorMessageBox(wa,rep,(const char*)WQ_ERR026);
		sendPageEnd (rep);	
		return SAPDB_FALSE;
	}

	if(m_poDBFS->getConnection() == NULL){

		sendErrorMessageBox(wa,rep,(const char*)WQ_ERR009);
		sendPageEnd (rep);	
		return SAPDB_FALSE;
	}

	if(isCreateFolderRequest(req)){

		if(createFolder(wa,req,rep) == SAPDB_FALSE) return SAPDB_FALSE;		
	}
	else if(isDeleteFolderRequest (req)){

		if(deleteFolder(wa,req,rep) == SAPDB_FALSE) return SAPDB_FALSE;		
	}
	else if(isDeleteFileRequest (req)){

		if(deleteFile(wa,req,rep) == SAPDB_FALSE) return SAPDB_FALSE;		
	}
	else if(isRenameItemRequest (req)){

		if(renameItem(wa,req,rep) == SAPDB_FALSE) return SAPDB_FALSE;		
	}
	else if(isMoveToFolderRequest (req)){

		if(moveToFolder(wa,req,rep) == SAPDB_FALSE) return SAPDB_FALSE;		
	}
	else{
		sapdbwa_StringSeq paramNames;
		if (req.GetParameterNames(paramNames)) {

			if(paramNames[0] != NULL){
				//request for storing a dsql object
				if (strcmp((char *)paramNames[0],WQ_PATHTOSTORE) == 0){

					if(saveFile(wa,req,rep) == SAPDB_FALSE) return SAPDB_FALSE;							
				}
			}
		}
	}
	
	Tools_DynamicUTF8String sTree;
	SAPDB_Bool bTreeExists = buildTree(sTree);		

	if (bTreeExists) {
		
		Tools_DynamicUTF8String sUser = m_poDBFS->getConnection()->getDBUser();
		if(!sUser.Empty()){
			Tools_DynamicUTF8String tmp = sUser.SubStrElem(0,1);
			if(!tmp.Empty()){
				if(tmp == (SAPDB_Char*)"\""){
					sUser = sUser.SubStrElem(1,sUser.Size()-2);
				}
			}
		}
		Tools_DynamicUTF8String sName = sUser;
		sName += m_poDBFS->getConnection()->getDBName();

		rep.SendBody(WQ_JS_BEGIN);		
		//rep.SendBody(WQ_JS_NAVONE);
		rep.SendBody(WQ_JS_CLEAR_TREE);			
		rep.SendBody(WQ_JS_ADDNAV);
		//name
		rep.SendBody(sName.StrPtr());
		rep.SendBody("\", \"");

		//rootLabel
		sName = sUser;
		sName += WQ_WSTEXT_ON;
		sName += m_poDBFS->getConnection()->getDBName();
		rep.SendBody(sName.StrPtr());		

		//frameName, picturesURL, backgroundPicturesURL, withPlus, withToolTip, cookieDays		
		rep.SendBody("\", \"tree\", \"/WARoot/Images/\", \"\", true, true, 1));\n");			
		rep.SendBody(WQ_MARKSELECTEDITEM);		
		rep.SendBody(WQ_HREFASFUNCTION);		
	}		

	rep.SendBody(sTree.StrPtr(),sTree.BasisSize());

	if (bTreeExists) {		

		if(m_sRefreshPath.Size() > 0){

			rep.SendBody("parent.dsql.refreshDSQLStoredStatement('");
			escapeAsUTF8String(m_sRefreshPath);
			rep.SendBody(m_sRefreshPath.StrPtr());
			rep.SendBody("');");
		}		
		
		rep.SendBody(WQ_JS_END);
	}

	rep.SendBody(WQ_BODYOPEN2);
	rep.SendBody(WQ_BG_COLOR);
	if (bTreeExists){
		
		rep.SendBody(WQ_SHOWNAVIGATION);
	}
	
	rep.SendBody(WQ_CLOSETAG);

	sendPageEnd (rep);	

	return SAPDB_TRUE;
}


SAPDB_Bool StudioWeb_StoredQueryTree::checkForQueryTree()
{

	if(m_poDBFS == NULL) return SAPDB_FALSE;	
	if(m_poDBFS->getConnection() == NULL) return SAPDB_FALSE;	

	//check if inode table is installed

	SAPDB_Bool bFrameWithTree = checkUserNotExlusive(m_poDBFS->getConnection()->getDBCHandle(),m_poDBFS->getConnection()->getDBUser());
	if (bFrameWithTree == SAPDB_TRUE) {
		bFrameWithTree = checkForSystemTable(m_poDBFS->getConnection()->getDBCHandle(),(SAPDB_UTF8*)SYSDBA,(SAPDB_UTF8*)"WA_MY_INODE");
		if (bFrameWithTree == SAPDB_TRUE) {

			//check if container table is installed
			bFrameWithTree = checkForSystemTable(m_poDBFS->getConnection()->getDBCHandle(),(SAPDB_UTF8*)SYSDBA,(SAPDB_UTF8*)"WA_MY_CONTAINER");
		}
	}

	return bFrameWithTree;
}

SAPDB_Bool StudioWeb_StoredQueryTree::buildTree(Tools_DynamicUTF8String &sResponse)
{
	//varibles
	Tools_DynamicUTF8String sSubTree;	
	if (readFirstLevelTree(sSubTree) == SAPDB_FALSE){

		// error occurred, sSubTree contains error message
		sResponse = sSubTree;
		return SAPDB_FALSE;
	}
	else
		sResponse += sSubTree;		
	 
 
	return SAPDB_TRUE;   
}


SAPDB_Bool StudioWeb_StoredQueryTree::readFirstLevelTree(Tools_DynamicUTF8String &sResponse)
{
	
	//variables/////////////////////////////////////////////////////////////
	SAPDB_Int2 lfno	  = 0;
	SAPDB_Int2 nNodeCount = 0;
	sResponse.Erase();

	Tools_DynamicUTF8String sFolder = (SAPDB_UTF8*)WQ_STOREDQUERY_FOLDER;
	lfno = m_poDBFS->openDir(sFolder,sResponse);  
    if ( lfno == -1 )  return SAPDB_FALSE;
	////////////////////////////////////////////////////////////////////////

	//variables/////////////////////////////////////////////////////////////
	Tools_DynamicUTF8String sTreeItem;
	Tools_DynamicUTF8String sSubTree;
	//path in db file system
	Tools_DynamicUTF8String sPath;
	Tools_DynamicUTF8String sBuffer;
	Tools_DynamicUTF8String sError;

	//node path in java script
	Tools_DynamicUTF8String sJSPath = (SAPDB_UTF8*)WQ_JSROOT;		
	SAPDB_Bool bRet	 = SAPDB_FALSE;
	st_db_stat  Info;	
	////////////////////////////////////////////////////////////////////////

	bRet = m_poDBFS->getDirEntry(lfno,sBuffer);          
          
    while (bRet == SAPDB_TRUE )
	{   

		sPath = (SAPDB_UTF8*)WQ_STOREDQUERY_FOLDER;
		//if it is not root
		if (sPath != (SAPDB_UTF8*)"/")  sPath += (SAPDB_UTF8*)("/");		

		sPath += sBuffer; 

		if (m_poDBFS->status(sPath,Info,sError) == SAPDB_FALSE) break;

		//increase nodecount
		nNodeCount ++;

		//entry is a directory
        if (Info.mode == 1) {            

			// Define the items for the top-level of the tree menu.
			//parent.Navigations.items[0].addNode(new node("Test"));
			sResponse += sJSPath;
			sResponse += WQ_JS3;									
			sResponse += sBuffer;
			sResponse += (SAPDB_UTF8*) "\"));\n";
						
			//reinit
			sSubTree.Erase();
			//read directory and add items to tree
			if (readSubDir(sPath,sJSPath,nNodeCount,sSubTree) == SAPDB_FALSE){

				sResponse = sSubTree;
				return SAPDB_FALSE;
			}
			else
				sResponse += sSubTree;
		}
		else{
			sTreeItem.Erase();
			makeTreeItem((SAPDB_UTF8*)WQ_STOREDQUERY_FOLDER,
						 sJSPath,
						 sBuffer, 
						 -1,
						 sTreeItem);
			sResponse += sTreeItem;					
        }		
        bRet = m_poDBFS->getDirEntry(lfno,sBuffer);                  
	}    
      
	m_poDBFS->closeDir(lfno,sBuffer);

	return SAPDB_TRUE;       
}



SAPDB_Bool StudioWeb_StoredQueryTree::makeTreeItem(const Tools_DynamicUTF8String &sPath,
												   const Tools_DynamicUTF8String &sJSPath,
												   const Tools_DynamicUTF8String &sFileName,				  
												   SAPDB_Int2 nLevel,
												   Tools_DynamicUTF8String &sResponse)
{
	 
	

	char sBuf   [16];		
	Tools_DynamicUTF8String sCopyFileName;
	Tools_DynamicUTF8String sCopyPath;

	//init
	sResponse = sJSPath;
	if (nLevel > -1){

		sResponse += WQ_JS2;	 
		sprintf (sBuf,"%d",nLevel);
		sResponse +=  (SAPDB_UTF8*)sBuf;		
		sResponse += (SAPDB_UTF8*) "].";
	}
	sResponse += WQ_JS3;				
	sCopyFileName = sFileName;
	escapeAsUTF8String(sCopyFileName);
	sResponse += sCopyFileName; 
	sResponse += (SAPDB_UTF8*) "\",\"";
	sCopyPath = sPath;
	escapeAsUTF8String(sCopyPath);
	sResponse +=  sCopyPath;
	if(sPath != (SAPDB_UTF8*) "/")  sResponse += (SAPDB_UTF8*) "/";
	sCopyFileName = sFileName;		
	escapeAsUTF8String(sCopyFileName);
	sResponse +=  sCopyFileName;
	sResponse += (SAPDB_UTF8*) "\",\"";
	sResponse += WQ_DSQL_WINDOW;
	sResponse += (SAPDB_UTF8*) "\"));\r\n";
	
	return SAPDB_TRUE;
}





SAPDB_Bool StudioWeb_StoredQueryTree::readSubDir(const Tools_DynamicUTF8String &sParentPath,
												  Tools_DynamicUTF8String &sJSPath,
												  const SAPDB_Int2 nNodeCount,
												  Tools_DynamicUTF8String &sSubTree)
{    
	Tools_DynamicUTF8String sSubTree2;
	Tools_DynamicUTF8String sTreeItem;
	Tools_DynamicUTF8String	sPath;
	Tools_DynamicUTF8String	sBuffer;
	Tools_DynamicUTF8String sError;
	

	//make a copy for recursion
	Tools_DynamicUTF8String	sJSPath2 = sJSPath;
	
	char sBuf   [16];	
	SAPDB_Bool bRet    = SAPDB_FALSE;
	SAPDB_Int2 lfno    = 0;
	SAPDB_Int2 nNodeCount2 = 0;
	
	st_db_stat  Info;	
	
	lfno = m_poDBFS->openDir(sParentPath,sError);  
    if ( lfno == -1 )  return SAPDB_FALSE;      
	
	bRet = m_poDBFS->getDirEntry(lfno,sBuffer);	
      
    while (bRet == SAPDB_TRUE){   	
		
		nNodeCount2 ++;

		sPath = sParentPath;		
		//if it is not root
		if (sPath != (SAPDB_UTF8*)"/")  sPath += (SAPDB_UTF8*)"/";		
		
		sPath += sBuffer;
		//get some properties of the dir content
		
		if (m_poDBFS->status(sPath,Info,sError) == SAPDB_FALSE) break;

		//entry is a directory
        if (Info.mode == 1) {                
			
			//e.g.: parent.Navigations.items[0].nodes[0].nodes[0].nodes[1].addNode(new parent.node(\"SP\", \"waadmin\", \"Details\"));*/
			//build the new java script node path

			//set the start node in every loop
			sJSPath2 = sJSPath;
			sJSPath2 += WQ_JS2;
			sprintf (sBuf,"%d",(nNodeCount-1));
			sJSPath2 += (SAPDB_UTF8*)sBuf;
			sJSPath2 += (SAPDB_UTF8*) "].";

			sSubTree +=	sJSPath2;
			sSubTree += WQ_JS3;						
			sSubTree += sBuffer;
			sSubTree += (SAPDB_UTF8*) "\"));\n";

			//reinit
			sSubTree2.Erase();
			if (readSubDir(sPath,sJSPath2,nNodeCount2,sSubTree2) == SAPDB_FALSE){

				sSubTree = sSubTree2;
				return SAPDB_FALSE;
			}
			else				
				sSubTree +=sSubTree2;	//e.g. "parent.Navigations.items[0].nodes[2].nodes[0].addNode(new parent.node(\"Hallo\"));";
		}
		else{	
			sTreeItem.Erase();
			makeTreeItem(sParentPath,
						 sJSPath,
						 sBuffer, 
						 nNodeCount-1,
						 sTreeItem);
			sSubTree += sTreeItem;			
        }
		
        bRet = m_poDBFS->getDirEntry(lfno,sBuffer);
	} 
	return (m_poDBFS->closeDir(lfno,sError));    
}



SAPDB_Bool StudioWeb_StoredQueryTree::createFolder(sapdbwa_WebAgent     &wa,
												   sapdbwa_HttpRequest  &req,
												   sapdbwa_HttpReply    &rep)
{

	const char		*queryString = req.GetQueryString();

	if (queryString == NULL) return SAPDB_FALSE;	
	
	Tools_DynamicUTF8String sName;
	getParameterValue(WQ_ACTIONTOTAKE,req,sName); 

	if(sName.Empty()) return SAPDB_FALSE;

	Tools_DynamicUTF8String sError;
	if(m_poDBFS->createFolder(sName,sError) == SAPDB_FALSE){

		sendUtf8ErrorMessageBox(wa,rep,sError);
		return SAPDB_FALSE;
	}

	return SAPDB_TRUE;	
}



SAPDB_Bool StudioWeb_StoredQueryTree::deleteFolder(sapdbwa_WebAgent     &wa,
												   sapdbwa_HttpRequest  &req,
												   sapdbwa_HttpReply    &rep)
{

	const char		*queryString = req.GetQueryString();

	if (queryString == NULL) return SAPDB_FALSE;	
	
	Tools_DynamicUTF8String sName;
	getParameterValue(WQ_ACTIONTOTAKE,req,sName); 

	if(sName.Empty()) return SAPDB_FALSE;

	Tools_DynamicUTF8String sError;
	if(m_poDBFS->deleteFolder(sName,sError) == SAPDB_FALSE){

		sendUtf8ErrorMessageBox(wa,rep,sError);
		return SAPDB_FALSE;
	}

	return SAPDB_TRUE;	
}

SAPDB_Bool StudioWeb_StoredQueryTree::renameItem(sapdbwa_WebAgent     &wa,
												 sapdbwa_HttpRequest  &req,
												 sapdbwa_HttpReply    &rep)
{

	const char		*queryString = req.GetQueryString();

	if (queryString == NULL) return SAPDB_FALSE;	
	
	Tools_DynamicUTF8String sOldName;
	Tools_DynamicUTF8String sNewName;

	getParameterValue(WQ_ACTIONTOTAKE,req,sOldName); 
	if(sOldName.Empty()) return SAPDB_FALSE;

	getParameterValue(WQ_NEWITEM,req,sNewName); 
	if(sNewName.Empty()) return SAPDB_FALSE;	

	//determine file type
	st_db_stat Info;
	Tools_DynamicUTF8String sError;
	if(m_poDBFS->status(sOldName,Info,sError) == SAPDB_FALSE){
			
		sendUtf8ErrorMessageBox(wa,rep,sError);
		return SAPDB_FALSE;
	}
	
	//now change file name
	if(m_poDBFS->renameItem(sOldName,sNewName,sError) == SAPDB_FALSE){

		sendUtf8ErrorMessageBox(wa,rep,sError);
		return SAPDB_FALSE;
	}

	if(Info.mode == DB_FIL)
		//set the rename path for refreshing the sql window
		m_sRefreshPath = sNewName;

	
	return SAPDB_TRUE;	
}

SAPDB_Bool StudioWeb_StoredQueryTree::deleteFile(sapdbwa_WebAgent     &wa,
												 sapdbwa_HttpRequest  &req,
												 sapdbwa_HttpReply    &rep)
{

	const char		*queryString = req.GetQueryString();

	if (queryString == NULL) return SAPDB_FALSE;	
	
	Tools_DynamicUTF8String sName;
	getParameterValue(WQ_ACTIONTOTAKE,req,sName); 
	
	if(sName.Empty()) return SAPDB_FALSE;
	

	Tools_DynamicUTF8String sError;
	if(m_poDBFS->deleteFile(sName,sError) == SAPDB_FALSE){

		sendUtf8ErrorMessageBox(wa,rep,sError);		
		return SAPDB_FALSE;
	}

	return SAPDB_TRUE;	
}

SAPDB_Bool StudioWeb_StoredQueryTree::moveToFolder(sapdbwa_WebAgent     &wa,
												 sapdbwa_HttpRequest  &req,
												 sapdbwa_HttpReply    &rep)
{

	const char		*queryString = req.GetQueryString();

	if (queryString == NULL) return SAPDB_FALSE;
	
	
	Tools_DynamicUTF8String sOldName;
	Tools_DynamicUTF8String sNewName;

	getParameterValue(WQ_ACTIONTOTAKE,req,sOldName); 
	if(sOldName.Empty()) return SAPDB_FALSE;

	getParameterValue(WQ_NEWITEM,req,sNewName); 
	if(sNewName.Empty()) return SAPDB_FALSE;
	
	//determine file type
	st_db_stat Info;
	Tools_DynamicUTF8String sError;
	if(m_poDBFS->status(sOldName,Info,sError) == SAPDB_FALSE){
			
		sendUtf8ErrorMessageBox(wa,rep,sError);
		return SAPDB_FALSE;
	}

	if(m_poDBFS->moveItem(sOldName,sNewName,sError) == SAPDB_FALSE){

		sendUtf8ErrorMessageBox(wa,rep,sError);
		return SAPDB_FALSE;
	}
	
	return SAPDB_TRUE;	
}



SAPDB_Bool StudioWeb_StoredQueryTree::copyToFolder(sapdbwa_WebAgent     &wa,
												 sapdbwa_HttpRequest  &req,
												 sapdbwa_HttpReply    &rep)
{

	const char		*queryString = req.GetQueryString();

	if (queryString == NULL) return SAPDB_FALSE;	
	
	Tools_DynamicUTF8String sOldName;
	Tools_DynamicUTF8String sNewName;
	
	getParameterValue(WQ_ACTIONTOTAKE,req,sOldName); 
	if(sOldName.Empty()) return SAPDB_FALSE;

	getParameterValue(WQ_NEWITEM,req,sNewName); 
	if(sNewName.Empty()) return SAPDB_FALSE;	

	Tools_DynamicUTF8String sError;
	if(m_poDBFS->copyItem(sOldName,sNewName,sError) == SAPDB_FALSE){

		sendUtf8ErrorMessageBox(wa,rep,sError);
		return SAPDB_FALSE;
	}
	
	return SAPDB_TRUE;	
}


SAPDB_Bool StudioWeb_StoredQueryTree::saveFile(sapdbwa_WebAgent     &wa,
											   sapdbwa_HttpRequest  &req,
											   sapdbwa_HttpReply    &rep)
{

	const char		*queryString = req.GetQueryString();

	if (queryString == NULL) return SAPDB_FALSE;	
	
	Tools_DynamicUTF8String  sStatement;
	getParameterValue(WQ_SQLQUERYTEXT,req,sStatement);
	
	Tools_DynamicUTF8String  sCompletePath;
	getParameterValue(WQ_PATHTOSTORE,req,sCompletePath);
	if(sCompletePath.Empty()){

		sendErrorMessageBox(wa,rep,(const char*)WQ_ERR027);
		return SAPDB_FALSE;	
	}

	Tools_DynamicUTF8String  sOverWrite;
	getParameterValue(WQ_OVERWRITE,req,sOverWrite);
	if(sOverWrite.Empty()){

		sendErrorMessageBox(wa,rep,(const char*)WQ_ERR034);
		return SAPDB_FALSE;	
	}
		
	//check if file exists
	st_db_stat Info;
	SAPDB_Bool bOverWrite = SAPDB_FALSE;
	Tools_DynamicUTF8String sError;
	if(m_poDBFS->status(sCompletePath,Info,sError) == SAPDB_TRUE){
		
		if(Info.mode == DB_FIL){
			
			if(sOverWrite == WQ_NO){

				sendErrorMessageBox(wa,rep,(const char*)WQ_ERR001);
				return SAPDB_FALSE;
			}
			bOverWrite = SAPDB_TRUE;		
		}
	}

	Studio_StoredFileContent *poSFC = new Studio_StoredFileContent;
	if(poSFC == NULL){

		sendErrorMessageBox(wa,rep,(const char*)WQ_ERR026);
		return SAPDB_FALSE;
	}

	if(bOverWrite == SAPDB_TRUE){

		// first read stored sql statement to get information like 
		// isolevel,outocommit off ...
		// this information would be lost if just the sql information is stored
		Tools_DynamicUTF8String  sBuffer;
		SAPDB_Int4 lBytesRead = m_poDBFS->readFile(sCompletePath,4096,sBuffer);
		if (lBytesRead < 0){

			sendUtf8ErrorMessageBox(wa,rep,sBuffer);
			return SAPDB_FALSE;
		}

		if (poSFC->setObjectFromBuffer(sBuffer) == 0){

			sBuffer = (SAPDB_UTF8*)WQ_ERR012;
			sBuffer += sCompletePath;
			sendUtf8ErrorMessageBox(wa,rep,sBuffer);
			return SAPDB_FALSE;
		}				

		if(poSFC->getObjectType() != Studio_StoredFileContent::storedObjectTypeStatement){

			sBuffer = (SAPDB_UTF8*)WQ_ERR035;
			sendUtf8ErrorMessageBox(wa,rep,sBuffer);
			return SAPDB_FALSE;
		}
	}

	//the procedure to store the tree items is compatible to SQLStudio
	//items can be read/written in SQLStudio and webSQL
	poSFC->setIconType(Studio_StoredFileContent::iconStoredDSQL);
	poSFC->setObjectType(Studio_StoredFileContent::storedObjectTypeStatement);
	initSOFromRequest(poSFC,req);

	SAPDB_Bool bRet = storeDSQLObject(wa,rep,sCompletePath,poSFC,bOverWrite);

	//set the rename path for refreshing the sql window if a new file was stored
	if(bRet == SAPDB_TRUE && bOverWrite == SAPDB_FALSE) m_sRefreshPath = sCompletePath;

	//delete SO object
	delete poSFC;

	return bRet;
		
}


		


SAPDB_Bool StudioWeb_StoredQueryTree::storeDSQLObject(sapdbwa_WebAgent     &wa,
													  sapdbwa_HttpReply    &rep,
													  const Tools_DynamicUTF8String  &sCompletePath,
													  Studio_StoredFileContent *poSFC,
													  SAPDB_Bool bOverWrite)
{

	//check if file exists
	Tools_DynamicUTF8String sError;

	SAPDB_Int2 lfno = -1;
	if(bOverWrite){

		lfno = m_poDBFS->openFile(sCompletePath,sError);
	}
	else{
		lfno = m_poDBFS->createFile(sCompletePath,sError);
	}
	if(lfno == -1){

		sendUtf8ErrorMessageBox(wa,rep,sError);
		return SAPDB_FALSE;
	}
	
	Tools_DynamicUTF8String  sBuffer;	
	poSFC->getObjectToBuffer(sBuffer);
	
	SAPDB_Int4 byteswritten = m_poDBFS->writeFile(lfno,
												  (void*)sBuffer.StrPtr(),
												  sBuffer.BasisSize(),
												  sError);

	if(byteswritten < 0){

		m_poDBFS->closeFile(lfno,sError);
		sendUtf8ErrorMessageBox(wa,rep,sError);		
		return SAPDB_FALSE;

	}

	if(m_poDBFS->closeFile(lfno,sError) == SAPDB_FALSE){

		sendUtf8ErrorMessageBox(wa,rep,sError);
		return SAPDB_FALSE;
	}
	
	return SAPDB_TRUE;
}

