/****************************************************************************

  module      : SysView_ClassContainers.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "ClassContainers" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_ClassContainers.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_ClassContainers::Create()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_ClassContainers::Create", SysView_Trace, 5);

	m_Table->AppendCol (ITOCT_CHARBYTE ,SV_FILEID, 6, 8);
	m_Table->AppendCol (ITOCT_CHAR ,SV_CLASSGUID, 9, 40);
	m_Table->AppendCol (ITOCT_CHARUNICODE ,SV_SCHEMANAME, 10, 32);
	m_Table->AppendCol (ITOCT_FIXED ,SV_CONTAINERNUMBER, 15, 10);
	m_Table->AppendCol (ITOCT_CHAR ,SV_CLASSNAME, 9, 32);
	m_Table->AppendCol (ITOCT_FIXED ,SV_OBJECTSIZE, 10, 5);
	m_Table->AppendCol (ITOCT_CHAR ,SV_VAROBJECTCONTAINER, 18, 3);
	m_Table->AppendCol (ITOCT_CHAR ,SV_KEYDEFINED, 10, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_KEYPARTITIONCOUNT, 17, 5);
	m_Table->AppendCol (ITOCT_TIMESTAMP ,SV_CREATEDATE, 10, 0);
	m_Table->AppendCol (ITOCT_LONG ,SV_COMMENT, 7, 0);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_ClassContainers::GetColCount()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_ClassContainers::GetColCount", SysView_Trace, 5);

    return SV_CC_CLASSCONTAINERS;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_ClassContainers::EstimateRows()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_ClassContainers::EstimateRows", SysView_Trace, 5);

    return SV_ER_CLASSCONTAINERS;
}

/*---------------------------------------------------------------------------*/

void SysView_ClassContainers::Execute()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_ClassContainers::Execute", SysView_Trace, 5);

    m_Session->ThrowError (e_not_implemented, 1);
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
