/****************************************************************************

  module      : SysView_DataStatistics.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "DataStatistics" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_DataStatistics.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_DataStatistics::Create()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_DataStatistics::Create", SysView_Trace, 5);

	m_Table->AppendCol (ITOCT_FIXED ,SV_USABLESIZE, 10, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDSIZE, 8, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDSIZEPERCENTAGE, 18, 3);
	m_Table->AppendCol (ITOCT_CHAR ,SV_DATAFULL, 8, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDSIZEONVOLUMES, 17, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDSIZENOTONVOLUMES, 20, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDPERMANENTSIZE, 17, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDTEMPORARYSIZE, 17, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXUSEDSIZE, 11, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXUSEDPERMANENTSIZE, 20, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXUSEDTEMPORARYSIZE, 20, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_DIFFERENTIALBACKUPSIZE, 22, 20);
	m_Table->AppendCol (ITOCT_TIMESTAMP ,SV_LASTEMERGENCYHISTORYCLEANUP, 27, 0);
	m_Table->AppendCol (ITOCT_FIXED ,SV_CONVERTERVERSION, 16, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXDYNAMICDATAPAGENUMBER, 24, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXSTATICDATAPAGENUMBER, 23, 10);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_DataStatistics::GetColCount()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_DataStatistics::GetColCount", SysView_Trace, 5);

    return SV_CC_DATASTATISTICS;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_DataStatistics::EstimateRows()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_DataStatistics::EstimateRows", SysView_Trace, 5);

    return SV_ER_DATASTATISTICS;
}

/*---------------------------------------------------------------------------*/

void SysView_DataStatistics::Execute()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_DataStatistics::Execute", SysView_Trace, 5);

    m_Session->ThrowError (e_not_implemented, 1);
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
