/****************************************************************************

  module      : SysView_DataVolumes.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "DataVolumes" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "hgg01_3.h"
#include "IOManager/IOMan_IDataVolumeInfo.hpp"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_DataVolumes.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_DataVolumes::Create(void)
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_DataVolumes::Create", SysView_Trace, 5);

	m_Table->AppendCol (ITOCT_FIXED, SV_ID, 2, 5);
	m_Table->AppendCol (ITOCT_CHARUNICODE, SV_PATH, 4, 256);
	m_Table->AppendCol (ITOCT_FIXED, SV_CONFIGUREDSIZE, 14, 20);
	m_Table->AppendCol (ITOCT_FIXED, SV_USABLESIZE, 10, 20);
	m_Table->AppendCol (ITOCT_FIXED, SV_USEDSIZE, 8, 20);
	m_Table->AppendCol (ITOCT_FIXED, SV_USEDSIZEPERCENTAGE, 18, 3);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_DataVolumes::GetColCount(void)
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_DataVolumes::GetColCount", SysView_Trace, 5);

	return SV_CC_DATAVOLUMES;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_DataVolumes::EstimateRows(void)
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_DataVolumes::EstimateRows", SysView_Trace, 5);

	return SV_ER_DATAVOLUMES;
}

/*---------------------------------------------------------------------------*/

void SysView_DataVolumes::Execute(void)
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_DataVolumes::Execute", SysView_Trace, 5);

    IOMan_DataVolumeInfoIterator     iter;
    SAPDB_UInt4                      devNo;
    tsp00_VFilename                  devName;
    SAPDB_UInt4                      devSizeConfigured;
    SAPDB_UInt4                      devSizeUsable;
    SAPDB_UInt4                      devSizeUsed;
    SAPDB_UInt4                      devUsedPercent;
    

    m_Table->GetCatalogTable();
    
    iter.Begin();
    
    while ((!iter.End()) && (m_Session->GetCommandResult() == 0))
    {
        if ((*iter).IsVolumeOnline())
        {
            if ((*iter).GetVolumeName( devName ))
            {
                devNo = (SAPDB_UInt4)((*iter).GetVolumeNo());
                devSizeConfigured = (SAPDB_UInt4)(((*iter).GetVolumeSize() * (SAPDB_UInt4)(g01page_size / 1024)));
                devSizeUsable     = (SAPDB_UInt4)(((*iter).GetUsableVolumeSize() * (SAPDB_UInt4)(g01page_size / 1024)));
                devSizeUsed		= (SAPDB_UInt4)(((*iter).GetUsedVolumeSize() * (SAPDB_UInt4)(g01page_size / 1024)));
                
                if (devSizeUsable > 0)
                {
                    devUsedPercent = (SAPDB_UInt4)(((devSizeUsed * 100) / devSizeUsable)); 
                }
                else
                {
                    devUsedPercent = 0;
                }
                
                m_Table->MoveToCol (ITOVT_INT4, &devNo, 0);
                m_Table->MoveToCol (ITOVT_UTF8PTR, devName.asCharp(), devName.length());
                m_Table->MoveToCol (ITOVT_UINT4, &devSizeConfigured, 0);
                m_Table->MoveToCol (ITOVT_UINT4, &devSizeUsable, 0);
                m_Table->MoveToCol (ITOVT_UINT4, &devSizeUsed, 0);
                m_Table->MoveToCol (ITOVT_UINT4, &devUsedPercent, 0);
                
            }
            else
            {
                m_Session->ThrowError (e_devname_invalid, 1);
            }
        }
        
        ++iter;
    }
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
