/****************************************************************************

  module      : SysView_Files.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "Files" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_Files.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_Files::Create()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_Files::Create", SysView_Trace, 5);

	m_Table->AppendCol (ITOCT_CHARBYTE ,SV_FILEID, 6, 8);
	m_Table->AppendCol (ITOCT_CHARUNICODE ,SV_OWNER, 5, 32);
	m_Table->AppendCol (ITOCT_CHARUNICODE ,SV_SCHEMANAME, 10, 32);
	m_Table->AppendCol (ITOCT_CHARUNICODE ,SV_NAME, 4, 32);
	m_Table->AppendCol (ITOCT_CHARUNICODE ,SV_INDEXNAME, 9, 32);
	m_Table->AppendCol (ITOCT_FIXED ,SV_ROOT, 4, 10);
	m_Table->AppendCol (ITOCT_CHAR ,SV_TYPE, 4, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDSIZE, 8, 20);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_Files::GetColCount()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_Files::GetColCount", SysView_Trace, 5);

    return SV_CC_FILES;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_Files::EstimateRows()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_Files::EstimateRows", SysView_Trace, 5);

    return SV_ER_FILES;
}

/*---------------------------------------------------------------------------*/

void SysView_Files::Execute()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_Files::Execute", SysView_Trace, 5);

    m_Session->ThrowError (e_not_implemented, 1);
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
