/****************************************************************************

  module      : SysView_IOThreadStatistics.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "IOThreadStatistics" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_IOThreadStatistics.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_IOThreadStatistics::Create()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_IOThreadStatistics::Create", SysView_Trace, 5);

	m_Table->AppendCol (ITOCT_FIXED ,SV_THREADID, 8, 10);
	m_Table->AppendCol (ITOCT_CHAR ,SV_THREADNAME, 10, 16);
	m_Table->AppendCol (ITOCT_FIXED ,SV_READCOUNT, 9, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_READPAGECOUNT, 13, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_READTIME, 8, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WRITECOUNT, 10, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WRITEPAGECOUNT, 14, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WRITETIME, 9, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_PENDINGIOCOUNT, 14, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXPENDINGIOCOUNT, 17, 5);
	m_Table->AppendCol (ITOCT_CHAR ,SV_PATH, 4, 256);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_IOThreadStatistics::GetColCount()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_IOThreadStatistics::GetColCount", SysView_Trace, 5);

    return SV_CC_IOTHREADSTATISTICS;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_IOThreadStatistics::EstimateRows()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_IOThreadStatistics::EstimateRows", SysView_Trace, 5);

    return SV_ER_IOTHREADSTATISTICS;
}

/*---------------------------------------------------------------------------*/

void SysView_IOThreadStatistics::Execute()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_IOThreadStatistics::Execute", SysView_Trace, 5);

    m_Session->ThrowError (e_not_implemented, 1);
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
