/*!**************************************************************************

  module      : SysView_ISessionObj.hpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This contains an abstract interface the session object.

  last changed: 2002-08-13
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/


#ifndef SYSVIEW_ISESSIONOBJ_HPP
#define SYSVIEW_ISESSIONOBJ_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SAPDBCommon/SAPDB_Types.hpp"

 /*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

struct tgg00_TransContext;
struct tak_all_command_glob;
class SAPDBMem_IRawAllocator;

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/


/*!***************************************************************************

   class: SysView_ISysViewObj

   description: This interface class provides an interface for the SystemView
                table object.

******************************************************************************/

class SysView_ISessionObj
{

public:

    /* -----------------------------------------------------------------------*/
    /*! chapter: Constructors and initialization                              */
    /* -----------------------------------------------------------------------*/

    /*!-------------------------------------------------------------------------
    function    : ~SysView_ISessionObj()
    description : Class destructor. Is not used in the moment.
    arguments   : none
    return value: none
    --------------------------------------------------------------------------*/

    virtual ~SysView_ISessionObj()
    {}

    /* -----------------------------------------------------------------------*/
    /*! endchapter: Constructors and initialization                           */
    /* -----------------------------------------------------------------------*/

    /* -----------------------------------------------------------------------*/
    /*! chapter: Virtual functions                                            */
    /* -----------------------------------------------------------------------*/

    /*!-------------------------------------------------------------------------
    function    : GetPascalObj()
    description : Returns the underlying pascal object. Should be avoided!
    arguments   : none
    return value: [tak_all_command_glob] - AKA acv
    --------------------------------------------------------------------------*/

    virtual tak_all_command_glob * GetPascalObjPtr()
    { return (NULL); }

    /*!-------------------------------------------------------------------------
    function    : GetTransContextPtr()
    description : Returns the trans context object. 
    arguments   : none
    return value: [tgg00_TransContext] - trans context pointer
    --------------------------------------------------------------------------*/

    virtual tgg00_TransContext * GetTransContextPtr()
    { return (NULL); }

    /*!-------------------------------------------------------------------------
    function    : GetAllocatorPtr()
    description : Returns the local memory allocator pointer. 
    arguments   : none
    return value: [SAPDBMem_IRawAllocator] - local memory allocator pointer
    --------------------------------------------------------------------------*/

    virtual SAPDBMem_IRawAllocator * GetAllocatorPtr()
    { return (NULL); }

    /*!-------------------------------------------------------------------------
    function    : GetCommandResult()
    description : This function returns the actual result of the latest executed
                  command.
    arguments   : none
    return value: none
    --------------------------------------------------------------------------*/

    virtual SAPDB_Int4 GetCommandResult ( void )
    { return(0); }

    /*!-------------------------------------------------------------------------
    function    : SetCommandResult()
    description : This function sets a new command result code.
    arguments   : none
    return value: [SAPDB_Int4] - actual result code
    --------------------------------------------------------------------------*/

    virtual void SetCommandResult ( const SAPDB_Int4 ccode )
    {}

    /*!-------------------------------------------------------------------------
    function    : ThrowError()
    description : This function produces an internal system error.
    arguments   : [iError] - error code
                  [iOffet] - offset in SQL statement
    return value: none
    --------------------------------------------------------------------------*/

    virtual void ThrowError ( const SAPDB_Int2          iError,
                              const SAPDB_Int4          iOffset)
    {}

    /* -----------------------------------------------------------------------*/
    /*! endchapter: Virtual functions                                         */
    /* -----------------------------------------------------------------------*/

protected:

    /*!-------------------------------------------------------------------------
    function    : SysView_ISessionObj()
    description : This constructor ensures that no-one will instanciate this
                  class.
    arguments   : none
    return value: none
    --------------------------------------------------------------------------*/

	SysView_ISessionObj()
    {}


};


#endif  /* SYSVIEW_ISESSIONOBJ_HPP */

