/****************************************************************************

  module      : SysView_IndexSizeDetails.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "IndexSizeDetails" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_IndexSizeDetails.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_IndexSizeDetails::Create()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_IndexSizeDetails::Create", SysView_Trace, 5);

	m_Table->AppendCol (ITOCT_CHARUNICODE ,SV_OWNER, 5, 32);
	m_Table->AppendCol (ITOCT_CHARUNICODE ,SV_TABLENAME, 9, 32);
	m_Table->AppendCol (ITOCT_CHARUNICODE ,SV_INDEXNAME, 9, 32);
	m_Table->AppendCol (ITOCT_CHARUNICODE ,SV_COLUMNNAME, 10, 32);
	m_Table->AppendCol (ITOCT_FIXED ,SV_ROOT, 4, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDSIZE, 8, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDDATASIZE, 12, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_TREELEVELCOUNT, 14, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_AVGUSEDSIZEPERCENTAGEINLEAVES, 29, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MINUSEDSIZEPERCENTAGEINLEAVES, 29, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXUSEDSIZEPERCENTAGEINLEAVES, 29, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SECONDARYKEYCOUNT, 17, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_AVGSECONDARYKEYLENGTH, 21, 4);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MINSECONDARYKEYLENGTH, 21, 4);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXSECONDARYKEYLENGTH, 21, 4);
	m_Table->AppendCol (ITOCT_FIXED ,SV_AVGSEPARATORLENGTH, 18, 4);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MINSEPARATORLENGTH, 18, 4);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXSEPARATORLENGTH, 18, 4);
	m_Table->AppendCol (ITOCT_FIXED ,SV_AVGROWCOUNTPERSECONDARYKEY, 26, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MINROWCOUNTPERSECONDARYKEY, 26, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXROWCOUNTPERSECONDARYKEY, 26, 10);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_IndexSizeDetails::GetColCount()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_IndexSizeDetails::GetColCount", SysView_Trace, 5);

    return SV_CC_INDEXSIZEDETAILS;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_IndexSizeDetails::EstimateRows()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_IndexSizeDetails::EstimateRows", SysView_Trace, 5);

    return SV_ER_INDEXSIZEDETAILS;
}

/*---------------------------------------------------------------------------*/

void SysView_IndexSizeDetails::Execute()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_IndexSizeDetails::Execute", SysView_Trace, 5);

    m_Session->ThrowError (e_not_implemented, 1);
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
