/****************************************************************************

  module      : SysView_LockStatistics.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "LockStatistics" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_LockStatistics.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_LockStatistics::Create()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_LockStatistics::Create", SysView_Trace, 5);

	m_Table->AppendCol (ITOCT_FIXED ,SV_ENTRYCOUNT, 10, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDENTRYCOUNT, 14, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDENTRYCOUNTPERCENTAGE, 24, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_AVGUSEDENTRYCOUNT, 17, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_AVGUSEDENTRYCOUNTPERCENTAGE, 27, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXUSEDENTRYCOUNT, 17, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXUSEDENTRYCOUNTPERCENTAGE, 27, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_TABLELOCKCOUNT, 14, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_ROWLOCKCOUNT, 12, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_HOLDERCOUNT, 11, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_REQUESTORCOUNT, 14, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_LOCKESCALATIONVALUE, 19, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_LOCKESCALATIONCOUNT, 19, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SQLLOCKCOLLISIONCOUNT, 21, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_OMSLOCKCOLLISIONCOUNT, 21, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_DEADLOCKCOUNT, 13, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SQLREQUESTTIMEOUTCOUNT, 22, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_OMSREQUESTTIMEOUTCOUNT, 22, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_TRANSACTIONREGIONCOUNT, 22, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_TABLEREGIONCOUNT, 16, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_ROWREGIONCOUNT, 14, 5);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_LockStatistics::GetColCount()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_LockStatistics::GetColCount", SysView_Trace, 5);

    return SV_CC_LOCKSTATISTICS;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_LockStatistics::EstimateRows()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_LockStatistics::EstimateRows", SysView_Trace, 5);

    return SV_ER_LOCKSTATISTICS;
}

/*---------------------------------------------------------------------------*/

void SysView_LockStatistics::Execute()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_LockStatistics::Execute", SysView_Trace, 5);

    m_Session->ThrowError (e_not_implemented, 1);
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
