/****************************************************************************

  module      : SysView_LogStatistics.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "LogStatistics" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_LogStatistics.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_LogStatistics::Create()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_LogStatistics::Create", SysView_Trace, 5);

	m_Table->AppendCol (ITOCT_CHAR ,SV_ARCHIVELOGENABLED, 17, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USABLESIZE, 10, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDSIZE, 8, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDSIZEPERCENTAGE, 18, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_NOTSAVEDSIZE, 12, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_NOTSAVEDPERCENTAGE, 18, 3);
	m_Table->AppendCol (ITOCT_CHAR ,SV_LOGFULL, 7, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SAVEPOINTCOUNT, 14, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_TRANSACTIONCOUNT, 16, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WRITETRANSACTIONCOUNT, 21, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXUSEDSESSIONID, 16, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXUSEDTRANSACTIONNUMBER, 24, 15);
	m_Table->AppendCol (ITOCT_FIXED ,SV_PHYSICALWRITECOUNT, 18, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_QUEUESIZE, 9, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_QUEUEMAXUSED, 12, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_QUEUEINSERTCOUNT, 16, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_QUEUEOVERFLOWCOUNT, 18, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_GROUPCOMMITCOUNT, 16, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WAITCOUNT, 9, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXWAITCOUNTPERPAGE, 19, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_AVGWAITCOUNTPERPAGE, 19, 10);
	m_Table->AppendCol (ITOCT_CHAR ,SV_FLUSHMODE, 9, 20);
	m_Table->AppendCol (ITOCT_CHAR ,SV_DEVICESTATE, 11, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_QUEUECOUNT, 10, 5);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_LogStatistics::GetColCount()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_LogStatistics::GetColCount", SysView_Trace, 5);

    return SV_CC_LOGSTATISTICS;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_LogStatistics::EstimateRows()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_LogStatistics::EstimateRows", SysView_Trace, 5);

    return SV_ER_LOGSTATISTICS;
}

/*---------------------------------------------------------------------------*/

void SysView_LogStatistics::Execute()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_LogStatistics::Execute", SysView_Trace, 5);

    m_Session->ThrowError (e_not_implemented, 1);
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
