/****************************************************************************

  module      : SysView_MemoryAllocatorStatistics.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "MemoryAllocatorStatistics" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_MemoryAllocatorStatistics.hpp"
#include "RunTime/MemoryManagement/RTEMem_AllocatorInfoReader.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_MemoryAllocatorStatistics::Create()
{

	m_Table->AppendCol (ITOCT_CHAR, SV_ALLOCATORNAME, 13, 40);
	m_Table->AppendCol (ITOCT_FIXED, SV_USEDSIZE, 8, 20);
	m_Table->AppendCol (ITOCT_FIXED, SV_ALLOCATEDSIZE, 13, 20);
	m_Table->AppendCol (ITOCT_FIXED, SV_ALLOCATECOUNT, 13, 20);
	m_Table->AppendCol (ITOCT_FIXED, SV_DEALLOCATECOUNT, 15, 20);
	m_Table->AppendCol (ITOCT_FIXED, SV_FAILEDALLOCATECOUNT, 19, 20);
	m_Table->AppendCol (ITOCT_CHAR, SV_BASEALLOCATORNAME, 17, 40);
	m_Table->AppendCol (ITOCT_FIXED, SV_BASEALLOCATECOUNT, 17, 20);
	m_Table->AppendCol (ITOCT_FIXED, SV_BASEDEALLOCATECOUNT, 19, 20);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_MemoryAllocatorStatistics::GetColCount()
{
	return SV_CC_MEMORYALLOCATORSTATISTICS;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_MemoryAllocatorStatistics::EstimateRows()
{
	return SV_ER_MEMORYALLOCATORSTATISTICS;
}

/*---------------------------------------------------------------------------*/

void SysView_MemoryAllocatorStatistics::Execute()
{
    RTEMem_AllocatorInfoReader                  reader(*(m_Session->GetAllocatorPtr()));
    RTEMem_AllocatorInfoItem                    *pInfo;

    m_Table->GetCatalogTable();

    if (m_Session->GetCommandResult() == 0)
    {

        if (reader.First())
        {
            while (reader.Next (pInfo) )
            {
                m_Table->MoveToCol (ITOVT_CHARPTR, pInfo->siAllocatorName, strlen((SAPDB_Char *)pInfo->siAllocatorName));
                m_Table->MoveToCol (ITOVT_INT8, &pInfo->siUsedSize, 0);
                m_Table->MoveToCol (ITOVT_INT8, &pInfo->siAllocatedSize, 0);
                m_Table->MoveToCol (ITOVT_INT8, &pInfo->siAllocateCount, 0);
                m_Table->MoveToCol (ITOVT_INT8, &pInfo->siDeallocateCount, 0);
                m_Table->MoveToCol (ITOVT_NULL, NULL, 0);
                m_Table->MoveToCol (ITOVT_CHARPTR, pInfo->siBaseAllocatorName, (SAPDB_Int)strlen((SAPDB_Char *)pInfo->siBaseAllocatorName));
                m_Table->MoveToCol (ITOVT_INT8, &pInfo->siBaseAllocateCount, 0);
                m_Table->MoveToCol (ITOVT_INT8, &pInfo->siBaseDeallocateCount, 0);
            }
        }
    }
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
