/****************************************************************************

  module      : SysView_OMSLocks.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "OMSLocks" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "gsp00.h"
#include "vak001.h"
#include "liveCache/LVC_LibOmsInterface.hpp"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_OMSLocks.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_OMSLocks::Create()
{

	m_Table->AppendCol (ITOCT_CHARBYTE, SV_ID, 2, 8);
	m_Table->AppendCol (ITOCT_FIXED, SV_TASKID, 6, 10);
	m_Table->AppendCol (ITOCT_FIXED, SV_LOCKREQUESTTIMEOUT, 18, 10);
	m_Table->AppendCol (ITOCT_CHAR, SV_LOCKMODE, 8, 14);
	m_Table->AppendCol (ITOCT_CHAR, SV_REQUESTMODE, 11, 14);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_OMSLocks::GetColCount()
{
	return SV_CC_OMSLOCKS;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_OMSLocks::EstimateRows()
{
	return SV_ER_OMSLOCKS;
}

/*---------------------------------------------------------------------------*/

void SysView_OMSLocks::Execute()
{

    //OMS_LibOmsInterface     *pLibOmsInterface;
    void                    *pHandle;
    tgg00_OmsLockInfo       LockInfo;
    SAPDB_Bool              bExit;

    m_Table->GetCatalogTable();

    if ((0 == m_Session->GetCommandResult()) && (m_Session->GetPascalObjPtr()->a_ex_kind != only_parsing))
    {
        pHandle = NULL;
        bExit = false;

        while ((!bExit) && (LVC_LibOmsInterface::Instance()->NextOmsLockObjInfo(&pHandle, LockInfo)))
        {
            m_Table->MoveToCol (ITOVT_CHARPTR, LockInfo.oli_handle.asCharp(), sizeof(LockInfo.oli_handle));
            m_Table->MoveToCol (ITOVT_INT4, &LockInfo.oli_taskid, 0);
            if (LockInfo.oli_timeout < 0)
            {
                m_Table->MoveToCol (ITOVT_NULL, NULL, 0);
            }
            else
            {
                m_Table->MoveToCol (ITOVT_INT4, &LockInfo.oli_timeout, 0);
            }
            m_Table->MoveToCol (ITOVT_CHARPTR, LockInfo.oli_lockmode.asCharp(), LockInfo.oli_requestmode.length());
            m_Table->MoveToCol (ITOVT_CHARPTR, LockInfo.oli_requestmode.asCharp(), LockInfo.oli_requestmode.length());

            if (!pHandle)
            {
                bExit = true;
            }
        }
    }
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
