/****************************************************************************

  module      : SysView_PascalSessionObj.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements all "append column" functions.

  last changed: 2002-08-13
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "vak001.h"
#include "hak07.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SAPDBCommon/SAPDB_Types.hpp"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_IRawAllocator.hpp"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_NewDestroy.hpp"
#include "SystemViews/SysView_PascalSessionObj.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

SysView_PascalSessionObj::SysView_PascalSessionObj( tak_all_command_glob	&acv)
: m_ACV(acv)
{}


tak_all_command_glob * SysView_PascalSessionObj::GetPascalObjPtr()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalSessionObj::GetPascalObjPtr", SysView_Trace, 5);

    return (&m_ACV);
}

tgg00_TransContext * SysView_PascalSessionObj::GetTransContextPtr()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalSessionObj::GetTransContextPtr", SysView_Trace, 5);

    return (&(m_ACV.a_transinf.tri_trans));
}

SAPDBMem_IRawAllocator * SysView_PascalSessionObj::GetAllocatorPtr()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalSessionObj::GetAllocatorPtr", SysView_Trace, 5);

    return (reinterpret_cast<SAPDBMem_IRawAllocator *> (m_ACV.a_transinf.tri_trans.trAllocator_gg00));
}

SAPDB_Int4 SysView_PascalSessionObj::GetCommandResult ( void )
{ 
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalSessionObj::GetCommandResult", SysView_Trace, 5);

    return(m_ACV.a_return_segm->variant.C_4.sp1r_returncode_F); 
}

void SysView_PascalSessionObj::SetCommandResult ( const SAPDB_Int4 ccode )
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalSessionObj::SetCommandResult", SysView_Trace, 5);

    m_ACV.a_return_segm->variant.C_4.sp1r_returncode_F = ccode;
}

void SysView_PascalSessionObj::ThrowError ( const SAPDB_Int2    iError,
                                            const SAPDB_Int4    iOffset )
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalSessionObj::ThrowError", SysView_Trace, 5);

    a07_b_put_error (m_ACV, static_cast<tgg00_BasisError>(iError), iOffset);    
}

/*===========================================================================*
*  END OF CODE                                                              *
*===========================================================================*/
