/*!**************************************************************************

  module      : SysView_PascalSessionObj.hpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This contains an abstract interface of all system view classes.

  last changed: 2002-08-13
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/


#ifndef SYSVIEW_PASCALSESSIONOBJ_HPP
#define SYSVIEW_PASCALSESSIONOBJ_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SAPDBCommon/SAPDB_Types.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"

 /*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!***************************************************************************

   class: SysView_PascalSessionObj

   description: This interface class provides an interface for the SystemView
                table object.

******************************************************************************/


class SysView_PascalSessionObj : public SysView_ISessionObj
{

public:

    /* -----------------------------------------------------------------------*/
    /*! chapter: Constructors and initialization                              */
    /* -----------------------------------------------------------------------*/

    /*!-------------------------------------------------------------------------
    function    : SysView_PascalSessionObj()
    description : Class constructor.
    arguments   : [acv] - pascal session object
    return value: none
    --------------------------------------------------------------------------*/

    SysView_PascalSessionObj( tak_all_command_glob	&acv);
    /*!-------------------------------------------------------------------------
    function    : ~SysView_PascalSessionObj()
    description : Class destructor. Is not used in the moment.
    arguments   : none
    return value: none
    --------------------------------------------------------------------------*/

    ~SysView_PascalSessionObj()
    {}

    /* -----------------------------------------------------------------------*/
    /*! endchapter: Constructors and initialization                           */
    /* -----------------------------------------------------------------------*/

    /* -----------------------------------------------------------------------*/
    /*! chapter: Virtual functions                                            */
    /* -----------------------------------------------------------------------*/

    /*!-------------------------------------------------------------------------
    function    : GetPascalObj()
    description : Returns the underlying pascal object. Should be avoided!
    arguments   : none
    return value: [tak_all_command_glob] - AKA acv
    --------------------------------------------------------------------------*/

    tak_all_command_glob * GetPascalObjPtr();

    /*!-------------------------------------------------------------------------
    function    : GetTransContextPtr()
    description : Returns the trans context object. 
    arguments   : none
    return value: [tgg00_TransContext] - trans context pointer
    --------------------------------------------------------------------------*/

    tgg00_TransContext * GetTransContextPtr();

    /*!-------------------------------------------------------------------------
    function    : GetAllocatorPtr()
    description : Returns the local memory allocator pointer. 
    arguments   : none
    return value: [SAPDBMem_IRawAllocator] - local memory allocator pointer
    --------------------------------------------------------------------------*/

    SAPDBMem_IRawAllocator * GetAllocatorPtr();

    /*!-------------------------------------------------------------------------
    function    : GetCommandResult()
    description : This function returns the actual result of the latest executed
                  command.
    arguments   : none
    return value: none
    --------------------------------------------------------------------------*/

    SAPDB_Int4 GetCommandResult ( void );

    /*!-------------------------------------------------------------------------
    function    : SetCommandResult()
    description : This function sets a new command result code.
    arguments   : none
    return value: [SAPDB_Int4] - actual result code
    --------------------------------------------------------------------------*/

    void SetCommandResult ( const SAPDB_Int4 ccode );

    /*!-------------------------------------------------------------------------
    function    : ThrowError()
    description : This function produces an internal system error.
    arguments   : [iError] - error code
                  [iOffet] - offset in SQL statement
    return value: none
    --------------------------------------------------------------------------*/

    void ThrowError ( const SAPDB_Int2     iError,
                      const SAPDB_Int4     iOffset);

    /* -----------------------------------------------------------------------*/
    /*! endchapter: Virtual functions                                         */
    /* -----------------------------------------------------------------------*/


private:

    /* -----------------------------------------------------------------------*/
    /*! chapter: Local Variables                                              */
    /* -----------------------------------------------------------------------*/

    tak_all_command_glob	&m_ACV;

    /* -----------------------------------------------------------------------*/
    /*! endchapter: Local Variables                                           */
    /* -----------------------------------------------------------------------*/

};


#endif  /* SYSVIEW_PASCALSESSIONOBJ_HPP */

