/****************************************************************************

  module      : SysView_PascalTableObj.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements all "append column" functions.

  last changed: 2002-08-13
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "hgg01_3.h"
#include "ggg91.h"
#include "ggg92.h"
#include "hgg06.h"
#include "hgg10.h"
#include "gak40.h"
#include "hak40.h"
#include "hak42.h"
#include "vak001.h"
#include "hsp81.h"
#include "hak07.h"
#include "gkb05.h"
#include "SAPDBCommon/SAPDB_Types.hpp"
#include "SAPDBCommon/SAPDB_UTF8Basis.hpp"
#include "SystemViews/SysView_Defines.hpp"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_IRawAllocator.hpp"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_NewDestroy.hpp"
#include "SystemViews/SysView_PascalTableObj.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

SysView_PascalTableObj::SysView_PascalTableObj( tak_all_command_glob	&acv,
                                                tak40_show_glob         &a41v)
: m_ACV(acv),
  m_Show(a41v)
{}


tak40_show_glob * SysView_PascalTableObj::GetPascalObjPtr()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalTableObj::GetPascalObjPtr", SysView_Trace, 5);

    return (&m_Show);
}

/*---------------------------------------------------------------------------*/

void SysView_PascalTableObj::GetCatalogTable ( void )
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalTableObj::GetCatalogTable", SysView_Trace, 5);

    a40get_catalog_table (m_ACV, m_Show);

}

/*---------------------------------------------------------------------------*/

void SysView_PascalTableObj::AppendCol (SysView_ITOColType  iType,
                                        const char          *sColName, 
                                        int                 iColNameSize,
                                        int                 iColSize)
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalTableObj::AppendCol", SysView_Trace, 5);

    switch (iType)
    {
    case ITOCT_CHAR:
        a40AppVarCol  (m_Show, *(tsp_c32 *)sColName, iColNameSize, dcha, iColSize, 0);
        break;

    case ITOCT_CHARBYTE:
        a40AppVarCol  (m_Show, *(tsp_c32 *)sColName, iColNameSize, dchb, iColSize, 0);
        break;

    case ITOCT_CHARUNICODE:
        a40AppVarCol  (m_Show, *(tsp_c32 *)sColName, iColNameSize, dunicode, iColSize, 0);
        break;

    case ITOCT_FIXED:
        a40AppVarCol  (m_Show, *(tsp_c32 *)sColName, iColNameSize, dfixed, iColSize, 0);
        break;

    case ITOCT_LONG:
        if(g01unicode)
        {
            a40AppVarCol (m_Show, *(tsp_c32 *)sColName, iColNameSize, dstruni, mxsp_long_desc, 0);
        }
        else
        {
            a40AppVarCol (m_Show, *(tsp_c32 *)sColName, iColNameSize, dstra, mxsp_long_desc, 0);
        }
        break;

    case ITOCT_TIMESTAMP:
        a40AppVarCol  (m_Show, *(tsp_c32 *)sColName, iColNameSize, dtimestamp, mxsp_timestamp, 0);
        break;

    default:
        a07_b_put_error (m_ACV, e_wrong_first_param, static_cast<SAPDB_Int>(iType));
        break;
    }
}

/*---------------------------------------------------------------------------*/

void SysView_PascalTableObj::MoveToCol ( SysView_ITOVarType iType,
                                         void               *pVal,
                                         SAPDB_Int          iLength)
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalTableObj::MoveToCol", SysView_Trace, 5);

    SAPDB_Int2 int2Temp;

    switch (iType)
    {

    case ITOVT_NULL:
        a40move (m_ACV, m_Show, NULL, 0);
        break;

    case ITOVT_INT2:
        int2Temp = *(static_cast<SAPDB_Int2 *>(pVal));
        a40move_i4 (m_ACV, m_Show, static_cast<SAPDB_Int4>(int2Temp), !C_IS_UNDEF, 0, !C_ZERO_AS_UNDEF );
        break;
    case ITOVT_INT4:
        a40move_i4 (m_ACV, m_Show, *(static_cast<SAPDB_Int4 *>(pVal)), !C_IS_UNDEF, 0, !C_ZERO_AS_UNDEF );
        break;
    
    case ITOVT_UINT4:
        a40move_ui4 (m_ACV, m_Show, *(reinterpret_cast<SAPDB_UInt4 *>(pVal)), !C_IS_UNDEF, 0, !C_ZERO_AS_UNDEF );
        break;

    case ITOVT_INT8:
    case ITOVT_UINT8:
        MoveInt8 (*(reinterpret_cast<SAPDB_Int8 *>(pVal)));
        break;

    case ITOVT_CHARPTR:
        a40move_const (m_ACV, m_Show, reinterpret_cast<SAPDB_Char *>(pVal), iLength );
        break;

    case ITOVT_UTF8PTR:
        MoveUTF8Ptr (reinterpret_cast<SAPDB_UTF8 *>(pVal), iLength );
        break;

    case ITOVT_TRANSNO:
        MoveTransNo (*(reinterpret_cast<tgg91_TransNo *>(pVal)));
        break;

    case ITOVT_LOCKREQMODE:
        a42lock_req_mode (m_ACV, m_Show, *(reinterpret_cast<tgg00_LockReqMode_Enum *>(pVal)));
        break;

    case ITOVT_LOCKREQSTATE:
        a42lock_req_state (m_ACV, m_Show, *(reinterpret_cast<tgg00_LockReqState *>(pVal)));
        break;

    case ITOVT_SURROGATE:
        a40move (m_ACV, m_Show, *(reinterpret_cast<tgg00_Surrogate *>(pVal)), SURROGATE_MXGG00);
        break;

    case ITOVT_LONGCOLREF:
        a40long_descriptor(m_ACV, m_Show, *(reinterpret_cast<tgg00_Surrogate *>(pVal)), 0);
        break;

    case ITOVT_KNLIDENTIFIER:
        a40move (m_ACV, m_Show, reinterpret_cast<tsp00_KnlIdentifier *>(pVal), sizeof(*(reinterpret_cast<tsp00_KnlIdentifier *>(pVal))));
        break;

    case ITOVT_TRANSLOCKMODE:
        MoveTransLockMode (*(reinterpret_cast<tkb05_TransLockMode_Enum *>(pVal)));
        break;

    default:
        a07_b_put_error (m_ACV, e_move_error, static_cast<SAPDB_Int>(iType));
        break;
    }
}

/*---------------------------------------------------------------------------*/

void SysView_PascalTableObj::MoveInt8 ( SAPDB_Int8 &val )
{
#ifdef BIT64
    a40move_counter (m_ACV, m_Show, static_cast<long>(val), !C_IS_UNDEF, 0, !C_ZERO_AS_UNDEF);
#else
    a40move_counter (m_ACV, m_Show, static_cast<double>(val), !C_IS_UNDEF, 0, !C_ZERO_AS_UNDEF);
#endif
}

void SysView_PascalTableObj::MoveUTF8Ptr ( SAPDB_UTF8 *val, SAPDB_Int length )
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalTableObj::MoveToCol (UTF8 *)", SysView_Trace, 5);

    SAPDB_Int      byteLen = 0;

    byteLen = length;
    
    if (g01unicode)
    {
        SAPDB_UTF8Basis::UTF8ConstPointer   srcAt;
        tsp81_UCS2Char*                     destAt;
        SAPDB_UInt      					charCnt = 0;
        SAPDB_UInt                          byteCnt = 0;
        SAPDB_UTF8Basis::ConversionResult   res;
        SAPDBMem_IRawAllocator              *mAlloc;
        
        mAlloc = reinterpret_cast<SAPDBMem_IRawAllocator *> (m_ACV.a_transinf.tri_trans.trAllocator_gg00);

        res = SAPDB_UTF8Basis::Bytelen( (SAPDB_UTF8 *)val, (SAPDB_UTF8 *)(val + byteLen), srcAt, byteLen, byteCnt, charCnt);
        
        if (!( e_ok == HandleUTF8Error( res )))
        {
            return;
        }

        tsp81_UCS2Char* pUCS2 = reinterpret_cast<tsp81_UCS2Char*> (mAlloc->Allocate ((charCnt+1) * 2));  

        res = SAPDB_UTF8Basis::KernelConvertToUTF16 ((SAPDB_UTF8 *)val, (SAPDB_UTF8 *)(val + byteLen), srcAt, pUCS2, pUCS2 + charCnt, destAt, false);

        if (!( e_ok == HandleUTF8Error( res )))
        {
            return;
        }
    
        a40move (m_ACV, m_Show, reinterpret_cast<void *> (pUCS2), (2 * charCnt));
    
        mAlloc->Deallocate (pUCS2);
    }
    else
    {
        a40move_const (m_ACV, m_Show, reinterpret_cast<void *> (val), byteLen);
    }

}

/*---------------------------------------------------------------------------*/

void SysView_PascalTableObj::MoveTransNo ( tgg91_TransNo &val )
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalTableObj::MoveToCol (TransNo)", SysView_Trace, 5);

    SAPDB_Int8      counter;
    SAPDB_UInt      i;

    if (gg06IsNilTrans( val ))
    {
        a40move (m_ACV, m_Show, NULL, 0);
    }
    else
    {
        counter = 0;

        for (i=0; i<6; i++)
        {
            counter = (counter << 8) + val[i];
        }

        MoveInt8 (counter);
    }
}

/*---------------------------------------------------------------------------*/

void SysView_PascalTableObj::MoveTransLockMode ( tkb05_TransLockMode_Enum &val )
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalTableObj::MoveToCol (TransLockMode)", SysView_Trace, 5);

    switch (val)
    {
    case tlmFree_ekb05:
        a40move (m_ACV, m_Show, NULL, 0);
        break;
    case tlmShare_ekb05:
        MoveToCol (ITOVT_CHARPTR, (SAPDB_Char *)"share", 5);
        break;
    case tlmExcl_ekb05:
    case tlmEotExcl_ekb05:
        MoveToCol (ITOVT_CHARPTR, (SAPDB_Char *)"exclusive", 9);
        break;
    }
}

/*---------------------------------------------------------------------------*/

void SysView_PascalTableObj::CreateLongColContainer ( )
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalTableObj::CreateLongColContainer", SysView_Trace, 5);

    a40CreateLongColContainer (m_ACV, m_Show);
}

/*---------------------------------------------------------------------------*/

void SysView_PascalTableObj::MoveToLongColContainer ( tgg00_Rec &rec, tgg00_Surrogate &erg )
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalTableObj::MoveToLongColContainer", SysView_Trace, 5);

    a40AddToLongColContainer (m_ACV, m_Show, rec, erg);
}


/*---------------------------------------------------------------------------*/

int SysView_PascalTableObj::HandleUTF8Error(SAPDB_UTF8Basis::ConversionResult &res)
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalTableObj::HandleUTF8Error", SysView_Trace, 5);

    SAPDB_Int err = e_ok;
    
    switch (res)
    {
    case SAPDB_UTF8Basis::SourceExhausted:
        err = e_unitrans_src_too_short;
        break;
    case SAPDB_UTF8Basis::SourceCorrupted:
        err = e_not_translatable;
        break;
    case SAPDB_UTF8Basis::TargetExhausted:
        err = e_unitrans_dest_too_short;
        break;
    }

    if (e_ok != err)
    {
        a07_b_put_error (m_ACV, err, 1);
    }
    
    return (err);
}

/*---------------------------------------------------------------------------*/

SAPDB_Int4 SysView_PascalTableObj::GetShowKind( void )
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_PascalTableObj::GetShowKind", SysView_Trace, 5);

    return(m_Show.a4sh_kind);
}

/*===========================================================================*
*  END OF CODE                                                              *
*===========================================================================*/
