/****************************************************************************

  module      : SysView_RestartInformation.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "RestartInformation" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_RestartInformation.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_RestartInformation::Create()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_RestartInformation::Create", SysView_Trace, 5);

	m_Table->AppendCol (ITOCT_TIMESTAMP ,SV_LASTRESTARTDATE, 15, 0);
	m_Table->AppendCol (ITOCT_FIXED ,SV_BADINDEXCOUNT, 13, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_PREVIOUSCONVERTERVERSION, 24, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_LASTSAVEPOINTID, 15, 10);
	m_Table->AppendCol (ITOCT_CHAR ,SV_LASTSAVEPOINTREASON, 19, 20);
	m_Table->AppendCol (ITOCT_TIMESTAMP ,SV_LASTSAVEPOINTDATE, 17, 0);
	m_Table->AppendCol (ITOCT_FIXED ,SV_LASTSAVEPOINTSEQUENCE, 21, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_LASTSAVEPOINTOFFSETONDEVICE, 27, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_LASTSAVEPOINTOFFSETINPAGE, 25, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_OPENTRANSCOUNT, 14, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_OPENTRANSFILEROOT, 17, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_HISTORYDIRECTORYROOT, 20, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_FILEDIRECTORYROOT, 17, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_LONGDIRECTORYROOT, 17, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_RESTARTIOSEQUENCE, 17, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_RESTARTOFFSETOFPAGEONDEVICE, 27, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_RESTARTOFFSETOFENTRYINPAGE, 26, 10);
	m_Table->AppendCol (ITOCT_CHAR ,SV_RESTARTENTRYTYPE, 16, 20);
	m_Table->AppendCol (ITOCT_TIMESTAMP ,SV_RESTARTENTRYDATE, 16, 0);
	m_Table->AppendCol (ITOCT_FIXED ,SV_CYCLERELATEDENDOFFSET, 21, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_LASTKNOWNSEQUENCE, 17, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_LASTKNOWNOFFSET, 15, 10);
	m_Table->AppendCol (ITOCT_CHAR ,SV_DATABASEIDENTIFIER, 18, 100);
	m_Table->AppendCol (ITOCT_CHAR ,SV_LASTSAVEPOINTRELEASEVERSION, 27, 40);
	m_Table->AppendCol (ITOCT_CHAR ,SV_INSTALLSERVERDBVERSION, 22, 40);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_RestartInformation::GetColCount()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_RestartInformation::GetColCount", SysView_Trace, 5);

    return SV_CC_RESTARTINFORMATION;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_RestartInformation::EstimateRows()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_RestartInformation::EstimateRows", SysView_Trace, 5);

    return SV_ER_RESTARTINFORMATION;
}

/*---------------------------------------------------------------------------*/

void SysView_RestartInformation::Execute()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_RestartInformation::Execute", SysView_Trace, 5);

    m_Session->ThrowError (e_not_implemented, 1);
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
