/****************************************************************************

  module      : SysView_TableSizeDetails.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "TableSizeDetails" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_TableSizeDetails.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_TableSizeDetails::Create()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_TableSizeDetails::Create", SysView_Trace, 5);

	m_Table->AppendCol (ITOCT_CHARUNICODE ,SV_OWNER, 5, 32);
	m_Table->AppendCol (ITOCT_CHARUNICODE ,SV_TABLENAME, 9, 32);
	m_Table->AppendCol (ITOCT_FIXED ,SV_ROOT, 4, 10);
	m_Table->AppendCol (ITOCT_CHAR ,SV_PERMANENT, 9, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDSIZE, 8, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USEDDATASIZE, 12, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_TREELEVELCOUNT, 14, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_AVGUSEDSIZEPERCENTAGEINLEAVES, 29, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MINUSEDSIZEPERCENTAGEINLEAVES, 29, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXUSEDSIZEPERCENTAGEINLEAVES, 29, 3);
	m_Table->AppendCol (ITOCT_FIXED ,SV_ROWCOUNT, 8, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_AVGROWCOUNTPERLEAF, 18, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MINROWCOUNTPERLEAF, 18, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXROWCOUNTPERLEAF, 18, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_AVGROWLENGTH, 12, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MINROWLENGTH, 12, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXROWLENGTH, 12, 5);
	m_Table->AppendCol (ITOCT_FIXED ,SV_DEFINEDLONGCOLUMNCOUNT, 22, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_AVGLONGCOLUMNLENGTH, 19, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MINLONGCOLUMNLENGTH, 19, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXLONGCOLUMNLENGTH, 19, 10);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_TableSizeDetails::GetColCount()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_TableSizeDetails::GetColCount", SysView_Trace, 5);

    return SV_CC_TABLESIZEDETAILS;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_TableSizeDetails::EstimateRows()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_TableSizeDetails::EstimateRows", SysView_Trace, 5);

    return SV_ER_TABLESIZEDETAILS;
}

/*---------------------------------------------------------------------------*/

void SysView_TableSizeDetails::Execute()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_TableSizeDetails::Execute", SysView_Trace, 5);

    m_Session->ThrowError (e_not_implemented, 1);
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
