/****************************************************************************

  module      : SysView_TaskStatisticsReset.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "TaskStatisticsReset" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_TaskStatisticsReset.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_TaskStatisticsReset::Create()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_TaskStatisticsReset::Create", SysView_Trace, 5);

	m_Table->AppendCol (ITOCT_FIXED ,SV_TASKID, 6, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_COMMUNICATIONCOUNT, 18, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_BYTESRECEIVED, 13, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_BYTESREPLIED, 12, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_DISPATCHCOUNT, 13, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_TASKRESCEDULECOUNT, 18, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_REGIONACCESSCOUNT, 17, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SUSPENDCOUNT, 12, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SUSPENDRELATIVETIME, 19, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SUSPENDABSOLUTETIME, 19, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WAITCOUNT, 9, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WAITRELATIVETIME, 16, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WAITABSOLUTETIME, 16, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SLEEPCOUNT, 10, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SLEEPRELATIVETIME, 17, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SLEEPABSOLUTETIME, 17, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_CMDEXECUTETIME, 14, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_CMDWAITRELATIVETIME, 19, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_CMDWAITABSOLUTETIME, 19, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SELFREADCOUNT, 13, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SELFREADPAGECOUNT, 17, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SELFREADABSOLUTETIME, 20, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SELFWRITECOUNT, 14, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SELFWRITEPAGECOUNT, 18, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_SELFWRITEABSOLUTETIME, 21, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_READCOUNT, 9, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_READPAGECOUNT, 13, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_READRELATIVETIME, 16, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_READABSOLUTETIME, 16, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WRITECOUNT, 10, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WRITEPAGECOUNT, 14, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WRITERELATIVETIME, 17, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_WRITEABSOLUTETIME, 17, 20);
	m_Table->AppendCol (ITOCT_FIXED ,SV_USABLESTACKSIZE, 15, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_MAXUSEDSTACK, 12, 10);
	m_Table->AppendCol (ITOCT_FIXED ,SV_HOPCOUNT, 8, 20);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_TaskStatisticsReset::GetColCount()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_TaskStatisticsReset::GetColCount", SysView_Trace, 5);

    return SV_CC_TASKSTATISTICSRESET;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_TaskStatisticsReset::EstimateRows()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_TaskStatisticsReset::EstimateRows", SysView_Trace, 5);

    return SV_ER_TASKSTATISTICSRESET;
}

/*---------------------------------------------------------------------------*/

void SysView_TaskStatisticsReset::Execute()
{
    SAPDBTRACE_METHOD_DEBUG ("SysView_TaskStatisticsReset::Execute", SysView_Trace, 5);

    m_Session->ThrowError (e_not_implemented, 1);
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
