/****************************************************************************

  module      : SysView_Version.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the "Version" view class.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright  (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or  (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"
#include "gsp100.h"
#include "hsp100.h"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"
#include "SystemViews/SysView_Version.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS  (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

void SysView_Version::Create()
{

	m_Table->AppendCol (ITOCT_CHAR, SV_OPERATINGSYSTEM, 15, 12);
	m_Table->AppendCol (ITOCT_CHAR, SV_PROCESSORTYPE, 13, 12);
	m_Table->AppendCol (ITOCT_FIXED, SV_ADDRESSINGMODE, 14, 2);
	m_Table->AppendCol (ITOCT_CHAR, SV_ID, 2, 40);
	m_Table->AppendCol (ITOCT_FIXED, SV_MAJORRELEASE, 12, 2);
	m_Table->AppendCol (ITOCT_FIXED, SV_MINORRELEASE, 12, 2);
	m_Table->AppendCol (ITOCT_FIXED, SV_CORRECTIONLEVEL, 15, 2);
	m_Table->AppendCol (ITOCT_FIXED, SV_BUILD, 5, 2);
	m_Table->AppendCol (ITOCT_CHAR, SV_KERNELVARIANT, 13, 8);

}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_Version::GetColCount()
{
	return SV_CC_VERSION;
}

/*---------------------------------------------------------------------------*/

SAPDB_Int SysView_Version::EstimateRows()
{
	return SV_ER_VERSION;
}

/*---------------------------------------------------------------------------*/

void SysView_Version::Execute()
{
    SAPDB_Char      sOS[12+2];
    SAPDB_Char      sID[40+2];
    SAPDB_Char      sVariant[9+2];
    SAPDB_Int2      iAdrMode;
    SAPDB_Int2      iMayRel;
    SAPDB_Int2      iMinRel;
    SAPDB_Int2      iCorr;
    SAPDB_Int2      iBuild;

    m_Table->GetCatalogTable();

    if(m_Session->GetCommandResult() == 0)
    {
        GetOSDetails (sOS, iAdrMode);
        GetKernelDetails (sID, iBuild, sVariant, iMayRel, iMinRel, iCorr);

        m_Table->MoveToCol (ITOVT_CHARPTR, sOS, (SAPDB_Int)strlen(sOS));
        m_Table->MoveToCol (ITOVT_NULL, NULL, 0);
        m_Table->MoveToCol (ITOVT_INT2, &iAdrMode, 0);
        m_Table->MoveToCol (ITOVT_CHARPTR, sID, (SAPDB_Int)strlen(sID));
        m_Table->MoveToCol (ITOVT_INT2, &iMayRel, 0);
        m_Table->MoveToCol (ITOVT_INT2, &iMinRel, 0);
        m_Table->MoveToCol (ITOVT_INT2, &iCorr, 0);
        m_Table->MoveToCol (ITOVT_INT2, &iBuild, 0);
        m_Table->MoveToCol (ITOVT_CHARPTR, sVariant, (SAPDB_Int)strlen(sVariant));
    }
}

void SysView_Version::GetOSDetails (SAPDB_Char *sOS, 
                                    SAPDB_Int2 &iBit)
{
#ifdef COMP_NAME_RTE_NT_SP100 
    strcpy (sOS, "WINNT");
#else
    strcpy (sOS, &(COMP_NAME_RTE_SP100[4]));
#endif

#ifdef BIT64
    iBit = 64;
#else
    iBit = 32;
#endif
}

void SysView_Version::GetKernelDetails (SAPDB_Char  *sIDString, 
                                        SAPDB_Int2  &iBuildNummer, 
                                        SAPDB_Char  *sKernelVariant,
                                        SAPDB_Int2  &iMayorRel,
                                        SAPDB_Int2  &iMinorRel,
                                        SAPDB_Int2  &iCorrLevel)
{
    tsp100_VersionID2   versionID;
    tsp00_Versionc      sVersID;

#if COMPILEMODE_MEO00 == SLOW_MEO00 
    #if defined(OMSTST)
        sp100_GetVersionString ( COMP_NAME_OMSTSTKNL_SP100, s100buildnumber, sVersID );
    #else
        sp100_GetVersionString ( COMP_NAME_SLOWKNL_SP100, s100buildnumber, sVersID );
    #endif
    strcpy(sIDString, sVersID);
    strcpy (sKernelVariant, "slow");  

#else
    #if COMPILEMODE_MEO00 == QUICK_MEO00 
        #if defined(OMSTST)
            sp100_GetVersionString ( COMP_NAME_OMSTSTKNL_SP100, s100buildnumber, sVersID );
        #else
            sp100_GetVersionString ( COMP_NAME_QUICKKNL_SP100, s100buildnumber, sVersID );
        #endif
        strcpy(sIDString, sVersID);
        strcpy (sKernelVariant, "quick");
    #else
        #if defined(OMSTST)
            sp100_GetVersionString ( COMP_NAME_OMSTSTKNL_SP100, s100buildnumber, sVersID );
        #else
            sp100_GetVersionString ( COMP_NAME_KERNEL_SP100, s100buildnumber, sVersID );
        #endif
        strcpy(sIDString, sVersID);
        strcpy (sKernelVariant, "fast");
    #endif
#endif

    sp100_GetVersionID ( VersionIDType2_esp100, s100buildnumberEx, &versionID );

    iMayorRel = versionID.MajorVersion_sp100;
    iMinorRel = versionID.MinorVersion_sp100;
    iCorrLevel = versionID.CorrLevel_sp100;
    iBuildNummer = versionID.BuildNumberPrefix_sp100;
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
