/*!**************************************************************************

  module      : SysView_Version.hpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This contains a class object representing the VERSION view.

  last changed: 2002-08-14
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/


#ifndef SYSVIEW_VERSION_HPP
#define SYSVIEW_VERSION_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_SysViewBase.hpp"
#include "SystemViews/SysView_ISessionObj.hpp"
#include "SystemViews/SysView_ITableObj.hpp"

 /*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!***************************************************************************

   class: SysView_Version

   description: This class is used to handle the system table named
                'VERSION'. It mainly provides the Create() and 
                Execute() functions for this table.

******************************************************************************/

class SysView_Version : public SysView_SysViewBase
{

public:

    /* -----------------------------------------------------------------------*/
    /*! chapter: Constructors and initialization                              */
    /* -----------------------------------------------------------------------*/

    /*!-------------------------------------------------------------------------
    function    : SysView_Version()
    description : constructs an instance and calls the base ISysView constructor. 
    arguments   : [acv] - a reference to the globally used acv structure.
    return value: none
    --------------------------------------------------------------------------*/

    SysView_Version(tak_all_command_glob	&acv,
                    tak40_show_glob         &a41v)
		: SysView_SysViewBase (acv, a41v)
	{}

    /* -----------------------------------------------------------------------*/
    /*! endchapter: Constructors and initialization                           */
    /* -----------------------------------------------------------------------*/

    /* -----------------------------------------------------------------------*/
    /*! chapter: Table creation                                               */
    /* -----------------------------------------------------------------------*/

    /*!-------------------------------------------------------------------------
    function    : Ceate()
    description : fills an empty table object with the column definitions. 
    arguments   : [a41v] - a reference to the table object.
    return value: none
    --------------------------------------------------------------------------*/

    void	Create (void);

    /* -----------------------------------------------------------------------*/
    /*! endchapter: Table creation                                            */
    /* -----------------------------------------------------------------------*/

    /* -----------------------------------------------------------------------*/
    /*! chapter: Information access                                           */
    /* -----------------------------------------------------------------------*/

    /*!-------------------------------------------------------------------------
    function    : GetColCount()
    description : returns the number of columns in this table. 
    arguments   : none
    return value: [SAPDB_Int] - number of columns
    --------------------------------------------------------------------------*/

    SAPDB_Int		GetColCount (void);

    /*!-------------------------------------------------------------------------
    function    : EstimateRows()
    description : returns the estimated number of rows for this table. 
    arguments   : none
    return value: [SAPDB_Int] - estimated number of rows
    --------------------------------------------------------------------------*/

    SAPDB_Int		EstimateRows (void);

    /*!-------------------------------------------------------------------------
    function    : Execute()
    description : fills this table with information
    arguments   : [a41v] - reference to the pascal table object
    return value: none
    --------------------------------------------------------------------------*/

    void	Execute (void);

    /* -----------------------------------------------------------------------*/
    /*! endchapter: Information access                                        */
    /* -----------------------------------------------------------------------*/

private:

    void GetOSDetails (SAPDB_Char *sOS, 
                       SAPDB_Int2 &iBit);

    void GetKernelDetails (SAPDB_Char   *sIDString, 
                           SAPDB_Int2   &iBuildNummer, 
                           SAPDB_Char   *sKernelVariant,
                           SAPDB_Int2   &iMayorRel,
                           SAPDB_Int2   &iMinorRel,
                           SAPDB_Int2   &iCorrLevel);


};

/*!***************************************************************************

   endclass: SysView_Version

******************************************************************************/

#endif  /* SYSVIEW_VERSION_HPP */

