/****************************************************************************

  module      : Trace_MessBlockEntry.cpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area:
  description : 

  last changed: 2000-12-14  18:15
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/


#include "Trace/Trace_MessBlockEntry.hpp"
#include "Trace/Trace_Manager.hpp"


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  GLOBAL VARIABLES                                                         *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

Trace_MessBlockEntry::Trace_MessBlockEntry( 
                                           tgg00_TransContext   &Trans,
                                           tgg00_MessBlock      &MessBlock,
                                           tgg00_Debug          Layer,
                                           tgg00_VtraceType     Type )
                                           :
Trace_ILargeEntry( Trans, Layer, Type ),
m_CurrentPartNo( 1 ),
m_NumberOfParts( 0 ),
m_EntrySize( 0 ),
m_MessBlockIter( Trans, MessBlock, Layer, Type ),
m_MessBlockPart( Trans, Layer, Type, (*m_MessBlockIter).GetPart(), (*m_MessBlockIter).GetPartSize()),
m_MessBlockPartIter( m_MessBlockPart )
{
    for( m_MessBlockIter.Begin(); !m_MessBlockIter.End(); ++m_MessBlockIter )
    {
        m_MessBlockPart.ReInit( *m_MessBlockIter );
        
        m_NumberOfParts += m_MessBlockPart.GetNumberOfParts();
        m_EntrySize     += m_MessBlockPart.GetSize();
    }
};

/*---------------------------------------------------------------------------*/

Trace_MessBlockEntry::~Trace_MessBlockEntry()
{
    Trace_Manager::GetInstance().InsertLarge( m_Trans, *this );
};

/*---------------------------------------------------------------------------*/

tgg00_Debug
Trace_MessBlockEntry::GetLayer() const
{
    return( m_Layer ); 
};

/*---------------------------------------------------------------------------*/

SAPDB_Int4
Trace_MessBlockEntry::GetNumberOfParts() const
{
    return( m_NumberOfParts ); 
};

/*---------------------------------------------------------------------------*/

tsp00_BytePtr
Trace_MessBlockEntry::GetPart() const
{
    return( m_MessBlockPartIter.GetPart() );
};

/*---------------------------------------------------------------------------*/

SAPDB_Int4
Trace_MessBlockEntry::GetPartNo() const
{
    return( m_CurrentPartNo );
};

/*---------------------------------------------------------------------------*/

SAPDB_Int4
Trace_MessBlockEntry::GetPartSize() const
{
    return( m_MessBlockPartIter.GetPartSize() );
};

/*---------------------------------------------------------------------------*/

SAPDB_Int4
Trace_MessBlockEntry::GetSize() const 
{
    // This is the size of the complete entry. This includes all mess block
    // parts (header, trailer, ...) and the internal header size.
    return( m_EntrySize );
};

/*---------------------------------------------------------------------------*/

tgg00_VtraceType
Trace_MessBlockEntry::GetType() const
{
    return( m_Type );
};

/*---------------------------------------------------------------------------*/

bool
Trace_MessBlockEntry::PartEndReached() const
{
    return( m_CurrentPartNo > m_NumberOfParts );
};

/*---------------------------------------------------------------------------*/

void 
Trace_MessBlockEntry::SetToFirstPart()
{
    m_CurrentPartNo = 1;
    m_MessBlockIter.Begin();
    m_MessBlockPart.ReInit( *m_MessBlockIter );
    m_MessBlockPartIter.Begin();
};

/*---------------------------------------------------------------------------*/

void
Trace_MessBlockEntry::SetToNextPart()
{
    
    // Count the total number of parts in which the complete mess block
    // trace entry is splited.
    ++m_CurrentPartNo;
    
    // Set to the next part inside a mess block, e.g. from the first part of 
    // a mess block header to the second part of the same mess block header.
    // Note that each part has a size limitation of MaximumSize (=> 132 bytes).
    
    ++m_MessBlockPartIter;
    if (! m_MessBlockPartIter.End() ) return;
    
    // Set to the next part of the mess block, i.e. from header to trailer or
    // from the qual part to the data part.
    ++m_MessBlockIter;
    if( m_MessBlockIter.End() ) return;
    
    // In case of a valid mess block part the internal mess block part 
    // will be set to the new current mess block part.
    m_MessBlockPart.ReInit( *m_MessBlockIter );
    m_MessBlockPartIter.Begin();
};

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

Trace_MessBlockIterator::Trace_MessBlockIterator
( 
 tgg00_TransContext &Trans,
 tgg00_MessBlock    &MessBlock,
 tgg00_Debug        &Layer,
 tgg00_VtraceType   &Type
 ) 
: 
m_PartKind( E_MessHeader ),
m_TraceLayer( Layer),
m_TraceType( Type ),
m_Trans( Trans ),
m_MessBlock( MessBlock )
{
};

/*---------------------------------------------------------------------------*/

void 
Trace_MessBlockIterator::operator ++()
{
    switch( m_PartKind )
    {
    case E_MessHeader:
        {
            m_PartKind  = E_MessTrailer;
            return;
        };
    case E_MessTrailer:
        {
            if ( (NULL == m_MessBlock.mb_qual()) 
                 || (0 >= m_MessBlock.mb_qual_len()) )
                m_PartKind = E_MessData;       // PTS 1109936 TS 2001-04-06
            else
                m_PartKind = E_MessQual;
            return;
        };
     case E_MessQual:
        {
            m_PartKind  = E_MessData;
            return;
        };
     case E_MessData:
         {
            if ( (ak_send == m_TraceType)        &&
                 (NULL != m_MessBlock.mb_qual()) &&
                 (0 < m_MessBlock.mb_qual_len()) )
                 m_PartKind = E_MessStack;
            else if( 0 < m_MessBlock.mb_strat_len() ) // PTS 1112689 
                 m_PartKind = E_MessStrategy;
             else
                 m_PartKind  = E_MessEnd;
             return;
         };
     case E_MessStack:
         {
             if( 0 < m_MessBlock.mb_strat_len() ) // PTS 1112689
                 m_PartKind = E_MessStrategy;
             else
                 m_PartKind  = E_MessEnd;
             return;
         };
     case E_MessStrategy: // PTS 1112689
         ;
    default:
        {
            m_PartKind  = E_MessEnd;
            return;
        }
    }
};

/*---------------------------------------------------------------------------*/

Trace_MessBlockPart
Trace_MessBlockIterator::operator *() const
{
    return( Trace_MessBlockPart( m_Trans, m_TraceLayer, m_TraceType, 
        GetPart(), GetPartSize() ));
};

/*---------------------------------------------------------------------------*/

void 
Trace_MessBlockIterator::Begin()
{
    m_PartKind = E_MessHeader;
};

/*---------------------------------------------------------------------------*/

bool 
Trace_MessBlockIterator::End() const
{
   // return(( E_MessStack < m_PartKind ) || 
   //     (( E_MessStack == m_PartKind ) && ( 0 == GetPartSize() )));
    
    return( E_MessEnd <= m_PartKind );

    //return(( E_MessStrategy < m_PartKind ) || 
    //   (( E_MessStrategy == m_PartKind ) && ( 0 == GetPartSize() )));
};

/*---------------------------------------------------------------------------*/

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
