/*!**************************************************************************

  module      : Trace_Writer.cpp

  -------------------------------------------------------------------------

  responsible : TorstenS

  special area:
  description : 

  last changed: 2000-11-27  18:15
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "hkb35.h"      // PASCAL: k35dump
#include "hkb38.h"      // PASCAL: k38dump
#include "hkb52_1.h"    // PASCAL: k52dump
#include "hkb57.h"      // PASCAL: k57dump
#include "hkb90.h"      // PASCAL: k90dump
#include "hbd77.h"      // PASCAL: b77dump
#include "hbd90.h"      // PASCAL: b90dump
#include "hbd91.h"      // PASCAL: b91dump
#include "hbd20_7.h"    // PASCAL: b20dump
#include "hgg01.h"      // PASCAL: g01page_size, g01dump
#include "hgg02.h"      // PASCAL: g02dump

#include "heo51.h"      // RTE   : vfinish
#include "heo56.h"      // RTE   : Vsuspend/Vresume
#include "heo58.h"      // RTE   : vIsDumpWanted/vIsTraceWanted

#include "Converter/Converter_IDump.hpp"
#include "Pager/Pager_IDump.hpp"
#include "FreeBlockManagement/FBM_Manager.hpp"
#include "KernelCommon/Kernel_Dump.hpp"
#include "RunTime/MemoryManagement/RTEMem_Allocator.hpp"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_NewDestroy.hpp"
#include "Trace/Trace_Manager.hpp"
#include "Trace/Trace_Writer.hpp"
#include "RunTime/RTE_ISystem.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define TRACE_ENTER_REGIONS   1
#define TRUE                  1

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  GLOBAL VARIABLES                                                         *
 *===========================================================================*/

Trace_Writer*   Trace_Writer::m_Instance = NULL;


/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
 *===========================================================================*/



/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

/*---------------------------------------------------------------------------*/
/* Singleton interface                                                       */
/*---------------------------------------------------------------------------*/

bool
Trace_Writer::CreateInstance( tsp00_TaskId TaskId )
{
    if ( NULL == m_Instance )
    {
        m_Instance = new( RTEMem_Allocator::Instance() ) Trace_Writer( TaskId );
    }
    return( NULL != m_Instance );
}

/*---------------------------------------------------------------------------*/

Trace_Writer::Trace_Writer( tsp00_TaskId &TaskId )
        :
        m_Frame( RTE_ISystem::Instance().AllocSystemPages( g01page_size ), g01page_size ),
        m_TaskId( TaskId )
{
}

/*---------------------------------------------------------------------------*/

void
Trace_Writer::UserForcedDump(
    tsp00_TaskId           TaskId,
    const tsp00_VFilename  &FileName,
    tgg00_DumpStateSet     &DumpStateSet )
{
    Kernel_Dump DumpFile( FileName, m_Frame );

    Dump( TaskId, DumpFile, DumpStateSet, TRACE_ENTER_REGIONS );
}

/*---------------------------------------------------------------------------*/

void
Trace_Writer::SystemForcedDump()
{
    tgg00_DumpStateSet  DumpStateSet;
    tsp00_VFilename     FileName = Trace_Manager::GetInstance().GetDumpFile();
    Kernel_Dump         DumpFile( FileName, m_Frame );

    DumpStateSet.clear();
    DumpStateSet.addElement( dumpAll_egg00 );

    Dump( m_TaskId, DumpFile, DumpStateSet, ! TRACE_ENTER_REGIONS );
}

/*---------------------------------------------------------------------------*/

tsp00_TaskId
Trace_Writer::GetTaskId()
{
    return( m_TaskId );
}

/*---------------------------------------------------------------------------*/

void
Trace_Writer::Run()
{
    Trace_Manager::GetInstance().TraceWriterRegistration( m_TaskId );

    while( true )
    {
        Suspend();

        // In case of vabort is called the trace writer starts
        // to flush the complete trace and the dump
        if ( TRUE == vIsTraceWanted() )
        {
            Trace_Manager::GetInstance().Flush( m_TaskId );
            if( TRUE == vIsDumpWanted() )
                SystemForcedDump();
            break;
        }
        else
            Trace_Manager::GetInstance().Flush( m_TaskId );
    }
    vfinish();
}

/*---------------------------------------------------------------------------*/

void
Trace_Writer::Resume()
{
    vresume( m_TaskId, cgg_suspend_lvl_0 );
}

/*---------------------------------------------------------------------------*/

void
Trace_Writer::Suspend()
{
    vsuspend( m_TaskId, cgg_suspend_lvl_0, 49 );
}

/*---------------------------------------------------------------------------*/


void
Trace_Writer::Dump(
    tsp00_TaskId         TaskId,
    Kernel_Dump          &Dump,
    tgg00_DumpStateSet   &DumpStateSet,
    tsp00_Bool           bEnterRegions )
{

    if(
        ( DumpStateSet.includes( dumpConverter_egg00 )) ||
        ( DumpStateSet.includes( dumpAll_egg00       ))
    )
        Converter_IDump::Instance().Dump( TaskId, Dump, bEnterRegions );

    if(
        ( DumpStateSet.includes( dumpFbm_egg00 )) ||
        ( DumpStateSet.includes( dumpAll_egg00 ))
    )
        FBM_Manager::Instance().Dump( TaskId, Dump );

    if( ( DumpStateSet.includes( dumpPagerWriter_egg00 )) ||
			( DumpStateSet.includes( dumpAll_egg00 ))) {
        Pager_IDump::GetInstance().Dump( TaskId, Dump );
	}

    // ====================> ADD NEW C++ DUMP METHODS HERE!!!! <=============================
    //if( ( DumpStateSet.includes( dumpAll ))
    //    myDump();
    // ====================> ADD NEW C++ DUMP METHODS HERE!!!! <=============================

    Dump.Flush();

    // ===========================
    // support old pascal dump
    // ===========================
    if( !Dump.ErrorOccured() )
    {
        tsp00_PageAddr  pFrame = reinterpret_cast < tsp00_PageAddr>
                                 (m_Frame.GetPtrAt( 0, m_Frame.Capacity()));
        tsp00_VfReturn  dumpError;
        tgg00_VfFileref dumpFile;
        tsp00_ErrText   errorText;
        SAPDB_Int4      dumpPageNo  = Dump.GetPageNo();
        SAPDB_Int4      dumpPagePos = 1;

        dumpError.becomes( vf_ok );
        dumpFile.no      = Dump.GetFileNo();
        dumpFile.buf_cnt = 0;

        // Dump tree lock manager
        if(
            ( vf_ok == dumpError                             ) &&
            (( DumpStateSet.includes( dumpBdLocklist_egg00  )) ||
             ( DumpStateSet.includes( dumpAll_egg00        )))
        )
        {
            b77dump_locklist( dumpFile, *pFrame, dumpPageNo, dumpPagePos,
                              dumpError, errorText);
        }
        // Dump utility manager
        if(
            ( vf_ok == dumpError                        ) &&
            (( DumpStateSet.includes( dumpUtility_egg00)) ||
             ( DumpStateSet.includes( dumpAll_egg00   )))
        )
        {
            k35dump_utility( dumpFile, *pFrame, dumpPageNo, dumpPagePos,
                             dumpError, errorText);
        }
        // Dump backup and recovery manager
        if(
            ( vf_ok == dumpError                       ) &&
            (( DumpStateSet.includes( dumpBackup_egg00)) ||
             ( DumpStateSet.includes( dumpAll_egg00  )))
        )
        {
            k38dump_headmaster( dumpFile, *pFrame, dumpPageNo, dumpPagePos,
                                dumpError, errorText);
        }
        // Dump SQL lock manager
        if(
            ( vf_ok == dumpError                           ) &&
            (( DumpStateSet.includes( dumpKbLocklist_egg00)) ||
             ( DumpStateSet.includes( dumpAll_egg00      )))
        )
        {
            k52dump_locklist( TaskId, bEnterRegions, dumpFile, *pFrame,
                              dumpPageNo, dumpPagePos, dumpError, errorText);
        }
        // Dump restart record
        if(
            ( vf_ok == dumpError                           ) &&
            (( DumpStateSet.includes( dumpRestartRec_egg00)) ||
             ( DumpStateSet.includes( dumpAll_egg00      )))
        )
        {
            k57dump_restartrec( dumpFile, *pFrame, dumpPageNo, dumpPagePos,
                                dumpError, errorText);
        }
        // Dump server task manager
        if(
            ( vf_ok == dumpError                           ) &&
            (( DumpStateSet.includes( dumpNetServer_egg00 )) ||
             ( DumpStateSet.includes( dumpAll_egg00      )))
        )
        {
            k90dump_netserver( dumpFile, *pFrame, dumpPageNo, dumpPagePos,
                               dumpError, errorText);
        }
        // Dump common kernel globals
        if(
            ( vf_ok == dumpError                              ) &&
            (( DumpStateSet.includes( dumpConfiguration_egg00)) ||
             ( DumpStateSet.includes( dumpAll_egg00         )))
        )
        {
            g01dump( dumpFile, *pFrame, dumpPageNo, dumpPagePos,
                     dumpError, errorText);
        }
        // Dump kernel code pages
        if(
            ( vf_ok == dumpError                               ) &&
            (( DumpStateSet.includes( dumpTransformation_egg00)) ||
             ( DumpStateSet.includes( dumpAll_egg00          )))
        )
        {
            g02dump( dumpFile, *pFrame, dumpPageNo, dumpPagePos,
                     dumpError, errorText);
        }
        // Dump oms file directory
        if(
            ( vf_ok == dumpError                          ) &&
            (( DumpStateSet.includes( dumpObjFirDir_egg00)) ||
             ( DumpStateSet.includes( dumpAll_egg00     )))
        )
        {
            bd90DumpObjFileDir( dumpFile, m_Frame.Capacity(), *pFrame,
                                dumpPageNo, dumpPagePos, dumpError, errorText);
        }
        // Dump garbage collector
        if(
            ( vf_ok == dumpError                         ) &&
            (( DumpStateSet.includes( dumpGarbcoll_egg00)) ||
             ( DumpStateSet.includes( dumpAll_egg00    )))
        )
        {
            bd91DumpGarbColl( dumpFile, m_Frame.Capacity(), *pFrame,
                              dumpPageNo, dumpPagePos, dumpError, errorText);
        }
        // Dump data cache control blocks
        if(
            ( vf_ok == dumpError                          ) &&
            (( DumpStateSet.includes( dumpDataCache_egg00)) ||
             ( DumpStateSet.includes( dumpAll_egg00     )))
        )
        {
            bd20DumpDataCache( dumpFile, *pFrame, dumpPageNo, dumpPagePos,
                               dumpError, errorText);
        }
        // Dump data cache frames
        if(( vf_ok == dumpError ) && ( DumpStateSet.includes( dumpAll_egg00 )))
        {
            bd20DumpDataCacheFrames( dumpFile, *pFrame, dumpPageNo, dumpPagePos,
                                     dumpError, errorText);
        }
    }
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
