/****************************************************************************

  module      : LVC_AbstractObjKeyIterator.hpp

  -------------------------------------------------------------------------

  responsible : d025251

  special area: 
  description : 


  last changed: 2001-07-05  18:40
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/


#ifndef LVC_ABSTRACTOBJKEYITERATOR_HPP
#define LVC_ABSTRACTOBJKEYITERATOR_HPP

#include "SAPDBCommon/MemoryManagement/SAPDBMem_IRawAllocator.hpp"

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define CHECK_NUMBER 4711

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/*!*****************************************************************************

   class:        LVC_AbstractObjKeyIterator
   description:  This class implements a key range iterator for object files
   
	             

*******************************************************************************/

class LVC_AbstractObjKeyIterator
{
public:

    /*!-------------------------------------------------------------------------
    function:     GetObjFileId()
    description:  returns the id of the file the iterator is working on

    arguments:    none
    return value: (tgg00_FileId) id of the file the iterator is working on
    --------------------------------------------------------------------------*/
	tgg00_FileId   GetObjFileId()
	{
		return m_ObjFileId;
	}
	
	/*!-------------------------------------------------------------------------
    function:     GetOId()
    description:  dereferences the iterator,
	
	The OId of the object at which the iterator is currently pointing is retuirned 
	  
	arguments:    none
	return value: (tgg92_KernelOid) OId of the object at which the iterator is currently pointing
    --------------------------------------------------------------------------*/
	virtual tgg92_KernelOid	GetOId() = 0;

	/*!-------------------------------------------------------------------------
	function:     operator ++()
	description:  moves the iterator to the next object in the key range

		   
	arguments:    none
	return value: (tgg92_KernelOid) OId of the object at which the iterator is currently pointing
    --------------------------------------------------------------------------*/
	virtual void operator ++() = 0;

    /*!-------------------------------------------------------------------------
	function:     IsValid()
	description:  returns true if the iterator fulfills some basic requirements of an iterator

                  This function can be used to validate an iterator after a cast
				  from a void pointer. Thus this function reperesnts a simple
				  type check.
		   
	arguments:    none
	return value: (SAPDB_Bool) true if the iterator fulfills some basic requirements
    --------------------------------------------------------------------------*/
    virtual SAPDB_Bool  IsValid()
	{
		return (m_CheckNumber == CHECK_NUMBER);
	}

    /*!-------------------------------------------------------------------------
	function:     IsReady()
	description:  returns true if the end of the iterator scope is reached
		   
	arguments:    none
	return value: (SAPDB_Bool) true if the end of the iterator scope is reached
    --------------------------------------------------------------------------*/
    virtual SAPDB_Bool  IsReady() = 0;

	/*!-------------------------------------------------------------------------
	function:     constructor()
	description:  
		   
	arguments:    none
	return value: none
    --------------------------------------------------------------------------*/
	LVC_AbstractObjKeyIterator (tgg00_TransContext     &Trans,
		                        tgg00_FileId	       &ObjFileId,
		                        tsp00_Int4	            KeyLen,	
		                        void		           *pStopKey)
	:
	m_KeyLen               (KeyLen),
	m_ObjFileId            (ObjFileId),
	m_Trans                (Trans),
	m_CheckNumber          (CHECK_NUMBER)
	{
		memcpy (&m_StopKey,      pStopKey,  m_KeyLen);
	}

	/*!-------------------------------------------------------------------------
	function:     destructor()
	description:  releases all resources and mark the iterator as destroyed via the checknumber
		   
	arguments:    none
	return value: none
    --------------------------------------------------------------------------*/
	virtual ~LVC_AbstractObjKeyIterator(){};

protected: // member
	tsp00_Int4               m_KeyLen;
	tsp00_Key                m_StopKey;
	tgg00_FileId             m_ObjFileId;
	tgg00_TransContext      &m_Trans; 
	tsp00_Int4               m_CheckNumber;
};

/*!*****************************************************************************

   endclass: LVC_AbstractObjKeyIterator

*******************************************************************************/

#endif  /* LVC_ABSTRACTOBJKEYITERATOR_HPP */
