/****************************************************************************

  module      : LVC_SingleFileObjKeyIterator.hpp

  -------------------------------------------------------------------------

  responsible : d025251

  special area: 
  description : 


  last changed: 2001-07-18  12:49
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/


#ifndef LVC_SINGLEFILEOBJKEYITERATOR_HPP
#define LVC_SINGLEFILEOBJKEYITERATOR_HPP



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "liveCache/LVC_BufferedObjKeyIterator.hpp"
#include "liveCache/LVC_AbstractObjKeyIterator.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define DONT_STORE_KEYS false

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

 /*!*****************************************************************************

   class:        LVC_SingleFileObjKeyIterator
   description:  This class implements a key range iterator for object files
   
	             

*******************************************************************************/

class LVC_SingleFileObjKeyIterator : public LVC_AbstractObjKeyIterator
{
public:

	/*!-------------------------------------------------------------------------
    function:     GetOId()
    description:  dereferences the iterator,
	
	The OId of the object at which the iterator is currently pointing is retuirned 
	  
	arguments:    none
	return value: (tgg92_KernelOid) OId of the object at which the iterator is currently pointing
    --------------------------------------------------------------------------*/
	virtual tgg92_KernelOid	GetOId()
	{
		return m_BufferdObjKeyIterator.GetOId();
	}

    /*!-------------------------------------------------------------------------
	function:     IsReady()
	description:  returns true if the end of the iterator scope is reached
		   
	arguments:    none
	return value: (SAPDB_Bool) true if the end of the iterator scope is reached
    --------------------------------------------------------------------------*/
    virtual SAPDB_Bool  IsReady()
	{
		return m_BufferdObjKeyIterator.IsReady();
	}

	/*!-------------------------------------------------------------------------
	function:     operator ++();
	description:  moves the iterator to the next object in the key range

		   
	arguments:    none
	return value: (tgg92_KernelOid) OId of the object at which the iterator is currently pointing
    --------------------------------------------------------------------------*/
	virtual void operator ++()
	{
		 m_BufferdObjKeyIterator.GotoNextObject (m_Trans, 
			m_ObjFileId.fileObjFileNo_gg00(), m_KeyLen, &m_StopKey);
	}

	/*!-------------------------------------------------------------------------
	function:     constructor()
	description:  
		   
	arguments:    none
	return value: none
    --------------------------------------------------------------------------*/
	LVC_SingleFileObjKeyIterator (tgg00_TransContext     &Trans,
    		                      tgg00_FileId	         &ObjFileId,
		                          tsp00_Int4	          KeyLen,
		                          void		             *pStartKey,
		                          void		             *pStopKey,
		                          SAPDBMem_IRawAllocator &Allocator)
	:LVC_AbstractObjKeyIterator (Trans, ObjFileId, KeyLen, pStopKey),
	 m_BufferdObjKeyIterator    (Trans, ObjFileId.fileObjFileNo_gg00(), 0 /* = index first key file */, KeyLen, pStartKey, pStopKey, DONT_STORE_KEYS, Allocator)
	{}

private: // member

	LVC_BufferedObjKeyIterator  m_BufferdObjKeyIterator;
};

/*!*****************************************************************************

   endclass: LVC_SingleFileObjKeyIterator

*******************************************************************************/


#endif  /* LVC_SINGLEFILEOBJKEYITERATOR_HPP */
