
/****************************************************************************

  module      : vak400.cpp

  -------------------------------------------------------------------------

  responsible : UlrichJ

  special area: SystemViews
  description : This module implements the pascal interface functions for the
                new c++ system view classes.

  last changed: 2002-08-13
  see also    :

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2002

  ========== licence begin LGPL
  Copyright (C) 2002 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/
#include "gsp00.h"
#include "ggg00.h"
#include "gak40.h"
#include "hak07.h"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_IRawAllocator.hpp"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_NewDestroy.hpp"
#include "SystemViews/SysView_Defines.hpp"
#include "SystemViews/SysView_ISysView.hpp"
#include "SystemViews/SysView_ActiveConfiguration.hpp"
#include "SystemViews/SysView_BackupInformation.hpp"
#include "SystemViews/SysView_BackupThreads.hpp"
#include "SystemViews/SysView_CacheStatistics.hpp"
#include "SystemViews/SysView_CacheStatisticsReset.hpp"
#include "SystemViews/SysView_ClassContainers.hpp"
#include "SystemViews/SysView_ClassContainerStatistics.hpp"
#include "SystemViews/SysView_ClassContainerStatisticsReset.hpp"
#include "SystemViews/SysView_DataCache.hpp"
#include "SystemViews/SysView_DataStatistics.hpp"
#include "SystemViews/SysView_DataStatisticsReset.hpp"
#include "SystemViews/SysView_DataVolumes.hpp"
#include "SystemViews/SysView_Events.hpp"
#include "SystemViews/SysView_Files.hpp"
#include "SystemViews/SysView_HistoryCleanupStatistics.hpp"
#include "SystemViews/SysView_HistoryCleanupStatisticsReset.hpp"
#include "SystemViews/SysView_IndexSizeDetails.hpp"
#include "SystemViews/SysView_IOThreadStatistics.hpp"
#include "SystemViews/SysView_IOThreadStatisticsReset.hpp"
#include "SystemViews/SysView_Locks.hpp"
#include "SystemViews/SysView_LockStatistics.hpp"
#include "SystemViews/SysView_LockStatisticsReset.hpp"
#include "SystemViews/SysView_LogStatistics.hpp"
#include "SystemViews/SysView_LogStatisticsReset.hpp"
#include "SystemViews/SysView_LogVolumes.hpp"
#include "SystemViews/SysView_MemoryAllocatorStatistics.hpp"
#include "SystemViews/SysView_MemoryAllocatorStatisticsReset.hpp"
#include "SystemViews/SysView_MemoryHolders.hpp"
#include "SystemViews/SysView_Miscellaneous.hpp"
#include "SystemViews/SysView_OMSLocks.hpp"
#include "SystemViews/SysView_OMSProcedureStatistics.hpp"
#include "SystemViews/SysView_OMSProcedureStatisticsReset.hpp"
#include "SystemViews/SysView_OMSVersions.hpp"
#include "SystemViews/SysView_OptimizerInformation.hpp"
#include "SystemViews/SysView_PageLocks.hpp"
#include "SystemViews/SysView_RegionLocks.hpp"
#include "SystemViews/SysView_RegionStatistics.hpp"
#include "SystemViews/SysView_RegionStatisticsReset.hpp"
#include "SystemViews/SysView_RestartInformation.hpp"
#include "SystemViews/SysView_Schemas.hpp"
#include "SystemViews/SysView_Sessions.hpp"
#include "SystemViews/SysView_SpinLockStatistics.hpp"
#include "SystemViews/SysView_SpinLockStatisticsReset.hpp"
#include "SystemViews/SysView_TableSizeDetails.hpp"
#include "SystemViews/SysView_TaskGroupStatistics.hpp"
#include "SystemViews/SysView_TaskGroupStatisticsReset.hpp"
#include "SystemViews/SysView_TaskStatistics.hpp"
#include "SystemViews/SysView_TaskStatisticsReset.hpp"
#include "SystemViews/SysView_Transactions.hpp"
#include "SystemViews/SysView_Version.hpp"


/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

SysView_ISysView * SVGetInstanceFromSH ( tak_all_command_glob	&acv,
                                         tak40_show_glob        &a41v)
{
    SAPDBMem_IRawAllocator  *mAlloc;
    SysView_ISysView        *ret; 

    
    mAlloc = reinterpret_cast<SAPDBMem_IRawAllocator *> (acv.a_transinf.tri_trans.trAllocator_gg00);
    
    switch (a41v.a4sh_kind)
    {
    case sh_activeconfiguration:
    case sh_xparam:
        ret = new(*mAlloc) SysView_ActiveConfiguration(acv, a41v);
        break;
	case sh_backupinformation:
		ret = new(*mAlloc) SysView_BackupInformation( acv, a41v ); 
		break;
	case sh_backupthreads:
		ret = new(*mAlloc) SysView_BackupThreads( acv, a41v );
		break;
	case sh_cachestatistics:
		ret = new(*mAlloc) SysView_CacheStatistics( acv, a41v );
		break;
	case sh_cachestatisticsreset:
		ret = new(*mAlloc) SysView_CacheStatisticsReset( acv, a41v );
		break;
	case sh_classcontainers:
		ret = new(*mAlloc) SysView_ClassContainers( acv, a41v );
		break;
	case sh_classcontainerstatistics:
		ret = new(*mAlloc) SysView_ClassContainerStatistics( acv, a41v );
		break;
	case sh_classcontainerstatisticsreset:
		ret = new(*mAlloc) SysView_ClassContainerStatisticsReset( acv, a41v );
		break;
	case sh_datacache:
		ret = new(*mAlloc) SysView_DataCache( acv, a41v );
		break;
	case sh_datastatistics:
		ret = new(*mAlloc) SysView_DataStatistics( acv, a41v );
		break;
	case sh_datastatisticsreset:
		ret = new(*mAlloc) SysView_DataStatisticsReset( acv, a41v );
		break;
	case sh_datavolumes:
		ret = new(*mAlloc) SysView_DataVolumes( acv, a41v );
		break;
	case sh_events:
		ret = new(*mAlloc) SysView_Events( acv, a41v );
		break;
	case sh_files:
		ret = new(*mAlloc) SysView_Files( acv, a41v );
		break;
	case sh_historycleanupstatistics:
		ret = new(*mAlloc) SysView_HistoryCleanupStatistics( acv, a41v );
		break;
	case sh_historycleanupstatisticsreset:
		ret = new(*mAlloc) SysView_HistoryCleanupStatisticsReset( acv, a41v );
		break;
	case sh_indexsizedetails:
		ret = new(*mAlloc) SysView_IndexSizeDetails( acv, a41v );
		break;
	case sh_iothreadstatistics:
		ret = new(*mAlloc) SysView_IOThreadStatistics( acv, a41v );
		break;
	case sh_iothreadstatisticsreset:
		ret = new(*mAlloc) SysView_IOThreadStatisticsReset( acv, a41v );
		break;
	case sh_locks:
		ret = new(*mAlloc) SysView_Locks( acv, a41v );
		break;
	case sh_lockstatistics:
		ret = new(*mAlloc) SysView_LockStatistics( acv, a41v );
		break;
	case sh_lockstatisticsreset:
		ret = new(*mAlloc) SysView_LockStatisticsReset( acv, a41v );
		break;
	case sh_logstatistics:
		ret = new(*mAlloc) SysView_LogStatistics( acv, a41v );
		break;
	case sh_logstatisticsreset:
		ret = new(*mAlloc) SysView_LogStatisticsReset( acv, a41v );
		break;
	case sh_logvolumes:
		ret = new(*mAlloc) SysView_LogVolumes( acv, a41v );
		break;
	case sh_memoryallocatorstatistics:
		ret = new(*mAlloc) SysView_MemoryAllocatorStatistics( acv, a41v );
		break;
	case sh_memoryallocatorstatisticsreset:
		ret = new(*mAlloc) SysView_MemoryAllocatorStatisticsReset( acv, a41v );
		break;
	case sh_memoryholders:
		ret = new(*mAlloc) SysView_MemoryHolders( acv, a41v );
		break;
	case sh_miscellaneous:
		ret = new(*mAlloc) SysView_Miscellaneous( acv, a41v );
		break;
	case sh_omslocks:
		ret = new(*mAlloc) SysView_OMSLocks( acv, a41v );
		break;
	case sh_omsprocedurestatistics:
		ret = new(*mAlloc) SysView_OMSProcedureStatistics( acv, a41v );
		break;
	case sh_omsprocedurestatisticsreset:
		ret = new(*mAlloc) SysView_OMSProcedureStatisticsReset( acv, a41v );
		break;
	case sh_omsversions:
		ret = new(*mAlloc) SysView_OMSVersions( acv, a41v );
		break;
	case sh_optimizerinformation:
		ret = new(*mAlloc) SysView_OptimizerInformation( acv, a41v );
		break;
	case sh_pagelocks:
		ret = new(*mAlloc) SysView_PageLocks( acv, a41v );
		break;
	case sh_regionlocks:
		ret = new(*mAlloc) SysView_RegionLocks( acv, a41v );
		break;
	case sh_regionstatistics:
		ret = new(*mAlloc) SysView_RegionStatistics( acv, a41v );
		break;
	case sh_regionstatisticsreset:
		ret = new(*mAlloc) SysView_RegionStatisticsReset( acv, a41v );
		break;
	case sh_restartinformation:
		ret = new(*mAlloc) SysView_RestartInformation( acv, a41v );
		break;
	case sh_schemas:
		ret = new(*mAlloc) SysView_Schemas( acv, a41v );
		break;
	case sh_sessions:
		ret = new(*mAlloc) SysView_Sessions( acv, a41v );
		break;
	case sh_spinlockstatistics:
		ret = new(*mAlloc) SysView_SpinLockStatistics( acv, a41v );
		break;
	case sh_spinlockstatisticsreset:
		ret = new(*mAlloc) SysView_SpinLockStatisticsReset( acv, a41v );
		break;
	case sh_tablesizedetails:
		ret = new(*mAlloc) SysView_TableSizeDetails( acv, a41v );
		break;
	case sh_taskgroupstatistics:
		ret = new(*mAlloc) SysView_TaskGroupStatistics( acv, a41v );
		break;
	case sh_taskgroupstatisticsreset:
		ret = new(*mAlloc) SysView_TaskGroupStatisticsReset( acv, a41v );
		break;
	case sh_taskstatistics:
		ret = new(*mAlloc) SysView_TaskStatistics( acv, a41v );
		break;
	case sh_taskstatisticsreset:
		ret = new(*mAlloc) SysView_TaskStatisticsReset( acv, a41v );
		break;
	case sh_transactions:
		ret = new(*mAlloc) SysView_Transactions( acv, a41v );
		break;
	case sh_version:
		ret = new(*mAlloc) SysView_Version( acv, a41v );
		break;
    default:
        ret = NULL;
    }

    return (ret);
}

void SVReleaseInstance( tak_all_command_glob	&acv,
                        SysView_ISysView *		sv )
{
    SAPDBMem_IRawAllocator *mAlloc;
    
    
    if ( sv != NULL )
    {
        mAlloc = reinterpret_cast<SAPDBMem_IRawAllocator *> (acv.a_transinf.tri_trans.trAllocator_gg00);
 
        destroy(sv, *mAlloc);
    }
    
}


externPascal int a400SVGetColCount( tak_all_command_glob	&acv,
                                    tak40_show_glob			&a41v )
{
    
    SysView_ISysView *	sv;
    int					iCols;
    
    sv = SVGetInstanceFromSH ( acv, a41v );
    
    if( sv )
    {
        iCols = sv->GetColCount();
    }
    else
    {
        iCols = -1;
        a07_b_put_error (acv, e_unknown_tablename, 1);
    }
    
    SVReleaseInstance ( acv, sv );
    
    return ( iCols );
}


externPascal void a400SVDefSysViewTable( tak_all_command_glob	&acv,
                                         tak40_show_glob		&a41v ) 
{
    SysView_ISysView *	sv;
    
    sv = SVGetInstanceFromSH ( acv, a41v );
    
    if( sv )
    {
        sv->Create();
    }
    else
    {
        a07_b_put_error (acv, e_unknown_tablename, 1);
    }
    
    SVReleaseInstance ( acv, sv );
}


externPascal int a400SVEstimateRows( tak_all_command_glob	&acv,
                                     tak40_show_glob		&a41v )
{
    SysView_ISysView *	sv;
    int					iRows;
    
    sv = SVGetInstanceFromSH ( acv, a41v );
    
    if( sv )
    {
        iRows = sv->EstimateRows();
    }
    else
    {
        iRows = -1;
        a07_b_put_error (acv, e_unknown_tablename, 1);
    }
    
    SVReleaseInstance ( acv, sv );
    
    return ( iRows );
}

externPascal void a400SVExecute( tak_all_command_glob	&acv, 
                                 tak40_show_glob		&a41v)
{
    SysView_ISysView *	sv;
    
    sv = SVGetInstanceFromSH ( acv, a41v );

    if( sv )
    {
        sv->Execute( );
    }
    else
    {
        a07_b_put_error (acv, e_unknown_tablename, 1);
    }
    
    SVReleaseInstance ( acv, sv );
}
