/****************************************************************************

  module      : gbd700.h

  -------------------------------------------------------------------------

  responsible : AlexanderK

  special area: eventing 
  description : the basic event class is defined


  last changed: 1999-07-22  17:34
  see also    : example.html ...

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1999



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/

// CRS 1103110 Alexander Kley 20/07/99

#ifndef GBD700_H
#define GBD700_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gsp00.h"
#include "ggg00.h"
#include "vsp0031.h"
#include "heo60.h"
#include "hgg01_2.h"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/
#define MAX_NUM_EVENTS                    101
#define SINGLE_VALUE_ACTIVATE_DIFF_BD700  25     // in % * 100 ->  ~ 0.25%
#define OVERFLOW_BD700                    true
#define UNDERFLOW_BD700                   false          
#define NIL_GRID_DISTANCE_BD700           -1
/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/
#define MIN(a, b)  (((a) < (b)) ? (a) : (b)) 
#define MAX(a, b)  (((a) > (b)) ? (a) : (b)) 
/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

class cbd700_Event
{
	friend class cbd700_EventSet;

private: // members
	tsp00_Int4       evThreshold_bd700;   
    tsp31_event_prio evOverFlowPriority_bd700;
    tsp31_event_prio evUnderFlowPriority_bd700;
	bool             evReportOverFlow_bd700;
	bool             evReportUnderFlow_bd700;
	bool             evIsGridValue_bd700;

private: // methods 
    cbd700_Event ();
	void    bd700_Init();                                                            // PTS 1102939 AK 31-05-99
};

/*---------------------------------------------------------------------------*/

class cbd700_EventSet
{	
public:  // methods
	bool bd700AddEvent      (const int & Threshold, const bool bOverflow, const tsp31_event_prio_Param & Priority);
	// definiert ein Event, d.h beim ber/unterschreiten des Schwellwertes Threshold soll ein Event ausgelst werden
	bool bd700DelEvent      (const int & Threshold ,const bool bOverflow);
	// entfernt ein Event aus der Menge der Events
	bool bd700DelGridEvent  ();
	// lscht alle als Gridevents bezeichneten Events
	void bd700Control       (const int & Value);    
	//berwacht den Wert Value und lst ein Event aus, wenn einer der angegebenen Schwellwerte passiert wird
	void bd700GetEvents     (tsp31_short_event_desc & ShortEventDesc, const bool bOverflow);
    // fllt die Struktur ShortEventDesc mit allen gesetzten Schwellwerten und ihren Prioritten 
	bool bd700SetGridEvent  (const int MinThreshold, 
							 const int MaxThreshold, 
							 const int NumThresholds,
							 const tsp31_event_prio_Param &Priority);
	// setzt eine Menge von NumThresholds quidistant verteilten Schwellwerten zwischen den Schwellwerten MinThreshold und MaxThreshold
	cbd700_EventSet (const tsp31_event_ident_Param  OverFlowEventId, 
		             const tsp31_event_ident_Param  UnderFlowEventId,
					 const tsp31_event_ident_Param  PassGridValueEventId);
	
private: // methods
	bool bd700_AddEmptyEvent         (const int   Index, const int  Threshold);
	bool bd700_DelEventIfItIsEmpty   (const int   Index);
	int  bd700_FindEventIndex        (const int & Value, bool &bFound);
	void bd700_LookForOverFlowEvent  (const int & Value);
	void bd700_LookForUnderFlowEvent (const int & Value);
	void bd700_ReportEvent           (const tsp31_event_ident_Param EventId, 
		                              const tsp31_event_prio_Param  EventPriority, 
				                      const int                     EventThreshold);
	
private: // members
	tsp00_Int4         evsIndexOfFirstActiveEvent_bd700;
	tsp00_Int4         evsLastValue_bd700;               
	tsp00_Int4         evsNumEvents_bd700;
	tsp00_Int4         evsNumSingleEvents_bd700;
	tsp00_Int4         evsReActivateEventDiff_bd700;      
	tsp00_Int4         evsGridDistance_bd700;
	tsp31_event_ident  evsOverFlowSingleEventId_bd700; 
	tsp31_event_ident  evsUnderFlowSingleEventId_bd700;
	tsp31_event_ident  evsPassGridValueEventId_bd700; 
    tsp31_event_prio   evsPassGridValuePriority_bd700;

	cbd700_Event       evsEvent_bd700  [MAX_NUM_EVENTS - 1];
};

/*===========================================================================*
 *  DEFINITION OF CLASS METHODS                                              *
 *===========================================================================*/

/*---------------------------------------------------------------------------*/
/* CLASS cbd700_Event                                                        */
/*---------------------------------------------------------------------------*/

inline void
cbd700_Event::bd700_Init ()                              // PTS 1102939 AK 31-05-99
{
	ROUTINE_DBG_MSP00 ("cbd700_Event::bd700_Init");
	
	evThreshold_bd700        = 0; 
	evOverFlowPriority_bd700.becomes(sp31ep_nil);
	evUnderFlowPriority_bd700.becomes(sp31ep_nil);
	evReportOverFlow_bd700   = false; 
	evReportUnderFlow_bd700  = false;
	evIsGridValue_bd700      = false;
};

/*---------------------------------------------------------------------------*/

inline 
cbd700_Event::cbd700_Event ()
{
	ROUTINE_DBG_MSP00 ("cbd700_Event::cbd700_Event");

	bd700_Init();                                        // PTS 1102939 AK 31-05-99
};

/*---------------------------------------------------------------------------*/
/* CLASS cbd700_EventSet                                                     */
/*---------------------------------------------------------------------------*/

inline
cbd700_EventSet::cbd700_EventSet (const tsp31_event_ident_Param  OverFlowEventId, 
								  const tsp31_event_ident_Param  UnderFlowEventId,
								  const tsp31_event_ident_Param  PassGridValueEventId)
:
evsNumEvents_bd700               (0), 
evsIndexOfFirstActiveEvent_bd700 (0),
evsReActivateEventDiff_bd700     (SINGLE_VALUE_ACTIVATE_DIFF_BD700),
evsLastValue_bd700               (0),
evsGridDistance_bd700            (NIL_GRID_DISTANCE_BD700) 
{
	ROUTINE_DBG_MSP00 ("cbd700_EventSet::cbd700_EventSet");
	evsOverFlowSingleEventId_bd700.becomes (OverFlowEventId);    
	evsUnderFlowSingleEventId_bd700.becomes(UnderFlowEventId);
    evsPassGridValueEventId_bd700.becomes  (PassGridValueEventId);    
	evsPassGridValuePriority_bd700.becomes (sp31ep_nil);
}

/*---------------------------------------------------------------------------*/

inline int
cbd700_EventSet::bd700_FindEventIndex (const int &Value, bool &bFound)
{
	
	ROUTINE_DBG_MSP00 ("cbd700_EventSet::bd700_FindEventIndex");
	
	if (0 == evsNumEvents_bd700) 
	{
		bFound = false;
		return 0;
	}
	else
	{
		if (Value > evsEvent_bd700 [evsNumEvents_bd700-1].evThreshold_bd700)
		{
			bFound = false;
			return evsNumEvents_bd700;
		}
		else 
		{
			if (Value < evsEvent_bd700 [0].evThreshold_bd700)
			{
				bFound = false;
				return 0;
			}
			else
			{
				int MaxIndex  = evsNumEvents_bd700 - 1;
				int MinIndex  = 0;
				int DiffIndex = MaxIndex - MinIndex;
				int CurrIndex;
				
				while (DiffIndex > 1)
				{
					CurrIndex = (MaxIndex + MinIndex)/2;
					if (evsEvent_bd700 [CurrIndex].evThreshold_bd700 == Value)
					{
						bFound = true;
						return CurrIndex;
					}
					else if (evsEvent_bd700 [CurrIndex].evThreshold_bd700 > Value) 
						MaxIndex = CurrIndex;
					else
						MinIndex = CurrIndex;
					
					DiffIndex = MaxIndex - MinIndex;
				}
				
				if (Value == evsEvent_bd700 [MinIndex].evThreshold_bd700)
				{
					bFound = true;
					return MinIndex;
				} 
				else
				{
					if (Value == evsEvent_bd700 [MaxIndex].evThreshold_bd700)
						bFound = true;
					else
						bFound = false;
					return MaxIndex;
				}
			}
		}
	}
}

/*---------------------------------------------------------------------------*/

inline bool 
cbd700_EventSet::bd700_AddEmptyEvent (const int  Index, 
									  const int  Threshold)
{
	ROUTINE_DBG_MSP00 ("cbd700_EventSet::bd700_AddEmptyEvent");
	
	// create an new entry in the threshold list
	
	// check if there are not too many m_Events 	
	if (evsNumEvents_bd700 >= MAX_NUM_EVENTS) 
		return false;
	
	// update index of the last evsEvent_bd700
	if (Index < evsIndexOfFirstActiveEvent_bd700)
		evsIndexOfFirstActiveEvent_bd700 ++;
	
	// add space for new evsEvent_bd700
	for (int i = evsNumEvents_bd700 - 1; i >= Index; i--)
		evsEvent_bd700 [i+1] = evsEvent_bd700 [i];
	
	// insert new evsEvent_bd700
	evsEvent_bd700 [Index].bd700_Init ();                     // PTS 1102939 AK 31-05-99
	evsEvent_bd700 [Index].evThreshold_bd700 = Threshold;
	
	// increase number of Events
	evsNumEvents_bd700 ++;
	
	//return error status 
	return true;
}

/*---------------------------------------------------------------------------*/

inline bool 
cbd700_EventSet::bd700AddEvent(
							   const int                    &Threshold,
							   const bool                    bOverflow,
							   const tsp31_event_prio_Param &Priority
							   )
{
	ROUTINE_DBG_MSP00 ("cbd700_EventSet::bd700AddEvent");
	
	// find position of the new treshold
	bool bFound;
	int Index = bd700_FindEventIndex (Threshold, bFound);
	
	if (!bFound)
		if (! bd700_AddEmptyEvent (Index, Threshold))
			return (evsNumEvents_bd700>0);

	// set the priority and the purpose of the new event
	if (bOverflow)
	{  
		evsEvent_bd700 [Index].evReportOverFlow_bd700    = true;
		evsEvent_bd700 [Index].evOverFlowPriority_bd700.becomes(Priority);
	}
	else
	{
		evsEvent_bd700 [Index].evReportUnderFlow_bd700   = true;
		evsEvent_bd700 [Index].evUnderFlowPriority_bd700.becomes(Priority);
	}
	
	evsNumSingleEvents_bd700 ++;
	evsReActivateEventDiff_bd700 = SINGLE_VALUE_ACTIVATE_DIFF_BD700;

	return (evsNumEvents_bd700>0);
}

/*---------------------------------------------------------------------------*/

inline void
cbd700_EventSet::bd700_ReportEvent (const tsp31_event_ident_Param EventId, 
								    const tsp31_event_prio_Param  EventPriority, 
								    const int                     EventThreshold)
{
	ROUTINE_DBG_MSP00 ("bd700_ReportEvent");
	
	tsp31_event_description EventDescription;
    g01event_init (EventDescription);	
	EventDescription.sp31ed_priority.becomes(EventPriority);
	EventDescription.sp31ed_ident.becomes(EventId);
	EventDescription.sp31ed_text_len = 0;
	EventDescription.sp31ed_eventcnt = 0;
	EventDescription.sp31ed_value_1  = EventThreshold/100;
	EventDescription.sp31ed_value_2  = 0;  
	
	vinsert_event (EventDescription);
};

/*---------------------------------------------------------------------------*/

inline	bool 
cbd700_EventSet::bd700SetGridEvent  (const int MinThreshold, 
									 const int MaxThreshold, 
									 const int NumThresholds,
									 const tsp31_event_prio_Param &Priority)
{
	
	ROUTINE_DBG_MSP00 ("cbd700_EventSet::bd700SetGridEvent");
	
	// check input parameter 
	if (NumThresholds <= 0) 
		return (evsNumEvents_bd700>0);
	
	bool bFound;
	int  Index;
	int  Distance = (MaxThreshold - MinThreshold)/NumThresholds;
	
	// delete previous set of grid values since there can exist only one set of those values
	bd700DelGridEvent();
	
	// insert new set
	for (int iThreshold=MinThreshold; iThreshold<(MaxThreshold-Distance); iThreshold+=Distance)
	{
		Index = bd700_FindEventIndex (iThreshold, bFound);
		
		if (!bFound)
		{
			if (!bd700_AddEmptyEvent (Index, iThreshold))
			{
				bd700DelGridEvent();
				return (evsNumEvents_bd700>0);
			}
		}
		
		// Fill new Event
		evsEvent_bd700 [Index].evIsGridValue_bd700 = true;
	}

	evsPassGridValuePriority_bd700.becomes (Priority);
	evsGridDistance_bd700   = Distance;

	if (0 == evsNumSingleEvents_bd700)
		evsReActivateEventDiff_bd700 = evsGridDistance_bd700;

	return (evsNumEvents_bd700>0);
}

/*---------------------------------------------------------------------------*/

inline bool
cbd700_EventSet::bd700_DelEventIfItIsEmpty (const int Index)
{
	ROUTINE_DBG_MSP00 ("cbd700_EventSet::bd700_DelEventIfItIsEmpty");
	
	if (! evsEvent_bd700 [Index].evReportOverFlow_bd700  && 
		! evsEvent_bd700 [Index].evReportUnderFlow_bd700 &&
		! evsEvent_bd700 [Index].evIsGridValue_bd700)
	{
		// update index of the last Event
		if (Index < evsIndexOfFirstActiveEvent_bd700)
			evsIndexOfFirstActiveEvent_bd700--;
		
		for (int i = Index; i < evsNumEvents_bd700 - 1; i++)
			evsEvent_bd700 [i] = evsEvent_bd700 [i+1];		
		
		// decrease number of events
		evsNumEvents_bd700 --;	

		// Event was successfully removed
		return true;
	}

	// Event was not removed 
	return false;
}

/*---------------------------------------------------------------------------*/

inline bool
cbd700_EventSet::bd700DelEvent (const int & Threshold ,const bool bOverflow)
{
	ROUTINE_DBG_MSP00 ("cbd700_EventSet::bd700DelEvent");
	
	// check if there exists any event at all
	if (0 != evsNumEvents_bd700) 
	{		
		// find position of the threshold
		bool bFound; 
		const int  Index = bd700_FindEventIndex (Threshold, bFound);
		
		if  (bFound)
		{	
			if (bOverflow)
				evsEvent_bd700 [Index].evReportOverFlow_bd700  = false;
			else
				evsEvent_bd700 [Index].evReportUnderFlow_bd700 = false;

			evsNumSingleEvents_bd700 --;

			if (0 == evsNumSingleEvents_bd700)
				evsReActivateEventDiff_bd700 = evsGridDistance_bd700;
			
			// remove Event if neither the overflow nor the underflow 
			// event nor the passing of a grid value should be observed
			bd700_DelEventIfItIsEmpty (Index);
		}
	}
	return (evsNumEvents_bd700>0);
}

/*---------------------------------------------------------------------------*/

inline bool
cbd700_EventSet::bd700DelGridEvent  ()
{
	evsPassGridValuePriority_bd700.becomes (sp31ep_nil);

	for (int Index = 0; Index < evsNumEvents_bd700; ++Index)
	{
		evsEvent_bd700 [Index].evIsGridValue_bd700 = false;

		bd700_DelEventIfItIsEmpty (Index);
	}

	evsGridDistance_bd700 = NIL_GRID_DISTANCE_BD700;

	return (evsNumEvents_bd700>0);
}

/*---------------------------------------------------------------------------*/

inline void // PTS 1102814 UH 19990531
cbd700_EventSet::bd700GetEvents (
								 tsp31_short_event_desc &ShortEventDesc, 
								 const bool              bOverflow
								 )
{	
	ROUTINE_DBG_MSP00 ("cbd700_EventSet::bd700GetEvents");
	
	short & EventCnt = ShortEventDesc.sp31sed_eventcnt;    // PTS 1102939 AK 31-05-99
    EventCnt = 0;                                          
	
	for (int iEvent = 0; iEvent < evsNumEvents_bd700; ++iEvent)
	{
		// collect overflow events
		if (bOverflow && evsEvent_bd700 [iEvent].evReportOverFlow_bd700)
		{
			ShortEventDesc.sp31sed_eventarr[EventCnt].sp31oei_value_1  = evsEvent_bd700 [iEvent].evThreshold_bd700/100;
			ShortEventDesc.sp31sed_eventarr[EventCnt].sp31oei_priority = evsEvent_bd700 [iEvent].evOverFlowPriority_bd700;
			EventCnt ++;
			continue;
		}
		
		// collect underflow events
		if (!bOverflow && evsEvent_bd700 [iEvent].evReportUnderFlow_bd700)
		{
			ShortEventDesc.sp31sed_eventarr[EventCnt].sp31oei_value_1  = evsEvent_bd700 [iEvent].evThreshold_bd700/100;
			ShortEventDesc.sp31sed_eventarr[EventCnt].sp31oei_priority = evsEvent_bd700 [iEvent].evUnderFlowPriority_bd700;
			EventCnt ++;
		}
	}
}

/*---------------------------------------------------------------------------*/

inline void
cbd700_EventSet::bd700_LookForOverFlowEvent   (const int & Value)
{	
	ROUTINE_DBG_MSP00 ("cbd700_EventSet::bd700_LookForOverFlowEvent");
	
	if (evsIndexOfFirstActiveEvent_bd700 >= evsNumEvents_bd700) return ;
	
	cbd700_Event & CurrEvent = evsEvent_bd700 [evsIndexOfFirstActiveEvent_bd700];

	if(Value >  CurrEvent.evThreshold_bd700 + evsReActivateEventDiff_bd700)
	{
		++ evsIndexOfFirstActiveEvent_bd700;

		if (CurrEvent.evReportOverFlow_bd700 )
		{
			bd700_ReportEvent (evsOverFlowSingleEventId_bd700, CurrEvent.evOverFlowPriority_bd700, 
				CurrEvent.evThreshold_bd700);
		}

		if (CurrEvent.evIsGridValue_bd700)
		{
			bd700_ReportEvent (evsPassGridValueEventId_bd700, evsPassGridValuePriority_bd700, 
				CurrEvent.evThreshold_bd700);
		}
	}
}

/*---------------------------------------------------------------------------*/

inline void
cbd700_EventSet::bd700_LookForUnderFlowEvent  (const int & Value)
{
	ROUTINE_DBG_MSP00 ("cbd700_EventSet::bd700_LookForUnderFlowEvent");
	
	if (0 == evsIndexOfFirstActiveEvent_bd700) return;

	cbd700_Event & CurrEvent = evsEvent_bd700 [evsIndexOfFirstActiveEvent_bd700 - 1];
	
	if(Value  < CurrEvent.evThreshold_bd700 - evsReActivateEventDiff_bd700)
	{
		-- evsIndexOfFirstActiveEvent_bd700;

		if (CurrEvent.evReportUnderFlow_bd700)
		{
			bd700_ReportEvent (evsUnderFlowSingleEventId_bd700, CurrEvent.evUnderFlowPriority_bd700, 
				CurrEvent.evThreshold_bd700);
		}
		
		if (CurrEvent.evIsGridValue_bd700)
		{
			bd700_ReportEvent (evsPassGridValueEventId_bd700, evsPassGridValuePriority_bd700, 
				CurrEvent.evThreshold_bd700);
		}
	}
}

/*---------------------------------------------------------------------------*/

inline void
cbd700_EventSet::bd700Control (const int & Value)
{
	ROUTINE_DBG_MSP00 ("cbd700_EventSet::bd700Control");
	
	if (Value > evsLastValue_bd700)
		bd700_LookForOverFlowEvent  (Value);
	else if (Value < evsLastValue_bd700)
        bd700_LookForUnderFlowEvent(Value);
	
	evsLastValue_bd700 = Value;
}

/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/

#endif  /* GBD700_H */
