/* @lastChanged: "1998-10-22  8:37"
 * @filename:    gco01.h
 * @purpose:     "LiveCache Interface used by DB-Procedures"
 * @release:     7.2.0.0
 * @see:         ""
 *
 * @copyright:   "Copyright by SAP AG, 1999" 


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/

#if defined(GCO01_ONLY_MEMBER_FUNCS)
#undef  GCO01_H
#undef  VIRTUAL
#undef  PURE_VIRTUAL
#endif
#ifndef GCO01_H
#define GCO01_H

#include "geo00.h"
#include "ggg00.h"
#include "livecachetypes.h" // nocheck
//#include "ggg91.h"
#include "gco00.h"

#if defined(GCO01_ONLY_MEMBER_FUNCS)
#define VIRTUAL 
#define PURE_VIRTUAL
#else
#define VIRTUAL       virtual
#define PURE_VIRTUAL  =0

#include "gco01_1.h"

externC const IID IID_IliveCacheSink;

class SAPDBMem_RawAllocator;
class OmsAllocatorInterface;

/*!
  class :  IliveCacheSink

  description: Definition of liveCache kernel sink, i.e. definition of those functions that
               can be called from the liboms to obtain kernel support.
*/

externC {

  struct IliveCacheSink : public IUnknown
    {
    public:
#endif

         /*!-----------------------------------------------------------------------
  
         function:     ABAPRead

         description:  reads rows of an ABAP table into a buffer.
         
                       Since the ABAP table resides at the application site, a request to the precompiler runtime must 
                       be performed. This is realized by a call of vreply. The precompiler runtime answers by a
                       call of sql_request.

         arguments:
           tabHandle [in ] pointer to the identification of the ABAP table.
           count     [out] number or delivered rows of the ABAP table.
           bufSize   [out] size of the buffer identified by buf
           buf       [out] pointer of the buffer containing the required rows
           error     [out] error code of the operation, e_ok if no error occured.
         
         return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE ABAPRead( 
            /* [out][in] */ OmsTypeInt4   &tabHandle,
            /* [out]     */ OmsTypeInt4   &count,
            /* [out]     */ OmsTypeInt4   &bufSize,
            /* [out]     */ OmsTypeByte*  &buf,
            /* [out]     */ short         &error) PURE_VIRTUAL;
        
        /*!-----------------------------------------------------------------------
  
        function:    ABAPWrite
  
        description: writes rows from a buffer into an ABAP table.
        
                     Since the ABAP table resides
                     at the application site, a request to the precompiler runtime must be performed.
                     This is realized by a call of vreply. The precompiler runtime answers by a
                     call of sql_request.

        arguments:
          TabHandle[in ] pointer to the identification of the ABAP table
          row_size [in ] size of one row of the ABAP table.
          row_count[in]  number of rows stored in the buffer lpv.
          lpv      [in]  pointer to the buffer where the rows of the ABAP table stored
          lpb_error[out] error code of the operation, e_ok if no error occured.
         
        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE ABAPWrite( 
            /* [in] */ OmsTypeInt4 *TabHandle,
            /* [in] */ tsp00_Int4 row_size,
            /* [in] */ tsp00_Int4 row_count,
            /* [in] */ unsigned char  *lpv,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     AllocPacket
  
       description:  Allocates a piece of memory which is used as a sql packet.
       
                     In the course of a dbproc execution the sqlc++ class uses this packet to send sql requests
                     to the kernel.

       arguments:
         p [out]  pointer to the packet. NULL, if no more memory is available.  
         sz [out] size of the packet.

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/
        
        VIRTUAL HRESULT STDMETHODCALLTYPE AllocPacket( 
            /* [out] */ unsigned char  * *p,
            /* [out] */ tsp00_Int4  *sz) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     BasisError
  
       description:  Sets the sql code of the kernel. 

       arguments:
         b_error [in] identifies the error. This error is transformed into an sql error
                      by a call of a07_b_put_error.

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/
               
        VIRTUAL HRESULT STDMETHODCALLTYPE BasisError( 
        /* [in] */ short b_error) PURE_VIRTUAL;
        
        VIRTUAL HRESULT STDMETHODCALLTYPE CloseVersion( 
            /* [in] */ OmsVersionId  *lpVersionId,
            /* [in] */ tsp00_Int4 lastKey,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;
      
      /*!-----------------------------------------------------------------------
  
       function:     ConsoleMsg
  
       description:  writes a message into knldiag 

       arguments:
         buflen [in] length of the message
         lpv    [in] pointer to the message

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/    

        VIRTUAL HRESULT STDMETHODCALLTYPE ConsoleMsg( 
            /* [in] */ tsp00_Int4 buflen,
            /* [size_is][in] */ unsigned char  *lpv) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     CreateOpenVersion
  
       description:  checks, if the conditions required to open a version are fulfilled. 
       
                     This means that no updates must have been executed in the current transaction.
                     If the function is called for creating a version, a version context is constructed
                     and returned to the caller. This context must be provided for all following
                     OMS calls in that version.

       arguments:
         lpConsistentView [in]  the consistent view definining the version.
         lpVersionId      [in]  the name of the version
         createVersion    [in]  true, if the function is called for creating the version, false if
                               the function is called for opening the version.
         lpVersionContext [out] the version context. Has no meaning, if createVersion is false.
         lpb_error        [out] error code of the operation, e_ok if no error occured.

         return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/    
  
        VIRTUAL HRESULT STDMETHODCALLTYPE CreateOpenVersion( 
            /* [in] */ unsigned char  *lpConsistentView,
            /* [in] */ OmsVersionId  *lpVersionId,
            /* [in] */ boolean createVersion,
            /* [out] */ unsigned char  *lpVersionContext,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     CreateKeyRangeInterator
  
       description:  Creates an iterator running over a defined key range
             
             This function creates a key iterator which runs over all objects with
			       keys between the given start and stop keys. The iterator is returned via a
			       void pointer. If the pointer to the start key  is empty and the start key len is zero
			       the iterator starts with the smallest key. If the stop key pointer is nil and the stop key len
			       is zero the iterator stops at the largest key. The iterator can be dereferenced by the 
			       function NexObjFromKeyIterator. 

       arguments:
         consistentView  [in]  kb identification of the current consistent view.
         lpVersionContext[in]  NULL, if the function is called outside of a version. Otherwise
                               a version context returned by CreateOpenVersion.
         objFile         [in]  bd identification of the container containing the object.
                               If the fileid is of the type unknown (ObjFileId.fileObjFileType_gg00 == oftUnknownType_egg00)
                               the root and the fileno of the file containing the wanted object
                               is returned in the fileId.
         keyLen          [in]  the key length of the container.
         pStartKey       [in]  a pointer to the start key of length keyLen
         pStopKey        [in]  a pointer to the stop key of length keyLen
         e               [out] error code of the operation, e_ok if no error occured.

         return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/  

        VIRTUAL HRESULT STDMETHODCALLTYPE CreateKeyRangeIterator(
           tgg91_TransNo&          consistentView,
           tgg00_OmsVersionContext *lpVersionContext,
           tgg00_FileId&           objFile,
           tsp00_Int4              keyLen,
           const void*             pStartKey,
           const void*             pStopKey,
           void**                  pKeyRangeIterator,
           short&                  e) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:    DeleteAllObj
  
       description: Deletes all objects of a container.

                    All objects of the file given by the object file id are deleted. 
                    If there is any object which cannot be deleted since the current 
                    image does not belong to the consistent view of the current 
                    transaction the error e_object_dirty is returned. In principle 
                    this operations works like the following command b04DelObj 
                    which is applied to all objects of a file.

       arguments:
         lpConsistentView [in,out]  kb identification of the current consistent view.
         lpContainerId    [in]  bd identification of the container.
         lpVersionContext [in]  NULL, if the function is called outside of a version. Otherwise
                                a version context returned by CreateOpenVersion.
         deleted          [out] number of objects deleted.
         lpb_error        [out] error code of the operation, e_ok if no error occured.
         errorOid         [out] the oid of the object causing the error

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/    
        
        VIRTUAL HRESULT STDMETHODCALLTYPE DeleteAllObj( 
            /* [out][in] */ tgg91_TransNo&  consistentView,
            /* [in] */      unsigned char  *lpContainerId,
            /* [in] */      unsigned char  *lpVersionFile,
            /* [out] */     int            &lpDeleted,
            /* [out] */     short          &lpb_error,
            /* [out] */     OmsTypeOid     &errorOid) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:     DeleteObj
  
       description:  Deletes a single object
             
             The object identified by an object identifier (OID) is deleted.
             If the current image stored in the object frame given by the OId does not contain the object 
             with the wanted version the error e_object_dirty is returned. 

       arguments:
         lpConsistentView [in] kb identification of the current consistent view.
         lpContainerId    [in] bd identification of the container containing the object.
         lpVersionContext [in] NULL, if the function is called outside of a version. Otherwise
                               a version context returned by CreateOpenVersion.
         lpoid            [in] object identification identifying the object to be deleted.
         lpobjseq         [in] since a frame can contain a number of different objects by the time
                               a unique definition of an object requires beside the OId also a kind
                               of time stamp. WantedObjVers is such a time stamp (actually it is the log position). 
         lpb_error        [out] error code of the operation, e_ok if no error occured.
 
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/    
        
        VIRTUAL HRESULT STDMETHODCALLTYPE DeleteObj( 
            /* [in] */ unsigned char  *lpConsistentView,
            /* [in] */ unsigned char  *lpContainerId,
            /* [in] */ unsigned char  *lpVersionContext,
            /* [size_is][in] */ unsigned char  *lpoid,
            /* [size_is][in] */ unsigned char  *lpobjseq,
            /* [in] */ unsigned char  *lpcoldesc,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------

        function   : DestroyKeyRangeIterator
        
        description: Releases all the KB resource held by the iterator
        
        arguments:
          pKeyRangeIterator [in] pointer to the key range iterator
       -----------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE DestroyKeyRangeIterator(
            /* [in] */ void* pKeyRangeIterator) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:     EndConsistentView

       description:  ends the current consistent view. Subsequently, the current transaction
                     has an undefined consistent view state.
  
       arguments:
         lpConsistentView [in,out] kb identification of the consistent view.
         lpb_error        [out] error code of the operation, e_ok if no error occured.

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/    
        
        VIRTUAL HRESULT STDMETHODCALLTYPE EndConsistentView( 
            /* [out][in] */ unsigned char  *lpConsistentView,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:     EnterCriticalSection

       description:  enters a critical section by calling vbegexcl.
  
       arguments:
         regionId [in] identifies the critical section to be entered.

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/    
        
        VIRTUAL HRESULT STDMETHODCALLTYPE EnterCriticalSection(/*in */ short regionId) PURE_VIRTUAL;
 
      /*!-----------------------------------------------------------------------
  
       function:     GetErrorMsg

       description:  yields an error description for a given sql code.
  
       arguments:
         errorno  [in]     the sql code
         lpbuflen [in,out] in : the size of the buffer identified by lpv, out : the length
                          of the description.
         lpv      [out]    buffer which takes the error description.


       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/ 
        
        VIRTUAL HRESULT STDMETHODCALLTYPE GetErrorMsg( 
            /* [in] */ short errorno,
            /* [out][in] */ tsp00_Int4  *lpbuflen,
            /* [size_is][out][in] */ unsigned char  *lpv) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     GetObj

       description:  Reads a consistent image of an object via an object identified (OID)

             This function returns the image of an object defined by the given OId. The returned image must fit
             with the consistent view of the current transaction.

       arguments  : 
         consistentView [in]     kb identification of the current consistent view.
         containerId    [in,out] bd identification of the container containing the object.
                                 If the fileid is of the type unknown (ObjFileId.fileObjFileType_gg00 == oftUnknownType_egg00)
                                 the root and the fileno of the file containing the wanted object
                                 is returned in the fileId. 
         lpVersionContext [in]  NULL, if the function is called outside of a version. Otherwise
                                a version context returned by CreateOpenVersion.
         oid              [in]  object identifier of the  wanted object.
         doLock           [in]  true, if the object shall be locked; false otherwise.
         objSeq           [out] time stamp (=log position)of the consistent object image. 
         bodySize         [in]  Size of the destination buffer where the found object image has to be copied.
         pBody            [out] pointer to the destination buffer, i.e. memory where the object is returned.
         bodyLen          [out] length of the object.
         histLogCount     [out] number of jumps back in the history necessary to find the wanted object image.
         e                [out] error code of the operation, e_ok if no error occured.
      
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/ 
        
        VIRTUAL HRESULT STDMETHODCALLTYPE GetObj( 
            /* [out][in] */ tgg91_TransNo&                 consistentView,
            /* [out][in] */ tgg00_ContainerId&             containerId, 
            /*      [in] */ const tgg00_OmsVersionContext* pVersionContext,
            /*      [in] */ const OmsTypeOid&              oid,
            /*      [in] */ bool                           doLock,
            /* [out]     */ tgg91_PageRef&                 objSeq,
            /*      [in] */ tsp00_Int4                     bodySize,
            /* [out]     */ unsigned char*                 pBody,
            /* [out]     */ tsp00_Int4&                    bodyLen,
            /* [out]     */ tsp00_Int4&                    histLogCount,
            /* [out]     */ tgg00_BasisError&              e) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     GetObjWithKey

       description:  Reads a consistent image of an object via a key.

             This function returns the image of an object defined by a key. The returned image must fit
             with the consistent view of the current transaction.

       arguments  : 
         consistentView [in]     kb identification of the current consistent view.
         lptree         [in,out] bd identification of the container containing the object.
                                 If the fileid is of the type unknown (ObjFileId.fileObjFileType_gg00 == oftUnknownType_egg00)
                                 the root and the fileno of the file containing the wanted object
                                 is returned in the fileId. 
         pVersionContext  [in]  NULL, if the function is called outside of a version. Otherwise
                                a version context returned by CreateOpenVersion.
         keylen           [in]  length of the wanted key in bytes.
         lpkey            [in]  pointer to the wanted key.
         doLock           [in]  true, if the object shall be locked; false otherwise.
         bodySize         [in]  Size of the destination buffer where the found object image has to be copied.
         lpbody           [out] pointer to the destination buffer, i.e. memory where the object is returned.
         lpoid            [out] object identifier of the wanted object.
         lpobjseq         [out] time stamp (=log position)of the consistent object image. 
         histLogCount     [out] number of jumps back in the history necessary to find the wanted object image.
         e                [out] error code of the operation, e_ok if no error occured.
      
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/ 
      
        VIRTUAL HRESULT STDMETHODCALLTYPE GetObjWithKey( 
            /* [out][in] */ unsigned char  *lpConsistentView,
            /* [in] */ unsigned char  *lptree,
            /* [in] */ unsigned char  *lpVersionContext,
            /* [in] */ tsp00_Int4 keylen,
            /* [in] */ unsigned char  *lpkey,
            /* [in] */ boolean doLock,
            /* [in] */ tsp00_Int4 bodysize,
            /* [size_is][out] */ unsigned char  *lpbody,
            /* [out] */ OmsTypeOid  *lpoid,
            /* [size_is][out] */ unsigned char  *lpobjseq,
            /* [out] */ int  *hist_log_count,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:     HResultError

       description:  Sets the sql return code for a HRESULT error.

       arguments  : 
         HResultError [in] the HRESULT error.
      
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/ 
        
        VIRTUAL HRESULT STDMETHODCALLTYPE HResultError( 
            /* [in] */ HRESULT h_error) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     IsDebugEnvironment

       description: tells, whether a dbproc is running in the liveCache proxy.  

       arguments  : 
         none
      
       return value: 0, if the function is called in the kernel, 1 otherwise (liveCache proxy). 

       -------------------------------------------------------------------------*/ 
        
        VIRTUAL HRESULT STDMETHODCALLTYPE IsDebugEnvironment( void) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     IsLocked

       description:  check, if an object is locked by the caller.
  
       arguments:
         lpContainerId [in] bd identification of the container containing the object.
         lpoid         [in] object identification (OID) of the object.
         lpb_error     [out] error code of the operation, e_ok if the object is locked 
                             by the caller, e_object_not_locked if the object is not locked
                             by the caller.

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/    
        
        VIRTUAL HRESULT STDMETHODCALLTYPE IsLocked( 
            /* [in] */ unsigned char  *lpContainerId,
            /* [in] */ OmsTypeOid  *lpoid,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;
        
       /*!-----------------------------------------------------------------------
  
       function:     LeaveCriticalSection

       description:  leaves a critical section by calling vendexcl.
  
       arguments:
         regionId [in] identifies the critical section to leave.

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/    

        VIRTUAL HRESULT STDMETHODCALLTYPE LeaveCriticalSection(/*in */ short regionId) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     LockObj

       description:  Locks an object

             If an object cannot be locked since the wanted version does not correspond to the current image
             of the object the error e_object_dirty is returned. 

       arguments :
         lpConsistentView [in]     kb identification of the current consistent view.
         lpContainerId    [in,out] bd identification of the container containing the object.
                                   If the fileid is of the type unknown (ObjFileId.fileObjFileType_gg00 == oftUnknownType_egg00)
                                   the root and the fileno of the file containing the wanted object
                                   is returned in the fileId. 
         lpoid            [in]     object identifier of the object to be locked.
         lpobjseq         [in]     wanted time stamp (=log position) of the consistent object image. 
         lpb_error        [out]    error code of the operation, e_ok if no error occured.
      
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/ 
        
        VIRTUAL HRESULT STDMETHODCALLTYPE LockObj( 
            /* [in] */ unsigned char  *lpConsistentView,
            /* [in] */ unsigned char  *lpContainerId,
            /* [in] */ OmsTypeOid  *lpoid,
            /* [size_is][in] */ unsigned char  *lpobjseq,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

         
        VIRTUAL HRESULT STDMETHODCALLTYPE MultiPurpose( 
            /* [in] */ unsigned char MessType,
            /* [in] */ unsigned char MessType2,
            /* [out][in] */ tsp00_Int4  *lpbuflen,
            /* [out][in] */ unsigned char  *lpv,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:     NewConsistentView

       description:  creates a new consistent view. If the current transaction already holds a
                     consistent view when this method is called, this view is released and
                     a new consistent view is returned.
  
       arguments:
         lpConsistentView [in,out] kb identification of the consistent view.
         lpb_error        [out] error code of the operation, e_ok if no error occured.

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/        

        VIRTUAL HRESULT STDMETHODCALLTYPE NewConsistentView( 
            /* [out][in] */ unsigned char  *lpConsistentView,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     NewObj

       description:  description: Reserves an empty object frame for new object and returns it's OId. 

             If the object owns a key this key is written into the object frame.

       arguments :
         lpConsistentView [in]  kb identification of the current consistent view.
         lpContainerId    [in]  bd identification of the container containing the object.
                                If the fileid is of the type unknown (ObjFileId.fileObjFileType_gg00 == oftUnknownType_egg00)
                                the root and the fileno of the file containing the wanted object
                                is returned in the fileId. 
         lpVersionContext [in]  NULL, if the function is called outside of a version. Otherwise
                                a version context returned by CreateOpenVersion.
         KeyPos           [in]  position of the key (if present).
         KeyColSize       [in]  length of the key (if present).
         pKeyColumn       [in]  pointer to the key if present; NULL otherwise.
         lpoid            [out] object identifier of the new object.
         lpobjseq         [out] time stamp (=log position) of the new object image. 
         lpb_error        [out] error code of the operation, e_ok if no error occured.
      
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/ 
        
        VIRTUAL HRESULT STDMETHODCALLTYPE NewObj( 
            /* [in] */ unsigned char  *lpConsistentView,
            /* [in] */ unsigned char  *lpContainerId,
            /* [in] */ unsigned char  *lpVersionContext,
            /* [in] */ tsp00_Int4 KeyPos,
            /* [in] */ tsp00_Int4 KeyColSize,
            /* [in] */ unsigned char  *KeyColumn,
            /* [out] */ OmsTypeOid  *lpoid,
            /* [size_is][out] */ unsigned char  *lpobjseq,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     NextObj

       description: returns a consistent image of an object following a given object.
             
                    The returned image fits with the consistent view of the current transaction.

       arguments  : 
         lpConsistentView [in]     kb identification of the current consistent view.
         lpContainerId    [in,out] bd identification of the container containing the object.
                                   If the fileid is of the type unknown (ObjFileId.fileObjFileType_gg00 == oftUnknownType_egg00)
                                   the root and the fileno of the file containing the wanted object
                                   is returned in the fileId. 
         lpVersionContext [in]     NULL, if the function is called outside of a version. Otherwise
                                   a version context returned by CreateOpenVersion.
         lpoid            [in,out] as input defines the position from where the search for the next object should be started.
                                   If an object frame with a consistent object image is found it's OID is returned via lpoid. 
                                   If lpoid as input parameter is nil the very first object of the file is returned.  
         lpobjseq         [out]    time stamp (=log position)of the consistent object image. 
         bodySize         [in]     Size of the destination buffer where the found object image has to be copied.
         lpbody           [out]    pointer to the destination buffer, i.e. memory where the object is returned.
         histLogCount     [out]    number of jumps back in the history necessary to find the wanted object image.
         e                [out]    error code of the operation, e_ok if no error occured, e_no_next_object if no
                                   next object has been found.
      
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/ 

        VIRTUAL HRESULT STDMETHODCALLTYPE NextObj( 
            /* [out][in] */ unsigned char  *lpConsistentView,
            /* [in] */ unsigned char  *lpContainerId,
            /* [in] */ unsigned char  *lpVersionContext,
            /* [out][in] */ OmsTypeOid  *lpoid,
            /* [size_is][out] */ unsigned char  *lpobjseq,
            /* [in] */ tsp00_Int4 bodysize,
            /* [size_is][out] */ unsigned char  *lpbody,
            /* [out] */ tsp00_Int4& objHistReadCount,         // PTS 1107819 TA 13/09/2000
            /* [out] */ short       *lpb_error) PURE_VIRTUAL;
      
      /*!-----------------------------------------------------------------------

       function   : NextObjFromKeyIterator

       description: returns the object the KeyIterator points currently and 
                    moves the iterator to the next object              
                    If this function is called the first time after the iterator was created by 
                    CreateKeyIterator it returns the very first object which resides
                    in the scope defined when the iterator was created. 
                    If the this function is called consecutively the complete 
                    scope of the iterator can be scaned.
                    After the last object was delivered the iterator sets the error
                    state in the TransactionContect to e_no_next_object and returns a nil result oid.

       arguments:
         pKeyIterator     [in]   pointer to the key range iterator
         VersionContext   [in]   NULL, if the function is called outside of an unloaded version. Otherwise
                                 pointer to a version context.
         ObjFile          [in]   bd identification of the container to be traversed.
         ObjBodySize      [in]   Size of the memory provided for the object body
         pObjBody         [in]   pointer to the position in the meory where the body
                                 of the rsult object is copied to.
         ResultOId        [out]  OId of the next object in the iterator scope
         ResultObjVers    [out]  version of the rsult object
         LogReadAccesses  [out]  total number of read accesses to the log to find the next consisten object
         e                [out]  error code of the operation, e_ok if no error occured.

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 
      
      -------------------------------------------------------------------------*/
      
        VIRTUAL HRESULT STDMETHODCALLTYPE NextObjFromKeyIterator (
                            void                   * pKeyIterator, 
                            tgg00_OmsVersionContext* VersionContext,
                            tgg00_ContainerId&       ObjFile,
                            tsp00_Int4               ObjBodySize,
                            void*                    pObjBody,
                            OmsTypeOid              &ResultOId, 
                            tgg91_PageRef           &ResultObjVers,
                            tsp00_Int4              &LogReadAccesses,
                            tsp00_Int2              &e) PURE_VIRTUAL;

        VIRTUAL HRESULT STDMETHODCALLTYPE VersionNextKey(
                            tgg00_OmsVersionContext* VersionContext,
                            const tgg00_ContainerId& ObjFile,
                            void*                    pKey,
                            tsp00_Int4               KeyLen,
                            bool                     Inclusive,
                            const void*              pStopKey,
                            OmsTypeOid&              Oid,
                            tgg00_BasisError&        e) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:     RegisterClass

       description:  Registers a container of a persistent class. If the class is not already registered,
                     a new container is created and meta data decribing the class is stored
                     in the catalog.
       arguments:
         SchemaId      [in]  identifies the schema.
         lpGUID        [in]  global unique identifier idenitifying the class.
         size          [in]  sizeof of an instance of the class.
         ContainerNo   [in]  number of the container.
         lpcoldesc     [in]  descriptor of the key of the class. 
         lpContainerId [out] bd identification of the container.
         lpb_error     [out] error code of the operation, e_ok if no error occured.

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/      

        VIRTUAL HRESULT STDMETHODCALLTYPE RegisterClass(
            /* [in] */ tsp00_Int4          SchemaId,
            /* [in] */ GUID                *lpGUID,
            /* [in] */ tsp00_KnlIdentifier *lpClassName,
            /* [in] */ tsp00_Int4          size,
            /* [in] */ OmsContainerNo      ContainerNo,
            /* [out][in] */ unsigned char  *lpcoldesc,
            /* [out] */ unsigned char      *lpContainerId,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
   
        function:     ResetVersion
 
        description:  Resets an unloaded version to its inital state, i.e. empties the
                      files containing the changes made in that version.
        arguments:
           lpVersionContext [in]  version context identifying the version.
           error            [out] error code of the operation, e_ok if no error occured.
 
        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 
 
        -------------------------------------------------------------------------*/  
 
        VIRTUAL HRESULT STDMETHODCALLTYPE ResetVersion( 
             /* [in ] */ unsigned char *lpVersionContext,
             /* [out] */ short  &error) PURE_VIRTUAL;
 
       /*!-----------------------------------------------------------------------
  
       function:     Rollback

       description:  rollback of the current transaction.

       arguments:
         buflen    [in]  no meaning.
         lpv       [in]  no meaning.
         lpb_error [out] error code of the operation, e_ok if no error occured.

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/      
        
        VIRTUAL HRESULT STDMETHODCALLTYPE Rollback( 
            /* [in] */ tsp00_Int4 buflen,
            /* [size_is][in] */ unsigned char  *lpv,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;
             
        VIRTUAL HRESULT STDMETHODCALLTYPE SetError( 
            /* [in] */ short errorno,
            /* [in] */ tsp00_Int4 buflen,
            /* [size_is][in] */ const unsigned char  *lpv,
            /* [in] */ tsp00_CodeType codeType ) PURE_VIRTUAL;
        
       /*!-----------------------------------------------------------------------
  
       function:     SQLReceive

       description:  returns the result of a SQL request sent by SQLRequest.

       arguments:
         lpv       [out] pointer to the return packet.
 
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/    

        VIRTUAL HRESULT STDMETHODCALLTYPE SQLReceive( 
            /* [out] */ unsigned char  * *lpv) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:     SQLRequest

       description:  sends a packet containing a SQL request to the kernel. This function
                     is used by the sqlc++ class whenever a SQL statement has to be executed.

       arguments:
         buflen    [in]  length of the packet. Has no meaning for inproc execution.
         lpv       [in]  pointer to the packet containing the sql request.
         lpb_error [out] error code of the operation, e_ok if no error occured.
 
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/   
        
        VIRTUAL HRESULT STDMETHODCALLTYPE SQLRequest( 
            /* [in] */ tsp00_Int4 buflen,
            /* [size_is][in] */ unsigned char  *lpv,
            /* [out] */ short  *lp_error) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     SubtransCommit

       description:  commits the current subtransaction.

       arguments:
         lpb_error [out] error code of the operation, e_ok if no error occured.
 
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/ 
        
        VIRTUAL HRESULT STDMETHODCALLTYPE SubtransCommit( 
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:     SubtransRollback

       description:  rolls back the current subtransaction.

       arguments:
         lpb_error [out] error code of the operation, e_ok if no error occured.
 
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/
        
        VIRTUAL HRESULT STDMETHODCALLTYPE SubtransRollback( 
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     SubtransStart

       description:  creates a new subtransaction.

       arguments:
         lpb_error [out] error code of the operation, e_ok if no error occured.
 
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/

        
        VIRTUAL HRESULT STDMETHODCALLTYPE SubtransStart( 
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:    UnlockObj

       description: unlocks the object identified by an OID. 

                    the caller must own a lock for the specified object.
             
       arguments  : 
         lpContainerId   [in,out] bd identification of the container containing the object.
                                  If the fileid is of the type unknown (ObjFileId.fileObjFileType_gg00 == oftUnknownType_egg00)
                                  the root and the fileno of the file containing the wanted object
                                  is returned in the fileId. 
         lpoid           [in]     identifies the object that has to be unlocked.
         lpb_error       [out]    error code of the operation, e_ok if no error occured.
      
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/ 

        
        VIRTUAL HRESULT STDMETHODCALLTYPE UnlockObj( 
            /* [in] */ unsigned char  *lpContainerId,
            /* [in] */ OmsTypeOid  *lpoid,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;
       
      /*!-----------------------------------------------------------------------
  
       function:     UpdateObj

       description: Replaces the current image of an object by a new one.

       The caller must have locked the object.

       arguments  : 
         lpConsistentView [in]     kb identification of the current consistent view.
         lpContainerId    [in,out] bd identification of the container containing the object.
                                   If the fileid is of the type unknown (ObjFileId.fileObjFileType_gg00 == oftUnknownType_egg00)
                                   the root and the fileno of the file containing the wanted object
                                   is returned in the fileId. 
         lpVersionContext [in]     NULL, if the function is called outside of a version. Otherwise
                                   a version context returned by CreateOpenVersion.
         lpoid            [in,out] as input defines the position from where the search for the next object should be started.
                                   If an object frame with a consistent object image is found it's OID is returned via lpoid. 
                                   If lpoid as input parameter is nil the very first object of the file is returned.  
         lpobjseq         [out]    time stamp (=log position)of the consistent object image. 
         bodySize         [in]     Size of the destination buffer where the found object image has to be copied.
         lpbody           [out]    pointer to the destination buffer, i.e. memory where the object is returned.
         lpcoldesc        [out]    description of the key of the object.
         e                [out]    error code of the operation, e_ok if no error occured, e_no_next_object if no
                                   next object has been found.
      
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/ 
            
        VIRTUAL HRESULT STDMETHODCALLTYPE UpdateObj( 
            /* [in] */ unsigned char  *lpConsistentView,
            /* [in] */ unsigned char  *lpContainerId,
            /* [in] */ unsigned char  *lpVersionContext,
            /* [size_is][in] */ unsigned char  *lpoid,
            /* [size_is][out][in] */ unsigned char  *lpobjseq,
            /* [in] */ tsp00_Int4 bodysize,
            /* [size_is][in] */ unsigned char  *lpbody,
            /* [in] */ unsigned char  *lpcoldesc,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
       
        function: Vtrace

        description: writes a message into the kerne vtrace

        arguments:
          buflen [in] length of the message
          lpv    [in] pointer to the message

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.
        
       -------------------------------------------------------------------------*/ 
        
        VIRTUAL HRESULT STDMETHODCALLTYPE Vtrace( 
            /* [in] */ tsp00_Int4 buflen,
            /* [size_is][in] */ unsigned char  *lpv) PURE_VIRTUAL;

        /*!-----------------------------------------------------------------------
       
        function: GetDefaultContext

        description: returns a pointer stored by SetDefaultContext

        arguments:
          p [out] a pointer stored by SetDefaultContext
          pCancelByte [out] a pointer to the cancel byte
          taskId [out] the task identification

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.
        
        -------------------------------------------------------------------------*/  

        VIRTUAL HRESULT STDMETHODCALLTYPE GetDefaultContext(
          /* [out] */ void** p, 
          /* [out] */ pasbool** pCancelByte,
          /* [out] */ tsp00_TaskId& taskId /* PTS 1110315 */) PURE_VIRTUAL;
        
        /*!-----------------------------------------------------------------------
       
        function: SetDefaultContext

        description: stores a pointer which can be inquired by GetDefaultContext

        arguments:
          p [in] a pointer.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.
        
       -------------------------------------------------------------------------*/  
        
        VIRTUAL HRESULT STDMETHODCALLTYPE SetDefaultContext(/* [in] */ void* p) PURE_VIRTUAL;

        /*!-----------------------------------------------------------------------
       
        function: GetSqlSessionContext

        description: returns a pointer stored by SetSqlSessionContext

        arguments:
          p [out] a pointer stored by SetSqlSessionContext

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.
        
        new method since PTS 1120804

        -------------------------------------------------------------------------*/  

        VIRTUAL HRESULT STDMETHODCALLTYPE GetSqlSessionContext(
          /* [out] */ void** p) PURE_VIRTUAL;
        
        /*!-----------------------------------------------------------------------
       
        function: SetSqlSessionContext

        description: stores a pointer which can be inquired by GetSqlSessionContext

        arguments:
          p [in] a pointer.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.

        new method since PTS 1120804

       -------------------------------------------------------------------------*/  
        
        VIRTUAL HRESULT STDMETHODCALLTYPE SetSqlSessionContext(/* [in] */ void* p) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
       
        function:     OpenFile

        description:  Opens a message file

                      if doAppend is true and the file already exists it is opened but writing takes place at end of file.
                      MaxFileSize is ignored in such a case.
                      if doAppend is false or the file does not exist, a file is created. If file already exists and 
                      is not still opened it is overwritten! So be carefull, choosing the filename.

                      A second open to the same file, simply increases a reference count. 
                      MaxFileSize is ignored in such a case.

                      For each open, a close must be issued.
    
        arguments:
          lpFileName [in] the file name
          FileSize   [in] the maximum size of the file. If 0 the file size is ignored. fileSize has
                          no meaning, if the file already exists and is not overwritten.
                          If not 0 writing restarts at beginning of file after MaxFileSize was reached.
          doAppend   [in] true : if the file already exist it is opened but writing takes place at end of file.
                         false : if the file already exists, it is overwritten
          FileHandle [out] the index of the message file (NOT a file handle!!!)
          lpb_error  [out] error code of the operation, e_ok if no error occured.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.
        
       -------------------------------------------------------------------------*/  

        VIRTUAL HRESULT STDMETHODCALLTYPE OpenFile(/* [in] */ char  *lpFileName,
          /* [in]  */ tsp00_Uint4 FileSize,
          /* [in]  */ boolean     doAppend, 
          /* [out] */ tsp00_Int4  *FileHandle,
          /* [out] */ short  *lpb_error) PURE_VIRTUAL;
          
       /*!-----------------------------------------------------------------------

        function: WriteFile

        description:  writes a null terminated string into a message file
    
        This expects a 0 terminated 'C String' as input. The message written is not flushed
        immediatly, but stored in internal buffers. Flushing takes place on dbpCloseMessageFile.
        The length of the message is limited to 255 bytes.
        
        NOTE - If length of message exceeds MaxFileSize, only the last 'piece' of the message will
        be visible.

        arguments:
          FileHandle [in] The file index returned by OpenFile
          msg        [in] The message to be written.
          lpb_error  [out] error code of the operation, e_ok if no error occured.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.

        -------------------------------------------------------------------------*/  

        VIRTUAL HRESULT STDMETHODCALLTYPE WriteFile(/* [in] */ tsp00_Int4 FileHandle,
          /* [in]  */ char   *msg,
          /* [out] */ short  *lpb_error) PURE_VIRTUAL; 

       /*!-----------------------------------------------------------------------

        function:     CloseFile

        description:  closes a message file
    
        This flushes the content of the buffers and allows the index to be used again by a 
        different dbpOpenMessageFile call.

        If file was opened more than once, the reference count is decreased but the file
        is kept open.

        But always the file is implicitly synchronized as if SyncFile() was called.
  
        arguments:
          FileHandle [in] : the file index returned by dbpOpenMessageFile
          lpb_error  [out] error code of the operation, e_ok if no error occured.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.

        -------------------------------------------------------------------------*/ 

        VIRTUAL HRESULT STDMETHODCALLTYPE CloseFile(/* [in] */ tsp00_Int4 FileHandle,
          /* [out] */ short  *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------

        function: SyncFile

        description:  flushes buffered messages into a message file
    
        This flushes the content of the buffers without closing the file. Not really needed for
        small files, since flushing takes place automatically before dbpCloseMessageFile.

        NOTE: Using this after each write, will reduce performance of the database dramatically...

        arguments:
          lpFileName [in]  The name of the message file as specified in OpenFile. NULL if the
                           file is identified via FileHandle.
          FileHandle [in]  the file index returned by dbpOpenMessageFile, has no meaning, if
                           lpFileName is given.
          lpb_error  [out] error code of the operation, e_ok if no error occured.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.

        -------------------------------------------------------------------------*/ 

        VIRTUAL HRESULT STDMETHODCALLTYPE SyncFile(/* [in] */ char  *lpFileName,
          /* [in] */ tsp00_Int4 FileHandle,
          /* [out] */ short  *lpb_error) PURE_VIRTUAL;

        /*!-----------------------------------------------------------------------

        function: GetDateTime

        description:  return current date and time

        arguments:
          Date [out] current date.
          Time [out] current time.
 
        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.

        -------------------------------------------------------------------------*/ 

        VIRTUAL HRESULT STDMETHODCALLTYPE GetDateTime(/* [out] */ tsp00_Date* Date,
          /* [out] */ tsp00_Time* Time) PURE_VIRTUAL;
        
       /*!-----------------------------------------------------------------------

        function: GetClock

        description:  return the current clock

        arguments:
          Sec  [out] seconds.
          MSec [out] milliseconds
 
        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.

        -------------------------------------------------------------------------*/ 

        VIRTUAL HRESULT STDMETHODCALLTYPE GetClock(/* [out] */ tsp00_Int4* Sec,
          /* [out] */ tsp00_Int4* MSec) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------

        function: DropVersion

        description:  drops a version.

        arguments:
          lpVersionContext [in]  version context identifying the version.
          lpb_error        [out] error code of the operation, e_ok if no error occured.

 
        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case.

        -------------------------------------------------------------------------*/ 

        VIRTUAL HRESULT STDMETHODCALLTYPE DropVersion( /* in */ unsigned char  *lpVersionContext,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;
        
       /*!-----------------------------------------------------------------------
  
       function:     CreateSchema

       description:  creates a schema. Stored meta data describing the schema in the catalog

       arguments:
         lpSchemaName [in]  the schema name in wyde representation, must be unique.
         lpSchemaId   [out] the schema identification. Serves as as handle for following calls
         lpb_error    [out] error code of the operation, e_ok if no error occured.
         
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE CreateSchema(
            /* [in ] */ const OmsTypeWyde *lpSchemaName,
            /* [out] */ tsp00_Int4        *lpSchemaId,
            /* [out] */ short             *lpb_error) PURE_VIRTUAL;

      /*!-----------------------------------------------------------------------
  
       function:     ExistsSchema

       description:  checks the existence of a schema

       arguments:
         lpSchemaName [in]  the schema name in wyde representation.
         lpSchemaId   [out] schema handle. NULL if schema does not exist.
         lpb_error    [out] error code of the operation, e_ok if no error occured.
         
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE ExistsSchema(
            /* [in]  */ const OmsTypeWyde *lpSchemaName,
            /* [out] */ tsp00_Int4 *lpSchemaId,
            /* [out] */ short      *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:     DropSchema

       description:  drops a schema, i.e. deletes meta data describing the schema from
                     the catalog

       arguments:
         lpSchemaId   [in ] schema handle, identifies the schema.
         lpb_error    [out] error code of the operation, e_ok if no error occured.
         
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE DropSchema(/* [in] */ tsp00_Int4 SchemaId,
            /* [out] */ short      *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
  
       function:     GetSchemaName

       description:  yields a schema name for a given schema handle.

       arguments:
         lpSchemaId   [in ] schema handle, identifies the schema.
         lpSchemaName [out] the schema name in wyde reprensentation.
         lpb_error    [out] e_ok, if the schema exists; otherwise e_unknown_name
         
       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE GetSchemaName(
            /* [in] */ tsp00_Int4    SchemaId,
            /* [out] */ OmsTypeWyde* lpSchemaName,
            /* [out] */ short      * lpb_error) PURE_VIRTUAL;

        /*!-----------------------------------------------------------------------
      
        function:     ExistsFile
        description:  Checks for existence of file or directory
    
        Called to test for existence of file or directory. 
        Sets doesExist to true if file or directory exists, false otherwise or if error occured.
        If doesExist is false isDirectory is always set to false.

        arguments:
          lpFileName   [in]  The file/directory name
          existsFile   [out] Set to true if file or directory was found, false if not.
          isDirectory  [out] Set to true to indicate directory was found, false if not.
   
        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 
       
        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE ExistsFile(/* [in] */ char  *lpFileName,
          /* [out] */ bool    &existsFile,
          /* [out] */ bool    &isDirectory,
          /* [out] */ short   &error) PURE_VIRTUAL;
        
        /*!-----------------------------------------------------------------------
        
        function:     DropFile

        description:  drops a message file
    
        This tries to drop a named MessageFile. If the file does not exist or is still open,
        it will not be dropped and an error is thrown instead.

        arguments:
          lpFileName [in] The file name of the file to be dropped.
           lpb_error [out] error code of the operation, e_ok if no error occured.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/
          
        VIRTUAL HRESULT STDMETHODCALLTYPE DropFile(/* [in] */ char  *lpFileName,
          /* [out] */ short   &error) PURE_VIRTUAL; 

        /*!-----------------------------------------------------------------------
        
        function:     GetWorkingDirectory

        description:  Get path to directory where message files are stored
    
        This returns the path to the directory that is used for all relativ specified message files.
        It should be identical to the WorkingDirectory of the database.
    
        arguments:
          pathName  [out]  The path to the root of all relativ specified message files
          lpb_error [out] error code of the operation, e_ok if no error occured.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE GetWorkingDirectory(/* [out] */ tsp00_Pathc& pathName,
          /* [out] */ short  &error) PURE_VIRTUAL; 

       /*!-----------------------------------------------------------------------
        
        function:     GetDbrootSapPath

        description:  gets the DBROOT dependent "sap" path ( libraries, shared libraries... )
    
        arguments:
          pathName [out]  the required path. 
          lpb_error [out] error code of the operation, e_ok if no error occured.
          
        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE GetDbrootSapPath(/* [out] */ tsp00_Pathc& pathName,
          /* [out] */ short  &error) PURE_VIRTUAL;
          
       /*!-----------------------------------------------------------------------
        
        function:     LoadLibraryEx

        description: loads a shared library

        arguments:
          path         [in] The name of the library to be loaded. The name can 
                           include the path but it should no include the fileextension. 
          loadGobal    [in] true  : load with 'global' scope of symbol-resolution
                            false : load with 'local' scope of symbol-resolution
          hd           [out] handle for the loaded library.
          errtext      [out] error description, if the library could not be loaded.
          errtext_size [out] length of the error description.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE LoadLibraryEx( /* in */ const char* path, 
          /* in  */ bool        loadGlobal,
          /* out */ HANDLE      *hd,
          /* out */ tsp_errtext &errtext,
          /* in  */ int         errtext_size) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
        
        function:     FreeLibrary

        description: unloads a shared library


        arguments:
          hd           [in]  handle to the shared library.
          ok           [out] true, if no error occured; otherwise false.
          errtext      [out] error description, if the library could not be loaded.
          errtext_size [out] length of the error description.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/


        VIRTUAL HRESULT STDMETHODCALLTYPE FreeLibrary( /* in */ HANDLE hd,
          /* out */ bool        &ok,
          /* out */ tsp_errtext &errtext,
          /* in  */ int         errtext_size) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
        
        function:     DropContainer

        description:  drops a class container and deletes corresponding meta data from catalog.

        arguments:
          schema   [in] schema handle of the schema which contains the class container
          guid     [in] global unique identifier describing the class stored in the container.
          cno      [in] container number identifying the container.
          error    [out] error code of the operation, e_ok if no error occured.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE DropContainer(/* [in] */ OmsSchemaHandle schema,
            /* [in] */ const GUID&        guid,
            /* [in] */ OmsContainerNo     cno,
            /* [out] */ tgg00_BasisError& error) PURE_VIRTUAL;

        /*!-----------------------------------------------------------------------
        
        function:    GetContainerInfo

        description: yields information abaout a class container. 

        arguments:
          containerHandle [in]  container handle identifying the class container.
          guid            [out] global unique identifier describing the class stored in the container.
          containerHandle [out] schema handle of the schema the container belongs to.
          cno             [out] container number of the class container.
          error           [out] error code of the operation, e_ok if no error occured.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE GetContainerInfo(/* [in] */ tsp00_Uint4 containerHandle,
            /* [out] */ GUID& guid,
            /* [out] */ OmsSchemaHandle& schema,
            /* [out] */ OmsContainerNo&  cno,
            /* [out] */ short& error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
        
        function:    IsObjHistoryUsed

        description: 

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE IsObjHistoryUsed( 
            /* [out][in] */ tgg00_ContainerId& containerId, 
            /*      [in] */ const OmsTypeOid&  oid,
            /* [out]     */ bool&              isUsed,
            /* [out]     */ tgg00_BasisError&  e) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
        
        function:    Reschedule

        description: calls the dispatcher

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/


        VIRTUAL HRESULT STDMETHODCALLTYPE Reschedule() PURE_VIRTUAL; /* PTS 1107849 */
       
       /*!-----------------------------------------------------------------------
  
        function:     LockUpdObj

        description:  Locks an object and marks it as updated

             If an object cannot be locked since the wanted version does not correspond to the current image
             of the object the error e_object_dirty is returned. 

        arguments :
         lpConsistentView [in]     kb identification of the current consistent view.
         lpContainerId    [in,out] bd identification of the container containing the object.
                                   If the fileid is of the type unknown (ObjFileId.fileObjFileType_gg00 == oftUnknownType_egg00)
                                   the root and the fileno of the file containing the wanted object
                                   is returned in the fileId. 
         lpoid            [in]     object identifier of the object to be locked.
         lpobjseq         [in,out] wanted time stamp (=log position) of the consistent object image. 
         lpb_error        [out]    error code of the operation, e_ok if no error occured.
      
        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

       -------------------------------------------------------------------------*/ 

        VIRTUAL HRESULT STDMETHODCALLTYPE LockUpdObj( 
            /* [in] */ unsigned char  *lpConsistentView,
            /* [in] */ unsigned char  *lpContainerId,
            /* [in] */ OmsTypeOid     *lpoid,
            /* [in] [out] */ unsigned char  *lpobjseq,
            /* [out] */ short  *lpb_error) PURE_VIRTUAL;

       /*!-----------------------------------------------------------------------
       
       function:     Signal

       description : signals a task to continue execution after a wait call supending
                     the task.

       arguments :
         TaskId[in] the task identification of the task to be signaled
         ok[in]    true signals the task, that the resource the task was waiting for has been
                    granted (true). Otherwise a timeout has been occured (false)

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 
       
       -------------------------------------------------------------------------*/ 
  
        VIRTUAL HRESULT STDMETHODCALLTYPE Signal(
            /* [in] */ tsp00_TaskId TaskId, tsp00_Bool ok) PURE_VIRTUAL; // PTS 1110315

       /*!-----------------------------------------------------------------------
       
       function:     Wait

       description : suspends the current task, until it is signaled to continue execution.

       arguments :
         ok[out]    signals the task, wether the resource the task was waiting for has been
                    granted (true). Otherwise a timeout has been occured (false)

       return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 
       
       -------------------------------------------------------------------------*/ 

        VIRTUAL HRESULT STDMETHODCALLTYPE Wait(/*[out]*/ tsp00_Bool& ok) PURE_VIRTUAL; // PTS 1110315

        /*!-----------------------------------------------------------------------
        
        function:     ExistsContainer

        description:  yields, whether a container exists in the catalog.
        arguments:
          SchemaId    [in] schema handle of the schema which contains the class container
          Guid        [in] global unique identifier describing the class stored in the container.
          ContainerNo [in] container number identifying the container.
          error      [out] error code of the operation, e_ok if container exists

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE ExistsContainer(
            /* [in]  */ tsp00_Int4     SchemaId,
            /* [in]  */ const GUID&    Guid,
            /* [in]  */ OmsContainerNo ContainerNo,
            /* [out] */ short&         error) PURE_VIRTUAL; // PTS 1110334

        /*!-----------------------------------------------------------------------
        
        function:     GetSessionHeapUsage

        description: returns the current heap usage of the caller
        arguments:
          heapUsage [out] heap usage in bytes

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE GetSessionHeapUsage(
            /* [out] */ tsp00_8ByteCounter& heapUsage) PURE_VIRTUAL; 


        /*!-----------------------------------------------------------------------
        
        function:     EvalCallStatistics

        description: calculates the statistcis of the last dbproc call
        arguments:
          AllocatorOnly [in] if true, only allocator statistics are returned; otherwise
                             full info is returned.
          pMonitorInfo  [in,out] the address of the monitor info. This is an address lying
                            in the application dll. 

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE EvalCallStatistics(
             bool            AllocatorOnly,
             OmsMonitorData& MonitorData) PURE_VIRTUAL; 


        /*!-----------------------------------------------------------------------
        
        function:     SimCtlDumpToHtml

        description: Implementation of omsSimDumpToHtml* methods of OmsAbstractObject.
					 This makes sense only in simulator.
        arguments:
          handle	[in]	OMS handle
		  obj		[in]	OID of the object to dump
		  str		[in]	destination for the dump data

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE SimCtlDumpToHtml(
			class OmsHandle				&handle,
			const struct OmsObjectId	&obj,
			class OmsSimDumpCollector	&str) PURE_VIRTUAL; 


        /*!-----------------------------------------------------------------------
        
        function:     SimCtlSetObjectName

        description: Implementation of omsSimSetObjectName method of OmsAbstractObject.
					 This makes sense only in simulator.
        arguments:
          handle	[in]	OMS handle
		  obj		[in]	OID of the object
		  name		[in]	new name

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE SimCtlSetObjectName(
			class OmsHandle				&handle,
			const struct OmsObjectId	&obj,
			const char					*name) PURE_VIRTUAL; 


        /*!-----------------------------------------------------------------------
        
        function:     SimCtlGetObjectName

        description: Implementation of omsSimGetObjectName method of OmsAbstractObject.
					 This makes sense only in simulator.
        arguments:
          handle	[in]	OMS handle
		  obj		[in]	OID of the object
		  name		[in]	pointer where to store pointer to name

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE SimCtlGetObjectName(
			class OmsHandle				&handle,
			const struct OmsObjectId	&obj,
			const char					**name) PURE_VIRTUAL; 


        /*!-----------------------------------------------------------------------
        
        function:     SimCtlDestroyHandleCB

        description: This callback is called on destroy of OmsHandle in simulator
					 to detach the sink.
        arguments:
          handle	[in]	OMS handle
		  sinkptr	[in]	pointer to sink pointer - NULLed if last instance

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE SimCtlDestroyHandleCB(
			class OmsHandle				&handle,
			struct IliveCacheSink		**session) PURE_VIRTUAL; 

        /*!-----------------------------------------------------------------------
        
        function:     GetSessionAllocator

        description: returns a pointer to the session allocator
        arguments: 
          allocator [out] pointer to the session allocator.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE GetSessionAllocator(SAPDBMem_RawAllocator* &allocator) PURE_VIRTUAL; 


        /*!-----------------------------------------------------------------------
        
        function:     GetOmsSessionAllocator

        description: returns a pointer to the oms session allocator
        arguments: 
          allocator [out] pointer to the oms session allocator.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE GetOmsSessionAllocator(SAPDBMem_RawAllocator* &allocator) PURE_VIRTUAL; 

        /*!-----------------------------------------------------------------------
        
        function:     SimCtlGetHTMLRef

        description: Return HTML link to an object in simulator dump.
					 This makes sense only in simulator.
        arguments:
          handle	[in]	OMS handle
		  obj		[in]	OID of the object
		  data		[in]	pointer where to store pointer to HTML code
		  flags     [in]	dump flags

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE SimCtlGetHTMLRef(
			class OmsHandle				&handle,
			const struct OmsObjectId	&obj,
			const char					**data,
			int							flags) PURE_VIRTUAL; 

        /*!-----------------------------------------------------------------------
        
        function:     IsInRegion

        description: tells, if the current session is in the requested oms region.
        arguments:
          regionId [in] the required regionId.
          inRegion [out] true, if current session is in region

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE IsInRegion(
            /* [in]  */ int   regionId,
            /* [out] */ bool& inRegion) PURE_VIRTUAL; 


        /*!-----------------------------------------------------------------------
        
        function:     GetSqlClsAllocator

        description: returns a pointer to the sql class allocator
        arguments: 
          allocator [out] pointer to the sql class allocator.

        return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

        -------------------------------------------------------------------------*/

        VIRTUAL HRESULT STDMETHODCALLTYPE GetSqlClsAllocator(SAPDBMem_RawAllocator* &allocator) PURE_VIRTUAL; 

	/*!-----------------------------------------------------------------------

	function:     GetGlobalUserAllocator

	description: returns a pointer to the global user allocator
	arguments: 
	allocator [out] pointer to the allocator.
	name      [in]  name for named allocator, NULL otherwise

	return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

	-------------------------------------------------------------------------*/

	VIRTUAL HRESULT STDMETHODCALLTYPE GetGlobalUserAllocator(
		OmsAllocatorInterface* &allocator, const char *name) PURE_VIRTUAL; 

	/*!-----------------------------------------------------------------------

	function:     GetSessionUserAllocator

	description: returns a pointer to the session user allocator
	arguments: 
		allocator [out] pointer to the allocator.

	return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

	-------------------------------------------------------------------------*/

	VIRTUAL HRESULT STDMETHODCALLTYPE GetSessionUserAllocator(
		OmsAllocatorInterface* &allocator) PURE_VIRTUAL; 

	/*!-----------------------------------------------------------------------

	function:     GetTransactionUserAllocator

	description: returns a pointer to the transaction user allocator
	arguments: 
		allocator [out] pointer to the allocator.

	return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

	-------------------------------------------------------------------------*/

	VIRTUAL HRESULT STDMETHODCALLTYPE GetTransactionUserAllocator(
		OmsAllocatorInterface* &allocator) PURE_VIRTUAL; 

	/*!-----------------------------------------------------------------------

	function:     GetCOMRoutineUserAllocator

	description: returns a pointer to the COM Routine user allocator
	arguments: 
		allocator [out] pointer to the allocator.

	return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

	-------------------------------------------------------------------------*/

	VIRTUAL HRESULT STDMETHODCALLTYPE GetCOMRoutineUserAllocator(
		OmsAllocatorInterface* &allocator) PURE_VIRTUAL; 

	/*!-----------------------------------------------------------------------

	function:     UserAllocatorCtl

	description: control of user allocator
	arguments: 
		msg   [in] message
		param [in] parameter block

	return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

	-------------------------------------------------------------------------*/

	VIRTUAL HRESULT STDMETHODCALLTYPE UserAllocatorCtl(
		int msg, void *param) PURE_VIRTUAL; 

    /*!-----------------------------------------------------------------------

	function:    IsUnicodeInstance

	description: returns true, if liveCache user's (SAPR3) default code is unicode
	arguments: 
		isUnicodeInstance [out] true or false
        error             [out] basis error

	return value: HRESULT of the operation. Has no meaning (S_OK) in inproc case. 

	-------------------------------------------------------------------------*/

    VIRTUAL HRESULT STDMETHODCALLTYPE IsUnicodeInstance(/* [out] */ bool& isUnicodeInstance,
        /* [out] */ short& sqlError) PURE_VIRTUAL;

#if !defined(GCO01_ONLY_MEMBER_FUNCS)
    };
}
#endif
/*! endclass: IliveCacheSink */

#endif  /* GCO01_H */
