/*!================================================================
 module:        gco02.h
 
 responsible:   ThomasA
 
 special area:  OMS
 
 description:   LibOms Interface
 
 see:
 
 copyright:     SAP AG 1998 


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 ===================================================================*/



#if !defined (__GCO02_H)
#define __GCO02_H

#include "gsp00.h"
#include "ggg92.h"

struct IliveCacheSink;

typedef IliveCacheSink* (*GetSinkPtr_co02)(void);
typedef void (*SignalTimeoutFunc_co02)(tsp00_TaskId); // PTS 1110287
typedef tsp00_Int4 (*Clock_co02)(void); // PTS 1110287
typedef void* (*GetKnlVersion_co02)(void*,int); // PTS 1110913

#define LABEL_OMS_CLASS "OMSCLASS", 1901
#define LABEL_OMS_CONTAINER "CONTAINER", 1902
#define LABEL_OMS_VERSION "OMSVERS", 1903
#define LABEL_OMS_CONTEXT "OMSCNTXT", 1904
#define LABEL_OMS_SESSION "OMSSESS", 1905
#define LABEL_OMS_BEFORE_IMAGE "OMSBIMAGE", 1906
#define LABEL_OMS_OID_CACHE "OMSCACHE", 1907
#define LABEL_OMS_LOCAL_CLS_DIR "OMSLOCAL", 1908
#define LABEL_OMS_LOCAL_CLS_HASH "OMSLCLS ", 1909
#define LABEL_OMS_LOCAL_GUID_HASH "OMSLGUID", 1910
#define LABEL_OMS_LOCAL_CLASS "OMSLCLS", 1911
#define LABEL_OMS_BOUND_VERSION "OMSBNDV", 1912

class IDump_cco02 {
public :
  virtual void* GetOmsSession() = 0;
  virtual void  SetDumpLabel (const char* label, int dumpType) = 0;
  virtual void  Dump (void* p, int len) = 0;
  virtual bool  Synchronize() const = 0;
};

class ITrace_co02
{
public :
  virtual void Trace( const char*fmt, ... ) = 0;
};

/*!
  class      :  LibOmsInterface_cco02

  description:  Defines the interface of the liboms, i.e. those functions callable from the kernel  
  
*/

class LibOmsInterface_cco02 {
public :

  /*!-----------------------------------------------------------------------

  function: GetFirstVersion
  description: starts an iterator, which yields information about all oms 
               versions. 
         
  arguments:
    lcSink         [in ] Pointer to the kernel sink
    versionId      [out] the name of the first version
    createDate     [out] date when version has been created
    createTime     [out] time when version has been created
    openDate       [out] last open date of the version
    openTime       [out] last open time of the version
    consistentView [out] identification of the consistent view defined by the version
    isMarked       [out] true, if the version is marked via a call of omsMarkVersion
    isOpen         [out] true, if version is currently open
    isUnloaded     [out] true, if the version has been unloaded due to a lack of memory
    heapUsage      [out] memory occupied by the version (in bytes)
    hashSize       [out] size of oid hash header
    versionTree    [out] identification of the b* tree, which contains the changed objects
                         of the version. Has no meaning, if isUnloaded is false
    versionInvTree [out] identification of the b* tree, that implements keyed objects, if the
                         version is unloaded
    versionDesc    [out] user-defined description of a version   
    error          [out] error-code if an error occurred

  return value : true, if at least one version exists; false otherwise
    
  -------------------------------------------------------------------------*/

  virtual tsp00_Bool GetFirstVersion(
    IliveCacheSink*     lcSink,
    tsp00_C24&          versionId,
    tsp00_Date&         createDate,
    tsp00_Time&         createTime,
    tsp00_Date&         openDate,
    tsp00_Time&         openTime,
    tgg91_TransNo&      consistentView,
    tsp00_Bool&         isMarked,
    tsp00_Bool&         isOpen,
    tsp00_Bool&         isUnloaded,
    tsp00_8ByteCounter& heapUsage,
    tsp00_Int4&         hashSize,
    tgg00_FileId&       versionTree,
	  tgg00_FileId&       versionInvTree,
    tsp00_C512&         versionDesc, // PTS 1117690
    short&              error) = 0;  // PTS 1126698 

  /*!-----------------------------------------------------------------------

  function: GetMonitorInfo
  description: implements an iterator which returns monitor info of dbprocedures. Every call of the function
               yields a handle, that must be provided for the next call. In the first call
               the handle must be NULL.
          
  arguments:
    lcSink  [in]     pointer to the kernel sink
    handle  [in,out] handle, i.e. state of the iterator. Must be NULL for the first call.
    iid     [out]    Interface Identification
    dispid  [out]    displacement identification of the dbprocedure, i.e. index in vtable.
    monInfo [out]    monitor data

  return value : false, if end of iterator is reached, true otherwise
    
  -------------------------------------------------------------------------*/

  virtual tsp00_Bool GetMonitorInfo(
    IliveCacheSink*       lcSink,
    tsp00_Addr&           handle,
    tsp00_C16&            iid,
    tsp00_Int4&           dispid,
    tgg00_COMMonitorInfo& monInfo) = 0;

  /*!-----------------------------------------------------------------------
  
  function: GetNextVersion
  
  description: yields information about the next oms version. After a call of
               GetFirstVersion GetNextVersion must be called until it returns
               false.
         
  arguments:
    lcSink         [in ] pointer to the kernel sink
    versionId      [out] the name of the version
    createDate     [out] date when version has been created
    createTime     [out] time when version has been created
    openDate       [out] last open date of the version
    openTime       [out] last open time of the version
    consistentView [out] identification of the consistent view defined by the version
    isMarked       [out] true, if the version is marked via a call of omsMarkVersion
    isOpen         [out] true, if version is currently open
    isUnloaded     [out] true, if the version has been unloaded due to a lack of memory
    heapUsage      [out] memory occupied by the version (in bytes)
    hashSize       [out] size of oid hash header
    versionTree    [out] identification of the b* tree, which contains the changed objects
                         of the version. Has no meaning, if isUnloaded is false
    versionInvTree [out] identification of the b* tree, that implements keyed objects, if the
                         version is unloaded
    versionDesc    [out] user-defined description of a version    
    error          [out] error-code if an error occurred

  return value : false, if the iterator stands behind the last version, true otherwise
    
  -------------------------------------------------------------------------*/

  virtual tsp00_Bool GetNextVersion(
    IliveCacheSink*     lcSink,
    tsp00_C24&          versionId,
    tsp00_Date&         createDate,
    tsp00_Time&         createTime,
    tsp00_Date&         openDate,
    tsp00_Time&         openTime,
    tgg91_TransNo&      consistentView,
    tsp00_Bool&         isMarked,
    tsp00_Bool&         isOpen,
    tsp00_Bool&         isUnloaded,
    tsp00_8ByteCounter& heapUsage,
    tsp00_Int4&         hashSize,
    tgg00_FileId&       versionTree,
	  tgg00_FileId&       versionInvTree,
    tsp00_C512&         versionDesc, // PTS 1117690
    short&              error) = 0;  // PTS 1126698 

  /*!-----------------------------------------------------------------------
  
  function : AdviseClockFunc

  description : tells the liboms about the address of the clock function.
             
  parameters :
    func[in] the address of the function

  return value : none

  -------------------------------------------------------------------------*/

  virtual void AdviseClockFunc(Clock_co02 func) = 0;


  /*!-----------------------------------------------------------------------
  
  function : GetKnlVersionFunc

  description : tells the liboms about the address of the function
                returning the kernel version.
             
  parameters :
    func[in] the address of the function

  return value : none

  -------------------------------------------------------------------------*/

  virtual void AdviseGetKnlVersionFunc(GetKnlVersion_co02 func) = 0;

  /*!-----------------------------------------------------------------------

  function : AdviseGetSinkFunc

  description : tells the liboms about the address of a function, that can
                be called to ask for the kernel sink.

  parameters :
    func[in] the address of the function

  return value : none

  -------------------------------------------------------------------------*/

  virtual void AdviseGetSinkFunc(GetSinkPtr_co02 func) = 0;

  /*!-----------------------------------------------------------------------
  
  function : AdviseHeapLimit

  description : sets the maximal heap size used by the liboms.

  parameters :
    limit[in] the maximal heap size in KB

  return value : none

  -------------------------------------------------------------------------*/

  virtual void AdviseHeapLimit(tsp00_Uint4 limit) = 0; 

  /*!-----------------------------------------------------------------------
  
  function : AdviseHeapThreshold

  description : sets the heap threshold, i.e. the percentage of heap usage when
                liboms starts to reduce the memory usage of versions.

  parameters :
    threshold[in] the threshold value in percent

  return value : none

  -------------------------------------------------------------------------*/

  virtual void AdviseHeapThreshold(int threshold) = 0; /* PTS 1109340 */

  /*!-----------------------------------------------------------------------
  
  function : AdviseOmsVersionThreshold

  description : sets the oms version threshold, i.e. the heap usage of a version in KB when
                liboms starts to reduce the memory usage of versions.

  parameters :
    threshold[in] the threshold value in KB

  return value : none

  -------------------------------------------------------------------------*/

  virtual void AdviseOmsVersionThreshold(tsp00_Uint4 threshold) = 0; /* PTS 1109340 */

  /*!-----------------------------------------------------------------------
  
  function : AdviseOmsRegionCnt

  description : sets the maximal number of regions used by the liboms.

  parameters :
    rgnCnt[in] the maximal number of regions

  return value : none

  -------------------------------------------------------------------------*/

  virtual void AdviseOmsRegionCnt(tsp00_Int4 rgnCnt) = 0;

  /*!-----------------------------------------------------------------------
  
  function : AdviseRequestTimeout

  description : tells the liboms about the request timeout of OmsLockObjects

  parameters :
    timeout[in] timeout value in seconds

  return value : none

  -------------------------------------------------------------------------*/

  virtual void AdviseRequestTimeout(tsp00_Int4 timeout) = 0; // PTS 1110287

  /*!-----------------------------------------------------------------------
  
  function : AdviseSignalTimeoutFunc

  description : tells the liboms about the address of a function, that signals
                a timeout of a task.

  parameters :
    func[in] the address of the function

  return value : none

  -------------------------------------------------------------------------*/

  virtual void AdviseSignalTimeoutFunc(SignalTimeoutFunc_co02 func) = 0; // PTS 1110287

  /*!-----------------------------------------------------------------------
  
  function : AdviseTracer

  description : registers a tracer object in the liboms.

  parameters :
    pTraceObj[in] pointer to the tracer object

  return value : none

  -------------------------------------------------------------------------*/
 
  virtual void AdviseTracer(ITrace_co02* pTraceObj) = 0;

  /*!-----------------------------------------------------------------------
  
  function : Dump

  description : dumps global structures of liboms.

  parameters :
    dumpObj[in]  reference to dump object in kernel environment

  return value : none

  -------------------------------------------------------------------------*/

  virtual void Dump(IDump_cco02&) = 0;


  /*!-----------------------------------------------------------------------
  
  function : InitLibOms

  description : initializes global data structures of the liboms.

  parameters :
    lcSink[in]  pointer to the kernel sink

  return value : none

  -------------------------------------------------------------------------*/

  virtual void InitLibOms(IliveCacheSink* lcSink) = 0;

  /*!-----------------------------------------------------------------------
  
  function : InspectTimeout

  description : checks timeouts of OmsLockObjects.

  parameters : none

  return value : none

  -------------------------------------------------------------------------*/

  virtual void InspectTimeout() = 0; // PTS 1110287

  /*!-----------------------------------------------------------------------
  
  function : IsMonitorOn

  description : returns the state of the OMS monitor.

  parameters : none

  return value : true, if the monitor is enabled; false otherwise

  -------------------------------------------------------------------------*/

  virtual bool IsMonitorOn() const = 0;

  /*!-----------------------------------------------------------------------
  
  function : LockVersionDirectory

  description : locks the version directory of the liboms, i.e. disables create
                and drop operations of OMS versions. 

  parameters : 
    lcSink[in]  pointer to the kernel sink

  return value : none

  -------------------------------------------------------------------------*/

  //virtual void LockVersionDirectory(IliveCacheSink* lcSink) = 0; // PTS 1124170


  virtual bool NextOmsLockObjInfo(void**, tgg00_OmsLockInfo&) = 0;

  /*!-----------------------------------------------------------------------
  
  function : ResetMonitor

  description : Initializes the OMS monitor, i.e resets all counters.

  parameters : 
    lcSink[in]  pointer to the kernel sink

  return value : none

  -------------------------------------------------------------------------*/

  virtual void ResetMonitor(IliveCacheSink* lcSink) = 0;

  /*!-----------------------------------------------------------------------
  
  function:     SetDumpError
  
  description:  enable dump whenever an exception of specified error occurs

  arguments:
    errorNo [in] the error code that causes dump

  return value: none
  -------------------------------------------------------------------------*/

  virtual void SetDumpError(tsp00_Int4 errorNo) = 0;
  
  /*!-----------------------------------------------------------------------
  
  function : UnLockVersionDirectory

  description : unlocks the version directory of the liboms, i.e. enables create
                and drop operations of OMS versions. 

  parameters : 
    lcSink[in]  pointer to the kernel sink

  return value : none

  -------------------------------------------------------------------------*/
  
  //virtual void UnLockVersionDirectory(IliveCacheSink* lcSink) = 0; // PTS 1124170

  /*!-----------------------------------------------------------------------
  
  function : SetTraceLevel

  description : enables or disables liboms tracing.

  parameters : 
    lvl[in]    NULL terminated string describing the trace level ("OMS_ERROR",
               "OMS_INTERFACE","OMS_CONTAINERDIR","OMS_MEMORY","OMS_VAROBJECT",
               "OMS_SESSION")
    enable[in] true, if trace level has to be enabled, false otherwise

  return value : none

  -------------------------------------------------------------------------*/

  virtual bool SetTraceLevel(tsp00_KnlIdentifier& lvl, bool enable) = 0;

  /*!-----------------------------------------------------------------------
  
  function : StartStopMonitor

  description : starts or stops the OMS Monitor 

  parameters : 
    doStart[in]  true, if maonitor has to be started

  return value : none

  -------------------------------------------------------------------------*/

  virtual void StartStopMonitor(bool doStart) = 0;
};

/*! endclass: LibOmsInterface_cco02 */

typedef LibOmsInterface_cco02* (*ProcGetInterfacePtr_co02)(void);

typedef IliveCacheSink* (*GetSinkPtr_co02)(void);



# endif