/* @lastChanged: "2000-02-24  13:31"
  * @filename:   livecachesql.h
  * @purpose:    "SQL-Class Interface"
  * @release:    7.2.2.02
  * @see:        "-.-"
  *
  * @copyright:  "Copyright by SAP AG, 2000"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/

#if !defined (__liveCacheSQL_INCLUDED)
#define __liveCacheSQL_INCLUDED

#include "lcbasetypes.h"    /* no check */
#include "livecachebase.h"  /* no check */

#if !defined (EXT_CLIENT_SQL)
#if !defined (__liveCache_INCLUDED)
#define __liveCache_INCLUDED

DEFINE_GUID(IID_IliveCacheSink, 
            0x8580E621,0x243F,0x11d1,0xB5,0xA1,0xAC,0x7F,0x01,0x06,0x8C,0x36);

#include "ilivecachesink.h"  /* no check */
#endif //__liveCache_INCLUDED
#endif // EXT_CLIENT_SQL

#undef  AFX_EXT_CLASS
#if defined(EXT_CLIENT_SQL_STATIC_LINK)
#define AFX_EXT_CLASS 
#else
#define AFX_EXT_CLASS DECLSPEC_IMPORT
#endif

#include "livecachebase.h"  /* no check */

#if defined(SQL_STD_STRING_SUPPORTED)
#include <string>
#endif

class  SqlOpt {
public:
  typedef enum { SQLMODE_NIL, SQLMODE_SESSION, SQLMODE_ADABAS, SQLMODE_ANSI, SQLMODE_DB2, SQLMODE_ORACLE
  } SqlSessionMode;
};

class AFX_EXT_CLASS SqlCol {
private:  
    void     *dummy;    
public:
  /* symbolic values returned by sqlVarType()
   * used in:   SqlCol (void *p, SqlParmType t, long l, short f = 0);
   */
   typedef enum { 
     VBYTE_DB, VDEC, VINT, VUINT, VFLOAT, VCHAR, VSTDSTRING, VSTRING, VBOOL,
     /* following types will be deleted in the future */ 
      VUNS2, VUNS4, VINT2, VINT4, VINT8, VFLO4, VFLO8,
     /* new inserted types                            */
     VCHAR_UNICODE, VSTRING_UNICODE
# if !defined (EXT_CLIENT_SQL)
    /* special VCHAR type for ascii columns in an unicode liveCache, PTS 1121341 */
    , VCHAR_ASCII
# endif
   } SqlParmType;	
  
  /* symbolic values returned by sqlType ()
   * NOTYPE means: no DB information for this column		
   */
  typedef enum  {
    FIXED=0, FLOAT=1, CHAR=2, BYTE=4, LONG=6, LONGBYTE=8, 
    DATE=10, TIME=11, TIMESTAMP = 13, BOOLEAN=23, UNICODE=24, 
    SMALLINT = 29, INTEGER = 30, VARCHAR = 31, VARBYTE = 33, 
    STRINGUNICODE=34, LONGUNICODE=35, NOTYPE = -1
  } SqlDbColType; 		

  /* symbolic values returned by sqlInOut ()     */
  typedef enum {IO_IN, IO_OUT, IO_INOUT} SqlDbColIO;			

  /* symbolic values returned by sqlNullValue () */
  typedef enum {NOT_NULL, NULL_PERMITTED} SqlDbColNull;		 
  
  SqlCol ();						
  ~SqlCol();
  SqlCol (const SqlCol& right);
  SqlCol (int &p);                        // variable
  SqlCol (short &p);                      // description for
  SqlCol (long &p);                       // different base
  SqlCol (unsigned &p);                   // types,
  SqlCol (unsigned short &p);
  SqlCol (unsigned long &p);
  SqlCol (float &p);
  SqlCol (double &p);
#if defined(SQL_STD_STRING_SUPPORTED)
  SqlCol (std::string &p);                // standard C++ strings 
#endif
&if $MACH in [ _IBMR2 ]     
#if (__IBMCPP__ >= 500)           
&endif                          
  SqlCol (bool &p);
&if $MACH in [ _IBMR2 ]     
#endif
&endif                          
  SqlCol (char &p);
  SqlCol (signed char &p);
  SqlCol (unsigned char &p);
  
  /* constructor for application defined parameter types
   * p pointer to parameter variable
   * t type to be used for parameter
   * l size of parameter variable
   * f in case of VDEC fraction of parameter variable
   */
  SqlCol (void *p, SqlParmType t, long l, short f = 0);
  
  // const char* sqlName () const;    // returns a copy of a column name
  int     sqlType () const;           // returns column type (SqlDbColType)
  long    sqlLength () const;         // returns column length
  int     sqlFrac () const;           // returns column fraction length
  int     sqlInOut () const;          // returns column usage (SqlDbColIO)
  int     sqlVarType () const;        // returns variable type (SqlParmType)
  int     sqlNullValue () const;      // returns column mode (SqlDbColNull)
  long    sqlIndicator () const;      // returns indicator;
  int     sqlIsNull () const;         // (TRUE=1) if value returned is NULL
  int     sqlOverflow () const;       // (TRUE=1) if value returned is numeric Overflow
  int     sqlTruncated () const;      // (TRUE=1) if value returned has been truncated
  void    sqlSetNull ();              // sets a null value for input
  void    sqlSetLength (long, int=0); // sets collength and colfrac
  const SqlCol & operator= (const SqlCol& right);
  void*   operator new( size_t sz );
  void    operator delete (void* p);
  void*   operator new[] (size_t sz);
  void    operator delete[] (void* p);
};

class AFX_EXT_CLASS SQL {
 
private:
  void*           dummy;
  boolean         m_assign;
  
public:
  SQL();
  ~SQL();
  SQL(const SQL& sql);
  short   sqlNumCols () const;            // returns number of input and output parameters
  short   sqlNumOutCols () const;         // returns number of output parameters
  int     sqlExecute ();                  // executes a statement
  int     sqlPrepare ();                  // prepares a statement and returns the number
  int     operator() ();                  // like execute ()
&if $MACH IN [ SUN ]
  SQL	    &operator<< (const SqlCol);     // assignes a column
&else
  SQL	    &operator<< (const SqlCol&);    // assignes a column
&endif
  SqlCol  &operator[] (int);              // select a column by index
  const SQL & operator= (const SQL& right);
  void*   operator new( size_t sz );
  void    operator delete (void* p);
  void*   operator new[] (size_t sz);
  void    operator delete[] (void* p);
};

class AFX_EXT_CLASS SqlHandle 
#if !defined (EXT_CLIENT_SQL)
  : public DbpBase 
#endif
{
protected:
  SqlHandle ();
  ~SqlHandle ();
  void    sqlInitSession (IUnknown** s); /*  {m_session = (IUnknown**) s;}; */
public :
  SQL     sql(const char*);
  SQL     sql(const DbpTypeUnicode*);
  typedef void ( __cdecl * sqlHandler ) ( SqlHandle* p_sqlhandle ); 
  sqlHandler sqlSetHandler(sqlHandler p);     /* PTS 1120804 */
  //
  // C++ SQL Interface Methods    
  short   sqlCode () const;           // returns sql error code
  const char*	sqlState() const;       // returns a copy of sqlstate
  const char* sqlErrorMsg() const;    // returns a copy of sqlerrmc
  const DbpTypeUnicode* sqlUnicodeErrorMsg() const; // returns a copy of sqlerrmc_uni
  int     sqlIsUnicodeErrMsg() const;     //
  char    sqlWarning (int) const;     // returns sqlwarn [i] )
  long    sqlErrorPos () const;       // returns sql error position sqlerrd[5]
  long    sqlResultCount () const;    // returns sql result count sqlerrd[2]
  void    sqlSetMode(const SqlOpt::SqlSessionMode mode);
  //*** following method has been eliminated ! Use 'throw DbpError(...) instead )
  //  void    sqlSetError (short msgno, const char* errmsg);
  void    sqlRaiseError (short msgno, const char* errmsg);
  void    sqlRaiseError (short msgno, const DbpTypeUnicode* errmsg);
#if !defined (EXT_CLIENT_SQL)
  void    sqlExceptionHandler(DbpError&);					
  bool    sqlGetMethodCallMonitorInfo(SqlMonitorData& info) const;     /* PTS 1120804 */
#else
  void    inline dbpDebugBreak();
#endif
  
  void*               m_str_assign;
  void*               m_str_append;
private :
  sqlHandler          m_sqlhandler;
  IUnknown**          m_session;
  void*               dummy1;			
  void*               dummy2;			
  void*               dummy3;			
  void*               dummy4;			
};

void 
#if defined(WIN32)
__cdecl 
#endif
sqlDefaultHandler ( SqlHandle* p_sqlhandle );

#endif //__liveCacheSQL_INCLUDED
