/*!================================================================
 module:        gco10.h
 
 responsible:   ThomasA
 
 special area:  OMS
 
 description:   C++ Persistent Object Interface
 
 see:
 
 copyright:     SAP AG 1998 


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 ===================================================================*/



#if !defined (__GCO10_H)
#define __GCO10_H

#define MAX_SCHEMA_NAME_LENGTH_CO10 31
// PTS 1117690
// This size must equals half the size of the array in the procedure calls  
// in the AK layer and the definition of the array 'tak_oms_version_desc'  
// in vak001. The desciption is always stored in UCS2-format (2 bytes per 
// character, while the array size is given in bytes.
#define MAX_VERSION_DESC_LENGTH_CO10 256

// Assertions : compile with option /-D_ASSERT_OMS

#ifdef _ASSERT_OMS
#define _OMSASSERT(session,expr) if (!(expr)) { session->ThrowDBError(-28999, #expr);}
#else
#define _OMSASSERT(h,expr) 
#endif

#if defined(OMSTST)
#define OMS_CHECK_EXCEPTION(HANDLE) { (HANDLE).m_pSession->CheckCancelCountDown();\
                                      (HANDLE).m_pSession->CheckExceptionCountDown(); }
#else
#define OMS_CHECK_EXCEPTION(HANDLE)
#endif

#ifndef _DEBUG

/****************************************************************************
*
* Debug OFF
* Debug OFF
* Debug OFF
*
***************************************************************************/

#define _TRACE_METHOD_ENTRY(h,methodname)
#define _TRACE_OID(h,oid)
#else

/****************************************************************************
*
* Debug ON
* Debug ON
* Debug ON
*
***************************************************************************/

#define _TRACE_METHOD_ENTRY(h,methodname) (h)->omsTracef(methodname);

#define _TRACE_OID(h,oid) (h)->omsTraceOid(oid);

#endif

#define MEMBER_OFFSET(T,M) (((char*) &((T*) 0)->M) - (char*) 0)

#include "gco001.h"
#include "gco02.h"
#include "hco13.h"
#include "gco14.h"
#include "hco15.h"
#include "hco16.h"
#include "hco18.h"
#include "gco90.h"
#include "ggg92.h"
#include "ggg00.h"
#include "hco10.h"
#include "hco30-k.h"
#include "hco101OmsObjByKeyIterBase.h"
#include "gco103.h"
#include "Oms/OMS_StackTraceCallbackInterface.hpp"

#define FOR_UPD_CO10       true

const GUID VAR_OBJ_GUID_DEF_GCO10 = { 0x8fb0963d, 0xe5aa, 0x11d1, { 0xa9, 0x5c, 0x0, 0xa0, 0xc9, 0x43, 0x7, 0x31 } };
#ifdef	LIVECACHE_INTGUIDS
const ClassID VAR_OBJ_GUID_GCO10  = -1;
#else
#define VAR_OBJ_GUID_GCO10 VAR_OBJ_GUID_DEF_GCO10
#endif

/*----------------------------------------------------------------------*/
/* PTS 1115231 */
externCpp
bool omsIsUnicodeInstance();

/*----------------------------------------------------------------------*/

externCpp
void* omsMalloc(size_t size);

/*----------------------------------------------------------------------*/

externCpp
void* omsMallocArray(size_t size);

/*----------------------------------------------------------------------*/

externCpp
void omsFree(void* p);

/*----------------------------------------------------------------------*/

externCpp
void omsFreeArray(void* p);

/*----------------------------------------------------------------------*/

extern 
void omsGetLocalDateTimeString(char*, int);

/*----------------------------------------------------------------------*/

extern
void omsGetLocalDateTime(int& year,
                           int& month,
                           int& day,
                           int& hours,
                           int& minutes,
                           int& seconds);

/*----------------------------------------------------------------------*/

extern
void omsGetUTCDateTimeString(char*, int);

/*----------------------------------------------------------------------*/

extern
void omsGetUTCDateTime(int& year,
                           int& month,
                           int& day,
                           int& hours,
                           int& minutes,
                           int& seconds);

/*----------------------------------------------------------------------*/

extern
void omsGetLiveCacheVersion(char*, int);


/*----------------------------------------------------------------------------*/

class AFX_EXT_CLASS OmsDuplicateKey : public DbpError
{
public:
  OmsDuplicateKey(long e, const OmsObjectId& oid); 
};

class AFX_EXT_CLASS OmsLockTimeout : public DbpError
{
public:
  OmsLockTimeout(const OmsObjectId& oid);
};

class AFX_EXT_CLASS OmsOutOfDate : public DbpError
{
public:
  OmsOutOfDate();
  OmsOutOfDate(const OmsObjectId& oid);
};

class OmsContext_co12;
class OmsSession_co12;
class SqlHandle;     /* PTS 1120804 */

// XREF to hco00.h, veo38.cpp for other definitions
class OmsCallbackInterface {
  friend class OmsSession_co12;
  friend class OmsHandle;
public :
  enum Event {
      BeforeCommit,
      AfterCommit,
      AfterRollback,
      ByApplication
  };
  enum BlockType {
      BlockFreed        = -1,   // freeing already freed block
      BlockInvalid      = 0,    // freeing a block using invalid allocator
      BlockGlobal       = 1,    // global allocator
      BlockSession      = 2,    // session-specific allocator
      BlockTransaction  = 3,    // transaction-specific allocator
      BlockCOMRoutine   = 4,    // COMRoutine-specific allocator

      BlockUnregister   = 15    // unregister interfaces at session end
  };
private :
  virtual ~OmsCallbackInterface() {};
  virtual void omsValidate (OmsHandle&, Event, bool) = 0;
  virtual void omsInvalidate (OmsHandle&, Event, bool) = 0;
  virtual void omsDestroySelf() = 0;
  virtual void omsMethodCallEpilog(OmsHandle&) {};
  /* PTS 1120804 */
  virtual void omsMethodCallEpilog( OmsHandle& oh, SqlHandle& sh ) {
    omsMethodCallEpilog(oh);
  }

  /*!
    function: omsMemLeftOver

    description: Called, when some block is left over in the allocator

    parameters:
        blk         [in]  pointer to the block
        size        [in]  size of the block
        blockType   [in]  allocator ID (1=global, 2=session, 3=transaction, 4=COM routine)
  */
  virtual void omsMemLeftOver(const void *blk, size_t size, BlockType blockType)
#ifdef LIVECACHE_COMPATAPO30
    {}
#else
    = 0;
#endif

  /*!
    function: omsMemInvalidFree

    description: Called, when some block is freed in incorrect allocator

    parameters:
        blk         [in]  pointer to the block
        size        [in]  size of the block
        blockType   [in]  allocator ID (1=global, 2=session, 3=transaction, 4=COM routine)
        freeType    [in]  allocator ID in the block (like blockType; -1/0=invalid allocator used to free)
  */
  virtual void omsMemInvalidFree(const void *blk, size_t size, BlockType blockType, BlockType freeType)
#ifdef LIVECACHE_COMPATAPO30
    {}
#else
    = 0;
#endif
};

AFX_EXT_CLASS void *operator new(size_t sz, OmsAllocatorInterface &i);
AFX_EXT_CLASS void operator delete(void *ptr, OmsAllocatorInterface &i);

class AFX_EXT_CLASS OmsIOidReadIterator   // PTS 1115027
{
public:
  virtual void omsReset() = 0;
  virtual const OmsObjectId& omsGet() const = 0;
  virtual void operator++ () = 0;
  virtual operator bool () const = 0;
};

class AFX_EXT_CLASS OmsIOidAppender   // PTS 1115027
{
public:
  virtual bool omsAppend(const OmsObjectId& currOid) = 0;
}; 

class AFX_EXT_CLASS OmsHandle : public DbpBase {
  
  friend class  OmsAbstractObject;
  friend class  OmsCriticalSection;
  friend class  OmsLockObject;
  friend class  OmsStream;
  friend class  OmsObjByClsIterBase;
  friend class  cco13_BeforeImages;
  friend void   co10_SetCurrMonitorMethod(OmsHandle& h, tgg00_COMMonitorInfo* p);
  friend void   co10_MethodCallEpilog( OmsHandle& h, void* sql_handle, int runTime );     /* PTS 1120804 */
public :
    
  OmsHandle  (OmsTypeInt4 sz = 1007);
  ~OmsHandle ();
  HRESULT     omsAttachDefaultContext();
  void        omsCancel(); /* PTS 1113128 */
  bool        omsCancel(bool toCancel); /* PTS 1120411 */
  const OmsObjectId &omsCastOid(const ClassIDRef guid, const OmsObjectId& oid);
  bool        omsCheckOid    (const OmsObjectId&, const ClassIDRef guid);
  bool        omsCheckOid    (const OmsVarOid& );
  void        omsCleanUpAfterCrash();
  void        omsControlCancelFlag(int);          // PTS 1112358
  void        omsControlDbpException(int);        // PTS 1112358
  void        omsControlBadAllocExceptions(int);  // PTS 1112358
  OmsSchemaHandle omsCreateSchema(const char* SchemaName);
  OmsSchemaHandle omsCreateSchema(const OmsTypeWyde* SchemaName);
  const void* omsDeRef(const OmsVarOid&);
  void        omsDropSchema(OmsSchemaHandle);
  void        omsDropContainer(const ClassIDRef, OmsSchemaHandle, OmsContainerNo);
  void        omsDropContainer(const ClassIDRef, OmsSchemaHandle, OmsContainerNo, size_t);
  bool        omsExistsContainer(const ClassIDRef, OmsSchemaHandle, OmsContainerNo); // PTS 1110334
  void        omsOidInfo (const OmsObjectId&, ClassID&, OmsSchemaHandle&, char*, int, char*, int, OmsContainerNo&); 
  void        omsOidInfo (const OmsObjectId&, ClassID&, OmsSchemaHandle&, OmsTypeWyde*, int, char*, int, OmsContainerNo&); 
  int         omsCurrentSubtransLevel() const;
  void        omsLoadVarObject (const OmsVarOid& oid, unsigned long size, void* buf);
  void        omsLock( const OmsObjectId& );
  bool        omsTryLock( const OmsObjectId& );
  void        omsLock( const OmsVarOid& oid);
  bool        omsTryLock( const OmsVarOid& oid);
  void        omsLocks( const OmsObjectId*, int);
  void        omsMonitorLock();   // PTS 1112275
  void        omsMonitorUnlock(); // PTS 1112275
  bool        omsGetMethodCallMonitorInfo(OmsMonitorData& info) const;
  int         omsGetTimeOut() const;
  OmsContainerNo  omsGetContainerNo (const OmsObjectId&);
  void        omsGetObj(const OmsObjectId& oid, unsigned char* cnsView, int& ObjLen, unsigned char* obj); 
  OmsSchemaHandle omsGetSchemaHandle(const char* SchemaName) const;
  OmsSchemaHandle omsGetSchemaHandle(const OmsTypeWyde* SchemaName) const;
  OmsSchemaHandle omsGetSchemaHandle(const OmsObjectId&);
  void        omsGetSchemaName(OmsSchemaHandle,char*,int) const;
  void        omsGetSchemaName(OmsSchemaHandle,OmsTypeWyde*,int) const;
  long        omsGetStoredErrorCode() const;    // PTS 1122838
  OmsCallbackInterface* omsGetTransactionInterface (const ClassIDRef) const;
  unsigned long omsGetVarObjSize (const OmsVarOid& oid);
  void        omsInitSession (IUnknown** s);
  void        omsMethodCallEpilog(void* pSH);     /* PTS 1120804 */
  void        omsNewConsistentView();
  void        omsNewConsistentView (OmsObjectId* pOids, int cntOids, short timeout);
  void        omsNewConsistentView (OmsObjectId* pOids, int cntOids, short timeout, OmsObjectId*, int&);
  void        omsNewConsistentView (OmsIOidReadIterator& pOids, short timeout, OmsIOidAppender& pErrOids);   // PTS 1115027
  void        omsReleaseConsistentView();
  void        omsPrintOid (char* buf, size_t bufSize, const OmsObjectId oid);
  const       OmsVarOid& omsNewVarObject(OmsSchemaHandle, OmsContainerNo);
  bool        omsIsLocked ( const OmsObjectId& );
  bool        omsIsRegistered(const ClassIDRef, OmsSchemaHandle, OmsContainerNo);
  void		  omsRaiseError (short msgno, const OmsTypeUTF8* errmsg);
  void        omsReadOnly (bool = true);
  bool        omsGetReadOnly();
  void        omsRegClass (OmsSchemaHandle, const ClassIDRef guid, const char* ClassName, size_t PersistentSize, size_t ObjectSize, const ClassIDPtr pBaseClass, OmsAbstractObject*, OmsContainerNo, size_t = 0);
  void        omsRegClassAndKey (
    OmsSchemaHandle,
    const ClassIDRef guid,
    const char* ClassName, 
    size_t PersistentSize, 
    size_t KeySize, 
    size_t ObjectSize, 
    OmsAbstractObject*, 
    OmsContainerNo,
    bool partitionedKey); /* PTS 1109717 */
  void        omsRegTransactionInterface (const ClassIDRef, OmsCallbackInterface*);
  void        omsRelease ( const OmsObjectId& );
  void        omsRelease ( const OmsVarOid& oid );
  void        omsReleaseAll();
  void        omsReleaseUnconditional( const OmsObjectId& ); // not exported in livecache.h
  void        omsReleaseVarObject(const OmsVarOid& oid);
  HRESULT     omsReleaseDefaultContext();
  void        omsReschedule() const; /* PTS 1107849 */
  void        omsResetVersion(const OmsVersionId& version); /* PTS 1115134 */
  void        omsSetTimeOut( short );
  bool        omsSetCachedKeys(bool, const ClassIDRef, OmsSchemaHandle, OmsContainerNo);  // PTS 1117571
  void        omsStoreErrorCode(long errorCode);    // PTS 1122838
  void        omsStoreVarObject ( const OmsVarOid& oid, const void*, unsigned long objLength);
  short       omsCommit   (bool);
  short       omsCommitHandling(int&);
  void        omsRollback (bool, bool);
  void 		    omsDelete ( const OmsObjectId& );
  void        omsDelete ( const OmsVarOid& );
  void        omsDelete ( OmsAbstractObject* );
  void		    omsDeleteAll(const ClassIDRef guid, OmsSchemaHandle, OmsContainerNo);
  int         omsStartSubtrans();
  void        omsCommitSubtrans(int = -1);
  void        omsRollbackSubtrans( int = - 1);
  unsigned long  omsGetVarObjectSize (const OmsVarOid& );
  void           omsTerminate();
  void           omsTerminate(const char*); /* PTS 1109693 */
  void           omsTerminate(const OmsTypeUTF8*);
  void           omsTrace ( const char* tracemsg, tsp00_Int4 TraceMsgLen);
  void           omsTracef( const char*fmt, ... );
  void           omsTraceOid (const OmsObjectId& oid);
  void           omsTransRollback();
  void           omsUnlock( const OmsObjectId& ); 
  void           omsExceptionHandler(DbpError&);					
  void	         omsDump();  
  static void    omsDumpStackTrace();
  static unsigned int omsSaveStackTrace(void **buffer, unsigned int levels);
  static void         omsEvalStackTrace(void **buffer, unsigned int levels,
                        OmsStackTraceCallbackInterface *cb);
  const char*    omsSimDumpGetHtmlRef(const OmsObjectId &oid);
  
  const OmsVersionId* omsCurrentVersionId () const;
  void           omsCreateVersion(const OmsVersionId&);                                             // PTS 1117690
  void           omsCreateVersion(const OmsVersionId&, const OmsTypeWyde* pDesc);                    // PTS 1117690
  void           omsCreateVersion(const OmsVersionId&, const char* pDesc);                           // PTS 1117690
  bool           omsGetVersionInfo(const OmsVersionId&, OmsVersionInfo&, OmsTypeWyde* pDescBuf, int bufLen) const;    // PTS 1117690
  bool           omsGetVersionInfo(const OmsVersionId&, OmsVersionInfo&, char* pDescBuf, int bufLen) const;           // PTS 1117690
  void           omsSetCurrVersionDesc(const OmsTypeWyde* pDesc);                                    // PTS 1117690
  void           omsSetCurrVersionDesc(const char* pDesc);                                           // PTS 1117690
  void           omsMarkVersion      ( const OmsVersionId& );
  void           omsOpenVersion      ( const OmsVersionId& );
  void           omsCloseVersion     ( const OmsVersionId& );
  void           omsDropVersion      ( const OmsVersionId& );
  void           omsForceDropVersion ( const OmsVersionId& );
  bool           omsExistsVersion    ( const OmsVersionId& );
  void           omsRenameVersion    ( const OmsVersionId&, const OmsVersionId&);

  void           omsValidate();
  void           omsGetMonitorInfo (tgg00_COMMonitorTransferBuf* lpMonInfo);
  void           omsRegMonitor     (void* pMon); 
  void           omsSetCurrMonitorMethod(tgg00_COMMonitorInfo* p);
  void           omsTransEndEpilog(); // PTS 1110315
  void           TraceCommitPhase(int);
  OmsObjByKeyIterBase omsCreateKeyRangeIter(
      const ClassIDRef guid,
      OmsSchemaHandle sh, 
      OmsContainerNo  ContainerNo,
      const void*     pStartKey,
      const void*     pStopKey
      );
  OmsObjByClsIterBase omsAllOids( const ClassIDRef, OmsSchemaHandle, OmsContainerNo );
  OmsAbstractObject* omsNewObject ( const ClassIDRef, OmsSchemaHandle, OmsContainerNo );
  OmsAbstractObject* omsNewRegistryObject (size_t sz, const ClassIDRef guid);
  OmsAbstractObject* omsNewKeyedObject( const ClassIDRef guid, const unsigned char* key, OmsSchemaHandle, OmsContainerNo );
  tco13_ClsIdEntry*  omsAddContainerInfo (ContainerInfo_co18* ContainerInfo);
  void omsThrowCancelledError() const; // PTS 1110812

  static OmsHandle* omsGetOmsHandle();

  OmsAllocatorInterface &omsGetGlobalAllocator();
  OmsAllocatorInterface &omsGetGlobalAllocator(const char *name);
  OmsAllocatorInterface &omsGetSessionAllocator();
  OmsAllocatorInterface &omsGetTransactionAllocator();
  OmsAllocatorInterface &omsGetCOMRoutineAllocator();

  protected :
    OmsAbstractObject* omsDeRef      (const OmsObjectId& oid, const ClassIDRef guid, bool forUpd, bool do_lock);
    OmsAbstractObject* omsDeRefKey (unsigned char* key, const ClassIDRef guid, bool forUpd, bool do_lock, OmsSchemaHandle, OmsContainerNo);

  private :
    void CheckHeapThresholds();
    void SessionEnd();
    void IdentifierToWyde(const char* pSource, OmsTypeWyde* pDest) const; 
    void WydeToChar(const OmsTypeWyde* pSource, char* pDest, int destSize, const char* errorMsg) const;

    IliveCacheSink**      m_session;
    co90_Monitor*         m_monitor;
  public :
    OmsSession_co12*      m_pSession;
    unsigned char*        m_pCancelFlag;     // PTS 1110812
  private :
    unsigned char         m_dummyCancelFlag; // PTS 1110812
};

class AFX_EXT_CLASS OmsCriticalSection {
public :
  OmsCriticalSection(OmsHandle*,int);
  void omsEnter();
  void omsLeave();
  ~OmsCriticalSection();
private :
  OmsSession_co12* m_session;
  int              m_regionId;
  bool             m_inSection;
};

class AFX_EXT_CLASS OmsObjByClsIterBase {
  friend class OmsHandle;
  friend class OmsAbstractObject;
public :
  ~OmsObjByClsIterBase();
  OmsObjByClsIterBase(const OmsObjByClsIterBase&); /* PTS 1110401 */
  OmsObjByClsIterBase(OmsHandle&);
protected:
  operator bool () const;
  const OmsObjectId& operator() () const;
  OmsAbstractObject* omsDeRef(bool,bool) const;
  void omsNextNewObjectInVersion(OmsObjectContainerPtr);
  void operator++ ();
private:
  OmsSession_co12*       m_session;
  tco13_ClsIdEntry*      m_containerInfo;
  OmsObjectContainerPtr  m_curr;
  OmsObjectContainerPtr  m_next;
  bool                   m_iterNewObjectsInVersion;
  tsp00_Int4             m_headIndex;
};

class AFX_EXT_CLASS OmsSimDumpCollector {
public:
	virtual void printf(const char *fmt, ...) = 0;
	virtual const char* str() const = 0;
	virtual void flush() = 0;
	inline operator const char *() const { return str(); }
};

class AFX_EXT_CLASS OmsAbstractObject {
  
  friend class OmsObjectContainer;
  friend class OmsHandle;
  friend class cco13_OidHash;
  friend class OmsObjByClsIterBase;
  friend class OmsContext_co12;
  friend class OmsSession_co12;
  friend class OmsArrayObjectBase;
  friend class OmsArrayObjectIteratorBase;
protected :
  
public :
  enum OmsRegEnum { USE_IN_REGISTRY_ONLY };

  virtual void       omsCleanUp (OmsHandle&);
  virtual void       omsDump    (OmsHandle& h, const char* flags) const;
  void               omsDelete          (OmsHandle& h);
  void               omsFlush           (OmsHandle& h);
  OmsSchemaHandle    omsGetSchemaHandle(OmsHandle&) const;
  bool               omsIsLocked        (OmsHandle& h) const;
  void               omsLock            (OmsHandle& h);
  bool               omsTryLock         (OmsHandle& h);
  void               omsRelease         (OmsHandle& h);
  void               omsStore           (OmsHandle& h);
  virtual void       omsVerify  (OmsHandle& h) const;
  const OmsObjectId& omsGetOid() const;
protected :
#ifndef LIVECACHE_COMPATAPO30
  virtual void       omsKeyToBinary(const void* pKey, void* pDest) const;
  virtual void       omsBinaryToKey(void* pSource, void* pKey) const;
#endif
  OmsAbstractObject ();
  const OmsObjectId& omsOid() const;
  OmsAbstractObject* omsForUpdPtr(OmsHandle& h, bool doLock) const;
  bool  omsHistoryInUse(OmsHandle&) const;
  const OmsAbstractObject* omsDeRefNext(OmsHandle& Handle,bool do_lock) const;
  void omsKey(OmsHandle& Handle, unsigned char* key) const;
  static void omsDeleteAll(OmsHandle& h, const ClassIDRef guid, OmsSchemaHandle, OmsContainerNo);
  static OmsAbstractObject* omsNewObject (OmsHandle& Handle, const ClassIDRef guid, OmsSchemaHandle, OmsContainerNo );
  static OmsAbstractObject* omsNewRegistryObject(size_t sz, OmsHandle& Handle, const ClassIDRef guid);
  static OmsAbstractObject* omsNewKeyedObject (OmsHandle& Handle, const ClassIDRef guid, const unsigned char* key, OmsSchemaHandle, OmsContainerNo ); 
  static OmsObjByClsIterBase omsAllOids( OmsHandle& h, const ClassIDRef guid, OmsSchemaHandle, OmsContainerNo);
  static void omsRegClass (OmsHandle& Handle, const ClassIDRef Guid, const char* ClassName, size_t PersistentSize, size_t ObjectSize, const ClassIDPtr pBaseClass, OmsAbstractObject* Obj, OmsSchemaHandle, OmsContainerNo, size_t = 0);
  static void omsRegClassAndKey (
    OmsHandle&, 
    const ClassIDRef, 
    const char* ClassName, 
    size_t PersistentSize, 
    size_t KeySize, 
    size_t ObjectSize, 
    OmsAbstractObject* Obj, 
    OmsSchemaHandle, 
    OmsContainerNo,
    bool partitionedKey); /* PTS 1109717 */
  static void omsRegClassAndKey (
    OmsHandle& h, 
    const ClassIDRef guid, 
    const char* ClassName, 
    size_t PersistentSize, 
    size_t KeySize, 
    size_t ObjectSize, 
    OmsAbstractObject* Obj, 
    OmsSchemaHandle sh, 
    OmsContainerNo cno)
  {
    omsRegClassAndKey(h, guid, ClassName, PersistentSize, KeySize, ObjectSize, Obj, sh, cno, false);
  }
  static const OmsAbstractObject* omsDeRef (const OmsObjectId& oid, OmsHandle& Handle, const ClassIDRef guid);
  static OmsAbstractObject* omsDeRefForUpd (const OmsObjectId& oid, OmsHandle& Handle, const ClassIDRef guid, bool do_lock);
  static const OmsAbstractObject* omsDeRefKey (unsigned char* key, OmsHandle& Handle, const ClassIDRef guid, OmsSchemaHandle, OmsContainerNo);
  static OmsAbstractObject* omsDeRefKeyForUpd (unsigned char* key, OmsHandle& Handle, const ClassIDRef guid, bool do_lock, OmsSchemaHandle, OmsContainerNo);
private :
  OmsAbstractObject(const OmsAbstractObject&);              // no copy
  OmsAbstractObject& operator= (const OmsAbstractObject&r); // no assignment
  
  // extras for liveCache simulator
public:
  virtual void omsSimDumpToHtmlShort(OmsHandle&, OmsSimDumpCollector&) const;
  virtual void omsSimDumpToHtmlLong(OmsHandle&, OmsSimDumpCollector&) const;
  void omsSimSetObjectName(OmsHandle &h, const char *name);
  const char* omsSimGetObjectName(OmsHandle &h) const;
  static const char *omsSimDumpMakeOidRef(OmsHandle &h, const OmsObjectId &oid, int flags = 3);
  inline static const char *omsSimDumpMakeOidRef(const OmsObjectId &oid, int flags = 3)
  {
    return omsSimDumpMakeOidRef(*OmsHandle::omsGetOmsHandle(), oid, flags);
  }
};

class OmsObjectContainer{ 
  friend class OmsHandle;
  friend class cco13_OidHash;
public :
  enum ct_state {
    CN_STORE = 1, CN_LOCKED = 2, CN_DELETE = 4, CN_NEW = 8, CN_VAROBJ = 16, CN_IS_NEW = 32 };
  enum ct_ver_state { 
    CN_VER_NEW = 1, CN_VER_LINKED = 2 };

    OmsObjectContainerPtr m_hashnext;
    OmsObjectId           m_oid;
    tgg91_PageRef         m_objseq;
private :
    unsigned char         m_state;
    unsigned char         m_verstate;
public :
    tsp00_Uint4           m_beforeImages;
#if defined(BIT64)
    /*! Fill bytes which are necessary on 64 bit platforms for alignement. If this space is needed
    **  for new variables, then the filler should be removed */
    tsp00_Uint4           filler;
#endif
    tco13_ClsIdEntryPtr   m_containerInfo;
    OmsAbstractObject     m_pobj;
        
    static tsp00_Int4 headerSize() {
      return (sizeof (OmsObjectContainer) - sizeof (OmsAbstractObject));
    }

    inline void ChainFree(OmsContext_co12* context);

    bool DeletedFlag() const {
      return ((m_state & OmsObjectContainer::CN_DELETE) > 0);
    }

    bool existBeforeImage (int subtransLvl, bool inVersion) const {
      if (0 == subtransLvl) {
        return true;
      }
      if ((!inVersion) && (1 == subtransLvl)) {
        return true;
      }
      return ((m_beforeImages & (1 << (subtransLvl - 1))) > 0);
    }

    inline tco13_ClsIdEntry* GetContainerInfo(OmsContext_co12* context);
    
    inline tco13_ClsIdEntry* GetContainerInfoNoCheck(OmsContext_co12* context);

    inline bool IsContainerAssigned() const {
      return m_containerInfo != NULL;
    }

    tsp00_Uint4 GetContainerHandle() const {
      return m_containerInfo->GetContainerHandle();
    }

    unsigned char GetState() const
    {
      return m_state;
    }

    void InitObjContainer(tco13_GuidEntry& classInfo) {
      void** pvtptr  = REINTERPRET_CAST(void**, (&m_pobj));
      *pvtptr        = classInfo.GetVirtualTablePtr();
      this->InitState(&classInfo);
      m_verstate = 0;
      m_beforeImages = 0;
      m_hashnext     = NULL;
      m_containerInfo   = NULL;
    }

    void InitObjContainer(tco13_ClsIdEntry& clsinfo) { 
      void** pvtptr  = REINTERPRET_CAST(void**, (&m_pobj));
      *pvtptr        = clsinfo.GetVirtualTablePtr();
      this->InitState(&clsinfo);
      m_verstate = 0;
      m_beforeImages = 0;
      m_hashnext     = NULL;
      m_containerInfo   = NULL;
    }

    void InitState(tco13_GuidEntry* classInfo) {
      m_state = 0;
       if ((NULL != classInfo) && (classInfo->IsVarObject())) {
        this->MarkVarObj();
      }
    }

    void InitState(tco13_ClsIdEntry* clsinfo) { // TODO ?
      m_state = 0;
       if ((NULL != clsinfo) && (clsinfo->IsVarObject())) {
        this->MarkVarObj();
      }
    }

    void InitVerNew(tsp00_Int4 ofs)
    {
      void **ptr = (void**) (((char*) this) + ofs);
      ptr[0] = ptr[1] = NULL;
      m_verstate |= OmsObjectContainer::CN_VER_NEW;
    }


    bool IsNewObject() const
    {
      return ((m_state & OmsObjectContainer::CN_IS_NEW) > 0);
    }

    bool IsVerNewObject() const
    {
      return ((m_verstate & OmsObjectContainer::CN_VER_NEW) > 0);
    }

    bool IsVerNewRegistered() const
    {
      return ((m_verstate & OmsObjectContainer::CN_VER_LINKED) > 0);
    }

    void UnmarkVerNewObject()
    {
      m_verstate = 0;
    }

    bool LockedFlag() {
      return ((m_state & OmsObjectContainer::CN_LOCKED) > 0);
    }

    void MarkAsNew()
    {
      m_state |= OmsObjectContainer::CN_IS_NEW;
    }

    void MarkDeleted() {
      m_state &= (OmsObjectContainer::CN_STORE ^ 0xFF);
      m_state |= (OmsObjectContainer::CN_DELETE);
    }
   
    void MarkLocked() {
      m_state |= OmsObjectContainer::CN_LOCKED;
    }

    void MarkNew() {
      m_state |= OmsObjectContainer::CN_NEW;
    }
    
    void MarkVerNewRegistered() {
      m_verstate |= OmsObjectContainer::CN_VER_LINKED;
    }
    
    void MarkStored() {
      m_state |= OmsObjectContainer::CN_STORE;
    }

    void MarkVarObj() {
      m_state |= OmsObjectContainer::CN_VAROBJ;
    }

    bool NewFlag() {
      return ((m_state & OmsObjectContainer::CN_NEW) > 0);
    }

    void RemoveStoredFlag() {
      m_state &= (OmsObjectContainer::CN_STORE ^ 0xFF);
    }

    void setBeforeImage (int subtransLvl) {
      m_beforeImages |= (1 << (subtransLvl-1)); 
    }

    static void store (OmsHandle& h, OmsAbstractObject* o) {
      OmsObjectContainerPtr p = containerPtr(o);
      p->m_state |= OmsObjectContainer::CN_STORE;
    }

    void SetContainerInfo(tco13_ClsIdEntryPtr ContainerInfo) {
      m_containerInfo = ContainerInfo;
    }

    bool StoredFlag() {
      return ((m_state & OmsObjectContainer::CN_STORE) > 0);
    }

    bool VarObjFlag() {
      return ((m_state & OmsObjectContainer::CN_VAROBJ) > 0);
    }

    static OmsObjectContainerPtr containerPtr (OmsAbstractObject* pobj) {
      return (OmsObjectContainerPtr) (((unsigned char*) pobj) - ((sizeof(OmsObjectContainer)) - sizeof (OmsAbstractObject))); 
    }
   
    void Copy (OmsContext_co12* c, OmsObjectContainerPtr beforeImage, int ObjectSize);

public :
};

#include "hco12.h"

inline void OmsObjectContainer::ChainFree(OmsContext_co12* context) {
  tco13_ClsIdEntry* containerInfo = m_containerInfo;
  OmsObjectContainerPtr p = REINTERPRET_CAST(OmsObjectContainerPtr, this);
  containerInfo->chainFree(*context, p);
}

inline tco13_ClsIdEntry* OmsObjectContainer::GetContainerInfo(OmsContext_co12* context) {
  context->CheckIfContainerNotDropped(m_containerInfo);
  return m_containerInfo;
}

inline tco13_ClsIdEntry* OmsObjectContainer::GetContainerInfoNoCheck(OmsContext_co12* context) {
  return m_containerInfo;
}


class AFX_EXT_CLASS OmsTimeStamp
{
public:
  OmsTimeStamp(const OmsTypeABAPTimeStamp& timeStamp);
  
  typedef struct _TimeStamp
  {
    unsigned char nYear1   : 4;
    unsigned char nPad     : 4;
    
    unsigned char nYear3   : 4;
    unsigned char nYear2   : 4;
    
    unsigned char nMonth1  : 4;
    unsigned char nYear4   : 4;
    
    unsigned char nDay1    : 4;
    unsigned char nMonth2  : 4;
    
    unsigned char nHour1   : 4;
    unsigned char nDay2    : 4;
    
    unsigned char nMinute1 : 4;
    unsigned char nHour2   : 4;
    
    unsigned char nSecond1 : 4;
    unsigned char nMinute2 : 4;

    unsigned char nSign    : 4;
    unsigned char nSecond2 : 4;
  } TimeStamp;

  // Auslesen der Daten:
  const OmsTypeABAPTimeStamp& omsGetTimeStamp() const;
  tsp00_Uint4 omsSecsSince01011970() { return m_DateTime.getDTValue(); }  
protected:
  // Konvertierung der Daten:
  void ToTimeStamp();
  void ToDateTime();

  // Schnelle Multiplikations- und Divisionsfunktionen
  inline int mult10(int n)
  {
    return (n << 3) + (n << 1);
  }

  inline int mult100(int n)
  {
    return (n << 6) + (n << 5) + (n << 2);
  }

  inline int mult1000(int n)
  {
    register int a;
    a = mult100(n);
    return mult10(a);
  }

  inline void div10(int n, int& n10, int& n1)
  {
    n1  = n;
    n10 = 0;

    while (n1 >= 10)
    {
      n1 -= 10;
      n10++;
    }
  }

  inline void div1000(int n, int& n1000, int& n100, int& n10, int& n1)
  {
    n100  = n;
    n1000 = 0;

    while (n100 >= 1000)
    {
      n100 -= 1000;
      n1000++;
    }

    n10  = n100;
    n100 = 0;
    
    while (n10 >= 100)
    {
      n10 -= 100;
      n100++;
    }

    n1  = n10;
    n10 = 0;
    
    while (n1 >= 10)
    {
      n1 -= 10;
      n10++;
    }
  }
protected:
  cco15_DateTime m_DateTime;
  union {
    TimeStamp            m_TimeStamp;
    OmsTypeABAPTimeStamp m_ABAPTimeStamp;
  };
};

/*---------------------------------------------------------------------------*/

class cco10_OmsGlob {
public :
  OmsObjectId    m_nilOid;
  tgg91_PageRef  m_nilSeq;
  cco10_OmsGlob() : m_nilOid() {
   m_nilSeq.gg91SetNilRef();
  }
};

#if defined(WIN32)
/*int co10_EvalException (OmsHandle* pHandle, LPEXCEPTION_POINTERS lpexceptInfo); PTS 1106505 FF */
/*int co10_EvalException (IliveCacheSink* m_sysLcSink, LPEXCEPTION_POINTERS lpexceptInfo );PTS 1110787 FF */
int co10_EvalException (OmsSession_co12* omsSession, LPEXCEPTION_POINTERS lpexceptInfo ); /* PTS 1110787 FF */
#endif

/*---------------------------------------------------------------------------*/

inline void omsIfNilOidThrowError (OmsHandle* h, const OmsObjectId& oid, const char* method) {
  if (!oid) {
    h->m_pSession->ThrowDBError (e_nil_pointer, method); 
  }
};

/*----------------------------------------------------------------------*/

#define __HCO13_IMPL
#include "hco13.h" 

static cco10_OmsGlob OmsGlob_co10;

/*----------------------------------------------------------------------*/

void co10_MethodCallEpilog( OmsHandle& h, void* sql_handle, int runTime );     /* PTS 1120804 */

void co10_SetCurrMonitorMethod(OmsHandle& h, tgg00_COMMonitorInfo* p);

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr omsGetContainer(const OmsAbstractObject* pObj) {
  if (NULL == pObj) throw DbpError (DbpError::RTE_ERROR, e_nil_pointer);
  return (OmsObjectContainerPtr) ((unsigned char*) pObj - OmsObjectContainer::headerSize());
};

/*----------------------------------------------------------------------*/
/*           OmsObjectContainer  ---  inline methods                    */
/*----------------------------------------------------------------------*/

inline void OmsObjectContainer::Copy (OmsContext_co12* c, OmsObjectContainerPtr beforeImage, int ObjectSize) {
  tsp00_Int2 refCnt;
  if (this->VarObjFlag()) {
    _OMSASSERT(c->m_session,
        REINTERPRET_CAST(OmsVarObjInfo_co12*, &beforeImage->m_pobj)->m_vobjInfo.m_vobjNext == 
        REINTERPRET_CAST(OmsVarObjInfo_co12*, &this->m_pobj)->m_vobjInfo.m_vobjNext);
    refCnt = REINTERPRET_CAST(OmsVarObjInfo_co12*, &m_pobj)->m_refCnt;
    REINTERPRET_CAST(OmsVarObjInfo_co12*, &m_pobj)->freeVarObjData(c);
  }
  memcpy (&m_oid, &beforeImage->m_oid, ObjectSize - sizeof(m_hashnext));
  if (this->VarObjFlag()) {
    REINTERPRET_CAST(OmsVarObjInfo_co12*, &m_pobj)->m_refCnt = refCnt;
    REINTERPRET_CAST(OmsVarObjInfo_co12*, &beforeImage->m_pobj)->m_pvobj = NULL;
  }
}

#ifdef	LIVECACHE_INTGUIDS
#define LIVECACHE_OBJECT_TMPLPAR(T, guid)	<T, guid>
#define LIVECACHE_KEYEDOBJECT_TMPLPAR(T, key, guid)	<T, key, guid>
#else
#define LIVECACHE_OBJECT_TMPLPAR(T, guid)	<T>
#define LIVECACHE_KEYEDOBJECT_TMPLPAR(T, key, guid)	<T, key>
#endif

// placeholder to allow OMS_ArrayObject.cpp to compile
template LIVECACHE_OBJECT_TMPLPAR(class T2, ClassID guid)
class OmsObject : public OmsAbstractObject
{
  friend class OmsHandle;
public:
#ifndef	LIVECACHE_INTGUIDS
  static const GUID guid;
#endif
};

#endif // __GCO10_H

