/*!***************************************************************************

  module      : SqlHandle.h 

  -------------------------------------------------------------------------

  responsible : FerdiF

  special area: C++ SQL-Class   
  description : SqlHandle - SQL Session Interface 
				
  
  last changed: 2000-03-08  11:22
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/

#ifndef GCO50SQLHANDLE_H
#define GCO50SQLHANDLE_H

#ifndef HCO30_H
#include "hco30-k.h"          
#endif

#ifndef GCO502SQLOPT_H
#include "gco502SqlOpt.h"
#endif

class SQL;
class co51CSqlSessionContext;

/*----------------------------------------------------------------------*/
/* class SqlHandle                                                      */
/*----------------------------------------------------------------------*/

class AFX_EXT_CLASS SqlHandle 
#if !defined (EXT_CLIENT_SQL)
: public DbpBase 
#endif
{
  friend class SQL;
  friend class co51CSqlSessionContext;
  friend class SqlSession;

protected:
  SqlHandle ();
  ~SqlHandle ();		
  void    sqlInitSession (IUnknown** s);
public :
  SQL     sql(const char*);
  SQL     sql(const DbpTypeUnicode*);
  typedef void (__cdecl * sqlHandler ) ( SqlHandle* p_sqlhandle );
  sqlHandler sqlSetHandler(sqlHandler p);     /* PTS 1120804 */
  void sqlSetDefaultHandler();                /* PTS 1120804 */
  //
// C++ SQL Interface Methods
    short   sqlCode () const;           // returns sql error code
    const char*	sqlState() const;       // returns a copy of sqlstate
    const char* sqlErrorMsg() const;    // returns a copy of sqlerrmc
    const DbpTypeUnicode* sqlUnicodeErrorMsg() const; // returns a copy of sqlerrmc_uni
    int     sqlIsUnicodeErrMsg() const; //
    char    sqlWarning (int) const;     // returns sqlwarn [i] )
    long    sqlErrorPos () const;       // returns sql error position sqlerrd[5]
    long    sqlResultCount () const;    // returns sql result count sqlerrd[2]
    void    sqlSetMode(const SqlOpt::SqlSessionMode mode);
    void    sqlRaiseError (short msgno, const char* errmsg);
    void    sqlRaiseError (short msgno, const DbpTypeUnicode* errmsg);
#if !defined (EXT_CLIENT_SQL)
    void    sqlExceptionHandler(DbpError&);					
    HRESULT  AdviseSqlSession();
    HRESULT  UnAdviseSqlSession();
    void    sqlSetMethodCallEpilog();                                    /* PTS 1120804 */
    void    sqlResetMethodCallEpilog();                                  /* PTS 1120804 */
    void    sqlSetCurrMethod(unsigned char* name);                       /* PTS 1120804 */
    void    sqlResetCurrMethod();                                        /* PTS 1120804 */
    bool    sqlGetMethodCallMonitorInfo(SqlMonitorData& info) const;     /* PTS 1120804 */
    void    sqlResetMonitor();                                           /* PTS 1120804 */
    void    sqlSetRuntime(unsigned int Runtime);                         /* PTS 1120804 */
#else
    inline void dbpDebugBreak( );
#endif
    void    sqlResetSession( );
    
    void*                       m_str_assign;
    void*                       m_str_append;
private:
  sqlHandler                    m_sqlhandler;
  IliveCacheSink**              m_session;
  class co51CSqlSessionContext* m_session_context;			
  void*                         dummy2;			
  void*                         dummy3;			
  void*                         dummy4;			
};
/*----------------------------------------------------------------------*/


#endif


