/*!***************************************************************************

  module      : co521CSqlStmt.h 

  -------------------------------------------------------------------------

  responsible : FerdiF

  special area: C++ SQL-Class   
  description : co521CSqlStmt 
				
  
  last changed: 2000-03-08  11:22
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/

#ifndef GCO521CSQLSTMT_H
#define GCO521CSQLSTMT_H

#ifndef HCO30_H
#include "hco30-k.h"
#endif
#ifndef GIN02_H
#include "gin02.h"
#endif
#include "hsp81.h"
#include "livecachetypes.h"   /*no check*/

class SqlCol;
class SQL;
class cin201_Part;
class co51CSqlSessionContext;

typedef struct co521LongDesc {
  tsp00_LongDescriptor dsc;
  int                  varPos;
  int                  bufPos;
  short                colIdx;
} co521LongDesc;

typedef enum { 
  ECO521_SQL,
  ECO521_FETCH_DESCRIBE
} eco521_stmt_type;

/*----------------------------------------------------------------------*/
/* class co521CSqlStmt                                                  */
/*----------------------------------------------------------------------*/

class co521CSqlStmt {
private:
  int                     m_ref_count;
  co51CSqlSessionContext& m_session_context;
  union { char* c; tsp81_UCS2Char* uc; DbpTypeUnicode* dc; } m_statement;
/*  char*                   m_statement; */ 
  boolean                 m_prepared;
  boolean                 m_reset_parms;    /* PTS 1109560 */
  eco521_stmt_type        m_stmt_type;
  tsp00_KnlIdentifierc    m_res_tabname;
  tin02_ParseId           m_parseid;
  tin02_ParseId           m_parseid_of_select;
  int                     m_parseid_len;
  int                     m_unicode;
  struct  structSqlDA {
    structSqlDA();   
    short    inp_col_idx; 
    short    sqlmax;		  /* number of allocated SqlCol entries   */
    short    sqln;		    /* number of in/out SqlCol entries      */
    short    long_col_num;/* number of long columns actually used */
    int      fix_buf_len;
    short    has_long_in_col;
    short    has_long_out_col;
    SqlCol   *sqlvar;
    co521LongDesc *longDesc;
  } m_sqlda;

  void reallocColDescs( short i = 20 );
public:
  co521CSqlStmt(co51CSqlSessionContext* sessContext, const char *stm);
  co521CSqlStmt(co51CSqlSessionContext* sessContext, const DbpTypeUnicode *stm);
  void appendNewColDesc(const SqlCol& p);
  
  int   addInputParms( cin201_Part * part );
  int   addOpenLongDataInput( cin201_Part * part );
  int   addOpenLongDataOutput( cin201_Part * part );
  int   addInputFinishLongData( cin201_Part * part ); 
  int   addRef();
  int   createLongDescriptors();
  int   dropLongDescriptors();
  int   execute();
  co521LongDesc* getLongDescPtr(short idx);
  int   getOutputParms(int argCount, int dataLen, const teo00_Byte * dataPtr); 
  int   getLongDesc(int argCount, int dataLen, const tsp00_Byte * dataPtr); 
  int   getLongDescAndValue(int argCount, int dataLen, const tsp00_Byte * dataPtr); 
  short hasLongInput();
  short hasLongOutput();
  int   prepare( const pasbool parsingAgain = false );
  int   release();
  boolean hasInputParms();
  void  setParsid( int len, const tin02_ParseId * data );
  void  setParsidOfSelect( int len, const tin02_ParseId * data );
  void  setResultTablename( int len, const char * data );
  void  setSqlDA_sqln( short argCount );
  void  setSqlDA_sfi( short i, const tsp1_param_info * sfi );
  void  setResetParms();              /* PTS 1109560 */
  short getNumVars () const;
  short getNumCols () const;
  short getNumOutCols () const;
  int   getParsid( const tin02_ParseId ** data ) const;
  int   getParsidInfo1() const;                  /* PTS 1116114 FF 2002-06-10 */
  int   getFixBufLength( ) const;
  short getLongColNum() const;
  boolean     getPreparedFlag() const;
  const char* getStatement() const;
  const tsp81_UCS2Char* getUniStatement() const;
  int   isUnicode() const;
  eco521_stmt_type getStmtType() const;
  void*   operator new( size_t sz );
  void    operator delete (void* p);
#if defined (EXT_CLIENT_SQL)
  int		connect (const char* hostname, const char* dbname, 
    const char* user, const char* pwd, const char* isolevel);
#endif
  SqlCol  &operator[] (int); 
};

#endif

/*----------------------------------------------------------------------*/
/* PTS 1116114 FF 2002-06-10 */
inline int   
co521CSqlStmt::getParsidInfo1() const
{
    return m_parseid[10];
};                         



