/* @lastChanged: "1998-09-21  9:43"

  * @filename:   hco00.h
  * @purpose:    "User Interface for C++ Persistent Objects"
  * @release:    7.1.0.0
  * @see:        "http://www.bea.sap-ag.de/doku/rel-7.1/hco00.html"
  *
  * @copyright:  "Copyright by   SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/

#if !defined (__liveCache_INCLUDED)
#define __liveCache_INCLUDED

#if _MSC_VER > 1100
#define PLACEMENT_DELETE 1
#endif

#undef  AFX_EXT_CLASS
#define AFX_EXT_CLASS DECLSPEC_IMPORT

#include "ilivecachesink.h" /* no check */
#include "livecachebase.h"  /* no check */

DEFINE_GUID(IID_IliveCacheSink, 
0x8580E621,0x243F,0x11d1,0xB5,0xA1,0xAC,0x7F,0x01,0x06,0x8C,0x36);

#define OMS_DEFAULT_SCHEMA_HANDLE 1

class OmsAbstractObject; // forward declarations
class OmsHandle;
class OmsExternalOid ;
class SqlHandle;     /* PTS 1120804 */

class AFX_EXT_CLASS OmsCriticalSection {
public :
  OmsCriticalSection(OmsHandle*, int id = 0);
  void omsEnter();
  void omsLeave();
  ~OmsCriticalSection();
private :
  void* m_handle;
  int   m_regionId;
  bool  m_inSection;
};

//
// Definition of HTML dump collector interface in simulator
//

class AFX_EXT_CLASS OmsSimDumpCollector {
public:
	virtual void printf(const char *fmt, ...) = 0;
	virtual const char* str() const = 0;
	virtual void flush() = 0;
	inline operator const char *() const { return str(); }
};


struct OmsObjectId : public OmsTypeOid
{
public :
  inline bool operator< ( const OmsObjectId&r ) const;
  inline bool operator<=( const OmsObjectId&r ) const;
  inline bool operator==( const OmsObjectId&r ) const;
  inline bool operator!=( const OmsObjectId&r ) const;
  inline unsigned int   getPno() const;
  inline int            getPagePos() const;
  inline int            getGeneration() const;
  inline long           omsHashValue(long upb) const; /* PTS 1103189 */
  inline bool operator! () const; 
  inline operator bool  () const; 
protected:
  inline OmsObjectId();
  inline void operator= (const OmsExternalOid&r);
  inline OmsObjectId(OmsTypeUInt4 p, unsigned short pp, unsigned short g);
};

struct OmsVarOid : public OmsObjectId
{
public :
  inline OmsVarOid();
  inline OmsVarOid(OmsHandle&h, const OmsExternalOid&r);
  inline bool operator< ( const OmsVarOid&r ) const;
  inline bool operator<=( const OmsVarOid&r ) const;
  inline bool operator==( const OmsVarOid&r ) const;
  inline bool operator!=( const OmsVarOid&r ) const;
  inline bool operator! () const; 
  inline operator bool  () const; 
  inline static bool omsCheckOid ( OmsHandle&, const OmsExternalOid& );
  inline bool omsCheckOid ( OmsHandle& ) const;
  inline const void* operator()(OmsHandle&) const;
  inline const void* omsDeRef(OmsHandle&) const;
};

// ADTs for collection of OIDs

class AFX_EXT_CLASS OmsIOidReadIterator   // PTS 1115027
{
public:
  virtual void omsReset() = 0;
  virtual const OmsObjectId& omsGet() const = 0;
  virtual void operator++ () = 0;
  virtual operator bool () const = 0;
};

class AFX_EXT_CLASS OmsIOidAppender   // PTS 1115027
{
public:
  virtual bool omsAppend(const OmsObjectId& currOid) = 0;
};

// External Oid

class OmsExternalOid : public OmsObjectId
{
public:
  OmsExternalOid() {}
  OmsExternalOid( const OmsObjectId&r ) : OmsObjectId(r) {}
  OmsExternalOid( const OmsTypeOid&r  ) : OmsObjectId(r.pno, r.pagePos, r.generation) {}
  OmsExternalOid( const OmsTypeByte* r) 
  { 
    OmsTypeByte* pDest = REINTERPRET_CAST(OmsTypeByte*, this);
    *pDest++ = *r++;
    *pDest++ = *r++;
    *pDest++ = *r++;
    *pDest++ = *r++;
    *pDest++ = *r++;
    *pDest++ = *r++;
    *pDest++ = *r++;
    *pDest   = *r;
  }
  void omsCopyTo( OmsTypeByte* pDest ) 
  {  
    OmsTypeByte* pSource = REINTERPRET_CAST(OmsTypeByte*, this);
    *pDest++ = *pSource++;
    *pDest++ = *pSource++;
    *pDest++ = *pSource++;
    *pDest++ = *pSource++;
    *pDest++ = *pSource++;
    *pDest++ = *pSource++;
    *pDest++ = *pSource++;
    *pDest   = *pSource;
  }
private:
};

// type save Oid

template<class T>
class OmsOid : public OmsObjectId
{
public:
  typedef bool (T::* OmsQualFunc) ();
  inline OmsOid();
  inline OmsOid( OmsHandle&h, const OmsExternalOid&r );
  inline OmsOid( OmsHandle&h, const OmsObjectId&from ); 
  inline static bool omsCheckOid ( OmsHandle&, const OmsExternalOid& r );
  inline bool omsCheckOid ( OmsHandle& ) const;
  inline const T* operator()(OmsHandle&) const;
  inline T* operator()(OmsHandle&,bool do_lock) const;
  inline const T* omsDeRef(OmsHandle&) const;
  inline T* omsDeRefForUpd(OmsHandle&,bool do_lock=true) const;
  inline bool operator==( const OmsOid<T>&r ) const;
  inline bool operator!=( const OmsOid<T>&r ) const;
  inline bool operator! () const;
  inline operator bool () const;
  // the following is compatibility support for old simulator
  inline void omsSimDumpToHtmlShort(OmsHandle&, OmsSimDumpCollector&, const OmsObjectId&) const;
  inline void omsSimDumpToHtmlLong(OmsHandle&, OmsSimDumpCollector&, const OmsObjectId&) const;
};

class AFX_EXT_CLASS OmsDuplicateKey : public DbpError
{
public:
  OmsDuplicateKey(long e);
};

class AFX_EXT_CLASS OmsLockTimeout : public DbpError
{
public:
  OmsLockTimeout(const OmsObjectId&);
};

class AFX_EXT_CLASS OmsOutOfDate : public DbpError
{
public:
  OmsOutOfDate();
  OmsOutOfDate(const OmsObjectId&);
};

// iterator for all objects of a class

class AFX_EXT_CLASS OmsObjByClsIterBase {
friend class OmsHandle;
public :
  ~OmsObjByClsIterBase();
  OmsObjByClsIterBase(const OmsObjByClsIterBase&);
  OmsObjByClsIterBase(OmsHandle&);
protected:
	operator bool () const;
	const OmsObjectId& operator() () const;
  OmsAbstractObject* omsDeRef(bool,bool) const;
 	void operator++ ();
private:
	OmsHandle*   m_phandle;
	void*        m_clsinfo;
	void*        m_curr;
  void*        m_next;
  bool         m_iterNewObjectsInVersion;
  int          m_headIndex;
};

template< class T3 >
class OmsObjByClsIter : public OmsObjByClsIterBase
{
public:
  OmsObjByClsIter( OmsHandle& h ) : OmsObjByClsIterBase (h) {}
  inline operator bool () const;
  inline const OmsOid<T3> operator() () const;                
  inline const T3* operator() ( OmsHandle&h ) const;         // old style
  inline T3* operator() ( OmsHandle&h, bool do_lock ) const; // old style
  inline const T3* omsDeRef () const;
  inline T3* omsDeRefForUpd (bool do_lock) const;
  inline T3* omsDeRefForUpd (OmsHandle&h, bool do_lock) const; // old style
  inline void operator++ ();
private :
  void operator= (const OmsObjByClsIter<T3>&); // no copy
};

// base class for key range iterators

class AFX_EXT_CLASS OmsObjByKeyIterBase 
{
public :
  OmsObjByKeyIterBase(const OmsObjByKeyIterBase&);
  OmsObjByKeyIterBase
    (
    OmsHandle&      handle, 
    const ClassIDRef guid, 
    OmsSchemaHandle Schema, 
    OmsContainerNo  ContainerNo,
    const void*     pStartKey,
    const void*     pStopKey
    );
  ~OmsObjByKeyIterBase();
protected:
  OmsObjByKeyIterBase() {}
  operator bool () const;
  OmsAbstractObject* omsDeRef(bool,bool) const;
  void operator++ ();
private:
    void* m_pBody;
};

template< class T >
class OmsKeyRangeIter : public OmsObjByKeyIterBase
{
public:
  inline operator bool () const;
  inline const T* omsDeRef () const;
  inline T* omsDeRefForUpd (bool do_lock) const;
  inline void operator++ ();
private:
  OmsKeyRangeIter() {}
  void operator= (const OmsKeyRangeIter<T>&); // no copy
};

// Call Back Interface

// XREF to gco10.h, veo38.cpp for other definitions
class OmsCallbackInterface {
  friend class OmsHandle;
public :
  enum Event {
      BeforeCommit,
      AfterCommit,
      AfterRollback,
      ByApplication
  };
  enum BlockType {
      BlockFreed        = -1,   // freeing already freed block
      BlockInvalid      = 0,    // freeing a block using invalid allocator
      BlockGlobal       = 1,    // global allocator
      BlockSession      = 2,    // session-specific allocator
      BlockTransaction  = 3,    // transaction-specific allocator
      BlockCOMRoutine   = 4,    // COMRoutine-specific allocator

      BlockUnregister   = 15    // unregister interfaces at session end
  };
protected :
  virtual ~OmsCallbackInterface() {};
  virtual void omsValidate (OmsHandle&, Event, bool inVersion) = 0;
  virtual void omsInvalidate (OmsHandle&, Event, bool inVersion) = 0;
  virtual void omsDestroySelf() = 0;
  virtual void omsMethodCallEpilog(OmsHandle&) {};
  /* PTS 1120804 */
  virtual void omsMethodCallEpilog( OmsHandle& oh, SqlHandle& sh ) {
    omsMethodCallEpilog(oh);
  }

public:
  /*!
    function: omsMemLeftOver

    description: Called, when some block is left over in the allocator

    parameters:
        blk         [in]  pointer to the block
        size        [in]  size of the block
        blockType   [in]  allocator ID (1=global, 2=session, 3=transaction, 4=COM routine)
  */
  virtual void omsMemLeftOver(const void *blk, size_t size, BlockType blockType)
#ifdef LIVECACHE_COMPATAPO30
    {}
#else
    = 0;
#endif

  /*!
    function: omsMemInvalidFree

    description: Called, when some block is freed in incorrect allocator

    parameters:
        blk         [in]  pointer to the block
        size        [in]  size of the block
        blockType   [in]  allocator ID (1=global, 2=session, 3=transaction, 4=COM routine)
        freeType    [in]  allocator ID in the block (like blockType; -1/0=invalid allocator used to free)
  */
  virtual void omsMemInvalidFree(const void *blk, size_t size, BlockType blockType, BlockType freeType)
#ifdef LIVECACHE_COMPATAPO30
    {}
#else
    = 0;
#endif
};


// general interface class for application allocators provided by liboms

class AFX_EXT_CLASS OmsAllocatorInterface {
public:
  virtual void *omsAllocate(size_t size) = 0; // allocation
  virtual void omsDeallocate(void *p) = 0;    // deallocation

  virtual bool omsCheckAllocatorEmpty() = 0;  // check for emptyness

  virtual void omsRegisterCallback(OmsCallbackInterface *cb) = 0;
  virtual OmsCallbackInterface *omsGetCallback() const = 0;

  // Return one of OmsCallbackInterface::BlockType constants.
  virtual int omsGetAllocatorType() const = 0;
};

AFX_EXT_CLASS void *operator new(size_t sz, OmsAllocatorInterface &i);
AFX_EXT_CLASS void operator delete(void *ptr, OmsAllocatorInterface &i);


/// Stack trace callback interface.
class AFX_EXT_CLASS OmsStackTraceCallbackInterface {
public:
  /*!
   * \brief Callback for one level in stack.
   *
   * \param frame pointer to the code segment,
   * \param symbol symbol name,
   * \param demangledSymbol demangled symbol name,
   * \param offsetToSymbol offset from the beginning of the symbol,
   * \param moduleName name of the module,
   * \param sourceFileName name of the source file (if available),
   * \param lineNumber line number in source file (if available).
   */
  virtual void omsStackTrace(const void *frame,
    const char *symbol, const char *demangledSymbol,
    unsigned int offsetToSymbol, const char *moduleName,
    const char *sourceFileName, int lineNumber) = 0;
};


// Handle

class OmsHandle : public DbpBase {
  
  friend class OmsAbstractObject;
  friend class DbpError;
  
public :
  enum OmsMode { OPTIMISTIC, PESSIMISTIC, IMPLICIT };
  
  OmsHandle (OmsTypeInt4 sz = 1007);
  ~OmsHandle ();
  void                omsCancel();
  bool                omsCancel(bool toCancel); /* PTS 1120411 */
  const OmsObjectId   &omsCastOid(const ClassIDRef guid, const OmsObjectId& oid);
  void                omsCommitSubtrans( int requiredLevel= -1 );
  inline void         omsCheckCancelFlag() const; // PTS 1110812
  bool                omsCheckOid ( const OmsObjectId&, const ClassIDRef guid );
  bool                omsCheckOid ( const OmsVarOid&);
  void                omsControlCancelFlag(int);          // PTS 1112358
  void                omsControlDbpException(int);        // PTS 1112358
  void                omsControlBadAllocExceptions(int);  // PTS 1112358
  void                omsCloseVersion ( const OmsVersionId& );
  OmsObjByKeyIterBase omsCreateKeyRangeIter(const ClassIDRef     guid,
                                            OmsSchemaHandle sh, 
                                            OmsContainerNo  ContainerNo,
                                            const void*     pStartKey,
                                            const void*     pStopKey);
#ifndef SAPDB_UNICODE
  OmsSchemaHandle     omsCreateSchema(const char* SchemaName);
  void                omsCreateVersion(const OmsVersionId&, const char* pDesc);        // PTS 1117690
#else
  OmsSchemaHandle     omsCreateSchema(const OmsTypeWyde* SchemaName);
  void                omsCreateVersion(const OmsVersionId&, const OmsTypeWyde* pDesc); // PTS 1117690
#endif
  void                omsCreateVersion(const OmsVersionId&);                    
  int                 omsCurrentSubtransLevel() const;
  const OmsVersionId* omsCurrentVersionId () const;
  void                omsDelete ( const OmsObjectId& );
  void                omsDelete ( const OmsVarOid& );
  void                omsDelete ( OmsAbstractObject* );
  const void*         omsDeRef  ( const OmsVarOid& );
  void                omsDropSchema(OmsSchemaHandle);
  void                omsDropContainer(const ClassIDRef, OmsSchemaHandle, OmsContainerNo);
  void                omsDropVersion( const OmsVersionId& );
#ifdef SAPDB_LVCSIM_EXTRAS
  // liveCache Simulator extra functionality
  inline void		  omsSimDump() { LVCSim::omsSimDump(*this, -1); }
  inline void		  omsSimDump(bool version) { LVCSim::omsSimDump(*this, version ? 1 : 0); }
#else
  // standard dump
  void                omsDump( void );
#endif
  static void         omsDumpStackTrace();
  static unsigned int omsSaveStackTrace(void **buffer, unsigned int levels);
  static void         omsEvalStackTrace(void **buffer, unsigned int levels,
                        OmsStackTraceCallbackInterface *cb);
  const char*         omsSimDumpGetHtmlRef(const OmsObjectId &oid);
  void                omsExceptionHandler(DbpError&);
  bool                omsExistsContainer(const ClassIDRef, OmsSchemaHandle, OmsContainerNo); // PTS 1110334
  bool                omsExistsVersion( const OmsVersionId& );
  void                omsForceDropVersion ( const OmsVersionId& );
  OmsContainerNo      omsGetContainerNo (const OmsObjectId&);
  bool                omsGetMethodCallMonitorInfo(OmsMonitorData& info) const;
  OmsSchemaHandle     omsGetSchemaHandle(const OmsObjectId&);
  long                omsGetStoredErrorCode() const;    // PTS 1122838
#ifndef SAPDB_UNICODE
  OmsSchemaHandle     omsGetSchemaHandle(const char* SchemaName) const;
  void                omsGetSchemaName(OmsSchemaHandle,char*,int) const;
  bool                omsGetVersionInfo(const OmsVersionId&, OmsVersionInfo&, char* pDesc_buf, int buf_len) const;         // PTS 1117690
#else
  OmsSchemaHandle     omsGetSchemaHandle(const OmsTypeWyde* SchemaName) const;
  void                omsGetSchemaName(OmsSchemaHandle,OmsTypeWyde*,int) const;
  bool                omsGetVersionInfo(const OmsVersionId&, OmsVersionInfo&, OmsTypeWyde* pDesc_buf, int buf_len) const;  // PTS 1117690
#endif
  OmsCallbackInterface* omsGetTransactionInterface (const ClassIDRef) const;
  int                 omsGetTimeOut() const;
  unsigned long       omsGetVarObjectSize (const OmsVarOid& oid);
  bool                omsIsLocked ( const OmsObjectId& );
  bool                omsIsRegistered(const ClassIDRef, OmsSchemaHandle, OmsContainerNo);
  void                omsInitSession (IUnknown** s);
  void                omsLoadVarObject (const OmsVarOid& oid, unsigned long size, void*);
  void                omsLock( const OmsObjectId&);
  bool                omsTryLock( const OmsObjectId& );
  void                omsLock( const OmsVarOid& );
  bool                omsTryLock( const OmsVarOid& );
  void                omsLocks( const OmsObjectId*, int);
  void                omsMarkVersion ( const OmsVersionId& );
  void                omsNewConsistentView();
  void                omsNewConsistentView(OmsObjectId* pOids, int cntOids, short timeout);
  void                omsNewConsistentView(OmsObjectId* pOids, int cntOids, short timeout, OmsObjectId* pErrOids, int& cntErrOids);
  void                omsNewConsistentView(OmsIOidReadIterator& pOids, short timeout, OmsIOidAppender& pErrOids);   // PTS 1115027
  const OmsVarOid&    omsNewVarObject(OmsSchemaHandle,OmsContainerNo);
#ifndef SAPDB_UNICODE
  void                omsOidInfo (const OmsObjectId&, ClassID&, OmsSchemaHandle&, 
                                  char* SchemaName, int SchemaNameBufSize,
                                  char* ClassName,  int ClassNameBufSize, OmsContainerNo&);
#else
  void                omsOidInfo (const OmsObjectId&, ClassID&, OmsSchemaHandle&, 
                                  OmsTypeWyde* SchemaName, int SchemaNameBufSize,
                                  char* ClassName,  int ClassNameBufSize, OmsContainerNo&);
#endif
  void                omsOpenVersion( const OmsVersionId& );
  void                omsPrintOid (char* buf, size_t bufSize, const OmsObjectId oid);
  void                omsReadOnly (bool = true);
  bool                omsGetReadOnly();
  long                omsRefresh();
  void                omsRegTransactionInterface (const ClassIDRef, OmsCallbackInterface*);
  void                omsRelease  (const OmsObjectId&);
  void                omsRelease  (const OmsVarOid&);
  void                omsReleaseAll ();
  void                omsReleaseConsistentView();
  void                omsReleaseVarObject(const OmsVarOid&);
  void                omsRollbackSubtrans( int requiredLevel = -1 );
  void                omsRenameVersion( const OmsVersionId& oldId, const OmsVersionId& newId); 
  void                omsReschedule() const;
  void                omsResetVersion(const OmsVersionId& );
  void                omsSetTimeOut( short );
  bool                omsSetCachedKeys(bool, const ClassIDRef, OmsSchemaHandle, OmsContainerNo);  // PTS 1117571
#ifndef SAPDB_UNICODE
  void                omsSetCurrVersionDesc(const char* pDesc);          // PTS 1117690
#else
  void                omsSetCurrVersionDesc(const OmsTypeWyde* pDesc);   // PTS 1117690
#endif
  int                 omsStartSubtrans();
  void                omsStoreErrorCode(long errorCode);    // PTS 1122838
  void                omsStoreVarObject ( const OmsVarOid& oid, const void*, unsigned long objLength);
  void                omsTerminate();
#ifndef SAPDB_UNICODE
  void                omsTerminate(const char*);
#else
  void                omsTerminate(const OmsTypeUTF8*);      // PTS 1109693
#endif
  void                omsThrowCancelledError() const; // PTS 1110812
  void                omsTraceOid  (const OmsObjectId& oid);
  void                omsTransRollback();

  static OmsHandle*   omsGetOmsHandle();      

  OmsAllocatorInterface &omsGetGlobalAllocator();
  OmsAllocatorInterface &omsGetGlobalAllocator(const char *name);
  OmsAllocatorInterface &omsGetSessionAllocator();
  OmsAllocatorInterface &omsGetTransactionAllocator();
  OmsAllocatorInterface &omsGetCOMRoutineAllocator();

#ifdef SAPDB_LVCSIM_EXTRAS
  // liveCache Simulator extra functionality
  short				  omsCommit(bool endSession);
  void				  omsRollback(bool endSession, bool ignoreErrors);

  inline void		  omsCommit() { LVCSim::Commit(*this); }
  inline void		  omsRollback() { LVCSim::Rollback(*this); }
#endif // SAPDB_LVCSIM_EXTRAS

private :
  OmsHandle (const OmsHandle&);             // no copy
  OmsHandle& operator= (const OmsHandle&r); // no assignment
  void*          m_dummy1;
  void*          m_dummy2;
  void*          m_dummy3;
  unsigned char* m_pCancelFlag;
  unsigned char  m_dummy4;
};


//
// Definition of abstract persistent Object
//
class AFX_EXT_CLASS OmsAbstractObject {
  
  friend class OmsHandle;
  friend class OmsArrayObjectBase;
  friend class OmsArrayObjectIteratorBase;
public :
  enum OmsRegEnum { USE_IN_REGISTRY_ONLY };
  
  virtual void      omsCleanUp  ( OmsHandle& );
  virtual void      omsDump     ( OmsHandle&, const char* flags) const;
  void              omsDelete   ( OmsHandle& );
  void              omsFlush    ( OmsHandle& );
  const OmsObjectId &omsGetOid  () const;
  bool              omsIsLocked ( OmsHandle& ) const;
  void              omsLock     ( OmsHandle& );
  bool              omsTryLock  ( OmsHandle& );
  void              omsRelease  ( OmsHandle& );
  void              omsStore    ( OmsHandle& );
  virtual void      omsVerify   ( OmsHandle& ) const;
protected :
#ifndef LIVECACHE_COMPATAPO30
  virtual void      omsKeyToBinary(const void* pKey, void* pDest) const;
  virtual void      omsBinaryToKey(void* pSource, void* pKey) const;
#endif
  OmsAbstractObject ();
  const OmsObjectId &omsOid() const;
  OmsAbstractObject* omsForUpdPtr(OmsHandle& h, bool doLock) const;
  bool   omsHistoryInUse(OmsHandle&) const;
  void   omsKey(OmsHandle& Handle, unsigned char* key) const;
  static void omsDeleteAll(OmsHandle& h, const ClassIDRef guid, OmsSchemaHandle, OmsContainerNo ContainerNo);
  static OmsAbstractObject* omsNewObject (OmsHandle&, const ClassIDRef, OmsSchemaHandle, OmsContainerNo );
  static OmsAbstractObject* omsNewRegistryObject (size_t, OmsHandle&, const ClassIDRef);
  static OmsAbstractObject* omsNewKeyedObject( OmsHandle& Handle, const ClassIDRef, const unsigned char* key, OmsSchemaHandle, OmsContainerNo );
  static OmsObjByClsIterBase omsAllOids( OmsHandle& h, const ClassIDRef guid, OmsSchemaHandle, OmsContainerNo);
  static void omsRegClass (
    OmsHandle&, 
    const ClassIDRef,
    const char* ClassName,
    size_t      PersistentSize,
    size_t      ObjectSize, 
    const ClassIDPtr pBaseClass,
    OmsAbstractObject* Obj,
    OmsSchemaHandle,
    OmsContainerNo,
    size_t = 0);
  static void omsRegClassAndKey (OmsHandle& Handle,
    const ClassIDRef Guid, 
    const char* ClassName,
    size_t PersistentSize, 
    size_t KeySize, 
    size_t ObjectSize,
    OmsAbstractObject* Obj,
    OmsSchemaHandle,
    OmsContainerNo);
  static void omsRegClassAndKey (OmsHandle& Handle,
    const ClassIDRef Guid, 
    const char* ClassName,
    size_t PersistentSize, 
    size_t KeySize, 
    size_t ObjectSize,
    OmsAbstractObject* Obj,
    OmsSchemaHandle,
    OmsContainerNo,
    bool partitionedKey);
  static const OmsAbstractObject* omsDeRef (const OmsObjectId& oid, OmsHandle& Handle, const ClassIDRef guid);
  static OmsAbstractObject* omsDeRefForUpd (const OmsObjectId& oid, OmsHandle& Handle, const ClassIDRef guid, bool);
  static const OmsAbstractObject* omsDeRefKey (unsigned char* key, OmsHandle&, const ClassIDRef, OmsSchemaHandle, OmsContainerNo);
  static OmsAbstractObject* omsDeRefKeyForUpd (unsigned char* key, OmsHandle&, const ClassIDRef, bool, OmsSchemaHandle, OmsContainerNo);
private :
  OmsAbstractObject(const OmsAbstractObject&);              // no copy
  OmsAbstractObject& operator= (const OmsAbstractObject&r); // no assignment

  // extras for liveCache simulator
public:
  virtual void omsSimDumpToHtmlShort(OmsHandle&, OmsSimDumpCollector&) const;
  virtual void omsSimDumpToHtmlLong(OmsHandle&, OmsSimDumpCollector&) const;
  void omsSimSetObjectName(OmsHandle &h, const char *name);
  const char* omsSimGetObjectName(OmsHandle &h) const;
  static const char *omsSimDumpMakeOidRef(OmsHandle &h, const OmsObjectId &oid, int flags = 3);
  inline static const char *omsSimDumpMakeOidRef(const OmsObjectId &oid, int flags = 3)
  {
    return omsSimDumpMakeOidRef(*OmsHandle::omsGetOmsHandle(), oid, flags);
  }

#ifdef	SAPDB_LVCSIM_EXTRAS
#ifdef	SAPDB_LVCSIM_DEPRECATED
public:
  static void omsRegClass(OmsHandle &h, const ClassIDRef guid, size_t psoSize, size_t objSize, 
	  const char*cls_name, const ClassIDPtr base_class, const OmsAbstractObject*pobj, 
	  OmsSchemaHandle sh, OmsContainerNo cno);
  static void omsRegClassAndKey(OmsHandle &h, const ClassIDRef guid, size_t psoSize, 
	  size_t objSize, size_t keySize, const char*cls_name, const ClassIDPtr base_class, 
	  const OmsAbstractObject*pobj, OmsSchemaHandle sh, OmsContainerNo cno);
#endif
#endif
};

// template for persistent objects

#ifdef	LIVECACHE_INTGUIDS
#define LIVECACHE_OBJECT_TMPLPAR(T, guid)	<T, guid>
#define LIVECACHE_KEYEDOBJECT_TMPLPAR(T, key, guid)	<T, key, guid>
#else
#define LIVECACHE_OBJECT_TMPLPAR(T, guid)	<T>
#define LIVECACHE_KEYEDOBJECT_TMPLPAR(T, key, guid)	<T, key>
#endif

#if defined(WIN32)
// Switch off warnings which are caused by private destructor
#pragma warning( push )
#pragma warning( disable : 4624 )
#endif

template LIVECACHE_OBJECT_TMPLPAR(class T2, ClassID guid)
class OmsObject : public OmsAbstractObject
{
  friend class OmsHandle;
  friend class OmsOid<T2>;
public:
#ifndef	LIVECACHE_INTGUIDS
  static const GUID guid;
#endif
  inline const OmsOid<T2> omsGetOid() const;
  inline T2* omsForUpdPtr(OmsHandle& h, bool doLock=true) const; 
  inline void* operator new ( size_t sz, OmsHandle&ah, OmsSchemaHandle, OmsContainerNo);
  inline void* operator new ( size_t sz, OmsHandle& ah, OmsRegEnum e);
  inline void* operator new ( size_t sz, OmsHandle& ah, OmsRegEnum e, size_t rsz);
  void operator delete ( void* p, OmsHandle&ah, OmsSchemaHandle, OmsContainerNo) {}
  void operator delete ( void* p, OmsHandle& ah, OmsRegEnum e) {}
  void operator delete (void* p, OmsHandle& ah, OmsRegEnum e, size_t rsz) {}
  inline bool  omsHistoryInUse(OmsHandle&) const;
  static void  omsDeleteAll ( OmsHandle&h, OmsSchemaHandle sh, OmsContainerNo ContainerNo )
  {
    OmsAbstractObject::omsDeleteAll( h, guid, sh, ContainerNo );
  }
  static OmsObjByClsIter<T2> omsAllOids( OmsHandle&h, OmsSchemaHandle sh, OmsContainerNo ContainerNo )
  { 
    OmsObjByClsIterBase iter = OmsAbstractObject::omsAllOids( h, guid, sh, ContainerNo );
    return *REINTERPRET_CAST (OmsObjByClsIter<T2>*, &iter);
  }
  static bool omsOidIsInstanceOf(OmsHandle&h, const OmsExternalOid&oid) {
	return T2::omsCheckOid(h, reinterpret_cast<const OmsObjectId&>(oid));
  }
  static void omsRegClass( OmsHandle&ah, const char* className, 
    size_t sz, OmsObject LIVECACHE_OBJECT_TMPLPAR(T2, guid)* pobj, OmsSchemaHandle sh, OmsContainerNo ContainerNo)
  { 
    OmsAbstractObject::omsRegClass( ah, guid, className, sz, sizeof(T2), NULL, pobj, sh, ContainerNo ); 
  }

  static void omsRegClass( OmsHandle&ah, const char* className, 
    size_t sz, const ClassIDRef BaseClassGuid, OmsObject LIVECACHE_OBJECT_TMPLPAR(T2, guid)* pobj, OmsSchemaHandle sh, OmsContainerNo ContainerNo)
  { 
#ifdef LIVECACHE_INTGUIDS
    OmsAbstractObject::omsRegClass( ah, guid, className, sz, sizeof(T2), BaseClassGuid, pobj, sh, ContainerNo ); 
#else
    OmsAbstractObject::omsRegClass( ah, guid, className, sz, sizeof(T2), &BaseClassGuid, pobj, sh, ContainerNo ); 
#endif
  }

  static bool omsCheckOid(OmsHandle &h, const OmsObjectId &id)
  {
	return h.omsCheckOid(id, guid);
  }

  static const ClassIDRef omsGetGuid()
  {
    return guid;
  }

  static const OmsObjectId &omsCastOid(OmsHandle &h, const OmsObjectId &from)
  {
	return h.omsCastOid(guid, from);
  }

#ifdef	SAPDB_LVCSIM_EXTRAS
  // extras for liveCache simulator
#ifdef	SAPDB_LVCSIM_DEPRECATED
  static inline void omsRegClass(OmsHandle&h, size_t sz, const char*cls_name, const OmsAbstractObject*pobj, OmsSchemaHandle sh, OmsContainerNo cno )
  { OmsAbstractObject::omsRegClass(h, guid, sz, sizeof(T2), cls_name, 0, pobj, sh, cno ); }
  static inline void omsRegClass(OmsHandle&h, size_t sz, const char*cls_name, const ClassIDRef base_class, const OmsAbstractObject*pobj, OmsSchemaHandle sh, OmsContainerNo cno )
#ifdef LIVECACHE_INTGUIDS 
  { OmsAbstractObject::omsRegClass(h, guid, sz, sizeof(T2), cls_name, base_class, pobj, sh, cno ); }
#else
  { OmsAbstractObject::omsRegClass(h, guid, sz, sizeof(T2), cls_name, &base_class, pobj, sh, cno ); }
#endif
#endif
#endif

protected:
  OmsObject() {}
#if defined(WIN32)
private:
#endif
  ~OmsObject() {} // derived class must not be destructed
#if !defined(WIN32)
private :
#endif

  static const T2* omsDeRef( const OmsOid<T2>&aoid, OmsHandle&ah )   
  { 
    return (T2*)OmsAbstractObject::omsDeRef( aoid, ah, guid );
  }
  static T2* omsDeRefForUpd( const OmsOid<T2>&aoid, OmsHandle&ah, bool doLock )
  { 
    return (T2*)OmsAbstractObject::omsDeRefForUpd( aoid, ah, guid, doLock );
  }
  OmsObject( const OmsObject LIVECACHE_OBJECT_TMPLPAR(T2, guid)& );       // no copy
  void operator= ( const OmsObject LIVECACHE_OBJECT_TMPLPAR(T2, guid)& ); // no assignment
};

// template for persistent objects with key

template LIVECACHE_KEYEDOBJECT_TMPLPAR(class Obj, class Key, ClassID guid)
class OmsKeyedObject : public OmsObject LIVECACHE_OBJECT_TMPLPAR(Obj, guid)
{
public:
  inline void  omsGetKey( OmsHandle&ah, Key& k ) const;
  void* operator new ( size_t sz, OmsHandle&ah, const Key&akey ,OmsSchemaHandle sh, OmsContainerNo ContainerNo)
  {
    return (void*) OmsAbstractObject::omsNewKeyedObject( ah, guid, (unsigned char*) &akey, sh, ContainerNo );
  }
  void* operator new ( size_t sz, OmsHandle& ah, OmsAbstractObject::OmsRegEnum e)
  {
    return (void*) OmsAbstractObject::omsNewRegistryObject( sz + sizeof(Key), ah, guid); 
  }
  void operator delete (void* p, OmsHandle&ah, const Key&akey ,OmsSchemaHandle sh, OmsContainerNo ContainerNo) {}
  void operator delete (void* p, OmsHandle& ah, OmsAbstractObject::OmsRegEnum e) {}
  static void  omsRegClass( OmsHandle&ah, const char* className, size_t sz, OmsObject LIVECACHE_OBJECT_TMPLPAR(Obj, guid)* pobj, OmsSchemaHandle sh, OmsContainerNo cno )
  { 
    OmsAbstractObject::omsRegClassAndKey( ah, guid, className, sz, sizeof(Key), sizeof(Obj), pobj, sh, cno, false ); 
  }
  static void  omsRegClass(
    OmsHandle&ah, 
    const char* className, 
    size_t sz, 
    OmsObject LIVECACHE_OBJECT_TMPLPAR(Obj, guid)* pobj, 
    OmsSchemaHandle sh, 
    OmsContainerNo cno,
    bool partitionedKey)
  { 
    OmsAbstractObject::omsRegClassAndKey( ah, guid, 
      className, sz, sizeof(Key), sizeof(Obj), pobj, sh, cno, partitionedKey ); 
  }


#ifdef	SAPDB_LVCSIM_EXTRAS
  // extras for liveCache simulator
#ifdef	SAPDB_LVCSIM_DEPRECATED
  static inline void  omsRegClass(OmsHandle &ah, size_t sz, const char*cls_name, OmsAbstractObject*pobj, OmsSchemaHandle sh, OmsContainerNo cno)
  { OmsAbstractObject::omsRegClassAndKey(ah, guid, sz, sizeof(Obj), sizeof(Key), cls_name, 0, pobj, sh, cno); }
  static inline void  omsRegClass(OmsHandle &ah, const char*cls_name, size_t sz, const ClassIDRef base_class, OmsAbstractObject*pobj, OmsSchemaHandle sh, OmsContainerNo cno)
#ifdef LIVECACHE_INTGUIDS
  { OmsAbstractObject::omsRegClassAndKey(ah, guid, sz, sizeof(Obj), sizeof(Key), cls_name, base_class, pobj, sh, cno); }
#else
  { OmsAbstractObject::omsRegClassAndKey(ah, guid, sz, sizeof(Obj), sizeof(Key), cls_name, &base_class, pobj, sh, cno); }
#endif
#endif
#endif

  static const Obj* omsKeyAccess( OmsHandle&h, const Key&key, OmsSchemaHandle shdl, OmsContainerNo cno )  
  {
    return (Obj*)OmsAbstractObject::omsDeRefKey( (unsigned char*)(&key), h, guid, shdl, cno );
  }
  static Obj* omsKeyAccessForUpd( OmsHandle&h, const Key&key, bool doLock, OmsSchemaHandle sh, OmsContainerNo cno )  
  {
    return (Obj*)OmsAbstractObject::omsDeRefKeyForUpd((unsigned char*)(&key), h, guid, doLock, sh, cno );
  }
  static OmsKeyRangeIter<Obj> omsFirstKey(
    OmsHandle&      h, 
    OmsSchemaHandle sh, 
    OmsContainerNo  ContainerNo,
    const Key&      startKey,
    const Key&      stopKey)
  { 
#ifdef LIVECACHE_COMPATAPO30
    OmsObjByKeyIterBase iter = h.omsCreateKeyRangeIter( guid, 
      sh, ContainerNo, &startKey, &stopKey );
#else
    unsigned char binStartKey[sizeof(Key)];
    unsigned char binStopKey [sizeof(Key)];
    startKey.omsKeyToBinary(&binStartKey[0]);
    stopKey.omsKeyToBinary(&binStopKey[0]);
    OmsObjByKeyIterBase iter = h.omsCreateKeyRangeIter( guid, 
      sh, ContainerNo, &binStartKey[0], &binStopKey[0] );
#endif
    return *REINTERPRET_CAST (OmsKeyRangeIter<Obj>*, &iter);
  }
protected:
#ifndef LIVECACHE_COMPATAPO30
  virtual void  omsKeyToBinary(const void* pKey, void* pDest) const 
  {
    REINTERPRET_CAST(const Key*, pKey)->omsKeyToBinary(pDest);
  }
  virtual void omsBinaryToKey(void* pSource, void* pKey) const 
  {
    REINTERPRET_CAST(Key*, pKey)->omsBinaryToKey(pSource);
  }
#endif
};

#if defined(WIN32)
// Reset old settings of pragma
#pragma warning( pop )
#endif


// lock object

class AFX_EXT_CLASS OmsLockObject {
public :
   OmsLockObject(OmsHandle&);
   ~OmsLockObject();
                           
   void omsEnable ();
   void omsEnable (const OmsLockHandle&);

   const OmsLockHandle& omsGetLockHandle() const;
   void  omsLockShare();
   void  omsLockExclusive();
   void  omsUnlock();
   bool  omsIsLocked() const;
   bool  omsShareLocked() const;
   bool  omsExclusiveLocked() const;

private :
  enum OmsLockMode{ NO_LOCK, LOCK_SHARE, LOCK_EXCLUSIVE };
  OmsHandle*     m_omsHandle;
  void*          m_void;
  OmsLockHandle  m_lockHandle;
  OmsLockMode    m_mode;
};

// abap table stream

class AFX_EXT_CLASS OmsStream {
protected :
  OmsStream(OmsHandle* h, OmsTypeABAPTabHandle& sh, long rsz, long sz, bool istream = true);    
  OmsStream(OmsHandle* h, OmsTypeStreamHandle& sh, long rsz, long sz, bool istream = true);
  ~OmsStream();
  void           omsClose();
  void           omsFlush();
  unsigned char* omsNxt();
  void           omsWrite(unsigned char*);
private :
  OmsStream(const OmsStream&);       // no copy
  void operator= (const OmsStream&); // assignment
  void* m_body;
};

// abap table input stream

template <class HandleType, class RowType> class OmsIStream : public OmsStream {
public :
  OmsIStream (OmsHandle* h, HandleType& th, long cached_rows = -1) :
       OmsStream (h, th.Handle, sizeof(RowType), cached_rows) {
         RowType* p = th.RowDef;
		 p;
       };
       RowType* omsRead() { return (RowType*) omsNxt(); };
};

// abap table output stream

template <class HandleType, class RowType> class OmsOStream : public OmsStream {
public :
  OmsOStream (OmsHandle* h, HandleType& th, long cached_rows = -1) :
       OmsStream (h, th.Handle, sizeof(RowType), cached_rows, false) {
         RowType* p = th.RowDef;
		 p;
       };
  void omsClose () { OmsStream::omsClose (); };
  void omsWrite (const RowType& row) { OmsStream::omsWrite ((unsigned char*) &row); };     
};

// abap timestamp

class AFX_EXT_CLASS OmsTimeStamp
{
public:
  OmsTimeStamp(const OmsTypeABAPTimeStamp& timeStamp);
  const    OmsTypeABAPTimeStamp& omsGetTimeStamp() const;
  unsigned long omsSecsSince01011970();
private :
  unsigned long        m_DateTime;
  OmsTypeABAPTimeStamp m_ABAPTimeStamp;
};

#define OMS_REG_CLASS(a,s,cno,h) {a::omsRegClass(h, #a, sizeof(a), new(h, OmsObject LIVECACHE_OBJECT_TMPLPAR(a, 0)::USE_IN_REGISTRY_ONLY) a, s,cno);}
#define OMS_REG_CLASS_WITH_PARTITIONED_KEY(a,s,cno,h) {a::omsRegClass(h, #a, sizeof(a), new(h, OmsObject LIVECACHE_OBJECT_TMPLPAR(a, 0)::USE_IN_REGISTRY_ONLY) a, s,cno,true);}
#define OMS_REG_CLASS_PREFIX(a,prefixSize,h) {a::omsRegClass(h, #a, prefixSize, new(h, OmsObject LIVECACHE_OBJECT_TMPLPAR(a, 0)::USE_IN_REGISTRY_ONLY) a);}
#define OMS_REG_CLASS_INH(a,s,cno,b,h) {a::omsRegClass(h, #a, sizeof(a), b::omsGetGuid(),new(h, OmsObject LIVECACHE_OBJECT_TMPLPAR(a, 0)::USE_IN_REGISTRY_ONLY) a, s,cno);}

extern
bool omsIsUnicodeInstance();

extern
void* omsMalloc(size_t size);

extern 
void omsFree(void* p);

extern
void* omsMallocArray(size_t size);

extern 
void omsFreeArray(void* p);

extern 
void omsGetLocalDateTimeString(char*, int);

extern
void omsGetLocalDateTime(int& year,
                           int& month,
                           int& day,
                           int& hours,
                           int& minutes,
                           int& seconds);

extern
void omsGetUTCDateTimeString(char*, int);

extern
void omsGetUTCDateTime(int& year,
                           int& month,
                           int& day,
                           int& hours,
                           int& minutes,
                           int& seconds);

extern
void omsGetLiveCacheVersion(char*, int);

extern
void omsSimConsoleMessage(const char*);

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                           Start of Implementation part                                             //
////////////////////////////////////////////////////////////////////////////////////////////////////////

#define NIL_OBJ_PAGE_NO  2147483647
#define NIL_OBJ_PAGE_POS 0
#define NIL_OBJ_VERSION  0

inline OmsObjectId::OmsObjectId() {
  pno        = NIL_OBJ_PAGE_NO;
  pagePos    = NIL_OBJ_PAGE_POS;
  generation = NIL_OBJ_VERSION;
}

inline bool OmsObjectId::operator< ( const OmsObjectId&r ) const
{
#ifdef LIVECACHE_INTGUIDS
  if (pno < r.pno) {
	return true;
  }
  if (pno > r.pno) {
    return false;
  }
  return *((&pno) + 1) < *((&r.pno) + 1);
#else
  register unsigned char* pThis = (unsigned char*) this;
  register unsigned char* pR    = (unsigned char*) &r;

  for (int ix = 0; ix < sizeof(r) - 1; ++ix)
  {
    if (*pThis != *pR)
    {
      break;
    }
    else
    {
      ++pThis;
      ++pR;
    }
  }
  return (*pThis < *pR); 
#endif
}

inline bool OmsObjectId::operator<=( const OmsObjectId&r ) const
{
#ifdef LIVECACHE_INTGUIDS
  if (pno < r.pno) {
	return true;
  }
  if (pno > r.pno) {
    return false;
  }
  return *((&pno) + 1) <= *((&r.pno) + 1);
#else
  register unsigned char* pThis = (unsigned char*) this;
  register unsigned char* pR    = (unsigned char*) &r;

  for (int ix = 0; ix < sizeof(r) - 1; ++ix)
  {
    if (*pThis != *pR)
    {
      break;
    }
    else
    {
      ++pThis;
      ++pR;
    }
  }
  return (*pThis <= *pR);
#endif
}

inline bool OmsObjectId::operator==( const OmsObjectId&r ) const 
{ 
    return ((pno == r.pno) && (pagePos == r.pagePos) && (generation == r.generation)); 
}
inline bool OmsObjectId::operator!=( const OmsObjectId&r ) const
{
    return ((pno != r.pno) || (pagePos != r.pagePos) || (generation != r.generation)); 
}

inline OmsObjectId::OmsObjectId(OmsTypeUInt4 p, unsigned short pp, unsigned short g)
{ 
  pno        = p;
  pagePos    = pp;
  generation = g;
}

inline void OmsObjectId::operator= (const OmsExternalOid&r) 
{
  pno        = r.pno;
  pagePos    = r.pagePos;
  generation = r.generation;
}

inline bool OmsObjectId::operator! () const 
{
  return ((NIL_OBJ_PAGE_NO==pno) && (NIL_OBJ_PAGE_POS==pagePos) && (NIL_OBJ_VERSION==generation));
}

inline OmsObjectId::operator bool () const 
{ 
  return ((NIL_OBJ_PAGE_NO!=pno) || (NIL_OBJ_PAGE_POS!=pagePos) || (NIL_OBJ_VERSION!=generation));
}

inline unsigned int OmsObjectId::getPno() const {
  return pno;
}

inline int OmsObjectId::getPagePos() const {
  return pagePos;
}

inline int OmsObjectId::getGeneration() const {
  return generation;
}

/* PTS 1103189 */
inline long OmsObjectId::omsHashValue(long upb) const { 
  return (((getPno() << 13) + pagePos) % upb);
}

// OmsVarOid

inline OmsVarOid::OmsVarOid() : OmsObjectId() {};

inline OmsVarOid::OmsVarOid( OmsHandle&h, const OmsExternalOid&r )
{
  OmsObjectId::operator=( r );
  if (!omsCheckOid( h )) {
    throw DbpError(E_INCOMPATIBLE_OID);
  }
}
  
inline bool OmsVarOid::operator< ( const OmsVarOid&r ) const
{ 
  return OmsObjectId::operator<(r);
}

inline bool OmsVarOid::operator<=( const OmsVarOid&r ) const
{
  return OmsObjectId::operator<=(r);
}

inline bool OmsVarOid::operator==( const OmsVarOid&r ) const 
{ 
  return OmsObjectId::operator==(r);
}

inline bool OmsVarOid::operator!=( const OmsVarOid&r ) const
{
  return OmsObjectId::operator!=(r);
}

inline bool OmsVarOid::operator! () const 
{ 
  return (OmsObjectId::operator!()) ;
}  

inline OmsVarOid::operator bool () const 
{ 
  return (OmsObjectId::operator bool()) ;  
}

inline bool OmsVarOid::omsCheckOid ( OmsHandle&h ) const
{
  return h.omsCheckOid(*this);
}

inline bool OmsVarOid::omsCheckOid ( OmsHandle&h, const OmsExternalOid&r )
{
  return h.omsCheckOid(reinterpret_cast<const OmsVarOid&>(r));
}

inline const void* OmsVarOid::operator()(OmsHandle& h) const 
{
  return h.omsDeRef(*this);
}

inline const void* OmsVarOid::omsDeRef(OmsHandle& h) const
{
  return h.omsDeRef(*this);
}

// OmsOid

template<class T>
inline
OmsOid<T>::OmsOid() : OmsObjectId(NIL_OBJ_PAGE_NO, NIL_OBJ_PAGE_POS, NIL_OBJ_VERSION) {}

template<class T>
inline 
OmsOid<T>::OmsOid( OmsHandle&h, const OmsExternalOid&r )
{
  OmsObjectId::operator=( r );
  T::omsDeRef( *this, h ); // PTS 1109694 
}

template<class T>
inline OmsOid<T>::OmsOid( OmsHandle&h, const OmsObjectId&from )
{
  OmsObjectId::operator=(T::omsCastOid(h, from));
}

template<class T>
inline
bool OmsOid<T>::omsCheckOid( OmsHandle&h, const OmsExternalOid& r )
{
	return T::omsCheckOid(h, reinterpret_cast<const OmsObjectId&>(r));
}

template<class T>
inline
bool OmsOid<T>::omsCheckOid( OmsHandle&h ) const
{
	return T::omsCheckOid(h, reinterpret_cast<const OmsObjectId&>(*this));
}

template<class T> 
inline 
const T* OmsOid<T>::operator() ( OmsHandle& ah ) const 
{
    return T::omsDeRef( *this, ah ); 
}

template<class T> 
inline 
T* OmsOid<T>::operator() ( OmsHandle& ah, bool doLock) const 
{
    return T::omsDeRefForUpd( *this, ah, doLock ); 
}

template<class T> 
inline 
const T* OmsOid<T>::omsDeRef ( OmsHandle& ah ) const 
{
    return T::omsDeRef( *this, ah ); 
}

template<class T> 
inline 
T* OmsOid<T>::omsDeRefForUpd ( OmsHandle& ah, bool doLock) const 
{
    return T::omsDeRefForUpd( *this, ah, doLock ); 
}

template<class T> 
inline 
bool OmsOid<T>::operator==( const OmsOid<T>&r ) const
{ 
    return ((pno == r.pno) && (pagePos == r.pagePos) && (generation == r.generation));
}
template<class T> 
inline 
bool OmsOid<T>::operator!=( const OmsOid<T>&r ) const
{ 
    return ((pno != r.pno) || (pagePos != r.pagePos) || (generation != r.generation));
}
template<class T> 
inline 
bool OmsOid<T>::operator! () const 
{
  return (OmsObjectId::operator! ());
}
template<class T> 
inline 
OmsOid<T>::operator bool () const
{ 
  return (OmsObjectId::operator bool ()); 
}

template<class T> 
inline 
void OmsOid<T>::omsSimDumpToHtmlShort(OmsHandle &h, OmsSimDumpCollector &s, const OmsObjectId &ref) const
{
  if (!*this) {
    s.printf("NULL OID");
  } else if (this->omsCheckOid(h)) {
    const T *p = this->omsDeRef(h);
    p->omsSimDumpToHtmlShort(h, s);
  } else {
    s.printf("<B><FONT COLOR=#FF0000> Invalid OID %s </FONT></B>", 
      OmsAbstractObject::omsSimDumpMakeOidRef(*this));
    s.printf("<B><FONT COLOR=#FF0000>Ref OID %s </FONT></B>", 
      OmsAbstractObject::omsSimDumpMakeOidRef(ref));
  }
}

template<class T> 
inline 
void OmsOid<T>::omsSimDumpToHtmlLong(OmsHandle &h, OmsSimDumpCollector &s, const OmsObjectId &ref) const
{
  if (!*this) {
    s.printf("NULL OID");
  } else if (this->omsCheckOid(h)) {
    const T *p = this->omsDeRef(h);
    p->omsSimDumpToHtmlLong(h, s);
  } else {
    s.printf("<B><FONT COLOR=#FF0000> Invalid OID %s </FONT></B>", 
      OmsAbstractObject::omsSimDumpMakeOidRef(*this));
    s.printf("<B><FONT COLOR=#FF0000>Ref OID %s </FONT></B>", 
      OmsAbstractObject::omsSimDumpMakeOidRef(ref));
  }
}

template< class T3 >
inline
OmsObjByClsIter<T3>::operator bool () const
{ 
    return OmsObjByClsIterBase::operator bool ();
}

template< class T3 >
inline
const T3* OmsObjByClsIter<T3>::operator() ( OmsHandle&h ) const 
{ 
    return REINTERPRET_CAST(T3*, OmsObjByClsIterBase::omsDeRef(false, false));
}

template< class T3 >
inline
T3* OmsObjByClsIter<T3>::operator() ( OmsHandle&h, bool do_lock ) const 
{ 
    return REINTERPRET_CAST(T3*, OmsObjByClsIterBase::omsDeRef(true, do_lock));
}

template< class T3 >
inline
const T3* OmsObjByClsIter<T3>::omsDeRef () const 
{ 
    return REINTERPRET_CAST(T3*, OmsObjByClsIterBase::omsDeRef(false,false));
}

template< class T3 >
inline
T3* OmsObjByClsIter<T3>::omsDeRefForUpd ( bool do_lock ) const 
{ 
    return REINTERPRET_CAST(T3*, OmsObjByClsIterBase::omsDeRef(true,do_lock));
}

template< class T3 >
inline
void OmsObjByClsIter<T3>::operator++ () 
{
    OmsObjByClsIterBase::operator++();
}

template< class T3 >
inline
const OmsOid<T3> OmsObjByClsIter<T3>::operator() () const 
{
    return *REINTERPRET_CAST(const OmsOid<T3>*, &OmsObjByClsIterBase::operator()());
}

template< class T >
inline
OmsKeyRangeIter<T>::operator bool () const
{
  return OmsObjByKeyIterBase::operator bool ();
}

template< class T >
inline
const T* OmsKeyRangeIter<T>::omsDeRef () const
{
  return REINTERPRET_CAST(T*, OmsObjByKeyIterBase::omsDeRef(false, false));
}

template< class T >
inline
T* OmsKeyRangeIter<T>::omsDeRefForUpd (bool do_lock) const
{
  return REINTERPRET_CAST(T*, OmsObjByKeyIterBase::omsDeRef(true, do_lock));
}

template< class T >
inline
void OmsKeyRangeIter<T>::operator++ ()
{
  OmsObjByKeyIterBase::operator++();
}

template LIVECACHE_OBJECT_TMPLPAR(class T2, ClassID guid)
inline 
const OmsOid<T2> OmsObject LIVECACHE_OBJECT_TMPLPAR(T2, guid)::omsGetOid() const 
{ 
    return *(OmsOid<T2>*)(&omsOid());
}

template LIVECACHE_OBJECT_TMPLPAR(class T2, ClassID guid)
inline
bool OmsObject LIVECACHE_OBJECT_TMPLPAR(T2, guid)::omsHistoryInUse(OmsHandle& h) const
{
  return OmsAbstractObject::omsHistoryInUse(h);
}

template LIVECACHE_OBJECT_TMPLPAR(class T2, ClassID guid)
inline 
T2* OmsObject LIVECACHE_OBJECT_TMPLPAR(T2, guid)::omsForUpdPtr(OmsHandle& h, bool doLock) const
{
  return (T2*) OmsAbstractObject::omsForUpdPtr(h, doLock); 
}

template LIVECACHE_OBJECT_TMPLPAR(class T2, ClassID guid)
inline 
void* OmsObject LIVECACHE_OBJECT_TMPLPAR(T2, guid)::operator new ( size_t sz, OmsHandle&ah, OmsSchemaHandle sh, OmsContainerNo cno )
{
    return (void*) OmsAbstractObject::omsNewObject( ah, guid, sh, cno );
}

template LIVECACHE_OBJECT_TMPLPAR(class T2, ClassID guid)
inline 
void* OmsObject LIVECACHE_OBJECT_TMPLPAR(T2, guid)::operator new ( size_t sz, OmsHandle& ah, OmsRegEnum e ) 
{
    return (void*) OmsAbstractObject::omsNewRegistryObject( sz, ah, guid ); 
}

template LIVECACHE_OBJECT_TMPLPAR(class T2, ClassID guid)
inline 
void* OmsObject LIVECACHE_OBJECT_TMPLPAR(T2, guid)::operator new ( size_t sz, OmsHandle& ah, OmsRegEnum e, size_t rsz ) 
{
    return (void*) OmsAbstractObject::omsNewRegistryObject( rsz, ah, guid ); 
}

template LIVECACHE_KEYEDOBJECT_TMPLPAR(class Obj, class Key, ClassID guid)
inline
void OmsKeyedObject LIVECACHE_KEYEDOBJECT_TMPLPAR(Obj, Key, guid)::omsGetKey( OmsHandle&ah, Key& k ) const
{ 
    omsKey ( ah, REINTERPRET_CAST(unsigned char*, &k) );
}

inline bool operator== (const OmsTypePacked_8_3& l, const OmsTypePacked_8_3& r) 
{
  return ((l.m_packed[0] == r.m_packed[0]) &&
          (l.m_packed[1] == r.m_packed[1]) && 
          (l.m_packed[2] == r.m_packed[2]) && 
          (l.m_packed[3] == r.m_packed[3]) &&
          (l.m_packed[4] == r.m_packed[4]) &&
          (l.m_packed[5] == r.m_packed[5]) &&
          (l.m_packed[6] == r.m_packed[6]) &&
          (l.m_packed[7] == r.m_packed[7])); 
}

inline bool operator!= (const OmsTypePacked_8_3& l, const OmsTypePacked_8_3& r) 
{
  return ((l.m_packed[0] != r.m_packed[0]) ||
          (l.m_packed[1] != r.m_packed[1]) || 
          (l.m_packed[2] != r.m_packed[2]) || 
          (l.m_packed[3] != r.m_packed[3]) ||
          (l.m_packed[4] != r.m_packed[4]) ||
          (l.m_packed[5] != r.m_packed[5]) ||
          (l.m_packed[6] != r.m_packed[6]) ||
          (l.m_packed[7] != r.m_packed[7])); 
}

inline void OmsHandle::omsCheckCancelFlag() const // PTS 1110812
{
  if (*m_pCancelFlag)
  {
    this->omsThrowCancelledError();
  }
}


#include "Oms/OMS_ArrayObject.hpp"

# endif

