/*!================================================================
 module:        hco12.h
 
 responsible:   ThomasA
 
 special area:  OMS
 
 description:   Implementation of the OMS Context   
 
 see:
 
 copyright:     SAP AG 1998 


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 ===================================================================*/

#ifndef HCO12_H
#define HCO12_H


/*****************************************************
 *     RefType: C++
 *****************************************************/


/*****************************************************
 *     I N C L U D E S                               *
 *****************************************************/

#include "gco11ISessionLockObjects.h"
#include "hco13.h"
#include "hco16.h"
#include "ggg92.h"
#include "ggg250.h"
#include "ggg251.h"
#include "gco001.h" 
#include "liveCache/LVC_KernelInterface.hpp"
#include "Oms/OMS_Monitor.hpp"
#include "SAPDBCommon/MemoryManagement/SAPDBMem_RawAllocator.hpp"
#include "Oms/OMS_Monitor.hpp"
#include "Oms/OMS_VersionDirRegions.hpp"
#include "Oms/OMS_NewObjCache.hpp"

extern      co16_SharedMemAllocator SharedMemAllocator_co16;
extern int  co10_GetLockIdForVersionDirectory (const OmsVersionId &versionId);
extern bool co10_GetLockMethodForVersionDictionary();


/*****************************************************
 *     P R O T O T Y P E S                           *
 *****************************************************/

enum VarObjLockType_co12 {
  VarObjNoLock,
  VarObjShareLock,
  VarObjExclusiveLock
};

class CUnknownContainerId_co12 : public tgg00_ContainerId {
public :
  CUnknownContainerId_co12() {
    fileObjFileType_gg00().becomes(oftUnknownType_egg00);
  }
  tsp00_Uint4 GetContainerHandle() {
    return fileObjFileNo_gg00();
  }
  bool IsVarObjFile() {
    return (oftVarLenObjFile_egg00 == fileObjFileType_gg00());
  }
};

class OmsSession_co12;

/*!
  class : OmsContext_co12
  description : Implements an OMS Context. A Context is a consistent view togethher 
                with cached Objects belonging to this view.
*/
class OmsContext_co12 {
  
#ifndef USE_SYSTEM_ALLOC_CO13
  SAPDBMem_RawAllocator m_heap; // caution! must be declared prior to m_oidDir, m_containerDir etc to
                                                  // ensure correct sequence of destruction
#endif

public :
   enum LockType {
      NoLock,
      Lock,
      TryLock
   };
   cco13_OidHash            m_oidDir;
   cco13_ClsIdHash          m_containerDir;
   OMS_NewObjCache          m_newObjCache;
   tgg91_TransNo            m_consistentView;
   OmsSession_co12*         m_session;
   OmsContext_co12(OmsSession_co12* session, const OmsVersionId* vid = NULL, const tgg00_OmsVersionContext* vctxt = NULL);
   /*!
     function : CloseVersion()

     description : closes the current open version 
   */
   ~OmsContext_co12();
   inline void                  AssignLcSink(IliveCacheSink* pSink);
   inline bool                  IsBoundToTrans() { return m_boundToTrans; }
   inline void                  CheckDBError(tsp00_Int2 e, const char* msg, const OmsObjectId& oid);
   inline void                  CheckCompatibility(const ClassIDRef, OmsObjectContainerPtr);
   inline void                  CheckIfContainerNotDropped(tco13_ClsIdEntry*);
   inline bool                  CheckOid (const OmsObjectId& oid, const ClassIDRef guid);
   inline bool                  CheckOid (const OmsVarOid& oid);
          void                  CleanContainerDir();
   inline void                  ClearObjCache();
   inline void                  CloseVersion();
   inline tsp00_Int4            CurrentVersionKey() const;
          void                  DeleteSelf();
   inline void                  DecNewObjectsToFlush();
   inline void                  DeleteObjInContext (OmsObjectContainerPtr p, tco13_ClsIdEntry*);
          void                  Dump(IDump_cco02& dumpObj) const;
          void                  DumpObjCache(OmsHandle&);
          void                  EmptyObjCache(tsp00_Uint4 containerHandle = 0xFFFFFFFF);
   inline bool                  ExistsConsistentView();
   inline OmsObjectContainerPtr FindObjInContext (const OmsObjectId* pOid);
   inline OmsObjectContainerPtr FindVarObjInContext (const OmsObjectId& oid, VarObjLockType_co12, unsigned long size, void* buf);
          void                  FlushObj(OmsObjectContainerPtr p);
          void                  FlushDelete (OmsObjectContainerPtr pObj, bool deleteInHash); 
          void                  FlushObjCache(bool emptyObjCache, bool adaptOidHash = true);
          void                  FlushLockUpdObj(OmsObjectContainerPtr p);
          void                  FlushVarObject (OmsObjectContainerPtr p); 
   inline void                  deallocate(void* p);
   inline void                  FreeVarObj(unsigned char* p, size_t sz);
   inline tco13_GuidEntry*      GetClassInfo(const ClassIDRef guid);
   inline tco13_ClsIdEntry*     GetContainerInfo(tsp00_Uint4 containerHandle);
   inline tco13_ClsIdEntry*     GetContainerInfoReg(tsp00_Uint4 containerHandle);
   inline tco13_ClsIdEntry*     GetContainerInfoNoCheck(tsp00_Uint4 containerHandle);
   inline OmsSession_co12*      GetCurrentSession();
   inline tsp00_Int4            GetHashSize() const;
   inline OmsContext_co12*      GetNext() const;
   inline OmsContext_co12*      GetNextUnloaded() const;
   inline OmsObjectContainerPtr GetObj(const OmsObjectId& oid, bool doLock);
   inline OmsObjectContainerPtr GetObj(const ClassIDRef guid, const OmsObjectId& oid, bool doLock);
   inline OmsObjectContainerPtr GetObjFromLiveCacheBase(const OmsObjectId& oid, LockType doLock, bool mayBeVarObject = true);
   inline OmsObjectContainerPtr GetObjFromLiveCacheBase(const ClassIDRef, const OmsObjectId& oid, LockType);
   inline OmsObjectContainerPtr GetObjViaCachedKey(tco13_ClsIdEntry& clsinfo,    // PTS 1117571
	                                               unsigned char* key, bool doLock);
   inline OmsObjectContainerPtr GetObjViaKey(tco13_ClsIdEntry& clsinfo, unsigned char* key, bool doLock);
   const tsp00_Date&            GetCreateDate() const { return m_date; }
   const tsp00_Time&            GetCreateTime() const { return m_time; }
   const tsp00_Date&            GetOpenDate() const { return m_lastOpenDate; }
   const tsp00_Time&            GetOpenTime() const { return m_lastOpenTime; }
   const OmsVersionId&          GetVersionId() { return m_version; }
   const OmsTypeWyde*           GetVersionDesc() const { return m_versionDesc; }  // PTS 1117690
   inline int                   GetVarObjFromLiveCacheBase(const OmsObjectId& oid, bool);
   inline void                  IncNewObjectsToFlush();
   inline void                  IncVersionKey ();
   bool                         IsDropped() const { return m_isDropped; }
   bool                         IsMarked() { return m_marked; } 
   inline bool                  IsVersionOid (const OmsObjectId& oid) const;   // PTS 1117571
   inline bool                  IsVersionOpen();
   inline bool                  IsOpenVersion (const OmsVersionId& versionId);
   inline bool                  IsUnloadable();
   inline bool                  IsUnloaded() const { return (m_pVersionContext != NULL); }
   inline bool                  IsVersion();
   IliveCacheSink*              LcSink() const { return m_currLcSink; }
   inline OmsObjectContainerPtr LoadVarObject (const OmsVarOid& oid, VarObjLockType_co12, tsp00_Int4 objSize, OmsVarObjChunk_co001& chunk, void* buf);
   inline void                  LockObj(const OmsObjectId&oid);
   inline bool                  TryLockObj(const OmsObjectId&oid);
   inline void                  LockResult (tsp00_Int2 DBError, OmsObjectContainerPtr p, const OmsObjectId& oid, const char* msg); 
   inline bool                  TryLockResult (tsp00_Int2 DBError, OmsObjectContainerPtr p, const OmsObjectId& oid, const char* msg); 
   inline void*                 allocate(size_t sz);
   void                         MarkBoundToTrans() { m_boundToTrans = true; }
   void                         MarkDropped() { m_isDropped = true; }
   void                         MarkNotBoundToTrans(bool callFromDestructor);
   void                         MarkVersion() { m_marked = true; }
   void                         NewConsistentView();
   void                         OpenVersion(OmsSession_co12* session);
   inline void                  PutObjectIntoContext (OmsObjectContainerPtr p, tsp00_Uint4 ContainerHandle);
   inline bool                  PutObjectIntoContextReg (OmsObjectContainerPtr p, tsp00_Uint4 ContainerHandle);
   inline void                  RenameVersion(const OmsVersionId& OldId, const OmsVersionId& NewId);
          void                  ReduceHeapUsage(); /* PTS 1109340 */
   inline void                  ResetNewObjectsToFlush();
   inline void                  ResetConsistentView ();
          void                  ResetLcSink();
          void                  ResetVersion(OmsSession_co12&); /* PTS 1115134 */
   inline void                  RemoveUnlockedObjFromCache();
   inline void                  SetConsistentView(const tgg91_TransNo&);
   void                         SetVersionDesc(const OmsTypeWyde*);       // PTS 1117690
   inline void                  SetNext(OmsContext_co12*);
   inline void                  SetNextUnloaded(OmsContext_co12*);
          void                  UnLoad();
          void                  VersionClearObjCache();
   inline unsigned char*        VersionContext();
   inline OmsObjectId           VersionNewOid();
   inline void*                 operator new(size_t sz);
#if defined(PLACEMENT_DELETE_MEO00)
   inline void                  operator delete (void* p);
#endif
   inline tsp00_8ByteCounter    HeapUsed();
          tgg00_FileId&         VersionFile()    { return m_versionContext.ovc_tree; }
          tgg00_FileId&         VersionInvFile() { return m_versionContext.ovc_index_tree; }
private :
  OmsVersionId             m_version;
  tgg00_OmsVersionContext  m_versionContext;
  tgg00_OmsVersionContext* m_pVersionContext;
  private :
    OmsContext_co12*         m_next;
    OmsContext_co12*         m_nextUnloaded;
    tsp00_Uint4              m_lastDropId;
    bool                     m_isOpen;
    bool                     m_isDropped;
    bool                     m_isVersion;
    bool                     m_boundToTrans;
    bool                     m_marked;
    tsp00_Date               m_date;
    tsp00_Time               m_time;
    tsp00_Date               m_lastOpenDate;
    tsp00_Time               m_lastOpenTime;
    IliveCacheSink*          m_currLcSink;
    IliveCacheSink*          m_sysLcSink; 
    int                      m_cntNewObjectsToFlush;    // PTS 1115545
    OmsTypeWyde*             m_versionDesc;             // PTS 1117690  
};
/*! endclass: OmsContext_co12 */

/*!
  class : OmsSession_co12
  description : Implements an OMS Session representing a kernel session in the OMS.
*/

typedef OmsContext_co12* OmsContextPtr_co12;

class OmsSession_co12 {   
public :
#define                        RESCHEDULE_DISTANCE 1000 // call dispatcher every 1000 derefs
  
  class MethodCallEplilogScope
  {
  public :
    MethodCallEplilogScope(OmsSession_co12& session);
    ~MethodCallEplilogScope();
  private :
    OmsSession_co12& m_session;
  };

  friend class OmsSession_co12::MethodCallEplilogScope;

                               OmsSession_co12(
                                 IliveCacheSink* lcSink,
                                 int cntRegions, 
                                 pasbool* pToCancel,
                                 tsp00_TaskId taskId);
                               ~OmsSession_co12();
  inline void* operator new(size_t sz);
#if defined(PLACEMENT_DELETE_MEO00)
  inline void  operator delete(void* p);
#endif
  inline void                  ActivateDefaultContext();
         void                  AssignLcSink(IliveCacheSink* pSink); /* PTS 1109371 */
  inline void                  AddContainerInfo(tco13_ClsIdEntry* classInfo);
  inline void                  AssertNotReadOnly(const char* callerMsg);
  const  OmsObjectId&          CastOid(const ClassIDRef castToGuid, const OmsObjectId& oid);
         void                  ChangedConsistentView();
#if defined(OMSTST)
  inline void                  CheckCancelCountDown();    // PTS 1112358
  inline void                  CheckExceptionCountDown(); // PTS 1112358
#endif
         void                  CleanAfterDropSchema();
         void                  ClearDefaultContext();
  inline void                  ClearFreeLists();
  inline void                  CloseVersion();
  inline void                  CommitSubtrans(int requiredLevel);
  inline void                  ClearTransVersions();
  inline void                  CreateDefaultContext();
         void                  CreateVersion(const OmsVersionId& versionId, const OmsTypeWyde* desc);  // PTS 1117690
  inline int                   CurrentSubtransLevel() const;
  inline OmsContext_co12*      CurrentContext() const;
         void                  CurrentMethodCallEpilog(tsp00_Int4 runTime);
  inline void                  DecSubtransLevel();
         void                  DeleteAll(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo);
  inline void                  DeleteSession();
         void                  DeleteVarObject(const OmsVarOid& oid);
  inline OmsAbstractObject*    DeRef (const OmsObjectId& oid, const ClassIDRef guid, bool forUpd, bool doLock);
  inline OmsAbstractObject*    DeRefViaKey (unsigned char* key, const ClassIDRef, bool forUpd, bool do_lock, OmsSchemaHandle, OmsContainerNo);
  const  void*                 DeRefVarObject(const OmsVarOid& oid);
         void                  DropVersion(OmsContext_co12* context);
         void                  DropVersionEpilog(OmsContext_co12* context); 
         void                  DropVersionProlog(OmsContext_co12* context);
         void                  Dump(IDump_cco02& dumpObj) const;
  inline void                  ExecuteSubtransCommit(int requiredLevel);
  inline void                  deallocate(void* p);
  inline bool                  ForceReadOnly();
  inline void                  FreeBeforeImages();
  inline void                  FreeStackHeap();
  inline OmsAbstractObject*    ForUpdPtr(OmsObjectContainerPtr pObj);
  inline OmsCallbackInterface* GetCallbackInterface() const;
  inline OmsCallbackInterface* GetCallbackInterface(const ClassIDRef guid) const;
  inline tco13_ClsIdEntry*     GetClsInfo(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo);
  inline tco13_ClsIdEntry*     GetClsInfoForReg(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo);
  inline tco13_GuidEntry*      GetClassInfo(const ClassIDRef guid);
  inline tco13_ClsIdEntry*     GetContainerInfo(tsp00_Uint4 containerHandle);
  inline long                  GetStoredErrorCode() const;           // PTS 1122838
  inline OmsObjectContainerPtr GetMemory (tco13_ClsIdEntry&);
  inline OmsObjectContainerPtr GetMemory (tco13_GuidEntry& classInfo);
  inline OmsObjectContainerPtr GetMemoryInVersion (tco13_ClsIdEntry&);
         bool                  GetMethodCallMonitorInfo (OmsMonitorData& info) const;
  inline bool                  GetReadOnly() const { return m_read_only; }
  inline ISessionLockObjects_co11* GetSessionLockObject() const;
  inline tsp00_TaskId          GetTaskId() const; // PTS 1110315
  inline unsigned char*        GetVarObjMemory(size_t sz);
  inline tsp00_Int4            GetVarObjSize(const OmsVarOid& oid);
         bool                  HistoryInUse(const OmsObjectId&);
  inline bool                  InVersion();
  inline void                  IncRefCnt();
  inline void                  IncSubtransLevel ();
  inline void                  IncCacheHit(const OmsObjectId& oid); // PTS 1117571
  inline void                  IncCacheMiss();                      // PTS 1117571
  inline void                  IncDeref();
  inline void                  IncDerefIter_LC(); // PTS 1107819 TA 13/09/2000  
  inline void                  IncDereflC();
  inline void                  IncDerefKey();
  inline void                  IncStore(); 
  inline void                  IncStore_lC();
  inline void                  IncOmsTerminate();
  inline void                  IncDelete (); 
  inline void                  IncDelete_lC(); 
  inline void                  IncDelete_lC(tsp00_Int4 cntDeleted); 
  inline void                  IncLock();
  inline void                  IncLogHop(int cnt); 
  inline void                  IncLogHopIter(int cnt); // PTS 1107819 TA 13/09/2000  
  inline void                  IncExceptions();
  inline void                  IncOutOfDate();
  inline void                  IncOutOfMemory();
  inline void                  IncTimeout(); 
  inline void                  IncSubtransCommit(); 
  inline void                  IncSubtransRollback(); 
  inline void                  IncLoadVarObj(); 
  inline void                  IncLoadVarObjLC(); 
  inline void                  IncStoreVarObj();
  inline void                  IncStoreVarObjLC();
  inline void                  IncReadStreamBuffer();
  inline void                  IncRehash();               // PTS 1119061
  inline void                  IncWriteStreamBuffer();
  inline void                  IncReadStreamRow();
  inline void                  IncWriteStreamRow();
  inline void                  InsertTransVersion(OmsContext_co12* context);
         bool                  IsLocked (const OmsObjectId& oid);
  inline OmsObjectContainerPtr LoadVarObject (const OmsVarOid& oid, VarObjLockType_co12 lockReq, tsp00_Int4 size, void* buf);
  inline void                  MaxHashChainLen (int len); // PTS 1119061
  inline void                  MonitorSubtransLevel(); 
  inline void                  SetCacheSize(tsp00_Uint4 sz); 
  inline void                  IncCreateVersion (); 
  inline void                  IncOpenVersion (); 
  inline void                  IncCloseVersion (); 
  inline void                  IncDropVersion (); 
  inline void                  IncDeleteVarObject (); 
  inline void                  IncDeleteVarObjectLC (); 
  inline void                  IncCntWaitOmsLockObj();
  inline void                  IncWaitOmsLockObjSecs(tsp00_Int4);
  inline bool                  InsertBeforeImage (OmsObjectContainerPtr p);
  inline void                  InsertNewBeforeImage (OmsObjectContainerPtr p, tco13_ClsIdEntry* clsinfo);
  inline bool                  IsDefaultContext(OmsContext_co12* context);
  inline bool                  IsSubtransOpen();
  inline void                  LockObj(OmsObjectContainerPtr p);
  inline bool                  TryLockObj(OmsObjectContainer* p);
  inline void*                 allocate(size_t sz);
  inline void                  MonitorVarObjSize(tsp00_Int4);
  inline void                  MonitorWaitNewConsistentView(tsp00_Int4);
         void                  NewConsistentView(OmsIOidReadIterator& pOids, short timeout, OmsIOidAppender& pErrOids);   // PTS 1115027
         OmsAbstractObject*    NewKeyedObject(const ClassIDRef guid, const unsigned char* key, OmsSchemaHandle Schema, OmsContainerNo ContainerNo);
         OmsAbstractObject*    NewObject(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo);
         const OmsVarOid&      NewVarObject(OmsSchemaHandle Schema, OmsContainerNo ContainerNo);
  inline void                  OpenVersion(OmsContext_co12* context);
         void                  RegContainer(OmsSchemaHandle Schema,
                                 const ClassIDRef   guid,
                                 const char*        ClassName, 
                                 size_t             PersistentSize,
                                 size_t             ObjectSize,
                                 const ClassIDPtr   pBaseClass,
                                 OmsAbstractObject* Obj, 
                                 OmsContainerNo     ContainerNo,
                                 size_t             arrayByteSize = 0);
         void                  RegVarObjectContainer(OmsSchemaHandle Schema, OmsContainerNo cno);
         void                  RegisterCallbackInterface(const ClassIDRef guid, OmsCallbackInterface* pInterface);
#if defined(OMSTST)
  inline bool                  RequiredExceptionThrown() const; // PTS 1112358
#endif
  inline void                  ResetSubtransLevel ();
         void                  ReleaseCallbackInterface();
  inline void                  ReleaseObj(const OmsObjectId& oid);
  inline void                  ReleaseObj(OmsObjectContainerPtr p);
  inline void                  ReleaseObjUnconditional(const OmsObjectId& oid);
         void                  ReleaseVarObject(const OmsVarOid& oid);
  inline void                  Reschedule(); /* PTS 1107849 */  
         void                  RemoveFromTransVersion(OmsContext_co12* context);
  inline void                  RollbackSubtrans(int requiredLevel, bool callFromRollbackTrans);
  inline void                  SetContext(OmsContext_co12* context) {m_context = context;}
         void                  SetCurrentMonitorMethod(tgg00_COMMonitorInfo* p); // PTS 1107731
#if defined(OMSTST)
  inline void                  SetCancelCountDown(int countDown);
  inline void                  SetExceptionCountDown(int countDown);
#endif
         void                  SetReadOnly(bool readOnly);
  inline void                  SetSessionLockObject(ISessionLockObjects_co11* pLockObj);
         void                  SetTimeout(short timeout);
	inline void                  Signal(tsp00_TaskId TaskId, tsp00_Bool ok); // PTS 1110315
  inline void*                 StackHeapMalloc(size_t size);
  inline int                   StartSubtrans();
  inline void                  StoreErrorCode(long errorCode);   // PTS 1122838
  inline void                  StoreVarObject (const OmsVarOid& oid, const void* pVarObj, unsigned long objLength);
         void                  ThrowDBError(tsp00_Int2 e, const char* msg);
         void                  ThrowDBError(tsp00_Int2 e, const char* msg, const OmsObjectId& oid);
         void                  ThrowReturnCode(tsp00_Int2 e, const char* msg);
         void                  Trace( const char*fmt, ... );
         void                  TransEnd();
  inline void                  TransEndEpilog();
         void                  TriggerDump(tsp00_Int4);
  inline void                  UnforceReadOnly(bool);
         void                  UnlockObj (const OmsObjectId& oid);
	       bool                  VersionBoundByMe(OmsContext_co12* context);
         void                  Wait();
  ClassID					m_callbackInterfaceGuid;
  OmsCallbackInterface*     m_callbackInterface;
  IliveCacheSink*           m_lcSink;
  OmsTypeInt64              m_sizeVarobj; 
  OmsTypeInt64              m_cntVarobj;
  tsp00_Int4                m_timeout;
  int                       m_refCnt;
  int                       m_subtrans_lvl;
  bool                      m_stream_io;
  bool                      m_read_only;
  
  bool*                     m_critical_section_in_use;
  ISessionLockObjects_co11* m_lockObjects;
  OmsContext_co12*          m_context;
  OmsContext_co12*          m_defaultContext;
  cco13_BeforeImages        m_beforeImages;
 
  OmsVarObjChunk_co001     m_currVarObjChunk;
  cgg251DCUList<OmsContext_co12*,OmsSession_co12> m_versionsBoundToTrans;
  cgg251DCList<OmsHandle*,OmsSession_co12>        m_handleList;
  private :
    bool                     m_allowReadOnly;
    bool                     m_inMethodCallEpilog;
    OMS_Monitor              m_monitor;
    tgg00_COMMonitorInfo*    m_monitor_curr;
#ifndef USE_SYSTEM_ALLOC_CO13
    co16_OmsStackHeap        m_stackHeap;
#endif
    int                      m_rescheduleDistance;
    tsp00_8ByteCounter       m_heapAtMethodBegin;
    pasbool&                 m_toCancel;
    long                     m_errorCode;               // PTS 1122838
    tsp00_TaskId             m_taskId;                  // PTS 1110315
    int                      m_exceptionCountDown;      // PTS 1112358
    int                      m_cancelCountDown;         // PTS 1112358
    bool                     m_requiredExceptionThrown; // PTS 1112358
};
/*! endclass: OmsSession_co12 */


class OmsVarObjInfoNode_co12 {
public :
  OmsVarObjInfoNode_co12() 
    : m_vobjOid()
    , m_vobjSeq()
    , m_refCnt(1)
    , m_vobjNext(NULL)
  {
  }

  OmsVarObjInfoNode_co12(const OmsObjectId& oid, const tgg91_PageRef& objSeq) 
    : m_vobjOid(oid)
    , m_vobjSeq(objSeq)
    , m_refCnt(1)
    , m_vobjNext(NULL)
  {
  }

  static void* operator new(size_t size, OmsContext_co12& c) {
    if (c.IsVersion()) {
      return c.allocate(sizeof(OmsVarObjInfoNode_co12));
    }
    else {
      return c.m_session->StackHeapMalloc(sizeof(OmsVarObjInfoNode_co12));
    }
  }
#if defined(PLACEMENT_DELETE_MEO00)
  void operator delete(void* p, OmsContext_co12& c) {
#ifndef USE_SYSTEM_ALLOC_CO13
    if (c.IsVersion()) {
#endif
      c.deallocate(p);
#ifndef USE_SYSTEM_ALLOC_CO13
    }
#endif
  }
#endif

  void deleteSelf (OmsContext_co12& c)
  {
#ifndef USE_SYSTEM_ALLOC_CO13
    if (c.IsVersion())
    {
#endif
      c.deallocate(this);
#ifndef USE_SYSTEM_ALLOC_CO13
    }
#endif
  }
  OmsObjectId             m_vobjOid;
  tgg91_PageRef           m_vobjSeq;
  tsp00_Int2              m_refCnt;
  OmsVarObjInfoNode_co12* m_vobjNext;
private :
};

class OmsVarObjInfo_co12 : public OmsAbstractObject {  
public :
  OmsVarObjInfo_co12() : m_vobjSize(0), m_refCnt(0), m_pvobj(NULL), m_vobjInfo() {}
  
  OmsVarObjInfo_co12(const OmsObjectId& oid, tsp00_Uint4 size, const tgg91_PageRef& objSeq) : 
  m_vobjSize(size), m_refCnt(0), m_pvobj(NULL), m_vobjInfo(oid, objSeq) {}
  
  void freeVarObjData(OmsContext_co12* context) 
  {
    context->FreeVarObj(m_pvobj, m_vobjSize);
    m_pvobj = NULL;
  }

  void freeVarObj (OmsContext_co12* context) {
    context->FreeVarObj(m_pvobj, m_vobjSize);
#ifndef USE_SYSTEM_ALLOC_CO13
    if (context->IsVersion())
    {
#endif
      OmsVarObjInfoNode_co12** prev = &m_vobjInfo.m_vobjNext;
      OmsVarObjInfoNode_co12* next  = m_vobjInfo.m_vobjNext;
      while (NULL != next)
      {
        OmsVarObjInfoNode_co12* toFree = next;
        --toFree->m_refCnt;
        next = next->m_vobjNext;
        if (0 == toFree->m_refCnt)
        {
          *prev = next;
          context->deallocate(toFree);
        }
        else
        {
          prev = &toFree->m_vobjNext;
        }
      }
#ifndef USE_SYSTEM_ALLOC_CO13
    }
#endif
    m_pvobj = NULL;
  }

  void incRefCnt()
  {
    OmsVarObjInfoNode_co12* curr = m_vobjInfo.m_vobjNext;
    while (NULL != curr)
    {
      ++curr->m_refCnt;
      curr = curr->m_vobjNext;
    }
  }
  
  void lock(VarObjLockType_co12 lockReq) {
    switch (lockReq) {
    case VarObjNoLock :
      return;
    case VarObjShareLock :
      if (m_refCnt >= 0) {
        ++m_refCnt;
        return;
      }
      break;
    case VarObjExclusiveLock :
      if (0 == m_refCnt) {
        m_refCnt = -1;
        return;
      }
    }
    throw DbpError (DbpError::DB_ERROR, e_duplicate_object);
  }
  
  void unlock() {
    if (m_refCnt > 0) {
      /* share lock */
      --m_refCnt;
    }
    else {
      if (m_refCnt < 0) {
        /* exclusive lock */
        m_refCnt = 0;
      }
    }
  }
  
  void* operator new(size_t sz, OmsAbstractObject* pObj) {
    return pObj;
  }
#if defined(PLACEMENT_DELETE_MEO00)
  void operator delete(void* p, OmsAbstractObject* pObj) {}
#endif
  
  tsp00_Uint4            m_vobjSize;
  tsp00_Int2             m_refCnt;
public :
  unsigned char*         m_pvobj;
  OmsVarObjInfoNode_co12 m_vobjInfo;
  
};


class CriticalSection_co12 : public CriticalSection_co001 {
public :
  CriticalSection_co12(OmsSession_co12* session,int regionId) : CriticalSection_co001(session->m_lcSink, regionId) {} 
};

/*****************************************************
*     I M P L E M E N T A T I O N                   *
*****************************************************/

inline void OmsContext_co12::AssignLcSink(IliveCacheSink* pSink) {
  m_currLcSink = pSink; 
  m_sysLcSink  = co10_GetLcSink();
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::CheckDBError(tsp00_Int2 e, const char* msg, const OmsObjectId& oid) {
  if (0 != e) {
    m_session->ThrowDBError (e, msg, oid);
  }
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::CheckCompatibility(const ClassIDRef guid, OmsObjectContainerPtr pObj) {
  if (NULL == pObj) {
    return;
  }
  if (!omsIsCompatibleGuid(pObj->GetContainerInfo(this)->GetGuid(), guid)) {
    tco13_ClsIdEntry* pContainerInfo = pObj->GetContainerInfo(this);
    if (NULL != pContainerInfo) {
      if (pContainerInfo->IsDerivedClassOf(guid)) {
        return;
      }
    }
    tco13_GuidEntry* pClassInfo = GetClassInfo(guid);
    if (pClassInfo->IsBaseClass()) {
    }
    m_session->ThrowDBError (e_object_not_found, "OmsContext_co12::CheckCompatibility");
  }
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::CheckIfContainerNotDropped(tco13_ClsIdEntry* pContainerInfo) {
  m_containerDir.CheckIfNotDropped(pContainerInfo);
}

/*----------------------------------------------------------------------*/

inline bool OmsContext_co12::CheckOid (const OmsObjectId& oid, const ClassIDRef guid) {
  
  if (oid) {
    try {
      OmsObjectContainerPtr pObj = this->FindObjInContext (&oid);
      if (NULL != pObj) {
        if (pObj->DeletedFlag()) {
          return false;
        } 
        CheckCompatibility(guid, pObj); // PTS 1121441
      }
      else {
        if ((!this->IsVersion() || !this->IsUnloaded()) && (VERSION_OBJ_PAGE_NO_GG92 == oid.getPno()))
        {
          return false;
        }
        const bool mayBeVarObject = true;
        pObj = GetObjFromLiveCacheBase(oid, OmsContext_co12::NoLock, !mayBeVarObject);
        if (NULL == pObj) {
          return false;
        }
        CheckCompatibility(guid, pObj); // PTS 1121441
      }
    }
    catch (DbpError& e) {
        if ((-102 == e.dbpError()) && co10_IsOmsTestLib())
        {
            throw;
        }
        return false;
    }
  }
  return true;
}

/*----------------------------------------------------------------------*/

inline bool OmsContext_co12::CheckOid (const OmsVarOid& oid)
{
  if (oid) {
    try {
      tco13_ClsIdEntry* pContainerInfo;
      OmsObjectContainerPtr pObj = this->FindObjInContext (&oid);
      if (NULL != pObj) {
        if (pObj->DeletedFlag()) {
          return false;
        } 
        pContainerInfo = pObj->GetContainerInfo(this);
        if (!pContainerInfo->IsVarObject()) {
          return false;
        }
      }
      else {
        if (!this->IsVersion() && (VERSION_OBJ_PAGE_NO_GG92 == oid.getPno()))
        {
          return false;
        }
        pObj = GetObjFromLiveCacheBase(oid, OmsContext_co12::NoLock);
        if (NULL == pObj) {
          return false;
        }
        if (!pObj->VarObjFlag()) {
          return false;
        }
      }
    }
    catch (DbpError& e) {
        if ((-102 == e.dbpError()) && co10_IsOmsTestLib())
        {
            throw;
        }
        return false;
    }
  }
  return true;  
}

/*----------------------------------------------------------------------*/

inline void  OmsContext_co12::ClearObjCache() {
  m_oidDir.Clear();
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::CloseVersion() {
  m_containerDir.ClearFreeLists();
  m_isOpen = false;
}

/*----------------------------------------------------------------------*/

inline tsp00_Int4 OmsContext_co12::CurrentVersionKey() const {
  return m_versionContext.ovc_key;
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::DecNewObjectsToFlush()
{
  --m_cntNewObjectsToFlush;
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::DeleteObjInContext (OmsObjectContainerPtr p, tco13_ClsIdEntry* clsinfo) {
  OMS_DETAIL_TRACE(p->IsNewObject() ? (omsTrContext | omsTrNewObj) : omsTrContext, 
    m_session->m_lcSink, "OmsContext_co12::DeleteObjInContext : " << p->m_oid
    << ", class: " << clsinfo->GetClassInfoPtr()->GetClassName());
  if (p->IsVerNewRegistered())
    m_newObjCache.removeObject(p, this);
    const bool c_updateKeyCache = true;
	m_oidDir.HashDelete(p->m_oid, c_updateKeyCache);

  if (NULL != clsinfo) {
    clsinfo->chainFree(*this, p);
  }
}

/*----------------------------------------------------------------------*/

inline bool OmsContext_co12::ExistsConsistentView() {
  return (!(m_consistentView.gg90IsNil()));
}

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsContext_co12::FindObjInContext (const OmsObjectId* pOid) {
  return m_oidDir.HashFind(pOid);
}

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsContext_co12::FindVarObjInContext (const OmsObjectId& oid, VarObjLockType_co12 lockReq, unsigned long size, void* buf) {
  const char* msg = "OmsContext_co12::FindVarObjInContext";
  OmsObjectContainerPtr po = this->FindObjInContext (&oid);
  if (NULL != po) {
    if (po->DeletedFlag()) {
      m_session->ThrowDBError (e_object_not_found, msg, oid);
    }
    OmsVarObjInfo_co12* pObjInfo = REINTERPRET_CAST (OmsVarObjInfo_co12*, &po->m_pobj);
    pObjInfo->lock(lockReq);
    if (pObjInfo->m_vobjSize > size) {
      m_session->ThrowDBError (e_too_small_packet_size, msg, oid);
    }
    if (NULL != buf) {
      memcpy (buf, pObjInfo->m_pvobj, pObjInfo->m_vobjSize);
    }
    return po;
  }
  return NULL;
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::deallocate(void* p) {
#ifdef USE_SYSTEM_ALLOC_CO13
  delete[] (char*) p;
#else
  m_heap.Deallocate(p);
#endif
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::FreeVarObj(unsigned char* p, size_t sz) {
  this->deallocate (p);
}

/*----------------------------------------------------------------------*/

inline tco13_GuidEntry* OmsContext_co12::GetClassInfo(const ClassIDRef guid) {
  return m_containerDir.GetClassInfo (guid);
}

/*----------------------------------------------------------------------*/

inline tco13_ClsIdEntry* OmsContext_co12::GetContainerInfo(tsp00_Uint4 containerHandle) {
  return m_containerDir.HashFindViaContainerHandle(containerHandle);
}

/*----------------------------------------------------------------------*/

inline tco13_ClsIdEntry* OmsContext_co12::GetContainerInfoReg(tsp00_Uint4 containerHandle) {
  return m_containerDir.HashFindViaContainerHandle(containerHandle, true, true);
}

/*----------------------------------------------------------------------*/

inline tco13_ClsIdEntry* OmsContext_co12::GetContainerInfoNoCheck(tsp00_Uint4 containerHandle) {
  return m_containerDir.HashFindViaContainerHandle(containerHandle, false);
}

/*----------------------------------------------------------------------*/

inline OmsSession_co12* OmsContext_co12::GetCurrentSession() {
  return m_session;
}

/*----------------------------------------------------------------------*/


inline tsp00_Int4 OmsContext_co12::GetHashSize() const
{
  return m_oidDir.GetHashSize();
}

/*----------------------------------------------------------------------*/

inline OmsContext_co12* OmsContext_co12::GetNext() const
{
  return m_next;
}

/*----------------------------------------------------------------------*/

inline OmsContext_co12* OmsContext_co12::GetNextUnloaded() const
{
  return m_nextUnloaded;
}

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsContext_co12::GetObj(const OmsObjectId& oid, bool doLock) {
  m_session->IncDeref();
  OmsObjectContainerPtr pObj = m_oidDir.HashFind(&oid);//this->FindObjInContext(&oid);
  if (NULL != pObj) {
    if (pObj->DeletedFlag()) {
      return NULL; // PTS 1107036
    }
    if ((doLock) && (!pObj->LockedFlag()) && (!this->IsVersion())) {
      this->LockObj(pObj->m_oid);
    }
  }
  else {
    pObj = this->GetObjFromLiveCacheBase(oid, doLock ? OmsContext_co12::Lock : OmsContext_co12::NoLock);
  }
  return pObj;
}


/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsContext_co12::GetObj(const ClassIDRef guid, const OmsObjectId& oid, bool doLock) {
  m_session->IncDeref();
  OmsObjectContainerPtr pObj = this->FindObjInContext(&oid);
  if (NULL != pObj) {
    if (pObj->DeletedFlag()) {
      return NULL; // PTS 1107036
    }
    if ((doLock) && (!pObj->LockedFlag()) && (!this->IsVersion())) {
      this->LockObj(pObj->m_oid);
    }
  }
  else {
    pObj = this->GetObjFromLiveCacheBase(guid, oid, doLock ? OmsContext_co12::Lock : OmsContext_co12::NoLock);
  }
  this->CheckCompatibility(guid, pObj);
  
  return pObj;
}

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsContext_co12::GetObjFromLiveCacheBase(const OmsObjectId& oid,
                                                                      LockType           doLockObj,
                                                                      bool               mayBeVarObject)
                                                                      
{
  const char* msg = "OmsContext_co12::GetObjFromLiveCacheBase_1";
  int              objLen;
  int              objHistReadCount;
  tgg00_BasisError DBError;
  if (this->IsVersion()) {
    doLockObj = OmsContext_co12::NoLock;
  }
  CUnknownContainerId_co12 FileId;
  m_session->m_currVarObjChunk.Clear();
  CHResult_co001 hr = m_currLcSink->GetObj (
    m_consistentView,
    FileId,
    m_pVersionContext,
    *REINTERPRET_CAST(OmsTypeOid*, CONST_CAST(OmsObjectId*, &oid)),
    doLockObj != NoLock,  
    m_session->m_currVarObjChunk.m_seq,
    sizeof(m_session->m_currVarObjChunk.m_container), 
    ((unsigned char*) &m_session->m_currVarObjChunk.m_container), 
    objLen,
    objHistReadCount,
    DBError);
  if (0 == DBError) {
    OmsObjectContainerPtr pObj;
    if (FileId.IsVarObjFile()) {
      if (!mayBeVarObject)
      {
          return NULL;
      }
      m_session->m_currVarObjChunk.m_containerHandle = FileId.GetContainerHandle();
      if (objLen <= VAR_OBJ_CHUNK_SIZE) {
        m_session->m_currVarObjChunk.m_container.m_vobjSize = objLen;
      }
      pObj = this->LoadVarObject (*REINTERPRET_CAST(const OmsVarOid*, &oid), VarObjNoLock, 
        m_session->m_currVarObjChunk.m_container.m_vobjSize, m_session->m_currVarObjChunk, NULL);
    }
    else {
      tco13_ClsIdEntry* containerInfo = this->GetContainerInfo(FileId.GetContainerHandle());
      pObj = m_session->GetMemory (*containerInfo);
      memcpy (REINTERPRET_CAST(unsigned char*, &pObj->m_pobj) + sizeof(void*), &m_session->m_currVarObjChunk.m_container,
        containerInfo->GetPersistentSize());
      pObj->m_oid    = oid;
      pObj->m_objseq = m_session->m_currVarObjChunk.m_seq;
      PutObjectIntoContext(pObj, FileId.GetContainerHandle());
    }
    if (doLockObj) {
      pObj->MarkLocked();
    }
    return pObj;
  }
  else {
    if (e_object_not_found != DBError) {
      switch (doLockObj) {
      case OmsContext_co12::Lock:
        LockResult (DBError, NULL, oid, msg);
        break;
      case OmsContext_co12::TryLock:
        if (!TryLockResult (DBError, NULL, oid, msg)) {
          // retry without the lock
          return GetObjFromLiveCacheBase(oid, NoLock, mayBeVarObject);
        }
        break;
      }
      this->CheckDBError(DBError, msg, oid);
    }
    return NULL;
  }
}

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsContext_co12::GetObjFromLiveCacheBase(const ClassIDRef   guid, 
                                                                      const OmsObjectId& oid,
                                                                      LockType           doLockObj)
                                                                      //bool               addIntoObjCache) 
{
  int              dummyLen;
  int              objHistReadCount;
  tgg00_BasisError DBError;
  const char* msg = "OmsContext_co12::GetObjFromLiveCacheBase_2";
  
  m_session->IncDereflC();
  tco13_GuidEntry* pClassInfo = this->GetClassInfo(guid);
  if (pClassInfo->IsBaseClass()) {
    // oid may identifiy an object of a derived class, i.e. size is unknown
    return this->GetObjFromLiveCacheBase(oid, doLockObj);
  }
  else {
    OmsObjectContainerPtr pObj = m_session->GetMemory (*pClassInfo);
    if (this->IsVersion()) {
      doLockObj = OmsContext_co12::NoLock;;
    }
    CUnknownContainerId_co12 FileId;
    CHResult_co001 hr = m_currLcSink->GetObj (
      m_consistentView,
      FileId,
      m_pVersionContext,
      *REINTERPRET_CAST(OmsTypeOid*, CONST_CAST(OmsObjectId*, &oid)),
      doLockObj != NoLock,  
      pObj->m_objseq,
      pClassInfo->GetObjectSize() - OmsObjectContainer::headerSize() - sizeof(void*), 
      ((unsigned char*) &pObj->m_pobj) + sizeof(void*), 
      dummyLen,
      objHistReadCount,
      DBError);
    if (0 != DBError) {
      pClassInfo->ChainFree(*this, pObj); 
      if (e_object_not_found != DBError) {
        switch (doLockObj) {
        case OmsContext_co12::Lock:
          LockResult(DBError, pObj, oid, msg);
          break;
        case OmsContext_co12::TryLock:
          if (!TryLockResult(DBError, pObj, oid, msg)) {
            // retry without the lock
            return GetObjFromLiveCacheBase(guid, oid, NoLock);
          }
          break;
        }
        this->CheckDBError(DBError, msg, oid);
      }
      return NULL;
    }
    pObj->m_oid = oid;
    if (doLockObj != OmsContext_co12::NoLock) {
      pObj->MarkLocked();
    }
    //if (addIntoObjCache) {
    PutObjectIntoContext(pObj, FileId.GetContainerHandle());
    //};
    m_session->IncLogHop(objHistReadCount);
    return pObj;
  }
}


/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsContext_co12::GetObjViaKey(tco13_ClsIdEntry& ContainerInfo, unsigned char* key, bool doLock) {
  
  const char* msg = "OmsContext_co12::GetObjViaKey";

  m_session->IncDerefKey();
  bool                isVersionKey = true;
  int                 ObjHistReadCount;
  tgg00_BasisError    DBError;
  OmsObjectContainer* obj = m_session->GetMemory(ContainerInfo);
#ifdef LIVECACHE_COMPATAPO30
  unsigned char* pBinaryKey = key;
#else
  // transform key into binary representation
  unsigned char* pBinaryKey = ContainerInfo.GetKeyPtr(obj);
  // introduce pAbstractObject, without this code call will not be virtual on some platforms
  OmsAbstractObject* pAbstractObject = &obj->m_pobj;
  pAbstractObject->omsKeyToBinary(key, pBinaryKey);
#endif
  if (this->IsVersion()) {
    doLock = false;
  }
  CHResult_co001 hr = m_currLcSink->GetObjWithKey (
    (unsigned char*) &m_consistentView,
    (unsigned char*) &ContainerInfo.GetFileId(),
    VersionContext(),
    (*ContainerInfo.GetInvColDesc().colHashList_gg00)[0].elen_var(),
    pBinaryKey, 
    doLock, 
    ContainerInfo.GetPersistentSize(),
    ((unsigned char*) &obj->m_pobj) + sizeof(void*),
    REINTERPRET_CAST(OmsTypeOid*, &obj->m_oid),
    (unsigned char*) &obj->m_objseq,
    &ObjHistReadCount,
    &DBError);
  m_session->IncLogHop(ObjHistReadCount);  
  OmsObjectContainerPtr found = NULL;
  if ( DBError != 0 ) {
    if (e_hash_key_not_found != DBError) {
      ContainerInfo.chainFree(*this, obj);
      this->LockResult (DBError, NULL, OmsObjectId(), msg);
    }
    else {
      if (this->IsVersion()) {
        found = ContainerInfo.VersionFindKey(pBinaryKey);
        if (NULL != found) {
            isVersionKey = true;
            DBError = 0;
        }
      }
      ContainerInfo.chainFree(*this, obj);
    }
  }
  if (0 == DBError) {
    if (NULL != found) {
        if (found->DeletedFlag()) 
        {
            if (isVersionKey 
                || 
                ((found = ContainerInfo.VersionFindKey(pBinaryKey)) && (found->DeletedFlag()))
                )
                {
                    return NULL;
                }
        }
      obj = found;
    }
    else {
      if (doLock) {
        this->LockResult (DBError, obj, OmsObjectId(), msg);
      }
      found = this->FindObjInContext (&obj->m_oid);
      if (NULL == found) {
        this->PutObjectIntoContext (obj, ContainerInfo.GetContainerHandle());
      }
      else {
        found->m_objseq = obj->m_objseq;
        if (doLock) {
          found->MarkLocked();
        }
        ContainerInfo.chainFree(*this, obj);
        if (found->DeletedFlag()) {
            if (this->IsVersion()) 
            {
                found = ContainerInfo.VersionFindKey(pBinaryKey);
                if ((NULL != found) && !found->DeletedFlag()) {
                    return found;
                }
            }
          return NULL;
        }
        obj = found;
      }
    }  
    return obj;
  }
  return NULL;
}

//----------------------------------------------------------------------
// Function: Reading object with the specified key and if it does not
//   already exist in the current context then store it in the context,
//   too. In contrast to 'GetObjViaKey' first the m_index tree is checked, 
//   whether the object does already exist in the current context. 
//   If so no access to the kernel is needed
//----------------------------------------------------------------------
inline OmsObjectContainerPtr 
OmsContext_co12::GetObjViaCachedKey(tco13_ClsIdEntry& ContainerInfo, 
																		unsigned char* key, 
																		bool doLock) {
	
	const char* msg = "OmsContext_co12::GetObjViaCachedKey";
	
	int                 ObjHistReadCount;    
	tgg00_BasisError    DBError;
	 
	
	m_session->IncDerefKey();

	OmsObjectContainer* obj = m_session->GetMemory(ContainerInfo);
	
#ifdef LIVECACHE_COMPATAPO30
  unsigned char* pBinaryKey = key;
#else
	// transform key into binary representation
	unsigned char* pBinaryKey = ContainerInfo.GetKeyPtr(obj);
	
	// introduce pAbstractObject, without this code call will not be virtual 
	// on some platforms
	OmsAbstractObject* pAbstractObject = &obj->m_pobj;
	pAbstractObject->omsKeyToBinary(key, pBinaryKey);
#endif
	
	// ********** Try to read from cache **********
	OmsObjectContainerPtr found = ContainerInfo.VersionFindKey(pBinaryKey);
  if (found) {
    m_session->IncCacheHit(found->m_oid);

		// Release memory
		ContainerInfo.chainFree(*this, obj);

    if (doLock && !found->LockedFlag() && !IsVersionOid(found->m_oid)) {
      // Get lock if requested. As the object has been read from the kernel
      // without aquireing a lock, the kernel must be informed about the lock, too.
      m_session->LockObj(found); 
			found->MarkLocked();
    }
		
		// If object is not marked as deleted, then return cached Entry.
		return found->DeletedFlag() ? NULL : found;
	}

  // ********** Check whether key is known as a cache miss **********
  else if (ContainerInfo.IsCacheMiss(pBinaryKey)) {
    m_session->IncCacheMiss();		
    
    // Release memory
		ContainerInfo.chainFree(*this, obj);

    return NULL;
  }
  // ********** Get object from kernel **********
	else {

		CHResult_co001 hr = m_currLcSink->GetObjWithKey (
			(unsigned char*) &m_consistentView,
			(unsigned char*) &ContainerInfo.GetFileId(),
			VersionContext(),
			(*ContainerInfo.GetInvColDesc().colHashList_gg00)[0].elen_var(),
			pBinaryKey, 
			doLock && !IsVersion(),  // lock only if not in version 
			ContainerInfo.GetPersistentSize(),
			((unsigned char*) &obj->m_pobj) + sizeof(void*),
			REINTERPRET_CAST(OmsTypeOid*, &obj->m_oid),
			(unsigned char*) &obj->m_objseq,
			&ObjHistReadCount,
			&DBError);
		
		m_session->IncLogHop(ObjHistReadCount);  
		
		if (DBError) {
			// Read from kernel was not successful 

			// Release Memory
			ContainerInfo.chainFree(*this, obj);


      if (DBError == e_hash_key_not_found) {
        // Object does not exist neither in the cache nor in the kernel
        // => Insert Object into the cache miss structure
        ContainerInfo.InsertCacheMiss(pBinaryKey, this);   
      }
      else {
			  // ONLY Errorhandling as object==NULL
		  	LockResult (DBError, NULL, OmsObjectId(), msg);    
      }

			return NULL;
		}
		else {
		  // Read from kernel was successful

			found = FindObjInContext (&obj->m_oid);
			if (!found) {

        // Insert entry into cached key structure 
        ContainerInfo.VersionAddKey(obj);    

				PutObjectIntoContext (obj, ContainerInfo.GetContainerHandle()); 

			  if (doLock) {
				  LockResult (DBError, obj, OmsObjectId(), msg);
				}
			
			  return obj;
			}
			else {
				// Object is already in the context without being registered in
				// the cache structure. This can either happen if object was not read
        // via the key or if the caching was switched on after objects have 
        // already been loaded into the context

				found->m_objseq = obj->m_objseq;
				ContainerInfo.chainFree(*this, obj);

				if (doLock && !found->LockedFlag()) {
					found->MarkLocked();
				}

		    // Insert info for cached keys 
		    ContainerInfo.VersionAddKey(found);   
				 
				return found->DeletedFlag() ? NULL : found;
			}
		}
	}	
}

	

/*----------------------------------------------------------------------*/

inline int OmsContext_co12::GetVarObjFromLiveCacheBase(const OmsObjectId& oid, bool doLock) {
  
  int                 objLen;
  int                 objHistReadCount;
  tgg00_BasisError    DBError;
  CUnknownContainerId_co12 FileId;
  const char* msg = "OmsContext_co12::GetVarObjFromLiveCacheBase";
  m_session->IncLoadVarObjLC();
  CHResult_co001 hr = m_currLcSink->GetObj (
    m_consistentView,
    FileId,
    m_pVersionContext,
    *REINTERPRET_CAST(OmsTypeOid*, CONST_CAST(OmsObjectId*, &oid)),
    doLock, 
    m_session->m_currVarObjChunk.m_seq, 
    sizeof(m_session->m_currVarObjChunk.m_container),
    (unsigned char*) &m_session->m_currVarObjChunk.m_container,
    objLen, objHistReadCount, DBError);
  if ( DBError != 0 ) 
  { 
    if (e_object_not_found != DBError) 
    {
      if (doLock) 
      {
        LockResult (DBError, NULL, oid, msg);
      }
    }
    this->CheckDBError(DBError, msg, oid);
    return -1;
  }
  if (!FileId.IsVarObjFile()) {
    m_session->ThrowDBError (e_object_not_found, msg, oid);
  }
  m_session->m_currVarObjChunk.m_containerHandle = FileId.GetContainerHandle();
  m_session->IncLogHop(objHistReadCount);
  m_session->m_currVarObjChunk.m_oid = oid;
  if (objLen <= VAR_OBJ_CHUNK_SIZE) {
    m_session->m_currVarObjChunk.m_container.m_vobjSize = objLen;
  }
  return m_session->m_currVarObjChunk.m_container.m_vobjSize;
}

/*----------------------------------------------------------------------*/

inline tsp00_8ByteCounter OmsContext_co12::HeapUsed() {
#ifdef USE_SYSTEM_ALLOC_CO13
  return 0;
#else
  SAPDB_ULong BytesUsed;
  SAPDB_ULong BytesControlled;
  m_heap.CalcStatisticsConst(BytesUsed,BytesControlled); 
  return BytesUsed;
#endif
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::IncNewObjectsToFlush()
{
  ++m_cntNewObjectsToFlush;
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::IncVersionKey () {
  ++m_versionContext.ovc_key;
}

/*----------------------------------------------------------------------*/

inline bool OmsContext_co12::IsOpenVersion (const OmsVersionId& versionId) {
  return (0 == memcmp (versionId, m_version, sizeof(versionId)));
}

/*----------------------------------------------------------------------*/

inline bool OmsContext_co12::IsUnloadable() {
  return ((!m_isOpen) && (!m_boundToTrans));
}

/*----------------------------------------------------------------------*/

inline bool OmsContext_co12::IsVersion() {
  return m_isVersion;
}

/*----------------------------------------------------------------------*/

inline bool OmsContext_co12::IsVersionOid (const OmsObjectId& oid) const {
  return (oid.getPno() == VERSION_OBJ_PAGE_NO_GG92);
}

/*----------------------------------------------------------------------*/

inline bool OmsContext_co12::IsVersionOpen() {
	return m_isOpen;
}

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsContext_co12::LoadVarObject (const OmsVarOid& oid, VarObjLockType_co12 lockReq, 
                                                             tsp00_Int4 objSize, OmsVarObjChunk_co001& chunk, void* buf) {
  OmsVarObjInfo_co12*     pObjInfo;
  unsigned char*          p;
  tsp00_Int2              DBError;
  OmsVarObjInfoNode_co12* pInfoNode;
  OmsObjectId             nextOid;
  int                     objLen;
  int                     ObjHistReadCount;

  /* first chunk of var object ist expected in parameter chunk ! */

  const char* msg = "OmsContext_co12::LoadVarObject";
  
  if (objSize < 0)
  {
      DbpBase b(m_currLcSink);
      b.dbpOpError("negative length in LoadVarObject %d", objSize);
      m_session->ThrowDBError(e_invalid, msg);
  }
  CUnknownContainerId_co12   FileId;
  tco13_GuidEntry* pClassInfo   = this->GetClassInfo(VAR_OBJ_GUID_GCO10);
  OmsObjectContainerPtr po      = m_session->GetMemory(*pClassInfo);
  tsp00_Int4            moveLen = co001_Min (objSize, VAR_OBJ_CHUNK_SIZE);
  po->m_oid     = oid;
  po->m_objseq  = chunk.m_seq;
  po->MarkVarObj();
  pObjInfo = new(&po->m_pobj) OmsVarObjInfo_co12(oid, objSize, chunk.m_seq); 
  pObjInfo->lock(lockReq);
  pObjInfo->m_pvobj               = m_session->GetVarObjMemory(objSize);
  pObjInfo->m_vobjInfo.m_vobjOid  = oid;
  pObjInfo->m_vobjInfo.m_vobjSeq  = chunk.m_seq;
  pObjInfo->m_vobjInfo.m_vobjNext = NULL;
  pInfoNode = &pObjInfo->m_vobjInfo;
  objSize  -= moveLen;
  if (!this->PutObjectIntoContextReg (po, chunk.m_containerHandle)) {
    // auto register varobj container
    GUID			   guid;
    OmsSchemaHandle  schema;
    OmsContainerNo   cno;
    tgg00_BasisError DBError;
    m_currLcSink->GetContainerInfo (chunk.m_containerHandle, guid, schema, cno, DBError);
    if (0 == DBError) {
      m_session->RegVarObjectContainer(schema, cno);
      this->PutObjectIntoContext (po, chunk.m_containerHandle);
    }
    else {
      m_session->ThrowDBError(DBError, msg);
    }
  }
  memcpy (pObjInfo->m_pvobj, &chunk.m_container, moveLen);
  p  = pObjInfo->m_pvobj + moveLen;
  try {
    while (objSize > 0) {
      nextOid = chunk.m_container.m_vobjNext;
      moveLen = co001_Min (objSize, VAR_OBJ_CHUNK_SIZE);
      CHResult_co001 hr = m_currLcSink->GetObj (
        m_consistentView,
        FileId,
        m_pVersionContext,
        *REINTERPRET_CAST(OmsTypeOid*, &nextOid),
        false,  
        chunk.m_seq,
        sizeof (chunk.m_container),
        (unsigned char*) &chunk.m_container, 
        objLen,
        ObjHistReadCount,
        DBError);
      this->CheckDBError(DBError, msg, oid);
      memcpy (p, &chunk.m_container, moveLen);
      p       += moveLen;
      objSize -= moveLen;
      pInfoNode->m_vobjNext = new(*this) OmsVarObjInfoNode_co12 (nextOid, chunk.m_seq);
      pInfoNode = pInfoNode->m_vobjNext;
    }
    if (buf != NULL) {
      memcpy (buf, pObjInfo->m_pvobj, pObjInfo->m_vobjSize);
    }
  } 
  catch (DbpError &e) {
    // remove varobj from context in case of error
    pObjInfo->unlock();
    this->DeleteObjInContext(po, po->GetContainerInfo(this));
    throw e;
  }
  return po;
}

/*----------------------------------------------------------------------*/


inline void OmsContext_co12::LockObj( const OmsObjectId&oid ) {
  
  const char* msg = "OmsContext_co12::LockObj";
    
  m_session->AssertNotReadOnly(msg);
  if (this->IsVersion()) {
    return;
  }
  OmsObjectContainerPtr pObj = this->FindObjInContext(&oid);
  if (NULL != pObj) {
    if (pObj->DeletedFlag()) {
      m_session->ThrowDBError(e_object_not_found, msg, oid);
    }
    if (pObj->LockedFlag()) {
      // Object already locked
      return;
    }
    m_session->LockObj(pObj);
  }
  else {
    pObj = this->GetObjFromLiveCacheBase(oid, OmsContext_co12::Lock); 
    if (NULL == pObj) {
      m_session->ThrowDBError (e_object_not_found, msg, oid);
    }
  }
}

/*----------------------------------------------------------------------*/

inline bool OmsContext_co12::TryLockObj( const OmsObjectId&oid ) 
{
  const char* msg = "OmsContext_co12::LockObj";
    
  m_session->AssertNotReadOnly(msg);
  if (this->IsVersion()) {
    return true;
  }
  OmsObjectContainerPtr pObj = this->FindObjInContext(&oid);
  if (NULL != pObj) {
    if (pObj->DeletedFlag()) {
      m_session->ThrowDBError(e_object_not_found, msg, oid);
    }
    if (pObj->LockedFlag()) {
      // Object already locked
      return true;
    }
    return m_session->TryLockObj(pObj);
  }
  else {
    pObj = this->GetObjFromLiveCacheBase(oid, OmsContext_co12::TryLock); 
    if (NULL == pObj) {
      m_session->ThrowDBError (e_object_not_found, msg, oid);
    }
    return pObj->LockedFlag();
  }
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::LockResult (tsp00_Int2 DBError, OmsObjectContainerPtr p, const OmsObjectId& oid, const char* msg) {
  if (DBError != 0) {
    if (e_object_dirty == DBError) {
      m_session->IncOutOfDate();
      throw OmsOutOfDate(oid);
    }
    else {
      m_session->ThrowDBError (DBError, msg, oid);
    }
    return;
  }
  else {
    if (NULL != p) {
      p->MarkLocked();
    }
    return;
  }
}

/*----------------------------------------------------------------------*/

inline bool OmsContext_co12::TryLockResult (tsp00_Int2 DBError, OmsObjectContainerPtr p, const OmsObjectId& oid, const char* msg) 
{
  if (DBError != 0) {
    if (e_object_dirty == DBError) {
      m_session->IncOutOfDate();
      return false;
    }
    else if (e_request_timeout == DBError) {
      return false;
    }
    else {
      tsp00_Int4  BufLen = sizeof(DBError);
      tsp00_Int2  SqlError;
      // transform basis error into sql return code
      m_session->m_lcSink->MultiPurpose (m_change, mm_nil, &BufLen, 
        (unsigned char*) &DBError, &SqlError);
      if (SqlError == 400 || SqlError == 500) {
        m_session->IncOutOfDate();
        return false;
      }
      m_session->ThrowDBError (DBError, msg, oid);
    }
    return true;
  }
  else {
    if (NULL != p) {
      p->MarkLocked();
    }
    return true;
  }
}

/*----------------------------------------------------------------------*/

inline void* OmsContext_co12::allocate(size_t sz) 
{
    if (co10_IsOmsTestLib())
    {
        co10_GetKernelInterface()->TestBadAlloc();
    }
#ifdef USE_SYSTEM_ALLOC_CO13
    return new char[sz];
#else
    return m_heap.Allocate((SAPDB_ULong) sz);
#endif
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::PutObjectIntoContext (OmsObjectContainerPtr p, tsp00_Uint4 ContainerHandle)
{
  const char* msg = "PutObjectIntoContext";
  
  tco13_ClsIdEntry* pContainerInfo = this->GetContainerInfo(ContainerHandle);

  if (NULL != pContainerInfo) {
    p->SetContainerInfo(pContainerInfo);
    OMS_DETAIL_TRACE(p->IsNewObject() ? (omsTrContext | omsTrNewObj) : omsTrContext, 
      m_session->m_lcSink, "OmsContext_co12::PutObjectIntoContext : " << p->m_oid
      << ", class: " << p->GetContainerInfo(this)->GetClassInfoPtr()->GetClassName());

    m_oidDir.HashInsert(p);
    if (p->IsVerNewObject()) {
      tsp00_Int4 size = (pContainerInfo->GetObjectSize() + sizeof(void *) - 1) & ~(sizeof(void *) - 1);
      m_newObjCache.registerObject(ContainerHandle, p, (OmsObjectContainer**) (((char*) p) + size));
      p->MarkVerNewRegistered();
    }
  }
  else {
    m_session->ThrowDBError(e_unknown_guid, msg);
  }
}

/*----------------------------------------------------------------------*/

inline bool OmsContext_co12::PutObjectIntoContextReg (OmsObjectContainerPtr p, tsp00_Uint4 ContainerHandle)
{
  const char* msg = "PutObjectIntoContextReg";

  try {
    tco13_ClsIdEntry* pContainerInfo = this->GetContainerInfoReg(ContainerHandle);

    if (NULL != pContainerInfo) {
      p->SetContainerInfo(pContainerInfo);
      OMS_DETAIL_TRACE(p->IsNewObject() ? (omsTrContext | omsTrNewObj) : omsTrContext, 
        m_session->m_lcSink, "OmsContext_co12::PutObjectIntoContext : " << p->m_oid
        << ", class: " << p->GetContainerInfo(this)->GetClassInfoPtr()->GetClassName());

      m_oidDir.HashInsert(p);
      if (p->IsVerNewObject()) {
        tsp00_Int4 size = (pContainerInfo->GetObjectSize() + sizeof(void *) - 1) & ~(sizeof(void *) - 1);
        m_newObjCache.registerObject(ContainerHandle, p, (OmsObjectContainer**) (((char*) p) + size));
        p->MarkVerNewRegistered();
      }
      return true;
    }
    else {
      return false;
    }
  } catch (DbpError &e) {
    // this is for added security, if something throws DbpError
    if (e_unknown_guid == e.dbpError()) {
      return false;
    }
    throw;
  }
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::RenameVersion(const OmsVersionId& OldId, const OmsVersionId& NewId) {
  memcpy (m_version, NewId, sizeof(m_version));
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::ResetNewObjectsToFlush()
{
  m_cntNewObjectsToFlush = 0;
}
/*----------------------------------------------------------------------*/

inline void OmsContext_co12::ResetConsistentView () {
  m_consistentView.gg90SetNil();
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::SetConsistentView(const tgg91_TransNo& v) {
   m_consistentView = v;
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::SetNext(OmsContext_co12* pContext)
{
  m_next = pContext;
}

/*----------------------------------------------------------------------*/

inline void OmsContext_co12::SetNextUnloaded(OmsContext_co12* pContext)
{
  m_nextUnloaded = pContext;
}

/*----------------------------------------------------------------------*/
// PTS 1117690
inline void OmsContext_co12::SetVersionDesc(const OmsTypeWyde* desc){
  if (m_versionDesc != NULL){
    deallocate(m_versionDesc);
    m_versionDesc = NULL;
  }

  if (desc == NULL){
    return;
  }

  // Get length of desciption
  int len = 0;
  while (desc[len] != 0 && len < MAX_VERSION_DESC_LENGTH_CO10){
    ++len;
  }

  // Allocate memory and copy description.
  // Every desciption longer than MAX_... will be truncated.
  // Memory will be freed in destructor of omsContext_co12.
  m_versionDesc = reinterpret_cast<OmsTypeWyde*>(allocate((len + 1) * sizeof(OmsTypeWyde)));
  memcpy(m_versionDesc, desc, len * sizeof(OmsTypeWyde));
  m_versionDesc[len] = 0;  // ensure that string in null-terminated
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::RemoveUnlockedObjFromCache() {
  m_oidDir.RemoveUnlockedObjFromCache();
}

/*----------------------------------------------------------------------*/

inline unsigned char*  OmsContext_co12::VersionContext() {
  return REINTERPRET_CAST(unsigned char*, m_pVersionContext);
}

/*----------------------------------------------------------------------*/

inline OmsObjectId OmsContext_co12::VersionNewOid()
{
  OmsObjectId oid;
  oid.setPno(VERSION_OBJ_PAGE_NO_GG92);
  IncVersionKey();
  tgg92_OmsTypeOid* pOid = REINTERPRET_CAST(tgg92_OmsTypeOid*, &oid);
  pOid->generation = CurrentVersionKey() / (256 * 256);  
  pOid->pagePos    = CurrentVersionKey() % (256 * 256);
  return oid;
}

/*----------------------------------------------------------------------*/

inline void* OmsContext_co12::operator new(size_t sz)
{
  return SharedMemAllocator_co16.allocate(sz);
}

/*----------------------------------------------------------------------*/
#if defined(PLACEMENT_DELETE_MEO00)
inline void OmsContext_co12::operator delete (void* p)
{
  REINTERPRET_CAST(OmsContext_co12*, p)->DeleteSelf();
}
#endif

/*----------------------------------------------------------------------*/
/*               Implementation of OmsSession_co12                      */
/*----------------------------------------------------------------------*/

inline void* OmsSession_co12::operator new(size_t sz)
{
  return SharedMemAllocator_co16.allocate(sz);
}

/*----------------------------------------------------------------------*/
#if defined(PLACEMENT_DELETE_MEO00)
inline void OmsSession_co12::operator delete(void* p)
{
  REINTERPRET_CAST(OmsSession_co12*, p)->DeleteSession();
}
#endif

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::ActivateDefaultContext() {
  m_context = m_defaultContext;
  this->ChangedConsistentView();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::AddContainerInfo(tco13_ClsIdEntry* containerInfo) {
  m_context->m_containerDir.HashInsert (containerInfo);
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::AssertNotReadOnly(const char* callerMsg) 
{
  if (m_read_only)
  {
    this->ThrowDBError (e_oms_read_only, callerMsg);
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::CloseVersion() {
  this->IncCloseVersion();
  m_context->CloseVersion();
  this->ActivateDefaultContext();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::ClearFreeLists() {
  m_defaultContext->m_containerDir.ClearFreeLists();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::ClearTransVersions() {
  if (!m_versionsBoundToTrans.empty()) {
    //CriticalSection_co12 region(this, RGN_VERSION_DIR);
    //region.Enter();

    cgg251dclIterator<OmsContext_co12*,OmsSession_co12> iter = m_versionsBoundToTrans.begin();
    while (iter) {
      //cgg251dclIterator<OmsContext_co12*,OmsSession_co12> delIter = iter;
      //++iter;
      //(*delIter())->CleanContainerDir();
      //if (!(*delIter())->IsVersionOpen()) {
      //  (*delIter())->MarkNotBoundToTrans(false);
      //  m_versionsBoundToTrans.remove(delIter);
      cgg251dclIterator<OmsContext_co12*,OmsSession_co12> delIter = iter;
      OmsContext_co12* pContext = *iter();
      ++iter;
      
      { // Begin of exclusive critical section.    // PTS 1124170
        ExclusiveVersionDirRgn rgn(co10_GetLockIdForVersionDirectory(pContext->GetVersionId()),
                                   co10_GetLockMethodForVersionDictionary());

        pContext->CleanContainerDir();
        if (!pContext->IsVersionOpen()) {
          pContext->MarkNotBoundToTrans(false);
          m_versionsBoundToTrans.remove(delIter);
        }
      } // End of exclusive critical section.
    }
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::CreateDefaultContext() {
  m_defaultContext = new OmsContext_co12(this);
  m_context        = m_defaultContext;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::CommitSubtrans(int requiredLevel) {

  if (!this->IsSubtransOpen()) {
	  return;
  }
  if (-1 == requiredLevel) {
    requiredLevel = this->CurrentSubtransLevel();
  }
  this->IncSubtransCommit();
  this->ExecuteSubtransCommit(requiredLevel);
}

/*----------------------------------------------------------------------*/

#if defined(OMSTST)
inline void OmsSession_co12::CheckCancelCountDown()
{
  if (m_cancelCountDown > 0)
  {
    --m_cancelCountDown;
    if (0 == m_cancelCountDown)
    {
       m_cancelCountDown         = -1;
       m_toCancel                = true;
       m_requiredExceptionThrown = true;
    }
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::CheckExceptionCountDown()
{
  if (m_exceptionCountDown > 0)
  {
    --m_exceptionCountDown;
    if (0 == m_exceptionCountDown)
    {
       m_exceptionCountDown      = -1;
       m_requiredExceptionThrown = true;
       this->ThrowDBError (e_requested_dbperror, "user requested exception");
    }
  }
}
#endif

/*----------------------------------------------------------------------*/

inline OmsContext_co12* OmsSession_co12::CurrentContext() const {
  return m_context;
}

/*----------------------------------------------------------------------*/

inline int OmsSession_co12::CurrentSubtransLevel() const {
  return m_subtrans_lvl;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::DecSubtransLevel() {
  --m_subtrans_lvl;
}

/*----------------------------------------------------------------------*/

inline OmsAbstractObject* OmsSession_co12::DeRef (const OmsObjectId& oid, const ClassIDRef guid, bool forUpd, bool doLock) {
 
  const char* msg = "OmsSession_co12::DeRef";
   
  OmsObjectContainerPtr found = m_context->GetObj (guid, oid, doLock);
  if (found) {
    if ((doLock) && (!found->LockedFlag()) && (!m_context->IsVersion())) {
      m_context->LockObj(found->m_oid);
    }
    if (forUpd) {
      return this->ForUpdPtr(found);
    }

    return &found->m_pobj;
  }
  else {
    this->ThrowDBError (e_object_not_found, msg, oid);
    return NULL;
  }
}

/*----------------------------------------------------------------------*/

inline OmsAbstractObject* OmsSession_co12::DeRefViaKey (unsigned char*  key, 
                                                        const ClassIDRef guid, 
                                                        bool            forUpd, 
                                                        bool            doLock, 
                                                        OmsSchemaHandle Schema,
                                                        OmsContainerNo  ContainerNo)
{
  tco13_ClsIdEntry*     clsinfo = this->GetClsInfo (guid, Schema, ContainerNo);
  OmsObjectContainerPtr found = NULL;
  if (clsinfo->UseCachedKeys()) {    // PTS 1117571
    found = m_context->GetObjViaCachedKey(*clsinfo, key, doLock);
  }
  else {
    found   = m_context->GetObjViaKey(*clsinfo, key, doLock);
  }


  if (NULL != found) {
    if (forUpd) {
      return this->ForUpdPtr(found);
    }
    return &found->m_pobj;
  }
  return NULL;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::DeleteSession() {
  --m_refCnt;
  if (0 == m_refCnt) {
    this->OmsSession_co12::~OmsSession_co12();
    SharedMemAllocator_co16.deallocate(this);
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::ExecuteSubtransCommit(int requiredLevel) {
  tsp00_Int2     DBError;
  CHResult_co001 hr;
  m_beforeImages.subtransCommit (requiredLevel); 
  while (this->CurrentSubtransLevel() >= requiredLevel) {
    if (this->CurrentSubtransLevel() > 1) {
      hr = m_lcSink->SubtransCommit( &DBError);
      if ( 0 != DBError ) { 
        this->ThrowReturnCode (DBError, "omsCommitSubtrans");
      }
    }
    this->DecSubtransLevel();
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::deallocate(void* p) {
  SharedMemAllocator_co16.deallocate(p);
}

/*----------------------------------------------------------------------*/

inline bool OmsSession_co12::ForceReadOnly()
{
  bool ret        = m_read_only;
  m_read_only     = true;
  m_allowReadOnly = false;
  return ret;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::FreeStackHeap() {
#ifndef USE_SYSTEM_ALLOC_CO13
  m_stackHeap.Free();
#endif
}

/*----------------------------------------------------------------------*/

inline OmsAbstractObject* OmsSession_co12::ForUpdPtr(OmsObjectContainerPtr pObj) {
  const char* msg = "OmsSession_co12::ForUpdPtr";
  this->AssertNotReadOnly(msg);
  this->InsertBeforeImage (pObj);
  return &pObj->m_pobj;
}

/*----------------------------------------------------------------------*/

inline tco13_GuidEntry* OmsSession_co12::GetClassInfo(const ClassIDRef guid) {
  return m_context->GetClassInfo (guid);
}

/*----------------------------------------------------------------------*/

inline OmsCallbackInterface* OmsSession_co12::GetCallbackInterface() const {
  return m_callbackInterface;
}

/*----------------------------------------------------------------------*/

inline OmsCallbackInterface* OmsSession_co12::GetCallbackInterface(const ClassIDRef guid) const {
  if (m_callbackInterfaceGuid == guid) {
    return m_callbackInterface;
  }
  else {
    return NULL;
  }
}

/*----------------------------------------------------------------------*/

inline tco13_ClsIdEntry* OmsSession_co12::GetClsInfo(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) {
  return m_context->m_containerDir.HashFind (guid, Schema, ContainerNo);
}

/*----------------------------------------------------------------------*/

inline tco13_ClsIdEntry* OmsSession_co12::GetClsInfoForReg(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) {
  return m_context->m_containerDir.HashFindForReg (guid, Schema, ContainerNo);
}

/*----------------------------------------------------------------------*/

inline tco13_ClsIdEntry* OmsSession_co12::GetContainerInfo(tsp00_Uint4 containerHandle) {
  return m_context->GetContainerInfo(containerHandle);
}

/*----------------------------------------------------------------------*/

inline long OmsSession_co12::GetStoredErrorCode() const {  // PTS 1122838
  return m_errorCode;
}

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsSession_co12::GetMemory (tco13_ClsIdEntry& clsid) { 
  OmsObjectContainerPtr p;

  p = clsid.m_freeHead->free;
  if (NULL != p) { 
    clsid.m_freeHead->free = p->m_hashnext;
  }
  else {
    if (this->InVersion()) {
      p = (OmsObjectContainerPtr) m_context->allocate(clsid.GetObjectSize());
    }
    else {
      p = (OmsObjectContainerPtr) StackHeapMalloc(clsid.GetObjectSize());
    }
  }
  if (NULL != p) {
    p->InitObjContainer(clsid);
    if (p->VarObjFlag()) // PTS 1111943
    {
      new(&p->m_pobj) OmsVarObjInfo_co12();
    }
  }
  return p;
}

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsSession_co12::GetMemoryInVersion (tco13_ClsIdEntry& clsid) { 
  OmsObjectContainerPtr p;

  tsp00_Int4 size = (clsid.GetObjectSize() + sizeof(void *) - 1) & ~(sizeof(void *) - 1);
  p = clsid.m_freeHeadInVersion->free;
  if (NULL != p) { 
    clsid.m_freeHeadInVersion->free = p->m_hashnext;
  }
  else {
    p = (OmsObjectContainerPtr) m_context->allocate(size + 2 * sizeof(void*));
  }
  if (NULL != p) {
    p->InitObjContainer(clsid);
    p->InitVerNew(size);
  }
  return p;
}

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsSession_co12::GetMemory (tco13_GuidEntry& classInfo) {

  OmsObjectContainerPtr p;

  p = classInfo.m_freeHead->free;
  if (NULL != p) { 
    classInfo.m_freeHead->free = p->m_hashnext;
  }
  else {
    if (this->InVersion()) {
      p = (OmsObjectContainerPtr) m_context->allocate(classInfo.GetObjectSize());
    }
    else {
      p = (OmsObjectContainerPtr) StackHeapMalloc(classInfo.GetObjectSize());
    }
  }
  if (NULL != p) {
    p->InitObjContainer(classInfo);
    if (p->VarObjFlag()) // PTS 1111943
    {
      new(&p->m_pobj) OmsVarObjInfo_co12();
    }
  }
  return p;
}

/*----------------------------------------------------------------------*/

inline ISessionLockObjects_co11* OmsSession_co12::GetSessionLockObject() const
{
  return m_lockObjects; 
}

/*----------------------------------------------------------------------*/

inline tsp00_TaskId  OmsSession_co12::GetTaskId() const // PTS 1110315
{
  return m_taskId;
}

/*----------------------------------------------------------------------*/

inline unsigned char* OmsSession_co12::GetVarObjMemory(size_t sz) {
  this->MonitorVarObjSize((tsp00_Int4) sz);
  return REINTERPRET_CAST(unsigned char*, m_context->allocate(sz));
}

/*----------------------------------------------------------------------*/

inline tsp00_Int4 OmsSession_co12::GetVarObjSize(const OmsVarOid& oid) {

  const char* msg = "OmsContext_co12::GetVarObjSize";

  OmsObjectContainerPtr pObj = m_context->FindObjInContext (&oid);
  if (NULL != pObj) {
    if (pObj->DeletedFlag()) {
      ThrowDBError (e_object_not_found, msg, oid);
    }
    OmsVarObjInfo_co12* pObjInfo;
    pObjInfo = REINTERPRET_CAST (OmsVarObjInfo_co12*, &pObj->m_pobj);
    return pObjInfo->m_vobjSize;
  }
  tsp00_Int4 objSize = m_context->GetVarObjFromLiveCacheBase(oid, false);
  m_context->LoadVarObject(oid, VarObjNoLock, objSize, m_currVarObjChunk, NULL);
  return objSize;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncRefCnt() {
  ++m_refCnt;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncCacheHit(const OmsObjectId& oid) {
  if (!InVersion()) 
  {
    m_monitor.IncOmsReadCacheHit();
  }
  else if (CurrentContext()->IsVersionOid(oid)){
    m_monitor.IncOmsReadVersion();
  }
  else {
    m_monitor.IncOmsReadCacheHit();
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncCacheMiss() {
    m_monitor.IncOmsReadCacheMiss();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncDeref() {
  --m_rescheduleDistance;
  if (m_rescheduleDistance <= 0) { /* PTS 1107849 */
    if (m_toCancel) 
    {
      m_toCancel = false;
      ThrowDBError(e_cancelled, "IncDeref");
    }
    m_rescheduleDistance = RESCHEDULE_DISTANCE;
    this->Reschedule();
  }  
  m_monitor.IncDeref();
}

/*----------------------------------------------------------------------*/

inline void  OmsSession_co12::IncDerefIter_LC() { // PTS 1107819 TA 13/09/2000 
  m_monitor.IncDerefIter();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncDereflC() {
  if (InVersion()) 
  {
    m_monitor.IncLCDerefInVersion();
  } 
  else 
  {
    m_monitor.IncDerefLCBase();
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncDerefKey() {
  if (InVersion()) 
  {
    m_monitor.IncLCKeyDerefInVersion();
  }
  else {
    m_monitor.IncKeyDeref();
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncOmsTerminate()
{
  m_monitor.IncOmsTerminate();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncStore() 
{
  m_monitor.IncStore();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncStore_lC() 
{
  if (InVersion())
  {
    m_monitor.IncStoreInVersion();
    
  }
  else 
  {
    m_monitor.IncLCStore();
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncReadStreamBuffer()
{
  m_monitor.IncReadSteam();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncRehash(){ // PTS 1119061
  m_monitor.IncRehash();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncWriteStreamBuffer()
{
  m_monitor.IncWriteSteam();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncReadStreamRow()
{
  m_monitor.IncReadStreamRow();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncWriteStreamRow()
{
  m_monitor.IncWriteStreamRow();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncDelete () {
  m_monitor.IncDelete();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncDelete_lC(tsp00_Int4 cntDeleted) {
  if (InVersion()) 
  {
    m_monitor.IncDeleteInVersion(cntDeleted);
  }
  else 
  {
    m_monitor.IncLCDelete(cntDeleted);
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncDelete_lC() 
{
  IncDelete_lC(1);
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncLock() 
{
  m_monitor.IncLock();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncLogHop(int cnt) 
{
  m_monitor.IncLogHops(cnt);
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncLogHopIter(int cnt) 
{ // PTS 1107819 TA 13/09/2000
  m_monitor.IncLogHopsIter(cnt);
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncExceptions() 
{
  m_monitor.IncExceptions();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncOutOfDate() 
{
  m_monitor.IncOutOfDate();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncOutOfMemory() 
{
  m_monitor.IncBadAlloc();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncTimeout() 
{
  m_monitor.IncTimeout();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncSubtransCommit() 
{
  m_monitor.IncSubtransCommit();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncSubtransRollback() {
  m_monitor.IncSubtransRollback();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncLoadVarObj() {
  m_monitor.IncLoadVarObject();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncLoadVarObjLC() {
  m_monitor.IncLoadVarObjectLC();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncStoreVarObj() {
  m_monitor.IncStoreVarObject();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncStoreVarObjLC() {
  m_monitor.IncLCStoreVarObj();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncCntWaitOmsLockObj() {
     m_monitor.IncCntWaitOmsLockObj();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncWaitOmsLockObjSecs(tsp00_Int4 secs)
{
    m_monitor.IncWaitOmsLockObjSecs(secs);
}
 
/*----------------------------------------------------------------------*/

inline void OmsSession_co12::MaxHashChainLen (int len){ // PTS 1119061
  m_monitor.MaxHashChainLen(len);
}

/*----------------------------------------------------------------------*/
 
 inline void OmsSession_co12::MonitorSubtransLevel() 
{
  m_monitor.MaxSubtransLevel(m_subtrans_lvl);
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::SetCacheSize(tsp00_Uint4 sz) 
{
  m_monitor_curr->cmiCounters_gg00[cmiCacheSize] = sz;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncCreateVersion () 
{
  m_monitor.IncCreateVersion();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncOpenVersion () {
  m_monitor.IncOpenVersion();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncCloseVersion () 
{
  m_monitor.IncCloseVersion();
}   

/*----------------------------------------------------------------------*/
        
inline void OmsSession_co12::IncDropVersion () 
{
  m_monitor.IncDropVersion();
}             

/*----------------------------------------------------------------------*/          

inline void OmsSession_co12::IncDeleteVarObject () {
   m_monitor.IncDeleteVarObject();
}      

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncDeleteVarObjectLC () {
   m_monitor.IncLCDeleteVarObj();
}              

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::IncSubtransLevel () {
  ++m_subtrans_lvl;
  this->MonitorSubtransLevel();
}

/*----------------------------------------------------------------------*/

inline bool OmsSession_co12::InsertBeforeImage (OmsObjectContainerPtr p) {
  int subtransLvl = this->CurrentSubtransLevel();
  if (!(p->existBeforeImage (subtransLvl, m_context->IsVersion()))) {
    if ((subtransLvl > 1) || (this->InVersion())) {
      m_beforeImages.insertBeforeImage (p, subtransLvl);
      return true;
    }
  }
  return false;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::InsertNewBeforeImage (OmsObjectContainerPtr p, tco13_ClsIdEntry* clsinfo) {
  int subtransLvl = this->CurrentSubtransLevel();
  if ((subtransLvl > 1) || (this->InVersion())) {
    m_beforeImages.insertNewBeforeImage (p, clsinfo, subtransLvl);
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::InsertTransVersion(OmsContext_co12* context) {
  if (!this->VersionBoundByMe(context)) {
    m_versionsBoundToTrans.insert(context);
  }
  context->MarkBoundToTrans();
}

/*----------------------------------------------------------------------*/

inline bool OmsSession_co12::InVersion() {
  return (m_context != m_defaultContext);
}

/*----------------------------------------------------------------------*/

inline bool OmsSession_co12::IsDefaultContext( OmsContext_co12* context) {
  return (context == m_defaultContext);
}

/*----------------------------------------------------------------------*/

inline bool OmsSession_co12::IsSubtransOpen() {
  return (m_subtrans_lvl > 1);
}

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr OmsSession_co12::LoadVarObject (const OmsVarOid& oid, VarObjLockType_co12 lockReq, tsp00_Int4 size, void* buf) {
  
  const char* msg = "OmsSession_co12::LoadVarObject";
  
  this->IncLoadVarObj();
  OmsObjectContainerPtr p = m_context->FindVarObjInContext(oid, lockReq, size, buf);
  if (NULL != p) {
    return p;
  }
  tsp00_Int4 objSize = m_context->GetVarObjFromLiveCacheBase(oid, false);
  if ((size < objSize) && (buf != NULL)) {
    this->ThrowDBError (e_too_small_packet_size, msg, oid);
  }
  p = m_context->LoadVarObject(oid, lockReq, objSize, m_currVarObjChunk, buf);
  m_currVarObjChunk.Clear();
  return p;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::LockObj(OmsObjectContainerPtr p) {
  tgg00_BasisError    DBError;
  CUnknownContainerId_co12 FileId;
  CHResult_co001 hr = m_lcSink->LockObj (
    (unsigned char*) &m_context->m_consistentView,
    (unsigned char*) &FileId, //TODO so lassen ??
    REINTERPRET_CAST(OmsTypeOid*, &p->m_oid),
    (unsigned char*) &p->m_objseq,
    &DBError);
  m_context->LockResult (DBError, p, p->m_oid, "OmsSession_co12::LockObj");
  this->IncLock();
}

/*----------------------------------------------------------------------*/

inline bool OmsSession_co12::TryLockObj(OmsObjectContainerPtr p)
{
  tgg00_BasisError    DBError;
  CUnknownContainerId_co12 FileId;
  CHResult_co001 hr = m_lcSink->LockObj (
    (unsigned char*) &m_context->m_consistentView,
    (unsigned char*) &FileId, //TODO so lassen ??
    REINTERPRET_CAST(OmsTypeOid*, &p->m_oid),
    (unsigned char*) &p->m_objseq,
    &DBError);
  bool retval = m_context->TryLockResult (DBError, p, p->m_oid, "OMS_Session::TryLockObj");
  this->IncLock();
  return retval;
}

/*----------------------------------------------------------------------*/

inline void* OmsSession_co12::allocate(size_t sz) {
  return SharedMemAllocator_co16.allocate(sz);
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::MonitorVarObjSize(tsp00_Int4 size) {
  m_monitor.MonitorVarObjSize(size);
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::MonitorWaitNewConsistentView(tsp00_Int4 runTime) {
  m_monitor.MonitorNewConsistentView(runTime);
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::OpenVersion(OmsContext_co12* context) {
  short   DBError;
  HRESULT hr;
  m_defaultContext->VersionClearObjCache();
  if (IsDefaultContext(m_context)) {
    /* current context is default context, release current consistent view */
    if (m_context->ExistsConsistentView()) {
      hr = m_lcSink->EndConsistentView(
        (unsigned char*) &m_context->m_consistentView, &DBError);
      if FAILED (hr) {
         throw DbpError (DbpError::HRESULT_ERROR, hr);
      }
      if ( 0 != DBError ) { 
        throw DbpError (DbpError::DB_ERROR, DBError, "OmsSession::OpenVersion");
      }
    }
  }
  m_context = context;
  m_context->OpenVersion(this);
  this->ChangedConsistentView();
  this->InsertTransVersion(context);
}

/*----------------------------------------------------------------------*/


inline void OmsSession_co12::ReleaseObj(const OmsObjectId& oid) {
  OmsObjectContainerPtr found = m_context->FindObjInContext(&oid);
  if (found) {
    this->ReleaseObj(found);
  }
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::ReleaseObj(OmsObjectContainerPtr p) {
  const bool releaseUpdatedObj = ((!m_context->IsVersion()) && (!this->IsSubtransOpen()));
  if (p->DeletedFlag()) 
  {
    if (releaseUpdatedObj) 
    {
      if (p->IsNewObject())
      {
        OMS_DETAIL_TRACE(omsTrNewObj, m_lcSink, 
          "OmsSession_co12::ReleaseObj : new deleted obj " << p->m_oid 
          << ", class: " << p->GetContainerInfo(m_context)->GetClassInfoPtr()->GetClassName());
        m_context->DecNewObjectsToFlush();
      }
      m_context->FlushDelete (p, true);
    }
    return;
  }
  if (p->StoredFlag()) 
  {
    if (releaseUpdatedObj) 
    {
      if (p->IsNewObject())
      {
        OMS_DETAIL_TRACE(omsTrNewObj, m_lcSink,
          "OmsSession_co12::ReleaseObj : new stored obj " << p->m_oid 
          << ", class: " << p->GetContainerInfo(m_context)->GetClassInfoPtr()->GetClassName());
        m_context->DecNewObjectsToFlush();
      }
      m_context->FlushObj(p);
    }
    else {
      return;
    }
  }
  else {
    if (p->LockedFlag()) {
      if ((releaseUpdatedObj) && (!p->VarObjFlag())) {
        m_context->FlushLockUpdObj (p);
      }
      else {
        return;
      }
    }
  }
  m_context->DeleteObjInContext(p, p->GetContainerInfo (m_context));
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::ReleaseObjUnconditional(const OmsObjectId& oid) {
  OmsObjectContainerPtr found = m_context->FindObjInContext(&oid);
  if (found) {
    m_context->DeleteObjInContext(found, found->GetContainerInfo (m_context));
  }
}
/*----------------------------------------------------------------------*/

#if defined(OMSTST)

inline bool OmsSession_co12::RequiredExceptionThrown() const
{
  return m_requiredExceptionThrown;
}

#endif

/*----------------------------------------------------------------------*/

/* PTS 1107849 */  
inline void OmsSession_co12::Reschedule() {
  m_lcSink->Reschedule();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::RollbackSubtrans(int requiredLvl, bool callFromRollbackTrans) {
  tgg00_BasisError DBError;
  CHResult_co001    hr;
 
  if (-1 == requiredLvl) {
    requiredLvl = this->CurrentSubtransLevel();
  }
  m_beforeImages.subtransRollback (requiredLvl); 
  while (this->CurrentSubtransLevel() >= requiredLvl) {
    if ((this->CurrentSubtransLevel() > 1) && (!callFromRollbackTrans)) 
    {
      hr = m_lcSink->SubtransRollback( &DBError);
      if ( e_ok != DBError ) { 
        this->ThrowReturnCode (DBError, "RollbackSubtrans");
      }
    }
    this->DecSubtransLevel();
  }
  m_currVarObjChunk.Clear();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::ResetSubtransLevel () {
  m_subtrans_lvl = 1;
}

/*----------------------------------------------------------------------*/
#if defined(OMSTST)

inline void OmsSession_co12::SetCancelCountDown(int countDown)
{
  if (0 == countDown) 
  {
    countDown = 1;
  }
  else
  {
    if (countDown < 0)
    {
      countDown = -1;
    }
  }
  m_cancelCountDown = countDown;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::SetExceptionCountDown(int countDown)
{
  if (0 == countDown) 
  {
    countDown = 1;
  }
  else
  {
    if (countDown < 0)
    {
      countDown = -1;
    }
  }
  m_exceptionCountDown = countDown;
}
#endif

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::SetSessionLockObject(ISessionLockObjects_co11* pLockObj)
{
  m_lockObjects = pLockObj;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::Signal(tsp00_TaskId TaskId, tsp00_Bool ok) // PTS 1110315
{
  m_lcSink->Signal(TaskId, ok);
}

/*----------------------------------------------------------------------*/

inline void* OmsSession_co12::StackHeapMalloc(size_t size) {
#ifdef USE_SYSTEM_ALLOC_CO13
  return new char[size];
#else
  return m_stackHeap.Malloc(size);
#endif
}

/*----------------------------------------------------------------------*/

inline int OmsSession_co12::StartSubtrans() {
  
  const char* msg = "OmsSession_co12::StartSubtrans";
  tsp00_Int2  DBError;
  if (this->CurrentSubtransLevel() >= MAX_SUBTRANS_LEVEL) {
    this->ThrowDBError (e_too_many_subtrans, msg);      
  }
  CHResult_co001 hr = m_lcSink->SubtransStart(&DBError);
  if ( 0 != DBError ) { 
    this->ThrowDBError (DBError, msg);
  }
  this->IncSubtransLevel();
  return this->CurrentSubtransLevel();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::StoreErrorCode(long errorCode)   // PTS 1122838
{
  m_errorCode = errorCode;
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::StoreVarObject (const OmsVarOid& oid, const void* pVarObj, unsigned long objLength) {
 
  bool                  freeBeforeImage = true;
  OmsObjectContainerPtr found           = this->LoadVarObject (oid, VarObjNoLock, -1, NULL);
  if (NULL == found) {
      this->ThrowDBError (e_object_not_found, "StoreVarObject", oid);
  }
  if (!found->LockedFlag()) {
    if (!IsLocked(oid)) {
      this->ThrowDBError (e_object_not_locked, "StoreVarObject", oid);
    }
  }
  this->IncStoreVarObj(); 
  if (m_currVarObjChunk.m_oid == oid) {
    m_currVarObjChunk.Clear();
  }
  OmsVarObjInfo_co12* objInfo = REINTERPRET_CAST (OmsVarObjInfo_co12*, &found->m_pobj);
  if (!(found->existBeforeImage (this->CurrentSubtransLevel(), this->InVersion()))) { 
    this->ForUpdPtr(found); 
    freeBeforeImage = false;
  }
  if ((objInfo->m_vobjSize != objLength) || (!freeBeforeImage)) {
    if (freeBeforeImage) {
      objInfo->freeVarObjData(m_context);
    }
    objInfo->m_pvobj = NULL;
    objInfo->m_pvobj = this->GetVarObjMemory(objLength);
  }
  objInfo->m_vobjSize = objLength;
  memcpy (objInfo->m_pvobj, pVarObj, objLength);
  found->MarkStored();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::TransEndEpilog()
{
  m_lockObjects->TransactionEnd();
}

/*----------------------------------------------------------------------*/

inline void OmsSession_co12::UnforceReadOnly(bool readOnly)
{
  m_read_only     = readOnly;
  m_allowReadOnly = true;
}

/*----------------------------------------------------------------------*/

inline void  OmsSession_co12::FreeBeforeImages() {
  /* free before images */
  m_beforeImages.freeBeforeImages();
  m_subtrans_lvl = 1;
}
   
#endif  // __HCO12_H
