/* @lastChanged: "1999-04-21  8:45"

  * @filename:   hco13.h
  * @purpose:    "C++ Persistent Object Hashing"
  * @release:    7.2.0.0
  * @see:        "http://www.bea.sap-ag.de/doku/rel-7.1/hco13.html"
  *
  * @copyright:  "Copyright by   SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/

#if !defined (__HCO13_H)
#define __HCO13_H

#include "ilivecachesink.h" // no check
#include "ggg00.h"
#include "ggg92.h"
#include "gco001.h"
#include "gco102.h"
#include "hco18.h"
#include "ggg250.h"

#define MAX_SUBTRANS_LEVEL 32


// use system allocator instead of own allocators and freelists
// #define USE_SYSTEM_ALLOC_CO13

#ifdef USE_SYSTEM_ALLOC_CO13
#include "geo573.h"
#endif


class OmsAbstractObject;
class OmsHandle;
class OmsObjectContainer;
class OmsObjByClsIterBase;
class OmsStream;
class LogEntry_co10;
class OmsSession_co12;
class OmsContext_co12;

typedef short tco13_ClsId;
typedef OmsObjectContainer* OmsObjectContainerPtr; 

struct tco13_FreeListHeader {
  tco13_FreeListHeader* next;
  OmsObjectContainerPtr free;
  size_t                size;
  inline tco13_FreeListHeader(size_t sz);
  inline void* operator new(size_t sz, OmsContext_co12* context);
#if defined(PLACEMENT_DELETE_MEO00)
  inline void operator delete(void* p, OmsContext_co12* context);
#endif
  inline void DeleteSelf(OmsContext_co12* context);
  void operator delete(void*p); // not implemented, attempt to call will force linker error
};

struct tco13_GuidEntry {
  tco13_GuidEntry*      m_hashNext;
  ClassInfo_co18*       m_classInfo;
  tco13_FreeListHeader* m_freeHead;
  tco13_FreeListHeader* m_freeHeadInVersion;
  tco13_GuidEntry(tco13_GuidEntry* next, ClassInfo_co18* ci, tco13_FreeListHeader* fl, tco13_FreeListHeader* flv) :
  m_hashNext(next),  m_classInfo(ci), m_freeHead(fl), m_freeHeadInVersion(flv) {} 
  inline void* operator new (size_t sz, OmsContext_co12* context);
  void          ChainFree (OmsContext_co12&c, OmsAbstractObject* p);
  void          ChainFree (OmsContext_co12&c, OmsObjectContainerPtr& p);
  inline void   DeleteSelf(OmsContext_co12* context);
  inline const ClassIDRef GetBaseGuid() const;
  inline int         GetKeyLen();
  inline tsp00_Int4  GetObjectSize();
  inline void*       GetVirtualTablePtr();
  inline bool        IsBaseClass() const;
  inline bool        IsDerivedClass() const;
  inline bool        IsDerivedClassOf(const ClassIDRef guid) const;
  inline bool        IsVarObject();
  inline bool        IsKeyedObject() const;
#if defined(PLACEMENT_DELETE_MEO00)
  inline void operator delete(void* p, OmsContext_co12* context);
#endif
  void operator delete(void*p); // not implemented, attempt to call will force linker error
};

typedef tco13_GuidEntry* tco13_GuidEntryPtr;

class tco13_ClassHash : public IGuidHash_co001 {
public :
#define GUID_HASH_ENTRIES_CO13 49
  
  tco13_ClassHash();
  ~tco13_ClassHash();
  void Create(OmsContext_co12* context);
  void HashInsert(ClassInfo_co18* pClassInfo, tco13_FreeListHeader* freeHead, tco13_FreeListHeader* freeHeadInVersion);
  tco13_GuidEntryPtr HashFind(const ClassIDRef guid);
  void Dump(IDump_cco02& dumpObj) const;
private :
  OmsContext_co12*   m_context;
  tco13_GuidEntryPtr m_head[GUID_HASH_ENTRIES_CO13];
};


class tco13_CacheMissEntry {   // PTS 1117571
public:
  int             m_keyLen;
  // Attention! m_key must be the last member in the declaration list.
  //   As the key-length is unknown in advance, the necessary space is 
  //   allocated in the construktor and placed directly behind (and 'coalesced') 
  //   with the member m_key. Because m_key has already length 1, the additionally
  //   allocated space is one byte less than the keyLen. 
  unsigned char m_key[1];

  inline static void* operator new (size_t sz, int keyLen, OmsContext_co12* pContext);
#if defined(PLACEMENT_DELETE_MEO00)
  inline static void  operator delete (void* p, int keyLen, OmsContext_co12* pContext);
#endif
  
  inline tco13_CacheMissEntry (const unsigned char*, int);
  inline ~tco13_CacheMissEntry ();
  inline void setKey(const unsigned char*, int);
  inline void deleteSelf (OmsContext_co12* pContext);
  inline int Compare(const tco13_CacheMissEntry*, const tco13_CacheMissEntry*);
};


struct tco13_ClsIdEntry {
  tco13_ClsIdEntry*                                               m_clsidHashNext;
  tco13_ClsIdEntry*                                               m_guidHashNext;
  ContainerInfo_co18*                                             m_containerInfo;
  cgg250AvlTree<unsigned char*,tco13_ClsIdEntry,OmsContext_co12>  m_index;
  tco13_FreeListHeader*                            m_freeHead;
  tco13_FreeListHeader*                            m_freeHeadInVersion;

  // Members for cache miss structure      // PTS 1117571
  cgg250AvlTree<tco13_CacheMissEntry*,tco13_CacheMissEntry,OmsContext_co12> 
                                   m_cacheMiss;      // search-structure; currently AVL-tree
  bool                             m_useCachedKeys;  // should cached keys be used 
  tco13_CacheMissEntry*            m_cacheMissCmp;   // needed for compare function
  

  tco13_ClsIdEntry (ContainerInfo_co18* classInfo,OmsContext_co12* c);
  inline void* operator new    (size_t sz, OmsContext_co12* session);
#if defined(PLACEMENT_DELETE_MEO00)
  inline void  operator delete (void* p, OmsContext_co12* session);
#endif
  inline int                     Compare(const unsigned char*, const unsigned char*);
  inline void                    DeleteSelf(OmsContext_co12* session);
  inline const ClassInfo_co18*   GetClassInfoPtr() const;
  inline ContainerInfo_co18*     GetContainerInfoPtr() const;
  inline tsp00_Uint4             GetContainerHandle() const;
  inline OmsContainerNo          GetContainerNo() const;
  inline OmsSchemaHandle         GetSchema() const;
  inline tgg00_ContainerId&      GetFileId() const;
  inline const ClassIDRef        GetGuid() const;
  inline const tgg00_ObjColDesc& GetInvColDesc() const;
  inline const tgg00_StackEntry& GetKeyDesc() const;
  inline int                     GetKeyLen() const;
  inline unsigned char*          GetKeyPtr(OmsObjectContainerPtr p);
  inline tsp00_Int4              GetObjectSize() const;
  inline tsp00_Int4              GetPersistentSize() const;
  inline void*                   GetVirtualTablePtr() const;
  inline bool                    IsDerivedClassOf(const ClassIDRef) const;
	inline bool                    IsKeyedObject() const;                    // PTS 1117571
  inline bool                    IsVarObject();
  bool                           SetCachedKeys(bool, OmsContext_co12*);    // PTS 1117571 
  inline bool                    UseCachedKeys() const;                    // PTS 1117571

  void                           chainFree (OmsContext_co12&c, OmsAbstractObject* p);
  void                           chainFree (OmsContext_co12&c, OmsObjectContainerPtr& p);
  inline void                    VersionAddKey(OmsObjectContainerPtr);
  inline void                    VersionDelKey(OmsObjectContainerPtr);
  void                           VersionDelIndex(bool cachedKeysOnly, OmsContext_co12*);
  inline OmsObjectContainer*     VersionFindKey(const unsigned char*);
  inline OmsObjectContainer*     VersionGetInfoFromNode(const unsigned char* const*);  // PTS 1117571
  inline void                    VersionReplaceOrAddKeyContainerPtr(OmsObjectContainerPtr p);

  inline bool                    IsCacheMiss(const unsigned char* key);                                 // PTS 1117571
  inline void                    DeleteCacheMiss(const unsigned char* key, OmsContext_co12* pContext);  // PTS 1117571
  inline void                    InsertCacheMiss(const unsigned char* key, OmsContext_co12* pContext);  // PTS 1117571
  void                           DropCacheMisses(OmsContext_co12* pContext);                            // PTS 1117571
private :
  void operator delete(void*);
};

typedef tco13_ClsIdEntry* tco13_ClsIdEntryPtr;

class cco13_ClsIdHash : public IGuidHash_co001 {
    tsp00_Int4            headentries;
    tsp00_Int4            m_count;
    tco13_ClsIdEntryPtr*  m_clsidHead;
    tco13_ClsIdEntryPtr*  m_guidHead;
    tco13_FreeListHeader* freeListHeader;
    OmsContext_co12*      m_context;
    tco13_ClassHash       m_classDir;
public :
    class Iter {
        friend class cco13_ClsIdHash;
        cco13_ClsIdHash*    hash;
        long                headIndex;
        tco13_ClsIdEntryPtr curr;
    public :
        inline Iter(cco13_ClsIdHash*);
        inline Iter(const Iter&);
        inline void operator++();
        inline operator bool() const;
        inline tco13_ClsIdEntryPtr operator()() const;
    };
    cco13_ClsIdHash  ();
    ~cco13_ClsIdHash ();
    tco13_ClsIdEntry*        AutoRegister (const ClassIDRef guid, OmsSchemaHandle sh, OmsContainerNo ContainerNo);
    tco13_ClsIdEntry*        AutoRegisterForReg (const ClassIDRef guid, OmsSchemaHandle sh, OmsContainerNo ContainerNo);
    inline bool              CheckIfNotDropped(tco13_ClsIdEntry* curr, bool excsafe = false);
    void                     Clean();                    
    void                     Create(OmsContext_co12* context);
    void                     Dump(IDump_cco02& dumpObj) const;
    inline Iter              First ();
    inline tco13_GuidEntry*  GetClassInfo(const ClassIDRef guid);
    inline void              HashInsert (tco13_ClsIdEntry* h);
    inline tco13_ClsIdEntry* HashFindViaContainerHandle (tsp00_Uint4 containerHandle, bool checkExistence = true, bool excsafe = false);
    inline tco13_ClsIdEntry* HashFind   (const ClassIDRef k, OmsSchemaHandle, OmsContainerNo);
    inline tco13_ClsIdEntry* HashFindForReg   (const ClassIDRef k, OmsSchemaHandle, OmsContainerNo);
    void                     HashDelete (tco13_ClsIdEntry* containerInfo);
    bool                     IsBaseClassOf(const ClassIDRef, const ClassIDRef);
    bool                     IsDerivedClassOf(const ClassIDRef , const ClassIDRef);
    void                     ClearCachedKeys(OmsContext_co12*);    // PTS 1117571
    inline void              ClearFreeLists();
    tco13_FreeListHeader*    GetFreeListHeader(size_t size);
    void                     Resize();
    void                     ThrowUnknownContainer(tco13_ClsIdEntryPtr);
    void                     ThrowUnknownContainer(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo);
    void                     UnregisterAll(); /* PTS 1110149 */
    friend class cco13_ClsIdHash::Iter;
};

// Minimal size of the (adaptable) hash array in the class cco13_OidHash
#define MINIMAL_HASH_SIZE 8192
#define DEFAULT_HASH_SIZE 131072
#define MAXIMAL_HASH_SIZE 2147483648

class cco13_OidHash {
    friend class OmsHandle;
    friend class OmsObjByClsIterBase;
    tsp00_Int4             m_headentries;
    tsp00_Int4             m_mask;      // PTS 1119061
    tsp00_Int4             m_count;     // PTS 1119061
    tsp00_Int4             m_maxCount;  // PTS 1119061

    OmsContext_co12*       m_context;
    OmsObjectContainerPtr* m_head;
    OmsObjectContainerPtr* m_headcurr;
public :
    cco13_OidHash ();
    ~cco13_OidHash ();
    void                         Clean();
    void                         Clear();
    tgg00_BasisError             ClearForOpenVersion();
    void                         Create (OmsContext_co12* s, const tsp00_Int4 sz = 131072); // PTS 1119061
    void                         Dump(IDump_cco02& dumpObj) const;
    inline tsp00_Int4            GetHashSize() const;
    inline void                  HashFree       ();
    inline void                  HashInit       (tsp00_Int4 sz);
    inline void                  HashInsert     (OmsObjectContainer* h);
    inline void                  HashSlotInsert (OmsObjectContainer* h);
    inline OmsAbstractObject*    HashFind       (const OmsObjectId& k);
    inline OmsObjectContainerPtr HashFind       (const OmsObjectId* k);
    void                         HashDelete     (const OmsObjectId& key, bool updateKeyCache);
    void                         HashResize     (int newHeadEntries, bool rehash = true); // PTS 1119061
    void                         HashCheck      ();
    void                         CheckChain     (OmsObjectContainerPtr curr);
    void                         RemoveUnlockedObjFromCache();
    void                         SetEmpty       (bool adaptOidHash = true);

    void                         IncCount       ();                          // PTS 1119061
    void                         DecCount       ();                          // PTS 1119061
    void                         ResetCount     (bool adaptOidHash = true);  // PTS 1119061

    class OidIter {
        friend class cco13_OidHash;
        cco13_OidHash*         m_hash;
        long                   m_headIndex;
        int                    m_entries;
        OmsObjectContainerPtr  m_curr;
    public :
        inline OidIter(cco13_OidHash*);
        inline OidIter(const OidIter&);
        inline ~OidIter();
        inline void operator++();
        inline void operator++(int);
        inline operator bool() const;
        inline OmsObjectContainerPtr operator()() const;
    };
    friend class cco13_OidHash::OidIter;
    inline OidIter               First ();
    
    private :
        OmsObjectContainerPtr* HeadPtr (const OmsObjectId& key) {
          //tsp00_Uint4 idx = ((key.getPno() << 13) + key.getPos()) % m_headentries;
          //tsp00_Uint4 idx = (((tsp00_Uint4) key.getPos() << 19) ^ key.getPno()) % m_headentries;
          /* For the computation of the hash value the generation is considered because of 
          ** objects which are created in a version. */
          tsp00_Uint4 idx =(    key.getPno() ^ (key.getPno() << 7) ^ (key.getPno() >> 7)    
                             ^ (key.getPos() >> 3) ^ (key.getPos() << 9)
                             ^ (key.getVersion() << 16))
                           & m_mask;
          m_headcurr = &m_head[idx];
          return m_headcurr;
        }
};


class cco13_BeforeImages {
public :
  cco13_BeforeImages ();
  ~cco13_BeforeImages();
  void Dump(IDump_cco02& dumpObj) const;
  void freeBeforeImages     ();
  void init(OmsSession_co12* c);
  void insertBeforeImage    (OmsObjectContainerPtr, int subtransLvl);
  void insertNewBeforeImage (OmsObjectContainerPtr, tco13_ClsIdEntry* clsinfo, int subtransLvl);
  void removeContext        (OmsContext_co12* pContext);
  void subtransRollback     (int subtransLvl);
  void subtransCommit       (int subtransLvl);
private :
  OmsObjectContainerPtr m_beforeImages[MAX_SUBTRANS_LEVEL]; 
  OmsSession_co12*      m_session;
};

#endif // __HCO013_H

#if defined (__HCO13_IMPL)
#if !defined (__HCO13_H_I)
#define __HCO13_H_I

inline tco13_FreeListHeader::tco13_FreeListHeader(size_t sz) : next(NULL), size(sz), free(NULL) {
}

/*----------------------------------------------------------------------*/

inline void* tco13_FreeListHeader::operator new(size_t sz, OmsContext_co12* context) {
  return context->allocate(sz);
}

/*----------------------------------------------------------------------*/

#if defined(PLACEMENT_DELETE_MEO00)
inline void tco13_FreeListHeader::operator delete (void* p, OmsContext_co12* context) {
  context->deallocate(p);
}
#endif

/*----------------------------------------------------------------------*/

inline void tco13_FreeListHeader::DeleteSelf (OmsContext_co12* context) {
  context->deallocate(this);
}

/*----------------------------------------------------------------------*/

inline void* tco13_GuidEntry::operator new (size_t sz, OmsContext_co12* context) {
  return context->allocate(sz);
}

/*----------------------------------------------------------------------*/

inline void tco13_GuidEntry::DeleteSelf(OmsContext_co12* context) {
  context->deallocate(this);
}

/*----------------------------------------------------------------------*/

inline const ClassIDRef tco13_GuidEntry::GetBaseGuid() const {
  return m_classInfo->GetBaseGuid();
}

/*----------------------------------------------------------------------*/

inline int tco13_GuidEntry::GetKeyLen() {
  return m_classInfo->GetKeyDesc().elen_var();
}

/*----------------------------------------------------------------------*/

inline tsp00_Int4 tco13_GuidEntry::GetObjectSize() {
  return m_classInfo->GetObjectSize();
}

/*----------------------------------------------------------------------*/

inline void* tco13_GuidEntry::GetVirtualTablePtr() {
  return m_classInfo->GetVTPtr();
}

/*----------------------------------------------------------------------*/

inline bool tco13_GuidEntry::IsBaseClass() const {
  return m_classInfo->IsBaseClass();
}

/*----------------------------------------------------------------------*/

inline bool tco13_GuidEntry::IsDerivedClass() const {
  return m_classInfo->IsDerivedClass();
}

/*----------------------------------------------------------------------*/

inline bool tco13_GuidEntry::IsDerivedClassOf(const ClassIDRef guid) const {
  return m_classInfo->IsDerivedClassOf(guid);
}

/*----------------------------------------------------------------------*/

inline bool tco13_GuidEntry::IsVarObject() {
  return m_classInfo->IsVarObject();
}

/*----------------------------------------------------------------------*/

inline bool tco13_GuidEntry::IsKeyedObject() const {
  return m_classInfo->IsKeyedObject();
}

/*----------------------------------------------------------------------*/

#if defined(PLACEMENT_DELETE_MEO00)
inline void tco13_GuidEntry::operator delete(void* p, OmsContext_co12* context) {
  context->deallocate(p);
}
#endif

/*----------------------------------------------------------------------*/

inline void* tco13_ClsIdEntry::operator new (size_t sz, OmsContext_co12* context) {
  return context->allocate(sz);
}

/*----------------------------------------------------------------------*/

#if defined(PLACEMENT_DELETE_MEO00)
inline void tco13_ClsIdEntry::operator delete (void* p, OmsContext_co12* context) {
  context->deallocate(p);
}
#endif

/*----------------------------------------------------------------------*/
// Check whether key is already registered as a cache miss     
inline bool tco13_ClsIdEntry::IsCacheMiss(const unsigned char *key) { 
  m_cacheMissCmp->setKey(key, GetKeyLen());
  return m_cacheMiss.Find(m_cacheMissCmp) != NULL ? true : false;
}

/*----------------------------------------------------------------------*/
// Delete specified entry from the cache miss structure           
inline void tco13_ClsIdEntry::DeleteCacheMiss(const unsigned char *key, OmsContext_co12* pContext) {
  m_cacheMissCmp->setKey(key, GetKeyLen());
  
  // Find pointer to the node in the AVL-Tree
  tco13_CacheMissEntry*const* pCacheMissEntry = m_cacheMiss.Find(m_cacheMissCmp); 
  tco13_CacheMissEntry*       pDelEntry       = const_cast<tco13_CacheMissEntry*>(*pCacheMissEntry);

  // Delete the node in the AVL-Tree
  tgg00_BasisError rc = m_cacheMiss.Delete(m_cacheMissCmp);
  if (e_ok != rc) {
    // Object does not existent is the structure
    co10_Throw (DbpError (DbpError::DB_ERROR, rc, "tco13_ClsIdEntry::DeleteCacheMiss")); 
  }

  // Delete the cache miss entry 
  pDelEntry->deleteSelf(pContext);
}

/*----------------------------------------------------------------------*/
// Insert entry into the cache miss structure 
inline void tco13_ClsIdEntry::InsertCacheMiss(const unsigned char *key,
                                              OmsContext_co12* pContext) {
  // Create object for cache miss structure
  tco13_CacheMissEntry* pCacheMiss 
    = new (GetKeyLen(),pContext) tco13_CacheMissEntry (key, GetKeyLen());

  tgg00_BasisError rc 
    = m_cacheMiss.Insert(const_cast<tco13_CacheMissEntry*>(pCacheMiss));
  if (e_ok != rc) {
    // Object is already existent is the structure
    co10_Throw (DbpError (DbpError::DB_ERROR, rc, "tco13_ClsIdEntry::InsertCacheMiss")); 
  }
}

/*----------------------------------------------------------------------*/


inline int tco13_ClsIdEntry::Compare(const unsigned char* l, const unsigned char* r) {
  int cmp = memcmp (l, r, GetKeyLen());
  if (0 != cmp) {
    if (cmp > 0) {
      return 1;
    }
    else {
      return -1;
    }
  }
  else {
    return 0;
  }
}

/*----------------------------------------------------------------------*/

inline void tco13_ClsIdEntry::DeleteSelf(OmsContext_co12* pContext) {
  // Delete complete structure for new version objects / cached keys 
  VersionDelIndex(false, pContext);

  // PTS 1117571
  // Delete complete structure of the cache misses
  if (UseCachedKeys()){
    DropCacheMisses(pContext);
    if (m_cacheMissCmp != NULL){
      m_cacheMissCmp->deleteSelf(pContext);
      m_cacheMissCmp = NULL;
    }
  }
  
  // release reference to global container dictionary 
  co10_DetachContainerInfo(pContext->LcSink(), m_containerInfo);
  // destroy self
  pContext->deallocate(this);
}

/*----------------------------------------------------------------------*/

inline const ClassInfo_co18* tco13_ClsIdEntry::GetClassInfoPtr() const {
  return m_containerInfo->GetClassInfoPtr();
}

/*----------------------------------------------------------------------*/

inline ContainerInfo_co18* tco13_ClsIdEntry::GetContainerInfoPtr() const {
  return m_containerInfo;
}

/*----------------------------------------------------------------------*/

inline tsp00_Uint4 tco13_ClsIdEntry::GetContainerHandle() const {
  return m_containerInfo->GetContainerHandle();
}

/*----------------------------------------------------------------------*/

inline OmsContainerNo tco13_ClsIdEntry::GetContainerNo() const {
  return m_containerInfo->m_ContainerNo;
}

/*----------------------------------------------------------------------*/

inline tgg00_FileId& tco13_ClsIdEntry::GetFileId() const { 
  return m_containerInfo->m_ContainerId;
}

/*----------------------------------------------------------------------*/

inline const ClassIDRef tco13_ClsIdEntry::GetGuid() const {
  return m_containerInfo->GetGuid();
}

/*----------------------------------------------------------------------*/

inline const tgg00_ObjColDesc& tco13_ClsIdEntry::GetInvColDesc() const {
  return m_containerInfo->GetInvColDesc();
}

/*----------------------------------------------------------------------*/

inline const tgg00_StackEntry& tco13_ClsIdEntry::GetKeyDesc() const {
  return m_containerInfo->GetKeyDesc();
}

/*----------------------------------------------------------------------*/

inline int tco13_ClsIdEntry::GetKeyLen() const {
  return m_containerInfo->GetKeyDesc().elen_var();
}

/*----------------------------------------------------------------------*/

inline unsigned char* tco13_ClsIdEntry::GetKeyPtr(OmsObjectContainerPtr p) {
  return (((unsigned char*) &p->m_pobj) + /* vtblptr */ sizeof (void*) + GetKeyDesc().epos() - 1);
}

/*----------------------------------------------------------------------*/

inline tsp00_Int4 tco13_ClsIdEntry::GetObjectSize() const {
  return m_containerInfo->GetObjectSize();
}

/*----------------------------------------------------------------------*/

inline tsp00_Int4 tco13_ClsIdEntry::GetPersistentSize() const {
  return (tsp00_Int4) m_containerInfo->m_PersistentSize;
}

/*----------------------------------------------------------------------*/

inline OmsContainerNo tco13_ClsIdEntry::GetSchema() const {
  return m_containerInfo->m_schema;
}

/*----------------------------------------------------------------------*/

inline void* tco13_ClsIdEntry::GetVirtualTablePtr() const {
   return m_containerInfo->GetVirtualTablePtr();
}

/*----------------------------------------------------------------------*/

inline bool tco13_ClsIdEntry::IsDerivedClassOf(const ClassIDRef guid) const {
  return m_containerInfo->m_clsInfo->IsDerivedClassOf(guid);
}

/*----------------------------------------------------------------------*/

inline bool tco13_ClsIdEntry::IsKeyedObject() const {  
  return  m_containerInfo->m_clsInfo->IsKeyedObject();;
}

/*----------------------------------------------------------------------*/

inline bool tco13_ClsIdEntry::IsVarObject() {
  return m_containerInfo->m_clsInfo->IsVarObject();
}

/*----------------------------------------------------------------------*/

inline bool tco13_ClsIdEntry::UseCachedKeys() const {  
  return m_useCachedKeys;
}

/*----------------------------------------------------------------------*/

void tco13_ClsIdEntry::VersionAddKey(OmsObjectContainerPtr p) {
  tgg00_BasisError rc;
  rc = m_index.Insert(GetKeyPtr(p));
  if (e_ok != rc) {
    co10_Throw (DbpError (DbpError::DB_ERROR, rc, "tco13_ClsIdEntry::VersionAddKey")); 
  }
}

/*----------------------------------------------------------------------*/

void tco13_ClsIdEntry::VersionDelKey(OmsObjectContainerPtr p) {
   m_index.Delete(GetKeyPtr(p));
}

/*----------------------------------------------------------------------*/

OmsObjectContainer* tco13_ClsIdEntry::VersionFindKey(const unsigned char* k) {
  unsigned char* const* p = m_index.Find(CONST_CAST(unsigned char*, k));
    
  return p != NULL ? VersionGetInfoFromNode(p) : NULL;
}

/*----------------------------------------------------------------------*/
// Returns the user-data (pointer to a object-container) from a node of 
// the version/cached-key search structure 
OmsObjectContainer* tco13_ClsIdEntry::VersionGetInfoFromNode(const unsigned char* const* p) {
    return REINTERPRET_CAST(OmsObjectContainer*, 
                 CONST_CAST(unsigned char*, (*p) 
                                            + GetKeyLen() 
                                            - GetObjectSize()));
}

/*----------------------------------------------------------------------*/

void tco13_ClsIdEntry::VersionReplaceOrAddKeyContainerPtr(OmsObjectContainerPtr pObj)
{
  unsigned char* const* p = m_index.Find(GetKeyPtr(pObj));
  if (NULL != p)
  {
      // this is allowed since key has not been changed !
      *CONST_CAST(unsigned char**, p) = GetKeyPtr(pObj);
  }
  else
  {
     this->VersionAddKey(pObj);
  }
}

/*----------------------------------------------------------------------*/
/* Implementation of inline methods of class cco13_OidHash              */
/*----------------------------------------------------------------------*/

inline tsp00_Int4 cco13_OidHash::GetHashSize() const
{
  return m_headentries;
}

/*----------------------------------------------------------------------*/

inline OmsAbstractObject* cco13_OidHash::HashFind (const OmsObjectId& k) {

  OmsObjectContainer* curr = *HeadPtr(k);
  int cnt = 1;  

  while (curr) {
    if (curr->m_oid == k){
      m_context->m_session->MaxHashChainLen(cnt); // PTS 1119061
			return &curr->m_pobj;
    }
    else {
			curr = curr->m_hashnext;
      ++cnt;
    }
  }
  m_context->m_session->MaxHashChainLen(cnt); // PTS 1119061
  return NULL;
};

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr cco13_OidHash::HashFind (const OmsObjectId* k) {
	
	OmsObjectContainer* curr = *HeadPtr(*k);
  int cnt = 1;  

	while (curr)
  {
		if (curr->m_oid == *k){
      m_context->m_session->MaxHashChainLen(cnt); // PTS 1119061
			return curr;
    }
    else {
			curr = curr->m_hashnext;
      ++cnt;
    }
  }
  m_context->m_session->MaxHashChainLen(cnt); // PTS 1119061
	return NULL;
};

/*----------------------------------------------------------------------*/

inline void cco13_OidHash::HashInsert(OmsObjectContainer* h) {

  h->m_hashnext = *HeadPtr(h->m_oid);
	*m_headcurr     = h;
  IncCount();          // PTS 1119061
#ifdef _ASSERT_OMS
  HashCheck();
#endif
};

/*----------------------------------------------------------------------*/

inline void cco13_OidHash::HashSlotInsert (OmsObjectContainer* h) {
	h->m_hashnext = *m_headcurr;
	*m_headcurr     = h;
  IncCount();          // PTS 1119061
#ifdef _ASSERT_OMS
  HashCheck();
#endif
};

/*===========================================================================*/
/// Increase number of entries in the hash and if necessary adapt the hash size
/*! Rehashing is necessary, if the number of entries is larger than twice
**  the size of the hash array (The hash chains have a mean size of 2). 
**  In this case the size of the hash array will be doubled */
/*===========================================================================*/
inline void cco13_OidHash::IncCount(){  // PTS 1118855
  ++m_count;

  if (m_count > m_maxCount){
    m_maxCount = m_count;
  }

  if (m_count > (m_headentries << 1)){
    HashResize(2 * m_headentries);
  }
}

/*===========================================================================*/
/// Decrease number of entries in the hash.
/*===========================================================================*/
inline void cco13_OidHash::DecCount(){  // PTS 1118855
  --m_count;
}
 
/*===========================================================================*/
/// Reset the number of entries to zero and if necessary adapt the hash size.
/*! Rehashing is considered as necessary if the maximal number of entries 
**  is less than 1/4 of the number of hash buckets. Rehashing will reduce the 
**  size of the hash array by half. To reduce the number of rehashing actions
**  transactions are only considered if they access at least one object and the
**  minimal size of the hash array is specified by the constant MINIMAL_HASH_SIZE. */
/*===========================================================================*/
inline void cco13_OidHash::ResetCount(bool adaptOidHash){  // PTS 1118855
  if (   adaptOidHash
      && m_maxCount != 0 
      && m_maxCount < (m_headentries >> 2) 
      && m_headentries > MINIMAL_HASH_SIZE){
    HashResize(m_headentries / 2, false);
  }

  m_count    = 0;
  m_maxCount = 0;
}

/*----------------------------------------------------------------------*/

inline cco13_OidHash::OidIter cco13_OidHash::First ()
{
  OidIter iter(this);
  /* PTS 1122897 FF 2003-07-03 if there is no element within the OidHash don't iterate */
  for (iter.m_headIndex = 0; (0 < m_count) && (iter.m_headIndex < m_headentries); ++iter.m_headIndex) {
    iter.m_curr = this->m_head[iter.m_headIndex];
    if (NULL != iter.m_curr) 
    {
      iter.m_entries = 1;
      break;
    }
  }
  return iter;
};

/*----------------------------------------------------------------------*/

inline cco13_OidHash::OidIter::OidIter(cco13_OidHash* h) : m_hash(h), m_entries(0), m_curr(NULL), m_headIndex(-1) {
}

/*----------------------------------------------------------------------*/

inline cco13_OidHash::OidIter::OidIter(const OidIter& iter) {
  *this = iter;
}

/*----------------------------------------------------------------------*/

inline cco13_OidHash::OidIter::~OidIter()
{
  /* PTS 1119061
  if ((m_entries > 2 * m_hash->m_headentries) &&
     (m_entries > m_hash->m_resize))
  {
    m_hash->m_resize = m_entries;
  }
  */
}

/*----------------------------------------------------------------------*/

inline void cco13_OidHash::OidIter::operator++() {
  if (NULL != m_curr) {
    m_curr = m_curr->m_hashnext;
  }
  if (NULL == m_curr) {
    ++m_headIndex; 
    while (m_headIndex < m_hash->m_headentries) {
      m_curr = m_hash->m_head[m_headIndex];
      if (NULL != m_curr) 
      {
        break;
      }
      ++m_headIndex;
    }
  }
  ++m_entries;
}

/*----------------------------------------------------------------------*/

inline void cco13_OidHash::OidIter::operator++(int dummy) {
  operator++();
}

/*----------------------------------------------------------------------*/

inline cco13_OidHash::OidIter::operator bool() const {
  return (m_curr != NULL);
}

/*----------------------------------------------------------------------*/

inline OmsObjectContainerPtr cco13_OidHash::OidIter::operator()() const {
  return m_curr;
}

/*----------------------------------------------------------------------*/
/* Implementation of class cco13_ClsIdHash inline methods               */
/*----------------------------------------------------------------------*/

inline bool cco13_ClsIdHash::CheckIfNotDropped(tco13_ClsIdEntry* curr, bool excsafe) {
  if (curr->m_containerInfo->IsDropped()) {
    if (excsafe) {
      return false;
    }
    this->ThrowUnknownContainer(curr);
  }
  return true;
}

/*----------------------------------------------------------------------*/

inline void cco13_ClsIdHash::ClearFreeLists() {
  tco13_FreeListHeader* p = freeListHeader;
  while (p != NULL) {
    if (m_context->IsVersion()) {
      OmsObjectContainerPtr curr = p->free;
      OmsObjectContainerPtr toFree;
      while (NULL != curr) {
        toFree = curr;
        curr   = curr->m_hashnext;
        m_context->deallocate(toFree);
      }
    }
    p->free = NULL;
    p = p->next;
  }
}

/*----------------------------------------------------------------------*/

inline void cco13_ClsIdHash::HashInsert (tco13_ClsIdEntry* h)
{
  int hashSlot;
  hashSlot              = (h->GetContainerHandle()) % headentries;
  h->m_clsidHashNext    = m_clsidHead[hashSlot];
  m_clsidHead[hashSlot] = h;

  hashSlot = HashValue(h->GetGuid(), h->GetSchema(), h->GetContainerNo(), headentries);
  h->m_guidHashNext    = m_guidHead[hashSlot];
  m_guidHead[hashSlot] = h;

  h->m_freeHead = this->GetFreeListHeader(h->GetObjectSize()); 
  h->m_freeHeadInVersion = this->GetFreeListHeader((h->GetObjectSize() + 
    3*sizeof(void*) - 1) & ~(sizeof(void*) - 1));
  m_classDir.HashInsert(h->m_containerInfo->m_clsInfo, h->m_freeHead, h->m_freeHeadInVersion);
  ++m_count;
  if (m_count > 2 * headentries)
  {
      this->Resize();
  }
};

/*----------------------------------------------------------------------*/

inline tco13_ClsIdEntry* cco13_ClsIdHash::HashFindViaContainerHandle (tsp00_Uint4 containerHandle, bool checkExistence, bool excsafe) 
{
  int hashSlot           = containerHandle % headentries;
  tco13_ClsIdEntry* curr = m_clsidHead[hashSlot];
  while (curr != NULL) {
    if (curr->GetContainerHandle() == containerHandle) {
      if (checkExistence) {
        if (!CheckIfNotDropped(curr, excsafe)) {
          return NULL;
        }
      }
      return curr;
    }
    curr = curr->m_clsidHashNext;
  }
  ContainerInfo_co18* pContainerInfo = co10_FindContainerInfo(m_context->LcSink(), containerHandle);
  if (NULL != pContainerInfo) {
    curr = new(m_context) tco13_ClsIdEntry (pContainerInfo, m_context);
    if (NULL != curr) {
      this->HashInsert (curr);
    }
    return curr;
  }
  OMS_TRACE(omsTrAlways, m_context->LcSink(), "cco13_ClsIdHash::HashFindViaContainerHandle : " << containerHandle << " not found");
  if (!excsafe) {
    throw DbpError (DbpError::DB_ERROR, e_unknown_guid);
  } else {
    return NULL;
  }
};

/*----------------------------------------------------------------------*/

inline tco13_ClsIdEntry* cco13_ClsIdHash::HashFind (const ClassIDRef guid, OmsSchemaHandle sh, OmsContainerNo ContainerNo) {
  int hashSlot = HashValue(guid, sh, ContainerNo, headentries);
  tco13_ClsIdEntry* curr = m_guidHead[hashSlot];
  while (curr != NULL) {
    if (omsIsCompatibleGuid(curr->GetGuid(), guid) && (curr->GetSchema() == sh) && (curr->GetContainerNo() == ContainerNo)) {
      CheckIfNotDropped(curr);
      return curr;
    }
    curr = curr->m_guidHashNext;
  }
  return this->AutoRegister(guid, sh, ContainerNo);
}

/*----------------------------------------------------------------------*/

inline tco13_ClsIdEntry* cco13_ClsIdHash::HashFindForReg (const ClassIDRef guid, OmsSchemaHandle sh, OmsContainerNo ContainerNo) {
  int hashSlot = HashValue(guid, sh, ContainerNo, headentries);
  tco13_ClsIdEntry* curr = m_guidHead[hashSlot];
  while (curr != NULL) {
    if (omsIsCompatibleGuid(curr->GetGuid(), guid) && (curr->GetSchema() == sh) && (curr->GetContainerNo() == ContainerNo)) {
      CheckIfNotDropped(curr);
      return curr;
    }
    curr = curr->m_guidHashNext;
  }
  return this->AutoRegisterForReg(guid, sh, ContainerNo);
}

/*----------------------------------------------------------------------*/

inline cco13_ClsIdHash::Iter cco13_ClsIdHash::First ()
{
  Iter iter(this);
  for (iter.headIndex = 0; iter.headIndex < headentries; ++iter.headIndex) {
    iter.curr = m_clsidHead[iter.headIndex];
    if (NULL != iter.curr) {
      break;
    }
  }
  return iter;
};

/*----------------------------------------------------------------------*/

inline tco13_GuidEntry* cco13_ClsIdHash::GetClassInfo(const ClassIDRef guid) {
  tco13_GuidEntry* found = m_classDir.HashFind(guid);
  if (NULL == found) {
    ClassInfo_co18* pClassInfo = co10_FindClassInfo(m_context->LcSink(), guid);
    if (NULL != pClassInfo) {
      tco13_FreeListHeader* freeHead = this->GetFreeListHeader(pClassInfo->GetObjectSize()); 
      tco13_FreeListHeader* freeHeadInVersion = this->GetFreeListHeader(
        (pClassInfo->GetObjectSize() + 3*sizeof(void*) - 1) & ~(sizeof(void*) - 1));
      m_classDir.HashInsert(pClassInfo, freeHead, freeHeadInVersion);
      found = m_classDir.HashFind(guid); 
    }
    else {
      OMS_TRACE(omsTrAlways, m_context->LcSink(), "cco13_ClsIdHash::GetClassInfo : " << guid << " not found");
      throw DbpError (DbpError::DB_ERROR, e_unknown_guid);
    }
  }
  return found;
}

/*----------------------------------------------------------------------*/

inline cco13_ClsIdHash::Iter::Iter(cco13_ClsIdHash* h) : hash(h), curr(NULL), headIndex(0) {
}

/*----------------------------------------------------------------------*/

inline cco13_ClsIdHash::Iter::Iter(const Iter& iter) {
  *this = iter;
}

/*----------------------------------------------------------------------*/

inline void cco13_ClsIdHash::Iter::operator++() {
  if (NULL != curr) {
    curr = curr->m_clsidHashNext;
  }
  if (NULL == curr) {
    ++headIndex; 
    while (headIndex < hash->headentries) {
      curr = hash->m_clsidHead[headIndex];
      if (NULL != curr) {
        break;
      }
      ++headIndex;
    }
  }
}

/*----------------------------------------------------------------------*/

inline cco13_ClsIdHash::Iter::operator bool() const {
  return (curr != NULL);
}

/*----------------------------------------------------------------------*/

inline tco13_ClsIdEntryPtr cco13_ClsIdHash::Iter::operator()() const {
  return curr;
}


/*----------------------------------------------------------------------*/
/* Implementation of class tco13_CacheMissEntry inline methods          */
/*----------------------------------------------------------------------*/

inline void* tco13_CacheMissEntry::operator new (size_t sz, 
                                                 int keyLen, 
                                                 OmsContext_co12* pContext){
  return pContext->allocate(sz + keyLen - 1);
}

//----------------------------------------------------------------------/

inline void tco13_CacheMissEntry::deleteSelf(OmsContext_co12* pContext){
   pContext->deallocate(this);
}

//----------------------------------------------------------------------/

#if defined(PLACEMENT_DELETE_MEO00)
inline void tco13_CacheMissEntry::operator delete (void* p,
                                                   int keyLen,
                                                   OmsContext_co12* pContext){
  pContext->deallocate(p);
}
#endif

//----------------------------------------------------------------------/

inline tco13_CacheMissEntry::tco13_CacheMissEntry (const unsigned char *key,     
                                                   int                 keyLen){
  if (keyLen > 0){
    m_keyLen = keyLen;
  }
  else {
    m_keyLen = 0;
  }

  if (key != NULL){
    memcpy(&m_key, key, m_keyLen);
  }
  else {
    memset(&m_key, '\0', m_keyLen);   
  }
}

//----------------------------------------------------------------------/

inline tco13_CacheMissEntry::~tco13_CacheMissEntry (){
    
}

//----------------------------------------------------------------------/

inline void tco13_CacheMissEntry::setKey (const unsigned char *key, int keyLen){
  if (key == NULL || keyLen <= 0){
    throw DbpError(DbpError::DB_ERROR, e_system_error);
  }
  
  // Copy key
  memcpy(&m_key, key, keyLen);
}

//----------------------------------------------------------------------/

inline int tco13_CacheMissEntry::Compare(const tco13_CacheMissEntry* l, 
                                         const tco13_CacheMissEntry* r) {
  int cmp = memcmp (l->m_key, r->m_key, m_keyLen);
  if (cmp) {
    return cmp > 0 ? 1 : -1;
  }
  else {
    return 0;
  }
}

//----------------------------------------------------------------------/

#endif // __HCO13_H_I
#endif // __HCO13_IMPL



