/* @lastChanged: "1998-07-14  8:07"

  * @filename:   hco002.h
  * @purpose:    "OMS Memory Management"
  * @release:    7.1.0.0
  * @see:        "http://www.bea.sap-ag.de/doku/rel-7.1/gco10.html"
  *
  * @copyright:  "Copyright by   SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/

#if !defined (__GCO16_H)
#define __GCO16_H
#include "ggg00.h"
#include "gco00.h"
#include "ilivecachesink.h" /* no check */
#include "hco10.h"
#include "gco01.h"
#include "gco001.h"

#define CHUNK_SIZE_CO16 64000
#define ALIGNMENT_CO16  sizeof(void*)

struct co16_MemChunk {
    co16_MemChunk*  m_next;
    unsigned char m_mem[CHUNK_SIZE_CO16 - sizeof(co16_MemChunk*)];
    long memSize() { return sizeof(m_mem); }
};

class co16_OmsStackHeap {
public :
  co16_OmsStackHeap() : 
    m_usedChunks(NULL), m_rest(0), m_used(0) {}
  void* Malloc( size_t requiredSize);
  void  Free();
  tsp00_8ByteCounter HeapUsed() const { return m_used; } /* PTS 1107731 */
private :
  co16_MemChunk*     m_usedChunks;
  unsigned char*     m_curr;
  size_t             m_rest;
  tsp00_8ByteCounter m_used; /* PTS 1107731 */
};


class co16_SharedMemAllocator {
public :
  co16_SharedMemAllocator();
  inline void* allocate   (size_t sz);
  inline void  deallocate (void* pz);
private :
  tsp00_8ByteCounter m_sharedMemUsed;
};

extern co16_SharedMemAllocator SharedMemAllocator_co16;

/*----------------------------------------------------------------------*/

inline size_t co16Aligned(size_t val) {
  return ((val + (ALIGNMENT_CO16 - 1)) & ~(ALIGNMENT_CO16 - 1)); 
}

/*----------------------------------------------------------------------*/

inline void* co16_OmsStackHeap::Malloc(size_t requiredSize) {
  void * p;
  size_t alignedSize = co16Aligned(requiredSize);
  if (m_rest >= alignedSize) {
    m_rest -= alignedSize;
    p       = m_curr;
    m_curr += alignedSize;
    return p;
  }
  else {
    co16_MemChunk* c = REINTERPRET_CAST(co16_MemChunk*, co10_malloc(sizeof(co16_MemChunk)));
    if (c != NULL) {
      c->m_next    = m_usedChunks;
      m_usedChunks = c;
      m_rest       = c->memSize() - alignedSize;
      m_curr       = c->m_mem + alignedSize;
      m_used      += c->memSize(); /* PTS 1107731 */
      return c->m_mem;
    }
    else {
      throw DbpError (DbpError::RTE_ERROR, e_new_failed);
    }
  }
}

/*----------------------------------------------------------------------*/

inline void co16_OmsStackHeap::Free() {
  co16_MemChunk* pCurrChunk = m_usedChunks;
  while (NULL != pCurrChunk)
  {
    co16_MemChunk* pDelete = pCurrChunk;
    pCurrChunk = pCurrChunk->m_next;
    co10_free(pDelete);
  }
  m_usedChunks = NULL;
  m_rest       = 0;
  m_used       = 0; /* PTS 1107731 */
}

/*----------------------------------------------------------------------*/

inline void* co16_SharedMemAllocator::allocate(size_t sz) {
  return co10_malloc(sz);
}

/*----------------------------------------------------------------------*/

inline void co16_SharedMemAllocator::deallocate(void *p) {
  co10_free(p);
}

#endif
