/*!================================================================
 module:        hco18.h
 
 responsible:   ThomasA
 
 special area:  OMS
 
 description:   Oms Class and Container Dictionary   
 
 see:
 
 copyright:     SAP AG 1998 


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 ===================================================================*/

#ifndef HCO18_H
#define HCO18_H

#include "ilivecachesink.h" // no check
#include "ggg00.h"
#include "gco01.h"
#include "gco001.h"
#include "gco102.h"

struct ClassDumpInfo_co18
{
  ClassDumpInfo_co18(tsp00_Int4 hashSlot) 
    :
  m_hashSlot(hashSlot)
  {
  }
  void*                m_addr;
  void*                m_pBaseClass;
  ClassID              m_guid;
  tsp00_C64            m_ClassName;
  tsp00_Int4           m_ObjectSize;
  tsp00_Int4           m_hashSlot;
  void*                m_VTBLPtr;
  bool                 m_isBaseClass;
  bool                 m_isDerivedClass;
  bool                 m_isVarObject;
  bool                 m_filler;
};

struct ClassInfo_co18 {
public :
  ClassInfo_co18(const char*             ClassName, 
                 const ClassIDRef        Guid,
                 const ClassInfo_co18*   pBaseClass,
                 const tgg00_ObjColDesc& InvColDesc,
                 size_t                  ObjSize, 
                 void*                   vtptr,
                 ClassInfo_co18*         HashNext);
  void* operator new(size_t sz);
  void           DeleteSelf();
#if defined(PLACEMENT_DELETE_MEO00)
  void  operator delete (void* p);
#endif
  void                    Dump(ClassDumpInfo_co18&);
  bool                    IsBaseClass()   const { return m_isBaseClass; }
  bool                    IsDerivedClass() const { return m_isDerivedClass; }
  bool                    IsDerivedClassOf(const ClassIDRef guid) const 
  {
    const ClassInfo_co18* pBaseClass = m_pBaseClass; 
    while (NULL != pBaseClass)
    {
      if (pBaseClass->GetGuid() == guid)
      {
        return true;
      }
      else
      {
        pBaseClass = pBaseClass->m_pBaseClass;
      }
    }
    return false;
  }
  bool                    IsVarObject()   const { return m_isVarObject; }
  bool                    IsKeyedObject() const { return m_isKeyedObject; }
  const ClassIDRef        GetBaseGuid()   const { return m_pBaseClass->GetGuid(); }
  const char*             GetClassName()  const { return REINTERPRET_CAST(const char*, &m_ClassName[0]); }
  const ClassIDRef        GetGuid()       const { return m_guid; }
  const tgg00_ObjColDesc& GetInvColDesc() const { return m_InvColDesc; }
  const tgg00_StackEntry& GetKeyDesc()    const { return m_KeyDesc; }
  tsp00_Int4              GetObjectSize() const { return (tsp00_Int4) m_ObjectSize; }
  void*                   GetVTPtr()      const { return m_VTPtr; }
  void                    SetBaseClass()        { m_isBaseClass = true; }
  ClassInfo_co18*         m_HashNext;
private :
  ClassID               m_guid;
  tsp00_C64             m_ClassName;
  tgg00_ObjColDesc      m_InvColDesc;
  tgg00_StackEntry      m_KeyDesc; 
  size_t                m_ObjectSize;
  void*                 m_VTPtr;
  const ClassInfo_co18* m_pBaseClass;
  bool                  m_isBaseClass;
  bool                  m_isDerivedClass;
  bool                  m_isVarObject;
  bool                  m_isKeyedObject;
};

struct ContainerInfo_co18;

struct ContainerDumpInfo_co18 
{
  void*                m_addr;
  ContainerInfo_co18*  m_clsidHashNext;
  ContainerInfo_co18*  m_guidHashNext;
  ClassInfo_co18*      m_clsInfo;
  tsp00_Int4           m_ContainerNo;
  tsp00_Int4           m_schema;
  tsp00_Int4           m_PersistentSize;
  short                m_refCnt;
  bool                 m_dropped;
  bool                 m_filler;
  tgg00_ContainerId    m_ContainerId;
};

struct ContainerInfo_co18 {
public :
  ContainerInfo_co18*  m_clsidHashNext;
  ContainerInfo_co18*  m_guidHashNext;
  ClassInfo_co18*      m_clsInfo;
  OmsContainerNo       m_ContainerNo;
  OmsSchemaHandle      m_schema;
  tgg00_ContainerId    m_ContainerId;
  size_t               m_PersistentSize;
  bool                 m_dropped;
  short                m_refCnt;
  ContainerInfo_co18 ( ClassInfo_co18* pClassInfo, const tgg00_ContainerId&, tsp00_Uint4 containerHandle, 
    size_t psz, OmsSchemaHandle sh, OmsContainerNo cno); 
  void* operator new(size_t sz);
  void           DeleteSelf();
#if defined(PLACEMENT_DELETE_MEO00)
  void  operator delete (void* p);
#endif
  inline short            DecRefCnt();
  void                    Dump(ContainerDumpInfo_co18&);
  const ClassInfo_co18*   GetClassInfoPtr()    const { return m_clsInfo; }
  tsp00_Uint4             GetContainerHandle() const { return m_ContainerId.fileObjFileNo_gg00(); }
  const ClassIDRef        GetGuid()            const { return m_clsInfo->GetGuid(); }
  const tgg00_ObjColDesc& GetInvColDesc()      const { return m_clsInfo->GetInvColDesc(); }
  const tgg00_StackEntry& GetKeyDesc()         const { return m_clsInfo->GetKeyDesc(); }  
  tsp00_Int4              GetObjectSize()      const { return (tsp00_Int4) m_clsInfo->GetObjectSize(); }
  OmsSchemaHandle         GetSchema()          const { return m_schema; }
  void*                   GetVirtualTablePtr() const { return m_clsInfo->GetVTPtr(); }
  void                    IncRefCnt()          { ++m_refCnt; }
  inline void             IncRefCnt(IliveCacheSink* lcSink);
  bool                    IsDropped()          const { return m_dropped; } 
  void                    MarkDropped()        { m_dropped = true; }
};

typedef ContainerInfo_co18* pContainerInfo_co18;
typedef ClassInfo_co18*     pClassInfo_co18;

class CContainerDictionary_co18 : public IGuidHash_co001 {
  enum { MaxHeadEntries = 2013 };
  tsp00_Uint4         m_dropId;
  pClassInfo_co18     m_classInfoHead[MaxHeadEntries];
  pContainerInfo_co18 m_clsidHead[MaxHeadEntries];
  pContainerInfo_co18 m_guidHead [MaxHeadEntries];
  ClassInfo_co18*     FindGuid (const ClassIDRef guid); 
  ContainerInfo_co18* FindGuid (const ClassIDRef guid, OmsSchemaHandle sh, OmsContainerNo cno);
  void                Insert   (ContainerInfo_co18* pClassInfo);
  void                Init();
public :
  class ContainerIterator {
   CContainerDictionary_co18* m_clsDir;
    int                     m_index;
    pContainerInfo_co18     m_curr;
  public :
    ContainerIterator() : m_clsDir(NULL), m_index(0), m_curr(NULL) {}
    ContainerIterator(CContainerDictionary_co18*);
    void reset(CContainerDictionary_co18*);
    void operator++();
    inline operator bool() const;
    inline pContainerInfo_co18 operator()() const;
  };

  CContainerDictionary_co18();
  void                Delete (IliveCacheSink* lcSink, ContainerInfo_co18* pContainerInfo);
  tgg00_BasisError    DropContainer          (IliveCacheSink* lcSink, const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo cno, size_t arrayByteSize);
  tgg00_BasisError    DropSchema             (IliveCacheSink* lcSink, OmsSchemaHandle Schema);
  void                Dump                   (IDump_cco02& dumpObj);
  ClassInfo_co18*     FindGuid               (IliveCacheSink* lcSink, const ClassIDRef guid); 
  ContainerInfo_co18* FindViaContainerHandle (IliveCacheSink* lcSink, tsp00_Uint4 containerHandle);
  tsp00_Uint4         GetLastDropId() const  { return m_dropId; }
  ContainerInfo_co18* FindViaGuid (IliveCacheSink* lcSink, const ClassIDRef guid, OmsSchemaHandle sh, OmsContainerNo cno);
  ClassInfo_co18* RegisterClass (IliveCacheSink* lcSink, 
                                 const char* ClassName,
                                 const ClassIDRef guid,
                                 const ClassIDPtr pBaseClass,
                                 const tgg00_ObjColDesc &ColDesc,
                                 size_t ObjectSize, 
                                 void* vtptr);
  ContainerInfo_co18* RegisterContainer (IliveCacheSink* lcSink, 
                                         const ClassIDRef guid, 
                                         const char* ClassName, 
                                         size_t PersistentSize, 
                                         size_t ObjectSize, 
                                         const ClassIDPtr pBaseClass,
                                         const tgg00_ObjColDesc &ColDesc,
                                         OmsSchemaHandle sh,
                                         OmsContainerNo cno,
                                         void* vtptr,
                                         size_t arrayByteSize = 0);

  void ReleaseContainerInfo(IliveCacheSink* lcSink, pContainerInfo_co18);
  void Shutdown();
  friend class CContainerDictionary_co18::ContainerIterator;
};

/*----------------------------------------------------------------------------------*/

inline CContainerDictionary_co18::ContainerIterator::operator bool() const {
  return (NULL != m_curr);
}

/*----------------------------------------------------------------------------------*/

inline pContainerInfo_co18 CContainerDictionary_co18::ContainerIterator::operator()() const {
  return m_curr;
}

/*----------------------------------------------------------------------------------*/

inline short ContainerInfo_co18::DecRefCnt() {
  --m_refCnt;
  return m_refCnt; 
}

/*----------------------------------------------------------------------------------*/

inline void ContainerInfo_co18::IncRefCnt(IliveCacheSink* lcSink) {
  CriticalSection_co001 cs(lcSink, RGN_CLASS_DIR);
  ++m_refCnt;
  OMS_TRACE(omsTrContainerDir, lcSink, "ContainerInfo::IncRefCnt " << m_refCnt);
  OMS_TRACE(omsTrContainerDir, lcSink, "Name : " << GetClassInfoPtr()->GetClassName() << "Schema : " << GetSchema() << "CNo : "  << m_ContainerNo);
}

#endif