/*!================================================================
 module:        hco19.h
 
 responsible:   ThomasA
 
 special area:  OMS
 
 description:   Oms Monitor   
 
 see:
 
 copyright:     SAP AG 1998 


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 ===================================================================*/

#ifndef _HCO19_H
#define _HCO19_H

#include "hco16.h"
#include "ggg00.h"
#include "ggg250.h"

class CompareMonitorKey_co19;
class co90_Monitor;

/*!
  class :  MonitorKey_co19

  description: defines the key of a monitoring info of a method of an instance
               of the class identified by its iid. the Method is identified by
               its corresponding dispid.
*/
class MonitorKey_co19 {
public :
/*
  Function:     
  description:  MonitorKey_co19
    constructor, initializes the key.

  arguments:
    iid   [in]  : the CLSID (GUID) of the com object.
    dispid[in]  : the index of the method in the vtable
  return value: none
*/
  MonitorKey_co19(const CLSID& iid, short dispId) : m_iid(iid), m_dispid(dispId) {}
/*
  Function: GetIid     
  description:  yields the iid.

  arguments: none
  return value: iid
*/
  const CLSID& GetIid()    const { return m_iid; }
/*
  Function: GetDispId     
  description:  yields the dispid.

  arguments: none
  return value: dispid 
*/
  short        GetDispId() const { return m_dispid; }
private :
  friend class CompareMonitorKey_co19;
  CLSID m_iid;
  short m_dispid;
};
/*! endclass: MonitorKey_co19 */


/*!
  class :  CompareMonitorKey_co19

  description: defines comparism of monitor keys
*/
class CompareMonitorKey_co19 {
public :
  int Compare(const MonitorKey_co19& k1, const MonitorKey_co19& k2);
};
/*! endclass: CompareMonitorKey_co19 */

/*!
  class :  Monitor_co19

  description: implements the monitor dictionary. This dictionary holds monitor data of
               com instances, that have already been destructed.
               The data is stored as rows of an AVL-Tree using a key structure defined by
               MonitorKey_co19. 
*/
class Monitor_co19 {
public :
  class Iterator : public cgg250AvlInfoTree<MonitorKey_co19,tgg00_COMMonitorInfo,CompareMonitorKey_co19,co16_SharedMemAllocator>::Iterator
  {
  public :
    inline void Get(CLSID& iid, tsp00_Int4& dispid, tgg00_COMMonitorInfo& monInfo);
  };
/*
  Function:      Monitor_co19
  description:   constructor

  arguments  :  none
*/
  Monitor_co19();
/*
  Function:     Insert
  description:  Inserts the monitor data into the monitor dictionary

  arguments  :  monitor, holds the monitor data of a com instance
*/
  void   Insert(co90_Monitor* monitor);
/*
  Function     : begin
  description  : returns an iterator for the monitor dictionary

  arguments    : none
  return value : iterator
*/
  inline Iterator begin(); 
/*
  Function     : Reset
  description  : clears the monitor dictionary

  arguments    : none
*/
  void   Reset();
private :
  void Insert(const CLSID& iid, short dispId, tgg00_COMMonitorInfo& info);
  cgg250AvlInfoTree<MonitorKey_co19,tgg00_COMMonitorInfo,CompareMonitorKey_co19,co16_SharedMemAllocator> m_monitorDir;
};
/*! endclass: Monitor_co19 */

inline Monitor_co19::Iterator  Monitor_co19::begin() 
{
  cgg250AvlInfoTree<MonitorKey_co19,tgg00_COMMonitorInfo,CompareMonitorKey_co19,co16_SharedMemAllocator>::Iterator
    iter = m_monitorDir.First(); 
  return *REINTERPRET_CAST(Iterator*, &iter);
}

inline void Monitor_co19::Iterator::Get(CLSID& iid, tsp00_Int4& dispid, tgg00_COMMonitorInfo& monInfo) {
  iid    = this->operator()()->GetIid();
  dispid = this->operator()()->GetDispId();
  memcpy (&monInfo, this->GetInfo(), sizeof(monInfo));
}

#endif