/*


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end


*/

#include "lcbasetypes.h" /* no check */

static const   CLSID CLSID_Intro =
{0xA435CDA8, 0xA24E, 0x11D3,
{0xA9, 0x96, 0x00, 0xA0, 0xC9, 0x43, 0x11, 0xA5}};

static const   IID   IID_Intro =
{0xFC2F8867, 0x6983, 0x11D2,
{0xA9, 0x7F, 0x00, 0xA0, 0xC9, 0x43, 0x11, 0xA5}};

typedef void ( __stdcall *CO_INTROSPECTFUNCTION) ( 
  unsigned short       ParmIdx,
  char                 **lplpMode,
  char                 **lplpIOtype,
  short int            *lpDatatype,
  short int            *lpSubdatatype,
  short int            *lpLength,
  short int            *lpDim,
  long                 *lpOffset,
  unsigned char        *lpFrac,
  LPGUID               *lplpGUID,
  char                 **lplpParmName);

typedef struct
{
  unsigned short        FuncIdx;
  char                  *lpMethod;
  unsigned short        ParamCount;
  CO_INTROSPECTFUNCTION lpFunction;
} t_FuncBufDesc;

typedef struct
{
  char                 *Mode;
  char                 *IOtype;
  short int            Datatype;
  short int            Subdatatype;
  short int            Length;
  short int            Dim;
  long  int            Offset;
  unsigned char        Frac;
  GUID                 guid;
  char                 *lpParmName;
} t_ParmsDesc;

static void __stdcall co_IntrospectFunction1 (
  unsigned short       ParmIdx,
  char                 **lplpMode,
  char                 **lplpIOtype,
  short int            *lpDatatype,
  short int            *lpSubdatatype,
  short int            *lpLength,
  short int            *lpDim,
  long                 *lpOffset,
  unsigned char        *lpFrac,
  LPGUID               *lplpGUID,
  char                 **lplpParmName)
{
  static const t_ParmsDesc CO_ParmDesc[] = 
  {
   (char*)"OPT", (char*)"IN", 28, 201, 0, 250, 0, 0, {0xEE4B31FA, 0xCF89, 0x11D1, {0xA9, 0x55, 0x00, 0xA0, 0xC9, 0x43, 0x07, 0x31}}, (char*)"DllName",
   (char*)"OPT", (char*)"IN", 206, 29, 0, 1, 0, 0, {0xB1397DF1, 0x9962, 0x11D1, {0xA9, 0x4F, 0x00, 0xA0, 0xC9, 0x43, 0x07, 0x31}}, (char*)"Handle",
   (char*)"MAN", (char*)"I/O", 17, 0, 0, 0, 0, 0, {0x00000000, 0x0000, 0x0000, {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}}, (char*)"",
   (char*)"MAN", (char*)"IN", 22, 0, 0, 0, 0, 0, {0x00000000, 0x0000, 0x0000, {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}}, (char*)"regDll"
  };

  if (( ParmIdx > (sizeof(CO_ParmDesc) / sizeof(t_ParmsDesc))) || ( ParmIdx < 1 ))
  {
    *lplpMode = NULL;
  }
  else
  {
    *lplpMode       = CO_ParmDesc[ParmIdx - 1].Mode;
    *lplpIOtype     = CO_ParmDesc[ParmIdx - 1].IOtype;
    *lpDatatype     = CO_ParmDesc[ParmIdx - 1].Datatype;
    *lpSubdatatype  = CO_ParmDesc[ParmIdx - 1].Subdatatype;
    *lpLength       = CO_ParmDesc[ParmIdx - 1].Length;
    *lpDim          = CO_ParmDesc[ParmIdx - 1].Dim;
    *lpOffset       = CO_ParmDesc[ParmIdx - 1].Offset;
    *lpFrac         = CO_ParmDesc[ParmIdx - 1].Frac;
    *lplpGUID       = (GUID*)&CO_ParmDesc[ParmIdx - 1].guid;
    *lplpParmName   = CO_ParmDesc[ParmIdx - 1].lpParmName;
  }
};


static void __stdcall co_IntrospectFunction2 (
  unsigned short       ParmIdx,
  char                 **lplpMode,
  char                 **lplpIOtype,
  short int            *lpDatatype,
  short int            *lpSubdatatype,
  short int            *lpLength,
  short int            *lpDim,
  long                 *lpOffset,
  unsigned char        *lpFrac,
  LPGUID               *lplpGUID,
  char                 **lplpParmName)
{
  static const t_ParmsDesc CO_ParmDesc[] = 
  {
   (char*)"OPT", (char*)"IN", 28, 201, 0, 250, 0, 0, {0x00000000, 0x0000, 0x0000, {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}}, (char*)"pProgId"
  };

  if (( ParmIdx > (sizeof(CO_ParmDesc) / sizeof(t_ParmsDesc))) || ( ParmIdx < 1 ))
  {
    *lplpMode = NULL;
  }
  else
  {
    *lplpMode       = CO_ParmDesc[ParmIdx - 1].Mode;
    *lplpIOtype     = CO_ParmDesc[ParmIdx - 1].IOtype;
    *lpDatatype     = CO_ParmDesc[ParmIdx - 1].Datatype;
    *lpSubdatatype  = CO_ParmDesc[ParmIdx - 1].Subdatatype;
    *lpLength       = CO_ParmDesc[ParmIdx - 1].Length;
    *lpDim          = CO_ParmDesc[ParmIdx - 1].Dim;
    *lpOffset       = CO_ParmDesc[ParmIdx - 1].Offset;
    *lpFrac         = CO_ParmDesc[ParmIdx - 1].Frac;
    *lplpGUID       = (GUID*)&CO_ParmDesc[ParmIdx - 1].guid;
    *lplpParmName   = CO_ParmDesc[ParmIdx - 1].lpParmName;
  }
};


static void __stdcall co_IntrospectFunction3 (
  unsigned short       ParmIdx,
  char                 **lplpMode,
  char                 **lplpIOtype,
  short int            *lpDatatype,
  short int            *lpSubdatatype,
  short int            *lpLength,
  short int            *lpDim,
  long                 *lpOffset,
  unsigned char        *lpFrac,
  LPGUID               *lplpGUID,
  char                 **lplpParmName)
{
  static const t_ParmsDesc CO_ParmDesc[] = 
  {
   (char*)"OPT", (char*)"IN", 28, 201, 0, 250, 0, 0, {0xEE4B31FA, 0xCF89, 0x11D1, {0xA9, 0x55, 0x00, 0xA0, 0xC9, 0x43, 0x07, 0x31}}, (char*)"Path"
  };

  if (( ParmIdx > (sizeof(CO_ParmDesc) / sizeof(t_ParmsDesc))) || ( ParmIdx < 1 ))
  {
    *lplpMode = NULL;
  }
  else
  {
    *lplpMode       = CO_ParmDesc[ParmIdx - 1].Mode;
    *lplpIOtype     = CO_ParmDesc[ParmIdx - 1].IOtype;
    *lpDatatype     = CO_ParmDesc[ParmIdx - 1].Datatype;
    *lpSubdatatype  = CO_ParmDesc[ParmIdx - 1].Subdatatype;
    *lpLength       = CO_ParmDesc[ParmIdx - 1].Length;
    *lpDim          = CO_ParmDesc[ParmIdx - 1].Dim;
    *lpOffset       = CO_ParmDesc[ParmIdx - 1].Offset;
    *lpFrac         = CO_ParmDesc[ParmIdx - 1].Frac;
    *lplpGUID       = (GUID*)&CO_ParmDesc[ParmIdx - 1].guid;
    *lplpParmName   = CO_ParmDesc[ParmIdx - 1].lpParmName;
  }
};


static void __stdcall co_IntrospectFunction4 (
  unsigned short       ParmIdx,
  char                 **lplpMode,
  char                 **lplpIOtype,
  short int            *lpDatatype,
  short int            *lpSubdatatype,
  short int            *lpLength,
  short int            *lpDim,
  long                 *lpOffset,
  unsigned char        *lpFrac,
  LPGUID               *lplpGUID,
  char                 **lplpParmName)
{
    *lplpMode = NULL;
};


static void __stdcall co_IntrospectFunction5 (
  unsigned short       ParmIdx,
  char                 **lplpMode,
  char                 **lplpIOtype,
  short int            *lpDatatype,
  short int            *lpSubdatatype,
  short int            *lpLength,
  short int            *lpDim,
  long                 *lpOffset,
  unsigned char        *lpFrac,
  LPGUID               *lplpGUID,
  char                 **lplpParmName)
{
  static const t_ParmsDesc CO_ParmDesc[] = 
  {
   (char*)"MAN", (char*)"OUT", 26, 3, 0, 0, 0, 0, {0x00000000, 0x0000, 0x0000, {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}}, (char*)"chptId"
  };

  if (( ParmIdx > (sizeof(CO_ParmDesc) / sizeof(t_ParmsDesc))) || ( ParmIdx < 1 ))
  {
    *lplpMode = NULL;
  }
  else
  {
    *lplpMode       = CO_ParmDesc[ParmIdx - 1].Mode;
    *lplpIOtype     = CO_ParmDesc[ParmIdx - 1].IOtype;
    *lpDatatype     = CO_ParmDesc[ParmIdx - 1].Datatype;
    *lpSubdatatype  = CO_ParmDesc[ParmIdx - 1].Subdatatype;
    *lpLength       = CO_ParmDesc[ParmIdx - 1].Length;
    *lpDim          = CO_ParmDesc[ParmIdx - 1].Dim;
    *lpOffset       = CO_ParmDesc[ParmIdx - 1].Offset;
    *lpFrac         = CO_ParmDesc[ParmIdx - 1].Frac;
    *lplpGUID       = (GUID*)&CO_ParmDesc[ParmIdx - 1].guid;
    *lplpParmName   = CO_ParmDesc[ParmIdx - 1].lpParmName;
  }
};


static void __stdcall co_IntrospectFunction6 (
  unsigned short       ParmIdx,
  char                 **lplpMode,
  char                 **lplpIOtype,
  short int            *lpDatatype,
  short int            *lpSubdatatype,
  short int            *lpLength,
  short int            *lpDim,
  long                 *lpOffset,
  unsigned char        *lpFrac,
  LPGUID               *lplpGUID,
  char                 **lplpParmName)
{
  static const t_ParmsDesc CO_ParmDesc[] = 
  {
   (char*)"MAN", (char*)"IN", 3, 0, 0, 0, 0, 0, {0x00000000, 0x0000, 0x0000, {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}}, (char*)"chptId"
  };

  if (( ParmIdx > (sizeof(CO_ParmDesc) / sizeof(t_ParmsDesc))) || ( ParmIdx < 1 ))
  {
    *lplpMode = NULL;
  }
  else
  {
    *lplpMode       = CO_ParmDesc[ParmIdx - 1].Mode;
    *lplpIOtype     = CO_ParmDesc[ParmIdx - 1].IOtype;
    *lpDatatype     = CO_ParmDesc[ParmIdx - 1].Datatype;
    *lpSubdatatype  = CO_ParmDesc[ParmIdx - 1].Subdatatype;
    *lpLength       = CO_ParmDesc[ParmIdx - 1].Length;
    *lpDim          = CO_ParmDesc[ParmIdx - 1].Dim;
    *lpOffset       = CO_ParmDesc[ParmIdx - 1].Offset;
    *lpFrac         = CO_ParmDesc[ParmIdx - 1].Frac;
    *lplpGUID       = (GUID*)&CO_ParmDesc[ParmIdx - 1].guid;
    *lplpParmName   = CO_ParmDesc[ParmIdx - 1].lpParmName;
  }
};


static void __stdcall co_IntrospectFunction7 (
  unsigned short       ParmIdx,
  char                 **lplpMode,
  char                 **lplpIOtype,
  short int            *lpDatatype,
  short int            *lpSubdatatype,
  short int            *lpLength,
  short int            *lpDim,
  long                 *lpOffset,
  unsigned char        *lpFrac,
  LPGUID               *lplpGUID,
  char                 **lplpParmName)
{
    *lplpMode = NULL;
};


static void __stdcall co_IntrospectFunction8 (
  unsigned short       ParmIdx,
  char                 **lplpMode,
  char                 **lplpIOtype,
  short int            *lpDatatype,
  short int            *lpSubdatatype,
  short int            *lpLength,
  short int            *lpDim,
  long                 *lpOffset,
  unsigned char        *lpFrac,
  LPGUID               *lplpGUID,
  char                 **lplpParmName)
{
    *lplpMode = NULL;
};

static void __stdcall co_IntrospectFunction9 (
  unsigned short       ParmIdx,
  char                 **lplpMode,
  char                 **lplpIOtype,
  short int            *lpDatatype,
  short int            *lpSubdatatype,
  short int            *lpLength,
  short int            *lpDim,
  long                 *lpOffset,
  unsigned char        *lpFrac,
  LPGUID               *lplpGUID,
  char                 **lplpParmName)
{
    *lplpMode = NULL;
};

#define PROCEDURE_NUM 9
#define VERSION_NUMBER 1
