/*


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end


*/
&ifdef KERNEL
/*!*****************************************************************************

  module:       hco31.h

  ------------------------------------------------------------------------------

  responsible:  ThomasA

  lastChanged:  2000-01-19

  purpose:      definition of liveCache error class

  generatedBy:

  ------------------------------------------------------------------------------

  copyright: Copyright by SAP AG, 1999

********************************************************************************/
&else
/********************************************************************************
*
* Copyright by SAP AG, 1999
*
********************************************************************************/
&endif

&ifdef KERNEL
#ifndef HCO31_H
#define HCO31_H


#include "ggg92.h"
#include "gco00.h"
#include "ilivecachesink.h" /* no check */
#include "gco01.h"

/* PTS 1115295 FF 2002-04-19 */
#ifdef KERNEL
#undef AFX_EXT_CLASS
#define AFX_EXT_CLASS
#endif

struct AFX_EXT_CLASS OmsObjectId : public tgg92_OmsObjectId
{
public :

  OmsObjectId()
  {
    setNil();
  };

  OmsTypeOid* pOid() {
    return REINTERPRET_CAST (OmsTypeOid*, &m_oid);
  }
protected :
  OmsObjectId( unsigned short pos, unsigned int l )
  { m_oid.pagePos = pos; setPno(l); m_oid.generation = NIL_OBJ_VERSION; }
};
&else
#ifndef _LIVECACHE_DBPERROR
#define _LIVECACHE_DBPERROR

enum {
    E_INCOMPATIBLE_OID   = -28007,
    E_NUMBER_OVERFLOW    = -28008
};

struct OmsObjectId;
&endif

/*!
 * \brief Base Class for all errors thrown in OMS
 *
 * \note All classes thrown in a db-procedure should be derived from 
 *  DbpErrorBase.
 */
class AFX_EXT_CLASS DbpErrorBase
{
public :
  DbpErrorBase() {}
};


/// Interface for checking the DbpError in DbpError constructor
class AFX_EXT_CLASS DbpErrorCallbackInterface {
public:
  /*!
   * \brief Check exception before throwing it.
   *
   * This method should process the exception before it is thrown, e.g.,
   * by writing a message and a stack trace into log file or by triggering
   * a debug breakpoint for certain exceptions.
   *
   * Define in an own handler class derived from this one.
   *
   * \see DbpError::dbpSetCallback
   * \see \ref omsdebug_cb
   */
  virtual void dbpCheckError(const class DbpError &e) = 0;
};


/// Exception class for SQL-Class and OMS
class AFX_EXT_CLASS DbpError : public DbpErrorBase
{
public :
  enum DbpErrorKinds { DB_ERROR, HRESULT_ERROR, RTE_ERROR, USER_DEFINED,
  DB_ERROR_UNICODE, USER_DEFINED_UNICODE };

&ifdef KERNEL
  DbpError (DbpErrorKinds ErrorKind, long ErrorNo);
  DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const char* errtext);
  DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const DbpTypeUnicode* errtext);
  DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const char* errtext, const OmsObjectId& oid);
  DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const DbpTypeUnicode* errtext, const OmsObjectId& oid);
&endif
  DbpError (long ErrorNo);
  DbpError (long ErrorNo, const char* errtext);
  DbpError (long ErrorNo, const DbpTypeUnicode* errtext);
  long  dbpError () const;
  short dbpDBError () const;
  short dbpRteError () const;
  long  dbpHresultError() const;
&ifndef KERNEL
#if defined(SAPDB_UNICODE)
&endif
  const OmsTypeWyde* dbpErrorTextWyde() const;
&ifndef KERNEL
#else
&endif
  const char* dbpErrorText() const;
&ifndef KERNEL
#endif
&endif
  const DbpTypeUnicode* dbpErrorTextUnicode() const;
  const OmsObjectId& dbpErrorOid() const;

  /// Set a new callback interface to pre-process the exception
  static void dbpSetCallback(DbpErrorCallbackInterface *cb);
  /// Query current callback interface
  static DbpErrorCallbackInterface *dbpGetCallback();

  DbpErrorKinds m_errorKind;
  mutable bool  m_isWyde;
  int           m_errorNo;
&ifdef KERNEL
  OmsObjectId   m_oid;
&else
  OmsTypeOid    m_oid;
&endif
  mutable char  m_errorText[256];
};
#endif