/* @lastChanged: "1999-04-21  16:14"

  * @filename:   vco10.cpp
  * @purpose:    "Implementation of C++ Persistent Object Interface"
  * @release:    7.2.0.0
  * @see:        "http://www.bea.sap-ag.de/doku/rel-7.1/vco10.html"
  *
  * @copyright:  "Copyright by   SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/


/* ================================================================================ */
/* ==========              REPLACEMENTS BY THE PREPROCESSOR               ========= */
/* ================================================================================ */



/* ================================================================================ */
/* ==========                 DECLARATION OF TYPES                        ========= */
/* ================================================================================ */




/* ================================================================================ */
/* ==========                 DECLARATION OF FUNCTIONS                    ========= */
/* ================================================================================ */

#define EXTERN_CO102
#include "gco102.h"
#include "geo00.h"
#include "geo573.h"
#include "gco00.h"
#include "ggg00.h"
#include "gco10.h"
#include "hco11.h"
#include "gco11ISessionLockObjects.h"
#include "ggg200.h"
#include "hco17.h"
#include "hco18.h"
#include "hco19.h"
#include "RunTime/Synchronisation/RTESync_Spinlock.hpp"
#include "RunTime/System/RTESys_Time.h"
#include "hsp78.h"    
#include "hsp77.h"     // PTS 1125307

#include "SAPDBCommon/MemoryManagement/SAPDBMem_IRawAllocator.hpp"
#include "Oms/OMS_LibOmsInterfaceInstance.hpp"
#include "Oms/OMS_KernelDefaultInterface.hpp"
#include "Oms/OMS_DummyKernelCallback.hpp"
#include "liveCache/LVC_KernelInterface.hpp"
#include "SAPDBCommon/SAPDB_UTF8Basis.hpp"
#include "Oms/OMS_OidArrayReader.hpp"
#include "Oms/OMS_OidArrayWriter.hpp"
#include "liveCache/LVC_LockRequest.hpp"

/*---------------------------------------------------------------------------*/
/*  external Globals                                                         */
/*---------------------------------------------------------------------------*/

externCpp bool omsInUnicodeMode = false;

/*---------------------------------------------------------------------------*/
/*                              Globals                                      */
/*---------------------------------------------------------------------------*/

const char* VAR_OBJ_CLASS_NAME_CO10 = "VarObjContainer";

static OMS_DummyKernelCallback DummySink_co10;
static IliveCacheSink*         ProxySinkAddr_co10() { return &DummySink_co10; }

static GetSinkPtr_co02         GetSinkAddrFunc_co10 = ProxySinkAddr_co10;
static LVC_KernelInterface*    KernelInterface_co10 = NULL;
static bool                    co10_monitor_malloc  = false;  // PTS 1110287

class CResetSink {
public :
  CResetSink() {}
  ~CResetSink() {
    if (NULL != KernelInterface_co10)
    {
      KernelInterface_co10->SignalUnloadLiboms();
    }
    GetSinkAddrFunc_co10 = ProxySinkAddr_co10;
  }
};

class CGlob_co10 {
public :
  CGlob_co10();
  ~CGlob_co10() 
  {
  }
  void Dump(IDump_cco02& dumpObj);
  bool DumpRequested(tsp00_Int4 errorNo);
  void DumpVersionDirectory(IDump_cco02& dumpObj);
  bool InSimulator() const;
  bool IsOmsTestLib() const { return m_isOmsTestLib;} 
  void SetDumpError(tsp00_Int4);
  void SetHost(bool isKernel);
  void SetOmsVersionThreshold(tsp00_Uint4 threshold); /* PTS 1110149 */
  tsp00_Uint4 GetOmsVersionThreshold() const          /* PTS 1110149 */
  {
    return m_omsVersionThreshold;
  }
  ITrace_co02* GetTracer();
  void SetTracer(ITrace_co02* pTraceObj);
  void ResetInfo()
  {
    // TODO: this is not perfect, but it will do the job...
    VersionDictionary.Shutdown();
    ClassDictionary.Shutdown();
  }
#if defined(_IBMR2) && defined (BIT64)
  // PTS 1107999
//  void* operator new(size_t sz, CGlob_co10& g)  { return &g; }
#endif
  //
  // caution : order of member variables is important !!
  //

  CVersionDictionary_co17    VersionDictionary;
  CContainerDictionary_co18  ClassDictionary;
  Monitor_co19               Monitor;
  // global version iterator, used for 'select .. from sysdd.oms_versions
  CVersionDictionary_co17::Iter  VersionIter;  // PTS 1124170
  // global container iterator
  CContainerDictionary_co18::ContainerIterator	ContainerIter;
private :
  enum { MAX_DUMP_ERRORS = 64 };
  enum LibHost
  {
      HostUnknown,
      HostKernel,
      HostSimulator
  };
  bool               m_isOmsTestLib;
  LibHost            m_host;
  int                m_cntDumpErrors;
  int                m_heapThreshold;       /* PTS 1110149 */
  tsp00_Uint4        m_omsVersionThreshold; /* PTS 1110149 */
  ITrace_co02*       m_tracer;
  RTESync_Spinlock   m_syncDumpError;
  tsp00_Int4         m_dumpOnError[MAX_DUMP_ERRORS];

  // ResetSink must be last member, because the destructor ensures, that the
  // destructors of the other members don't ask for the kernel environment !
  CResetSink       ResetSink;
};

static CGlob_co10                  *Glob_co10 = NULL;
static double                      Glob_co10_space[sizeof(CGlob_co10) / sizeof(double) + 1];
static OMS_LibOmsInterfaceInstance *LibOmsInterface_co10 = NULL;
static void                        *LibOmsInterface_co10_Place = NULL;

/* ================================================================================ */
/* ==========                 DEFINITION OF GLOBAL FUNCTIONS              ========= */
/* ================================================================================ */

/*----------------------------------------------------------------------*/
// PTS 1124170
int co10_GetLockIdForVersionDirectory (const OmsVersionId &versionId){
  return Glob_co10->VersionDictionary.GetSingleLockId(versionId);
}

/*----------------------------------------------------------------------*/
// PTS 1124170
bool co10_GetLockMethodForVersionDictionary(){
  return Glob_co10->VersionDictionary.UseRWLocks();
}

/*----------------------------------------------------------------------*/

externC void co10_initSingletons()
{
  // initialize all singletons to sensible values - on load in xregcomp
  if (Glob_co10 == NULL) Glob_co10 = new(&Glob_co10_space) CGlob_co10();
  if (KernelInterface_co10 == NULL) KernelInterface_co10 = &OMS_KernelDefaultInterface::Instance();
  if (LibOmsInterface_co10 == NULL)
    LibOmsInterface_co10 = new(&LibOmsInterface_co10_Place) OMS_LibOmsInterfaceInstance();

}

/*----------------------------------------------------------------------*/

tsp00_Int4 co10_clock() // PTS 1110287
{
  return KernelInterface_co10->Clock();
}

/*----------------------------------------------------------------------*/

void* co10_malloc (size_t size)
{
  return KernelInterface_co10->OmsAllocate(size);
}

/*----------------------------------------------------------------------*/

void co10_DetachContainerInfo(IliveCacheSink* lcSink, pContainerInfo_co18 p) {
  Glob_co10->ClassDictionary.ReleaseContainerInfo (lcSink, p);
}

/*----------------------------------------------------------------------*/

bool co10_dumpRequested(tsp00_Int4 errorNo)
{
  return Glob_co10->DumpRequested(errorNo);
}

/*----------------------------------------------------------------------*/

void co10_free(void* p) {
  if (NULL == p) { 
    return;
  }
  KernelInterface_co10->OmsDeallocate(p);
 }

/*----------------------------------------------------------------------*/

LVC_KernelInterface* co10_GetKernelInterface()
{
  if (KernelInterface_co10 == NULL) KernelInterface_co10 = &OMS_KernelDefaultInterface::Instance();
  return KernelInterface_co10;
}

/*----------------------------------------------------------------------*/

IliveCacheSink* co10_GetLcSink() {
  return KernelInterface_co10->GetSinkPtr();
}

/*----------------------------------------------------------------------*/
/* PTS 1115231, 1120474 */
externCpp
bool omsIsUnicodeInstance()
{
   IliveCacheSink* pSink = co10_GetLcSink();
   bool  isUnicodeInstance = false;
   if (pSink)
   {
       short error;
       pSink->IsUnicodeInstance(isUnicodeInstance, error);
       if (0 != error)
       {
           throw DbpError(error, "omsIsUnicodeInstance");
       }
   }
   return isUnicodeInstance;
}

/*----------------------------------------------------------------------*/

externCpp
void omsGetLiveCacheVersion(char* pBuf, int bufSize)
{
  if (NULL == KernelInterface_co10->GetKnlVersion(pBuf, bufSize))
  {
    throw DbpError (e_buffer_too_small, "omsGetLiveCacheVersion");
  }
}

/*----------------------------------------------------------------------*/

externCpp
void omsSimConsoleMessage(const char* msg)
{
  if (KernelInterface_co10) KernelInterface_co10->ConsoleMessage(msg);
}

/*----------------------------------------------------------------------*/

externCpp 
void omsGetLocalDateTimeString(char* pBuf, int bufSize)
{
  if (bufSize < sizeof(RTESys_Timestamp) + 1)
  {
    throw DbpError (e_buffer_too_small, "omsGetLocalDateTime");
  }
  RTESys_FillProtocolTimeStamp(*REINTERPRET_CAST(RTESys_Timestamp*, pBuf));
  pBuf[sizeof(RTESys_Timestamp)] = 0;
}

/*----------------------------------------------------------------------*/

externCpp
void omsGetLocalDateTime(int& year,
                    int& month,
                    int& day,
                    int& hours,
                    int& minutes,
                    int& seconds)
{
  char ts[sizeof(RTESys_Timestamp) + 1];
  omsGetLocalDateTimeString(&ts[0], sizeof(ts));
  sscanf(ts, SAPDB_TIMESTAMP_FORMAT, &year, &month, &day, &hours, &minutes, &seconds);
}

/*----------------------------------------------------------------------*/

externCpp 
void omsGetUTCDateTimeString (char* pBuf, int bufSize)
{
  if (bufSize < sizeof(RTESys_Timestamp) + 1)
  {
    throw DbpError (e_buffer_too_small, "omsGetLocalDateTime");
  }
  RTESys_FillProtocolGMTTimeStamp(*REINTERPRET_CAST(RTESys_Timestamp*, pBuf));
  pBuf[sizeof(RTESys_Timestamp)] = 0;
}

/*----------------------------------------------------------------------*/

externCpp
void omsGetUTCDateTime(int& year,
                    int& month,
                    int& day,
                    int& hours,
                    int& minutes,
                    int& seconds)
{
  char ts[sizeof(RTESys_Timestamp) + 1];
  omsGetUTCDateTimeString(&ts[0], sizeof(ts));
  sscanf(ts, SAPDB_TIMESTAMP_FORMAT, &year, &month, &day, &hours, &minutes, &seconds);
}

/*----------------------------------------------------------------------*/

externCpp
void* omsMalloc(size_t size) 
{
  return KernelInterface_co10->Allocate(size);
}

/*----------------------------------------------------------------------*/

externCpp
void* omsMallocArray(size_t size) 
{
  return KernelInterface_co10->AllocateArray(size);
}

/*----------------------------------------------------------------------*/

externCpp
void omsFree(void* p) 
{
  KernelInterface_co10->Deallocate(p);
}

/*----------------------------------------------------------------------*/

externCpp
void omsFreeArray(void* p) 
{
  KernelInterface_co10->DeallocateArray(p);
}

/*----------------------------------------------------------------------*/

void co10_MethodCallEpilog (OmsHandle& h, void* sql_handle, int runTime)     /* PTS 1120804 */
{
  h.m_pSession->CurrentMethodCallEpilog(runTime);
  OmsSession_co12::MethodCallEplilogScope epilogScope(*h.m_pSession);
  /* only in this scope a call of omsGetMethodCallMonitorInfo returns any result */
  h.omsMethodCallEpilog(sql_handle);     /* PTS 1120804 */
}

/*----------------------------------------------------------------------*/

externC void* co10_GetInterface() {
  if (LibOmsInterface_co10 == NULL)
    LibOmsInterface_co10 = new(&LibOmsInterface_co10_Place) OMS_LibOmsInterfaceInstance();
  return REINTERPRET_CAST(void*, LibOmsInterface_co10);
}

/*----------------------------------------------------------------------*/

void co10_MarkUnloadable(OmsContext_co12* pContext, bool callFromDestructor) {
  Glob_co10->VersionDictionary.MarkUnloadable(pContext, callFromDestructor);
}

/*----------------------------------------------------------------------*/

ContainerInfo_co18* co10_RegisterContainer (IliveCacheSink*        lcSink, 
                                            const ClassIDRef       guid, 
                                            const char*            ClassName, 
                                            size_t                 PersistentSize, 
                                            size_t                 ObjectSize, 
                                            const ClassIDPtr       pBaseClass,
                                            const tgg00_ObjColDesc &ColDesc,
                                            OmsSchemaHandle        sh,
                                            OmsContainerNo         cno,
                                            void*                  vtptr,
                                            size_t                 arrayByteSize)
{
  return Glob_co10->ClassDictionary.RegisterContainer (lcSink, guid, ClassName, PersistentSize,
    ObjectSize, pBaseClass, ColDesc, sh, cno, vtptr, arrayByteSize); 
}

/*----------------------------------------------------------------------*/

void co10_ResetOMSForSimulator()
{
  Glob_co10->ResetInfo();
}

/*----------------------------------------------------------------------*/

void co10_SetCurrMonitorMethod(OmsHandle& h, tgg00_COMMonitorInfo* p) 
{
  h.m_pSession->SetCurrentMonitorMethod(p); // PTS 1107731
}

/*----------------------------------------------------------------------*/

void co10_Signal(tsp00_TaskId TaskId, bool timeout) // PTS 1110287
{
  KernelInterface_co10->Signal(TaskId, timeout);
}

/*----------------------------------------------------------------------*/

void co10_Throw(const DbpError& e)
{
  if (Glob_co10->DumpRequested(e.dbpError()))
  {
    IliveCacheSink* lcSink = co10_GetLcSink();
    short      DummyError;
    tsp00_Int4 errorNo = e.dbpError();
    tsp00_Int4 BufLen  = sizeof(errorNo);
    CHResult_co001 hr  = lcSink->MultiPurpose (m_outcopy, mm_nil, &BufLen, 
      (unsigned char*) &errorNo, &DummyError);
  }
  if (e_new_failed == e.dbpError()) // PTS 1109338
  {
    pasbool*         pToCancel;
    OmsSession_co12* pSession;
    tsp00_TaskId     taskId;
    IliveCacheSink* lcSink = co10_GetLcSink();
    lcSink->GetDefaultContext(REINTERPRET_CAST(void**, &pSession), &pToCancel, taskId);
    if (NULL != pSession)
    {
      pSession->IncOutOfMemory();
    }
  }
  throw e;
}

/*----------------------------------------------------------------------*/

void co10_ToKnlIdentifier (const char* Source, tsp00_KnlIdentifier& Dest) {
  size_t copyLen = strlen(Source);
  if (copyLen > sizeof(Dest)) {
    co10_Throw(DbpError (DbpError::DB_ERROR, -2014, "too long identifier"));
  }
  memcpy(&Dest[0], Source, copyLen);
  memset(&Dest[copyLen], 0, sizeof(Dest) - copyLen);
}

/*----------------------------------------------------------------------*/

//PTS 1117690
// strlen on strings of type OmsTypeWyde
int co10_strlen (const OmsTypeWyde* s){
  int len=-1;
  while (s[++len]);
  return len;
}

/*----------------------------------------------------------------------*/
// PTS 1107993

CGlob_co10::CGlob_co10()  
: VersionDictionary()
, ClassDictionary()
, Monitor()
, VersionIter()
#if defined(OMSTST)
, m_isOmsTestLib(true)
#else
, m_isOmsTestLib(false)
#endif 
, m_host(HostUnknown)
, m_cntDumpErrors(0)
, m_heapThreshold(100)
, m_omsVersionThreshold(MAX_INT4_SP00)
, m_syncDumpError()
, m_tracer(0)
, ResetSink() 
{
}

/*----------------------------------------------------------------------*/

void CGlob_co10::Dump(IDump_cco02& dumpObj)
{
  this->ClassDictionary.Dump(dumpObj);
  this->DumpVersionDirectory(dumpObj);
  OmsSession_co12* pSession = REINTERPRET_CAST(OmsSession_co12*, dumpObj.GetOmsSession());
  if (NULL != pSession)
  {
    pSession->Dump(dumpObj);
  }
}

/*----------------------------------------------------------------------*/

bool CGlob_co10::DumpRequested(tsp00_Int4 errorNo) 
{
  RTESync_LockedScope sync(m_syncDumpError);
  for (int ix = 0; ix < m_cntDumpErrors; ++ix)
  {
    if (m_dumpOnError[ix] == errorNo)
    {
      /* remove error to be sure that error is only dumped once */
      while (ix < m_cntDumpErrors - 1)
      {
        m_dumpOnError[ix] = m_dumpOnError[ix+1];
      }
      --m_cntDumpErrors;
      return true;
    }
  }
  return false;
}

/*----------------------------------------------------------------------*/

void CGlob_co10::DumpVersionDirectory(IDump_cco02& dumpObj)
{
  dumpObj.SetDumpLabel(LABEL_OMS_VERSION);

  for (CVersionDictionary_co17::Iter VersionIter(dumpObj.Synchronize() ? SHARED : NO_LOCK, VersionDictionary); 
       VersionIter; ++VersionIter)
  {
    OmsContext_co12* pContext = VersionIter();
    pContext->Dump(dumpObj);
  }
}

/*----------------------------------------------------------------------*/

bool CGlob_co10::InSimulator() const
{
    return (m_host == HostSimulator);
}

/*----------------------------------------------------------------------*/

void CGlob_co10::SetDumpError(tsp00_Int4 e)
{
  RTESync_LockedScope sync(m_syncDumpError);
  if (m_cntDumpErrors < MAX_DUMP_ERRORS)
  {
    m_dumpOnError[m_cntDumpErrors] = e;
    ++m_cntDumpErrors;
  }
}

/*----------------------------------------------------------------------*/

void CGlob_co10::SetHost(bool isKernel)
{
    m_host = (isKernel) ? HostKernel : HostSimulator;
}

/*----------------------------------------------------------------------*/

void CGlob_co10::SetOmsVersionThreshold(tsp00_Uint4 threshold) /* PTS 1110149 */
{
  // 100 has been wrong default, set to wanted default
  m_omsVersionThreshold = (100 == threshold) ? 2097152 : threshold;
}

/*----------------------------------------------------------------------*/

ITrace_co02* CGlob_co10::GetTracer()
{
  return m_tracer;
}

/*----------------------------------------------------------------------*/

void CGlob_co10::SetTracer(ITrace_co02* pTraceObj)
{
  m_tracer = pTraceObj;
}

/*----------------------------------------------------------------------*/

void OMS_LibOmsInterfaceInstance::AdviseKernelInterface(LVC_KernelInterface& knlInterface, bool isKernel)
{
  KernelInterface_co10 = &knlInterface;

  if (!Glob_co10) Glob_co10 = new(&Glob_co10_space) CGlob_co10();
  Glob_co10->SetHost(isKernel);    
  omsInUnicodeMode = KernelInterface_co10->IsUnicodeInstance();

  for (int i=0; i<OMS_VDIR_SIZE; ++i){
    // Get single lock for every hash-array   // PTS 1124170
    LVC_LockRequest lock(LVC_LockRequest::RWLOCK_CREATE, i + Glob_co10->VersionDictionary.GetVDirStartCnt());
    short err = knlInterface.LockRequest(lock);
  }

  // PTS 1124170
  Glob_co10->VersionDictionary.m_useRWLocks = (knlInterface.UseReaderWriterLocks() ? true : false);
}

/*----------------------------------------------------------------------*/

void OMS_LibOmsInterfaceInstance::InitLibOms(IliveCacheSink* lcSink) 
{
  if (!Glob_co10) Glob_co10 = new(&Glob_co10_space) CGlob_co10();
  struct VirtualObject {
    void* vtptr;
  }; 
  tgg00_ObjColDesc ColDesc;
  OmsVarObjInfo_co12 info;
  ColDesc.colHashCount_gg00 = 0;
  ColDesc.colInvCount_gg00  = 0;
  ColDesc.colHashList_gg00  = NULL;
  ColDesc.colInvList_gg00   = NULL;
  Glob_co10->ClassDictionary.RegisterClass (lcSink, VAR_OBJ_CLASS_NAME_CO10, VAR_OBJ_GUID_GCO10, NULL,
    ColDesc, sizeof(info) + OmsObjectContainer::headerSize(), (REINTERPRET_CAST(VirtualObject*, &info))->vtptr);
  Glob_co10->SetOmsVersionThreshold(KernelInterface_co10->GetOmsVersionThreshold());
  co11_SetTimeout(KernelInterface_co10->GetLockRequestTimeout());
}

/*----------------------------------------------------------------------*/

void OMS_LibOmsInterfaceInstance::AdviseTracer(ITrace_co02* pTraceObj)
{
  if (!Glob_co10) Glob_co10 = new(&Glob_co10_space) CGlob_co10();
  Glob_co10->SetTracer(pTraceObj);
}

/*----------------------------------------------------------------------*/

void OMS_LibOmsInterfaceInstance::Dump(IDump_cco02& dumpObj)
{
  Glob_co10->Dump(dumpObj);
}

/*----------------------------------------------------------------------*/

tsp00_Bool OMS_LibOmsInterfaceInstance::GetFirstVersion(IliveCacheSink*     lcSink,
                                                  tsp00_C24&          versionId,
                                                  tsp00_Date&         createDate,
                                                  tsp00_Time&         createTime,
                                                  tsp00_Date&         openDate,
                                                  tsp00_Time&         openTime,
                                                  tgg91_TransNo&      consistentView,
                                                  tsp00_Bool&         isMarked,
                                                  tsp00_Bool&         isOpen,
                                                  tsp00_Bool&         isUnloaded,
                                                  tsp00_8ByteCounter& heapUsage,
                                                  tsp00_Int4&         hashSize,
                                                  tgg00_FileId&       versionTree,
                                                  tgg00_FileId&       versionInvTree,
                                                  tsp00_C512&         versionDesc,
                                                  short&              error)    // PTS 1126698
{ 
  try {
    // Region is only used to synchonize the access to the global iterator
    lcSink->EnterCriticalSection(RGN_VERSION_DIR);
   
    // Reset the global iterator to the first element
    Glob_co10->VersionIter.first(EXCLUSIVE, Glob_co10->VersionDictionary);
  } catch (DbpError &e){  // PTS 1126698
    Glob_co10->VersionIter.stop();
    lcSink->LeaveCriticalSection(RGN_VERSION_DIR);
    error = e.m_errorNo;
    return false;
  }

  return GetNextVersion(lcSink, versionId, createDate, createTime, 
    openDate, openTime, consistentView, isMarked, isOpen, isUnloaded,
    heapUsage, hashSize, versionTree, versionInvTree, versionDesc, error);
}

/*----------------------------------------------------------------------*/

tsp00_Bool OMS_LibOmsInterfaceInstance::GetMonitorInfo(
                                                 IliveCacheSink*       lcSink,
                                                 tsp00_Addr&           handle,
                                                 tsp00_C16&            iid,
                                                 tsp00_Int4&           dispid,
                                                 tgg00_COMMonitorInfo& monInfo)
{
  static Monitor_co19::Iterator Iter;
  if (NULL == handle) {
    lcSink->EnterCriticalSection(RGN_MONITOR);
    Iter = Glob_co10->Monitor.begin();
    handle = REINTERPRET_CAST(tsp00_Addr, &Iter);
  }
  if (Iter) {
    Iter.Get(*REINTERPRET_CAST(CLSID*, &iid), dispid, monInfo);
    ++Iter;
    return true;
  }
  else {
    lcSink->LeaveCriticalSection(RGN_MONITOR);
  }
  return false;
}

/*----------------------------------------------------------------------*/

tsp00_Bool OMS_LibOmsInterfaceInstance::GetNextVersion(IliveCacheSink*     lcSink,
                                                 tsp00_C24&          versionId,
                                                 tsp00_Date&         createDate,
                                                 tsp00_Time&         createTime,
                                                 tsp00_Date&         openDate,
                                                 tsp00_Time&         openTime,
                                                 tgg91_TransNo&      consistentView,
                                                 tsp00_Bool&         isMarked,
                                                 tsp00_Bool&         isOpen,
                                                 tsp00_Bool&         isUnloaded,
                                                 tsp00_8ByteCounter& heapUsage,
                                                 tsp00_Int4&         hashSize,
                                                 tgg00_FileId&       versionTree,
                                                 tgg00_FileId&       versionInvTree,
                                                 tsp00_C512&         versionDesc,
                                                 short&              error) {  // PTS 1126698
  try {
  error = e_ok;
  while (true) {
    if (Glob_co10->VersionIter) {
      OmsContext_co12* pContext = Glob_co10->VersionIter();
      if (pContext->IsDropped()) {
        // version is marked as dropped, try to drop it now
        //++Glob_co10->VersionIter;       // PTS 1126698
        if (!pContext->IsBoundToTrans()) {
          // version no longer bound to transaction, i.e. can be dropped now
          pContext->AssignLcSink(lcSink);  /* PTS 1107209 */
          Glob_co10->VersionDictionary.MarkNotUnloadable(lcSink, pContext);
          Glob_co10->VersionDictionary.DropVersion(pContext->GetVersionId()); 
          Glob_co10->VersionIter.getNext(true /*dropCurr*/);
          pContext->DeleteSelf();
        }
        else {                           // PTS 1126698
          ++Glob_co10->VersionIter;
        }
      }
      else {
        memset (&versionId[0], ' ', sizeof(versionId));
        memcpy(&versionId[0], &pContext->GetVersionId()[0], sizeof(OmsVersionId)); 
        createDate     = pContext->GetCreateDate();
        createTime     = pContext->GetCreateTime();
        openDate       = pContext->GetOpenDate();
        openTime       = pContext->GetOpenTime();
        consistentView = pContext->m_consistentView;
        isMarked       = pContext->IsMarked();
        isOpen         = pContext->IsVersionOpen();
        isUnloaded     = pContext->IsUnloaded();
        heapUsage      = pContext->HeapUsed();
        hashSize       = pContext->GetHashSize();
        versionTree    = pContext->VersionFile();
        versionInvTree = pContext->VersionInvFile();
        
        // PTS 1117690
        // Initialize version-desciption
        if (omsInUnicodeMode){
          // initialize with blanks in kernel layout (big endian)
          for (int ix = 0; ix < sizeof(versionDesc); ix += 2)
          {
            versionDesc[ix  ] = 0;
            versionDesc[ix+1] = ' ';
          }
        }
        else {
          memset(versionDesc, ' ', sizeof(versionDesc));
        }
        
        if (pContext->GetVersionDesc() != NULL){
          int len = co10_strlen(pContext->GetVersionDesc());
          
          if (omsInUnicodeMode){
            // Source and Target in UCS2 => No Convertion needed but output string could be
            // too long
            int len_byte = (len * sizeof(OmsTypeWyde) > sizeof(versionDesc)
                         ? sizeof(versionDesc)
                         : len * sizeof(OmsTypeWyde));

            memcpy(versionDesc, pContext->GetVersionDesc(), len_byte); 
            
            // check if endianness of maschine is the same as the endianness of the kernel
            if (sp77encodingUCS2Native != sp77encodingUCS2){
              // translate to kernel layout
              for (int ix = 0; ix < len_byte; ix += 2){
                // swap
                char aux = versionDesc[ix];
                versionDesc[ix  ] = versionDesc[ix+1];
                versionDesc[ix+1] = aux;
              }
            }
          }
          else {
            tsp00_Uint4 destBytesWritten;
            tsp00_Uint4 srcBytesParsed;
            tsp78ConversionResult error;

            // Convertion from UCS2 to ASCII
            error = sp78convertString(sp77encodingAscii,              // destEndcoding
                                      versionDesc,                    // destBuffer
                                      sizeof(versionDesc),            // destBufferLengthInBytes
                                      &destBytesWritten,              // destBytesWritten
                                      false,                          // addZeroTerminator
                                      sp77encodingUCS2Native,         // srcEncoding
                                      pContext->GetVersionDesc(),     // srcBuffer
                                      len * sizeof(OmsTypeWyde),      // srcBufferLengthInBytes
                                      &srcBytesParsed);               // srcBytesParsed
          }
        }
        
        ++Glob_co10->VersionIter;
        return true;
      }
    }
    else {
      lcSink->LeaveCriticalSection(RGN_VERSION_DIR);
      return false;
    }
  }
  } catch (DbpError &e){  // PTS 1126698
    Glob_co10->VersionIter.stop();
    lcSink->LeaveCriticalSection(RGN_VERSION_DIR);
    error = e.m_errorNo;
    return false;
  }
}

/*----------------------------------------------------------------------*/

bool OMS_LibOmsInterfaceInstance::GetFirstClassInfo(OMS_LibOmsInterfaceClassInfo &info)
{
  Glob_co10->ContainerIter.reset(&Glob_co10->ClassDictionary);
  return GetNextClassInfo(info);
}

/*----------------------------------------------------------------------*/

bool OMS_LibOmsInterfaceInstance::GetNextClassInfo(OMS_LibOmsInterfaceClassInfo &info)
{
  if (Glob_co10->ContainerIter) {
    ContainerInfo_co18 *i = Glob_co10->ContainerIter();
    info.handle = i->GetContainerHandle();
    info.schema = i->GetSchema();
    info.container = i->m_ContainerNo;
	GUID_TEMP(lguid, i->GetGuid());
    info.guid = lguid;
    info.keyDesc = i->GetKeyDesc();
    info.persSize = i->GetObjectSize();
    const ClassInfo_co18 *ci = i->GetClassInfoPtr();
    info.name = ci->GetClassName();
    info.isVarObject = ci->IsVarObject();
    info.isBaseClass = ci->IsBaseClass();
    info.isDerivedClass = ci->IsDerivedClass();
    info.isDropped = i->IsDropped();
    ++(Glob_co10->ContainerIter);
    return true;
  } else {
    return false;
  }
}

/*----------------------------------------------------------------------*/

bool OMS_LibOmsInterfaceInstance::IsMonitorOn() const {
  return co10_monitor_malloc;
}

/*----------------------------------------------------------------------*/

void OMS_LibOmsInterfaceInstance::InspectTimeout() // PTS 1110287
{
  co11_InspectTimeout();
}

/*----------------------------------------------------------------------*/

void OMS_LibOmsInterfaceInstance::ResetMonitor(IliveCacheSink* lcSink) {
  CriticalSection_co001 cs(lcSink, RGN_MONITOR);
  cs.Enter();
  Glob_co10->Monitor.Reset();
}

/*----------------------------------------------------------------------*/

void OMS_LibOmsInterfaceInstance::SetDumpError(tsp00_Int4 errorNo)
{
  Glob_co10->SetDumpError(errorNo);
}

/*----------------------------------------------------------------------*/
// PTS 1110287
bool OMS_LibOmsInterfaceInstance::NextOmsLockObjInfo(void** pVoidLockInfo, tgg00_OmsLockInfo& lockInfo)
{
  return co11_NextLockInfo(pVoidLockInfo, lockInfo);
}

/*----------------------------------------------------------------------*/

bool OMS_LibOmsInterfaceInstance::SetTraceLevel(tsp00_KnlIdentifier& lvl, bool enable) {
  return TraceLevel_co102.Set(lvl, enable);
}

/*----------------------------------------------------------------------*/

void OMS_LibOmsInterfaceInstance::StartStopMonitor(bool doStart) {
  co10_monitor_malloc = doStart;
}

/*----------------------------------------------------------------------*/
   
void OMS_LibOmsInterfaceInstance::UnAdviseKernelInterface()
{
   KernelInterface_co10 = NULL;
}

/*----------------------------------------------------------------------*/

bool OMS_LibOmsInterfaceInstance::VersionUnloaded()
{
  return Glob_co10->VersionDictionary.UnloadOldestVersion(co10_GetLcSink());;
}

/*----------------------------------------------------------------------*/

static void co10_TraceGuid (OmsHandle *h, const ClassIDRef guid) {
#ifdef	LIVECACHE_INTGUIDS
  h->omsTracef ("guid : %8X", guid);
#else
  h->omsTracef ("guid : %8X-%4X-%4X-%1X%1X%1X%1X%1X%1X%1X%1X", guid.Data1, guid.Data2, guid.Data3,
  guid.Data4[0], guid.Data4[1], guid.Data4[2], guid.Data4[3], guid.Data4[4], guid.Data4[5],
  guid.Data4[6], guid.Data4[7]);
#endif
}

/*----------------------------------------------------------------------*/

externCpp
ClassInfo_co18* co10_FindClassInfo(IliveCacheSink* lcSink, const ClassIDRef guid) {
  return Glob_co10->ClassDictionary.FindGuid(lcSink, guid);
}

/*----------------------------------------------------------------------*/

externCpp
ContainerInfo_co18* co10_FindContainerInfo(IliveCacheSink* lcSink, tsp00_Uint4 containerHandle) {
  return Glob_co10->ClassDictionary.FindViaContainerHandle(lcSink, containerHandle);
}

/*----------------------------------------------------------------------*/

externCpp
ContainerInfo_co18* co10_FindContainerInfo(IliveCacheSink* lcSink, const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) {
  return  Glob_co10->ClassDictionary.FindViaGuid(lcSink, guid, Schema, ContainerNo);
}

/*----------------------------------------------------------------------*/

externCpp 
bool co10_IsOmsTestLib()
{
    return Glob_co10->IsOmsTestLib();
}

/*----------------------------------------------------------------------*/

externCpp
tsp00_Uint4 co10_GetLastDropId() {
  return Glob_co10->ClassDictionary.GetLastDropId();
}

/*----------------------------------------------------------------------*/

externCpp
ITrace_co02* co10_GetTracer()
{
  return Glob_co10->GetTracer();
}

/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/* Implementation of class OmsCriticalSection                           */
/*----------------------------------------------------------------------*/

OmsCriticalSection::OmsCriticalSection(OmsHandle* h, int regionId) : m_session(h->m_pSession), 
  m_inSection(false), m_regionId(regionId + INTERNAL_CRITICAL_SECTIONS_CO001)
{
	if ((regionId < 0) || !(m_regionId < KernelInterface_co10->GetOmsRegionCnt())) // PTS 1112197
	{
		h->dbpOpError("OmsCriticalSection : invalid regionId %d", regionId); 
		m_session->ThrowDBError (e_invalid_parameter, "OmsCriticalSection");
	}
}

/*----------------------------------------------------------------------*/

void OmsCriticalSection::omsEnter() {
  if ((m_inSection) || (m_session->m_critical_section_in_use[m_regionId-INTERNAL_CRITICAL_SECTIONS_CO001])) {
    m_session->ThrowDBError (e_already_in_critical_section, "omsEnter");
  }
  m_session->m_critical_section_in_use[m_regionId-INTERNAL_CRITICAL_SECTIONS_CO001] = true;
  m_session->m_lcSink->EnterCriticalSection(m_regionId);
  m_inSection = true;
}

/*----------------------------------------------------------------------*/

void OmsCriticalSection::omsLeave() {
  if (m_inSection) {
    m_session->m_lcSink->LeaveCriticalSection(m_regionId);
    m_session->m_critical_section_in_use[m_regionId-INTERNAL_CRITICAL_SECTIONS_CO001] = false;
    m_inSection = false;
  }
  else {
    m_session->ThrowDBError ( e_not_in_critical_section, "omsLeave");
  }
}

/*----------------------------------------------------------------------*/

OmsCriticalSection::~OmsCriticalSection() {
  if (m_inSection) {
    omsLeave();
  }
}

/*----------------------------------------------------------------------*/
/* Implementation of class OmsDuplicateKey                              */
/*----------------------------------------------------------------------*/

OmsDuplicateKey::OmsDuplicateKey(long e, const OmsObjectId& oid) : 
  DbpError(DbpError::DB_ERROR, e, "Duplicate Key", oid) {
};

/*----------------------------------------------------------------------*/ 
/* Implementation of application allocators                             */
/*----------------------------------------------------------------------*/ 

void *operator new(size_t sz, OmsAllocatorInterface &i)
{
  return i.omsAllocate(sz);
}

/*----------------------------------------------------------------------*/

void operator delete(void *ptr, OmsAllocatorInterface &i)
{
  i.omsDeallocate(ptr);
}

/*----------------------------------------------------------------------*/ 
/* Implementation of class OmsHandle                                    */
/*----------------------------------------------------------------------*/ 

OmsHandle::OmsHandle (OmsTypeInt4 sz)
  : m_session(NULL)   
  , m_pSession(NULL)
  , m_monitor(NULL)
  , m_dummyCancelFlag(false) // PTS 1110812
  , m_pCancelFlag(&m_dummyCancelFlag)  
{
};

/*----------------------------------------------------------------------*/

HRESULT OmsHandle::omsAttachDefaultContext() {
  pasbool*     pToCancel;
  tsp00_TaskId taskId;
  HRESULT hr = (*m_session)->GetDefaultContext(REINTERPRET_CAST(void**, &m_pSession),  
    &pToCancel, taskId);
  if ((!FAILED(hr)) && (NULL == m_pSession)) {
    m_pSession = new OmsSession_co12(*m_session, KernelInterface_co10->GetOmsRegionCnt(), pToCancel, taskId); /* PTS 1107849 */
    if (NULL == m_pSession) {
      return E_OUTOFMEMORY;
    }
    m_pCancelFlag = pToCancel;
    try {
       m_pSession->CreateDefaultContext();
    }
    catch (DbpError&) {
      OMS_TRACE(omsTrSession, (*m_session), "create session failed : out of memory");
      m_pSession->IncRefCnt();
      m_pSession->DeleteSession();
      return E_OUTOFMEMORY;
    }
    hr = (*m_session)->SetDefaultContext((void*) m_pSession);
    OMS_TRACE(omsTrSession, (*m_session), "New Session : " << m_pSession); 
  }
  // PTS 1116693
  // Insert actual OmsHandle into the HandleList of the SessionContext
  // Needed for omsGetOmsHandle
  m_pSession->m_handleList.push_back (this);

  dbpAdviseLcSink(*m_session);
  m_pSession->IncRefCnt();
  OMS_TRACE(omsTrSession, (*m_session), "Session " << m_pSession << " attached to Hndl " << this << " RefCnt " << m_pSession->m_refCnt);
  return hr;
}

/*----------------------------------------------------------------------*/

tco13_ClsIdEntry* OmsHandle::omsAddContainerInfo (ContainerInfo_co18* ContainerInfo) {
  tco13_ClsIdEntry* curr = new(m_pSession->m_context) tco13_ClsIdEntry (ContainerInfo, m_pSession->m_context);
  m_pSession->AddContainerInfo (curr);
  return curr;
}

/*----------------------------------------------------------------------*/

OmsObjByClsIterBase OmsHandle::omsAllOids( const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo ) 
{
  _TRACE_METHOD_ENTRY(this,"omsAllOids");
  OMS_CHECK_EXCEPTION(*this);
  
  OmsObjByClsIterBase iterator(*this);
  tsp00_Int2            DBError;
  CHResult_co001        hr;
  tsp00_Int4            objHistReadCount; // PTS 1107819 TA 13/09/2000  
  OmsObjectContainerPtr obj = NULL;
  iterator.m_containerInfo = m_pSession->GetClsInfo (guid, Schema, ContainerNo);
  tsp00_Int4 size          = iterator.m_containerInfo->GetPersistentSize();
  iterator.m_next          = REINTERPRET_CAST(OmsObjectContainerPtr, 
    m_pSession->allocate (iterator.m_containerInfo->GetObjectSize()));
  iterator.m_next->InitObjContainer(*iterator.m_containerInfo);
  iterator.m_next->m_oid = OmsObjectId();
  do {
    m_pSession->IncDerefIter_LC(); // PTS 1107819 TA 13/09/2000 
    hr = (*m_session)->NextObj (
      (unsigned char*) &m_pSession->m_context->m_consistentView,
      (unsigned char*) &iterator.m_containerInfo->GetFileId(),
      m_pSession->m_context->VersionContext(),
      iterator.m_next->m_oid.pOid(), 
      (unsigned char*) &iterator.m_next->m_objseq, size,
      ((unsigned char*) &iterator.m_next->m_pobj) + sizeof(void*), 
      objHistReadCount, &DBError);
    if ( DBError != 0 ) {
      if (e_no_next_object == DBError) {
        iterator.m_next->m_oid.setNil();
        if (!m_pSession->InVersion()) {
          return iterator; /* PTS 1107750 */
        }
        break;
      }
      else {
        m_pSession->ThrowDBError (DBError, "omsSelectOids");
      }
    }
    m_pSession->IncLogHopIter(objHistReadCount); // PTS 1107819 TA 13/09/2000 
    obj = m_pSession->m_context->m_oidDir.HashFind (&iterator.m_next->m_oid);
    if (NULL != obj) {
      if (!obj->DeletedFlag()) {
        break;
      }
    }
    else {
      break;
    }
  }
  while (true);
  /* PTS 1107750 */
  // We are in a version, look for new version objects
  if (!(iterator.m_next->m_oid)) {
    iterator.m_iterNewObjectsInVersion = true;
    iterator.omsNextNewObjectInVersion(NULL);
  }
  iterator.m_curr = REINTERPRET_CAST(OmsObjectContainerPtr, 
    m_pSession->allocate (iterator.m_containerInfo->GetObjectSize()));
  iterator.m_curr->InitObjContainer(*iterator.m_containerInfo);
  iterator.operator++();
  return iterator;
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsCancel() /* PTS 1113128 */
{
  *m_pCancelFlag = true;
}

/*----------------------------------------------------------------------*/

bool OmsHandle::omsCancel(bool toCancel) /* PTS 1120411 */
{
	bool cancelFlag = *m_pCancelFlag != 0;
    *m_pCancelFlag  = toCancel;
	return cancelFlag;
}

/*----------------------------------------------------------------------*/

const OmsObjectId &OmsHandle::omsCastOid(const ClassIDRef guid, const OmsObjectId& oid) {
  OMS_CHECK_EXCEPTION(*this);
  return m_pSession->CastOid(guid, oid);
}

/*----------------------------------------------------------------------*/

bool OmsHandle::omsCheckOid ( const OmsObjectId& oid, const ClassIDRef guid) {
  OMS_TRACE(omsTrInterface, m_pSession->m_lcSink, "omsCheckOid : " << guid << "," << oid);
  OMS_CHECK_EXCEPTION(*this);
  return m_pSession->CurrentContext()->CheckOid(oid, guid);
}

/*----------------------------------------------------------------------*/

bool OmsHandle::omsCheckOid (const OmsVarOid& oid)
{
  OMS_TRACE(omsTrInterface, m_pSession->m_lcSink, "omsCheckOid : " << oid);
  OMS_CHECK_EXCEPTION(*this);
  return m_pSession->CurrentContext()->CheckOid(oid);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsCloseVersion ( const OmsVersionId& versionId )
{
  OMS_CHECK_EXCEPTION(*this);
  tsp00_Int2 DBError;
  if (!m_pSession->InVersion()) {
    return;
  }
  OmsContext_co12* pVersion = m_pSession->m_context;
  if (pVersion->IsOpenVersion(versionId)) {
    OMS_TRACE(omsTrInterface, m_pSession->m_lcSink, "omsCloseVersion : " << CFixedLenChar_co102(versionId, sizeof(versionId)));
    if (1 < m_pSession->CurrentSubtransLevel()) {
      m_pSession->ThrowDBError (e_not_implemented, "Close Version : Open Subtrans");
    }
// PTS 1106619
// PTS 1109449
#if ( defined (AIX) && (__IBMCPP__ >= 500) )
    OmsVersionId LocalVersionID ;
    memcpy(LocalVersionID, versionId, sizeof(OmsVersionId) );
    CHResult_co001 hr = (*m_session)->CloseVersion (&LocalVersionID,
      m_pSession->m_context->CurrentVersionKey(), &DBError);  
#else
    CHResult_co001 hr = (*m_session)->CloseVersion (CONST_CAST(OmsVersionId*, &versionId),
      m_pSession->m_context->CurrentVersionKey(), &DBError);  
#endif

    if ( DBError != 0 ) {
      m_pSession->ThrowDBError (DBError, "omsCloseVersion");
    }
    bool isDropped = false;
    { // Begin of exclusive critical section.    // PTS 1124170
      ExclusiveVersionDirRgn rgn(Glob_co10->VersionDictionary.GetSingleLockId(versionId),
                                 Glob_co10->VersionDictionary.UseRWLocks());      
      
      m_pSession->CloseVersion();
      if (pVersion->IsDropped())
        isDropped = true;
    } // End of exclusive critical section.

    if (isDropped) {
      // version has been marked as to be deleted, do it now
      try {
        omsForceDropVersion(versionId);
      }
      catch (DbpError&) {
      }
    }
  }
}

/*----------------------------------------------------------------------*/
/* PTS 1107849 */

void OmsHandle::omsReschedule() const {
  OMS_CHECK_EXCEPTION(*this);
  m_pSession->Reschedule();
}

/*----------------------------------------------------------------------*/

short OmsHandle::omsCommitHandling (int& phase) {
  tsp00_Uint4            listCount  = 0;
  tsp00_Uint4            entryCount = 0;
  bool                   inVersion  = m_pSession->InVersion();
  try {
    // no application wanted exceptions in the course of transaction end
    this->omsControlCancelFlag(-1);
    this->omsControlDbpException(-1);
    OmsCallbackInterface* callbackInterface = m_pSession->GetCallbackInterface();
    if (NULL != callbackInterface) {
      callbackInterface->omsValidate (*this, OmsCallbackInterface::BeforeCommit,
        inVersion);
    }
    phase = 1;
    m_pSession->ExecuteSubtransCommit (1);
    phase = 2;
    m_pSession->m_defaultContext->FlushObjCache(false);
    phase = 3;
    m_pSession->FreeBeforeImages();
    phase = 4;
    m_pSession->ClearDefaultContext();
    phase = 5;
    if (NULL != callbackInterface) {
      bool read_only          = m_pSession->ForceReadOnly();
      callbackInterface->omsInvalidate (*this, OmsCallbackInterface::AfterCommit,
        m_pSession->InVersion());
      m_pSession->m_lcSink->UserAllocatorCtl(OmsCallbackInterface::BlockTransaction, NULL);  // check TX allocator empty
      m_pSession->UnforceReadOnly(read_only);
      m_pSession->ClearDefaultContext();
    }
    phase = 6;
    m_pSession->TransEnd();
    phase = 7;
    if (inVersion)
    {
      this->CheckHeapThresholds(); /* PTS 1110149 */
    }
  }
  catch (DbpError& e) {
    dbpOpError ("omsCommit : DbpError %d", e.dbpError());
    TraceCommitPhase(phase);
    omsRollback(false, false);
    return e_work_rolled_back;
  }
  return 0;
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsControlCancelFlag(int countDown)     // PTS 1112358
{
#if defined(OMSTST)
  m_pSession->SetCancelCountDown(countDown);
#endif
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsControlDbpException(int countDown)   // PTS 1112358
{
#if defined(OMSTST)
  m_pSession->SetExceptionCountDown(countDown);
#endif
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsControlBadAllocExceptions(int countDown)  // PTS 1112358
{
#if defined(OMSTST)
  tsp00_Int2  DummyError;
  tsp00_Int4  BufLen = 0;
  tsp_int4    Buf = countDown;
  (*m_session)->MultiPurpose (m_diagnose, mm_down, &BufLen, 
    (unsigned char*) &Buf, &DummyError);
#endif
}

/*----------------------------------------------------------------------*/

void  OmsHandle::TraceCommitPhase(int phase) {
  char msg[30];
  switch (phase) {
  case 0 :
    strcpy (msg, "omsValidate");
    break;
  case 1 :
    strcpy (msg, "SubtransCommit");
    break;
  case 2 :
    strcpy (msg, "FlushObjCache");
    break;
  case 3 :
    strcpy (msg, "FreeBeforeImages");
    break;
  case 4 :
    strcpy (msg, "ClearDefaultContext");
    break;
  case 5 :
    strcpy (msg, "OmsInvalidate");
    break;
  case 6 :
    strcpy (msg, "TransEnd");
    break;
  default :
    strcpy(msg, "After TransEnd");
  }
  dbpTrace("Commit Phase : %s", msg);
}

/*----------------------------------------------------------------------*/

short OmsHandle::omsCommit (bool doRelease) {
  
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsCommit");
  short  e;
  int    phase = 0;
#if defined(WIN32)
  __try {
#else
    try {
#endif
      e = omsCommitHandling(phase);
      if (0 != e) {
        return e;
      }
    }
#if defined(WIN32)
/*    __except(co10_EvalException (this, GetExceptionInformation ())) { PTS 1106505 FF */
/*    __except(co10_EvalException (m_pSession->m_lcSink, GetExceptionInformation ())) {PTS 1110787 FF*/
    __except( co10_EvalException ( m_pSession, GetExceptionInformation ())) { /*PTS 1110787 FF*/
      dbpOpError ("omsCommit failed");
      TraceCommitPhase(phase);
      omsRollback(false, false);
      return e_work_rolled_back;
    }
#else
    catch (...) {
      dbpOpError ("omsCommit : unknown exception");
      TraceCommitPhase(phase);
      omsRollback(false, false);
      return e_work_rolled_back;
    }
#endif
    if (doRelease) {
      this->SessionEnd(); /* PTS 1107209 */
    }
    return 0;
  }

/*----------------------------------------------------------------------*/

OmsObjByKeyIterBase OmsHandle::omsCreateKeyRangeIter
(
 const ClassIDRef guid,
 OmsSchemaHandle Schema, 
 OmsContainerNo  ContainerNo,
 const void*     pStartKey,
 const void*     pStopKey
 )
{
  OMS_CHECK_EXCEPTION(*this);
  OmsObjByKeyIterBase iterator(*this, guid, Schema, ContainerNo, pStartKey, pStopKey);
  return iterator;   
}

/*----------------------------------------------------------------------*/

void OmsHandle::IdentifierToWyde(const char* pSource, OmsTypeWyde* pDest) const
{
  pDest[MAX_SCHEMA_NAME_LENGTH_CO10] = 0;
  for (int ix = 0; ix < MAX_SCHEMA_NAME_LENGTH_CO10; ++ix)
  {
    *pDest = *pSource;
    if (0 == *pSource)
    {
      break;
    }
    ++pSource;
    ++pDest;
  }
}

/*----------------------------------------------------------------------*/

void OmsHandle::CheckHeapThresholds() /* PTS 1110149 */
{
  if ((KernelInterface_co10->HeapThresholdReached())
    ||
    (m_pSession->m_context->HeapUsed() > (tsp00_8ByteCounter) (Glob_co10->GetOmsVersionThreshold() * 1024)))
  {
    m_pSession->m_context->ReduceHeapUsage();
  }
}

/*----------------------------------------------------------------------*/

void OmsHandle::SessionEnd()
{
    /* PTS 1107209 */
    if (m_pSession->InVersion()) {
        OmsContext_co12* pContext = m_pSession->CurrentContext();
        m_pSession->CloseVersion();
        if (pContext->IsDropped()) {
            omsForceDropVersion(pContext->GetVersionId());
        }
        else
        {
            pContext->MarkNotBoundToTrans(true);
        }
    }
    m_pSession->ClearDefaultContext();
    m_pSession->ReleaseCallbackInterface();

    m_pSession->m_lcSink->UserAllocatorCtl(OmsCallbackInterface::BlockUnregister, NULL);
    m_pSession->m_lcSink->UserAllocatorCtl(OmsCallbackInterface::BlockSession, NULL);
}

/*----------------------------------------------------------------------*/

void OmsHandle::WydeToChar(const OmsTypeWyde* pSource, char* pDest, int destSize, const char* errorMsg) const
{
  int ix = -1;
  do
  {
    ++ix;
    if (pSource[ix] > 255)
    {
      m_pSession->ThrowDBError (e_not_translatable, errorMsg);
    }
    if (ix == destSize)
    {
      m_pSession->ThrowDBError (e_buffer_too_small, errorMsg);
    }
    pDest[ix] = (char) pSource[ix];
  }
  while (0 != pSource[ix]);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsCommitSubtrans( int required_lvl ) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsCommitSubtrans : reqlvl = " << required_lvl);
  OMS_CHECK_EXCEPTION(*this);
  m_pSession->CommitSubtrans(required_lvl);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsCleanUpAfterCrash() {
  OMS_TRACE (omsTrAlways, m_pSession->m_lcSink, "omsCleanUpAfterCrash");
  CVersionDictionary_co17::Iter iter (EXCLUSIVE, Glob_co10->VersionDictionary);
  while (iter) {
    OmsContext_co12* pContext = iter();
    //++iter;
    if ((pContext->IsBoundToTrans()) && (m_pSession->VersionBoundByMe(pContext))) {
      OMS_TRACE (omsTrAlways, m_pSession->m_lcSink, "Implicit Drop Version : " << 
        CFixedLenChar_co102(pContext->GetVersionId(), sizeof(pContext->GetVersionId())));
      Glob_co10->VersionDictionary.MarkNotUnloadable(m_pSession->m_lcSink, pContext);
      m_pSession->DropVersionProlog(pContext);
      //Glob_co10.VersionDictionary.DropVersion(pContext->GetVersionId());
      iter.getNext(true /*dropCurr*/);
      m_pSession->DropVersionEpilog(pContext);
    }
    else {
      ++iter;
    }
  }
  omsRollback(true, true);
}

/*----------------------------------------------------------------------*/

OmsSchemaHandle OmsHandle::omsCreateSchema(const char* SchemaName) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsCreateSchema : " << SchemaName);
  OmsTypeWyde SchemaNameWyde[MAX_SCHEMA_NAME_LENGTH_CO10 + 1];
  this->IdentifierToWyde(SchemaName, &SchemaNameWyde[0]);
  return omsCreateSchema(&SchemaNameWyde[0]);
}

/*----------------------------------------------------------------------*/

OmsSchemaHandle OmsHandle::omsCreateSchema(const OmsTypeWyde* SchemaName) 
{
  OMS_CHECK_EXCEPTION(*this);
  int length = 0;
  while (SchemaName[length] != 0)
  {
    ++length;
  }
  if (length > MAX_SCHEMA_NAME_LENGTH_CO10)
  {
    CONST_CAST(OmsHandle*, this)->dbpOpError("omsCreateSchema : too long identifier : %d, ", length);
    m_pSession->ThrowDBError (e_identifier_too_long, "omsCreateSchema");
  }
  tsp00_Int4          SchemaId = 0;
  tgg00_BasisError    DBError;

  CHResult_co001 hr = (*m_session)->CreateSchema (SchemaName, &SchemaId, &DBError);
  if ( DBError != 0 ) {
    m_pSession->ThrowDBError (DBError, "omsCreateSchema");
  }
  return SchemaId;
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsCreateVersion(const OmsVersionId& versionId){
  omsCreateVersion(versionId, reinterpret_cast<OmsTypeWyde*>(NULL));
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsCreateVersion(const OmsVersionId& versionId, const char* desc){
  if (desc == NULL){
    omsCreateVersion(versionId, reinterpret_cast<OmsTypeWyde*>(NULL));
  }
  else{
    OmsTypeWyde desc_wyde[MAX_VERSION_DESC_LENGTH_CO10 + 1];
    tsp00_Uint4           destBytesWritten;
    tsp00_Uint4           srcBytesParsed;
    tsp78ConversionResult error;
    int len_wyde = (MAX_VERSION_DESC_LENGTH_CO10 + 1) * sizeof(OmsTypeWyde);
    
    // truncate input-string if necessary
    int len = strlen(desc);
    if (len > MAX_VERSION_DESC_LENGTH_CO10){
      len = MAX_VERSION_DESC_LENGTH_CO10;
    }

    // Convertion from ASCII to UCS2
    error = sp78convertString(sp77encodingUCS2Native,        // destEndcoding
                              desc_wyde,                     // destBuffer
                              len_wyde,                      // destBufferLengthInBytes
                              &destBytesWritten,             // destBytesWritten
                              true,                          // addZeroTerminator
                              sp77encodingAscii,             // srcEncoding
                              desc,                          // srcBuffer
                              len,                           // srcBufferLengthInBytes
                              &srcBytesParsed);              // srcBytesParsed
        
    if (error != sp78_Ok){
      // Reset array again
      memset(desc_wyde, 0, len_wyde);
    }

    omsCreateVersion(versionId, desc_wyde);
  }
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsCreateVersion( const OmsVersionId& versionId, const OmsTypeWyde* desc ){
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsCreateVersion : " << CFixedLenChar_co102(versionId, sizeof(versionId)));
  OMS_CHECK_EXCEPTION(*this);
  tsp00_Int2              DBError;
  OmsContext_co12*        initContext = m_pSession->CurrentContext();

  if (m_pSession->IsSubtransOpen()) {
    m_pSession->ThrowDBError (e_too_many_subtrans, "Create Version : Open Subtrans");
  }
  if (m_pSession->InVersion()) {
    m_pSession->ThrowDBError (e_already_in_version, "omsCreateVersion"); 
  }
  m_pSession->CreateVersion(versionId, desc); 

  { // Begin of exclusive critical section.   // PTS 1124170
    ExclusiveVersionDirRgn rgn(Glob_co10->VersionDictionary.GetSingleLockId(versionId),
                               Glob_co10->VersionDictionary.UseRWLocks());

    try
    {
        DBError = Glob_co10->VersionDictionary.InsertVersion(versionId, m_pSession->CurrentContext());
    }
    catch (BAD_ALLOC_GEO573)
    {
        DBError = e_no_more_memory;
        //DebugBreak();  // This should not happen  
    } 
  }  // End of exclusive critical section.

  if (DBError != 0) {
      m_pSession->RemoveFromTransVersion(m_pSession->m_context);
      m_pSession->m_context->DeleteSelf();
      m_pSession->SetContext(initContext);
      if (e_duplicate_key == DBError)
      {
          m_pSession->ThrowDBError (e_duplicate_name, "omsCreateVersion");
      }
      else
      {
          m_pSession->ThrowDBError (DBError, "omsCreateVersion");
      }
  }
  // End PTS 1121889
}

/*----------------------------------------------------------------------*/

int OmsHandle::omsCurrentSubtransLevel() const {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsCurrentSubtransLevel : " << m_pSession->CurrentSubtransLevel() - 1);
  OMS_CHECK_EXCEPTION(*this);
  return m_pSession->CurrentSubtransLevel() - 1;
}

/*----------------------------------------------------------------------*/

const OmsVersionId*  OmsHandle::omsCurrentVersionId () const {
  OMS_CHECK_EXCEPTION(*this);
  if (m_pSession->InVersion()) {
    return REINTERPRET_CAST(const OmsVersionId*, &m_pSession->CurrentContext()->GetVersionId()[0]);
  }
  return NULL;
}

/*----------------------------------------------------------------------*/
// PTS 1122838
long OmsHandle::omsGetStoredErrorCode() const
{
  return m_pSession->GetStoredErrorCode();
}

/*----------------------------------------------------------------------*/
// PTS 1117690
bool OmsHandle::omsGetVersionInfo(const OmsVersionId& versionId, 
                                  OmsVersionInfo& info,
                                  char* pDescBuf, 
                                  int bufLen) const { 

  if (pDescBuf == NULL || bufLen <= 0){
    return omsGetVersionInfo(versionId, info, reinterpret_cast<OmsTypeWyde*>(NULL), 0);
  }
  else{
    OmsTypeWyde descBuf_wyde[MAX_VERSION_DESC_LENGTH_CO10 + 1];

    // PTS 1118475
    if (bufLen > MAX_VERSION_DESC_LENGTH_CO10){
      bufLen = MAX_VERSION_DESC_LENGTH_CO10;
    }

    // PTS 1118593
    // Initialize output-buffer if requested
    memset(pDescBuf, 0, bufLen);

    bool found = omsGetVersionInfo(versionId, info, descBuf_wyde, bufLen);

    if (found && descBuf_wyde[0] != 0){
      tsp00_Uint4 destBytesWritten;
      tsp00_Uint4 srcBytesParsed;
      tsp78ConversionResult error;
      int bufLen_wyde = bufLen * sizeof(OmsTypeWyde);

      // Convertion from UCS2 to ASCII 
      error = sp78convertString(sp77encodingAscii,      // destEndcoding
                                pDescBuf,               // destBuffer
                                bufLen,                 // destBufferLengthInBytes
                                &destBytesWritten,      // destBytesWritten
                                true,                   // addZeroTerminator
                                sp77encodingUCS2Native, // srcEncoding
                                descBuf_wyde,           // srcBuffer
                                bufLen_wyde,            // srcBufferLengthInBytes
                                &srcBytesParsed);       // srcBytesParsed
        
      if (error != sp78_Ok){
        // Reset array again
        memset(pDescBuf, 0, bufLen);
      }
    }

    return found;
  }
}

/*----------------------------------------------------------------------*/
// PTS 1117690
bool OmsHandle::omsGetVersionInfo (const OmsVersionId& versionId, 
                                   OmsVersionInfo& info, 
                                   OmsTypeWyde* pDescBuf, 
                                   int bufLen) const {  
  OMS_CHECK_EXCEPTION(*this);

  if (pDescBuf != NULL && bufLen > 0){
     // Initialize output-buffer if requested
    int bufLen_byte = bufLen * sizeof(OmsTypeWyde);
    memset(pDescBuf, 0, bufLen_byte);
  }

  OmsContext_co12* pContext = NULL;
  if (m_pSession->InVersion()){
    if (!memcmp(&versionId[0], &(*omsCurrentVersionId())[0], sizeof(OmsVersionId))){
      // ***  Check if information can be read from the version which is currently open  ***
      pContext = m_pSession->CurrentContext();
    }
  }
  
  if (pContext == NULL){
    // ***  Loop over all versions and look for specified version  ***
    m_pSession->m_lcSink->EnterCriticalSection(RGN_VERSION_DIR);
    CVersionDictionary_co17::Iter iter(SHARED, Glob_co10->VersionDictionary);
    
    while (iter) {
      OmsContext_co12* pIterContext = iter();
      if (!pIterContext->IsDropped()) {
        if (!memcmp(&versionId[0], &pIterContext->GetVersionId()[0], sizeof(OmsVersionId))){
          pContext = pIterContext;
          iter.stop();  // PTS 
          break;
        }
      }
      ++iter;
    }
    m_pSession->m_lcSink->LeaveCriticalSection(RGN_VERSION_DIR);
  }

  if (pContext == NULL){
    // there is no version which matches specified id
    return false;
  }
  
  *((tsp00_Date*)&info.createDate)  = pContext->GetCreateDate();
  *((tsp00_Time*)&info.createTime)  = pContext->GetCreateTime();
  *((tsp00_Date*)&info.openDate)    = pContext->GetOpenDate();
  *((tsp00_Time*)&info.openTime)    = pContext->GetOpenTime();
  info.consistentView = pContext->m_consistentView.gg90GetInt4();
  info.isMarked       = pContext->IsMarked();
  info.isOpen         = pContext->IsVersionOpen();
  info.isUnloaded     = pContext->IsUnloaded();
  info.heapUsage      = (OmsTypeInt8)pContext->HeapUsed();
  info.hashSize       = pContext->GetHashSize();

  if (pDescBuf != NULL && bufLen > 0 && pContext->GetVersionDesc() != NULL){
    // truncate desciption string if necessary
    int len = co10_strlen(pContext->GetVersionDesc());
    if (bufLen < len + 1){
      len = bufLen - 1;
    }

    // copy desciption
    memcpy(pDescBuf, pContext->GetVersionDesc(), len * sizeof(OmsTypeWyde));
    pDescBuf[len] = 0;  // ensure that string is null-terminated
  }

  return true;
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsDelete( const OmsObjectId& oid ) 
{
  const char* msg = "omsDelete";

  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, msg << " : " << oid);
  OMS_CHECK_EXCEPTION(*this);
  m_pSession->AssertNotReadOnly(msg);
  m_pSession->IncDelete ();
  OmsObjectContainerPtr found = m_pSession->CurrentContext()->GetObj(oid, false);
  if (!found) 
  {
    m_pSession->ThrowDBError (e_object_not_found, msg, oid);
  }
  if (!found->LockedFlag() && !m_pSession->IsLocked(oid))
  {
    m_pSession->ThrowDBError (e_object_not_locked, msg, oid);
  }
  OmsAbstractObject* pObj = &found->m_pobj; /* PTS 1107093 */
  pObj->omsCleanUp(*this);
  m_pSession->InsertBeforeImage (found);
  found->MarkDeleted();
};

/*----------------------------------------------------------------------*/

void OmsHandle::omsDelete( OmsAbstractObject* pobj ) 
{
  omsDelete (omsGetContainer(pobj)->m_oid);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsDelete( const OmsVarOid& oid ) {
  OMS_TRACE (omsTrInterface | omsTrVarObject, m_pSession->m_lcSink, "omsDelete(Var) : " << oid);
  OMS_CHECK_EXCEPTION(*this);
  if (!m_pSession->IsLocked(oid))
  {
    m_pSession->ThrowDBError (e_object_not_locked, "omsDelete(varobject)", oid);
  }
  m_pSession->DeleteVarObject(oid);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsDeleteAll (const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsDeleteAll : " << guid << ", " 
    << Schema << ", " << ContainerNo);
  OMS_CHECK_EXCEPTION(*this);
  m_pSession->DeleteAll (guid, Schema, ContainerNo);
}

/*----------------------------------------------------------------------*/

const void* OmsHandle::omsDeRef(const OmsVarOid& oid) {
  omsIfNilOidThrowError (this, oid, "omsDeRef : VarObject");
  OMS_TRACE (omsTrInterface | omsTrVarObject, m_pSession->m_lcSink, "omsDeRef(Var) : " << oid);
  OMS_CHECK_EXCEPTION(*this);
  return m_pSession->DeRefVarObject(oid);
}

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsHandle::omsDeRef (const OmsObjectId& oid, const ClassIDRef guid, bool forUpd, bool do_lock )
{
  OMS_CHECK_EXCEPTION(*this);
  if (!oid) {
    return NULL;
  }
  else {
    return m_pSession->DeRef (oid, guid, forUpd, do_lock);
  }
};

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsHandle::omsDeRefKey (unsigned char*  key, 
                                           const ClassIDRef guid, 
                                           bool            forUpd, 
                                           bool            do_lock,
                                           OmsSchemaHandle Schema,
                                           OmsContainerNo  ContainerNo) 
{
  OMS_CHECK_EXCEPTION(*this);
  return m_pSession->DeRefViaKey (key, guid, forUpd, do_lock, Schema, ContainerNo);  
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsDropSchema(OmsSchemaHandle Schema) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsDropSchema : " << Schema);
  OMS_CHECK_EXCEPTION(*this);
  if (m_pSession->InVersion()) {
    m_pSession->ThrowDBError (e_missing_privilege, "omsDropSchema : not allowed in version");
  }
  if (OMS_DEFAULT_SCHEMA_HANDLE == Schema) {
    m_pSession->ThrowDBError (e_missing_privilege, "omsDropSchema : OMS Schema not allowed");
  }
  tgg00_BasisError DBError = Glob_co10->ClassDictionary.DropSchema(m_pSession->m_lcSink, Schema);
  if ( 0 != DBError ) {
    m_pSession->ThrowDBError (DBError, "omsDropSchema");
  }
  m_pSession->CleanAfterDropSchema();
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsDropContainer(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo,
                                 size_t arrayByteSize)
{
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsDropContainer : " << guid << ", " << Schema << ", " << ContainerNo );
  OMS_CHECK_EXCEPTION(*this);
  if (m_pSession->InVersion()) {
    m_pSession->ThrowDBError (e_missing_privilege, "omsDropContainer : not allowed in version");
  }
  tgg00_BasisError DBError = Glob_co10->ClassDictionary.DropContainer(
    m_pSession->m_lcSink, guid, Schema, ContainerNo, arrayByteSize);
  if ( 0 != DBError ) {
    m_pSession->ThrowDBError (DBError, "omsDropSchema");
  }
  m_pSession->CleanAfterDropSchema();
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsDropContainer(const ClassIDRef guid, OmsSchemaHandle sh, OmsContainerNo cn)
{
  omsDropContainer(guid, sh, cn, 0);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsDropVersion( const OmsVersionId& versionId )
{
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsDropVersion : " << CFixedLenChar_co102(versionId, sizeof(versionId)));
  OMS_CHECK_EXCEPTION(*this);
  OmsContext_co12*   context;
  {
    { // Begin of exclusive critical section.    // PTS 1124170
      ExclusiveVersionDirRgn rgn(Glob_co10->VersionDictionary.GetSingleLockId(versionId),
                                 Glob_co10->VersionDictionary.UseRWLocks());

      context = Glob_co10->VersionDictionary.FindVersion(versionId);
      if (NULL == context) {
        m_pSession->ThrowDBError (e_unknown_version, "omsDropVersion");
      }
      Glob_co10->VersionDictionary.MarkNotUnloadable(this->m_pSession->m_lcSink, context);
      m_pSession->DropVersionProlog(context);
      Glob_co10->VersionDictionary.DropVersion(versionId);
    } // End of exclusive critical section.
  }
  m_pSession->DropVersionEpilog(context);
}

/*----------------------------------------------------------------------*/

const char* OmsHandle::omsSimDumpGetHtmlRef(const OmsObjectId &oid)
{
  const char *data;
  m_pSession->m_lcSink->SimCtlGetHTMLRef(*this, oid, &data, 3);
  return data;
}

/*----------------------------------------------------------------------*/

bool OmsHandle::omsExistsContainer(const ClassIDRef guid, OmsSchemaHandle schema, OmsContainerNo cno) // PTS 1110334
{
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsExistsContainer : " << guid << "," << schema << "," << cno);
  OMS_CHECK_EXCEPTION(*this);
  tgg00_BasisError DBError;
  GUID_TEMP(lguid, guid);
  CHResult_co001 hr = (*m_session)->ExistsContainer (schema, lguid, cno , DBError);
  if (0 == DBError)
  {
    return true;
  }
  else
  {
    if (e_sysinfo_not_found !=DBError)
    {
      m_pSession->ThrowDBError (DBError, "omsExistsContainer");
    }
  }
  return false;
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsForceDropVersion( const OmsVersionId& versionId )
{
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsDropVersion : " << CFixedLenChar_co102(versionId, sizeof(versionId)));
  OMS_CHECK_EXCEPTION(*this);
  OmsContext_co12*   context;
  {  // Begin of exclusive critical section.    // PTS 1124170 
    ExclusiveVersionDirRgn rgn(Glob_co10->VersionDictionary.GetSingleLockId(versionId),
                               Glob_co10->VersionDictionary.UseRWLocks());

    context = Glob_co10->VersionDictionary.FindVersion(versionId);
    if (NULL == context) {
      m_pSession->ThrowDBError (e_unknown_version, "omsDropVersion");
    }
    Glob_co10->VersionDictionary.MarkNotUnloadable(this->m_pSession->m_lcSink, context);
    try {
      m_pSession->DropVersionProlog(context);
      Glob_co10->VersionDictionary.DropVersion(versionId);
    }
    catch (DbpError& e) {
      if (-5001 == e.dbpError()) {
        context->MarkDropped();
        return;
      }
      else {
        throw e;
      }
    }
  } // End of exclusive critical section.
  
  m_pSession->DropVersionEpilog(context);
}
/*----------------------------------------------------------------------*/

void OmsHandle::omsDump () {
  m_pSession->CurrentContext()->DumpObjCache(*this);
};

/*----------------------------------------------------------------------*/

void OmsHandle::omsDumpStackTrace()
{
  KernelInterface_co10->DumpStackTrace();
}

/*----------------------------------------------------------------------*/

unsigned int OmsHandle::omsSaveStackTrace(void **buffer, unsigned int levels)
{
  return KernelInterface_co10->SaveStackTrace(buffer, levels);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsEvalStackTrace(void **buffer, unsigned int levels,
  OmsStackTraceCallbackInterface *cb)
{
  KernelInterface_co10->EvalStackTrace(buffer, levels, cb);
}

/*----------------------------------------------------------------------*/

bool OmsHandle::omsExistsVersion( const OmsVersionId& versionId )
{
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsExistsVersion : " << CFixedLenChar_co102(versionId, sizeof(versionId)));
  OMS_CHECK_EXCEPTION(*this);

  bool isDropped = false;
  OmsContext_co12* context = NULL;
  { // Begin of shared critical section.    // PTS 1124170
    SharedVersionDirRgn(Glob_co10->VersionDictionary.GetSingleLockId(versionId),
                        Glob_co10->VersionDictionary.UseRWLocks());

    context = Glob_co10->VersionDictionary.FindVersion(versionId);
    if (context && context->IsDropped())  /* PTS 1107277 */
      isDropped = true;
  } // End of shared critical section.  

  if (isDropped) {  /* PTS 1107209 */ 
    try {
      omsForceDropVersion(versionId);
      context = NULL;
    }
    catch(DbpError&) {
    }
  }
  return (context != NULL);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsExceptionHandler(DbpError& e) {
  m_pSession->CurrentMethodCallEpilog(-1); // PTS 1126793
  OmsTypeUTF8 utf8Buffer[sizeof(e.m_errorText)*2];
  OmsTypeUTF8* pMsgText;
  if (e.m_isWyde)
  {
    
    SAPDB_UTF8Basis::UCS2ConstPointer srcAt;
    SAPDB_UTF8Basis::UTF8Pointer      destAt;
    pMsgText = &utf8Buffer[0];
    register const OmsTypeWyde* pStart = e.dbpErrorTextWyde();
    register const OmsTypeWyde* pEnd   = pStart;
    while (0 != *pEnd)
    {
      ++pEnd;
    }
    if (!KernelInterface_co10->IsUnicodeInstance())
    {
      int ix = 0;
      do
      {
        if (*pStart > 255)
        {
          dbpOpError ("omsExceptionHandler : msg not translatable");
          utf8Buffer[ix] = 0;
          break;
        }
        utf8Buffer[ix] = (unsigned char) *pStart;
        if (pStart == pEnd)
        {
          break;
        }
        ++pStart;
        ++ix;
       }
      while (true);
    }
    else
    {
      SAPDB_UTF8Basis::ConversionResult res = SAPDB_UTF8Basis::ConvertFromUCS2 (
        (SAPDB_UTF8Basis::UCS2ConstPointer) pStart,
        (SAPDB_UTF8Basis::UCS2ConstPointer) ++pEnd,
        srcAt,
        &utf8Buffer[0],
        &utf8Buffer[sizeof(e.m_errorText)*2],
        destAt);
    }
  }
  else
  {
    pMsgText = (OmsTypeUTF8*) &e.m_errorText[0];
  }
  switch (e.m_errorKind) {
  case DbpError::DB_ERROR :
    {
      omsRaiseError ((short) e.m_errorNo, pMsgText);
      break;
    }
  case DbpError::HRESULT_ERROR :
    {
      (*m_session)->HResultError (e.m_errorNo);
      break;
    }
  case DbpError::RTE_ERROR :
    {
      omsRaiseError ((short) e.m_errorNo, pMsgText);
      break;
    } 
  case DbpError::USER_DEFINED :
    {
      omsRaiseError ((short) e.m_errorNo, pMsgText);
      break;
    } 
  default :
    dbpOpError ("Invalid DbpError, kind = %d, errno = %d", e.m_errorKind, e.m_errorNo);
    throw;
  }
} 

/*----------------------------------------------------------------------*/

OmsContainerNo OmsHandle::omsGetContainerNo(const OmsObjectId& oid) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsGetContainerNo : " << oid);
  OMS_CHECK_EXCEPTION(*this);
  omsIfNilOidThrowError (this, oid, "omsGetContainerNo");
  OmsObjectContainerPtr pObj = m_pSession->CurrentContext()->GetObj(oid, false);
  if (NULL != pObj) {
    tco13_ClsIdEntry* pContainerInfo = pObj->GetContainerInfo(m_pSession->CurrentContext());
    return pContainerInfo->GetContainerNo();
  }
  else {
    m_pSession->ThrowDBError (e_object_not_found, "omsGetContainerNo", oid);
    return 0;
  }
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsGetMonitorInfo (tgg00_COMMonitorTransferBuf* lpMonInfo) {

  if (lpMonInfo->ctbDispId_gg00 < 0) {
    // reset statistics
    m_monitor->Reset();
    return;
  }
  while ((lpMonInfo->ctbDispId_gg00 < m_monitor->size()) &&  
    (0 == (*m_monitor)[lpMonInfo->ctbDispId_gg00].m_monInfo.cmiCallCnt_gg00)) {
    ++lpMonInfo->ctbDispId_gg00;
  }
  if (lpMonInfo->ctbDispId_gg00 >= m_monitor->size()) {
    lpMonInfo->ctbDispId_gg00 = -1;
    return;
  }
  memcpy (&lpMonInfo->ctbMonInfo_gg00, 
    &((*m_monitor)[lpMonInfo->ctbDispId_gg00].m_monInfo),
    sizeof (lpMonInfo->ctbMonInfo_gg00));
  if (csp_maxint4 == lpMonInfo->ctbMonInfo_gg00.cmiCounters_gg00[cmiMinWaitNewConsView]) {
    lpMonInfo->ctbMonInfo_gg00.cmiCounters_gg00[cmiMinWaitNewConsView] = 0;
  }
}

/*----------------------------------------------------------------------*/

bool OmsHandle::omsGetMethodCallMonitorInfo (OmsMonitorData& info) const
{
   return m_pSession->GetMethodCallMonitorInfo(info);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsGetObj(const OmsObjectId& oid, unsigned char* cnsView, int& ObjLen, unsigned char* obj)
// exported for diagnostics only, not included into interface
{
  tgg91_TransNo cView;
  tgg91_TransNo currCView = m_pSession->m_defaultContext->m_consistentView;
  memcpy (&cView, cnsView, sizeof(cView));
  m_pSession->m_defaultContext->SetConsistentView(cView);
  m_pSession->m_defaultContext->SetConsistentView(cView);
  OmsObjectContainerPtr pObj = m_pSession->m_defaultContext->GetObjFromLiveCacheBase(oid, OmsContext_co12::NoLock);
  if (NULL != pObj) {
    tco13_ClsIdEntry* pContainerInfo = pObj->GetContainerInfo(m_pSession->m_defaultContext);
    ObjLen = pContainerInfo->GetClassInfoPtr()->GetObjectSize() - OmsObjectContainer::headerSize() - sizeof(void*);
    memcpy (obj, &pObj->m_pobj, ObjLen); 
  }
  else {
    ObjLen = 0;
    obj    = NULL;
  }
  m_pSession->m_defaultContext->SetConsistentView(currCView);
}

/*----------------------------------------------------------------------*/

OmsHandle* OmsHandle::omsGetOmsHandle()  {  // PTS 1116693
   OmsSession_co12 *pSession;
   pasbool         *pToCancel;
   tsp00_TaskId    taskId;
  
   IliveCacheSink* pSink = co10_GetLcSink();
   pSink->GetDefaultContext(REINTERPRET_CAST(void**, &pSession), &pToCancel, taskId);
   return pSession == NULL ? NULL : pSession->m_handleList.front();
}
 
/*----------------------------------------------------------------------*/

OmsSchemaHandle OmsHandle::omsGetSchemaHandle(const char* SchemaName) const 
{
  OmsTypeWyde SchemaNameWyde[MAX_SCHEMA_NAME_LENGTH_CO10 + 1];
  this->IdentifierToWyde(SchemaName, &SchemaNameWyde[0]);
  return omsGetSchemaHandle(&SchemaNameWyde[0]);
}

/*----------------------------------------------------------------------*/

OmsSchemaHandle OmsHandle::omsGetSchemaHandle(const OmsTypeWyde* SchemaName) const 
{
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsGetSchemaHandle");
  OMS_CHECK_EXCEPTION(*this);
  tsp00_Int4          SchemaId = 0;
  tgg00_BasisError    DBError;
  CHResult_co001 hr = (*m_session)->ExistsSchema (SchemaName, &SchemaId, &DBError);
  if ( 0 != DBError ) {
    m_pSession->ThrowDBError (DBError, "omsGetSchemaHandle");
  }
  return SchemaId;
}

/*----------------------------------------------------------------------*/

OmsSchemaHandle OmsHandle::omsGetSchemaHandle(const OmsObjectId& oid) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsGetSchemaHandle : " << oid);
  OMS_CHECK_EXCEPTION(*this);
  
  omsIfNilOidThrowError (this, oid, "omsGetSchemaHandle");
  OmsObjectContainerPtr pObj = m_pSession->CurrentContext()->GetObj(oid, false);
  if (NULL != pObj) {
    tco13_ClsIdEntry* pContainerInfo = pObj->GetContainerInfo(m_pSession->CurrentContext());
    return pContainerInfo->GetSchema();
  }
  else {
    m_pSession->ThrowDBError (e_object_not_found, "omsGetSchemaHandle", oid);
    return 0;
  }
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsGetSchemaName(OmsSchemaHandle Schema, char* SchemaName, int SchemaNameBufSize) const 
{
  const char* msg = "omsGetSchemaName";

  OmsTypeWyde SchemaNameWyde[MAX_SCHEMA_NAME_LENGTH_CO10 + 1];
  this->omsGetSchemaName(Schema, &SchemaNameWyde[0], MAX_SCHEMA_NAME_LENGTH_CO10 + 1);
  this->WydeToChar(&SchemaNameWyde[0], SchemaName, SchemaNameBufSize, msg);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsGetSchemaName(OmsSchemaHandle Schema, OmsTypeWyde* SchemaName, int SchemaNameBufSize) const 
{
  OMS_CHECK_EXCEPTION(*this);
  OmsTypeWyde SchemaNameWyde[MAX_SCHEMA_NAME_LENGTH_CO10 + 1];
  tgg00_BasisError    DBError;
  CHResult_co001 hr = m_pSession->m_lcSink->GetSchemaName(Schema, &SchemaNameWyde[0], &DBError);
  if (0 != DBError)
  {
    DbpBase base(m_pSession->m_lcSink);
    base.dbpOpError("omsGetSchemaName : unknown schema handle %d", Schema);
    m_pSession->ThrowDBError (DBError, "omsGetSchemaName");
  }
  if (SchemaNameBufSize > 0) 
  {
    int ix = 0;
    while (ix < SchemaNameBufSize)
    {
      SchemaName[ix] = SchemaNameWyde[ix];
      if (0 == SchemaNameWyde[ix])
      {
        return;
      }
      ++ix;
    }
    m_pSession->ThrowDBError (e_buffer_too_small, "omsGetSchemaName");
  }
}

/*----------------------------------------------------------------------*/

int OmsHandle::omsGetTimeOut() const {
  return m_pSession->m_timeout;
}

/*----------------------------------------------------------------------*/

OmsCallbackInterface* OmsHandle::omsGetTransactionInterface (const ClassIDRef guid) const {
  return m_pSession->GetCallbackInterface(guid);
}

/*----------------------------------------------------------------------*/

unsigned long OmsHandle::omsGetVarObjectSize (const OmsVarOid& oid) 
{
  OMS_TRACE (omsTrInterface | omsTrVarObject, m_pSession->m_lcSink, "omsGetVarObjSize : " << oid);
  OMS_CHECK_EXCEPTION(*this);

  return m_pSession->GetVarObjSize(oid);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsInitSession (IUnknown** s) {
  m_session = (IliveCacheSink**) s;

  // this will do an attach in simulator, ignored otherwise
  KernelInterface_co10->SimCtlAttachSink(*this);
}

/*----------------------------------------------------------------------*/

bool OmsHandle::omsIsLocked (const OmsObjectId& oid) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsIsLocked : " << oid);
  return m_pSession->IsLocked(oid);
}

/*----------------------------------------------------------------------*/

bool OmsHandle::omsIsRegistered(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) 
{
    return NULL != m_pSession->GetClsInfoForReg(guid, Schema, ContainerNo);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsLoadVarObject (const OmsVarOid& oid, unsigned long size, void* buf) {
  OMS_TRACE (omsTrInterface | omsTrVarObject, m_pSession->m_lcSink, "omsLoadVarObject : " << oid << ", size " << size);
  OMS_CHECK_EXCEPTION(*this);

  omsIfNilOidThrowError (this, oid, "omsLoadVarObject");
  m_pSession->LoadVarObject(oid, VarObjExclusiveLock, size, buf); 
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsLock( const OmsObjectId&oid )
{
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsLock : " << oid);
  OMS_CHECK_EXCEPTION(*this);

  m_pSession->CurrentContext()->LockObj(oid);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsLock( const OmsVarOid& oid) { 
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsLock(Var) : " << oid);
  OMS_CHECK_EXCEPTION(*this);

  m_pSession->CurrentContext()->LockObj((OmsObjectId) oid);
}

 /*----------------------------------------------------------------------*/
 
bool OmsHandle::omsTryLock( const OmsObjectId&oid )
{
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsTryLock : " << oid);
  OMS_CHECK_EXCEPTION(*this);

  return m_pSession->CurrentContext()->TryLockObj(oid);
}

/*----------------------------------------------------------------------*/

bool OmsHandle::omsTryLock( const OmsVarOid& oid)
{ 
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsTryLock(Var) : " << oid);
  OMS_CHECK_EXCEPTION(*this);

  return m_pSession->CurrentContext()->TryLockObj((OmsObjectId) oid);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsLocks( const OmsObjectId*p, int n )
{ 
  _TRACE_METHOD_ENTRY(this,"omsLocks")
  OMS_CHECK_EXCEPTION(*this);
    
  tsp00_Int4 ix;
  for (ix = 0; ix < n; ++ix) {
    omsLock(*p);
    ++p;
  }
  return;
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsMarkVersion ( const OmsVersionId& versionId ) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsMarkVersion : " << CFixedLenChar_co102(versionId, sizeof(versionId)));
  OMS_CHECK_EXCEPTION(*this);

  OmsContext_co12*     context;
  { // Begin of exclusive critical section.    // PTS 1124170
    ExclusiveVersionDirRgn rgn(Glob_co10->VersionDictionary.GetSingleLockId(versionId),
                               Glob_co10->VersionDictionary.UseRWLocks());

    context = Glob_co10->VersionDictionary.FindVersion(versionId);
    if (NULL == context) {
      m_pSession->ThrowDBError (e_unknown_version, "omsMarkVersion");
    }
    context->MarkVersion();
  } // End of exclusive critical section.
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsMethodCallEpilog(void* pSH) {     /* PTS 1120804 */
  OmsCallbackInterface* cbi = m_pSession->GetCallbackInterface();
  if (NULL != cbi) {
    /* PTS 1120804 */
    if (NULL != pSH) {
      cbi->omsMethodCallEpilog(*this,*REINTERPRET_CAST(SqlHandle*,pSH));
    }
    else {
      cbi->omsMethodCallEpilog(*this);
    }
  }
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsMonitorLock()
{
  (*m_session)->EnterCriticalSection(RGN_MONITOR);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsMonitorUnlock()
{
  (*m_session)->LeaveCriticalSection(RGN_MONITOR);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsNewConsistentView () {
  
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsNewConsistentView : ");
  if (m_pSession->CurrentContext()->IsVersion()) {
    return;
  }
  OMS_CHECK_EXCEPTION(*this);

  m_pSession->CurrentContext()->RemoveUnlockedObjFromCache();
  m_pSession->CurrentContext()->NewConsistentView();
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsNewConsistentView (OmsObjectId* pOids, int cntOids, short timeout)
{
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsNewConsistentView : " << "cnt " << cntOids << "timeout " << timeout);

  int dummy = 0;
  omsNewConsistentView (pOids, cntOids, timeout, NULL, dummy);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsNewConsistentView (OmsObjectId* pOids, int cntOids, short timeout, OmsObjectId* pErrOids, int& cntErrOids) {
  OMS_CHECK_EXCEPTION(*this);
  
  gg200QuickSort(&pOids[0], cntOids);

  // PTS 1115027

  OMS_OidArrayReader pOidsIter(pOids, cntOids);
  OMS_OidArrayWriter pErrOidsApp(pErrOids, cntErrOids);

  m_pSession->NewConsistentView(pOidsIter, timeout, pErrOidsApp);
  cntErrOids = pErrOidsApp.omsGetCntr();

}

/*----------------------------------------------------------------------*/

// PTS 1115027

void OmsHandle::omsNewConsistentView (OmsIOidReadIterator& pOids, short timeout, OmsIOidAppender& pErrOids)
{
  OMS_CHECK_EXCEPTION(*this);
  
  m_pSession->NewConsistentView(pOids, timeout, pErrOids);
}

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsHandle::omsNewKeyedObject(const ClassIDRef	 guid, 
                                                const unsigned char* key,
                                                OmsSchemaHandle      Schema,
                                                OmsContainerNo       ContainerNo )
{
  _TRACE_METHOD_ENTRY(this,"omsNewKeyObject");
  OMS_CHECK_EXCEPTION(*this);

  return m_pSession->NewKeyedObject (guid, key, Schema, ContainerNo);
}

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsHandle::omsNewObject( const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo )
{
  _TRACE_METHOD_ENTRY(this,"omsNewObject")
  OMS_CHECK_EXCEPTION(*this);

  return m_pSession->NewObject(guid, Schema, ContainerNo);    
}

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsHandle::omsNewRegistryObject (size_t sz, const ClassIDRef guid) {
  
  OmsObjectContainerPtr p;
  sz += OmsObjectContainer::headerSize();
  p = (OmsObjectContainerPtr) m_pSession->m_context->allocate(sz);
  if (p) {
    p->InitState(REINTERPRET_CAST(tco13_GuidEntry*, NULL));
    p->UnmarkVerNewObject();
    return &p->m_pobj;
  }
  return NULL;  
}

/*----------------------------------------------------------------------*/

const OmsVarOid& OmsHandle::omsNewVarObject(OmsSchemaHandle Schema, OmsContainerNo ContainerNo)
{
  OMS_TRACE (omsTrInterface | omsTrVarObject, m_pSession->m_lcSink, "omsNewVarObj : " << Schema << ", " << ContainerNo);
  OMS_CHECK_EXCEPTION(*this);

  m_pSession->RegVarObjectContainer (Schema, ContainerNo);
  return m_pSession->NewVarObject(Schema, ContainerNo);
}

/*----------------------------------------------------------------------*/

void  OmsHandle::omsPrintOid (char* buf, size_t bufSize, const OmsObjectId oid)
{

   char auxbuf[256];
   //sprintf ( auxbuf, "OID : %d.%d (vers %d)", oid.getPno(), oid.getPos(), oid.getVersion());                   // PTS 1125307
   sp77sprintf ( auxbuf, sizeof(auxbuf), "OID : %d.%d (vers %d)", oid.getPno(), oid.getPos(), oid.getVersion()); // PTS 1125307
   if (strlen(auxbuf) < bufSize) {
     strcat (buf, auxbuf);
   }
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsOidInfo (const OmsObjectId& oid, ClassID &Guid, 
                            OmsSchemaHandle& Schema, char* SchemaName, int SchemaNameBufSize,
                            char* ClassName, int ClassNameBufSize, OmsContainerNo& ContainerNo)
{
   OmsTypeWyde SchemaNameWyde[MAX_SCHEMA_NAME_LENGTH_CO10 + 1];
   this->omsOidInfo (oid, Guid, Schema, SchemaNameWyde, MAX_SCHEMA_NAME_LENGTH_CO10 + 1,
     ClassName, ClassNameBufSize, ContainerNo);
   this->WydeToChar(&SchemaNameWyde[0], SchemaName, SchemaNameBufSize, "omsOidInfo");
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsOidInfo (const OmsObjectId& oid, ClassID &Guid, 
                            OmsSchemaHandle& Schema, OmsTypeWyde* SchemaName, int SchemaNameBufSize,
                            char* ClassName, int ClassNameBufSize, OmsContainerNo& ContainerNo)
{
  OMS_CHECK_EXCEPTION(*this);
  
  omsIfNilOidThrowError (this, oid, "omsOidInfo");
  OmsObjectContainerPtr pObj = m_pSession->CurrentContext()->GetObj(oid, false);
  if (NULL != pObj) {
    tco13_ClsIdEntry* pContainerInfo = pObj->GetContainerInfo(m_pSession->CurrentContext());
    Guid        = pContainerInfo->GetGuid();
    Schema      = pContainerInfo->GetSchema();
    ContainerNo = pContainerInfo->GetContainerNo();
    omsGetSchemaName(Schema, SchemaName, SchemaNameBufSize);
    if (ClassNameBufSize > 0) {
      int ClassNameLength = (int) strlen(pContainerInfo->GetClassInfoPtr()->GetClassName());
      if (ClassNameLength >= ClassNameBufSize) {
        ClassNameLength = ClassNameBufSize - 1;
      }
      memcpy (ClassName, pContainerInfo->GetClassInfoPtr()->GetClassName(), ClassNameLength); 
      ClassName[ClassNameLength] = 0;
    }
  }
  else {
    m_pSession->ThrowDBError (e_object_not_found, "omsOidInfo", oid);
  }
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsOpenVersion( const OmsVersionId& versionId )
{
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsOpenVersion : " << CFixedLenChar_co102(versionId, sizeof(versionId)));
  OMS_CHECK_EXCEPTION(*this);

  OmsContext_co12*        context;
  const char*             msg = "omsOpenVersion";
  { // Begin of exclusive critical section.    // PTS 1124170
    ExclusiveVersionDirRgn rgn(Glob_co10->VersionDictionary.GetSingleLockId(versionId),
                               Glob_co10->VersionDictionary.UseRWLocks());

    context = Glob_co10->VersionDictionary.FindVersion(versionId);
    if (NULL == context) {
      m_pSession->ThrowDBError (e_unknown_version, msg);
    }
    if (context->IsDropped()) {
      goto Drop;  // The deletion itself is done outside the critical section
    }
    if (m_pSession->IsSubtransOpen()) {
      m_pSession->ThrowDBError (e_too_many_subtrans, "Open Version : Open Subtrans"); 
    }
    if ((context->IsBoundToTrans()) && (!m_pSession->VersionBoundByMe(context))) 
    {
      if (context->IsVersionOpen())
      {
        m_pSession->ThrowDBError (e_version_already_open, msg);
      }
      else
      {
        m_pSession->ThrowDBError (e_version_bound_by_trans, msg);
      }
    }
    if (m_pSession->InVersion()) {
      m_pSession->ThrowDBError (e_already_in_version, msg); 
    }
    m_pSession->OpenVersion(context);
    m_pSession->IncOpenVersion();
    Glob_co10->VersionDictionary.MarkNotUnloadable(m_pSession->m_lcSink, context);
  } // End of exclusive critical section.
  return;

Drop: 
  {
    try {
      omsForceDropVersion(versionId);
    }
    catch(DbpError&) {
    }
    m_pSession->ThrowDBError (e_unknown_version, msg);
  }
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsRaiseError (short msgno, const OmsTypeUTF8* errmsg) 
{
  _TRACE_METHOD_ENTRY(this,"omsRaiseError");
  const SAPDB_UTF8* pMsg    = errmsg;
  int               charCnt = 0;
  bool              toUCS2  = false;
  if (KernelInterface_co10->IsUnicodeInstance())
  {
    while (true)
    {
      if (0 == *pMsg)
      {
        break;
      }
      SAPDB_UInt characterSize = SAPDB_UTF8Basis::CharacterSize(pMsg);
      if (0 == characterSize)
      {
        return;
      }
      if (characterSize > 1)
      {
        toUCS2 = true;
      }
      ++charCnt;
      for (SAPDB_UInt ix = 0; ix < characterSize; ++ix)
      {
        ++pMsg;
      }
    }
  }
  else
  {
    charCnt = (int) strlen((char*) &errmsg[0]); 
  }
  if (toUCS2)
  {
    tsp81_UCS2Char* pUCS2 = REINTERPRET_CAST(tsp81_UCS2Char*, co10_malloc((charCnt+1) * 2));  
    SAPDB_UTF8Basis::UTF8ConstPointer srcAt;
    tsp81_UCS2Char*                   destAt;
    SAPDB_UTF8Basis::ConversionResult res = SAPDB_UTF8Basis::KernelConvertToUTF16 (errmsg, pMsg,
      srcAt, pUCS2, pUCS2 + charCnt, destAt, false);
    if (SAPDB_UTF8Basis::Success != res)
    {
      charCnt = 0;
    }
    m_pSession->m_lcSink->SetError (msgno, 2 * charCnt, 
      REINTERPRET_CAST(unsigned char*, pUCS2), csp_unicode);
  }
  else
  {
    m_pSession->m_lcSink->SetError (msgno, charCnt, errmsg, csp_ascii);
  }
};

/*----------------------------------------------------------------------*/

void OmsHandle::omsReadOnly (bool read_only) 
{
  m_pSession->SetReadOnly(read_only);
}

/*----------------------------------------------------------------------*/

bool OmsHandle::omsGetReadOnly()
{
  return m_pSession->GetReadOnly();
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsRegClass (OmsSchemaHandle    Schema,
                             const ClassIDRef   guid,
                             const char*        ClassName, 
                             size_t             PersistentSize,
                             size_t             ObjectSize,
                             const ClassIDPtr   pBaseClass,
                             OmsAbstractObject* Obj, 
                             OmsContainerNo     ContainerNo,
                             size_t             arrayByteSize)
{
  OMS_CHECK_EXCEPTION(*this);

  m_pSession->RegContainer(Schema, guid, ClassName, PersistentSize, ObjectSize, 
    pBaseClass, Obj,  ContainerNo, arrayByteSize);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsRegClassAndKey (OmsSchemaHandle    Schema, 
                                   const ClassIDRef   guid,
                                   const char*        ClassName,
                                   size_t             PersistentSize, 
                                   size_t             KeySize, 
                                   size_t             ObjectSize, 
                                   OmsAbstractObject* Obj, 
                                   OmsContainerNo     ContainerNo,
                                   bool               partitionedKey /* PTS 1109717 */)
{
  _TRACE_METHOD_ENTRY(this,"OmsHandle::omsRegClassAndKey");
  OMS_CHECK_EXCEPTION(*this);
  
  struct VirtualObject {
    void* vtptr;
  };    
  tco13_ClsIdEntry* pContainerInfo;
  bool              isRegistered = false;
  
  //try {                                    // PTS 1121231
    pContainerInfo = m_pSession->GetClsInfoForReg(guid, Schema, ContainerNo);
    isRegistered = (pContainerInfo != NULL);
  //}                                        // PTS 1121231
  //catch (DbpError& e) {                    // PTS 1121231
  //  if (e_unknown_guid != e.dbpError()) {  // PTS 1121231
  //    throw e;                             // PTS 1121231
  //  }                                      // PTS 1121231
  //}                                        // PTS 1121231
  if (!isRegistered) {
    tgg00_ObjColDesc    InvColDesc;
    tgg00_StackEntry    KeyDesc;
    ObjectSize     += (OmsObjectContainer::headerSize() + KeySize);
    PersistentSize += (KeySize - /* virtual table ptr */ sizeof (void*)); 
    InvColDesc.colHashCount_gg00 = 1;
    InvColDesc.colInvCount_gg00  = partitionedKey; /* PTS 1109717 */
    InvColDesc.colHashList_gg00  = REINTERPRET_CAST(tgg00_StackListPtr, &KeyDesc);
    InvColDesc.colInvList_gg00   = NULL;
    KeyDesc.etype().becomes(st_object_col);
    KeyDesc.eop().becomes(op_none);
    KeyDesc.epos()     = (tsp00_Int2) (PersistentSize - KeySize + 1);
    KeyDesc.elen_var() = (tsp00_Int2) KeySize;
    KeyDesc.ecol_pos() = -1;
    ContainerInfo_co18* p = Glob_co10->ClassDictionary.RegisterContainer (*m_session, guid, 
      ClassName, PersistentSize, ObjectSize, NULL, InvColDesc, Schema, ContainerNo, 
      (REINTERPRET_CAST(VirtualObject*, Obj))->vtptr);
    pContainerInfo = omsAddContainerInfo (p);
  }
  /* PTS 1122376, don't chain free object, because we don't know its size any more */
  m_pSession->m_context->deallocate(OmsObjectContainer::containerPtr(Obj));
};

/*----------------------------------------------------------------------*/

void OmsHandle::omsRegMonitor (void* pMon) { 
  m_monitor = REINTERPRET_CAST(co90_Monitor*, (pMon)); 
} 

/*----------------------------------------------------------------------*/

void OmsHandle::omsRegTransactionInterface (const ClassIDRef guid, OmsCallbackInterface* pInterface) {
  m_pSession->RegisterCallbackInterface(guid, pInterface);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsRelease( const OmsObjectId& oid ) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsRelease : " << oid);
  m_pSession->ReleaseObj(oid);
};

/*----------------------------------------------------------------------*/

void OmsHandle::omsRelease ( const OmsVarOid& oid ) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsRelease : " << oid);
  m_pSession->ReleaseObj ((OmsObjectId) oid); 
};

/*----------------------------------------------------------------------*/

void OmsHandle::omsReleaseAll() {
  
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsReleaseAll");
  
  const bool emptyObjCache = true;
  if (!m_pSession->IsSubtransOpen()) {
    if(m_pSession->IsDefaultContext(m_pSession->CurrentContext())) {
      m_pSession->CurrentContext()->FlushObjCache(emptyObjCache);
    }
    m_pSession->FreeBeforeImages(); 
  }
}

/*----------------------------------------------------------------------*/

HRESULT OmsHandle::omsReleaseDefaultContext() 
{
  if (NULL != m_monitor) {
    CriticalSection_co12 region(this->m_pSession, RGN_MONITOR);
    region.Enter();
    Glob_co10->Monitor.Insert(m_monitor); /* PTS 1107731 */
    region.Leave();
  }
  
  OMS_TRACE(omsTrSession, (*m_session), " Release Session " << m_pSession << " of Hndl " << this << " RefCnt " << m_pSession->m_refCnt);
  m_pSession->DeleteSession();
  return S_OK;
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsReleaseUnconditional( const OmsObjectId& oid) {
  m_pSession->ReleaseObjUnconditional (oid); 
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsReleaseVarObject(const OmsVarOid& oid) {
  OMS_TRACE (omsTrInterface | omsTrVarObject, m_pSession->m_lcSink, "omsReleaseVarObject : " << oid);
  m_pSession->ReleaseVarObject(oid);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsReleaseConsistentView() {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsReleaseConsistentView");
  tsp00_Int2  DummyError;
  tsp00_Int4  BufLen = 0;
  tsp00_Int4  Buf;
  CHResult_co001 hr = (*m_session)->MultiPurpose (m_end, mm_nil, &BufLen, 
    (unsigned char*) &Buf, &DummyError);
}

/*----------------------------------------------------------------------*/

/* PTS 1115134 */
void OmsHandle::omsResetVersion(const OmsVersionId& versionId)
{
  //DebugBreak();
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsResetVersion : " << CFixedLenChar_co102(versionId, sizeof(versionId)));
  OMS_CHECK_EXCEPTION(*this);

  const char*             msg = "omsResetVersion";
  bool isDropped = false;
  { // Begin of exclusive critical section.    // PTS 1124170
    ExclusiveVersionDirRgn rgn(Glob_co10->VersionDictionary.GetSingleLockId(versionId),
                               Glob_co10->VersionDictionary.UseRWLocks());

    OmsContext_co12* context = Glob_co10->VersionDictionary.FindVersion(versionId);
    if (NULL == context) {
      m_pSession->ThrowDBError (e_unknown_version, msg);
    }
    if (context->IsDropped()) {
      isDropped = true;  // The deletion itself is done outside the critical section
    }
    if (!isDropped){
      if (m_pSession->IsSubtransOpen()) {
        m_pSession->ThrowDBError (e_too_many_subtrans, "Open Version : Open Subtrans"); 
      }
      if (context->IsBoundToTrans()) 
      {
          m_pSession->ThrowDBError (e_version_already_open, msg);
      }
      context->ResetVersion(*m_pSession);
    }
  } // End of exclusive critical section.

  if (isDropped){
    try {
      omsForceDropVersion(versionId);
    }
    catch(DbpError&) {
    }
    m_pSession->ThrowDBError (e_unknown_version, msg);
  }
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsRollback (bool doRelease, bool ignoreErrors) {

  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsRollback");
  int phase = 1;
  try
  {
    // no application wanted exceptions in the course of transaction end
    this->omsControlCancelFlag(-1);
    this->omsControlDbpException(-1);
    if (!ignoreErrors)
    {
      m_pSession->RollbackSubtrans(1, true);
    }
    phase = 2;
    m_pSession->FreeBeforeImages();
    phase = 3;
    m_pSession->ClearDefaultContext();
    phase = 4;
    OmsCallbackInterface* callbackInterface = m_pSession->GetCallbackInterface();
    phase = 5;
    if (NULL != callbackInterface) {
      bool read_only = m_pSession->ForceReadOnly();
      try {
        callbackInterface->omsInvalidate (*this, OmsCallbackInterface::AfterRollback,
          m_pSession->InVersion());
        m_pSession->m_lcSink->UserAllocatorCtl(OmsCallbackInterface::BlockTransaction, NULL);  // check TX allocator empty
      }
      catch (DbpError e) {
        dbpOpError("omsInvalidate failed with DbpError %d", e.dbpError());
        m_pSession->m_currVarObjChunk.m_oid.setNil();
        m_pSession->ResetSubtransLevel();
        m_pSession->UnforceReadOnly(read_only);
        m_pSession->ClearDefaultContext();
        throw e;
      }
      catch (...) {
        dbpOpError("omsInvalidate failed with unknown exception");
        m_pSession->m_currVarObjChunk.m_oid.setNil();
        m_pSession->ResetSubtransLevel();
        m_pSession->UnforceReadOnly(read_only);
        m_pSession->ClearDefaultContext();
        throw;
      }
      m_pSession->m_currVarObjChunk.m_oid.setNil();
      m_pSession->UnforceReadOnly(read_only);
      if (!m_pSession->InVersion()) {
        m_pSession->ResetSubtransLevel();
        m_pSession->ClearDefaultContext();
      }
    }
    phase = 6;
	m_pSession->TransEnd();
	phase = 7;
    if (doRelease) 
	{
      this->SessionEnd();  /* PTS 1107209 */
    }
  }
  catch (DbpError& e)
  {
    if (!ignoreErrors)
    { 
      dbpOpError("omsRollback failed with error %d in phase %d", e.dbpError(), phase);
      omsTerminate();
    }
  }
  catch (...)
  {
    if (!ignoreErrors)
    { 
      dbpOpError("omsRollback failed with unknown exception in phase %d", phase);
      omsTerminate();
     }
  }
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsRollbackSubtrans( int required_lvl ) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsRollbackSubtrans : " << required_lvl);
  if (!m_pSession->IsSubtransOpen()) {
    return;
  }
  OMS_CHECK_EXCEPTION(*this);
  m_pSession->IncSubtransRollback();
  m_pSession->RollbackSubtrans(required_lvl, false);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsRenameVersion ( const OmsVersionId& oldId, const OmsVersionId& newId) {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsRenameVersion : " << CFixedLenChar_co102(oldId, sizeof(oldId)) 
    << " to " << CFixedLenChar_co102(newId, sizeof(newId)));
  OMS_CHECK_EXCEPTION(*this);

  const char* msg = "omsRenameVersion";

  { // Begin of exclusive critical section.    // PTS 1124170
    int   pLock[2];
    const OmsVersionId *pVersionId[2];
    pVersionId[0] = &newId;
    pVersionId[1] = &oldId;
    int size = 2;
    Glob_co10->VersionDictionary.GetMultiLockIds(pVersionId, size, &pLock[0]);
    ExclusiveVersionDirRgn rgn(&pLock[0], size,
                               Glob_co10->VersionDictionary.UseRWLocks());

    OmsContext_co12* context = Glob_co10->VersionDictionary.FindVersion(oldId);
    if (NULL == context) {
      m_pSession->ThrowDBError(e_unknown_version, msg);
    }

    // Check that version is not bound by another session
    if ((context->IsBoundToTrans()) && (!m_pSession->VersionBoundByMe(context))){
      m_pSession->ThrowDBError (e_version_bound_by_trans, msg);
    }

    // PTS 1125426 Changed calling sequence of drop and insert
    Glob_co10->VersionDictionary.DropVersion(oldId); 
    context->RenameVersion (oldId, newId);

    tgg00_BasisError e = Glob_co10->VersionDictionary.InsertVersion(newId, context);
    if (e_ok != e) {
      if (e_duplicate_key == e) {
        e = e_duplicate_name;
      }
      // PTS 1125426 If error occurred, then reinsert version with old name
      context->RenameVersion (newId, oldId);
      tgg00_BasisError e = Glob_co10->VersionDictionary.InsertVersion(oldId, context);
      
      m_pSession->ThrowDBError (e_duplicate_name, msg);
    }
  } // End of exclusive critical section.
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsSetTimeOut( short timeout ) {
  tsp00_Int2  DummyError;
  tsp00_Int4  BufLen = sizeof(timeout);
  m_pSession->m_timeout = timeout;
  CHResult_co001 hr = (*m_session)->MultiPurpose (m_set, mm_lock_timeout, &BufLen, 
    (unsigned char*) &timeout, &DummyError);
}

// PTS 1117571
/*----------------------------------------------------------------------*/
// Switch ON or OFF the feature of CACHED KEYS. 
bool OmsHandle::omsSetCachedKeys(bool on, 
                                 const ClassIDRef guid, 
                                 OmsSchemaHandle schema, 
                                 OmsContainerNo containerNo){
  OMS_TRACE (omsTrInterface | omsTrContainerDir, m_pSession->m_lcSink, "omsSetCachedKeys: " << (on?"ON":"OFF"));
  OMS_CHECK_EXCEPTION(*this);

  tco13_ClsIdEntry* pClsInfo = m_pSession->GetClsInfo (guid, schema, containerNo);
  return pClsInfo->SetCachedKeys(on, m_pSession->CurrentContext());
}

/*----------------------------------------------------------------------*/
// PTS 1117690
void OmsHandle::omsSetCurrVersionDesc (const char* desc){ 

  if (desc == NULL){
    omsSetCurrVersionDesc(reinterpret_cast<OmsTypeWyde*>(NULL));
  }
  else{
    OmsTypeWyde descBuf_wyde[MAX_VERSION_DESC_LENGTH_CO10 + 1];
    int len = strlen(desc);
    int len_byte = (MAX_VERSION_DESC_LENGTH_CO10 + 1) * sizeof(OmsTypeWyde);

    tsp00_Uint4 destBytesWritten;
    tsp00_Uint4 srcBytesParsed;
    tsp78ConversionResult error;

    // Convertion from ASCII to UCS2  
    error = sp78convertString(sp77encodingUCS2Native,  // destEndcoding
                              descBuf_wyde,            // destBuffer
                              len_byte,                // destBufferLengthInBytes
                              &destBytesWritten,       // destBytesWritten
                              true,                    // addZeroTerminator
                              sp77encodingAscii,       // srcEncoding
                              desc,                    // srcBuffer
                              len,                     // srcBufferLengthInBytes
                              &srcBytesParsed);        // srcBytesParsed
        
    if (error == sp78_Ok){
      // Description is only set if there are no errors in convertion
      omsSetCurrVersionDesc(descBuf_wyde);
    }
  }
}

/*----------------------------------------------------------------------*/
// PTS 1117690
void OmsHandle::omsSetCurrVersionDesc (const OmsTypeWyde* desc) { 

  m_pSession->CurrentContext()->SetVersionDesc(desc);
}

/*----------------------------------------------------------------------*/


int OmsHandle::omsStartSubtrans() {
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsStartSubtrans");
  OMS_CHECK_EXCEPTION(*this);

  return m_pSession->StartSubtrans();
}

/*----------------------------------------------------------------------*/
// PTS 1122838
void OmsHandle::omsStoreErrorCode(long errorCode) 
{
  m_pSession->StoreErrorCode(errorCode);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsStoreVarObject ( const OmsVarOid& oid, const void* pVarObj, unsigned long objLength)
{
  OMS_TRACE (omsTrInterface | omsTrVarObject, m_pSession->m_lcSink, "omsStoreVarObject : " << oid << ", length is " << objLength);
  OMS_CHECK_EXCEPTION(*this);

  if (!m_pSession->IsLocked(oid))
  {
    m_pSession->ThrowDBError (e_object_not_locked, "omsStoreVarObject", oid);
  }
  m_pSession->StoreVarObject(oid, pVarObj, objLength);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsTerminate()
{
  omsTerminate((OmsTypeUTF8*) "");
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsTerminate(const char* msg)
{
  omsTerminate((OmsTypeUTF8*) msg);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsTerminate(const OmsTypeUTF8* msg) /* PTS 1109963 */
{
  dbpOpMsg("omsTerminate called %s", msg);
  tgg00_BasisError e = e_terminate; /* PTS 1107666 */
  tsp00_Int4  BufLen = sizeof(e);
  tsp00_Int2  SqlError;
  m_pSession->IncOmsTerminate();

#if defined(OMSTST)
  // As variable is reset in CurrentMethodCallEpilog, remember current value.
  // PTS 1128645
  bool requiredExceptionThrown = m_pSession->RequiredExceptionThrown();
#endif

  m_pSession->CurrentMethodCallEpilog(-1); // PTS 1126793
  // transform basis error into sql return code
  m_pSession->m_lcSink->MultiPurpose (m_change, mm_nil, &BufLen, 
    (unsigned char*) &e, &SqlError);

#if defined(OMSTST)
  if (requiredExceptionThrown)
  {
    char* noReturnCodeMsg = (char*) "-28999";
    omsRaiseError (SqlError, (OmsTypeUTF8*) noReturnCodeMsg);
  }
#endif
/*  omsRaiseError (SqlError, msg); PTS 1123201, PG
    co10_GetKernelInterface()->IsUnicodeInstance() ist TRUE iff parameter _UNICODE == YES.
    omsIsUnicodeInstance() is TRUE iff 1. parameter _UNICODE == YES and
                                       2. either USERS.DEFAULTCODE == UNICODE for the standard LC user
                                          or     USERS.DEFAULTCODE == DEFAULT for the standard LC user
                                                 and parameter DEFAULT_CODE == UNICODE.

    Standard LC user is the user under which the Dbsl connects to the LC.
    Newer versions of the 6.20 Dbsl and higher allow only connections 
      - from an Ascii SAP system to an Ascii LC and
      - from an Unicode SAP system to an Unicode LC.
    Where   Ascii LC means omsIsUnicodeInstance() == FALSE
      and Unicode LC means omsIsUnicodeInstance() == TRUE.
    So, in case co10_GetKernelInterface()->IsUnicodeInstance() && !omsIsUnicodeInstance()
    the error text needs to be adjusted. Otherwise question marks occur in ABAP short dumps. */

  if ( co10_GetKernelInterface()->IsUnicodeInstance() 
       && !omsIsUnicodeInstance() ) {

    tsp00_Uint4           destBytesWritten;
    tsp00_Uint4           srcBytesParsed;
    tsp78ConversionResult error;
    tsp00_Int4            srcLenByte = (int)strlen((char*)&msg[0]);
    int                   charCnt = 2 * srcLenByte;
    tsp00_Int4            destLenByte = charCnt + 2;

    OmsTypeUTF8 * pDest = (OmsTypeUTF8 *)m_pSession->m_context->allocate(destLenByte);
    error = sp78convertString( sp77encodingUCS2,      // destEndcoding
                               pDest,                 // destBuffer
                               destLenByte,           // destBufferLengthInBytes
                               &destBytesWritten,     // destBytesWritten
                               true,                  // addZeroTerminator
                               sp77encodingUTF8,      // srcEncoding
                               msg,                   // srcBuffer
                               srcLenByte,            // srcBufferLengthInBytes
                               &srcBytesParsed );     // srcBytesParsed

    if (sp78_Ok == error) {
      m_pSession->m_lcSink->SetError( SqlError, charCnt, pDest, csp_ascii );
    }
    else { /* If the conversion fails, call omsRaiseError as before. */
      omsRaiseError( SqlError, msg );
    }

  }

  else {
    omsRaiseError( SqlError, msg );
  }

}

/*----------------------------------------------------------------------*/
// PTS 1110812
void OmsHandle::omsThrowCancelledError() const
{
  *m_pCancelFlag = false;
  co10_Throw(DbpError (DbpError::DB_ERROR, -102, "cancelled"));
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsTrace (const char* tracemsg, tsp00_Int4 TraceMsgLen) {
  if (NULL != *m_session)
    (*m_session)->Vtrace (TraceMsgLen, (unsigned char*) tracemsg);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsTracef( const char*fmt, ... ) {
#if defined(WIN32)
  char buffer[256];
#else
  char buffer[1024];
#endif

  va_list args;
  va_start (args, fmt);
#if defined(WIN32)
  _vsnprintf ( buffer, sizeof(buffer), fmt, args );
#else
  vsprintf ( buffer, fmt, args );
  buffer[256] = '\0';
#endif
  omsTrace (buffer, (tsp00_Int4) strlen(buffer));
  va_end (args);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsTraceOid (const OmsObjectId& oid) {
  omsTracef ("OID : %d.%d (vers %d)", oid.getPno(), oid.getPos(), oid.getVersion());
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsTransRollback() {
  tsp00_Int2 DBError;
  CHResult_co001 hr = (*m_session)->Rollback (0, NULL, &DBError);
  if ( DBError != 0 ) {
    m_pSession->ThrowDBError (DBError, "omsTransRollback");
  }
  omsRollback(false, false);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsUnlock (const OmsObjectId& oid)
{
  OMS_TRACE (omsTrInterface, m_pSession->m_lcSink, "omsUnlock : " << oid);
  m_pSession->UnlockObj(oid);
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsTransEndEpilog() // PTS 1110287
{
  m_pSession->TransEndEpilog();
}

/*----------------------------------------------------------------------*/

void OmsHandle::omsValidate() {
};

/*----------------------------------------------------------------------*/

OmsAllocatorInterface &OmsHandle::omsGetGlobalAllocator()
{
  OmsAllocatorInterface *alloc;
  m_pSession->m_lcSink->GetGlobalUserAllocator(alloc, NULL);
  return *alloc;
}

/*----------------------------------------------------------------------*/

OmsAllocatorInterface &OmsHandle::omsGetGlobalAllocator(const char *name)
{
  OmsAllocatorInterface *alloc;
  m_pSession->m_lcSink->GetGlobalUserAllocator(alloc, name);
  return *alloc;
}

/*----------------------------------------------------------------------*/

OmsAllocatorInterface &OmsHandle::omsGetSessionAllocator()
{
  OmsAllocatorInterface *alloc;
  m_pSession->m_lcSink->GetSessionUserAllocator(alloc);
  return *alloc;
}

/*----------------------------------------------------------------------*/

OmsAllocatorInterface &OmsHandle::omsGetTransactionAllocator()
{
  OmsAllocatorInterface *alloc;
  m_pSession->m_lcSink->GetTransactionUserAllocator(alloc);
  return *alloc;
}

/*----------------------------------------------------------------------*/

OmsAllocatorInterface &OmsHandle::omsGetCOMRoutineAllocator()
{
  OmsAllocatorInterface *alloc;
  m_pSession->m_lcSink->GetCOMRoutineUserAllocator(alloc);
  return *alloc;
}

/*----------------------------------------------------------------------*/

OmsHandle::~OmsHandle () {
    _TRACE_METHOD_ENTRY(this,"~OmsHandle");
    // Remove Handle from HandleList         // PTS 1116693
  	if (m_pSession) {
  		for (cgg251dclIterator<OmsHandle*,OmsSession_co12> iter 
  		    = m_pSession->m_handleList.begin(); iter; ++iter) {
            if (this == *iter()) {
                m_pSession->m_handleList.remove(iter);
                break;
            }
        }
    }

    if (Glob_co10->InSimulator() && m_pSession && m_pSession->m_lcSink) {
        m_pSession->m_lcSink->SimCtlDestroyHandleCB(*this, &(m_pSession->m_lcSink));
    }
}

/*----------------------------------------------------------------------*/
/* Implementation of OmsLockTimeout                                     */
/*----------------------------------------------------------------------*/

OmsLockTimeout::OmsLockTimeout(const OmsObjectId& oid) :
  DbpError(DbpError::DB_ERROR, -51, "Lock Request Timeout", oid) {
}

/*----------------------------------------------------------------------*/
/* Implementation of OmsOutOfDate                                       */
/*----------------------------------------------------------------------*/

OmsOutOfDate::OmsOutOfDate(const OmsObjectId& oid) :
  DbpError(DbpError::DB_ERROR, e_object_dirty, "Object out of date", oid) {
}

/*----------------------------------------------------------------------*/

OmsOutOfDate::OmsOutOfDate() :
  DbpError(e_object_dirty, "Object out of date") {
}

/*----------------------------------------------------------------------*/

//
// Implementation of OmsAbstractObject
//

OmsAbstractObject::OmsAbstractObject() {};

/*----------------------------------------------------------------------*/

OmsObjByClsIterBase OmsAbstractObject::omsAllOids( OmsHandle& h, const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) {
  OMS_TRACE(omsTrInterface, h.m_pSession->m_lcSink, "OmsAbstractObject::omsAllOids : " << guid << "," << Schema << "," << ContainerNo); 
  return h.omsAllOids (guid, Schema, ContainerNo);
};

/*----------------------------------------------------------------------*/

#ifndef LIVECACHE_COMPATAPO30
void OmsAbstractObject::omsBinaryToKey(void* pSource, void* pKey) const
{
	throw DbpError(-9000, "invalid call of OmsAbstractObject::omsBinaryToKey");
}
#endif

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsCleanUp (OmsHandle&) {};

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsDelete (OmsHandle& Handle) {
  
  OMS_CHECK_EXCEPTION(Handle);

  const char* msg = "OmsAbstractObject::omsDelete";

  Handle.m_pSession->AssertNotReadOnly(msg);
  Handle.m_pSession->IncDelete ();
  OmsObjectContainerPtr pObjContainer = omsGetContainer(this);
  
  OMS_TRACE(omsTrInterface, Handle.m_pSession->m_lcSink, msg << " : " << pObjContainer->m_oid);
  
  if (pObjContainer->DeletedFlag()) {
    Handle.m_pSession->ThrowDBError (e_object_not_found, msg, pObjContainer->m_oid);
  }
  if (!pObjContainer->LockedFlag() && !Handle.m_pSession->IsLocked(pObjContainer->m_oid))
  {
    Handle.m_pSession->ThrowDBError (e_object_not_locked, msg, pObjContainer->m_oid);
  }
  omsCleanUp (Handle);
  pObjContainer->MarkDeleted();
}

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsDeleteAll(OmsHandle& h, const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) 
{
  const char* msg = "OmsAbstractObject::omsDeleteAll";

  OMS_TRACE(omsTrInterface, h.m_pSession->m_lcSink, msg << " : "
    << guid << "," << Schema << "," << ContainerNo);
  h.omsDeleteAll (guid, Schema, ContainerNo);
}

/*----------------------------------------------------------------------*/

const OmsAbstractObject* OmsAbstractObject::omsDeRef (const OmsObjectId& oid, OmsHandle& Handle, const ClassIDRef guid) {
  OMS_TRACE(omsTrInterface, Handle.m_pSession->m_lcSink, "OmsAbstractObject::omsDeRef : " << oid << ", " << guid); 
  OMS_CHECK_EXCEPTION(Handle);

#if defined(TEST_INLINE)
  OmsObjectContainerPtr p = Handle.m_pSession->m_context->FindObjInContext(&oid);
  if (NULL != p) {
    if (p->DeletedFlag()) {
      p = NULL;
    }
  }
  else {
    if (oid) {
      p = Handle.m_pSession->m_context->GetObjFromLiveCacheBase(oid, OMS_Context::NoLock);
    }
    else {
      return NULL;
    }
  }
  if (NULL != p) {
    Handle.m_pSession->m_context->CheckCompatibility(guid, p);
    return &p->m_pobj;
  }
  else {
    Handle.m_pSession->ThrowDBError (e_object_not_found, "OmsAbstractObject::omsDeRef", oid);
  }
  return NULL;
#else
  if (!oid) {
    return NULL;
  }
  else {
    return REINTERPRET_CAST(OmsAbstractObject*, Handle.m_pSession->DeRef (oid, guid, !FOR_UPD_CO10, false));
  }
#endif
};

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsAbstractObject::omsDeRefForUpd (const OmsObjectId& oid, OmsHandle& Handle, const ClassIDRef guid, bool do_lock) {
  OMS_TRACE(omsTrInterface, Handle.m_pSession->m_lcSink, "OmsAbstractObject::omsDeRefForUpd : " << oid 
    << ", " << guid << ", do_lock : " << do_lock); 
  OMS_CHECK_EXCEPTION(Handle);

  if (oid) {
    return REINTERPRET_CAST(OmsAbstractObject*, Handle.m_pSession->DeRef(oid, guid, FOR_UPD_CO10, do_lock));
  }
  else {
    return NULL;
  }
};

/*----------------------------------------------------------------------*/

const OmsAbstractObject* OmsAbstractObject::omsDeRefKey (unsigned char*  key,
                                                         OmsHandle&      Handle, 
                                                         const ClassIDRef guid, 
                                                         OmsSchemaHandle h,
                                                         OmsContainerNo  ContainerNo) 
{
  OMS_TRACE(omsTrInterface, Handle.m_pSession->m_lcSink, "OmsAbstractObject::omsDeRefKey : " << guid
    << "CNo : " << ContainerNo << " Key : " << CUnsignedChar_co102(key, Handle.m_pSession->GetClassInfo(guid)->GetKeyLen()));
  OMS_CHECK_EXCEPTION(Handle);
  return REINTERPRET_CAST(OmsAbstractObject*, Handle.m_pSession->DeRefViaKey(key, guid, !FOR_UPD_CO10, false, h, ContainerNo));
};

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsAbstractObject::omsDeRefKeyForUpd (unsigned char*  key,
                                                         OmsHandle&      Handle, 
                                                         const ClassIDRef guid, 
                                                         bool            do_lock,
                                                         OmsSchemaHandle h,
                                                         OmsContainerNo  ContainerNo)
{
  OMS_TRACE(omsTrInterface, Handle.m_pSession->m_lcSink, "OmsAbstractObject::omsDeRefKeyForUpd : " << guid
    << "CNo : " << ContainerNo << " Key : " << CUnsignedChar_co102(key, Handle.m_pSession->GetClassInfo(guid)->GetKeyLen())
    << " do_lock : " << do_lock);
  OMS_CHECK_EXCEPTION(Handle);

  return REINTERPRET_CAST(OmsAbstractObject*, Handle.m_pSession->DeRefViaKey(key, guid, FOR_UPD_CO10, do_lock, h, ContainerNo));
};

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsDump (OmsHandle& Handle, const char* flags) const {
  OmsObjectId oid = omsOid();
  Handle.omsTracef ("OID : %d.%d (vers %d) %s", 
    oid.getPno(), oid.getPos(), oid.getVersion(), flags); 
}

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsFlush (OmsHandle& Handle) {
  OmsObjectContainerPtr pObjContainer = omsGetContainer(this);
  Handle.m_pSession->m_context->FlushObj(pObjContainer);
};

/*----------------------------------------------------------------------*/

const OmsObjectId& OmsAbstractObject::omsGetOid() const {
  return omsOid();
}

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsAbstractObject::omsForUpdPtr(OmsHandle& h, bool doLock) const {
  OMS_CHECK_EXCEPTION(h);

  OmsObjectContainerPtr pObjContainer = omsGetContainer(this);
  
  h.m_pSession->AssertNotReadOnly("OmsAbstractObject::omsForUpdPtr"); 

  if (doLock) {
    if (!(pObjContainer->LockedFlag()) && (!h.m_pSession->InVersion())) {
      ((OmsAbstractObject*) this)->omsLock(h);
    }
  }
  h.m_pSession->InsertBeforeImage (pObjContainer);
  return (OmsAbstractObject*) this;
}

/*----------------------------------------------------------------------*/

OmsSchemaHandle OmsAbstractObject::omsGetSchemaHandle(OmsHandle& Handle) const {
  OMS_CHECK_EXCEPTION(Handle);

  OmsObjectContainerPtr pObjContainer = omsGetContainer(this);
  tco13_ClsIdEntry* pContainerInfo = pObjContainer->GetContainerInfo(Handle.m_pSession->CurrentContext());
  return pContainerInfo->GetSchema();
}

/*----------------------------------------------------------------------*/

bool OmsAbstractObject::omsHistoryInUse(OmsHandle& h) const {
  OmsObjectContainerPtr pObjContainer = omsGetContainer(this);
  OMS_TRACE(omsTrInterface, h.m_pSession->m_lcSink, "OmsAbstractObject::omsHistoryInUse : " << pObjContainer->m_oid);
  OMS_CHECK_EXCEPTION(h);

  return h.m_pSession->HistoryInUse(pObjContainer->m_oid);
}

/*----------------------------------------------------------------------*/

bool OmsAbstractObject::omsIsLocked (OmsHandle& Handle) const {
  
  _TRACE_METHOD_ENTRY(&Handle,"OmsAbstractObject::omsIsLocked");

  return Handle.m_pSession->IsLocked(omsGetContainer(this)->m_oid);
}

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsKey(OmsHandle& Handle, unsigned char* key) const {

  OmsObjectContainerPtr pObjContainer = omsGetContainer(this);
  tco13_ClsIdEntry* pContainerInfo = pObjContainer->GetContainerInfo (Handle.m_pSession->CurrentContext());
#ifdef LIVECACHE_COMPATAPO30
  memcpy(key, pContainerInfo->GetKeyPtr(pObjContainer), pContainerInfo->GetKeyLen());
#else
  this->omsBinaryToKey(pContainerInfo->GetKeyPtr(pObjContainer), key);
#endif
}

/*----------------------------------------------------------------------*/

#ifndef LIVECACHE_COMPATAPO30
void OmsAbstractObject::omsKeyToBinary(const void* pKey, void* pDest) const
{
	throw DbpError (-9000, "invalid call of OmsAbstractObject::omsKeyToBinary");
}
#endif

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsLock (OmsHandle& h) {
  
  OMS_CHECK_EXCEPTION(h);

  const char* msg = "OmsAbstractObject::omsLock"; 

  h.m_pSession->AssertNotReadOnly(msg);  

  OmsObjectContainerPtr pObjContainer = omsGetContainer(this);

  OMS_TRACE(omsTrInterface, h.m_pSession->m_lcSink, msg << " : " << pObjContainer->m_oid);
 
  if ((pObjContainer->LockedFlag()) || (h.m_pSession->InVersion())) {
    // instance already locked
    return;
  }
  h.m_pSession->LockObj(pObjContainer);
}

/*----------------------------------------------------------------------*/

bool OmsAbstractObject::omsTryLock (OmsHandle& h)
{
  OMS_CHECK_EXCEPTION(h);

  const char* msg = "OmsAbstractObject::omsTryLock"; 

  h.m_pSession->AssertNotReadOnly(msg);  

  OmsObjectContainerPtr pObjContainer = omsGetContainer(this);

  OMS_TRACE(omsTrInterface, h.m_pSession->m_lcSink, msg << " : " << pObjContainer->m_oid);

  if ((pObjContainer->LockedFlag()) || (h.m_pSession->InVersion())) {
    // instance already locked
    return true;
  }
  return h.m_pSession->TryLockObj(pObjContainer);
}

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsAbstractObject::omsNewObject (OmsHandle&      Handle, 
                                                    const ClassIDRef guid, 
                                                    OmsSchemaHandle h, 
                                                    OmsContainerNo  ContainerNo ) {
  OMS_TRACE(omsTrInterface, Handle.m_pSession->m_lcSink, "OmsAbstractObject::omsNewObject : " << guid 
    << "CNo : " << ContainerNo);

  return Handle.omsNewObject ( guid, h, ContainerNo ); 
};

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsAbstractObject::omsNewRegistryObject(size_t sz, OmsHandle& Handle, const ClassIDRef guid) {
  return Handle.omsNewRegistryObject (sz, guid);
};

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsAbstractObject::omsNewKeyedObject (OmsHandle&           Handle, 
                                                         const ClassIDRef     guid, 
                                                         const unsigned char* key,
                                                         OmsSchemaHandle      h,
                                                         OmsContainerNo       ContainerNo ) {
  OMS_TRACE(omsTrInterface, Handle.m_pSession->m_lcSink, "OmsAbstractObject::omsNewKeyedObject : " << guid
    << " CNo : " << ContainerNo << " Key : " << CUnsignedChar_co102(key, Handle.m_pSession->GetClassInfo(guid)->GetKeyLen()));

  return Handle.omsNewKeyedObject ( guid, key, h, ContainerNo ); 
};

/*----------------------------------------------------------------------*/

const OmsObjectId& OmsAbstractObject::omsOid() const {
  return omsGetContainer(this)->m_oid;
};

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsRegClass (OmsHandle&         Handle,
                                     const ClassIDRef   Guid, 
                                     const char*        ClassName, 
                                     size_t             PersistentSize, 
                                     size_t             ObjectSize,
                                     const ClassIDPtr   pBaseClass,
                                     OmsAbstractObject* Obj,
                                     OmsSchemaHandle    h,
                                     OmsContainerNo     ContainerNo,
                                     size_t             arrayByteSize) 
{
  Handle.omsRegClass (h, Guid, ClassName, PersistentSize, ObjectSize, pBaseClass,
    Obj, ContainerNo, arrayByteSize);
};

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsRegClassAndKey (OmsHandle&         Handle,
                                           const ClassIDRef   Guid,
                                           const char*        ClassName, 
                                           size_t             PersistentSize, 
                                           size_t             KeySize, 
                                           size_t             ObjectSize,
                                           OmsAbstractObject* Obj,
                                           OmsSchemaHandle    h,
                                           OmsContainerNo     ContainerNo,
                                           bool               partitionedKey)
{
  Handle.omsRegClassAndKey (h, Guid, ClassName, PersistentSize, KeySize, 
    ObjectSize, Obj, ContainerNo, partitionedKey);
};

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsRelease(OmsHandle& Handle) {
 
 OMS_TRACE(omsTrInterface, Handle.m_pSession->m_lcSink, "OmsAbstractObject::omsRelease : " << omsGetContainer(this)->m_oid);
 Handle.m_pSession->ReleaseObj(omsGetContainer(this));
}

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsStore (OmsHandle& Handle) {
 
  OMS_TRACE(omsTrInterface, Handle.m_pSession->m_lcSink, "OmsAbstractObject::omsStore : " << omsGetContainer(this)->m_oid);
  OMS_CHECK_EXCEPTION(Handle);

  const char* msg = "OmsAbstractObject::omsStore"; 

  Handle.m_pSession->AssertNotReadOnly(msg);  

  Handle.m_pSession->IncStore();
  // omsVerify(Handle); PTS 1121886
  
  OmsObjectContainerPtr pObjContainer = omsGetContainer(this);
  _TRACE_OID(&Handle, pObjContainer->m_oid);
  if (!pObjContainer->LockedFlag() && !Handle.m_pSession->IsLocked(pObjContainer->m_oid))
  {
    Handle.m_pSession->ThrowDBError (e_object_not_locked, msg, pObjContainer->m_oid);
  }
  if (!(pObjContainer->existBeforeImage (Handle.m_pSession->CurrentSubtransLevel(), Handle.m_pSession->InVersion()))) {
    Handle.m_pSession->ThrowDBError (e_missing_before_image, "missing before image");
  }
  else {
    pObjContainer->MarkStored();
  }
}

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsVerify (OmsHandle& h) const {}

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsSimDumpToHtmlShort(OmsHandle &Handle, OmsSimDumpCollector &str) const
{
  Handle.m_pSession->m_lcSink->SimCtlDumpToHtml(Handle, omsOid(), str);
}

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsSimDumpToHtmlLong(OmsHandle &Handle, OmsSimDumpCollector &str) const
{
  omsSimDumpToHtmlShort(Handle, str);
}

/*----------------------------------------------------------------------*/

void OmsAbstractObject::omsSimSetObjectName(OmsHandle &Handle, const char *name)
{
  Handle.m_pSession->m_lcSink->SimCtlSetObjectName(Handle, omsOid(), name);
}

/*----------------------------------------------------------------------*/

const char* OmsAbstractObject::omsSimGetObjectName(OmsHandle &Handle) const
{
  const char *name;
  Handle.m_pSession->m_lcSink->SimCtlGetObjectName(Handle, omsOid(), &name);
  return name;
}

/*----------------------------------------------------------------------*/

const char *OmsAbstractObject::omsSimDumpMakeOidRef(OmsHandle &h, const OmsObjectId &oid, int flags)
{
  const char *ref;
  h.m_pSession->m_lcSink->SimCtlGetHTMLRef(h, oid, &ref, flags);
  return ref;
}

/*----------------------------------------------------------------------*/

//
// Implementation of OmsObjByClsIterBase, PTS 1110401
//
OmsObjByClsIterBase::OmsObjByClsIterBase(const OmsObjByClsIterBase& iter) 
{
  memcpy (this, &iter, sizeof(iter));
  if (NULL != iter.m_curr)
  {
    m_curr = NULL;
    m_curr = REINTERPRET_CAST(OmsObjectContainerPtr, 
      m_session->allocate (m_containerInfo->GetObjectSize()));
    memcpy(m_curr, iter.m_curr, m_containerInfo->GetObjectSize());
  }
  if (NULL != iter.m_next)
  {
    m_next = NULL;
    m_next = REINTERPRET_CAST(OmsObjectContainerPtr, 
      m_session->allocate (m_containerInfo->GetObjectSize()));
    memcpy(m_next, iter.m_next, m_containerInfo->GetObjectSize());
  }
}

/*----------------------------------------------------------------------*/

OmsObjByClsIterBase::OmsObjByClsIterBase( OmsHandle& h ) : 
m_session(h.m_pSession), m_containerInfo(), m_iterNewObjectsInVersion(false), m_headIndex(-1), m_curr(NULL), m_next(NULL) {
};

/*----------------------------------------------------------------------*/

OmsObjByClsIterBase::operator bool () const {
  return ((m_curr) && (m_curr->m_oid)) ? true : false;
}

/*----------------------------------------------------------------------*/

const OmsObjectId& OmsObjByClsIterBase::operator() () const {
  return (m_curr) ? m_curr->m_oid : OmsGlob_co10.m_nilOid;
}

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsObjByClsIterBase::omsDeRef (bool forUpd, bool doLock) const{
  if ((m_curr) && (m_curr->m_oid)) {
    m_session->IncDeref();
    OmsObjectContainerPtr found = m_session->m_context->FindObjInContext (&m_curr->m_oid);
    if (found) {
      if (found->DeletedFlag()) {
        m_session->ThrowDBError (e_object_not_found, "OmsObjByClsIterBase()", m_curr->m_oid);
        return NULL;
      }
      else {
        if ((doLock) && (!found->LockedFlag()) && (!m_session->m_context->IsVersion())) {
          m_session->m_context->LockObj(found->m_oid);
        }
        if (forUpd) {
          return m_session->ForUpdPtr(found);
        }
        return &found->m_pobj;
      }
    }
    else {
      OmsObjectContainerPtr obj = m_session->GetMemory (*m_containerInfo); 
      memcpy (obj, m_curr,  m_containerInfo->GetObjectSize());
      m_session->m_context->PutObjectIntoContext (obj, m_containerInfo->GetContainerHandle());
      if ((doLock) && (!m_session->m_context->IsVersion())) {
        m_session->m_context->LockObj(obj->m_oid);
      }
      if (forUpd) {
        return m_session->ForUpdPtr(obj);
      }
      return &obj->m_pobj;
    }
  }
  return NULL;
}

/*----------------------------------------------------------------------*/

void OmsObjByClsIterBase::omsNextNewObjectInVersion(OmsObjectContainerPtr curr) {
  const OmsObjectContainer *obj = curr ? m_session->m_context->m_oidDir.HashFind(&curr->m_oid) : NULL;
  do {
    obj = m_session->m_context->m_newObjCache.nextObject(m_containerInfo->GetContainerHandle(),
      m_containerInfo->GetObjectSize(), obj);
    if (!obj) break;
    if ((!(obj->DeletedFlag())) && (!obj->m_objseq.gg91IsDummyRef())) {
      memcpy (m_next, obj, m_containerInfo->GetObjectSize());
      m_next->UnmarkVerNewObject();
      return;
    }
  } while (true);
/*
  // This is the old coding, expected pointer to *next* object (as parameter obj)
  do {
    while (NULL != obj) {
      if ((VERSION_OBJ_PAGE_NO_GG92 == obj->m_oid.getPno()) && (obj->GetContainerHandle() == m_containerInfo->GetContainerHandle())) { 
        if ((!(obj->DeletedFlag())) && (!obj->m_objseq.gg91IsDummyRef())) {
          memcpy (m_next, obj, m_containerInfo->GetObjectSize());
          return;
        }
      }
      obj = obj->m_hashnext;
    }
    ++m_headIndex;
    if (m_headIndex < m_session->m_context->m_oidDir.m_headentries) {
      obj = m_session->m_context->m_oidDir.m_head[m_headIndex];
    }
    else {
      break;
    }
  }
  while (true);
*/
  m_next->m_oid.setNil();
}

/*----------------------------------------------------------------------*/

void OmsObjByClsIterBase::operator++ ()
{
#if defined(OMSTST)
  m_session->CheckCancelCountDown();
  m_session->CheckExceptionCountDown();
#endif
  OmsObjectContainerPtr curr = m_curr;
  m_curr = m_next;
  m_next = curr;
  if ((m_curr) && (m_curr->m_oid)) {
    OmsObjectContainerPtr obj = NULL;
    tsp00_Int2            DBError;
    tsp00_Int4            size = m_containerInfo->GetPersistentSize();
    
    if (m_iterNewObjectsInVersion) {
      omsNextNewObjectInVersion(m_curr);
      /*
      // in old coding was the pointer to next:
      omsNextNewObjectInVersion(m_curr->m_hashnext);
      */
    }
    else {
      m_next->m_oid = m_curr->m_oid;
      do {
        tsp00_Int4 objHistReadCount;  // PTS 1107819 TA 13/09/2000  
        m_session->IncDerefIter_LC(); // PTS 1107819 TA 13/09/2000 
        CHResult_co001 hr = m_session->m_lcSink->NextObj (
          (unsigned char*) &m_session->m_context->m_consistentView,
          (unsigned char*) &m_containerInfo->GetFileId(),
          m_session->m_context->VersionContext(),
          m_next->m_oid.pOid(), 
          (unsigned char*) &m_next->m_objseq, size, 
          ((unsigned char*) &m_next->m_pobj) + sizeof(void*), 
          objHistReadCount, &DBError);
         if ( DBError != 0 ) { 
          if ( DBError == e_no_next_object) {
            if (m_session->InVersion()) {
              m_iterNewObjectsInVersion = true;
              omsNextNewObjectInVersion(NULL);
            }
            else {
              m_next->m_oid.setNil();
            }
            return;
          }
          else {
            m_session->ThrowDBError (DBError, "Iter op++");
          }
        }
        else {
          m_session->IncLogHopIter(objHistReadCount); // PTS 1107819 TA 13/09/2000 
          obj = m_session->m_context->m_oidDir.HashFind (&m_next->m_oid);
          if (obj) {
            if (!obj->DeletedFlag()) {
              break;
            }
          }
          else {
            break;
          }
        }
      }
      while (true);
    }
  }
}

/*----------------------------------------------------------------------*/

OmsObjByClsIterBase::~OmsObjByClsIterBase() {
  if (m_curr) {
    m_session->deallocate (m_curr);
  }
  if (m_next) {
    m_session->deallocate (m_next);
  }
}

/*----------------------------------------------------------------------*/

//
// Implementation of OmsTimeStamp
//

OmsTimeStamp::OmsTimeStamp(const OmsTypeABAPTimeStamp& timeStamp)
{
  memcpy (&m_TimeStamp, &timeStamp, sizeof(timeStamp));
  ToDateTime();
}

/*----------------------------------------------------------------------*/

const OmsTypeABAPTimeStamp& OmsTimeStamp::omsGetTimeStamp() const
{
  return m_ABAPTimeStamp;
}

/*----------------------------------------------------------------------*/

void OmsTimeStamp::ToDateTime()
{
  int nSec   = m_TimeStamp.nSecond1 * 10 + m_TimeStamp.nSecond2;
  int nMin   = m_TimeStamp.nMinute1 * 10 + m_TimeStamp.nMinute2;
  int nHour  = m_TimeStamp.nHour1   * 10 + m_TimeStamp.nHour2;
  int nDay   = m_TimeStamp.nDay1    * 10 + m_TimeStamp.nDay2;
  int nMonth = m_TimeStamp.nMonth1  * 10 + m_TimeStamp.nMonth2;
  int nYear  = m_TimeStamp.nYear1 * 1000 + m_TimeStamp.nYear2 * 100 +
    m_TimeStamp.nYear3 * 10 + m_TimeStamp.nYear4;
  
  if (nYear < 1970)
  {
    nYear = 1970;
  }
  else if (nYear > 2099)
  {
    nYear = 2099;
  }
  m_DateTime = cco15_DateTime(nYear, nMonth, nDay, nHour, nMin, nSec);
}
/*----------------------------------------------------------------------*/

void OmsTimeStamp::ToTimeStamp()
{
  int n1, n2, n3, n4;

  div10(m_DateTime.seconds(),  n1, n2);
  m_TimeStamp.nSecond1 = n1;
  m_TimeStamp.nSecond2 = n2;
  
  div10(m_DateTime.minutes(),  n1, n2);
  m_TimeStamp.nMinute1 = n1;
  m_TimeStamp.nMinute2 = n2;

  div10(m_DateTime.hours(), n1, n2);
  m_TimeStamp.nHour1 = n1;
  m_TimeStamp.nHour2 = n2;

  div10(m_DateTime.day(), n1, n2);
  m_TimeStamp.nDay1 = n1;
  m_TimeStamp.nDay2 = n2;

  div10(m_DateTime.month(),  n1, n2);
  m_TimeStamp.nMonth1 = n1;
  m_TimeStamp.nMonth2 = n2;

  div1000(m_DateTime.year(), n1, n2, n3, n4);
  m_TimeStamp.nYear1 = n1;
  m_TimeStamp.nYear2 = n2;
  m_TimeStamp.nYear3 = n3;
  m_TimeStamp.nYear4 = n4;
}

/*----------------------------------------------------------------------*/

#if defined(WIN32)
/*int co10_EvalException (OmsHandle* pHandle, LPEXCEPTION_POINTERS lpexceptInfo )  PTS 1106505 FF */
/*int co10_EvalException (IliveCacheSink* m_sysLcSink, LPEXCEPTION_POINTERS lpexceptInfo )PTS 1110787 FF*/
int co10_EvalException (OmsSession_co12* omsSession, LPEXCEPTION_POINTERS lpexceptInfo ) /* PTS 1110787 FF*/
{

#define STATUS_UNHANDLED_CPP_EXCEPTION (unsigned long)0xE06D7363L   
  int  ret = EXCEPTION_EXECUTE_HANDLER; 
  /* char cExceptCode[32]; PTS 1110787 FF */
  switch (lpexceptInfo->ExceptionRecord->ExceptionCode)
  {
  /* PTS 1110787 FF

  case STATUS_WAIT_0:                       // 0x00000000L    
    strcpy(cExceptCode,"WAIT_0"); break;
  case STATUS_ABANDONED_WAIT_0:             // 0x00000080L    
    strcpy(cExceptCode,"ABANDONED_WAIT_0"); break;
  case STATUS_USER_APC:                     // 0x000000C0L    
    strcpy(cExceptCode,"USER_APC"); break;
  case STATUS_TIMEOUT:                      // 0x00000102L    
    strcpy(cExceptCode,"TIMEOUT"); break;
  case STATUS_PENDING:                      // 0x00000103L    
    strcpy(cExceptCode,"PENDING"); break;
  case STATUS_SEGMENT_NOTIFICATION:         // 0x40000005L    
    strcpy(cExceptCode,"SEGMENT_NOTIFICATION"); break;
  case STATUS_GUARD_PAGE_VIOLATION:         // 0x80000001L    
    strcpy(cExceptCode,"GUARD_PAGE_VIOLATION"); break;
  case STATUS_NONCONTINUABLE_EXCEPTION:     // 0xC0000025L    
    strcpy(cExceptCode,"GUARD_PAGE_VIOLATION"); break;
  case STATUS_BREAKPOINT:                   // 0x80000003L
    return EXCEPTION_CONTINUE_SEARCH;
  case STATUS_SINGLE_STEP:                  // 0x80000004L    
    strcpy(cExceptCode,"SINGLE_STEP"); break;
  case STATUS_IN_PAGE_ERROR:                // 0xC0000006L    
    strcpy(cExceptCode,"IN_PAGE_ERROR"); break;
  case STATUS_ILLEGAL_INSTRUCTION:          // 0xC000001DL    
    strcpy(cExceptCode,"ILLEGAL_INSTRUCTION"); break;
  case STATUS_PRIVILEGED_INSTRUCTION:       // 0xC0000096L    
    strcpy(cExceptCode,"PRIVILEGED_INSTRUCTION"); break;
  case STATUS_STACK_OVERFLOW:               // 0xC00000FDL    
    strcpy(cExceptCode,"STACK_OVERFLOW"); break;
  case STATUS_CONTROL_C_EXIT:               // 0xC000013AL    
    strcpy(cExceptCode,"CONTROL_C_EXIT"); break;
  case STATUS_DATATYPE_MISALIGNMENT:        // 0x80000002L   
    strcpy(cExceptCode,"DATATYPE_MISALIGNMENT"); break;
  case STATUS_ACCESS_VIOLATION:             // 0xC0000005L    
    strcpy(cExceptCode,"ACCESS_VIOLATION"); break;
  case STATUS_INVALID_HANDLE:               // 0xC0000008L    
    strcpy(cExceptCode,"INVALID_HANDLE"); break;
  case STATUS_NO_MEMORY:                    // 0xC0000017L    
    strcpy(cExceptCode,"NO_MEMORY"); break;
  case STATUS_INVALID_DISPOSITION:          // 0xC0000026L    
    strcpy(cExceptCode,"INVALID_DISPOSITION"); break;
  case STATUS_ARRAY_BOUNDS_EXCEEDED:        // 0xC000008CL    
    strcpy(cExceptCode,"ARRAY_BOUNDS_EXCEEDED"); break;
  case STATUS_FLOAT_DENORMAL_OPERAND:       // 0xC000008DL    
    strcpy(cExceptCode,"FLOAT_DENORMAL_OPERAND"); break;
  case STATUS_FLOAT_DIVIDE_BY_ZERO:         // 0xC000008EL    
    strcpy(cExceptCode,"FLOAT_DIVIDE_BY_ZERO"); break;
  case STATUS_FLOAT_INEXACT_RESULT:         // 0xC000008FL    
    strcpy(cExceptCode,"FLOAT_INEXACT_RESULT"); break;
  case STATUS_FLOAT_INVALID_OPERATION:      // 0xC0000090L    
    strcpy(cExceptCode,"FLOAT_INVALID_OPERATION"); break;
  case STATUS_FLOAT_OVERFLOW:               // 0xC0000091L    
    strcpy(cExceptCode,"FLOAT_OVERFLOW"); break;
  case STATUS_FLOAT_STACK_CHECK:            // 0xC0000092L    
    strcpy(cExceptCode,"FLOAT_STACK_CHECK"); break;
  case STATUS_FLOAT_UNDERFLOW:              // 0xC0000093L    
    strcpy(cExceptCode,"FLOAT_UNDERFLOW"); break;
  case STATUS_INTEGER_DIVIDE_BY_ZERO:       // 0xC0000094L    
    strcpy(cExceptCode,"INTEGER_DIVIDE_BY_ZERO"); break;
  case STATUS_INTEGER_OVERFLOW:             // 0xC0000095L    
    strcpy(cExceptCode,"INTEGER_OVERFLOW"); break;
  case STATUS_UNHANDLED_CPP_EXCEPTION:      // 0xE06D7363L   
    strcpy(cExceptCode,"UNHANDLED_C++_EXCEPTION"); break;  
  default:
    sprintf(cExceptCode,"ExceptionCode: 0x%8.8X ", 
      lpexceptInfo->ExceptionRecord->ExceptionCode);
    ret = EXCEPTION_CONTINUE_SEARCH;
    */
    /* ++++++  PTS 1110787 ++++++ */
  case STATUS_FLOAT_DIVIDE_BY_ZERO:         // 0xC000008EL    
  case STATUS_FLOAT_INEXACT_RESULT:         // 0xC000008FL    
  case STATUS_FLOAT_OVERFLOW:               // 0xC0000091L    
  case STATUS_FLOAT_UNDERFLOW:              // 0xC0000093L    
  case STATUS_INTEGER_DIVIDE_BY_ZERO:       // 0xC0000094L    
  case STATUS_INTEGER_OVERFLOW:             // 0xC0000095L    
  case STATUS_UNHANDLED_CPP_EXCEPTION:      // 0xE06D7363L   
    ret = EXCEPTION_EXECUTE_HANDLER;                
    break;  
  default:
    ret = EXCEPTION_CONTINUE_SEARCH;                
    break;
    /* ------  PTS 1110787 ------ */
  }
//  pHandle->dbpOpError("%s at address 0x%8.8X", cExceptCode,   PTS 1106505 FF
//    lpexceptInfo->ExceptionRecord-> ExceptionAddress);        PTS 1106505 FF
  if ( ret == EXCEPTION_EXECUTE_HANDLER )                    /* PTS 1110787 FF */
  {                                                          /* PTS 1110787 FF */
    __try                                                    /* PTS 1110787 FF */
    {                                                        /* PTS 1110787 FF */
/* PTS 1106505 FF +++++++++++++++++++++  */
  tsp00_Int2  DummyError;
  tsp00_Int4  BufLen = sizeof(tsp00_Addr);
  tsp00_Addr  Buf = REINTERPRET_CAST(tsp00_Addr,lpexceptInfo);
  IliveCacheSink* m_sysLcSink = omsSession->m_lcSink;      /* PTS 1110787 FF */
  CHResult_co001 hr = m_sysLcSink->MultiPurpose (m_diagnose, mm_dump, &BufLen, 
    (unsigned char*) &Buf, &DummyError);
/* PTS 1106505 FF ---------------------  */
/* PTS 1106505 FF ---------------------  */
    }                                                         /* PTS 1110787 FF */
    __except (EXCEPTION_EXECUTE_HANDLER)                      /* PTS 1110787 FF */
    {                                                         /* PTS 1110787 FF */
      ret = EXCEPTION_CONTINUE_SEARCH;                        /* PTS 1110787 FF */
    }                                                         /* PTS 1110787 FF */
  }                                                           /* PTS 1110787 FF */
  return ret;
}

#endif


