/*


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end


*/
#include "gsp00.h"
#include "gco00.h"
#include "gco102.h"
#include "hco10.h"
#include "gco10.h"
#include "hco12.h"
#include "hsp77.h"

/*----------------------------------------------------------------------*/

OmsContext_co12::OmsContext_co12
(
 OmsSession_co12* session, const OmsVersionId* vid,
 const tgg00_OmsVersionContext* vctxt
 ) 
 : m_session(session)
 , m_oidDir(), m_containerDir()
 , m_consistentView()
 , m_pVersionContext(NULL)
 , m_isOpen(false)
 , m_isDropped(false)
 , m_isVersion(false)
#ifndef USE_SYSTEM_ALLOC_CO13
 , m_heap((const SAPDB_UTF8*) "", *co10_GetKernelInterface()->GetOmsAllocator(), 32 * 1024, 32 * 1024, 
          SAPDBMem_RawAllocator::FREE_RAW_EXTENDS) 
 , m_newObjCache(m_heap)
#endif
 , m_next(0)
 , m_nextUnloaded(0) 
 , m_lastDropId(0)
 , m_boundToTrans(false)
 , m_marked(false)
 , m_date()
 , m_time()
 , m_lastOpenDate()
 , m_lastOpenTime()
 , m_currLcSink(session->m_lcSink)
 , m_sysLcSink(co10_GetLcSink())
 , m_cntNewObjectsToFlush(0)
 , m_versionDesc(NULL)             
{
    session->m_lcSink->GetDateTime(&m_date, &m_time);
    ResetConsistentView();
    m_oidDir.Create(this);
    m_containerDir.Create(this);
    SAPDB_UTF8 heapName[41];
    if (NULL != vid) {
        m_isVersion      = true;
        m_versionContext = *vctxt;
        memcpy (m_version, vid, sizeof(*vid));
        m_isOpen         = true;
        sp77sprintf (REINTERPRET_CAST(char*, &heapName[0]), sizeof(heapName), "OMS Version %s", &vid[0]);
    }
    else
    {
        sp77sprintf (REINTERPRET_CAST(char*, &heapName[0]), sizeof(heapName), "OMS default context T%03d", session->GetTaskId());
    }
#ifndef USE_SYSTEM_ALLOC_CO13
    m_heap.SetIdentifier(&heapName[0]);
    co10_GetKernelInterface()->RegisterAllocator(m_heap.GetAllocatorInfo());
#endif
};

/*----------------------------------------------------------------------*/

OmsContext_co12::~OmsContext_co12() 
{
#ifdef USE_SYSTEM_ALLOC_CO13
    if (m_isVersion)
    {
        
        for (cco13_ClsIdHash::Iter iter = m_containerDir.First(); iter; ++iter) {
            iter()->DeleteSelf(this);       
        }
        m_oidDir.Clear();
        if (m_versionDesc != NULL){  // PTS 1117690
            deallocate(m_versionDesc);
        }
    }
#else
    co10_GetKernelInterface()->DeregisterAllocator(m_heap.GetAllocatorInfo());
#endif
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::CleanContainerDir() {
  tsp00_Uint4 lastDropId = co10_GetLastDropId();
  if (lastDropId != m_lastDropId) {    
    m_containerDir.Clean();
    m_lastDropId = lastDropId;
  }
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::DeleteSelf() {
    tgg00_BasisError DBError;
    if (m_isVersion) {                /* FF 16.Nov.99  PTS 1104668 */
        m_versionContext.ovc_trans_version = m_consistentView;
        CHResult_co001 hr = m_currLcSink->DropVersion((unsigned char*) &m_versionContext, &DBError);
        if (0 != DBError)
        {
            // just write a message into knldiag and ignore error
            DbpBase opMsg(m_currLcSink);
            opMsg.dbpOpError("drop version, error %d ignored", DBError);
        }
    }
    this->OmsContext_co12::~OmsContext_co12();
    SharedMemAllocator_co16.deallocate(this);
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::Dump(IDump_cco02& dumpObj) const
{
  struct ContextDumpInfo
  {
    void*                    m_this;
    OmsVersionId             m_version;
    tsp00_Int2               m_filler1;
    OmsSession_co12*         m_session;
    IliveCacheSink*          m_currLcSink;
    IliveCacheSink*          m_sysLcSink; 
    tgg00_OmsVersionContext* m_pVersionContext;
    tsp00_Uint4              m_lastDropId;
    tgg91_TransNo            m_consistentView;
    bool                     m_isOpen;
    bool                     m_isDropped;
    bool                     m_isVersion;
    bool                     m_boundToTrans;
    bool                     m_marked;
    bool                     m_filler2;
  } contextDumpInfo;

  contextDumpInfo.m_this            = CONST_CAST(OmsContext_co12*, this);
  memcpy(&contextDumpInfo.m_version[0], &m_version[0], sizeof(m_version));
  contextDumpInfo.m_filler1         = 0;
  contextDumpInfo.m_session         = m_session;
  contextDumpInfo.m_currLcSink      = m_currLcSink;
  contextDumpInfo.m_sysLcSink       = m_sysLcSink; 
  contextDumpInfo.m_pVersionContext = m_pVersionContext;
  contextDumpInfo.m_lastDropId      = m_lastDropId;
  contextDumpInfo.m_consistentView  = m_consistentView;
  contextDumpInfo.m_isOpen          = m_isOpen;
  contextDumpInfo.m_isDropped       = m_isDropped;
  contextDumpInfo.m_isVersion       = m_isVersion;
  contextDumpInfo.m_boundToTrans    = m_boundToTrans;
  contextDumpInfo.m_marked          = m_marked;
  contextDumpInfo.m_filler2         = false;
  dumpObj.SetDumpLabel(LABEL_OMS_CONTEXT);
  dumpObj.Dump(&contextDumpInfo, sizeof(contextDumpInfo));
  m_oidDir.Dump(dumpObj);
  m_containerDir.Dump(dumpObj);
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::DumpObjCache(OmsHandle& h) {
  char                  flags[4];
  flags[sizeof(flags)-1] = 0;
  OmsObjectContainerPtr curr;
  for (cco13_OidHash::OidIter iter = m_oidDir.First(); iter; ++iter) {
    curr = iter();
    if (curr->DeletedFlag()) {
        flags[0] = 'D';
      }
      else {
        flags[0] = ' ';
      }
      if (curr->StoredFlag()) {
        flags[1] = 'U';
      }
      else {
        flags[1] = ' ';
      }
      if (curr->LockedFlag()) {
        flags[2] = 'L';
      }
      else {
        flags[2] = ' ';
      }
      curr->m_pobj.omsDump(h, flags);
  }
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::EmptyObjCache(tsp00_Uint4 containerHandle) {
  OmsObjectContainerPtr curr;
  tco13_ClsIdEntry* pContainerInfo ;
  cco13_OidHash::OidIter iter = m_oidDir.First();

  while (iter) {
    curr = iter();
    ++iter;
    if (0xFFFFFFFF == containerHandle || curr->GetContainerHandle() == containerHandle) 
    {
      pContainerInfo = curr->GetContainerInfo(this);
      if (curr->IsNewObject())
      {
        OMS_DETAIL_TRACE(omsTrNewObj, m_session->m_lcSink, 
          "OmsContext_co12::EmptyObjCache : new obj " << curr->m_oid 
          << ", class: " << pContainerInfo->GetClassInfoPtr()->GetClassName());
        this->DecNewObjectsToFlush();
      }
      DeleteObjInContext(curr, pContainerInfo);
    }
  }
};

/*----------------------------------------------------------------------*/

void OmsContext_co12::FlushObjCache(bool emptyObjCache, bool adaptOidHash) {
  bool                   errorOccured = false;
  bool                   checkHash    = false;
  int                    newObjFlushed = 0;
  OmsObjectContainerPtr  curr;
  tco13_ClsIdEntry*      clsinfo;
  DbpError               error(0);

  cco13_OidHash::OidIter iter = m_oidDir.First();
  bool varflushed = true;
  while (iter) {
    if (checkHash) {
    m_oidDir.HashCheck();
    }
    curr = iter();
    ++iter;
    curr->m_beforeImages = 0;
    if (!errorOccured) {
      try {
        if (curr->DeletedFlag()) 
        {
          if (curr->IsNewObject())
          {
            ++newObjFlushed;
          }
          FlushDelete(curr, !emptyObjCache);
        }
        else {
          if (curr->VarObjFlag()) {
            varflushed = false;
          }
          if (curr->StoredFlag()) 
          { 
            if (curr->IsNewObject())
            {
              ++newObjFlushed;
            }
            FlushObj (curr);
          }
          else { /* CR 1103570 FF 12-Aug-1999 */
            if ((curr->LockedFlag()) && (!IsVersion()))
            { 
              FlushLockUpdObj (curr);
            }
          }
        }
      }
      catch (DbpError& e) {
        error        = e;
        errorOccured = true;
      }
    }
    if (emptyObjCache) {
      clsinfo = curr->GetContainerInfo(this);
      clsinfo->chainFree(*this, curr);
    }
    else {
      if (!varflushed) {
        REINTERPRET_CAST(OmsVarObjInfo_co12*, &curr->m_pobj)->freeVarObj(this);
        varflushed = true;
      }
    }
  }

#ifdef USE_SYSTEM_ALLOC_CO13
  ClearObjCache();
#endif
  m_oidDir.SetEmpty(adaptOidHash);
  m_newObjCache.SetEmpty();

  int cntNewObjToFlush = m_cntNewObjectsToFlush;
  m_cntNewObjectsToFlush = 0;
  if (errorOccured) {
    co10_Throw(error);
  }
  if ((!this->IsVersion()) && (newObjFlushed != cntNewObjToFlush))
  {
    OMS_DETAIL_TRACE(omsTrNewObj, m_session->m_lcSink, 
      "OmsContext_co12::FlushObjCache : flushed new objects: "
      << newObjFlushed << ", expected to flush: " << cntNewObjToFlush);
    DbpBase b(m_currLcSink);
    b.dbpOpError ("Flushed new objects           : %d", newObjFlushed);
    b.dbpOpError ("expected new objects to flush : %d", cntNewObjToFlush);
    co10_Throw(DbpError(e_missing_object_flush));
  }
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::FlushObj(OmsObjectContainerPtr p) {
  if (p->VarObjFlag()) {
    FlushVarObject (p);
  }
  else {
    tsp00_Int2  DBError;
    tco13_ClsIdEntry* clsinfo = p->GetContainerInfo(this);
    CHResult_co001 hr = m_currLcSink->UpdateObj (
      (unsigned char*) &m_consistentView,
      (unsigned char*) &clsinfo->GetFileId(),
      VersionContext(),
      (unsigned char*) &p->m_oid,
      (unsigned char*) &p->m_objseq, 
      clsinfo->GetPersistentSize(), 
      ((unsigned char*) &p->m_pobj) + sizeof(void*), 
      (unsigned char*) &clsinfo->GetInvColDesc(), 
      &DBError);
    this->CheckDBError(DBError, "OmsContext_co12::FlushObj", p->m_oid); 
    p->RemoveStoredFlag();
    m_session->IncStore_lC();
  }
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::FlushDelete (OmsObjectContainerPtr pObj, bool deleteInHash) {
  
  tsp00_Int2     DBError;
  CHResult_co001 hr;
  tco13_ClsIdEntry* clsinfo = pObj->GetContainerInfo(this);
  if (pObj->VarObjFlag()) {
    m_session->IncDeleteVarObjectLC ();
    bool                    isFirst = true;
    OmsVarObjInfoNode_co12* pInfoNode; 
    OmsVarObjInfo_co12*     objInfo = REINTERPRET_CAST (OmsVarObjInfo_co12*, &pObj->m_pobj);
    pInfoNode = &objInfo->m_vobjInfo;
    while (NULL != pInfoNode) {
      if ((!isFirst) && (!IsVersion())) {
        // ADIS 1001523, T.A. 24.09.1998
        hr = m_currLcSink->LockObj (
          (unsigned char*) &m_consistentView,
          (unsigned char*) &clsinfo->GetFileId(),
          REINTERPRET_CAST(OmsTypeOid*, &pInfoNode->m_vobjOid),
          (unsigned char*) &pInfoNode->m_vobjSeq,
          &DBError);
        this->CheckDBError(DBError, "OmsContext_co12::FlushDelete (VarObj)", pInfoNode->m_vobjOid);
      }
      hr = m_currLcSink->DeleteObj (
        (unsigned char*) &m_consistentView,
        (unsigned char*) &clsinfo->GetFileId(),
        VersionContext(),
        (unsigned char*) &pInfoNode->m_vobjOid,
        (unsigned char*) &pInfoNode->m_vobjSeq, 
        (unsigned char*) &clsinfo->GetInvColDesc(),
        &DBError);
      this->CheckDBError(DBError, "OmsContext_co12::FlushDelete (VarObj)", pInfoNode->m_vobjOid);
      pInfoNode = pInfoNode->m_vobjNext;
      isFirst   = false;
    }
  } 
  else { // no Var Object
    m_session->IncDelete_lC();
    hr = m_currLcSink->DeleteObj (
      (unsigned char*) &m_consistentView,
      (unsigned char*) &clsinfo->GetFileId(),
      VersionContext(),
      (unsigned char*) &pObj->m_oid, 
      (unsigned char*) &pObj->m_objseq,
      (unsigned char*) &clsinfo->GetInvColDesc(),
      &DBError);
    this->CheckDBError(DBError, "OmsContext_co12::FlushDelete", pObj->m_oid);
  }
  if (deleteInHash) {
    DeleteObjInContext (pObj, clsinfo);
  }
}


/*----------------------------------------------------------------------*/

void OmsContext_co12::FlushLockUpdObj(OmsObjectContainerPtr p) {
  tsp00_Int2  DBError;
  tco13_ClsIdEntry* clsinfo = p->GetContainerInfo(this);
  CHResult_co001 hr = m_currLcSink->LockUpdObj (
    (unsigned char*) &m_consistentView,
    (unsigned char*) &clsinfo->GetFileId(),
    REINTERPRET_CAST(OmsTypeOid*, &p->m_oid),
    (unsigned char*) &p->m_objseq, 
    &DBError);
  this->CheckDBError(DBError, "OmsContext_co12::FlushLockUpdObj", p->m_oid); 
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::FlushVarObject (OmsObjectContainerPtr pObj) {

  CHResult_co001 hr;
  tsp00_Int2     DBError;
  tsp00_Uint4    moveLen;
  int            vObjLen;
  tgg91_PageRef  newObjSeq;
  bool           isNew    = true;
  bool           isLocked = false;
  bool           isFirst  = true;
  tsp00_Uint4    objLength;
  OmsVarObjContainer_co001 varObjContainer;
  OmsVarObjInfoNode_co12*  pInfoNode; 
  OmsVarObjInfoNode_co12*  pLastNode;

  m_session->IncStoreVarObjLC();
  tco13_ClsIdEntry* clsinfo = pObj->GetContainerInfo(this);
  OmsVarObjInfo_co12* objInfo  = REINTERPRET_CAST (OmsVarObjInfo_co12*, &pObj->m_pobj);
  unsigned char* p             = objInfo->m_pvobj;
  pInfoNode                    = &objInfo->m_vobjInfo;
  objLength                    = objInfo->m_vobjSize;
  varObjContainer.m_vobjSize   = objInfo->m_vobjSize;
  do {
    isLocked = isNew;
    isNew    = false;
    moveLen = co001_Min (VAR_OBJ_CHUNK_SIZE, objLength);
    if (moveLen >= objLength) {
      varObjContainer.m_vobjNext.setNil();
    }
    else {
      if (pInfoNode->m_vobjNext) {
        memcpy (&varObjContainer.m_vobjNext, &pInfoNode->m_vobjNext->m_vobjOid, 
          sizeof (pInfoNode->m_vobjNext->m_vobjOid));
      }
      else {
        if (IsVersion()) /* PTS 1110149 */
        {
          OmsObjectId newOid = VersionNewOid();
          memcpy(&varObjContainer.m_vobjNext, &newOid, sizeof(varObjContainer.m_vobjNext));
          newObjSeq.gg91SetNilRef();
        }
        else
        {
          hr = m_currLcSink->NewObj (
            (unsigned char*) &m_consistentView,
            (unsigned char*) &clsinfo->GetFileId(), 
            VersionContext(),
            0,
            0,
            NULL,
            REINTERPRET_CAST(OmsTypeOid*, &varObjContainer.m_vobjNext),
            (unsigned char*) &newObjSeq,
            &DBError);
          this->CheckDBError(DBError, "OmsContext_co12::FlushVarObject", pObj->m_oid); 
        }
        isNew                 = true;
        pInfoNode->m_vobjNext = new(*this) OmsVarObjInfoNode_co12 (varObjContainer.m_vobjNext, newObjSeq);
      }
    }
    memcpy (&varObjContainer.vobjInfo[0], p, moveLen);
    if ((!isLocked) && (!IsVersion())) {
      hr = m_currLcSink->LockObj (
        (unsigned char*) &m_consistentView,
        (unsigned char*) &clsinfo->GetFileId(),
        REINTERPRET_CAST(OmsTypeOid*, &pInfoNode->m_vobjOid),
        (unsigned char*) &pInfoNode->m_vobjSeq, &DBError);
      this->CheckDBError(DBError, "OmsContext_co12::FlushVarObject", pInfoNode->m_vobjOid); 
    }
    objLength -= moveLen;
    vObjLen    = (objLength > 0) ? (int) sizeof(varObjContainer) : moveLen;
    hr = m_currLcSink->UpdateObj (
      (unsigned char*) &m_consistentView,
      (unsigned char*) &clsinfo->GetFileId(),
      VersionContext(),
      (unsigned char*) &pInfoNode->m_vobjOid,
      (unsigned char*) &pInfoNode->m_vobjSeq,
      vObjLen, 
      (unsigned char*) &varObjContainer, 
      (unsigned char*) &clsinfo->GetInvColDesc(),
      &DBError);
    this->CheckDBError(DBError, "OmsContext_co12::FlushVarObject", pInfoNode->m_vobjOid); 
    if (isFirst) {
      pObj->m_objseq = pInfoNode->m_vobjSeq;
      isFirst        = false;
    }
    p         += moveLen;
    pLastNode  = pInfoNode;
    pInfoNode  = pInfoNode->m_vobjNext;
  }
  while (objLength > 0);
  pLastNode->m_vobjNext = NULL;
  if (!IsVersion()) {
    objInfo->freeVarObj(this);
  }
  while (pInfoNode) {
    if (!IsVersion()) {
      hr = m_currLcSink->LockObj (
        (unsigned char*) &m_consistentView,
        (unsigned char*) &clsinfo->GetFileId(),
        REINTERPRET_CAST(OmsTypeOid*, &pInfoNode->m_vobjOid),
        (unsigned char*) &pInfoNode->m_vobjSeq, &DBError);
    }
    if (0 == DBError) {
      hr = m_currLcSink->DeleteObj (
        (unsigned char*) &m_consistentView,
        (unsigned char*) &clsinfo->GetFileId(),
        VersionContext(),
        (unsigned char*) &pInfoNode->m_vobjOid,
        (unsigned char*) &pInfoNode->m_vobjSeq, 
        (unsigned char*) &clsinfo->GetInvColDesc(),
        &DBError);
    }
    this->CheckDBError(DBError, "OmsContext_co12::FlushVarObject", pInfoNode->m_vobjOid); 
    pLastNode = pInfoNode;
    pInfoNode = pInfoNode->m_vobjNext;
    pLastNode->deleteSelf(*this);
  }
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::MarkNotBoundToTrans(bool callFromDestructor) { 
  m_boundToTrans = false;
  if (!m_isOpen) {
    co10_MarkUnloadable(this, callFromDestructor);
    this->ResetLcSink();
  }
}

/*----------------------------------------------------------------------*/

void  OmsContext_co12::OpenVersion(OmsSession_co12* session) {
  AssignLcSink(session->m_lcSink);
  m_currLcSink->GetDateTime(&m_lastOpenDate, &m_lastOpenTime);
  m_session    = session;
  m_isOpen     = true;
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::ReduceHeapUsage() /* PTS 1110149 */
{
  this->UnLoad();
  m_containerDir.UnregisterAll();
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::ResetLcSink() {
  m_currLcSink = NULL;
  m_sysLcSink  = NULL;
}

/*----------------------------------------------------------------------*/

/* PTS 1115134 */

void  OmsContext_co12::ResetVersion(OmsSession_co12& session) 
{
  if (m_isVersion)
  {
    this->AssignLcSink(session.m_lcSink);
    this->EmptyObjCache();
    for (cco13_ClsIdHash::Iter iter = m_containerDir.First(); iter; ++iter)
    {
      iter()->VersionDelIndex(false, this);
    }
    this->AssignLcSink(NULL);
    tgg00_BasisError DBError;
    session.m_lcSink->ResetVersion((unsigned char*) &m_versionContext, DBError);
    this->CheckDBError(DBError, "OmsContext_co12::ResetVersion", OmsObjectId());
  }
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::UnLoad() {
  const bool emptyObjCache = true;
  pasbool*     pToCancel;
  tsp00_TaskId taskId;
  // reassign pointer to session, may not be valid, if called from
  // bad alloc handling
  co10_GetLcSink()->GetDefaultContext(REINTERPRET_CAST(void**, &this->m_session), &pToCancel, taskId);
  m_pVersionContext = &m_versionContext;
  FlushObjCache(emptyObjCache, false); // No hash-adaptation PTS 1119061
  m_containerDir.ClearFreeLists();
  for (cco13_ClsIdHash::Iter iter = m_containerDir.First(); iter; ++iter) {
    iter()->VersionDelIndex(false, this);
  }
  co10_GetKernelInterface()->IncOmsVersionUnloadCounter();
}

/*----------------------------------------------------------------------*/

void  OmsContext_co12::VersionClearObjCache() {

  OmsObjectContainerPtr curr;

  for (cco13_OidHash::OidIter iter = m_oidDir.First(); iter; ++iter) {
    curr = iter();
    if ((curr->StoredFlag()) || (curr->DeletedFlag())) {
      this->CheckDBError(e_transaction_end_required, "OmsContext_co12::VersionClearObjCache", OmsObjectId()); 
    }
    if (curr->VarObjFlag()) {
      REINTERPRET_CAST(OmsVarObjInfo_co12*, &curr->m_pobj)->freeVarObj(this);
    }
    else {
      if (curr->LockedFlag()) {
        this->FlushObj(curr);
      }
    }
  }  
  m_oidDir.SetEmpty();
  m_newObjCache.SetEmpty();
}

/*----------------------------------------------------------------------*/

OmsSession_co12::OmsSession_co12
(
 IliveCacheSink* lcSink, 
 int             cntRegions,    
 pasbool*        pToCancel,
 tsp00_TaskId    taskId) 
 :
m_callbackInterface(NULL),
m_lcSink(lcSink), m_sizeVarobj(0), m_cntVarobj(0), m_timeout(0), m_refCnt(0),
m_subtrans_lvl(1),m_stream_io(true), m_read_only(false), m_allowReadOnly(true), m_inMethodCallEpilog(false),
m_lockObjects(NULL),
m_context(NULL), m_defaultContext(NULL), m_beforeImages(),
#ifndef USE_SYSTEM_ALLOC_CO13
m_stackHeap(),
#endif
m_monitor(taskId),
m_monitor_curr(NULL),
m_versionsBoundToTrans(NULL), m_rescheduleDistance(RESCHEDULE_DISTANCE), m_heapAtMethodBegin(0),
m_toCancel(*pToCancel), /* PTS 1107849 */
m_taskId(taskId),
m_exceptionCountDown(-1),
m_cancelCountDown(-1),   
m_requiredExceptionThrown(false),
m_handleList(NULL),       // PTS 1116693
m_errorCode(0)              // PTS 1122838
{
  memset(&m_callbackInterfaceGuid, 0, sizeof(m_callbackInterfaceGuid));
  int ix;
  m_lockObjects = ISessionLockObjects_co11::Create(*this);
  m_beforeImages.init(this);
  m_versionsBoundToTrans.advise_allocator(this);
  m_handleList.advise_allocator(this);
  m_critical_section_in_use = REINTERPRET_CAST(bool*, allocate(cntRegions * sizeof(bool)));
  for (ix = 0; ix < cntRegions; ++ix) {
    m_critical_section_in_use[ix] = false;
  }
}

/*----------------------------------------------------------------------*/

OmsSession_co12::~OmsSession_co12() {

  m_lockObjects->Destroy();
  if (this->InVersion()) {
    OmsContext_co12* pVersion = m_context;
    this->CloseVersion();
    pVersion->MarkNotBoundToTrans(true);
  }
  m_lcSink->SetDefaultContext(NULL);
  if (NULL != m_defaultContext) {
    m_defaultContext->DeleteSelf();
  }
  deallocate(m_critical_section_in_use);
  // Set Pointer in OmsHandles to this instance to NULL   // PTS 1116693
  for (cgg251dclIterator<OmsHandle*,OmsSession_co12> iter 
    = m_handleList.begin(); iter; ++iter) {
 	(*iter())->m_pSession = NULL; 
  }
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::AssignLcSink(IliveCacheSink* pSink) /* PTS 1109371 */
{
  m_lcSink = pSink;
  m_defaultContext->AssignLcSink(pSink);
  m_context->AssignLcSink(pSink);
}

/*----------------------------------------------------------------------*/

const OmsObjectId& OmsSession_co12::CastOid(const ClassIDRef castToGuid, const OmsObjectId& oid) {
  
  if (!oid) { // Null Oid given
    return oid;
  }
  OmsObjectContainerPtr p = m_context->GetObj(oid, false);
  tco13_ClsIdEntry*     pFrom = p->GetContainerInfo(m_context);
  if (pFrom->GetGuid() == castToGuid) {
    return oid;
  }
  if (m_context->m_containerDir.IsDerivedClassOf(pFrom->GetGuid(), castToGuid)) {
    return oid;
  }
  if (m_context->m_containerDir.IsBaseClassOf(castToGuid, pFrom->GetGuid())) {
    return oid;
  }
  char msg[80];
#ifdef LIVECACHE_INTGUIDS
  //sprintf (msg, "guid : %8X", castToGuid);                   // PTS 1125307
  sp77sprintf (msg, sizeof(msg), "guid : %8X", castToGuid);    // PTS 1125307 
#else
  //sprintf (msg, "guid : %8X-%4X-%4X-%1X%1X%1X%1X%1X%1X%1X%1X", castToGuid.Data1, castToGuid.Data2, castToGuid.Data3,
  //  castToGuid.Data4[0], castToGuid.Data4[1], castToGuid.Data4[2], castToGuid.Data4[3], castToGuid.Data4[4], castToGuid.Data4[5],
  //  castToGuid.Data4[6], castToGuid.Data4[7]);               // PTS 1125307 
  sp77sprintf (msg, sizeof(msg), "guid : %8X-%4X-%4X-%1X%1X%1X%1X%1X%1X%1X%1X", castToGuid.Data1, castToGuid.Data2, castToGuid.Data3,
    castToGuid.Data4[0], castToGuid.Data4[1], castToGuid.Data4[2], castToGuid.Data4[3], castToGuid.Data4[4], castToGuid.Data4[5],
    castToGuid.Data4[6], castToGuid.Data4[7]);                 // PTS 1125307
#endif
  this->ThrowDBError (e_incompatible_oid, msg, oid);
  return oid;
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::ChangedConsistentView() {
  m_currVarObjChunk.Clear();
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::CleanAfterDropSchema() {
  m_defaultContext->m_oidDir.Clean();
  m_defaultContext->CleanContainerDir();
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::ClearDefaultContext() {
  m_defaultContext->ClearObjCache();
  this->ClearFreeLists();
  this->FreeStackHeap();
}

/*----------------------------------------------------------------------*/

bool OmsSession_co12::IsLocked (const OmsObjectId& oid) {
 
  if (m_context->IsVersion()) {
    return true;
  }
  OmsObjectContainerPtr found = m_context->FindObjInContext (&oid);
  if ((NULL != found) && (found->LockedFlag())) {
    return true;
  }
  tgg00_BasisError         DBError;
  CUnknownContainerId_co12 UnknownFileId;
  CHResult_co001 hr = m_lcSink->IsLocked ((unsigned char*) &UnknownFileId, 
    REINTERPRET_CAST(OmsTypeOid*, CONST_CAST(OmsObjectId*, &oid)), &DBError);
  if (e_ok == DBError) {
    return true;
  } 
  else {
    if (e_object_not_locked == DBError) {
      return false;
    }
    else {
      this->ThrowDBError (DBError, "IsLocked", oid);
    }
  }
  return false;
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::CreateVersion(const OmsVersionId& versionId, const OmsTypeWyde* desc) {

  const char* msg = "OmsSession_co12::CreateVersion";

  tsp00_Int2              DBError;
  tgg00_OmsVersionContext versionContext;
  OmsContext_co12*        context = NULL;

  this->IncCreateVersion   ();
  CHResult_co001 hr = m_lcSink->CreateOpenVersion (
      (unsigned char*) &m_context->m_consistentView,
      (OmsVersionId*)  &versionId,
      true,
      (unsigned char*) &versionContext,
      &DBError);
  if (DBError != 0) {
      this->ThrowDBError (DBError, msg, OmsObjectId());
  }
  /* create a new context for the version */
  try {
    context = new OmsContext_co12(this, &versionId, &versionContext);
    context->NewConsistentView();
    context->ResetLcSink();
    this->OpenVersion(context);
    context->SetVersionDesc(desc);     // PTS 1117690
  }
  catch (DbpError& e) {
    if (context != NULL) {
      context->AssignLcSink(m_lcSink); // PTS 1108278
      context->DeleteSelf();
    }
    OMS_TRACE(omsTrError, m_lcSink, "Create Version Error : " << e.dbpError());
    throw e;
  }
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::CurrentMethodCallEpilog(tsp00_Int4 runTime)
{
  m_requiredExceptionThrown = false;
  tsp00_8ByteCounter heapUsed;
  m_lcSink->GetSessionHeapUsage(heapUsed);
  if (runTime > 0) // PTS 1126793
  {
      m_monitor.SetRunTime(runTime);
  }
  if (NULL != m_monitor_curr) // PTS 1112146
  {
    m_monitor.AggregateCallStatistics(m_monitor_curr);
    OmsMonitorData info;
    const bool AllocatorOnly = true;
    m_lcSink->EvalCallStatistics(AllocatorOnly, info);
    m_monitor_curr->cmiCounters_gg00[cmiOutOfMemory] += info.m_cntOutOfMemory; // PTS 1126793
    if (info.m_cntOperatorNew > 0)
    {
      m_monitor_curr->cmiCounters_gg00[cmiMallocCnt] += info.m_cntOperatorNew;
      if (info.m_minChunkAllocated < m_monitor_curr->cmiCounters_gg00[cmiMallocMin])
      {
        m_monitor_curr->cmiCounters_gg00[cmiMallocMin] = info.m_minChunkAllocated; 
      }
      if (info.m_maxChunkAllocated > m_monitor_curr->cmiCounters_gg00[cmiMallocMax])
      {
        m_monitor_curr->cmiCounters_gg00[cmiMallocMax] = info.m_maxChunkAllocated; 
      }
      m_monitor_curr->cmiCounters_gg00[cmiMallocSum] += info.m_avgChunkAllocated * info.m_cntOperatorNew;
    }
    if (info.m_cntOperatorDelete > 0)
    {
        m_monitor_curr->cmiCounters_gg00[cmiFreeCnt] += info.m_cntOperatorDelete;
        if (info.m_minChunkDeleted < m_monitor_curr->cmiCounters_gg00[cmiFreeMin])
        {
            m_monitor_curr->cmiCounters_gg00[cmiFreeMin] = info.m_minChunkDeleted; 
        }
        if (info.m_maxChunkDeleted > m_monitor_curr->cmiCounters_gg00[cmiFreeMax])
        {
            m_monitor_curr->cmiCounters_gg00[cmiFreeMax] = info.m_maxChunkDeleted; 
        }
        m_monitor_curr->cmiCounters_gg00[cmiFreeSum] += info.m_avgChunkDeleted * info.m_cntOperatorDelete;
    }
    if ((heapUsed > m_heapAtMethodBegin) &&
        ( heapUsed > m_monitor_curr->cmiCounters_gg00[cmiCacheSize]))
		{
      m_monitor_curr->cmiCounters_gg00[cmiCacheSize] = heapUsed;
		}
		tsp00_8ByteCounter heapDelta = heapUsed - m_heapAtMethodBegin;  
    if (heapDelta < m_monitor_curr->cmiCounters_gg00[cmiDeltaMin])
    {
      m_monitor_curr->cmiCounters_gg00[cmiDeltaMin] = heapDelta; 
    }
    if (heapDelta > m_monitor_curr->cmiCounters_gg00[cmiDeltaMax])
    {
      m_monitor_curr->cmiCounters_gg00[cmiDeltaMax] = heapDelta;
    }
    tsp00_8ByteCounter callCnt = m_monitor_curr->cmiCallCnt_gg00;
    if (callCnt > 0)
    {
      m_monitor_curr->cmiCounters_gg00[cmiDeltaAvg] =
        (m_monitor_curr->cmiCounters_gg00[cmiDeltaAvg] * (callCnt - 1) + heapDelta) / callCnt;
    }
  }
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::DeleteAll(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) {

  const char* msg = "OmsSession_co12::DeleteAll"; 

  this->AssertNotReadOnly(msg);
  tco13_ClsIdEntry* clsinfo = this->GetClsInfo (guid, Schema, ContainerNo);
  tsp00_Int4  cntDeleted;
  tsp00_Int2  DBError;
  OmsTypeOid  errorOid;
  CHResult_co001 hr = m_lcSink->DeleteAllObj (m_context->m_consistentView, (unsigned char*) &clsinfo->GetFileId(), m_context->VersionContext(),
    cntDeleted, DBError, errorOid);
  m_context->LockResult (DBError, NULL, *REINTERPRET_CAST(OmsObjectId*, &errorOid), msg);
  m_context->EmptyObjCache(clsinfo->GetContainerHandle());
  this->IncDelete_lC (cntDeleted);
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::DeleteVarObject(const OmsVarOid& oid) 
{
  const char* msg = "OmsSession_co12::DeleteVarObject";

  this->AssertNotReadOnly(msg);
  this->IncDeleteVarObject();
  OmsObjectContainerPtr found = this->LoadVarObject(oid, VarObjExclusiveLock, -1, NULL);
  if (!found) {
    this->ThrowDBError (e_object_not_found, msg, oid);
  }
  OmsVarObjInfo_co12* pObjInfo = REINTERPRET_CAST (OmsVarObjInfo_co12*, &found->m_pobj); /* PTS 1106727 */
  pObjInfo->unlock();
  if (!found->LockedFlag()) {
    if (!IsLocked(oid)) {
      this->ThrowDBError (e_object_not_locked, msg, oid);
    }
  }
  if (!this->InsertBeforeImage (found)) {
    m_context->FreeVarObj(pObjInfo->m_pvobj, pObjInfo->m_vobjSize);
  }
  pObjInfo->m_pvobj     = NULL;
  pObjInfo->m_vobjSize  = 0;
  found->MarkDeleted();
  if (oid == m_currVarObjChunk.m_oid) {
    m_currVarObjChunk.Clear();
  }
}

/*----------------------------------------------------------------------*/

const  void* OmsSession_co12::DeRefVarObject(const OmsVarOid& oid) {
 
  this->IncLoadVarObj();
  OmsObjectContainerPtr p = m_context->FindVarObjInContext(oid, VarObjShareLock, 0xFFFFFFFF, NULL);
  if (NULL == p) {
    tsp00_Uint4 objSize = m_context->GetVarObjFromLiveCacheBase(oid, false);
    p = m_context->LoadVarObject(oid, VarObjShareLock, objSize, m_currVarObjChunk, NULL);
    m_currVarObjChunk.Clear();
  }
  OmsVarObjInfo_co12* pObjInfo = REINTERPRET_CAST (OmsVarObjInfo_co12*, &p->m_pobj);
  return pObjInfo->m_pvobj;
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::DropVersion(OmsContext_co12* context) 
{
  DropVersionProlog (context);
  DropVersionEpilog (context);
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::DropVersionProlog(OmsContext_co12* context) {

  const char* msg = "OmsSession_co12::DropVersionProlog";
  if (context->IsBoundToTrans()) {
    /* version is bound to trans, bound by myself ? */
    if ((context != CurrentContext()) && (!this->VersionBoundByMe(context))) {
      this->ThrowDBError (e_missing_privilege, msg);
    }
    context->CloseVersion();
    this->RemoveFromTransVersion(context);
  }
  else {
    context->AssignLcSink(m_lcSink);
  }
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::DropVersionEpilog(OmsContext_co12* context)
{
  m_beforeImages.removeContext(context);
  /* free context */
  context->DeleteSelf();
  /* return to default context */
  this->ActivateDefaultContext();
  this->IncDropVersion();
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::Dump(IDump_cco02& dumpObj) const
{
  struct SessionDumpInfo 
  {
    OmsSession_co12*          m_this;
    ClassID				      m_callbackInterfaceGuid;
    OmsCallbackInterface*     m_callbackInterface;
    IliveCacheSink*           m_lcSink;
    bool*                     m_critical_section_in_use;
    ISessionLockObjects_co11* m_lockObjects;
    OmsContext_co12*          m_context;
    OmsContext_co12*          m_defaultContext;
    tsp00_Int4                m_timeout;
    tsp00_Int2                m_refCnt;
    tsp00_Int2                m_subtrans_lvl;
    bool                      m_stream_io;
    bool                      m_read_only;
    tsp00_Int2                m_filler1;
    tsp00_Int4                m_filler2;
  } sessionDumpInfo;
  sessionDumpInfo.m_this                     = CONST_CAST(OmsSession_co12*, this);
  sessionDumpInfo.m_callbackInterfaceGuid    = m_callbackInterfaceGuid;
  sessionDumpInfo.m_callbackInterface        = m_callbackInterface; 
  sessionDumpInfo.m_lcSink                   = m_lcSink;
  sessionDumpInfo.m_critical_section_in_use  = m_critical_section_in_use;
  sessionDumpInfo.m_lockObjects              = m_lockObjects;
  sessionDumpInfo.m_context                  = m_context;
  sessionDumpInfo.m_defaultContext           = m_defaultContext;
  sessionDumpInfo.m_timeout                  = m_timeout;
  sessionDumpInfo.m_refCnt                   = m_refCnt;
  sessionDumpInfo.m_subtrans_lvl             = m_subtrans_lvl;
  sessionDumpInfo.m_stream_io                = m_stream_io;
  sessionDumpInfo.m_read_only                = m_read_only;
  dumpObj.SetDumpLabel(LABEL_OMS_SESSION);
  dumpObj.Dump(&sessionDumpInfo, sizeof(sessionDumpInfo));
  m_context->Dump(dumpObj);
  m_beforeImages.Dump(dumpObj);
  dumpObj.SetDumpLabel(LABEL_OMS_BOUND_VERSION);
  OmsSession_co12* self = CONST_CAST(OmsSession_co12*, this);
  for (cgg251dclIterator<OmsContext_co12*,OmsSession_co12> iter = self->m_versionsBoundToTrans.begin();
    iter; ++iter)
  {
    struct ListDumpInfo 
    {
      void*            m_thisPtr;
      void*            m_nextPtr;
      void*            m_prevPtr;
      OmsContext_co12* m_context;
    } listDumpInfo;
    
    iter.Dump(listDumpInfo.m_thisPtr, listDumpInfo.m_nextPtr, listDumpInfo.m_prevPtr); 
    listDumpInfo.m_context = *iter();
    dumpObj.Dump(&listDumpInfo, sizeof(listDumpInfo));
  }
}

/*----------------------------------------------------------------------*/

bool OmsSession_co12::GetMethodCallMonitorInfo (OmsMonitorData& info) const
{
  if (m_inMethodCallEpilog)
  {
    m_monitor.GetCallStatistics(info);
    const bool AllocatorOnly = true;
    CHResult_co001 hr = m_lcSink->EvalCallStatistics(!AllocatorOnly, info);
    memcpy(&info.m_methodName[0], &m_monitor_curr->cmiMethodName_gg00[0], sizeof(info.m_methodName));
    return true;
  }
  return false;
}

/*----------------------------------------------------------------------*/

bool OmsSession_co12::HistoryInUse(const OmsObjectId& oid) {
  tgg00_BasisError         LCError;
  bool                     isUsed = true;
  CUnknownContainerId_co12 UnknownFileId;    
  CHResult_co001 hr = m_lcSink->IsObjHistoryUsed(UnknownFileId, *REINTERPRET_CAST(const OmsTypeOid*, &oid), isUsed, LCError);
  if (0 != LCError) {
    this->ThrowDBError(LCError, "OmsSession_co12::HistoryInUse", oid);
  }
  return isUsed;
}

/*----------------------------------------------------------------------*/

void OmsContext_co12::NewConsistentView() {
  tsp00_Int2 DBError;

  tgg91_TransNo consistentView = m_consistentView;
  CHResult_co001 hr = m_currLcSink->NewConsistentView ((unsigned char*) &m_consistentView, &DBError);
  if (0 != DBError) {
    m_consistentView = consistentView;
    m_session->ThrowDBError (DBError, "omsNewConsistentView", OmsObjectId());
    return;
  }
  m_session->ChangedConsistentView();
}

/*----------------------------------------------------------------------*/

// PTS 1115027

void OmsSession_co12::NewConsistentView (OmsIOidReadIterator& pOids, short timeout, OmsIOidAppender& pErrOids) {
  tsp00_Int2        DBError = e_ok;
  int               ix = 0;
  tsp00_Int4        oldTimeout = m_timeout;
  CHResult_co001    hr;
  tgg91_PageRef     pageRef;
  tsp00_Int4        start_time_sec; 
  tsp00_Int4        start_time_msec;
  tsp00_Int4        curr_time_sec; 
  tsp00_Int4        curr_time_msec;
  OmsTypeInt8       startTime;
  OmsTypeInt8       runTime;
  OmsTypeInt8       runTimeLimit = timeout * 1000000;

  if (this->CurrentContext()->IsVersion()) {
    return;
  }
  this->CurrentContext()->RemoveUnlockedObjFromCache();
  
  m_timeout  = timeout; 
  m_lcSink->GetClock(&start_time_sec, &start_time_msec);
  startTime = start_time_sec * 1000000 + start_time_msec;
  
  OmsObjectId* pCurrOid = 0;
  while (bool(pOids))
  {
    pCurrOid = CONST_CAST(OmsObjectId*, &(pOids.omsGet()));
    if ((ix > 0) && (timeout > 0))
    {
      m_lcSink->GetClock(&curr_time_sec, &curr_time_msec);
      runTime = curr_time_sec * 1000000 + curr_time_msec - startTime;
      if (runTime > runTimeLimit) 
      {
        m_timeout = oldTimeout;
        this->MonitorWaitNewConsistentView((tsp00_Int4) (runTime / 1000000));
        this->SetTimeout(oldTimeout);
        this->ThrowDBError(e_lock_collision, "OmsSession_co12::NewConsistentView", *pCurrOid);
      }
      else
      {
        timeout = (short) ((runTimeLimit - runTime) / 1000000);
        if (0 == timeout)
          timeout = 1;
      }
    }
    this->SetTimeout(timeout); 
    CUnknownContainerId_co12 UnknownFileId;    
    pageRef.gg91SetDummyRef();
    hr = m_lcSink->LockObj (
      (unsigned char*) &this->CurrentContext()->m_consistentView,
      (unsigned char*) &UnknownFileId,
      REINTERPRET_CAST(OmsTypeOid*, pCurrOid),
      (unsigned char*) &pageRef,
      &DBError);
    if (e_object_dirty == DBError)
      if (pErrOids.omsAppend(*pCurrOid))
        DBError = 0;
    if (e_ok != DBError)
      break;
    ++pOids;
    ++ix;
  }

  if (e_ok != DBError)
  {
    int jx;
    pOids.omsReset();
    OmsObjectId* pCurrLockOid = 0;
    for (jx = 0; jx < ix; jx++) 
    {
      pCurrLockOid = CONST_CAST(OmsObjectId*, &(pOids.omsGet()));
      /* PTS 1111734 */
      try
      {
        if (!this->CurrentContext()->FindObjInContext (pCurrLockOid))
          this->UnlockObj (*pCurrLockOid);
      }
      catch (DbpError&)
      {
      }
      ++pOids;
    }
  }

  this->SetTimeout(oldTimeout);
  this->MonitorWaitNewConsistentView((tsp00_Int4) (runTime / 1000000)); 
  m_timeout = oldTimeout;
  if (e_ok == DBError) {
    this->CurrentContext()->NewConsistentView();
  }
  else {
    if (e_object_dirty == DBError) {
      this->IncOutOfDate();
      throw OmsOutOfDate(*pCurrOid);
    }
    else {
      this->ThrowDBError(DBError, "OmsSession_co12::NewConsistentView", *pCurrOid);
    }
  }
}

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsSession_co12::NewKeyedObject(const ClassIDRef     guid, 
                                                   const unsigned char* key, 
                                                   OmsSchemaHandle      Schema, 
                                                   OmsContainerNo       ContainerNo) {
  
  const char* msg = "OmsSession_co12::NewKeyedObject";

  m_monitor.IncNewKeyedObject();

  bool        newOid           = true;
  bool        writeBeforeImage = true;
  bool        incNewObjectCnt  = true;
  tsp00_Int2  DBError;
  OmsObjectId errOid;

  HRESULT hr;
  tco13_ClsIdEntry*   pContainerInfo = this->GetClsInfo (guid, Schema, ContainerNo);
  OmsObjectContainer* po;
  if (m_context->IsVersion()) {
    po = this->GetMemoryInVersion(*pContainerInfo);
  } else {
    po = this->GetMemory(*pContainerInfo);
  }
#ifdef LIVECACHE_COMPATAPO30
  const unsigned char* pBinaryKey = key;
#else
  // transform key into binary representation
  unsigned char* pBinaryKey = pContainerInfo->GetKeyPtr(po);
  // introduce pAbstractObject, without this code call will not be virtual on some platforms
  OmsAbstractObject* pAbstractObject = &po->m_pobj;
  pAbstractObject->omsKeyToBinary(key, pBinaryKey);
#endif
  if (m_context->IsVersion()) {
    int                   ObjHistReadCount;
    OmsObjectContainerPtr found;
    found = pContainerInfo->VersionFindKey(pBinaryKey);
    bool keyInCache = (NULL != found);
    if (NULL != found && m_context->IsVersionOid(found->m_oid)) { // PTS 1117571
      if (!found->DeletedFlag()) {
        pContainerInfo->chainFree(*m_context, po);
        throw OmsDuplicateKey (e_duplicate_hash_key, found->m_oid);
      }
    }
    else {
      found = NULL;
      hr = m_lcSink->GetObjWithKey (
        (unsigned char*) &m_context->m_consistentView,
        (unsigned char*) &pContainerInfo->GetFileId(),
        m_context->VersionContext(),
        pContainerInfo->GetKeyLen(), 
        (unsigned char*) pBinaryKey, 
        false, pContainerInfo->GetPersistentSize(), ((unsigned char*) &po->m_pobj) + sizeof(void*),
        REINTERPRET_CAST(OmsTypeOid*, &po->m_oid), (unsigned char*) &po->m_objseq, &ObjHistReadCount, &DBError);
      if FAILED (hr) {
        pContainerInfo->chainFree(*m_context, po);
      }
      if ( DBError != 0 ) {
        if ( e_hash_key_not_found != DBError ) {
          pContainerInfo->chainFree(*m_context, po);
          this->ThrowDBError (DBError, msg);
        }
      }
      else {
        this->IncLogHop(ObjHistReadCount);
        OmsObjectContainerPtr found = m_context->FindObjInContext (&po->m_oid);
        if (found) {
          if (!found->DeletedFlag()) {
            pContainerInfo->chainFree(*m_context, po);
            throw OmsDuplicateKey (e_duplicate_hash_key, found->m_oid);
          }
        }
        else {
            OmsObjectId oid = po->m_oid;
            pContainerInfo->chainFree(*m_context, po);
            throw OmsDuplicateKey (e_duplicate_hash_key, oid);
        }
      }
    }
    if (newOid) {
      po->m_oid = m_context->VersionNewOid();
      po->m_objseq.gg91SetNilRef();

      if (pContainerInfo->UseCachedKeys()) { 
        // Check if object is registerd as cache-miss and if so 
        // remove it from cache miss structure
        if (pContainerInfo->IsCacheMiss(pBinaryKey)) {
          pContainerInfo->DeleteCacheMiss(pBinaryKey, CurrentContext());
        }
      }

      if (keyInCache)
      {
          // delete key in cache, will be reinserted, see below
          pContainerInfo->VersionDelKey(po);
      }
      // Always add entry to cached key structure, because this object
      // is created in a version
      pContainerInfo->VersionAddKey(po); 

      m_context->PutObjectIntoContext (po, pContainerInfo->GetContainerHandle());
      newOid = false;
    }
  }
  if (newOid) {
    HRESULT hr = m_lcSink->NewObj (
      (unsigned char*) &m_context->m_consistentView,
      (unsigned char*) &pContainerInfo->GetFileId(),
      m_context->VersionContext(),
      pContainerInfo->GetKeyDesc().epos(),
      pContainerInfo->GetKeyLen(), 
      (unsigned char*) pBinaryKey,
      REINTERPRET_CAST(OmsTypeOid*, &po->m_oid), 
      (unsigned char*) &po->m_objseq,
      &DBError);
    if FAILED (hr) { 
      pContainerInfo->chainFree(*m_context, po);
      //omsThrowHResult (hr); 
      return NULL; 
    }
    else {
      if ( DBError ) {
        OmsObjectContainerPtr found = NULL;
        if (e_duplicate_hash_key == DBError) {
          // check, if current transcation already deleted object. If true, reuse it
          tsp00_Int2 getDBError;
          int        dummy;
          hr = m_lcSink->GetObjWithKey (
            (unsigned char*) &m_context->m_consistentView,
            (unsigned char*) &pContainerInfo->GetFileId(),
            m_context->VersionContext(),
            pContainerInfo->GetKeyLen(),
            CONST_CAST(unsigned char*, pBinaryKey), 
            false,
            pContainerInfo->GetPersistentSize(), 
            ((unsigned char*) &po->m_pobj) + sizeof(void*),
            REINTERPRET_CAST(OmsTypeOid*, &po->m_oid),
            (unsigned char*) &po->m_objseq,
            &dummy,
            &getDBError);
          if (!FAILED(hr) && (0 == getDBError)) {
            errOid = po->m_oid;
            found  = m_context->FindObjInContext (&po->m_oid);
            if (found) 
            {
              if (found->DeletedFlag()) 
              {
                if (found->IsNewObject())
                {
                  // oid has already been counted as new
                  incNewObjectCnt = false;
                }
                writeBeforeImage = false;
                this->InsertBeforeImage (found);
              }
              else {
                found = NULL;
              }
            }
          }
        }
        pContainerInfo->chainFree(*m_context, po);
        if (found) {
          po = found;
          po->InitState(pContainerInfo);
        }
        else {
          if (e_duplicate_hash_key == DBError)
          {
            throw OmsDuplicateKey (DBError, errOid);
          }
          else {
            this->ThrowDBError (DBError, "omsNewKeyedObject");
          }
          return NULL;
        }
      }
      else {
        if (pContainerInfo->UseCachedKeys()) {
          // Check whether object exists already in the context.
          // This might be possible in the following scenario
          // Tr 1:  Create(a) Commit         Delete(a) Commit
          // Tr 2:                   Read(a)                  Create(a)
          OmsObjectContainerPtr found = pContainerInfo->VersionFindKey(pBinaryKey);
          if (found) {
            //m_context->DeleteObjInContext(found, pContainerInfo);
            pContainerInfo->VersionDelKey(found);
          }

          // Check if object is registerd as cache-miss and if so 
          // remove it from cache miss structure
          if (pContainerInfo->IsCacheMiss(pBinaryKey)) {
            pContainerInfo->DeleteCacheMiss(pBinaryKey, CurrentContext());
          }
          
          // Insert object into cached key structure
          pContainerInfo->VersionAddKey(po);  
        }

        m_context->PutObjectIntoContext (po, pContainerInfo->GetContainerHandle());
      }
    }
  }
  po->MarkLocked();
  po->MarkStored();
  po->MarkAsNew();
  if (writeBeforeImage) {
    this->InsertNewBeforeImage (po, pContainerInfo);
  }
  OMS_DETAIL_TRACE(omsTrNewObj, m_lcSink, "OmsSession_co12::NewKeyedObject : "
    << (incNewObjectCnt ? "to flush: " : "no flush: ")
    << po->m_oid << ", class: " << pContainerInfo->GetClassInfoPtr()->GetClassName());
  if (incNewObjectCnt)
  {
    m_context->IncNewObjectsToFlush();
  }
  return &po->m_pobj;
}

/*----------------------------------------------------------------------*/

OmsAbstractObject* OmsSession_co12::NewObject(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) {
  
  const char* msg = "OmsSession_co12::NewObject";

  this->AssertNotReadOnly(msg);

  m_monitor.IncNewObject();

  tco13_ClsIdEntry* pContainerInfo = this->GetClsInfo (guid, Schema, ContainerNo);
  OmsObjectContainerPtr po;
  if (m_context->IsVersion()) {
    po = this->GetMemoryInVersion(*pContainerInfo);
  } else {
    po  = this->GetMemory(*pContainerInfo);
  }
  if (this->InVersion()) {
    po->m_oid = m_context->VersionNewOid();
    po->m_objseq.gg91SetNilRef();
  }
  else {
    tsp00_Int2 DBError;
    CHResult_co001 hr = m_lcSink->NewObj (
      (unsigned char*) &m_context->m_consistentView,
      (unsigned char*) &pContainerInfo->GetFileId(),
      m_context->VersionContext(),
      0,
      0,
      NULL,
      REINTERPRET_CAST(OmsTypeOid*, &po->m_oid), 
      (unsigned char*) &po->m_objseq,
      &DBError);
    if ( DBError != 0 ) {
      pContainerInfo->chainFree(*this->CurrentContext(), po);
      this->ThrowDBError (DBError, msg);
    }
  }
  m_context->PutObjectIntoContext (po, pContainerInfo->GetContainerHandle());
  po->MarkLocked();
  po->MarkStored();
  po->MarkAsNew();
  this->IncStore(); 
  this->InsertNewBeforeImage (po, pContainerInfo);
  OMS_DETAIL_TRACE(omsTrNewObj, m_lcSink, "OmsSession_co12::NewObject : to flush: "
    << po->m_oid << ", class: " << pContainerInfo->GetClassInfoPtr()->GetClassName());
  m_context->IncNewObjectsToFlush();

  return &po->m_pobj;
}

/*----------------------------------------------------------------------*/

const OmsVarOid& OmsSession_co12::NewVarObject(OmsSchemaHandle Schema, OmsContainerNo ContainerNo) {

  const char* msg = "OmsSession_co12::NewVarObject";


  AssertNotReadOnly(msg);

  m_monitor.IncNewObject();

  tco13_ClsIdEntry* pContainerInfo = this->GetClsInfo (VAR_OBJ_GUID_GCO10, Schema, ContainerNo);
  OmsObjectContainerPtr po  = this->GetMemory(*pContainerInfo);
  if (this->InVersion())
  {
    po->m_oid = m_context->VersionNewOid(); 
    po->m_objseq.gg91SetNilRef();
  }
  else {
    tgg00_BasisError DBError;
    CHResult_co001 hr = m_lcSink->NewObj (
      (unsigned char*) &m_context->m_consistentView,
      (unsigned char*) &pContainerInfo->GetFileId(),
      m_context->VersionContext(),
      0, 0, NULL,
      REINTERPRET_CAST(OmsTypeOid*, &po->m_oid),
      (unsigned char*) &po->m_objseq,
      &DBError);
    if ( DBError != 0 ) {
      pContainerInfo->chainFree(*CurrentContext(), po);
      this->ThrowDBError (DBError, msg);
    }
  }
  m_context->PutObjectIntoContext (po, pContainerInfo->GetContainerHandle());
  po->MarkLocked();
  po->MarkVarObj();
  po->MarkAsNew();
  OmsVarObjInfo_co12* pObjInfo = new(&po->m_pobj) OmsVarObjInfo_co12(po->m_oid, 0, po->m_objseq);
  this->InsertNewBeforeImage (po, pContainerInfo);
  OMS_DETAIL_TRACE(omsTrNewObj, m_lcSink, "OmsSession_co12::NewVarObject : to flush: "
    << po->m_oid << ", class: " << pContainerInfo->GetClassInfoPtr()->GetClassName());
  m_context->IncNewObjectsToFlush();

  return *(REINTERPRET_CAST(OmsVarOid*, CONST_CAST(OmsObjectId*, &po->m_oid)));
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::RegContainer (OmsSchemaHandle Schema,
                             const ClassIDRef   guid,
                             const char*        ClassName, 
                             size_t             PersistentSize,
                             size_t             ObjectSize,
                             const ClassIDPtr   pBaseClass,
                             OmsAbstractObject* Obj, 
                             OmsContainerNo     ContainerNo,
                             size_t             arrayByteSize)
{
  struct VirtualObject {
    void* vtptr;
  };  
  tco13_ClsIdEntry* pContainerInfo; 
  bool              isVarObject;
  bool              isRegistered = false;
  isVarObject = (0 == memcmp (&VAR_OBJ_GUID_GCO10, &guid, sizeof(guid)));
  //try {  // PTS 1121231
    pContainerInfo = this->GetClsInfoForReg(guid, Schema, ContainerNo);
    isRegistered = (pContainerInfo != NULL);
  //}  // PTS 1121231
  //catch (DbpError& e) {
  //  if (e_unknown_guid != e.dbpError()) {
  //    throw e;
  //  }
  //}
  if (!isRegistered) {
    tgg00_ObjColDesc    InvColDesc;
    ObjectSize     += OmsObjectContainer::headerSize();
    PersistentSize -= /* virtual table ptr */ sizeof (void*);
    InvColDesc.colHashCount_gg00 = 0;
    InvColDesc.colInvCount_gg00  = 0;
    InvColDesc.colHashList_gg00  = NULL;
    InvColDesc.colInvList_gg00   = NULL;
    if  (isVarObject) {
      PersistentSize = sizeof(OmsVarObjContainer_co001);
    }
    ContainerInfo_co18* p = co10_RegisterContainer (m_lcSink, guid, ClassName, PersistentSize,
      ObjectSize, pBaseClass, InvColDesc, Schema, ContainerNo, 
      (REINTERPRET_CAST(VirtualObject*, Obj))->vtptr, arrayByteSize); 
    pContainerInfo = new(m_context) tco13_ClsIdEntry (p, m_context);
    this->AddContainerInfo (pContainerInfo);
  }
  if (!isVarObject) {
    pContainerInfo->chainFree(*this->CurrentContext(), Obj);
  }
};

/*----------------------------------------------------------------------*/

void OmsSession_co12::RegVarObjectContainer(OmsSchemaHandle Schema, OmsContainerNo cno)
{
  OmsVarObjInfo_co12 info;
  RegContainer (Schema, VAR_OBJ_GUID_GCO10, "VarObjContainer", sizeof(info), sizeof(info), NULL, &info, cno);  
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::RegisterCallbackInterface(const ClassIDRef guid, OmsCallbackInterface* pInterface) {
  if (NULL != m_callbackInterface) {
    if (m_callbackInterfaceGuid == guid) {
      return;
    }
    ReleaseCallbackInterface();
  }
  m_callbackInterface     = pInterface;
  m_callbackInterfaceGuid = guid;
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::ReleaseVarObject(const OmsVarOid& oid)
{
  OmsObjectContainerPtr found = m_context->FindObjInContext (&oid);
  if (NULL != found) {
    OmsVarObjInfo_co12* pObjInfo = REINTERPRET_CAST (OmsVarObjInfo_co12*, &found->m_pobj);
    pObjInfo->unlock();
  }
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::RemoveFromTransVersion(OmsContext_co12* context) {
  for (cgg251dclIterator<OmsContext_co12*,OmsSession_co12> iter = m_versionsBoundToTrans.begin(); iter; ++iter) {
    if (*iter() == context) {
      m_versionsBoundToTrans.remove(iter);
      return;
    }
  }
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::ReleaseCallbackInterface()
{
  // Set callback-interface to NULL before calling 'omsDestroySelf'. 
  // PTS 1128149
  OmsCallbackInterface *pInterface = m_callbackInterface;
  m_callbackInterface = NULL;
  if (NULL != pInterface) {
    try {
      pInterface->omsDestroySelf();
    } catch (DbpError &e) {
      DbpBase(m_lcSink).dbpOpError("omsDestroySelf failed with error %d", e.dbpError());
      // make sure we don't have partially destroyed class as callback
      // this makes memory leaks possible
    } catch (...) {
      DbpBase(m_lcSink).dbpOpError("omsDestroySelf failed with unknown exception");
      // make sure we don't have partially destroyed class as callback
      // this makes memory leaks possible
    }
    //m_callbackInterface = NULL; PTS 1128149
  }
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::SetCurrentMonitorMethod(tgg00_COMMonitorInfo* p) 
{
  /* PTS 1107731 */
  tsp00_8ByteCounter heapUsed;
  m_lcSink->GetSessionHeapUsage(heapUsed);
  // start of procedure call
  m_heapAtMethodBegin = heapUsed;
  m_monitor_curr      = p;
  m_monitor.Reset();
}
 
/*----------------------------------------------------------------------*/

void OmsSession_co12::SetReadOnly(bool readOnly)
{
  if (readOnly && !m_allowReadOnly)
  {
    this->ThrowDBError (e_missing_privilege, "OmsSession_co12::SetReadOnly");
  }
  m_read_only = readOnly;
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::SetTimeout(short timeout) {
  short       DummyError;
  tsp00_Int4  BufLen = sizeof(timeout);
  this->m_timeout = timeout;
  CHResult_co001 hr = m_lcSink->MultiPurpose (m_set, mm_lock_timeout, &BufLen, 
    (unsigned char*) &timeout, &DummyError);
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::ThrowDBError(tsp00_Int2 e, const char* msg) {
  this->ThrowDBError (e, msg, OmsObjectId());
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::ThrowDBError(tsp00_Int2 e, const char* msg, const OmsObjectId& oid) {
  tsp00_Int4  BufLen = sizeof(e);
  tsp00_Int2  SqlError;
  // transform basis error into sql return code
  m_lcSink->MultiPurpose (m_change, mm_nil, &BufLen, 
    (unsigned char*) &e, &SqlError);
  if (co10_dumpRequested(SqlError))
  {
    this->TriggerDump(SqlError); 
  }
  switch (SqlError) {
  case 400 :
  case 500 : /* PTS 1102829 */
    OMS_TRACE(omsTrError, m_lcSink, "throw LockTimeout");
    IncTimeout();
    throw OmsLockTimeout(oid);
  default :
    OMS_TRACE(omsTrError, m_lcSink, "Error : " << e << "sqlError : " << SqlError << ", " << msg << ", " << oid);
    IncExceptions();
    throw DbpError (DbpError::DB_ERROR, SqlError, msg, oid);
  }
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::ThrowReturnCode (tsp00_Int2 e, const char* msg) {
  OMS_TRACE(omsTrError, m_lcSink, "SqlError : " << e << ", " << msg);
  co10_Throw (DbpError (DbpError::DB_ERROR, e, msg));
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::Trace( const char*fmt, ... ) {
#if defined(WIN32)
  char buffer[256];
#else
  char buffer[1024];
#endif

  va_list args;
  va_start (args, fmt);
#if defined(WIN32)
  _vsnprintf ( buffer, sizeof(buffer), fmt, args );
#else
  vsprintf ( buffer, fmt, args );
  buffer[256] = '\0';
#endif
  m_lcSink->Vtrace ((tsp00_Int4) strlen(buffer), (unsigned char*) &buffer[0]);
  va_end (args);
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::TransEnd() {
  short DBError = 0;
  if (m_defaultContext->ExistsConsistentView()) {
    CHResult_co001 hr = m_lcSink->EndConsistentView(
      (unsigned char*) &m_defaultContext->m_consistentView, &DBError);
  }
  m_currVarObjChunk.Clear();
  this->ClearFreeLists();
  this->ClearTransVersions();
  m_defaultContext->CleanContainerDir();

  m_defaultContext->ResetNewObjectsToFlush();
  if ( 0 != DBError ) { 
    this->ThrowDBError (DBError, "omsTransEnd");
  }
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::TriggerDump(tsp00_Int4 errorNo) 
{
  short DummyError;
  tsp00_Int4  BufLen = sizeof(errorNo);
  CHResult_co001 hr = m_lcSink->MultiPurpose (m_outcopy, mm_nil, &BufLen, 
    (unsigned char*) &errorNo, &DummyError);
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::UnlockObj (const OmsObjectId& oid)
{
  tsp00_Int2               DBError;
  CUnknownContainerId_co12 UnknownFileId; 
  CHResult_co001 hr = m_lcSink->UnlockObj ((unsigned char*) &UnknownFileId,
          REINTERPRET_CAST(OmsTypeOid*, CONST_CAST(OmsObjectId*, &oid)), &DBError);
  if ( DBError != 0 ) { 
    this->ThrowDBError (DBError, "OmsSession_co12::Unlock", oid);
  }
}

/*----------------------------------------------------------------------*/

bool OmsSession_co12::VersionBoundByMe(OmsContext_co12* context) {
  for (cgg251dclIterator<OmsContext_co12*,OmsSession_co12> iter = m_versionsBoundToTrans.begin(); iter; ++iter) {
    if (*iter() == context) {
      return true;
    }
  }
  return false;
}

/*----------------------------------------------------------------------*/

void OmsSession_co12::Wait()
{
  tsp00_Bool ok;
  this->IncCntWaitOmsLockObj();
  tsp00_Int4 waitStart = co10_GetKernelInterface()->Clock();
  m_lcSink->Wait(ok);
  this->IncWaitOmsLockObjSecs(co10_GetKernelInterface()->Clock() - waitStart);
  if (!ok)
  {
    OMS_TRACE(omsTrAlways, m_lcSink, "oms lock request timeout " << this->GetTaskId());
    this->ThrowDBError(e_lock_collision, "OmsSession_co12::Wait", OmsObjectId());
  }
}

/*----------------------------------------------------------------------*/

OmsSession_co12::MethodCallEplilogScope::MethodCallEplilogScope(OmsSession_co12& session)
: m_session(session)
{
  m_session.m_inMethodCallEpilog = true;
}

/*----------------------------------------------------------------------*/

OmsSession_co12::MethodCallEplilogScope::~MethodCallEplilogScope()
{
  m_session.m_inMethodCallEpilog = false;
};
