
/* @lastChanged: "1998-09-22  8:46"

  * @filename:   vco13.cpp
  * @purpose:    "Implementation Runtime Functions of Persistent Object Interface"
  * @release:    7.1.0.0
  * @see:        "http://www.bea.sap-ag.de/doku/rel-7.1/vco13.html"
  *
  * @copyright:  "Copyright by   SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/


/* ================================================================================ */
/* ==========              REPLACEMENTS BY THE PREPROCESSOR               ========= */
/* ================================================================================ */



/* ================================================================================ */
/* ==========                 DECLARATION OF TYPES                        ========= */
/* ================================================================================ */




/* ================================================================================ */
/* ==========                 DECLARATION OF FUNCTIONS                    ========= */
/* ================================================================================ */

#include "geo00.h"
#include "geo573.h"
#include "gco00.h" 
#include "gco10.h"
#include "hsp77.h"
#define __HCO13_IMPL
#include "hco13.h"
#include <new.h>

/* ================================================================================ */
/* ==========                 DEFINITION OF GLOBAL FUNCTIONS              ========= */
/* ================================================================================ */

#if defined( USE_SYSTEM_ALLOC_CO13 ) && defined( WIN32 )
void *operator new(size_t sz) THROW_NEW_GEO573
{
	return omsMalloc(sz);
}

void *operator new[](size_t sz) THROW_NEW_GEO573
{
	return omsMallocArray(sz);
}

void operator delete(void *p) THROW_DELETE_GEO573
{
	omsFree(p);
}

void operator delete[](void *p) THROW_DELETE_GEO573
{
	omsFreeArray(p);
}
#endif


void tco13_GuidEntry::ChainFree (OmsContext_co12&c, OmsAbstractObject* po) {
  if (NULL != po) {
    OmsObjectContainerPtr p = OmsObjectContainer::containerPtr(po);
#ifdef USE_SYSTEM_ALLOC_CO13
    if (p->VarObjFlag()) {
      REINTERPRET_CAST(OmsVarObjInfo_co12*, po)->freeVarObj(&c);
    }
    c.deallocate(p);
#else
    if (p->IsVerNewObject()) {
#ifdef _ASSERT_OMS
      OmsObjectContainerPtr curr = m_freeHeadInVersion->free;
      while (curr) {
        if (curr == p) {
          throw DbpError (-10, "Duplicate in freelist"); 
        }
        curr = curr->m_hashnext;
      }
#endif
      p->m_hashnext    = m_freeHeadInVersion->free;
      m_freeHeadInVersion->free = p;
    } else {
#ifdef _ASSERT_OMS
      OmsObjectContainerPtr curr = m_freeHead->free;
      while (curr) {
        if (curr == p) {
          throw DbpError (-10, "Duplicate in freelist"); 
        }
        curr = curr->m_hashnext;
      }
#endif
      if (p->VarObjFlag()) {
        REINTERPRET_CAST(OmsVarObjInfo_co12*, po)->freeVarObj(&c);
      }
      p->m_hashnext    = m_freeHead->free;
      m_freeHead->free = p;
    }
#endif
  }
}

/*----------------------------------------------------------------------*/

void tco13_GuidEntry::ChainFree (OmsContext_co12&c, OmsObjectContainerPtr& p) {
  if (NULL != p) {
#ifdef USE_SYSTEM_ALLOC_CO13
    if (p->VarObjFlag()) {
      REINTERPRET_CAST(OmsVarObjInfo_co12*, &p->m_pobj)->freeVarObj(&c);
    }
	c.deallocate(p);
	p = NULL;
#else
    if (p->IsVerNewObject()) {
#ifdef _ASSERT_OMS
      OmsObjectContainerPtr curr = m_freeHeadInVersion->free;
      while (curr) {
        if (curr == p) {
          throw DbpError (-10, "Duplicate in freelist"); 
        }
        curr = curr->m_hashnext;
      }
#endif
      p->m_hashnext    = m_freeHeadInVersion->free;
      m_freeHeadInVersion->free = p;
      p = NULL;
    } else {
#ifdef _ASSERT_OMS
      OmsObjectContainerPtr curr = m_freeHead->free;
      while (curr) {
        if (curr == p) {
          throw DbpError (-10, "Duplicate in freelist"); 
        }
        curr = curr->m_hashnext;
      }
#endif
      if (p->VarObjFlag()) {
        REINTERPRET_CAST(OmsVarObjInfo_co12*, &p->m_pobj)->freeVarObj(&c);
      }
      p->m_hashnext    = m_freeHead->free;
      m_freeHead->free = p;
      p = NULL;
    }
#endif
  }
}

/*----------------------------------------------------------------------*/

tco13_ClassHash::tco13_ClassHash() : IGuidHash_co001(), m_context(NULL) {
  for (int ix = 0; ix < GUID_HASH_ENTRIES_CO13; ++ix) {
    m_head[ix] = NULL;
  }
}

/*----------------------------------------------------------------------*/

tco13_ClassHash::~tco13_ClassHash() {
  for (int ix = 0; ix < GUID_HASH_ENTRIES_CO13; ++ix) {
    tco13_GuidEntryPtr pDel;
    tco13_GuidEntryPtr curr = m_head[ix];
    m_head[ix] = NULL;
    while (NULL != curr) {
      pDel = curr;
      curr = curr->m_hashNext;
      pDel->DeleteSelf(m_context);
    }
  }
  new(this) tco13_ClassHash();
}

/*----------------------------------------------------------------------*/

void tco13_ClassHash::Create(OmsContext_co12* context) {
  m_context = context;
}

/*----------------------------------------------------------------------*/

void tco13_ClassHash::Dump(IDump_cco02& dumpObj) const
{
  struct ClassHashDumpInfo 
  {
    tco13_GuidEntry*      m_this;
    tco13_GuidEntry*      m_hashNext;
    ClassInfo_co18*       m_classInfo;
    tco13_FreeListHeader* m_freeHead;
    tco13_FreeListHeader* m_freeHeadInVersion;
    tsp00_Int4            m_slot;
    tsp00_Int4            m_filler;
  } classHashDumpInfo;

  dumpObj.SetDumpLabel(LABEL_OMS_LOCAL_CLASS);
  for (int ix = 0; ix < GUID_HASH_ENTRIES_CO13; ++ix) {
    tco13_GuidEntryPtr curr = m_head[ix];
    while (NULL != curr) {
      classHashDumpInfo.m_this      = curr;
      classHashDumpInfo.m_hashNext  = curr->m_hashNext;
      classHashDumpInfo.m_classInfo = curr->m_classInfo;
      classHashDumpInfo.m_freeHead  = curr->m_freeHead;
      classHashDumpInfo.m_freeHeadInVersion  = curr->m_freeHeadInVersion;
      classHashDumpInfo.m_slot      = ix;
      dumpObj.Dump(&classHashDumpInfo, sizeof(classHashDumpInfo));
      curr = curr->m_hashNext;
    }
  }
}

/*----------------------------------------------------------------------*/


void tco13_ClassHash::HashInsert(ClassInfo_co18* pClassInfo, tco13_FreeListHeader* freeHead,
                                 tco13_FreeListHeader* freeHeadInVersion) {
  if (!HashFind(pClassInfo->GetGuid())) {
    int slot = HashValue(pClassInfo->GetGuid(), GUID_HASH_ENTRIES_CO13);
    m_head[slot] = new(m_context) tco13_GuidEntry(m_head[slot], pClassInfo, freeHead, freeHeadInVersion); 
  }
}

/*----------------------------------------------------------------------*/
 
tco13_GuidEntryPtr tco13_ClassHash::HashFind(const ClassIDRef guid) {
  int slot = HashValue(guid, GUID_HASH_ENTRIES_CO13);
  tco13_GuidEntryPtr p = m_head[slot];
  while (NULL != p) {
    if (p->m_classInfo->GetGuid() == guid) {
      return p;
    }
    p = p->m_hashNext;
  }
  return NULL;
}

/*----------------------------------------------------------------------*/
// Implementation tco13_ClsIdEntry
/*----------------------------------------------------------------------*/

tco13_ClsIdEntry::tco13_ClsIdEntry (ContainerInfo_co18* pContainerInfo,OmsContext_co12* c) 
  : m_clsidHashNext(NULL), 
    m_guidHashNext(NULL), 
    m_containerInfo(pContainerInfo), 
    m_index(), 
    m_freeHead(NULL), 
    m_freeHeadInVersion(NULL), 
    m_useCachedKeys(false),  
    m_cacheMiss(),           
    m_cacheMissCmp(NULL)     
{
  m_index.AdviseCompare(this);
  m_index.AdviseAllocator(c);
  pContainerInfo->IncRefCnt(c->GetCurrentSession()->m_lcSink);
}

/*----------------------------------------------------------------------*/

void tco13_ClsIdEntry::chainFree (OmsContext_co12&c, OmsAbstractObject* po) {
  if (po) {
    OmsObjectContainerPtr p = OmsObjectContainer::containerPtr(po);
#ifdef USE_SYSTEM_ALLOC_CO13
    if (p->VarObjFlag()) {
      REINTERPRET_CAST(OmsVarObjInfo_co12*, po)->freeVarObj(&c);
    }
    c.deallocate(p);
#else
    if (p->IsVerNewObject()) {
#ifdef _ASSERT_OMS
      OmsObjectContainerPtr curr = m_freeHeadInVersion->free;
      while (curr) {
        if (curr == p) {
          throw DbpError (-10, "Duplicate in freelist"); 
        }
        curr = curr->m_hashnext;
      }
#endif
      p->m_hashnext    = m_freeHeadInVersion->free;
      m_freeHeadInVersion->free = p;
    } else {
#ifdef _ASSERT_OMS
      OmsObjectContainerPtr curr = m_freeHead->free;
      while (curr) {
        if (curr == p) {
          throw DbpError (-10, "Duplicate in freelist"); 
        }
        curr = curr->m_hashnext;
      }
#endif
      if (p->VarObjFlag()) {
        REINTERPRET_CAST(OmsVarObjInfo_co12*, po)->freeVarObj(&c);
      }
      p->m_hashnext    = m_freeHead->free;
      m_freeHead->free = p;
    }
#endif
  }
}

/*----------------------------------------------------------------------*/

void tco13_ClsIdEntry::chainFree (OmsContext_co12&c, OmsObjectContainerPtr& p) {
  if (NULL != p) {
#ifdef USE_SYSTEM_ALLOC_CO13
    if (p->VarObjFlag()) {
      REINTERPRET_CAST(OmsVarObjInfo_co12*, &p->m_pobj)->freeVarObj(&c);
    }
    c.deallocate(p);
	p = NULL;
#else
    if (p->IsVerNewObject()) {
#ifdef _ASSERT_OMS
      OmsObjectContainerPtr curr = m_freeHeadInVersion->free;
      while (curr) {
        if (curr == p) {
          throw DbpError (-10, "Duplicate in freelist"); 
        }
        curr = curr->m_hashnext;
      }
#endif
      p->m_hashnext    = m_freeHeadInVersion->free;
      m_freeHeadInVersion->free = p;
      p = NULL;
    } else {
#ifdef _ASSERT_OMS
      OmsObjectContainerPtr curr = m_freeHead->free;
      while (curr) {
        if (curr == p) {
          throw DbpError (-10, "Duplicate in freelist"); 
        }
        curr = curr->m_hashnext;
      }
#endif
      if (p->VarObjFlag()) {
        REINTERPRET_CAST(OmsVarObjInfo_co12*, &p->m_pobj)->freeVarObj(&c);
      }
      p->m_hashnext    = m_freeHead->free;
      m_freeHead->free = p;
      p = NULL;
    }
#endif
  }
}

/*----------------------------------------------------------------------*/
// PTS 1117571
// Delete complete cache miss structure                           
void tco13_ClsIdEntry::DropCacheMisses(OmsContext_co12* pContext) { 
  // Delete dynamically created cache-miss-entries 
  cgg250AvlTree<tco13_CacheMissEntry*,tco13_CacheMissEntry,OmsContext_co12>::Iterator nodeIter;
  tco13_CacheMissEntry*const* pCurrEntry;

  // Delete the cache miss entries
  nodeIter = m_cacheMiss.First();
  while (nodeIter) {
    pCurrEntry = nodeIter();
    (*pCurrEntry)->deleteSelf(pContext); 
      
    ++nodeIter;
  }

  // Delete the tree itself
  m_cacheMiss.DeleteAll();
}

/*----------------------------------------------------------------------*/

void tco13_ClsIdEntry::VersionDelIndex(bool cachedKeysOnly, OmsContext_co12* pContext) {

  const char* msg = "tco13_ClsIdEntry::VersionDelIndex";

  if (!cachedKeysOnly || !pContext->IsVersion()){
    // Delete the complete tree. This is possible, when either
    // - all entries should be deleted and not only those which are added by the
    //   cached key feature, or
    // - when no version is active
    m_index.DeleteAll();
  }
  else {
    // Delete only those entries which were added by the cached key feature
    // As it is not possible to delete entries while looping over an iterator
    // a temporary list is created with those entries which should be remain,
    // and after the whole index is deleted those entries are inserted again.
    cgg250AvlTree<unsigned char*, tco13_ClsIdEntry, OmsContext_co12>::Iterator objectIter;
    cgg251DCList<OmsObjectContainer*, OmsContext_co12> tmpList;    
    OmsObjectContainer* pCurrObject;

    tmpList.advise_allocator(pContext);

    objectIter = m_index.First();
    while (objectIter) {
      pCurrObject = VersionGetInfoFromNode(objectIter());          
      if (!pCurrObject) {
        throw DbpError (e_system_error, msg); 
      }

      if (pContext->IsVersionOid (pCurrObject->m_oid)) {
        tmpList.push_back(pCurrObject);
      }
      ++objectIter;
    }

    m_index.DeleteAll();
    for (cgg251dclIterator<OmsObjectContainer*,OmsContext_co12> listIter 
      = tmpList.begin(); listIter; ++listIter) {
      VersionAddKey(*listIter());
    }
    tmpList.remove_all();
  }
}

/*----------------------------------------------------------------------*/

// PTS 1117571
bool tco13_ClsIdEntry::SetCachedKeys(bool on, OmsContext_co12* pContext) {  
  // Cached Keys are only defined on keyed objects
  if (!IsKeyedObject()){
    return false;
  }
  
  // No change => nothing to do
  if (on == UseCachedKeys()){
    return false;
  }

  if (on) {
    // Cached Keys are switched on

    // Initialization of structures for cache misses
    m_cacheMissCmp = new (GetKeyLen(), pContext) tco13_CacheMissEntry (NULL, GetKeyLen());
    m_cacheMiss.AdviseCompare(m_cacheMissCmp);               
    m_cacheMiss.AdviseAllocator(pContext);  

    m_useCachedKeys = true;
  }
  else {
    // Cached Keys are switched off

    // Delete all entries which are not created in a version
    VersionDelIndex(true, pContext);

    // Reset of structures for cache misses
    DropCacheMisses(pContext);
    if (m_cacheMissCmp != NULL){
      m_cacheMissCmp->deleteSelf(pContext); 
      m_cacheMissCmp = NULL;
    }
    m_cacheMiss.AdviseCompare(NULL);               
    m_cacheMiss.AdviseAllocator(NULL);  
    
    m_useCachedKeys = false;
  }

  return true;
}

/*----------------------------------------------------------------------*/
// Implementation cco13_OidHash
/*----------------------------------------------------------------------*/

cco13_OidHash::cco13_OidHash () 
  : m_headentries(0), 
    //m_resize(0), // PTS 1118855
    m_mask(0),     // PTS 1118855 
    m_count(0),    // PTS 1118855
    m_maxCount(0), // PTS 1118855
    m_head(NULL),	
    m_headcurr(NULL), 
    m_context(NULL)
{
};

// end constructor

/*----------------------------------------------------------------------*/

cco13_OidHash::~cco13_OidHash () 
{
  HashFree();	
  new(this) cco13_OidHash();
};

// end destructor

/*----------------------------------------------------------------------*/

void cco13_OidHash::Clean() {
  // removes all objects of dropped containers from OID Hash
  OmsObjectContainerPtr* prev;
  OmsObjectContainerPtr  curr;
  OmsObjectContainerPtr  toFree;
  tsp00_Int4             ix;
  for (ix = 0; ix < m_headentries; ix++) {
    prev = &m_head[ix];
    curr = *prev;
    while (NULL != curr) {
      tco13_ClsIdEntry* containerInfo = curr->GetContainerInfoNoCheck(m_context);
      if (containerInfo->GetContainerInfoPtr()->IsDropped()) {
        DecCount();         // PTS 1119061
        toFree = curr;
        *prev = curr->m_hashnext;
        curr  = curr->m_hashnext;

        // PTS 1117571
	      if (containerInfo->UseCachedKeys() && containerInfo->IsKeyedObject()) {
      	  // Delete object out of cached key structure  
			    containerInfo->VersionDelKey(toFree);
        }

        if (toFree->IsNewObject())
        {
          m_context->DecNewObjectsToFlush();
        }
        containerInfo->chainFree(*m_context, toFree);
      }
      else {
        prev = &curr->m_hashnext;
        curr  = curr->m_hashnext;
      }
    }
  }
}

/*----------------------------------------------------------------------*/

void cco13_OidHash::Clear() { 
  tsp00_Int4  ix;
  tsp00_Int4 listCount  = 0;
  tsp00_Int4 entryCount = 0;
  OmsObjectContainerPtr curr;

  for (ix = 0; ix < m_headentries; ++ix) {
    curr = m_head[ix];
    if (NULL != curr) {
      ++listCount;
      do {
        ++entryCount;
        if (curr->VarObjFlag()) {
          REINTERPRET_CAST(OmsVarObjInfo_co12*, &curr->m_pobj)->freeVarObj(m_context);
        }
#ifdef USE_SYSTEM_ALLOC_CO13
        OmsObjectContainerPtr toFree = curr;
        tco13_ClsIdEntry* containerInfo = curr->GetContainerInfoNoCheck(m_context);
        curr  = curr->m_hashnext;
        containerInfo->chainFree(*m_context, toFree);
#else
        curr  = curr->m_hashnext;
#endif
      }
      while (NULL != curr);
    }
    m_head[ix] = NULL;
  }

  // PTS 1117571
  // reset search structure for cached keys
  m_context->m_containerDir.ClearCachedKeys(m_context);

  ResetCount(); // PTS 1119061
}

/*----------------------------------------------------------------------*/

void cco13_OidHash::Create (OmsContext_co12* c, const tsp00_Int4 sz) {
  m_context = c;
  HashInit(sz);
}

/*----------------------------------------------------------------------*/

void cco13_OidHash::CheckChain(OmsObjectContainerPtr curr) {
  OmsObjectContainerPtr p = curr->m_hashnext;
  while (p != NULL) {
    if (p == curr) {
      throw DbpError (-9, "Loop in Oms Cache"); 
    }
    p = p->m_hashnext;
  }
}

/*----------------------------------------------------------------------*/

void cco13_OidHash::Dump(IDump_cco02& dumpObj) const
{
  struct OidHashDumpInfo 
  {
    void*                 m_this;
    OmsObjectContainerPtr m_hashnext;
    OmsObjectId           m_oid;
    tgg91_PageRef         m_objseq;
    unsigned char         m_state;
    unsigned char         m_filler;
    tsp00_Uint4           m_beforeImages;
    tsp00_Uint4           m_containerHandle;
    tsp00_Int4            m_slot;
  } oidHashDumpInfo;

  dumpObj.SetDumpLabel(LABEL_OMS_OID_CACHE);
  for (tsp00_Int4 ix = 0; ix < m_headentries; ix++) {
    oidHashDumpInfo.m_slot = ix;
    OmsObjectContainerPtr curr = m_head[ix];
    while (NULL != curr)
    {
      memcpy(&oidHashDumpInfo.m_hashnext, curr,
        sizeof(oidHashDumpInfo) -
        sizeof(oidHashDumpInfo.m_this) -
        sizeof(oidHashDumpInfo.m_slot));
      dumpObj.Dump(&oidHashDumpInfo, sizeof(oidHashDumpInfo));
      curr = curr->m_hashnext;
    }
  }
}

/*----------------------------------------------------------------------*/

void cco13_OidHash::HashCheck() {
  
  for (tsp00_Int4 ix = 0; ix < m_headentries; ix++) {
    if (m_head[ix] != NULL) {
      CheckChain (m_head[ix]);
    }
  }
}

/*----------------------------------------------------------------------*/

inline void cco13_OidHash::HashFree () 
{
  if (NULL != m_context)
  {
    m_context->deallocate(m_head);
  }
}

/*----------------------------------------------------------------------*/

inline void cco13_OidHash::HashInit (tsp00_Int4 sz)
{
  // Ensure that sz is a power of 2 and is between MINIMAL_HASH_SIZE and the 
  // maximal possible size              // PTS 1119061
  if (sz <= 0){
    sz = DEFAULT_HASH_SIZE;
  }
  else if (sz <= MINIMAL_HASH_SIZE){
    sz = MINIMAL_HASH_SIZE;
  }
  else {
    tsp00_Int4 size = (MINIMAL_HASH_SIZE << 1);
    while (sz > size && size < MAXIMAL_HASH_SIZE){
      size <<= 1;
    }
    sz = size;
  }

  m_head = REINTERPRET_CAST(OmsObjectContainerPtr*, m_context->allocate(sizeof(OmsObjectContainerPtr) * sz));
  m_headentries = sz;     // PTS 1110780
  m_mask        = sz - 1; // PTS 1119061
  
  for (tsp00_Int4 ix = 0; ix < sz; ix++)
  {
		m_head[ix] = NULL;
	};
	m_headcurr = NULL;
  m_count    = 0;     // PTS 1119061
  m_maxCount = 0;     // PTS 1119061


  // PTS 1117571
  // reset search structure for cached keys 
  m_context->m_containerDir.ClearCachedKeys(m_context);
}

/*----------------------------------------------------------------------*/

void cco13_OidHash::HashDelete (const OmsObjectId& key, bool updateKeyCache)
{
    
    OmsObjectContainerPtr* prev = HeadPtr(key);
    OmsObjectContainerPtr  curr = *prev;
    while (curr) {
        if (curr->m_oid == key) {
            if (curr->VarObjFlag()) {
                OmsVarObjInfo_co12* objInfo;
                objInfo = REINTERPRET_CAST(OmsVarObjInfo_co12*, (&curr->m_pobj));
                objInfo->freeVarObj(m_context);
            }
            
            if (updateKeyCache)
            {
                // PTS 1117571
                tco13_ClsIdEntry* containerInfo = curr->GetContainerInfoNoCheck(m_context);
                if (containerInfo->UseCachedKeys() && containerInfo->IsKeyedObject()) {
                    // Delete object out of cached key structure  
                    containerInfo->VersionDelKey(curr);
                }
            }
            
            *prev = curr->m_hashnext;
            DecCount();   // PTS 1119061
            return;
        }
        else {
            prev = &curr->m_hashnext;
            curr = curr->m_hashnext;
        }
    }
#ifdef _ASSERT_OMS
    HashCheck();
#endif
    return;
};

/*----------------------------------------------------------------------*/

void cco13_OidHash::HashResize(int newHeadEntries, bool rehash)
{
    OmsObjectContainerPtr* oldHead        = m_head;
    tsp00_Int4             oldCount       = m_count;
    tsp00_Int4             oldHeadEntries = m_headentries;

    float avgChainLen = 0;  // Only for analysing purpose

    bool error = false;
    try
    {
      // Allocate new hash array
      HashInit(newHeadEntries);
    }
    catch (DbpError&){
      error = true;
    }
    catch (BAD_ALLOC_GEO573&){
      error = true;
    }

    if (error){
      // If allocation of new hash array fails, then use old array again
      m_head        = oldHead;
      m_headentries = oldHeadEntries;
      m_mask        = oldHeadEntries - 1;
      m_count       = oldCount;
      return;
    }


    // Rehash if rehashing is requested and there are entries in the old hash 
    if (rehash && oldCount > 0){
      int nonEmptySlots   = 0;
      m_context->m_session->IncRehash();
      for (tsp00_Int4 slot = 0; slot < oldHeadEntries; ++slot) {
        OmsObjectContainerPtr curr = oldHead[slot];
        if (curr != NULL){
          ++nonEmptySlots;
          while (NULL != curr){
            OmsObjectContainerPtr pInsert = curr;
            curr = curr->m_hashnext;
            HashInsert(pInsert);
          }
        }
      }
      avgChainLen = (float) oldCount / (float) (nonEmptySlots + 1);
    }

    // Free memory of old hash array
    m_context->deallocate (oldHead);
}
// end HashDelete

/*----------------------------------------------------------------------*/

void cco13_OidHash::RemoveUnlockedObjFromCache() {
  tsp00_Int4             ix;
  OmsObjectContainerPtr* prev;
  OmsObjectContainerPtr  curr;
  OmsObjectContainerPtr  toFree;
  tco13_ClsIdEntry*      pContainerInfo;
 
  for (ix = 0; ix < m_headentries; ix++) {
    prev       = &m_head[ix];
    curr       = m_head[ix];
    m_head[ix] = NULL;
    while (NULL != curr) {
      toFree = curr;
      curr   = curr->m_hashnext;
      if ((toFree->LockedFlag()) || (toFree->m_beforeImages != 0)) 
      {
        *prev              = toFree;
        toFree->m_hashnext = NULL;
        prev               = &toFree->m_hashnext;
      }
      else {
        pContainerInfo = toFree->GetContainerInfo(m_context);

        // PTS 1117571
        if (pContainerInfo->UseCachedKeys() && pContainerInfo->IsKeyedObject()) {
          // Delete object out of cached key structure
          pContainerInfo->VersionDelKey(toFree);
        }

        DecCount();  // PTS 1119061
        pContainerInfo->chainFree(*m_context, toFree);
      }
    }
  }
}

/*----------------------------------------------------------------------*/

void cco13_OidHash::SetEmpty(bool adaptOidHash) {
  tsp00_Int4 ix;
  for (ix = 0; ix < m_headentries; ++ix) {
    m_head[ix] = NULL;
  }
  ResetCount(adaptOidHash);  // PTS 1119061

  // PTS 1117571
  // reset search structure for cached keys 
  m_context->m_containerDir.ClearCachedKeys(m_context);
}


/*----------------------------------------------------------------------*/
/* Implementation of ClsId HashList                                     */
/*----------------------------------------------------------------------*/

cco13_ClsIdHash::cco13_ClsIdHash () : headentries(0), m_count(0), m_clsidHead(NULL), m_guidHead(NULL), freeListHeader(NULL), m_context(NULL) 
{
};

/*----------------------------------------------------------------------*/

tco13_ClsIdEntry* cco13_ClsIdHash::AutoRegister (const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) {
  tco13_ClsIdEntry*   curr           = NULL;
  ContainerInfo_co18* pContainerInfo = NULL;            // PTS 1121231
  try {                                                 // PTS 1121231
    pContainerInfo = co10_FindContainerInfo(m_context->GetCurrentSession()->m_lcSink, guid, Schema, ContainerNo);
  } catch (DbpError &e){                                // PTS 1121231
    if (e_unknown_guid != e.dbpError())                 // PTS 1121231
      throw e;                                          // PTS 1121231
    else                                                // PTS 1121231
      ThrowUnknownContainer(guid, Schema, ContainerNo); // PTS 1121231
  }

  if (NULL != pContainerInfo) {
    curr = new(m_context) tco13_ClsIdEntry (pContainerInfo, m_context);
    if (NULL != curr) {
      HashInsert (curr);
    }
  }
  else {
    this->ThrowUnknownContainer(guid, Schema, ContainerNo);
  }
  return curr;
}

/*----------------------------------------------------------------------*/

tco13_ClsIdEntry* cco13_ClsIdHash::AutoRegisterForReg (const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) {
  tco13_ClsIdEntry*   curr           = NULL;
  ContainerInfo_co18* pContainerInfo = NULL;            // PTS 1121231
  try {                                                 // PTS 1121231
    pContainerInfo = co10_FindContainerInfo(m_context->GetCurrentSession()->m_lcSink, guid, Schema, ContainerNo);
  } catch (DbpError &e){                                // PTS 1121231
    if (e_unknown_guid != e.dbpError())                 // PTS 1121231
      throw e;                                          // PTS 1121231
    else                                                // PTS 1121231
      ThrowUnknownContainer(guid, Schema, ContainerNo); // PTS 1121231
  }

  if (NULL != pContainerInfo) {
    curr = new(m_context) tco13_ClsIdEntry (pContainerInfo, m_context);
    if (NULL != curr) {
      HashInsert (curr);
    }
  }
  else {
    curr = NULL;
  }
  return curr;
}

/*----------------------------------------------------------------------*/

void cco13_ClsIdHash::Clean() {
  // remove all infos of dropped containers
  cco13_ClsIdHash::Iter iter = this->First();
  while (iter) {
    tco13_ClsIdEntry* curr = iter();
    ++iter;
    if (curr->m_containerInfo->IsDropped()) {
      this->HashDelete(curr);
    }
  }
}

/*----------------------------------------------------------------------*/

// PTS 1117571
// Clear structures which are used for cached keys and cache misses.
// As the structure for the cached keys is also used for those objects
// which are newly created in a version, delete only those objects, which
// do not belong to a version. 
void cco13_ClsIdHash::ClearCachedKeys(OmsContext_co12* pContext) {

  tco13_ClsIdEntry*     pCurrContainer; 
  cco13_ClsIdHash::Iter containerIter  = First();
  while (containerIter) {
    pCurrContainer = containerIter();
    if (pCurrContainer->UseCachedKeys() && pCurrContainer->IsKeyedObject()) {
      // Reset structure for cache hits 
      if (!m_context->IsVersion()) {
        // As we are not in a version, objects which are created in a version
        // cannot exist. Therefore the complete structure can be reset
        pCurrContainer->VersionDelIndex(false, pContext);
      }
      else {
        // Delete only those entries which are not created in a version
        pCurrContainer->VersionDelIndex(true, pContext);
      }

      // Reset structure for cache misses if requested
      pCurrContainer->DropCacheMisses(pContext);
    }
    ++containerIter;
  }
}

/*----------------------------------------------------------------------*/

void cco13_ClsIdHash::Create (OmsContext_co12* context) {
#define HEAD_ENTRIES 257 
  m_context   = context;
  m_clsidHead = REINTERPRET_CAST(tco13_ClsIdEntryPtr*, 
    m_context->allocate(sizeof(tco13_ClsIdEntryPtr) * HEAD_ENTRIES));
  m_guidHead = REINTERPRET_CAST(tco13_ClsIdEntryPtr*, 
    m_context->allocate(sizeof(tco13_ClsIdEntryPtr) * HEAD_ENTRIES));
  headentries = HEAD_ENTRIES;
  for (tsp00_Int4 ix = 0; ix < headentries; ++ix) {
    m_clsidHead[ix]= NULL;
    m_guidHead[ix] = NULL;
  };
  m_classDir.Create(context);
  freeListHeader = new(m_context) tco13_FreeListHeader(0 /* PTS 1109561 */);
};

// end constructor

/*----------------------------------------------------------------------*/

void cco13_ClsIdHash::Dump(IDump_cco02& dumpObj) const
{
  struct ClsIdHashDumpInfo 
  {
    cco13_ClsIdHash*      m_this;
    tco13_ClsIdEntryPtr*  m_clsidHead;
    tco13_ClsIdEntryPtr*  m_guidHead;
    tco13_FreeListHeader* m_freeListHeader;
    OmsContext_co12*      m_context;
    tsp00_Int4            m_headentries;
  } clsIdHashDumpInfo;
  
  dumpObj.SetDumpLabel(LABEL_OMS_LOCAL_CLS_DIR);
  clsIdHashDumpInfo.m_this           = CONST_CAST(cco13_ClsIdHash*, this);
  clsIdHashDumpInfo.m_clsidHead      = m_clsidHead;
  clsIdHashDumpInfo.m_guidHead       = m_guidHead;
  clsIdHashDumpInfo.m_freeListHeader = freeListHeader;
  clsIdHashDumpInfo.m_context        = m_context;
  clsIdHashDumpInfo.m_headentries    = headentries;
  dumpObj.Dump(&clsIdHashDumpInfo, sizeof(clsIdHashDumpInfo));

  struct ClsIdEntryDumpInfo 
  {
    tco13_ClsIdEntryPtr       m_this;
    tco13_ClsIdEntry*         m_clsidHashNext;
    tco13_ClsIdEntry*         m_guidHashNext;
    ContainerInfo_co18*       m_containerInfo;
    tco13_FreeListHeader*     m_freeHead;
    tco13_FreeListHeader*     m_freeHeadInVersion;
  } clsIdEntryDumpInfo;
  
  dumpObj.SetDumpLabel(LABEL_OMS_LOCAL_CLS_HASH);
  int ix;
  for (ix = 0; ix < headentries; ++ix) 
  {
    tco13_ClsIdEntryPtr curr = m_clsidHead[ix];
    while (NULL != curr)
    {
      clsIdEntryDumpInfo.m_this          = curr;
      clsIdEntryDumpInfo.m_clsidHashNext = curr->m_clsidHashNext;
      clsIdEntryDumpInfo.m_guidHashNext  = curr->m_guidHashNext;
      clsIdEntryDumpInfo.m_containerInfo = curr->m_containerInfo;
      clsIdEntryDumpInfo.m_freeHead      = curr->m_freeHead;
      clsIdEntryDumpInfo.m_freeHeadInVersion = curr->m_freeHeadInVersion;
      dumpObj.Dump(&clsIdEntryDumpInfo, sizeof(clsIdEntryDumpInfo));
      curr = curr->m_clsidHashNext;
    }
  }
  dumpObj.SetDumpLabel(LABEL_OMS_LOCAL_GUID_HASH);
  for (ix = 0; ix < headentries; ++ix) 
  {
    tco13_ClsIdEntryPtr curr = m_guidHead[ix];
    while (NULL != curr)
    {
      clsIdEntryDumpInfo.m_this          = curr;
      clsIdEntryDumpInfo.m_clsidHashNext = curr->m_clsidHashNext;
      clsIdEntryDumpInfo.m_guidHashNext  = curr->m_guidHashNext;
      clsIdEntryDumpInfo.m_containerInfo = curr->m_containerInfo;
      clsIdEntryDumpInfo.m_freeHead      = curr->m_freeHead;
      clsIdEntryDumpInfo.m_freeHeadInVersion = curr->m_freeHeadInVersion;
      dumpObj.Dump(&clsIdEntryDumpInfo, sizeof(clsIdEntryDumpInfo));
      curr = curr->m_guidHashNext;
    }
  }
  m_classDir.Dump(dumpObj);
}

/*----------------------------------------------------------------------*/

tco13_FreeListHeader* cco13_ClsIdHash::GetFreeListHeader(size_t size) 
{
  tco13_FreeListHeader* p = freeListHeader;
  while (p != NULL) {
    if (p->size == size) {
      return p;
    }
    else {
      p = p->next;
    }
  }
  p       = new(m_context) tco13_FreeListHeader(size);
  p->next = freeListHeader->next;
  freeListHeader->next = p;
  return p;
}

/*----------------------------------------------------------------------*/

void cco13_ClsIdHash::HashDelete (tco13_ClsIdEntry* containerInfo) {
  // remove from ContainerHandle Hash 
  int hashSlot              = containerInfo->GetContainerHandle() % headentries;
  tco13_ClsIdEntryPtr* prev = &m_clsidHead[hashSlot];
  tco13_ClsIdEntryPtr  curr = *prev;
  while (curr != NULL) {
    if (curr == containerInfo) {
      *prev = curr->m_clsidHashNext;
      --m_count;
      break;
    }
    else {
      prev = &curr->m_clsidHashNext;
      curr = curr->m_clsidHashNext;
    }
  }
  // remove from Guid,Schema,ContainerNo Hash
  hashSlot = HashValue(containerInfo->GetGuid(), containerInfo->GetSchema(), 
    containerInfo->GetContainerNo(), headentries);
  prev     = &m_guidHead[hashSlot];
  curr     = *prev;
  while (curr != NULL) {
    if (curr == containerInfo) {
      *prev = curr->m_guidHashNext;
      break;
    }
    else {
      prev = &curr->m_guidHashNext;
      curr = curr->m_guidHashNext;
    }
  }
  // delete local ContainerInfo
  containerInfo->DeleteSelf(m_context);
}

/*----------------------------------------------------------------------*/

bool cco13_ClsIdHash::IsBaseClassOf(const ClassIDRef baseGuid, const ClassIDRef derivedGuid) {
  return  IsDerivedClassOf(baseGuid, derivedGuid);
}

/*----------------------------------------------------------------------*/

bool cco13_ClsIdHash::IsDerivedClassOf(const ClassIDRef baseGuid, const ClassIDRef derivedGuid) {
   tco13_GuidEntry* pDerivedClassInfo = GetClassInfo(derivedGuid);
   do {
     if (pDerivedClassInfo->IsDerivedClassOf(baseGuid)) {
       return true;
     }
     if (pDerivedClassInfo->IsDerivedClass()) {
       pDerivedClassInfo = GetClassInfo(pDerivedClassInfo->GetBaseGuid());
     }
     else {
       return false;
     }
   }
   while(1);
}

/*----------------------------------------------------------------------*/

void cco13_ClsIdHash::Resize()
{
    tco13_ClsIdEntryPtr*  oldClsIdHead = m_clsidHead;
    tco13_ClsIdEntryPtr*  oldGuidHead  = m_guidHead;
    try
    {
        int ix;
        // calculate new header size
        int newHeadEntries = m_count - 1;
        bool     isPrime   = false;
        while (!isPrime) {
            newHeadEntries++;
            isPrime = (1 == (newHeadEntries % 2)) ? true : false; 
            ix = 3;
            while (((ix * ix) <= newHeadEntries) && (isPrime)) {
                isPrime = (0 == (newHeadEntries % ix)) ? false : true;
                ix += 2;
            }
        }
        // allocate and init new headers
        m_clsidHead = NULL;
        m_clsidHead = REINTERPRET_CAST(tco13_ClsIdEntryPtr*, 
            m_context->allocate(sizeof(tco13_ClsIdEntryPtr) * newHeadEntries));
        m_guidHead = REINTERPRET_CAST(tco13_ClsIdEntryPtr*, 
            m_context->allocate(sizeof(tco13_ClsIdEntryPtr) * newHeadEntries));
        int oldHeadEntries = headentries;
        headentries        = newHeadEntries;
        for (ix = 0; ix < headentries; ++ix) {
            m_clsidHead[ix]= NULL;
            m_guidHead[ix] = NULL;
        };
        // rehash 
        m_count = 0;
        for (ix = 0; ix < oldHeadEntries; ++ix)
        {
            tco13_ClsIdEntry* next = oldClsIdHead[ix];
            while (next)
            {
                tco13_ClsIdEntry* curr = next; 
                next = next->m_clsidHashNext;
                int hashSlot = (curr->GetContainerHandle()) % headentries;
                curr->m_clsidHashNext  = m_clsidHead[hashSlot];
                m_clsidHead[hashSlot]  = curr;
                hashSlot = HashValue(curr->GetGuid(), curr->GetSchema(), curr->GetContainerNo(), headentries);
                curr->m_guidHashNext = m_guidHead[hashSlot];
                m_guidHead[hashSlot] = curr;
            }
        }
        // free old headers
        m_context->deallocate(oldClsIdHead);
        m_context->deallocate(oldGuidHead);
    }
    catch (BAD_ALLOC_GEO573&)
    {
        // reset old state in case of missing memory
        m_context->deallocate(m_clsidHead);
        m_clsidHead = oldClsIdHead;
        m_guidHead  = oldGuidHead;
    }
}

/*----------------------------------------------------------------------*/

void cco13_ClsIdHash::ThrowUnknownContainer(tco13_ClsIdEntryPtr containerInfo) {
  this->ThrowUnknownContainer(containerInfo->GetGuid(), containerInfo->GetSchema(), containerInfo->GetContainerNo());
}

/*----------------------------------------------------------------------*/


void cco13_ClsIdHash::ThrowUnknownContainer(const ClassIDRef guid, OmsSchemaHandle Schema, OmsContainerNo ContainerNo) {
  char buf[256];
  OmsTypeWyde         Identifier[MAX_SCHEMA_NAME_LENGTH_CO10];
  tgg00_BasisError    DBError;
  m_context->LcSink()->GetSchemaName(Schema, &Identifier[0], &DBError);
  if (0 != DBError)
  {
#ifdef LIVECACHE_INTGUIDS
    //sprintf (buf, "guid : %8.8X, Schema(%8X) CNo(%8X)", guid, Schema, ContainerNo);                  // PTS 1125307
    sp77sprintf (buf, sizeof(buf), "guid : %#8.8X, Schema(%d) CNo(%d)", guid, Schema, ContainerNo);   // PTS 1125307
#else
    //sprintf (buf, "guid : %8.8X-%4.4X-%4.4X-%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X, Schema(%8X) CNo(%8X)", guid.Data1, guid.Data2, guid.Data3,
    //  guid.Data4[0], guid.Data4[1], guid.Data4[2], guid.Data4[3], guid.Data4[4], guid.Data4[5],
    //  guid.Data4[6], guid.Data4[7], Schema, ContainerNo);                                            // PTS 1125307
    sp77sprintf (buf, sizeof(buf),"guid : %8.8X-%4.4X-%4.4X-%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X, Schema(%d) CNo(%d)", guid.Data1, guid.Data2, guid.Data3,
      guid.Data4[0], guid.Data4[1], guid.Data4[2], guid.Data4[3], guid.Data4[4], guid.Data4[5],
      guid.Data4[6], guid.Data4[7], Schema, ContainerNo);                                              // PTS 1125307
#endif
  }
  else
  { 
#ifdef LIVECACHE_INTGUIDS
      sp77sprintfUnicode(sp77encodingUTF8, buf, sizeof(buf), 
          "guid : %#8.8X, Schema(%d:'%S') CNo(%d)", guid, Schema, &Identifier[0], ContainerNo);
#else
      sp77sprintfUnicode(sp77encodingUTF8, buf, sizeof(buf), 
          "guid : %8.8X-%4.4X-%4.4X-%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X%2.2X, Schema(%d:'%S') CNo(%d)", guid.Data1, guid.Data2, guid.Data3,
          guid.Data4[0], guid.Data4[1], guid.Data4[2], guid.Data4[3], guid.Data4[4], guid.Data4[5],
          guid.Data4[6], guid.Data4[7], Schema, &Identifier[0], ContainerNo);
#endif
  }
  co10_Throw(DbpError (DbpError::DB_ERROR, e_unknown_guid, buf)); 
}

/*----------------------------------------------------------------------*/

void cco13_ClsIdHash::UnregisterAll() /* PTS 1110149 */
{
  /* delete hash */
  tco13_ClsIdEntry* curr;
  tco13_ClsIdEntry* delClsInfo;
  for (tsp00_Int4 ix = 0; ix < headentries; ix++) {
    curr = m_clsidHead[ix];
    m_clsidHead[ix] = NULL;
    m_guidHead[ix] = NULL;
    while (curr != NULL) {
      delClsInfo = curr;
      curr       = curr->m_clsidHashNext;
      delClsInfo->DeleteSelf(m_context);
    }
  }
  /* delete free list headers except sentinel */
  if (0 != freeListHeader)
  {
    tco13_FreeListHeader* toDel;
    tco13_FreeListHeader* p = freeListHeader->next;
    freeListHeader->next = NULL;
    while (p != NULL) {
      toDel = p;
      p     = p->next;
      toDel->DeleteSelf(m_context);
    }
  }
  m_classDir.~tco13_ClassHash();
  m_classDir.Create(m_context);
}

/*----------------------------------------------------------------------*/

cco13_ClsIdHash::~cco13_ClsIdHash () 
{
  if (NULL != m_context)
  {
    this->UnregisterAll();
    if (0 != freeListHeader)
    {
      freeListHeader->DeleteSelf(m_context);
    }
    m_context->deallocate(m_clsidHead);
    m_context->deallocate(m_guidHead);
  }
  new(this) cco13_ClsIdHash();
};

// end destructor

// end Guid HashFind

/*----------------------------------------------------------------------*/

cco13_BeforeImages::cco13_BeforeImages () : m_session(NULL) {
  int ix;
  for (ix = 0; ix < MAX_SUBTRANS_LEVEL; ix++) {
    m_beforeImages[ix] = NULL;
  }
}

/*----------------------------------------------------------------------*/

cco13_BeforeImages::~cco13_BeforeImages () {
  
}

/*----------------------------------------------------------------------*/

void cco13_BeforeImages::Dump(IDump_cco02& dumpObj) const
{
  struct BeforeImageDumpInfo 
  {
    OmsObjectContainerPtr m_this;
    OmsObjectContainerPtr m_hashNext;
    OmsContext_co12*      m_context;
    tco13_ClsIdEntry*     m_containerInfo;
    OmsObjectId           m_oid;
    tsp00_Int2            m_subtransLevel;
    unsigned char         m_state;
    bool                  m_filler1;
    tsp00_Int4            m_filler2;
  } beforeImageDumpInfo;
  OmsObjectContainerPtr pBeforeImage;
  bool isFirst = true;
  int  subtransLvl = m_session->CurrentSubtransLevel();
  while (subtransLvl > 0) {
    beforeImageDumpInfo.m_subtransLevel = subtransLvl;
    pBeforeImage = m_beforeImages[subtransLvl-1];
    while (pBeforeImage) {
      if (isFirst)
      {
        isFirst = false;
        dumpObj.SetDumpLabel(LABEL_OMS_BEFORE_IMAGE);
      }
      beforeImageDumpInfo.m_this     = pBeforeImage;
      beforeImageDumpInfo.m_hashNext = pBeforeImage->m_hashnext;
      beforeImageDumpInfo.m_context = *REINTERPRET_CAST(OmsContext_co12**, &pBeforeImage->m_pobj);
      beforeImageDumpInfo.m_containerInfo = 
        pBeforeImage->GetContainerInfo(beforeImageDumpInfo.m_context);
      beforeImageDumpInfo.m_state = pBeforeImage->GetState();
      beforeImageDumpInfo.m_oid   = pBeforeImage->m_oid;
      dumpObj.Dump(&beforeImageDumpInfo, sizeof(beforeImageDumpInfo));
      pBeforeImage   = pBeforeImage->m_hashnext;
    }
    subtransLvl--; 
  }
}

/*----------------------------------------------------------------------*/

void cco13_BeforeImages::init (OmsSession_co12* s) {
   m_session = s;
}

/*----------------------------------------------------------------------*/

void cco13_BeforeImages::freeBeforeImages () {

  OmsObjectContainerPtr pBeforeImage;
  OmsObjectContainerPtr pToFree;
  tco13_ClsIdEntry*     pContainerInfo;
  OmsContext_co12*      context;
  int                   subtransLvl = m_session->CurrentSubtransLevel();
  while (subtransLvl > 0) {
    pBeforeImage                  = m_beforeImages[subtransLvl-1];
    m_beforeImages[subtransLvl-1] = NULL;
    while (pBeforeImage) {
      context        = *REINTERPRET_CAST(OmsContext_co12**, &pBeforeImage->m_pobj);
      pContainerInfo = pBeforeImage->GetContainerInfo(context);
      pToFree        = pBeforeImage;
      pBeforeImage   = pBeforeImage->m_hashnext;
      pContainerInfo->chainFree(*context, pToFree);
    }
    subtransLvl--; 
  }
}

/*----------------------------------------------------------------------*/

void cco13_BeforeImages::removeContext (OmsContext_co12* pContext) {
  OmsObjectContainerPtr  pBeforeImage;
  OmsObjectContainerPtr* pPrev;
  OmsObjectContainerPtr  pToFree;
  tco13_ClsIdEntry*      pContainerInfo;
  OmsContext_co12*       currContext;
  int                    subtransLvl = m_session->CurrentSubtransLevel();
  while (subtransLvl > 0) {
    pBeforeImage = m_beforeImages[subtransLvl-1];
    pPrev        = &m_beforeImages[subtransLvl-1];
    while (NULL != pBeforeImage) {
      currContext = *REINTERPRET_CAST(OmsContext_co12**, &pBeforeImage->m_pobj);
      if (currContext == pContext) {
        pContainerInfo = pBeforeImage->GetContainerInfo(currContext);
        pToFree        = pBeforeImage;
        pBeforeImage   = pBeforeImage->m_hashnext;
        *pPrev         = pBeforeImage;
        pContainerInfo->chainFree(*currContext, pToFree);
      }
      else {
        pPrev        = &pBeforeImage->m_hashnext;
        pBeforeImage = pBeforeImage->m_hashnext;
      }
    }
    subtransLvl--; 
  } 
}

/*----------------------------------------------------------------------*/

void cco13_BeforeImages::subtransCommit (int wantedSubtransLvl) {
  OmsObjectContainerPtr pBeforeImage, p;
  OmsObjectContainerPtr found;
  OmsContext_co12*      context;
  int         subtransLvl = m_session->CurrentSubtransLevel();
  tsp00_Uint4 mask = (1 == wantedSubtransLvl) ? 0 : (0xFFFFFFFF >> (MAX_SUBTRANS_LEVEL - wantedSubtransLvl + 1));
  while (subtransLvl >= wantedSubtransLvl) {
    subtransLvl--;
    pBeforeImage                = m_beforeImages[subtransLvl];
    m_beforeImages[subtransLvl] = NULL;
    OmsObjectContainerPtr* pInsert = &m_beforeImages[wantedSubtransLvl-2];
    while (pBeforeImage) {
      p                  = pBeforeImage;
      pBeforeImage       = pBeforeImage->m_hashnext;
      context            = *REINTERPRET_CAST(OmsContext_co12**, &p->m_pobj);
      found = context->FindObjInContext(&p->m_oid);
      if (found) {
        found->m_beforeImages &= mask;
        if (!found->existBeforeImage(wantedSubtransLvl-1, m_session->InVersion())) 
        {
            p->m_hashnext = *pInsert;
            *pInsert      = p;
            pInsert       = &p->m_hashnext;
            found->setBeforeImage (wantedSubtransLvl-1);
        }
        else 
        {
          p->ChainFree(context);
        }
      }
      else {
        // object must have been release by application, i.e. omsRelease of an
        // object, that has been read for update without omsStore, omsDelete
      }
    }
  }
}

/*----------------------------------------------------------------------*/

void cco13_BeforeImages::subtransRollback (int wantedSubtransLvl) {
  OmsObjectContainerPtr pBeforeImage, p;
  OmsObjectContainerPtr found;
  tco13_ClsIdEntry*     pContainerInfo;
  OmsContext_co12*      context;
  int         subtransLvl = m_session->CurrentSubtransLevel();
  tsp00_Uint4 mask = (1 == wantedSubtransLvl) ? 0 : (0xFFFFFFFF >> (MAX_SUBTRANS_LEVEL - wantedSubtransLvl + 1));
  while (subtransLvl >= wantedSubtransLvl) {
    subtransLvl--;
    pBeforeImage                = m_beforeImages[subtransLvl];
    m_beforeImages[subtransLvl] = NULL;
    while (pBeforeImage) {
      p                  = pBeforeImage;
      p->m_beforeImages &= mask; 
      pBeforeImage       = pBeforeImage->m_hashnext;
      context            = *REINTERPRET_CAST(OmsContext_co12**, &p->m_pobj);
      if (p->NewFlag()) {
        pContainerInfo = p->GetContainerInfo(context);
        OmsObjectContainerPtr pObj = context->FindObjInContext (&p->m_oid);
        if (context->IsVersion()) {
            if (pContainerInfo->IsKeyedObject()) {
                if (pObj && (pObj == pContainerInfo->VersionFindKey(pContainerInfo->GetKeyPtr(pObj))))
                {
                    pContainerInfo->VersionDelKey(pObj);
                }
          }
        }
        if (NULL != pObj)
        {
          context->DecNewObjectsToFlush();
          context->DeleteObjInContext(pObj, pContainerInfo);
        }
        pContainerInfo->chainFree(*context, p);
      }
      else {
        pContainerInfo = p->GetContainerInfo(context);
        found = context->FindObjInContext(&p->m_oid);
        if (NULL != found) 
        {
          if (found->IsNewObject() && ! p->IsNewObject())
          {
            context->DecNewObjectsToFlush();
          }
          found->Copy (context, p, pContainerInfo->GetObjectSize());
          if ((context->IsVersion()) && context->IsVersionOid(found->m_oid) && (!found->DeletedFlag()) && (pContainerInfo->IsKeyedObject()))
          {
              pContainerInfo->VersionReplaceOrAddKeyContainerPtr(found);
          }
          /* restore vtable ptr */
          *REINTERPRET_CAST(void**, &found->m_pobj) = pContainerInfo->GetVirtualTablePtr();
          pContainerInfo->chainFree(*context, p);
        }
        else {
          // object must have been release by application, i.e. omsRelease of an
          // object, that has been read for update without omsStore, omsDelete
        }
      }
    }
  }
}

/*----------------------------------------------------------------------*/

void cco13_BeforeImages::insertBeforeImage (OmsObjectContainerPtr pObj, int subtransLvl) {
  if (subtransLvl > 0) {
#ifdef _ASSERT_OMS
    OmsObjectContainerPtr p = m_beforeImages[subtransLvl-1];
    while (p) {
      _OMSASSERT (m_session, pObj->m_oid != p->m_oid);
      p = p->m_hashnext;
    }
#endif
    tco13_ClsIdEntry* pContainerInfo = pObj->GetContainerInfo(m_session->m_context);
    OmsObjectContainerPtr pBeforeImage;
    if (pObj->IsVerNewObject()) {
      pBeforeImage = m_session->GetMemoryInVersion (*pContainerInfo);
    } else {
      pBeforeImage = m_session->GetMemory (*pContainerInfo);
    }
    memcpy (pBeforeImage, pObj, pContainerInfo->GetObjectSize());
    pBeforeImage->m_hashnext      = m_beforeImages[subtransLvl-1];
    m_beforeImages[subtransLvl-1] = pBeforeImage;
    pObj->setBeforeImage (subtransLvl);
    /* use the vtable ptr of the before image to store the context the object is from */
    *REINTERPRET_CAST(OmsContext_co12**, (&pBeforeImage->m_pobj)) = m_session->m_context;
    if (pObj->VarObjFlag())
    {
      OmsVarObjInfo_co12* objInfo = REINTERPRET_CAST (OmsVarObjInfo_co12*, &pObj->m_pobj);
      objInfo->incRefCnt();
    }
  }
}

/*----------------------------------------------------------------------*/

void cco13_BeforeImages::insertNewBeforeImage (OmsObjectContainerPtr pObj, tco13_ClsIdEntry* pContainerInfo, int subtransLvl) {

  if (subtransLvl > 0) {
#ifdef _ASSERT_OMS
    OmsObjectContainerPtr p = m_beforeImages[subtransLvl-1];
    while (p) {
      _OMSASSERT (m_session, pObj->m_oid != p->m_oid);
      p = p->m_hashnext;
    }
#endif
    OmsObjectContainerPtr pBeforeImage;
    if (pObj->IsVerNewObject()) {
      pBeforeImage = m_session->GetMemoryInVersion (*pContainerInfo);
    } else {
      pBeforeImage = m_session->GetMemory (*pContainerInfo);
    }
    memcpy (pBeforeImage, pObj, pContainerInfo->GetObjectSize());
    pBeforeImage->m_hashnext      = m_beforeImages[subtransLvl-1];
    pBeforeImage->MarkNew();
    m_beforeImages[subtransLvl-1] = pBeforeImage;
    pObj->setBeforeImage (subtransLvl);
    /* use the vtable ptr of the before image to store the context the object is from */
    *REINTERPRET_CAST(OmsContext_co12**, (&pBeforeImage->m_pobj)) = m_session->m_context;
  }
}



