/* @lastChanged: "1998-09-17  13:47"
 * @filename:    vco15.cpp
 * @purpose:     "Date/Time"
 * @release:     M.m.c.p
 * @see:         ""
 *
 * @copyright:   "Copyright by SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 */

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "geo00.h"
#include "gco00.h"
#include "hco15.h"
#undef  AFX_EXT_CLASS
#define AFX_EXT_CLASS
#include "gco001.h"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#ifdef NO_INLINES
# define _INLINE
#else
# define _INLINE	inline
#endif

#define SEC_PER_DAY 86400L  // 24*60*60

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/



/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/



/*===========================================================================*
 *  GLOBAL VARIABLES                                                         *
 *===========================================================================*/

static int co15_mtbl[13]={ 0, 31, 61, 92, 122, 153, 184, 214, 245, 275, 306, 337, 367 };

/*===========================================================================*
 *  LOCAL VARIABLES                                                          *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL FUNCTIONS (PROTOTYPES)                                             *
 *===========================================================================*/

_INLINE int co15_day2year( unsigned long v );
_INLINE unsigned long co15_ymd2date( int y, int m, int d );
_INLINE unsigned long co15_year2day( int v );
_INLINE int co15_getDay  ( unsigned long date ); 
_INLINE int co15_getMonth( unsigned long date );
 
/*===========================================================================*
 *  GLOBAL FUNCTIONS (CODE)                                                  *
 *===========================================================================*/

cco15_DateTime::cco15_DateTime( long year, long month, long day, long hour, long min, long sec )
{
  dtValue = co15_ymd2date (year, month, day) * SEC_PER_DAY + (sec+60*(min+60*hour));
}

int cco15_DateTime::day ( void ) const 
{
  return co15_getDay (dtValue/SEC_PER_DAY);
}

int cco15_DateTime::month ( void ) const
{
  return co15_getMonth (dtValue/SEC_PER_DAY);
}

int cco15_DateTime::year ( void ) const
{
  return co15_day2year (dtValue/SEC_PER_DAY);
}

/*===========================================================================*
 *  LOCAL FUNCTIONS (CODE)                                                   *
 *===========================================================================*/

_INLINE int co15_day2year( unsigned long v )
{
  return int(1970L + (v+0.5)/365.25);
}

_INLINE unsigned long co15_year2day( int v )
{
  return int((v-1970L)*365.25+0.25);
}

_INLINE unsigned long co15_ymd2date( int y, int m, int d )
{
  m--; d--;
  if( y<100 ) y+=1900;
  if( y<1970 || y>2099 ) throw DbpError (DbpError::RTE_ERROR,  e_invalid_timestamp);
  if( m<0 || m>=12 ) throw DbpError (DbpError::RTE_ERROR,  e_invalid_timestamp);
  unsigned long x=co15_year2day(y) + d;

  const int off = ((y%4)==0) ? 31+29 : 31+28;
  if( m>=2 ) x+=off; else x-=306;

  if( m<2 ) m+=10; else m-=2;
  x+=co15_mtbl[m];

  return x;
}

_INLINE int co15_getDay( unsigned long date ) 
{
  long x=date;             // Tag seit BASE_YEAR
  int j=co15_day2year(x);  // Jahr dazu
  x-=co15_year2day(j);     // Tag seit Anfang des Jahres j

  int off = ((j%4)==0) ? 31+29 : 31+28;
  if( x>=off ) x-=off; else x+=306;
  int m=0;
  while( x >= co15_mtbl[m] ) m++;
  m--;
  int t=(int)(x-co15_mtbl[m]);
  t++;
  return t;
}

_INLINE int co15_getMonth( unsigned long date )
{
  long x=date;             // Tag seit BASE_YEAR
  int j=co15_day2year(x);  // Jahr dazu
  x-=co15_year2day(j);     // Tag seit Anfang des Jahres j

  int off = ((j%4)==0) ? 31+29 : 31+28;
  if( x>=off ) x-=off; else x+=306;
  int m=0;
  while( x >= co15_mtbl[m] ) m++;
  m--;
  if( m>=10 ) m-=10; else m+=2;
  m++;
  return m;
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/