/* @lastChanged: "1999-04-21  16:14"

  * @filename:   vco18.cpp
  * @purpose:    "Implementation of OMS Class Dictionary"
  * @release:    7.2.0.0
  * @see:        "http://www.bea.sap-ag.de/doku/rel-7.1/vco10.html"
  *
  * @copyright:  "Copyright by   SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/


/* ================================================================================ */
/* ==========              REPLACEMENTS BY THE PREPROCESSOR               ========= */
/* ================================================================================ */



/* ================================================================================ */
/* ==========                 DECLARATION OF TYPES                        ========= */
/* ================================================================================ */




/* ================================================================================ */
/* ==========                 DECLARATION OF FUNCTIONS                    ========= */
/* ================================================================================ */

#include "geo00.h"
#include "gco00.h"
#include "gco10.h"
#include "hco16.h"
#include "hco18.h"

extern co16_SharedMemAllocator SharedMemAllocator_co16;

tsp00_Uint4 ContainerHandle_co18 = 0;

static void co18_ToKnlIdentifier (const char* Source, tsp00_KnlIdentifier& Dest) {
  size_t copyLen = strlen(Source);
  if (copyLen > sizeof(Dest)) {
    copyLen = sizeof(Dest);
  }
  memcpy(&Dest[0], Source, copyLen);
  memset(&Dest[copyLen], ' ', sizeof(Dest) - copyLen);
}

/*----------------------------------------------------------------------------------*/

ClassInfo_co18::ClassInfo_co18(const char*             ClassName, 
                               const ClassIDRef        Guid,
                               const ClassInfo_co18*   pBaseClass,
                               const tgg00_ObjColDesc& ColDesc,
                               size_t                  ObjSize, 
                               void*                   vtptr, 
                               ClassInfo_co18*         HashNext)
                               :  
                               m_guid(Guid),
                               m_ObjectSize(ObjSize),
                               m_VTPtr(vtptr),
                               m_pBaseClass(pBaseClass),
                               m_HashNext(HashNext),
                               m_isBaseClass(false), 
                               m_isDerivedClass(pBaseClass != NULL),
                               m_isVarObject(false)
{
  m_InvColDesc = ColDesc;
  size_t ClassNameLength = strlen(ClassName);
  if (ClassNameLength >= sizeof(m_ClassName)) {
    ClassNameLength = sizeof(m_ClassName) - 1;
  }
  memcpy (&m_ClassName[0], ClassName, ClassNameLength);
  m_ClassName[ClassNameLength] = 0;
  if (0 < m_InvColDesc.colHashCount_gg00) {
    m_KeyDesc    = (*ColDesc.colHashList_gg00)[0];
    m_InvColDesc.colHashList_gg00  = REINTERPRET_CAST(tgg00_StackListPtr, (&m_KeyDesc));
  }
#ifdef LIVECACHE_INTGUIDS
  m_isVarObject = (Guid == VAR_OBJ_GUID_GCO10);
#else
  m_isVarObject = (0 == memcmp(&Guid, &VAR_OBJ_GUID_GCO10, sizeof(Guid)));
#endif
  m_isKeyedObject = ColDesc.colHashCount_gg00 > 0;
}

/*----------------------------------------------------------------------------------*/

void* ClassInfo_co18::operator new(size_t sz) {
  return SharedMemAllocator_co16.allocate(sz);
}

/*----------------------------------------------------------------------------------*/

void ClassInfo_co18::DeleteSelf()
{
#if defined(_ASSERT_OMS)
  memset (this, 0, sizeof(*this));
#endif
  SharedMemAllocator_co16.deallocate(this);
}

/*----------------------------------------------------------------------------------*/

#if defined(PLACEMENT_DELETE_MEO00)

void  ClassInfo_co18::operator delete (void* p)
{
  SharedMemAllocator_co16.deallocate(p);
}

#endif

/*----------------------------------------------------------------------------------*/

void ClassInfo_co18::Dump (ClassDumpInfo_co18& dumpInfo)
{
  dumpInfo.m_addr           = this;
  dumpInfo.m_guid           = m_guid;
  dumpInfo.m_pBaseClass     = CONST_CAST(ClassInfo_co18*, m_pBaseClass);
  memset(&dumpInfo.m_ClassName[0], ' ', sizeof(dumpInfo.m_ClassName));
  memcpy(&dumpInfo.m_ClassName[0], &m_ClassName[0], strlen(m_ClassName));
  dumpInfo.m_ObjectSize     = (tsp00_Int4) m_ObjectSize;
  dumpInfo.m_VTBLPtr        = m_VTPtr;
  dumpInfo.m_isBaseClass    = m_isBaseClass;
  dumpInfo.m_isDerivedClass = m_isDerivedClass;
  dumpInfo.m_isVarObject    = m_isVarObject;
  dumpInfo.m_filler         = 0;
}

/*----------------------------------------------------------------------------------*/

ContainerInfo_co18::ContainerInfo_co18 (ClassInfo_co18* pClassInfo, const tgg00_ContainerId& t, 
                                tsp00_Uint4 containerHandle, size_t psz, OmsSchemaHandle sh, OmsContainerNo cno) :
m_clsidHashNext(NULL), m_guidHashNext(NULL), m_clsInfo(pClassInfo),
m_PersistentSize(psz), m_ContainerNo(cno), m_schema(sh), m_dropped(false), m_refCnt(0) 
{
  memcpy (&m_ContainerId, &t, sizeof (m_ContainerId));
}

/*----------------------------------------------------------------------------------*/

void* ContainerInfo_co18::operator new(size_t sz)
{
  return SharedMemAllocator_co16.allocate(sz);
}

/*----------------------------------------------------------------------------------*/

void ContainerInfo_co18::DeleteSelf() {
#if defined(_ASSERT_OMS)
  memset (this, 0, sizeof(*this));
#endif
  SharedMemAllocator_co16.deallocate(this);
}

/*----------------------------------------------------------------------------------*/
#if defined(PLACEMENT_DELETE_MEO00)

void  ContainerInfo_co18::operator delete (void* p) {
  SharedMemAllocator_co16.deallocate(p);
}

#endif

/*----------------------------------------------------------------------------------*/

void ContainerInfo_co18::Dump(ContainerDumpInfo_co18& dumpInfo)
{
  dumpInfo.m_addr          = this;
  dumpInfo.m_clsidHashNext = m_clsidHashNext;
  dumpInfo.m_guidHashNext  = m_guidHashNext;
  dumpInfo.m_clsInfo       = m_clsInfo;
  dumpInfo.m_ContainerNo   = m_ContainerNo;
  dumpInfo.m_schema        = m_schema;
  dumpInfo.m_ContainerId   = m_ContainerId;
  dumpInfo.m_PersistentSize= (tsp00_Int4) m_PersistentSize;
  dumpInfo.m_refCnt        = m_refCnt;
  dumpInfo.m_dropped       = m_dropped;
  dumpInfo.m_filler        = false;
}

/*----------------------------------------------------------------------------------*/

CContainerDictionary_co18::CContainerDictionary_co18() : IGuidHash_co001(), m_dropId(0) {
  this->Init();
}

/*----------------------------------------------------------------------------------*/

tgg00_BasisError CContainerDictionary_co18::DropContainer (IliveCacheSink* lcSink, 
                                                           const ClassIDRef guid, 
                                                           OmsSchemaHandle schema,
                                                           OmsContainerNo cno,
                                                           size_t arrayByteSize)
{
  tgg00_BasisError DBError = e_ok;
  ClassID rguid = guid;
  if (arrayByteSize) {
    omsMakeArrayGuid(rguid, arrayByteSize);
  }
  CriticalSection_co001 cs(lcSink, RGN_CLASS_DIR);
  cs.Enter();
  GUID_TEMP(lguid, rguid);
  CHResult_co001 hr = lcSink->DropContainer (schema, lguid, cno, DBError);
  if (e_ok == DBError) {
    ContainerInfo_co18* p = this->FindGuid(guid, schema, cno);
    if (NULL != p) {
      p->MarkDropped();
      ++m_dropId; // PTS 1121231
    }
  }
  return DBError;
}

/*----------------------------------------------------------------------------------*/

tgg00_BasisError CContainerDictionary_co18::DropSchema (IliveCacheSink* lcSink, OmsSchemaHandle Schema) {
  tgg00_BasisError DBError = 0;
  CriticalSection_co001 cs(lcSink, RGN_CLASS_DIR);
  cs.Enter();
  CHResult_co001 hr = lcSink->DropSchema (Schema, &DBError);
  if (0 == DBError) {
    for (ContainerIterator iter(this); iter; ++iter) {
      if (iter()->GetSchema() == Schema) {
        iter()->MarkDropped();
      }
    }
    ++m_dropId;
  }
  return DBError;
}

/*----------------------------------------------------------------------------------*/

void  CContainerDictionary_co18::Dump(IDump_cco02& dumpObj)
{
  IliveCacheSink* lcSink = NULL;
  if (dumpObj.Synchronize())
  {
    lcSink = co10_GetLcSink();
  }
  CriticalSection_co001 cs(lcSink, RGN_CLASS_DIR);
  dumpObj.SetDumpLabel(LABEL_OMS_CLASS);
  if (dumpObj.Synchronize())
  {
    cs.Enter();
  }
  for (int hashSlot = 0; hashSlot < CContainerDictionary_co18::MaxHeadEntries; ++hashSlot)
  {
    ClassDumpInfo_co18 dumpInfo(hashSlot);
    ClassInfo_co18* p = m_classInfoHead[hashSlot];
    while (NULL != p)
    {
      p->Dump(dumpInfo);
      dumpObj.Dump(&dumpInfo, sizeof(dumpInfo));
      p = p->m_HashNext;
    }
  }
  dumpObj.SetDumpLabel(LABEL_OMS_CONTAINER);
  for (ContainerIterator iter(this); iter; ++iter)
  {
    ContainerDumpInfo_co18 dumpInfo;
    iter()->Dump(dumpInfo);
    dumpObj.Dump(&dumpInfo, sizeof(dumpInfo));
  }
}

/*----------------------------------------------------------------------------------*/

ClassInfo_co18* CContainerDictionary_co18::FindGuid (const ClassIDRef guid) {
  int HashSlot  = HashValue(guid, CContainerDictionary_co18::MaxHeadEntries);
  ClassInfo_co18* p = m_classInfoHead[HashSlot];
  while (NULL != p) {
    if (p->GetGuid() == guid) {
      return p;
    }
    else {
      p = p->m_HashNext;
    }
  }
  return NULL;
}

/*----------------------------------------------------------------------------------*/

ClassInfo_co18* CContainerDictionary_co18::FindGuid (IliveCacheSink* lcSink, const ClassIDRef guid) {
  CriticalSection_co001 cs(lcSink, RGN_CLASS_DIR);
  cs.Enter();
  ClassInfo_co18* p = this->FindGuid(guid);
  return p;
}

/*----------------------------------------------------------------------------------*/

ContainerInfo_co18* CContainerDictionary_co18::FindViaContainerHandle  (IliveCacheSink* lcSink, tsp00_Uint4 containerHandle) {
  CriticalSection_co001 cs(lcSink, RGN_CLASS_DIR);
  cs.Enter();
  int clsidHashSlot = containerHandle % CContainerDictionary_co18::MaxHeadEntries;
  ContainerInfo_co18* p = m_clsidHead[clsidHashSlot];
  while (NULL != p) {
    if (p->GetContainerHandle() == containerHandle) {
      if (p->IsDropped()) {
        p = NULL;
      }
      break;
    }
    else {
      p = p->m_clsidHashNext;
    }
  }
  return p;
}

/*----------------------------------------------------------------------------------*/

ContainerInfo_co18* CContainerDictionary_co18::FindViaGuid (IliveCacheSink* lcSink, const ClassIDRef guid, OmsSchemaHandle sh, OmsContainerNo cno) {
  CriticalSection_co001 cs(lcSink, RGN_CLASS_DIR);
  cs.Enter();
  ContainerInfo_co18* p = this->FindGuid(guid, sh, cno);
  if ((p != NULL) && (p->IsDropped())) {
    p = NULL;
    co10_Throw(DbpError (DbpError::DB_ERROR, e_unknown_guid));  // PTS 1121231
  }
  return p;
}

/*----------------------------------------------------------------------------------*/

ContainerInfo_co18* CContainerDictionary_co18::FindGuid(const ClassIDRef guid, OmsSchemaHandle sh, OmsContainerNo cno) {
  int guidHashSlot  = HashValue(guid, sh, cno, CContainerDictionary_co18::MaxHeadEntries);
  ContainerInfo_co18* p = m_guidHead[guidHashSlot];
  while (NULL != p) {
    if ((p->m_schema == sh) && (p->m_ContainerNo == cno) && omsIsCompatibleGuid(p->m_clsInfo->GetGuid(), guid)) {
      break;
    }
    else {
      p = p->m_guidHashNext;
    }
  }
  return p;
}

/*----------------------------------------------------------------------------------*/

void CContainerDictionary_co18::Init() {
  int ix;
  for (ix = 0; ix < MaxHeadEntries; ++ ix) {
    m_classInfoHead[ix] = NULL;
    m_clsidHead[ix]     = NULL;
    m_guidHead [ix]     = NULL;
  }
}

/*----------------------------------------------------------------------------------*/

ClassInfo_co18* CContainerDictionary_co18::RegisterClass (IliveCacheSink*     lcSink, 
                                                       const char*            ClassName,
                                                       const ClassIDRef       guid,
                                                       const ClassIDPtr       pBaseClassGuid,
                                                       const tgg00_ObjColDesc &ColDesc,
                                                       size_t                 ObjectSize, 
                                                       void*                  vtptr)
{
  ClassInfo_co18* p = FindGuid(guid);
  if (NULL == p) 
  {
    ClassInfo_co18* pBaseClass = NULL;
    int HashSlot = HashValue(guid, CContainerDictionary_co18::MaxHeadEntries);
#ifdef	LIVECACHE_INTGUIDS
    // now register GUID for array objects, if needed
    ClassID arrayBaseGuid = omsMaskGuid(guid);
    if (arrayBaseGuid != guid) {
      // we need to register base guid for array objects of this class
      ClassInfo_co18 *pb = RegisterClass(lcSink, ClassName, arrayBaseGuid,
        pBaseClassGuid, ColDesc, sizeof(void*), vtptr);
      pb->SetBaseClass();
    }
    if (0 != pBaseClassGuid) {
      pBaseClass = FindGuid(pBaseClassGuid);
#else
    if (NULL != pBaseClassGuid) {
      pBaseClass = FindGuid(*pBaseClassGuid);
#endif
      if (NULL == pBaseClass)
      {
        co10_Throw(DbpError (DbpError::DB_ERROR, e_unknown_guid, "RegisterClass (Base Class)"));
      }
      else
      {
        pBaseClass->SetBaseClass();
      }
    }
    p = new ClassInfo_co18 (ClassName, guid, pBaseClass, ColDesc, ObjectSize, vtptr, m_classInfoHead[HashSlot]);
    m_classInfoHead[HashSlot] = p;
  }
  else
  { // check, if previous and current definition are compatible
    if (ObjectSize != (size_t) p->GetObjectSize())
    {
      DbpBase base(lcSink);
      base.dbpOpError("Register Class :  Size old %d, new Size %d", 
        ObjectSize, p->GetObjectSize());
      co10_Throw(DbpError (DbpError::DB_ERROR, e_incompatible_datatypes, "RegisterClass"));
    }
  }
  return p;
}

/*----------------------------------------------------------------------------------*/

ContainerInfo_co18* CContainerDictionary_co18::RegisterContainer (IliveCacheSink* lcSink,
                                                       const ClassIDRef       guid, 
                                                       const char*            ClassName, 
                                                       size_t                 PersistentSize, 
                                                       size_t                 ObjectSize,
                                                       const ClassIDPtr       pBaseClass,
                                                       const tgg00_ObjColDesc &ColDesc,
                                                       OmsSchemaHandle        sh, 
                                                       OmsContainerNo         ContainerNo,
                                                       void*                  vtptr,
                                                       size_t                 arrayByteSize)
{
  HRESULT      hr = S_OK;
  tsp00_Int2   DBError = 0;
  ClassID      rguid = guid;
  if (arrayByteSize) {
    omsMakeArrayGuid(rguid, arrayByteSize);
  }
  CriticalSection_co001 cs(lcSink, RGN_CLASS_DIR);
  cs.Enter();
  ContainerInfo_co18* p = FindGuid (rguid, sh, ContainerNo);
  if (NULL == p) {
    tgg00_ContainerId   containerId;
    tsp00_KnlIdentifier knlIdentifier;
    co18_ToKnlIdentifier(ClassName, knlIdentifier);
	  GUID_TEMP(lguid, rguid);
    hr = lcSink->RegisterClass (sh, CONST_CAST(GUID*, &lguid), &knlIdentifier, (tsp00_Int4) PersistentSize,
      ContainerNo, (unsigned char*) &ColDesc, (unsigned char*) &containerId, &DBError);
    if ((!FAILED(hr)) && (0 == DBError)) {
      ++ContainerHandle_co18;
      ClassInfo_co18* pClass = RegisterClass (lcSink, ClassName, rguid, pBaseClass, ColDesc, ObjectSize, vtptr);
      p = new ContainerInfo_co18 (pClass, containerId, ContainerHandle_co18, PersistentSize, sh, ContainerNo);
      this->Insert(p);
    }
  }
  if (FAILED(hr)) {
    co10_Throw (DbpError (DbpError::HRESULT_ERROR, hr));
  }
  if (0 != DBError) {
    co10_Throw(DbpError (DbpError::DB_ERROR, DBError, "RegisterClass"));
  }
  return p;
}

/*----------------------------------------------------------------------------------*/

void CContainerDictionary_co18::Delete (IliveCacheSink* lcSink, ContainerInfo_co18* pContainerInfo) {
  OMS_TRACE(omsTrContainerDir, lcSink, "CContainerDictionary_co18::Delete : " << pContainerInfo->GetClassInfoPtr()->GetClassName());
  int clsidHashSlot = pContainerInfo->GetContainerHandle() % CContainerDictionary_co18::MaxHeadEntries;
  pContainerInfo_co18* prev = &m_clsidHead[clsidHashSlot]; 
  pContainerInfo_co18  curr = *prev;
  bool found = false;
  while (NULL != curr) {
    if (curr == pContainerInfo) {
      *prev = pContainerInfo->m_clsidHashNext;
      found = true;
      break;
    }
    else {
      prev = &curr->m_clsidHashNext;
      curr = curr->m_clsidHashNext;
    }
  }
  if (found) {
    found = false;
    int guidHashSlot  = HashValue(pContainerInfo->m_clsInfo->GetGuid(), pContainerInfo->m_schema,
      pContainerInfo->m_ContainerNo, CContainerDictionary_co18::MaxHeadEntries);
    prev = &m_guidHead[guidHashSlot];
    curr = *prev;
    while (NULL != curr) {
      if (curr == pContainerInfo) {
        *prev = pContainerInfo->m_guidHashNext;
        found = true;
        break;
      }
      else {
        prev = &curr->m_guidHashNext;
        curr = curr->m_guidHashNext;
      }
    }
  }
  if (found) {
    pContainerInfo->DeleteSelf();
  }
  else {
    co10_Throw (DbpError (DbpError::DB_ERROR, -28999, "CContainerDictionary_co18::Delete"));
  }
}

/*----------------------------------------------------------------------------------*/

void CContainerDictionary_co18::Insert (ContainerInfo_co18* pContainerInfo) { 
  int clsidHashSlot = pContainerInfo->GetContainerHandle() % CContainerDictionary_co18::MaxHeadEntries;
  int guidHashSlot  = HashValue(pContainerInfo->m_clsInfo->GetGuid(), pContainerInfo->m_schema,
    pContainerInfo->m_ContainerNo, CContainerDictionary_co18::MaxHeadEntries);
  pContainerInfo->m_clsidHashNext = m_clsidHead[clsidHashSlot]; 
  m_clsidHead[clsidHashSlot]      = pContainerInfo;
  pContainerInfo->m_guidHashNext  = m_guidHead[guidHashSlot];
  m_guidHead[guidHashSlot]        = pContainerInfo;
}

/*----------------------------------------------------------------------------------*/

void CContainerDictionary_co18::ReleaseContainerInfo(IliveCacheSink* lcSink, pContainerInfo_co18 p) {
  CriticalSection_co001 cs(lcSink, RGN_CLASS_DIR);
  cs.Enter();
  OMS_TRACE(omsTrContainerDir, lcSink, "ReleaseContainerInfo, RefCnt : " << p->m_refCnt);
  OMS_TRACE(omsTrContainerDir, lcSink, "Name : " << p->GetClassInfoPtr()->GetClassName() << "Schema : " 
    << p->GetSchema() << ", CNo : " << p->m_ContainerNo);
  if ((0 == p->DecRefCnt()) && (p->IsDropped())) {
    this->Delete(lcSink, p);
  }
}

/*----------------------------------------------------------------------------------*/

void CContainerDictionary_co18::Shutdown()
{
  int hashSlot;
  for (hashSlot = 0; hashSlot < CContainerDictionary_co18::MaxHeadEntries; ++hashSlot)
  {
    ClassInfo_co18* p = m_classInfoHead[hashSlot];
    while (NULL != p)
    {
      ClassInfo_co18* pToFree = p;
      p = p->m_HashNext;
      pToFree->DeleteSelf();
    }
  }
  for (hashSlot = 0; hashSlot < CContainerDictionary_co18::MaxHeadEntries; ++hashSlot)
  {
    ContainerInfo_co18* curr = m_clsidHead[hashSlot];
    while (NULL != curr)
    {
      ContainerInfo_co18* pToFree = curr;
      curr = curr->m_clsidHashNext;
      pToFree->DeleteSelf();
    }
  }
  this->Init();
}

/*----------------------------------------------------------------------------------*/

CContainerDictionary_co18::ContainerIterator::ContainerIterator(CContainerDictionary_co18* clsDir) : 
m_clsDir(clsDir), m_index(0), m_curr(clsDir->m_clsidHead[0]) {
  while ((NULL == m_curr) && (++m_index < CContainerDictionary_co18::MaxHeadEntries)) {
    m_curr = m_clsDir->m_clsidHead[m_index];
  }
}

/*----------------------------------------------------------------------------------*/

void CContainerDictionary_co18::ContainerIterator::reset(CContainerDictionary_co18* clsDir)
{
  m_clsDir = clsDir;
  m_index = 0;
  m_curr = clsDir->m_clsidHead[0];
  while ((NULL == m_curr) && (++m_index < CContainerDictionary_co18::MaxHeadEntries)) {
    m_curr = m_clsDir->m_clsidHead[m_index];
  }
}

/*----------------------------------------------------------------------------------*/

void CContainerDictionary_co18::ContainerIterator::operator++() {
  if (NULL != m_curr) { 
    m_curr = m_curr->m_clsidHashNext;
    while (NULL == m_curr) { // PTS 1106820
      ++m_index;
      if (m_index < CContainerDictionary_co18::MaxHeadEntries) {
        m_curr = m_clsDir->m_clsidHead[m_index];
      }
      else {
        return;
      }
    }
  }
}

/*----------------------------------------------------------------------------------*/

