 /*!***************************************************************************

  module:       vco19.cpp

  ------------------------------------------------------------------------------

  responsible:  ThomasA

  special area: OMS

  description:  OMS Monitor

  see also:     

  ------------------------------------------------------------------------------

  copyright:            Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

  *****************************************************************************/


/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "hco19.h"
#include "gco90.h"
#include "geo573.h"

/*===========================================================================*
 *  GLOBAL VARIABLES                                                         *
 *===========================================================================*/

static CompareMonitorKey_co19 glob_MonitorKeyComparator;

/*---------------------------------------------------------------------------*/

int CompareMonitorKey_co19::Compare(const MonitorKey_co19& k1, const MonitorKey_co19& k2) {
  int cmp = memcmp(&k1.m_iid, &k2.m_iid, sizeof(k1.m_iid));
  if (0 != cmp) {
    if (cmp < 0) {
      return -1;
    }
    else {
      return 1;
    }
  }
  else {
    if (k1.m_dispid == k2.m_dispid) {
      return 0;
    }
    else {
      if (k1.m_dispid < k2.m_dispid) {
        return -1;
      }
      else {
        return 1;
      }
    }
  }
}

/*---------------------------------------------------------------------------*/

Monitor_co19::Monitor_co19() : m_monitorDir(&glob_MonitorKeyComparator, &SharedMemAllocator_co16)
{
}

/*---------------------------------------------------------------------------*/

void Monitor_co19::Insert(const CLSID& iid, short dispId, tgg00_COMMonitorInfo& info)
{
  MonitorKey_co19 monKey(iid, dispId);
  tgg00_COMMonitorInfo* found = m_monitorDir.Find(monKey);
  if (NULL != found) {
    found->cmiCallCnt_gg00+= info.cmiCallCnt_gg00;
    for (int monEnum = cmiRunTime; monEnum <= cmiCacheSize; ++monEnum) {
      switch (monEnum) {
      case cmiRunTimeMin :
      case cmiMinWaitNewConsView :
      case cmiMallocMin :
      case cmiFreeMin :
      case cmiDeltaMin :
      case cmiMinReadApplCallbacks :
      case cmiMinStreamRowsRead :
      case cmiMinWriteApplCallbacks :
      case cmiMinStreamRowsWrite :
        if (info.cmiCounters_gg00[monEnum] < found->cmiCounters_gg00[monEnum]) {
          found->cmiCounters_gg00[monEnum] = info.cmiCounters_gg00[monEnum];
        }
        break;
      case cmiRunTimeMax :
      case cmiMaxSubtransLevel :
      case cmiMaxWaitNewConsView :
      case cmiMallocMax :
      case cmiFreeMax :
      case cmiDeltaMax :
      case cmiMaxStreamRowsRead :
      case cmiMaxReadApplCallbacks :
      case cmiMaxWriteApplCallbacks :
      case cmiMaxStreamRowsWrite :
      case cmiOmsMaxHashChainLen :
      case cmiCacheSize :
        if (info.cmiCounters_gg00[monEnum] > found->cmiCounters_gg00[monEnum]) {
          found->cmiCounters_gg00[monEnum] = info.cmiCounters_gg00[monEnum];
        }
        break;
      default :
        {
          found->cmiCounters_gg00[monEnum] += info.cmiCounters_gg00[monEnum];
        }
      }
    }
  }
  else {
    tgg00_BasisError rc;
    try
    {
      found = m_monitorDir.Insert(monKey, rc);
      if (0 == rc) {
        memcpy(found, &info, sizeof(info));
      }
    }
    catch (BAD_ALLOC_GEO573&)
    {
      // if no memory is available, monitoring info cannot be saved
      // ignore it !
    }
  }
}

/*---------------------------------------------------------------------------*/

void Monitor_co19::Insert(co90_Monitor* monitor) /* PTS 1107731 */
{
  for (co90_Monitor::Iterator iter = monitor->begin(); iter; ++iter) {
    this->Insert(monitor->getIID(), iter.getDispId(), iter().m_monInfo);
  }
}

/*---------------------------------------------------------------------------*/

void Monitor_co19::Reset()
{
  m_monitorDir.DeleteAll();
}

/*---------------------------------------------------------------------------*/

