/****************************************************************************

  module      : vco30.cpp
  
  -------------------------------------------------------------------------
    
  responsible : ThomasA
      
  special area: Common base of SQL-Class and OMS
  description : Defines Functions used by SQL-Class and OMS
        
  last changed: 2000-01-10  9:34
           
  -------------------------------------------------------------------------
            
  copyright:    Copyright by SAP AG, 1999
              


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


/*===========================================================================*
*  INCLUDES                                                                 *
*===========================================================================*/


#include "hco30-k.h"
#include "heo02x.h"
#include "hsp77.h"

/*===========================================================================*
*  DEFINES                                                                  *
*===========================================================================*/


DbpBase::DbpBase() : m_lcSink(NULL) {
}

/*----------------------------------------------------------------------*/

DbpBase::DbpBase(IUnknown * pSink) : m_lcSink(REINTERPRET_CAST(IliveCacheSink*,pSink)) {
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpAdviseLcSink (IliveCacheSink* pSink) {
  m_lcSink = pSink;
}

/*----------------------------------------------------------------------*/

#if !defined(WIN32)
void DbpBase::dbpDebugBreak() {
};
#endif

/*----------------------------------------------------------------------*/

void DbpBase::dbpDropMessageFile(const char* fn) {
  tgg00_BasisError DBError;
  m_lcSink->DropFile (CONST_CAST(char*, fn), DBError);
  if (0 != DBError) {
    throw DbpError(DbpError::DB_ERROR, DBError);
  }
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpExistsFile(const char* fn, bool& doesExist, bool& isDirectory) {
  tgg00_BasisError DBError;
  m_lcSink->ExistsFile (CONST_CAST(char*, fn), doesExist, isDirectory, DBError);
  if (0 != DBError) {
    throw DbpError(DbpError::DB_ERROR, DBError);
  }
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpFreeLibrary (DbpHandle hd) {
  tsp_errtext errtext;
  bool        ok;
  HRESULT     hr;
  hr = m_lcSink->FreeLibrary(hd, ok, errtext, sizeof(tsp_errtext)-1 );
//  if (!sqlFreeLibrary(hd, errtext, sizeof(errtext) -1)) {
  if (!ok) {
    throw DbpError(DbpError::DB_ERROR, e_lib_not_released, errtext);
  }
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpGetDbrootSapPath(char* path, int bufSize) {
  tgg00_BasisError DBError;
  tsp00_Pathc      pathBuf; 
  m_lcSink->GetDbrootSapPath (pathBuf, DBError);
  if (0 != DBError) {
    throw DbpError(DbpError::DB_ERROR, DBError);
  }
  int len = (int)strlen(pathBuf);
  if (len < bufSize) {
    memcpy (path, &pathBuf[0], len + 1);
  }
  else {
    throw DbpError(DbpError::DB_ERROR, e_directory_not_found);
  }
}

/*----------------------------------------------------------------------*/

DbpPROC DbpBase::dbpGetProcAddress(DbpHandle hd, const char *pProcName) {
  tsp_errtext errtext;
  DbpPROC     p;
  p = sqlGetProcAddress(hd, pProcName,
      REINTERPRET_CAST(char*, &errtext[0]), sizeof(errtext-1));
  if (NULL == p) {
    throw DbpError (DbpError::DB_ERROR, e_proc_address_not_found, errtext);
  }
  return p;
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpGetWorkingDirectory(char* path, int bufSize) {
  tgg00_BasisError DBError;
  tsp00_Pathc      pathBuf; 
  m_lcSink->GetWorkingDirectory (pathBuf, DBError);
  if (0 != DBError) {
    throw DbpError(DbpError::DB_ERROR, DBError);
  }
  int len = (int)strlen(pathBuf);
  if (len < bufSize) {
    memcpy (path, &pathBuf[0], len + 1);
  }
  else {
    throw DbpError(DbpError::DB_ERROR, e_directory_not_found);
  }
}

/*----------------------------------------------------------------------*/

DbpHandle DbpBase::dbpLoadLibrary(const char* path, bool loadGlobal) {
  DbpHandle       hd;
  tsp_errtext     errtext;
  m_lcSink->LoadLibraryEx(path, loadGlobal, &hd, errtext, sizeof(tsp_errtext)-1 );
  return hd;
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpOpError( const char*fmt, ... ) {

  char        buffer[256];
  tsp00_Int2  DBError;
  tsp00_Int4  BufLen;
  HRESULT     hr;
  
  va_list args;
  va_start (args, fmt);
  sp77vsprintf ( buffer, sizeof(buffer), fmt, args );
  BufLen = (tsp00_Int4)strlen(buffer);
  hr = m_lcSink->MultiPurpose (m_show, mm_nil, &BufLen, 
    (unsigned char*) &buffer[0], &DBError);
  va_end (args);
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpOpErrorVarg( const char*fmt, va_list args ) {

  char        buffer[256];
  tsp00_Int2  DBError;
  tsp00_Int4  BufLen;
  HRESULT     hr;
  
  sp77vsprintf ( buffer, sizeof(buffer), fmt, args );
  BufLen = (tsp00_Int4)strlen(buffer);
  hr = m_lcSink->MultiPurpose (m_show, mm_nil, &BufLen, 
    (unsigned char*) &buffer[0], &DBError);
}

/*----------------------------------------------------------------------*/

OmsTypeInt4 DbpBase::dbpOpenMessageFile (const char* fn, unsigned long fileSize, bool doAppend) {
  tsp00_Int4 FileHandle;
  tgg00_BasisError DBError;
  m_lcSink->OpenFile (CONST_CAST(char*, fn), fileSize, doAppend, &FileHandle, &DBError);
  if (0 != DBError) {
    throw DbpError(DbpError::DB_ERROR, DBError);
  }
  return FileHandle;
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpOpMsg( const char*fmt, ... ) {

  char        buffer[256];
  tsp00_Int2  DBError;
  tsp00_Int4  BufLen;
  HRESULT     hr;

  va_list args;
  va_start (args, fmt);
  sp77vsprintf ( buffer, sizeof(buffer), fmt, args );
  BufLen = (tsp00_Int4)strlen(buffer);
  hr = m_lcSink->MultiPurpose (m_show, mm_log, &BufLen, 
    (unsigned char*) &buffer[0], &DBError);
  va_end (args);
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpOpMsgVarg( const char*fmt, va_list args ) {

  char        buffer[256];
  tsp00_Int2  DBError;
  tsp00_Int4  BufLen;
  HRESULT     hr;

  sp77vsprintf ( buffer, sizeof(buffer), fmt, args );
  BufLen = (tsp00_Int4)strlen(buffer);
  hr = m_lcSink->MultiPurpose (m_show, mm_log, &BufLen, 
    (unsigned char*) &buffer[0], &DBError);
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpTrace( const char*fmt, ... ) {
  va_list args;
  va_start (args, fmt);
  char buffer[256];
  sp77vsprintf ( buffer, sizeof(buffer) - 4, fmt, args );
  m_lcSink->Vtrace ((tsp00_Int4)strlen(buffer), (unsigned char*) &buffer[0]);
  va_end (args);
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpTraceVarg( const char*fmt, va_list args ) {
  char buffer[256];
  sp77vsprintf ( buffer, sizeof(buffer) - 4, fmt, args );
  m_lcSink->Vtrace ((tsp00_Int4)strlen(buffer), (unsigned char*) &buffer[0]);
}


/*----------------------------------------------------------------------*/

void DbpBase::dbpCloseMessageFile(OmsTypeInt4 fileHandle) {
  tgg00_BasisError DBError;
  m_lcSink->CloseFile (fileHandle, &DBError);
  if (0 != DBError) {
    throw DbpError(DbpError::DB_ERROR, DBError);
  }
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpSyncMessageFile (OmsTypeInt4 fileHandle) {
  tgg00_BasisError DBError;
  m_lcSink->SyncFile (NULL, fileHandle, &DBError);
  if (0 != DBError) {
    throw DbpError(DbpError::DB_ERROR, DBError);
  }
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpSyncMessageFile (const char* fn) {
  tgg00_BasisError DBError;
  m_lcSink->SyncFile (CONST_CAST(char*, fn), 0, &DBError);
  if (0 != DBError) {
    throw DbpError(DbpError::DB_ERROR, DBError);
  }
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpWriteMessageFile(OmsTypeInt4 fileHandle, const char*fmt, ... ) {
  char        buffer[256];
  tsp00_Int2  DBError;
  tsp00_Int4  BufLen;

  va_list args;
  va_start (args, fmt);
  sp77vsprintf ( buffer, sizeof(buffer), fmt, args );
  BufLen = (tsp00_Int4)strlen(buffer);
  m_lcSink->WriteFile (fileHandle, buffer, &DBError);
  va_end (args);
  if (0 != DBError) {
    throw DbpError(DbpError::DB_ERROR, DBError);
  }
}

/*----------------------------------------------------------------------*/

void DbpBase::dbpWriteMessageFileVarg(OmsTypeInt4 fileHandle, const char*fmt, va_list args ) {
  char        buffer[256];
  tsp00_Int2  DBError;
  tsp00_Int4  BufLen;

  sp77vsprintf ( buffer, sizeof(buffer), fmt, args );
  BufLen = (tsp00_Int4)strlen(buffer);
  m_lcSink->WriteFile (fileHandle, buffer, &DBError);
  if (0 != DBError) {
    throw DbpError(DbpError::DB_ERROR, DBError);
  }
}

/*===========================================================================*
*  END OF CODE                                                              *
*===========================================================================*/
