/****************************************************************************

  module      : vco31.cpp
  
  -------------------------------------------------------------------------
    
  responsible : ThomasA
      
  special area: liveCache Error Class
  description : Defines Functions of an error thrown by OMS and SQL class
        
  last changed: 2000-01-10  9:34
           
  -------------------------------------------------------------------------
            
  copyright:    Copyright by SAP AG, 1999
              


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


/*===========================================================================*
*  INCLUDES                                                                 *
*===========================================================================*/


#include "hco31-k.h"
#include "hsp81.h"
#include "geo573.h"

/*===========================================================================*
 *  METHODS                                                                  *
 *===========================================================================*/

static DbpErrorCallbackInterface *dbpErrorCallback = NULL;

/*----------------------------------------------------------------------*/

void dbpErrorCallbackFunc()
{
  // this causes call to callback in constructor
  DbpError tmp(-28000, "bad_alloc");
}

/*----------------------------------------------------------------------*/

DbpError::DbpError (DbpErrorKinds ErrorKind, long ErrorNo)
: m_errorKind(ErrorKind)
, m_isWyde   (false)
, m_errorNo  (ErrorNo)
, m_oid()
{
  m_errorText[0] = 0;
  if (dbpErrorCallback) 
    dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const char* errtext) 
: m_errorKind(ErrorKind)
, m_isWyde   (false)
, m_errorNo  (ErrorNo)
, m_oid()
{
  strcpy (m_errorText, errtext);
  if (dbpErrorCallback) 
    dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const DbpTypeUnicode* errtext) 
: m_errorKind(ErrorKind)
, m_isWyde   (true)
, m_errorNo  (ErrorNo)
, m_oid()
{
  sp81UCS2strcpy ((tsp81_UCS2Char*)(m_errorText), (tsp81_UCS2Char*)errtext);
  if (dbpErrorCallback) 
    dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const char* errtext, const OmsObjectId& oid) 
: m_errorKind(ErrorKind)
, m_isWyde (false)
, m_errorNo (ErrorNo)
, m_oid(oid)
{
  strcpy (m_errorText, errtext);
  if (dbpErrorCallback) 
    dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const DbpTypeUnicode* errtext, const OmsObjectId& oid)
: m_errorKind(ErrorKind)
, m_isWyde (true)
, m_errorNo (ErrorNo)
, m_oid(oid)
{
  sp81UCS2strcpy ((tsp81_UCS2Char*)(m_errorText), (tsp81_UCS2Char*)errtext);
  if (dbpErrorCallback) 
    dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (long ErrorNo)
: m_errorKind(USER_DEFINED)
, m_isWyde   (false)
, m_errorNo  (ErrorNo)
, m_oid() 
{
  m_errorText[0] = 0;
  if (dbpErrorCallback) 
    dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (long ErrorNo, const char* errtext) 
: m_errorKind(USER_DEFINED)
, m_isWyde   (false)
, m_errorNo  (ErrorNo)
, m_oid() 
{
  strcpy (m_errorText, errtext);
  if (dbpErrorCallback) 
    dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (long ErrorNo, const DbpTypeUnicode* errtext)
: m_errorKind(USER_DEFINED_UNICODE)
, m_isWyde   (true)
, m_errorNo  (ErrorNo)
, m_oid()  
{
  sp81UCS2strcpy ((tsp81_UCS2Char*)(m_errorText), (tsp81_UCS2Char*)errtext);
  if (dbpErrorCallback) 
    dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

void DbpError::dbpSetCallback(DbpErrorCallbackInterface *cb)
{
  if (cb) {
    dbpErrorCallback = cb;
    geo573_BadAllocHandler::setCallback(dbpErrorCallbackFunc);
  } else {
    dbpErrorCallback = NULL;
    geo573_BadAllocHandler::setCallback(NULL);
  }
}

/*----------------------------------------------------------------------*/

DbpErrorCallbackInterface *DbpError::dbpGetCallback()
{
  return dbpErrorCallback;
}

/*----------------------------------------------------------------------*/

long DbpError::dbpError () const { 
  return m_errorNo; 
};

/*----------------------------------------------------------------------*/

short DbpError::dbpDBError () const {
  return (m_errorKind == DB_ERROR) ? (short)m_errorNo : 0; 
};

/*----------------------------------------------------------------------*/

short DbpError::dbpRteError () const { 
  return (m_errorKind == RTE_ERROR) ? (short)m_errorNo : 0; 
};

/*----------------------------------------------------------------------*/

long  DbpError::dbpHresultError() const { 
  return (m_errorKind == HRESULT_ERROR) ? m_errorNo : 0; 
};

/*----------------------------------------------------------------------*/

const char* DbpError::dbpErrorText() const 
{
  if (m_isWyde)
  {
    throw DbpError(-9000, "dbpErrorText : not implemented : wyde to char");
  }
  return &m_errorText[0]; 
};

/*----------------------------------------------------------------------*/

const OmsTypeWyde* DbpError::dbpErrorTextWyde() const  
{ 
  if (!m_isWyde)
  {
    int length = (int) strlen(&m_errorText[0]);
    if (length >= sizeof(m_errorText) / 2)
    {
      length = sizeof(m_errorText) / 2;
      m_errorText[length-1] = 0;
    }
    int idx = length;
    OmsTypeWyde* pWyde = REINTERPRET_CAST(OmsTypeWyde*, &m_errorText[0]);
    while (idx >= 0)
    {
      pWyde[idx] = (unsigned char) m_errorText[idx];
      --idx;
    }
    m_isWyde = true;
  }
  return REINTERPRET_CAST(OmsTypeWyde*, &m_errorText[0]); 
};

/*----------------------------------------------------------------------*/

const DbpTypeUnicode* DbpError::dbpErrorTextUnicode() const {
  return (DbpTypeUnicode*)&m_errorText[0]; 
};

/*----------------------------------------------------------------------*/

const OmsObjectId& DbpError::dbpErrorOid() const { 
  return m_oid; 
};