/*!***************************************************************************

  module      : vco501SQL.cpp 

  -------------------------------------------------------------------------

  responsible : FerdiF

  special area: C++ SQL-Class   
  description : SQL - Application Interface for an SQL-Statement 
				
  
  last changed: 2000-03-08  11:22
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/

#include "geo00.h"

#ifndef GCO00_H
#include "gco00.h"
#endif
#ifndef GCO50SQLHANDLE_H
#include "gco50SqlHandle.h"
#endif
#ifndef GCO501SQL_H
#include "gco501SQL.h"
#endif
#ifndef GCO51CSQLSESSIONCONTEXT_H
#include "gco51CSqlSessionContext.h"
#endif
#ifndef GCO521CSQLSTMT_H
#include "gco521CSqlStmt.h"
#endif
#ifndef GCO52SQLCOL_H
#include "gco52SqlCol.h"
#endif

/*----------------------------------------------------------------------*/

SQL::SQL ( ) {
  m_sqlstatement = NULL;
};

/*----------------------------------------------------------------------*/

SQL::SQL (SqlHandle* handle, const char *stm) {
  m_sqlstatement = NULL;
  handle->m_session_context->createSqlStmt(*this, stm);
};

/*----------------------------------------------------------------------*/

SQL::SQL (SqlHandle* handle, const DbpTypeUnicode *stm) {
  m_sqlstatement = NULL;
  handle->m_session_context->createSqlStmt(*this, stm);
};

/*----------------------------------------------------------------------*/

SQL::~SQL () {
  if ( NULL != m_sqlstatement ) {
    m_sqlstatement->release();
    m_sqlstatement = NULL;
  }
}

/*----------------------------------------------------------------------*/

#if defined(EXT_CLIENT_SQL)
int SQL::connect (const char* hostname, const char* dbname, const char *user, const char *passwd, const char* isolevel) {
  if ( m_sqlstatement ) {
      return m_sqlstatement->connect(hostname, dbname, user, passwd, isolevel);
  }
  else
    return 0;
}
#endif

/*----------------------------------------------------------------------*/

SQL::SQL(const SQL& sql) {
  m_sqlstatement = sql.m_sqlstatement;
  if ( m_sqlstatement )
    m_sqlstatement->addRef();
}

/*----------------------------------------------------------------------*/

int SQL::sqlExecute () {
  if ( m_sqlstatement )
    return m_sqlstatement->execute();
  else
    return 0;
}

/*----------------------------------------------------------------------*/

short SQL::sqlNumCols () const {
  if ( m_sqlstatement )
    return m_sqlstatement->getNumCols();
  else
    return 0; 
}

/*----------------------------------------------------------------------*/

short SQL::sqlNumOutCols () const {
  if ( m_sqlstatement )
    return m_sqlstatement->getNumOutCols();
  else
    return 0; 
}

/*----------------------------------------------------------------------*/

int SQL::sqlPrepare () {
  if ( m_sqlstatement )
    return m_sqlstatement->prepare();
  else
    return 0;
}

/*----------------------------------------------------------------------*/

SqlCol &SQL::operator[] (int i) {
  if ( m_sqlstatement ) {
    return (*m_sqlstatement)[i];
  }
  else 
    return *new(SqlCol);
}

/*----------------------------------------------------------------------*/

int SQL::operator() () {
    return sqlExecute ();
}

/*----------------------------------------------------------------------*/

#if defined( SUN )
SQL &SQL::operator<< (const SqlCol p)
#else
SQL &SQL::operator<< (const SqlCol &p)
#endif
{
  if ( m_sqlstatement ) {
    m_sqlstatement->appendNewColDesc(p);
  }
  return *this;
}


/*----------------------------------------------------------------------*/

const SQL & SQL::operator= (const SQL& right) {
  if ( NULL != m_sqlstatement )
    m_sqlstatement->release();
  m_sqlstatement = right.m_sqlstatement;
  if ( NULL != m_sqlstatement )
    m_sqlstatement->addRef();
  return *this;
}

/*----------------------------------------------------------------------*/

void* SQL::operator new(size_t sz) {
#if defined (EXT_CLIENT_SQL)
  void *p = malloc(sz);
  if (!p)
    throw DbpError(e_new_failed,"new failed");
  return p;
#else
# if defined(OMSTST)
  co10_GetKernelInterface()->TestBadAlloc();
# endif
  SAPDBMem_RawAllocator *m_alloc;
  co10_GetLcSink()->GetSqlClsAllocator(m_alloc);
  return m_alloc->Allocate(sz);   
#endif
};

/*----------------------------------------------------------------------*/

void  SQL::operator delete (void* p) {
# if defined (EXT_CLIENT_SQL)
  free(p);
# else
  SAPDBMem_RawAllocator *m_alloc;
  co10_GetLcSink()->GetSqlClsAllocator(m_alloc);
  m_alloc->Deallocate(p);
# endif
}

/*----------------------------------------------------------------------*/

void* SQL::operator new[] (size_t sz) {
#if defined (EXT_CLIENT_SQL)
  void *p = malloc(sz);
  if (!p)
    throw DbpError(e_new_failed,"new failed");
  return p;
#else
# if defined(OMSTST)
  co10_GetKernelInterface()->TestBadAlloc();
# endif
  SAPDBMem_RawAllocator *m_alloc;
  co10_GetLcSink()->GetSqlClsAllocator(m_alloc);
  return m_alloc->Allocate(sz);   
#endif
};

/*----------------------------------------------------------------------*/

void  SQL::operator delete[] (void* p) {
# if defined (EXT_CLIENT_SQL)
  free(p);
# else
  SAPDBMem_RawAllocator *m_alloc;
  co10_GetLcSink()->GetSqlClsAllocator(m_alloc);
  m_alloc->Deallocate(p);
# endif
}
