/*!***************************************************************************

  module      : vco509Utils.cpp

  -------------------------------------------------------------------------

  responsible : FerdiF

  special area: C++ SQL-Class   
  description : helper functions 
				
  
  last changed: 2000-03-08  11:22
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/

#if !defined(EXT_CLIENT_SQL)
extern "C" {
  void  sqlabort(){int x=0; x=1/x;};
};
#endif

#include "gco00.h"
#include "geo00.h"
#include "hco509Utils.h"
#include "hsp81.h"

#if defined(EXT_CLIENT_SQL)
#include "hco31-k.h"

/*----------------------------------------------------------------------*/
/* Implementation of class DbpError                                     */
/*----------------------------------------------------------------------*/

static DbpErrorCallbackInterface *dbpErrorCallback = NULL;

DbpError::DbpError (DbpErrorKinds ErrorKind, long ErrorNo) {
  m_errorKind    = ErrorKind;
  m_errorNo      = ErrorNo;
  m_errorText[0] = 0;
  if (dbpErrorCallback) dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const char* errtext) {
  m_errorKind    = ErrorKind;
  m_errorNo      = ErrorNo;
  strcpy (m_errorText, errtext);
  if (dbpErrorCallback) dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const DbpTypeUnicode* errtext) {
  m_errorKind    = ErrorKind;
  m_errorNo      = ErrorNo;
  sp81UCS2strcpy ((tsp81_UCS2Char*)(m_errorText), (tsp81_UCS2Char*)errtext);
  if (dbpErrorCallback) dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const char* errtext, const OmsObjectId& oid){
  m_errorKind    = ErrorKind;
  m_errorNo      = ErrorNo;
  strcpy (m_errorText, errtext);
  if (dbpErrorCallback) dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (DbpErrorKinds ErrorKind, long ErrorNo, const DbpTypeUnicode* errtext, const OmsObjectId& oid) {
  m_errorKind    = ErrorKind;
  m_errorNo      = ErrorNo;
  m_oid          = oid;
  sp81UCS2strcpy ((tsp81_UCS2Char*)(m_errorText), (tsp81_UCS2Char*)errtext);
  if (dbpErrorCallback) dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (long ErrorNo) {
  m_errorKind   = USER_DEFINED;
  m_errorNo      = ErrorNo;
  m_errorText[0] = 0;
  if (dbpErrorCallback) dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (long ErrorNo, const char* errtext) {
  m_errorKind    = USER_DEFINED;
  m_errorNo      = ErrorNo;
  strcpy (m_errorText, errtext);
  if (dbpErrorCallback) dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

DbpError::DbpError (long ErrorNo, const DbpTypeUnicode* errtext) {
  m_errorKind    = USER_DEFINED_UNICODE;
  m_errorNo      = ErrorNo;
  sp81UCS2strcpy ((tsp81_UCS2Char*)(m_errorText), (tsp81_UCS2Char*)errtext);
  if (dbpErrorCallback) dbpErrorCallback->dbpCheckError(*this);
};

/*----------------------------------------------------------------------*/

void DbpError::dbpSetCallback(DbpErrorCallbackInterface *cb)
{
  dbpErrorCallback = cb;
}

/*----------------------------------------------------------------------*/

DbpErrorCallbackInterface *DbpError::dbpGetCallback()
{
  return dbpErrorCallback;
}

/*----------------------------------------------------------------------*/

long DbpError::dbpError () const { 
  return m_errorNo; 
};

/*----------------------------------------------------------------------*/

short DbpError::dbpDBError () const { 
  return (m_errorKind == DB_ERROR) ? (/*ia64*/short)m_errorNo : 0; 
};

/*----------------------------------------------------------------------*/

short DbpError::dbpRteError () const { 
  return (m_errorKind == RTE_ERROR) ? (/*ia64*/short)m_errorNo : 0; 
};

/*----------------------------------------------------------------------*/

long  DbpError::dbpHresultError() const { 
  return (m_errorKind == HRESULT_ERROR) ? m_errorNo : 0; 
};

/*----------------------------------------------------------------------*/

const char* DbpError::dbpErrorText() const { 
  return &m_errorText[0]; 
};

/*----------------------------------------------------------------------*/

const DbpTypeUnicode* DbpError::dbpErrorTextUnicode() const {
  return (DbpTypeUnicode*)&m_errorText[0]; 
};

#endif

/*----------------------------------------------------------------------*/

unsigned int dbpStrLen( const DbpTypeUnicode *s ) {
  return sp81UCS2strlen( (const tsp81_UCS2Char*) s );
};

/*----------------------------------------------------------------------*/

DbpTypeUnicode * dbpStrCat(DbpTypeUnicode *dest, 
                           const DbpTypeUnicode *src) {
  return (DbpTypeUnicode *) sp81UCS2strcat((tsp81_UCS2Char*)dest, (tsp81_UCS2Char*)src);
};

/*----------------------------------------------------------------------*/

DbpTypeUnicode * dbpStrnCat(DbpTypeUnicode *dest, 
                            const DbpTypeUnicode *src, 
                            unsigned int maxlen ) {
  return (DbpTypeUnicode *) sp81UCS2strncat((tsp81_UCS2Char*)dest, (tsp81_UCS2Char*)src, maxlen);
};

/*----------------------------------------------------------------------*/

DbpTypeUnicode * dbpStrChr( const DbpTypeUnicode *s, const DbpTypeUnicode c ) {
  tsp81_UCS2Char    UCS2char;
  UCS2char.s = c;
  return (DbpTypeUnicode *) sp81UCS2strchr((tsp81_UCS2Char*)s, UCS2char );
}

/*----------------------------------------------------------------------*/

DbpTypeUnicode * dbpStrnChr(const DbpTypeUnicode *s, 
                            const DbpTypeUnicode c, 
                            unsigned int maxlen ) {
  tsp81_UCS2Char    UCS2char;
  UCS2char.s = c;
  return (DbpTypeUnicode *) sp81UCS2strnchr((tsp81_UCS2Char*)s, UCS2char, maxlen );
}

/*----------------------------------------------------------------------*/

/* PTS 1111387 dbpStrCmp doesn't return pointer to string DbpTypeUnicode * */
int dbpStrCmp(DbpTypeUnicode *dest, 
              const DbpTypeUnicode *src) {
  return /*(DbpTypeUnicode *)*/ sp81UCS2strcmp((tsp81_UCS2Char*)dest, (tsp81_UCS2Char*)src);
};

/*----------------------------------------------------------------------*/

/* PTS 1111387 dbpStrnCmp doesn't return pointer to string DbpTypeUnicode * */
int dbpStrnCmp(DbpTypeUnicode *dest, 
               const DbpTypeUnicode *src, 
               unsigned int maxlen ) {
  return /*(DbpTypeUnicode *)*/ sp81UCS2strncmp((tsp81_UCS2Char*)dest, (tsp81_UCS2Char*)src, maxlen);
};

/*----------------------------------------------------------------------*/

DbpTypeUnicode * dbpStrCpy(DbpTypeUnicode *dest, 
                           const DbpTypeUnicode *src) {
  return (DbpTypeUnicode *) sp81UCS2strcpy((tsp81_UCS2Char*)dest, (tsp81_UCS2Char*)src);
};

/*----------------------------------------------------------------------*/

DbpTypeUnicode * dbpStrnCpy(DbpTypeUnicode *dest, 
                            const DbpTypeUnicode *src, 
                            unsigned int maxlen ) {
  return (DbpTypeUnicode *) sp81UCS2strncpy((tsp81_UCS2Char*)dest, (tsp81_UCS2Char*)src, maxlen);
};

/*----------------------------------------------------------------------*/

DbpTypeUnicode * dbpStrrChr(const DbpTypeUnicode *s, 
                            const DbpTypeUnicode c ) {
  tsp81_UCS2Char    UCS2char;
  UCS2char.s = c;
  return (DbpTypeUnicode *) sp81UCS2strrchr((tsp81_UCS2Char*)s, UCS2char );
}

/*----------------------------------------------------------------------*/

DbpTypeUnicode * dbpStrnrChr(const DbpTypeUnicode *s, 
                             const DbpTypeUnicode c, 
                             unsigned int maxlen ) {
  tsp81_UCS2Char    UCS2char;
  UCS2char.s = c;
  return (DbpTypeUnicode *) sp81UCS2strnrchr((tsp81_UCS2Char*)s, UCS2char, maxlen );
}

