/*!***************************************************************************

  module      : vco512CParseSQLReply.cpp

  -------------------------------------------------------------------------

  responsible : FerdiF

  special area: C++ SQL-Class   
  description : co512CParseSQLReply - Parse SQL Reply Parser 
				
  
  last changed: 2000-03-08  11:22
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/

#include "gco512CParseSQLReply.h"

#ifndef GCO51CSQLSESSIONCONTEXT_H
#include "gco51CSqlSessionContext.h"
#endif
#ifndef GCO521CSQLSTMT_H
#include "gco521CSqlStmt.h"
#endif

/*----------------------------------------------------------------------*/

co512CParseSQLReply::co512CParseSQLReply( co51CSqlSessionContext& sessionContext,
    co521CSqlStmt& stmt )
  : m_sessCtxt(sessionContext), m_stmt(stmt) {
};

/*----------------------------------------------------------------------*/

int co512CParseSQLReply::ParseReplyData( tsp1_packet* m_reply_packet ) {

  cin203_ReplyPacket rplyPacket( m_reply_packet );
  cin204_ReplyParser::AcceptKind  accepted;
  cin202_ReplySegment segment = rplyPacket.GetFirstSegment();
  accepted = unknown_e;
  while (segment.IsValid () && (accepted != abort_e)) {
    accepted = cin204_ReplyParser::ParseReply ( segment );
    segment = rplyPacket.GetNextSegment( segment );
  }
  return ( accepted != abort_e );
};

/*----------------------------------------------------------------------*/

cin204_ReplyParser::AcceptKind 
co512CParseSQLReply::HandleData ( int argCount, int dataLen, const teo00_Byte * dataPtr)
{
  m_stmt.getOutputParms ( argCount, dataLen, dataPtr );
# if !defined(EXT_CLIENT_SQL)           /* PTS 1120804 */
    m_sessCtxt.setDataLen(dataLen);     /* PTS 1120804 */
# endif                                 /* PTS 1120804 */
  return cin204_ReplyParser::handled_e;
};

/*----------------------------------------------------------------------*/

cin204_ReplyParser::AcceptKind 
co512CParseSQLReply::HandleLongdata ( int argCount, int dataLen, const tsp00_Byte * dataPtr)
{
  if ( dataLen == argCount * (int)(sizeof(tsp00_LongDescriptor) + 1) )
    m_stmt.getLongDesc ( argCount, dataLen, dataPtr );
  else
    m_stmt.getLongDescAndValue ( argCount, dataLen, dataPtr );
  return cin204_ReplyParser::handled_e;
};

/*----------------------------------------------------------------------*/

cin204_ReplyParser::AcceptKind 
co512CParseSQLReply::HandleErrortext ( int argCount, int len, const char * data) {
  m_sessCtxt.setErrortext ( argCount, len, data );
  return cin204_ReplyParser::handled_e;
};

/*----------------------------------------------------------------------*/

cin204_ReplyParser::AcceptKind 
co512CParseSQLReply::HandleParsid ( int argCount, int len, const tin02_ParseId * data) {
  m_stmt.setParsid( len, data );
  return cin204_ReplyParser::handled_e;
};

/*----------------------------------------------------------------------*/

cin204_ReplyParser::AcceptKind 
co512CParseSQLReply::HandleParsidOfSelect ( int argCount, int len, const tin02_ParseId * data) {
  m_stmt.setParsidOfSelect( len, data );
  return cin204_ReplyParser::handled_e;
};

/*----------------------------------------------------------------------*/

cin204_ReplyParser::AcceptKind 
co512CParseSQLReply::HandleResultcount ( int argCount, int len, long data ) {
  m_sessCtxt.setResultCount( data );
  return cin204_ReplyParser::handled_e;
}

/*----------------------------------------------------------------------*/

cin204_ReplyParser::AcceptKind 
co512CParseSQLReply::HandleResulttablename ( int argCount, int len, const char * data ) {
  m_stmt.setResultTablename( len, data );
  return cin204_ReplyParser::handled_e;
}

/*----------------------------------------------------------------------*/

cin204_ReplyParser::AcceptKind 
co512CParseSQLReply::HandleShortinfo ( int argCount, int len, const tsp1_param_info * data ) {
  m_stmt.setSqlDA_sqln( argCount );
  for ( int ix=0; ix < argCount; ix++ ) {
    m_stmt.setSqlDA_sfi( ix, data + ix );
  }
  return cin204_ReplyParser::handled_e;
}
