/*!***************************************************************************

  module      : vco51CSqlSessionContext.cpp

  -------------------------------------------------------------------------

  responsible : FerdiF

  special area: C++ SQL Class   
  description : Implementation of co51CSqlSessionContext 
				
  
  last changed: 2000-03-08  11:22
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/

#include "geo00.h"

#ifndef GCO00_H
#include "gco00.h"
#endif
#if !defined (EXT_CLIENT_SQL)
// hco10.h   is needed because of omsHeap functions
#include "hco10.h"
#include "liveCache/LVC_KernelInterface.hpp"
#endif
#ifndef GCO501SQL_H
#include "gco501SQL.h"
#endif
#ifndef GCO51CSQLSESSIONCONTEXT_H
#include "gco51CSqlSessionContext.h"
#endif
#ifndef GCO521CSQLSTMT_H
#include "gco521CSqlStmt.h"
#endif
#ifndef HIN204_H
#include "hin204.h"
#endif
#ifndef HIN205_H
#include "hin205.h"
#endif
#include "gco512CParseSQLReply.h"

#if defined (EXT_CLIENT_SQL)
#include "gsp01.h"
#include "heo02.h"
#include "heo03.h"
#include "hsp02.h"
#include <new.h>
#endif
#include "hsp81.h"

#if !defined (EXT_CLIENT_SQL)
const int C_MIN_REPLY_SIZE = 1024;
#endif

#include "hsp100.h"    /* PTS 1112481 */

const short C_SQL_PARSE_AGAIN = -8;   /* PTS 1112690 */
const pasbool C_PARSING_AGAIN = true; /* PTS 1112690 */

template<class T> class auto_ptr 
{
    public:
        explicit auto_ptr(T *p = 0)  { pointee = p; };
        auto_ptr<T>& operator=(T *p) { pointee = p; };
        T* release() 
        {
            T* p = pointee;
            pointee = NULL;
            return p;
        };
        ~auto_ptr() { 
            if (pointee)
              pointee->release(); 
        };
    private:
        T* pointee;
};

/*----------------------PTS 1112481 ------------------------------------*/
static void co51InitVersion ( cin203_RequestPacket &reqPacket ) {
    tsp00_C5c vers;                                           
    sp100_GetSenderIDVersion( vers );                     
#if defined (EXT_CLIENT_SQL)
    reqPacket.InitVersion("CPP", vers );                    
#else
    reqPacket.InitVersion("C++", vers );                    
#endif
};

/*----------------------------------------------------------------------*/

co51CSqlSessionContext::SessionHeap::SessionHeap()
{
#if !defined (EXT_CLIENT_SQL)
  co10_GetLcSink()->GetSqlClsAllocator(m_alloc);
#endif
}

/*----------------------------------------------------------------------*/

co51CSqlSessionContext::SessionHeap::~SessionHeap()
{
}

/*----------------------------------------------------------------------*/

co51CSqlSessionContext::co51CSqlSessionContext ( SqlHandle& pHandle ) :
m_heap(), 
m_CSqlStmtDir(&m_heap), m_sqlhandle(pHandle), m_request_packet_size(0),
m_request_packet(0), m_reply_packet(0), m_deleteSelf(false)
, m_sqlhandler(NULL) /* 1121294 */
/* PTS 1120804 */
, m_ref_count(0)
#if !defined(EXT_CLIENT_SQL)
, m_pSink(co10_GetLcSink())             /* PTS 1121948 */
, m_data_len(0)
, m_inMethodCallEpilog(false)
{
  SAPDBMem_RawAllocator *m_alloc;
  co10_GetLcSink()->GetSqlClsAllocator(m_alloc);
  m_curr_method = (tsp00_KnlIdentifier*)m_alloc->Allocate(sizeof(tsp00_KnlIdentifier));
  m_sqlhandle.m_session = &m_pSink;     /* PTS 1121948 */
  short error = 0;                                          /* PTS 1121341 */
  m_pSink->IsUnicodeInstance( m_isUnicodeInstance, error ); /* PTS 1121341 */
#else
{
  m_sqlhandle.m_session = NULL;         /* PTS 1121948 */
#endif
  m_sqlhandle.m_session_context = this; /* PTS 1121948 */
  m_sqlmode.becomes(sp1sm_adabas);
};

/*----------------------------------------------------------------------*/

void co51CSqlSessionContext::sqlSetMode(const SqlOpt::SqlSessionMode mode) 
{
  switch ( mode ) 
  {
  case SqlOpt::SQLMODE_NIL:
    m_sqlmode.becomes(sp1sm_nil); break;
  case SqlOpt::SQLMODE_SESSION:
    m_sqlmode.becomes(sp1sm_session_sqlmode); break;
  case SqlOpt::SQLMODE_ADABAS:
    m_sqlmode.becomes(sp1sm_adabas); break;
  case SqlOpt::SQLMODE_ANSI:
    m_sqlmode.becomes(sp1sm_ansi); break;
  case SqlOpt::SQLMODE_DB2:
    m_sqlmode.becomes(sp1sm_db2); break;
  case SqlOpt::SQLMODE_ORACLE:
    m_sqlmode.becomes(sp1sm_oracle); break;
  }
};

/*----------------------------------------------------------------------*/

void co51CSqlSessionContext::deleteSelf() {
/* PTS 1120804 */
# if !defined (EXT_CLIENT_SQL)
    decRefCnt();
# endif
  if (m_ref_count == 0) {
    m_deleteSelf = true;
    if ( m_CSqlStmtDir.empty() ) {
      m_heap.~SessionHeap(); 
#     if defined (EXT_CLIENT_SQL)
        free(this);
#     else
        SAPDBMem_RawAllocator *m_alloc;
        co10_GetLcSink()->GetSqlClsAllocator(m_alloc);
        m_alloc->Deallocate(m_curr_method);
        m_alloc->Deallocate(this);
        co10_GetLcSink()->SetSqlSessionContext(NULL);
#     endif
    }
  }
};

/*----------------------------------------------------------------------*/

void* co51CSqlSessionContext::operator new(size_t sz) {
#if defined (EXT_CLIENT_SQL)
  void *p = malloc(sz);
  if (!p)
    throw DbpError(e_new_failed,"new failed");
  return p;
#else
# if defined(OMSTST)
  co10_GetKernelInterface()->TestBadAlloc();
# endif
  SAPDBMem_RawAllocator *m_alloc;
  co10_GetLcSink()->GetSqlClsAllocator(m_alloc);
  return m_alloc->Allocate(sz);
#endif
};

/*----------------------------------------------------------------------*/

void  co51CSqlSessionContext::operator delete (void* p) {
};

#if defined (EXT_CLIENT_SQL)

/*----------------------------------------------------------------------*/

int	co51CSqlSessionContext::connect(co521CSqlStmt* pSqlStmt, const char* hostname, 
                                    const char* dbname, const char* user, 
                                    const char* pwd, const char* isolevel) {
  cin201_Part * part;
  char            l_stm[256];
  int             rc = 0;

  m_sqlca.sqlCAInit();
  
  tsp00_ErrTextc errtext;
  tsp01_CommErr_Enum commErr =
    SqlAConnect (CONST_CAST(char*,hostname),          
    REFCAST_MEO00 (tsp00_DbNamec) dbname,                   
    srvUser_esp01, 1, &m_connection,
    &m_request_packet_size, (void**)&m_request_packet, errtext);
  if ( commErr ) {
    switch ( commErr ) {
    case commErrShutdown_esp01:
      setRtError(e_dbshutdown, dbname, hostname, NULL );
      break;
    case commErrTasklimit_esp01:
      setRtError(e_too_many_users, dbname, hostname, NULL );
      break;
    case commErrNotOk_esp01:
    case commErrTimeout_esp01:
    case commErrCrash_esp01:
    case commErrStartRequired_esp01:
    case commErrSendLineDown_esp01:
    case commErrReceiveLineDown_esp01:
    case commErrPacketLimit_esp01:
    case commErrReleased_esp01:
    case commErrWouldBlock_esp01:
    case commErrUnknownRequest_esp01:
    case commErrServerOrDBUnknown_esp01:
    default:
      setRtError(e_dbconnect_notok, dbname, hostname, NULL );
    }
    m_connection          = -1;
    m_request_packet_size = 0;
    m_sqlhandle.m_session = NULL;
    return 0;
    
  }
  m_sqlhandle.m_session  = (IliveCacheSink**)&m_connection;

  IliveCacheSink* pILCSink = initSession();
  if ( NULL != pILCSink ) {  
    cin203_RequestPacket reqPacket( m_request_packet, m_request_packet_size,
      pSqlStmt->isUnicode());
    co51InitVersion( reqPacket );                         /* PTS 1112481 */
    cin205_RequestWriter reqWriter( reqPacket );
    
    reqWriter.Reset();
    
    /*
    * connect statement
    */
    strcpy (l_stm, "connect \"");
    strcat (l_stm, user);
    strcat (l_stm, "\" identified by ? sqlmode adabas isolation level ");
    strcat (l_stm, isolevel);
    reqWriter.InitDbs (m_sqlmode, l_stm);  /* SQL-Mode PTS 1108446 */
    
    /*
    * convert pwd
    */
    tsp00_Name      pasString;
    tsp00_CryptPw   stCrypt;
    s02applencrypt(pasString.c2p(pwd), stCrypt);
    part = reqWriter.AddDataPart();
    part->AddParameterArg (stCrypt, 1, sizeof(stCrypt), 50, 'x');
    
    tsp00_TermId termid;  
    // sqltermid (termid); later: use module var initialized by environment
    memcpy (termid, "<unknown><unknown>", sizeof (termid));
    part->AddParameterArg (termid, 51, sizeof(termid), sizeof(termid) + 1, ' ');
    
    reqWriter.Close();
    rc = executeDBRequest( pILCSink, reqPacket );
    if ( rc ) {
      co512CParseSQLReply rplyParser( *this, *pSqlStmt );
      rc = rplyParser.ParseReplyData( m_reply_packet );  
    };
  }

  if ( (m_sqlhandler) && (m_sqlca.sqlcode != 0) )     /* PTS 1120804 */
    m_sqlhandler(&m_sqlhandle);                       /* PTS 1120804 */

  return ( m_sqlca.sqlcode == 0 );
};
#endif

/*----------------------------------------------------------------------*/

void co51CSqlSessionContext::createSqlStmt(SQL& sql, const char *stmt) {
  sql.m_sqlstatement = new co521CSqlStmt(this, stmt);
  auto_ptr<co521CSqlStmt>  autoCSqlStmt(sql.m_sqlstatement);
  m_CSqlStmtDir.push_front(sql.m_sqlstatement);
  autoCSqlStmt.release();
};

/*----------------------------------------------------------------------*/

void co51CSqlSessionContext::createSqlStmt(SQL& sql, const DbpTypeUnicode *stmt) {
  sql.m_sqlstatement = new co521CSqlStmt(this, stmt);
  auto_ptr<co521CSqlStmt>  autoCSqlStmt(sql.m_sqlstatement);
  m_CSqlStmtDir.push_front(sql.m_sqlstatement);
  autoCSqlStmt.release();
};

/*----------------------------------------------------------------------*/

void co51CSqlSessionContext::dropSqlStmt(co521CSqlStmt* pSqlStmt) {
  for ( cgg251dclIterator<co521CSqlStmt*,SessionHeap> iter = m_CSqlStmtDir.begin(); iter; ++iter) {
    if ( pSqlStmt == *iter() ) {
      m_CSqlStmtDir.remove(iter);
      break;
    }
  }  
  if ( pSqlStmt->getPreparedFlag() ) {
    dropParsid(pSqlStmt);
  }
  m_heap.deallocate(pSqlStmt);
  pSqlStmt = NULL;

  if ( m_deleteSelf ) 
    deleteSelf();
};

/*----------------------------------------------------------------------*/

IliveCacheSink* co51CSqlSessionContext::initSession(const unsigned char reqType) {
  m_sqlca.sqlCAInit(reqType);
  IliveCacheSink* pILCSink = NULL;
  
  if ( m_sqlhandle.m_session )
    pILCSink =  *m_sqlhandle.m_session;
  if ( NULL == pILCSink ) {
    setRtError( e_dbconnection_down, NULL, NULL, NULL ); 
    return NULL;
  }
#if !defined (EXT_CLIENT_SQL)
  if ( NULL == m_request_packet ) {
    pILCSink->AllocPacket( (unsigned char**)&m_request_packet, &m_request_packet_size);
    if (( NULL == m_request_packet ) || ( 0 == m_request_packet_size )) {
      setRtError( e_dbconnection_down, NULL, NULL, NULL );
      return NULL;
    }
    m_request_packet_size = m_request_packet_size - C_MIN_REPLY_SIZE; 
  }
#endif
  return pILCSink;
}

/*----------------------------------------------------------------------*/

int co51CSqlSessionContext::executeImmediateSQL(IliveCacheSink* pILCSink,
                                                co521CSqlStmt* pSqlStmt ) {
  int rc = 0;

  cin203_RequestPacket reqPacket(m_request_packet, m_request_packet_size, 
    pSqlStmt->isUnicode());
  co51InitVersion( reqPacket );                         /* PTS 1112481 */
  cin205_RequestWriter reqWriter( reqPacket );
  
  reqWriter.Reset();
  if ( pSqlStmt->isUnicode() )
  {
    /* PTS 1117853 FF 2002-Sep-16 */    
    if (NULL == reqWriter.InitDbs (m_sqlmode, pSqlStmt->getUniStatement())) /* SQL-Mode PTS 1108446 */     
    {
      setRtError(e_cmd_too_long,NULL,NULL,NULL);
      return rc;
    }
  }
  else
  {
    /* PTS 1117853 FF 2002-Sep-16 */    
    if (NULL == reqWriter.InitDbs (m_sqlmode, pSqlStmt->getStatement()))  /* SQL-Mode PTS 1108446 */
    {
      setRtError(e_cmd_too_long,NULL,NULL,NULL);
      return rc;
    }
  }
  reqWriter.Close();
  pSqlStmt->setResetParms();                  /* PTS 1109560 */
  rc = executeDBRequest (pILCSink, reqPacket);
  if ( rc ) {
     co512CParseSQLReply rplyParser( *this, *pSqlStmt );
     rc = rplyParser.ParseReplyData( m_reply_packet );  
     /* PTS 1120804 */
#    if !defined (EXT_CLIENT_SQL)
       monitorRequest(m_reply_packet->sp1_segm().sp1r_function_code());
#    endif
  };
  return rc;  
}

/*----------------------------------------------------------------------*/

int co51CSqlSessionContext::executePreparedSQL(IliveCacheSink* pILCSink,
                                                co521CSqlStmt* pSqlStmt ) {
  int rc = 1;
  cin201_Part * part;
  const tin02_ParseId * parseId;
  int len = pSqlStmt->getParsid( &parseId );
  
  cin203_RequestPacket reqPacket( m_request_packet, m_request_packet_size,
    pSqlStmt->isUnicode());
  co51InitVersion( reqPacket );                         /* PTS 1112481 */
  cin205_RequestWriter reqWriter( reqPacket );
  
  reqWriter.Reset();
  reqWriter.InitExecute (m_sqlmode, parseId ); /* SQL-Mode PTS 1108446 */
  if ( pSqlStmt->hasInputParms()) {
    part = reqWriter.AddDataPart();
    rc = pSqlStmt->addInputParms( part );
    if ( !rc ) {
      return rc;
    }
#if !defined(EXT_CLIENT_SQL)           /* PTS 1120804 */
    else {                             /* PTS 1120804 */
      m_data_len = part->Length();     /* PTS 1120804 */
    }                                  /* PTS 1120804 */
#endif                                 /* PTS 1120804 */
  }
  reqWriter.Close();
  pSqlStmt->setResetParms();                  /* PTS 1109560 */
  rc = executeDBRequest( pILCSink, reqPacket );
  if ( rc ) {
    co512CParseSQLReply rplyParser( *this, *pSqlStmt );
    rc = rplyParser.ParseReplyData( m_reply_packet );  
    /* PTS 1120804 */
#   if !defined (EXT_CLIENT_SQL)
      monitorRequest(m_reply_packet->sp1_segm().sp1r_function_code());
#   endif
  };
  if ( 0 == m_sqlca.sqlcode ) {
    if ( pSqlStmt->hasLongInput() && rc ) {
      rc = executeLongInput( pILCSink, reqPacket, pSqlStmt );
    }
    if ( pSqlStmt->hasLongOutput() && rc ) {
      rc = executeLongOutput( pILCSink, reqPacket, pSqlStmt );
    }
  }
  return rc;  
}

/*----------------------------------------------------------------------*/

int co51CSqlSessionContext::executeLongInput(IliveCacheSink* pILCSink,
                                             cin203_RequestPacket &reqPacket,
                                             co521CSqlStmt* pSqlStmt ) {
  int rc = 1;
  cin205_RequestWriter reqWriter( reqPacket );
  do { 
    reqWriter.Reset();
    reqWriter.AddSegment(sp1m_putval, m_sqlmode); /* SQL-Mode PTS 1108446 */
    cin201_Part *part = reqWriter.AddPart (sp1pk_longdata);
    rc = pSqlStmt->addOpenLongDataInput( part );
    reqWriter.Close();
    rc = executeDBRequest( pILCSink, reqPacket );
    if ( rc ) {
      co512CParseSQLReply rplyParser( *this, *pSqlStmt );
      rc = rplyParser.ParseReplyData( m_reply_packet ); 
      /* PTS 1120804 */
#     if !defined (EXT_CLIENT_SQL)
        monitorRequest(m_reply_packet->sp1_segm().sp1r_function_code());
#     endif
    };
  } while ( pSqlStmt->hasLongInput() && rc );

  reqWriter.Reset();
  reqWriter.AddSegment(sp1m_putval, m_sqlmode); /* SQL-Mode PTS 1108446 */
  cin201_Part *part = reqWriter.AddPart (sp1pk_longdata);
  rc = pSqlStmt->addInputFinishLongData( part );
  reqWriter.Close();
  rc = executeDBRequest( pILCSink, reqPacket );
  if ( rc ) {
    co512CParseSQLReply rplyParser( *this, *pSqlStmt );
    rc = rplyParser.ParseReplyData( m_reply_packet );  
    /* PTS 1120804 */
#   if !defined (EXT_CLIENT_SQL)
      monitorRequest(m_reply_packet->sp1_segm().sp1r_function_code());
#   endif
  };

  return rc;
}

/*----------------------------------------------------------------------*/

int co51CSqlSessionContext::executeLongOutput(IliveCacheSink* pILCSink,
                                              cin203_RequestPacket &reqPacket,
                                              co521CSqlStmt* pSqlStmt ) {
  int rc = 1;
  cin205_RequestWriter reqWriter( reqPacket );
  do { 
    reqWriter.Reset();
    reqWriter.AddSegment(sp1m_getval, m_sqlmode); /* SQL-Mode PTS 1108446 */
    cin201_Part *part = reqWriter.AddPart (sp1pk_longdata);
    rc = pSqlStmt->addOpenLongDataOutput( part );
    reqWriter.Close();
    rc = executeDBRequest( pILCSink, reqPacket );
    if ( rc ) {
      co512CParseSQLReply rplyParser( *this, *pSqlStmt );
      rc = rplyParser.ParseReplyData( m_reply_packet );  
      /* PTS 1120804 */
#     if !defined (EXT_CLIENT_SQL)
        monitorRequest(m_reply_packet->sp1_segm().sp1r_function_code());
#     endif
    };
  } while ( pSqlStmt->hasLongOutput() && rc );

  return rc;
}

/*----------------------------------------------------------------------*/

int co51CSqlSessionContext::executeDBRequest(IliveCacheSink* pILCSink,
                                             cin203_RequestPacket &reqPacket) {
  unsigned char	warnset[2];
  
#if defined (EXT_CLIENT_SQL)
  tsp01_CommErr      commErr;
  tsp00_Int4         outLen;
  tsp00_ErrText      errtext;
  sqlarequest (m_connection, reqPacket.GetRawPacket(), reqPacket.Length(), errtext, &commErr);
  if ( commErr ) {
    setRtError(e_notok, (char*)"sqlarequest", NULL, NULL );
    return 0;
  }    

  sqlareceive (m_connection, REINTERPRET_CAST (void**, &m_reply_packet), &outLen, errtext, &commErr);
  if ( commErr ) {
    setRtError(e_notok, (char*)"sqlareceive", NULL, NULL );
    return 0;
  }
#else
  short  error;
  m_request_packet->sp1_header.sp1h_varpart_size = m_request_packet->sp1_header.sp1h_varpart_size
    + C_MIN_REPLY_SIZE;
  pILCSink->SQLRequest( reqPacket.Length(), 
    (unsigned char*)reqPacket.GetRawPacket(), 
    &error );
  m_request_packet->sp1_header.sp1h_varpart_size = m_request_packet->sp1_header.sp1h_varpart_size
    - C_MIN_REPLY_SIZE;
  pILCSink->SQLReceive((unsigned char**)&m_reply_packet);
#endif
  m_sqlca.sqlcode   = m_reply_packet->sp1_segm().sp1r_returncode();
  m_sqlca.sqlerrpos = m_reply_packet->sp1_segm().sp1r_errorpos();
  memcpy( m_sqlca.sqlstate,&m_reply_packet->sp1_segm().sp1r_sqlstate(),sizeof(tsp_sqlstate) );
  memcpy(&warnset,
    &m_reply_packet->sp1_segm().sp1s_segm_header().sp1r_extern_warning(),
    sizeof(warnset));
  if (warnset[0] != 0) {
    short i;
    char warnb = 0x01;
    
    for (i = 0; i < 8; warnset[0]=warnset[0]>>1, i++) {
      if (warnset[0] & warnb) {
        m_sqlca.sqlwarn [i] = 'W';
      }
      else {
        m_sqlca.sqlwarn [i] = ' ';
      }
    }
    for (i = 8; i < 16; warnset[1]=warnset[1]>>1, i++) {
      if (warnset[1] & warnb) {
        m_sqlca.sqlwarn [i] = 'W';
      }
      else {
        m_sqlca.sqlwarn [i] = ' ';
      }
    }
  }
  return 1;  
}

/*----------------------------------------------------------------------*/

int co51CSqlSessionContext::executeSqlStmt( co521CSqlStmt* pSqlStmt ) 
{    
    IliveCacheSink* pILCSink = initSession(pSqlStmt->isUnicode());
    if (NULL != pILCSink) 
    {
        if ( true /* PTS 1111883 pSqlStmt->getNumVars() || pSqlStmt->getPreparedFlag()*/ ) 
        { 
            int rc = 1;
            if (!pSqlStmt->getPreparedFlag()) 
            {
                // PTS 1111883 rc = prepareSqlStmt( pSqlStmt );
                rc = pSqlStmt->prepare();  // PTS 1111883
            }
            /* 'prepared sql statement' -> request buffer  */
            if ((rc) 
                && (csp1_p_command_executed != pSqlStmt->getParsidInfo1())) // PTS 1116114 FF 2002-06-10
                executePreparedSQL( pILCSink, pSqlStmt );
        }
        else 
        {
            /* 'immediate sql statement' -> request buffer */
            executeImmediateSQL( pILCSink, pSqlStmt );
        }
        
        /* +++ **** PTS 1112690 **** +++ */
        if (C_SQL_PARSE_AGAIN == m_sqlca.sqlcode) 
        {
            m_sqlca.sqlCAInit( pSqlStmt->isUnicode() );
            if ( ( NULL != pILCSink ) && pSqlStmt->prepare( C_PARSING_AGAIN ) )
                executePreparedSQL( pILCSink, pSqlStmt );
        }
        /* --- **** PTS 1112690 **** --- */
    }
    if ((m_sqlhandler) && (m_sqlca.sqlcode != 0))     /* PTS 1120804 */
        m_sqlhandler(&m_sqlhandle);                   /* PTS 1120804 */
    return ( m_sqlca.sqlcode == 0 );
};

/*----------------------------------------------------------------------*/

/* PTS 1112690 parsingAgain added */
int co51CSqlSessionContext::prepareSqlStmt(co521CSqlStmt* pSqlStmt,
                                           const pasbool parsingAgain ) {
  bool		do_parsing = true;					      /* PTS 1112690  */
  pasbool	l_parsingAgain = parsingAgain;		/* PTS 1112690  */
  
  IliveCacheSink* pILCSink = initSession(pSqlStmt->isUnicode());
  if ( NULL != pILCSink ) {
    /* PTS 1112690 with 7.4.2.00 the parseid is dropped by the kernel   * 
     *                                              when -8 is signaled */
    if ( pSqlStmt->getPreparedFlag() && !parsingAgain ) {
      dropParsid(pSqlStmt);
    }
    
    if ( !pSqlStmt->dropLongDescriptors()) return 0;
    
    while ( do_parsing ) {         /* PTS 1112690  */
      cin203_RequestPacket reqPacket( m_request_packet, m_request_packet_size,
        pSqlStmt->isUnicode() );
      co51InitVersion( reqPacket );                         /* PTS 1112481 */
      cin205_RequestWriter reqWriter( reqPacket );
      
      if ( ECO521_FETCH_DESCRIBE == pSqlStmt->getStmtType() ) {
        reqWriter.Reset();
        /* SQL-Mode PTS 1108446 & PTS 1112690 parsingAgain added */
        reqWriter.InitDbs (m_sqlmode, "DESCRIBE", -1, l_parsingAgain); 
        reqWriter.Close();
        if ( executeDBRequest( pILCSink, reqPacket ) ) {
          /* PTS 1120804 */
#         if !defined (EXT_CLIENT_SQL)
            m_monitor.IncPrepare();
#         endif
          co512CParseSQLReply rplyParser( *this, *pSqlStmt );
          rplyParser.ParseReplyData( m_reply_packet );  
        };
        if ( !pSqlStmt->dropLongDescriptors()) return 0;
      }
      
      reqWriter.Reset();
      /* SQL-Mode PTS 1108446 & PTS 1112690 parsingAgain added */
      if ( pSqlStmt->isUnicode() )
      {
        /* PTS 1117853 FF 2002-Sep-16 */
        if (NULL == reqWriter.InitParse (m_sqlmode, pSqlStmt->getUniStatement(), -1, l_parsingAgain)) 
        {
            setRtError(e_cmd_too_long,NULL,NULL,NULL);
            return 0;
        }
      }
      else
      {
        /* PTS 1117853 FF 2002-Sep-16 */
        if (NULL == reqWriter.InitParse (m_sqlmode, pSqlStmt->getStatement(), -1, l_parsingAgain)) 
        {
            setRtError(e_cmd_too_long,NULL,NULL,NULL);
            return 0;
        }
      }
      reqWriter.Close();
      
      pSqlStmt->setResetParms();                  /* PTS 1109560 */
      if ( executeDBRequest( pILCSink, reqPacket ) ) {
        /* PTS 1120804 */
#       if !defined (EXT_CLIENT_SQL)
          m_monitor.IncPrepare();
#       endif
        co512CParseSQLReply rplyParser( *this, *pSqlStmt );
        rplyParser.ParseReplyData( m_reply_packet );  
      };
      pSqlStmt->createLongDescriptors();
      /* +++ **** PTS 1112690 **** +++ */
      if ( C_SQL_PARSE_AGAIN == m_sqlca.sqlcode && !l_parsingAgain ) {
        l_parsingAgain = true;
      }
      else 
        do_parsing = false;
      /* --- **** PTS 1112690 **** --- */
    }
  }
  
  if ( (m_sqlhandler) && (m_sqlca.sqlcode != 0) )     /* PTS 1120804 */
    m_sqlhandler(&m_sqlhandle);                       /* PTS 1120804 */
  return ( m_sqlca.sqlcode == 0 );
  
};

/*----------------------------------------------------------------------*/

void* co51CSqlSessionContext::allocate(size_t sz) 
{
    return m_heap.allocate(sz);
};

/*----------------------------------------------------------------------*/

void  co51CSqlSessionContext::deallocate (void* p) {
  m_heap.deallocate(p);
};

/*----------------------------------------------------------------------*/

int  co51CSqlSessionContext::dropParsid( co521CSqlStmt* pSqlStmt ) 
{
    /* drop parseid MUST NOT destroy previous error */
    structSqlCa ca_save = m_sqlca;   // PTS 1111883 
    int rc = 0;
    if (pSqlStmt->getPreparedFlag() 
        && (!m_deleteSelf) 
        && (csp1_p_command_executed != pSqlStmt->getParsidInfo1())) // PTS 1116114 FF 2002-06-10) 
    {        
        IliveCacheSink* pILCSink =  *m_sqlhandle.m_session;
        if (NULL == pILCSink) 
        {
            m_sqlca = ca_save;          // PTS 1111883
            return 0; 
        }
        
        cin203_RequestPacket reqPacket( m_request_packet, m_request_packet_size,
            pSqlStmt->isUnicode());
        co51InitVersion( reqPacket );                         /* PTS 1112481 */
        cin205_RequestWriter reqWriter( reqPacket );
        
        reqWriter.Reset();
        reqWriter.InitDbs (m_sqlmode, "DROP PARSEID"); /* SQL-Mode PTS 1108446 */
        cin201_Part* part = reqWriter.AddPart (sp1pk_parsid);
        const tin02_ParseId * data;
        int len = pSqlStmt->getParsid( &data );
        part->AddArgument (REINTERPRET_CAST(const void *,data), len);  
        reqWriter.Close();
        rc = executeDBRequest( pILCSink, reqPacket );
        if ( rc ) {
          /* PTS 1120804 */
#           if !defined (EXT_CLIENT_SQL)
              m_monitor.IncDropParseid();
#           endif
            co512CParseSQLReply rplyParser( *this, *pSqlStmt );
            rc = rplyParser.ParseReplyData( m_reply_packet );  
        };
    }
    m_sqlca = ca_save;            // PTS 1111883
    return rc;  
};

/*----------------------------------------------------------------------*/

void  co51CSqlSessionContext::setRtError(tgg00_BasisError e, 
                                         const char* p1, const char* p2, const char* p3 ) {
  const char * msg = getErrorDesc(e);
  m_sqlca.sqlcode = e;
  m_sqlca.sqlerrml  = (short)strlen( msg );
  memcpy( m_sqlca.sqlerrmc, msg, m_sqlca.sqlerrml );

  if ( p1 ) {
    if (( m_sqlca.sqlerrml + 1 + strlen(p1)) < sizeof(m_sqlca.sqlerrmc)) { 
      strcat( m_sqlca.sqlerrmc, "\255" );
      strcat( m_sqlca.sqlerrmc, p1 );
      m_sqlca.sqlerrml = m_sqlca.sqlerrml + 1 + (short)strlen(p1);
    }
  }
  
  if ( p2 ) {
    if (( m_sqlca.sqlerrml + 1 + (short)strlen(p2)) < sizeof(m_sqlca.sqlerrmc)) { 
      strcat( m_sqlca.sqlerrmc, "\255" );
      strcat( m_sqlca.sqlerrmc, p2 );
      m_sqlca.sqlerrml = m_sqlca.sqlerrml + 1 + (short)strlen(p1);
    }
  }

  if ( p3 ) {
    if (( m_sqlca.sqlerrml + 1 + (short)strlen(p3)) < sizeof(m_sqlca.sqlerrmc)) { 
      strcat( m_sqlca.sqlerrmc, "\255" );
      strcat( m_sqlca.sqlerrmc, p3 );
      m_sqlca.sqlerrml = m_sqlca.sqlerrml + 1 + (short)strlen(p1);
    }
  }
};

/*----------------------------------------------------------------------*/

void  co51CSqlSessionContext::setError(tgg00_BasisError e, const char* msg ) {
  m_sqlca.sqlcode = e;
  m_sqlca.sqlerrml  = (short)strlen( msg );
  memcpy( m_sqlca.sqlerrmc, msg, m_sqlca.sqlerrml );
};

/*----------------------------------------------------------------------*/

void  co51CSqlSessionContext::setErrortext (int argCount, 
                                            int len, 
                                            const char * data ) {
  unsigned int converted;
  int          UCS2Swap;
  tsp81_UCS2Char UCS2Blank;
  UCS2Blank.s = 32;
  UCS2Swap = (UCS2Blank.c[0] ? 1 : 0 ); 

  if ( m_sqlca.sqlerrmt ) {
    if ( sizeof(m_sqlca.sqlerrmc) <= len ) 
      len = sizeof(m_sqlca.sqlerrmc) - 2;
    memcpy( m_sqlca.sqlerrmuc, data, len );
    m_sqlca.sqlerrml  = len/2;
    sp81UCS2toASCII((tsp00_Byte*)(m_sqlca.sqlerrmc), sizeof(m_sqlca.sqlerrmc)-1, 
      &converted, (tsp81_UCS2Char*)data, len/2, UCS2Swap );
  }
  else {
    if ( sizeof(m_sqlca.sqlerrmc) <= len ) 
      len = sizeof(m_sqlca.sqlerrmc) - 1;
    m_sqlca.sqlerrml  = len;
    memcpy( m_sqlca.sqlerrmc, data, len );
    sp81ASCIItoUCS2((tsp81_UCS2Char*)(m_sqlca.sqlerrmuc), 
      len , UCS2Swap, &converted, (const unsigned char*)data, len );
  }
};

/*----------------------------------------------------------------------*/

void  co51CSqlSessionContext::setResultCount ( int resultCnt ) {
  m_sqlca.sqlrescount = resultCnt;
}

/*----------------------------------------------------------------------*/

void  co51CSqlSessionContext::setWarnTrunc ( ) {
  m_sqlca.sqlwarn[0] = 'W';
  m_sqlca.sqlwarn[1] = 'W';
}

/*----------------------------------------------------------------------*/

short  co51CSqlSessionContext::sqlCode () const {
  return m_sqlca.sqlcode;
};

/*----------------------------------------------------------------------*/

const char* co51CSqlSessionContext::sqlErrorMsg() const {
  return &m_sqlca.sqlerrmc[0];
};

/*----------------------------------------------------------------------*/

int co51CSqlSessionContext::sqlIsUnicodeErrMsg() const {
  return m_sqlca.sqlerrmt;
};

/*----------------------------------------------------------------------*/

const DbpTypeUnicode* co51CSqlSessionContext::sqlUnicodeErrorMsg() const {
  return &m_sqlca.sqlerrmuc[0];
};

/*----------------------------------------------------------------------*/

const char*	co51CSqlSessionContext::sqlState() const {
  return &m_sqlca.sqlstate[0];
};       

/*----------------------------------------------------------------------*/

char  co51CSqlSessionContext::sqlWarning (int i) const {
  return m_sqlca.sqlwarn[i];
};

/*----------------------------------------------------------------------*/

long  co51CSqlSessionContext::sqlErrorPos () const {
  return m_sqlca.sqlerrpos;
};       

/*----------------------------------------------------------------------*/

long  co51CSqlSessionContext::sqlResultCount () const {
  return m_sqlca.sqlrescount;
};    

/*----- PTS 1120804 ----------------------------------------------------*/

void  co51CSqlSessionContext::setHandler(sqlHandler p) {
  m_sqlhandler = p;
};

/*----- PTS 1120804 ----------------------------------------------------*/

sqlHandler co51CSqlSessionContext::getHandler() {
  return m_sqlhandler;
};

# if !defined (EXT_CLIENT_SQL)

/*----- PTS 1121948 ----------------------------------------------------*/

void co51CSqlSessionContext::setSink(IliveCacheSink * pSink) {
  m_pSink = pSink;
}

/*----- PTS 1121341 ----------------------------------------------------*/

bool co51CSqlSessionContext::isUnicodeInstance() {
  return m_isUnicodeInstance;
}

/*----- PTS 1120804 ----------------------------------------------------*/

void  co51CSqlSessionContext::monitorRequest(int fc) {
  switch (fc) {
    case csp1_insert_fc: 
      m_monitor.IncInsert();
      if (m_data_len > 0) {
        m_monitor.AddInsertDataLen(m_data_len);
        m_data_len = 0;
      }
      break;
    case csp1_select_fc:
      m_monitor.IncSelect();
      break;
    case csp1_update_fc:
      m_monitor.IncUpdate();
      if (m_data_len > 0) {
        m_monitor.AddUpdateDataLen(m_data_len);
        m_data_len = 0;
      }
      break;
    case csp1_delete_fc:
      m_monitor.IncDelete();
      if (m_data_len > 0) {
        m_monitor.AddDeleteDataLen(m_data_len);
        m_data_len = 0;
      }
      break;
    case csp1_select_direct_fc:
      m_monitor.IncSelect();
      break;
    case csp1_select_first_fc:
      m_monitor.IncSelect();
      break;
    case csp1_select_last_fc:
      m_monitor.IncSelect();
      break;
    case csp1_select_next_fc:
      m_monitor.IncSelect();
      break;
    case csp1_select_prev_fc:
      m_monitor.IncSelect();
      break;
    case csp1_fetch_first_fc:
      m_monitor.IncFetch();
      if (m_data_len > 0) {
        m_monitor.AddFetchDataLen(m_data_len);
        m_data_len = 0;
      }
      break;
    case csp1_fetch_last_fc:
      m_monitor.IncFetch();
      if (m_data_len > 0) {
        m_monitor.AddFetchDataLen(m_data_len);
        m_data_len = 0;
      }
      break;
    case csp1_fetch_next_fc:
      m_monitor.IncFetch();
      if (m_data_len > 0) {
        m_monitor.AddFetchDataLen(m_data_len);
        m_data_len = 0;
      }
      break;
    case csp1_fetch_prev_fc:
      m_monitor.IncFetch();
      if (m_data_len > 0) {
        m_monitor.AddFetchDataLen(m_data_len);
        m_data_len = 0;
      }
      break;
    case csp1_fetch_pos_fc:
      m_monitor.IncFetch();
      if (m_data_len > 0) {
        m_monitor.AddFetchDataLen(m_data_len);
        m_data_len = 0;
      }
      break;
    case csp1_fetch_same_fc:
      m_monitor.IncFetch();
      if (m_data_len > 0) {
        m_monitor.AddFetchDataLen(m_data_len);
        m_data_len = 0;
      }
      break;
    case csp1_select_into_fc:
      m_monitor.IncSelect();
      break;
    case csp1_fetch_relative_fc:
      m_monitor.IncFetch();
      break;
    default:
      m_monitor.IncOther();
  }
}

/*----- PTS 1120804 ----------------------------------------------------*/

void  co51CSqlSessionContext::incRefCnt() {
  ++m_ref_count;
}; 

/*----- PTS 1120804 ----------------------------------------------------*/

void  co51CSqlSessionContext::decRefCnt() {
  --m_ref_count;
};

/*----- PTS 1120804 ----------------------------------------------------*/

SQL_Monitor*  co51CSqlSessionContext::getMonitor() {
  return &m_monitor;
}

/*----- PTS 1120804 ----------------------------------------------------*/

void co51CSqlSessionContext::setMethodCallEpilog() {
  m_inMethodCallEpilog = true;
}

/*----- PTS 1120804 ----------------------------------------------------*/

void co51CSqlSessionContext::resetMethodCallEpilog() {
  m_inMethodCallEpilog = false;
}

/*----- PTS 1120804 ----------------------------------------------------*/

void co51CSqlSessionContext::setCurrMethod(unsigned char* name) {
  memcpy( &m_curr_method[0], name, sizeof (tsp00_KnlIdentifier) );
}

/*----- PTS 1120804 ----------------------------------------------------*/

void co51CSqlSessionContext::resetCurrMethod() {
  memset ( &m_curr_method[0], ' ', sizeof (tsp00_KnlIdentifier) );
}

/*----- PTS 1120804 ----------------------------------------------------*/

bool co51CSqlSessionContext::getMethodCallMonitorInfo(SqlMonitorData& info) const {
  if (m_inMethodCallEpilog) {
    m_monitor.GetCallStatistics(info);
    memcpy( &info.m_MethodName[0], m_curr_method[0], sizeof(info.m_MethodName) );
    return true;
  }
  return false;
}

/*----- PTS 1120804 ----------------------------------------------------*/

void co51CSqlSessionContext::setDataLen(int DataLen) {
  m_data_len = DataLen;
}

#endif
