/*!***************************************************************************

  module      : vco521CSqlStmt.cpp

  -------------------------------------------------------------------------

  responsible : FerdiF

  special area: C++ SQL Class   
  description : Implementation of co521CSqlStmt 
				
  
  last changed: 2000-03-08  11:22
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/
#include "geo00.h"
#ifndef GCO00_H
#include "gco00.h"
#endif
#ifndef GCO501SQL_H
#include "gco501SQL.h"
#endif
#ifndef GCO521CSQLSTMT_H
#include "gco521CSqlStmt.h"
#endif
#ifndef GCO51CSQLSESSIONCONTEXT_H
#include "gco51CSqlSessionContext.h"
#endif
#ifndef GCO52SQLCOL_H
#include "gco52SqlCol.h"
#endif
#ifndef GCO522CCOLUMNDESC_H
#include "gco522CColumnDesc.h"
#endif
#ifndef HIN201_H
#include "hin201.h"
#endif
#include "hsp81.h"

/*----------------------------------------------------------------------*/

template<class S, class T> class auto_ptr 
{
    public:
        explicit auto_ptr(S *s, T *p = NULL)  {
            sessCtx = s;
            pointee = p; 
        };
        auto_ptr<S,T>& operator=(T *p) { 
            pointee = p; 
            return *this;
        };
        T* operator() (){
            return pointee;
        };
        T* release() 
        {
            T* p = pointee;
            sessCtx = NULL;
            pointee = NULL;
            return p;
        };
        ~auto_ptr() { 
            if (sessCtx)
              sessCtx->deallocate( pointee ); 
        };
    private:
        S* sessCtx;
        T* pointee;
};

/*----------------------------------------------------------------------*/
/* primitive token generator                                            */

inline char *strntok (char *s, const char c, int &n) {                              
  char *pc = s;
  for ( ;*pc == c; pc++ );
  for ( n=0; *(pc + n) != '\0'; n++ )
    if ( *(pc + n) == c ) break;
  return pc;
}

inline tsp81_UCS2Char *strntok (tsp81_UCS2Char *s, const tsp81_UCS2Char c, 
                                int &n) {                              
  tsp81_UCS2Char *pc = s;
  for ( ;pc->s == c.s; pc++ );
  for ( n=0; (*(pc + n)).s != 0; n++ )
    if ( (*(pc + n)).s == c.s ) break;
  return pc;
}

/*----------------------------------------------------------------------*/
/* upper case conversion of a null terminated s                         */

inline void strupper (char *s) {
  char *c = s;
  while (*c) {
    if (islower (*c))
      *c = toupper (*c);
    c++;
  }
}

inline void strupper (tsp81_UCS2Char *s, int size, int swapped) {
  if ( swapped )
    sp81UCS2SwappedStringToupper (s,size);
  else
    sp81UCS2StringToupper (s,size);
}

/*----------------------------------------------------------------------*/
/* compares a token generated by  strntok to a null terminated t        */

static int eqtok (const char *s, const char *t, int n) {
  int i;
  const char *cs = s;
  const char *ct = t;
  
  for (i = 0; (i < n) && (*cs != '\0'); i++) {
    if (*ct++ != *cs++)
      i = n;
  }
  return (i == n && !*ct);
}

static int eqtok (const tsp81_UCS2Char *s, const char *t, int n, int swapped ) {
  int i;
  const tsp81_UCS2Char *cs = s;
  tsp81_UCS2Char ct[64];
  unsigned int converted;

  sp81ASCIItoUCS2(ct, sizeof(ct)/2 , swapped, &converted, (const unsigned char*) t, (unsigned int)strlen(t) );

  for (i = 0; (i < n) && (cs->s != 0); i++, cs++ ) {
    if (ct[i].s != cs->s)
      i = n;
  }
  return (i == n && ! ct[i].s);
}

/* ================================================================================ */
/* ==========                 DECLARATION EXTERNAL C FUNCTIONS            ========= */
/* ================================================================================ */

co521CSqlStmt::structSqlDA::structSqlDA() {
  inp_col_idx       = 0;
  sqlmax            = 0;
  sqln              = 0;
  sqlvar            = NULL;
  long_col_num      = 0;
  longDesc          = NULL;
  fix_buf_len       = 0;
  has_long_in_col   = 0;
  has_long_out_col  = 0;
};

/*----------------------------------------------------------------------*/

#if defined(EXT_CLIENT_SQL)
int co521CSqlStmt::connect (const char* hostname, const char* dbname, 
                            const char *user, const char *passwd, const char* isolevel) {
  return m_session_context.connect(this, hostname, dbname, user, passwd, isolevel);
}
#endif

/*----------------------------------------------------------------------*/

int co521CSqlStmt::addInputParms( cin201_Part * part ) {
  int rc = 1; 
  int i;

  m_sqlda.fix_buf_len = 0;
  if ( m_sqlda.has_long_in_col ) {
    /* calculate "fixed column length" to get the first position
       to be used by long columns                                      */
    for (i = 0; ( i < m_sqlda.sqln ) && rc; i++) {
      SqlCol *var = m_sqlda.sqlvar + i;
      if ( m_sqlda.fix_buf_len < ( var->m_desc->sfi.sp1i_bufpos 
        + var->m_desc->sfi.sp1i_in_out_len )) {
        m_sqlda.fix_buf_len = var->m_desc->sfi.sp1i_bufpos 
          + var->m_desc->sfi.sp1i_in_out_len - 1;
      }
      switch ( var->m_desc->sfi.sp1i_data_type ) {
      case dstra:
      case dstrb:
	  case dstruni:
	  case dlonguni:
        ( m_sqlda.longDesc + var->m_desc->long_idx )->varPos = 0; 
        ( m_sqlda.longDesc + var->m_desc->long_idx )->bufPos = 
          var->m_desc->sfi.sp1i_bufpos;
        break;
      default:
        break;
      }
    }
  }
  for (i = 0; ( i < m_sqlda.sqln ) && rc; i++) {
    SqlCol *var = m_sqlda.sqlvar + i;
    if (var->sqlInOut () == sp1io_input ||
      var->sqlInOut() == sp1io_inout)
    {	
      rc = var->m_desc->addParmToBuffer( part, m_session_context, *this, i + 1 );
    }
  }
  return rc;
};

/*----------------------------------------------------------------------*/

int  co521CSqlStmt::addOpenLongDataInput( cin201_Part * part ) {
  int rc = 1; 
  for (int i = 0; ( i <= m_sqlda.long_col_num ) && rc; i++) {
    co521LongDesc *longDescDestPtr = m_sqlda.longDesc + i;
    SqlCol *var = m_sqlda.sqlvar + longDescDestPtr->colIdx;
    int byteLen;
    switch ( var->sqlVarType())
    {
    case SqlCol::VCHAR_UNICODE:
    case SqlCol::VSTRING_UNICODE:
      byteLen = var->m_desc->vfi.vlen * 2;
      break;
    default:
      byteLen = var->m_desc->vfi.vlen;
    }
    if (( var->m_desc->vfi.ind == 0 ) &&
      ( longDescDestPtr->varPos < byteLen ) &&
      ( var->sqlInOut () == sp1io_input ||
      var->sqlInOut() == sp1io_inout)) {
      m_sqlda.fix_buf_len = part->Length() + sizeof(tsp00_LongDescriptor) + 1;
      longDescDestPtr->bufPos = part->Length() + 1;
      rc = var->m_desc->addOpenLongDataToBuffer( part, m_session_context, *this, 
        longDescDestPtr->colIdx + 1);
    }
  }
  return rc;
};

/*----------------------------------------------------------------------*/

int  co521CSqlStmt::addOpenLongDataOutput( cin201_Part * part ) {
  int rc = 1; 
  for (int i = 0; ( i <= m_sqlda.long_col_num ) && rc; i++) {
    co521LongDesc *longDescDestPtr = m_sqlda.longDesc + i;
    SqlCol *var = m_sqlda.sqlvar + longDescDestPtr->colIdx;
    if (( var->m_desc->vfi.ind >= 0 ) &&
      (!(vm_lastdata ==longDescDestPtr->dsc.ld_valmode() ||
      vm_alldata ==longDescDestPtr->dsc.ld_valmode() ||
      vm_data_trunc ==longDescDestPtr->dsc.ld_valmode())) &&
      ( var->sqlInOut () == sp1io_output ||
      var->sqlInOut() == sp1io_inout)) {
      m_sqlda.fix_buf_len = part->Length() + sizeof(tsp00_LongDescriptor) + 1;
      longDescDestPtr->bufPos = part->Length() + 1;
      rc = var->m_desc->addOpenLongDescriptor( part, m_session_context, *this, 
        longDescDestPtr->colIdx + 1);
    }
  }
  return rc;
};

/*----------------------------------------------------------------------*/

int  co521CSqlStmt::addInputFinishLongData( cin201_Part * part ) {
  int rc = 1; 
  co521LongDesc *longDescDestPtr = m_sqlda.longDesc;
  SqlCol *var = m_sqlda.sqlvar + longDescDestPtr->colIdx;
  if (( var->m_desc->vfi.ind == 0 ) &&
    ( var->sqlInOut () == sp1io_input ||
    var->sqlInOut() == sp1io_inout)) {
    m_sqlda.fix_buf_len = -1;
    longDescDestPtr->bufPos = part->Length() + 1;
    rc = var->m_desc->addOpenLongDataToBuffer( part, m_session_context, *this, 
      longDescDestPtr->colIdx + 1);
  }
  return rc;
};

/*----------------------------------------------------------------------*/

void co521CSqlStmt::reallocColDescs( short new_max ) {
  short   ix;
  auto_ptr<co51CSqlSessionContext,SqlCol> var(&m_session_context); 

  var = REINTERPRET_CAST(SqlCol*,m_session_context.allocate( new_max * sizeof(SqlCol) ));
  /* initialize SqlCol Array to avoid ~SqlCol() to use undefined memory */
  for ( ix = 0; ix < new_max; ix++ )
    (*( var() + ix )).m_desc = NULL;
  for ( ix = m_sqlda.sqlmax; ix < new_max; ix++ )
    (*( var() + ix )).sqlColInit();
  for ( ix = 0; ix < m_sqlda.sqlmax; ix++ ) 
    (*( var() + ix )).m_desc = (*(m_sqlda.sqlvar + ix)).m_desc;            //  PTS 1112897
  m_session_context.deallocate( m_sqlda.sqlvar );
  m_sqlda.sqlmax = new_max;  /* PTS 1115380 FF 2002-04-17 because of execption safety */
  m_sqlda.sqlvar = var.release();
};

/*----------------------------------------------------------------------*/

boolean co521CSqlStmt::hasInputParms( ) {
  for (int i = 0; i < m_sqlda.sqln; i++) {
    SqlCol *var = m_sqlda.sqlvar + i;
    if (var->sqlInOut () == sp1io_input ||
      var->sqlInOut() == sp1io_inout)
    {	
      return true;
    }
  }
  return false;
}

/*----------------------------------------------------------------------*/

co521CSqlStmt::co521CSqlStmt(co51CSqlSessionContext* sessContext, const char *stm)
  : m_session_context(*sessContext), m_ref_count(1), m_prepared(false), 
    m_stmt_type(ECO521_SQL), m_unicode(0), /* PTS 1109560 */m_reset_parms( true ) {
  m_res_tabname[0] = '\0';
  tsp00_KnlIdentifierc  token;
  int i,n;

  if (stm) {
    m_statement.c = REINTERPRET_CAST(char*,m_session_context.allocate( strlen(stm) + 1));
    if ( m_statement.c ) {
      strcpy (m_statement.c, stm);
      char *s = strntok (m_statement.c, ' ', n);
      strncpy(token,s,n);
      token[n] = '\0';
      strupper(token);
#if !defined (EXT_CLIENT_SQL)
      if (eqtok (token, (char*)"COMMIT", n) || eqtok (token, (char*)"ROLLBACK", n)) {
        s[0] = 'X';
      }
#endif
      if (eqtok (token, (char*)"FETCH", n)) {
        static char *dir [5] = {(char*)"FIRST", (char*)"LAST", (char*)"NEXT", (char*)"PREV", (char*)"SAME"},
          *pos = (char*)"POS", *into [3] = {(char*)"INTO", (char*)"USING", (char*)"DESCRIPTOR"};
        s = strntok (s + n, ' ', n);
        strncpy(token,s,n);
        token[n] = '\0';
        strupper(token);
        
        for (i = 0; i < 5; i++) if (eqtok (token, dir [i], n)) i = 5;
        if (i > 5) {
          s = strntok (s + n, ' ', n);
          strncpy(token,s,n);
          token[n] = '\0';
          strupper(token);
        }
        else if (eqtok (s, pos, n))
        {
          s = strntok (s + n, ' ', n);
          if (*s == '(')
          {
            s = strntok (s, ')', n);
            if (*(s + n) == ')') s++;
            s = strntok (s + n, ' ', n);
            strncpy(m_res_tabname, s, n);
            m_res_tabname[n] = '\0';
            strncpy(token,s,n);
            token[n] = '\0';
            strupper(token);
          }
        }
        if (!(eqtok (token, into [0], n) || eqtok (token, into [1], n)))
        {
          s = strntok (s + n, ' ', n);
          strncpy(m_res_tabname, s, n);
          m_res_tabname[n] = '\0';
          strncpy(token,s,n);
          token[n] = '\0';
          strupper(token);
        }
//#if defined(COMATABILITY)
        /* following code for compatability only                       */
        /* describe isn't supported anymore                            */
        if (eqtok (token, into [0], n) || eqtok (token, into [1], n))
        {
          if (eqtok (token, into [1], n))
            for ( i=0; i<n; i++ ) s[i] = ' ';
          s = strntok (s + n, ' ', n);
          strncpy(token,s,n);
          token[n] = '\0';
          strupper(token);
          if (eqtok (token, into [2], n)) {
            //m_stmt_type = ECO521_FETCH_DESCRIBE; only usefull if column names are needed
            for ( i=0; i<n; i++ ) s[i] = ' ';
            s = strntok (s + n, ' ', n);
            strncpy(m_res_tabname, s, n);
            m_res_tabname[n] = '\0';
          }
        }
//#endif
      }
    }
  }
  else 
  {
    m_statement.c = REINTERPRET_CAST(char*,m_session_context.allocate( 1 ) );
    *m_statement.c = 0;
  }  
};

/*----------------------------------------------------------------------*/

co521CSqlStmt::co521CSqlStmt(co51CSqlSessionContext* sessContext, const DbpTypeUnicode *stm)
  : m_session_context(*sessContext), m_ref_count(1), m_prepared(false), 
    m_stmt_type(ECO521_SQL), m_unicode(1), /* PTS 1109560 */m_reset_parms( true ) {
  m_res_tabname[0] = '\0';
  tsp81_UCS2Char uc_token[ sizeof(tsp00_KnlIdentifierc) ];
  int i,n;
  tsp81_UCS2Char    UCS2Char;
  tsp81_UCS2Char    UCS2Blank;
  int               UCS2Swap;
  UCS2Blank.s = 32;
  UCS2Swap =  (UCS2Blank.c[0] ? 1 : 0 ); 

  if (stm) {
    m_statement.uc = REINTERPRET_CAST(tsp81_UCS2Char*,
      m_session_context.allocate( (sp81UCS2strlen((tsp81_UCS2Char*)stm) + 1) * 2));
    if ( m_statement.uc ) {
      sp81UCS2strcpy (m_statement.uc, (tsp81_UCS2Char*)stm);
      tsp81_UCS2Char *s = strntok (m_statement.uc, UCS2Blank, n);
      sp81UCS2strncpy(uc_token,s,sizeof(uc_token)/sizeof(tsp81_UCS2Char));
      uc_token[n].s = 0;
      strupper(uc_token, sizeof(uc_token)/2, UCS2Swap);
#if !defined (EXT_CLIENT_SQL)
      if (eqtok(uc_token, (char*)"COMMIT", n, UCS2Swap) || 
        eqtok (uc_token, (char*)"ROLLBACK", n, UCS2Swap)) {
        s->s = 'X';
      }
#endif
      if (eqtok ((tsp81_UCS2Char*)uc_token, (char*)"FETCH", n, UCS2Swap)) {
        static char *dir [5] = {(char*)"FIRST", (char*)"LAST", (char*)"NEXT", (char*)"PREV", (char*)"SAME"},
          *pos = (char*)"POS", *into [3] = {(char*)"INTO", (char*)"USING", (char*)"DESCRIPTOR"};
        s = strntok (s + n, UCS2Blank, n);
        sp81UCS2strncpy (uc_token, s, n);
        uc_token[n].s = 0;
        strupper(uc_token, sizeof(uc_token)/2, UCS2Swap);
        
        for (i = 0; i < 5; i++) if (eqtok (uc_token, dir [i], n,UCS2Swap)) i = 5;
        if (i > 5) {
          s = strntok (s + n, UCS2Blank, n);
          sp81UCS2strncpy (uc_token, s, n);
          uc_token[n].s = 0;
          strupper(uc_token, sizeof(uc_token)/2, UCS2Swap);
        }
        else if (eqtok (s, pos, n,UCS2Swap))
        {
          s = strntok (s + n, UCS2Blank, n);
          if (s->s == '(')
          {
            UCS2Char.s = ')';
            s = strntok (s, UCS2Char, n);
            if ((s + n)->s == ')') s++;
            s = strntok (s + n, UCS2Blank, n);
/***** TO DO TO DO            strncpy(m_res_tabname, s, n); */
/***** TO DO            m_res_tabname[n] = '\0';            */
            sp81UCS2strncpy (uc_token, s, n);
            uc_token[n].s = 0;
            strupper(uc_token, sizeof(uc_token)/2, UCS2Swap);
          }
        }
        if (!(eqtok (uc_token, into [0], n, UCS2Swap) || eqtok (uc_token, into [1], n, UCS2Swap)))
        {
          s = strntok (s + n, UCS2Blank, n);
/*          strncpy(m_res_tabname, s, n);
          m_res_tabname[n] = '\0';   */
            sp81UCS2strncpy (uc_token, s, n);
            uc_token[n].s = 0;
            strupper(uc_token, sizeof(uc_token)/2, UCS2Swap);
        }
#if defined(COMATABILITY)
        /* following code for compatability only                       */
        /* describe isn't supported anymore                            */
        if (eqtok (token, into [0], n, UCS2Swap) || eqtok (token, into [1], n, UCS2Swap))
        {
          for ( i=0; i<n; i++ ) s[i] = ' ';
          s = strntok (s + n, UCS2Blank, n);
          strncpy(token,s,n);
          token[n] = '\0';
          strupper(token);
          if (eqtok (token, into [1], n, UCS2Swap)) {
            m_stmt_type = ECO521_FETCH_DESCRIBE;
            for ( i=0; i<n; i++ ) s[i] = ' ';
            s = strntok (s + n, UCS2Blank, n);
            strncpy(m_res_tabname, s, n);
            m_res_tabname[n] = '\0';
          }
        }
#endif
      }
    }
  }
  else 
  {
    m_statement.uc = REINTERPRET_CAST(tsp81_UCS2Char*,m_session_context.allocate( 2 ) );
    *m_statement.dc = 0;
  }  
};

/*----------------------------------------------------------------------*/

void co521CSqlStmt::appendNewColDesc(const SqlCol& p) {
  if ( m_reset_parms ) {                 /* PTS 1109560 */ 
    m_sqlda.inp_col_idx = 0;             /* PTS 1109560 */ 
    m_reset_parms = false;               /* PTS 1109560 */ 
  };                                     /* PTS 1109560 */ 
  m_sqlda.inp_col_idx++;
  if (m_sqlda.inp_col_idx > m_sqlda.sqlmax) 
    reallocColDescs( m_sqlda.sqlmax + 10 );
  SqlCol *var = m_sqlda.sqlvar + ( m_sqlda.inp_col_idx - 1);
  var->m_desc->assignVar( *p.m_desc );  
}
/*----------------------------------------------------------------------*/

int co521CSqlStmt::createLongDescriptors() { 
  int rc = 1;
  short i = m_sqlda.long_col_num + 1;
  if ( i > 0 ) {
    co521LongDesc * longDescPtr = NULL;
    m_sqlda.longDesc = REINTERPRET_CAST(co521LongDesc*,
      m_session_context.allocate( i * sizeof(co521LongDesc) ));
    if ( m_sqlda.longDesc ) {
      for ( i = 0; i <= m_sqlda.long_col_num; i++ ) {
        longDescPtr = m_sqlda.longDesc + i;
        memset(longDescPtr,'\0',sizeof(*longDescPtr));
      }
    }
    else {
      rc = 0;
    }
    for (int i = 0; i < m_sqlda.sqln; i++) {
      SqlCol *var = m_sqlda.sqlvar + i;
      if ( var->m_desc ) {
        switch ( var->m_desc->sfi.sp1i_data_type ) {
        case dstra:
        case dstrb:
		case dstruni:
        case dlonguni:
          if (var->sqlInOut () == sp1io_output) {
            m_sqlda.has_long_out_col++;
          }
          else if (var->sqlInOut() == sp1io_inout){
            m_sqlda.has_long_in_col++;
            m_sqlda.has_long_out_col++;
          } else if (var->sqlInOut() == sp1io_input){
            m_sqlda.has_long_in_col++;
          }
          ( m_sqlda.longDesc + var->m_desc->long_idx )->dsc.ld_used_in_ak()[0] = 
            (unsigned char)var->m_desc->long_idx; 
          ( m_sqlda.longDesc + var->m_desc->long_idx )->colIdx = i;
          break;
        default:
          break;
        }
      }
    }
  }
  else {
    m_sqlda.has_long_in_col  = 0;
    m_sqlda.has_long_out_col = 0;
  }
  return rc;
}
/*----------------------------------------------------------------------*/

int co521CSqlStmt::dropLongDescriptors() {
  m_sqlda.long_col_num = -1;
  m_session_context.deallocate( m_sqlda.longDesc );
  m_sqlda.longDesc = NULL;
  return 1;
}

/*----------------------------------------------------------------------*/

int  co521CSqlStmt::execute() {
  return m_session_context.executeSqlStmt(this);
};


/*----------------------------------------------------------------------*/

int  co521CSqlStmt::getOutputParms (int argCount, 
                                    int dataLen, 
                                    const teo00_Byte * dataPtr ) {
  int rc = 1; 
  co521LongDesc * longDescPtr = NULL;
  for (int i = 0; ( i < m_sqlda.sqln ) && rc; i++) {
    SqlCol *var = m_sqlda.sqlvar + i;
    switch ( var->m_desc->sfi.sp1i_data_type ) {
    case dstra:
    case dstrb:
	case dstruni:
    case dlonguni:
      longDescPtr = m_sqlda.longDesc + var->m_desc->long_idx;
      longDescPtr->bufPos = var->m_desc->sfi.sp1i_bufpos;
      longDescPtr->dsc.ld_valpos() = 0;
      longDescPtr->varPos = 0;
      break;
    default:
      break;
    }
    if (var->sqlInOut () == sp1io_output ||
      var->sqlInOut() == sp1io_inout)
    {	
      rc = var->m_desc->getParmFromBuffer( dataPtr, m_session_context, 
        *this, i + 1 );
    }
  }
  return rc;
};  

/*----------------------------------------------------------------------*/
/* PTS 1112690 parsingAgain added */
int  co521CSqlStmt::prepare( const pasbool parsingAgain) {
  m_res_tabname[0] = '0';
  int rc = m_session_context.prepareSqlStmt(this, parsingAgain);
  if (rc) m_prepared = true;
  return rc;
};

/*----------------------------------------------------------------------*/

void co521CSqlStmt::setParsid( int len, const tin02_ParseId *data ) {
  memcpy( &m_parseid[0], data, sizeof(m_parseid) ); 
  m_parseid_len = len;
};

/*----------------------------------------------------------------------*/

void co521CSqlStmt::setParsidOfSelect( int len, const tin02_ParseId *data ) {
  memcpy( &m_parseid_of_select[0], data, sizeof(m_parseid_of_select) ); 
  m_parseid_len = len;
};

/*----------------------------------------------------------------------*/

void co521CSqlStmt::setResultTablename( int len, const char * data ) {
  memcpy( &m_res_tabname[0], data, len );
  m_res_tabname[len] = '\0';
};

/*----------------------------------------------------------------------*/

void co521CSqlStmt::setSqlDA_sqln( short argCount ) {
  if ( m_sqlda.sqlmax < argCount ) {
    reallocColDescs( argCount ); 
  }
  m_sqlda.sqln = argCount;
};

/*----------------------------------------------------------------------*/

void co521CSqlStmt::setSqlDA_sfi( short i, const tsp1_param_info * sfi ) {
  SqlCol *var = m_sqlda.sqlvar + i;
  if ( var->m_desc ) {
    memcpy(&var->m_desc->sfi, sfi, sizeof(*sfi) );
    switch ( var->m_desc->sfi.sp1i_data_type ) {
    case dstra:
    case dstrb:
	case dstruni:
    case dlonguni:
      m_sqlda.long_col_num++;
      var->m_desc->long_idx = m_sqlda.long_col_num;
      break;
    default:
      break;
    }
  }
  if ( m_sqlda.fix_buf_len < ( var->m_desc->sfi.sp1i_bufpos 
    + var->m_desc->sfi.sp1i_in_out_len )) {
    m_sqlda.fix_buf_len = var->m_desc->sfi.sp1i_bufpos 
      + var->m_desc->sfi.sp1i_in_out_len - 1;
  }

};

/*----------------------------------------------------------------------*/

void co521CSqlStmt::setResetParms() {              /* PTS 1109560 */
    m_reset_parms = true;
};

/*----------------------------------------------------------------------*/

int  co521CSqlStmt::getFixBufLength( ) const {
  return m_sqlda.fix_buf_len;
};

/*----------------------------------------------------------------------*/

short co521CSqlStmt::getNumCols () const {
  return m_sqlda.sqln;
};

/*----------------------------------------------------------------------*/

short co521CSqlStmt::getNumVars () const{
  return m_sqlda.inp_col_idx;
};

/*----------------------------------------------------------------------*/

short co521CSqlStmt::getLongColNum () const {
  return m_sqlda.long_col_num;
};

/*----------------------------------------------------------------------*/

short co521CSqlStmt::getNumOutCols () const {
  short numOutCols = 0; 
  for (int i = 0; i < m_sqlda.sqln; i++) {
    SqlCol *var = m_sqlda.sqlvar + i;
    if (var->sqlInOut () == sp1io_output ||
      var->sqlInOut() == sp1io_inout)
    {	
      numOutCols++;
    }
  }
  return numOutCols;
};

/*----------------------------------------------------------------------*/

#define OFFSET(T,M) (((char*) &((T*) 0)->M) - (char*) 0) // 1113808 FF 30-JAN-2002 

int co521CSqlStmt::getLongDesc(int argCount, int dataLen, 
                               const tsp00_Byte * dataPtr) {

  unsigned char * longDescSrcPtr  = NULL;	// 1113808 FF 30-JAN-2002 
  co521LongDesc * longDescDestPtr = NULL;  

  for (int i = 0; i < argCount; i++) {
    longDescSrcPtr = (unsigned char *)(dataPtr + i * (sizeof(tsp00_LongDescriptor) + 1) + 1);
    longDescDestPtr = m_sqlda.longDesc + *( longDescSrcPtr +  OFFSET(tsp00_LongDescriptor,ld_used_in_ak()) );
    memcpy( &longDescDestPtr->dsc, longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
  }
  m_sqlda.fix_buf_len = argCount * (1 + sizeof(tsp00_LongDescriptor));

  return 1;
}; 

/*----------------------------------------------------------------------*/

int co521CSqlStmt::getLongDescAndValue(int argCount, int dataLen, 
                                       const tsp00_Byte * dataPtr) {
  
  unsigned char * longDescSrcPtr  = NULL;	// 1113808 FF 30-JAN-2002 
  co521LongDesc * longDescDestPtr = NULL;  
  int descPos = 1;
  int rc = 1;

  for (int i = 0; i < argCount; i++) {
    longDescSrcPtr = (unsigned char *)(dataPtr + descPos);
    longDescDestPtr = m_sqlda.longDesc + *( longDescSrcPtr +  OFFSET(tsp00_LongDescriptor,ld_used_in_ak()) );
    memcpy( &longDescDestPtr->dsc, longDescSrcPtr, sizeof(tsp00_LongDescriptor) );
    SqlCol *var = m_sqlda.sqlvar + longDescDestPtr->colIdx;
    if (( vm_datapart == longDescDestPtr->dsc.ld_valmode() ) ||
      ( vm_alldata == longDescDestPtr->dsc.ld_valmode() ) ||
      ( vm_lastdata == longDescDestPtr->dsc.ld_valmode() )) {
      descPos = longDescDestPtr->dsc.ld_valpos() + 
        longDescDestPtr->dsc.ld_vallen();
      rc = var->m_desc->getOpenLongDataFromBuffer( dataPtr, m_session_context, *this, 
        longDescDestPtr->colIdx + 1);
    }
    else
      descPos = descPos + sizeof(tsp00_LongDescriptor) + 1;
  }
  m_sqlda.fix_buf_len = argCount * (1 + sizeof(tsp00_LongDescriptor));

  return rc;
}; 

/*----------------------------------------------------------------------*/

int  co521CSqlStmt::getParsid( const tin02_ParseId ** data ) const {
  *data = &m_parseid;
  return m_parseid_len;
};
/*----------------------------------------------------------------------*/
co521LongDesc* co521CSqlStmt::getLongDescPtr( short idx ) {
    return ( m_sqlda.longDesc + idx );
};

/*----------------------------------------------------------------------*/

boolean co521CSqlStmt::getPreparedFlag() const {
  return m_prepared;
};

/*----------------------------------------------------------------------*/

const char* co521CSqlStmt::getStatement() const {
  return m_statement.c;
};

/*----------------------------------------------------------------------*/

const tsp81_UCS2Char* co521CSqlStmt::getUniStatement() const {
  return m_statement.uc;
};

/*----------------------------------------------------------------------*/

eco521_stmt_type co521CSqlStmt::getStmtType() const {
  return m_stmt_type;
};

/*----------------------------------------------------------------------*/

int co521CSqlStmt::isUnicode() const {
  return m_unicode;
};

/*----------------------------------------------------------------------*/

short  co521CSqlStmt::hasLongInput() {
  if ( m_sqlda.has_long_in_col ) {
    for ( int i=0; i <= m_sqlda.long_col_num; i++) {
      co521LongDesc *longDescDestPtr = m_sqlda.longDesc + i;
      SqlCol *var = m_sqlda.sqlvar + longDescDestPtr->colIdx;
      if (( var->m_desc->vfi.ind == 0 ) &&         
        ( var->sqlInOut () == sp1io_input ||
        var->sqlInOut() == sp1io_inout)) {
        int byteLen;
        switch ( var->sqlVarType())
        {
        case SqlCol::VCHAR_UNICODE:
        case SqlCol::VSTRING_UNICODE:
          byteLen = var->m_desc->vfi.vlen * 2;
          break;
        default:
          byteLen = var->m_desc->vfi.vlen;
        } 
        if ( longDescDestPtr->varPos < byteLen )
          return 1;
      }
    };
  }
  return 0;
};

/*----------------------------------------------------------------------*/

short  co521CSqlStmt::hasLongOutput() {
  if ( m_sqlda.has_long_out_col ) {
    for ( int i=0; i <= m_sqlda.long_col_num; i++) {
      co521LongDesc *longDescDestPtr = m_sqlda.longDesc + i;
      SqlCol *var = m_sqlda.sqlvar + longDescDestPtr->colIdx;
      if (( var->m_desc->vfi.ind == 0 ) &&         
        ( var->sqlInOut () == sp1io_output ||
        var->sqlInOut() == sp1io_inout)) {
        if (!(vm_alldata == longDescDestPtr->dsc.ld_valmode() ||
              vm_data_trunc == longDescDestPtr->dsc.ld_valmode() ||
              vm_lastdata == longDescDestPtr->dsc.ld_valmode() ))
          return 1;
      }
    };
  }
  return 0;
};

/*----------------------------------------------------------------------*/

int  co521CSqlStmt::addRef() {
  ++m_ref_count;
  return m_ref_count;
};

/*----------------------------------------------------------------------*/

int  co521CSqlStmt::release() {
  --m_ref_count;
  if ( 0 == m_ref_count ) {
    m_session_context.deallocate( m_statement.c );
    m_statement.c = NULL;
    for ( int i = 0; i < m_sqlda.sqlmax; i++ )
      (*( m_sqlda.sqlvar + i )).~SqlCol();
    m_session_context.deallocate( m_sqlda.sqlvar );
    m_sqlda.sqlvar = NULL;
    m_sqlda.long_col_num = 0;
    m_session_context.deallocate( m_sqlda.longDesc );
    m_sqlda.longDesc = NULL;
    m_session_context.dropSqlStmt(this);
    return 0;
  }
  else
    return m_ref_count;
};

/*----------------------------------------------------------------------*/

SqlCol& co521CSqlStmt::operator[] (int i) {
  if ( (i >= 0) && (i < m_sqlda.sqlmax) ) 
      return *(m_sqlda.sqlvar + i);
  else
    return *new(SqlCol);
}; 

/*----------------------------------------------------------------------*/

void* co521CSqlStmt::operator new(size_t sz) {
#if defined (EXT_CLIENT_SQL)
  void *p = malloc(sz);
  if (!p)
    throw DbpError(e_new_failed,"new failed");
  return p;
#else
# if defined(OMSTST)
  co10_GetKernelInterface()->TestBadAlloc();
# endif
  SAPDBMem_RawAllocator *m_alloc;
  co10_GetLcSink()->GetSqlClsAllocator(m_alloc);
  return m_alloc->Allocate(sz);
#endif
};

/*----------------------------------------------------------------------*/

void  co521CSqlStmt::operator delete (void* p) {
# if defined (EXT_CLIENT_SQL)
  free(p);
# else
  SAPDBMem_RawAllocator *m_alloc;
  co10_GetLcSink()->GetSqlClsAllocator(m_alloc);
  m_alloc->Deallocate(p);
# endif
};
