/****************************************************************************

  module      : gen50.h

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Kernel LZU UNIX
  description : Kernel interne Strukturen

  last changed: 2001-01-03  10:22
  see also    : example.html ...

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1998



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef GEN50_H
#define GEN50_H

#ifdef  USE_KGS

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gen00.h"
#include "geo00_3.h"
#include "geo52.h"
#include "heo00.h"
#include "heo07.h"
#include "vsp002.h"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define NO_DISTRIBUTION

/*
   --- Kernel threads stack sizes 
   These sizes are used as input for sqlbeginthread() calls
 */
#if defined (HP_IA64)
#define UKT_BASE_STACK_SIZE             ( 512 * 1024)
#define UKT_TEMP_STACK_SIZE             ( 512 * 1024)
#define KT_WORKER_STACK_SIZE            ( 512 * 1024)
#define KT_TIMER_STACK_SIZE             ( 512 * 1024)
#define KT_CLOCK_STACK_SIZE             ( 512 * 1024)
#define KT_DEV0_STACK_SIZE              ( 512 * 1024)
#define KT_DEVI_STACK_SIZE              ( 512 * 1024)
#define KT_CONSOLE_STACK_SIZE           ( 512 * 1024)
#define KT_ASYNC0_STACK_SIZE            ( 512 * 1024)
#define KT_ASYNCi_STACK_SIZE            ( 512 * 1024)
#define KT_REQUESTOR_STACK_SIZE         ( 512 * 1024)
#define KT_COORDINATOR_STACK_SIZE       ( 512 * 1024)
#define KT_DEV_WORKER_STACK_SIZE        ( 512 * 1024)
#else
#define UKT_BASE_STACK_SIZE             ( 128 * 1024)
#define UKT_TEMP_STACK_SIZE             ( 128 * 1024)
#define KT_WORKER_STACK_SIZE            ( 128 * 1024)
#define KT_TIMER_STACK_SIZE             ( 128 * 1024)
#define KT_CLOCK_STACK_SIZE             ( 128 * 1024)
#define KT_DEV0_STACK_SIZE              ( 128 * 1024)
#define KT_DEVI_STACK_SIZE              ( 128 * 1024)
#define KT_CONSOLE_STACK_SIZE           ( 128 * 1024)
#define KT_ASYNC0_STACK_SIZE            ( 128 * 1024)
#define KT_ASYNCi_STACK_SIZE            ( 128 * 1024)
#define KT_REQUESTOR_STACK_SIZE         ( 128 * 1024)
#define KT_COORDINATOR_STACK_SIZE       ( 128 * 1024)
#define KT_DEV_WORKER_STACK_SIZE        ( 128 * 1024)
#endif

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/

/*---------------------------------------------------------------------------------
 *  Macros for dispatching
 */
#if 0
#if defined(HPUX)
/* PTS 1114487 */
/* The HPUX compiler supports a much more effectiv way to use thread local storage... */
   extern __thread void *en81ThisUktControl;
#  define THIS_UKT_CTRL ((ten50_UKT_Control *)en81ThisUktControl)
#else
externC void *en81_Get_KT_TLS( void );
#  define THIS_UKT_CTRL ((ten50_UKT_Control *) en81_Get_KT_TLS())
#endif /* HPUX */
#else
#  define THIS_UKT_CTRL en88GetThisUkt()
#endif
/*---------------------------------------------------------------------------------
 *  Macro for retrieving a task control block from a task index
 */
#if COMPILEMODE_MEO00 >= SLOW_MEO00

#define PID_TCB_EN50(pid_,tcb_) \
  {\
    ten50_UKT_Control * this_ukt = THIS_UKT_CTRL; \
    if ( this_ukt->curr_task->index != pid_ ) \
    { \
      MSGALL (( ERR_VXXXX_WRONG_TASK, "PID_TCB_Mapping", pid_ )); \
      vabort( WRITE_CORE ) ; \
    } \
    if ( pid_ < 1 || pid_ > kgs->ulNumOfTasks )\
    {\
      MSGCD (( ERR_KERN_PARAM_PID_OUT_OF_RANGE, pid_ ));\
      vabort ( WRITE_CORE );\
    }\
    tcb_ = kgs->pFirstTaskCtrl + pid_ - 1 ;\
  }

#else /* COMPILEMODE_MEO00 < SLOW_MEO00 */

/*
 * Since in slow mode pid is checked, we accept it without check in fast mode...
 */
#define PID_TCB_EN50(pid_,tcb_) tcb_ = kgs->pFirstTaskCtrl + (pid_) - 1

#endif /* COMPILEMODE_MEO00 < SLOW_MEO00 */

#  define THREAD_INDEX(_t) ((tsp00_Int4)((_t)->index))

#define GOTO_DISP(_ppUKT) {                                                            \
   struct TASK_TYPE*   oldTask_ = (*_ppUKT)->curr_task ;                               \
  (*_ppUKT)->curr_task->begexcl_count = 0 ;                                            \
                                                                                       \
  en71Dispatcher (*_ppUKT);                                                            \
                                                                                       \
  if ( oldTask_ != (*_ppUKT)->curr_task )                                              \
  {                                                                                    \
      (*_ppUKT)->taskSwitches++;                                                       \
      en88TaskContextSwitch( oldTask_, (*_ppUKT)->curr_task );                         \
      *_ppUKT = oldTask_->ukt;                                                         \
  }                                                                                    \
}

#define RELEASE_FREELIST_ELEM(_pReq)                                \
      e73_dl_enqu (&kgs->freelist, _pReq)
#define RESERVE_FREELIST_ELEM()                                     \
      e73_dl_dequ( &kgs->freelist );

#define RESUME_TRACEWRITER(_pTW)                                    \
    {                                                               \
    struct DOUBLY_LINKED *  _pRequest = RESERVE_FREELIST_ELEM();    \
    _pRequest->req_type = REQ_RESUME_TW_EO00;                       \
    en71_iocEnquAndWake( _pTW->ukt, _pTW, _pRequest );              \
    }



/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/


/*
 *  Specific event parameter field definitions.
 */
struct REQ_PARMS_CONNECT
{
    tsp00_Int4                            task_index ;
    tsp00_Int4                            filler;
};

struct REQ_PARMS_VATTACH
{
    tsp00_Int4                            devno;
    tsp00_Int4                            ndevi;
    struct ten50_UKT_Control              *kp;
    tsp00_Uint4                           ok;
    tsp00_Uint4                           errcode;
};

struct REQ_PARMS_VBLOCKIO
{
    tsp00_Int4                            devno;
    tsp00_Int4                            devpno;
    tsp00_Int4                            op_code;
    tsp00_Int4                            buf_count; /*JH*/
    tsp00_PageAddr                        p;
    tsp00_Uint4                           ok;
    tsp00_Uint4                           errcode;
};

struct REQ_PARMS_VDETACH
{
    tsp00_Int4                            devno;
    tsp00_Int4                            ndevi;
    struct         DOUBLY_LINKED          *this_req;
    struct         ten50_UKT_Control      *kp;
    tsp00_Uint4                           ok;
    tsp00_Uint4                           errcode;
    tsp00_Bool                            rewind;
    tsp00_Bool                            filler[7];
};

struct REQ_PARMS_ASYNOPEN
{
    tsp00_Int4                            devno;
    tsp00_Int4                            ndevi;
    struct ten50_UKT_Control              *kp;
    tsp00_Int4                            block_size;
    tsp00_BoolAddr                        cancelPointer;
    tsp00_Int4                            max_block_cnt;
    tsp00_Uint4                           is_devspace;
    tsp00_Uint4                           for_writing;
    tsp00_Uint4                           ok;
    tsp00_Uint4                           errcode;
    tsp00_Int4                            fileType;
    tsp00_Int4                            filler;
};

struct REQ_PARMS_ASYNIO
{
    tsp00_Int4                            io_id;
    tsp00_Int4                            devno;
    tsp00_Int4                            devpno;
    tsp00_Int4                            block_siz;
    tsp00_Int4                            block_cnt;
    tsp00_Uint4                           finished;
    tsp00_Uint4                           ok;
    tsp00_Uint4                           errcode;
    tsp_maxiobufaddr                      block_ptr;
    long                                  io_count;
    struct TASK_TYPE                      *initiator ;
    void                                  *aio_op;
    tsp00_Int4                            aio_errno ;
    tsp00_Int4                            filler;
};

struct REQ_PARMS_ASYNCNTL
{
    tsp00_Uint4                           ok;
    tsp00_Uint4                           errcode;
    tsp00_Int4                            cntlRequest;
    tsp00_Bool                            destructive;
    tsp_vf_type                           fileType;
    tsp00_Bool                            filler[2];
    tsp00_Int4                            hostfileno;
};

struct REQ_PARMS_VSIGNAL
{
    tsp00_Uint4                           ok;
    tsp00_Int4                            filler;
};

struct REQ_PARMS_TIMER
{
    tsp00_Int4                            time_to_wake;
    tsp00_Int4                            filler;
};

struct REQ_PARMS_VSUSPEND
{
    tsp00_Int4                            susp_count;
    tsp00_Int4                            filler;
};

struct REQ_PARMS_VNSHUTDOWN
{
    tsp00_Int4                            pathid ;
    tsp00_Int4                            filler;
    struct TASK_TYPE                      *initiator ;
};

struct REQ_PARMS_VVECTORIO
{
  tsp00_Int4                            devno;
  tsp00_Int4                            devpno;
  tsp00_Int4                            pageCount;
  tsp00_PageAddr                        *pageVector;
  tsp00_Uint4                           ok;
  tsp00_Uint4                           errcode;
};

struct REQ_PARMS_MOVE_TASK_MESS_INT
{
  SAPDB_UInt2                           timeVar;         /* Value 0 .. TIME_VAL_ARRAY_SIZE */
  SAPDB_UInt8                           taskMoveIntervalStartTime;
};

struct REQ_PARMS_MOVE_TASK
{
  SAPDB_UInt8                           taskMoveIntervalStartTime; /* used to skip old requests */
  SAPDB_UInt8                           stopWaitForBestFitTaskAt; 
  struct ten50_UKT_Control              *moveTaskToUKT;            /* pointer to the UKT to which a Task has to be moved */
};

union REQ_ARGS
{
    struct REQ_PARMS_VSUSPEND             vsu_parms ;
    struct REQ_PARMS_VATTACH              vat_parms ;
    struct REQ_PARMS_VBLOCKIO             vbl_parms ;
    struct REQ_PARMS_VDETACH              vde_parms ;
    struct REQ_PARMS_ASYNOPEN             vas_parms ;
    struct REQ_PARMS_ASYNIO               vai_parms ;
    struct REQ_PARMS_ASYNCNTL             vac_parms ;
    struct REQ_PARMS_VSIGNAL              vsig ;
    struct REQ_PARMS_TIMER                timer_auftrag ;
    struct REQ_PARMS_CONNECT              conn_auftrag ;
    struct REQ_PARMS_VNSHUTDOWN           vns_parms ;
    struct REQ_PARMS_VVECTORIO            vve_parms ;
    struct REQ_PARMS_MOVE_TASK            mt_parms ;
    struct REQ_PARMS_MOVE_TASK_MESS_INT   mtmi_parms ;
};
   
/*
 * Some Queue elements  are initially  listed in the Free-list |
 */

/*
 * The UKT2UKT queue (alias XXX) element
 * contains pointer to nexdt in fixed ring as well as all parameters needed for the function call
 */

struct XXX_QUEUE
{
    struct XXX_QUEUE          *next;
    VOIDFUNC_PTR                       to_execute;
    void                               *func_param ;
    struct         TASK_TYPE           *task;
    struct         TASK_TYPE           *FromTask;
};


/*
 * The UKT2UKT-queue (alias XXX) is a fixed list of maxtasks elements pointing in ring
 */
struct XXX_QUEUE_HEAD
{
    tsp00_Uint4               num_overflows ;
    tsp00_Uint4               filler;
    struct XXX_QUEUE *xx_work;
    struct XXX_QUEUE *element;
    struct XXX_QUEUE *xx_request;
   ALONE_IN_CACHE(dum_before) 
    volatile ALONE_UKP_STRUCT   exclusive;
   ALONE_IN_CACHE(dum_behind)
    /*
     * xx_work        is only used by server
     * xx_request     is used to enter requests,
     * entries.task is nulled when completed
     * xx_work should always follow the xx_request
     */
};

/*
 * The ioc-queue is a fixed list pointing in ring
 */
struct IOC_QUEUE_HEAD
{
    struct IOC_QUEUE          *first;
    struct IOC_QUEUE          *last;
   ALONE_IN_CACHE(dum_before) 
    volatile ALONE_UKP_STRUCT exclusive;
   ALONE_IN_CACHE(dum_behind) 
    /*
     * first is first free element
     * last is last used element
     */
};


/*
 * The ior-queue is a fixed list pointing in ring
 */
struct IOR_QUEUE_HEAD
{
    struct IOR_QUEUE           *first;
    struct IOR_QUEUE           *last;
    /*
     * first is first free element
     * last is last used element
     */
};

/*
 * allocate one table of MAX_TPP + 1 for each DEV_PCB, where io requests
 * and corresponding replies are stored
 * The table is circularly linked, and upon completion of a request
 * the completed result is moved one position backward
 */

struct IOC_QUEUE
{
    struct IOC_QUEUE           *next;
    struct IOC_QUEUE           *previous;
    struct DOUBLY_LINKED       *request;
};

struct IOR_QUEUE
{
    struct IOR_QUEUE           *next;
    struct IOR_QUEUE           *previous;
    struct DOUBLY_LINKED       *request;
};

struct DOUBLY_LINKED
{
    struct DOUBLY_LINKED       *forward;
    struct DOUBLY_LINKED       *backward;
    struct         TASK_TYPE            *taskid;
    union          REQ_ARGS            args;
    tsp00_Uint4                        req_type;
    tsp00_Uint4                        req_type_org;
    tsp00_Int4                         filler;
};


struct DOUBLE_Q_HEAD
{
    struct DOUBLY_LINKED       *first;
    struct DOUBLY_LINKED       *last;
   ALONE_IN_CACHE(dum_before) 
    volatile ALONE_UKP_STRUCT   dq_exclusive;
   ALONE_IN_CACHE(dum_behind)
    char                               name[ALIGN_EO00(sizeof(tsp00_C8),sizeof(void *))];
    /*
     * first is as forward, last is as backward used
     */
};


struct          RUN_QUEUE    
{
  struct TASK_TYPE            *Task ;
  struct DOUBLY_LINKED        *Req  ;
  struct TASK_TYPE            *FromTask ;
} ;


/*
 *  DEVICE - I/O
 */
struct dev_info
{
    tsp00_Int4            block_size ;
    tsp00_Int4            max_block_cnt ;
    long                  asyn_io_count ;
    char                  devspace[ALIGN_EO00(sizeof(tsp00_VFilenamec),sizeof(void *))];
    char                  devspace_label[ALIGN_EO00(sizeof(tsp_vf_label),sizeof(void *))];
    tsp00_Uint4           asyn_write ;
	tsp00_Bool            IsDevspace ;   /* PTS 1106176 */
    tsp2_dev_type         devtype;
    tsp00_Bool            filler[2];
};

struct TASK_TYPE
{
    /*
     *  pointer to itself and to the next task in the user kernel thread
     */
    struct TASK_TYPE                * this_tcb ;
    struct TASK_TYPE                * prev_task ;  /* Since load balancing, the task chain     */
    struct TASK_TYPE                * next_task ;  /* should only be used by the UKT its self  */
                                                     
    /*
     *  pointer to the thread control block
     *  of the user kernel thread which runs this task
     */
    struct ten50_UKT_Control        * ukt ;
    struct ten50_UKT_Control        * bigComSegBelongsToUKT; /* we need this because of task load balancing */
    /*
     *  stack management
     */
    tsp00_Byte                      * StackChunkStart ;
    tsp00_Byte                      * StackChunkEnd ;
    tsp00_Byte                      * GuardPageBegin ;
    tsp00_Byte                      * GuardPageEnd ;

    /*
     *  Communication Information.
     */
    struct connection_info          * connection ;                        
    tsp00_Int4                      * reg_excl_cnt ;
    tsp00_Int4                      * reg_coll_cnt ;
    char                            * DevNameWaitingFor;
    struct TASK_TYPE                * next_in_com ;
    /*
     *  two special queue elements, reserved for this task
     */
    struct DOUBLY_LINKED              qa ;
    struct DOUBLY_LINKED              qb ;
    struct DOUBLY_LINKED            * asyn_request ;
    void                            * aio_op;

    /*
     * Special pointer for tracing stack
     */
    void                            * pTrackBuffer;
    /*
     *  kernel communication record (cancel byte,...)
     */
    tsp_rte_comm_rec                  rte_comm ;
    void                             *AcvPointer ;        /* CR 1103950 */
    void                             (*AkDumpProc)() ;    /* CR 1103950 */

   ALONE_IN_CACHE(dum_before) 
    volatile ALONE_UKP_STRUCT         exclusive;
   ALONE_IN_CACHE(dum_behind)

/* -------------------- task statistic ------------------------ */
    teo52_TaskStateStatisticRec       TaskStateStat;
    teo52_TaskQueueStatistic          QueStat;
    teo52_TaskPrioStatistic           PrioStat;
    teo52_TaskOMSStatisticRec         TaskOMSStat; /* 1106969 */

    /* set to EO52_RESET_TASK_STAT_FULL_PENDING or EO52_RESET_TASK_STAT_PARTIAL_PENDING by console (RESET TASKS). */
    /* reset by dispatcher */
    SAPDB_Byte                        TaskStatRecResetPending; 
    SAPDB_Bool                        QueueStatRecResetPending; 

    SAPDB_Bool                      TimeCollectionEnabled;     /* actual time enabled flag (copy of KGS->fTimeMeasurte)*/
    SAPDB_Bool                      MoveTimeCollectionEnabled; /* actual move time enabled flag (copy of   XPARAM(uLoadBalancingCheck))*/
    SAPDB_Bool                      SuppressTimeCollection;    /* suppress time collection flag (fast fast self reschedule) */

    teo00_Longint                     cputim ;
    teo00_Longint                     dirio ;
    teo00_Longint                     disp_counter ;
    SAPDB_ULong                       ulBegExclCalls;
    teo00_Longint                     begexcl_count ; /* Count for vbegexcl without dispatcher call */
    teo00_Longint                     command_count ;
    teo00_Longint                     self_susp_cnt ; 
    teo00_Longint                     lWaitForRoot ;
    teo00_Longint                     lWaitForDevno;
    teo00_Longint                     lWaitForTask;

    /*
     *  Resume counter and flag for vsignal.
     *  Vsignal sets 'vsignal_ok' to 1 for 'TRUE' and to -1 for 'FALSE'.
     *  Zero means that no vsignal was issued.
     */
#define MAX_SUSPEND_LEVEL 3
    tsp00_Int4                              resume_count [ MAX_SUSPEND_LEVEL ] ;
#define MAX_RESUMER_PIDS 3
    tsp00_Int4                              totalResumerCount [ MAX_SUSPEND_LEVEL ] ;
    tsp00_TaskId                            resumerPid [ MAX_RESUMER_PIDS ] [ MAX_SUSPEND_LEVEL ] ;

    /*
     *  the index of the task control block in the kernel shared segment
     *  is used as task 'pid' for the database kernel code
     */
    tsp00_Int4                              index ;
    tsp00_Int4                              type ;
    volatile tsp00_Int4                     state ;
    volatile tsp00_Int4                     TaskAlive;  /* 0 Task did not set alive flag recently */
                                                        /* 1 Task set alive flag */
    SAPDB_Bool                              connectable;
    tsp00_Int4                              state_reason ;
    tsp00_Int4                              stack_size ;
    /*
     *  time for task's timeout if still in VRECEIVE
     */
    time_t                                  task_timeout ;
    time_t                                  CmdWaitTime  ;
    time_t                                  wait_time ;

    tsp00_Int4                              signal_count ;
    tsp00_Int4                              vsignal_ok ;
    tsp00_Uint4                             prio_flag ;
    tsp00_Uint4                             save_prio ;
    tsp00_Uint4                             ulDynPrio ;
    tsp00_Uint4                             sid ;
    tsp00_Uint4                             excl_schachtel ; /* UUHH! */
    tsp00_Uint4                             is_in_queue ;
    tsp00_Uint4                             aio_in_progress ;
    tsp00_Int4                              aio_errno ;

    tsp00_Int4                              lVStopCounter;
            /* PTS 1108470 */
    teo07_Thread                            taskThread;
			/* PTS 1111201 */
    teo07_ThreadId                          taskThreadId;

    struct RTE_CoroutineContext *           taskContext; /* used only if coroutines enabled */

    /* PTS 1110829 */
    SAPDB_UInt4 *                   pCurrStackPointer;  /* last stack pointer value on entry in dispatcher */

    /* PTS 1110953 */
    SAPDB_UInt4 *                   maxStackPointer;    /* deepest stack position of all dispatcher calls */
    SAPDB_UInt4                     commandMaxStack;    /* maximum stack usage in bytes per command */
    SAPDB_UInt4                     totalMaxStack;      /* maximum stack usage in bytes over all commands */
    tsp00_PageNo                    lockedPageNo;       /* page number set if suspended via vSuspendForPageLock() */

    union { 
      struct {
        SAPDB_UInt2                   temporary;      /* used for temporary locking */
        SAPDB_UInt2                   longTerm;       /* used for long term locking */
      }                              type;

      SAPDB_UInt4                    val;             /* used for lock checks */
    }                               moveLock;

    SAPDB_UInt2                     lastTimeVar;      /* Value 0 .. TIME_VAL_ARRAY_SIZE */
    SAPDB_UInt8                     runnableMicroSeconds[TIME_VAL_ARRAY_SIZE];
    SAPDB_UInt8                     maxRunnableMicroSeconds[TIME_VAL_ARRAY_SIZE];

    SAPDB_Bool                      fTaskIsWaitingForExclusiveAccess; /* ... to a RWRegion */
    struct TASK_TYPE               *nextTaskInWaitQueue ;
    /* no filler */
}; /* TASK_TYPE */

struct UKP_ASYNC_IO_STRUCT
{
    tsp00_Int4                              aio_in_progress ;
    tsp00_Int4                              aio_complete ;
    tsp00_Uint4                             TotalAIOReq ;
    tsp00_Uint4                             TotalAIOSig ;
};

struct UKP_IO_STRUCT
{
  tsp00_Int4                                ukp_attached_fd ;
  tsp00_Uint4                               single_read ;
  tsp00_Uint4                               single_read_pages ;
  tsp00_Uint4                               single_write ;
  tsp00_Uint4                               single_write_pages ;
  tsp00_Int4                                filler;
};


struct DEV_PCB
{
    teo07_Thread                              thread;
    teo07_ThreadId                            tid;
    tsp00_Int4                                semid;
    volatile tsp00_Int4                       state;
    tsp00_Int4                                cpu_id;
    time_t                                    start_time;
    time_t                                    stopTime;
    volatile tsp00_Int4                       fSemAlreadyPosted;
    /*
     * The entries so far MUST be kept identical to the entries for the
     * ten50_KT_Control
     */
    struct ten50_UKT_Control                  *ukt;
    struct DOUBLY_LINKED                      *request;
/*    INTFUNC_PTR                        io_f[2]; */
    int (*io_f[2])( struct DEV_PCB *, int, char *, long, long, tsp00_Uint4 *);
    teo52_DevIOStatistic                      DevIOStat ;
    volatile struct IOC_QUEUE                 *io_work;
    volatile struct IOC_QUEUE                 *io_request;
    struct         IOC_QUEUE                  *ios;
    /*
     * points to functions to read and write for this threads' devspace
     */
   ALONE_IN_CACHE(dum_before) 
    volatile ALONE_UKP_STRUCT                 alone ;
   ALONE_IN_CACHE(dum_behind) 
    off_t                                     old_io_endpos;
    tsp00_Int4                                fd;  /* file decriptor */
    tsp00_Int4                                dev_type;
    tsp00_Int4                                sfd; /* same file opened with O_SYNC */
    tsp00_Int4                                set_io_error ;
    tsp00_Int4                                devcap;
    tsp00_Int4                                filler;
};

typedef int (*INTFUNC_PTR)( struct DEV_PCB *, int, char *, long, long, tsp00_Uint4 *);

struct DEV_PCB0
{
    teo07_Thread                              thread;
    teo07_ThreadId                            tid;
    tsp00_Int4                                semid;
    volatile tsp00_Int4                       state;
    tsp00_Int4                                cpu_id;
    time_t                                    start_time;
    time_t                                    stopTime;
    volatile tsp00_Int4                       fSemAlreadyPosted;
    /*
     * The entries so far MUST be kept identical to the entries for the
     * ten50_KT_Control
     */
    volatile struct IOC_QUEUE                 *io_work;
    struct dev_info                           *dev ;
    struct         IOC_QUEUE                  *ios;
    volatile struct IOC_QUEUE                 *io_request;
    /*
     * points to functions to read and write for this threads' devspace
     */
   ALONE_IN_CACHE(dum_before) 
    volatile ALONE_UKP_STRUCT                 alone ;
   ALONE_IN_CACHE(dum_behind) 
    /*
     * Dependencies from Taskcluster !!
     */
    tsp00_Int4                                nice;
    tsp00_Uint4                               fix_pri;
    tsp00_Uint4                               dis_pre;
    tsp00_Int4                                filler;
};

/*
 * There is one ten50_UKT_Control for each Kernel Thread ( KT ) and for each
 * User Kernel Thread  ( UKT )
 * 
 * state values only changed by the thread itself !!
 *
 */
typedef struct ten50_UKT_Control
{
    teo07_Thread                       thread;
    teo07_ThreadId                     tid;
    tsp00_Int4                         semid;
    volatile tsp00_Int4                state;
    tsp00_Int4                         cpu_id;
    time_t                             start_time;
    time_t                             stopTime;
    volatile tsp00_Int4                fSemAlreadyPosted;
    /*
     * The entries so far MUST be kept identical to the entries for the
     * ten50_KT_Control
     */
    struct connection_info              *connection ;

/*
 * Used to get a pointer to the current task
 */
    struct TASK_TYPE                    *curr_task;
/*
 * Used to transfer current_queue element between tasks
 */
    struct DOUBLY_LINKED                *curr_quu;
/*
 * This threads first task
 */
    struct         TASK_TYPE            *pFirstTaskCtrl;
    struct         IOC_QUEUE_HEAD       ioc_queue;
    struct         IOC_QUEUE            *iocs;
    struct         IOR_QUEUE_HEAD       aioc_queue;
    struct         IOR_QUEUE            *aiocs;
    struct         TASK_TYPE            *com_queue;
    struct         RUN_QUEUE            *RunQueue;
    /*
     * each KT has its own set of DEVi
     */
    struct UKP_IO_STRUCT                *UKPIO;
    struct XXX_QUEUE_HEAD              **xxx;
    struct TASK_TYPE                    *last_com_task ;

    struct    UKP_ASYNC_IO_STRUCT       aio ;

    ALONE_IN_CACHE(dum_before) 
    volatile ALONE_UKP_STRUCT           exclusive ;
    ALONE_IN_CACHE(dum_behind)

    SAPDB_UInt8                         SleepingStateEnterMicroSeconds; /* - set when UKT is entering sleep state */
    SAPDB_UInt8                         RunningStateEnterMicroSeconds; /* - set when UKT is running again */
    SAPDB_UInt8                         IdleTimeMeasureBeginMicroseconds; /* - set when UKT resets idle statistic */
#ifndef HAS_CONSOLE_PIPE
    SAPDB_UInt8                         SysIdleTimeMeasureBeginMicroSeconds; /* needed for scaling internal microseconds */
#endif
    teo52_AbsoluteTime                  UKTRunningStat; /* - collect absolute running time */
    teo52_AbsoluteTime                  UKTIOWaitStat; /* - collect absolute time spend waiting for i/o */
    SAPDB_Bool                          SleepStatRecResetPending; /* Flag set by console thread, handled and reset by dispatcher entry code */

    /*
     *  the index of the thread in the kernel shared segment
     *  is used as threads 'pid' for the database kernel code
     */
    tsp00_Int4                          index ;
    tsp00_Int4                          stacksize ;
    tsp00_Int4                          io_was_completed;
    /*
     * The entries so far MUST be kept identical to the entries for the
     * ten50_KT_Control
     */
    SAPDB_Bool                          tasksAllowedToMoveTo; /* specifies that a task is allowed to move to or from this UKT. */
    SAPDB_UInt4                         numOfMovableTasks;
    struct ten50_UKT_Control            *moveTaskToUKT;      /*  pointer to the UKT to which a Task  */

    SAPDB_UInt2                         currTimeVar;         /* Value 0 .. TIME_VAL_ARRAY_SIZE */
    SAPDB_UInt8                         taskMoveIntervalStartTime;
    SAPDB_UInt8                         UKTSleepTimeMicroSeconds[TIME_VAL_ARRAY_SIZE];
    SAPDB_UInt8                         runnableMicroSeconds[TIME_VAL_ARRAY_SIZE];
    SAPDB_UInt8                         maxRunnableMicroSeconds[TIME_VAL_ARRAY_SIZE];
    SAPDB_UInt8                         stopWaitForBestFitTaskAt; 
    teo00_Longint                       dispCounterDuringMesIntReq;
    teo00_Longint                       dispCounterDuringSendMoveReq;

    SAPDB_UInt4                         activeTasks;

    SAPDB_UInt4                         numServTasks;
    SAPDB_UInt4                         activeServTasks;

    /* --- connectable tasks */
    SAPDB_UInt4                         numConnectableTasks;
    SAPDB_UInt4                         numUserTasks;
    SAPDB_UInt4                         activeUserTasks;
    SAPDB_UInt4                         numUtilityTasks;
    SAPDB_UInt4                         activeUtilityTasks;
    SAPDB_UInt4                         numEventTasks;
    SAPDB_UInt4                         activeEventTasks;

    tsp00_Int4                          RunQueueLen  ;
    tsp00_Uint8                         disp_counter;
    tsp00_Uint8                         cmd_counter;
    time_t                              ukp_timeout;  /* RND_TIMEOUT rounded value of all tasks timeout */
    tsp00_Uint4                         RegLockSlice;
    tsp00_Uint4                         SelfDispatch;
    tsp00_Uint4                         NumRegWait;
    tsp00_Int4                          io_is_completed;
    tsp00_Uint4                         ukp_is_awaiting;
    tsp00_Uint4                         ActTasksPrioCmdWait ;
    tsp00_Uint4                         ActTasksPrio ;
    tsp00_Uint4                         ActTasksVwait;
    tsp00_Uint4                         ulTotVsuspWhenSleep ;
    tsp00_Uint4                         ulTotVwaitWhenSleep ;
    tsp00_Uint4                         ulTotPrioWhenSleep ;
    tsp00_Int4                          ukp_detach_count ;
    tsp00_Uint4                         allow_single_io ;
    tsp00_Int4                          wait_for_ioproc_io ;

    /*
     * Dependencies from Taskcluster !!
     */
    tsp00_Int4                          nice;
    tsp00_Uint4                         fix_pri;
    tsp00_Uint4                         dis_pre;
    tsp00_Int4                          delay_active ; /* vdelay parameter */
    tsp00_Int4                          delay_prio ;
    tsp00_Int4                          act_com_elem ; /* dispatcher parameter */
    tsp00_Int4                          act_rex_elem ;
    tsp00_Int4                          end_switch_cnt ;

    tsp00_TaskId                        DispatcherLockingTask ;

    /* PTS 1108768 Space for Tracebuffer used for InMemory tracing */
    tsp00_PageAddr                      traceBuffer;
    tsp00_Int4                          traceBufferSizeInPages;

    tsp00_Uint4                         taskSwitches;  /* PTS 1108715 */

    struct RTE_CoroutineContext *       removeTaskContext; /* used only if coroutines enabled */

} ten50_UKT_Control; /* ten50_UKT_Control */

externC ten50_UKT_Control *en88GetThisUkt();

/*
 * used for the requestor, coordinator, timer
 */
typedef struct ten50_KT_Control
{
    teo07_Thread                       thread;
    teo07_ThreadId                     tid;
    tsp00_Int4                         semid;
    volatile tsp00_Int4                state;
    tsp00_Int4                         cpu_id;
    time_t                             start_time;
    time_t                             stopTime;
    volatile tsp00_Int4                fSemAlreadyPosted;
    /*
     * The entries so far MUST be kept identical to the entries for the
     * ten50_UKT_Control
     */
    struct         DOUBLE_Q_HEAD       work;
    /* no filler */
} ten50_KT_Control;

struct N_QUEUE
{
    struct         N_QUEUE          *next;
    struct         TASK_TYPE        *taskid;
    /* no filler */
};

struct SEM_LISTE
{
   ALONE_IN_CACHE(dum_before) 
    volatile ALONE_UKP_STRUCT        sem_exclusive;
   ALONE_IN_CACHE(dum_behind) 
    struct         N_QUEUE          *first;
    struct         N_QUEUE          *last;
    struct         N_QUEUE          *sem ;
};

struct DDB4_SEMA
{
    volatile struct TASK_TYPE     * owner ;
    GUARDWORD( guard_before ) /* 1995-Sep-08 J.B. */
    struct SEM_LISTE                semaliste ;
    GUARDWORD( guard_after  ) /* 1995-Sep-08 J.B. */
    tsp00_Uint4                     get_count ;
    tsp00_Uint4                     begexcl_tas_count ;
    tsp00_Uint4                     endexcl_tas_count ;
    tsp00_Uint4                     collision_count ;
    tsp00_Uint4                     wait_count ;
    tsp00_Uint4                     act_waiting;
    char                            semaname[ALIGN_EO00(sizeof(tsp00_C8),sizeof(void *))];
};

typedef struct dcom_mem_ctrl_record
  {
 ALONE_IN_CACHE(dum_before) 
  volatile ALONE_UKP_STRUCT lExclusive; /* - used to grant exclusive exccess */
 ALONE_IN_CACHE(dum_behind) 
  void*                             lphead;            
  void*                             lpfree;
  void*                             lpdebug;
} DCOM_MEM_CTRL_REC, ten50_DCOMMemCtrl;


/*
   +---------------------------------------------------------------+
   |   TASK MOVE STATISTIC                                         |
   +---------------------------------------------------------------+
*/

struct last_task_move_entry
{
  SAPDB_UInt8                 moveTime;
  struct TASK_TYPE*           task;
  SAPDB_UInt8                 taskMaxRunnableMicroSeconds;
  struct ten50_UKT_Control*   srcUKT;
  SAPDB_UInt8                 srcMaxRunnableMicroSeconds;
  struct ten50_UKT_Control*   dstUKT;
  SAPDB_UInt8                 dstMaxRunnableMicroSeconds;
};

struct last_task_move_record
{
  SAPDB_UInt2                     currEntry;
  struct last_task_move_entry     entries[NUM_OF_TASK_MOVE_STAT_RECS];

  ALONE_IN_CACHE(dum_before) 
   volatile ALONE_UKP_STRUCT      exclusive;
  ALONE_IN_CACHE(dum_behind)
};



struct KERNEL_GLOBAL_SECTION
{
    struct KERNEL_GLOBAL_SECTION    * this_kgs ;
    struct SHARED_SEGMENT           * seg ;
    tsp00_Int4                      * kgsshmid ;
    struct ten50_UKT_Control        * first_kp ;
    struct ten50_UKT_Control        * last_kp ;
    struct DEV_PCB                  * first_dev ;
    struct DEV_PCB                  * first_asdev ;
    struct DEV_PCB                  * last_dev ;
    struct TASK_TYPE                * pFirstTaskCtrl ;
    struct TASK_TYPE                * pLastTaskCtrl ;
    struct TASK_TYPE                * ti ;
    struct TASK_TYPE                * al ;
    struct TASK_TYPE                * tw ;
    struct TASK_TYPE                * cs ;
    struct TASK_TYPE                * ut ;
    struct TASK_TYPE                * first_datawriter ;
    struct TASK_TYPE                * first_event_task ;
    struct TASK_TYPE                * first_backup_task ;
    struct TASK_TYPE                * first_garbage_collector ;
    struct TASK_TYPE                * first_objectlog_writer ;
    struct TASK_TYPE                * first_server ;
    struct TASK_TYPE                * first_user ;

    struct DOUBLE_Q_HEAD            freelist ;
    struct DOUBLY_LINKED            * first_free ;
    struct DOUBLY_LINKED            * last_free ;
    struct XXX_QUEUE_HEAD           * first_xxx ;
    struct XXX_QUEUE_HEAD           * last_xxx ;

    struct DDB4_SEMA                * semaphore ;
    tsp00_Bool                      * dumpflag ;
    tsp00_Bool                      * traceflag ;

    UCHAR                           * dynpool_bot ;
    UCHAR                           * dynpool_ptr ;
    UCHAR                           * dynpool_top ;

    tsp00_PageAddr                  datacache_bot ;
    tsp00_PageAddr                  datacache_ptr ;
    tsp00_PageAddr                  datacache_top ;

    tsp00_PageAddr                  convcache_bot ;
    tsp00_PageAddr                  convcache_ptr ;
    tsp00_PageAddr                  convcache_top ;

    tsp00_PageAddr                  dyndata_bot ;
    tsp00_PageAddr                  dyndata_ptr ;
    tsp00_PageAddr                  dyndata_top ;

    struct connection_info          * connection ;

    void                            * alone_thread;
    struct rte_xparam_record        * XParam ;
    void                            * Eventing;
    /* PTS 1103824 */
    void                            * MessageBufferSpace;

    struct ten50_KT_Control         coord ;
    struct ten50_KT_Control         console ;
    struct ten50_KT_Control         timer ;
    struct ten50_KT_Control         requestor ;
    struct DEV_PCB0                 dev0 ;

    DCOM_MEM_CTRL_REC               DCOM_MEM;   /* - anchor for DCOM hashing */

  ALONE_IN_CACHE(dum_before) 
    volatile ALONE_UKP_STRUCT       attach_detach_alone;
  ALONE_IN_CACHE(dum_behind) 

    long                            seg_size ;
    long                            kgs_shm_part_len ;

    unsigned long                   ulDBPageShift;

    tsp00_Versionc                  RTEVersion;
    tsp00_Versionc                  KernelVersion;
    tsp00_NodeIdc                   servernode ;
    tsp00_DbNamec                   serverdb ;

    tsp100_VersionID2               RTEVersionID;
    uid_t                           uid ;
    pid_t                           pid ;
    pid_t                           parentPid ;

    /*
     *  Database state
     */
    volatile tsp00_Uint4            state ;
    /*
     *  Database stop/kill mode
     */
    tsp00_Uint4                     mode ;

    tsp00_Int4                      devpsets ;
    tsp00_Int4                      max_asyn_request;
    tsp00_Int4                      maxtapes ;
    tsp00_Int4                      maxdevspaces ;
    tsp00_Int4                      max_ukpio_devs;
    tsp00_Int4                      dev_queue ;

    tsp00_Int4                      ulNumOfTasks;
    tsp00_Int4                      NumOfUserUKTs;

    tsp00_Int4                      codesize ;

    tsp00_Uint4                     restarted ;
    tsp00_Int4                      restart_timeout ;

    time_t                          start_time ;

    /*
     * MWa Spinlock
     */
#ifdef SPINLOCK_NO_MORE
    spinlock_t                      m_locks ;
#endif
    tsp00_Bool                      alone_flag ;
    long                            alone_pcnt ;
    tsp00_Int4                      alone_count ;
    tsp00_Int4                      alone_collision ;

    tsp00_Int4                      ulNumOfUKTs ;
    tsp00_Int4                      dev_blocksize ;
    tsp00_Int4                      kgs_detach_count ;

    /*
     *  Communication information for some threads and tasks.
     */
    tsp00_Int4                        connection_cnt ;

    /*
     * debuglevel can be modified by an external tool
     */
    tsp00_Int4                        ConsoleModifiedKgs ;
    tsp00_Int4                        debuglevel ;
    tsp00_Int4                        fTimeMeasure;
    /*
     * coretaskindex can be set by an external tool to request a core
     * dump of a specific task. The kernel thread should then fork,
     * load the task's stack, and core dump.
     */
    tsp00_Int4                        wantDump ;
    tsp00_Int4                        coretaskindex ;
    tsp00_Int4                        corethreadindex ;
    tsp00_Uint4                       TotalDataIOCnt ;
    tsp00_Uint4                       SuspendReason[MAX_SUSPEND_REASONS] ;

    volatile tsp00_Int4               traceWriterGone ;
    tsp00_Uint4                       ulSingleDebugBreakPos;

  ALONE_IN_CACHE(dum2_before) 
    volatile ALONE_UKP_STRUCT         ExclusiveVallocat;
  ALONE_IN_CACHE(dum2_behind) 

  ALONE_IN_CACHE(dum3_before) 
    volatile ALONE_UKP_STRUCT         ExclusiveVnewbuf;
  ALONE_IN_CACHE(dum3_behind) 

  /* PTS 1108471 Space for Tracebuffer used for InMemory tracing */
  tsp00_PageAddr                      traceBufferSpace;
  tsp00_Int4                          totalTraceBufferSizeInPages;
  tsp00_Int4                          traceNeedsSaving;
  tsp00_Bool                          dumpDiagFiles;   /* PTS 1115180 */
  tsp00_Bool                          pipeCheckWriteDone; /* PTS 1117651 */

  key_t                               ipcBaseKey;   /* Basis key for all ipc resources */

  SAPDB_Bool                          taskMoved;
  SAPDB_UInt2                         currTimeVar;   /* Value 0 .. TIME_VAL_ARRAY_SIZE */
  SAPDB_UInt8                         taskMoveIntervalStartTimes[TIME_VAL_ARRAY_SIZE];
  SAPDB_UInt2                         taskMoveMeasIntervals;
  struct last_task_move_record        lastTaskMoves;
  SAPDB_Bool                          fBusyWaitingForbidden;
};
/* KERNEL_GLOBAL_SECTION */


struct SHARED_SEGMENT
{
    struct SHARED_SEGMENT             * this_seg ;
    struct KERNEL_GLOBAL_SECTION      * kgs ;
    long                              seg_size ;
};
/* SHARED_SEGMENT */

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/

/*===========================================================================*
 *  EXTERNAL VARIABLES                                                       *
 *===========================================================================*/

#ifdef MAIN
#define PUBLIC
#define SEP ,
#define INITLIST( _v ) = _v
#else
#define PUBLIC extern
#define INITLIST( _v )
#endif

PUBLIC struct KERNEL_GLOBAL_SECTION * kgs ;

#ifdef strings
PUBLIC struct sembuf semsnd INITLIST( { 0 SEP  1 SEP 0        } );
PUBLIC struct sembuf semrcv INITLIST( { 0 SEP -1 SEP 0        } );
PUBLIC struct sembuf semrea INITLIST( { 0 SEP  0 SEP 0        } );
PUBLIC struct sembuf semadd INITLIST( { 0 SEP  1 SEP SEM_UNDO } );
#endif

#undef PUBLIC
#undef INITVAL
#ifdef MAIN
#undef SEP
#endif
    
externC void en88TaskContextSwitch( struct TASK_TYPE *pOld, struct TASK_TYPE *pNew );

#endif  /*USE_KGS*/

#endif  /* GEN50_H */
