/*!**************************************************************************

  module      : gen950.h

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: sql_run_definitions
  description : Definitions for sql_run


  last changed: 2001-09-21  17:00
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/

/*-------------------------- includes -----------------------------------*/
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <stdlib.h>

#ifdef WIN32

#include <windows.h>
#include <process.h>
#include <sys/utime.h>
#include <direct.h>

#define F_READ  0200

#else

#include        <unistd.h>
#include        <utime.h>
#include        <errno.h>
#include        <termio.h>
#if ! T31 && ! T33 && ! T35
#include        <termios.h>
#endif
#if SVR4
#include        <wait.h>
#else
#if SCO
#include        <sys/wait.h>
#endif
#endif

#define TERM   "TERM"
#define DBTERM "DBTERM"

#define F_READ  S_IRUSR

#endif
/*-----------------------------------------------------------------------*/

/*-------------------------- global defines -----------------------------*/

#define EXPORT
#define IMPORT extern

#define CNULL  (char *) 0
#define TRUE            1
#define FALSE           0
#define NOTOK          -1
#define NO_TERM     FALSE
#define INIT_TERM    TRUE
#define INIT_WIN        2

#define MAX_DBNAME_LEN 8
#define MAX_HLP_FILES 10
#define MAX_HLP_LINE_LEN 80
#define MAX_PATH_LEN 255

#define HELP_ID  "h"
#define ALTERNATE_HELP_ID "?"
#define NO_HELP         (char **) 0
#define NOT_AVAIL       ""
#define NO_ALTERN       CNULL

/*--------------------- environment-vars using by SQL-DB ---------------*/
#define BIN "bin"
#define PGM "pgm"

#define SQLOPT "SQLOPT"

/*------------------- permissions using by stat / open-------------------*/
#define F_EX 0
#define F_WRITE 0400
#define F_RW 0600
#define F_RWX 0700

#define GETPARAM "getparam"
#define RUNDIRECTORY "RUNDIRECTORY"
#define KERNELDIAGFILE  "_KERNELDIAGFILE"
#define TMP_FILE "getparam.tmp"
#define ADABAS_KERNELS     { "kernel","quickknl","slowknl", "omststknl" }
#define ADABAS_GATEWAYS    { "fstgwada","qckgwada","slwgwada" }
#define ORACLE_GATEWAYS    { "fstgwora","qckgwora","slwgwora" }

/*-------------------------- USAGES for bad parameters ------------------*/
#define PRI_USAGE(x) { { printf x; } }
#define SQL_HELP_HEADER "\ncorrect use of %s is:\n\n"

#define SQL_USAGE "USAGE: "

#define USAGE_CLEAR(progname) \
        PRI_USAGE (("\n%s %s <serverdb>\n", SQL_USAGE, progname ))

#define USAGE_SHOW(progname) \
        PRI_USAGE (("\n%s %s [-c]\n", SQL_USAGE, progname ))


#define USAGE_PARAM(progname) \
      PRI_USAGE (("\n%7s %s [-u <colduser,coldpassword>] [-d <serverdb>]\n", \
                  SQL_USAGE, progname )) ; \
      PRI_USAGE (("%7s %s [<option>]\noption::= -l | -ll | -V | -CLEAR <serverdb>\n", \
                  "", progname )) ;

#define USAGE_PARAM_CLEAR(progname) \
      PRI_USAGE (("\n%s %s -%s <serverdb>\n", \
                  SQL_USAGE, progname, PARAM_OPT_CLEAR ))

#if _WIN32
#define USAGE_SERVER(progname) \
        PRI_USAGE (("\n%s %s [ %s | %s | %s | %s ]\n", SQL_USAGE, progname, \
                  SERVER_OPT_START, SERVER_OPT_STOP,                        \
                  SERVER_OPT_INSTALL, SERVER_OPT_REMOVE ));
#else
#define USAGE_SERVER(progname) \
        PRI_USAGE (("\n%s %s [ %s | %s ]\n", SQL_USAGE, progname, \
                                      SERVER_OPT_START, SERVER_OPT_STOP ));
#endif

/*-------------------------- defines OS/2 and WIN32 -------------------------*/
#ifdef WIN32

#define USAGE_CONS(progname) \
        PRI_USAGE (("\n%s %s [<dbname>] <command>\n", SQL_USAGE, progname ))

#  define       IS_OPT_CHR(c) ( c == '-' || c == '/' )
#  define       TERMIO_INFO             int
#  define       GET_TERMIO(f,t)
#  define       SET_TERMIO(f,t)

#define BEG_ENV_VAR '%'
#define END_ENV_VAR '%'

#define PARAM_CHK   "xpcheck.prt"
#define PARAM_DBNAME_F "xparam.dbn"
#define PARAM_BACKUP_EXT ".qui"
#define WRK_CONFIG "CONFIG"
#define PARAM_TMP   ""
#define LIST_PARAM_SHORT  "cmd /c dir /B"
#define LIST_PARAM_LONG   "cmd /c dir"
#define PARAM_CLEAR "del"

#define SQLDB_START  "x_start" , "xstart", PGM, "strt.exe"       , NO_TERM
#define SQLDB_GWSTART  "x_gwstart" , "xgwstart", PGM, "strt.exe" , NO_TERM
#define SQLDB_STOP   "x_stop"  , "xstop" , PGM, "stp.exe"        , NO_TERM
#define SQLDB_CONS   "x_cons"  , "xcons" , PGM, "cons.exe"       , NO_TERM
#define SQLDB_SHOW   "x_show"  , "xshow" , PGM, "show.exe"       , NO_TERM
#define SQLDB_SERV   "x_server","xserver", PGM, "serv.exe"       , NO_TERM
#define SQLDB_PARAM  "x_param" , "xparam", PGM, "param.exe"      , INIT_TERM
#define SQLDB_DIAG   "x_diag" , NO_ALTERN, PGM, "diagnose.exe"   , INIT_TERM
#define SQLDB_QUERY  "xquery" , NO_ALTERN, PGM, "query.exe"      , INIT_TERM
#define SQLDB_LOAD   "xload"  , NO_ALTERN, PGM, "load.exe"       , INIT_TERM
#define SQLDB_DOMAIN "xdomain", NO_ALTERN, PGM,     \
                         "sqlpl.exe\0tplrun.exe\0plrun.exe"      , INIT_TERM
#define SQLDB_EASY   "xeasy"  , NO_ALTERN, PGM, "easy.exe"       , INIT_TERM
#define SQLDB_PL     "xpl"    , NO_ALTERN, PGM, "sqlpl.exe"      , INIT_TERM
#define SQLDB_PLRUN  "xplrun" , NO_ALTERN, PGM, "plrun.exe"      , INIT_TERM
#define SQLDB_TOPL   "xtopl"  , NO_ALTERN, PGM, "topl.exe"       , INIT_TERM
#define SQLDB_TOPLR  "xtplrun", NO_ALTERN, PGM, "tplrun.exe"     , INIT_TERM
#define SQLDB_USER   "xuser"  , NO_ALTERN, PGM, "userx.exe"      , INIT_TERM
#define SQLDB_AUDIT  "xaudit" , NO_ALTERN, PGM, "audit.exe"      , INIT_TERM
#define SQLDB_CPC    "cpc"    , NO_ALTERN, PGM, "callpc3.exe"    , NO_TERM
#define SQLDB_CNTRL "xcontrol", NO_ALTERN, PGM, "plrun.exe"      , INIT_TERM
#define SQLDB_CLEAR  NOT_AVAIL ,NO_ALTERN, BIN, "x_clear.exe"    , NO_TERM
#define SQLDB_CI     "xci"    , NO_ALTERN, PGM, "slowci.exe"     , INIT_TERM
#define SQLDB_CI_O   "xoldci" , NO_ALTERN, PGM, "oldci.exe"      , INIT_TERM
#define SQLDB_FILTER "xsql"   , NO_ALTERN, PGM, "sqlfilt.exe"    , NO_TERM
#define SQLDB_FILTER_RO "xsqlro", NO_ALTERN, PGM, "sqlfiltn.exe" , NO_TERM
#define SQLDB_MAKETI NOT_AVAIL, NO_ALTERN, BIN, "x_maketi.exe"   , NO_TERM
#define SQLDB_VTTEST NOT_AVAIL, NO_ALTERN, PGM, "vttest.exe"     , NO_TERM

#define SQLDB_SERVICE  "x_service" , "xservice", PGM, "service.exe", NO_TERM
#define SQLDB_BACKUP   "xbackup"   , NO_ALTERN , PGM, "backup.exe",  NO_TERM
#define SQLDB_RESTORE  "xrestore"  , NO_ALTERN , PGM, "restore.exe", NO_TERM

#define SQLDB_KERNPROT "xkernprot" , NO_ALTERN , PGM, "diagnose.exe", INIT_TERM
/*------------------------- function prototypes  -----------------------*/

int e952_call_start(struct program_id*,int, char**);
int e952_call_gwstart(struct program_id*,int, char**);
int e952_call_stop(struct program_id*,int, char**);
int e952_call_console(struct program_id*,int, char**);
int e952_call_show(struct program_id*,int, char**);
int e951_call_param(struct program_id*,int, char**);
int e952_call_vserver(struct program_id*,int, char**);
int e953_call_domain(struct program_id*,int, char**);
int e953_call_control(struct program_id*,int, char**);
int e950_call_component(struct program_id*,int, char**);
int e952_call_sqlfilter(struct program_id*,int, char**);
int e952_call_backup(struct program_id*,int, char**);
int e952_call_restore(struct program_id*,int, char**);
int e953_call_kernprot(struct program_id*,int, char**);

#else

/*-------------------------- defines UNIX only --------------------------*/


#define USAGE_CONS(progname) \
        PRI_USAGE (("\n%s %s <dbname> [ command [ interval [ count ] ] ] \n", \
                   SQL_USAGE, progname ))


#define IS_OPT_CHR(c) ( c == '-' )

#if     defined(NCCS) && defined(TCSADRAIN)
#  define       TERMIO_INFO             struct termios
#  define       GET_TERMIO(f,t)         tcgetattr(f,t)
#  define       SET_TERMIO(f,t)         tcsetattr(f,TCSADRAIN,t)
#else
#  define       TERMIO_INFO             struct termio
#  define       GET_TERMIO(f,t)         ioctl(f,TCGETA,t)
#  define       SET_TERMIO(f,t)         ioctl(f,TCSETAW,t)
#endif

#define BEG_ENV_VAR '{'
#define END_ENV_VAR '}'

#define PARAM_DBNAME_F "/tmp/xparam.dbname"
#define PARAM_BACKUP_EXT ""

#define DIR_SPOOL_DIAG "/usr/spool/sql/diag"
#define DIR_SPOOL_IPC  "/usr/spool/sql/ipc"
#define RUNNING_DB_ID "db:"

#define WRK_CONFIG "wrk/config"
#define PARAM_CHK   "xpcheck.prot"
#define PARAM_TMP   "/tmp"
#define LIST_PARAM_SHORT  "ls"
#define LIST_PARAM_LONG   "ls -l"
#define PARAM_CLEAR "rm"

#define SQLDB_START  "x_start", NO_ALTERN, PGM, "start"       , NO_TERM
#define SQLDB_GWSTART  "x_gwstart", NO_ALTERN, PGM, "start"   , NO_TERM
#define SQLDB_STOP   "x_stop" , NO_ALTERN, PGM, "stop"        , NO_TERM
#define SQLDB_CLEAR  "x_clear", NO_ALTERN, PGM, "clr_kernel\0clr_ps_ipc",NO_TERM
#define SQLDB_CONS   "x_cons" , NO_ALTERN, PGM, "console"     , NO_TERM
#define SQLDB_SERV   "x_server",NO_ALTERN, PGM, "vserver"     , NO_TERM
#define SQLDB_PARAM  "x_param" ,NO_ALTERN, PGM, "param"       , INIT_TERM
#define SQLDB_QUERY  "xquery" , NO_ALTERN, PGM, "query"       , INIT_TERM
#define SQLDB_LOAD   "xload"  , "xdbload", PGM, "load"        , INIT_TERM
#define SQLDB_EASY   "xeasy"  , NO_ALTERN, PGM, "easy"        , INIT_TERM
#define SQLDB_PL     "xpl"    , NO_ALTERN, PGM, "sqlpl"       , INIT_TERM
#define SQLDB_PLRUN  "xplrun" , NO_ALTERN, PGM, "plrun"       , INIT_TERM
#define SQLDB_TOPL   "xtopl"  , NO_ALTERN, PGM, "topl"        , INIT_TERM
#define SQLDB_TOPLR  "xtplrun", NO_ALTERN, PGM, "tplrun"      , INIT_TERM
#define SQLDB_AUDIT  "xaudit" , NO_ALTERN, PGM, "audit"       , INIT_TERM
#define SQLDB_DIAG   "x_diagnose", NO_ALTERN, PGM, "diagnose" , INIT_TERM
#define SQLDB_CI     "xci"    , NO_ALTERN, PGM, "slowci"      , INIT_TERM
#define SQLDB_CI_O   "xoldci" , NO_ALTERN, PGM, "oldci"       , INIT_TERM
#define SQLDB_MAKETI "x_maketi",NO_ALTERN, PGM, "maketi"      , INIT_TERM
#define SQLDB_VTTEST "xvttest", NO_ALTERN, PGM, "vttest"      , INIT_TERM
#define SQLDB_DOMAIN "xdomain", NO_ALTERN, PGM, "sqlpl\0tplrun\0plrun",INIT_TERM
#define SQLDB_USER   "xuser"  , NO_ALTERN, PGM, "userx"       , INIT_TERM
#define SQLDB_CNTRL "xcontrol", NO_ALTERN, PGM, "plrun"       , INIT_TERM
#define SQLDB_FILTER "xsql"   , NO_ALTERN, PGM, "sqlfilter"   , NO_TERM
#define SQLDB_FILTER_RO "xsqlro", NO_ALTERN, PGM, "sqlfiltern"   , NO_TERM
#define SQLDB_SHOW   "x_show" , NO_ALTERN, BIN, "x_show"      , NO_TERM
#define SQLDB_CPC    NOT_AVAIL, NO_ALTERN, BIN, "cpc"         , NO_TERM

#define SQLDB_KERNPROT "xkernprot", NO_ALTERN, PGM, "diagnose", INIT_TERM

int e952_call_start() ;
int e952_call_gwstart() ;
int e952_call_stop() ;
int e952_call_console() ;
int e952_call_show() ;
int e951_call_param() ;
int e952_call_vserver() ;
int e953_call_domain() ;
int e953_call_control() ;
int e950_call_component() ;
int e952_call_sqlfilter() ;
int e952_call_clear() ;
#if SUNOS || ( SUN && SVR4 )
int e953_call_ci() ;
int e953_call_user() ;
int e953_call_audit() ;
int e953_call_utility() ;
int e953_call_diagnose() ;
#endif
int e953_call_kernprot() ;

#endif


/*------------------------- structure definitions -----------------------*/

#ifdef _WIN32
 typedef int             (* INT_FUNC_PTR  )(struct program_id*,int, char**);
#else
 typedef int             (* INT_FUNC_PTR  )();
#endif

typedef struct          program_id
                        { char *progid , *alter_progid;
                          char *subpath;
                          char *progname;
                          int init_term ;
                          INT_FUNC_PTR call_func ;
                          char **prog_helpfiles ;
                        } PROG_ID ;

/*-----------------------------------------------------------------------*/


/*------------------- global vars for MAIN-MUDUL only -------------------*/
#ifdef DEF_PROGLIST

static char *start_hlpfiles[]= { "xstart.use", CNULL } ;
static char *gwstart_hlpfiles[]= { "xgwstart.use", CNULL } ;
static char *stop_hlpfiles[] = { "xstop.use", CNULL } ;
static char *param_hlpfiles[]= { "xp.use", CNULL } ;
static char *query_hlpfiles[]= { "general.use", "qu.use", CNULL } ;
static char *load_hlpfiles[] = { "general.use", "lo.use", CNULL } ;
static char *dom_hlpfiles[]  = { "general.use", CNULL } ;
static char *easy_hlpfiles[] = { "general.use", "ye.use", CNULL } ;
static char *pl_hlpfiles[]   = { "general.use", "plrun.use",
                                "plimp.use"  , "plexp.use", CNULL } ;
static char *plrun_hlpfiles[] = { "general.use", "plrun.use", CNULL } ;
static char *topl_hlpfiles[]  = { "general.use", CNULL } ;
static char *tplrun_hlpfiles[]= { "general.use", "plrun.use",
                                "plimp.use"  , CNULL } ;
static char *audit_hlpfiles[] = { "xtau.use" , CNULL } ;
static char *cntrl_hlpfiles[] = { "control.use" , CNULL } ;

#ifdef  WIN32
static char *server_hlpfiles[] = { "serv.use" , CNULL } ;
#else
static char *server_hlpfiles[] = { "vserver.use" , CNULL } ;
#endif

static char *kernprot_hlpfiles[] = { "kernprot.use" , CNULL } ;

static PROG_ID  proglist[] = {
                { SQLDB_QUERY , e950_call_component , query_hlpfiles } ,
                { SQLDB_LOAD  , e950_call_component , load_hlpfiles } ,
                { SQLDB_DOMAIN, e953_call_domain    , dom_hlpfiles } ,
                { SQLDB_EASY  , e950_call_component , easy_hlpfiles } ,
                { SQLDB_PL    , e950_call_component , pl_hlpfiles } ,
                { SQLDB_PLRUN , e950_call_component , plrun_hlpfiles } ,
                { SQLDB_TOPL  , e950_call_component , topl_hlpfiles } ,
                { SQLDB_TOPLR , e950_call_component , tplrun_hlpfiles } ,
                { SQLDB_CONS  , e952_call_console   , NO_HELP } ,
                { SQLDB_SHOW  , e952_call_show      , NO_HELP } ,
                { SQLDB_MAKETI, e950_call_component , NO_HELP } ,
                { SQLDB_VTTEST, e950_call_component , NO_HELP } ,
                { SQLDB_CPC   , e950_call_component , NO_HELP } ,
#if SUNOS || ( SUN && SVR4 )
                { SQLDB_USER  , e953_call_user      , NO_HELP } ,
                { SQLDB_DIAG  , e953_call_diagnose  , NO_HELP } ,
                { SQLDB_AUDIT , e953_call_audit     , audit_hlpfiles } ,
                { SQLDB_CI    , e953_call_ci        , NO_HELP } ,
#else
                { SQLDB_USER  , e950_call_component , NO_HELP } ,
                { SQLDB_DIAG  , e950_call_component , NO_HELP } ,
                { SQLDB_AUDIT , e950_call_component , audit_hlpfiles } ,
                { SQLDB_CI    , e950_call_component , NO_HELP } ,
                { SQLDB_CI_O  , e950_call_component , NO_HELP } ,
#endif
                { SQLDB_START , e952_call_start,      start_hlpfiles} ,
                { SQLDB_GWSTART , e952_call_gwstart,  gwstart_hlpfiles} ,
                { SQLDB_STOP  , e952_call_stop ,      stop_hlpfiles } ,
                { SQLDB_SERV  , e952_call_vserver   , server_hlpfiles } ,
                { SQLDB_PARAM , e951_call_param     , param_hlpfiles } ,
                { SQLDB_CNTRL , e953_call_control   , cntrl_hlpfiles } ,
                { SQLDB_FILTER, e952_call_sqlfilter , NO_HELP } ,
                { SQLDB_FILTER_RO, e952_call_sqlfilter , NO_HELP } ,
#ifdef WIN32
                { SQLDB_SERVICE, e950_call_component, NO_HELP } ,
                { SQLDB_BACKUP , e952_call_backup   , NO_HELP } ,
                { SQLDB_RESTORE, e952_call_restore  , NO_HELP } ,
#else
                { SQLDB_CLEAR , e952_call_clear,      NO_HELP } ,
#endif
                { SQLDB_KERNPROT, e953_call_kernprot, kernprot_hlpfiles } ,
                } ;

#define PROGLIST_SIZE     (sizeof( proglist )/sizeof (PROG_ID))
EXPORT int debug ;
#else
IMPORT int debug ;
#endif
/*-----------------------------------------------------------------------*/

#define DBG0(x)         { if ( debug ) { printf x; } }
