/*!***************************************************************************

  module      : hen78.h

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: DCOM Registration for UNIX
  description : Registration Functions for Class implementation and Kernel.
				
  
  last changed: 1998-09-29  11:22
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1998



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef HEN78_H
#define HEN78_H

/*!
  Chapter: Convention

  description:
    This API handles DCOM Registration function.

	'en78_ComObject' Prefix is used for all routines in this API
	All functions return an enumeration ten78_Result.
	Memory has to allocated by the caller. Input/Output parameter
	must point to sufficient space. This is easy to realize by
	using the predefined types ten78_... only. Strings are always 
	terminated by 0.
 */
/*!
  EndChapter: Convention
 */

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define PROG_ID_PFX_MXEN78 32
#define PROJECT_MXEN78     11 
#define VERSION_MXEN78     12
/*
 * Programmatic Identifier  consists of the three items above plus
 * two delimiters (dots) replacing the 0 bytes... This is the maximum
 * approach...
 * Ferdinand Flohe mentioned a necessary size limitation for AS/400.
 */
#ifdef AS400
#define PROG_ID_MXEN78 20
#else
#define PROG_ID_MXEN78 \
	(PROG_ID_PFX_MXEN78 + PROJECT_MXEN78 + VERSION_MXEN78)
#endif
#define CLSID_MXEN78       39
#define MODULE_NAME_MXEN78 256

/*===========================================================================*
 *  TYPES                                                                    *
 *===========================================================================*/

typedef char ten78_ProgIDPfx[PROG_ID_PFX_MXEN78];
typedef char ten78_Project[PROJECT_MXEN78];
typedef char ten78_Version[VERSION_MXEN78];
typedef char ten78_ProgID[PROG_ID_MXEN78];
typedef char ten78_ClsID[CLSID_MXEN78];
typedef char ten78_ModuleName[MODULE_NAME_MXEN78];
typedef enum ten78_Result {
	success_en78,
    illegal_parameter_en78,
	access_denied_en78,
	doubled_clsid_en78,
	entry_not_found_en78,
	clsid_version_mismatch_en78
} ten78_Result;

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

/* none */

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/

/*!
  Chapter: Registration Functions
 */
/*!  
  Function: sqlComObjectRegister
  
  description:
	This call registers a specified Com Object

    It is possible to register more then one version of the com object.
	In such a case the last registration wins! That means the last 
	registered version overwrites the version independent programmatic
	identifier entry. CLSID must be unique in the system.

  arguments:
	ProgIDPfx  [in]  Project Prefix, i.e. "LiveCache"
    Project    [in]  The Project name identifying the Class
    Version    [in]  The Version
	ClsID      [in]  Class Identifier 
	ModuleName [in]  Name of DLL containing the Class implementation
				
  return value: 
	  The code returned
	<UL>
		<LI>success_en78:           Success: Entry was already in registry
		<LI>entry_not_found_en78:   Success: Entry added to registry
		<LI>illegal_parameter_en78: Input parameter check failed
		<LI>access_denied_en78:     Failed to access registry
		<LI>doubled_clsid_en78:     CLSID already used for other registered project
	</UL>
*/
extern "C" ten78_Result sqlComObjectRegister( 
						      ten78_ProgIDPfx   ProgIDPfx,
						      ten78_Project     Project,
						      ten78_Version     Version,
						      ten78_ClsID       ClsID,
						      ten78_ModuleName  ModuleName );

/*!  
  Function: sqlComObjectUnregister
  
  description:
	This call unregisters a specified Com Object.

    The information registered has to be removed. In the special case, 
	that the current version is removed, there is no other version
	select as current version, but no version is current version.
	Check is done for matching version/ClsID combination. No deletion
	is done if mismatch found.

  arguments:
	ProgIDPfx  [in]  Project Prefix, i.e. "LiveCache"
    Project    [in]  The Project name identifying the Class
    Version    [in]  The Version
	ClsID      [in]  Class Identifier 
				
  return value: 
	  The code returned
	<UL>
		<LI>success_en78:           Success
		<LI>illegal_parameter_en78: Input parameter check failed
		<LI>access_denied_en78:     Failed to access registry (i.e. dones not exist)
		<LI>clsid_version_mismatch_en78: CLSID/ProjectVersion mismatch
		<LI>entry_not_found_en78:   Project not found
	</UL>
*/
extern "C" ten78_Result sqlComObjectUnregister( 
						      ten78_ProgIDPfx ProgIDPfx,
						      ten78_Project   Project,
						      ten78_Version   Version,
						      ten78_ClsID     ClsID );

/*!  
  Function: sqlComObjectGet
  
  description:
	This call retrieves a module name and ClsID usign given programmatic ID.

    This call is used as helper function for the DCOM implementation.

  arguments:
	ProgID     [in]  Programmatic Identifier
    ClsID      [out] Filled with ClsID found
    ModuleName [out] Filled with ModuleName found
				
  return value: 
	  The code returned
	<UL>
		<LI>success_en78:           Success
		<LI>illegal_parameter_en78: Input parameter check failed
		<LI>access_denied_en78:     Failed to access registry (i.e. dones not exist)
		<LI>entry_not_found_en78:   Programmatic Identifier not found
	</UL>
*/
extern "C" ten78_Result sqlComObjectGet(
						      ten78_ProgID     ProgID,
						      ten78_ClsID      ClsID,
						      ten78_ModuleName ModuleName);

/*!  
  Function: sqlComObjectGetByClsID
  
  description:
	This call retrieves a module name using given ClsID

    This call is used as helper function for the DCOM implementation.
    Used by Kernel or Proxy server only.

  arguments:
	ClsID      [in]  Class Identifier 
    ModuleName [out] Filled with ModuleName found
				
  return value: 
	  The code returned
	<UL>
		<LI>success_en78:           Success
		<LI>illegal_parameter_en78: Input parameter check failed
		<LI>access_denied_en78:     Failed to access registry (i.e. dones not exist)
	    <LI>invalid_clsid_en78:     ClsID not found
	</UL>
*/
extern "C" ten78_Result sqlComObjectGetByClsID(
						      ten78_ClsID      ClsID,
						      ten78_ModuleName ModuleName );

/*!
  EndChapter: Registration Functions
 */
#endif  /* HEN78_H */
