/*!**************************************************************************

  module      : ven51.c

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Kernel RunTime
  description : Kernel Initialization Functions

  last changed: 2001-09-21  12:09
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/
#define MOD__  "ven51.c:"

#include "gen00.h"
#include "geo50_0.h"
#include "geo00_2.h"
/*
 *  DEFINES
 */

/* exported functions */

#include "heo51.h"
#include "hen51.h"

/* imported functions */

#include "heo46.h"
#include "gen71.h"
#include "gen72.h"
#include "gen73.h"
#include "gen75.h"
#include "gen81.h"
#include "gen88.h"
#include "hen50.h" /* vabort () */
#include "hen56.h" /* vsleep () */
#include "hen40.h" /* sqlabort () */
#include "geo007_1.h"
#include "heo15.h"
            /* PTS 1108470 */
/* after dynamic thread allocation is implemented.... */
/* include "geo07.h" */ /* sqlendthread () */

/* exported data */

/* used to inform Coordinator that a core dump is wanted */
global volatile int     en51NoCoreDumpWanted = 0;
volatile int en51_NeverStop = 1;

/* local functions */

static  void    en51_start_single_task   ( struct TASK_TYPE *tcb );
static  void	en51_start_multiple_tasks( struct TASK_TYPE *first,
										   unsigned long     count );
/* #define TEST_MESSAGE_FILE Jawoll */
#ifdef TEST_MESSAGE_FILE
static  void    en51_ProduceSomeMessages( void );
#endif

/*
 * =====================================================================
 */
#undef  MF__
#define MF__ MOD__"vinitok"
void    vinitok ( )
{
    int                            i;
    struct TASK_TYPE             * tcb;
    ten50_UKT_Control            * lkp;
    ten50_UKT_Control            * this_ukt = THIS_UKT_CTRL;

    DBGIN_T (this_ukt->curr_task->index );

    if ( this_ukt->curr_task->type != TT_TI_EO00 )
    {
        MSGCD (( ERR_VINITOK_NOT_TIMEOUT ));
        vabort ( WRITE_CORE );
    }

    /*
     *  wake all non user kernel tasks
     */
	en51_start_single_task( KGS->al );
	en51_start_single_task( KGS->tw );
	en51_start_single_task( KGS->ut );

	en51_start_multiple_tasks( KGS->first_datawriter, 
								XPARAM(ulMaxDataWriter) );
	en51_start_multiple_tasks( KGS->first_event_task, 
								XPARAM(ulMaxEventTasks) );
	en51_start_multiple_tasks( KGS->first_backup_task, 
								XPARAM(ulMaxBackupTasks) );
	en51_start_multiple_tasks( KGS->first_garbage_collector, 
								XPARAM(ulMaxGarbageCollector) ) ;

    /*
     *  wake all server kernel tasks
     */
	if ( XPARAM(ulMaxServer) > 0 ) 
	{
		/*
		 *	 Only first must be tested, others are in same state
		 */
	    if ( KGS->first_server->state == TSK_INITIALIZED_EO00 )
		{
			en51_start_multiple_tasks( KGS->first_server, XPARAM(ulMaxServer) ) ;
		}
	}

    /*
     *  wake all user kernel tasks (only first must be tested, 
	 *  others are in same state)
     */
	if ( XPARAM(ulMaxUser) > 0 
	  && KGS->first_user->state == TSK_INITIALIZED_EO00 )
	{
		en51_start_multiple_tasks( KGS->first_user, XPARAM(ulMaxUser) ) ;
	}

    /*
     *  wake all user kernel processes
     */
    for ( lkp = KGS->first_kp; lkp <= KGS->last_kp ; lkp ++ )
    {
        if ( lkp->semid > 0 )
        {
            e72_wake ( lkp );
        }
    }

    DBGOUT_T ( this_ukt->curr_task->index );
}

/*
 *=======================================================================
 */
#undef  MF__
#define MF__ MOD__"vinit"
void    vinit ( 
tsp_rte_comm_rec ** rte_comm_ptr,
tsp_int2            len )
{
    ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;

    DBGIN_T (this_ukt->curr_task->index);

    if ( len != sizeof(tsp_rte_comm_rec) )
    {
        MSGCD (( ERR_WRONG_COMM_REC_SIZE , len ))
        vabort ( WRITE_CORE );
    }

    switch ( this_ukt->curr_task->type )
    {
    case TT_US_EO00:
        this_ukt->curr_task->task_timeout = MAXTIMEVALUE ;
        break;

    default:
        break;
    }

    memset ( &this_ukt->curr_task->runnableMicroSeconds,    0, sizeof ( this_ukt->curr_task->runnableMicroSeconds ) );
    memset ( &this_ukt->curr_task->maxRunnableMicroSeconds, 0, sizeof ( this_ukt->curr_task->maxRunnableMicroSeconds ) );

    /* - alter task activ counters */
    en71AlterTaskCnt( this_ukt->curr_task, ADD_ACTIVE_COUNTER );

    if (( this_ukt->curr_task->moveLock.type.longTerm != NEVER_MOVE_TASK ) &&
        ( this_ukt->curr_task->connectable ))
    {   /* Set temporary move lock. Will be removed by the first 'vreceive' */
        this_ukt->curr_task->moveLock.type.temporary = 1; 
    }

    /* return address of rte_comm */
    * rte_comm_ptr = & this_ukt->curr_task->rte_comm;

    DBGOUT_T (this_ukt->curr_task->index);
}

/*
 *=======================================================================
 */

#undef  MF__
#define MF__ MOD__"vdebug_break"
void    vdebug_break ( 
tsp_int4 i4DebugBreakPosition )
{
#ifdef DEBUG_RTE
    ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
#endif /* DEBUG_RTE */
  DBGIN;

  if ( i4DebugBreakPosition < 10000 )
  {
    MSGALL (( ERR_WRONG_DEBUG_BREAK_POS, i4DebugBreakPosition, 10000 ));
    DBGOUT;
    return;
  }

  if ( KGS->ulSingleDebugBreakPos == i4DebugBreakPosition )
  {
      KGS->ulSingleDebugBreakPos = 0;

      en81CallDebugger();
  }

  DBGOUT;
  return;
}

/*
 *=======================================================================
 */

#undef  MF__
#define MF__ MOD__"vgetuktid"
void    vgetuktid ( 
tsp00_Int4 *uktId )
{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
    DBGIN_T (this_ukt->curr_task->index);

    *uktId = this_ukt->index;

    DBGOUT_T(this_ukt->curr_task->index);
}

/*
 *=======================================================================
 */

/* void vgetpid(tsp00_TaskId *pid) now implemented in ven88.cpp */

/*
 *=======================================================================
 */

#undef  MF__
#define MF__ MOD__"vgetfirstuserpid"
void    vgetfirstuserpid ( 
PROCESS_ID *pid )
{
#ifdef DEBUG_RTE
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;

	DBGIN_T (this_ukt->curr_task->index);
#endif
    

    *pid = (PROCESS_ID) KGS->first_user->index ;
	DBG1((MF__,"returning pid %ld\n", *pid));
#ifdef DEBUG_RTE
    DBGOUT_T(this_ukt->curr_task->index);
#endif
}

/*
 *=======================================================================
 */

#undef  MF__
#define MF__ MOD__"vptype"
void    vptype ( 
PROCESS_ID pid ,
tsp2_process_type *proc_type )
{
        ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
    DBGIN_T (this_ukt->curr_task->index);

    if ( (long) pid != (long) this_ukt->curr_task->index )
    {
        MSGD (( ERR_VXXXX_WRONG_TASK , "vptype", (long) pid ));
    }

    switch ( this_ukt->curr_task->type )
    {
    case TT_UT_EO00:
        *proc_type = sp2pt_utility;
        break;
    case TT_TI_EO00:
        *proc_type = sp2pt_timeout;
        break;
    case TT_US_EO00:
        *proc_type = sp2pt_user;
        break;
    case TT_AL_EO00:
        *proc_type = sp2pt_log_writer;
        break;
    case TT_TW_EO00:
        *proc_type = sp2pt_trace_writer;
        break;
    case TT_SV_EO00:
        *proc_type = sp2pt_server;
        break;
    case TT_DW_EO00:
        *proc_type = sp2pt_data_writer;
        break;
    case TT_EV_EO00:
        *proc_type = sp2pt_event;
        break;
    case TT_GC_EO00:
        *proc_type = sp2pt_garbage_collector;
        break;
    case TT_BUP_EO00:
        *proc_type = sp2pt_backup;
        break;
    default:
        MSGCD (( ERR_VPTYPE_WRONG_TASK_TYPE, this_ukt->curr_task->type )) ;
        vabort ( WRITE_CORE );
        break;
    }

    DBG1 (( MF__,"returning %d \n", * proc_type ));

	DBGOUT_T (this_ukt->curr_task->index);
}

/*
 *=======================================================================
 */

#undef  MF__
#define MF__ MOD__"vversion"
void    vversion ( tsp00_Version kernelversion, 
				   tsp00_Version rteversion )
{
    static int everCalled = 0;
#ifdef DEBUG_RTE
    ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
    DBGIN_T(this_ukt->curr_task->index);
#endif /* DEBUG_RTE */

    eo46PtoC ( KGS->KernelVersion, kernelversion, sizeof(tsp00_Version) );
    eo46CtoP ( rteversion, KGS->RTEVersion, sizeof(tsp00_Version) );
    if ( 0 == everCalled )
    {
        everCalled = 1;
        if ((strncmp ( XPARAM(szKernelVersion) + BEG_OF_1ST_VERS_SUBSTR_SP100,
		 KGS->KernelVersion + BEG_OF_1ST_VERS_SUBSTR_SP100,
		 END_OF_1ST_VERS_SUBSTR_SP100 - BEG_OF_1ST_VERS_SUBSTR_SP100  + 1 )) ||
            (strncmp ( XPARAM(szKernelVersion) + BEG_OF_2CND_VERS_SUBSTR_SP100,
		 KGS->KernelVersion + BEG_OF_2CND_VERS_SUBSTR_SP100,
		 END_OF_2CND_VERS_SUBSTR_SP100 - BEG_OF_2CND_VERS_SUBSTR_SP100  + 1 )))
        {
            MSGALL (( ERR_PARAM_AND_KERNEL_INCOMP ,
                   KGS->KernelVersion , XPARAM(szKernelVersion) ))
            vabort(!WRITE_CORE);
        }
        else
        {
            MSGCD (( INFO_VERSION , KGS->KernelVersion ));
        }
    }
#ifdef DEBUG_RTE
    DBGOUT_T(this_ukt->curr_task->index)
#endif /* DEBUG_RTE */
}

/*
 * =======================================================================
 */
#undef  MF__
#define MF__ MOD__"vutil_startup"
tsp1_utility_startup_Param  vutil_startup( )
{
#ifdef DEBUG_RTE
ten50_UKT_Control * this_ukt = THIS_UKT_CTRL;
	DBGIN_T(this_ukt->curr_task->index);
	DBGOUT_T(this_ukt->curr_task->index);
#endif
    return (sp1us_connect);
}

/*
 * =======================================================================
 */

#undef  MF__
#define MF__ MOD__"vrestart"
/*ARGSUSED*/ /* sysdevspace unused */
void    vrestart ( 
PROCESS_ID  pid ,
tsp_devname sysdevspace )
{
ten50_UKT_Control * this_ukt = THIS_UKT_CTRL;
	DBGIN_T(this_ukt->curr_task->index);

    if ( (long) pid != (long) this_ukt->curr_task->index )
    {
        MSGD (( ERR_VXXXX_WRONG_TASK, "vrestart", (long) pid ));
    }

    KGS->restarted = TRUE;
    KGS->state = SERVER_WARM;

#ifdef TEST_MESSAGE_FILE
    en51_ProduceSomeMessages();
#endif /* TEST_MESSAGE_FILE */

    MSGD (( INFO_CHANGE_STATE, "ONLINE", KGS->state ));

	DBGOUT_T(this_ukt->curr_task->index);
}

/* PTS 1115383
 *=======================================================================
 */
#undef  MF__
#define MF__ MOD__"voffline"
void    voffline ( PROCESS_ID pid,
                   tsp00_ShutdownMode_Param mode)
{
#undef  MF__
#define MF__ MOD__"voffline"
	ten50_UKT_Control            * this_ukt = THIS_UKT_CTRL;

	DBGIN_T(this_ukt->curr_task->index)

    if ( (long) pid != (long) this_ukt->curr_task->index )
    {
        MSGD (( ERR_VXXXX_WRONG_TASK , "voffline", (long) pid ));
    }

    KGS->state = SERVER_SHUTDOWN;
	MSGD (( INFO_CHANGE_STATE, "SHUTDOWN", KGS->state ));

    if (mode == shtShutdKill_esp00 )
    {
        KGS->dumpDiagFiles = true;
    }

#if defined(LINUX)
	exit(0);	/* LINUX does not support real threading, so _exit(0) will only exit a single clone process... */
#else
	_exit(0);	/* this is the normal end of SAP DB Kernel... */
#endif
}

/*
 * ===========================================================================
 */

void en51Finish()
{
#   undef  MF__
#   define MF__ MOD__"en51Finish"
    ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;

    DBGIN_T (this_ukt->curr_task->index);

    if ( this_ukt->curr_task->state != TSK_INITIALIZED_EO00 )
    {
        if (( this_ukt->curr_task->moveLock.type.longTerm != NEVER_MOVE_TASK ) &&
            ( this_ukt->curr_task->connectable ))
        {
            en71TempMoveLock ( this_ukt->curr_task, 1, true ); /* don't move this task during finish and next startup phase...
                                                               move lock will be initialized during next 'vinit' via 'sql70k_init_task_ctrl'! */
        }

        /* PTS 1110953 */
        if ( XPARAM(showMaxStackUse) )
        {
            vsShowMaxStackUse( this_ukt->curr_task->index, eo92GetTaskTypeName(this_ukt->curr_task->type) );
        }

        /* - alter task activ counters */
        en71AlterTaskCnt( this_ukt->curr_task, SUB_ACTIVE_COUNTER );

        if ( this_ukt->curr_task == KGS->tw )
        {
            /*
            *  Crash server after bufwriter wrote trace (and dump).
            */
            switch ( KGS->state )
            {
            case SERVER_SHUTDOWNKILL :
            case SERVER_SHUTDOWNREINIT :
                KGS->state = SERVER_KILL;
                MSGD (( INFO_CHANGE_STATE, "KILL", KGS->state ));
                /*FALLTHROUGH*/
            case SERVER_KILL :
            case SERVER_STOP :
                en81_NotifyCoordinator(1);
                /* PTS 1108470 */
                while( en51_NeverStop )
                {
                    sqlyieldthread();
                }
                /* PTS 1111155 */
                break;

            default:
                break;
            }

            MSGD (( ERR_VFINISH_TW_DUR_NORM_OP )) ;
        }

        this_ukt->curr_task->state = TSK_INITIALIZED_EO00;

        /* PTS 1108470 */
        /* after dynamic thread allocation is implemented.... */
        /*
        sqlendthread(0);
        */
    }

    DBGOUT_T(this_ukt->curr_task->index)
}


/*
 * ===========================================================================
 */

void    vfinish ( )
{
#   undef  MF__
#   define MF__ MOD__"vfinish"

    void en51Finish();
}
/*
 * ===========================================================================
 */

void vmovelock ( PROCESS_ID  pid )
{
  ten50_UKT_Control            * this_ukt = THIS_UKT_CTRL;
  /* set temporary move lock */
  if ( (long) pid != (long) this_ukt->curr_task->index )
  {
    MSGD (( ERR_VXXXX_WRONG_TASK , "vmovelock", (long) pid ));
  }

  en71TempMoveLock ( this_ukt->curr_task, 1, true );
}

/*
 * ===========================================================================
 */

void vmoveunlock ( PROCESS_ID  pid )
{
	ten50_UKT_Control            * this_ukt = THIS_UKT_CTRL;

  if ( (long) pid != (long) this_ukt->curr_task->index )
  {
    MSGD (( ERR_VXXXX_WRONG_TASK , "vmoveunlock", (long) pid ));
  }

  /* remove temporary move lock */
  en71TempMoveLock ( this_ukt->curr_task, 1, false );
}


/*
 * ===========================================================================
 */

#undef  MF__
#define MF__ MOD__"vos"
void    vos ( UINT1 *os )
{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;

    DBGIN_T (this_ukt->curr_task->index);

    *os = os_unix ;

    DBGOUT_T (this_ukt->curr_task->index);
}

/*
 * ========================== LOCAL FUNCTIONS =================================
 */

#undef  MF__
#define MF__ MOD__"en51_start_single_task"
static  void en51_start_single_task( struct TASK_TYPE *tcb )
{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
	struct DOUBLY_LINKED            * lquu;

	DBGIN
	lquu           = e73_dl_dequ ( & KGS->freelist );
	lquu->req_type = REQ_INITOK;
	en71_ioc ( tcb->ukt, tcb, lquu );
	DBGOUT
}

/*---------------------------------------------------------------------------*/

#undef  MF__
#define MF__ MOD__"en51_start_multiple_tasks"
static  void en51_start_multiple_tasks( struct TASK_TYPE *first,
										unsigned long     count )
{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;

	DBGIN
	if ( count > 0 )
	{
		unsigned long i;
		struct TASK_TYPE				* tcb;

		for ( i = 0, tcb = first; i < count; i++, tcb++ )
		{
			en51_start_single_task( tcb );
		}
	}
	DBGOUT
}

/*---------------------------------------------------------------------------*/

#ifdef TEST_MESSAGE_FILE
#include <heo15.h>
#undef  MF__
#define MF__ MOD__"en51_ProduceSomeMessages"
static  void    en51_ProduceSomeMessages( void )
{
    tsp00_VfReturn_Param ReturnStatus;
    tsp00_Int4 FileIndex;

    FileIndex = eo15OpenMessageFile("WarmMessage.txt", UNLIMITED_FILE_SIZE_EO15, &ReturnStatus);
    if ( vf_ok == ReturnStatus )
    {
        eo15WriteMessageFile(FileIndex, "Server is online\n", &ReturnStatus);
        eo15WriteMessageFile(FileIndex, "Don't let it get too hot...\n", &ReturnStatus);
        eo15WriteMessageFile(FileIndex, "Just a joke ;-)\n", &ReturnStatus);
    }

    FileIndex = eo15OpenMessageFile("MoreMessage.txt", 1024*8, &ReturnStatus);
    if ( vf_ok == ReturnStatus )
    {
        eo15WriteMessageFile(FileIndex, "Testing Message File output in Kernel\n", &ReturnStatus);
    }
    eo15CloseMessageFile(FileIndex, &ReturnStatus);
}
#endif
