/*!**************************************************************************

  module      : ven52.c

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Kernel RunTime
  description : date_time

                vclock, vdattime, vtimestamp

  last changed: 2001-09-21  12:04
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/
#define MOD__ "ven52.c:"

#include "gen00.h"
#include "heo00.h"
#include "geo50_0.h"

/* exported functions */

#include "gen52.h"
#include "geo007_1.h"
#include "RunTime/System/RTESys_Time.h"
#include "RunTime/System/RTESys_MicroTime.h"

#define EN52_CENTURY_ADJUST 1900
#define EN52_MICROSECONDS_PER_SECOND (1000000U)
/*
 * ===========================================================================
 */
#undef MF__
#define MF__ MOD__"en52Localtime_r"
externC struct tm *en52Localtime_r(const time_t *clock, struct tm *res)
{
    return localtime_r(clock, res);
}

#undef MF__
#define MF__ MOD__"en52GMTime_r"
static struct tm *en52_GMTime_r(const time_t *clock, struct tm *res)
{
    return gmtime_r(clock, res);
}

/*
 * ===========================================================================
 */
#undef MF__
#define MF__ MOD__"e52_init_time"
externC	void	e52_init_time ( )
{
    time_t startTime;
    /*
     *  Get start_time in seconds
     */
    startTime = time((time_t *)0);

    /* Synchronize to beginn of next second */
    for ( KGS->start_time = startTime = time((time_t *)0); KGS->start_time == startTime; )
    {
        KGS->start_time = time((time_t *)0);
    }
    /* Initialize microsecond timer now. This allows a smaller jitter for calculating actual real time */
    RTESys_InitMicroSecTimer();
}


/*
 * ===========================================================================
 */
#undef MF__
#define MF__ MOD__"vclock"
externC	void    vclock ( 
tsp00_Int4			*sec ,
tsp00_Int4			*usec )
{
#ifdef DEBUG_RTE
ten50_UKT_Control *this_ukt = THIS_UKT_CTRL;
#endif

    SAPDB_UInt8 microSeconds = RTESys_MicroSecTimer();

    /*
     *  Return the time since the start of the database.
     */
    *sec =  (tsp00_Int4)(microSeconds / EN52_MICROSECONDS_PER_SECOND);
    *usec = (tsp00_Int4)(microSeconds % EN52_MICROSECONDS_PER_SECOND);

#ifdef DEBUG_RTE
    DBG1 (( MF__,"T%d sec %ld usec %ld \n", (int) this_ukt->curr_task->index ,
		 (long) *sec , (long) *usec ));
#endif
}

/*
 * ===========================================================================
 */
#undef MF__
#define MF__ MOD__"vdattime"
externC	void    vdattime ( 
tsp00_Date dat ,
tsp00_Time tim )
{
#ifdef DEBUG_RTE
ten50_UKT_Control *this_ukt = THIS_UKT_CTRL;
#endif

    time_t              time_sec;
    struct tm			tm_buffer ;
    struct tm			*tmbuf ;
    tsp00_Byte		    *ptr ;
    SAPDB_UInt4         tmpValue;

    time_sec = time((time_t *)0);

    /*
     *  The adjustment to local time is done by 'localtime'.
     */
    tmbuf = en52Localtime_r ( &time_sec, &tm_buffer );

    /*
     * ASCII  '0'
     */
    (void) memset ( dat , '0' , sizeof(tsp00_Date) );
    (void) memset ( tim , '0' , sizeof(tsp00_Time) );

    /*
     *  WARNING: Using "usec" as a temporary variable.
     *           Its value is no longer needed in this function.
     */
    ptr = (&dat[0]) + sizeof(tsp00_Date) ;
		 tmpValue = tmbuf->tm_mday ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue /= 10 ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue = tmbuf->tm_mon + 1 ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue /= 10 ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue = tmbuf->tm_year + EN52_CENTURY_ADJUST ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue /= 10 ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue /= 10 ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue /= 10 ;
    *(-- ptr) += tmpValue %  10 ;

    ptr = (&tim[0]) + sizeof(tsp00_Time) ;
		 tmpValue = tmbuf->tm_sec ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue /= 10 ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue = tmbuf->tm_min ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue /= 10 ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue = tmbuf->tm_hour ;
    *(-- ptr) += tmpValue %  10 ;
		 tmpValue /= 10 ;
    *(-- ptr) += tmpValue %  10 ;

#ifdef DEBUG_RTE
    DBG1 (( MF__,"T%d %.8s %.8s \n", (int) this_ukt->curr_task->index , dat , tim ));
#endif
}

/*-------------------------------------------*/
/* vtimestamp now in veo52.c */
/*-------------------------------------------*/

externC tsp00_Longreal vTimeZoneDelta()
{
   /* Weekdays may differ due to timezone between localtime and GMT */
   /* Since weekday information [0-6] is a range, rather than an absolute */
   /* value, the difference 0 - 6 and 6 - 0 are special cases. To handle these */
   /* cases, the following array is used. Its index is the difference biased by 6 */
   /* to guarantee a non negativ number */
#define SAPDB_WEEKDAY_DIFF_BIAS 6
#define SAPDB_ILLEGAL_WEEKDAY_DIFF -2
   static int weekdayDifference[13] = {
     1, /* -6 = 0 - 6 ->  1 localtime sunday, GMT saturday */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /* -5 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /* -4 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /* -3 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /* -2 = illegal */
    -1, /* -1               localtime one day behind GMT */
     0, /*  0                same day */
     1, /*  1               localtime one day ahead of GMT */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /*  2 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /*  3 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /*  4 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /*  5 = illegal */
    -1  /*  6 = 6 - 0 -> -1 localtime saturday, GMT sunday */
   };
   struct tm gmt;
   struct tm our;
   time_t now;

   now = time((time_t *)0);
   en52Localtime_r(&now, &our); 
   en52_GMTime_r(&now, &gmt);

   if ( weekdayDifference[SAPDB_WEEKDAY_DIFF_BIAS + our.tm_wday - gmt.tm_wday] == SAPDB_ILLEGAL_WEEKDAY_DIFF )
   {
       MSGD(( WRN_TIMEZONE_DELTA, (int)our.tm_wday, (int)gmt.tm_wday ));
       return 0.;
   }

   /* 
      Return difference (localtime - GMT) in hours:
      Weekday difference see above
    */
   return ( (our.tm_min  - gmt.tm_min)/60.
           + our.tm_hour - gmt.tm_hour
           + weekdayDifference[SAPDB_WEEKDAY_DIFF_BIAS + our.tm_wday - gmt.tm_wday] * 24. );
}
