/*!**************************************************************************

  module      : ven55.c

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Kernel RunTime
  description : Mutual Exclusion


  last changed: 2001-09-21  11:42
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/
#define MOD__ "ven55.c:"

#include "gen00.h"
#include "heo00.h"
#include "geo002.h"
#include "geo50_0.h"
#include "gen500.h"
#include "gen55.h"
#include "hen55.h"

/* imported functions */
#include "gen71.h"
#include "hen50.h" /* vabort () */
#include "hen56.h" /* vsleep () */

/* local functions */
#if OSF1_MSEM
#include <c_asm.h>
#define e55_interukp(a,b)  INTERLOCK(a)

#define e55_clearukp(a,b)  { asm(" mb \n stl $31, 0($16) \n",a) ; }
#else

#ifndef INTERLOCK_TREE 
#define e55_clearukp(a,b)  CLEARLOCK(a)
#define e55_interukp(a,b)  INTERLOCK(a)
#else
static  int                             e55_interukp ( );
static  void                            e55_clearukp ( );
#endif
#endif

#define DO_WHEN_DUMMY_REGION(_RegId,_Action) \
        { if ( _RegId == -1 ) _Action ; }

#define MAX_INTERLOCK_LOOPS -1
#define CHECK_INDEFINITELY_LOOP(tas_count, sid, funcname) \ 
                if ( ! -- tas_count )                    \
                  if ( l_curr_task->type == TT_TW &&       \
                       l_curr_task->ukt->activeTasks == 1 )\
                    { \
					  MSGD (( INFO_EXCL_TW_IS_SLEEPING )) \
                      vsleep(l_curr_task->index,1) ;       \
                      tas_count = MAX_INTERLOCK_LOOPS;   \
                    }                                    \
                  else                                   \
                    { \
					  MSGALL (( IERR_EXCL_TW_LOOPS_INDEF, funcname, sid )); \
                      vabort ( WRITE_CORE );       \
                    } 

#define GOTO_REX_AND_RESCHEDULE(funcname) \
         { \
	   l_curr_task->qa.req_type = REQ_TEST_AND_SET ; \
           en71_FastEnquRex ( this_ukt, true ); \
           DBG1 (( funcname,"before goto_disp T%d \n", l_curr_task->index));\
           GOTO_DISP (&this_ukt); \
           DBG1 (( funcname,"after  goto_disp T%d \n", l_curr_task->index));\
         }

#define INSERT_IN_SEMALIST                                                \
        scbs->first->taskid = l_curr_task;                                \
        scbs->first         = scbs->first->next;                          

#define INSERT_IN_SEMALIST_AND_GOTO_DISP(funcname)                         \ 
                  {                                                        \
        INSERT_IN_SEMALIST ;                                               \
        e55_clearukp ( scbs->sem_exclusive, l_curr_task->ukt->index );     \
        scb->wait_count++ ;                                                \
        this_ukt->NumRegWait++ ;                                           \
        DBG1 (( funcname,"before goto_disp T%d \n", l_curr_task->index )); \
        l_curr_task->SuppressTimeCollection = false;                       \
        GOTO_DISP (&this_ukt);                                             \
        l_curr_task->SuppressTimeCollection = true;                        \
        DBG1 (( funcname,"after goto_disp T%d \n", l_curr_task->index ));  \
    }

#define INIT_VBEGEXCL_FUNC(funcname)                                       \
    DO_WHEN_DUMMY_REGION(sid,return);                                      \
    DBG1 (( funcname,"called, T%d, sid =%d\n", l_curr_task->index,sid ));\
    scb = KGS-> semaphore + sid - 1 ;                                        \
    if ( scb->owner == l_curr_task )                                         \
    { MSGALL(( IERR_SECOND_REGION_REQUEST, funcname,sid, scb->semaname)); \
      vabort ( WRITE_CORE ) ;                                              \
    }                                                                      \
    if ( KGS-> traceflag && * KGS-> traceflag && l_curr_task->type == TT_TW )  \
        return;                                                            \
    l_curr_task->state = TSK_VBEGEXCL;                                     \
    l_curr_task->sid   = sid;                                              \
    l_curr_task->begexcl_count++;                                          \
    l_curr_task->ulBegExclCalls++;                                         \
    l_curr_task->excl_schachtel++;                                         \
    l_curr_task->SuppressTimeCollection = true;                            \
    DBG1 (( funcname, "l_curr_task->sid = %d \n", l_curr_task->sid ));     \
    scbs = & scb->semaliste;                                               \
    DBG1 (( funcname,"region   '%.8s' \n", scb->semaname ));               \
    scb->get_count++;  


static  void                            init_semaliste (
struct DDB4_SEMA                * scb);

static  void                            vbegexcl_one_ukp(
PROCESS_ID	pid , REGISTER REGION_ID	sid );
static  void                            vbegexcl_n_ukps_loop(
PROCESS_ID	pid , REGISTER REGION_ID	sid );
static  void  							vbegexcl_n_ukps_noloop (
PROCESS_ID	pid , REGISTER REGION_ID	sid );
static  void                            vbegexcl_dirty_read_loop(
PROCESS_ID	pid , REGISTER REGION_ID	sid );
static  void                            vbegexcl_no_semalist(
PROCESS_ID	pid , REGISTER REGION_ID	sid );
global	VOIDFUNC_PTR           	        vbegexcl_func_ptr =
                                                 vbegexcl_n_ukps_noloop ;

/* imported data */


/* local data */


/*
 *=======================================================================
 */
#undef MF__
#define MF__ MOD__"e55_init_vbegexcl_func_ptr"
void    e55_init_vbegexcl_func_ptr ( 
int nr_of_ukps )
{
	DBGIN;
  
  if ( nr_of_ukps == 1 )
      vbegexcl_func_ptr = vbegexcl_one_ukp ;
  else
      if ( XPARAM(ulRegionCollisionLoop) == 0 )
	  vbegexcl_func_ptr = vbegexcl_n_ukps_noloop ;
      else
        if ( XPARAM(fBegExclDirtyRead) )
          { 
            if ( ! XPARAM(fExclQueue) )
	      vbegexcl_func_ptr = vbegexcl_no_semalist ;
            else
	      vbegexcl_func_ptr = vbegexcl_dirty_read_loop ;
          }
        else
	  vbegexcl_func_ptr = vbegexcl_n_ukps_loop ;

	DBGOUT;
  return ;  
}

/*
 *=======================================================================
 */
#undef MF__
#define MF__ MOD__"init_semaliste"
static  void    init_semaliste ( 
struct DDB4_SEMA                * scb)
{
    int                             i;
    struct N_QUEUE                  * ior1;
    struct N_QUEUE                  * ior2;

	DBGIN;
    DBG1 (( MF__,"called, scb 0x%lx \n", scb ));

    ior1 = & scb->semaliste.sem [ 0 ];
    for ( i = 0; i < KGS-> ulNumOfTasks ; i ++ )
    {
        ior1->taskid  = 0;
        ior2          = ior1 + 1;
        ior1->next    = ior2;
        ior1          = ior2;
    }

    ior1 --;
    ior2 = & scb->semaliste.sem [ 0 ];
    ior1->next = ior2;

    scb->semaliste.first = ior2;
    scb->semaliste.last  = ior2;

#if INTERLOCK_TREE
    for ( i = 0; i < LOCK_INT ; i ++ )
      scb->semaliste.sem_exclusive [ i ] = 0;
#else
    INIT_LOCK(scb->semaliste.sem_exclusive);
#endif

    DBGOUT;
}


/*
 *=======================================================================
 */

#ifdef	OLD_VBEGEXCL

void    vbegexcl ( 
PROCESS_ID		pid ,
REGISTER REGION_ID      sid )
{
    (*vbegexcl_func_ptr) ( pid , sid );
}

#endif	/*OLD_VBEGEXCL*/

/*
 *=======================================================================
 */
#undef MF__
#define MF__ MOD__"vbegexcl_one_ukp"
/*ARGSUSED*/ /* pid unused */
static  void    vbegexcl_one_ukp ( 
PROCESS_ID		pid ,
REGISTER REGION_ID	sid )
{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
    REGISTER struct DDB4_SEMA               * scb;
    REGISTER struct SEM_LISTE               * scbs;
    REGISTER struct TASK_TYPE               * l_curr_task = this_ukt->curr_task;

	DBGIN;

    INIT_VBEGEXCL_FUNC( "vbegexcl_one_ukp" ) ;

    l_curr_task->reg_excl_cnt [ sid-1 ]++;
    if ( ! scb->owner )
    {
        l_curr_task->SuppressTimeCollection = false;
        scb->owner = l_curr_task;
    }
    else
    {
        INSERT_IN_SEMALIST ;

#ifdef DEVELOPMENT_HUGE
        l_curr_task->wait_time = time ((time_t *)0);
#endif

        DBG1 (( MF__,"before goto_disp T%d \n", l_curr_task->index ));
        l_curr_task->SuppressTimeCollection = false;
        GOTO_DISP (&this_ukt);
        DBG1 (( MF__,"after  goto_disp T%d \n", l_curr_task->index ));
        scb->owner->reg_coll_cnt [ sid-1 ]++;
        scb->collision_count++;
    }

    l_curr_task->state = TSK_RUNNING;

    DBGOUT_T(( l_curr_task->index ));
}

/*
 *=======================================================================
 */

#undef MF__
#define MF__ MOD__"vbegexcl_n_ukps_loop"
/*ARGSUSED*/ /* pid unused */
static  void    vbegexcl_n_ukps_loop ( 
PROCESS_ID		pid ,
REGISTER REGION_ID	sid )
{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
    REGISTER struct DDB4_SEMA               * scb;
    REGISTER struct SEM_LISTE               * scbs;
    REGISTER struct TASK_TYPE               * l_curr_task = this_ukt->curr_task;
    REGISTER long                           tas_count = MAX_INTERLOCK_LOOPS;
    long loop_region_coll = XPARAM(ulRegionCollisionLoop) ;
    long region_reserved;
    long reserved_cnt = loop_region_coll;
    long num_coll ; 

	DBGIN;

    INIT_VBEGEXCL_FUNC( "vbegexcl_n_ukps_loop" ) ;

    l_curr_task->reg_excl_cnt [ sid-1 ]++;
    do  /* fh: if we have a multi-cpu machine, loop here */
      {
        while ( e55_interukp( scbs->sem_exclusive, l_curr_task->ukt->index ))
          { 
            CHECK_INDEFINITELY_LOOP(tas_count, sid, "vbegexcl");
            GOTO_REX_AND_RESCHEDULE("vbegexcl_n_ukps_loop"); 
          }  /* end-while interukp */
/*  loop_region_coll is > 0 !!!  	    if ( reserved_cnt ) */
              if ( ( region_reserved = scb->owner != (struct TASK_TYPE *) 0 ) 
                   && --reserved_cnt )
		{ e55_clearukp ( scbs->sem_exclusive , l_curr_task->ukt->index ) ;
		  l_curr_task->qa.req_type = REQ_EXCLUSIVE ;
		  en71_FastEnquRex ( this_ukt, l_curr_task->prio_flag );
          GOTO_DISP (&this_ukt);
        }                 /* else leave the loop */  
      }
    while ( region_reserved && reserved_cnt > 0 ) ;  /* fh: do-while end */

    /*
     * we are alone now
     */
    if ( ! scb->owner )
    {   
        scb->owner = l_curr_task;
        e55_clearukp ( scbs->sem_exclusive, l_curr_task->ukt->index );
    }
    else
      INSERT_IN_SEMALIST_AND_GOTO_DISP("vbegexcl_n_ukps_loop") ;

    scb->begexcl_tas_count += MAX_INTERLOCK_LOOPS - tas_count ;
    if ( ( num_coll = loop_region_coll - reserved_cnt ) )
      { scb->collision_count += num_coll ;
        l_curr_task->reg_coll_cnt [ sid-1 ] += num_coll ;
      }
    l_curr_task->SuppressTimeCollection = false;
    l_curr_task->state = TSK_RUNNING;

    DBGOUT_T (( l_curr_task->index ));
}

/*
 *=======================================================================
 */

#undef MF__
#define MF__ MOD__"vbegexcl_n_ukps_noloop"
/*ARGSUSED*/ /* pid unused */
static  void    vbegexcl_n_ukps_noloop (
PROCESS_ID		pid ,
REGISTER REGION_ID	sid )
{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
    REGISTER struct DDB4_SEMA               * scb;
    REGISTER struct SEM_LISTE               * scbs;
    REGISTER struct TASK_TYPE               * l_curr_task = this_ukt->curr_task;
    REGISTER long                           tas_count = MAX_INTERLOCK_LOOPS;

	DBGIN;

    INIT_VBEGEXCL_FUNC( "vbegexcl_n_ukps_noloop" ) ;

    l_curr_task->reg_excl_cnt [ sid-1 ]++;
    while ( e55_interukp( scbs->sem_exclusive, l_curr_task->ukt->index ))
      { 
        CHECK_INDEFINITELY_LOOP(tas_count, sid, "vbegexcl_n_ukps_noloop ");
        GOTO_REX_AND_RESCHEDULE("vbegexcl_n_ukps_noloop"); 
      }  /* end-while interukp */
    /*
     * we are alone now
     */
    if ( ! scb->owner )
    {
        scb->owner = l_curr_task;
        e55_clearukp ( scbs->sem_exclusive, l_curr_task->ukt->index );
    }
    else
      { INSERT_IN_SEMALIST_AND_GOTO_DISP("vbegexcl_n_ukps_noloop") ;
        scb->collision_count++; 
        scb->owner->reg_coll_cnt [ sid-1 ]++;
      }
    scb->begexcl_tas_count += MAX_INTERLOCK_LOOPS - tas_count ;
    l_curr_task->SuppressTimeCollection = false;
    l_curr_task->state = TSK_RUNNING;

    DBGOUT_T (( l_curr_task->index ));
}

/*
 *=======================================================================
 */

#undef MF__
#define MF__ MOD__"vbegexcl_no_semalist"
/*ARGSUSED*/ /* pid unused */
static  void    vbegexcl_no_semalist ( 
PROCESS_ID		pid ,
REGISTER REGION_ID	sid )

{
ten50_UKT_Control * this_ukt = THIS_UKT_CTRL;
    struct DDB4_SEMA               * scb = KGS-> semaphore + sid - 1 ;
    struct SEM_LISTE               * scbs= & scb->semaliste;
    struct TASK_TYPE               * l_curr_task = this_ukt->curr_task;
    long                           tas_count = MAX_INTERLOCK_LOOPS;
    long loop_region_coll = XPARAM(ulRegionCollisionLoop) ;
    long reserved_cnt = loop_region_coll, num_coll ; 

	DBGIN;

    INIT_VBEGEXCL_FUNC( "vbegexcl_dirty_read_loop" ) ;

    l_curr_task->reg_excl_cnt [ sid-1 ]++;

    do  /* fh: if we have a multi-cpu machine, loop here */
      {
        if ( scb->owner != (struct TASK_TYPE *) 0  ) 
        {
            --reserved_cnt ;
            l_curr_task->qa.req_type = REQ_EXCLUSIVE ;
            en71_FastEnquRex ( this_ukt, reserved_cnt > 0 || l_curr_task->excl_schachtel > 1 ) ;
            GOTO_DISP (&this_ukt);
        }                 /* else leave the loop */  
        else
          { while ( e55_interukp( scbs->sem_exclusive, l_curr_task->ukt->index ))
              { 
                CHECK_INDEFINITELY_LOOP(tas_count, sid, "vbegexcl");
      	        GOTO_REX_AND_RESCHEDULE("vbegexcl_no_semalist"); 
              }  /* end-while interukp */
              if ( scb->owner == (struct TASK_TYPE *) 0 )
              { 
                  scb->owner = l_curr_task;
                  e55_clearukp ( scbs->sem_exclusive, l_curr_task->ukt->index );
                  break ;
              }
              else
              { 
                  e55_clearukp ( scbs->sem_exclusive , l_curr_task->ukt->index );
                  --reserved_cnt ;
                  l_curr_task->qa.req_type = REQ_EXCLUSIVE ;
	              en71_FastEnquRex ( this_ukt, reserved_cnt > 0 || l_curr_task->excl_schachtel > 1 ) ;
                  GOTO_DISP (&this_ukt);
              }  /* else leave the loop */  
          }
      }
    while ( true ) ;

    /*
     * we are alone now
     */

    if ( ( num_coll = loop_region_coll - reserved_cnt ) )
      { scb->collision_count += num_coll ;
        l_curr_task->reg_coll_cnt [ sid-1 ] += num_coll ;
      }
    scb->begexcl_tas_count += MAX_INTERLOCK_LOOPS - tas_count ;
    l_curr_task->SuppressTimeCollection = false;
    l_curr_task->state = TSK_RUNNING;
    DBGOUT_T (( l_curr_task->index ));
}


/*
 *=======================================================================
 */

#undef MF__
#define MF__ MOD__"vbegexcl_dirty_read_loop"
/*ARGSUSED*/ /* pid unused */
static  void    vbegexcl_dirty_read_loop ( 
PROCESS_ID		pid ,
REGISTER REGION_ID	sid )
{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
    struct DDB4_SEMA               * scb = KGS-> semaphore + sid - 1 ;
    struct SEM_LISTE               * scbs= & scb->semaliste;
    struct TASK_TYPE               * l_curr_task = this_ukt->curr_task;
    long                           tas_count = MAX_INTERLOCK_LOOPS;
    long LoopEnd ;
    long loop_region_coll = XPARAM(ulRegionCollisionLoop) ;
    long reserved_cnt = loop_region_coll, num_coll ; 

	DBGIN;

    INIT_VBEGEXCL_FUNC( "vbegexcl_dirty_read_loop" ) ;

    l_curr_task->reg_excl_cnt [ sid-1 ]++;

    do  /* fh: if we have a multi-cpu machine, loop here */
      {
        LoopEnd    = reserved_cnt <= 0 /* || scbs->last->taskid */ ||
                     ( l_curr_task->prio_flag && 
                       XPARAM(fBegExclQueWhenPrio) ) ;    
        if ( scb->owner != (struct TASK_TYPE *) 0 && !LoopEnd ) 
        {
            --reserved_cnt ;
  	        l_curr_task->qa.req_type = REQ_EXCLUSIVE ;
	        en71_FastEnquRex ( this_ukt, reserved_cnt > 0 || l_curr_task->excl_schachtel > 1 ) ;
        GOTO_DISP (&this_ukt);
/*
    	    GOTO_REX_AND_RESCHEDULE("vbegexcl_dirty_read_loop"); 
*/
          }                 /* else leave the loop */  
        else
          { while ( e55_interukp( scbs->sem_exclusive, l_curr_task->ukt->index ))
              { 
                CHECK_INDEFINITELY_LOOP(tas_count, sid, "vbegexcl");
      	        GOTO_REX_AND_RESCHEDULE("vbegexcl_dirty_read_loop"); 
              }  /* end-while interukp */
              if ( scb->owner == (struct TASK_TYPE *) 0 )
                LoopEnd = true ;
              else
              if ( ! LoopEnd )
                { e55_clearukp ( scbs->sem_exclusive , l_curr_task->ukt->index );
                  --reserved_cnt ;
  	          l_curr_task->qa.req_type = REQ_EXCLUSIVE ;
	          en71_FastEnquRex ( this_ukt, reserved_cnt > 0 || l_curr_task->excl_schachtel > 1 ) ;
              GOTO_DISP (&this_ukt);
                }                 /* else leave the loop */  
          }
      }
    while ( !LoopEnd ) ;

    /*
     * we are alone now
     */
    if ( ! scb->owner )
    {   
        scb->owner = l_curr_task;
        e55_clearukp ( scbs->sem_exclusive, l_curr_task->ukt->index );
    }
    else
      INSERT_IN_SEMALIST_AND_GOTO_DISP("vbegexcl_dirty_read_loop") ;

    if ( ( num_coll = loop_region_coll - reserved_cnt ) )
      { scb->collision_count += num_coll ;
        l_curr_task->reg_coll_cnt [ sid-1 ] += num_coll ;
      }
    scb->begexcl_tas_count += MAX_INTERLOCK_LOOPS - tas_count ;
    l_curr_task->SuppressTimeCollection = false;
    l_curr_task->state = TSK_RUNNING;
    DBGOUT_T (( l_curr_task->index ));
}

/*
 *=======================================================================
 */

#undef MF__
#define MF__ MOD__"vendexcl"
/*ARGSUSED*/ /* pid unused */
void    vendexcl ( 
PROCESS_ID		pid ,
REGISTER REGION_ID	sid )

{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
    REGISTER struct DDB4_SEMA   * scb;
    REGISTER struct SEM_LISTE   * scbs;
    REGISTER struct TASK_TYPE   * l_curr_task = this_ukt->curr_task;
    REGISTER long                 tas_count = MAX_INTERLOCK_LOOPS ;

	DBGIN_T(l_curr_task->index);

    l_curr_task->SuppressTimeCollection = true;

    DO_WHEN_DUMMY_REGION( sid  ,return );

    /*JH frickel*/
    if ( KGS-> traceflag && * KGS-> traceflag && l_curr_task->type == TT_TW )
        return;

    l_curr_task->state = TSK_VENDEXCL;

    scb  = KGS-> semaphore + sid - 1;
    scbs = & scb->semaliste;
 
    DBG1 (( MF__,"region   '%.8s' \n", scb->semaname ));
    if ( scb->owner != l_curr_task )
    {
		MSGALL (( IERR_VENDEXCL_WITHOUT_VBEG ,
                   l_curr_task->index,  sid, scb->semaname ));
		vabort ( WRITE_CORE ) ;
    }

    if ( XPARAM(fExclQueue ) )
      while ( e55_interukp( scbs->sem_exclusive, l_curr_task->ukt->index ))
      {
        CHECK_INDEFINITELY_LOOP(tas_count, sid, "vendexcl");
        if ( ! XPARAM(fEndExclBusyWait)  )
	  GOTO_REX_AND_RESCHEDULE("vendexcl"); 
      }
    /*
     * we are alone now
     */

    if ( scbs->last->taskid  )
    {
        /*
         * somebody else is waiting for begexcl
         */
        scb->owner             = scbs->last->taskid  ;
        scbs->last->taskid     = 0;
        scbs->last             = scbs->last->next;
        e55_clearukp ( scbs->sem_exclusive, l_curr_task->ukt->index );
        scb->endexcl_tas_count += MAX_INTERLOCK_LOOPS - tas_count ;
        if ( --l_curr_task->excl_schachtel == 0 )
          l_curr_task->sid = 0;
        if ( scb->owner->ukt == this_ukt )
        {
            this_ukt->NumRegWait-- ;
	    scb->owner->qa.req_type = REQ_EXCLUSIVE ;
            e71_EnquRav ( (struct TASK_TYPE *)scb->owner, true );
	    if ( !l_curr_task->prio_flag )
	      { l_curr_task->qa.req_type = REQ_RESCHEDULE ;
                l_curr_task->state = TSK_RUNNABLE;
	        en71_FastEnquRex ( this_ukt, false ) ;
            GOTO_DISP (&this_ukt);
              }
        }
        else
        {
            en71EnqueueUkt2Ukt ( this_ukt, 
                             (struct TASK_TYPE *)scb->owner, 
                             this_ukt->curr_task,
                             e55_remactive, 
                            (void *)sid );
        }
    }
    else
    {
        /*
         * Queue is empty
         */
        scb->owner               = (struct TASK_TYPE *) 0;
        if ( XPARAM(fExclQueue) )
          { e55_clearukp ( scbs->sem_exclusive, l_curr_task->ukt->index );
            scb->endexcl_tas_count += MAX_INTERLOCK_LOOPS - tas_count ;
          }
        --l_curr_task->excl_schachtel;
        if ( ( l_curr_task->begexcl_count > XPARAM(ulRegLockSlice) ||
                                         /* this_ukt->RegLockSlice */
               this_ukt->SelfDispatch > 0 ) &&
             !l_curr_task->prio_flag && l_curr_task->excl_schachtel == 0)
        {   if ( this_ukt->SelfDispatch > 0 ) this_ukt->SelfDispatch-- ; 
            l_curr_task->state = TSK_RUNNABLE;
            l_curr_task->self_susp_cnt++ ;
	    l_curr_task->qa.req_type = REQ_RESCHEDULE ;
	    en71_FastEnquRex ( this_ukt, false );
        GOTO_DISP (&this_ukt);
        }
    }

/*
    scb->last_owner = l_curr_task ;  
*/
    l_curr_task->SuppressTimeCollection = false;
    l_curr_task->state = TSK_RUNNING;
    l_curr_task->sid   = 0 ;
	DBGOUT_T(l_curr_task->index);
}

#if INTERLOCK_TREE
/*
 *=======================================================================
 */

/* The lock algorithm works on a tree of integers. Each first-level int
 * handles the exclusion between four processes, one proc per byte.
 * At each further level, one int handles the exclusion bewteen the
 * four winners of four previous ints, again one contestant per byte.
 * So the scheme is a four-fold tree of ints, where each leaf level
 * byte corresponds to one process.
 *
 * This tree is stored in an array of ints, the root is at index 0
 * (= bytes 0..3), each level down is stored at higher indexes.
 * For simplicity of index calculations, several elements are unused
 * (including byte 0).
 * Also, the tree is not balanced.
 */
#define IND_WORD(byt_ind)   ( byt_ind / 4 )
#define BYT_POS(byt_ind)    ( byt_ind % 4 )
#define IND_LEV_1(index)    ( LEV_1_INT * 4 + index )
#define IND_LEV_OK(index)   ( index >= 4 )

#undef MF__
#define MF__ MOD__"e55_interukp"
static int e55_interukp ( 
volatile ALONE_UKP_STRUCT a,
tsp_int4         b)
{
    UINT4            lockval;
    UINT4            pat;
    UCHAR            * str;
    int              ind_word_curr_byt;
    int              curr_byt;
    volatile UINT4   * curr_word;
    int              lev;

    DBG1 (( MF__,"entered by P%d\n", b ));
    curr_byt = IND_LEV_1 ( b - 1 );  /* UKPs count from 1, but index from 0 */
    lev = 0;

NOCHMAL:

    pat  = 0xffffffff;        /* compute the mask of competing entries. */
    str  = (UCHAR *) &pat;
    str += BYT_POS ( curr_byt );
    *str = 0x00;
    lev ++;

    DBG1 (( MF__,"level is    %d\n", lev ));
    DBG1 (( MF__,"byte pos is %d\n", curr_byt ));
    DBG1 (( MF__,"pattern is  0x%08x\n", pat ));

    ind_word_curr_byt = IND_WORD ( curr_byt );
    curr_word = & a [ ind_word_curr_byt ];
    str  = (UCHAR *) curr_word;
    str += BYT_POS ( curr_byt );
#ifdef DEBUG_RTE
    lockval = * curr_word;
    DBG1 (( MF__,"before == 0x%08x\n", lockval ));
#endif
    lockval = (* curr_word) & pat;                /* check the others only ! */
    if ( lockval )
    {
		DBG1 (( MF__,"check before ... 0x%08x\n", lockval ));
		DBG1 (( MF__,"... at level %d\n", lev ));
		return ( 1 );
    }
    *str = 0xff;
    lockval = * curr_word;
    DBG1 (( MF__,"after  == 0x%08x\n", lockval ));
    lockval = lockval & pat;
    DBG1 (( MF__,"masked == 0x%08x\n", lockval ));
    if ( lockval )
    {
        *str = 0x0;
        DBG1 (( MF__,"masked == 0x%08x ... at level %d\n", lockval, lev ));
        sleep ( 0 );
        goto NOCHMAL;
    }

    if ( IND_LEV_OK ( curr_byt ) )
    {
        curr_byt = ind_word_curr_byt;
        goto NOCHMAL;
    }

    return ( 0 );
}

/*
 *=======================================================================
 */

static void e55_clearukp ( 
ALONE_UKP_STRUCT a,
tsp_int4         b)
{
    UCHAR            * str;
    int              curr_byt;
    int              ind_word_curr_byt;
    UINT4            * curr_word;

    curr_byt = IND_LEV_1 ( b - 1 ); /* UKPs count from 1, but index from 0 */

NOCHMAL:

    ind_word_curr_byt = IND_WORD ( curr_byt );

    curr_word = & a [ ind_word_curr_byt ];
    str  = (UCHAR *) curr_word;
    str += BYT_POS ( curr_byt );
    *str = 0x00;

    if ( IND_LEV_OK ( curr_byt ) )
    {
        curr_byt = ind_word_curr_byt;
        goto NOCHMAL;
    }
}

/*
 *=======================================================================
 */
#endif /* INTERLOCK_TREE */

/*
 * ===========================================================================
 */
#undef MF__
#define MF__ "e55_remactive"
void    e55_remactive ( 
void *param ,
struct TASK_TYPE *FromTask)
{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
	DBGIN_T(this_ukt->curr_task->index);
    /*
     * this procedure is a dummy, only to ensure that the task is activated
     */
    this_ukt->NumRegWait-- ;
    DBGOUT_T(this_ukt->curr_task->index);
}

/*
 *=======================================================================
 */
#undef MF__
#define MF__ MOD__"visexcl"
tsp00_Bool visexcl ( 
PROCESS_ID		pid ,
REGION_ID               sid)

{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
    struct DDB4_SEMA                * scb;

	DBGIN;

    DO_WHEN_DUMMY_REGION( sid , return(true) );

    if ( sid == -1 )
      return ( this_ukt->curr_task->excl_schachtel != 0 ) ? true : false;

    scb  = KGS-> semaphore + sid - 1;

    if ( scb->owner == this_ukt->curr_task )
    {
        return (true);
    }
    else
    {
        return (false);
    }
	DBGOUT;
}

/*
 *=======================================================================
 */
#undef MF__
#define MF__ MOD__"vsemaphid"
void    vsemaphid ( 
tsp_c8                          semaname,
REGION_ID                       * sid)
{
ten50_UKT_Control               * this_ukt = THIS_UKT_CTRL;
    int                             i;
    struct DDB4_SEMA                * scb;

	DBGIN_T(this_ukt->curr_task->index);
    DBG1 (( MF__,"region   '%.8s' \n", semaname ));

    /*
     * Initialisation already done
     */
    for ( i = 0; i < XPARAM(ulNoOfRegions) ; i ++ )
    {
        if ( KGS-> semaphore [ i ].semaname [ 0 ] == '\0' ) break;
    }
    if ( i >= XPARAM(ulNoOfRegions) )
    {
        MSGCD (( IERR_TOO_MANY_REGIONS_REQ ))  
#ifdef  DEBUG_RTE
        for ( i = 0; i < XPARAM(ulNoOfRegions) ; i ++ )
        {
            DBG1 (( MF__,"region '%.8s' \n", KGS-> semaphore [ i ].semaname ));
        }
#endif
        vabort ( !WRITE_CORE );
    }
    * sid = i + 1;
    scb = & KGS-> semaphore [ i ];
	eo46PtoC ( scb->semaname, semaname, sizeof(tsp00_C8) );
    scb->owner = (struct TASK_TYPE *) 0;
    init_semaliste ( scb );
    
    DBG1 (( MF__,"returning, sid %d \n", * sid ));
	DBGOUT;
}


/*
 *=======================================================================
 */
#undef MF__
#define MF__ MOD__"vsemainfo"
void vsemainfo ( 
REGION_ID                       sid,
tsp_c8                          semaname)
{
  DBGIN;
  if ( sid <= XPARAM(ulNoOfRegions) )
  {
	  eo46CtoP ( semaname, KGS-> semaphore [sid-1].semaname, sizeof(tsp00_C8) );
      DBG1 (( MF__,"region %d '%.8s' \n", sid, KGS-> semaphore [sid-1].semaname ));
  } 
  DBGOUT;
}
