/****************************************************************************

  module      : ven780.cpp

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: DCOM Registry
  description : Test program


  last changed: 2000-02-04  17:45
  see also    : example.html ...

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1998



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <iostream.h>
#include "heo01.h"
#include "hen78.h"
#include "gen780.h"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/

#ifdef TEST

#define ROUTINE_DBG_MEO00(routname)   TRACER_CLASS_MEO00 localTrace (routname)
#define TRACEF_MEO00(args)            localTrace.trace args

#if defined(DEBUG_RTE)

#undef TRACER_CLASS_MEO00
#define TRACER_CLASS_MEO00 teo200_Tracer

#  define ROUTINE_DBG_MEO200(name)    ROUTINE_DBG_MEO00(name)
#  define DBG_MEO200(args) TRACEF_MEO00(args)
#  define FUNCTION_NAME_MEO200 localTrace.currentFuncName()

# else

// fast + no debug

#  define ROUTINE_DBG_MEO200(name) \
    static const char *currentFuncName = #name
#  define DBG_MEO200(args)
#  define FUNCTION_NAME_MEO200 currentFuncName

#endif

class teo200_Tracer
{
private:
	const char *name;
	int   level;
public:

    teo200_Tracer (const char *nameparm, int levelparm =1)
	:name(nameparm),
	level(levelparm)
	{
		printf("%s -->called\n", name);
	}

    ~teo200_Tracer ()
	{
		printf("%s <--returning\n", name);
	}

    void trace (int level, const char *format, ...) const
	{
		va_list args;

		printf("%s:", name);
		va_start(args, format);
		vprintf(format, args);
		va_end(args);
	}

	// for MSG macros...
    const char *currentFuncName() const { return name; }

};

/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/

typedef char ten78_ProgProject[PROG_ID_PFX_MXEN78 + PROJECT_MXEN78];

typedef struct reg_slot {
	ten78_ProgProject ProgProject;	/* Version independent programmatic ID */
	ten78_Version     Version; /* The given version string */
	ten78_ClsID       ClsID;
	ten78_ModuleName  ModuleName;
	int               is_current;
} reg_slot;

/*===========================================================================*
 *  GLOBAL VARIABLES                                                         *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL VARIABLES                                                          *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL FUNCTIONS (PROTOTYPES)                                             *
 *===========================================================================*/

static char *interpret_result(ten78_Result res);

static void show_result_difference(ten78_Result expected, 
								   ten78_Result result);
static void show_get_by_clsid_difference(ten78_Result expected, 
										 char *module,
										 ten78_Result result, 
										 char *res_module);
static void show_get_difference(ten78_Result expected, 
								char *clsid,
								char *module,
								ten78_Result result, 
								char *res_clsid, 
								char *res_module);
static void dump_registry(int verbose);

#endif /* TEST */

/*===========================================================================*
 *  GLOBAL FUNCTIONS (CODE)                                                  *
 *===========================================================================*/

/*
 * Testprogram to test different functions
 */
int main(int argc, char *argv[])
{
#ifndef TEST
	en78_DumpRegistry(stdout);
#else
	ROUTINE_DBG_MEO200("main");

	int verbose = 0;
	ten78_ProgID      ProgID;
	ten78_ProgID      ProgID1;
	ten78_ProgID      ProgID2;
	ten78_ProgID      ProgID3;
	ten78_ProgIDPfx   ProgIDPfx;
	ten78_Project     Project;
	ten78_Version     Version;
	ten78_Version     Version2;
	ten78_ClsID       ClsID;
	ten78_ClsID       ClsID2;
	ten78_ClsID       ReturnedClsID;
	ten78_ModuleName  ModuleName;
	ten78_ModuleName  ReturnedModuleName;
	ten78_Result res;

	if ( argc > 1 )
	{
		verbose = strncmp(argv[1], "-v", 2);
	}

	if ( !verbose )
	{
		DBG_MEO200((1, "Use -v to dump registry in between\n"));
	}

	memset(ProgID, 0, sizeof(ProgID));
	strcpy(ProgID, "LiveCache.Sample1");
	memset(ProgID1, 0, sizeof(ProgID));
	strcpy(ProgID1, "LiveCache.Sample1.1.2.3.4");
	memset(ProgID2, 0, sizeof(ProgID));
	strcpy(ProgID2, "LiveCache.Sample1.1.2.3.5");
	memset(ProgID3, 0, sizeof(ProgID));
	strcpy(ProgID3, "LiveCache.Sample1.1.2.3.6");
	memset(ProgIDPfx, 0, sizeof(ProgIDPfx));
	strcpy(ProgIDPfx, "LiveCache");
	memset(Project, 0, sizeof(Project));
	strcpy(Project, "Sample1");
	memset(Version, 0, sizeof(Version));
	strcpy(Version, "1.2.3.4");
	memset(Version2, 0, sizeof(Version));
	strcpy(Version2, "1.2.3.5");
	memset(ClsID, 0, sizeof(ClsID));
	strcpy(ClsID, "{12345678-1234-1234-1234-123456789000}");
	memset(ClsID2, 0, sizeof(ClsID));
	strcpy(ClsID2, "{12345678-1234-1234-1234-123456789001}");
	memset(ModuleName, 0, sizeof(ModuleName));
	strcpy(ModuleName, "/usr/spool/Sample1.1.2.3.4.dll");

	DBG_MEO200((1, "Use -v to dump registry in between\n"));

	dump_registry(verbose);

    printf( "sqlComObjectUnregister(%s,%s,%s,%s)\n",
			ProgIDPfx, Project, Version, ClsID );
	res = sqlComObjectUnregister( ProgIDPfx, Project, Version, ClsID);
	show_result_difference(entry_not_found_en78, res);

	printf(  "sqlComObjectGetByClsID(%s, [OUT]ModuleName)\n", ClsID );
	res = sqlComObjectGetByClsID( ClsID, ReturnedModuleName );
	show_result_difference(entry_not_found_en78, res);

	printf(  "sqlComObjectGet(Unknown, [OUT]ClsID, [OUT]ModuleName)\n" );
	res = sqlComObjectGet( "Unknown", ReturnedClsID, ReturnedModuleName );
	show_result_difference(entry_not_found_en78, res);

	memset(ReturnedClsID, 0, sizeof(ClsID));
	memset(ReturnedModuleName, 0, sizeof(ModuleName));
	printf(  "sqlComObjectGet(%s, [OUT]ClsID, [OUT]ModuleName)\n",
		ProgID );
	res = sqlComObjectGet( ProgID, ReturnedClsID, ReturnedModuleName );
	show_get_difference(entry_not_found_en78, "", "", 
						res, ReturnedClsID, ReturnedModuleName);

	dump_registry(verbose);

    printf( "sqlComObjectRegister(%s, %s, %s, %s, %s)\n",
			ProgIDPfx, Project, Version, ClsID,  ModuleName );
	res = sqlComObjectRegister( ProgIDPfx, Project, Version, ClsID, ModuleName);
	show_result_difference(success_en78, res);

	dump_registry(verbose);

    printf( "sqlComObjectRegister(%s, %s, %s, %s, %s)\n",
			ProgIDPfx, Project, Version2, ClsID2,  ModuleName );
	res = sqlComObjectRegister( ProgIDPfx, Project, Version2, ClsID2, ModuleName);
	show_result_difference(success_en78, res);

	dump_registry(verbose);

	printf(  "sqlComObjectGetByClsID(%s, [OUT]ModuleName)\n", ClsID );
	res = sqlComObjectGetByClsID( ClsID, ReturnedModuleName );
	show_get_by_clsid_difference(success_en78, ModuleName, 
								res, ReturnedModuleName);

	printf(  "sqlComObjectGet(%s, [OUT]ClsID, [OUT]ModuleName)\n", ProgID );
	res = sqlComObjectGet( ProgID, ReturnedClsID, ReturnedModuleName );
	show_get_difference(success_en78, ClsID2, ModuleName, 
						res, ReturnedClsID, ReturnedModuleName);

    printf(  "sqlComObjectRegister(Unknown, %s, %s, %s, %s)\n",
		Project, Version, ClsID, ModuleName );
	res = sqlComObjectRegister( "Unknown", Project, Version, ClsID, ModuleName);
	show_result_difference(doubled_clsid_en78, res);

    printf(  "sqlComObjectUnregister(Unknown, %s, %s, %s)\n", 
		Project, Version, ClsID );
	res = sqlComObjectUnregister( "Unknown", Project, Version, ClsID);
	show_result_difference(clsid_version_mismatch_en78, res);

    printf(  "sqlComObjectUnregister(%s, %s, 0.0.0.0, %s\n",
		ProgIDPfx, Project, ClsID );
	res = sqlComObjectUnregister( ProgIDPfx, Project, "0.0.0.0", ClsID);
	show_result_difference(clsid_version_mismatch_en78, res);

    printf(  "sqlComObjectUnregister(%s, %s, %s, %s\n",
		ProgIDPfx, Project, Version2, ClsID );
	res = sqlComObjectUnregister( ProgIDPfx, Project, Version2, ClsID2);
	show_result_difference(success_en78, res);

	dump_registry(verbose);

	memset(ReturnedClsID, 0, sizeof(ClsID));
	memset(ReturnedModuleName, 0, sizeof(ModuleName));
	printf(  "sqlComObjectGet(%s, [OUT]ClsID, [OUT]ModuleName)\n", 
		ProgID );
	res = sqlComObjectGet( ProgID, ReturnedClsID, ReturnedModuleName );
	show_get_difference(entry_not_found_en78, "", "", 
						res, ReturnedClsID, ReturnedModuleName);

	memset(ReturnedClsID, 0, sizeof(ClsID));
	memset(ReturnedModuleName, 0, sizeof(ModuleName));
	printf(  "sqlComObjectGet(%s, [OUT]ClsID, [OUT]ModuleName)\n", 
		ProgID1 );
	res = sqlComObjectGet( ProgID1, ReturnedClsID, ReturnedModuleName );
	show_get_difference(success_en78, ClsID, ModuleName, 
						res, ReturnedClsID, ReturnedModuleName);

	memset(ReturnedClsID, 0, sizeof(ClsID));
	memset(ReturnedModuleName, 0, sizeof(ModuleName));
	printf(  "sqlComObjectGet(%s, [OUT]ClsID, [OUT]ModuleName)\n", 
		ProgID2 );
	res = sqlComObjectGet( ProgID2, ReturnedClsID, ReturnedModuleName );
	show_get_difference(entry_not_found_en78, "", "", 
						res, ReturnedClsID, ReturnedModuleName);

	memset(ReturnedClsID, 0, sizeof(ClsID));
	memset(ReturnedModuleName, 0, sizeof(ModuleName));
	printf(  "sqlComObjectGet(%s, [OUT]ClsID, [OUT]ModuleName)\n", 
		ProgID3 );
	res = sqlComObjectGet( ProgID3, ReturnedClsID, ReturnedModuleName );
	show_get_difference(entry_not_found_en78, "", "", 
						res, ReturnedClsID, ReturnedModuleName);

    printf(  "sqlComObjectUnregister(%s, %s, %s, %s)\n", 
		ProgIDPfx, Project, Version, ClsID );
	res = sqlComObjectUnregister( ProgIDPfx, Project, Version, ClsID);
	show_result_difference(success_en78, res);

	dump_registry(verbose);

    printf(  "sqlComObjectUnregister(%s, %s, %s, %s)\n", 
		ProgIDPfx, Project, Version, ClsID );
	res = sqlComObjectUnregister( ProgIDPfx, Project, Version, ClsID);
	show_result_difference(entry_not_found_en78, res);

	dump_registry(verbose);
#endif /* TEST */
	return 0;
}

/*===========================================================================*
 *  LOCAL FUNCTIONS (CODE)                                                   *
 *===========================================================================*/

#ifdef TEST

static void dump_registry(int verbose)
{
	ROUTINE_DBG_MEO200("dump_registry");

	if ( verbose )
	{
		en78_DumpRegistry(stdout);
	}
}

static void show_result_difference(ten78_Result expected, ten78_Result result)
{
	ROUTINE_DBG_MEO200("show_result_difference");

	if ( expected != result )
	{
		printf(  "Difference!\n" );
		printf(  "expected: %s\n", interpret_result(expected) );
		printf(  "result  : %s\n", interpret_result(result) );
	}
	else
	{
		printf(  "Ok\n" );
	}
}

static void show_get_by_clsid_difference(ten78_Result expected,
										 char *module,
										 ten78_Result result,
										 char *res_module)
{
	ROUTINE_DBG_MEO200("show_get_by_clsid_difference");

	if ( expected != result 
	  || strcmp(module, res_module) )
	{
		printf(  "Difference!\n" );
		printf(  "expected: %s ModuleName [%s]\n", 
			interpret_result(expected), module );
		printf(  "result  : %s ModuleName [%s]\n", 
			interpret_result(result), res_module );
	}
	else
	{
		printf(  "Ok\n" );
	}
}

static void show_get_difference(ten78_Result expected,
								   char *clsid,
								   char *module,
								   ten78_Result result,
								   char *res_clsid,
								   char *res_module)
{
	ROUTINE_DBG_MEO200("show_get_difference");

	if ( expected != result
	  || strcmp(module, res_module)
	  || strcmp(clsid, res_clsid) )
	{
		printf(  "Difference!\n" );
		printf(  "expected: %s ClsID %s ModuleName %s\n", 
			interpret_result(expected), clsid, module );
		printf(  "result  : %s ClsID %s ModuleName %s\n",
			interpret_result(result), res_clsid, res_module );
	}
	else
	{
		printf(  "Ok\n" );
	}
}

static char *interpret_result(ten78_Result res)
{
	ROUTINE_DBG_MEO200("interpret_result");

	switch(res)
	{
	case success_en78:
		return "success";
    case illegal_parameter_en78:
		return "illegal_parameter";
	case access_denied_en78:
		return "access_denied";
	case doubled_clsid_en78:
		return "doubled_clsid";
	case entry_not_found_en78:
		return "entry_not_found";
	case clsid_version_mismatch_en78:
		return "clsid_version_mismatch";
	default:
		break;
	}
	return "unknown result code";
}
#endif /* TEST */

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
