/*!***************************************************************************

  module      : ven87.c

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Kernel Runtime
  description : CONSOLE Thread
  

  last changed: 2001-09-19  11:55
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/
#define MOD__ "ven87.c:"

#include      "gen00.h"
#include      "heo00.h"
#include      <ctype.h>
#include      "geo50_0.h"
#include      "gen500.h"
#include      "heo07.h"
#include      "gen41.h"
#include      "gen60.h"
#include      "gen81.h"
#include      "gen87.h"
#include      "hen40.h"

/* imported functions */

extern  void                            vabort ();

/* local functions */

static  int     e87_get_line (  /* IN */  int fd, 
								/* OUT */ char *msg );

/* thread local data */
#define MSG_LINE_MXEN87    256
#define BUFFER_SIZE_MXEN87 5120

/*
 * Buffer for pipe readout
 */
static   int      buf_pos = 0;
static   char     buffer [ BUFFER_SIZE_MXEN87 ];
static   int      buf_lgt = 0;
static   int      LineContinue = FALSE ;

/* imported data */

extern tsp00_DbNamec				    en81_dbname;

static void en87_GetFifoName( tsp00_Pathc VAR_ARRAY_REF file );

/*
 * ======================================================================
 */
#undef MF__
#define MF__ MOD__"en87_console"
/*ARGSUSED*/
externC void *en87_console ( void *arg )
{
	tsp00_VFilenamec file ;
    int              buf_fd  = 0;
/*
 * Buffer for single message line
 */
    char     opmsg [ MSG_LINE_MXEN87 ];
    int	     oplgt = 0;

	DBGIN;

	MSGCD (( INFO_PROCESS_STARTED, "CONSOLE" )) 

	KGS->console.start_time = time ( (time_t *) 0 );
    KGS->console.state = KT_RUNNING;

	DBG1 (( MF__,"thread %ld \n", (long) KGS->console.tid ));

	buf_fd = en87_OpenDiagFifo( O_RDONLY );
	/*
	 * Blocks until COORDINATOR calls en60_UseDiagFifo
	 */
	if ( buf_fd < 0 )
	{
	    DBG1 (( MF__,"Failed to open diag fifo\n" ));
	    MSGCD (( ERR_CONSOLE_FIFO_OPEN, sqlerrs() ));
	    vabort ( !WRITE_CORE );
	}

	while ( (oplgt = e87_get_line ( buf_fd, opmsg )) != 0 )
	{
	    e60_deliver_msg (opmsg, oplgt);
	}

	(void) close ( buf_fd );
	(void) unlink ( &file[0] );

	MSGCD (( INFO_PROCESS_STOPPED, "CONSOLE" )) 

	DBGOUT;
    return NULL;
}

/*
 * ======================================================================
 */
#undef MF__
#define MF__ MOD__"e87_get_line"
static  int     e87_get_line ( 
int buf_fd, 
char *opmsg )
{
register int c ;
register int oplgt ;

	DBGIN;

	oplgt = ( LineContinue ? 1 : 0 ) ; 
	while ( oplgt < (MSG_LINE_MXEN87-1) )
	{
	    if ( buf_lgt <= 0 )
	    {
			buf_pos = 0 ;
			do
			{
				buf_lgt = read ( buf_fd , 
								 buffer , 
								 BUFFER_SIZE_MXEN87 );
			} while ( (buf_lgt < 0) && (errno == EINTR) );

			if ( buf_lgt < 0 )
			{
				MSGCD (( ERR_CONSOLE_FIFO_READ , sqlerrs() ));
				vabort ( !WRITE_CORE );
			}

			if ( buf_lgt == 0 ) 
			{
			/*
			 * FIFO is closed, but some output lines could still be in buffer
			 */
				break;
			}
	    }
	    c = buffer [ buf_pos ++ ];
	    buf_lgt -- ;
	    opmsg [ oplgt ++ ] = c ;
	    if ( c == '\n' ) break ;
	}
	LineContinue = ( oplgt == (sizeof(opmsg)-1) ) ; 
	opmsg [ oplgt ] = '\0' ;

	DBGOUT;

	/*
	 * Return only 0 if all output is done...
	 */

	return ( oplgt );
}

/*
 * ======================================================================
 */

#undef MF__
#define MF__ MOD__"en87_create_diag_fifo"
externC int en87_create_diag_fifo ()
{
	int              rc ;
	tsp00_Pathc      file ;

	DBGIN;

	en87_GetFifoName( file );

	/*
	 *  The diag fifo might exist from a previous run.
	 */
	(void) unlink ( &file[0] );

	rc = mknod ( &file[0] , S_IFIFO | 0622 , 0 );
	if ( rc < 0 )
	{
		MSGCD (( ERR_CONSOLE_FIFO_CREATE , sqlerrs() ));
		return ( 2 );
	}

	rc = chown ( &file[0] , getuid() , getgid() );
	if ( rc < 0 )
	{
		MSGCD (( ERR_CONSOLE_FIFO_CHOWN , sqlerrs() ));
		return ( 2 );
	}

	DBGOUT;

	return ( 0 );
}

#undef MF__
#define MF__ MOD__"en87_GetFifoName"
static void en87_GetFifoName( tsp00_Pathc VAR_ARRAY_REF file )
{
	DBGIN;

	sql41_get_diag_dir ( &file[0] );
	(void) strcat ( &file[0] , en81_dbname );

	DBG1 (( MF__,"fifo file %s \n", &file[0] ));

	DBGOUT;
}

#undef MF__
#define MF__ MOD__"en87_open_diag_fifo"
externC int en87_OpenDiagFifo ( int mode )
{
	int fd;

	tsp00_Pathc      file ;

	DBGIN;

	en87_GetFifoName( file );

	do
	{
		fd = open ( &file[0] , mode, 0 );
	} while ( fd == -1 && errno == EINTR );

	DBG1 (( MF__,"fifo file descriptor %d\n", fd ));

	DBGOUT;

	return fd;
}
