/*!***************************************************************************

  module      : ven90.c

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Client RunTime
  description : xstart main

  Start a Kernel or a Gateway

  last changed: 2001-09-19  13:46
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/
#define MOD__ "ven90.c:"

#if SVR4
#include	<wait.h>
#endif

#include "gen00.h"
#include "heo00.h"
#include "hen40.h"
#include "heo01.h"
#include "gip00.h"

#ifndef IERR_TYPE
#define IERR_TYPE   0
#define ERR_TYPE    1
#define WRN_TYPE    2
#define INFO_TYPE   3
#endif

#define ERR_START_USAGE              10101,ERR_TYPE,"start   ","usage: start kernel_program"
#define ERR_START_DBROOT_UNDEF       10102,ERR_TYPE,"start   ","DBROOT undefined"
#define ERR_START_DBROOT_EMPTY       10103,ERR_TYPE,"start   ","DBROOT empty"
#define ERR_START_DBROOT_LONG        10104,ERR_TYPE,"start   ","DBROOT too long"
#define ERR_START_DBNAME_UNDEF       10105,ERR_TYPE,"start   ","SERVERDB (DBNAME) undefined"
#define ERR_START_DBNAME_EMPTY       10106,ERR_TYPE,"start   ","SERVERDB (DBNAME) empty"
#define ERR_START_DBNAME_LONG        10107,ERR_TYPE,"start   ","SERVERDB (DBNAME) too long"
#define ERR_START_KNLPGM             10108,ERR_TYPE,"start   ","kernel program missing '%s'"
#define ERR_START_LINK               10109,ERR_TYPE,"start   ","cannot link kernel program to '%s'"
#define ERR_START_SRVPGM             10110,ERR_TYPE,"start   ","server program missing '%s'"
#define ERR_START_NO_LINK            10111,ERR_TYPE,"start   ","link from kernel to server missing"
#define ERR_START_SIGUSR1            10112,ERR_TYPE,"start   ","cannot catch SIGUSR1, %s"
#define ERR_START_SIGCLD             10113,ERR_TYPE,"start   ","cannot catch SIGCLD, %s"
#define ERR_START_SIGWINCH           10114,ERR_TYPE,"start   ","cannot ignore SIGWINCH, %s"
#define ERR_START_EXEC               10115,ERR_TYPE,"start   ","exec failed , %s"
#define ERR_START_FORK               10116,ERR_TYPE,"start   ","fork failed, %s"
#define ERR_START_WAIT               10117,ERR_TYPE,"start   ","Wait returned %ld: %s, timeout signal %d"
#define ERR_START_KILLING            10118,ERR_TYPE,"start   ","Killing server '%s'"
#define ERR_START_DIED               10119,ERR_TYPE,"start   ","Server '%s' died prematurely."
#define ERR_CANT_GET_DBROOT_PGM      10121,ERR_TYPE, "Can't get DBROOT/pgm : %s"

#define INFO_START_SUCCESS           10200,INFO_TYPE,"start   ","Server '%s' successfully started."

#define MSG_O(_x) { sqloutwrite _x ; }
/* imported functions */
 
extern  void                            sqlinit ();
 
/* local functions */
 
static  SIGNALFUNCTYPE                  catch_signal ();
static  int     get_servername ( tsp00_Pathc  servername );
 
/* local data */
 
static  int                             timeout = 0;
static  char                          * dbname_ptr = NULL;
 
/*
 * ======================================================================
 */
 
int c_main ( int argc , char *argv[] )
{
    int                             statloc , rc ;
    pid_t                           serverpid , childpid ;
    char                          * kernelpgm ;
    tsp00_Pathc                     servername ;
 
    sqlinit ( "start" , (char*)0 );
    DBG1 (( MF__,"XSTART: called \n" ));
 
    /* PTS 1105151
     *  File permission mode is always specified absolutely
     */
    (void) umask ( 0 );
 
    if ( argc < 2 )
    {
	MSG_O (( ERR_START_USAGE ))
	return ( 1 );
    }
    kernelpgm = argv [ 1 ];
 
    /*
     *  get DBNAME
     */
    dbname_ptr = getenv ( "SERVERDB" );
    if ( ! dbname_ptr )
	{
	MSG_O (( ERR_START_DBNAME_UNDEF ))
	return ( 1 );
	}
    if ( (int) strlen ( dbname_ptr ) < 1 )
	{
	MSG_O (( ERR_START_DBNAME_EMPTY ))
	return ( 1 );
	}
    if ( (int) strlen ( dbname_ptr ) > (int) sizeof ( SQL_DBNAMEC ) )
	{
	MSG_O (( ERR_START_DBNAME_LONG ))
	return ( 1 );
	}
 
    /*
     *  get servername
     */
    if ( !get_servername ( servername ) )
      return ( 1 );
 
    /*
     *  Create server process
     */
 
    timeout = 0;
    if ( signal ( SIGUSR1 , catch_signal ) == SIG_ERR )
	{
	MSG_O (( ERR_START_SIGUSR1 , sqlerrs() ))
	return ( 2 );
	}
    /*
     *  SIGCLD is required for the wait system call below.
     */
    if ( signal ( SIGCLD  , SIG_DFL      ) == SIG_ERR )
	{
	MSG_O (( ERR_START_SIGCLD , sqlerrs() ))
	return ( 2 );
	}
#ifdef SIGWINCH
    if ( signal ( SIGWINCH , SIG_IGN ) == SIG_ERR )
	{
	MSG_O (( ERR_START_SIGWINCH , sqlerrs() ))
	return ( 2 );
	}
#endif

    /* PTS 1105182
     *  It is necessary to add '$DBROOT/lib' to LD_LIBRARY_PATH
     *  Otherwise DCOM runs into severe problems.
     */
    {
      static char *envLDLIB = NULL;
      tsp00_Pathc LibPath;
      tsp01_RteError RteError;

      DBG1 (( MF__,"preparing " LDLIBPATH_IP00 ));

      if ( envLDLIB != NULL )
      {
        free(envLDLIB); /* prevent some memory leak, accept that last malloc is not freed... */
        envLDLIB = NULL;
      }

      if ( sqlGetDbrootLibPath(LibPath, 0, &RteError ) )
      {
        DBG1 (( MF__,"got LibPath %s", LibPath ));

        if ( getenv(LDLIBPATH_IP00) != NULL ) 
        {
          envLDLIB = (char *)malloc(strlen(LDLIBPATH_IP00 "=") + strlen(LDLIBPATH_IP00) + strlen(":") + sizeof(tsp00_Pathc));
        }
        else
        {
          envLDLIB = (char *)malloc(strlen(LDLIBPATH_IP00 "=") + sizeof(tsp00_Pathc));
        }
      }

      /*
       * Prepend the $DBROOT/lib and store via putenv
       */
      if ( envLDLIB != NULL )
      {
        strcpy(envLDLIB, LDLIBPATH_IP00 "=");
        strcat(envLDLIB,LibPath);

        if ( getenv(LDLIBPATH_IP00) != NULL ) 
        {
          strcat(envLDLIB,":");
          strcat(envLDLIB,getenv(LDLIBPATH_IP00));
        }

        DBG1 (( MF__,"try putenv(%s)", envLDLIB ));

        if ( putenv(envLDLIB) < 0 )
        {
          free(envLDLIB);
          envLDLIB = NULL;
        }
      }

      if ( envLDLIB == NULL )
      {
        MSG_O (( ERR_START_DBROOT_LONG ));
        return (1);
      }
    }
    
#if defined(AIX)
#  if defined(BIT64)
     putenv("EXTSHM=OFF");
#  else
    /* set EXTSHM on 32 Bit AIX to avoid shm attach problems
      PTS 1106620 
     */
     putenv("EXTSHM=ON");
#  endif
#endif

    /*
     *  Fork and exec database kernel.
     */
    serverpid = fork ();
    if ( serverpid == 0 )
	{
	/* child process */
 
	/*
	 *  IST 1995-12-29
	 *  This allows starting the kernel from a debugger.
	 */
	kernelpgm = getenv ( "DBDEBUG" );
	if ( kernelpgm )
	{
	    char		cmdlin [ 1024 ];
 
	    (void) sprintf ( cmdlin , "%s %s %s" , kernelpgm , argv[1], dbname_ptr );
	    for ( rc = 3 ; argv[rc] ; rc ++ )
	    {
		(void) strcat ( cmdlin , " " );
		(void) strcat ( cmdlin , argv[rc] );
	    }
	    (void) printf ( "%s\n", cmdlin );
	    (void) fflush ( stdout );
	    (void) system ( cmdlin );
	    exit ( 0 );
	}
 
	DBG1 (( MF__,"XSTART: starting '%s' \n", servername ));
	rc = execl ( argv[1], argv[1], dbname_ptr ,
		            argv[3] , argv[4] ,
                    argv[5] , argv[6] , NULL );
		    /* one of them will be NULL and terminating the execl list */
 
	MSG_O (( ERR_START_EXEC , sqlerrs() ))
	return ( 3 );
	}
	/* end of child process */
 
    if ( serverpid == (pid_t) -1 )
	{
	/* parent, fork failed */
 
	MSG_O (( ERR_START_FORK , sqlerrs() ))
	return ( 2 );
	}
 
    /* parent, fork ok */
 
#if _IBMR2 || SUN_ucb || MI_ULT || LINUX
    (void) siginterrupt ( SIGUSR1 , TRUE );
#endif
 
    DBG1 (( MF__,"XSTART: waiting ... \n" ));
    childpid = wait ( & statloc );
    DBG1 (( MF__,"XSTART: Wait returned  %ld \n", (long) childpid ));
    DBG1 (( MF__,"XSTART:         error  %s \n", sqlerrs() ));
    DBG1 (( MF__,"XSTART:       statloc  0x%x \n", statloc ));
    DBG1 (( MF__,"XSTART: timeout signal %d \n", timeout ));
    sleep(0);
    if ( childpid != serverpid )
    {
	switch ( timeout )
	{
	case SIGUSR1 :
        { MSG_O (( INFO_START_SUCCESS , dbname_ptr ))
		}
	    return ( 0 );
 
	default :
	    MSG_O (( ERR_START_WAIT , (long) childpid , sqlerrs() , timeout ))
	}
	MSG_O (( ERR_START_KILLING , dbname_ptr ))
	kill ( serverpid, SIGKILL );
	return ( 2 );
    }
       { MSG_O (( ERR_START_DIED , dbname_ptr ))
       }
    return ( 2 );
}
 
/*
 * =====================================================================
 */

/* PTS 1105631 */
static  int     get_servername ( tsp00_Pathc  servername )
{
    tsp01_RteError RteError ;
    int Ok ; 
 
    DBG1 (( MF__,"get_servername: called \n" ));

    Ok = sqlGetDbrootPgmPath ( servername, TERM_WITH_DELIMITER_EO01, &RteError ) ;
 
    if ( Ok ) 
    {
      if ( sizeof(tsp00_Pathc) > ( strlen(servername) + strlen("db:") + strlen(dbname_ptr) )) 
      {
          (void) strcat ( servername , "db:" );
        (void) strcat ( servername , dbname_ptr );
      }
      else
      {
        Ok = 0 ;
   	    MSG_O (( ERR_START_DBROOT_LONG ))
      }
    }
    else
    {
 	  MSG_O (( ERR_CANT_GET_DBROOT_PGM, RteError.RteErrText ))
    }

    DBG1 (( MF__,"get_servername: returning, '%s' \n", servername ));
    return ( Ok );
}
 
/*
 * =====================================================================
 */
 
static  SIGNALFUNCTYPE  catch_signal ( int sig )
{
    timeout = sig;
    (void) signal ( sig , catch_signal );
#if _IBMR2 || SUN_ucb || MI_ULT || LINUX
    (void) siginterrupt ( sig , TRUE );
#endif
}
