/*!***************************************************************************

  module      : ven901.c

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Client Runtime
  description : start,stop,clear as functions

  last changed: 2001-09-19  12:09
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/
#define MOD__ "ven901.c:"

#include "gen00.h"
#include "heo00.h"
#include "gen50.h"
#include "geo007_1.h"
#include "gen950.h"
#include "gsp09.h"
#include "heo05.h"
#include "gen904.h"
#include "gen45.h"
#include "hsp100.h"
#include "heo01.h"
#include "gip00.h"
#include "hen40.h"
#define XPARAM_NAMES_ONLY
#include "geo002.h" /* for PAN_RUNDIR */
#if SVR4
#include        <wait.h>
#endif
#if defined(OSF1) || defined(LINUX)
#include        <sys/wait.h>
#endif
 
/* imported functions */

#include "gen41.h"
extern  tsp9_cstr               en903dbname (void);
extern  tsp9_cstr               en903dbroot (void);
extern  int                     en903GetDbPidStateFromShm(pid_t *, pid_t *, tsp00_Uint4 *, tsp00_Bool);
extern  int                     e950_put_dbname_to_env (tsp9_cstr dbname);

/* imported data */

/* exported data */

char                                  * dbname ;

/* local functions */
 
static  SIGNALFUNCTYPE                  en901_CatchColdState ();
static  void    RemoveRunningProcesses ( tsp9_cstr dbn, tsp9_rte_xerror* xerror );
static  int     CheckForDbStartable ( tsp9_cstr dbname, 
                                      tsp9_cstr dbroot,
                                      char *full_knl_path,
                                      tsp9_pgm_kind pgmKind,
                                      tsp00_Pathc rundir,
                                      tsp9_rte_xerror* xerror);
static  char   *GetExecutable ( tsp9_pgm_kind pgmKind);
static  void    UpdateInternalValues( tsp9_cstr *pDbname, tsp9_cstr *pDbroot );
static  int     CheckForRunningProcesses ( tsp9_cstr dbname, tsp9_rte_xerror* xerror );
static  int     RundirExists ( tsp9_cstr dbname,
                               tsp00_Pathc rundir,
                               tsp9_rte_xerror* xerror);
#define STOP_DONE       0
#define STOP_DIRTY      1
#define STOP_IMPOSSIBLE 2
static  int     DoStop ( tsp9_cstr dbname, tsp9_cstr dbroot, tsp00_Bool dumpflag, tsp9_rte_xerror* xerror);
static  int     DoWaitForPid( pid_t kernelPid );
static  int     DoStart ( tsp9_cstr     dbname, 
                          tsp9_pgm_kind pgmKind, 
                          int argc,
                          char* argv[],
                          tsp00_Pathc kernelpgm,
                          tsp00_Pathc rundir );
#define DO_START_SUCCEEDED              0
#define DO_START_CANT_REDIRECT_STDOUT   1
#define DO_START_CANT_REDIRECT_STDERR   2
#define DO_START_CANT_OPEN_DEV_NULL     3
#define DO_START_CANNOT_FORK            4
#define DO_START_CANT_CHANGE_TO_RUN_DIR 5
#define DO_START_DIED_BEFORE_COLD_STATE 6
/* DO_START_KERNEL_EXIT_CODE_OFFSET must be highest number, since exit_code is added */
#define DO_START_KERNEL_EXIT_CODE_OFFSET 7

#define DO_WAIT_FOR_PID_SLEEP           2
/* PTS 1107119 */
/* Number of loops polling before additional search of state in shared memory segment */
#define DO_WAIT_FOR_PID_LOOPS          30
/* Number of seconds sleeping between polling kernel start signal indication */
#define DO_WAIT_FOR_COLD_STATE_SLEEP    1

/* PTS 1107044 */
static int en901_UpdateEnvironment(const char *dbroot, tsp9_rte_xerror* xerror);

/* PTS 1107204 */
static int CheckFileAccess ( char *path, int mode );
static int CheckFileDoesNotExist ( char *path );
static int CreatePath ( char* path_to_create, unsigned short mode );
static int GetRunDirectory ( tsp9_cstr dbname, tsp00_Pathc ParamRundir);

/* PTS 1109708 */
static pid_t SaveWaitpid(pid_t pidToWaitFor, int *pStatus, int flags);

/* local data */
 
static volatile int en901_ColdStateIndicator = 0;

/**\
--------------------------------------- MF__ MOD__ (sqlxstart)
\**/

#define maxArgsC 10

    #undef MF__
    #define MF__ MOD__"sqlxstart"
void sqlxstart (
    tsp9_cstr dbname, 
    tsp9_pgm_kind pgmKind, 
    tsp9_cstr dbroot, 
    tsp00_Bool is_gateway,
    int argc,
    char* argv[],
    tsp9_rte_xerror* xerror)
{
    int                         rc;
    pid_t                       serverpid;
    pid_t                       returnedPid;
    tsp00_Pathc                 kernelpgm;  /* PTS 1105631 */
    tsp00_Pathc                 rundir;     /* PTS 1117575 */
    int                         startProcessStatus;

    DBGIN;

    eo44initError (xerror);

    if ( is_gateway )
    {
        MSG_9 ((xerror, -11987, "Gateway %s no longer supported", dbname ));
        return;
    }


    if ( strlen(dbname) > sizeof(tsp00_DbName) )
    {
        MSG_9 ((xerror, -11987, "Database name %s too long", dbname ));
        return;
    }

    UpdateInternalValues(&dbname, &dbroot);

	rc = CheckForRunningProcesses(dbname, xerror);
	if ( rc != 0 )
	{
        DBG1 (( MF__,"still processes running using '%s'", dbname ));
        MSG_9 ((xerror, -11987, "Database %s already started", dbname ));
        return;
    }

    rc = CheckForDbStartable ( dbname,
                               dbroot,
                               kernelpgm,
                               pgmKind,
                               rundir,
                               xerror);
    if ( rc != 0 )
    {
        return;
    }

    /*
     *  create IPC directories
     */
    {
        char ipcdb [ MAX_PATH_LEN ];
        char ipcus [ MAX_PATH_LEN ];
        int saved_umask;

        /*
         *  File permission mode is always specified absolutely
         */
        saved_umask = umask ( 0 );
        sql41_get_ipc_dir( ipcdb );
        strcat( ipcdb, "/db:" );
        strcat( ipcdb, dbname );
        sql41_get_ipc_dir( ipcus );
        strcat( ipcus, "/us:" );
        strcat( ipcus, dbname );

        if ( !CreatePath ( ipcdb, 0777 ) )
        {
            MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxclear", 
                "Cannot create ipc db directories for %s", dbname ));
            umask(saved_umask);
            return;
        }
        if ( !CreatePath ( ipcus, 0777 ) )
        {
            MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxclear", 
                "Cannot create ipc us directories for %s", dbname ));
            umask(saved_umask);
            return;
        }

        /* 
         * PTS 1105151
         * All resources created, umask can be reset
         */
        umask(saved_umask);
    }
    
    /*
     *  Create server process
     */
    {
      struct sigaction sa;

      sa.sa_handler = en901_CatchColdState;
      sigemptyset( &sa.sa_mask );
      sa.sa_flags = 0;
      if ( sigaction( SIGUSR1, &sa, NULL) != 0 )
      {
        MSG_ERR ((xerror, 10112,ERR_TYPE,"start   ","cannot catch SIGUSR1, %s" , sqlerrs() ));
        return;
      }

      /*
       *  SIGCLD is required for the wait system call below.
       */
      sa.sa_handler = SIG_DFL;
      sigemptyset( &sa.sa_mask );
      sa.sa_flags = 0;
      if ( sigaction( SIGCLD, &sa, NULL) != 0 )
      {
        MSG_ERR ((xerror, 10113,ERR_TYPE,"start   ","cannot catch SIGCLD, %s" , sqlerrs() ));
        return;
      }

#ifdef SIGWINCH
      sa.sa_handler = SIG_IGN;
      sigemptyset( &sa.sa_mask );
      sa.sa_flags = 0;
      if ( sigaction( SIGWINCH, &sa, NULL) != 0 )
      {
        MSG_ERR ((xerror, 10114,ERR_TYPE,"start   ","cannot ignore SIGWINCH, %s" , sqlerrs() ));
        return;
      }
#endif
    }

    /* PTS 1107044 */
    if ( en901_UpdateEnvironment( dbroot, xerror ) == 0 )
    {
      return;
    }
    /*
     *  Fork and exec database kernel.
     */
    serverpid = fork ();
    if ( serverpid == 0 ) 
    {
      /* child process */
      exit ( DoStart(dbname, pgmKind, argc, argv, kernelpgm, rundir) );
    }

    returnedPid = SaveWaitpid(serverpid, &startProcessStatus, 0);
    if ( returnedPid == ((pid_t)-1) )
    {
        MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxstart", 
                 "Cannot wait for kernel starter: %s", sqlerrs() ));
    } else if ( !WIFEXITED(startProcessStatus) ) {
        MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxstart", 
                 "Kernel starter process did not exit normally: %d", startProcessStatus ));
    } else {
      switch( WEXITSTATUS(startProcessStatus) )
      {
        case DO_START_SUCCEEDED:
          break;

        case DO_START_CANT_REDIRECT_STDOUT:
          MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxstart", 
                   "Cannot redirect stdout to '/dev/null'" ));
          break;

        case DO_START_CANT_REDIRECT_STDERR:
          MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxstart", 
                   "Cannot redirect stderr to '/dev/null'" ));
          break;
        case DO_START_CANT_OPEN_DEV_NULL:
          MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxstart", 
                   "Cannot open '/dev/null' for redirection" ));
          break;
        case DO_START_CANNOT_FORK:
          MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxstart", 
                   "Cannot fork to start kernel" ));
          break;
        case DO_START_CANT_CHANGE_TO_RUN_DIR:
          MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxstart", 
                   "Cannot change to rundir '%s'", rundir ));
          break;
        case DO_START_DIED_BEFORE_COLD_STATE:
          MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxstart", 
                   "Check knldiag! Kernel died before reaching ADMIN state" ));
          break;
        default:
          MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxstart", 
                   "Check knldiag! Kernel exited with '%d' before reaching ADMIN state", 
                   WEXITSTATUS(startProcessStatus) - DO_START_KERNEL_EXIT_CODE_OFFSET ));
          break;
      }
    }

    DBGOUT;
    return;
}
 
/**\
--------------------------------------- MF__ MOD__ (sqlxclear)
\**/

    #undef MF__
    #define MF__ MOD__"sqlxclear"
void sqlxclear (
    tsp9_cstr dbname,
    tsp9_cstr dbroot,
    tsp9_rte_xerror* xerror)
{
    int rc;

    DBGIN;
    UpdateInternalValues(&dbname, &dbroot);

    eo44initError (xerror);

    RemoveRunningProcesses (dbname, xerror);

    rc = sql41_remove_ipc_resources ( dbname );

    DBGOUT;

    return;
}

/**\
--------------------------------------- MF__ MOD__ (sqlxcanstart)
\**/

    #undef MF__
    #define MF__ MOD__"sqlxcanstart"
externC void sqlxcanstart ( tsp9_rte_xerror *xerror )
{
    int rc = 0;
    tsp9_cstr dbname; 
    tsp9_cstr dbroot;
    tsp00_Pathc                 kernelpgm;
    tsp00_Pathc                 rundir;
    tsp00_Pathc                 LocalDbroot;

    DBGIN;

    dbroot = en903dbroot ();
    dbname = en903dbname ();

    eo44initError (xerror);

    if ( !*dbroot )
    {
        /*
         * Try to get dbroot from dbname...
         */
        /* PTS 1105115 */
        if ( !sqlGetDBRootOfDB(dbname, LocalDbroot, sizeof(LocalDbroot) ) )
        {
            eo44anyError ( xerror, "Missing dbroot for dbname\n" );
            return;
        }
        dbroot = LocalDbroot;
    }

    rc = sqlXParamCheckExists(dbname);

    if ( rc )
    {
	    rc = CheckForRunningProcesses(dbname, xerror);
	    if ( rc != 0 )
	    {
            DBG1 (( MF__,"still processes running using '%s'", dbname ));
            MSG_9 ((xerror, -11987, "Database %s already started", dbname ));
            return;
        }
    }

    DBG1(( MF__,"dbname %s dbroot %s can%s start", dbroot, dbname, (rc ? "not" : "") ));

    DBGOUT;
}
/**\
--------------------------------------- MF__ MOD__ (sqlxstop)
\**/

    #undef MF__
    #define MF__ MOD__"sqlxstop"
void sqlxstop (
    tsp9_cstr dbname, 
    tsp9_cstr dbroot,
    tsp00_Bool dumpflag,
    tsp9_rte_xerror* xerror)
{
    int rc;
    DBGIN;

    UpdateInternalValues(&dbname, &dbroot);

    rc = DoStop ( dbname, dbroot, dumpflag, xerror);
    
    switch( rc )
    {
    case STOP_DONE: /* Stop succeeded */
        eo44initError (xerror);
        break;
    case STOP_DIRTY: /* Stop wants Clear */
        eo44initError (xerror);
        sqlxclear (dbname, dbroot, xerror);
        break;
    default:
    case STOP_IMPOSSIBLE: /* Stop not possible */
        break;
    }

    DBGOUT;
    return;
}

/**\
--------------------------------------- MF__ MOD__ (sqlxstate)
\**/
    #undef MF__
    #define MF__ MOD__"sqlxdbstate"
tsp00_Uint4
sqlxdbstate( tsp9_cstr        dbname, 
             tsp9_rte_xerror  *xerror )
{
    int         rc;
    tsp00_Uint4 dbState;

    eo44initError (xerror);

    rc = en903GetDbPidStateFromShm( NULL, NULL, &dbState, 0 );
    if ( rc == 1 )
    {
        dbState = SERVER_OFFLINE;
    } else if ( rc == 2 )
    {
        MSG_7 ((xerror, -11202, "Getting Kernel state failed: %s\n", sqlerrs() ));
        dbState = 0;
    }

    return dbState;
}

/*
 * Wait for either a child or a process started by some other process...
 *
 * Return value: 0 if Waiting successfull
 *               1 if Waiting aborted
 */
    #undef MF__
    #define MF__ MOD__"DoWaitForPid"
static int DoWaitForPid( pid_t kernelPid )
{
    pid_t childPid;
    int ret = 0;

    DBGIN;

    DBG1 (( MF__,"Waiting for kernel %ld to stop", (long)kernelPid ));
    childPid = SaveWaitpid(kernelPid, (int *)0, 0 );
    if ( childPid == (pid_t)-1 )
    {
        if ( errno == ECHILD )
        {
            DBG1 (( MF__,"Not started by this process" ));
            /* some other process has started it... */

            /* PTS 1106476 This produced an endless loop if kernel became zombie... */
            /* Scenario: DBMGUI or dbmcli started kernel and was left open. */
            /* Second dbmcli stopped kernel and waited forever */
            /* but it now possible, since no more zombis are created (see DoStart()) */

            while ( kill( kernelPid, 0 ) == 0 )
            {
                if ( sleep(DO_WAIT_FOR_PID_SLEEP) != 0 )
                {
                    DBG1 (( MF__,"Kernel wait aborted: %s", sqlerrs() ));
                    ret = 1;
                    break;
                }
            }
#ifdef DEBUG_RTE
            if ( kill( kernelPid, 0 ) < 0 
              && errno == ESRCH )
            {
                DBG1 (( MF__,"Kernel stopped"));
            }
            else
            {
                DBG1 (( MF__,"Kernel wait impossible: %s", sqlerrs() ));
                ret = 1;
            }
#endif /* DEBUG_RTE */
        }
        else
        {
            DBG1 (( MF__,"Kernel wait impossible: %s", sqlerrs() ));
            ret = 1;
        }
    }
    else
    {
        DBG1 (( MF__,"Kernel stopped"));
    }
    DBGOUT;

    return ret;
}

/*
 * LOCAL SUBROUTINES
 */
static void SaveCloseError(int closedFd, int errorCode)
{
    int fd = open("/tmp/SaveCloseError.txt", O_RDWR);
    if ( fd >= 0 )
    {
        char errmsg[80];
        sp77sprintf(errmsg, sizeof(errmsg), "close(%d) reports error %d\n",
                closedFd, errorCode );
        lseek(fd, 0, SEEK_END);
        write(fd, errmsg, strlen(errmsg));
    }
    close(fd);
}

static void SaveDupError(int expectedFd, int duppedFd, int errorCode)
{
    int fd = open("/tmp/SaveCloseError.txt", O_RDWR);
    if ( fd >= 0 )
    {
        char errmsg[80];
        sp77sprintf(errmsg, sizeof(errmsg), "dup(%d) != %d reports error %d\n",
                duppedFd, expectedFd, errorCode );
        lseek(fd, 0, SEEK_END);
        write(fd, errmsg, strlen(errmsg));
    }
    close(fd);
}

/*PTS 1113454
  It happened, that sqlxstart was called with stdout closed. To prevent this from 
  producing a severe problem, channel 0/1/2 are opened with /dev/null if not already
  open and stdout/stderr are redirected to /dev/null
 */
static int RedirectStandardDescriptors()
{
  int fd;
  do {
      fd = open ( "/dev/null", O_WRONLY|O_CREAT|O_NDELAY , 0664 );
  } while ( fd >= 0 && fd <= 2 );

  if ( fd >= 0 )
  {
    int DummyFd ;

    while ( -1 == close(1) )
    {
        SaveCloseError(1, errno);
        if ( errno != EINTR )
        {
           return(DO_START_CANT_REDIRECT_STDOUT);
        }
    }
    DummyFd = dup ( fd );
    if ( DummyFd != 1 )
    {
        SaveDupError(1, DummyFd, errno);
      return(DO_START_CANT_REDIRECT_STDOUT);
    }

    while ( -1 == close(2) )
    {
        SaveCloseError(2, errno);
        if ( errno != EINTR )
        {
           return(DO_START_CANT_REDIRECT_STDERR);
        }
    }
    DummyFd = dup ( fd );
    if ( DummyFd != 2 )
    {
        SaveDupError(2, DummyFd, errno);
      return(DO_START_CANT_REDIRECT_STDERR);
    }

    (void) close (fd);
  }
  else
  {
    return(DO_START_CANT_OPEN_DEV_NULL);
  }
  return DO_START_SUCCEEDED;
}

        /* PTS 1106476 */
    #undef MF__
    #define MF__ MOD__"DoStart"
static int DoStart (
  tsp9_cstr   dbname,
  tsp9_pgm_kind pgmKind, 
  int argc,
  char* argv[],
  tsp00_Pathc kernelpgm,
  tsp00_Pathc rundir)
{
  /* child process */
  pid_t kernelPid;
  pid_t returnedPid;
  int   kernelExitStatus;
  char* args[maxArgsC + 1];
  int   argStart;
  int   i;
  int   rc;
  tsp00_DbNamec currentDbname;

  strcpy((char *)&currentDbname[0], dbname); /* Length is checked by caller */

  rc = RedirectStandardDescriptors();
  if ( rc != DO_START_SUCCEEDED )
  {
      return (rc);
  }

  if ( chdir(rundir) != 0 )
  {
      return DO_START_CANT_CHANGE_TO_RUN_DIR;
  }

  /* Put dbname in environment */
  e950_put_dbname_to_env (dbname);

  /* Prepare arguments for starting kernel */
  args [0] = kernelpgm;
  args [1] = (char *)&currentDbname[0];
  argStart = 2;
  argc = min (argc, maxArgsC - argStart);
  for (i = 0; i < argc; ++i) 
  {
      args [i + argStart] = argv [i];
  }
  args [i + argStart] = NULL;

  DBG1 (( MF__,"starting '%s'", kernelpgm ));

  /* Reset flag set by signal handler to indicate kernel is in COLD state */
  en901_ColdStateIndicator = 0;

  /* PTS 1106476
   * Why a second fork ?
   * Not so easy to explain... The kernel expect to have a parent process to whom
   * a startup signal is send. This forces at least one fork. The dbmsrv wants to
   * continue its work, so he cannot wait until the kernel is finished.
   * If he do not terminate after the call, the terminating kernel will become a zombie.
   * The second fork prevents this. Its only task is to start the kernel and wait until
   * it is in COLD state (indicated by a signal). Then the 'process in between' can 
   * terminate. When the kernel finally terminates its parent is already gone so he cannot
   * become a zombie!
   */
  kernelPid = fork();

  if ( kernelPid == -1 )
  {
    return(DO_START_CANNOT_FORK);
  }

  if ( kernelPid == 0 )
  {
    execv ( kernelpgm, args);
    /* after execv failed _exit() is better that exit() ... */
    _exit(1);
    /* end of child process */
  }
 
  /* parent, fork ok */
 
#if _IBMR2 || LINUX
  (void) siginterrupt ( SIGUSR1 , TRUE );
#endif
 
  DBG1 (( MF__,"waiting for kernel %ld...", (long)kernelPid ));

  {
    /* PTS 1107119 Prevent from hanging due to signal lost */
    int timeoutLoop = DO_WAIT_FOR_PID_LOOPS;

    /* PTS 1106045 */
    while ( en901_ColdStateIndicator != SIGUSR1 )
    {
      returnedPid = SaveWaitpid(kernelPid, &kernelExitStatus, WNOHANG);

      if ( returnedPid != 0 )
        break;

      /* Loop until kernel is in cold state */
      sleep(DO_WAIT_FOR_COLD_STATE_SLEEP);

      /* PTS 1107119 Prevent from hanging due to signal lost */
      if ( --timeoutLoop < 0 )
      {
        tsp00_Uint4 dbState;
        if ( en903GetDbPidStateFromShm( NULL, NULL, &dbState, 0 ) == 0 
          && dbState == SERVER_COLD ) /* PTS 1110060 */
        {
          en901_ColdStateIndicator = SIGUSR1;
        }
      }
    }
  }

  DBG1 (( MF__,"en901_ColdStateIndicator signal %d", en901_ColdStateIndicator ));

  if ( en901_ColdStateIndicator != SIGUSR1 )
  {
    if ( !WIFEXITED(kernelExitStatus) )
    {
      return(DO_START_DIED_BEFORE_COLD_STATE);
    }

    return(DO_START_KERNEL_EXIT_CODE_OFFSET + WEXITSTATUS(kernelExitStatus));
  }

  return(DO_START_SUCCEEDED);
}

/*---------------------------------------------------------------------------*/

    #undef MF__
    #define MF__ MOD__"DoStop"
static int DoStop (
    tsp9_cstr dbname, 
    tsp9_cstr dbroot,
    tsp00_Bool dumpflag,
    tsp9_rte_xerror* xerror)
{
    int                    rc ;
    pid_t                  kernelParentPid ;
    pid_t                  kernelPid ;

    eo44initError (xerror);

    rc = en903GetDbPidStateFromShm(&kernelParentPid, &kernelPid, NULL, dumpflag );
    if ( rc == 1 )
    {
        MSG_ERR ((xerror, 11000, ERR_TYPE, "sqlxstop",
                    "Cannot get PID for %s", dbname));
        return STOP_DIRTY;
    }

    if ( rc == 2 )
    {
        MSG_7 ((xerror, -11202, "STOP: Getting Kernel PID failed: %s\n", sqlerrs() ));
        return STOP_IMPOSSIBLE;
    }

    if ( kill ( kernelPid, SIGTERM ) != 0 )
    {
        MSG_7 ((xerror, -11202, "STOP: Kill failed: %s\n", sqlerrs() ));
        return ( errno == EPERM ? STOP_IMPOSSIBLE : STOP_DIRTY );
    }

    if ( DoWaitForPid(kernelParentPid) != 0 )
    {
        return STOP_DIRTY;
    }

    rc = sql41_remove_ipc_resources ( dbname );
    if ( rc == 0 )
    {
        rc = STOP_DONE;
    }
    else
    {
        rc = STOP_DIRTY;
    }

    DBGOUT;
    return rc;
}

    #undef MF__
    #define MF__ MOD__"CheckForRunningProcesses"
static  int     CheckForRunningProcesses (
    tsp9_cstr dbname,
    tsp9_rte_xerror* xerror)
{
    int                             rc ;
    int                             ec ;
    tsp00_TaskId                    dbPid;

    ec = 0 ;

    if ( en41GetPPIDFromFile(dbname, &dbPid) != 0 )
	{
        DBG1 (( MF__,"searching database %s watchdog processes %s error, %s", dbname, line, sqlerrs() ));
        MSG_7 (( xerror, -11987, "searching database watchdog processes error, %s\n", sqlerrs() ));
        return ( 1 );
    }
    if ( dbPid != 0 )
    {
        rc = kill ( (pid_t) dbPid , 0 ) ;
        if (rc == 0 || ( rc != 0 && errno != ESRCH ) )
	    {
            DBG1 (( MF__,"test database process id %ld, error %s", lpid, sqlerrs() ));
            MSG_7 (( xerror, -11987, "test database process id %ld, error %s\n", 
                        (long)dbPid, sqlerrs() ));
            ++ec;
        }
    }

    if ( en41GetPIDFromFile(dbname, &dbPid) != 0 )
	{
        DBG1 (( MF__,"searching database %s processes %s error, %s", dbname, line, sqlerrs() ));
        MSG_7 (( xerror, -11987, "searching database processes error, %s\n", sqlerrs() ));
        return ( 1 );
    }
    if ( dbPid != 0 )
    {
        rc = kill ( (pid_t) dbPid , 0 ) ;
        if (rc == 0 || ( rc != 0 && errno != ESRCH ) )
	    {
            DBG1 (( MF__,"test database process id %ld, error %s", lpid, sqlerrs() ));
            MSG_7 (( xerror, -11987, "test database process id %ld, error %s\n", 
                        (long)dbPid, sqlerrs() ));
            ++ec;
        }
    }

    if (ec != 0) 
	{
        MSG_7 ((xerror, -11987, "found %d already running database processes\n", ec ));
    }

    return ( ec > 0 );
}

    #undef MF__
    #define MF__ MOD__"GetExecutable"
static  char *GetExecutable (
    tsp9_pgm_kind pgmKind )
{
    static char* kernelNames[] = ADABAS_KERNELS ;
    char* result;
    
    switch (pgmKind) {
        case csp9_test_pgm:
        case csp9_fast_pgm:
        case csp9_quick_pgm:
        case csp9_slow_pgm:
            result = kernelNames [pgmKind];
            break;
        default:
            result = kernelNames [csp9_fast_pgm];
            break;
    }
    return result;
}
 
    #undef MF__
    #define MF__ MOD__"en901_CatchColdState"
static  SIGNALFUNCTYPE  en901_CatchColdState (int sig)
{
    en901_ColdStateIndicator = sig;
}

#define DONT_ASK_FOR_CREATION   FALSE

    #undef MF__
    #define MF__ MOD__"RundirExists"
static int 
RundirExists (
    tsp9_cstr dbname, 
    tsp00_Pathc rundir,
    tsp9_rte_xerror* xerror)
{
    int rd_exist;

    if ( !(rd_exist = sqlXParamCheckExists(dbname)) )
    {
      MSG_9 ((xerror, -11987, "Paramfile for serverdb <%s> not found!",dbname ));
    }
    else 
    {
      rd_exist = GetRunDirectory ( dbname, rundir );
      if (!rd_exist) 
      { 
        MSG_9 ((xerror, -11987, "Parameter RUNDIRECTORY is missing!" ));
      }
    }
    return (rd_exist);
}

    #undef MF__
    #define MF__ MOD__"CheckForDbStartable"
static int 
CheckForDbStartable (
    tsp9_cstr dbname,        /* in */
    tsp9_cstr dbroot,        /* in */
    char *full_knl_path,     /* out */
    tsp9_pgm_kind pgmKind,      /* in */
    tsp00_Pathc rundir,       /*out */
    tsp9_rte_xerror* xerror) /* out */
{
    int rc ;
    char ipcdb [ MAX_PATH_LEN ]; 
    char ipcus [ MAX_PATH_LEN ];
    char* shortpgm;

    if ( !dbroot || !dbroot[0] )
    {
        MSG_9 ((xerror, -11987, 
                "Internal usage error: No dbroot given for '%s'", dbname ));
        return 1;
    }

    DBG1 (( MF__,"serverdb <%s> ...", dbname ));

    shortpgm = GetExecutable (pgmKind);
    sprintf ( full_knl_path, "%s/pgm/%s", dbroot, shortpgm) ;
    /* PTS 1110606 */
    if ( !CheckFileAccess ( full_knl_path, S_IXUSR ) )
    {
        if ( CheckFileDoesNotExist( full_knl_path ) )
        {
            MSG_ERR ((xerror, 10108,ERR_TYPE,"start   ","kernel program missing '%s'" , full_knl_path ));
        }
        else
        {
            MSG_ERR ((xerror, 10108,ERR_TYPE,"start   ","kernel program not executable '%s'" , full_knl_path ));
        }
        return 1;
    }

    rc = sql41_remove_ipc_resources ( dbname );
    if ( 0 != rc )
    {
        sql41_get_ipc_dir( ipcdb );
        strcat( ipcdb, "/db:" );
        strcat( ipcdb, dbname );
        sql41_get_ipc_dir( ipcus );
        strcat( ipcus, "/us:" );
        strcat( ipcus, dbname );

        if ( CheckFileAccess ( ipcdb, S_IFDIR ) )
        {
            MSG_9 ((xerror, -11987, "IPC directory '%s' still exists", ipcdb ));
            return 1;
        }

        if ( CheckFileAccess ( ipcus, S_IFDIR ) )
        {
            MSG_9 ((xerror, -11987, "IPC directory '%s' still exists", ipcus ));
            return 1;
        }
    }

    DBG1 (( MF__,"accesses ok" ));
    rc = RundirExists ( dbname, rundir, xerror );
    if (rc) 
    {
        rc = CheckFileAccess ( rundir, F_RWX );
        if (!rc) 
        {
            MSG_9 ((xerror, -11987, 
                "You don't have read+write+execute permission on the run-directory '%s'",
                rundir ));
            return 1;
        }
    }

    /* allowed to start... */
    return 0;
}

    #undef MF__
    #define MF__ MOD__"RemoveRunningProcesses"
static void RemoveRunningProcesses (
    tsp9_cstr dbname,
    tsp9_rte_xerror* xerror)
{
    int                             rc ;
    tsp00_TaskId                    dbPid = 0;

    if ( en41GetPPIDFromFile(dbname, &dbPid) != 0 )
	{
        MSG_7 (( xerror, -11987, "searching database watchdog processes error, %s\n", sqlerrs() ));
    }
    else if ( dbPid != 0 )
    {
        do
        {
            rc = kill ( (pid_t) dbPid , 0 ) ;
            if ( 0 == rc )
            {
                rc = kill( (pid_t) dbPid, SIGKILL );
                if ( rc != 0 && errno != EINTR && errno != ESRCH )
                {
                    MSG_7 (( xerror, -11987, "killing database watchdog pid %ld, error %s\n", 
                                (long)dbPid, sqlerrs() ));
                    break;
                }
                sleep(0);
                rc = kill ( (pid_t) dbPid , 0 ) ;
            }
        } while ( rc == 0 || errno != ESRCH );
    }

    if ( en41GetPIDFromFile(dbname, &dbPid) != 0 )
	{
        MSG_7 (( xerror, -11987, "searching database processes error, %s\n", sqlerrs() ));
    }
    else if ( dbPid != 0 )
    {
        do
        {
            rc = kill ( (pid_t) dbPid , 0 ) ;
            if ( 0 == rc )
            {
                rc = kill( (pid_t) dbPid, SIGKILL );
                if ( rc != 0 && errno != EINTR && errno != ESRCH )
                {
                    MSG_7 (( xerror, -11987, "killing database pid %ld, error %s\n", 
                                (long)dbPid, sqlerrs() ));
                    break;
                }
                sleep(0);
                rc = kill ( (pid_t) dbPid , 0 ) ;
            }
        } while ( rc == 0 || errno != ESRCH );
    }
}

#undef MF__
#define MF__ MOD__"UpdateInternalValues"
static void UpdateInternalValues( tsp9_cstr *pDbname, tsp9_cstr *pDbroot )
{
    int ret = 0;
    int needUpdate = 0;

    /*
     *  get DBNAME
     */
    if ( *pDbname == NULL )
    {
        *pDbname = en903dbname ();
    }
    else
    {
        tsp9_cstr currentDbname = en903dbname();
        if ( currentDbname != NULL 
          && strcmp(currentDbname, *pDbname) != 0 )
        {
            needUpdate = 1;
        }
    }

    /*
     *  get DBROOT
     */
    if ( *pDbroot == NULL) 
    {
        *pDbroot = en903dbroot ();
    }
    else
    {
        tsp9_cstr currentDbroot = en903dbroot();
        if ( currentDbroot != NULL 
          && strcmp(currentDbroot, *pDbroot) != 0 )
        {
            needUpdate = 1;
        }
    }

    if ( needUpdate != 0 )
    {
        sqlxfinish();
        sqlxinit(*pDbname, *pDbroot);
    }
}

/*---------------------------------------------------------------------------*/

/*
 * function: en901_SearchSubPath
 * description:
 * Given path is read from environment. It contains the complete path specification for
 * all shared library paths configurated. Each path element is separated by a colon ':'.
 * arguments: GivenPath [in] the complete path environment string
 *            SubPath [in] the path to search
 * return value: 0 if subpath was not found, 1 if found
 */
#undef MF__
#define MF__ MOD__"en901_SearchSubPath"
static int en901_SearchSubPath(const char *GivenPath, const char *SubPath)
{
  int SearchResult = 0;
  const char *SearchPath = GivenPath;

  if ( SearchPath != NULL )
  {
    /* Loop over each path element */
    while ( *SearchPath != '\0' )
    {
      while ( *SearchPath == ':' ) ++SearchPath;

      if ( !strncmp(SearchPath, SubPath, strlen(SubPath)) )
      {
        SearchResult = 1;
        break;
      }

      while ( *SearchPath != ':' 
           && *SearchPath != '\0' ) ++SearchPath;
    }
  }

  return SearchResult;
}

/*---------------------------------------------------------------------------*/

/* PTS 1107044 */
/*
 * function: en901_UpdateEnvironment
 * description:
 * Update environment of starting kernel. It updates the sharedc library paths, DBROOT
 * and any extra environment needed platform dependend.
 * arguments: dbroot [in] the dbroot of the starting kernel
 *            xerror [out] the error return
 * return value: 0 if update failed (message is written), 1 if successfull
 */
#undef MF__
#define MF__ MOD__"en901_UpdateEnvironment"
static int en901_UpdateEnvironment(const char *dbroot, tsp9_rte_xerror* xerror)
{
  static char *envDBROOT = NULL;
  char *oldenvDBROOT = NULL;
  tsp01_RteError RteError;

  /*--------------------*/
  if ( !sqlUpdateLibPathEnvironment(&RteError) )
  {
    MSG_ERR ((xerror, 10104,ERR_TYPE,"start   ", RteError.RteErrText ));
    return 0;
  }

  /* Always set DBROOT
     PTS 1105164 
   */
  oldenvDBROOT = envDBROOT;
  envDBROOT = (char *)malloc(strlen("DBROOT=") + strlen(dbroot) + 1);

  if ( envDBROOT != NULL )
  {
    strcpy(envDBROOT, "DBROOT=");
    strcat(envDBROOT, dbroot);

    DBG1 (( MF__,"try putenv(%s)", envDBROOT ));

    if ( putenv(envDBROOT) < 0 )
    {
      MSG_ERR ((xerror, 10104,ERR_TYPE,"start   ","putenv DBROOT failed" ));
      free(envDBROOT);
      return 0;
    }
    else if ( oldenvDBROOT != NULL )
    {
      free(oldenvDBROOT); /* prevent some memory leak, accept that last malloc is not freed... */
    }
  }
  else
  {
    MSG_ERR ((xerror, 10104,ERR_TYPE,"start   ","DBROOT too long" ));
    return 0;
  }

  /* ------------------------------ */
  /* Platform dependend environment */
  /* ------------------------------ */

#if defined(AIX)
#  if defined(BIT64)
   if ( putenv("EXTSHM=OFF") != 0 )
   {
     MSG_ERR ((xerror, 10104,ERR_TYPE,"start   ","Cannot putenv EXTSHM=OFF" ));
     return 0;
   }
#  else
  /* set EXTSHM on 32 Bit AIX to avoid shm attach problems
    PTS 1106620 
   */
   if ( putenv("EXTSHM=ON") != 0 )
   {
     MSG_ERR ((xerror, 10104,ERR_TYPE,"start   ","Cannot putenv EXTSHM=ON" ));
     return 0;
   }
#  endif
#endif

  return 1;
}

/*-----------------------------------------------------------*/

static int CheckFileAccess ( char *path, int   mode )
{
  struct stat buf;
  int access_ok ;

  access_ok = stat ( path, &buf )  == 0 && (buf.st_mode & mode) == mode ;
  DBG1 (( "trying to access path <%s> with mode %02x\n", path, mode ));
  return ( access_ok ) ;
}

/*-----------------------------------------------------------*/

static int CheckFileDoesNotExist ( char *path )
{
  struct stat buf;

  return ( stat ( path, &buf ) != 0 && errno == ENOENT );
}

/*-----------------------------------------------------------*/

static int CreatePath ( char * path_to_create, unsigned short mode )
{
  int create_ok = TRUE ;
  char *act_pos = path_to_create , *end_pos;

  if ( *act_pos == PATH_DELIMITER_EO01 ) act_pos++ ;

  do
    { if ( (end_pos = (char *)strchr ( act_pos, PATH_DELIMITER_EO01 )) != CNULL )
        *end_pos = '\0' ;
      if ( !CheckFileAccess ( path_to_create , S_IFDIR ) )
      {
        DBG1 (( "\ncreating directory <%s>", path_to_create ));
        create_ok = mkdir ( path_to_create, mode ) == 0;
      }
      if ( end_pos != CNULL )
        { *end_pos = PATH_DELIMITER_EO01; act_pos = end_pos+1 ; }
    }
  while ( create_ok && end_pos != CNULL ) ;
  return ( create_ok ) ;
}

/*-----------------------------------------------------------*/

static char *expand_env_vars ( char *path )
{
  int len ;
  tsp00_Pathc TmpTmpFile,exp_path ;
  char *beg_env_var, *end_env_var, *env_var_val,
       *act_pos, *exp_path_pos;

  DBG1 (( "expand_env_vars: before expand <%s>\n", path ));
  
  do                                             /* PTS 1000713 */
  { 
    beg_env_var  = end_env_var = CNULL ;
    act_pos      = path;
    exp_path_pos = exp_path ;

    strcpy( TmpTmpFile, path ) ;
    while ( ( beg_env_var = strchr ( act_pos      , BEG_ENV_VAR ) ) != CNULL &&
          ( end_env_var = strchr ( beg_env_var+1, END_ENV_VAR ) ) != CNULL )
    { *end_env_var = '\0' ;
      if ( ( env_var_val = (char *)getenv ( beg_env_var+1 ) ) != CNULL )
        { len = (int ) (beg_env_var - act_pos) ;
#ifndef WIN32
          if ( * ( beg_env_var - 1 ) == '$' ) len-- ;
#endif
          if ( len > 0 )
            strncpy ( exp_path_pos, act_pos, len ) ;
          strcpy ( exp_path_pos + len, env_var_val );
          exp_path_pos += len + strlen ( env_var_val ) ;
          act_pos = end_env_var + 1 ;
        }
    }

    if ( end_env_var != CNULL || beg_env_var != CNULL )
    {
      strcpy ( exp_path_pos, act_pos ) ;
      strcpy ( path, exp_path ) ;
    }
    
  }
  while ( strcmp( TmpTmpFile, path ) != 0 ) ;

  DBG1 (( "expand_env_vars: after expand <%s>\n", path ));
  return ( path ) ;
}

/*-----------------------------------------------------------*/

static int GetRunDirectory ( tsp9_cstr dbname, tsp00_Pathc rundir)
{
  int rc;
  tsp00_C64c ParamVal;
  tsp01_RteError          RteError ;

  eo11GetParam ( dbname , PAN_RUNDIR, ParamVal, &RteError ) ;
  if ( (rc = RteError.RteErrCode == RTE_NO_ERROR_SP01) )
  {
      strcpy(rundir, ParamVal);
      expand_env_vars ( rundir ) ;
  }
  return ( rc ) ;
}

/*-----------------------------------------------------------*/

/* PTS 1109708 */
/*!
  Function: SaveWaitpid
  Description: EINTR save version of waitpid
   Difference from waitpid call. This call will only return, if
   the pid was really returned (collecting other waited for processes
   without notion)!

  Arguments: pidToWaitFor [in]
             pStatus [out] filled if != 0
             flags [in] if WNOHANG is set, 0 is a possible return value
  Return value: (pid_t)-1 on error other than EINTR
                pidToWaitFor if process exited
                0 if WNOHANG was specified and process did not already exited
 */
static pid_t SaveWaitpid(pid_t pidToWaitFor, int *pStatus, int flags)
{
    pid_t returnedPid;

    do {
      returnedPid = waitpid(pidToWaitFor, pStatus, flags);
    } while ( ( (returnedPid > 0)            && (returnedPid != pidToWaitFor) ) 
           || ( (returnedPid == ((pid_t)-1)) && (errno == EINTR)            ) );
    return returnedPid;
}
