/*!***************************************************************************

  module      : ven903.c

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Client RunTime
  description : init Control Session

  last changed: 2001-09-19  13:35
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/
#define MOD__ "ven903.c:"

#include "gen00.h"
#include "heo00.h"
#include "geo007_1.h"
#include "gen34.h"
#include "gsp09.h"
#include "hsp100.h"
#include "heo01.h"
#include "hen40.h"

/**\
--------------------------------------- MF__ MOD__ (imports)
\**/

extern  key_t                         sql41_get_server_key (tsp9_cstr dbname);


/**\
--------------------------------------- MF__ MOD__ (module functions)
\**/

void en903unmap_shm (void);

/**\
--------------------------------------- MF__ MOD__ (module variables)
\**/
static SQL_DBNAMEC                             dbname_MV = { 0 };
static tsp00_Pathc                             dbroot_MV = { 0 };
static int                                     shmid_MV = 0;
static int                                     accessMode_MV = 0;

/**\
--------------------------------------- MF__ MOD__ (session)
\**/

    #undef MF__
    #define MF__ MOD__"sqlxinit"
void
sqlxinit (
    tsp9_cstr dbname,
    tsp9_cstr dbroot)
{
    DBGIN;
    DBG1(( MF__,"OLD dbname: <%s> dbroot: <%s> kgs 0x%lx", 
              dbname_MV, dbroot_MV, kgs ));
    DBG1(( MF__,"dbname: <%s> dbroot: <%s>", 
              dbname ? dbname : "",
              dbroot ? dbroot : "" ));
    strcpy ((char *)dbname_MV, (char *)dbname);
    strcpy (dbroot_MV, dbroot);
    kgs = NULL;
    DBGOUT;
}

    #undef MF__
    #define MF__ MOD__"sqlxfinish"
void
sqlxfinish (void)
{
    DBGIN;
    DBG1(( MF__,"dbname: <%s> dbroot: <%s> kgs 0x%lx", 
              dbname_MV, dbroot_MV, kgs ));
    dbname_MV [0] = '\0';
    dbroot_MV [0] = '\0';
    if (kgs != NULL) {
        /* detach segment */
        en903unmap_shm ();
    }
    DBGOUT;
}

/**\
--------------------------------------- MF__ MOD__ (session parameters)
\**/

tsp9_cstr
en903dbname (void)
{
    return (tsp9_cstr)dbname_MV;
}

tsp9_cstr
en903dbroot (void)
{
    return (tsp9_cstr)dbroot_MV;
}

/**\
--------------------------------------- MF__ MOD__ (kernel kgs)
\**/

    #undef MF__
    #define MF__ MOD__"check_kernel_MF"
static	int
check_kernel_MF (void)
{
    int                         rc ;


    if ( kgs == (struct KERNEL_GLOBAL_SECTION *) 0 ||
         kgs == (struct KERNEL_GLOBAL_SECTION *) -1 )
    {
        return ( 1 );
    }
    rc = kill ( kgs->pid , 0 );
    if ( rc != 0 && errno == ESRCH )
    {
	    MSGD (( ERR_DATABASE_NOT_STARTED, dbname_MV));
        return ( 1 );
    }
    return ( 0 );
}

/**\
--------------------------------------- MF__ MOD__ (check for running processes)
\**/

    #undef MF__
    #define MF__ MOD__"check_kernel_MF"
static 
bool en903checkForRunningProcesses (
    bool *dbRunning )
{
    SAPDB_Int      rc ;
    tsp00_TaskId   dbWatchdogPid;
    tsp00_TaskId   dbKernelPid;

    if ( (en41GetPPIDFromFile(dbname_MV, &dbWatchdogPid) != 0) || (dbWatchdogPid == 0) )
	{   /* the watchdog process did not write its pid file. This is the normal case when */
        /* the database is simply not running at all. */
        *dbRunning = false;
        return (true);
    }

    rc = kill ( (pid_t) dbWatchdogPid , 0 ) ;
    if ( rc == 0 || errno != ESRCH )
    {   /* check explicit for ESRCH since watchdog process is running even 
           if current user is not authorized to send signal... */
        *dbRunning = true;
        return (true);
    }

    /* now we know watchdog process does not run */

    if ( (en41GetPIDFromFile(dbname_MV, &dbKernelPid) != 0) || (dbKernelPid == 0) )
	{   /* while kernel process did not write its pid file, state is OFFLINE */
        *dbRunning = false;
        return (true);
    }

    rc = kill ( (pid_t) dbKernelPid , 0 ) ;
    if(rc == 0 || errno != ESRCH)
    {   /* The watchdog does not exist (Terminated by unfriendly user?) but the
           kernel process does exist... Be friendly and call it 'running...' since
           we only loose the cleanup and flush of knltrace after kernel stopped */
        *dbRunning = true;
    }
    else
    {   /* both processes do not exist */
        *dbRunning = false;
    }

    return (true);
}

/**\
--------------------------------------- 
\**/


    #undef MF__
    #define MF__ MOD__"en903GetDbPidStateFromShm"
int
en903GetDbPidStateFromShm (
    pid_t *pParentPid,
    pid_t *pPid,
    tsp00_Uint4 *pState,
    tsp00_Bool wantDump )
{
    key_t                           commonRTEKey;
    int                             rc ;
    struct  KERNEL_GLOBAL_SECTION *KgsNotRemapped;
    bool dbRunning;
    tsp9_rte_xerror xerror;

    DBGIN;
    /* first check if the kernel is running at all - if it wasn't, looking in the shared memory would not make any sense */

    if(!en903checkForRunningProcesses( &dbRunning ))
    {
        return 2;
    }
    if(!dbRunning)
    {
        if ( pState )
        {
            *pState = SERVER_OFFLINE;
        }
    	MSGD (( ERR_DATABASE_NOT_STARTED, dbname_MV));
        return 1;
    }

    if (accessMode_MV != 0) /* already attached ? */
    {
        rc = check_kernel_MF ();
        if ( rc == 0 )
        {
            if ( pParentPid )
            {
                *pParentPid = kgs->parentPid;
            }

            if ( pPid )
            {
                *pPid = kgs->pid;
            }

            if ( pState )
            {
                *pState = kgs->state;
            }
        }
        return rc;
    }

    commonRTEKey = sql41_get_server_key ( dbname_MV );
    if ( commonRTEKey == (key_t)-1 )
    {
        DBG1 (( MF__,"Server key not creatable\n" ));
        return ( 1 );
    }

    if ( commonRTEKey == (key_t)-2 )
    {
        DBG1 (( MF__,"Server key file invalid\n" ));
        return ( 1 );
    }

    /*
     *  Get shmid
     */
    shmid_MV = shmget ( commonRTEKey , 0 , 0 );
    if ( shmid_MV < 0 )
    {
        MSGD (( ERR_IPC_SEMC_SEMGET, sqlerrs() ));

        if ( errno == ENOENT )
        {
    	    MSGD (( ERR_DATABASE_NOT_STARTED, dbname_MV));
            return ( 1 );
        }
        if ( errno == EPERM || errno == EACCES )
        {
            return ( 2 );
        }
        return ( 2 );
    }
    DBG1 (( MF__,"shmid_MV %d \n", shmid_MV ));

    /*
     *  it tries to get write permission, to flag dump wanted....
     */
    KgsNotRemapped = sql34MapKgsNotRemapped ( shmid_MV, wantDump );
    if ( !KgsNotRemapped )
    {
        return ( 2 );
    }
    DBG1 (( MF__,"attached at address 0x%lx \n", (long) KgsNotRemapped ));

    if ( IsEqual_esp100 != sp100_CompareVersionIDs ( &KgsNotRemapped->RTEVersionID ) )
    {
		tsp00_Versionc currentVersion;
		
		sp100_GetVersionString( COMP_NAME_RTE_SP100, 
								s100buildnumber,
								currentVersion);

        MSGD(( ERR_WRONG_DB_VERSION ));
        DBG1 (( MF__,"Incompatible version of running kernel and console!\n" ));
        DBG1 (( MF__,"Running kernel-version is: %s\n", KgsNotRemapped->RTEVersion ));
        DBG1 (( MF__,"Actual console-version is: %s\n", currentVersion ));
        sql34UnmapKgsNotRemapped(KgsNotRemapped);
        return ( 2 );
    }

    if ( pParentPid )
    {
        *pParentPid = KgsNotRemapped->parentPid;
    }

    if ( pPid )
    {
        *pPid = KgsNotRemapped->pid;
    }

    if ( pState )
    {
        *pState = KgsNotRemapped->state;
    }

    rc = kill ( KgsNotRemapped->pid, 0 );
    if ( rc != 0 && errno == ESRCH )
    {
    	MSGD (( ERR_DATABASE_NOT_STARTED, dbname_MV));
	    sql34UnmapKgsNotRemapped(KgsNotRemapped);
        return ( 1 );
    }

    if ( wantDump )
    {
        KgsNotRemapped->wantDump = 1;
    }

	/* PTS 1113038 */
	sql34UnmapKgsNotRemapped(KgsNotRemapped);

    DBGOUT;

    return ( 0 );
}

    #undef MF__
    #define MF__ MOD__"en903map_shm"
int
en903map_shm (
    int  shmflag)
{
    key_t                           commonRTEKey;
    int                             rc ;
    DBG1 (( MF__,"called,    shmflag %#o \n", shmflag ));

    if (accessMode_MV != 0) /* already attached ? */
    {
        DBG1 (( MF__,"en903map_shm: already attached, access mode %d \n", accessMode_MV ));
        if ( accessMode_MV == 1 && shmflag != SHM_RDONLY )
        {
            DBG1 (( MF__,"Permission denied \n" ))
            return ( 2 );
        }
        return ( check_kernel_MF () );
    }

    commonRTEKey = sql41_get_server_key ( dbname_MV );
    if ( commonRTEKey == (key_t)-1 )
    {
        DBG1 (( MF__,"Server key not creatable\n" ));
        return ( 1 );
    }

    if ( commonRTEKey == (key_t)-2 )
    {
        DBG1 (( MF__,"Server key file invalid\n" ));
        return ( 1 );
    }

    /*
     *  Get shmid
     */
    shmid_MV = shmget ( commonRTEKey , 0 , 0 );
    if ( shmid_MV < 0 )
    {
        MSGD (( ERR_IPC_SEMC_SEMGET, sqlerrs() ));
        if ( errno == ENOENT )
        {
    	    MSGD (( ERR_DATABASE_NOT_STARTED, dbname_MV));
            return ( 1 );
        }
        if ( errno == EPERM || errno == EACCES )
        {
            DBG1 (( MF__,"Cannot attach to Kernel Shared Memory Segment: \n" ));
            DBG1 (( MF__,"Permission denied \n" ));
            return ( 2 );
        }
        return ( 2 );
    }
    DBG1 (( MF__,"shmid_MV %d \n", shmid_MV ));

    /*
     *  if the flag parameter to 'map_kgs' is -1,
     *  it tries to get write permission, else to get read permission
     */
    accessMode_MV = 2 ;
    rc = sql34_map_kgs ( shmid_MV , -1 );
    if ( rc )
    {
        if ( rc == 1 )
        {
            accessMode_MV = 1 ;
            /* if ( shmflag == 0 ) */
            if ( shmflag != SHM_RDONLY )
            {
                DBG1 (( MF__,"Permission denied \n" ));
                return ( 2 );
            }
        }
        else
        {
            accessMode_MV = 0 ;
            return ( 2 );
        }
    }
    DBG1 (( MF__,"attached at address 0x%08lx \n", (long) kgs ));

    if ( kgs == (struct KERNEL_GLOBAL_SECTION *) 0 ||
         kgs == (struct KERNEL_GLOBAL_SECTION *) -1 )
    {
    	MSGD (( ERR_DATABASE_NOT_STARTED, dbname_MV));
        return ( 1 );
    }

    if ( IsEqual_esp100 != sp100_CompareVersionIDs ( &kgs->RTEVersionID ) )
    {
		tsp00_Versionc currentVersion;
		
		sp100_GetVersionString( COMP_NAME_RTE_SP100, 
								s100buildnumber,
								currentVersion);

        MSGD(( ERR_WRONG_DB_VERSION ));
        DBG1 (( MF__,"Incompatible version of running kernel and console!\n" ));
        DBG1 (( MF__,"Running kernel-version is: %s\n", kgs->RTEVersion ));
        DBG1 (( MF__,"Actual console-version is: %s\n", currentVersion ));
        return ( 2 );
    }

    if ( check_kernel_MF () )
    {
    	MSGD (( ERR_DATABASE_NOT_STARTED, dbname_MV));
        return ( 1 );
    }

    DBGOUT;
    return ( 0 );
}

    #undef MF__
    #define MF__ MOD__"en903unmap_shm"
void
en903unmap_shm (void)
{
    int         rc ;

    DBGIN;

    if ( kgs == (struct KERNEL_GLOBAL_SECTION *) 0 ||
         kgs == (struct KERNEL_GLOBAL_SECTION *) -1 )
    {
        return;
    }

    /* KWe
     * We can only detach the shm, if the accessMode_MV == 1 or 2
     */
    if (accessMode_MV != 0)
    {
        accessMode_MV = 0;
        rc = sql34_unmap_kgs () ;
        if ( rc != 0 )
        {
            DBG1 (( MF__,"sql34_unmap_kgs fails %s\n", sqlerrs() ))
        }
    }

    DBGOUT;
}
