/*!**************************************************************************

  module      : ven904.c

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Client RunTime
  description : Database and Installation  Registry

  last changed: 2001-09-21  14:24
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/
#define MOD__ "ven904.c:"
#include <sys/stat.h>
#include "gen00.h"
#include "heo00.h"
#include "gsp01.h"
#include "hen09.h"
#include "gsp09.h"
#include "gen950.h"
#include "geo001.h"
#include "geo57.h"
#include "gen904.h"

#define XOK(xerror)	((xerror)->xe_result == 0)
#define forUpdate_MC	TRUE
#define readOnly_MC	FALSE
#define DB_IPC_INTRO "db:"
#define DB_IPC_INTRO_LEN 3


#if defined (USE_MY_INI)
# define iniFileNameC "MySAP_DBTech.ini"
#else
# define iniFileNameC "SAP_DBTech.ini"
#endif 
#define globalSectionC        "Globals"
#define installationsSectionC "Installations"
#define databasesSectionC     "Databases"
#define maxVersionStringC	100

#if defined (DEBUG_TRACE) 
extern FILE* traceFileCS;
#define TRACE(x) fprintf x
#else
#define TRACE(x)
#endif

#if defined (ENUMINST_ONLY)

/* uses stdmalloc, otherwise, most of RTE has to be included */
#undef ALLOC_MEM_EO57
#define ALLOC_MEM_EO57(target, size)    ((*(target) = malloc (size)) == NULL)
#undef FREE_MEM_EO57
#define FREE_MEM_EO57(ptr)              free (ptr);

#else

/* PTS 1110181 */
#define registryDcomFileNameC "Registry_dcom.ini"
#define registryDcomPrefixC "Registry"
#define registryDcomExtensionC ".dcom"
#define registryIndexSectionC "RegistryIndex"
#define registryMapSectionC   "RegistryMap"
static void en904_CreateDcomRegistryLink(tsp09_cstr dbroot, tsp9_rte_xerror *xerror);
static void en904_RemoveDcomRegistryLink(tsp09_cstr dbroot, tsp9_rte_xerror *xerror);
static void en904_TryCopy  (  const char *SrcName, const char *DestName );

#endif

static char* kernelNames[] = ADABAS_KERNELS ;
static bool dbHasRunningKernel (tsp9_rte_dbinfo *dbinfo, const char *speed);


/**\
--------------------------------------- MF__ MOD__ (Installation Registry)
\**/

    #undef MF__
    #define MF__ MOD__"parseVersion"
static void
parseVersion (
    const char * versionString,
    tsp9_release_version * version)
{
    int versionTmp [csp9_version_digits];
    int i;
    memset (versionTmp, '\0', sizeof (version));
    sscanf (versionString, "%d.%d.%d.%d",
	    &versionTmp [0], &versionTmp [1], &versionTmp [2], &versionTmp [3]);
    for (i = 0; i < csp9_version_digits; ++i) {
	version->no [i] = (signed char) versionTmp [i];
    }
}

/**\
--------------------------------------- MF__ MOD__ (Installation Registry)
\**/

    #undef MF__
    #define MF__ MOD__"parseVersionNew"
static void
parseVersionNew (
    const char * versionString,
    tsp9_release_version * version,
    tsp00_Pathc            key)     /* PTS 1104801 */
{
    int versionTmp [csp9_version_digits];
    char FormatString[100] ;
    int i;

    memset (key, '\0', sizeof (tsp00_Pathc));
    memset (versionTmp, '\0', sizeof (versionTmp));
    sprintf(FormatString, "%%d.%%d.%%d.%%d,%%%ds", sizeof(tsp00_Path) ) ;

    DBG1 (( MF__,"FormatString : %s\n",FormatString ));
    DBG1 (( MF__,"versionString: %s\n",versionString ));

    sscanf (versionString, FormatString,
	    &versionTmp [0], &versionTmp [1], &versionTmp [2], &versionTmp [3], key);

    DBG1 (( MF__,"Version: %d.%d.%d.%d\n",
            versionTmp [0], versionTmp [1], versionTmp [2], versionTmp [3] ));
    DBG1 (( MF__,"Key    : %s\n", key ));

    for (i = 0; i < csp9_version_digits; ++i) 
    {
    	version->no [i] = (signed char) versionTmp [i];
    }
}

#if !defined (ENUMINST_ONLY)
/* PTS 1104330 */
/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"sqlxregisterglobal"
tsp00_Bool sqlxregisterglobal ( const char      *GlobalKey,
                                char            *GlobalValue,
                                tsp01_RteError  *RteError )
{
    tsp00_Bool    ok;
    unsigned char rc;
    tsp00_ErrText errtext;

    if ( !sqlPutConfigString (iniFileNameC, globalSectionC,
	                          GlobalKey, GlobalValue, errtext, &rc))
    {
      ok = FALSE ;
      eo46_set_rte_error (RteError, 0, errtext, GlobalKey);
	}
    else
      ok = TRUE ;
    return(ok);
}

/* PTS 1104330 */
/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"sqlxgetglobal"
tsp00_Bool sqlxgetglobal ( const char      *GlobalKey,
                                  char            *GlobalValue,
                                  int              MaxValueLen,
                                  tsp01_RteError  *RteError )
{
    tsp00_Bool    ok;
    unsigned char rc;
    tsp00_ErrText errtext;

    if (!sqlGetConfigString (iniFileNameC, globalSectionC,
	                         GlobalKey, GlobalValue, MaxValueLen,  errtext, &rc))
    {
      ok = FALSE ;
      eo46_set_rte_error (RteError, 0, errtext, GlobalKey);
	}
    else
      ok = TRUE ;
    return(ok);
}

/*----------------------------------------*/
/* PTS 1110181 removed sqlxregisterinst and added symbolic link generation for dcom registry */
/*!
  Function: en904_AddNewDcomRegistryFile
  Description: Add a new entry to a registry file for DCOM registry

  This is not optimized for performance.
  Beginning with lowest possible number a file name is created and the mapping entry searched.
  If a name is found, that has no corresponding entry, the name is used to create both mappings.

  Arguments: dbroot [in] path to dbroot
             registryFilePath [out] path to DCOM registry file
             xerror [out] set if path too long
 */
static void en904_AddNewDcomRegistryFile( tsp09_cstr dbroot, tsp00_Pathc registryFilePath, tsp9_rte_xerror *xerror)
{
    unsigned char rc;
    tsp00_ErrText errtext;
    int           numberToUse = 0;
    int           lengthOfConfigPath;
    tsp00_Pathc   mappedDbroot;

    memset(registryFilePath, 0, sizeof(tsp00_Pathc));
    sqlGetConfigPath(registryFilePath, true);
    lengthOfConfigPath = strlen(registryFilePath);

    do {
        int   number;
        char  numberBuffer[13];
        char *mostSignicantDigit;

        /* get decimal number */
        memset(numberBuffer, 0, sizeof(numberBuffer));
        mostSignicantDigit = &numberBuffer[sizeof(numberBuffer)-1];
        for ( number = ++numberToUse; number > 0; number /= 10 )
        {
            *--mostSignicantDigit = '0' + (number % 10);
        }
           
        if ( ( lengthOfConfigPath
             + strlen(registryDcomPrefixC) 
             + strlen(mostSignicantDigit)
             + strlen(registryDcomExtensionC) ) > sizeof(tsp00_Path) )
        {
            eo44anyError (xerror, "config path too long");
            return;
        }

        registryFilePath[lengthOfConfigPath] = 0;
        strcat(registryFilePath, registryDcomPrefixC);
        strcat(registryFilePath, mostSignicantDigit);
        strcat(registryFilePath, registryDcomExtensionC);

    } while ( sqlGetConfigString( registryDcomFileNameC, registryMapSectionC,
                                  registryFilePath, mappedDbroot, sizeof(tsp00_Pathc), errtext, &rc) );

    /* If file not exists, config_error_open_chen09 is returned */
    /* If section or key was not found, config_error_section_chen09 is returned */
    if ( rc != config_error_open_chen09 
      && rc != config_error_section_chen09 )
    {
        /* all other errors are deadly */
        eo44anyError (xerror, errtext);
        return;
    }

    if ( !sqlPutConfigString( registryDcomFileNameC, registryMapSectionC,
                              registryFilePath, dbroot, errtext, &rc ) )
    {
        eo44anyError (xerror, errtext);
        return;
    }

    if ( !sqlPutConfigString( registryDcomFileNameC, registryIndexSectionC,
                              dbroot, registryFilePath, errtext, &rc ) )
    {
        eo44anyError (xerror, errtext);
        sqlRemoveConfigString (registryDcomFileNameC, registryMapSectionC,
                        	   registryFilePath, errtext, &rc);
        return;
    }
}

/*
  Function: en904_TryCopy
  Description: Try to copy file, but don't care if it fails
  Arguments: SrcName [in] source file name
             DescName [in] destination file name
  Try to copy, ignoring any errors...
 */
static void en904_TryCopy  (  const char *SrcName,
                              const char *DestName )
{
  #undef  MF__
  #define MF__ MOD__"e541_filecopy"

  FILE    *src_stream;
  FILE    *dest_stream;
#define COPY_BUFFER_SIZE (32*1024)
  char buf [ COPY_BUFFER_SIZE ] ;
  int num_read, num_written;

  DBGIN;

  src_stream = fopen ( SrcName, "rb" ) ;
  if ( !src_stream ) return;

  dest_stream = fopen ( DestName, "wb" ) ;
  if ( dest_stream )
  {
      do
      {
          num_read = fread ( (char *)buf, sizeof(char),
								     sizeof ( buf ) , src_stream );
          if ( num_read < 0 )
	      {
              break;
	      }
          else
          {
              num_written = fwrite ( (char *)buf, sizeof(char), 
								    num_read, dest_stream ) ;
              if ( num_written < 0 )
              {
                  break;
              }
          }
      }
      while ( num_read > 0 && num_read == num_written ) ;

      fclose ( dest_stream ) ;
  }
  fclose ( src_stream ) ;

  DBGOUT;
}

/*----------------------------------------*/
/* PTS 1110181 removed sqlxregisterinst and added symbolic link generation for dcom registry */
/*!
  Function: en904_CreateDcomRegistryLink
  Description: Create entry and link to a registry file for DCOM registry

  The name of the DCOM registry file is found by scanning the Registry_dcom.ini file for dbroot.
  
  If entry does not exists, the lowest possible number that has no mapping is searched.
  With this a new entry is created.
  
  If an old symbolic link exists, its entry is read and compared with the new target.
  If an old file was referenced, it is tried to remove it.
  If the old symbolic link was wrong, it is also removed.

  If neccessary the new symbolic link is created.
  
  Arguments: dbroot [in] path to dbroot
             xerror [out] set if path too long
 */
static void en904_CreateDcomRegistryLink(tsp09_cstr dbroot, tsp9_rte_xerror *xerror)
{
    unsigned char rc;
    tsp00_ErrText errtext;
    tsp00_Pathc   registryLinkPath;
    tsp00_Pathc   registryFilePath;
    int           linkEstablished = 0;
    then09_ConfigHandle lockHandle;

    if ( ( strlen(dbroot) 
         + strlen("/wrk/") 
         + strlen(registryDcomPrefixC) 
         + strlen(registryDcomExtensionC) ) > sizeof(tsp00_Path) )
    {
        eo44anyError (xerror, "dbroot path too long");
        return;
    }

    memset(registryLinkPath, 0, sizeof(tsp00_Pathc));
    strcpy(registryLinkPath, dbroot);
    strcat(registryLinkPath, "/wrk/");
    strcat(registryLinkPath, registryDcomPrefixC);
    strcat(registryLinkPath, registryDcomExtensionC);

    memset(registryFilePath, 0, sizeof(tsp00_Pathc));

    do /* keep common registry locked while adding new Dcom Registry Entry */
    {
        lockHandle = sqlOpenConfigEnum (iniFileNameC, installationsSectionC, errtext, &rc);
        if ( lockHandle == (then09_ConfigHandle)0 )
        {
            sleep(1);
        }
    } while( lockHandle == (then09_ConfigHandle)0 );

    if ( !sqlGetConfigString( registryDcomFileNameC, registryIndexSectionC,
    	                      dbroot, registryFilePath, sizeof(tsp00_Pathc), errtext, &rc) )
    {
        /* Scan DCOM index registry to find spare entry */
        en904_AddNewDcomRegistryFile( dbroot, registryFilePath, xerror);
    }

    (void)sqlCloseConfigEnum(lockHandle, errtext, &rc); /* release common registry */

    /* Check maybe existing old symlink */
    {
        tsp00_Pathc   formerRegistryFilePath;

        /* try to recycle existing entry */
        if ( readlink(registryLinkPath, formerRegistryFilePath, sizeof(tsp00_Path)) != -1 )
        {
            linkEstablished = ( strcmp((char *)&registryFilePath[0], (char *)&formerRegistryFilePath[0]) == 0 );
            if ( !linkEstablished )
            {
                /* there was a link, but is pointed to a different file -> Remove both */
                (void)unlink(formerRegistryFilePath);
            }
        }
        else
        {
            if ( errno == EINVAL )
            {
                /* If no symbolic link is found, the original is copied. 
                   This migrates an existing DCOM registry to its new place */
                en904_TryCopy(registryLinkPath, registryFilePath);
            }
        }

        if ( !linkEstablished )
        {
            /* there was no link or another link, could have been a file -> Remove it */
            (void)unlink(registryLinkPath);
        }
    }

    /* establish new symlink (file must not be created here...) */
    if ( !linkEstablished )
    {
        if ( symlink(registryFilePath, registryLinkPath) == -1 )
        {
            eo44anyError (xerror, "symlink to DCOM registry failed");
            en904_RemoveDcomRegistryLink(dbroot, xerror);
        }
    }
}

/*----------------------------------------*/
/* PTS 1110181 removed sqlxregisterinst and added symbolic link generation for dcom registry */
/*!
  Function: en904_RemoveDcomRegistryLink
  Description: Remove entry and link and registry file for DCOM registry

  The name of the DCOM registry file is found by reading the content of the symbolic link found
  in 'dbroot/wrk/Registry.dcom'.
  
  This file, the symbolic link and the entry in Registry_dcom.ini are removed.

  Arguments: dbroot [in] path to dbroot
             xerror [out] set if path too long
 */
static void en904_RemoveDcomRegistryLink(tsp09_cstr dbroot, tsp9_rte_xerror *xerror)
{
/* 
   Has to wait until reregister of installation is possible 
   Otherwise inplace update of service database is impossible for installed release.
 */ 
#if 0
    unsigned char rc;
    tsp00_ErrText errtext;
    tsp00_Pathc   registryLinkPath;
    tsp00_Pathc   registryFilePath;

    if ( ( strlen(dbroot) 
         + strlen("/wrk/") 
         + strlen(registryDcomPrefixC) 
         + strlen(registryDcomExtensionC) ) > sizeof(tsp00_Path) )
    {
        eo44anyError (xerror, "dbroot path too long");
        return;
    }

    memset(registryLinkPath, 0, sizeof(tsp00_Pathc));
    strcpy(registryLinkPath, dbroot);
    strcat(registryLinkPath, "/wrk/");
    strcat(registryLinkPath, registryDcomPrefixC);
    strcat(registryLinkPath, registryDcomExtensionC);

    memset(registryFilePath, 0, sizeof(tsp00_Pathc));
    if ( readlink(registryLinkPath, registryFilePath, sizeof(tsp00_Path)) != -1 )
    {
        (void)unlink(registryFilePath);
        (void)sqlRemoveConfigString(registryDcomFileNameC, registryMapSectionC, registryFilePath, errtext, &rc);
    }
    (void)unlink(registryLinkPath);

    (void)sqlRemoveConfigString(registryDcomFileNameC, registryIndexSectionC, dbroot, errtext, &rc);
#endif
}

/*----------------------------------------*/
/* PTS 1110181 removed sqlxregisterinst and added symbolic link generation for dcom registry */
    #undef MF__
    #define MF__ MOD__"sqlxregisterinstNew"
void sqlxregisterinstNew (
    tsp09_RteInstallationInfoNew * info,
    tsp9_rte_xerror* xerror)
{
    int ok;
    unsigned char rc;
    tsp00_ErrText errtext;
    char versionString [maxVersionStringC+2+sizeof(tsp00_Pathc)]; /* PTS 1104801 */

    if ( (info->RecordVersion != INSTALLATION_RECORD_VERSION_V721)
      || (info->RecordLength != sizeof(tsp09_RteInstallationInfoNew)) )
    {
      eo44anyError (xerror, "Unsupported record version");
      return;
    }

    eo44initError (xerror);
    sprintf (versionString, "%d.%d.%d.%d,%s",
	    info->version.no [0], info->version.no [1], info->version.no [2], info->version.no [3],
        info->key );
    ok = sqlPutConfigString (iniFileNameC, installationsSectionC,
	    info->dbroot, versionString, errtext, &rc);
    if (!ok) 
    {
    	eo44anyError (xerror, errtext);
    }
    else
    {
        en904_CreateDcomRegistryLink(&info->dbroot[0], xerror);
    }
}


/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"sqlxunregisterinst"
void sqlxunregisterinst (
    tsp9_cstr dbroot, 
    tsp9_rte_xerror* xerror)
{
    int ok;
    unsigned char rc;
    tsp00_ErrText errtext;

    eo44initError (xerror);
    ok = sqlRemoveConfigString (iniFileNameC, installationsSectionC,
	    dbroot, errtext, &rc);
    en904_RemoveDcomRegistryLink(dbroot, xerror);
    if (!ok) {
    	eo44anyError (xerror, errtext);
    }
}

#endif
/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"sqlxopen_installation_enum"
void sqlxopen_installation_enum (
    void** handle,
    tsp9_rte_xerror* xerror)
{
    int ok;
    unsigned char rc;
    tsp00_ErrText errtext;

    eo44initError (xerror);
    *handle = (void *)sqlOpenConfigEnum (iniFileNameC, installationsSectionC,
	    errtext, &rc);
    if (rc != config_error_ok_chen09) {
    	eo44anyError (xerror, errtext);
    }
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"sqlxnext_installation"
void sqlxnext_installation (
    void* handle,
    tsp9_rte_installationinfo* instinfo,
    tsp9_rte_xerror* xerror)
{
    int ok;
    unsigned char rc;
    tsp00_ErrText errtext;
    char versionString [maxVersionStringC+2+sizeof(tsp00_Pathc)]; /* PTS 1104801 */

    eo44initError (xerror);
    ok = sqlNextConfig ((then09_ConfigHandle)((long)handle),
                        (char *)instinfo->dbroot,
                        sizeof(instinfo->dbroot),
                        versionString,
                        sizeof(versionString),
                        errtext,
                        &rc);
    if (ok) {
	parseVersion (versionString, &instinfo->version);
    }
    else {
	if (rc == config_error_end_chen09) {
	    eo44eoshowError (xerror);
	}
	else {
	    eo44anyError (xerror, errtext);
	}
    }
}

/*------------------------------*/
void sqlxnext_installationNew (
    void* handle,
    tsp09_RteInstallationInfoNew * info,
    tsp09_rte_xerror* xerror)
{
    int ok;
    unsigned char rc;
    tsp00_ErrText errtext;
    char versionString [maxVersionStringC+2+sizeof(tsp00_Pathc)]; /* PTS 1104801 */

    if ( (info->RecordVersion != INSTALLATION_RECORD_VERSION_V721)
      || (info->RecordLength != sizeof(tsp09_RteInstallationInfoNew)) )
    {
      eo44anyError (xerror, "Unsupported record version");
      return;
    }

    eo44initError (xerror);
    ok = sqlNextConfig ((then09_ConfigHandle)((long)handle),
                        (char *)info->dbroot,
                        sizeof(info->dbroot),
                        versionString,
                        sizeof(versionString),
                        errtext, 
                        &rc);
    if (ok) 
    {
    	parseVersionNew (versionString, &info->version, info->key);
    }
    else 
    {
        if (rc == config_error_end_chen09) 
        {
	        eo44eoshowError (xerror);
    	}
	    else
        {
	        eo44anyError (xerror, errtext);
    	}
    }
}

/*----------------------------------------*/

  #undef MF__
  #define MF__ MOD__"sqlxclose_installation_enum"
void sqlxclose_installation_enum (
    void* handle)
{
    int ok;
    unsigned char rc;
    tsp00_ErrText errtext;

    sqlCloseConfigEnum ((long)handle, errtext, &rc);
}


#if !defined (ENUMINST_ONLY)
/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"versionCompare_MF"
static int versionCompare_MF (
    tsp9_release_version* version1,
    tsp9_release_version* version2)
{
    int i;

    for (i = 0; i < csp9_version_digits; ++i) {
	if (version1->no [i] < version2->no [i]) {
	    return -1;
	}
	else if (version1->no [i] > version2->no [i]) {
	    return 1;
	}
    }
    return 0;
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"en904dbrootOfRelease"
tsp00_Bool en904dbrootOfRelease (
    tsp9_release_version* version,
    char * dbroot)
{
    tsp00_Bool	found = FALSE;
    int		ok;
    unsigned char rc;
    tsp00_ErrText errtext;
    void       *handle;
    tsp9_rte_installationinfo instinfo;
    tsp9_rte_xerror	xerror;

    dbroot [0] = '\0';
    eo44initError (&xerror);
    sqlxopen_installation_enum (&handle, &xerror);
    if (!XOK (&xerror)) {
      while (!found && XOK (&xerror)) {
	sqlxnext_installation (handle, &instinfo, &xerror);
	if (XOK (&xerror) && (versionCompare_MF (version, &instinfo.version) == 0)) {
	    strcpy (dbroot, instinfo.dbroot);
	    found = TRUE;
	}
      }
      sqlCloseConfigEnum ((long)handle, errtext, &rc);
    }
    return found;
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"en904_VersionOfDBRoot"
static tsp00_Bool en904_VersionOfDBRoot (
    char * dbroot,
    tsp9_release_version* version)
{
    int		ok;
    unsigned char rc;
    tsp00_ErrText errtext;
    char versionString [maxVersionStringC+2+sizeof(tsp00_Pathc)]; /* PTS 1104801 */

    ok = sqlGetConfigString (iniFileNameC, installationsSectionC,
	    dbroot, versionString, sizeof (versionString),
	    errtext, &rc);
    if (ok) {
    	parseVersion (versionString, version);
    }
    return ok;
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"newestDBRoot_MF"
static tsp00_Bool newestDBRoot_MF (
    char * dbroot,
    const char * serverpgm)
{
  /*
   * search through all installations and pick the newest
   */
    tsp9_rte_installationinfo instnfo;
    tsp9_rte_xerror	xerror;
    tsp00_Bool		found = FALSE;
    tsp9_rte_installationinfo  instinfo;
    tsp9_release_version newestVersion;
    void       *handle;
    tsp00_Pathc  testPath;
    struct stat  statInfo;
    int          statRC;

    eo44initError (&xerror);
    newestVersion.no [0] = csp9_invalid_version;
    sqlxopen_installation_enum (&handle, &xerror);
    if (!XOK (&xerror)) {
	return FALSE;
    }
    while (XOK (&xerror)) {
	sqlxnext_installation (handle, &instinfo, &xerror);
	if (XOK (&xerror)) {

	    if (versionCompare_MF (&newestVersion, &instinfo.version) < 0) {

                 strcpy ((char *)testPath, instinfo.dbroot);
                 strcat ((char *)testPath, "/pgm/");
                 strcat ((char *)testPath, serverpgm);
                 statRC = stat ((char *)testPath, &statInfo);
                 if ((statRC == 0)                && 
                     (statInfo.st_mode & S_IFREG) &&   /* PTS 1105115 */
                     (statInfo.st_mode & S_IEXEC) )  
                 {
                     strcpy (dbroot, instinfo.dbroot);
                     newestVersion = instinfo.version;
                     found = TRUE;
        	     }
	    }
	}
    }
    sqlxclose_installation_enum (handle);
    return found;
}

/**\
--------------------------------------- MF__ MOD__ (DB Registry)
\**/

typedef struct DBEnumT {
    then09_ConfigHandle  registry;
    tsp00_Bool	         fullInfo;
    tsp9_pgm_kind	     pgmKind;  /* Last reported Program Kind */
    tsp9_release_version version; /* Last reported installation version */
} DBEnumT;

    #undef MF__
    #define MF__ MOD__"sqlxregisterdb"
void sqlxregisterdb (
    tsp9_cstr		dbname, 
    tsp9_cstr		dbroot,
    tsp9_pgm_kind	pgmKind, /* not used under UNIX, all speeds are registered by a single call */
    tsp9_server_options options, /* not used under UNIX */
    tsp9_rte_xerror*	xerror)
{
    int ok;
    unsigned char rc;
    tsp00_ErrText errtext;

    eo44initError (xerror);
    ok = sqlPutConfigString (iniFileNameC, databasesSectionC,
	    dbname, (char*) dbroot, errtext, &rc);
    if (!ok) {
    	eo44anyError (xerror, errtext);
    }
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"sqlxunregisterdb"
void sqlxunregisterdb (
    tsp9_cstr		dbname, 
    tsp9_pgm_kind	pgmKind,	/* not used under UNIX */
    tsp9_rte_xerror*	xerror)
{
    tsp9_rte_dbinfo	dbInfo;
    tsp00_Bool	ok;
    unsigned char rc;
    tsp00_ErrText errtext;

    eo44initError (xerror);
    ok = sqlRemoveConfigString (iniFileNameC, databasesSectionC,
	    dbname, errtext, &rc);
    if (!ok) {
    	eo44anyError (xerror, errtext);
    }
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"dbInRegistry_MF"
static tsp00_Bool dbInRegistry_MF (
    tsp9_cstr dbname)
{
    int ok;
    unsigned char rc;
    tsp00_ErrText errtext;
    tsp00_Pathc   dbroot;

    ok = sqlGetConfigString (iniFileNameC, databasesSectionC,
	   dbname, (char *)dbroot, sizeof (dbroot), errtext, &rc);
    return ok;
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"dbHasSpecificProcess_MF"
static tsp00_Bool dbHasSpecificProcess_MF (
    tsp9_cstr dbname,
    const char *kernelpgm)
{
    char   source [30];

    sprintf (source, "cat /tmp/db.%d", getpid() );

    return en41FindDatabaseProcess(source, dbname, kernelpgm);
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"dbHasLink_MF"
static tsp00_Bool dbHasLink_MF (
    tsp9_rte_dbinfo *dbinfo)
{
    struct stat		statInfo;
    tsp00_Pathc  	fullpath;
    int			    rc;

    sprintf ((char *)fullpath, "%s/pgm/db:%s", dbinfo->dbroot, dbinfo->dbname);
    rc = stat ((char *)fullpath, &statInfo);
    return (rc == 0);
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"sqlDBSpeed"
tsp9_pgm_kind sqlDBSpeed( const tsp00_DbNamec szDbName )
{
    char speedBuffer[24];
    tsp00_TaskId pid;
    tsp9_cstr dbname = (tsp9_cstr)&szDbName[0];

    if ( 0 != en41GetPPIDFromFile(dbname, &pid)
      || pid == 0
      || 0 != en41GetPIDFromFile(dbname, &pid)
      || pid == 0
      || 0 != en41GetDbSpeedFromFile(dbname, speedBuffer, sizeof(speedBuffer)) )
    {
        return csp9_any_pgm;
    }

    if ( strcmp("FAST", speedBuffer) == 0 )
    {
        return csp9_fast_pgm;
    }
    if ( strcmp("QUICK", speedBuffer) == 0 )
    {
        return csp9_quick_pgm;
    }
    if ( strcmp("TEST", speedBuffer) == 0 )
    {
        return csp9_test_pgm;
    }
    if ( strcmp("SLOW", speedBuffer) == 0 )
    {
        return csp9_slow_pgm;
    }

    /* Default... */
    return csp9_fast_pgm;
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"dbHasRunningKernel"
static bool dbHasRunningKernel (
    tsp9_rte_dbinfo *dbinfo,
    const char *speed)
{
    char speedBuffer[24];
    tsp00_TaskId pid;

    if ( 0 != en41GetPPIDFromFile(dbinfo->dbname, &pid)
      || pid == 0
      || 0 != en41GetPIDFromFile(dbinfo->dbname, &pid)
      || pid == 0
      || 0 != en41GetDbSpeedFromFile(dbinfo->dbname, speedBuffer, sizeof(speedBuffer)) )
    {
        return false;
    }

    dbinfo->state = ( 0 == strcmp(speedBuffer, speed) ? csp9_state_running : csp9_state_off );
    return true;
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"dbHasExecutable_MF"
static tsp00_Bool dbHasExecutable_MF (
    const char *dbroot,
    const char *kernelName )
{
    struct stat		statInfo;
    tsp00_Pathc 	fullpath;
    int		    	rc;

    sprintf ((char *)fullpath, "%s/pgm/%s", dbroot, kernelName);
    rc = stat ((char *)fullpath, &statInfo);
    return (rc == 0);
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"sqlxopen_db_enum"
void sqlxopen_db_enum (
    void** handle,
    tsp00_Bool fullinfo,
    tsp9_rte_xerror* xerror)
{
    DBEnumT	       *dbEnum;
    char		cmdBuf [255];
    int ok;
    unsigned char rc;
    tsp00_ErrText errtext;

    *handle = NULL ; /* PTS 1106394 */
    eo44initError (xerror);
    if ( ALLOC_MEM_EO57( (void **)&dbEnum, sizeof (DBEnumT)) != NO_ERROR_EO001 )
    {
    	eo44anyError (xerror, "Out of memory");
	    return;
    }
    memset(dbEnum, 0, sizeof(DBEnumT));
    dbEnum->registry = sqlOpenConfigEnum (iniFileNameC, databasesSectionC,
	    errtext, &rc);
    if (rc != config_error_ok_chen09) 
    {
    	eo44anyError (xerror, errtext);
        FREE_MEM_EO57 (dbEnum);
        return;
    }

    dbEnum->pgmKind = csp9_any_pgm;
    dbEnum->fullInfo = fullinfo;
    if (fullinfo) 
    {
        /* build process list ( needed only to support releases < 7.4 ) */
	    sprintf (cmdBuf, "ps_all > /tmp/db.%d 2>/dev/null", getpid ());
	    system (cmdBuf);
    }
    *handle = dbEnum;
}

/*----------------------------------------*/

/* PTS 1104875 */
/*
 * Iterate through all types of kernels supported. Restart if last reported was 'any_pgm'
 */
    #undef MF__
    #define MF__ MOD__"nextRegisteredDB_MF"
static void
nextRegisteredDB_MF (
    DBEnumT* dbEnum,
    tsp9_rte_dbinfo* dbinfo,
    tsp9_rte_xerror* xerror)
{
    char cmdBuf [255];
    int pid;
    tsp00_Bool ok;
    unsigned char rc;
    tsp00_ErrText errtext;
    tsp9_pgm_kind	pgmKind;

#define BOOLSTR(x)    (x ? "True" : "False")
    TRACE ((traceFileCS, "!atEnd: %s; error: %d\n",
	    BOOLSTR (!(dbEnum->registry == NULL)),
	    xerror->xe_result));

    ok = ( dbEnum->pgmKind != csp9_quick_pgm && dbEnum->pgmKind != csp9_any_pgm );
    if ( !ok )
    {
        ok = sqlNextConfig (dbEnum->registry,
                            dbinfo->dbname,
                            sizeof(dbinfo->dbname),
  	                        dbinfo->dbroot, 
                            sizeof(dbinfo->dbroot),
                            errtext, 
                            &rc);
        if ( !en904_VersionOfDBRoot (dbinfo->dbroot, &dbEnum->version) )
        {
            int i;
            for (i = 0; i < csp9_version_digits; ++i) {
            	dbEnum->version.no [i] = 0;
            }
        }
    }

    if (ok) 
    {
        switch ( dbEnum->pgmKind )
        {
            default:
            case csp9_quick_pgm: pgmKind = csp9_fast_pgm;  break;
            case csp9_fast_pgm:  pgmKind = csp9_slow_pgm;  break;
            case csp9_slow_pgm:  pgmKind = csp9_test_pgm;  break;
            case csp9_test_pgm:  pgmKind = csp9_quick_pgm; break;
        }

        dbEnum->pgmKind = pgmKind;

        dbinfo->state = csp9_state_unknown;
	    dbinfo->options = csp9_no_options;
        dbinfo->pgmKind = pgmKind;

        if( !dbHasExecutable_MF(dbinfo->dbroot, kernelNames[pgmKind]) )
        {
            int i;
            for (i = 0; i < csp9_version_digits; ++i) {
            	dbinfo->version.no [i] = 0;
            }
        }
        else
        {
            int i;
            for (i = 0; i < csp9_version_digits; ++i) {
            	dbinfo->version.no [i] = dbEnum->version.no [i];
            }
        }

        if ( dbinfo->version.no [0] != 0 
          && dbEnum->fullInfo )
        {
            bool foundState = false;

            if ( dbinfo->version.no[0] > 7
              || ( dbinfo->version.no[0] == 7
                && dbinfo->version.no[1] >= 3 ) )
            {
                const char *speed;
                     if ( pgmKind == csp9_fast_pgm )  speed = "FAST";
                else if ( pgmKind == csp9_quick_pgm ) speed = "QUICK";
                else if ( pgmKind == csp9_slow_pgm )  speed = "SLOW";
                else if ( pgmKind == csp9_test_pgm )  speed = "TEST";
                else speed = "UNKNOWN"; /* Strange type.... */

                foundState = dbHasRunningKernel (dbinfo, speed);
            }

            if ( !foundState )
            {
                if ( dbHasSpecificProcess_MF (dbinfo->dbname, kernelNames[pgmKind]) )
                {
		            dbinfo->state = csp9_state_running;
	            }
	            else 
                {
                    /* If another speed of this database is running, it must be offline... */
                    if ( dbHasSpecificProcess_MF (dbinfo->dbname, NULL) )
                    {
		                dbinfo->state = csp9_state_off;
                    }
                    else
                    {
                        /* If the link still exist a crash is very likely */
                        if (dbHasLink_MF (dbinfo)) 
                        {
		                    dbinfo->state = csp9_state_ruin;
		                }
    		            else
                        {
		                    dbinfo->state = csp9_state_off;
		                }
                    }
	            }
            }
	    }
    }
    else 
    {
	    sqlCloseConfigEnum (dbEnum->registry, errtext, &rc);
	    dbEnum->registry = NULL;
	    eo44eoshowError (xerror);
    }
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"sqlxnext_db"
void sqlxnext_db (
    void* handle,
    tsp9_rte_dbinfo* dbinfo,
    tsp9_rte_xerror* xerror)
{
    DBEnumT    *dbEnum = (DBEnumT*) handle;
    tsp00_Bool infoRead = FALSE;

    eo44initError (xerror);
    if (dbEnum->registry != NULL) {
    	nextRegisteredDB_MF (dbEnum, dbinfo, xerror);
    }
    else 
    {
	    eo44eoshowError (xerror);
    }
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"sqlxclose_db_enum"
void sqlxclose_db_enum (
    void* handle)
{
    if ( handle != NULL ) /* PTS 1106394 */
    {
      tsp9_rte_xerror	 xerror;
      DBEnumT		*dbEnum = (DBEnumT*) handle;
      tsp00_Pathc	tmpName;
      int			pid;

      xerror.xe_result = 0;
      if (dbEnum->fullInfo) 
      {
	    sprintf ((char *)tmpName, "/tmp/db.%d", getpid () );
	    unlink ((char *)tmpName);
      }
      FREE_MEM_EO57 (dbEnum);
    }
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"sqlGetDBRootOfDB"
tsp00_Bool sqlGetDBRootOfDB (
    tsp9_cstr dbname, 
    char* dbroot,
    int   MaxDBRootLen)
{
    int ok;
    unsigned char rc;
    tsp00_ErrText errtext;

    DBG1 (( MF__,"sqlGetConfigString for '%s' in section '%s'\n",dbname, databasesSectionC ));
    ok = sqlGetConfigString (iniFileNameC, databasesSectionC,
	    dbname, dbroot, MaxDBRootLen, errtext, &rc);
    if ( !ok )
    {
    DBG1 (( MF__,"sqlGetConfigString failed: %s\n", errtext ));
    }
    else
    {
    DBG1 (( MF__,"sqlGetConfigString for %s got %s\n",dbname, dbroot ));
    }
    return ok;
}

/*----------------------------------------*/

    #undef MF__
    #define MF__ MOD__"RTE_CheckIfInstallationRegistered"
tsp00_Bool RTE_CheckIfInstallationRegistered (
    char const * dbroot )
{
    int ok;
    unsigned char rc;
    tsp00_ErrText errtext;
    tsp00_Pathc dummy;

    DBG1 (( MF__,"sqlGetConfigString for '%s' in section '%s'\n",dbroot, installationsSectionC ));
    ok = sqlGetConfigString (iniFileNameC, installationsSectionC,
	    dbroot, &dummy[0], sizeof(dummy), errtext, &rc);
    return ( 0 != ok );
}

/*----------------------------------------*/

tsp00_Bool
sql904_findControlServer (
    char *executable,		    /* OUT		*/
    const int  MaxExecutableLen,/* IN       */
    const char *dbname,		    /* IN		*/
    const char *serverpgm,      /* IN       */ 
    char *dbroot,		        /* IN/OUT	*/
    const int  MaxDBRootLen)    /* IN       */
{
#undef	MF__
#define	MF__	MOD__"sql904_findControlServer"
    tsp00_Bool found = FALSE;
    struct stat statInfo;
    int rc;

    if ((dbname != NULL) && (dbname [0] != '\0'))
    {
	  /* first precedence: explicit dbname */
      DBG1 (( MF__,"calling sqlGetDBRootOfDB for %s\n",dbname ));
 	  found = sqlGetDBRootOfDB (dbname, dbroot, MaxDBRootLen);
      if ( found )
      {
          found = RTE_CheckIfInstallationRegistered(dbroot);
      }
    }
    else 
    if ((dbroot != NULL) && (dbroot [0] != '\0')) 
    {
    	/* second precedence: explicit dbroot */
    	found = true; /* PTS 1125785 RTE_CheckIfInstallationRegistered(dbroot); */
    }
    else 
    {
    	/* last chance: get newest installation */
    	found = newestDBRoot_MF (dbroot, serverpgm);
    }

    if (found) 
    {
	    strcpy (executable, dbroot);
	    strcat (executable, "/pgm/");
	    strcat (executable, serverpgm);
        DBG1 (( MF__,"sqlGetDBRootOfDB executable: %s\n",executable ));
        rc = stat (executable, &statInfo);
	    if ((rc != 0)                     || 
            !(statInfo.st_mode & S_IFREG) ||  /* PTS 1105115 */
            !(statInfo.st_mode & S_IEXEC) )
        {
            DBG1 (( MF__,"bad fileinformation (stat) of executable: %s\n",executable ));
	        found = FALSE;
	    }
    }

    DBG1 (( MF__,"executable: %s %sfound",executable, found ? "" : "NOT "  ));
    return found;
}
#endif
