/*!**************************************************************************

  module      : ven906.c

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Client RunTime
  description : CServer communication


  last changed: 2001-09-21  14:36
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/
#define MOD__ "ven906.c:"

#include "gsp00.h"
#include "gen00.h"
#include "gen003.h"
#include "gsp09.h"
#include <unistd.h>
#include "geo001.h"
#include "geo57.h"
#ifndef AIX
/* on AIX this produces a confilct with stdlib.h definition of setkey.... */
#include <crypt.h>
#endif /* AIX */
#include <pwd.h>

#if defined(HPUX) || defined(SUN) || defined(LINUX) || defined(NMP)
#include <shadow.h>
#endif /* SUN | LINUX | NMP */

#if defined(AIX)
#include <userpw.h>
#endif /* AIX */

#if defined(OSF1)
#include <sys/types.h>
#include <sys/security.h>
#include <prot.h>
#endif /* OSF1 */

static tsp00_Bool verbose_MV = FALSE;

static FILE* traceFile_MV = NULL;
#if defined (DEBUG_RTE)
#define TRC(x)	fprintf x
#else
#define TRC(x)
#endif


/*----------------------------------------*/
static const char *en906_FindCryptedPassword(const char *Username, const struct passwd *pwEntry);
/*----------------------------------------*/

/* PTS 1106827 */
    #undef MF__
    #define MF__ MOD__"sqlxchown"
void
sqlxchown (
    const char      *Username,
    tsp00_Pathc     Filename,
    tsp9_rte_xerror *xerror )
{
  struct passwd *pwEntry;

  pwEntry = getpwnam (Username);

  if ( ! pwEntry )
  {
    eo44anyError (xerror, "cannot find user in passwd\n");
  }
  else
  {
    if ( geteuid() != pwEntry->pw_uid
      && chown(Filename, pwEntry->pw_uid, pwEntry->pw_gid) != 0 )
    {
      eo44sysError(xerror, errno );
    }
    else
    {
      eo44initError (xerror);
    }
  }
}

#ifdef DUMP_XLOGON_INFO
  FILE *xlogonError = 0;
#endif /*DUMP_XLOGON_INFO */


/*----------------------------------------*/
    #undef MF__
    #define MF__ MOD__"sqlxlogon"
void
sqlxlogon (
    const char      *username,
    const char      *passwd,
    tsp9_rte_xerror *xerror )
{
    struct passwd *pwEntry;

#ifdef DUMP_XLOGON_INFO

#define DUMP_INFO_0(_fmt)                   fprintf(xlogonError, _fmt);
#define DUMP_INFO_1(_fmt,_arg0)             fprintf(xlogonError, _fmt, _arg0);
#define DUMP_INFO_2(_fmt,_arg0,_arg1)       fprintf(xlogonError, _fmt, _arg0, _arg1);
#define DUMP_INFO_3(_fmt,_arg0,_arg1,_arg2) fprintf(xlogonError, _fmt, _arg0, _arg1, _arg2);
#define DUMP_ERR_0(_fmt)                   DUMP_INFO_0(_fmt); fclose(xlogonError);
#define DUMP_ERR_1(_fmt,_arg0)             DUMP_INFO_1(_fmt,_arg0); fclose(xlogonError);
#define DUMP_ERR_2(_fmt,_arg0,_arg1)       DUMP_INFO_2(_fmt,_arg0,_arg1); fclose(xlogonError);
#define DUMP_ERR_3(_fmt,_arg0,_arg1,_arg2) DUMP_INFO_3(_fmt,_arg0,_arg1,_arg2); fclose(xlogonError);

    system("touch /tmp/xlogin.err");
    system("date >> /tmp/xlogin.err");
    xlogonError = fopen("/tmp/xlogin.err", "a");
    DUMP_INFO_2("sqlxlogon(%s,%s) called\n", 
		!username ? "" : username, !passwd ? "" : passwd);

#else

#define DUMP_INFO_0(_fmt)
#define DUMP_INFO_1(_fmt, _arg0)
#define DUMP_INFO_2(_fmt, _arg0, _arg1)
#define DUMP_INFO_3(_fmt, _arg0, _arg1, _arg2)
#define DUMP_ERR_0(_fmt)                        
#define DUMP_ERR_1(_fmt, _arg0)                 
#define DUMP_ERR_2(_fmt, _arg0, _arg1)          
#define DUMP_ERR_3(_fmt, _arg0, _arg1, _arg2)   

#endif /* DUMP_XLOGON_INFO */

    eo44anyError (xerror, "logon failed");
    if ( !username || !*username )
    {
        DUMP_ERR_0("No username given\n")
        return;
    }

    pwEntry = getpwnam (username);
    if (pwEntry == NULL) 
    {
    	DUMP_ERR_0("getpwnam failed\n")
        return;
    }

    /* PTS 1120387 If not started as 'setuid root', it is not needed to check the password, if the effective 
       user already matches... */
    if ( geteuid() == pwEntry->pw_uid
      && 0 != pwEntry->pw_uid )
    {
        DUMP_ERR_0("geteuid() already matches. Nothing to do!\n" );
        eo44initError (xerror);
        return;
    }

    /* On UNIX checking the password is no longer possible,
     * if you are not superuser....
     * Dependency on the operating system on how password can be
     * found is multiple... Some use Shadow files, some use Security
     * subdirectories. But common is that checking the password
     * is a job only a setuid root program can do.
     * Therefore IF THIS will be added, the searching should be
     * put into an external process...
     */
    if ( passwd )
    {
        const char *CryptedPassword = en906_FindCryptedPassword(username, pwEntry);
        const char *GeneratedPassword = "";

        if ( *passwd )
        {
            if ( CryptedPassword[0]
              && CryptedPassword[1] ) /* check for minimum length = 2 */
            {
                GeneratedPassword = crypt (passwd, CryptedPassword);
                if ( !GeneratedPassword )
                {
                    GeneratedPassword = "??";
                }
            }
        }
        else
        {
            /* Empty password is not crypted... */
            GeneratedPassword = passwd;
        }

        if ( strncmp(CryptedPassword, GeneratedPassword, strlen(GeneratedPassword)) != 0 )
        {
        	DUMP_ERR_2("Password mismatch: Found '%s' Generated '%s'\n",
                        CryptedPassword, GeneratedPassword )
            return;
        }
    }
    else
    {
        DUMP_INFO_0("No Password given -> no check\n");
    }

    /* PTS 1104335 Order important! If setuid is done, setgid must fail... */
    if ( getegid() != pwEntry->pw_gid )
    {
        if (setgid((uid_t)pwEntry->pw_gid) < 0) 
        {
        	DUMP_ERR_2( "setgid to %d failed getgid returns %d\n", (int)pwEntry->pw_gid, (int)getgid() )
            return;
        }
        else
        {
        	DUMP_INFO_1( "setgid to %d succeeded\n", (int)pwEntry->pw_gid )
        }
    }
    else
    {
        	DUMP_INFO_1( "setgid to %d not needed\n", (int)pwEntry->pw_gid )
    }

    if ( geteuid() != pwEntry->pw_uid )
    {
        /* PTS 1109037 supplementary groups were not initialized */
        if ( initgroups(username, pwEntry->pw_gid) < 0 )
        {
        	DUMP_ERR_2( "initgroups for %s failed. initgroups returns %d\n", username, errno )
            return;
        }
        else
        {
        	DUMP_INFO_1( "initgroups for %s succeeded\n", username )
        }

        if (setuid((uid_t)pwEntry->pw_uid) < 0) 
        {
        	DUMP_ERR_2( "setuid to %d failed getuid returns %d\n", (int)pwEntry->pw_uid, (int)getuid() )
            return;
        }
        else
        {
        	DUMP_INFO_1( "setuid to %d succeeded\n", (int)pwEntry->pw_uid )
        }
    }
    else
    {
        	DUMP_INFO_1( "setuid to %d not needed\n", (int)pwEntry->pw_uid )
    }

    /* set error to OK */
    eo44initError (xerror);

    DUMP_ERR_0( "sqlxlogon succeeded\n")
}

/*----------------------------------------*/
    #undef MF__
    #define MF__ MOD__"en906_FindCryptedPassword"
static const char *en906_FindCryptedPassword(const char *Username, const struct passwd *pwEntry)
{
    const char *FoundCryptedPassword = pwEntry->pw_passwd;

    if ( FoundCryptedPassword
     && (*FoundCryptedPassword) == '\0' )
    {
        return FoundCryptedPassword;
    }

    if ( !FoundCryptedPassword
      || !FoundCryptedPassword[0]
      || !FoundCryptedPassword[1] ) /* check minimum length = 2, core dump free version of strlen() < 2 ;-) */
    {
        char srchUsername[64];
        if ( strlen(Username) > 63 )
        {
            return "?";
        }

        strcpy(srchUsername, Username);
        {
#if defined(AIX)
            struct userpw *spwdEntry;

            DUMP_INFO_1( "First found crypted password invalid:%s Try getuserpw\n", FoundCryptedPassword )
            spwdEntry = getuserpw (srchUsername);
            if (spwdEntry != NULL) 
            {
                FoundCryptedPassword = spwdEntry->upw_passwd;
            }
#elif defined(OSF1)
            /* PTS 1111197 */
            struct pr_passwd *spwdEntry;
            static int setAuthCalled = 0;
            if ( !setAuthCalled )
            {
                char *myArgv[] = { "x", 0 };
                setAuthCalled = 1;
            /* Manual page forces this call to be first in main(), but this is not possible... */
            /* So we live with this inconvenience here, that after this call starting_...      */
            /* calls return wrong result... */
                set_auth_parameters(1, myArgv );
            }
            DUMP_INFO_1( "First found crypted password invalid:%s Try getprpwnam\n", FoundCryptedPassword )
            spwdEntry = getprpwnam(srchUsername);
            if ( spwdEntry != NULL )
            {
                FoundCryptedPassword = spwdEntry->ufld.fd_encrypt;
            }
#else
            struct spwd *spwdEntry;
            DUMP_INFO_1( "First found crypted password invalid:%s Try getspnam\n", FoundCryptedPassword )
            spwdEntry = getspnam (srchUsername);
            if (spwdEntry != NULL) 
            {
                FoundCryptedPassword = spwdEntry->sp_pwdp;
            }
#endif
        }

        if ( !FoundCryptedPassword
          || !FoundCryptedPassword[0]
          || !FoundCryptedPassword[1] ) /* check minimum length = 2, core dump free version of strlen() < 2 ;-) */
        {
            DUMP_INFO_1( "Second crypted password invalid. errno:%d\n", errno )
            FoundCryptedPassword = "?";
        }
    }

    return FoundCryptedPassword;
}
