/*!***************************************************************************

  module      : ven91.c

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Client RunTime
  description : xstop - Stop a Kernel

  last changed: 2001-09-19  13:57
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/
#define MOD__ "ven91.c:"

#include "gen00.h"
#include "heo00.h"
#include "gen50.h"
#include "hsp100.h"

/* imported functions */

#include "gen41.h"
#include "hen40.h"
extern  int                 sql34_map_kgs ();
extern	void				sqlinit ();

/* imported data */

/* exported data */

extern struct KERNEL_GLOBAL_SECTION * kgs ;

/*==========================================================================*/

#undef MF__
#define MF__ MOD__"STOP"
int     c_main ( 
int     argc ,
char ** argv )
{
char                          * dbname ;
int                             dumpflag , shmid , rc ;
key_t                           key ;
struct TASK_TYPE              * tcb ;
struct DOUBLY_LINKED          * lquu ;
struct IOR_QUEUE_HEAD         * head ;
ten50_UKT_Control             * lkp ;


    DBGIN;

    /* PTS 1105151
     *  File permission mode is always specified absolutely
     */
	(void) umask ( 0 );

	if ( argc < 2 || argc > 3 )
    {
		(void) fprintf ( stderr, "USAGE: stop dbname [ dump ] \n", 0 );
		return ( 1 );
    }

	sqlinit ( "stop" , (char*)0 );
	dbname = argv [ 1 ] ;
	dumpflag = 0 ;
	if ( argc > 2 && argv [ 2 ] [ 0 ] != '\0' ) dumpflag = 1 ;
    DBG1 (( MF__,"with%s dump\n", dumpflag ? "" : "out" ));

	/*
	 *  Compute the key for ipc resources from the database name
	 */
	key = sql41_get_server_key ( dbname );
	if ( key == (key_t) -1 )
	{
		DBG1 (( -11987, "Database '%s' is not running \n", 
			dbname ));
		return ( 2 );
	}
	if ( key == (key_t) -2 )
	{
		DBG1 (( MF__,"getserverid error, dbname '%s' \n", dbname ));
		return ( 2 );
	}
	DBG1 (( MF__,"key 0x%08x \n", key ));

	/*
	 *  Get the shmid of the Kernel Shared Memory Segment
	 *  and test read/write accessibility
	 */
	shmid = shmget ( key , 0 , 0600 );
	if ( shmid < 0 )
    {
		if ( errno == ENOENT )
		{
			DBG1 (( -11107, "STOP: Kernel Shared Memory Segment not found \n" ));
			DBG1 (( -11109, "STOP: Database not running '%s' \n", dbname ));
			return ( 1 );
		}
		if ( errno == EPERM || errno == EACCES )
		{
			DBG1 (( MF__, "Cannot attach Kernel Shared Memory Segment \n" ));
			DBG1 (( MF__, "Permission denied \n" ));
			return ( 2 );
		}
		DBG1 (( MF__, "Kernel Shared Memory Segment shmget error, %s\n", sqlerrs() ));
		return ( 2 );
    }

	DBG1 (( MF__,"STOP: shmid %d \n", shmid ));

	rc = sql34_map_kgs ( shmid , 0 );
	if ( rc )
    {
	    DBG1 (( MF__, "Mapping of shm failed!\n" ));
		return ( 2 );
    }

    if ( IsEqual_esp100 != sp100_CompareVersionIDs ( &kgs->RTEVersionID ) )
    {
		tsp00_Versionc currentVersion;
		
		sp100_GetVersionString( COMP_NAME_RTE_SP100, 
								s100buildnumber,
								&currentVersion[0]);

	    DBG1 (( MF__, "Incompatible version of running kernel and stop!\n" ))
	    DBG1 (( MF__, "Running kernel-version is: %s\n", kgs->RTEVersion ))
	    DBG1 (( MF__, "Actual    stop-version is: %s\n", currentVersion ))
	    return ( 2 );
	}

    kgs->wantDump = dumpflag;

    rc = kill ( kgs->pid , SIGTERM );

    DBG1 (( MF__,"Waiting for kernel to stop \n" ));

    rc = kill ( kgs->pid , 0 );
    if ( (rc == -1) && (errno == ESRCH) )
    {
         DBG1 (( MF__, "Server '%s' did not run \n", dbname ));
         (void) sleep ( 1 );
    }
	else
	{
		while ( kill ( kgs->pid , 0 ) != -1 ) 
			(void) sleep ( 1 );
	}

	rc = sql41_remove_ipc_resources ( dbname );

	DBG1 (( MF__,"returning %d \n", rc ));

	return ( rc );
}
