/*!**************************************************************************

  module       : geo002.h

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: XParam
  description : Parameter names and structure used inside LZU for parameter access

  last changed: 2002-06-06  11:37
  see also    : example.html ...
  first created:2000-11-22  15:19

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2000



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef GEO002_H
#define GEO002_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

# ifndef XPARAM_NAMES_ONLY
#include "gsp00.h"
#include "vsp002.h"
#include "geo00_0.h"
#include "RunTime/RTE_Types.h"

# endif /* XPARAM_NAMES_ONLY */

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

/* names used inside param file */
#define         PAN_INSTANCE_TYPE             "INSTANCE_TYPE"

#define         PAN_PARAM_KERNELVERSION       "KERNELVERSION"
#define         PAN_SERVERDB                  "SERVERDB"
#define         PAN_SYSDEV                    "SYSDEVSPACE"
#define         PAN_MIRR_SYSDEV               "MIRR_SYSDEVSPACE"
#define         PAN_TARNSACTION_LOG           "TRANSACTION_LOG"
#define         PAN_TARNSACTION_LOG_SIZE      "TRANS_LOG_SIZE"
#define         PAN_ARCHIVE_LOG               "ARCHIVE_LOG"
#define         PAN_MIRR_ARCHIVE_LOG          "MIRR_ARCHIVE_LOG"
#define         PAN_CONTROLID                 "CONTROLUSERID"
#define         PAN_CONTROLPW                 "CONTROLPASSWORD"
#define         PAN_MAXBACKUPDEVS             "MAXBACKUPDEVS"
#define         PAN_ALLOW_LBACK_TO_PIPE       "LOG_BACKUP_TO_PIPE"
#define		      MAX_BACKUP_DEVS		            16

#define         PAN_MAXTRANS                  "_MAXTRANS"
#define         PAN_DATA_CACHE_PAGS           "DATA_CACHE"
#define         PAN_SEQUENCE_CACHE            "SEQUENCE_CACHE"
#define         PAN_CATALOG_CACHE             "CAT_CACHE_SUPPLY"
#define         PAN_SERVER_DESC_CACHE         "_SERVER_DESC_CACHE"
#define         PAN_SERVER_CMD_CACHE          "_SERVER_CMD_CACHE"
#define         PAN_USE_THRD_FOR_TSK          "_USE_THRD_FOR_TSK"  /* PTS 1108525 */
#define         PAN_USE_PROC_SCOPE            "_USE_PROC_SCOPE"    /* PTS 1109046 */
#define         PAN_MAXACTIVE                 "MAXUSERTASKS"
#define         PAN_MAXSERVERDB               "_MAXSERVERDB"
#define         PAN_MIN_SERVER_DESC           "_MIN_SERVER_DESC"
#define         PAN_MAXSERVER                 "MAXSERVERTASKS"
#define         PAN_LOG_IO_QUEUE              "LOG_IO_QUEUE"
#define         PAN_MAXLOCKS                  "MAXLOCKS"
#define         PAN_DEADLOCK_DETECTION        "DEADLOCK_DETECTION"
#define         PAN_LOCK_SUPPLY_BLOCK         "_LOCK_SUPPLY_BLOCK"
#define         PAN_FBM_CACHE_PAGES           "_FBM_CACHE"
#define         PAN_CONV_CACHE_PAGES          "CONVERTER_CACHE"
#define         PAN_USM_CACHE_PAGES           "_USM_CACHE"
#define         PAN_BACKUP_BLOCK_CNT          "BACKUP_BLOCK_CNT"
#define         PAN_DEFAULT_LRU               "LRU_FOR_SCAN"

#define         PAN_JOIN_SEARCH_LEVEL         "JOIN_SEARCH_LEVEL"
#define         PAN_JOIN_MAXTAB_LEVEL4        "JOIN_MAXTAB_LEVEL4"
#define         PAN_JOIN_MAXTAB_LEVEL9        "JOIN_MAXTAB_LEVEL9"

#define         PAN_OPTIM_JOIN_FETCH          "OPTIM_JOIN_FETCH"
#define         PAN_OPTIM_MAX_MERGE           "OPTIM_MAX_MERGE"
#define         PAN_OPTIM_INV_ONLY            "OPTIM_INV_ONLY"
#define         PAN_OPTIM_CACHE               "OPTIM_CACHE"

#define         PAN_DYNDATA                   "_SHAREDDYNDATA"
#define         PAN_DYNPOOL                   "_SHAREDDYNPOOL"
#define         PAN_PNOPOOLSIZE               "_PNO_SUPPLY_SIZE"

#define         PAN_RUNDIR                    "RUNDIRECTORY"
#define         PAN_MAXDEVSPACES              "_MAXDEVSPACES"

#define         PAN_TASKCLUSTER_N             "_TASKCLUSTER_"
#define         PAN_TASKCLUSTER_N_LN          13

#define         PAN_MAXSYSDEVSPACES           "_MAXSYSDEVSPACES"
#define         PAN_SYS_N                     "SYSDEV_"
#define         PAN_SYS_N_LN                  7
#define         PAN_SYS_N_SIZE                "SYS_SIZE_"
#define         PAN_SYS_N_SIZE_LN             8

#define         PAN_M_SYS_N                   "M_SYSDEV_"
#define         PAN_M_SYS_N_LN                9

#define         PAN_MAXARCHIVELOGS            "MAXARCHIVELOGS"
#define         PAN_ARCHIVE_LOG_N             "ARCHIVE_LOG_"
#define         PAN_ARCHIVE_LOG_N_LN          12
#define         PAN_ARCHIVE_LOG_N_SIZE        "ARCHIVE_SIZE_"
#define         PAN_ARCHIVE_LOG_N_SIZE_LN     13

#define         PAN_M_ARCHIVE_LOG_N           "M_ARCHIVE_LOG_"
#define         PAN_M_ARCHIVE_LOG_N_LN        14

#define         PAN_LOG_MODE                  "LOG_MODE"
#define         PAN_LOG_SEGMENT_SIZE          "LOG_SEGMENT_SIZE"
#define         PAN_DEFAULT_CODE              "DEFAULT_CODE"
#define         PAN_DATE_TIME_FORMAT          "DATE_TIME_FORMAT"
#define         PAN_SESSION_TIMEOUT           "SESSION_TIMEOUT"
#define         PAN_LOCK_TIMEOUT              "LOCK_TIMEOUT"
#define         PAN_REQUEST_TIMEOUT           "REQUEST_TIMEOUT"
#define         PAN_NET_TIMEOUT               "_NET_MINTIMEOUT"
#define         PAN_RESTART_TIME              "_RESTART_TIME"

#define         PAN_MAXDATADEVSPACES          "MAXDATADEVSPACES"
#define         PAN_DATADEV_N                 "DATADEV_"
#define         PAN_DATADEV_N_LN              8
#define         PAN_DATADEV_N_SIZE            "DATA_SIZE_"
#define         PAN_DATADEV_N_SIZE_LN         10

#define         PAN_MIRR_DATA                 "_MIRRORED_DATA"
#define         PAN_M_DATADEV_N               "M_DATADEV_"
#define         PAN_M_DATADEV_N_LN            10

#define         PAN_USE_ASYNC_IO              "_USE_ASYNC_IO"
#define         PAN_MAXIOTHREADS              "_IOPROCS_PER_DEV"
#define         PAN_IOTHREADS_FOR_PRIO        "_IOPROCS_FOR_PRIO"
#define         PAN_DEV_QUEUE                 "_IOPROCS_SWITCH"
#define         PAN_USE_IOTHREAD_ONLY         "_USE_IOPROCS_ONLY"

#define         PAN_MAXDATAPAGES              "MAXDATAPAGES"
#define         PAN_MAXCPU                    "MAXCPU"
#define         PAN_PRIO_BASE_U2U             "_PRIO_BASE_U2U"
#define         PAN_PRIO_BASE_IOC             "_PRIO_BASE_IOC"
#define         PAN_PRIO_BASE_RAV             "_PRIO_BASE_RAV"
#define         PAN_PRIO_BASE_COM             "_PRIO_BASE_COM"
#define         PAN_PRIO_BASE_REX             "_PRIO_BASE_REX"
#define         PAN_PRIO_FACTOR               "_PRIO_FACTOR"
#define         PAN_DELAY_LW                  "_DELAY_LOGWRITER"
#define         PAN_DELAY_COMMIT              "_DELAY_COMMIT"
#define         PAN_SUBTREE                   "SUBTREE"
#define         PAN_MAXRGN_REQUEST            "MAXRGN_REQUEST"
#define         PAN_MP_RGN_LOOP               "MP_RGN_LOOP"
#define         PAN_MP_RGN_QUEUE              "_MP_RGN_QUEUE"
#define         PAN_MP_RGN_BUSY_WAIT          "_MP_RGN_BUSY_WAIT"
#define         PAN_MP_RGN_PRIO               "_MP_RGN_PRIO"
#define         PAN_MP_RGN_DIRTY_READ         "_MP_RGN_DIRTY_READ"
#define         PAN_MP_DISP_PRIO              "_MP_DISP_PRIO"
#define         PAN_MP_DISP_LOOPS             "_MP_DISP_LOOPS"
#define         PAN_DATACACHE_REGIONS         "_DATA_CACHE_RGNS"
#define         PAN_TREE_REGIONS              "_TREE_RGNS"
#define         PAN_TRANS_REGIONS             "_TRANS_RGNS"
#define         PAN_TAB_REGIONS               "_TAB_RGNS"
#define         PAN_ROW_REGIONS               "_ROW_RGNS"
#define         PAN_OMS_REGIONS               "_OMS_RGNS"
#define         PAN_MAXUSERSTACK              "_MAXTASK_STACK"

#define         PAN_MAXDATAWRITER             "_MAXDATAWRITER"
#define         PAN_COPY_IO_BLOCK_CNT         "_COPY_IO_BLOCK_CNT"
#define         PAN_MULT_IO_BLOCK_CNT         "_MULT_IO_BLOCK_CNT"
#define         PAN_REALTIME_PRIO             "REALTIME_PRIO"

#define         PAN_UNICODE_CATALOG           "_UNICODE"

#define         PAN_PACKET_SIZE               "_PACKET_SIZE"
#define         PAN_MIN_REPLY_SIZE            "_MINREPLY_SIZE"

#define         PAN_MB_DATA_SIZE              "_MBLOCK_DATA_SIZE"
#define         PAN_MB_QUAL_SIZE              "_MBLOCK_QUAL_SIZE"
#define         PAN_MB_STACK_SIZE             "_MBLOCK_STACK_SIZE"
#define         PAN_KB_WORK_ST_SIZE           "_WORKSTACK_SIZE"
#define         PAN_KB_WORKBUF_SIZE           "_WORKDATA_SIZE"
#define         PAN_CATALOG_CACHE_SIZE        "_CAT_CACHE_MINSIZE"

#define         PAN_GATEWAYTYPE               "GATEWAYTYPE"
#define         PAN_DW_COPY_BOUNDARY          "_DW_COPY_BOUNDARY"
#define         PAN_DW_IO_AREA_SIZE           "_DW_IO_AREA_SIZE"
#define         PAN_DW_IO_AREA_FLUSH          "_DW_IO_AREA_FLUSH"
#define         PAN_DW_LRU_TAIL_FLUSH         "_DW_LRU_TAIL_FLUSH"
#define         PAN_FBM_DEVSP_BALANCE         "_FBM_DEVSP_BALANCE"
#define         PAN_FBM_DEVSP_COMPR           "_FBM_DEVSP_COMPR"
#define         PAN_FBM_LOW_IO_RATE           "_FBM_LOW_IO_RATE"
#define         PAN_BACKUP_HISTFILE           "_BACKUP_HISTFILE"
#define         PAN_BACKUP_MED_DEF            "_BACKUP_MED_DEF"
#define         PAN_KERNELDUMPFILE            "_KERNELDUMPFILE"
#define         PAN_KERNELTRACEFILE           "_KERNELTRACEFILE"
#define         PAN_KERNELTRACESIZE           "KERNELTRACESIZE"
#define         PAN_DIAGFILE                  "_KERNELDIAGFILE"
#define         PAN_DIAG_SIZE                 "KERNELDIAGSIZE"
#define         PAN_UTILITY_PROTFILE          "_UTILITY_PROTFILE"
#define         PAN_UTILITY_PROTSIZE          "UTILITY_PROTSIZE"
#define         PAN_SVP_1_CONV_FLUSH          "_SVP_1_CONV_FLUSH"
#define         PAN_RTEDUMPFILE               "_RTEDUMPFILE"
#define         PAN_FORMAT_DATAVOLUME         "FORMAT_DATAVOLUME"

#define         PAN_EVENTFILE                 "_EVENTFILE"
#define         PAN_EVENT_SIZE                "_EVENTSIZE"
#define         PAN_MAXEVENTTASKS             "_MAXEVENTTASKS"
#define         PAN_EVENTPRIO                 "_EVENTPRIO"
#define         PAN_MAXEVENTS                 "_MAXEVENTS"
#define         PAN_XPARAM_CHANGED            "__PARAM_CHANGED___" /*XP_PARAM_CHANGED*/
#define         PAN_PARAM_VERSION_TIME        "__PARAM_VERIFIED__" /*XP_PARAM_VERSION_TIME*/
#define         PAN_PAGE_SIZE                 "_PAGE_SIZE"

#define         PAN_OMS_HEAP_LIMIT            "OMS_HEAP_LIMIT"
/* PTS 1110149 */
#define         PAN_OMS_HEAP_THRESHOLD        "OMS_HEAP_THRESHOLD"
#define         PAN_OMS_VERS_THRESHOLD        "OMS_VERS_THRESHOLD"
#define         PAN_HEAP_CHECK_LEVEL          "HEAP_CHECK_LEVEL" 
/* END PTS 1110149 */
#define         PAN_OMS_HEAP_COUNT            "OMS_HEAP_COUNT"
#define         PAN_OMS_HEAP_BLOCKSIZE        "OMS_HEAP_BLOCKSIZE"
/* PTS 1115635 */
#define         PAN_OMS_STREAM_TIMEOUT        "OMS_STREAM_TIMEOUT"

#define         PAN_MAXGARBAGE_COLLECTOR      "_MAXGARBAGE_COLL"
#define         PAN_MAX_BACKUP_TASKS          "_MAX_BACKUP_TASKS"
#define         PAN_READAHEAD_BLOBS           "_READAHEAD_BLOBS"
#define         PAN_SERVERDB_FOR_SAP          "_SERVERDB_FOR_SAP"
#define         PAN_EVENT_ALIVE_CYCLE         "_EVENT_ALIVE_CYCLE"

#define         PAN_INDEXFILE_LIST_SIZE       "_IDXFILE_LIST_SIZE"
#define         PAN_AK_DUMP_ALLOWED           "_AK_DUMP_ALLOWED"
#define         PAN_MAX_MESSAGE_FILES         "_MAX_MESSAGE_FILES"
#define         PAN_TASK_STACK_AREA           "_TASK_STACK_AREA"

/* New Converter ... */
#define         PAN_DEVNO_BIT_COUNT           "DEVNO_BIT_COUNT"
#define         PAN_CONVERTER_REGIONS         "CONVERTER_REGIONS"
#define         PAN_STAT_FREEPOOL_SIZE        "STAT_FREEPOOL_SIZE"
#define         PAN_DYN_FREEPOOL_SIZE         "DYN_FREEPOOL_SIZE"
#define         PAN_CACHE_SIZE                "CACHE_SIZE"

#define         PAN_CRASH_HISTORY_NUM         "DIAG_HISTORY_NUM"
#define         PAN_CRASH_HISTORY_PATH        "DIAG_HISTORY_PATH"
#define         PAN_CRASH_SEM                 "_DIAG_SEM"

#define         PAN_LOAD_BALANCING_CHK        "LOAD_BALANCING_CHK"
#define         PAN_LOAD_BALANCING_DIFF       "LOAD_BALANCING_DIF"
#define         PAN_LOAD_BALANCING_EQUALITY   "LOAD_BALANCING_EQ"
  

/***************************************************************/
#if defined(_WIN32)
/***************************************************************/

#define         PAN_DYNAMIC_SERVER           "DYNAMIC_SERVER"
#define         PAN_ALLOW_COMPR_DEVSP        "ALLOW_COMPR_DEVSP"
#define         PAN_RESTART_SHUTDOWN         "RESTART_SHUTDOWN"
#define         PAN_DCOM_DEBUG               "DCOM_DEBUG"
#define         PAN_USE_MEM_ENHANCEMENT      "USE_MEM_ENHANCE"
#define         PAN_MEM_ENHANCEMENT_LIM      "MEM_ENHANCE_LIMIT"
#define         PAN_USE_FIBERS               "USE_FIBERS"
#define         PAN_HIRES_TIMER_TYPE         "HIRES_TIMER_TYPE"

/***************************************************************/
#else
/***************************************************************/

#define         PAN_SHMCHUNK                  "_SHMCHUNK"
#define         PAN_SHMKERNEL                 "_SHMKERNEL"

#define         PAN_DYNAMIC_TASK_STACK        "_DYN_TASK_STACK"
#define         PAN_OPMSG1                    "OPMSG1"
#define         PAN_OPMSG2                    "OPMSG2"

#define         PAN_USE_COROUTINES            "USE_COROUTINES"
#define         PAN_USE_STACK_ON_STACK        "USE_STACK_ON_STACK"
#define         PAN_USE_UCONTEXT              "USE_UCONTEXT"

/***************************************************************/
#endif

/* PTS 1108976 Trace Size for each task type in pages */
#define         PAN_TRACE_PAGES_TI            "TRACE_PAGES_TI"
#define         PAN_TRACE_PAGES_GC            "TRACE_PAGES_GC"
#define         PAN_TRACE_PAGES_AL            "TRACE_PAGES_AL"
#define         PAN_TRACE_PAGES_DW            "TRACE_PAGES_DW"
#define         PAN_TRACE_PAGES_US            "TRACE_PAGES_US"
#define         PAN_TRACE_PAGES_UT            "TRACE_PAGES_UT"
#define         PAN_TRACE_PAGES_SV            "TRACE_PAGES_SV"
#define         PAN_TRACE_PAGES_EV            "TRACE_PAGES_EV"
#define         PAN_TRACE_PAGES_TW            "TRACE_PAGES_TW"
#define         PAN_TRACE_PAGES_CS            "TRACE_PAGES_CS"
#define         PAN_TRACE_PAGES_BUP           "TRACE_PAGES_BUP"

#define         PAN_SHOW_MAX_STACK_USE        "SHOW_MAX_STACK_USE"

#define         PAN_MMAP_MEM_POOL             "MMAP_MEM_POOL"
#define         PAN_SHM_MEM_POOL              "SHM_MEM_POOL"
#define         PAN_VALLOC_MEM_POOL           "VALLOC_MEM_POOL"
#define         PAN_HEAP_MEM_POOL             "HEAP_MEM_POOL"
#define         PAN_MEMORY_SLOT_SIZE          "MEMORY_SLOT_SIZE"

#define         PAN_COMMENT                   "_COMMENT"

#define         PAN_SUPPRESS_CORE             "SUPPRESS_CORE"
#define         PAN_FORMATTING_MODE           "FORMATTING_MODE"
#define         PAN_OFFICIAL_NODE             "OFFICIAL_NODE"

#define         PAN_EXPAND_COM_TRACE          "EXPAND_COM_TRACE"

#define         PAN_ALLOW_DEMANGLE            "SYMBOL_DEMANGLING"

#define         PAN_LOG_MIRRORED              "LOG_MIRRORED"

#ifndef XPARAM_NAMES_ONLY

#define DEVSPACE_UNUSED  -1

#define NUM_DEVSP_TYPES 8

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/

#if defined(_WIN32)
# define XPARAM(_param_elem) (kgs.XParam->_param_elem)
#else /* UNIX */
# define XPARAM(_param_elem) (kgs->XParam->_param_elem)
#endif

/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

/* --- RTE_XPARAM_REC ------------------------------------------------------ */

typedef struct  dev_descr
  {
    tsp2_devnamec               szDevspaceName;
    tsp00_Int                   lDevspaceSize;
    tsp00_Int                   lDevspaceType;
    RTE_DevID                   physicalDevID; /* 1109850 */
  } DEV_DESCR;

typedef struct  devspaces_descr
  {
    tsp00_Uint                ulNumDevs;
    DEV_DESCR                 *Devspaces;
  } DEVSPACES_DESCR;

typedef struct  rte_xparam_record
  {
  tsp00_VFilenamec            szRunDirectory;
  tsp00_CryptPw               crypControlPassw;
  tsp00_KnlIdentifierc        szControlUser;
  tsp00_Uint                  ulDBPageSize;
  tsp00_Int                   lNoOfBackupDevs;
  tsp00_Bool                  fLogBackToPipe; 
  tsp00_Int                   lMaxSysDevspaces; /* for Migration only ... */
  tsp00_Int                   lMaxArchiveLogs;
  tsp00_Int                   lMaxDataDevspaces;
  tsp00_Int                   lLogMode;
  tsp00_Int                   lLogSegmentSize;
  tsp00_Int                   lSessionTimeout;
  tsp00_Int                   lLockTimeout;
  tsp00_Int                   lRequestTimeout;
  tsp00_Int                   lNetTimeout;
  tsp00_Int                   lRestartTime;
  tsp00_C18c                  szDefaultCode;
  tsp00_C18c                  szDateTimeFormat;
  tsp00_Int                   lMaxTrans;
  tsp00_Int                   lSequenceCache;
  tsp00_Int                   lDataCachePages;
  tsp00_Int                   lCatalogCachePages;
  tsp00_Int                   lDistribDescCache;
  tsp00_Int                   lMaxDataPages;
  tsp00_Int                   lFbmCachePages;
  tsp00_Bool                  fUseThreadForTask;         /* PTS 1108525 */
  tsp00_Bool                  fUseProcessScope;          /* PTS 1109046 */
  tsp00_Uint                  ulMaxUsUtStack;
  tsp00_Int                   lMaxLocks;
  tsp00_Uint                  ulDeadlockDetection;
  tsp00_Int                   ulLockSupplyBlock;
  tsp00_Int                   lServerCmdCache;
  tsp00_Int                   lMinServerDesc;
  tsp00_Int                   lMaxServerDB;
  tsp00_Int                   lLogQueuePages;
  tsp00_Int                   lConvCachePages;
  tsp00_Int                   lUsmCachePages;
  tsp00_Int                   lBackUpBlockCount;
  tsp00_Int                   lMaxDevspaces;
  tsp00_Int                   lJoinPrefetchPercent;
  tsp00_Int                   lJoinSearchLevel;
  tsp00_Int                   lJoinMaxTabLevel4;
  tsp00_Int                   lJoinMaxTabLevel9;
  tsp00_Int                   lMaxMerge;
  tsp00_Int                   lDwCopyBoundary;
  tsp00_Int                   lDwIOAreaSize;
  tsp00_Int                   lDwIOAreaFlush;
  tsp00_Int                   lDwLRUTailFlush;
  tsp00_Int                   lFbmBalance;
  tsp00_Int                   lFbmCompression;
  tsp00_Int                   lFbmLowIORate;

  tsp00_C40c                  szKernelVersion;

  tsp00_Int                   lSharedDynData;
  tsp00_Int                   lSharedDynPool;

  tsp00_VFilenamec            szDiagFile;
  tsp00_Uint                  ulDiagSize;

  tsp00_VFilenamec            szEventFile;
  tsp00_Uint                  ulEventSize;
  tsp00_Uint                  ulMaxEvents;
  tsp00_Uint                  ulEventAliveCycle;

  tsp00_VFilenamec            szBackupHistFile;
  tsp00_VFilenamec            szBackupMedDefFile;

  tsp00_VFilenamec            szUtilityProtFile;
  tsp00_Uint                  ulUtilityProtSize;

  tsp00_VFilenamec            szDumpFilename;

  tsp2_devnamec               szTraceDevspace;
  tsp00_Int                   lTraceDevspaceSize;

  tsp00_Int                   lDeviPerDevspace;
  tsp00_Int                   lPrioDeviPerDevspace;
  tsp00_Int                   lDevQueue;

  tsp00_Int                   lRealtimePrio;
  tsp00_Uint                  ulMaxCPU;
  tsp00_Uint                  ulRegionCollisionLoop;
  tsp00_Uint                  ulNoOfRegions;
  char                        **TaskClusterArray;

  tsp00_Uint                  ulRegLockSlice;
  tsp00_Int                   lPrioBaseU2U;
  tsp00_Int                   lPrioBaseRAV;
  tsp00_Int                   lPrioBaseIOC;
  tsp00_Int                   lPrioBaseCOM;
  tsp00_Int                   lPrioBaseREX;
  tsp00_Int                   lPrioFactor;
  tsp00_Int                   lDelayLW;

  tsp00_Bool                  fServerDBForSAP;
  tsp00_Bool                  fDefaultLRU;
  tsp00_Bool                  fDelayCommit;
  tsp00_Bool                  fDbSockets;

  tsp00_Bool                  fAllowSingleIO;
  tsp00_Bool                  fExclQueue;
  tsp00_Bool                  fBegExclQueWhenPrio;
  tsp00_Bool                  fBegExclDirtyRead;

  tsp00_Bool                  fEndExclBusyWait;
  tsp00_Bool                  fDynDispQueSrch;
  tsp00_Bool                  fSubtree;
  tsp00_Bool                  fUnicode;

  tsp00_Bool                  fMirrData;
  tsp00_Bool                  fSvpFstConvFlush;
  tsp00_Bool                  fUseMemEnhancement;
  tsp00_Uint                  lMemEnhancementLimit;
  tsp00_Bool                  fUseAsyncIO;

  tsp00_Bool                  fFormatDataVolume;
  tsp00_Bool                  fEventPrio;
  tsp00_Bool                  fAKDumpAllowed;
  tsp00_Bool                  fStackStack;

  tsp00_Uint                  ulMultIOBlockCount;

  tsp00_Uint                  ulMaxDataWriter;
  tsp00_Uint                  ulMaxEventTasks;
  tsp00_Uint                  ulMaxBackupTasks;
  tsp00_Uint                  ulMaxGarbageCollector;
  tsp00_Uint                  ulMaxObjectLogWriter;
  tsp00_Uint                  ulMaxArchiveLogWriter;
  tsp00_Uint                  ulMaxUser;
  tsp00_Uint                  ulMaxServer;

  tsp00_Uint                  ulDwIOBufSize;
  tsp00_Uint                  ulDispLoopsBefSemop;
  tsp00_Uint                  ulNumDataCacheReg;
  tsp00_Uint                  ulNumTreeReg;
  tsp00_Uint                  ulNumTransReg;
  tsp00_Uint                  ulNumTabReg;
  tsp00_Uint                  ulNumRowReg;
  tsp00_Uint                  ulNumOMSReg;
  tsp00_Uint                  ulPacketSize;
  tsp00_Uint                  ulMinReplySize;
  tsp00_Uint                  ulPnoPoolSize;
  tsp00_Uint                  ulMBDataSize;
  tsp00_Uint                  ulMBQualSize;
  tsp00_Uint                  ulMBStackSize;
  tsp00_Uint                  ulKBWorkStSize;
  tsp00_Uint                  ulKBWorkBufSize;
  tsp00_Uint                  ulCatalogCacheSize;
  tsp00_Uint                  ulXparamChanged;
  DEVSPACES_DESCR             DevspDescr   [NUM_DEVSP_TYPES];
  tsp00_VFilenamec            szRTEDumpFilename;
  tsp00_Uint                  ulReadAheadBlobsGT;
  tsp00_Int                   lIndexfileListSize;
  tsp00_Uint                  ulMaxMessageFiles;
  tsp00_Uint                  ulOMSHeapLimit;            /* CR 1105220 */
  tsp00_Uint                  ulOmsHeapUsageThreshold;   /* PTS 1110149 */
  tsp00_Uint                  ulOmsVersionFreeThreshold; /* PTS 1110149 */
  tsp00_Uint                  ulHeapCheckLevel;          /* PTS 1117756 */
  tsp00_Uint                  ulOmsHeapCount;            /* PTS 1110828 */
  tsp00_Uint                  ulOmsHeapBlocksize;        /* PTS 1110828 */
  tsp00_Uint                  ulOmsStreamTimeout;        /* PTS 1115635 */
  tsp00_Uint                  uCrashHistoryNum;
  tsp00_VFilenamec            szCrashHistoryPath;
  tsp00_Uint                  uCrashSem;
  tsp00_Uint                  uLoadBalancingCheck;       /* PTS 1117672 */
  tsp00_Uint                  uLoadBalancingDiff;        /* PTS 1117672 */
  tsp00_Uint                  uLoadBalancingEquality;    /* PTS 1117672 */
/***************************************************************/
#if defined(_WIN32)
/***************************************************************/
  tsp00_Bool                  fDynamicServerTasks;
  tsp00_Bool                  fAllowCompressedDevsp;
  tsp00_Bool                  fAutoRestartShutdown;
  tsp00_Bool                  fProfileDCOMObject;       /* CR 1000241 */
  tsp00_Bool                  fUseFibers;
  tsp00_C16c                  szHiResTimerType;         /* PTS 1117672 */
/***************************************************************/
#else
/***************************************************************/
  tsp00_VFilenamec            szOpMsg1;
  tsp00_VFilenamec            szOpMsg2;
  tsp00_Bool                  fDynTaskStack;
  tsp00_Bool                  fUseSharedMMU;	/* CR 1000130 */
  tsp00_Int                   lShmChunk;
  tsp00_Int                   lShmKernel;
  tsp00_Bool                  fUseCoroutines;
  tsp00_Bool                  fUseStackOnStack;
  tsp00_Bool                  fUseUcontext;
/***************************************************************/
#endif
/***************************************************************/
/* PTS 1105389 */
  tsp00_C8c                   szInstanceType;
  tsp00_Bool                  fOptimCache;
  tsp00_Bool                  fOptimInvOnly;

  tsp00_Int                   devnoBitCount;
  tsp00_Int                   converterRegions;
  tsp00_Int                   statFreepoolSize;
  tsp00_Int                   dynFreepoolSize;
  tsp00_Int                   cacheSize;

  /* PTS 1108976 */
  tsp00_Int                   tracePagesTI;
  tsp00_Int                   tracePagesGC;
  tsp00_Int                   tracePagesAL;
  tsp00_Int                   tracePagesDW;
  tsp00_Int                   tracePagesUS;
  tsp00_Int                   tracePagesUT;
  tsp00_Int                   tracePagesSV;
  tsp00_Int                   tracePagesEV;
  tsp00_Int                   tracePagesTW;
  tsp00_Int                   tracePagesCS;
  tsp00_Int                   tracePagesBUP;

  tsp00_Bool                  showMaxStackUse;

  tsp00_Int                   mmapMemPoolSize;
  tsp00_Int                   shmMemPoolSize;
  tsp00_Int                   vallocMemPoolSize;
  tsp00_Int                   heapMemPoolSize;
  tsp00_Int                   memorySlotSize;

  /* PTS 1113405 */
  tsp00_Bool                  suppressCore;
  /* PTS 1109850 */
  tsp00_Uint                  ulFormattingMode;
  /* PTS 1116151 */
  tsp00_NodeIdc               szOfficialNode;
  /* PTS 1118987 */
  tsp00_Bool                  allowDemangle;
  /* PTS 1120119 */
  tsp00_Bool                  fLogMirrored;
  } RTE_XPARAM_REC;

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/

/*================================================*/

/* --- VEO79.C -------------------------------------------------------------- */

void   sql79k_create_taskcluster(   RTE_XPARAM_REC  *XParam,
                                    char            **TaskCluster );

void   sql79k_free_taskcluster  (   RTE_XPARAM_REC  *XParam,
                                    char            *TaskCluster );


#endif /* XPARAM_NAMES_ONLY */

#endif  /* GEO002_H */
