/****************************************************************************

  module      : geo007_2.h

  -------------------------------------------------------------------------

  responsible : JoergM
				Raymond Roedling (D024857 - Raymond.Roedling@SAP-AG.de)
				Frank Hoffmann (D024858 - Frank.Hoffmann@SAP-AG.de)

  special area: RTE User visible Strings 
  
  description : Kernel, Common and User RTE show messages for UNIX and NT. 
				To ease the typographic problem fixing and translation
				all such strings are contained in this module.
				NOTE: Messages are seperated from this and contained in geo007_1.h

  last changed: 2000-10-13  19:11
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1998



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef GEO007_2_H
#define GEO007_2_H



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/


/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

/* =============================== ERROR MESSAGES ============================== */

#define ERRMSG_CANT_INSTALL_SIG_HANDLER     "could not install signal handler"
#define ERRMSG_SET_EXIT_HANDLER_FAILED      "set exit handler failed [%lu]"
#define ERRMSG_MISSING_DBROOT               "missing "DBROOT_ENV_VAR" "
#define ERRMSG_CANT_FREE_MEM                "could not free memory"
#define ERRMSG_ALLOC_MEMORY                 "could not allocate memory"
/*
 --- remote comm. ---
*/

#define ERRMSG_COM_DATABASE_NOT_STARTED     "database not running"
#define ERRMSG_COM_TO_MANY_DB_SESSIONS      "too many database sessions active"
#define ERRMSG_COM_SERVER_OR_DB_NOT_FOUND   "database or server not found"

#define ERRMSG_COM_CANT_WRITE_TO_PIPE       "could not write to pipe"
#define ERRMSG_COM_PACK_WRITE_SIZE_ERROR    "pipe write size error"
#define ERRMSG_COM_CONN_PACKET_READ_ERR     "connect packet read pipe error"
#define ERRMSG_COM_CANT_SND_TO_SOCKET       "could not send to socket"
#define ERRMSG_COM_CONN_PACKET_REC_ERR      "connect packet receive error"
#define ERRMSG_COM_PACK_SEND_SIZE_ERROR     "socket send size error"
#define ERRMSG_COM_CONN_SERV_REJECT         "server rejected connection"
#define ERRMSG_COM_CONN_PACKET_GARBLED      "connect packet garbled"
#define ERRMSG_COM_ILL_PACKET_SIZE          "illegal packet size"
#define ERRMSG_COM_SOCKET_SND_ERROR         "socket send error"
#define ERRMSG_COM_SOCKET_REC_ERROR         "socket receive error"
#define ERRMSG_COM_CONN_BROKEN              "connection broken"
#define ERRMSG_COM_CANT_SET_OBJ_SEC         "could not set object security"
#define ERRMSG_COM_CANT_GET_OBJ_SEC         "could not get object security"
#define ERRMSG_COM_SOCKET_SELECT_ERROR      "socket select error"
#define ERRMSG_COM_SOCKET_SET_SNDBUF_ERROR  "socket set send buf error"
#define ERRMSG_COM_SOCKET_SET_RCVBUF_ERROR  "socket set recv buf error"
#define ERRMSG_COM_BAD_RTE_RETURN_CODE      "unknown RTE return code received"
#define ERRMSG_COM_NOT_OK                   "communication broken [not ok]"
#define ERRMSG_COM_TIMEOUT                  "command timeout"
#define ERRMSG_COM_CRASH                    "connection aborted"
#define ERRMSG_COM_PACKET_LIMIT             "packet limit reached"
#define ERRMSG_COM_RELEASED                 "session released"
#define ERRMSG_COM_UNKNOWN_REQUEST          "unknown request"
#define ERRMSG_COM_SHUTDOWN                 "database shutdown"


/*===========================================================================*
 *                            C O M M U N I C A T I O N                      *
 *===========================================================================*/

#define ERRMSG_SERVERDB_NOT_STARTED         "SERVERDB not started"
#define ERRMSG_NETW_NOT_RESTARTED           "network not restarted"
#define ERRMSG_COM_SERVER_OR_DB_NOT_ACC     "XSERVER or SERVERDB not accessible"
#define ERRMSG_COM_CANT_CREATE_COM_SEM      "could not create comm. semaphore"
#define ERRMSG_COM_CANT_ALLOC_COM_SEG       "could not create comm. shared memory"
#define ERRMSG_COM_CANT_GIVE_COM_SEG        "could not share memory with kernel"
#define ERRMSG_COM_CANT_OPEN_UKT_SEM        "could not open kernel semaphore"
#define ERRMSG_COM_CANT_WRITE_COM_QUE       "could not write to comm. queue"
#define ERRMSG_CONN_TIMEOUT                 "connect timeout"
#define ERRMSG_COM_TIMEOUT                  "command timeout"
#define ERRMSG_REPLY_TIMEOUT                "reply timeout"
#define ERRMSG_COM_WAIT_COM_SEM             "communication semaphore error"
#define ERRMSG_COM_POST_UKT_SEM             "kernel semaphore error"
#define ERRMSG_COM_CONN_REFUSED             "connection refused"
#define ERRMSG_COM_CONN_BROKEN              "connection broken"
#define ERRMSG_COM_UNBALANCED_REQUEST_REPLY "unbalanced request/reply"
#define ERRMSG_COM_CANT_FREE_MEM            "could not free memory"
#define ERRMSG_COM_TO_MANY_SESSIONS         "too many sessions"
#define ERRMSG_COM_UTIL_ALREADY_CONN        "utility session is already in use"
#define ERRMSG_NO_TCPIP_INSTALLED           "no TCP/IP to connect remote server"
#define ERRMSG_WRONG_TCPIP_VERSION          "wrong DLL-Version, (check: SQLTCPx.DLL)"
#define ERRMSG_SERV_BY_NAME_ERROR           "unknown TCP/IP service"
#define ERRMSG_MISSING_TCP_SERVICE_ENTRY    "missing TCP/IP service entry: "
#define ERRMSG_CANT_CREATE_SOCKET           "could not create socket"
#define ERRMSG_INCORRECT_MAXSIZE            "got incorrect maxsize"
#define ERRMSG_CANT_SET_BUF_SIZE            "could not set socket buffer"
#define ERRMSG_OUT_OF_SOCK_BUF_MEM          "Out of socket buffer memory"
#define ERRMSG_GET_HOST_BY_NAME_ERR         "unknown host name"
#define ERRMSG_REQ_SOCKET_CONNECT           "could not connect to socket"
#define ERRMSG_COM_ILL_REFERENCE            "illegal reference"
#define ERRMSG_COM_CORRUPTED_INTERNAL_DATA  "internal: corrupted connection data"
#define ERRMSG_COM_APPLIC_FORKED            "application forked"
#define ERRMSG_COM_WRONG_CONN_STATE         "wrong connection state"
#define ERRMSG_COM_WRONG_PACKET_LEN         "wrong packet length"
#define ERRMSG_COM_WRONG_PACKET_ADDR        "packet address differs from SQLREQUEST"
#define ERRMSG_KEEP_ALIVE                   "error setting 'KEEPALIVE' socket option"
#define ERRMSG_LINGER                       "error setting 'LINGER' socket option"
#define ERRMSG_XSERVER_NOT_ACTIVE           "XSERVER might be inactive"
#define ERRMSG_CONNECT_TIMED_OUT            "connect request timed out"
#define ERRMSG_CREATING_REC_THREAD          "error creating 'ASYNC RECEIVE' thread"
#define ERRMSG_KILL_REC_THREAD              "could not kill 'ASYNC RECEIVE' thread"
#define ERRMSG_CREATING_REC_SEM             "error creating 'ASYNC RECEIVE' semaphore"
#define ERRMSG_CLOSE_REC_SEM                "error closing 'ASYNC RECEIVE' semaphore"
#define ERRMSG_COM_POST_ASYNC_SEM           "async receive semaphore error"
#define ERRMSG_COM_CLOSE_QUE_MAIL           "could not close queue/mailslot"
#define ERRMSG_COM_CLOSE_SERV_SEM           "could not close server semaphore"
#define ERRMSG_COM_CLOSE_CLIENT_SEM         "could not close client semaphore"
#define ERRMSG_INVALID_PACKET_SIZE          "invalid packet size"
#define ERRMSG_INVALID_MIN_REPL_SIZE        "invalid minimum reply data size"
#define ERRMSG_INVALID_MAX_DATA_LEN         "invalid maximum data length"
#define ERRMSG_INVALID_REPL_PACKET_SIZE     "invalid reply packet size"
#define ERRMSG_GET_OFFICIAL_NODE_NAME       "could not get node name"
#define ERRMSG_GET_HOST_NAME                "could not get host name"
#define ERRMSG_COM_CLOSE_SERV_PROCESS       "could not close server process handle"

#define WRNMSG_COM_NODE_PL_DBNAME_TOO_LONG  "SERVERNODE + SERVERDB too long"
#define ERRMSG_COM_SERVER_NODE_NAME_TO_LONG "server node name too long"

#define ERRMSG_ILL_SWAP_TYPE                  "illegal swap type"
#define ERRMSG_COM_ILL_MAXSEGMENTSIZE         "illegal max. segment size"
#define ERRMSG_COM_PACKET_GARBLED             "received a garbled packet"
#define ERRMSG_COM_CONN_CLOSED_BY_COM_PART    "connection closed by comm. partner"

#define ERRMSG_COM_TO_MANY_OPEN_SESSIONS      "too many SERVERDB sessions active"
#define ERRMSG_PROTOCOL_ERROR                 "protocol error"
#define ERRMSG_UNSUPPORTED_PROTOCOL           "unsupported protocol"
#define ERRMSG_COM_RTE_COMM_ERROR             "communication error"
#define ERRMSG_COM_COMMUNICATION_TIMEOUT      "communication timeout"
#define ERRMSG_COM_COMMSEG_HEADER_CORRUPT     "communication segment header corrupt"

#define ERRMSG_NICOM_NOT_SUPPORTED            "SAP NI not supported"
#define ERRMSG_NICOM_INIT_ERROR               "NI: init failed"
#define ERRMSG_NICOM_FINISH_ERROR             "NI: finish failed"
#define ERRMSG_NICOM_LISTEN_ERROR             "NI: listen failed"
#define ERRMSG_NICOM_ACCEPT_ERROR             "NI: accept failed"
#define ERRMSG_NICOM_CONNECT_ERROR            "NI: connect failed"
#define ERRMSG_NICOM_PEEK_ERROR               "NI: peek error"
#define ERRMSG_NICOM_SOCKET_SND_ERROR         "ni-socket send error"
#define ERRMSG_NICOM_REC_ERROR                "ni receive error"

/*
 --- XUSER ---
*/
#define ERRMSG_CANNOT_CREATE_XUSER_FILE     "could not create USER file"
#define ERRMSG_CANNOT_OPEN_XUSER_FILE       "could not open USER file"
#define ERRMSG_CANNOT_CLOSE_XUSER_FILE      "could not close USER file"
#define ERRMSG_CANNOT_READ_XUSER_FILE       "could not read USER file"
#define ERRMSG_CANNOT_WRITE_XUSER_FILE      "could not write to USER file"
#define ERRMSG_CANNOT_SEEK_XUSER_FILE       "could not seek on USER file"
#define ERRMSG_READ_ERROR_XUSER_FILE        "error while reading USER file"
#define ERRMSG_READ_ERROR_XUSER_PAGE        "error while reading USER page"
#define ERRMSG_READ_ERROR_XUSER_EUSER       "error while reading effective USER"
#define ERRMSG_WRITE_ERROR_XUSER_FILE       "error while writing to USER file"
#define ERRMSG_BLANK_USERKEY                "blank user key not allowed"
#define ERRMSG_NO_XUSER_ENTRY_FOUND         "the USERKEY is unknown"
#define ERRMSG_NO_XUSER_FILE                "could not find USER file"
#define ERRMSG_DIFFERENT_SIDS               "incorrect USER data owner"
#define ERRMSG_ILLEGAL_USER_ENTRY           "illegal USER entry"
#define ERRMSG_ILLEGAL_USER_INDEX           "illegal USER index"
#define ERRMSG_DELETE_XUSER_FILE            "could not delete USER file"
#define ERRMSG_GET_LOGON_USER_NAME          "could not get logon USER name"
#define ERRMSG_NO_SPACE_LEFT                "space for USER entries exhausted"
#define ERRMSG_READ_XUSER_REG_DATA          "could not read USER data"
#define ERRMSG_WRITE_XUSER_REG_DATA         "could not write USER data"
#define ERRMSG_REMOVE_XUSER_REG_DATA        "could not remove USER data"
#define ERRMSG_WRONG_XUSER_REG_DATA_LEN     "wrong USER data length"
#define ERRMSG_NO_GUEST_ACCESS              "missing permission to create USER data"
#define ERRMSG_GET_SID_STRING               "could not get SID"
#define ERRMSG_DATA_NEWER_THAN_COMPONENT    "USER data newer than component"

/*
 --- SQLPRINT ---
*/
#define ERRMSG_WRITE_SPOOLFILE              "could not write on spool file"
#define ERRMSG_READ_SPOOLFILE               "could not read on spool file"

/*
 --- SQLEXEC/SQLAT ---
*/
#define ERRMSG_SYNC_SAME                    "could not start of the same session"
#define ERRMSG_CMD_LENGTH                   "command length too long"
#define ERRMSG_SYNC_START                   "could not start of a sync session"
#define ERRMSG_START_SESSION                "could not start a new session"
#define ERRMSG_END_SESSION                  "could not end session"
#define ERRMSG_SYNC_END                     "undefined end of a session"
#define ERRMSG_CANNOT_EXECUTE_CMD           "could not execute command"
#define ERRMSG_WRONG_TIME_VALUE             "wrong time value"
#define ERRMSG_NO_CMD                       "no command to execute"
#define ERRMSG_SET_STD_HANDLE               "could not set standard handle"
#define ERRMSG_CREATE_PIPE                  "could not create 'BATCH' pipe"
#define ERRMSG_CREATE_THREAD                "could not create 'BATCH' thread"
#define ERRMSG_RESUMING_THREAD              "could not resume 'BATCH' thread"
#define ERRMSG_SCHEDULE_SERVICE_NOT_STARTED "service '%s' not started"
#define ERRMSG_STARTING_FUNC_FROM_DLL       "could start function from DLL"
#define ERRMSG_FUNCTION_NOT_AVAIL           "function not available"

/*
 --- VIRTUAL TERMINAL ---
*/
#define ERRMSG_VT_WRONG_WINDOW_SIZE         "the window size is to large"
#define ERRMSG_VT_GET_VIDEO_MODE            "could not examine Video mode, rc = %hu"
#define ERRMSG_VT_OPEN_KEYBOARD             "could not examine Keyboard, rc = %lu"
#define ERRMSG_VT_CANT_CHANGE_CURSOR        "could not change cursor type, rc = %hu"

/*
 --- VIRTUAL FILE ---
*/
#define ERRMSG_VF_ILL_HOST_FILE_NAME        "illegal host file name"
#define ERRMSG_VF_ILL_READ_FORMAT_COMB      "illegal read/format combination"
#define ERRMSG_VF_BAD_FILENO                "invalid file number"
#define ERRMSG_VF_WRONG_FILE_NAME           "wrong file name"
#define ERRMSG_VF_ILL_DIREC                 "illegal direction"
#define ERRMSG_VF_ILL_FILE_DESC             "illegal file description"
#define ERRMSG_VF_NON_CONST_REC_LEN         "non-constant record length"
#define ERRMSG_VF_SEEK_REACHED_EOF          "seeking overflows the eof"
#define ERRMSG_VF_FTYPE_NOT_TO_SEEK         "wrong file type to seek"
#define ERRMSG_VF_NO_REG_FILE               "could not delete non-regular files"
#define ERRMSG_VF_DELETE                    "could not delete file"
#define ERRMSG_VF_INVALID_TAPE_CONTENTS     "invalid/no tape contents"

/* --- common */
#define ERRMSG_VF_ALLOC_BUFFER_SPACE        "could not allocate buffer space"
#define ERRMSG_VF_FILE_EXIST                "file already exists"
#define ERRMSG_VF_FILE_SHARING_VIOLATION    "this file/tape/pipe is already in use"
#define ERRMSG_VF_FREE_BUFFER_SPACE         "could not free buffer space"
#define ERRMSG_VF_FREE_BUFPOOL_COUNT_DIFF   "free Buffer pool: block count error"
#define ERRMSG_VF_GET_FILE_STATUS           "error by get file status"
#define ERRMSG_VF_GET_FILE_TYPE             "error by get file type"
#define ERRMSG_VF_ILL_APPEND_DEVICE         "illegal device for direction append"
#define ERRMSG_VF_ILL_DIREC                 "illegal direction"
#define ERRMSG_VF_ILL_FILE_DESC             "illegal file description"
#define ERRMSG_VF_ILL_FILE_SPEC             "illegal file specification"
#define ERRMSG_VF_ILL_DATA_SIZE             "illegal data size"
#define ERRMSG_VF_ILL_NEXT_REC_LEN          "length of next record is out of range"
#define ERRMSG_VF_ILL_REC_LEN               "illegal record length"
#define ERRMSG_VF_ILL_RESOURCE              "illegal resource"
#define ERRMSG_VF_MAX_REC                   "record too big   (> max record length)"
#define ERRMSG_VF_MIN_REC                   "record too small (< min record length)"
#define ERRMSG_VF_NO_FREE_FILE_DESCRIPTOR   "No free file descriptor found"
#define ERRMSG_VF_OPEN                      "could not open file/tape/pipe"
#define ERRMSG_VF_READ                      "could not read file/tape/pipe"
#define ERRMSG_VF_SEEK                      "could not seek on file/tape/pipe"
#define ERRMSG_VF_WRITE                     "could not write to file/tape/pipe"
#define ERRMSG_VF_TAPE_END_OF_MEDIA         "end-of-tape marker reached"
#define ERRMSG_VF_TAPE_WRITE_MARK           "Could not write filemark or setmark"
#define ERRMSG_VF_TAPE_FILE_SET_MARK        "file or setmark reached"
#define ERRMSG_VF_TAPE_NOT_PATITIONED       "tape not partitioned"
#define ERRMSG_VF_TAPE_NO_MEDIA_IN_DRIVE    "missing tape media in drive"
#define ERRMSG_VF_TAPE_MEDIA_CHANGED        "tape media changed"
#define ERRMSG_VF_REWIND_TAPE               "could not rewind tape"
#define ERRMSG_VF_DEVICE_NOT_READY          "device not ready"
#define ERRMSG_VF_TAPE_PARAM                "could not get/set tape parameter"
#define ERRMSG_VF_TAPE_LOCK_UNLOCK          "could not lock/unlock tape"
#define ERRMSG_VF_TAPE_LOAD_UNLOAD          "could not load/unload tape"
#define ERRMSG_VF_TAPE_WRONG_PART_CNT       "more then one partition on the tape"
#define ERRMSG_VF_CLOSE                     "could not close file"
#define ERRMSG_VF_DELETE                    "could not delete file"
#define ERRMSG_VF_ACCESS_DENIED             "access denied"
#define ERRMSG_VF_DISK_FULL                 "not enough space on disk"
#define ERRMSG_VF_WRONG_PATH                "wrong path"
#define ERRMSG_VF_WRONG_FILE_DEVICE_NAME    "wrong file or device name"
#define ERRMSG_VF_MEDIA_WRITE_PROTECTED     "the media is write protected"

#define ERRMSG_VF_CREATE_DIR                "could not create dir:"
#define ERRMSG_VF_REMOVE_DIR                "could not remove dir:"
#define ERRMSG_VF_CHANGE_DIR                "could not change dir:"
#define ERRMSG_VF_GETCWD                    "could not get curr. dir:"

/*
 --- SQLSETUSEROPTIONS ---
*/
#define ERRMSG_WRITE_USER_DIAG_OPT_VALS     "Could not write diagfile option values"
#define ERRMSG_READ_USER_DIAG_OPT_VALS      "Could not read diagfile option values"
#define ERRMSG_WRITE_CHAR_SET_OPT_VALS      "Could not write CHARACTERSET option values"
#define ERRMSG_READ_CHAR_SET_OPT_VALS       "Could not read CHARACTERSET option values"
#define ERRMSG_WRITE_SQLOPT_OPT_VALS        "Could not write SQLOPT option values"
#define ERRMSG_READ_SQLOPT_OPT_VALS         "Could not read SQLOPT option values"

/*
 --- SQLARG3 ---
*/
#define ERRMSG_NO_VALID_OPTION              "no valid option"
#define ERRMSG_NO_COMP_SPECIFIED            "no component specified"
#define ERRMSG_WRONG_OPTION_SEQUENCE        "wrong option sequence"
#define ERRMSG_MISSING_QUOTE                "missing quote"
#define ERRMSG_ILLEGAL_ARGUMENT             "illegal argument string"
#define ERRMSG_ILLEGAL_DIALOG_OBJ           "illegal component object"
#define ERRMSG_DOUBLE_SEPERATOR             "illegal seperator sequence"


/*
 --- XPARAM ---
*/
#define ERRMSG_PATH_TOO_LONG_FOR_XPF        "path too long for PARAM file"
#define ERRMSG_UNDEFINED_OPEN_MODE          "undefined open mode of PARAM file"
#define ERRMSG_ACCESS_XPARAM_FILE           "cannot access PARAM file"
#define ERRMSG_XP_FILE_READONLY             "put failed: PARAM file opened READONLY"
#define ERRMSG_BLANK_XP_KEY                 "blank keys not allowed"
#define ERRMSG_KEY_NOT_FOUND                "key not found"
#define ERRMSG_CANNOT_WRITE_XP_KEY          "cannot write key"
#define ERRMSG_CANNOT_WRITE_XP_VALUE        "cannot write value"
#define ERRMSG_CANNOT_WRITE_XP_FILE         "cannot write to PARAM file"
#define ERRMSG_WRITE_ERROR                  "write error"
#define ERRMSG_CANNOT_READ_XP_KEY           "cannot read key"
#define ERRMSG_CANNOT_READ_XP_VALUE         "cannot read value"
#define ERRMSG_READ_ERROR                   "read error"
#define ERRMSG_CANNOT_FIND_START_POINT      "cannot find start point"
#define ERRMSG_CANNOT_FIND_PREV_ENTRY       "cannot find previous entry"
#define ERRMSG_CANNOT_FIND_ACT_ENTRY        "cannot find actual entry"
#define ERRMSG_EOF_REACHED                  "end of file reached"
#define ERRMSG_PARAM_UPDATE_NOT_ALLOWED     "parameter update not allowed"
#define ERRMSG_BAD_FORMAT                   "bad format"

/*
 --- SERVICE ---
*/
#define SERVICE_USAGE                       "usage: service [[-g|-G] | [-d <..>][-r][-n <..>][-R <..>][-u][-q|-s][-a]]\r\n"\
                                            "\n"                                                                \
                                            "     -d <serverdb>           - SERVERDB name       \n"             \
                                            "     -n <servernode>         - server node name    \n"             \
                                            "     -R <dbroot>             - DBROOT              \n"             \
                                            "     -G                      - install SAPDB operator group\n"     \
                                            "     -g                      - delete SAPDB operator group\n"      \
                                            "     -q                      - quick kernel        \n"             \
                                            "     -s                      - slow kernel         \n"             \
											"     -A <account,password>   - use this account    \n"             \
                                            "     -a                      - set autostart option\n"             \
											"     -P <parameter,value>    - alter parameter     \n"             \
                                            "     -r                      - removes service control entry\n"    \
                                            "     -u                      - update service control entry\n"

/*
 --- SHOW ---
*/
#if defined(_WIN32)
 #define SHOW_USAGE_95                      "usage: show [-n <servernode>]\r\n"            \
                                            "\n"                                           \
                                            "     -n <servernode> - server node name    \n"

 #define SHOW_USAGE                         "usage: show [-n <servernode>][-v|-r]\r\n"        \
                                            "\n"                                              \
                                            "     -n <servernode> - server node name    \n"   \
                                            "     -v              - show SERVERDB-Versions\n" \
                                            "     -r              - show DBROOTs\n"           \
											"     -A              - show service accounts\n"          
#else
 #define SHOW_USAGE                         "usage: show\r\n"
#endif

/*
 --- XSTART ---
*/
#if defined(_WIN32)
 #define XSTART_USAGE_NT                    "usage: strt [-d <..>] [-slow|-quick|-n <servernode>]\r\n"\
                                            "\n"                                            \
                                            "     -d <serverdb>    - SERVERDB name                 \n"\
                                            "     -n <servernode>  - server node name              \n"\
                                            "     -i               - no automatic restart          \n"\
                                            "     -g               - start gateway                 \n"\
                                            "     -l               - start with kernel low priority\n"\
                                            "     -slow            - start slow  kernel            \n"\
                                            "     -quick           - start quick kernel            \n"

 #define XSTART_USAGE                       "usage: strt [-d <..>] [-slow|-quick]                  \r\n"\
                                            "\n"                                                       \
                                            "     -d <serverdb>    - SERVERDB name                 \n"\
                                            "     -i               - no automatic restart          \n"\
                                            "     -l               - start with kernel low priority\n"\
                                            "     -slow            - start slow  kernel            \n"\
                                            "     -quick           - start quick kernel            \n"
#else
 #define XSTART_USAGE                       "usage: strt [-d <..>] [-slow|-quick] [-a]\r\n" \
                                            "\n"                                            \
                                            "     -d <serverdb>    - SERVERDB name       \n"\
                                            "     -i               - no automatic restart\n"\
                                            "     -slow            - start slow  kernel   \n"\
                                            "     -quick           - start quick kernel   \n"\
                                            "     -a               - close window automatically\n"
#endif

/*
 --- XSTOP ---
*/
#if defined(_WIN32)
 #define XSTOP_USAGE_NT                     "usage: stp -d <..> [-p|-n <servernode>]\r\n"\
                                            "\n"                                         \
                                            "     -d <serverdb>      - SERVERDB name    \n"\
                                            "     -n <servernode>    - server node name \n"\
                                            "     -p                 - dump             \n"
 #define XSTOP_USAGE                        "usage: stp -d <..> [-p]\r\n"\
                                            "\n"                                           \
                                            "     -d <serverdb>      - SERVERDB name    \n"\
                                            "     -p                 - dump             \n"
#else
 #define XSTOP_USAGE                        "usage: stp -d <..> [-p]\r\n"\
                                            "\n"                                           \
                                            "     -d <serverdb>      - SERVERDB name    \n"\
                                            "     -p                 - dump             \n"

#endif
#define XSTOP_WRONG_VERS_KERNEL             "       Kernel  RTE Version :  %s\n"
#define XSTOP_WRONG_VERS_XSTOP              "       XSTOP   RTE Version :  %s\n"

/*
 --- GETPARAM ---
*/
#define GETPARAM_USAGE                      "usage: getparam <ServerDB> <ParamName> \r\n"
#define GETPARAM_ERR_TEXT                   "getparam : %s\n"

/*
 --- PUTPARAM ---
*/
#define PUTPARAM_USAGE                      "usage: %s <ServerDB> <ParamName> <ParamValue>\r\n"
#define RENPARAM_USAGE                      "usage: %s <ServerDB> <OldParamName> <NewParamName>\r\n"
#define PUTPARAM_ERR_TEXT                   "%s : %-18s: %s\n"


/*
 --- DBPARAMS ---
*/
#define DBPARAMS_USAGE                      "usage: dbparams" \
                                            "[ DROP TABLE ]\r\n"
#define DBPARAMS_CMD_DROP_TABLE             "DROP"
#define DBPARAMS_PAR_DROP_TABLE             "TABLE"

/*
 --- DBTIMES ---
*/
#define DBTIMES_USAGE                       "usage: dbtimes" \
                                            "[ DROP TABLE ]\r\n"
#define DBTIMES_CMD_DROP_TABLE              "DROP"
#define DBTIMES_PAR_DROP_TABLE              "TABLE"

/*
 --- XSERVER ---
*/
#define XSERVER_NO_VALID_OPTION   "No valid option! Use -h to get help.\r\n"
#define XSERVER_ACTIVE            "Remote SQL Server is active"


#if defined(_WIN32) && defined(RUN_AS_SERVICE)
 #define XSERVER_USAGE_NT                                                    \
   "usage: serv [-D<l>|-i|-r|-s|-k][-n <servernode>]\n"                      \
   "\n"                                                                      \
   "     -D <l>          -  debug level ( l = 0 .. 9 )                    \n"\
   "                        1       error messages, rejected connections  \n"\
   "                        2       connections                           \n"\
   "                        3       start/stop data transfer              \n"\
   "                        4       start/stop server threads             \n"\
   "                        8       internal control flow                 \n"\
   "     -i              -  install as service                            \n"\
   "     -n <servernode> -  server node name                              \n"\
   "     -r              -  remove service from service table             \n"\
   "     -s              -  start                                         \n"\
   "     -k              -  kill (stop) service                           \n"

 #define XSERVER_USAGE_WIN95                                                 \
   "usage: serv [[-D<l>] |-k]\n"                                             \
   "\n"                                                                      \
   "     -D <l>          -  debug level ( l = 0 .. 9 )                    \n"\
   "                        0       error messages, rejected connections  \n"\
   "                        1       all connections                       \n"\
   "                        2       data transfer                         \n"\
   "                        7       data                                  \n"\
   "                        8       internal control flow                 \n"\
   "     -k              -  kill a running xserver                        \n"
#else
 #define XSERVER_USAGE                                                      \
   "usage: serv [-D<l>] [-k]\n"                                             \
   "\n"                                                                     \
   "     -D <l>          - debug level ( l = 0 .. 9 )                    \n"\
   "                       0       error messages, rejected connections  \n"\
   "                       1       all connections                       \n"\
   "                       2       data transfer                         \n"\
   "                       7       data                                  \n"\
   "                       8       internal control flow                 \n"\
   "     -k              - kill a running xserver                        \n"
#endif

/*
 --- XCONS ---
*/
#if defined(_WIN32)
  #define XCONS_SYS                        "Win"
#elif defined(WIN32)
  #define XCONS_SYS                        "OS/2"
#else
  #define XCONS_SYS                        "UNIX"
#endif

#define XCONS_CMD_SHOW                     "SHOW      "
#define XCONS_CMD_UPDATE                   "UPDATE    "
#define XCONS_CMD_CANCEL                   "CANCEL    "
#define XCONS_CMD_KILL                     "KILL      "
#define XCONS_CMD_DROP                     "DROP      "
#define XCONS_CMD_HELP1                    "HELP      "
#define XCONS_CMD_HELP2                    "?         "
#define XCONS_CMD_TOP                      "TOP       "
#define XCONS_CMD_TIME_MEASURE             "TIME      "
#define XCONS_CMD_RESET_CNT                "RESET     "
#define XCONS_CMD_GETPARAM                 "GETPARAM  "
#define XCONS_CMD_PUTPARAM                 "PUTPARAM  "
#define XCONS_CMD_CRASH                    "CRASH     "
#define XCONS_CMD_DUMP                     "DUMP      "
#define XCONS_CMD_ENABLE                   "ENABLE    "
#define XCONS_CMD_DISABLE                  "DISABLE   "
#define XCONS_CMD_DEBUGLEVEL               "DEBUGLEVEL"
#define XCONS_CMD_PROCESSOR_MASK           "PROCMASK  "
#define XCONS_CMD_UKT_AFFINITY             "AFFINITY  "
#define XCONS_CMD_ERRIOR                   "ERRIOR    "
#define XCONS_CMD_ERRIOW                   "ERRIOW    "
#define XCONS_CMD_DEBUG_BREAK              "DEBUGBREAK"
#define XCONS_CMD_DEBUG_TASK               "DEBUGTASK "
#define XCONS_CMD_SET_LB_CHK               "LD_BAL_CHK"

#define XCONS_MIN_CMD_LEN_SHOW             1
#define XCONS_MIN_CMD_LEN_UPDATE           1
#define XCONS_MIN_CMD_LEN_CANCEL           2
#define XCONS_MIN_CMD_LEN_KILL             1
#define XCONS_MIN_CMD_LEN_DROP             1
#define XCONS_MIN_CMD_LEN_HELP1            1
#define XCONS_MIN_CMD_LEN_HELP2            1
#define XCONS_MIN_CMD_LEN_TIME_MEASURE     2
#define XCONS_MIN_CMD_LEN_RESET_CNT        2
#define XCONS_MIN_CMD_LEN_GETPARAM         8
#define XCONS_MIN_CMD_LEN_PUTPARAM         8
#define XCONS_MIN_CMD_LEN_CRASH            5
#define XCONS_MIN_CMD_LEN_DUMP             4
#define XCONS_MIN_CMD_LEN_ENABLE           1
#define XCONS_MIN_CMD_LEN_DISABLE          2
#define XCONS_MIN_CMD_LEN_DEBUGLEVEL       2
#define XCONS_MIN_CMD_LEN_UKT_AFFINITY     2
#define XCONS_MIN_CMD_LEN_PROCESSOR_MASK   3
#define XCONS_MIN_CMD_LEN_ERRIO            6
#define XCONS_MIN_CMD_LEN_DEBUG_BREAK      10
#define XCONS_MIN_CMD_LEN_DEBUG_TASK       9 
#define XCONS_MIN_CMD_SET_LB_CHK_TASK      2

#define XCONS_CMD_IO                       "IO        "
#define XCONS_CMD_AIO                      "AIO       "
#define XCONS_CMD_STORAGE                  "STORAGE   "
#define XCONS_CMD_TASKS                    "TASKS     "
#define XCONS_CMD_ACTIVE_TASKS             "ACTIVE    "
#define XCONS_CMD_RUNNABLE_TASKS           "RUNNABLE  "
#define XCONS_CMD_VERSIONS                 "VERSIONS  "
#define XCONS_CMD_REGIONS                  "REGIONS   "
#define XCONS_CMD_STATE                    "STATE     "
#define XCONS_CMD_RTE                      "RTE       "
#define XCONS_CMD_ALL                      "ALL       "
#define XCONS_CMD_TABLES                   "TABLES    "
#define XCONS_CMD_TASK_STAT                "T_STAT    "
#define XCONS_CMD_TASK_COUNTS              "T_CNT     "
#define XCONS_CMD_QUEUE_STAT               "T_QUEUE   "
#define XCONS_CMD_TOTAL_COUNTS             "TOTAL     "
#define XCONS_CMD_UKT_SLEEP_STAT           "SLEEP     "
#define XCONS_CMD_SUSPENDS                 "SUSPENDS  "
#define XCONS_CMD_UKT_QUEUES               "QUEUES    "
#define XCONS_CMD_SET                      "SET       "
#define XCONS_CMD_CLEAR                    "CLEAR     "
#define XCONS_CMD_THREAD_TIMES             "THRD_TIMES"
#define XCONS_CMD_AWE                      "AWE       "
#define XCONS_CMD_CONNECTIONS              "CONNECTIONS"
#define XCONS_CMD_MOVE_INFO                "MOVEINFO  "
#define XCONS_CMD_TASK_MOVE                "T_MOVE    "


#define XCONS_MIN_CMD_LEN_IO               2
#define XCONS_MIN_CMD_LEN_AIO              3
#define XCONS_MIN_CMD_LEN_STORAGE          3
#define XCONS_MIN_CMD_LEN_TASKS            1
#define XCONS_MIN_CMD_LEN_ACTIVE_TASKS     2
#define XCONS_MIN_CMD_LEN_RUNNABLE_TASKS   2
#define XCONS_MIN_CMD_LEN_VERSIONS         1
#define XCONS_MIN_CMD_LEN_REGIONS          1
#define XCONS_MIN_CMD_LEN_STATE            3
#define XCONS_MIN_CMD_LEN_RTE              1
#define XCONS_MIN_CMD_LEN_ALL              2
#define XCONS_MIN_CMD_LEN_TABLES           6
#define XCONS_MIN_CMD_LEN_TASK_STAT        3
#define XCONS_MIN_CMD_LEN_TASK_COUNTS      3
#define XCONS_MIN_CMD_LEN_QUEUE_STAT       3
#define XCONS_MIN_CMD_LEN_TOTAL_COUNTS     2
#define XCONS_MIN_CMD_LEN_UKT_SLEEP_STAT   2
#define XCONS_MIN_CMD_LEN_SUSPENDS         2
#define XCONS_MIN_CMD_LEN_UKT_QUEUES       1
#define XCONS_MIN_CMD_LEN_SET              1
#define XCONS_MIN_CMD_LEN_CLEAR            1
#define XCONS_MIN_CMD_LEN_THREAD_TIMES     2
#define XCONS_MIN_CMD_LEN_AWE              3
#define XCONS_MIN_CMD_LEN_CONNECTIONS      2
#define XCONS_MIN_CMD_LEN_TASK_MOVE        3
#define XCONS_MIN_CMD_LEN_MOVE_INFO        4

#if defined(_WIN32)

#define XCONS_USAGE                        "usage: x_cons [<serverdb>] <Command> [<Interval> [<Repeat>]]\r\n" \
                                           "\r\n"                                                            \
                                           "    SERVERDB                         ::   <serverdb>\n"             \
                                           "\r\n"                                                            \
                                           "    Commands: \r\n"                                           \
                                           "      time measurement               ::  TIME   <ENABLE | DISABLE> \r\n"   \
                                           "      cancel the command of task     ::  CANCEL <taskindex>\r\n"   \
                                           "      kill the session of task       ::  KILL <taskindex>  \r\n"   \
                                           "      set UKT affinity               ::  AFFINITY <uktindex>\r\n"  \
                                           "      set processor affinity mask    ::  PROCMASK <procmask> [ DW | SV | US | GC]\r\n"  \
                                           "      show statistics/states         ::  SHOW IO           \r\n"   \
                                           "                                     ::  SHOW AIO       (backup only) \r\n"   \
                                           "                                     ::  SHOW STORAGE      \r\n"   \
                                           "                                     ::  SHOW TASKS        \r\n"   \
                                           "                                     ::  SHOW ACTIVE    [ DW | SV | US | GC]\r\n"   \
                                           "                                     ::  SHOW RUNNABLE  [ DW | SV | US | GC]\r\n"   \
                                           "      show move info (load balancing)::  SHOW MOVEINFO     \r\n"   \
                                           "      show task counts               ::  SHOW T_CNT     [ DW | SV | US | GC]\r\n"   \
                                           "      show task queues               ::  SHOW T_QUEUE      \n"   \
                                           "      show task regions              ::  SHOW T_REG        \n"   \
                                           "      show task statistics           ::  SHOW T_STAT       \n"   \
                                           "      show tasks move info           ::  SHOW T_MOVE       \r\n"   \
                                           "                                         SHOW VERSIONS     \r\n"   \
                                           "                                         SHOW REGIONS      \r\n"   \
                                           "                                         SHOW STATE        \r\n"   \
                                           "                                         SHOW RTE          \r\n"   \
                                           "                                         SHOW QUEUES       \r\n"   \
                                           "      suspend reasons                ::  SHOW SUSPENDS     \r\n"   \
                                           "      UKT sleep statistic            ::  SHOW SLEEP        \r\n"   \
                                           "      Thread time usage              ::  SHOW THRD_TIMES   \r\n"   \
                                           "                                         SHOW ALL          \r\n"

#else

#define XCONS_USAGE                        "usage: x_cons [<serverdb>] <Command> [<Interval> [<Repeat>]]\n" \
                                           "\n"                                                            \
                                           " SERVERDB                    ::  <serverdb>\n"             \
                                           "\n"                                                            \
                                           " Commands: \n"                                           \
                                           "  time measurement           ::  TIME   <ENABLE | DISABLE> \n"   \
                                           "  cancel the command of task ::  CANCEL <taskindex>\n"   \
                                           "  kill the session of task   ::  KILL <taskindex>  \n"   \
                                           "  show statistics/states     ::  SHOW IO           \n"   \
                                           "                                 SHOW AIO       (backup only) \n"   \
                                           "                                 SHOW DEV_IO    (volume only)\n"   \
                                           "                                 SHOW STORAGE      \n"   \
                                           "                                 SHOW TASKS     [ DW | SV | US ]\n"   \
                                           "     alternative for TASKS   ::  SHOW PROCESSES [ DW | SV | US ]\n"   \
                                           "                                 SHOW ACTIVE    [ DW | SV | US ]\n"   \
                                           "                                 SHOW RUNNABLE  [ DW | SV | US ]\n"   \
                                           "  show task counts           ::  SHOW T_CNT     [ DW | SV | US | T<taskindex>]\n"   \
                                           "  show task queues           ::  SHOW T_QUEUE      \n"   \
                                           "  show task regions          ::  SHOW T_REG        \n"   \
                                           "  show task statistics       ::  SHOW T_STAT       \n"   \
                                           "                                 SHOW VERSIONS     \n"   \
                                           "                                 SHOW REGIONS      \n"   \
                                           "                                 SHOW STATE        \n"   \
                                           "                                 SHOW RTE          \n"   \
                                           "                                 SHOW QUEUES       \n"   \
                                           "  suspend reasons            ::  SHOW SUSPENDS     \n"   \
                                           "  UKT sleep statistic        ::  SHOW SLEEP        \n"   \
                                           "  Thread stacks              ::  SHOW TSTACK       \n"   \
                                           "                                 SHOW ALL          \n"

#endif

#define XCONS_WRONG_VERS_KERNEL            "       Kernel  RTE Version :  %s\n"
#define XCONS_WRONG_VERS_XCONS             "       x_cons  RTE Version :  %s\n"

#define XCONS_SHOW_VERS_KERNEL             "\nKernel Version  '%s'\n"
#define XCONS_SHOW_VERS_RTE                "RTE    Version  '%s'\n"
#define XCONS_SERVERDB                     "\nSERVERDB: %s\n"
#define XCONS_GATEWAY                      "\nGateway: %s\n"

/* CR 1103476 */
/* CR 1115172 */
#define XCONS_SHOW_TASK_MOVE_HEADER        "\n"                                                            \
                                           "%-3.3s  %3.3s %-7.7s  %-14.14s  %11.11s %11.11s %-10.10s \n"   \
                                           "%-3.3s  %3.3s %-7.7s  %-14.14s  %11.11s %11.11s %-10.10s \n",  \
                                           "ID", "UKT", "TASK", "Current", "Max.",     "Sum",      "Move", \
                                	         "",   "",    "type", "state",   "Runnable", "Runnable", "lock"

#define XCONS_SHOW_TASK_MOVE_INFO          "T%-3d %3d %-7.7s %c%-14.14s%c %4I64d.%06I64d %4I64d.%06I64d %-10.10s\n"
#define XCONS_SHOW_TASK_HEADER             "\n"                                                                   \
                                           "%-3.3s  %3.3s %4.4s   %-7.7s %7.7s %-14.14s %8.8s %-8.8s   %-4.4s \n" \
                                           "%-3.3s  %3.3s %4.4s   %-7.7s %7.7s %-14.14s %8.8s %-8.8s   %-4.4s \n", \
                                           "ID", "UKT", XCONS_SYS, "TASK", "APPL", "Current", "Timeout",  "Region",   "Wait", \
                                	         "",   "",    "tid",     "type", "pid",  "state",   "priority", "cnt try", "item"

#define XCONS_SHOW_TASK_INFO               "T%-3d %3d %6s %-7.7s %7s%c%-14.14s%c%c%c  %4.4s %-4d %3s %6s %-ld(%c)\n"
#define XCONS_SHOW_SERVER_UNUSED           "\n%5d Server Task(s) is/are unused"
#define XCONS_SHOW_TASK_CONN_INFO          "\n%5d Task(s) is/are in State %s\n"
#define XCONS_SHOW_TASK_NO_TIMEOUT         "no"

#define XCONS_SHOW_IO_HEADER               "\n"                                         \
                                           "%-44.44s %5.5s %8.8s %8.8s %8.8s\n"               \
                                           "%-44.44s %5.5s %8.8s %8.8s %8.8s\n",              \
                                           "Volume", "Devs.", "Read(s)",  "Write(s)", "Total",\
                                           "Name",     "No.",   "Count",    "Count",  "Count"

#define XCONS_SHOW_IO_INFO                 "%-44.44s %5d %8d %8d %8d\n"
#define XCONS_SHOW_IO_INFO_TOTAL_STR       "total I/O:"
#define XCONS_SHOW_IO_INFO_TOTAL           "%44.44s       %8d %8d %8d \n"

#define XCONS_SHOW_REGION_HEADER           "\n"                                                                           \
                                           "%5s  %-8.8s  %-5.5s %10.10s %10.10s %10.10s %8.8s %4.4s   %s\n",                       \
                                           "Index", "Region", "Owner", "Get-Cnt", "Tas-Cnt","Coll.", "Waits", "Excl","Coll % "
#define XCONS_SHOW_REGION_INFO             "%5d  %-8.8s  %-5.5s %10u %10u %10u %8u %4d %6.2f %%\n"
#define XCONS_SHOW_REGION_WAIT_HD          "       waiting:"
#define XCONS_SHOW_REGION_WAIT_I           "  T%-4d"
#define XCONS_SHOW_REGION_WAIT_ILF         "\n                 T%-4d"

#define XCONS_SHOW_STOR_DYN_POOL           "\n" \
                                           "Type                      allocated         used       unused\n"\
                                           "Shared dyn pool         %8d KB  %8d KB  %8d KB\n"
#define XCONS_SHOW_STOR_DYN_DATA           "Shared dyn data         %8d KB  %8d KB  %8d KB\n"
#define XCONS_SHOW_STOR_RTE_ALLOC_POOL_MX  "\n" \
                                           "Type                      allocated          max       cnt\n"\
                                           "RTE allocation pool     %8d KB  %8d MB  %8d\n"
#define XCONS_SHOW_STOR_RTE_ALLOC_POOL     "\n" \
                                           "Type                      allocated       cnt\n"\
                                           "RTE allocation pool     %8d KB  %8d\n"

#define XCONS_SHOW_STOR_ALLOC_STCK         "\n"    \
                                           "Type                      allocated\n"\
                                           "Task stacks             %8d KB\n"
#define XCONS_SHOW_STOR_KGS                "Kernel global section   %8d KB\n"
#define XCONS_SHOW_STOR_KSS                "Kernel shared section   %8d KB\n"

#define XCONS_SHOW_STOR_TSK_HEADER         "\n"                                       \
                                           "%-5.5s %-7.7s  %10.10s  %10.10s\n"        \
                                           "%-5.5s %-7.7s  %10.10s  %10.10s \n",      \
                                           "Task", "Type", "Stack used", "Stack free",\
                                           "ID",   "",     "KB",          "KB"
#define XCONS_SHOW_STOR_TSK_INFO           "T%-4d %-7.7s  %10.1f  %10.1f\n"

#define XCONS_SHOW_RTE_KT_HEADER           "\nKernel Threads:\n"            \
                                           " %-11.11s %10.10s  %-12.12s\n"  \
                                           " %-11.11s %10.10s  %-12.12s\n", \
                                           "Thread", XCONS_SYS,"State",     \
                                           "Name",  "Tid",  ""
#define XCONS_SHOW_RTE_KT_INFO             " %-11.11s %10.10s  %-12.12s\n"
#define XCONS_SHOW_RTE_KT_COORD            "COORDINATOR"
#define XCONS_SHOW_RTE_KT_TIMER            "TIMER"
#define XCONS_SHOW_RTE_KT_CLOCK            "CLOCK"
#define XCONS_SHOW_RTE_KT_DEV0             "DEV0"
#define XCONS_SHOW_RTE_KT_DCOM0            "DCOM0"
#define XCONS_SHOW_RTE_KT_ASYNC0           "ASYNC0"
#define XCONS_SHOW_RTE_KT_CONSOLE          "CONSOLE"
#define XCONS_SHOW_RTE_KT_REQUESTOR        "REQUESTOR"
#define XCONS_SHOW_RTE_DEV_WORKER_HEADER   "\nIO Worker Threads:\n"                                             \
                                           " %-12.12s  %10.10s  %-11.11s  %-13.13s\n"             \
                                           " %-12.12s  %10.10s  %-11.11s  %-13.13s\n",            \
                                           "Thread", XCONS_SYS, "State", "IO", \
                                           "Name",   "Tid",     "",      "Counter"
#define XCONS_SHOW_RTE_DEV_WORKER_INFO     " IO-WORKER%-3d  %10.10s  %-11.11s  %-10d\n"
#define XCONS_SHOW_RTE_CONCURR_DEV_WORKER  "\n %d IO Worker threads executing concurrently\n"

#define XCONS_SHOW_RTE_UKT_HEADER          "\nUser Kernel Threads:\n"                                             \
                                           " %-6.6s  %10.10s  %-11.11s  %13.13s  %6.6s  %5.5s   %s\n"             \
                                           " %-6.6s  %10.10s  %-11.11s  %13.13s  %6.6s  %5.5s   %s\n",            \
                                           "Thread", XCONS_SYS, "State", "Dispatch", "Active" , "Total" , "Task", \
                                           "Name",   "Tid",     "",      "Counter",  "Tasks"  , "Tasks" , "Cluster"
#define XCONS_SHOW_RTE_UKT_INFO            " UKT%-3d  %10.10s  %-11.11s %14.0f  %6d  %5d   %s \n"
#define XCONS_SHOW_RTE_UKT_AFFIN_HEADER    "\nUser Kernel Threads affinity:\n"
#define XCONS_SHOW_RTE_UKT_AFFIN_INFO      " UKT%-3d <-> CPU:%3d\n"

#define XCONS_SHOW_RTE_IO_HEADER           "\nI/O via UKTs and I/O Threads:\n"                                    \
                                           " %-6.6s  %10.10s  %-20.20s %5.5s  %8.8s  %8.8s    %5.5s\n"            \
                                           " %-6.6s  %10.10s  %-20.20s %5.5s  %8.8s  %8.8s  %4.4s %4.4s\n",       \
                                           "Thread",XCONS_SYS,"Volume", "Devs.", "Read",  "Write" , "Queue",    \
                                           "Name",  "Tid",    "Name",     "No.",   "Count", "Count" , "Len.", "Max."
#define XCONS_SHOW_RTE_IO_UKT_INFO         " UKT%-3d  %10.10s  %-20.20s %5d  %8d  %8d   --  (--)\n"
#define XCONS_SHOW_RTE_IO_DEV_INFO         " I/O%-3d  %10.10s  %-20.20s %5d  %8d  %8d   %2d  (%2d)\n"

#define XCONS_SHOW_ASYNCIO_HEADER          "\nI/O via asynchronous I/O Threads:\n"                                 \
                                           " %-20.20s %10.10s  %3.3s %6.6s %6.6s %8.8s %8.8s   %5.5s\n"            \
                                           " %-20.20s %10.10s  %3.3s %6.6s %6.6s %8.8s %8.8s %4.4s %4.4s\n",       \
                                           "Device", XCONS_SYS, "R/W", "Max   ",  "Block","Read",  "Write" , "Queue",    \
                                           "Name",   "Tid",     "",    "Blocks",  "Size", "Count", "Count" , "Len.", "Max."
#define XCONS_SHOW_ASYNCIO_INFO            " %-20.20s %10.10s  %3s %6d %6d %8d %8d  %2d  (%2d)\n"

#define XCONS_SHOW_AWE_INFO                "\nAWE infos:\n"                                   \
                                           " Number of physical pages                %8d\n" \
                                           " Number of free physical pages           %8d\n" \
                                           " Number of map area pages (cache)        %8d\n" \
                                           " Number of free map area pages (cache)   %8d\n" \
                                           " Number of map calls                     %8d\n" \
                                           " Number of unmap calls                   %8d\n"


#if defined (_WIN64)
# define XCONS_SHOW_AWE_MAP_PAGE_HEADER    "\nAWE map page infos:\n\n"               \
                                           " %-18.18s   %-10.10s %-4.4s %-20.20s\n",\
                                           "MapAddress", "PhysPageNo", "Res.",   "Map Count"
# define XCONS_SHOW_AWE_MAP_PAGE_INFO      " 0x%016I64X - %-10d YES  %-d\n"
# define XCONS_SHOW_AWE_UNMAP_PAGE_INFO    " 0x%016I64X - NO         %-4.4s %-d\n"
#else
# define XCONS_SHOW_AWE_MAP_PAGE_HEADER    "\nAWE map page infos:\n\n"               \
                                           " %-10.10s   %-10.10s %-4.4s %-20.20s\n",\
                                           "MapAddress", "PhysPageNo", "Res.",   "Map Count"
# define XCONS_SHOW_AWE_MAP_PAGE_INFO      " 0x%08X - %-10d YES  %-d\n"
# define XCONS_SHOW_AWE_UNMAP_PAGE_INFO    " 0x%08X - NO         %-4.4s %-d\n"
#endif
#define XCONS_SHOW_AWE_PHYS_PAGE_HEADER    "\nAWE physical page infos:\n\n"            \
                                           " %-10.10s   %-4.4s %-20.20s\n",            \
                                           "PageNo", "Res.",   "Map Count"

#define XCONS_SHOW_AWE_PHYS_PAGE_INFO      " %-10d - %-4.4s %-d\n"

#define XCONS_FORMAT_SHOW_CONNECTION_TASK                     " Task T%d UKT%d  Task type: %s\n"
#define XCONS_FORMAT_SHOW_CONNECTION_STATE_NOT_CONNECTED      "    state '%s' connect time not set\n"
#define XCONS_FORMAT_SHOW_CONNECTION_STATE_TIME               "    state '%s' connect time %s"
#define XCONS_FORMAT_SHOW_CONNECTION_STATE_DUMPTIME           "    state '%s' connect time not convertable[%ld]\n"
#define XCONS_FORMAT_SHOW_CONNECTION_LOCAL                    "    LOCAL CONNECTION  peer pid %d\n"
#define XCONS_FORMAT_SHOW_CONNECTION_REMOTE	                  "    REMOTE CONNECTION peer pid %d\n" \
		                                                      "                      remote pid %s remote node %s\n"
#define XCONS_FORMAT_SHOW_CONNECTION_NO_TASK_CONNECTED		  "\n no Task connected\n"

#define XCONS_SHOW_MOVE_INFO               "\nBalancing Info:\n"          \
                                           " Load balancing      -  %s\n" \
                                           " Interval time       -  %d (s)\n" \
                                           " Last interval start -  %s (ddd:hh:mm:ss)\n"  \
                                           " Move intervals      -  %d\n"  \
                                           " Minimum diff.       -  %d (%%)\n"  \
                                           " Value equality      -  %d (%%)\n"

#define XCONS_SHOW_LAST_TASK_MOVES_HEADER  "\n\nLast task moves:\n"                                     \
                                           " %-14.14s  %-5.5s %11.11s   %-3.3s %11.11s   %-3.3s %11.11s\n"  \
                                           " %14.14s  %-5.5s %11.11s   %-3.3s %11.11s   %-3.3s %11.11s\n", \
                                           "Task moved",      "Task",    "Max.",     "Src", "Max.",     "Dst",  "Max.", \
                                           "(ddd:hh:mm:ss)",   "",        "Runnable", "UKT",    "Runnable", "UKT",    "Runnable"
#define XCONS_SHOW_LAST_TASK_MOVES_INFO    " %13.13s   T%-4d %4I64d.%06I64d   %-3d %4I64d.%06I64d   %-3d %4I64d.%06I64d\n"

#define XCONS_SHOW_MOVE_UKT_HEADER         "\n\nUser Kernel Threads:\n"                                            \
                                           " %-3.3s  %-7.7s  %-14.14s  %11.11s  %11.11s  %11.11s\n"             \
                                           " %-3.3s  %-7.7s  %14.14s  %11.11s  %11.11s  %11.11s\n",            \
                                           "UKT",    "Movable",   "Interval start",  "Max.",      "Sum" ,       "UKT", \
                                           "",       "Tasks",     "(ddd:hh:mm:ss)",   "Runnable",  "Runnable"  , "Idle Time"
#define XCONS_SHOW_MOVE_UKT_INFO           " %3d  %-7d  %13.13s   %4I64d.%06I64d  %4I64d.%06I64d  %4I64d.%06I64d \n"

#define XCONS_INFO_LINE                    "-------------------------------------------------------------------------------"

#define XCONS_SHOW_QUEUE_STAT_HEADER       "\nTask Queue Statistics:\n"                           \
                                           "\n%s\n%5s      %9s     %9s    %9s\n%s\n",             \
                                           XCONS_INFO_LINE, "Task ", "UToU", "UKT", "COM", XCONS_INFO_LINE

#define XCONS_SHOW_QUEUE_STAT_INFO         "T%-4d     %10u    %10u   %10u  \n"

#define TASK_COUNT_FORMAT_TWO_COUNTS       "%16s %-10lu   %40s %-lu\n"
#define TASK_COUNT_FORMAT_THREE_COUNTS     "%16s %-8d %17s %-8d %17s %-u\n"
#define FORMAT_CNT_AND_TIME                "%16s %-10u   %40s %-3.4f\n"
#define FORMAT_TWO_COUNTS_AND_TIME         "%16s %-8d %17s %-8d %17s %-3.4f\n"
#define FORMAT_COUNT_AND_TWO_TIMES         "%16s %-8d %17s %-8.4f %17s %-8.4f\n"

#define XCONS_SHOW_TASK_COUNT_HEADER \
                                                   "\n%19.19s T%-4d %12.12s ( pid = %-10.10s ) %19.19s\n"
#define XCONS_SHOW_TASK_COUNT_TRAILER              "%s\n",XCONS_INFO_LINE

#define XCONS_SHOW_TASK_COUNT_REM_NODE_AND_PID     "%16s %-30.30s   %20s %s\n"
#define XCONS_SHOW_TASK_COUNT_REM_NODE             "remote_node   :"
#define XCONS_SHOW_TASK_COUNT_REM_PID              "remote_pid    :"

/* PTS 1106969 */
#define XCONS_SHOW_TASK_COUNT_OMS_PROCEDURE        "%16s %-62.62s\n"
#define XCONS_SHOW_TASK_COUNT_OMS_PROCEDURE_NAME   "COM procedure :"

#define XCONS_SHOW_TASK_COUNT_OMS_CALLBACK_STAT    "%16s %-30.30s   %20s %d\n"
#define XCONS_SHOW_TASK_COUNT_OMS_CALLBACK_METHOD  "OMS callback  :"
#define XCONS_SHOW_TASK_COUNT_OMS_CALLBACK_CNT     "callback_cnt  :"

#define XCONS_SHOW_TASK_COUNT_DISP_COMMAND_CNT     TASK_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TASK_COUNT_KT_DISP_CALL_CNT     "dispatcher_cnt:"
#define XCONS_SHOW_TASK_COUNT_KT_COMMAND_CNT       "command_cnt   :"

#define XCONS_SHOW_TASK_COUNT_EXCL_SELF_SUSP_CNT   TASK_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TASK_COUNT_KT_EXCLUSIVE_CNT     "exclusive_cnt :"
#define XCONS_SHOW_TASK_COUNT_KT_SELF_SUSP_CNT     "self_susp_cnt :"

#define XCONS_SHOW_TASK_COUNT_WAIT_FOR_DEV         " I/O on volume: %s\n"

#define XCONS_SHOW_TASK_COUNT_PG_PER_IO            "%16s %-10u   %40s %-4.1f\n"
#define XCONS_SHOW_TASK_COUNT_IO_CNT               TASK_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TASK_COUNT_ASYN_CNT_PG          TASK_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TASK_COUNT_STATE_CNT_TIME       FORMAT_COUNT_AND_TWO_TIMES
#define XCONS_SHOW_TASK_COUNT_IO_CNT_PG_TIME       FORMAT_TWO_COUNTS_AND_TIME
#define XCONS_SHOW_TASK_COUNT_SELF_R_CNT_AVG_TIME  FORMAT_CNT_AND_TIME
#define XCONS_SHOW_TASK_COUNT_SELF_W_CNT_AVG_TIME  FORMAT_CNT_AND_TIME
#define XCONS_SHOW_TASK_COUNT_DEV_R_CNT_AVG_TIME   FORMAT_CNT_AND_TIME
#define XCONS_SHOW_TASK_COUNT_DEV_W_CNT_AVG_TIME   FORMAT_CNT_AND_TIME
#define XCONS_SHOW_TASK_COUNT_ASYN_R_CNT_AVG_TIME  FORMAT_CNT_AND_TIME
#define XCONS_SHOW_TASK_COUNT_ASYN_W_CNT_AVG_TIME  FORMAT_CNT_AND_TIME

#define XCONS_SHOW_TASK_COUNT_KT_PG_PER_IO         "pages_per_io  :"
#define XCONS_SHOW_TASK_COUNT_KT_SELF_R_CNT        "self_read_io  :"
#define XCONS_SHOW_TASK_COUNT_KT_SELF_R_PG         "self_read_pg  :"
#define XCONS_SHOW_TASK_COUNT_KT_SELF_W_CNT        "self_write_io :"
#define XCONS_SHOW_TASK_COUNT_KT_SELF_W_PG         "self_write_pg :"
#define XCONS_SHOW_TASK_COUNT_KT_SELF_R_AVG_TIME   "avg_self_rd_tm:"
#define XCONS_SHOW_TASK_COUNT_KT_SELF_W_AVG_TIME   "avg_self_wr_tm:"

#define XCONS_SHOW_TASK_COUNT_KT_DEV_R_CNT         "dev_read_io   :"
#define XCONS_SHOW_TASK_COUNT_KT_DEV_R_PG          "dev_read_pg   :"
#define XCONS_SHOW_TASK_COUNT_KT_DEV_W_CNT         "dev_write_io  :"
#define XCONS_SHOW_TASK_COUNT_KT_DEV_W_PG          "dev_write_pg  :"
#define XCONS_SHOW_TASK_COUNT_KT_DEV_R_REL_TIME    "rel_dev_rd_tm :"
#define XCONS_SHOW_TASK_COUNT_KT_DEV_R_ABS_TIME    "abs_dev_rd_tm :"
#define XCONS_SHOW_TASK_COUNT_KT_DEV_W_REL_TIME    "rel_dev_wr_tm :"
#define XCONS_SHOW_TASK_COUNT_KT_DEV_W_ABS_TIME    "abs_dev_wr_tm :"

#define XCONS_SHOW_TASK_COUNT_KT_ASYN_R_CNT        "asyn_read_io  :"
#define XCONS_SHOW_TASK_COUNT_KT_ASYN_R_PG         "asyn_read_pg  :"
#define XCONS_SHOW_TASK_COUNT_KT_ASYN_W_CNT        "asyn_write_io :"
#define XCONS_SHOW_TASK_COUNT_KT_ASYN_W_PG         "asyn_write_pg :"
#define XCONS_SHOW_TASK_COUNT_KT_ASYN_R_REL_TIME   "rel_asyn_rd_tm:"
#define XCONS_SHOW_TASK_COUNT_KT_ASYN_R_ABS_TIME   "abs_asyn_rd_tm:"
#define XCONS_SHOW_TASK_COUNT_KT_ASYN_W_REL_TIME   "rel_asyn_wr_tm:"
#define XCONS_SHOW_TASK_COUNT_KT_ASYN_W_ABS_TIME   "abs_asyn_wr_tm:"

#define XCONS_SHOW_TASK_COUNT_RESUME_CNT " SuspendLevel %d resume count %-2d total %-10d History ["
#define XCONS_SHOW_TASK_COUNT_RESUME_HISTORY " T%d"
#define XCONS_SHOW_TASK_COUNT_RESUME_END_HISTORY " ]\n"

#define XCONS_SHOW_TASK_COUNT_WAIT_SLEEP_SUSP_CNT  \
                                                   TASK_COUNT_FORMAT_THREE_COUNTS
#define XCONS_SHOW_TASK_COUNT_VWAIT_CNT_AVG_TIME   FORMAT_CNT_AND_TIME
#define XCONS_SHOW_TASK_COUNT_KT_STATE_VWAIT_CNT   "state_vwait   :"
#define XCONS_SHOW_TASK_COUNT_KT_VWAIT_REL_TIME    "rel_wait_time :"
#define XCONS_SHOW_TASK_COUNT_KT_VWAIT_ABS_TIME    "abs_wait_time :"
#define XCONS_SHOW_TASK_COUNT_VSLEEP_CNT_AVG_TIME  FORMAT_CNT_AND_TIME
#define XCONS_SHOW_TASK_COUNT_KT_STATE_VSLEEP_CNT  "state_vsleep  :"
#define XCONS_SHOW_TASK_COUNT_KT_VSLEEP_REL_TIME   "rel_sleep_time:"
#define XCONS_SHOW_TASK_COUNT_KT_VSLEEP_ABS_TIME   "abs_sleep_time:"
#define XCONS_SHOW_TASK_COUNT_VSUSP_CNT_AVG_TIME   FORMAT_CNT_AND_TIME
#define XCONS_SHOW_TASK_COUNT_KT_STATE_VSUSP_CNT   "state_vsusp   :"
#define XCONS_SHOW_TASK_COUNT_KT_VSUSP_REL_TIME    "rel_susp_time :"
#define XCONS_SHOW_TASK_COUNT_KT_VSUSP_ABS_TIME    "abs_susp_time :"
#define XCONS_SHOW_TASK_COUNT_RCV_RPL_INFO         FORMAT_TWO_COUNTS_AND_TIME
#define XCONS_SHOW_TASK_COUNT_KT_RCV_RPL_CNT       "rcv_rpl_count :"
#define XCONS_SHOW_TASK_COUNT_KT_RCV_RPL_TO_LONG   "rcv_rpl_long  :"
#define XCONS_SHOW_TASK_COUNT_KT_RCV_RPL_AVG_TIME  "avg_rcv_rpl_t :"
#define XCONS_SHOW_TASK_COUNT_RPL_RCV_INFO         FORMAT_CNT_AND_TIME
#define XCONS_SHOW_TASK_COUNT_KT_RPL_RCV_CNT       "rpl_rcv_count :"
#define XCONS_SHOW_TASK_COUNT_KT_RPL_RCV_REL_TIME  "rel_rpl_rcv_t :"
#define XCONS_SHOW_TASK_COUNT_KT_RPL_RCV_ABS_TIME  "abs_rpl_rcv_t :"

#define XCONS_SHOW_TASK_COUNT_PRIO_TOT_OTH         TASK_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TASK_COUNT_KT_PRIO_CNT          "prio_total_cnt:"
#define XCONS_SHOW_TASK_COUNT_KT_PRIO_FROM_OTHER   "from_oth_ukt:  "

#define XCONS_SHOW_TOTAL_STAT_HEADER                \
                                                   "\nTotal counter statistics:\n=========================\n\n"

#define TOTAL_COUNT_FORMAT_ONE_COUNT               "%16s %-10u\n"
#define TOTAL_COUNT_FORMAT_TWO_COUNTS              "%16s %-10u   %30s %-u\n"
#define TOTAL_COUNT_FORMAT_TWO_DBL_COUNTS          "%16s %-10.0f   %30s %-.0f\n"

#define XCONS_SHOW_TOTAL_DISP_COMMAND_CNT          TOTAL_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TOTAL_KT_DISP_CALL_CNT          "total_disp_calls    :"
#define XCONS_SHOW_TOTAL_KT_COMMAND_CNT            "total_commands      :"

#define XCONS_SHOW_TOTAL_KT_STATE_VWAIT_VSUSP_CNT  TOTAL_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TOTAL_KT_STATE_VWAIT_CNT        "total_state_vwait   :"
#define XCONS_SHOW_TOTAL_KT_STATE_VSUSP_CNT        "total_state_vsuspend:"

#define XCONS_SHOW_TOTAL_KT_STATE_VSLEEP           TOTAL_COUNT_FORMAT_ONE_COUNT
#define XCONS_SHOW_TOTAL_KT_STATE_VSLEEP_CNT       "total_state_vsleep  :"

#define XCONS_SHOW_TOTAL_SELFIO_DEVIO_CNT          TOTAL_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TOTAL_KT_SELFIO_CNT             "total_self_io       :"
#define XCONS_SHOW_TOTAL_KT_DEVIO_CNT              "total_dev_io        :"

#define XCONS_SHOW_TOTAL_SELFIO_DEVIO_CNT          TOTAL_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TOTAL_KT_SELFIO_CNT             "total_self_io       :"
#define XCONS_SHOW_TOTAL_KT_DEVIO_CNT              "total_dev_io        :"

#define XCONS_SHOW_TOTAL_REG_ACC_BEGEXCL_COLL      TOTAL_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TOTAL_KT_REG_ACCESS             "total_region_access :"
#define XCONS_SHOW_TOTAL_KT_BEGEXCL_COLL           "begexcl_tas_coll    :"

#define XCONS_SHOW_TOTAL_REG_COLL_ENDEXCL_COLL     TOTAL_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TOTAL_KT_REG_COLL               "total_region_coll   :"
#define XCONS_SHOW_TOTAL_KT_ENDEXCL_COLL           "endexcl_tas_coll    :"

#define XCONS_SHOW_TOTAL_KT_REG_WAIT               TOTAL_COUNT_FORMAT_ONE_COUNT
#define XCONS_SHOW_TOTAL_KT_REG_WAIT_CNT           "total_region_wait   :"

#define XCONS_SHOW_TOTAL_PRIO_CNT_AND_OTHER        TOTAL_COUNT_FORMAT_TWO_COUNTS
#define XCONS_SHOW_TOTAL_KT_PRIO_CNT               "total_prio_calls    :"
#define XCONS_SHOW_TOTAL_KT_PRIO_OTHER             "total_prio_other    :"

#define XCONS_SHOW_UKT_SLEEP_STAT_HEADER \
                                                   "\n%s\n            Idle   dd:hh:mm:ss             I/O Wait   dd:hh:mm:ss         RQlen\n%s\n", \
                                                   XCONS_INFO_LINE , \
                                                   XCONS_INFO_LINE

#define XCONS_SHOW_UKT_SLEEP_STAT_INFO             "UKT%-2d %10d  %s  %6.2f%%  %10d  %s  %6.2f%% %4d\n"

#define XCONS_SHOW_SUSPEND_HEADER \
                                                   "\nList of suspend reasons:\n%s\n","========================"
#define XCONS_SHOW_SUSPEND_TOTAL                   "\ntotal suspends: %7d\n\n"
#define XCONS_SHOW_SUSPEND_CNT_PERCENT             "%s : %7d ( %6.2f%% ) %s\n"
 
#define XCONS_SHOW_QUEUES_HEAD                     "UKT%d tid 0x%x \n"
#define XCONS_SHOW_UKT_QUEUE_HEAD                  "    UKTQueue: \n"
#define XCONS_SHOW_UKT_QUEUE_ELEMENT               "          task T%d \n"
#define XCONS_SHOW_UKT_QUEUE_PRIO_ELEMENT          "          task T%d, prio %d\n"
#define XCONS_SHOW_COM_QUEUE_HEAD                  "    COMQueue: \n"
#define XCONS_SHOW_COM_QUEUE_ELEMENT               "          task T%d \n"
#define XCONS_SHOW_COM_QUEUE_PRIO_ELEMENT          "          task T%d, prio %d\n"
#define XCONS_SHOW_RUN_QUEUE_HEAD                  "    RUNQueue: \n"
#define XCONS_SHOW_RUN_QUEUE_ELEMENT               "          task T%-3d (%s), dyn_prio %d, %s\n"
#define XCONS_SHOW_RUN_QUEUE_ELEMENT_REQ           "          task T%-3d (%s), dyn_prio %d, %s %s\n"

#define XCONS_SHOW_THREAD_TIMES_HEADER     "\n%s\n %-6.6s%8.8s  %-28.28s %-15.15s   %-15.15s\n"        \
                                               " %-6.6s%8.8s  %-28.28s %-15.15s   %-15.15s\n%s\n",     \
                                           XCONS_INFO_LINE, "Thread", "Win", "Task",   "   System", "   User" , \
                                           "Name",   "Tid", "Cluster"," d hh:mm:ss:ms"," d hh:mm:ss:ms", XCONS_INFO_LINE

#define XCONS_FORMAT_SHOW_THREAD_TIMES     " %-6.6s%8d  %-28.28s %2.2s %2d:%02d:%02d:%03d   %2.2s %2d:%02d:%02d:%03d\n"
#define XCONS_FORMAT_SHOW_THREAD_TOT_TIMES "%s\n %-44.44s %2.2s %2d:%02d:%02d:%03d   %2.2s %2d:%02d:%02d:%03d\n", \
                                           XCONS_INFO_LINE,"total time used by shown threads: " 

/* --- SERVERDB states */

#define XCONS_SHOW_STATE_INFO              "\nThe SERVERDB state is %s\n"
#define XCONS_SHOW_STATE_INFO_TERM_TIMEOUT "\nThe SERVERDB state is %s  - Termination timeout: %d\n"
#define XCONS_SHOW_STATE_STARTING          "STARTING      "
#define XCONS_SHOW_STATE_COLD              "ADMIN         "
#define XCONS_SHOW_STATE_WARM              "ONLINE        "
#define XCONS_SHOW_STATE_SHTDWN            "SHUTDOWN      "
#define XCONS_SHOW_STATE_SHTDWNREI         "SHUTDOWNREINIT"
#define XCONS_SHOW_STATE_SHTDWNKIL         "SHUTDOWNKILL  "
#define XCONS_SHOW_STATE_STOP              "STOP          "
#define XCONS_SHOW_STATE_KILL              "KILL          "
#define XCONS_SHOW_STATE_ABORT             "ABORT         "
#define XCONS_SHOW_STATE_UNKNOWN           "UNKNOWN       "


/* --- task states */

#define XCONS_TSK_STATE_INITIALIZE         "Not Connected "
#define XCONS_TSK_STATE_INACTIVE           "Inactive      "
#define XCONS_TSK_STATE_MARKED_FOR_STRT    "Mark for Start"
#define XCONS_TSK_STATE_RUNNING            "Running       "
#define XCONS_TSK_STATE_CONNECTWAIT        "Connect wait  "
#define XCONS_TSK_STATE_VDEVSIZE           "Vdevsize      "
#define XCONS_TSK_STATE_VATTACH            "Vattach       "
#define XCONS_TSK_STATE_V2INFO             "V2info        "
#define XCONS_TSK_STATE_VBLOCKIO           "Vblockio      "
#define XCONS_TSK_STATE_V2BLOCKIO          "V2blockio     "
#define XCONS_TSK_STATE_VDETACH            "Vdetach       "
#define XCONS_TSK_STATE_VFOPEN             "Vfopen        "
#define XCONS_TSK_STATE_VFREAD             "Vfread        "
#define XCONS_TSK_STATE_VFWRITE            "Vfwrite       "
#define XCONS_TSK_STATE_VFCLOSE            "Vfclose       "
#define XCONS_TSK_STATE_VRELEASE           "Vrelease      "
#define XCONS_TSK_STATE_VSHUTDOWN          "Vshutdown     "
#define XCONS_TSK_STATE_VRECEIVE           "Command wait  "
#define XCONS_TSK_STATE_VREPLY             "Command reply "
#define XCONS_TSK_STATE_VNRESTART          "Vnrestart     "
#define XCONS_TSK_STATE_VNSHUTDOWN         "Vnshutdown    "
#define XCONS_TSK_STATE_VNOPEN             "Vnopen        "
#define XCONS_TSK_STATE_VNREQUEST          "Vnrequest     "
#define XCONS_TSK_STATE_VNRECEIVE          "Net Cmd wait  "
#define XCONS_TSK_STATE_VNCLOSE            "Vnclose       "
#define XCONS_TSK_STATE_VBEGEXCL           "Vbegexcl      "
#define XCONS_TSK_STATE_VENDEXCL           "Vendexcl      "
#define XCONS_TSK_STATE_VSUSPEND           "Vsuspend      "
#define XCONS_TSK_STATE_VWAIT              "Vwait         "
#define XCONS_TSK_STATE_VSLEEP             "Vsleep        "
#define XCONS_TSK_STATE_VDIAGINIT          "Diaginit      "
#define XCONS_TSK_STATE_VOPMSG             "Vopmsg        "
#define XCONS_TSK_STATE_LOCKED             "Locked        "
#define XCONS_TSK_STATE_TERMINATED         "Terminated    "
#define XCONS_TSK_STATE_RUNNABLE           "Runnable      "
#define XCONS_TSK_STATE_ASYNOPEN           "AsynOpen      "
#define XCONS_TSK_STATE_ASYNCLOSE          "AsynClose     "
#define XCONS_TSK_STATE_ASYNIO             "AsynIO        "
#define XCONS_TSK_STATE_ASYNWAIT_READ      "AsynWaitRead  "
#define XCONS_TSK_STATE_ASYNWAIT_WRITE     "AsynWaitWrite "
#define XCONS_TSK_STATE_VCREATE            "Vcreate       "
#define XCONS_TSK_STATE_VKILL              "Vkill         "
#define XCONS_TSK_STATE_IO_READ            "IO Wait (R)   "
#define XCONS_TSK_STATE_IO_WRITE           "IO Wait (W)   "
#define XCONS_TSK_STATE_IO2_READ           "IO2 Wait (R)  "
#define XCONS_TSK_STATE_IO2_WRITE          "IO2 Wait (W)  "
#define XCONS_TSK_STATE_VACKNOWLEDGE       "Vacknowledge  "
#define XCONS_TSK_STATE_VNCLEAR            "Vnclear       "
#define XCONS_TSK_STATE_DCOM_OBJ_CALLED    "DcomObjCalled "
#define XCONS_TSK_STATE_RESCHEDULE_MSEC    "RescheduleMsec"
#define XCONS_TSK_STATE_ASYNCNTL           "Asyncntl      "
#define XCONS_TSK_STATE_INSERT_EVENT       "InsertEvent   "
#define XCONS_TSK_STATE_WAIT_FOR_EVENT     "WaitForEvent  "
#define XCONS_TSK_STATE_STOPPED            "Stopped       "
#define XCONS_TSK_STATE_VVECTORIO          "Vvectorio     "
#define XCONS_TSK_STATE_VDUALVECTORIO      "Vdualvectorio "
#define XCONS_TSK_STATE_VCOPYVOLUME        "Vcopyvolume   "
#define XCONS_TSK_STATE_UNKNOWN            "UNKNOWN       "


/* --- KT/UKT states */

#define XCONS_KT_STATE_INITIALIZED         "Initialized"
#define XCONS_KT_STATE_RUNNING             "Running    "
#define XCONS_KT_STATE_TERMINATING         "Terminating"
#define XCONS_KT_STATE_TERMINATED          "Terminated "
#define XCONS_KT_STATE_SLEEPING            "Sleeping   "
#define XCONS_KT_STATE_DETACHED            "Detached   "
#define XCONS_KT_STATE_ATTACHED            "Attached   "
#define XCONS_KT_STATE_UNKNOWN             "UNKNOWN"

#define XCONS_UKT_STATE_INITIALIZE         "Initialized"
#define XCONS_UKT_STATE_RUNNING            "Running    "
#define XCONS_UKT_STATE_TERMINATED         "Terminated "
#define XCONS_UKT_STATE_SLEEPING           "Sleeping   "
#define XCONS_UKT_STATE_UNKNOWN            "UNKNOWN"


/* --- task types */

#define XCONS_TSK_TYPE_TI                  "Timer   "
#define XCONS_TSK_TYPE_TL                  "TLogwr. "
#define XCONS_TSK_TYPE_AL                  "ALogwr. "
#define XCONS_TSK_TYPE_TW                  "Tracewr."
#define XCONS_TSK_TYPE_DW                  "Datawr. "
#define XCONS_TSK_TYPE_SN                  "Sender  "
#define XCONS_TSK_TYPE_RC                  "Receiv. "
#define XCONS_TSK_TYPE_CS                  "Convsc. "
#define XCONS_TSK_TYPE_UT                  "Utility "
#define XCONS_TSK_TYPE_OL                  "ObjLogwr"
#define XCONS_TSK_TYPE_SV                  "Server  "
#define XCONS_TSK_TYPE_US                  "User    "
#define XCONS_TSK_TYPE_EV                  "EventTsk"
#define XCONS_TSK_TYPE_GC                  "GarbColl"
#define XCONS_TSK_TYPE_BUP                 "Backup  "
#define XCONS_TSK_TYPE_UNKNOWN             "UNKNOWN"

/* --- task request types */

#define XCONS_REQ_VATTACH                     "REQ_VATTACH"
#define XCONS_REQ_VBLOCKIO                    "REQ_VBLOCKIO"
#define XCONS_REQ_VDETACH                     "REQ_VDETACH"
#define XCONS_REQ_INITOK                      "REQ_INITOK"
#define XCONS_REQ_VRESUME                     "REQ_VRESUME"
#define XCONS_REQ_VSIGNAL                     "REQ_VSIGNAL"
#define XCONS_REQ_VSLEEP                      "REQ_VSLEEP"
#define XCONS_REQ_CONNECT                     "REQ_CONNECT"
#define XCONS_REQ_RELEASE                     "REQ_RELEASE"
#define XCONS_REQ_VRECEIVE                    "REQ_VRECEIVE"
#define XCONS_REQ_SHUTDOK                     "REQ_SHUTDOK"
#define XCONS_REQ_SENDER_JOB_FREE             "REQ_SENDER_JOB_FREE"
#define XCONS_REQ_VENDEXCL                    "REQ_VENDEXCL"
#define XCONS_REQ_RESCHEDULE                  "REQ_RESCHEDULE"
#define XCONS_REQ_SHUTDOWN_RESUME_TW          "REQ_SHUTDOWN_RESUME_TW"
#define XCONS_REQ_ASYNOPEN                    "REQ_ASYNOPEN"
#define XCONS_REQ_ASYNCLOSE                   "REQ_ASYNCLOSE"
#define XCONS_REQ_ASYNIO                      "REQ_ASYNIO"
#define XCONS_REQ_ASYNWAIT                    "REQ_ASYNWAIT"
#define XCONS_REQ_VNSHUTDOWN_REPLY            "REQ_VNSHUTDOWN_REPLY"
#define XCONS_REQ_OUTGOING_CONNECT            "REQ_OUTGOING_CONNECT"
#define XCONS_REQ_OUTGOING_CONNECT_REPLY      "REQ_OUTGOING_CONNECT_REPLY"
#define XCONS_REQ_OPEN_WAKEUP_SD              "REQ_OPEN_WAKEUP_SD"
#define XCONS_REQ_CONNECT_RC                  "REQ_CONNECT_RC"
#define XCONS_REQ_RELEASE_RC                  "REQ_RELEASE_RC"
#define XCONS_REQ_CLOSE_OUTGOING              "REQ_CLOSE_OUTGOING"
#define XCONS_REQ_CLOSE_OUTGOING_REPLY        "REQ_CLOSE_OUTGOING_REPLY"
#define XCONS_REQ_FINISH_SV                   "REQ_FINISH_SV"
#define XCONS_REQ_ASYNCNTL                    "REQ_ASYNCNTL"
#define XCONS_REQ_VMARK_BAD_DEV               "REQ_VMARK_BAD_DEV"
#define XCONS_REQ_TEST_AND_SET                "REQ_TEST_AND_SET"
#define XCONS_REQ_BIG_CONNECT                 "REQ_BIG_CONNECT"
#define XCONS_REQ_ADDRESS_CONNECT             "REQ_ADDRESS_CONNECT"
#define XCONS_REQ_BIG_ADDR_CONNECT            "REQ_BIG_ADDR_CONNECT"
#define XCONS_REQ_EXCLUSIVE                   "REQ_EXCLUSIVE"
#define XCONS_REQ_VNSHUTDOWN                  "REQ_VNSHUTDOWN"
#define XCONS_REQ_IO_FINISHED                 "REQ_IO_FINISHED"
#define XCONS_REQ_DCOM_CREATE_DCOMI           "REQ_DCOM_CREATE_DCOMI"
#define XCONS_REQ_DCOM_REMOVE_DCOMI           "REQ_DCOM_REMOVE_DCOMI"
#define XCONS_REQ_DCOM_PARAMINFO			  "REQ_DCOM_PARAMINFO"
#define XCONS_REQ_DCOM_CO_CREATE_INSTANCE     "REQ_DCOM_CO_CREATE_INSTANCE"
#define XCONS_REQ_DCOM_RELEASE_INSTANCE		  "REQ_DCOM_RELEASE_INSTANCE"
#define XCONS_REQ_DCOM_CO_GET_CLASSOBJECT     "REQ_DCOM_CO_GET_CLASSOBJECT"
#define XCONS_REQ_DCOM_TRANSFER			      "REQ_DCOM_TRANSFER"
#define XCONS_REQ_DCOM_RELASE_ALL			  "REQ_DCOM_RELASE_ALL"
#define XCONS_REQ_VSTOP                       "REQ_VSTOP"
#define XCONS_REQ_VVECTORIO                   "REQ_VVECTORIO"
#define XCONS_REQ_TYPE_UNKNOWN                "REQ_UNKNOWN"

/* --- task storage descriptions */

#define XCONS_STOR_DES_CODESIZE            "CODESIZE"
#define XCONS_STOR_VDES_CODESIZE           "KB"
#define XCONS_STOR_DES_TSK_STACK_SIZE      "TASK_STACKSIZE"
#define XCONS_STOR_VDES_TSK_STACK_SIZE     "KB"
#define XCONS_STOR_DES_KGS                 "KERNEL_GLOB_SECTION"
#define XCONS_STOR_VDES_KGS                "KB"
#define XCONS_STOR_DES_KSS                 "KERNEL_SHRD_SECTION"
#define XCONS_STOR_VDES_KSS                "KB"
#define XCONS_STOR_DES_SHRD_DYN_POOLFREE   "UNUSED_SHARED_DYN_POOL"
#define XCONS_STOR_VDES_SHRD_DYN_POOLFREE  "KB"
#define XCONS_STOR_DES_SHRD_DYN_POOL_USED  "USED_SHARED_DYN_POOL"
#define XCONS_STOR_VDES_SHRD_DYN_POOL_USED "KB"
#define XCONS_STOR_DES_SHRD_DYN_DATFREE    "UNUSED_SHARED_DYN_DATA"
#define XCONS_STOR_VDES_SHRD_DYN_DATFREE   "PAGES"
#define XCONS_STOR_DES_SHRD_DYN_DAT_USED   "USED_SHARED_DYN_DATA"
#define XCONS_STOR_VDES_SHRD_DYN_DAT_USED  "PAGES"
#define XCONS_STOR_DES_RTE_ALL_POOL_USED   "USED_RTE_ALLOC_POOL"
#define XCONS_STOR_VDES_RTE_ALL_POOL_USED  "KB"
#define XCONS_STOR_DES_RTE_ALL_POOL_MAX    "MAX_RTE_ALLOC_POOL"
#define XCONS_STOR_VDES_RTE_ALL_POOL_MAX   "MB"
#define XCONS_STOR_DES_KRN_ALL_POOL_USED   "USED_KERNEL_ALLOC_POOL"
#define XCONS_STOR_VDES_KRN_ALL_POOL_USED  "KB"
#define XCONS_STOR_DES_KRN_ALL_POOL_MAX    "MAX_KERNEL_ALLOC_POOL"
#define XCONS_STOR_VDES_KRN_ALL_POOL_MAX   "KB"
#define XCONS_STOR_DES_EXISTING_TASKS      "EXISTING_TASKS"
#define XCONS_STOR_VDES_EXISTING_TASKS     ""

/* --- task rte descriptions */

#define XCONS_RTE_DESC_DEV_VTRACE          "DEV-VTRACE"
#define XCONS_RTE_DESC_DEV                 "DEV"
#define XCONS_RTE_DESC_UKT                 "UKT%-7d"

/* --- task version descriptions */
#define XCONS_VERS_DESC_KERNEL             "KERNEL"
#define XCONS_VERS_DESC_RTE                "RTE"


/*
 --- XSHOW ---
*/
#define XSHOW_ADABAS_GATEWAYS              "\nADABAS for R/3 - Gateways: \n"
#define XSHOW_ADABAS_SERVERDBS             "\nADABAS for R/3 - SERVERDBs: \n"
#define XSHOW_GATEWAYS                     "\nGateways: \n"
#define XSHOW_SERVERDBS                    "\nSERVERDBs: \n"
#define XSHOW_SERVERDB_GW_RUNNING          "  - %-18.18s  [Running]\n"
#define XSHOW_SERVERDB_GW_STOPPED          "  - %-18.18s  [Stopped]\n"
#define XSHOW_SERVERDB_GW_VER_RUNNING      "  - %-18.18s '%-40.40s'  [Running]\n"
#define XSHOW_SERVERDB_GW_VER_STOPPED      "  - %-18.18s '%-40.40s'  [Stopped]\n"
#define XSHOW_SERVERDB_GW_ACC_RUNNING      "  - %-18.18s '%-40.41s  [Running]\n"
#define XSHOW_SERVERDB_GW_ACC_STOPPED      "  - %-18.18s '%-40.41s  [Stopped]\n"
#define XSHOW_SERVERDB_GW_DBR_RUNNING      "  - %-18.18s '%-41.41s  [Running]\n"
#define XSHOW_SERVERDB_GW_DBR_STOPPED      "  - %-18.18s '%-41.41s  [Stopped]\n"
#define XSHOW_NO_SERVERDBS_GATEWAYS        "  [ N O N E ]\n"
#define XSHOW_XSERVER_ACTIVE               "\nXSERVER is active.\n"
#define XSHOW_XSERVER_NOT_ACTIVE           "\nXSERVER is not active.\n"
#define XSHOW_XSERVER_NOT_INSTALLED        "\nXSERVER is not installed.\n"
#define XSHOW_XSERVER_VER_ACTIVE           "\nXSERVER '%-40.40s' is active.\n"
#define XSHOW_XSERVER_VER_NOT_ACTIVE       "\nXSERVER '%-40.40s' is not active.\n"
#define XSHOW_XSERVER_DBR_ACTIVE           "\nXSERVER '%-41.41s is active.\n"
#define XSHOW_XSERVER_DBR_NOT_ACTIVE       "\nXSERVER '%-41.41s is not active.\n"



/*
 --- DIAGNOSE / MESSAGES OUTPUT ---
*/
#define LAST_CHANCE_INFO         "Last Chance Information"
#define ERR_LONG_DESCRIPTION     "ERROR"
#define IERR_LONG_DESCRIPTION    "INTERNAL ERROR"
#define INFO_LONG_DESCRIPTION    "INFO"
#define WRN_LONG_DESCRIPTION     "WARNING"

#define BIG_HEADER_INFO_LINE     "--------------------------------------------------------------------------------\r\n"\
                                 "Date       Time       TID(hex) Typ MsgID Label    Message-Text\r\n"\
                                 "--------------------------------------------------------------------------------\r\n"
#define TIME_HEADER_INFO_LINE    "--------------------------------------------------------------------------------\r\n"\
                                 "Date       Time     Message-Text\r\n"\
                                 "--------------------------------------------------------------------------------\r\n"

#define EVENT_FILE_INFO_LINE     "--------------------------------------------------------------------------------\r\n"\
                                 "Date     Time     _________# Id P ____Value1 ____Value2 Message-Text\r\n"\
                                 "--------------------------------------------------------------------------------\r\n"

#define ERR_FILE_OPEN_LINE       "%19.19s                               --- Starting ---\r\n"
#define ERR_FILE_STARTING_LINE   "%19.19s                               --- Starting GMT %s %s\r\n"
#define BIG_HEADER_FORMAT_STR    "%19.19s %10.10s %3.3s %5d %-8.8s "
#define TIME_HEADER_FORMAT_STR   "%04d-%02d-%02d %02d:%02d:%02d"
#define SMALL_HEADER_FORMAT_STR  "%5d %s: "
#define E_LOG_HEADER_FORMAT_STR  ":\r\n - %s   TID: %s  PID: %s  MsgID: %d\r\n  "


#define BEGIN_OF_DIAG_WRITE_CYCLE  "\r\n============================================ begin of write cycle ==================\r\n\r\n\r\n"

#define CURR_DIAG_WRITE_POS            "-------------------------------------------- current write position ----------------"
#define END_OF_DIAG_WRITE_CYCLE    "\r\n===========================================  end of write cycle  ==================="

/* --- KERNEL VEN35KC 'errortab' messages ----------------------------------------------------------------------------------------- */

#define ERRMSG_DCOM_CREATE_DCOMI                "could not create DCOMi-thread"
#define ERRMSG_DCOM_RESUME_DCOMI                "could not resume DCOMi-thread"
#define ERRMSG_DCOM_COINITIALIZE_FAILED         "CoInitialize failed"
#define ERRMSG_DCOMI_CLOSE_EVENT_SEM            "could not close DCOMi event semaphore"
#define ERRMSG_DCOM_MAX_DCOMIS_REACHED          "max DCOMis reached"
#define ERRMSG_DCOM_GET_DCOMI_CTRL              "could not get DCOMi-Ctrl"
#define ERRMSG_DCOM_DEBUG_NOT_ENABLED           "DCOM-Debug not enabled"
#define ERRMSG_DCOMI_NO_RUNNING                 "DCOMi not running"
#define ERRMSG_DCOM_REM_UKT_DCOMi_NOT_ALLOWED   "remove of UKT related DCOMi-thread not allowed"

/* --- KERNEL VEN54KC 'errortab' messages ----------------------------------------------------------------------------------------- */

#define ERRMSG_DEVIO_DEVCREA                    "could not create volume"
#define ERRMSG_DEVIO_MAXDEVNO                   "maximum attached volumes reached"
#define ERRMSG_DEVIO_ALRATT                     "volume already attached"
#define ERRMSG_DEVIO_DEVICREA                   "could not create parallel IO thread"
#define ERRMSG_DEVIO_WRITE                      "could not write on volume"
#define ERRMSG_DEVIO_READ                       "could not read from volume"
#define ERRMSG_DEVIO_SEEK                       "could not seek on volume"
#define ERRMSG_DEVIO_INVPNO                     "invalid pageno, no io performed"
#define ERRMSG_DEVIO_LOCK                       "could not lock volume"
#define ERRMSG_DEVIO_DEV_POS                    "wrong volume position"
#define ERRMSG_DEVIO_SEEK_POS                   "unexpected volume position"
#define ERRMSG_DEVIO_NOT_ATT                    "volume not attached"
#define ERRMSG_DEVIO_DEVSPACE_ON_DIR            "no volumes on directory"
#define ERRMSG_DEVIO_DEVSPACE_ON_SYS            "no volumes on system file"
#define ERRMSG_DEVIO_DEVSPACE_ON_RD_ONLY        "no volumes on read-only file"
#define ERRMSG_DEVIO_DEVSPACE_COMPRESSED        "no volumes on compressed file"
#define ERRMSG_DEVIO_ERROR_ON_DEVSPACE          "error on volume"
#define ERRMSG_DEVIO_OPEN_DEVSPACE              "could not open volume"
#define ERRMSG_DEVIO_ACCESS_DENIED              "access on volume denied"
#define ERRMSG_DEVIO_LOCK_DEVSPACE              "could not lock volume"
#define ERRMSG_DEVIO_WRONG_PATH                 "wrong path specified"
#define ERRMSG_DEVIO_GET_FILE_INFO              "could not query file info"
#define ERRMSG_DEVIO_UNLOCK_DEVSPACE            "could not unlock volume"
#define ERRMSG_DEVIO_CREATE_DEVSPACE            "could not create volume"
#define ERRMSG_DEVIO_DURING_Q_FILE_SIZE         "query file-info for volume"
#define ERRMSG_DEVIO_CLOSE_DEVSPACE             "could not close the volume"
#define ERRMSG_DEVIO_SET_DEVSPACE_OWNER         "could not set volume owner"
#define ERRMSG_DEVIO_WRITE_ON_DEVSPACE          "could not write on volume"
#define ERRMSG_DEVIO_READ_FROM_DEVSPACE         "could not read from volume"
#define ERRMSG_DEVIO_SINGLEIO_DEVSP_UNDEF       "read/write on undefined volume"
#define ERRMSG_DEVIO_SEEK_ON_DEVSPACE           "could not seek on volume"
#define ERRMSG_DEVIO_DISK_FULL                  "not enough space on disk"
#define ERRMSG_DEVIO_INVALID_DEVNO              "invalid despace no."
#define ERRMSG_DEVIO_DIF_SIZE_OF_MIR            "different sizes of mirrored volumes"
#define ERRMSG_DEVIO_DEV0_NOT_ACTIVE            "main I/O thread not active"
#define ERRMSG_DEVIO_DEVSPACE_LOCKED            "volume locked by another process"
#define ERRMSG_DEVIO_INV_DEVSPACE_SIZE          "invalid volume size"
#define ERRMSG_DEVIO_CANT_CHECK_DRIVE_CONT      "error while checking drive contents"
#define ERRMSG_DEVIO_DEV_DRIVE_NOT_EMPTY        "drive must be empty to initialize"
#define ERRMSG_DEVIO_CANT_REDET_MEDIA           "could not redetermine media"
#define ERRMSG_DEVIO_DEV_POS                    "wrong volume position"
#define ERRMSG_DEVIO_SEEK_POS                   "unexpected volume position"
#define ERRMSG_DEVIO_WRONG_FILE_ATTRIB          "wrong file atrribute"
#define ERRMSG_DEVIO_DEVICREA_TIMEOUT           "parallel IO thread creation timeout"
#define ERRMSG_DEVIO_EVAL_DEVSPACE              "error evaluating volume"
#define ERRMSG_DEVIO_CREATE_COMPL_PORT          "error creating io completion port"
#define ERRMSG_DEVIO_USER_REQ_ERROR             "user requested error"
#define ERRMSG_DEVIO_ALLOC_MEM                  "could not allocate memory"


/* --- KERNEL VEN59KC error messages ---------------------------------------------------------------------------------------------- */

#define ERRMSG_EXEC_COMMAND                     "could not execute command"
#define ERRMSG_CMD_FILE_NOT_FOUND               "CMD.EXE not found"

/* --- KERNEL VEN65KC 'errortab' messages ----------------------------------------------------------------------------------------- */

#define ERRMSG_ASYNCIO_SURPRISING_ERROR_CODE    "surprising error code"
#define ERRMSG_ASYNCIO_ALRATT                   "device already attached"
#define ERRMSG_ASYNCIO_MAXDEVICES               "maximum attached devices reached"
#define ERRMSG_ASYNCIO_ASYNCI_CREATE            "could not create async IO thread"
#define ERRMSG_ASYNCIO_ASYNC0_NOT_ACTIVE        "main async I/O thread not active"
#define ERRMSG_ASYNCIO_ACCESS_DENIED            "access denied"
#define ERRMSG_ASYNCIO_WRONG_PATH               "wrong path specified"
#define ERRMSG_ASYNCIO_GET_FILE_INFO            "could not query file info"
#define ERRMSG_ASYNCIO_IO_ON_DIR                "file I/O on directoriy not allowed"
#define ERRMSG_ASYNCIO_IO_ON_SYS                "file I/O on system file not allowed"
#define ERRMSG_ASYNCIO_OPEN_FILE_DEV_PIPE       "could not open file/device/pipe"
#define ERRMSG_ASYNCIO_WRONG_DEVICE_TYPE        "wrong device type"
#define ERRMSG_ASYNCIO_CLOSE_FILE_DEV_PIPE      "could not close file/device/pipe"
#define ERRMSG_ASYNCIO_CLOSE_EVENT_SEM          "could not close semaphore"
#define ERRMSG_ASYNCIO_ASYNCI_REMOVE            "could not remove async IO thread"
#define ERRMSG_ASYNCIO_FILE_DEV_PIPE_NOT_OPEN   "file/device/pipe not open"
#define ERRMSG_ASYNCIO_WRONG_DEVICENO           "wrong deviceno"
#define ERRMSG_ASYNCIO_WAIT_FIRST               "call wait first"
#define ERRMSG_ASYNCIO_NO_REQ_ACTIVE            "no I/O requests active"
#define ERRMSG_ASYNCIO_CANT_SEEK_FILE_DEV_PIPE  "could not seek on file/device/pipe"
#define ERRMSG_ASYNCIO_CANT_READ_FILE_DEV_PIPE  "could not read from file/device/pipe"
#define ERRMSG_ASYNCIO_CANT_WRITE_FILE_DEV_PIPE "could not write to file/device/pipe"
#define ERRMSG_ASYNCIO_END_OF_FILE_DEV_PIPE     "end of file"
#define ERRMSG_ASYNCIO_ALLOC_MEM                "could not allocate mameory"
#define ERRMSG_ASYNCIO_FREE_MEM                 "could not free memory"
#define ERRMSG_ASYNCIO_WRONG_BLOCK_SIZE         "wrong block size specified"
#define ERRMSG_ASYNCIO_WRONG_FILE_ATTRIB        "wrong file atrribute"
#define ERRMSG_ASYNCIO_MEDIA_WRITE_PROTECTED    "the media is write protected"
#define ERRMSG_ASYNCIO_DEVICE_NOT_READY         "device not ready"
#define ERRMSG_ASYNCIO_DEVICE_BUSY              "device busy"
#define ERRMSG_ASYNCIO_END_OF_MEDIA             "end of media reached"
#define ERRMSG_ASYNCIO_FILE_SET_MARK            "filemark or setmark was reached"
#define ERRMSG_ASYNCIO_MEDIA_NOT_PATITIONED     "no partition information found"
#define ERRMSG_ASYNCIO_NO_MEDIA_IN_DEVICE       "no media in device"
#define ERRMSG_ASYNCIO_MEDIA_CHANGED            "media changed"
#define ERRMSG_ASYNCIO_BAD_LENGTH_READ_WRITTEN  "bad length of data written/read"
#define ERRMSG_ASYNCIO_GET_TAPE_DRIVE_INFO      "could not get drive info"
#define ERRMSG_ASYNCIO_GET_TAPE_MEDIA_INFO      "could not get media info"
#define ERRMSG_ASYNCIO_SET_TAPE_DRIVE_INFO      "could not set drive info"
#define ERRMSG_ASYNCIO_SET_TAPE_MEDIA_INFO      "could not set media info"
#define ERRMSG_ASYNCIO_REWIND_TAPE              "could not rewind tape"
#define ERRMSG_ASYNCIO_LOCK_UNLOCK_TAPE         "could not lock/unlock tape"
#define ERRMSG_ASYNCIO_WRITE_TAPE_MARK          "could not write tape mark"
#define ERRMSG_ASYNCIO_CRC_ERROR                "CRC error, data/media might be damaged"
#define ERRMSG_ASYNCIO_SHARING_VIOLATION        "file is already used by another process"
#define ERRMSG_ASYNCIO_WRONG_FILE_DEVICE_NAME   "invalid file or device name"
#define ERRMSG_ASYNCIO_UNRECOGNIZED_MEDIA       "unrecognized media"
#define ERRMSG_ASYNCIO_TENSION_TAPE             "could not tension tape"
#define ERRMSG_ASYNCIO_DISK_FULL                "not enough space on the disk."
#define ERRMSG_ASYNCIO_LOAD_UNLOAD_TAPE         "could not load/unload tape"
#define ERRMSG_ASYNCIO_ILL_BLOCK_COUNT          "blockcount mismatch"
#define ERRMSG_ASYNCIO_IO_READ_ONLY             "cannot write on read-only file"
#define ERRMSG_ASYNCIO_PIPE_CONN_CANCELLED      "pipe operation cancelled"
#define ERRMSG_ASYNCIO_NO_DATA_DETECTED         "no more data on media"
#define ERRMSG_ASYNCIO_OPERATION_CANCELED       "operation was canceled by user"  // --- AIDS 1008631
#define ERRMSG_ASYNCIO_INVALID_BLOCKSIZE        "invalid tape blocksize"

/* --- Virtual file - general error messages -------------------------------------------------------------------------------------- */

#define ERRMSG_VF_ILL_HOST_FILE_NAME            "illegal host file name"
#define ERRMSG_VF_ILL_READ_FORMAT_COMB          "illegal read/format combination"
#define ERRMSG_VF_WRONG_FILE_NAME               "wrong file name"
#define ERRMSG_VF_ILL_DIREC                     "illegal direction"
#define ERRMSG_VF_BAD_FILENO                    "invalid file number"
#define ERRMSG_VF_ILL_FILE_DESC                 "illegal file description"
#define ERRMSG_VF_NON_CONST_REC_LEN             "non-constant record length"

/* --- Win95/NT Console ----------------------------------------------------------------------------------------------------------- */

#define KERNEL_HINT_SERVERDB_STARTING           "SERVERDB '%s' is starting"
#define KERNEL_HINT_STARTING                    "SERVERDB is starting"
#define KERNEL_HINT_SERVERDB_COLD               "SERVERDB '%s' is ADMIN - %d user(s) connected"
#define KERNEL_HINT_COLD                        "SERVERDB is ADMIN - %d user(s) connected"
#define KERNEL_HINT_SERVERDB_WARM               "SERVERDB '%s' is ONLINE - %d user(s) connected"
#define KERNEL_HINT_WARM                        "SERVERDB is ONLINE - %d user(s) connected"
#define KERNEL_HINT_SERVERDB                    "SERVERDB '%s' - %d user(s) connected"

/* --- OS/2 Console --------------------------------------------------------------------------------------------------------------- */

#define KERNEL_WARNING                          "Warning"
#define KERNEL_INFORMATION                      "Information"
#define KERNEL_QUESTION                         "Question"
#define KERNEL_CONSOLE                          " Console "
#define KERNEL_DIAG                             " Diagnostic file "
#define KERNEL_FONT_TITLE                       "Font"
#define KERNEL_DB_STATE_WARM                    "ONLINE"
#define KERNEL_DB_STATE_COLD                    "ADMIN"
#define KERNEL_DB_STATE_STOPPED                 "STOPPED"
#define KERNEL_XSERVER_NOT_ACTIVE               "Remote SQL Server"
#define KERNEL_XSERVER_ACTIVE                   "Remote SQL Server"
#define KERNEL_STRING_NOT_FOUND                 "String not found!"
#define KERNEL_DELETE_CONSOLE                   "Do you want to delete the console contents ?\n\n"
#define KERNEL_CANCEL_EXIT_WARM_CONSOLE         "To stop the SERVERDB it must be in ADMIN mode.\n\n"
#define KERNEL_EXIT_WARM_CONSOLE                "The SERVERDB is ONLINE. Do you really want to stop the SERVERDB?\n\n"
#define KERNEL_EXIT_CONSOLE                     "Do you want to stop the SERVERDB?\n\n"
#define KERNEL_ISO_FONT_FAILED                  "The font selected is not compliant with the"     \
                                                " ISO 9241 standard on the IBM 9515, 9517, or"    \
                                                " 9518 displays.\n\n"
#define KERNEL_ISO_FONT_NOT_TESTED              "The font selected is not tested for compliance"  \
                                                " with the ISO 9241 standard.\n\n"


/* --- Kernel Communication ------------------------------------------------------------------------------------------------------ */

#define ERRMSG_NETW_NOT_RESTARTED               "network not restarted"
#define ERRMSG_NOT_YET_IMPLEMENTED              "not yet implemented"
#define ERRMSG_NETW_SHUTDOWN                    "network shutdown"
#define ERRMSG_REQUEST_TIMEOUT                  "request timeout"
#define ERRMSG_RECEIVE_TIMEOUT                  "receive timeout"
#define ERRMSG_SERVERDB_SHUTDOWN                "SERVERDB shutdown"
#define ERRMSG_ILLEGAL_PATHID                   "illegal pathid"
#define ERRMSG_PATH_NOT_CONNECTED               "path not connected"
#define ERRMSG_PEER_SEM_ERROR                   "peer semaphore error"
#define ERRMSG_TOO_MANY_SESSIONS                "too many sessions"
#define ERRMSG_WRONG_PACKET_LEN                 "wrong packet length"
#define ERRMSG_WRONG_PACKET_REQ                 "wrong packet requested"
#define ERRMSG_PROTOCOL_ERROR                   "protocol error"
#define ERRMSG_CONNECTION_BROKEN                "connection broken"
#define ERRMSG_CONNECTION_CLOSED_BY_C_PARTNER   "connection closed by comm. partner"

/* --- Multithreading API ------------------------- */

#define ERRMSG_STACKSIZE                        "stack size is to small."
#define ERRMSG_RESOURCE_LIMIT                   "resource limit."
#define ERRMSG_PROCADDRESS_NULL                 "address of thread proc is NULL."
#define ERRMSG_UNKNOWN_THREADID                 "thread id cannot be found."
#define ERRMSG_INVALID_SIGNAL                   "invalid signal number."
#define ERRMSG_DEADLOCK                         "failed to prevent a deadlock."
#define ERRMSG_INVALID_TLSKEY                   "invalid local storage key."
#define ERRMSG_TLSKEY_BUSY                      "local storage key is busy."




/* --- Microsoft Cluster Server ------------------- */

#define ERRMSG_CS_OPEN_REG                      _T("[SAP DBTech] Unable to open Parameters key. Error: %1!u!.")
#define ERRMSG_CS_ALLOC_RESOURCE                _T("[SAP DBTech] Unable to allocate resource entry structure. Error: %1!u!.")
#define ERRMSG_CS_NONEXIST_REQUEST              _T("[SAP DBTech] Request for a nonexistent resource id %1!u!.")
#define ERRMSG_CS_SANITY_CHECK_FAILED           _T("[SAP DBTech] Resource sanity check failed! resource id: %1!u!.")
#define ERRMSG_CS_START_WORKER_THRD             _T("[SAP DBTech] Online: Unable to start thread, Status %1!u!.")
#define ERRMSG_CS_ERR_READ_PROPERTY             _T("[SAP DBTech] Unable to read the '%1' property. Error: %2!u!.")
#define ERRMSG_CS_STARTING_RESOURCE             _T("[SAP DBTech] Error %1!u! bringing resource '%2' online.")
#define ERRMSG_CS_STOPPING_RESOURCE             _T("[SAP DBTech] Error %1!u! bringing resource '%2' offline.")


#define INFOMSG_CS_CLOSE_REQUEST                _T("[SAP DBTech] Close request.")
#define INFOMSG_CS_ONLINE_REQUEST               _T("[SAP DBTech] Online request.")
#define INFOMSG_CS_OFFLINE_REQUEST              _T("[SAP DBTech] Offline request.")
#define INFOMSG_CS_TERMINATE_REQUEST            _T("[SAP DBTech] Terminate request.")
#define INFOMSG_CS_RES_CTRL_REQUEST             _T("[SAP DBTech] Resource control request.")
#define INFOMSG_CS_IO_PENDING                   _T("[SAP DBTech] IO pending.")


#define OPTION_HELP_STR                         _T("help")
#define OPTION_ERROR_MESSAGES_TO_DBOX_STR       _T("send error messages to dialog box")
#define OPTION_VERBOSE_ERROR_MESSAGES_STR       _T("print verbose error messages")
#define OPTION_MOVE_DRIVE_TO_GROUP_STR          _T("move drive to SAPDB group")
#define OPTION_ONLINE_GROUP_STR                 _T("bring SAPDB group online")
#define OPTION_OFFLINE_GROUP_STR                _T("bring SAPDB group offline")
#define OPTION_BUILDUP_GROUP_STR                _T("build up SAPDB group")
#define OPTION_REMOVE_SAPDB_RESOURCES_STR       _T("remove all SAPDB resources")
#define OPTION_CREATE_RESOURCE_TYPES_STR        _T("create SAPDB resource types")
#define OPTION_REMOVE_RESOURCE_TYPES_STR        _T("remove SAPDB resource types")
#define OPTION_ONLINE_XSERVER_RESOURCE_STR      _T("bring XSERVER online")
#define OPTION_OFFLINE_XSERVER_RESOURCE_STR     _T("bring XSERVER offline")
#define OPTION_ONLINE_SERVERDB_RESOURCE_STR     _T("bring SERVERDB online")
#define OPTION_OFFLINE_SERVERDB_RESOURCE_STR    _T("bring SERVERDB offline")
#define OPTION_ADD_SERVERDB_RESOURCE_STR        _T("add SERVERDB resource")
#define OPTION_REMOVE_SERVERD_BRESOURCE_STR     _T("remove SERVERDB resource")
#define OPTION_ENABLE_CLUSTERLOG_STR            _T("enable cluster log")
#define OPTION_DISABLE_CLUSTERLOG_STR           _T("disable cluster log")
#define OPTION_REGISTER_DEBUGEX_STR             _T("")
#define OPTION_UNREGISTER_DEBUGEX_STR           _T("")

#endif  /* GEO007_2_H */
