/* @lastChanged: "1998-06-22  11:16"
 * @filename:    geo38.h
 * @purpose:     "DCOM - Senke"
 * @release:     7.2.0.0
 * @see:         ""
 *
 * @copyright:   "Copyright by SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 */


#ifndef GEO38_H
#define GEO38_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gco01.h"
#include "RunTime/Synchronisation/RTESync_Spinlock.hpp"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define   C_HASH_SESS_HDR_SIZE               3277

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

struct tak_all_command_glob;
struct tsp92_RteTaskDetailsInfo;
class  SAPDBMem_RawAllocator;
class  LVCMem_CheckedUserAllocator;

class CIliveCacheSink : public IliveCacheSink
{
public:
  class AllocPool;

  // The value for OMS_SYS_REGION must match the define INTERNAL_CRITICAL_SECTIONS_CO001
  // in the file gco001.h
  enum { OMS_SYS_REGION = 25 };  /* + 17 PTS 1124170 */
  tak_all_command_glob     *lpSessionContext;
  void                     *lpDefaultContext;
  void                     *lpTaskContext;       /* PTS 1106969 */
  tsp92_RteTaskDetailsInfo *lpCallTaskInfo;
  void                     *lpSqlSessionContext;     /* PTS 1120804 */
  SAPDBMem_RawAllocator    *lpOmsSessionAllocator;
  SAPDBMem_RawAllocator    *lpSqlClsAllocator;
  LVCMem_CheckedUserAllocator      *m_pSessionUserAllocator;
  LVCMem_CheckedUserAllocator      *m_pTransactionUserAllocator;
  LVCMem_CheckedUserAllocator      *m_pCOMRoutineUserAllocator;
  SAPDBMem_RawAllocator    *m_pSessionRawAllocator;
  teo00_Bool                m_inRegion[OMS_SYS_REGION];
  bool                     sqlErrorFlag;                    
  bool                     CallbackIsActive;
  static RTESync_Spinlock  m_allocatorPoolLock;
  static AllocPool         *m_allocatorPool;
  static AllocPool         *m_allocatorPoolFreelist;

  STDMETHODIMP         QueryInterface(REFIID, PVOID*);
  
  STDMETHODIMP_(ULONG) AddRef(void);
  
  STDMETHODIMP_(ULONG) Release(void);

#define GCO01_ONLY_MEMBER_FUNCS
#include "gco01.h" 
  CIliveCacheSink();
  bool InitAllocator();
  void Destruct(); 

  static CIliveCacheSink* GetSessionSink();
  
  void *operator new( size_t stAllocateBlock, void *addr );
#if defined(PLACEMENT_DELETE_MEO00)
  void operator delete(void* p, void *addr ) {}
#endif
  void operator delete(void* p);
};

typedef enum 
{
  SearchALL_eeo38,
    SearchEQ_eeo38,
    SearchNE_eeo38   
} teo38_HashSearchType;

typedef enum { 
  Free_eeo38,   
    CLSID_eeo38,
    Invalid_CLSID_eeo38,
    IID_eeo38,
    Invalid_IID_eeo38,
    DbgFactory_eeo38,
    DbgInterface_eeo38
} teo38_HashEntryType;

class RTE_DBProcError
{
public:
  short error_number;
  RTE_DBProcError(short no);
};
inline RTE_DBProcError::RTE_DBProcError(short no) {error_number = no;}

/*===========================================================================*
*  CHashEntry                                                               *
*===========================================================================*/
class CHashEntry{
protected:
  CHashEntry           *m_lpnext;
  tsp00_Uint4           m_session;
  GUID                  m_iid;
  LPUNKNOWN             m_IUnknown;
  teo38_HashEntryType   m_type;
  bool                  m_idebug_used_flag;
  bool                  m_is_committed;
  bool                  m_is_remote_debug;
public:
  CHashEntry ();
  bool        getDebugFlag( void );
  bool        getRemoteDbgFlag( void );
  GUID        *getGuid();
  LPUNKNOWN   getIUnknown( void );
  bool        getIsCommittedFlag( void );
  CHashEntry  *getNextEntry();
  tsp00_Uint4 getSession();
  teo38_HashEntryType getType( void );
  bool        GuidEQ( GUID *lpguid );
  void        init(tsp00_Uint4   session, 
    LPGUID      lpGuid, 
    LPUNKNOWN   lpInterface, 
    bool        remoteDebugFlag,
    teo38_HashEntryType hType );
  bool sessionEQ( tsp00_Uint4   session );
  bool sessionNE( tsp00_Uint4   session );
  void setDebugFlag(bool flag);
  void setIsCommittedFlag(bool flag);
  void setNextEntry( CHashEntry* next );
  void setType( teo38_HashEntryType type );
  void setSession( tsp00_Uint4   session );
  void* operator new( size_t sz );  /* PTS 1110403 */
  void operator delete(void* p);    /* PTS 1110403 */
  const CHashEntry & operator= (const CHashEntry& right); /* PTS 1116802 */
};
typedef CHashEntry *CHashEntryPtr;

inline bool         CHashEntry::getDebugFlag( void ) { 
  return m_idebug_used_flag; }
inline bool         CHashEntry::getRemoteDbgFlag( void ) { 
  return m_is_remote_debug; }
inline bool         CHashEntry::getIsCommittedFlag( void ) { 
  return m_is_committed; }
inline GUID         *CHashEntry::getGuid() { 
  return &m_iid; }
inline CHashEntry   *CHashEntry::getNextEntry(){ 
  return m_lpnext;}
inline LPUNKNOWN    CHashEntry::getIUnknown( void ) { 
  return m_IUnknown; }
inline tsp00_Uint4  CHashEntry::getSession() { 
  return m_session; }
inline teo38_HashEntryType CHashEntry::getType( void ){ 
  return m_type; }
inline bool         CHashEntry::GuidEQ( GUID *lpguid ) {
      if (memcmp (lpguid, &m_iid, sizeof (GUID)) ) return false;
      else return true; }
inline bool         CHashEntry::sessionEQ( tsp00_Uint4   session ) {
  if ( session == m_session )  return true;
  else return false; }
inline bool CHashEntry::sessionNE( tsp00_Uint4   session ) {
  if ( session != m_session ) return true;
  else return false; }
inline void CHashEntry::setDebugFlag(bool flag) { 
  m_idebug_used_flag  = flag; };
inline void CHashEntry::setIsCommittedFlag(bool flag) { 
  m_is_committed  = flag; };
inline void CHashEntry::setNextEntry( CHashEntry* next ) { 
  m_lpnext = next;}
inline void CHashEntry::setType( teo38_HashEntryType type ){ 
  m_type = type; }
inline void CHashEntry::setSession( tsp00_Uint4   session ){ 
  m_session = session; }

/*===========================================================================*
*  CCoClassHash                                                              *
*===========================================================================*/
class CCoClassHash {
private:
 	tsp00_Uint4           m_HashHdrSize;
 	CHashEntryPtr        *m_HashHdrPtr;
 	CHashEntryPtr         m_HashFreePtr;
  CHashEntryPtr         m_HashCurrPtr;
  tsp00_Uint4           m_HashCurrIdx;
  tsp00_Uint4           m_session;
  teo38_HashSearchType  m_searchType;
public :
  CCoClassHash ();
  inline tsp00_Uint4 SessionHashIdx( tsp00_Uint4   session );
  tsp00_Uint4        HashInit (const tsp00_Uint4   lcSessionHashHdrSize = C_HASH_SESS_HDR_SIZE );
  void               chainEntry(CHashEntryPtr p);
  void               ResetEntrySearch(tsp00_Uint4   session, 
    teo38_HashSearchType searchType );
  CHashEntryPtr getFreeEntry();
  CHashEntryPtr NextEntry( void );
  CHashEntryPtr GetRechainFirstEntry( 
    teo38_HashEntryType *lpEntryType );
  CHashEntryPtr FindHashEntry( GUID *lpGuid, tsp00_Uint4   session );
  void chainFreeEntry(CHashEntryPtr p);
  void* operator new( size_t sz );  /* PTS 1110403 */
  void operator delete(void* p);    /* PTS 1110403 */
};
typedef CCoClassHash *CCoClassHashPtr;

/*---------------------------------------------------------------------------*/
inline tsp00_Uint4 CCoClassHash::SessionHashIdx( tsp00_Uint4 session ){
  if ( 0 != session )
    return  ( session % (m_HashHdrSize - 1)); 
  else
    return (m_HashHdrSize - 1);
};


#endif  /* GEO38_H */
