/* @lastChanged: "2000-10-21  15:30"

  * @filename:   geo39.h
  * @purpose:    "COM"
  * @release:    7.2.0.0
  * @see:        "-.-"
  *
  * @copyright:  "Copyright by SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/


#ifndef GEO39_H
#define GEO39_H

#include "livecachetypes.h" /* no check */
#include "ggg00.h"
#include "ggg92.h"

#define MARSHAL_BUF_MXEO39        33000

typedef struct teo39_TransendBufMsg {
  tgg00_MessType  mtype;
  tgg00_MessType2 mtype2;
} teo39_TransendBufMsg;

/* struct describing 'stack' passed to transend */
typedef struct teo39_TransendBuf {
  union {
    teo39_TransendBufMsg msg;
    tgg00_BasisError     err;
  };
} teo39_TransendBuf;

typedef struct teo39_Transend {
    union {
      tsp00_Int4  buflen;
      tsp00_Int4* lpbuflen;
    };
    teo39_TransendBuf* lpbuf;
} teo39_Transend;

typedef struct  teo39_Variant
{
  union 
  {
    PVOID   byref;
    tsp00_Int4    *plVal;
  };
#if !defined(BIT64)
  tsp00_Int4  reserved1;
#endif
  short         vt;
  unsigned char io_flag;
  unsigned char ptr_flag;
  tsp00_Int4          len;
} teo39_Variant;
    
typedef struct teo39_Transend_desc {
    teo39_Variant lpbuflen;
    teo39_Variant lpbuf;
} teo39_Transend_desc;

typedef struct teo39_dbsink_ABAPRead {
    OmsTypeInt4 TabHandle;
    tsp00_Int4  count;
    tsp00_Int4  buflen;
} teo39_dbsink_ABAPRead;

typedef struct teo39_dbsink_ABAPRead_rply {
    OmsTypeInt4 TabHandle;
    tsp00_Int4  count;
    tsp00_Int2  e;
} teo39_dbsink_ABAPRead_rply;

typedef struct teo39_dbsink_ABAPWrite {
    OmsTypeInt4 TabHandle;
	  tsp00_Int4  row_size;
    tsp00_Int4  row_count;
} teo39_dbsink_ABAPWrite;
		
typedef struct teo39_dbsink_ABAPWrite_rply {
    tsp00_Int2 e;
} teo39_dbsink_ABAPWrite_rply;

typedef struct  teo39_dbsink_AllocPacket_rply
{
    tsp00_Int4             sz;
} teo39_dbsink_AllocPacket_rply;

typedef struct  teo39_dbsink_GetObj
{
    tgg91_TransNo           consistentView; /* prefix of reply, don't change */
    tgg00_ContainerId       containerId;    /*          "    "               */
    tgg00_OmsVersionContext versionContext;
    OmsTypeOid              oid;
    bool                    do_lock;
    bool                    inVersion;
    tsp00_Int4              rec_size;
} teo39_dbsink_GetObj;

typedef struct  teo39_dbsink_GetObj_rply
{
    tgg91_TransNo     consistentView;
    tgg00_ContainerId containerId;
    tgg91_PageRef     ResultSeq;
    tsp00_Int2        e;
    tsp00_Int4        ObjLen; /* +++ Change Request 1000203, T.A. 01.12.1998 */
} teo39_dbsink_GetObj_rply;

typedef struct  teo39_dbsink_GetObjWithKey
{
    tgg91_TransNo           consistentView;  /* prefix of reply, don't change */
    tgg00_ContainerId       containerId;     /*             "                 */
    tgg00_OmsVersionContext versionContext;
    tsp00_Int4              keylen;
    char                    getConsistent;
    char                    inVersion;
    tsp00_Int4              bodysize;
} teo39_dbsink_GetObjWithKey;

typedef struct  teo39_dbsink_GetObjWithKey_rply
{
    tgg91_TransNo        consistentView;
    tgg00_ContainerId    containerId;
    tsp00_Int2           e;
    tgg92_KernelOidSpace oid;
    tsp00_C6             ResultSeq;
} teo39_dbsink_GetObjWitkKey_rply;

typedef struct  teo39_dbsink_NextObj
{
    tgg92_KernelOidSpace    oid;            /* prefix of reply */
    tgg91_TransNo           consistentView; /* don't change !  */
    char                    do_lock;
    char                    with_object;
    tgg00_ContainerId       containerId;
    tgg00_OmsVersionContext versionContext;
    tsp00_Int4              rec_size;
    char                    inVersion;
} teo39_dbsink_NextObj;

typedef struct  teo39_dbsink_NextObj_rply
{
    tgg92_KernelOidSpace oid;
    tgg91_TransNo   consistentView;
    tsp00_C6        ResultSeq;
    tsp00_Int2      e;
} teo39_dbsink_NextObj_rply;


typedef struct  teo39_dbsink_NewObj
{
    tgg91_TransNo           consistentView;
    tgg00_ContainerId       containerId;
    tgg00_OmsVersionContext versionContext;
    tsp00_Int4              KeyPos;
    tsp00_Int4              KeySize;
    char                    inVersion;
} teo39_dbsink_NewObj;

typedef struct  teo39_dbsink_NewObj_rply
{
    tgg92_KernelOidSpace oid;
    tsp00_C6        ResultSeq;
    tsp00_Int2      e;
} teo39_dbsink_NewObj_rply;

typedef struct  teo39_dbsink_UpdateObj
{
    tgg91_TransNo           consistentView;
    tgg00_ContainerId       containerId;
    tgg00_OmsVersionContext versionContext;
    tgg92_KernelOidSpace    oid;
    tsp00_C6                WantedSeq;
    char                    inVersion;
    tsp00_Int4              bodysize;
    tsp00_Int2              colHashCount_gg00;
    tsp00_Int2              colInvCount_gg00;
} teo39_dbsink_UpdateObj;

typedef struct  teo39_dbsink_LockObj
{
    tgg91_TransNo        consistentView;
    tgg00_ContainerId    containerId;
    tgg92_KernelOidSpace oid;
    tsp00_C6             WantedSeq;
} teo39_dbsink_LockObj;

typedef struct  teo39_dbsink_UnlockObj
{
    tgg00_ContainerId    containerId;
    tgg92_KernelOidSpace oid;
} teo39_dbsink_UnlockObj;

typedef struct  teo39_dbsink_RegisterClass
{
    tgg00_ObjColDesc    invcoldesc;
    tsp00_C16           guid;
    tsp00_KnlIdentifier className;
    tsp00_Int4          size;
    tsp00_Int4          schemaId;
    tsp00_Int2          keylen;
    OmsContainerNo      container_no;
} teo39_dbsink_RegisterClass;

typedef struct  teo39_dbsink_RegisterClass_rply
{
    tgg00_ObjColDesc   invcoldesc;
    tgg00_ContainerId  containerId;
    tsp00_Int2         e;
} teo39_dbsink_RegisterClass_rply;

typedef struct  teo39_dbsink_ErrorMsg
{
    short          errorno;
    tsp00_CodeType msgType;
    unsigned char  filler;
    tsp00_Int4     msglen;
} teo39_dbsink_ErrorMsg;

typedef struct teo39_dbsink_MultiPurpose {
    unsigned char MessType;
    unsigned char MessType2;
    tsp00_Int4          buflen;
} teo39_dbsink_MultiPurpose;

typedef struct teo39_dbsink_MultiPurpose_rply {
    short         b_error;
    tsp00_Int4          buflen;
} teo39_dbsink_MultiPurpose_rply;

typedef struct  teo39_dbsink_DeleteAllObj
{
    tgg00_ContainerId       containerId;
    tgg00_OmsVersionContext VersionContext;
    char                    inVersion;
} teo39_dbsink_DeleteAllObj;

typedef struct  teo39_dbsink_DeleteAllObj_rply
{
    tsp00_Int4      cntDeleted;
    tsp00_Int2      e;
} teo39_dbsink_DeleteAllObj_rply;

typedef struct  teo39_dbsink_CloseVersion
{
    OmsVersionId versionId;
    tsp00_Int4   lastKey;
} teo39_dbsink_CloseVersion;


typedef struct  teo39_dbsink_CreateOpenVersion
{
    tgg91_TransNo consistentView;
    char          createVersion;
    OmsVersionId  versionId;
} teo39_dbsink_CreateOpenVersion;

typedef struct  teo39_dbsink_CreateOpenVersion_rply
{
    tgg00_OmsVersionContext VersionContext;
    tsp00_Int2              e;
} teo39_dbsink_CreateOpenVersion_rply;

typedef struct teo39_dbsink_CriticalSection
{
  short RegionId;
} teo39_dbsink_CriticalSection;

typedef struct  teo39_dbsink_RenameVersion
{
    OmsVersionId oldVersionId;
    OmsVersionId newVersionId;
} teo39_dbsink_RenameVersion;

typedef struct  teo39_dbsink_EndConsistentView
{
    tgg91_TransNo consistentView;
} teo39_dbsink_EndConsistentView;

typedef struct  teo39_dbsink_EndConsistentView_rply
{
    tgg91_TransNo consistentView;
    tsp00_Int2    e;
} teo39_dbsink_EndConsistentView_rply;

typedef struct  teo39_dbsink_MessageFile
{
  tsp00_Int4       i4;
  tgg00_BasisError e;
  boolean          doAppend;
  char             msg[4096*4];
} teo39_dbsink_MessageFile;

typedef struct  teo39_dbsink_ExistsFile
{
  bool             doesExist;
  bool             isDirectory;
  tgg00_BasisError e;
  char             msg[4096*4];
} teo39_dbsink_EsistsFile;


typedef struct  teo39_dbsink_GetDateTime_rply
{
    tsp00_Date date;
    tsp00_Time time;
} teo39_dbsink_GetDateTime_rply;

typedef struct  teo39_dbsink_GetClock_rply
{
    tsp00_Int4 sec;
    tsp00_Int4 msec;
} teo39_dbsink_GetClock_rply;

typedef struct teo39_dbsink_DropVersion 
{
   tgg00_OmsVersionContext versionContext;
} teo39_dbsink_DropVersion;

typedef struct teo39_dbsink_HistoryUsed
{
  tgg00_BasisError     e;
  bool                 isUsed;
  tgg00_ContainerId    containerId;
  OmsTypeOid           oid;
} teo39_dbsink_HistoryUsed;

typedef struct  teo39_vdcomReleaseAll
{
  bool         remoteFlag;
} teo39_vdcomReleaseAll;

typedef struct  teo39_vdcomReleaseAll_rply
{
  HRESULT      hr;
} teo39_vdcomReleaseAll_rply;

typedef struct teo39_dbsink_CreateSchema 
{
  tsp00_KnlIdentifier schemaName;
} teo39_dbsink_CreateSchema; 

typedef struct teo39_dbsink_CreateSchema_rply
{
  tgg00_BasisError e;
  tsp00_Int4       schemaId;
} teo39_dbsink_CreateSchema_rply;

typedef struct teo39_dbsink_DropSchema 
{
  tsp00_Int4 schemaId;
} teo39_dbsink_DropSchema; 

typedef struct teo39_dbsink_GetSchemaName 
{
  tsp00_Int4 schemaId;
} teo39_dbsink_GetSchemaName; 

typedef struct teo39_dbsink_GetSchemaName_rply 
{
  tgg00_BasisError    e;
  tsp00_KnlIdentifier schemaName;
} teo39_dbsink_GetSchemaName_rply; 

typedef struct teo39_dbsink_Path_rply
{
  tgg00_BasisError    e;
  tsp00_Pathc         path;
} teo39_dbsink_Path_rply;

typedef struct teo39_dbsink_DropContainer
{
  OmsSchemaHandle schemaId;
  GUID            guid;
  OmsContainerNo  cno;
} teo39_dbsink_DropContainer;

typedef struct teo39_dbsink_GetContainerInfo
{
  tsp00_Uint4 containerHandle;
} teo39_dbsink_GetContainerInfo;

typedef struct teo39_dbsink_GetContainerInfo_rply
{
  tgg00_BasisError e;
  GUID             guid;
  OmsSchemaHandle  schema;
  OmsContainerNo   cno;
} teo39_dbsink_GetContainerInfo_rply;

/* ** PTS 1107355 ** FF * 21-Aug-2000 */
typedef struct  teo39_vdcomSetEnvironment
{
  char             envStr[256];
} teo39_vdcomSetEnvironment;

typedef enum {
    vdcomRequest_eo39, 
    vdcomReply_eo39, 
    iSinkRequest_eo39, 
    iSinkReply_eo39, 
    iSinkRmtRequest_eo39, 
    iSinkRmtReply_eo39 
} teo39_MsgKind;

typedef enum { 
    vdcomGetDispaddr_eo39,
    coGetClassObject_eo39,
    vdcomCreateInstance_eo39, 
    vdcomDbproc_eo39,
    vdcom_paraminfo_eo39,
    vdcomTransEnd_eo39,
    dbsink_ABAPRead_eo39,
    dbsink_ABAPWrite_eo39,
    dbsink_AllocPacket_eo39,
    dbsink_SQLRequest_eo39,
    dbsink_SQLReceive_eo39,
    dbsink_GetErrorMsg_eo39,
    dbsink_SetError_eo39,
    dbsink_BasisError_eo39,
    dbsink_HResultError_eo39,
    dbsink_Vtrace_eo39,
    dbsink_LockObj_eo39,
    dbsink_Rollback_eo39,
    dbsink_GetObj_eo39,
    dbsink_GetObjWithKey_eo39,
    dbsink_UpdateObj_eo39,
    dbsink_NewObj_eo39,
    dbsink_RegisterClass_eo39,
    dbsink_NextObj_eo39,
    dbsink_DeleteObj_eo39,
    dbsink_SubtransCommit_eo39,
    dbsink_SubtransRollback_eo39,
    dbsink_SubtransStart_eo39,
    dbsink_MultiPurpose_eo39,
    dbsink_UnlockObj_eo39,
    dbsink_ConsoleMsg_eo39,
    dbsink_IsLocked_eo39,
    dbsink_DeleteAllObj_eo39,
    dbsink_EnterCriticalSection_eo39,
    dbsink_LeaveCriticalSection_eo39,
    dbsink_NewConsistentView_eo39,
    dbsink_EndConsistentView_eo39,
    dbsink_CloseVersion_eo39,
    dbsink_CreateOpenVersion_eo39,
    dbsink_RenameVersion_eo39,
    dbsink_CloseFile_eo39,
    dbsink_DropFile_eo39,
    dbsink_ExistsFile_eo39,
    dbsink_OpenFile_eo39,
    dbsink_SyncFile_eo39,
    dbsink_WriteFile_eo39,
    dbsink_GetDateTime_eo39,
    dbsink_GetClock_eo39,
    dbsink_DropVersion_eo39,
    vdcomReleaseAll_os39,
    dbsink_CreateSchema_eo39,
    dbsink_ExistsSchema_eo39,
    dbsink_DropSchema_eo39,
    dbsink_GetSchemaName_eo39,
    dbsink_GetWorkingDir_eo39,
    dbsink_GetDbrootSapDir_eo39,
    dbsink_LoadLibrary,
    dbsink_DropContainer_eo39,
    dbsink_GetContainerInfo_eo39,
    dbsink_HistoryUsed_eo39,
    dbsink_ExistsContainer_eo39,   /* PTS 1110334 */
    vdcomSetEnvironment_os39                             /* PTS 1107355 */
} teo39_ParamKind;

typedef struct  
{
    tsp00_Int4         hdbgBufSize;
    tsp00_Int4         hdbgBufLen;
    teo39_MsgKind      hdbgMsgKind;
    tsp00_Int4         hdbgParamCount;
} teo39_DebugMsgHdr;

typedef struct
{
    teo39_ParamKind     pdbgKind;
    union
    {
        tsp_dcom_co_create_instance_parms   CoCreateP;
        tsp_dcom_dispatch_parms             DispP;
        tsp_dcom_paraminfo_parms            ParamInfoP;
        teo39_dbsink_ABAPRead               ABAPReadP;
        teo39_dbsink_ABAPRead_rply          ABAPRead_rplyP;
        teo39_dbsink_ABAPWrite              ABAPWriteP;
        teo39_dbsink_ABAPWrite_rply         ABAPWrite_rplyP;
        teo39_dbsink_AllocPacket_rply       AllocPacket_rplyP;
        teo39_dbsink_DeleteAllObj           DeleteAllObjP;
        teo39_dbsink_DeleteAllObj_rply      DeleteAllObj_rplyP;
        teo39_dbsink_ErrorMsg               ErrorMsgP;
		    short							                  BasisErrorP;
		    HRESULT							                HResultErrorP;
		    tsp00_Int4						              BuflenP;
        teo39_dbsink_GetObj                 GetObjP;
        teo39_dbsink_GetObj_rply            GetObj_rplyP;
        teo39_dbsink_GetObjWithKey          GetObjWithKeyP;
        teo39_dbsink_GetObjWithKey_rply     GetObjWithKey_rplyP;
        teo39_dbsink_LockObj                LockObjP;
        teo39_dbsink_NextObj                NextObjP;
        teo39_dbsink_NextObj_rply           NextObj_rplyP;
        teo39_dbsink_UpdateObj              UpdateObjP;
        teo39_dbsink_NewObj                 NewObjP;
        teo39_dbsink_NewObj_rply            NewObj_rplyP;
        teo39_dbsink_RegisterClass          RegisterClassP;
        teo39_dbsink_RegisterClass_rply     RegisterClass_rplyP;
        teo39_dbsink_MultiPurpose           MultiPurposeP;
        teo39_dbsink_MultiPurpose_rply      MultiPurpose_rplyP;
        teo39_dbsink_UnlockObj              UnlockObjP;
        teo39_dbsink_CloseVersion           CloseVersionP;
        teo39_dbsink_CreateOpenVersion      CreateOpenVersionP;
        teo39_dbsink_CreateOpenVersion_rply CreateOpenVersion_rplyP;
        teo39_dbsink_EndConsistentView      EndConsistentViewP;
        teo39_dbsink_EndConsistentView_rply EndConsistentView_rplyP;
        teo39_dbsink_RenameVersion          RenameVersionP;
        teo39_dbsink_CriticalSection        CriticalSectionP;
        teo39_dbsink_MessageFile            MessageFileP;
        teo39_dbsink_ExistsFile             ExistsFileP;
        teo39_dbsink_GetDateTime_rply       GetDateTime_rplyP;
        teo39_dbsink_GetClock_rply          GetClock_rplyP;
        teo39_dbsink_DropVersion            DropVersionP;
        teo39_vdcomReleaseAll_rply          ReleaseAll_rplyP;
        teo39_vdcomReleaseAll               ReleaseAllP;
        teo39_dbsink_CreateSchema           CreateSchemaP;
        teo39_dbsink_CreateSchema_rply      CreateSchema_rplyP;
        teo39_dbsink_DropSchema             DropSchemaP;
        teo39_dbsink_GetSchemaName          GetSchemaNameP;
        teo39_dbsink_GetSchemaName_rply     GetSchemaName_rplyP;
        teo39_dbsink_Path_rply              Path_rplyP;
        teo39_dbsink_DropContainer          DropContainerP;
        teo39_dbsink_GetContainerInfo       GetContainerInfoP;
        teo39_dbsink_GetContainerInfo_rply  GetContainerInfo_rplyP;
        teo39_dbsink_HistoryUsed            HistoryUsedP;
        teo39_vdcomSetEnvironment           SetEnvironment;            /* PTS 1107355 */
    } v;
} teo39_DebugMsgParm;

typedef struct  teo39_DebugMsg
{
    teo39_DebugMsgHdr   dbghdr;
    teo39_DebugMsgParm  dbgMsgParm;
} teo39_DebugMsg;

typedef struct  teo39_TransferP
{
    teo39_DebugMsg      *dbgMsg;
    tsp00_Int4          session;
    unsigned char       *SQLBuffer_ptr;
    unsigned char       *SQLBuffer_ptr2;
    tsp00_Int4          SQLBuffer2_size;
    long                SQLBuffer2_length; 
    HRESULT             hr;
} teo39_TransferP;

#define in_eeo39		0 
#define out_eeo39		1 
#define inout_eeo39		2

typedef struct teo39_TStack
{
    union
    {
        unsigned char   i1;
        unsigned short  i2;
        tsp00_Int4      i4;
        float           r4;
        double          r8;
        void*           ptr;
    } value;
    unsigned short      vt;
    unsigned char       inout_param; 
    bool                is_ptr;
    tsp00_Int4          length;
} teo39_TStack;
 
#endif
