/*!***************************************************************************

  module      : heo12.h

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: getopt Wrapper
  description : Needed to hide POSIXLY INCORRECT GNU getopt implementation


  last changed: 2000-01-27  17:59
  see also    : example.html ...

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1999



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef HEO12_H
#define HEO12_H



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "geo00_0.h"
#include "gsp00.h"

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/

/*!
  Function:     sqlgetopt
  description:  This function warppes to support POSIX CORRECT getopt implementation
    
    See getopt() man pages for detailed information. This wrapper was needed to
    support LINUX or other ports that use GNU getopt() implementation


  arguments:
    argc [in] argument count
    argv [in] argument vector
    optstring [in] option parsing input
    extern int opterr Set this to 0 to suppress error messages of getopt
    
  return value: parsed option character
    <UL>
        <LI>EOF: End of options
        <LI>'?':   Unknown option or missing option argument found
        <LI>extern char *optarg points to the start of the option argument
        <LI>extern int   optind is the index into argv of the next argument to process
        <LI>extern int   optopt is the value of the error producing option character
  </UL>
 */
externC int sqlgetopt(int argc, char * const argv[], const char *optString);

/*!
  Function:     sqlgetpid
  description:  This function return the current process id
    
  Dont mix this up with the SAPDB kernel idea of PROCESS_ID, which is 
  a tsp00_TaskId. 

  arguments:
    pPid [out] Pointer of process id to be returned
 */
externC void sqlgetpid ( tsp00_Int4 *pid );

/*!
  Function: sqlSetOptionEncoding
  Description: Set the encoding type for command line options

  Only the programmer of the main routine knows the command line option
  encoding. This call allows him to tell his setting to other routines.
  This function should be called once before option parsing via sqlarg3.
  It influences the conversion of options especially for username and
  password.

  If not called before sqlarg3, sqlarg3 must assume arguments are passed as ASCII8

  Arguments: isUTF8encoded [in] true if arguments are UTF8 encoded, false if ASCII8
 */
externC void sqlSetOptionEncodingUTF8(bool isUTF8encoded);

/*!
  Function: sqlIsOptionEncodingUTF8
  Description: Get the encoding type for command line options

  If sqlSetOptionEncodingUTF8 was never called before,
  the default false (meaning ASCII8) is returned.

  Return value: true (UTF8) or false (ASCII8).
 */
externC bool sqlIsOptionEncodingUTF8();

#endif  /* HEO12_H */
