/****************************************************************************

  module      : heo52.h

  -------------------------------------------------------------------------

  responsible : FrankH

  special area: time 
  description : description ...


  last changed: 1999-10-28  13:20
  see also    : example.html ...

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1998



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef HEO52_H
#define HEO52_H


/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/
#include "gsp00.h"
#include "heo00.h"
#include "geo52.h"

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/

/*!
  Function: vclock
  Description: Return seconds and microseconds since start of database
  Arguments: Sec      [out] seconds since start
             MicroSec [out] microseconds of last second (0-999999)
 */
externC void
vclock            ( teo00_Int4 * Sec,
                    teo00_Int4 * MicroSec);

/*!
  Function: vtimestamp
  Description: Return current time as timestamp

  Format of timestamp is  YYYYMMDDhhmmssuuuuuu

  Y Year digits
  M month digits
  D day digits
  h hour digits
  m minute digits
  s second digits
  u microsecond digits

  Arguments: pStamp [out]timestamp to fill
 */
externC void
vtimestamp        ( tsp00_Timestamp VAR_ARRAY_REF pStamp);

/*!
  Function: vUTCtimestamp
  Description: Return current UTC time as timestamp

  Format of timestamp is  YYYYMMDDhhmmssuuuuuu

  Y Year digits
  M month digits
  D day digits
  h hour digits
  m minute digits
  s second digits
  u microsecond digits

  Arguments: pStamp [out]timestamp to fill
 */
externC void
vUTCtimestamp        ( tsp00_Timestamp VAR_ARRAY_REF pStamp);

/*!
  Function: vdattime
  Description: Return current date and time

  Format of date is YYYYMMDD. Format of time is hhmmss

  Y Year digits
  M month digits
  D day digits

  h hour digits
  m minute digits
  s second digits

  Arguments: pDate [out] date to fill
             pTime [out] time to fill
 */
externC void
vdattime          ( tsp00_Date VAR_ARRAY_REF pDate,
                    tsp00_Time VAR_ARRAY_REF pTime );

/*
  Function: vTimeZoneDelta
  Description: Return difference between local time and GMT in hours

  Berlin winter time result 1.

  Berlin summer time result 2.

  New Zealand Chatham Islands result 12,75 !!!!

  NOTE: If weekday difference between localtime and systemtime is larger
  than a day, 0. is returned and a warning message is issued in knldiag.

  Return value: Difference in hours
 */
externC tsp00_Longreal vTimeZoneDelta();

/*!
  Function: eo52UpdateTaskStateStatisticRec
  Description: Update relative and absolute time record

  The difference (leaveDispatcher - enterDispatcher) is added to the AbsoluteTime record.
  The difference (enterRunqueue - enterDispatcher) is added to the RelativeTime record.

  Arguments: pRec [inout] record to update
             pState [in] record containing dispatcher timestamps
 */
externC void eo52UpdateTaskStateStatisticRec(
 teo52_RelativeAndAbsoluteTime *pRec,
 teo52_TaskStateStatisticRec   *pState );

#endif  /* HEO52_H */
