/*!***************************************************************************

  module      : heo78.h

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: DCOM Registration for UNIX
  description : Registration Functions for Class implementation and Kernel.
				
  
  last changed: 1998-09-29  11:22
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1998



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/


#ifndef HEO78_H
#define HEO78_H

/*!
  Chapter: Convention

  description:
    This API handles DCOM Registration function.

	'eo78_ComObject' Prefix is used for all routines in this API
	All functions return an enumeration teo78_Result.
	Memory has to allocated by the caller. Input/Output parameter
	must point to sufficient space. This is easy to realize by
	using the predefined types teo78_... only. Strings are always 
	terminated by 0.
 */
/*!
  EndChapter: Convention
 */

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define PROG_ID_PFX_MXEO78 32
#define PROJECT_MXEO78     32 
#define VERSION_MXEO78     12
/*
 * Programmatic Identifier  consists of the three items above plus
 * two delimiters (dots) replacing the 0 bytes... This is the maximum
 * approach...
 * Ferdinand Flohe mentioned a necessary size limitation for AS/400.
 */
#ifdef AS400
#define PROG_ID_MXEO78 20
#else
#define PROG_ID_MXEO78 \
	(PROG_ID_PFX_MXEO78 + PROJECT_MXEO78 + VERSION_MXEO78)
#endif
#define CLSID_MXEO78       39
#define MODULE_NAME_MXEO78 256

/*===========================================================================*
 *  TYPES                                                                    *
 *===========================================================================*/

typedef char teo78_ProgIDPfx[PROG_ID_PFX_MXEO78];
typedef char teo78_Project[PROJECT_MXEO78];
typedef char teo78_Version[VERSION_MXEO78];
typedef char teo78_ProgID[PROG_ID_MXEO78];
typedef char teo78_ClsID[CLSID_MXEO78];
typedef char teo78_ModuleName[MODULE_NAME_MXEO78];
typedef enum teo78_Result {
	success_eo78,
  illegal_parameter_eo78,
	access_denied_eo78,
	doubled_clsid_eo78,
	entry_not_found_eo78,
	clsid_version_mismatch_eo78,
  system_error_eo78
} teo78_Result;

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

/* none */

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/

/*!
  Chapter: Registration Functions
 */
/*!  
  Function: sqlComObjectRegister
  
  description:
	This call registers a specified Com Object

    It is possible to register more then one version of the com object.
	In such a case the last registration wins! That means the last 
	registered version overwrites the version independent programmatic
	identifier entry. CLSID must be unique in the system.

  arguments:
	ProgIDPfx  [in]  Project Prefix, i.e. "LiveCache"
    Project    [in]  The Project name identifying the Class
    Version    [in]  The Version
	ClsID      [in]  Class Identifier 
	ModuleName [in]  Name of DLL containing the Class implementation
				
  return value: 
	  The code returned
	<UL>
		<LI>success_eo78:           Success: Entry was already in registry
		<LI>entry_not_found_eo78:   Success: Entry added to registry
		<LI>illegal_parameter_eo78: Input parameter check failed
		<LI>access_denied_eo78:     Failed to access registry
		<LI>doubled_clsid_eo78:     CLSID already used for other registered project
	</UL>
*/
extern "C" teo78_Result sqlComObjectRegister( 
						      teo78_ProgIDPfx   ProgIDPfx,
						      teo78_Project     Project,
						      teo78_Version     Version,
						      teo78_ClsID       ClsID,
						      teo78_ModuleName  ModuleName );

/*!  
  Function: sqlComObjectUnregister
  
  description:
	This call unregisters a specified Com Object.

    The information registered has to be removed. In the special case, 
	that the current version is removed, there is no other version
	select as current version, but no version is current version.
	Check is done for matching version/ClsID combination. No deletion
	is done if mismatch found.

  arguments:
	ProgIDPfx  [in]  Project Prefix, i.e. "LiveCache"
    Project    [in]  The Project name identifying the Class
    Version    [in]  The Version
	ClsID      [in]  Class Identifier 
				
  return value: 
	  The code returned
	<UL>
		<LI>success_eo78:           Success
		<LI>illegal_parameter_eo78: Input parameter check failed
		<LI>access_denied_eo78:     Failed to access registry (i.e. dones not exist)
		<LI>clsid_version_mismatch_eo78: CLSID/ProjectVersion mismatch
		<LI>entry_not_found_eo78:   Project not found
	</UL>
*/
extern "C" teo78_Result sqlComObjectUnregister( 
						      teo78_ProgIDPfx ProgIDPfx,
						      teo78_Project   Project,
						      teo78_Version   Version,
						      teo78_ClsID     ClsID );

/*!  
  Function: sqlComObjectGet
  
  description:
	This call retrieves a module name and ClsID usign given programmatic ID.

    This call is used as helper function for the DCOM implementation.

  arguments:
	ProgID     [in]  Programmatic Identifier
    ClsID      [out] Filled with ClsID found
    ModuleName [out] Filled with ModuleName found
				
  return value: 
	  The code returned
	<UL>
		<LI>success_eo78:           Success
		<LI>illegal_parameter_eo78: Input parameter check failed
		<LI>access_denied_eo78:     Failed to access registry (i.e. dones not exist)
		<LI>entry_not_found_eo78:   Programmatic Identifier not found
	</UL>
*/
extern "C" teo78_Result sqlComObjectGet(
						      teo78_ProgID     ProgID,
						      teo78_ClsID      ClsID,
						      teo78_ModuleName ModuleName);

/*!  
  Function: sqlComObjectGetByClsID
  
  description:
	This call retrieves a module name using given ClsID

    This call is used as helper function for the DCOM implementation.
    Used by Kernel or Proxy server only.

  arguments:
	ClsID      [in]  Class Identifier 
    ModuleName [out] Filled with ModuleName found
				
  return value: 
	  The code returned
	<UL>
		<LI>success_eo78:           Success
		<LI>illegal_parameter_eo78: Input parameter check failed
		<LI>access_denied_eo78:     Failed to access registry (i.e. dones not exist)
	    <LI>invalid_clsid_eo78:     ClsID not found
	</UL>
*/
extern "C" teo78_Result sqlComObjectGetByClsID(
						      teo78_ClsID      ClsID,
						      teo78_ModuleName ModuleName );

/*!
  EndChapter: Registration Functions
 */
#endif  /* HEO78_H */
