/* @lastChanged: "1999-05-22  9:25"

  * @filename:   veo36.cpp
  * @purpose:    "DCOM Inspection for DB-Procedure characteristics"
  * @release:    7.2.0.0
  * @see:        "-.-"
  * @author:     Ferdinand Flohe (D024849 - Ferdinand.Flohe@SAP-AG.de)
  * @copyright:  "Copyright by SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/

/* ================================================================================ */
/* ==========              REPLACEMENTS BY THE PREPROCESSOR               ========= */
/* ================================================================================ */

#define MAXPARAMS_EO36 256
#define INITGUID     1        /* Define needed CoClassIDs + IIDs */


/* ========================================================================== */
/* ==========              DECLARATION OF TYPES                     ========= */
/* ========================================================================== */

#include "gsp00.h"
#include "vsp001.h"

#include "heo60.h"
#include "hak34.h"
#include "hak341.h"
#include "geo60.h"

#include "gco00.h"
#include "geo007_1.h"

#include "geo00d.h"
#include "livecachetypes.h"  /* no check */
#include "heo36.h"
#include "hsp26.h"
#include "geo36.h"
#include "heo38.h"

typedef char t_part_data[sizeof(tsp00_KnlIdentifier) + 2];
typedef tsp1_dcom_param_info *dcom_param_info_ptr;
typedef BSTR                  ParamNames_eo36[MAXPARAMS_EO36];

#define ERROR_NO_STREAM ERROR_FILE_NOT_FOUND

/* ========================================================================== */
/* ==========              DECLARATION OF CONSTANTS                 ========= */
/* ========================================================================== */

/* ========================================================================== */
/* ==========               EXTERN FUNCTION PROTOTYPES              ========= */
/* ========================================================================== */

externC
{
  typedef void ( __stdcall *CO_INTROSPECTOBJECT) (  LPCLSID        *lpClsId,
    LPIID          *lpIId,
    unsigned short *lpNum,
    long           *lpVersion );

  enum Co_AccessType { get_by_name, get_by_index };

  typedef void ( __stdcall *CO_INTROSPECTMETHOD) ( Co_AccessType   accessType,
    char            **lpMethod,
    unsigned short  Num,
    unsigned short  *lpIdx,
    unsigned short  *lpParamCount,
    void            **lplpIntrospecFunc );

  typedef void ( __stdcall *CO_INTROSPECTMETHOD_MO) ( Co_AccessType   accessType,
    char            **lpMethod,
    unsigned short  Num,
    unsigned short  *lpIdx,
    unsigned short  *lpParamCount,
    void            **lplpIntrospecFunc_MO );
}

typedef void ( __stdcall *CO_INTROSPECTFUNCTION) (unsigned short ParmIdx,
                                                  char           **lplpMode,
                                                  char           **lplpIOtype,
                                                  short int      *lpDatatype,
                                                  short int      *lpSubdatatype,
                                                  short int      *lpLength,
                                                  short int      *lpDim,
                                                  long           *lpOffset,
                                                  unsigned char  *lpFrac,
                                                  LPGUID         *lplpGUID,
                                                  char           **lplpParmName );

typedef void ( __stdcall *CO_INTROSPECTFUNCTION_MO) (unsigned short ParmIdx,
                                                     char           **lplpMode,
                                                     char           **lplpIOtype,
                                                     short int      *lpDatatype,
                                                     short int      *lpSubdatatype,
                                                     short int      *lpLength,
                                                     short int      *lpDim,
                                                     int            *lpOffsetServer,
                                                     int            *lpOffsetAscClient,
                                                     int            *lpOffsetUniClient,
                                                     unsigned char  *lpFrac,
                                                     LPGUID         *lplpGUID,
                                                     char           **lplpParmName );

struct teo36_ParamGlob {
    tsp1_segment_ptr            lpSegment;
    tsp1_part_header            *lpPartHdr;
    tsp_dcom_paraminfo_parms    *Parms;
    dcom_param_info_ptr         lpParamInfo;
    dcom_param_info_ptr         lpStreamParamInfo;
    int                         OutParams;
    tsp00_C4                    session;
    CO_INTROSPECTOBJECT         co_IntroSpectObject;
    CO_INTROSPECTMETHOD         co_IntroSpectMethod;
    CO_INTROSPECTMETHOD_MO      co_IntroSpectMethod_MO;
    CO_INTROSPECTFUNCTION       co_IntroSpectFunction;
    CO_INTROSPECTFUNCTION_MO    co_IntroSpectFunction_MO;
};


/* ========================================================================== */
/* ==========               LOCAL FUNCTION PROTOTYPES               ========= */
/* ========================================================================== */

void eo36_DescribeParamNames (teo36_ParamGlob &param_glob,
                              ParamNames_eo36    &ParmNames,
                              unsigned int       cArgs);

HRESULT eo36_GetInterface (teo36_ParamGlob          &param_glob,
                           tsp_dcom_paraminfo_parms *parms,
                           unsigned short           *pNumParms);

HRESULT eo36_GetMethod (teo36_ParamGlob          &param_glob,
                        tsp_dcom_paraminfo_parms *parms,
                        unsigned short           *pNumParms,
                        BOOLEAN                  &is_last_func);

void eo36_Init (teo36_ParamGlob       &param_glob,
                   tsp_dcom_paraminfo_parms *parms);

void eo36_InitDcomParamInfo(tsp1_dcom_param_info &paramInfo);

HRESULT eo36_NewParamInfo(teo36_ParamGlob &param_glob,
                          int                vt);

HRESULT eo36_OneParameter (teo36_ParamGlob          &param_glob,
                           tsp_dcom_paraminfo_parms *parms,
                           unsigned short           idx);

void eo36_SetError (long e,
                   teo36_ParamGlob &param_glob);


/* ========================================================================== */
/* ==========              DEFINITION OF GLOBAL FUNCTIONS           ========= */
/* ========================================================================== */

externC void
eo36ParamInfo (tsp_dcom_paraminfo_parms * parms)
{
  CHRESULT_EO00D     hr(parms->session,"sql37k_paraminfo");
  tsp1_packet_ptr    lpPacket;
  unsigned short     i;
  unsigned short     cArgs;
  ParamNames_eo36    ParmNames;
  BOOLEAN            is_function = 1;
  BOOLEAN            is_last_func = FALSE;
  teo36_ParamGlob parm_glob;

  eo36_Init (parm_glob, parms);

  /*                                                                   */
  /* Visual Basic uses as ProgID: '<Project>.<Class>'                  */
  /*            as Componentname: '<Class>'                            */
  /*            as Interfacename: '_<Class>'                           */
  /*               as Methodname: '<Method>'                           */
  /*                                                                   */
  /* <Project>.<Class> is used as ProgID to be found in the Registry   */
  /*                   from the ProgID the Components CLSID is found   */
  /*                   from the CLSID the TypeLibID is extracted       */
  /*                                                                   */
  /* Within the TypeLib the CLSID is used to localize the Component    */
  /* description.                                                      */
  /* It is assumed, that the Component has ONLY ONE Interface.         */
  /* Then this Interface is searched for the <Method>.                 */
  /*                                                                   */

  if (NULL == parms->typeinfo_ptr)
    hr = eo36_GetInterface ( parm_glob, parms, &cArgs );
  else
    hr = eo36_GetMethod ( parm_glob, parms, &cArgs, is_last_func );

  if (ERROR_SUCCESS == hr.hresult())
  {
    for (i = 1; (i < (cArgs + 1)) && (hr.succeeded()); i++)
    {
      hr = eo36_OneParameter (parm_glob, parms, i);
    }
  }
  if (ERROR_SUCCESS == hr.hresult())
  {
    /* +++           PTS 1110591                                          +++ */
    tsp1_part_ptr  partPtr = REINTERPRET_CAST(tsp1_part_ptr,parm_glob.lpPartHdr); 
    if (( s26partlen(*partPtr) - (int)sizeof (partPtr->sp1p_part_header()) ) > 
      partPtr->sp1p_buf_size() )
    {
      eo36_SetError ( GEO00D_DCOM_BUFFER_LIMIT, parm_glob);
    }
    else
    {
      /* ---           PTS 1110591                                        --- */
      s26finish_part ( REINTERPRET_CAST(tsp1_packet_ptr,parms->packet),
        *REINTERPRET_CAST(tsp1_part_ptr,parm_glob.lpPartHdr) );
      eo36_DescribeParamNames (parm_glob, ParmNames, cArgs);
      lpPacket = REINTERPRET_CAST(tsp1_packet_ptr,parms->packet);
      lpPacket->sp1_header.sp1h_varpart_len =   sizeof(lpPacket->sp1_header) +
        lpPacket->sp1_segm().sp1s_segm_header().sp1s_segm_len();
    }
  }
  else
    eo36_SetError (hr.hresult(), parm_glob);

  if (is_last_func || ( ERROR_SUCCESS != hr.hresult()) ||
     (parms->db_obj_type == dbo_dbproc ) ||
     (dbo_dbfunc == parms->db_obj_type) || (dbo_package == parms->db_obj_type))
  {
    if ( NULL != parms->typeinfo_ptr )
    {
      bool ok;
      char errtext[64];
      eo38FreeLibrary((HANDLE)parms->typeinfo_ptr, &ok, errtext, sizeof(errtext));
      parms->typeinfo_ptr = NULL;
    }
  }
}

/* ========================================================================== */
/* ==========           IMPLEMENTATION OF LOCAL FUNCTIONS           ========= */
/* ========================================================================== */

HRESULT eo36_GetInterface (teo36_ParamGlob          &param_glob,
                           tsp_dcom_paraminfo_parms *parms,
                           unsigned short           *pNumParms)
{
    CHRESULT_EO00D hr(parms->session, "eo36_GetInterface");

    HINSTANCE      hInstance;
    int            prog_id_len;
    int            ix;
    char           cProgID[sizeof (tsp00_KnlIdentifier) * 2 + 1];
    char           cMethod[sizeof (tsp00_KnlIdentifier) + 1];
    CLSID          ClsID;
    LPCLSID        pClsID;
    LPIID          pIID;
    unsigned short numMethods, dispid;
    long           version;
    char           *pc;
    int            len;

    param_glob.co_IntroSpectObject      = NULL;
    param_glob.co_IntroSpectMethod      = NULL;
    param_glob.co_IntroSpectMethod_MO   = NULL;
    param_glob.co_IntroSpectFunction    = NULL;
    param_glob.co_IntroSpectFunction_MO = NULL;

    parms->typeinfo_ptr = NULL;

    for (prog_id_len = sizeof (parms->prog_id);
       ((prog_id_len > 0) && (parms->prog_id[prog_id_len - 1] == ' '));
       prog_id_len--);
    for (ix = 0; ix < prog_id_len; ix++)
        cProgID[ix] = parms->prog_id[ix];
    cProgID[prog_id_len] = 0;

    /*
     * Load TypeLib
     */
      tsp00_C4          session;
      memcpy(session,parms->session,sizeof(session));
      tsp1_packet*      SQLBuffer_ptr;
      char*             pIliveCacheSink;
      ak341GetDebugContext (
        REINTERPRET_CAST(tak_all_command_glob*, parms->session_context),
        *REINTERPRET_CAST(tgg91_SessionNo*, &session),
        *REINTERPRET_CAST(tsp00_Addr*, &pIliveCacheSink),
        SQLBuffer_ptr );
      memcpy(session,&pIliveCacheSink,sizeof(session));
    hr = sql36LoadTypeLibFromProgId (true,
                                     &cProgID[0],
                                     ClsID,
                                     &hInstance,
                                     session,
                                     parms->errtext);

    /* map ven36.cpp error code to a HResult */
    if (ERROR_SUCCESS == hr.hresult())
    {
      parms->typeinfo_ptr = (char*)hInstance;
      memcpy (&parms->coclsid[0], &ClsID, sizeof (CLSID));

      param_glob.co_IntroSpectObject = (CO_INTROSPECTOBJECT)sql36GetProcAddress(hInstance,"Co_IntrospectObject");
      if ( NULL == param_glob.co_IntroSpectObject )
      {
        hr = GEO00D_DCOM_INSPEC_ROUTINE_NOT_FOUND;
        MSGD ((ERR_DCOM_HRESULT, "eo36Loadtypelibfromprogid:GetProcAddress(Co_IntrospectObject)", hr.hresult() ));
        return hr.hresult();
      }

      param_glob.co_IntroSpectObject ( &pClsID, &pIID, &numMethods, &version );
      memcpy(cMethod, parms->method, sizeof(parms->method) );
      memcpy (&parms->iid[0], pIID, sizeof (IID));
      pc = strchr( cMethod, ' ' );
      *pc = '\0';

      param_glob.co_IntroSpectMethod_MO = (CO_INTROSPECTMETHOD)sql36GetProcAddress(hInstance,"Co_IntrospectMethod_MO");
      if ( NULL == param_glob.co_IntroSpectMethod_MO )
      {
        param_glob.co_IntroSpectMethod = (CO_INTROSPECTMETHOD)sql36GetProcAddress(hInstance,"Co_IntrospectMethod");
        if ( NULL == param_glob.co_IntroSpectMethod )
        {
          hr = GEO00D_DCOM_INSPEC_ROUTINE_NOT_FOUND;
          MSGD ((ERR_DCOM_HRESULT, "eo36Loadtypelibfromprogid:GetProcAddress(Co_IntrospectMethod)", hr.hresult() ));
          return hr.hresult();
        }
      }

      pc = &cMethod[0];
      if ( param_glob.co_IntroSpectMethod_MO ) 
        param_glob.co_IntroSpectMethod_MO ( get_by_name, &pc, 0, &dispid , pNumParms, (void **)&param_glob.co_IntroSpectFunction_MO );
      else
        param_glob.co_IntroSpectMethod ( get_by_name, &pc, 0, &dispid , pNumParms, (void **)&param_glob.co_IntroSpectFunction );
      parms->dispid = dispid;
      if (( NULL == param_glob.co_IntroSpectFunction_MO ) && ( NULL == param_glob.co_IntroSpectFunction ))
      {
        hr = GEO00D_DCOM_METHOD_NOT_FOUND;
        char msg_buf[128];
        sprintf(msg_buf,"eo36Loadtypelibfromprogid:co_IntroSpectMethod(%.64s)",&cMethod[0]);
        MSGD ((ERR_DCOM_HRESULT, msg_buf, hr.hresult() ));
        return hr.hresult();
      }
      memset (&parms->method[0], ' ', sizeof (parms->method));
      len = (int)strlen (pc);
      if (len > sizeof(parms->method))
        len = sizeof(parms->method);
      memcpy (parms->method, pc, len);
    }
    return hr.hresult();
}

/*----------------------------------------------------------------------------*/

HRESULT eo36_GetMethod (teo36_ParamGlob          &param_glob,
                        tsp_dcom_paraminfo_parms *parms,
                        unsigned short           *pNumParms,
                        BOOLEAN                  &is_last_func)
{
    CHRESULT_EO00D hr(parms->session, "eo36_GetMethod");
    HINSTANCE      hInstance;
    int            len;
    unsigned short DispID;
    LPCLSID        pClsID;
    LPIID          pIID;
    unsigned short num;
    long           version;
    char*          pc;

    hInstance = (HINSTANCE)parms->typeinfo_ptr;
    param_glob.co_IntroSpectObject = (CO_INTROSPECTOBJECT)sql36GetProcAddress(hInstance,"Co_IntrospectObject");
    if ( NULL == param_glob.co_IntroSpectObject )
    {
      hr = GEO00D_DCOM_INSPEC_ROUTINE_NOT_FOUND;
      MSGD ((ERR_DCOM_HRESULT, "eo36_GetMethod:GetProcAddress(Co_IntrospectObject)", hr.hresult() ));
      return hr.hresult();
    }

    param_glob.co_IntroSpectObject ( &pClsID, &pIID, &num, &version );

    param_glob.co_IntroSpectMethod = (CO_INTROSPECTMETHOD)sql36GetProcAddress(hInstance,"Co_IntrospectMethod");
    if ( NULL == param_glob.co_IntroSpectMethod )
    {
      hr = GEO00D_DCOM_INSPEC_ROUTINE_NOT_FOUND;
      MSGD ((ERR_DCOM_HRESULT, "eo36_GetMethod:GetProcAddress(Co_IntrospectMethod)", hr.hresult() ));
      return hr.hresult();
    }
    if ( param_glob.co_IntroSpectMethod_MO ) 
      param_glob.co_IntroSpectMethod_MO ( get_by_index, &pc, parms->func_idx - 1, &DispID , pNumParms, (void **)&param_glob.co_IntroSpectFunction_MO );
    else
      param_glob.co_IntroSpectMethod ( get_by_index, &pc, parms->func_idx - 1, &DispID , pNumParms, (void **)&param_glob.co_IntroSpectFunction );

    if (( NULL == param_glob.co_IntroSpectFunction_MO ) && ( NULL == param_glob.co_IntroSpectFunction ))
    {
      hr = GEO00D_DCOM_METHOD_NOT_FOUND;
      MSGD ((ERR_DCOM_HRESULT, "eo36_GetMethod:co_IntroSpectMethod", hr.hresult() ));
      return hr.hresult();
    }
    if ( parms->func_idx  == num ) is_last_func = TRUE;
    parms->dispid = DispID;
    memset (&parms->method[0], ' ', sizeof (parms->method));
    len = (int)strlen (pc);
    if (len > sizeof(parms->method))
      len = sizeof(parms->method);
    memcpy (parms->method, pc, len);

    return 0;
}

/*----------------------------------------------------------------------------*/

void eo36_Init (teo36_ParamGlob       &param_glob,
                tsp_dcom_paraminfo_parms *parms)
{
    tsp1_packet_ptr     lpPacket;
    tsp1_part_ptr       lpPart;

    memset (parms->errtext, ' ', sizeof (parms->errtext));
    param_glob.OutParams = 0;
    param_glob.Parms     = parms;
    s26first_segment_init ( REINTERPRET_CAST(tsp1_packet_ptr,parms->packet),
                            sp1sk_return, param_glob.lpSegment);
    param_glob.lpSegment->sp1r_returncode() = 0;
    lpPacket = REINTERPRET_CAST(tsp1_packet_ptr,parms->packet);
    lpPacket->sp1_header.sp1h_varpart_len =   sizeof(lpPacket->sp1_header) +
                                              sizeof(lpPacket->sp1_segm().sp1s_segm_header());
    s26new_part_init ( REINTERPRET_CAST(tsp1_packet_ptr,parms->packet),
                        *param_glob.lpSegment, lpPart);
    param_glob.lpPartHdr                     = &lpPart->sp1p_part_header();
    param_glob.lpPartHdr->sp1p_part_kind.becomes(sp1pk_shortinfo);
    param_glob.lpPartHdr->sp1p_attributes.clear();
    param_glob.lpPartHdr->sp1p_arg_count = 0;
    param_glob.lpParamInfo = REINTERPRET_CAST(tsp1_dcom_param_info *, & lpPart->sp1p_buf());
    memcpy(&param_glob.session[0],&parms->session[0],sizeof(param_glob.session));
}

/*----------------------------------------------------------------------------*/

void eo36_InitDcomParamInfo(tsp1_dcom_param_info &paramInfo) {
     paramInfo.sp1i_dcom_mode.clear();
     paramInfo.sp1i_dcom_mode.addElement(sp1ot_mandatory);
     paramInfo.sp1i_dcom_io_type.becomes(sp1io_inout);
     paramInfo.sp1i_dcom_datatype     = 0;
     paramInfo.sp1i_dcom_sub_datatype = 0;
     paramInfo.sp1i_dcom_length       = 0;
     paramInfo.sp1i_dcom_frac         = 0;
     paramInfo.sp1i_dcom_dim          = 0;
     memset (paramInfo.sp1i_dcom_type_uid, 0, sizeof (paramInfo.sp1i_dcom_type_uid));
}

/*----------------------------------------------------------------------------*/

HRESULT eo36_NewParamInfo(teo36_ParamGlob &param_glob,
                          int                vt)
{
  if (0 < param_glob.lpPartHdr->sp1p_arg_count)
    if ((long) ( sizeof (tsp1_part_header) /* PTS 1110591 part_header size added */
      + param_glob.lpPartHdr->sp1p_buf_len 
      + sizeof (tsp1_dcom_param_info)) >
      (long) param_glob.lpPartHdr->sp1p_buf_size)
      return GEO00D_DCOM_BUFFER_LIMIT;
    else
      param_glob.lpParamInfo++;
    param_glob.lpPartHdr->sp1p_arg_count++;
    param_glob.lpPartHdr->sp1p_buf_len += sizeof (tsp1_dcom_param_info);
    eo36_InitDcomParamInfo(*param_glob.lpParamInfo);
    param_glob.lpParamInfo->sp1i_dcom_datatype = vt;
    return S_OK;
}

/*----------------------------------------------------------------------------*/

void eo36_DescribeParamNames (teo36_ParamGlob &param_glob,
                             ParamNames_eo36    &ParmNames,
                             unsigned int       cArgs)
{
  HRESULT hr = ERROR_SUCCESS;  /* PTS 1110591 */
  unsigned int     ix;
  unsigned int     len;
  tsp1_part_ptr    lpPart;
  t_part_data      *lpPart_data;
  tsp1_part_header *lpPartHdr;
  tsp1_dcom_param_info ParamInfo;
  char    *lpcMode;
  char    *lpcType;
  char    *lpcName;
  GUID    *lpGuid;

  s26new_part_init ( REINTERPRET_CAST(tsp1_packet_ptr,param_glob.Parms->packet),
                     *param_glob.lpSegment, lpPart);
  lpPartHdr = &lpPart->sp1p_part_header();
  lpPartHdr->sp1p_part_kind.becomes(sp1pk_columnnames);
  lpPartHdr->sp1p_attributes.clear();
  lpPartHdr->sp1p_arg_count = cArgs;
  lpPart_data = (t_part_data *) & lpPart->sp1p_buf();
  lpPartHdr->sp1p_buf_len = 0;
  for (ix = 1; ix < ( cArgs + 1 ); ix++)
  {
    if ( param_glob.co_IntroSpectFunction_MO )
      param_glob.co_IntroSpectFunction_MO ( ix, &lpcMode, &lpcType,
      &ParamInfo.sp1i_dcom_datatype,
      &ParamInfo.sp1i_dcom_sub_datatype,
      &ParamInfo.sp1i_dcom_length,
      &ParamInfo.sp1i_dcom_dim,
      &param_glob.lpParamInfo->sp1i_dcom_cpp_offset,
      &param_glob.lpParamInfo->sp1i_dcom_ascii_offset,
      &param_glob.lpParamInfo->sp1i_dcom_unicode_offset,
      &ParamInfo.sp1i_dcom_frac,
      &lpGuid,
      &lpcName );
    else
    {
      long offset;
      param_glob.co_IntroSpectFunction ( ix, &lpcMode, &lpcType,
      &ParamInfo.sp1i_dcom_datatype,
      &ParamInfo.sp1i_dcom_sub_datatype,
      &ParamInfo.sp1i_dcom_length,
      &ParamInfo.sp1i_dcom_dim,
      &offset,
      &ParamInfo.sp1i_dcom_frac,
      &lpGuid,
      &lpcName );
      param_glob.lpParamInfo->sp1i_dcom_cpp_offset      = offset;
      param_glob.lpParamInfo->sp1i_dcom_ascii_offset    = offset;
      param_glob.lpParamInfo->sp1i_dcom_unicode_offset  = offset;
    }

    len = (int)strlen (lpcName);
    /* +++           PTS 1110591                                          +++ */
    if (( s26partlen (*lpPart) - (int)sizeof (tsp1_part_header) + (int)len ) > 
      lpPart->sp1p_buf_size() )
    {
      hr = GEO00D_DCOM_BUFFER_LIMIT;
      eo36_SetError ( hr , param_glob);
    }
    else
    {
    /* ---           PTS 1110591                                        --- */
      (*lpPart_data)[0] = len;
      lpPartHdr->sp1p_buf_len = lpPartHdr->sp1p_buf_len + len + 1;
      lpPart_data = (t_part_data *)(((char *) lpPart_data) + 1);
      memcpy ((char *) lpPart_data, lpcName, len);
      lpPart_data = (t_part_data *)(((char *) lpPart_data) + len);
    }  /* PTS 1110591  */
  }
  if ( ERROR_SUCCESS == hr )  /* PTS 1110591  */
    s26finish_part ( REINTERPRET_CAST(tsp1_packet_ptr,param_glob.Parms->packet), *lpPart);
}

/*----------------------------------------------------------------------------*/


HRESULT eo36_OneParameter (teo36_ParamGlob          &param_glob,
                           tsp_dcom_paraminfo_parms *parms,
                           unsigned short           idx)
{
  CHRESULT_EO00D hr(parms->session,"eo36_OneParameter");
  char    *lpcMode;
  char    *lpcType;
  char    *lpcName;
  GUID    *lpGuid;

  hr = eo36_NewParamInfo(param_glob, 0);
  if (!hr.succeeded())
    return hr.hresult();
  
  if ( param_glob.co_IntroSpectFunction_MO )
    param_glob.co_IntroSpectFunction_MO ( idx, &lpcMode, &lpcType,
    &param_glob.lpParamInfo->sp1i_dcom_datatype,
    &param_glob.lpParamInfo->sp1i_dcom_sub_datatype,
    &param_glob.lpParamInfo->sp1i_dcom_length,
    &param_glob.lpParamInfo->sp1i_dcom_dim,
    &param_glob.lpParamInfo->sp1i_dcom_cpp_offset,
    &param_glob.lpParamInfo->sp1i_dcom_ascii_offset,
    &param_glob.lpParamInfo->sp1i_dcom_unicode_offset,
    &param_glob.lpParamInfo->sp1i_dcom_frac,
    &lpGuid,
    &lpcName );
  else
  {
    long offset;
    param_glob.co_IntroSpectFunction ( idx, &lpcMode, &lpcType,
      &param_glob.lpParamInfo->sp1i_dcom_datatype,
      &param_glob.lpParamInfo->sp1i_dcom_sub_datatype,
      &param_glob.lpParamInfo->sp1i_dcom_length,
      &param_glob.lpParamInfo->sp1i_dcom_dim,
      &offset,
      &param_glob.lpParamInfo->sp1i_dcom_frac,
      &lpGuid,
      &lpcName );
    param_glob.lpParamInfo->sp1i_dcom_cpp_offset      = offset;
    param_glob.lpParamInfo->sp1i_dcom_ascii_offset    = offset;
    param_glob.lpParamInfo->sp1i_dcom_unicode_offset  = offset;
  }

  param_glob.lpParamInfo->sp1i_dcom_mode.clear();
  if ( 0 == strcmp( "MAN", lpcMode ))
     param_glob.lpParamInfo->sp1i_dcom_mode.addElement(sp1ot_mandatory);
  if ( 0 == strcmp( "OPT", lpcMode ))
     param_glob.lpParamInfo->sp1i_dcom_mode.addElement(sp1ot_optional);
  if ( 0 == strcmp( "DEF", lpcMode ))
     param_glob.lpParamInfo->sp1i_dcom_mode.addElement(sp1ot_default);
  if ( 0 == strcmp( "ESC", lpcMode ))
     param_glob.lpParamInfo->sp1i_dcom_mode.addElement(sp1ot_escape_char);

  if ( 0 == strcmp( "I/O", lpcType ))
     param_glob.lpParamInfo->sp1i_dcom_io_type.becomes(sp1io_inout);
  if ( 0 == strcmp( "OUT", lpcType ))
     param_glob.lpParamInfo->sp1i_dcom_io_type.becomes(sp1io_output);
  if ( 0 == strcmp( "IN", lpcType ))
     param_glob.lpParamInfo->sp1i_dcom_io_type.becomes(sp1io_input);

  memcpy( &param_glob.lpParamInfo->sp1i_dcom_type_uid[0], lpGuid, sizeof(GUID) );

  return S_OK;
};

/*----------------------------------------------------------------------------*/

void eo36_SetError (long e,
                   teo36_ParamGlob &param_glob)
{
    param_glob.lpSegment->sp1r_returncode()  = -1;
    param_glob.lpSegment->sp1r_errorpos()    = e;
}

