/*!**************************************************************************

  module      : veo52.c

  -------------------------------------------------------------------------

  responsible : JoergM

  special area: Time and Time Statistic Functions
  description : 


  last changed: 2001-09-21  13:41
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/


/*
 * INCLUDE FILES
 */
#include "heo52.h"
#include "geo50_0.h"
#include "RunTime/System/RTESys_MicroTime.h"

/*
 *  DEFINES
 */
#define MOD__  "veo52.c:"
#define EO52_CENTURY_ADJUST 1900
#define EO52_MICROSECONDS_PER_SECOND (1000000U)

static void eo52_FillPascalTimestamp ( tsp00_Timestamp stamp, SAPDB_Bool wantUTC );
/*
 * ========================== GLOBAL FUNCTIONS ================================
 */
externC void vtimestamp (
tsp00_Timestamp stamp )
{
    DBGPAS;
    eo52_FillPascalTimestamp( stamp, false);
}

/*---------------------*/

void eo52UpdateTaskStateStatisticRec(
 teo52_RelativeAndAbsoluteTime *pRec,
 teo52_TaskStateStatisticRec   *pState )
{
#undef MF__
#define MF__ MOD__"eo52UpdateRelativeAndAbsoluteTimeRec"
    pRec->ulAbsMicroSeconds += (pState->dispatcherLeaveTime - pState->dispatcherEnterTime);
    pRec->ulRelMicroSeconds += (pState->runqueueEnqueueTime - pState->dispatcherEnterTime);
}

/*---------------------*/

externC void vUTCtimestamp (
tsp00_Timestamp stamp )
{
    DBGPAS;
    eo52_FillPascalTimestamp( stamp, true);
}

/*
 * ========================== LOCAL FUNCTIONS ================================
 */
static void eo52_FillPascalTimestamp ( tsp00_Timestamp stamp, SAPDB_Bool wantUTC )
{
#undef  MF__
#define MF__ MOD__"vUTCtimestamp"
    SAPDB_Int4                    tmpValue;
    SAPDB_Char                   *cp;

#if defined(_WIN32)
    SYSTEMTIME                 SystemTime;
#else
    time_t              time_sec;     /* total seconds + start second */
    SAPDB_UInt8         microSeconds; /* total microseconds */
    struct tm			tm_buffer ;
    struct tm			*tmbuf ;
#endif

    DBGPAS;
    /*
     * Return date and time as YYYYMMDD and HHHHMMSS
     */
    cp = (SAPDB_Char*)&stamp[0];
    memset (cp, '0', sizeof(tsp00_Timestamp));

    /*
     * obtain current date and time
     */
#if defined(_WIN32)
    if ( wantUTC )
    {
        GetSystemTime(&SystemTime);
    }
    else
    {
        GetLocalTime(&SystemTime);
    }
#else
    microSeconds = RTESys_MicroSecTimer();
    time_sec = (time_t)(KGS->start_time + (microSeconds / EO52_MICROSECONDS_PER_SECOND));
    if ( wantUTC )
    {
        tmbuf = gmtime_r( &time_sec, &tm_buffer );
    }
    else
    {
        tmbuf = localtime_r( &time_sec, &tm_buffer );
    }
#endif

#if defined(_WIN32)
    tmpValue = (SAPDB_Int4)SystemTime.wYear;
#else
    tmpValue = tmbuf->tm_year + EO52_CENTURY_ADJUST ;
#endif
    *cp++ += tmpValue         / 1000; tmpValue %= 1000;
    *cp++ += tmpValue         / 100;  tmpValue %= 100;
    *cp++ += tmpValue         / 10;
    *cp++ += tmpValue         % 10;

#if defined(_WIN32)
    tmpValue = (SAPDB_Int4)SystemTime.wMonth;
#else
    tmpValue = tmbuf->tm_mon + 1 ;
#endif
    *cp++ += tmpValue        / 10;
    *cp++ += tmpValue        % 10;

#if defined(_WIN32)
    tmpValue = (SAPDB_Int4)SystemTime.wDay;
#else
    tmpValue = tmbuf->tm_mday ;
#endif
    *cp++ += tmpValue        / 10;
    *cp++ += tmpValue        % 10;

#if defined(_WIN32)
    tmpValue = (SAPDB_Int4)SystemTime.wHour;
#else
    tmpValue = tmbuf->tm_hour ;
#endif
    *cp++ += tmpValue        / 10;
    *cp++ += tmpValue        % 10;

#if defined(_WIN32)
    tmpValue = (SAPDB_Int4)SystemTime.wMinute;
#else
    tmpValue = tmbuf->tm_min ;
#endif
    *cp++ += tmpValue        / 10;
    *cp++ += tmpValue        % 10;

#if defined(_WIN32)
    tmpValue = (SAPDB_Int4)SystemTime.wSecond;
#else
    tmpValue = tmbuf->tm_sec ;
#endif
    *cp++ += tmpValue        / 10;
    *cp++ += tmpValue        % 10;

#if defined(_WIN32)
    /* NT Systemtimer only provides milliseconds */
    tmpValue = (SAPDB_Int4)SystemTime.wMilliseconds;
#else
    /* UNIX uses its microsecond timer */
    tmpValue = (SAPDB_Int4)(microSeconds % EO52_MICROSECONDS_PER_SECOND);
    *cp++ += tmpValue        / 100000;  tmpValue %= 100000;
    *cp++ += tmpValue        / 10000;  tmpValue %= 10000;
    *cp++ += tmpValue        / 1000;  tmpValue %= 1000;
#endif
    *cp++ += tmpValue        / 100;  tmpValue %= 100;
    *cp++ += tmpValue        / 10;
    *cp++ += tmpValue        % 10;
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
