
/*!**************************************************************************

    module      : vgg943.cpp

    -------------------------------------------------------------------------

    author      : JuergenA
    responsible : UweH

    special area: KernelCommon
    description : GG_TaskWaitStack_Interface

    last changed: 2001-01-19

    -------------------------------------------------------------------------

    copyright:    Copyright by SAP AG, 2001

    ========== licence begin LGPL
    Copyright (C) 2001 SAP AG
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/

/*==========================================================================*
*  INCLUDES                                                                 *
*===========================================================================*/

#include "KernelCommon/Kernel_WaitQueue.hpp"

#include "gsp00.h"
#include "gsp03.h"
#include "ggg00.h"
#include "hgg943.h" // check of PASCAL header vgg943
#include "hgg01_1.h"

/*==========================================================================*
*  DEFINES                                                                  *
*===========================================================================*/

#define ERRMSG_ALLOCATE_FAILED "WaitStack.Alloc failed  "

/*==========================================================================*
* FUNCTIONS                                                                 *
*===========================================================================*/

inline Kernel_WaitStackGlob::Kernel_WaitStackType WaitStackType (
           pasbool              IsReadWait,
           tgg00_MessType_Param MessType)
{
    if (m_autosave == MessType)
    {
        if (IsReadWait) return (Kernel_WaitStackGlob::wstAutoSaveRead);
        return (Kernel_WaitStackGlob::wstAutoSaveWrite); 
    }
    
    if (IsReadWait) return (Kernel_WaitStackGlob::wstBackupRead);
    return (Kernel_WaitStackGlob::wstBackupWrite);
}

/*==========================================================================*
* FUNCTIONS                                                                 *
*===========================================================================*/

// PASCAL interface
externPascal void gg943AllCheckptWaitsResume (tsp00_TaskId CallingTaskId)
{
    Kernel_WaitStackGlob::Instance().PopAllTaskIdsAndResume
        (CallingTaskId, Kernel_WaitStackGlob::wstCheckpoint);
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943AllBackupWaitsResume (tsp00_TaskId         CallingTaskId,
                                             tgg00_MessType_Param MessType)
{
    if (m_autosave == MessType)
    {
        Kernel_WaitStackGlob::Instance().PopAllTaskIdsAndResume
            (CallingTaskId, Kernel_WaitStackGlob::wstAutoSaveRead);

        Kernel_WaitStackGlob::Instance().PopAllTaskIdsAndResume
            (CallingTaskId, Kernel_WaitStackGlob::wstAutoSaveWrite);

        return;
    }
    Kernel_WaitStackGlob::Instance().PopAllTaskIdsAndResume
        (CallingTaskId, Kernel_WaitStackGlob::wstBackupRead);

    Kernel_WaitStackGlob::Instance().PopAllTaskIdsAndResume
        (CallingTaskId, Kernel_WaitStackGlob::wstBackupWrite);
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943AllLogFullWaitsResume (tsp00_TaskId CallingTaskId)
{
    Kernel_WaitStackGlob::Instance().PopAllTaskIdsAndResume
        (CallingTaskId, Kernel_WaitStackGlob::wstLogFull);
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943AllocateTaskWaitStacks (tsp00_TaskId  CallingTaskId,
                                               tsp00_Int4    MaxTasks)
{
    bool ok;
    
    Kernel_WaitStackGlob::AllocateInstance (ok);
    
    if ( ok )
    {
        Kernel_WaitStackGlob::Instance().AllocateStackElements (CallingTaskId, MaxTasks, ok);
    }
    
    if ( ! ok )
    {
        g01abort (csp3_allocat, csp3_n_memory, ERRMSG_ALLOCATE_FAILED, MaxTasks);
    }
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943AllSurrogateWaitsResume (tsp00_TaskId CallingTaskId)
{
    Kernel_WaitStackGlob::Instance().PopAllTaskIdsAndResume
        (CallingTaskId, Kernel_WaitStackGlob::wstSurrogate);
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943AnyBackupWaitResume (tsp00_TaskId         CallingTaskId,
                                            pasbool              IsReadWait,
                                            tgg00_MessType_Param MessType)
{  
    Kernel_WaitStackGlob::Instance().PopAnyTaskIdAndResume
        (CallingTaskId, WaitStackType (IsReadWait, MessType));
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal pasbool gg943AnySaveptWaitExists (tsp00_TaskId  CallingTaskId)
{
    return ( ! Kernel_WaitStackGlob::Instance().IsEmpty
        (CallingTaskId, Kernel_WaitStackGlob::wstSavepoint) );
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943BackupWaitInsertAtBottom (tsp00_TaskId         TaskId,
                                                 pasbool              IsReadWait,
                                                 tgg00_MessType_Param MessType)
{
    Kernel_WaitStackGlob::Instance().InsertTaskIdAtBottom
        (TaskId, WaitStackType (IsReadWait, MessType));
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943CheckptWaitInsert (tsp00_TaskId TaskId)
{
    Kernel_WaitStackGlob::Instance().PushTaskId
        (TaskId, Kernel_WaitStackGlob::wstCheckpoint);
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943GetSaveptWaitList (tsp00_TaskId       CallingTaskId,
                                          tsp00_Int4         DataSize,
                                          tgg00_DataPartPtr  DatapartPtr,
                                          tsp00_Int4        &DataLen)
{
    Kernel_WaitStackGlob::Instance().PopListOfSaveptTaskIds
        (CallingTaskId, DataSize, DatapartPtr, DataLen);
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943LogFullWaitInsert (tsp00_TaskId TaskId)
{
    Kernel_WaitStackGlob::Instance().PushTaskId
        (TaskId, Kernel_WaitStackGlob::wstLogFull);
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943SavepointPrepareWaitInsert (tsp00_TaskId TaskId)
{
    Kernel_WaitStackGlob::Instance().PushTaskId
        (TaskId, Kernel_WaitStackGlob::wstSaveptPrepare);
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943SavepointWaitInsert (tsp00_TaskId TaskId)
{
    Kernel_WaitStackGlob::Instance().PushTaskId
        (TaskId, Kernel_WaitStackGlob::wstSavepoint);
}

//----------------------------------------------------------------------------

// PASCAL interface
externPascal void gg943SurrogateWaitInsert (tsp00_TaskId TaskId)
{
    Kernel_WaitStackGlob::Instance().PushTaskId
        (TaskId, Kernel_WaitStackGlob::wstSurrogate);
}

// end of program
