/*!================================================================
 module:        hin203.h

 -------------------------------------------------------------------

 responsible:   DanielD

 special area:  SQL

 description:   Packet

 -------------------------------------------------------------------

 copyright:     SAP AG 1999



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 ===================================================================*/

#ifndef HIN203_H
#define HIN203_H

#include "vsp001.h"
#include "hin202.h"

/*!
  Class: cin203_SQLPacket
 */
class cin203_SQLPacket
{
protected:
    cin203_SQLPacket (tsp1_packet * packetPtr = NULL);
    ~cin203_SQLPacket ();

public:
    /*! Function: TraceOn

        description: dumps content of packet to @param{trace}.
     */
    void TraceOn (cin97_TraceStream * trace, int buflimit = -1);

    /*! Function: Length

        description: return the complete length in bytes
     */
    int Length () const;

    /*! Function: IsValid

        description:
     */
    bool IsValid () const;

    /*! Function: Invalidate

        description: makes the reference invalid.
     */
    void Invalidate ();

    /*! Function: GetRawPacket

        description:
        prototypes: 2
     */
    tsp1_packet * GetRawPacket ();
    const tsp1_packet * GetRawPacket () const;

    /*! Function: GetRawHeader

        description:
        prototypes: 2
     */
    tsp1_packet_header * GetRawHeader ();
    const tsp1_packet_header * GetRawHeader () const;

protected:
    tsp1_packet * rawPacket;
};

/*!
  EndClass: cin203_SQLPacket
 */

/*----------------------------------------*/

/*!
  Class: cin203_RequestPacket
 */
class cin203_RequestPacket : public cin203_SQLPacket
{
public:
    cin203_RequestPacket (tsp1_packet * packetPtr, int size, int unicode = 0);
    cin203_RequestPacket ();
    ~cin203_RequestPacket ();

    /*! Function: InitVersion

        description: sets the component string and version

        arguments:
            component   [in] a three character string
            version     [in] a five character string. A NULL value
                will query the version module vsp100 for the actual version.
     */
    void InitVersion (const char * component, const char * version = NULL);

    /*! Function: Reset

        description: reinits the package for a new request.
     */
    void Reset ();

    /*! Function: CloseSegment

        description:
     */
    void CloseSegment (cin202_RequestSegment & segment);

    /*! Function: AddSegment

        description: creates a new Segment.
     */
    /* PTS 1108446 sqlMode added */
    /* PTS 1112690 parsingAgain added */
    cin202_RequestSegment AddSegment (tsp1_cmd_mess_type_Param messType,
      const tsp1_sqlmode_Enum &sqlMode,
      const pasbool parsingAgain = false );


};

/*!
  EndClass: cin203_RequestPacket
 */

/*----------------------------------------*/

/*!
  Class: cin203_ReplyPacket
 */
class cin203_ReplyPacket : public cin203_SQLPacket
{
public:
    cin203_ReplyPacket (tsp1_packet * packetPtr = NULL);
    ~cin203_ReplyPacket ();

    /*! Function: GetFirstSegment

        description: returns the first Segment.
     */
    const cin202_ReplySegment GetFirstSegment () const;

    /*! Function: GetNextSegment

        description: returns the Segment following @parm{segment}.

            When @parm{segment} was the last Segment, an invalid Segment is
            returned.
     */
    const cin202_ReplySegment GetNextSegment (const cin202_ReplySegment & segment) const;
};

/*!
  EndClass: cin203_ReplyPacket
 */

#endif

