/*!================================================================
 module:        hin97.h

 -------------------------------------------------------------------

 responsible:   DanielD

 special area:  Tracing

 description:   Simple Trace Stream

 -------------------------------------------------------------------

 copyright:     SAP AG 1999



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 ===================================================================*/

#ifndef HIN97_H
#define HIN97_H

#include "gsp12.h"

/*!
  Class: cin97_TraceStream
 */
class cin97_TraceStream
{
public:
    /*!
     */
    cin97_TraceStream (isp12_OutputStream * outstream);
    virtual ~cin97_TraceStream ();
    virtual void procEntry (const char * name);
    virtual void procEntryParms (const char * name, const char * fmt, ...);
    virtual void procReturns (const char * fmt, ...);
    virtual void procExit (const char * name);
    virtual void writeln (const char * name, int len = -1);
    virtual void printf (const char * format, ...);
    virtual void vprintf (const char * format, va_list ap);
    virtual void setIndentation (int newIndent);
    virtual void hex (const char * comment, const void * buf, int len);
    virtual void hexFromTo (const char * comment, const void * buf,
        int startPos, int endPos);
    virtual void hexFromFor (const char * comment, const void * buf,
        int startPos, int len);

protected:
    isp12_OutputStream * outstream;
    int nesting;
    int indent;
};

typedef struct tin97_EnumDesc {
    int maxElement;
    const char * defaultFormat;
    const char ** enumStrings;
} tin97_EnumDesc;


#define tmpBufSize_cin97 50

/*! Function: in97EnumDump

    description: returns a character representation of an enumeration element.
 */
const char * in97EnumDump (
    const tin97_EnumDesc * enumDesc,
    int element,
    char * tmpBuf = NULL);


#endif

