/*! 
  -----------------------------------------------------------------------------
 
  module: vin101.cpp
 
  -----------------------------------------------------------------------------
 
  responsible:  BerndV
 
  special area: XDB interface
 
  description:  implementation  of XDB class

  version:      min 7.2.

  copyright:    by SAP AG, 1998

  -----------------------------------------------------------------------------
 
    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

  -----------------------------------------------------------------------------
*/
 

/* 
  -----------------------------------------------------------------------------
  includes
  -----------------------------------------------------------------------------
*/
#include "heo02.h"
#include "hsp02.h"
#include "gin100.h"
#include "hin100.h"
#include "hin101.h"
#include "hin110.h"

/* 
  -----------------------------------------------------------------------------
  private definition, declarations and macros
  -----------------------------------------------------------------------------
*/
#define CASESENS_MARK          '"'

#define XUK_DEFAULT            "DEFAULT"

#define XDBSQL_STARTED         "STARTED"
#define XDBSQL_RUNNING         "RUNNING"
#define XDBSQL_COMPLETED       "COMPLETED"

#define XDBSQL_46BCHECK        "SELECT TABLENAME FROM TABLES WHERE TABLENAME='DBAML'"
#define XDBSQL_46CCHECK        "SELECT TABLENAME FROM TABLES WHERE TABLENAME='DBATL'"

#define XDBSQL_R3SID           "SELECT SRCSYSTEM FROM SAPR3.TADIR WHERE OBJECT = 'SYST'"
#define XDBFIELD_R3SID         "SRCSYSTEM"

#define XDBSQL_46BINSSTMT      " INSERT SAPR3.DBAML (OPER_ID, FUNC_ID, UPD_OP, LOG_NAME,   PROGNAME, STATUS, BEGIN_OP) " \
                               " VALUES (            '%s',    '%s',    '%s',   'dba%s.%s', '%s',     '%s',   '%s')"

#define XDBSQL_46CINSDBATL     " INSERT SAPR3.DBATL (SYS_ID, OPER_ID, TASKNAME, JOB_NR, UPD_OP, TLOGNAME,   STATUS, BEGIN_OP) " \
                               " VALUES (            '%s',   '%s',    '%s',     '%s',   '%s',   'dba%s.%s', '%s',   '%s')"
#define XDBSQL_46CINSDBAML     " INSERT SAPR3.DBAML (SYS_ID, OPER_ID, TASKNAME, JOB_NR, TRIAL, UTIL_NR, FUNC_ID, UPD_OP, TOOLSET, PROGNAME, LOG_NAME,   STATUS, BEGIN_OP) " \
                               " VALUES (            '%s',   '%s',    '%s',      '%s',   '%s',  '%s',    '%s',    '%s',   '%s',    '%s',    'dba%s.%s', '%s',   '%s')"

#define XDBSQL_46BINSBASTMT    " INSERT SAPR3.DBABARL (OPER_ID, FUNC_ID, UPD_OP, OBJ_TYPE, BU_TYPE, BU_METHOD) " \
                               " VALUES (              '%s',    '%s',    '%s',   '%s',     '%s',    '%s')"

#define XDBSQL_46CINSDBABARL   " INSERT SAPR3.DBABARL (SYS_ID, OPER_ID, TASKNAME, JOB_NR, TRIAL, UTIL_NR, FUNC_ID, UPD_OP, OBJ_TYPE, BU_TYPE, BU_METHOD) " \
                               " VALUES (              '%s',   '%s',    '%s',     '%s',   '%s',  '%s',    '%s',    '%s',   '%s',     '%s',    '%s')"

#define XDBSQL_46BUPDSTMT      " UPDATE SAPR3.DBAML (UPD_OP, STATUS, END_OP, RC) " \
                               " VALUES (            '%s',   '%s',   '%s',   '%04d') " \
                               " WHERE OPER_ID ='%s' AND FUNC_ID = '%s' "

#define XDBSQL_46CUPDDBATL     " UPDATE SAPR3.DBATL (UPD_OP, STATUS, END_OP, RC) " \
                               " VALUES (            '%s',   '%s',   '%s',   '%04d') " \
                               " WHERE SYS_ID = '%s' AND OPER_ID = '%s' AND TASKNAME = '%s' AND JOB_NR = '%s' "

#define XDBSQL_46CUPDDBAML     " UPDATE SAPR3.DBAML (UPD_OP, STATUS, END_OP, RC) " \
                               " VALUES (            '%s',   '%s',   '%s',   '%04d') " \
                               " WHERE SYS_ID = '%s' AND OPER_ID = '%s' AND TASKNAME = '%s' AND JOB_NR = '%s' AND TRIAL = '%s' AND UTIL_NR = '%s' AND FUNC_ID = '%s'"

#define XDBSQL_46BCHECKFUNC    "SELECT FUNC_ID FROM SAPR3.DBAFID WHERE %s AND PROGNAME = '%s' AND FUNC_ID = '%s' AND TABLE_NAME = '%s'"
#define XDBSQL_46CCHECKFUNC    "SELECT * FROM SAPR3.DBAFID WHERE %s AND PROGNAME = '%s' AND FUNC_ID = '%s'"

#define XDBSQL_CHECKTRIAL      "SELECT FUNC_ID FROM SAPR3.DBATRIAL WHERE %s AND PROGNAME = '%s' AND FUNC_ID = '%s' AND TRIAL = '%d'"
#define XDBSQL_CHECKERR        "SELECT RC FROM SAPR3.DBAERR WHERE %s AND PROGNAME = '%s' AND RC = '%04d'"

#define XDBSQL_COMMIT          "COMMIT"
#define XDBSQL_ROLLBACK        "ROLLBACK"

#define XDBR3_46B              4602  // that means 4.6 + the second letter in alphabet
#define XDBR3_46C              4603  // that means 4.6 + the third  letter in alphabet
#define XDBR3_NONE               -1  // XDB not installed

#define XDB_TASKNAME           "Database Manager"
#define XDB_JOBNR              "0"
#define XDB_TRIAL              "1"
#define XDB_UTILNR             "1"

typedef struct {
  const char   * szFunction;
  const char   * szTable;
  const char   * szComment;
  const char   * szTopic;
  const char   * szComponent;
} tXDBOpTable; 

// XDB-Database
#define XDBDB_SAPDBCLAUSE        "(DBNAME LIKE  'ADA*' OR DBNAME = 'SAP DB')"        // SAP DB

// XDB-Tables
#define XDBTA_DBATL              "DBATL"       // Task Log 4.6C only
#define XDBTA_DBAML              "DBAML"       // Main Log
#define XDBTA_DBAOBJL            "DBAOBJL"     // Object Log 4.6C only
#define XDBTA_DBAPAHAL           "DBAPHAL"     // Phase Log 4.6C only
#define XDBTA_DBARCL             "DBARCL"      // Partial Operations  4.6B only

#define XDBTA_DBAFID             "DBAFID"      // FunctionIDs
#define XDBTA_DBAGRP             "DBAGRP"      // Groups 4.6C only
#define XDBTA_DBATRIAL           "DBATRIAL"    // Trials
#define XDBTA_DBAERR             "DBAERR"      // Errors

#define XDBTA_DBASPAL            "DBASPAL"     // Space Info
#define XDBTA_DBAREOL            "DBAREOL"     // Reorganisation
#define XDBTA_DBAEXTL            "DBAEXTL"     // Space Extension
#define XDBTA_DBABARL            "DBABARL"     // Backup
#define XDBTA_DBAOPTL            "DBAOPTL"     // Optimizer

/* 
  -----------------------------------------------------------------------------
  static members of tin101_XDBOperation
  -----------------------------------------------------------------------------
*/
bool        tin101_XDBOperation :: m_bEnabled  = false;
tsp00_Int4  tin101_XDBOperation :: m_R3Version = 0;

/* 
  -----------------------------------------------------------------------------
  implementation of class tin101_XDBOperation
  -----------------------------------------------------------------------------
*/

/* 
  -----------------------------------------------------------------------------
  public CONSTRUCTOR tin101_XDBOperation :: tin101_XDBOperation
  -----------------------------------------------------------------------------
*/
tin101_XDBOperation :: tin101_XDBOperation
    ( const tsp00_DbNamec        & szDb,  
      const tsp00_KnlIdentifierc & szUsr,
      const tsp00_Namec          & szPwd,
            bool                   bUnicode)
{
  tsp00_NodeIdc szNode;
  
  // do nothing if disabled
  if (!isEnabled()) {
    return;
  } // end if

  szNode.Init();

  initObject(szNode, szDb, szUsr, szPwd, bUnicode);

} // tin101_XDBOperation :: tin101_XDBOperation
/* 
  -----------------------------------------------------------------------------
*/
tin101_XDBOperation :: tin101_XDBOperation
    ( const tsp00_NodeIdc        & szNode,  
      const tsp00_DbNamec        & szDb,  
      const tsp00_KnlIdentifierc & szUsr,
      const tsp00_Namec          & szPwd,
            bool                   bUnicode)
{
  // do nothing if disabled
  if (!isEnabled()) {
    return;
  } // end if

  initObject(szNode, szDb, szUsr, szPwd, bUnicode);

} // tin101_XDBOperation :: tin101_XDBOperation

/* 
  -----------------------------------------------------------------------------
  public DESTRUCTOR tin101_XDBOperation :: ~tin101_XDBOperation
  -----------------------------------------------------------------------------
*/
tin101_XDBOperation :: ~tin101_XDBOperation
    ( )
{
  // do nothing if disabled
  if (!isEnabled()) {
    return;
  } // end if

  if (strcmp(m_pszState, XDBSQL_COMPLETED) != 0) {
    complete(XDBST_ERROR_IN101);
  } // end if

  disconnect(m_Session);
} // end tin101_XDBOperation :: ~tin101_XDBOperation

/* 
  -----------------------------------------------------------------------------
  public function tin101_XDBOperation :: lastEvent
  -----------------------------------------------------------------------------
*/
teo200_EventList & tin101_XDBOperation :: lastEvent
    ( ) 
{
  return m_EvtLast;
} // end tin101_XDBOperation :: lastEvent

/* 
  -----------------------------------------------------------------------------
  public function tin101_XDBOperation :: isXDBInstalled
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: isXDBInstalled
    ( ) 
{
  return (m_R3Version != -1);
} // end tin101_XDBOperation :: isXDBInstalled

/* 
  -----------------------------------------------------------------------------
  public function tin101_XDBOperation :: reset
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: reset
    ( )
{

  // do nothing if disabled
  if (!isEnabled()) {
    return true;
  } // end if

  // clear event
  tin101_XDBOperation :: m_EvtLast.eo200_ClearEventList ( );

  // send rollback
  execute(m_Session, XDBSQL_ROLLBACK, m_EvtLast);

  // disconnect
  disconnect(m_Session);

  // connect
  connect(m_Session, m_EvtLast);

  // clear members
  m_pszState       = XDBSQL_COMPLETED;
  m_szOperId[0]    = '\0';
  m_szFuncId[0]    = '\0';
  m_szComponent[0] = '\0';

  return !m_EvtLast;
} // end tin101_XDBOperation :: reset

/* 
  -----------------------------------------------------------------------------
  public function tin101_XDBOperation :: enable
  -----------------------------------------------------------------------------
*/
void tin101_XDBOperation :: enable
    (const bool bEnabled ) 
{
  
  tin101_XDBOperation :: m_bEnabled = bEnabled;

} // end tin101_XDBOperation :: enable

/* 
  -----------------------------------------------------------------------------
  public function tin101_XDBOperation :: disable
  -----------------------------------------------------------------------------
*/
void tin101_XDBOperation :: disable
    ( ) 
{
  
  m_bEnabled = false;

} // end tin101_XDBOperation :: disable

/* 
  -----------------------------------------------------------------------------
  public function tin101_XDBOperation :: isEnabled
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: isEnabled
    ( ) 
{
  
  return m_bEnabled;

} // end tin101_XDBOperation :: isEnabled

/* 
  -----------------------------------------------------------------------------
  public function tin101_XDBOperation :: start
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: start
    ( const char       * szFunction,
      const char       * szComponent,
      tsp00_Timestampc & szTimeStamp,
      const bool         bCommit ) 
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: start");
  
  szTimeStamp.Init();

  // do nothing if disabled
  if (!isEnabled() || !isXDBInstalled()) {
    return true;
  } // end if

  m_EvtLast.eo200_ClearEventList ( );

  if (!m_Session.is_connected) {
    if (!reset()) {
      return m_EvtLast;
    } // end if
  } // end if

  // check state
  if (strcmp(m_pszState, XDBSQL_COMPLETED) != 0) {
    teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBSEQU_IN100);
    tin100_GlobalLog :: writeEntry (lEvtList);
    m_EvtLast = lEvtList;
    return false;
  } // end if

  // check parameters
  const char   * pError = NULL;

  if (strlen(szFunction) > XDBMAX_FUNCID_IN101) {
    pError = szFunction;
  } else if (strlen(szComponent) > XDBMAX_COMPONENT_IN101) {
    pError = szComponent;
  } // end if
  if (pError != NULL) {
    teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBINVVAL_IN100, pError);
    tin100_GlobalLog :: writeEntry (lEvtList);
    m_EvtLast = lEvtList;
    return false;
  } // end if

  // generate timestamp
  timestamp(m_szOperId);
  // copy function id
  strcpy(m_szFuncId, szFunction);
  // copy component
  strcpy(m_szComponent, szComponent);
  // set state
  m_pszState = XDBSQL_STARTED;

  if (m_R3Version == XDBR3_46B) {
    return start46B(szTimeStamp, bCommit );
  } else if (m_R3Version == XDBR3_46C) {
    return start46C(szTimeStamp, bCommit );
  } // end if

  // m_R3Version must be 0 -> XDB not installed 
  return true;
} // end tin101_XDBOperation :: start

/* 
  -----------------------------------------------------------------------------
  public function tin101_XDBOperation :: run
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: run
    ( const int      nState,
      const char   * szApprEnd, 
      const bool     bCommit )
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: run");
 
  const char       * pError = NULL;

  // do nothing if disabled
  if (!isEnabled() || !isXDBInstalled()) {
    return true;
  } // end if

  m_EvtLast.eo200_ClearEventList ( );

  // check state
  if (strcmp(m_pszState, XDBSQL_COMPLETED) == 0) {
    teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBSEQU_IN100);
    tin100_GlobalLog :: writeEntry (lEvtList);
    m_EvtLast = lEvtList;
    return false;
  } // end if

  // check parameters
  if (nState < XDBMIN_STATE_IN101 || nState > XDBMAX_STATE_IN101) {
    pError = "nState";
  } else if (strlen(szApprEnd) > XDBMAX_TIMESTAMP_IN101) {
    pError = szApprEnd;
  } // end if
  if (pError != NULL) {
    teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBINVVAL_IN100, pError);
    tin100_GlobalLog :: writeEntry (lEvtList);
    m_EvtLast = lEvtList;
    return false;
  } // end if

  // check entry in DBAERR
  if (!checkErr(m_Session, m_szComponent, nState,  m_EvtLast)) {
    return false;
  } // end if

  // generate timestamp
  timestamp(m_szNowStamp);
  // set state
  m_pszState =  XDBSQL_RUNNING;

  if (m_R3Version == XDBR3_46B) {
    return run46B(nState, szApprEnd, bCommit );
  } else if (m_R3Version == XDBR3_46C) {
    return run46C(nState, szApprEnd, bCommit );
  } // end if

  // m_R3Version must be 0 -> XDB not installed 
  return true;
} // end tin101_XDBOperation :: run

/* 
  -----------------------------------------------------------------------------
  public function tin101_XDBOperation :: complete
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: complete
    ( const int      nState, 
      const bool     bCommit )
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: complete");
  
  const char       * pError = NULL;

  // do nothing if disabled
  if (!isEnabled() || !isXDBInstalled()) {
    return true;
  } // end if

  m_EvtLast.eo200_ClearEventList ( );

  // check state
  if (strcmp(m_pszState, XDBSQL_COMPLETED) == 0) {
    teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBSEQU_IN100);
    tin100_GlobalLog :: writeEntry (lEvtList);
    m_EvtLast = lEvtList;
    return false;
  } // end if

  // check parameters
  if (nState < XDBMIN_STATE_IN101 || nState > XDBMAX_STATE_IN101) {
    pError = "nState";
  } // end if
  if (pError != NULL) {
    teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBINVVAL_IN100, pError);
    tin100_GlobalLog :: writeEntry (lEvtList);
    m_EvtLast = lEvtList;
    return false;
  } // end if

  // check entry in DBAERR
  if (!checkErr(m_Session, m_szComponent, nState,  m_EvtLast)) {
    return false;
  } // end if

  // generate timestamp
  timestamp(m_szNowStamp);
  // set state
  m_pszState =  XDBSQL_COMPLETED;
  
  if (m_R3Version == XDBR3_46B) {
    return complete46B(nState, bCommit );
  } else if (m_R3Version == XDBR3_46C) {
    return complete46C(nState, bCommit );
  } // end if

  // m_R3Version must be 0 -> XDB not installed 
  return true;
} // end tin101_XDBOperation :: complete

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: start46B
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: start46B
    ( const tsp00_Timestampc & szTimeStamp,
      const bool               bCommit ) 
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: start46B");
  

  // check entry in DBAFID
  if (!checkFunc46B(m_Session, m_szFuncId, m_szComponent, XDBTA_DBAML, m_EvtLast)) {
    return false;
  } // end if

  // make statement for DBAML
  char           szStmtDBAML[1024];
  _stprintf(szStmtDBAML, XDBSQL_46BINSSTMT, 
                         m_szOperId.asCharp(),    // OPER_ID
                         m_szFuncId,              // FUNC_ID
                         m_szOperId.asCharp(),    // UPD_OP
                         m_szOperId.asCharp(), m_szFuncId,  // LOG_NAME
                         m_szComponent,           // PROGNAME
                         m_pszState,              // STATUS
                         m_szOperId.asCharp());   // BEGIN_OP


  if (execute(m_Session, &szStmtDBAML[0], m_EvtLast)) {
    if (bCommit) {
      return execute(m_Session, XDBSQL_COMMIT, m_EvtLast);
    } // end if
    return true;
  } // end if

  execute(m_Session, XDBSQL_ROLLBACK, m_EvtLast);
  
  return false;
} // end tin101_XDBOperation :: start46B

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: start46C
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: start46C
    ( const tsp00_Timestampc & szTimeStamp,
      const bool               bCommit ) 
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: start46C");

  // check entry in DBAFID
  if (!checkFunc46C(m_Session, m_szFuncId, m_szComponent, m_EvtLast)) {
    return false;
  } // end if

  // read the DBAFID record
  tin110_SQLRecord        aRecord(&m_Session);

  // make statement for DBATL
  char           szStmtDBATL[1024];
  _stprintf(szStmtDBATL, XDBSQL_46CINSDBATL,   
                         m_R3Sid.asCharp(),      // SYS_ID
                         m_szOperId.asCharp(),   // OPER_ID
                         XDB_TASKNAME,           // TASKNAME
                         XDB_JOBNR,              // JOB_NR
                         m_szOperId.asCharp(),   // UPD_OP
                         m_szOperId.asCharp(), m_szFuncId,  // TLOGNAME
                         m_pszState,             // STATUS
                         m_szOperId.asCharp());  // BEGIN_OP

  // make statement for DBAML
  char           szStmtDBAML[1024];
  _stprintf(szStmtDBAML, XDBSQL_46CINSDBAML,   
                         m_R3Sid.asCharp(),      // SYS_ID
                         m_szOperId.asCharp(),   // OPER_ID
                         XDB_TASKNAME,           // TASKNAME
                         XDB_JOBNR,              // JOB_NR
                         XDB_TRIAL,              // TRIAL
                         XDB_UTILNR,             // UTIL_NR
                         m_szFuncId,             // FUNC_ID
                         m_szOperId.asCharp(),   // UPD_OP
                         aRecord.fieldAsChar("TOOLSET"),  // TOOLSET
                         m_szComponent,          // PROGNAME
                         m_szOperId.asCharp(), m_szFuncId,  // LOG_NAME
                         m_pszState,             // STATUS
                         m_szOperId.asCharp());  // BEGIN_OP

  
  if (execute(m_Session, &szStmtDBATL[0], m_EvtLast)) {
    if (execute(m_Session, &szStmtDBAML[0], m_EvtLast)) {
      if (bCommit) {
        return execute(m_Session, XDBSQL_COMMIT, m_EvtLast);
      } // end if
      return true;
    } // end if
  } // end if

  execute(m_Session, XDBSQL_ROLLBACK, m_EvtLast);
  
  return false;
} // end tin101_XDBOperation :: start46C

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: run46B
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: run46B
    ( const int      nState,
      const char   * szApprEnd, 
      const bool     bCommit )
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: run46B");
  
  // make statement for DBAML
  char               szStmtDBAML[1024];
  _stprintf(szStmtDBAML, XDBSQL_46BUPDSTMT, 
                         m_szNowStamp.asCharp(),  // UPD_OP
                         m_pszState,              // STATUS
                         szApprEnd,               // END_OP
                         nState,                  // RC
                         m_szOperId.asCharp(),    // OPER_ID  (WHERE)
                         m_szFuncId);             // FUNC_ID  (WHERE)

  if (execute(m_Session, &szStmtDBAML[0], m_EvtLast)) {
    if (bCommit) {
      return execute(m_Session, XDBSQL_COMMIT, m_EvtLast);
    } // end if
    return true;
  } // end if

  execute(m_Session, XDBSQL_ROLLBACK, m_EvtLast);
  
  return false;
} // end tin101_XDBOperation :: run46B

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: run46C
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: run46C
    ( const int      nState,
      const char   * szApprEnd, 
      const bool     bCommit )
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: run46C");

  // make statement for DBATL
  char               szStmtDBATL[1024];
  _stprintf(szStmtDBATL, XDBSQL_46CUPDDBATL, 
                         m_szNowStamp.asCharp(),  // UPD_OP
                         m_pszState,              // STATUS
                         szApprEnd,               // END_OP
                         nState,                  // RC
                         m_R3Sid.asCharp(),       // SYS_ID   (WHERE)
                         m_szOperId.asCharp(),    // OPER_ID  (WHERE)
                         XDB_TASKNAME,            // TASKNAME (WHERE)
                         XDB_JOBNR);              // JOB_NR   (WHERE)

  // make statement for DBAML
  char               szStmtDBAML[1024];
  _stprintf(szStmtDBAML, XDBSQL_46CUPDDBAML, 
                         m_szNowStamp.asCharp(),  // UPD_OP
                         m_pszState,              // STATUS
                         szApprEnd,               // END_OP
                         nState,                  // RC
                         m_R3Sid.asCharp(),       // SYS_ID
                         m_szOperId.asCharp(),    // OPER_ID
                         XDB_TASKNAME,            // TASKNAME
                         XDB_JOBNR,               // JOB_NR
                         XDB_TRIAL,               // TRIAL
                         XDB_UTILNR,              // UTIL_NR
                         m_szFuncId);             // FUNC_ID

  

  if (execute(m_Session, &szStmtDBATL[0], m_EvtLast)) {
    if (execute(m_Session, &szStmtDBAML[0], m_EvtLast)) {
      if (bCommit) {
        return execute(m_Session, XDBSQL_COMMIT, m_EvtLast);
      } // end if
      return true;
    } // end if
  } // end if

  execute(m_Session, XDBSQL_ROLLBACK, m_EvtLast);
  
  return false;
} // end tin101_XDBOperation :: run46C

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: complete46B
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: complete46B
    ( const int      nState, 
      const bool     bCommit )
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: complete46B");
  
  // make statement
  char               szStmtDBAML[1024];
  _stprintf(szStmtDBAML, XDBSQL_46BUPDSTMT,        
                         m_szNowStamp.asCharp(),  // UPD_OP
                         m_pszState,              // STATUS
                         m_szNowStamp.asCharp(),  // END_OP
                         nState,                  // RC
                         m_szOperId.asCharp(),    // OPER_ID  (WHERE)
                         m_szFuncId);             // FUNC_ID  (WHERE)

  if (execute(m_Session, &szStmtDBAML[0], m_EvtLast)) {
    if (bCommit) {
      return execute(m_Session, XDBSQL_COMMIT, m_EvtLast);
    } // end if
    return true;
  } // end if

  execute(m_Session, XDBSQL_ROLLBACK, m_EvtLast);
  
  return false;
} // end tin101_XDBOperation :: complete46B

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: complete46C
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: complete46C
    ( const int      nState, 
      const bool     bCommit )
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: complete46C");

  // make statement for DBATL
  char               szStmtDBATL[1024];
  _stprintf(szStmtDBATL, XDBSQL_46CUPDDBATL, 
                         m_szNowStamp.asCharp(),  // UPD_OP
                         m_pszState,              // STATUS
                         m_szNowStamp.asCharp(),  // END_OP
                         nState,                  // RC
                         m_R3Sid.asCharp(),       // SYS_ID   (WHERE)
                         m_szOperId.asCharp(),    // OPER_ID  (WHERE)
                         XDB_TASKNAME,            // TASKNAME (WHERE)
                         XDB_JOBNR);              // JOB_NR   (WHERE)

  // make statement for DBAML
  char               szStmtDBAML[1024];
  _stprintf(szStmtDBAML, XDBSQL_46CUPDDBAML, 
                         m_szNowStamp.asCharp(),  // UPD_OP
                         m_pszState,              // STATUS
                         m_szNowStamp.asCharp(),  // END_OP
                         nState,                  // RC
                         m_R3Sid.asCharp(),       // SYS_ID   (WHERE)
                         m_szOperId.asCharp(),    // OPER_ID  (WHERE)
                         XDB_TASKNAME,            // TASKNAME (WHERE)
                         XDB_JOBNR,               // JOB_NR   (WHERE)
                         XDB_TRIAL,               // TRIAL    (WHERE)
                         XDB_UTILNR,              // UTIL_NR  (WHERE)
                         m_szFuncId);             // FUNC_ID  (WHERE)
  

  if (execute(m_Session, &szStmtDBATL[0], m_EvtLast)) {
    if (execute(m_Session, &szStmtDBAML[0], m_EvtLast)) {
      if (bCommit) {
        return execute(m_Session, XDBSQL_COMMIT, m_EvtLast);
      } // end if
      return true;
    } // end if
  } // end if

  execute(m_Session, XDBSQL_ROLLBACK, m_EvtLast);
  
  return false;
} // end tin101_XDBOperation :: complete46C

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: initObject
  -----------------------------------------------------------------------------
*/
void tin101_XDBOperation :: initObject
/*    
    ( const tsp00_NodeIdc        & szRNode,  
      const tsp00_DbNamec        & szRDb,  
      const tsp00_KnlIdentifierc & szRUsr,
      const tsp00_Namec          & szRPwd ) */
    ( const tsp00_NodeIdc        & szNode,  
      const tsp00_DbNamec        & szDb,  
      const tsp00_KnlIdentifierc & szUsr,
      const tsp00_Namec          & szPwd,
            bool                   bUnicode)
{
/*
  // fake target db
  tsp00_NodeIdc        szNode ;
  tsp00_DbNamec        szDb   ;   
  tsp00_KnlIdentifierc szUsr  ;
  tsp00_Namec          szPwd  ;
 
  szNode.rawAssign("p26615");
  szDb  .rawAssign("S10");  
  szUsr .rawAssign("SAPR3");
  szPwd .rawAssign("SAP");
*/
  // init other members
  i29initsession ( &m_Session, NULL );  
  m_EvtLast.eo200_ClearEventList ( );
  m_pszState       = XDBSQL_COMPLETED;
  m_szOperId[0]    = '\0';
  m_szNowStamp[0]  = '\0';
  m_szFuncId[0]    = '\0';
  m_szComponent[0] = '\0';
  m_R3Version      = 0;
  m_R3Sid.rawAssign(szDb);

  // connect and check
  if (connect ( szNode, szDb, szUsr, szPwd, bUnicode, m_Session, m_EvtLast )) {
    isXDBInstalled ( m_Session, m_EvtLast );
  } // end if

} // tin101_XDBOperation :: initObject

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: connect
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: connect
    ( const tsp00_NodeIdc          & szNode,  
      const tsp00_DbNamec          & szDb,  
      const tsp00_KnlIdentifierc   & szUsr,
      const tsp00_Namec            & szPwd,
            bool                     bUnicode,
            tin01_sql_session      & aSession,
            teo200_EventList       & aEvent) const
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: connect");

  int                   nRc;
  tsp00_ErrTextc        szErrtext;
  tsp00_KnlIdentifierc  szErrname;
  tsp00_Int2            nErrcode;
  tsp00_Int2            nErrpos;
 
  // init return
  aEvent.eo200_ClearEventList();

  // init session structure
  i29initsession(&aSession, NULL);

  // connect 
  i29lasterr_on(&aSession);
  nRc = i29usrconnect(&aSession, szDb, szNode, szUsr, szPwd, bUnicode, false);

  if (nRc != 0) {
    i29sqllasterr(&aSession, szErrtext, szErrname, &nErrcode, &nErrpos);
    teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBCONNECT_IN100, szErrtext.asCharp(), szErrname.asCharp(), nRc);
    tin100_GlobalLog :: writeEntry (lEvtList);
    aEvent = lEvtList;
  } // end if
  
  return !aEvent;
} // tin101_XDBOperation :: connect

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: connect
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: connect
    ( tin01_sql_session & aSession,
      teo200_EventList  & aEvent) const
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: connect");
 
  int                   nRc;
  tsp00_ErrTextc        szErrtext;
  tsp00_KnlIdentifierc  szErrname;
  tsp00_Int2            nErrcode;
  tsp00_Int2            nErrpos;

  // init return
  aEvent.eo200_ClearEventList();

  if (!isEnabled()) {
    return !aEvent;
  } // end if
  
  // connect 
  i29lasterr_on(&aSession);
  nRc = i29reconnect(&aSession);

  if (nRc != 0) {
    i29sqllasterr(&aSession, szErrtext, szErrname, &nErrcode, &nErrpos);
    teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBCONNECT_IN100, szErrtext.asCharp(), szErrname.asCharp(), nRc);
    tin100_GlobalLog :: writeEntry (lEvtList);
    aEvent = lEvtList;
  } // end if
  
  return !aEvent;
} // tin101_XDBOperation :: connect

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: disconnect
  -----------------------------------------------------------------------------
*/
void tin101_XDBOperation :: disconnect
    ( tin01_sql_session & aSession ) const
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: disconnect");

  tsp00_Bool Dummy = true;

  if (Dummy) {
    Dummy = false;
  } // end if

  if (aSession.is_connected ) {
    i29release(&aSession, true);
  } /* end if */

} // tin101_XDBOperation :: disconnect

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: execute
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: execute
    ( tin01_sql_session & aSession,
      const char        * pStatement, 
      teo200_EventList  & aEvent,
      const bool          bLog) const
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: execute");

  tsp00_C256c    szError;
  int            nRc        = cin01_db_ok;
  
  // init return
  aEvent.eo200_ClearEventList();

  if (!aSession.is_connected) {
    teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBNOSESS_IN100);
    tin100_GlobalLog :: writeEntry (lEvtList);
    aEvent = lEvtList;
  } else {
    // execute select
    nRc = i29adbs(&aSession, pStatement);

    // error handling
    if (nRc != cin01_db_ok) {
      i29errmsg(&aSession, szError);
      teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBSQL_IN100, nRc, szError.asCharp());
      if (bLog) {
        tin100_GlobalLog :: writeEntry (lEvtList);
      } // end if
      aEvent = lEvtList;
    } // end if
  } // end if

  return !aEvent;
} // tin101_XDBOperation :: execute

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: isXDBInstalled
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: isXDBInstalled
    ( tin01_sql_session & aSession, 
      teo200_EventList  & aEvent )
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: isXDBInstalled");

  // init return
  aEvent.eo200_ClearEventList();

  // execute select
  if (m_R3Version == 0) {
    if (!execute(aSession, XDBSQL_46BCHECK, aEvent, false)) {
      teo200_EventList lEvtList(/*aEvent,*/ FUNCTION_NAME_MIN100, ERR_XDBNOTINST_IN100);
      tin100_GlobalLog :: writeEntry (lEvtList);
      aEvent = lEvtList;
      m_R3Version = XDBR3_NONE;
    } else {
      m_R3Version = XDBR3_46B;
      if (execute(aSession, XDBSQL_46CCHECK, aEvent, false)) {
        m_R3Version = XDBR3_46C;
      } // end if

//    // read R/3-System Id
//    tin110_SQLResultTable   aTable(&aSession);
//    tin110_SQLRecord        aRecord;
//
//    aTable.execute(XDBSQL_R3SID);
//    if (aTable.sqlRc() == cin01_db_ok) {
//      aRecord = aTable.record();
//      m_R3Sid.rawAssign(aRecord.fieldAsChar(XDBFIELD_R3SID));
//    } // end if

    } // end if
  } // end if
   
  return !aEvent;
} // tin101_XDBOperation :: isXDBInstalled

/* 
  -----------------------------------------------------------------------------
  private function tin101_XDBOperation :: timestamp
  -----------------------------------------------------------------------------
*/
void tin101_XDBOperation :: timestamp
    ( tsp00_Timestampc & szTimestamp ) const
{
  tsp00_Date  DateP;
  tsp00_Time  TimeP;
  tsp00_Datec DateC;
  tsp00_Timec TimeC;
    
  sqldattime(DateP, TimeP);
  DateC.p2c(DateP);
  TimeC.p2c(TimeP);

  _stprintf(szTimestamp, "%.8s%.6s", DateC.asCharp(), (TimeC.asCharp() + 2));

} // end tin101_XDBOperation :: timestamp

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: checkFunc46B
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: checkFunc46B
    ( tin01_sql_session & aSession,
      const char        * szFunction,
      const char        * szComponent,
      const char        * szTable,
      teo200_EventList  & aEvent ) const
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: checkFunc46B");

  char               szStatement[1024];

  // make statement
  _stprintf(szStatement, XDBSQL_46BCHECKFUNC, 
                         XDBDB_SAPDBCLAUSE,
                         szComponent,
                         szFunction,
                         szTable);

  if (!execute(aSession, &szStatement[0], aEvent, false)) {
    teo200_EventList lEvtList(/*aEvent,*/ FUNCTION_NAME_MIN100, ERR_XDBINVFUNC_IN100, szFunction, szComponent);
    tin100_GlobalLog :: writeEntry (lEvtList);
    aEvent = lEvtList;
  } // end if
  
  return !aEvent;
} // tin101_XDBOperation :: checkFunc46B

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: checkFunc46C
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: checkFunc46C
    ( tin01_sql_session & aSession,
      const char        * szFunction,
      const char        * szComponent,
      teo200_EventList  & aEvent ) const
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: checkFunc46C");

  char               szStatement[1024];

  // make statement
  _stprintf(szStatement, XDBSQL_46CCHECKFUNC, 
                         XDBDB_SAPDBCLAUSE,
                         szComponent,
                         szFunction);

  tin110_SQLResultTable aTable (&aSession, szStatement);

  if (aTable.sqlRc() != cin01_db_ok) {
    teo200_EventList lEvtList(/*aEvent,*/ FUNCTION_NAME_MIN100, ERR_XDBINVFUNC_IN100, szFunction, szComponent);
    tin100_GlobalLog :: writeEntry (lEvtList);
    aEvent = lEvtList;
  } // end if
  
  return !aEvent;
} // tin101_XDBOperation :: checkFunc46C
/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: checkTrial
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: checkTrial
    ( tin01_sql_session & aSession,
      const char        * szFunction,
      const char        * szComponent,
      const int           nTrial,
      teo200_EventList  & aEvent ) const
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: checkTrial");

  char               szStatement[1024];

  // make statement
  _stprintf(szStatement, XDBSQL_CHECKTRIAL, 
                         XDBDB_SAPDBCLAUSE,
                         szComponent,
                         szFunction,
                         nTrial);

  if (!execute(aSession, &szStatement[0], aEvent, false)) {
    teo200_EventList lEvtList(/*aEvent,*/ FUNCTION_NAME_MIN100, ERR_XDBINVTRIAL_IN100, szFunction, szComponent, nTrial);
    tin100_GlobalLog :: writeEntry (lEvtList);
    aEvent = lEvtList;
  } // end if
  
  return !aEvent;
} // tin101_XDBOperation :: checkTrial

/* 
  -----------------------------------------------------------------------------
  protected function tin101_XDBOperation :: checkErr
  -----------------------------------------------------------------------------
*/
bool tin101_XDBOperation :: checkErr
    ( tin01_sql_session & aSession,
      const char        * szComponent,
      const int           nError,
      teo200_EventList  & aEvent ) const
{
  FUNCTION_DBG_MIN100("tin101_XDBOperation :: checkErr");

  char               szStatement[1024];

  // make statement
  _stprintf(szStatement, XDBSQL_CHECKERR, 
                         XDBDB_SAPDBCLAUSE,
                         szComponent,
                         nError);

  if (!execute(aSession, &szStatement[0], aEvent, false)) {
    teo200_EventList lEvtList(/*aEvent,*/ FUNCTION_NAME_MIN100, ERR_XDBINVERR_IN100, nError);
    tin100_GlobalLog :: writeEntry (lEvtList);
    aEvent = lEvtList;
  } // end if
  
  return !aEvent;
} // tin101_XDBOperation :: checkErr

/* 
  -----------------------------------------------------------------------------
  implementation of class tin101_XDBBackupOperation
  -----------------------------------------------------------------------------
*/

/* 
  -----------------------------------------------------------------------------
  public CONSTRUCTOR tin101_XDBBackupOperation :: tin101_XDBBackupOperation
  -----------------------------------------------------------------------------
*/
tin101_XDBBackupOperation :: tin101_XDBBackupOperation
    ( tsp00_DbNamec        szDb,  
      tsp00_KnlIdentifierc szUsr,
      tsp00_Namec          szPwd,
      bool                 bUnicode)
    : tin101_XDBOperation (szDb, szUsr, szPwd, bUnicode)
{
} // tin101_XDBBackupOperation :: tin101_XDBBackupOperation
/* 
  -----------------------------------------------------------------------------
*/
tin101_XDBBackupOperation :: tin101_XDBBackupOperation
    ( tsp00_NodeIdc        szNode,  
      tsp00_DbNamec        szDb,  
      tsp00_KnlIdentifierc szUsr,
      tsp00_Namec          szPwd,
      bool                 bUnicode)
    : tin101_XDBOperation (szNode, szDb, szUsr, szPwd, bUnicode)
{
} // tin101_XDBBackupOperation :: tin101_XDBBackupOperation

/* 
  -----------------------------------------------------------------------------
  public tin101_XDBBackupOperation :: start
  -----------------------------------------------------------------------------
*/
bool tin101_XDBBackupOperation :: start
    ( const char       * szFunction,
      const char       * szComponent,
      tsp00_Timestampc & szTimeStamp,
      const char       * szObject,
      const char       * szType,
      const char       * szMethod )
{
  FUNCTION_DBG_MIN100("tin101_XDBBackupOperation :: start");

  const char   * pError = NULL;

  szTimeStamp.Init();

  // do nothing if disabled
  if (!isEnabled() || !isXDBInstalled()) {
    return true;
  } // end if

  m_EvtLast.eo200_ClearEventList ( );

  if (!m_Session.is_connected) {
    if (!reset()) {
      return m_EvtLast;
    } // end if
  } // end if

  // check state
  if (strcmp(m_pszState, XDBSQL_COMPLETED) != 0) {
    teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBSEQU_IN100);
    tin100_GlobalLog :: writeEntry (lEvtList);
    m_EvtLast = lEvtList;
    return false;
  } // end if

  // check only backup parameters 
  if (strlen(szObject) > XDBMAX_BAOBJECT_IN101) {
    pError = szObject;
  } else if (strlen(szType) > XDBMAX_BATYPE_IN101) {
    pError = szType;
  } else if (strlen(szMethod) > XDBMAX_BAMETHOD_IN101) {
    pError = szMethod;
  } // end if
  if (pError != NULL) {
    teo200_EventList lEvtList(FUNCTION_NAME_MIN100, ERR_XDBINVVAL_IN100, pError);
    tin100_GlobalLog :: writeEntry (lEvtList);
    m_EvtLast = lEvtList;
    return false;
  } // end if

  if (m_R3Version == XDBR3_46B) {
    return start46B(szFunction, szComponent, szTimeStamp, szObject, szType, szMethod);
  } else if (m_R3Version == XDBR3_46C) {
    return start46C(szFunction, szComponent, szTimeStamp, szObject, szType, szMethod);
  } // end if

  // m_R3Version must be 0 -> XDB not installed 
  return true;
} // end tin101_XDBBackupOperation :: start

/* 
  -----------------------------------------------------------------------------
  protected tin101_XDBBackupOperation :: start46B
  -----------------------------------------------------------------------------
*/
bool tin101_XDBBackupOperation :: start46B
    ( const char       * szFunction,
      const char       * szComponent,
      tsp00_Timestampc & szTimeStamp,
      const char       * szObject,
      const char       * szType,
      const char       * szMethod )
{
  FUNCTION_DBG_MIN100("tin101_XDBBackupOperation :: start46B");

  // check entry in DBABARL
  if (!checkFunc46B(m_Session, szFunction, szComponent, XDBTA_DBABARL, m_EvtLast)) {
    return false;
  } // end if

  // call parent member start
  if (tin101_XDBOperation :: start(szFunction, szComponent, szTimeStamp, false)) {

    // make insert statement for DBABARL
    char           szStmtDBARL[1024];
    _stprintf(szStmtDBARL, XDBSQL_46BINSBASTMT, 
                           m_szOperId.asCharp(),
                           m_szFuncId,
                           m_szOperId.asCharp(),
                           szObject,
                           szType,
                           szMethod);

    if (execute(m_Session, &szStmtDBARL[0], m_EvtLast)) {
      return execute(m_Session, XDBSQL_COMMIT, m_EvtLast);
    } // end if

    execute(m_Session, XDBSQL_ROLLBACK, m_EvtLast);
  } // end if

  return false;
} // end tin101_XDBBackupOperation :: start46B

/* 
  -----------------------------------------------------------------------------
  protected tin101_XDBBackupOperation :: start46C
  -----------------------------------------------------------------------------
*/
bool tin101_XDBBackupOperation :: start46C
    ( const char       * szFunction,
      const char       * szComponent,
      tsp00_Timestampc & szTimeStamp,
      const char       * szObject,
      const char       * szType,
      const char       * szMethod )
{
  FUNCTION_DBG_MIN100("tin101_XDBBackupOperation :: start46C");

  // call parent member start
  if (tin101_XDBOperation :: start(szFunction, szComponent, szTimeStamp, false)) {

    // make statement for DBARL
    char           szStmtDBARL[1024];
    _stprintf(szStmtDBARL, XDBSQL_46CINSDBABARL,   
                           m_R3Sid.asCharp(),      // SYS_ID
                           m_szOperId.asCharp(),   // OPER_ID
                           XDB_TASKNAME,           // TASKNAME
                           XDB_JOBNR,              // JOB_NR
                           XDB_TRIAL,              // TRIAL
                           XDB_UTILNR,             // UTIL_NR
                           m_szFuncId,             // FUNC_ID
                           m_szOperId.asCharp(),   // UPD_OP
                           szObject,               // OBJ_TYPE
                           szType,                 // BU_TYPE
                           szMethod);              // BU_METHOD

    if (execute(m_Session, &szStmtDBARL[0], m_EvtLast)) {
      return execute(m_Session, XDBSQL_COMMIT, m_EvtLast);
    } // end if

    execute(m_Session, XDBSQL_ROLLBACK, m_EvtLast);

  } // end if

  return false;
} // end tin101_XDBBackupOperation :: start46C
