/*!================================================================
 module:        vin201.cpp

 -------------------------------------------------------------------

 responsible:   DanielD

 special area:  SQL

 description:   Part

 -------------------------------------------------------------------

 copyright:     SAP AG 1999



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 ===================================================================*/

#include "hsp77.h"
#include "hin201.h"

/*----------------------------------------*/

cin201_Part::cin201_Part (tsp1_part * partArg)
{
  this->rawPart = partArg;
}

/*----------------------------------------*/

cin201_Part::~cin201_Part ()
{
  this->rawPart = NULL;
}

/*----------------------------------------*/

void cin201_Part::Init (tsp1_part_kind_Param partKind, int segmOffset, int segmSize)
{
  tsp1_part_header * header = &this->rawPart->sp1p_part_header ();
  
  header->sp1p_part_kind.becomes (partKind);
  header->sp1p_attributes.clear ();
  header->sp1p_arg_count = 0;
  header->sp1p_segm_offset = segmOffset;
  header->sp1p_buf_len = 0;
  header->sp1p_buf_size = segmSize - segmOffset - sizeof (tsp1_part_header);
}

/*----------------------------------------*/

int cin201_Part::Length () const
{
  return this->rawPart->sp1p_buf_len ();
}

/*----------------------------------------*/

int cin201_Part::BytesRemaining () const
{
  return this->rawPart->sp1p_buf_size () - this->rawPart->sp1p_buf_len ();
}

/*----------------------------------------*/

bool cin201_Part::IsValid () const
{
  return (this->rawPart != NULL);
}

/*----------------------------------------*/

void cin201_Part::Invalidate ()
{
  this->rawPart = NULL;
}

/*----------------------------------------*/

tsp1_part * cin201_Part::GetRawPart ()
{
  return this->rawPart;
}

/*----------------------------------------*/

const tsp1_part * cin201_Part::GetRawPart () const
{
  return this->rawPart;
}

/*----------------------------------------*/

tsp1_part_header * cin201_Part::GetRawHeader ()
{
  return &this->rawPart->sp1p_part_header ();
}

/*----------------------------------------*/

const tsp1_part_header * cin201_Part::GetRawHeader () const
{
  return &this->rawPart->sp1p_part_header ();
}

/*----------------------------------------*/

const teo00_Byte * cin201_Part::GetReadData (int pos) const
{
  return REINTERPRET_CAST (teo00_Byte*, this->rawPart->sp1p_buf ().asCharp ());
}

/*----------------------------------------*/

const teo00_Byte * cin201_Part::GetParameterData (int pos) const
{
  const teo00_Byte * dataBuf;
  const teo00_Byte * result;
  
  /*
  * get pointer
  */
  dataBuf = REINTERPRET_CAST (const teo00_Byte* , this->rawPart->sp1p_buf ().asCharp ());
  /*
  * check for null value
  */
  if (dataBuf [pos - 1] == csp_undef_byte) {
    result = NULL;
  }
  else {
    result = dataBuf + pos;
  }
  /*
  * return
  */
  return result;
}

/*----------------------------------------*/

teo00_Byte * cin201_Part::GetWriteData ()
{
  return REINTERPRET_CAST (teo00_Byte*, this->rawPart->sp1p_buf ().asCharp ())
    + this->rawPart->sp1p_buf_len ();
}

/*----------------------------------------*/

void cin201_Part::ExtendLength (int increment)
{
  PRECONDITION_DBG_MSP00 (this->rawPart->sp1p_buf_len () + increment < this->rawPart->sp1p_buf_size ());
  this->rawPart->sp1p_buf_len () += increment;
}

/*----------------------------------------*/

void cin201_Part::AddData (const void * buf, int len)
{
  PRECONDITION_DBG_MSP00 (this->rawPart->sp1p_buf_len () + len < this->rawPart->sp1p_buf_size ());
  memcpy (this->rawPart->sp1p_buf () + this->rawPart->sp1p_buf_len (), buf, len);
  this->rawPart->sp1p_buf_len () += len;
}

/*----------------------------------------*/

void cin201_Part::AddDataAsciiToUCS2 (const void * buf, int len, int swapped)
{
  PRECONDITION_DBG_MSP00 (this->rawPart->sp1p_buf_len () + len * 2 < this->rawPart->sp1p_buf_size ());
  unsigned int converted;
  sp81ASCIItoUCS2((tsp81_UCS2Char*)(this->rawPart->sp1p_buf () + this->rawPart->sp1p_buf_len ()), 
    len , swapped, &converted, (const unsigned char*)buf, len );
  this->rawPart->sp1p_buf_len () += len * 2;
}

/*----------------------------------------*/

void cin201_Part::AddDataUCS2ToAscii (const void * buf, int len, int swapped)
{
  PRECONDITION_DBG_MSP00 (this->rawPart->sp1p_buf_len () + len < this->rawPart->sp1p_buf_size ());
  unsigned int converted;
  sp81UCS2toASCII((unsigned char*)(this->rawPart->sp1p_buf () + this->rawPart->sp1p_buf_len ()), 
    len , &converted, (tsp81_UCS2Char*)buf, len, swapped );
  this->rawPart->sp1p_buf_len () += len;
}
/*----------------------------------------*/

void cin201_Part::AddArgument (const void * buf, int len)
{
  if (buf != NULL) {
    this->AddData (buf, len);
  }
  ++this->rawPart->sp1p_part_header ().sp1p_arg_count;
}

/*----------------------------------------*/

void cin201_Part::AddArgument ()
{
  ++this->rawPart->sp1p_part_header ().sp1p_arg_count;
}

/*----------------------------------------*/

void cin201_Part::AddParameterArg (
                                   const void * buf,
                                   int pos,
                                   int len,
                                   int sqllen,
                                   char typechar)
{
  char * dataBuf;
  int copyLen;
  int padLen;
  
  --sqllen; // adjust for defined byte
            /*
            * get pointer
  */
  dataBuf = this->rawPart->sp1p_buf ().asCharp ();
  /*
  * check for null value
  */
  if (buf == NULL) {
    dataBuf [pos - 1] = (char) csp_undef_byte; // STATIC_CAST (char, csp_undef_byte);
    IFSLOW_MSP00 (memset (dataBuf + pos, 0xff, sqllen));
  }
  else {
    dataBuf [pos - 1] = typechar;
    /*
    * copy data
    */
    if (len < sqllen) {
      copyLen = len;
      padLen = sqllen - len;
    }
    else {
      copyLen = sqllen;
      padLen = 0;
    }
    memcpy (dataBuf + pos, buf, copyLen);
    if (padLen > 0) {
      memset (dataBuf + pos + copyLen, typechar, padLen);
    }
  }
  /*
  * adjust part header
  */
  this->rawPart->sp1p_buf_len () =
    MAX_EO00 (this->rawPart->sp1p_buf_len (), pos + sqllen);
  ++this->rawPart->sp1p_arg_count ();
}

/*----------------------------------------*/

void cin201_Part::AddParameterArgUCS2 (const void * buf, 
                                       int pos, 
                                       int len,
                                       int isAsciiInput,
                                       int isAsciiOutput,
                                       int swapped,
                                       int sqllen, 
                                       tsp81_UCS2Char typechar)
{
  char * dataBuf;
  int copyLen;
  int padLen;
  
  --sqllen; // adjust for defined byte
            /*
            * get pointer
  */
  dataBuf = this->rawPart->sp1p_buf ().asCharp ();
  /*
  * check for null value
  */
  if (buf == NULL) {
    dataBuf [pos - 1] = (char) csp_undef_byte; // STATIC_CAST (char, csp_undef_byte);
    IFSLOW_MSP00 (memset (dataBuf + pos, 0xff, sqllen));
  }
  else {
    dataBuf [pos - 1] = csp_unicode_def_byte;
    /*
    * copy data
    */
    if ( isAsciiInput ) {
      if ( isAsciiOutput ) {
        if (len < sqllen) {
          copyLen = len;
          padLen = sqllen - len;
        }
        else {
          copyLen = sqllen;
          padLen = 0;
        }
        memcpy (dataBuf + pos, buf, copyLen);
      }
      else {
        unsigned int converted;
        sp81ASCIItoUCS2 ((tsp81_UCS2Char*)(dataBuf + pos), sqllen / 2, swapped,
          &converted, (const unsigned char*)buf, len );
        copyLen = 2 * converted;
        padLen = sqllen - copyLen;
      }
    }
    else {
      if ( isAsciiOutput ) {
        unsigned int converted;
        sp81UCS2toASCII((tsp00_Byte*)(dataBuf + pos), sqllen, &converted,
          (tsp81_UCS2Char*)buf, len, swapped );
        copyLen = converted;
        padLen = sqllen - copyLen;
      }
      else {
        len = len * 2;
        if (len < sqllen) {
          copyLen = len;
          padLen = sqllen - copyLen;
        }
        else {
          copyLen = sqllen;
          padLen = 0;
        }
        memcpy (dataBuf + pos, buf, copyLen);
      }
    }
    while (padLen > 0) {
      if ( isAsciiOutput ) {
        if (padLen > 0) {
          memset (dataBuf + pos + copyLen, ' ', padLen);
        }
      }
      else
        sp81UCS2strncpy ((tsp81_UCS2Char*)(dataBuf + pos + copyLen), &typechar, 1);
      copyLen = copyLen + 2;
      padLen  = padLen - 2;
    }
  }
  /*
  * adjust part header
  */
  this->rawPart->sp1p_buf_len () =
    MAX_EO00 (this->rawPart->sp1p_buf_len (), pos + sqllen);
  ++this->rawPart->sp1p_arg_count ();
}

/*----------------------------------------*/

typedef void partDumper_FT (
                            cin97_TraceStream * trace,
                            int argCount,
                            int len,
                            const teo00_Byte * data,
                            int buflimit);

typedef struct PartDumpInfoT {
  const char * name;
  partDumper_FT * func;
} PartDumpInfoT;

/*----------------------------------------*/

static void in201_StdDump (
                           cin97_TraceStream * trace,
                           int argCount,
                           int len,
                           const teo00_Byte * data,
                           int buflimit)
{
  if (buflimit > 0) {
    len = MIN_EO00 (len, buflimit);
  }
  trace->hex (NULL, data, len);
}

/*----------------------------------------*/

static PartDumpInfoT dumpInfoC [] = {
  {"nil", in201_StdDump},
  {"appl_parameter_description", in201_StdDump},
  {"columnnames", in201_StdDump},
  {"command", in201_StdDump},
  {"conv_tables_returned", in201_StdDump},
  {"data", in201_StdDump},
  {"errortext", in201_StdDump},
  {"getinfo", in201_StdDump},
  {"modulname", in201_StdDump},
  {"page", in201_StdDump},
  {"parsid", in201_StdDump},
  {"parsid_of_select", in201_StdDump},
  {"resultcount", in201_StdDump},
  {"resulttablename", in201_StdDump},
  {"shortinfo", in201_StdDump},
  {"user_info_returned", in201_StdDump},
  {"surrogate", in201_StdDump},
  {"bdinfo", in201_StdDump},
  {"longdata", in201_StdDump},
  {"tablename", in201_StdDump},
  {"session_info_returned", in201_StdDump},
  {"output_cols_no_parameter", in201_StdDump},
  {"key", in201_StdDump},
  {"serial", in201_StdDump},
  {"relative_pos", in201_StdDump},
  {"abap_istream", in201_StdDump},
  {"abap_ostream", in201_StdDump},
  {"abap_info", in201_StdDump}
};

/*----------------------------------------*/

void cin201_Part::TraceOn (cin97_TraceStream * trace, int buflimit)
{
  tsp1_part_header * header = &this->rawPart->sp1p_part_header ();
  tsp1_part_kind_Enum partKind = header->sp1p_part_kind;
  PartDumpInfoT * dumpInfo;
  char tmpBuf [100];
  PartDumpInfoT tmpInfo;
  
  /*
  * get dump function
  */
  if ((sp1pk_nil <= partKind) && (partKind <= sp1pk_abap_info)) {
    dumpInfo = &dumpInfoC [partKind];
    
  }
  else {
    sp77sprintf (tmpBuf, sizeof (tmpBuf), "Unknown Part %d", partKind);
    tmpInfo.name = tmpBuf;
    tmpInfo.func = in201_StdDump;
    dumpInfo = &tmpInfo;
  }
  /*
  * dump header
  */
  trace->printf ("        %s Part %d Arguments (%d of %d bytes) (Segment at %d)\n",
    dumpInfo->name, header->sp1p_arg_count, header->sp1p_buf_len,
    header->sp1p_buf_size, header->sp1p_segm_offset);
    /*
    * call dump function
  */
  const void * raw = this->rawPart->sp1p_buf ();
  const teo00_Byte * data = REINTERPRET_CAST (const teo00_Byte *, raw);
  dumpInfo->func (trace, header->sp1p_arg_count, header->sp1p_buf_len,
    data, buflimit);
  
}

/*----------------------------------------*/

