#!/usr/bin/perl
#
# PackDefs.pm
#
# $Header: //sapdb/V74/c_03/cor/sys/src/install/perl/BuildPackage/PackDefs.pm#8 $
# $DateTime: 2004/02/03 15:58:10 $
# $Change: 63246 $
#

# added test diff

# Desc:	- define packages and profiles of SAP DB Packages
#		
#
#
#    ========== licence begin LGPL
#    Copyright (C) 2002 SAP AG
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#    ========== licence end
#

		
package BuildPackage::PackDefs;

use Exporter;
use BuildPackage::Vars;

my $os = $BuildPackage::Distrib::opt_os;

my $isWindows = 0;
if(defined $os){
	$isWindows = 1 if ($os =~ /WIN/); 	
}
else{
	$isWindows = 1 if ($^O =~ /mswin/i);
}


@ISA = ('Exporter');

@EXPORT = ('%profile_in_name','%profiles','@allpackages',
			'%profile_data','@installer_files','%ALL_RPMS',
			'%RPM_PROFILES','@alltestpackages');

#define profiles -> contain package names

%profiles=(
	'Server' => ['Base','Server Utilities','Database Kernel','JDBC',"PCR $RUNTIME_VERSION",'ODBC','DB Analyzer'],
	
	#'APO LiveCache' => ['Base','Server Utilities','Database Kernel',"PCR $RUNTIME_VERSION",'APO LC APPS'],

	'APO LiveCache' => ['APO LC APPS'],

&if not ($OS == WIN32 && defined BIT64)
	'Script Interface' => ['Base','Perl Interface','Python Interface'],
&else
	'Script Interface' => ['Base','Python Interface'],
&endif

	'ODBC' => ['Base','ODBC'],

	'JDBC' => ['Base','JDBC'],

	'C Precompiler' => ['Base','CPC Base',"CPC SDK $RUNTIME_VERSION","PCR $RUNTIME_VERSION"],

	'XML Indexing Engine' => ['Base','ODBC','XML Indexing Engine'],

	'Runtime For SAP AS' => ['Base',"PCR $RUNTIME_VERSION",'JDBC','ODBC'],
	
	'Webtools' => ['Base', 'ODBC', 'Webtools','XML Indexing Engine'],
	
	'DB Analyzer' => ['Base',$isWindows ? 'ODBC' : undef ,'DB Analyzer'],

&if not ($OS == WIN32 && defined BIT64)	
	'Develop' => ['Base',"PCR $RUNTIME_VERSION","SDK $RUNTIME_VERSION",'ODBC','JDBC','Perl Interface','Python Interface']
&else
	'Develop' => ['Base',"PCR $RUNTIME_VERSION","SDK $RUNTIME_VERSION",'ODBC','JDBC','Python Interface']
&endif

);

my $name_COR = $COR;
$name_COR =~ s/^0*//g;
$name_COR = 0 unless $name_COR =~ /\S/; 
my $name_BUILD = $BUILD;
$name_BUILD =~ s/^0*//g;
$name_BUILD = 0 unless $name_BUILD =~ /\S/; 






#
#	additional profile data defs needed by SAP Software Center
#


my ($apo_rel_for_lca_package_name) = ($APO_SHORT_VERS =~ /^(.{2})/);
my ($apo_build_for_lca_package_name) = ($APO_BUILD =~ /(\d{2})$/);	# swc supports only 2 digits for apo com build
																	# ignore leading ones 
%profile_data = (
				'Server' => {
				
					'inner_subdir' => "sapdb-server-$OS-".$BIT."bit-$ARCH-".$MAJ.'_'.$MIN.'_'.$name_COR.'_'.$name_BUILD,
					'archive_name' => "SAPDB${MAJ}${MIN}${COR}_${BUILD}",
					'archive_name_tmpl' => 'SAPDB%MAJ%MIN%COR_%BUILD',				
					'inner_subdir_tmpl' => 'sapdb-server-%OS-%BITbit-%ARCH-%MAJ_%MIN_%cor_%build'
				},
				'APO LiveCache' => {
					'inner_subdir' => "sapdb-apo${APO_SHORT_VERS}_livecache-$OS-".$BIT."bit-$ARCH-".$MAJ.'_'.$MIN.'_'.$name_COR.'_'.$name_BUILD,
					#'archive_name' => "LCA${MAJ}${MIN}${COR}${BUILD}_${APO_BUILD}"				
					'archive_name' => "LC${MAJ}${MIN}${COR}${BUILD}A${apo_rel_for_lca_package_name}_${apo_build_for_lca_package_name}"					
				},
				'ODBC' => {
					'inner_subdir' => "sapdb-odbc-$OS-".$BIT."bit-$ARCH-".$MAJ.'_'.$MIN.'_'.$name_COR.'_'.$name_BUILD,
					'archive_name' => "SAPODBC${MAJ}${MIN}${COR}_${BUILD}",
					'archive_name_tmpl' => 'SAPODBC%MAJ%MIN%COR_%BUILD',				
					'inner_subdir_tmpl' => 'sapdb-odbc-%OS-%BITbit-%ARCH-%MAJ_%MIN_%cor_%build'					
				},
				'JDBC' => {
					'inner_subdir' => "sapdb-jdbc-$OS-".$BIT."bit-$ARCH-".$MAJ.'_'.$MIN.'_'.$name_COR.'_'.$name_BUILD,
					'archive_name' => "SAPJDBC${MAJ}${MIN}${COR}_${BUILD}",
					'archive_name_tmpl' => 'SAPJDBC%MAJ%MIN%COR_%BUILD',				
					'inner_subdir_tmpl' => 'sapdb-jdbc-%OS-%BITbit-%ARCH-%MAJ_%MIN_%cor_%build'					
				},
				'C Precompiler' => {
					'inner_subdir' => "sapdb-precompiler-$OS-".$BIT."bit-$ARCH-".$MAJ.'_'.$MIN.'_'.$name_COR.'_'.$name_BUILD,
					'archive_name' => "SAPDBCPC${MAJ}${MIN}${COR}_${BUILD}",
					'archive_name_tmpl' => 'SAPDBCPC%MAJ%MIN%COR_%BUILD',				
					'inner_subdir_tmpl' => 'sapdb-precompiler-%OS-%BITbit-%ARCH-%MAJ_%MIN_%cor_%build'					
				},
				'Script Interface' => {
					'inner_subdir' => "sapdb-scriptif-$OS-".$BIT."bit-$ARCH-".$MAJ.'_'.$MIN.'_'.$name_COR.'_'.$name_BUILD,
					'archive_name' => "SAPDBSIF${MAJ}${MIN}${COR}_${BUILD}",
					'archive_name_tmpl' => 'SAPDBSIF%MAJ%MIN%COR_%BUILD',				
					'inner_subdir_tmpl' => 'sapdb-scriptif-%OS-%BITbit-%ARCH-%MAJ_%MIN_%cor_%build'					
				},
				'Runtime For SAP AS' => {
					'inner_subdir' => "sapdb-as_runtime-$OS-".$BIT."bit-$ARCH-".$MAJ.'_'.$MIN.'_'.$name_COR.'_'.$name_BUILD,
					'archive_name' => "SAPDBART${MAJ}${MIN}${COR}_${BUILD}",
					'archive_name_tmpl' => 'SAPDBART%MAJ%MIN%COR_%BUILD',				
					'inner_subdir_tmpl' => 'sapdb-as_runtime-%OS-%BITbit-%ARCH-%MAJ_%MIN_%cor_%build',
				},
				'Webtools' => {
					'inner_subdir' => "sapdb-webtools-$OS-".$BIT."bit-$ARCH-".$MAJ.'_'.$MIN.'_'.$name_COR.'_'.$name_BUILD,
					'archive_name' => "SAPDBWEB${MAJ}${MIN}${COR}_${BUILD}",
					'archive_name_tmpl' => 'SAPDBWEB%MAJ%MIN%COR_%BUILD',				
					'inner_subdir_tmpl' => 'sapdb-webtools-%OS-%BITbit-%ARCH-%MAJ_%MIN_%cor_%build'
					
				},
				'DB Analyzer' => {
					'inner_subdir' => "sapdb-analyzer-$OS-".$BIT."bit-$ARCH-".$MAJ.'_'.$MIN.'_'.$name_COR.'_'.$name_BUILD,
					'archive_name' => "SAPDBANA${MAJ}${MIN}${COR}_${BUILD}",
					'archive_name_tmpl' => 'SAPDBANA%MAJ%MIN%COR_%BUILD',				
					'inner_subdir_tmpl' => 'sapdb-analyzer-%OS-%BITbit-%ARCH-%MAJ_%MIN_%cor_%build'
				},
 				'XML Indexing Engine' => {
 					'inner_subdir' => "sapdb-xie-$OS-".$BIT."bit-$ARCH-".$MAJ.'_'.$MIN.'_'.$name_COR.'_'.$name_BUILD,
 					'archive_name' => "SAPDBXIE${MAJ}${MIN}${COR}_${BUILD}",
 					'archive_name_tmpl' => 'SAPDBXIE%MAJ%MIN%COR_%BUILD',				
 					'inner_subdir_tmpl' => 'sapdb-xie-%OS-%BITbit-%ARCH-%MAJ_%MIN_%cor_%build'
				}
);


my $exe_ext = $isWindows ? '.exe' : '';

#
# files of sapdb installer (without SDBUPD - only needed for Server / APO LiveCache)
#

@installer_files = ('SDBINST'.$exe_ext, 'SDBINST.TGZ');
push @installer_files, ($isWindows ? 'sdbrun.dll' : 'SDBRUN');
push @installer_files,'perl56.dll' if $isWindows;


&if $OS == WIN32 && $_MSC_VER == 13 && not defined BIT64
push @installer_files,'msvcr70.dll';
&endif



%profile_in_name = (

	'Runtime For SAP AS' => 'as_runtime',
	'APO LiveCache' => "apo${APO_SHORT_VERS}_livecache",
	'Script Interface' => 'scriptif'

);

	
	
#define packages
@allpackages=(
		{
			'name' => 'Base',
			'subdir' => 'base',
			'script' => 'instbase',
			'archive' => 'SAPDBBAS.TGZ',
			'filelist' => 'base.lst'
&if $OS == WIN32 && $_MSC_VER == 13 && not defined BIT64
			,'add_files' => ['mfc70.dll','msvcr70.dll']	
&endif		
		},
		{
			'name' => 'Server Utilities',
			'subdir' => 'utils',
			'script' => 'instutils',
			'archive' => 'SAPDBUTL.TGZ',
			'filelist' => 'utils.lst'	
		},
		{
			'name' => 'Database Kernel',
			'subdir' => 'kernel',
			'script' => 'instsrv',
			'archive' => 'SAPDBKRN.TGZ',
			'pmask' => 0227,
			'filelist' => 'kernel.lst',
			'additional_installer_files' => ['SDBUPD'.$exe_ext]
		},
		{
			'name' => "PCR $RUNTIME_VERSION",
			'subdir' => 'pcr',
			'script' => 'instpcr',
			'archive' => "PCR$MAJ$MIN$COR.TGZ",
			'filelist' => 'pcr.lst',
			'specialFunc' => \&BuildPackage::New::copyPCR
		},
		
		{
			'name' => 'DB Analyzer',
			'subdir' => 'dbana',
			'script' => 'instdbana',
			'archive' => 'SAPDBANA.TGZ',
			'filelist' => 'dbana.lst'	
		},
		{
			'name' => "CPC Base",
			'subdir' => 'cpcbase',
			'script' => 'instcpcbase',
			'archive' => "CPCBASE.TGZ",
			'filelist' => 'cpcbase.lst'	
		},
		{
			'name' => "CPC SDK $RUNTIME_VERSION",
			'subdir' => 'cpcsdk',
			'script' => 'instcpcsdk',
			'archive' => "CSDK$RUNTIME_VERSION.TGZ",
			'filelist' => 'cpcsdk.lst'	
		},
		{
			'name' => 'Webtools',
			'subdir' => 'web',
			'script' => 'instweb',
			'archive' => 'SAPDBWEB.TGZ',
			'filelist' => 'web.lst'	
		},
		{
			'name' => 'ODBC',
			'subdir' => 'odbc',
			'script' => 'instodbc',
			'archive' => 'SAPODBC.TGZ',
			'filelist' => 'odbc.lst'	
		},	
		{
			'name' => 'JDBC',
			'subdir' => 'jdbc',
			'script' => 'instjdbc',
			'archive' => 'SAPJDBC.TGZ',
			'filelist' => 'jdbc.lst'
			#'FileMappings' => $isWindows ? {"$RUNTIME_PATH/pgm/sapdbcbin.dll" => 'pgm/sapdbcbin.dll','runtime/jar/sapdbc.jar' => 'jar/sapdbc.jar'} : {'runtime/jar/sapdbc.jar' => 'jar/sapdbc.jar'} 	
		},
&if not ($OS == WIN32 && defined BIT64)
		{
			'name' => 'Perl Interface',
			'subdir' => 'perl',
			'script' => 'instperl',
			'archive' => 'SAPDBPL.TGZ',
			'filelist' => 'perl.lst'
		},
&endif
		{
			'name' => 'Python Interface',
			'subdir' => 'python',
			'script' => 'instpy',
			'archive' => 'SAPDBPY.TGZ',
			'filelist' => 'py.lst'	
		},
		{
			'name' => 'APO LC APPS',
			'subdir' => 'apocom',
			'script' => 'instapocom',
			'archive' => 'APOLCA.TGZ',
			'filelist' => 'apocom.lst'
		},
 		{
 			'name' => 'XML Indexing Engine',
 			'subdir' => 'xie',
 			'script' => 'instxie',
 			'archive' => 'SAPDBXIE.TGZ',
 			'filelist' => 'xie.lst',
 			'FileMappings' => {'config/sapdbxie.reg' => 'config/sapdbxie.reg.org','config/sapdbxie.ini' => 'config/sapdbxie.ini.org'}
		}
		
);


@alltestpackages=(
		{
			'name' => 'OMS Test Kernel',
			'subdir' => 'tstknl',
			'script' => 'insttstknl',
			'archive' => 'TSTKNL.TGZ',
			'filelist' => 'tstknl.lst'
		},
		{
			'name' => 'LC Simulator',
			'subdir' => 'lcsim',
			'script' => 'instlcsim',
			'archive' => 'LCSIMUL.TGZ',
			'filelist' => 'lcsim.lst'
		},
		{
			'name' => 'Test Apps',
			'pmask' => 0227,
			'subdir' => 'tstapps',
			'script' => 'insttstapps',
			'archive' => 'TSTAPPS.TGZ',
			'filelist' => 'tstapps.lst'
		}

);





%ALL_RPMS = (
	'ind' => 'sapdb-ind-%MAJ.%MIN.%cor.%build-1.i386.rpm',
	'srv74' => 'sapdb-srv74-%MAJ.%MIN.%cor.%build-1.i386.rpm',
	'web' => 'sapdb-web-%MAJ.%MIN.%cor.%build-1.i386.rpm',
	'callif' => 'sapdb-callif-%MAJ.%MIN.%cor.%build-1.i386.rpm',
	'scriptif' => 'sapdb-scriptif-%MAJ.%MIN.%cor.%build-1.i386.rpm',
	'testdb74' => 'sapdb-testdb74-%MAJ.%MIN.%cor.%build-1.i386.rpm',
	'precompiler' => 'sapdb-precompiler-%MAJ.%MIN.%cor.%build-1.i386.rpm'
);

%RPM_PROFILES = (
	'Server' => ['ind','srv74','testdb74'],
	'Runtime For SAP AS' => ['ind'],
	'DB Analyzer'  => ['ind'],
	'Webtools' => ['callif','web'],
	'ODBC' => ['callif'],
	'JDBC' => ['callif'],
	'SDK' => ['callif','scriptif','precompiler'],
	'C Precompiler' => ['precompiler']
);



1;