/****************************************************************************

  module      : hkb04.h

  -------------------------------------------------------------------------

  author      : JuergenA
  responsible : UweH

  special area: KB_object_handling_interface
  description : 


  last changed: 1999-06-22  15:48
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1998



    ========== licence begin  GPL
    Copyright (C) 2000 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/


#ifndef HKB04_H
#define HKB04_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg92.h"

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/

externCpp void
kb04CreateObjFile (tgg00_TransContext	&Trans,
                   tgg00_FileId 	    &ObjFile,
                   tsp00_Int4       	 ObjBodyLen,
                   tsp00_Int4            ObjKeyLen,
				   tsp00_Bool            bCreateSeveralObjKeyFiles);			   

/*!***************************************************************************
function   : kb04CreateKeyIterator
description: Creates an iterator running over a predefined key range
             
             This function creates a key iterator which runs over all objects with
			 keys between the given start and stop keys. The iterator is returned via a
			 void pointer. If the pointer to the start key  is empty and the start key len is zero
			 the iterator starts with the smallest key. If the stop key pointer is nil and the stop key len
			 is zero the iterator stops at the largest key. The iterator can be dereferenced by the 
			 function kb04NexObjFromKeyIterator. 

arguments : Trans            [in/out] TransContext
            ObjFileId        [in]   Id of the file to scan
            KeyLen           [in]   common length of the start and stop key 
            pStartKey        [in]   pointer to the start key
            pStopKey         [in]   pointer to the stop key
            pKeyIterator     [out]  pointer to the key range iterator
******************************************************************************/ 
externCpp void
kb04CreateKeyIterator (tgg00_TransContext &Trans,
				       tgg00_FileId		  &ObjFile,
				       tsp00_Int4	       KeyLen,
                       void		          *pStartKey,
                       void		          *pStopKey,
				       void              *&pKeyIterator);

/*---------------------------------------------------------------------------*/

externCpp void
kb04DeleteAllObj (tgg00_TransContext &Trans,
                  tgg00_FileId		 &ObjFile,
                  tsp00_Int4         &NumDelObj,
                  tgg92_KernelOid    &ErrorOid);

/*---------------------------------------------------------------------------*/

externCpp void
kb04DeleteObj (tgg00_TransContext &Trans,
               tgg00_FileId       &ObjFile,
               tgg92_KernelOid     Oid,
               tgg91_PageRef       WantedObjVers,
               tgg00_ObjColDesc   &ColDesc);


/*!***************************************************************************
function   : kb04DestroyKeyIterator
description: Releases all resources held by the iterator
             
arguments : Trans            [in/out]  TransContext
            pKeyIterator     [in]      pointer to the key range iterator
******************************************************************************/ 
externCpp void
kb04DestroyKeyIterator (tgg00_TransContext &Trans,
				        void              *&pKeyIterator);

/*!***************************************************************************
function   : kb04DropObjFile
description: Drops a complete file
             
             The complete object file including all auxiliary structures are removed
             asynchronously. The command does not consider whether all objects to be deleted
             belong to the consistent view of the current transaction. To avoid destruction of 
             consistent views of other transactions this command should be performed only if there
             exists no other open transaction.

arguments : Trans         [in/out]  TransContext
            ObjFileId     [in]      Id of the file to be deleted

******************************************************************************/                     
externCpp void
kb04DropObjFile (tgg00_TransContext    &Trans,
                 tgg00_FileId          &ObjFileId);

/*---------------------------------------------------------------------------*/

externCpp void
kb04EndConsistentTrans (tgg00_TransContext &Trans);


/*---------------------------------------------------------------------------*/

externCpp void
kb04GetObjConsistent (tgg00_TransContext &Trans,
                      tgg00_FileId       &ObjFile,     
                      tgg92_KernelOid     Oid,
                      tsp00_Int4          ObjBodySize,     //[inp=buffer size for object] 
                      void               *pObjBody,        //[inp]
                      bool                bDoLock,         //[inp] 
                      tsp00_Int4         &ObjBodyLen,      //[out=real object length]
                      tsp00_Int4         &LogReadAccesses, //[out] PTS 1107890 TA 13/09/2000
                      tgg91_PageRef      &ResultObjVers);

/*---------------------------------------------------------------------------*/

externCpp void
kb04GetObjWithKeyConsistent (tgg00_TransContext	 &Trans,
                             tgg00_FileId	       &ObjFile,
                             tsp00_Int4		        ObjBodySize,
                             void			           *pObjBody,
                             tsp00_Int4	          KeyLen,
                             void		             *pKey,
                             bool                 bDoLock,         //[inp] 
                             tsp00_Int4          &LogReadAccesses, //[out] PTS 1107890 TA 13/09/2000
                             tgg92_KernelOid     &ResultOid,
                             tgg91_PageRef       &ResultObjVers);

/*---------------------------------------------------------------------------*/

/* PTS 1106284 UH 07-04-2000 new */
externCpp void
kb04IsObjHistoryUsed (tgg00_TransContext &Trans,
                      tgg00_FileId       &ObjFile,
                      tgg92_KernelOid     Oid,
                      bool               &bHistoryIsUsed);

/*---------------------------------------------------------------------------*/

/* PTS 1001745 UH 23-11-1998 new */
externCpp void
kb04IsObjSelfLocked (tgg00_TransContext &Trans,
                     tgg00_FileId       &ObjFile,
                     tgg92_KernelOid     Oid,
                     bool               &IsSelfLocked);

/*---------------------------------------------------------------------------*/

externCpp void
kb04LockObj (tgg00_TransContext &Trans,
             tgg00_FileId       &ObjFile,    
             tgg92_KernelOid     Oid,
             tgg91_PageRef       WantedObjVers);

/*---------------------------------------------------------------------------*/

/* PTS 1109298 UH 2001-01-26 new */
externCpp void
kb04LockUpdObj (tgg00_TransContext &Trans,
                tgg00_FileId       &ObjFile,    
                tgg92_KernelOid     Oid,
                tgg91_PageRef       WantedObjVers);

/*---------------------------------------------------------------------------*/


externCpp void
kb04NewObj (tgg00_TransContext &Trans,
            tgg00_FileId       &ObjFile,
            tsp00_Int4          KeyLen,
            void               *KeyColumn,
            tgg92_KernelOid    &ResultOid,
            tgg91_PageRef      &ResultObjVers);

/*---------------------------------------------------------------------------*/

externCpp void
kb04NextObjConsistent (tgg00_TransContext &Trans,
                       tgg00_FileId       &ObjFile,    
                       tgg92_KernelOid    &CurrOid,
                       bool                bWithObject,
                       tsp00_Int4          ObjBodySize,
                       void               *pObjBody,
                       tsp00_Int4&        LogReadAccesses, // PTS 1107890 TA 13/09/2000
                       tgg91_PageRef      &ResultObjVers);

/*!***************************************************************************
function   : kb04NextObjFromKeyIterator
description: returns the object the KeyIterator points currently and moves the iterator to the next object 
             
            If this function is called the first time after the iterator was created by 
			kb04CreateKeyIterator it returns the very first object which resides
			in the scope defined when the iterator was created. 
			If the this function is called consecutively the complete scope of the iterator can be scaned.
			After the last object was delivered the iterator sets the error state in the TransactionContect
		    to e_no_next_object and returns a nil result oid.


arguments : Trans            [in/out] TransContext
            pUntypedKeyIterator  [in] pointer to the key range iterator
            bWithObject      [in]   if true not only the OId but also the object body is delivered
			ExtObjBodySize   [in]   Size of the memory provided for the object body
			pExtObjBody      [in]   pointer to the position in the meory where the body
			                        of the rsult object is copied to.
            ResultOId        [out]  OId of the next object in the iterator scope
            ResultObjVers    [out]  version of the rsult object
            ResultObjBodySize[out]  real size of the result object
            LogReadAccesses  [out]  total number of read accesses to the log to find the next consisten object
******************************************************************************/ 
externCpp void
kb04NextObjFromKeyIterator (tgg00_TransContext &Trans,
                            void               *pUntypedKeyIterator,    
                            bool                bWithObject,
                            tsp00_Int4          ExtObjBodySize,
                            void               *pExtObjBody,
                            tgg92_KernelOid    &ResultOId, 
                            tgg91_PageRef      &ResultObjVers,
                            tsp00_Int4         &ResultObjBodySize,
                            tsp00_Int4         &LogReadAccesses);

/*---------------------------------------------------------------------------*/

externCpp void
kb04StartConsistentTrans (tgg00_TransContext &Trans);


/*---------------------------------------------------------------------------*/

externCpp void
kb04UnlockObj (tgg00_TransContext &Trans,
               tgg00_FileId       &ObjFile, 
               tgg92_KernelOid     Oid);

/*---------------------------------------------------------------------------*/


externCpp void
kb04UpdateObj (tgg00_TransContext &Trans,
               tgg00_FileId       &ObjFile,
               tgg92_KernelOid     Oid,
               tgg91_PageRef      &ObjVers,
               tsp00_Int4          NewBodySize,
               void               *pNewBody,
               tgg00_ObjColDesc   &ColDesc);


/*---------------------------------------------------------------------------*/

#endif /* HKB04_H */
