
/*!**************************************************************************

    module      : vkb811.cpp

    -------------------------------------------------------------------------

    author      : JuergenA,UweH
    responsible : UweH

    special area: Logging
    description : KB_RedoManager_interface

    last changed: 2001-03-22

    -------------------------------------------------------------------------

    copyright:    Copyright by SAP AG, 2001

    ========== licence begin LGPL
    Copyright (C) 2001 SAP AG
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

*****************************************************************************/

/*==========================================================================*
*  INCLUDES                                                                 *
*===========================================================================*/

#include "RunTime/RTE_Message.hpp"
#include "SAPDBCommon/Tracing/SAPDBTrace_Usage.hpp"
#include "SAPDBCommon/ErrorsAndMessages/SAPDBErr_Assertions.hpp"
#include "Logging/Log_Types.hpp"
#include "Logging/Log_Exceptions.hpp"
#include "Logging/Log_RedoManager.hpp"
#include "ggg00.h"
#include "hkb811.h" // check of PASCAL header vkb811


/*==========================================================================*
* PASCAL INTERFACE                                                          *
*===========================================================================*/

Log_RedoManager *pRedoManagerInstance = NULL;
    
// --------------------------------------------------------------------------

externPascal void kb811RedoManager (
    tgg00_TransContext    VAR_VALUE_REF  TransContext,
    tsp00_Int4                           untilDate,
    tsp00_Int4                           untilTime,
    tsp00_PageNo                         untilIOSeq,
    pasbool                              restoreLog)
{
    SAPDBTRACE_ROUTINE_DEBUG ("kb811RedoManager", LogTrans_Trace, 5);

    Log_RedoManager RedoManager;
    
    // pRedoManagerInstance MUST NOT BE != NULL
    pRedoManagerInstance = &RedoManager;

    RedoManager.RedoLog (TransContext,
                         Kernel_Date(untilDate), Kernel_Time(untilTime),
                         Log_IOSequenceNo((SAPDB_UInt4)untilIOSeq),
                         restoreLog != 0);
    
    pRedoManagerInstance = NULL;
}

// --------------------------------------------------------------------------

externPascal void kb811AbortRedo (
    tgg00_BasisError                     abortReason)
{
    SAPDBTRACE_ROUTINE_DEBUG ("kb811AbortRedo", LogTrans_Trace, 5);

    if ( pRedoManagerInstance != NULL )
    {
        pRedoManagerInstance->AbortRedo(abortReason);
    }
}

// --------------------------------------------------------------------------

externPascal void kb811GetRedoProgressInfo (
    tsp00_Int4            VAR_VALUE_REF  transread,
    tsp00_Int4            VAR_VALUE_REF  transredone)
{
    SAPDBTRACE_ROUTINE_DEBUG ("kb811GetRedoProgressInfo", LogTrans_Trace, 5);
    
    if ( NULL == pRedoManagerInstance )
    {
        transread   = 0;
        transredone = 0;
        return;
    }
    SAPDB_UInt auxtransread   = 0;
    SAPDB_UInt auxtransredone = 0;
    pRedoManagerInstance->GetProgressInfo(auxtransread, auxtransredone);
    transread   = auxtransread;
    transredone = auxtransredone;
}

// --------------------------------------------------------------------------
// No header exists for this routine.
externC void  voffline ( tsp00_TaskId pid, SAPDB_Byte value );

externPascal void kb811ForceOffline (
    tsp00_TaskId                         taskid,
    tgg00_BasisError                     error)
{
    SAPDBTRACE_ROUTINE_DEBUG ("kb811ForceOffline", LogTrans_Trace, 5);
    // PTS 1117603 UH 2002-09-12 new
    Log_Exception exception (__CONTEXT__,LOG_FORCE_OFFLINE,SAPDB_ToString(error));
    
    #ifdef SAPDB_QUICK
    if (   error != e_incompatible_log
        && error != e_cancelled
        && error != e_save_skipped )
    {
        RTE_Crash(exception);
    }
    #endif

    if ( LogTrans_Check.ChecksLevel(5) )
        RTE_Crash(exception);
        
    RTE_Message(exception);
    voffline (taskid, shtShutdKill_esp00/*IN FUTURE ,error*/);
    // -------------------------
    // THE KERNEL IS OFFLINE NOW
    // -------------------------
}

/*===========================================================================*
*  END OF CODE                                                               *
*============================================================================*/
