/*! 
  -----------------------------------------------------------------------------
 
  module: hls22.h
 
  -----------------------------------------------------------------------------
 
  responsible:  SteffenS

  special area: Replication Manager
 
  description:  Fastload - filling db pages (definitions)

  version:      7.4.

  -----------------------------------------------------------------------------
 
  copyright:    SAP AG, 1999-2004
 
  -----------------------------------------------------------------------------



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end


*/

#ifndef HLS22
#define HLS22

#include "gls00.h"
//#include "gbd00.h"      // database bd layer (data page layout)


/*! Declaration: Special return codes */

#define STOP_LS22                    101
#define STOP_NODATA_LS22             102

/*! EndDeclaration: Special return codes */


/*!
  -----------------------------------------------------------------------------
  Chapter: global functions
  -----------------------------------------------------------------------------
*/

/*!
  -----------------------------------------------------------------------------
 
  function:     ls22LoadPages

  -----------------------------------------------------------------------------

  description:  Loads data pages to a given db table (FASTLOAD).
                Initializes file structure for the given data file, opens it,
                reads data records, builds up a db data page and sends the
                pages to the db.
  
  arguments:    FloadCmd  [IN]   - fastload structure built by the parser from
                                    given command
                FloadRec  [IN]   - fastload structure built by the server
                DBInfo    [IN]   - dbinfo structure
                ErrText   [OUT]  - error text

  returnvalue:  errOK_els00             - data loaded
                errors when loading data failed

                Logs the error in log file.
  -----------------------------------------------------------------------------
*/
int ls22LoadPages(tls00_FloadCmd *FloadCmd,
                  tls00_FloadRec *FloadRec,
                  tls00_DBInfo   *DBInfo,
                  tsp00_Addr      ErrText);

/*!
  -----------------------------------------------------------------------------
  End Chapter: global functions
  -----------------------------------------------------------------------------
 */

#endif /*HLS22*/
