/*!**************************************************************************

  module      : gos00cn.h

  -------------------------------------------------------------------------

  responsible : RaymondR

  special area: Console
  description : 


  last changed: 2001-09-21  12:20
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/


#ifndef GOS00CN_H
#define GOS00CN_H


/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "gos00.h"
#include "gos00k.h"
#include "heo46.h"
#include "heo00.h"
#include "geo52.h"
#include "geo002.h"

//
// +---------------------------------------------------------------+
// |   DEFINES                                                     |
// +---------------------------------------------------------------+
//
#define SECURITY_CHECK_ACCESS_RIGHTS    0x8000

#define CON_REQ_TASK                    1
#define CON_REQ_IO                      2
#define CON_REQ_REGION                  3
#define CON_REQ_STORAGE                 4
#define CON_REQ_STATE                   5
#define CON_REQ_RTE                     6
#define CON_REQ_CANCEL                  7 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_KILL                    8 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_TASK_STAT               9
#define CON_REQ_TASK_COUNTS            10
#define CON_REQ_QUEUE_STAT             11
#define CON_REQ_TOTAL_COUNTS           12
#define CON_REQ_ENABLE_TIME_MEASURE    13 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_DISABLE_TIME_MEASURE   14 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_UKT_SLEEPTIME_STAT     15
#define CON_REQ_SHOW_SUSPENDS          16
#define CON_REQ_THREAD_TIMES           17
#define CON_REQ_AWE                    18
#define CON_REQ_CONNECTIONS            19
#define CON_REQ_CONNECTIONS            19
#define CON_REQ_TASK_MOVE              20
#define CON_REQ_SET_CHECK_INTERVAL     21 | SECURITY_CHECK_ACCESS_RIGHTS

#define CON_REQ_CRASH_DUMP             27 | SECURITY_CHECK_ACCESS_RIGHTS

#define CON_REQ_RES_IO                 28 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_RES_REGION             29 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_RES_TASK_COUNTS        30 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_RES_UKT_SLEEPTIME_STAT 31 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_RES_QUEUE_STAT         32 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_RES_SUSPENDS           33 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_RES_ALL                34 | SECURITY_CHECK_ACCESS_RIGHTS

#define CON_REQ_GETPARAM               35 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_PUTPARAM               36 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_SET_DEBUGLEVEL         37 | SECURITY_CHECK_ACCESS_RIGHTS

#define CON_REQ_UKT_QUEUES             38
#define CON_REQ_SET_UKT_AFFINITY       40 | SECURITY_CHECK_ACCESS_RIGHTS

#define CON_REQ_AIO                    41 | SECURITY_CHECK_ACCESS_RIGHTS

#define CON_REQ_FORCE_IO_ERROR         42 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_DEBUG_BREAK            43 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_SET_PROCESSOR_MASK     44 | SECURITY_CHECK_ACCESS_RIGHTS
#define CON_REQ_DEBUG_TASK             45 | SECURITY_CHECK_ACCESS_RIGHTS

//
// --- RTE return codes ( cancel, kill, debug break )
//
#define CON_NO_ERROR                    0
#define CON_ERROR                       1
#define CON_WRONG_TASK                  2
#define CON_WRONG_TASK_TYPE             3
#define CON_WRONG_UKT                   4
#define CON_NO_PROCESSOR                5
#define CON_NO_INTERACT_WITH_DECTOP     6
#define CON_TASK_INACTIVE               7
#define CON_WRONG_VALUE                 8

//
// +---------------------------------------------------------------+
// |   TYPEDEFS AND STRUCTS                                        |
// +---------------------------------------------------------------+
//

//
// --- Common DEV info structs
//
typedef struct cons_dev_info_record
  {
  ULONG                           ulDevspaceNo;
  tsp2_devnamec                   DevspaceName;
  ULONG                           ulReadCnt;
  ULONG                           ulWriteCnt;
  } CONS_DEV_INFO_REC;

typedef CONS_DEV_INFO_REC         *PCONS_DEV_INFO_REC;

typedef struct cons_devi_info_record
  {
  TID                             Tid;
  tsp2_devnamec                   DevspaceName;
  LONG                            ulDEViIndex;
  ULONG                           ulDevspaceNo;
  teo52_DevIOStatistic            DevIOStat ;
  } CONS_DEVI_INFO_REC;

typedef CONS_DEVI_INFO_REC        *PCONS_DEVI_INFO_REC;

typedef struct cons_asynci_info_record
  {
  TID                             Tid;
  VF_FILENAMEC                    szLogDeviceName;
  BOOL                            fForWriting;
  BOOL                            ulBlockSize;
  BOOL                            ulMaxBlockCnt;
  teo52_DevIOStatistic            DevIOStat ;
  } CONS_ASYNCI_INFO_REC;

typedef CONS_ASYNCI_INFO_REC      *PCONS_ASYNCI_INFO_REC;


//
// --- Task info
//
typedef struct cons_single_task_info_record
  {
  ULONG                           ulTaskIndex;
  TID                             Tid;
  PID                             ApplPid;            // - process id of connected user procces
  SQL_NODEIDC                     szClientNode;
  ULONG                           ulStateReason;      // - more state infos
  tsp00_Longuint                  ulTaskTimeOut;      // - time for task's timeout if still in VRECEIVE
  REGION_ID                       Sid;
  ULONG                           ulBegExclCnt;
  LONG                            lWaitForTask;
  LONG                            lWaitForRoot ;
  LONG                            lWaitForDevno ;
  tsp2_devnamec                   DevNameWaitingFor;
  UINT1                           TaskState;          // - task state (TSK_RUNNING,...)
  UINT1                           TaskType;           // - task type (TT_US,...)
  UINT1                           PrioFlag;
  UINT1                           SavePrioFlag;
  UINT1                           QueLocation;
  UINT1                           CancelFlag;         /* CR 1103476 */
  UINT1                           OmsCallbackActive;  /* CR 1106969 */
  UINT1                           UktSleepingFlag;    /* CR 1115172 */
  tsp00_PageNo                    LockedPageNo;       /* CR 1115172 */
  ULONG                           ulUktIndex;         /* CR 1115172 */
  ULONG                           UktSleepCount;      /* CR 1115172 */
  LONG                            WaitTime;           /* CR 1115172 */
  ULONG                           ulDynPrio;          /* CR 1115172 */
  SAPDB_UInt8                     RunnableMicroSeconds;
  SAPDB_UInt8                     MaxRunnableMicroSeconds;
  SAPDB_UInt2                     TemporaryMoveLock;    
  SAPDB_UInt2                     LongTermMoveLock;  
  } CONS_SINGLE_TASK_INFO_REC;

typedef CONS_SINGLE_TASK_INFO_REC *PCONS_SINGLE_TASK_INFO_REC;


typedef struct cons_task_info_record
  {
  UINT1                           StateToShow ;
  UINT1                           TypeToShow ;
  LONG                            lNumOfActiveTasks;
  ULONG                           ulTaskInfoOffset;        // - CONS_SINGLE_TASK_INFO_REC * 'MAX_TASKS'
  } CONS_TASK_INFO_REC;

typedef CONS_TASK_INFO_REC        *PCONS_TASK_INFO_REC;


//
// --- IO info
//
typedef struct cons_io_info_record
  {
  LONG                            lNumOfDevspaces;
  ULONG                           ulDevspaceInfoOffset;    // - CONS_DEV_INFO_REC  * 'MAX_DEVSPACES'
  } CONS_IO_INFO_REC;

typedef CONS_IO_INFO_REC          *PCONS_IO_INFO_REC;

//
// --- AIO info
//
typedef struct cons_aio_info_record
  {
  LONG                            lNumOfASYNCis;
  ULONG                           ulASYNCiInfoOffset;      // - CONS_ASYNCI_INFO_REC * 'MAX_DEVSPACES' + 'MAX_BACKUP_DEVS'
  } CONS_AIO_INFO_REC;

typedef CONS_AIO_INFO_REC         *PCONS_AIO_INFO_REC;

//
// --- Region info
//
typedef struct cons_single_sem_info_record
  {
  ULONG                           ulSid;
  ULONG                           ulOwnerTaskIndex;
  C8C                             szSemaName;
  ULONG                           lGetCount;
  ULONG                           lBegExclTasCount;
  ULONG                           lEndExclTasCount;
  ULONG                           lCollisionCount;
  ULONG                           lWaitCount;
  LONG                            lExclusive;
  ULONG                           lNumWaitTasks;
  ULONG                           ulWaitTaskOffset;        // - ULONG * 'MAX_TASKS'
  } CONS_SINGLE_SEM_INFO_REC;

typedef CONS_SINGLE_SEM_INFO_REC  *PCONS_SINGLE_SEM_INFO_REC;


typedef struct cons_sem_info_record
  {
  LONG                            lNumOfSems;
  ULONG                           ulSingleSemInfoOffset;   // - CONS_SINGLE_SEM_INFO_REC * 'MAX_SEMAPHORES'
  } CONS_SEM_INFO_REC;

typedef CONS_SEM_INFO_REC         *PCONS_SEM_INFO_REC;



//
// --- Storage info
//

typedef struct cons_task_storage_info_record
  {
  ULONG                           ulTaskIndex;
  UINT1                           TaskType;           // - task type (TT_US,...)
  tsp00_Longint                   ulStackUsed;
  tsp00_Longint                   ulStackFree;
  } CONS_TASK_STORAGE_INFO_REC;

typedef CONS_TASK_STORAGE_INFO_REC *PCONS_TASK_STORAGE_INFO_REC;


typedef struct cons_storage_info_record
  {
  tsp00_Longint                   lDynDatAvail;
  tsp00_Longint                   lDynDatUsed;
  tsp00_Longint                   lDynPoolAvail;
  tsp00_Longint                   lDynPoolUsed;
  LONG                            lTaskStacks;
  ULONG                           ulKGSSize;
  ULONG                           ulKSSSize;
  ULONG                           ulTasksTotal;
  LONG                            lNumOfTasks;
  ULONG                           ulTaskStorInfoOffset;    // - CONS_TASK_STORAGE_INFO_REC * 'MAX_TASKS'
  } CONS_STORAGE_INFO_REC;

typedef CONS_STORAGE_INFO_REC     *PCONS_STORAGE_INFO_REC;



//
// --- State info
//
typedef struct cons_state_info_record
  {
  UINT1                           DatabaseState;         // - Database state
  ULONG                           ulTerminationTimeOut;
  } CONS_STATE_INFO_REC;

typedef CONS_STATE_INFO_REC       *PCONS_STATE_INFO_REC;

typedef struct cons_set_ukt_affinity_record
  {
  USHORT                          usReturnCode;
  ULONG                           ulUKTIndex;
  } CONS_SET_UKT_AFFINITY_REC;

typedef CONS_SET_UKT_AFFINITY_REC *PCONS_SET_UKT_AFFINITY_REC;

// PTS 1107070
typedef struct cons_set_processor_mask_record
  {
  USHORT                          usReturnCode;
  ULONG                           ProcessAffinityMask;
  UINT1                           TaskType;             // - task type (TT_US,...)
  } CONS_SET_PROCESSOR_MASK_REC;

typedef CONS_SET_PROCESSOR_MASK_REC *PCONS_SET_PROCESSOR_MASK_REC;

//
// --- RTE info
//
typedef struct cons_ukt_info_record
  {
  ULONG                           ulUKTIndex;
  TID                             Tid;
  INT4                            ThrdState;
  double                          dDispatcherCalls;
  ULONG                           ulActiveTasks;
  ULONG                           ulTotalTasks;
  CHAR                            szCluster[80];
  ULONG                           ulProcessor;
  LONG                            lNumOfDevspaces;
  ULONG                           ulDevspaceInfoOffset;    // - CONS_DEV_INFO_REC  * 'MAX_DEVSPACES'
  } CONS_UKT_INFO_REC;

typedef CONS_UKT_INFO_REC         *PCONS_UKT_INFO_REC;

typedef struct cons_dev_worker_info_record
  {
  TID                             Tid;
  INT4                            ThrdState;
  ULONG                           ulIOCount;
  } CONS_DEV_WORKER_INFO_REC;

typedef CONS_DEV_WORKER_INFO_REC *PCONS_DEV_WORKER_INFO_REC;


typedef struct cons_rte_info_record
  {
  LONG                            lNumOfDevWorkers;
  ULONG                           ulDevWorkerInfoOffset;

  LONG                            lNumOfUKTs;
  ULONG                           ulUKTInfoOffset;         // - CONS_UKT_INFO_REC * 'MAXUKT'

  LONG                            lNumOfDEVis;
  ULONG                           ulDEViInfoOffset;        // - CONS_DEVI_INFO_REC * 'MAX_DEVSPACES' * 'MAX_DEVI_P_DEVSPACE'

  ULONG                           ulNumOfConcurrentDevWorkers;
  LONG                            lTraceDevspaceNo;
  TID                             CoordTid;
  UINT1                           CoordState;
  TID                             ConsoleTid;
  UINT1                           ConsoleState;
  TID                             TimerTid;
  UINT1                           TimerState;
  TID                             ClockTid;
  UINT1                           ClockState;
  TID                             Dev0Tid;
  UINT1                           Dev0State;
  TID                             Async0Tid;
  UINT1                           Async0State;
  TID                             RequestorTid;
  UINT1                           RequestorState;
  } CONS_RTE_INFO_REC;

typedef CONS_RTE_INFO_REC         *PCONS_RTE_INFO_REC;


typedef struct cons_cancel_task_record
  {
  USHORT                          usReturnCode;
  ULONG                           ulTask;
  } CONS_CANCEL_TASK_REC;

typedef CONS_CANCEL_TASK_REC      *PCONS_CANCEL_TASK_REC;


typedef struct cons_kill_task_record
  {
  USHORT                          usReturnCode;
  ULONG                           ulTask;
  } CONS_KILL_TASK_REC;

typedef CONS_KILL_TASK_REC        *PCONS_KILL_TASK_REC;


typedef struct cons_crash_dump_record
  {
  USHORT                          usReturnCode;
  } CONS_CRASH_DUMP_REC;

typedef CONS_CRASH_DUMP_REC       *PCONS_CRASH_DUMP_REC;

//
// --- Task Statistic Counts
//
typedef struct cons_single_task_counts_record
  {
  ULONG                           ulTaskIndex;
  UINT1                           TaskType;           // - task type (TT_US,...)
  PID                             ApplPid;            // - process id of connected user process
  SQL_NODEIDC                     szClientNode;
  ULONG                           ulSelfSuspCount;
  ULONG                           ulBegExclCalls;
  teo52_TaskPrioStatistic         PrioStat;
  teo52_TaskStateStatisticRec     StateStat;
  teo52_TaskOMSStatisticRec       TaskOMSStat;        // 1106969

  ULONG                           ulCommands;
  ULONG                           ulDispatcherCalls;
  LONG                            lResumeCount[MAX_SUSPEND_LEVEL];
  tsp00_Int4 totalResumerCount[MAX_SUSPEND_LEVEL];
  tsp00_TaskId resumerPid[MAX_RESUMER_PIDS][MAX_SUSPEND_LEVEL];
  tsp2_devnamec                   DevNameWaitingFor; // PTS 1106969
  } CONS_SINGLE_TASK_COUNTS_INFO_REC;

typedef CONS_SINGLE_TASK_COUNTS_INFO_REC  *PCONS_SINGLE_TASK_COUNTS_INFO_REC;

typedef struct cons_task_counts_info_record
  {
  BOOL                             fTimeMeasureActive ;
  USHORT                           usReturnCode;           // -  PTS 1104727
  LONG                             lNumOfActiveTasks;
  ULONG                            ulTaskCountsInfoOffset; // -  CONS_SINGLE_TASK_COUNTS_INFO_REC * 'MAX_TASKS'
  } CONS_TASK_COUNTS_INFO_REC;

typedef CONS_TASK_COUNTS_INFO_REC *PCONS_TASK_COUNTS_INFO_REC;


//
// --- Queue Statistic info
//
typedef struct cons_single_queue_info_record
  {
  ULONG                           ulTaskIndex;
  teo52_TaskQueueStatistic        QueStat;
  } CONS_SINGLE_QUEUE_STAT_INFO_REC;

typedef CONS_SINGLE_QUEUE_STAT_INFO_REC *PCONS_SINGLE_QUEUE_STAT_INFO_REC;


typedef struct cons_queue_stat_info_record
  {
  LONG                            lNumOfActiveTasks;
  ULONG                           ulQueStatInfoOffset;     // -  CONS_SINGLE_QUEUE_STAT_INFO_REC * 'MAX_TASKS'
  } CONS_QUEUE_STAT_INFO_REC;

typedef CONS_QUEUE_STAT_INFO_REC  *PCONS_QUEUE_STAT_INFO_REC;

typedef struct cons_debug_break_record
  {
  USHORT                          usReturnCode;
  BOOL                            fClearPosition;
  ULONG                           ulBreakPosition;
  } CONS_DEBUG_BREAK_REC;

typedef CONS_DEBUG_BREAK_REC *PCONS_DEBUG_BREAK_REC;

typedef struct cons_debug_task_record
  {
  USHORT                          usReturnCode;
  ULONG                           ulTask;
  } CONS_DEBUG_TASK_REC;

typedef CONS_DEBUG_TASK_REC *PCONS_DEBUG_TASK_REC;

//
// --- Total Counter Statistic Info
//
typedef struct cons_total_counts_info_record
  {
  ULONG                           ulTotalDispCalls;
  ULONG                           ulTotalCommands;
  ULONG                           ulTotalVwait;
  ULONG                           ulTotalVsleep;
  ULONG                           ulTotalVsusp;
  ULONG                           ulTotalSelfIO;
  ULONG                           ulTotalDevIO;
  ULONG                           ulTotalRegAccess;
  ULONG                           ulTotalRegColl;
  ULONG                           ulTotalRegWait;
  ULONG                           ulTotalBegExclTasCount;
  ULONG                           ulTotalEndExclTasCount;
  teo52_TaskPrioStatistic         TotalPrioStat ;
  } CONS_TOTAL_COUNTS_INFO_REC;

typedef CONS_TOTAL_COUNTS_INFO_REC  *PCONS_TOTAL_COUNTS_INFO_REC;


typedef struct cons_reset_record
  {
  BOOL                            dummy;
  } CONS_RESET_REC;

typedef CONS_RESET_REC            *PCONS_RESET_REC;


typedef struct cons_time_measure_record
  {
  BOOL                            dummy;
  } CONS_TIME_MEASURE_REC;

typedef CONS_TIME_MEASURE_REC     *PCONS_TIME_MEASURE_REC;

typedef struct cons_set_debug_level_record
  {
  ULONG                           ulDebugLevel;
  } CONS_SET_DEBUGLEVEL_REC;

typedef CONS_SET_DEBUGLEVEL_REC     *PCONS_SET_DEBUGLEVEL_REC;

typedef struct cons_single_ukt_sl_stat_info_rec
  {
  ULONG                           ulUKTIndex;
  TID                             Tid;
  ULONG                           ulRUNQueueLen;
  SAPDB_UInt8                     measuredMicroSeconds;
  teo52_AbsoluteTime              UKTIdleStat;        
  teo52_AbsoluteTime              UKTIOWaitStat;
  } CONS_SINGLE_UKT_SL_STAT_INFO_REC;

typedef CONS_SINGLE_UKT_SL_STAT_INFO_REC *PCONS_SINGLE_UKT_SL_STAT_INFO_REC;


typedef struct cons_ukt_sleep_stat_info_record
  {
  LONG                             lNumOfActiveUKTs;
  ULONG                            ulSleepStatInfoOffset;  // - CONS_SINGLE_UKT_SL_STAT_INFO_REC * 'MAXUKT'
  } CONS_UKT_SLEEPTIME_REC;

typedef CONS_UKT_SLEEPTIME_REC    *PCONS_UKT_SLEEPTIME_REC;


typedef struct cons_show_suspend_rec
  {
  ULONG                           ulSuspends [ MAX_SUSPEND_REASONS ] ;
  } CONS_SHOW_SUSPEND_REC;

typedef CONS_SHOW_SUSPEND_REC     *PCONS_SHOW_SUSPEND_REC;


typedef struct cons_getparam_rec
  {
  RTE_XPARAM_REC                  XParam;
  } CONS_GETPARAM_REC;

typedef CONS_GETPARAM_REC         *PCONS_GETPARAM_REC;


typedef struct cons_putparam_rec
  {
  tsp00_C18c                      XParamId;
  ULONG                           OldVal;
  ULONG                           NewVal;
  } CONS_PUTPARAM_REC;

typedef CONS_PUTPARAM_REC         *PCONS_PUTPARAM_REC;

typedef struct cons_single_queue_info_rec
  {
  ULONG                           ulTaskIndex;
  ULONG                           ulDynPrio;
  ULONG                           ulReqType;
  ULONG                           ulFromTask;
  UINT1                           TaskState;
  UINT1                           PrioFlag;
  } CONS_SINGLE_QUEUE_INFO_REC;

typedef CONS_SINGLE_QUEUE_INFO_REC *PCONS_SINGLE_QUEUE_INFO_REC;

typedef struct cons_single_ukt_queue_info_rec
  {
  ULONG                           ulUKTIndex;
  TID                             Tid;
  ULONG                           ulRUNQueueLen;
  ULONG                           ulRUNQueueOffset;
  ULONG                           ulCOMQueueLen;
  ULONG                           ulCOMQueueOffset;
  ULONG                           ulUKTQueueLen;
  ULONG                           ulUKTQueueOffset;
  ULONG                           ulUToUQueueLen;
  ULONG                           ulUToUQueueOffset;
  } CONS_SINGLE_UKT_QUEUES_INFO_REC;

typedef CONS_SINGLE_UKT_QUEUES_INFO_REC *PCONS_SINGLE_UKT_QUEUES_INFO_REC;


typedef struct cons_ukt_queue_info_record
  {
  LONG                             lNumOfActiveUKTs;
  ULONG                            ulUKTQueuesOffset;  // - CONS_SINGLE_UKT_QUEUE_INFO_REC * 'MAXUKT'
  } CONS_UKT_QUEUES_REC;

typedef CONS_UKT_QUEUES_REC    *PCONS_UKT_QUEUES_REC;

typedef struct cons_force_io_error_record
  {
  ULONG                           ulDevno;
  BOOL                            fErrOnRead ;
  } CONS_FORCE_IO_ERROR_REC;

typedef struct cons_single_thread_times_record
  {
  TID                             Tid;
  CHAR                            szThreadName[20];
  CHAR                            szCluster[80];
  FILETIME                        SystemTime; // time the thread has spent in kernel mode 
  FILETIME                        UserTime;   // time the thread has spent in user mode 
  } CONS_SINGLE_THREAD_TIMES_INFO_REC ;

typedef CONS_SINGLE_THREAD_TIMES_INFO_REC *PCONS_SINGLE_THREAD_TIMES_INFO_REC;

typedef struct cons_thread_times_record
  {
  LONG                             lNumOfUKTs;
  ULONG                            ulThreadTimesOffset; // -  CONS_SINGLE_THREAD_TIMES_INFO_REC * 'lNumOfUKTs'
  } CONS_THREAD_TIMES_INFO_REC;

typedef CONS_THREAD_TIMES_INFO_REC *PCONS_THREAD_TIMES_INFO_REC;


//
// --- AWE info
//
typedef struct cons_awe_info_record
  {
  bool                            AWEAvailable;

  SAPDB_UInt4                     NumOfPhysPages;
  SAPDB_UInt4                     NumOfFreePhysPages;
  SAPDB_UInt4                     NumOfMapAreaPages;
  SAPDB_UInt4                     NumOfFreeMapAreaPages;
  SAPDB_UInt4                     NumOfMaps;
  SAPDB_UInt4                     NumOfUnMaps;

  SAPDB_UInt4                     NumOfAWEMapPageInfos;
  ULONG                           ulAWEMapPageInfoOffset;   // -  CONS_AWE_PHYS_PAGE_INFO_REC * 'NumOfAWEPhysPageInfos'
  SAPDB_UInt4                     NumOfAWEPhysPageInfos;
  ULONG                           ulAWEPhysPageInfoOffset;  // -  ulAWEMapPageInfoOffset + CONS_AWE_PHYS_PAGE_INFO_REC * 'NumOfAWEPhysPageInfos'
  } CONS_AWE_INFO_REC;

typedef CONS_AWE_INFO_REC          *PCONS_AWE_INFO_REC;

//
// --- AWE map page info
//
typedef struct cons_awe_map_pageinfo_record
  {
  void *                          pMappingPageAddr;
  SAPDB_UInt4                     PhysPageNo;
  SAPDB_UInt8                     Reserved:1,
                                  MapCount;
  } CONS_AWE_MAP_PAGE_INFO_REC;

typedef CONS_AWE_MAP_PAGE_INFO_REC          *PCONS_AWE_MAP_PAGE_INFO_REC;

//
// --- AWE phys page info
//
typedef struct cons_awe_phys_pageinfo_record
  {
  SAPDB_UInt8                     Reserved:1,
                                  MapCount;
  } CONS_AWE_PHYS_PAGE_INFO_REC;

typedef CONS_AWE_PHYS_PAGE_INFO_REC          *PCONS_AWE_PHYS_PAGE_INFO_REC;

//
// --- Connection info
//
typedef struct cons_single_connection_info_record
  {
  SAPDB_UInt4                     taskIndex;         /* Task index */
  SAPDB_UInt4                     uktIndex;          /* User kernel thread index */
  SAPDB_UInt4                     taskType;          /* Task type (TT_US,...) */
  SAPDB_UInt4                     connState;         /* Connection state */
  time_t                          connectTime;       /* Connect time in seconds */
  PID                             peerPid;           /* Local connected pid */
  PID                             remotePid;         /* Remote client pid */
  SQL_NODEIDC                     clientNode;        /* Client node */
  } CONS_SINGLE_CONNECTION_INFO_REC;

typedef CONS_SINGLE_CONNECTION_INFO_REC  *PCONS_SINGLE_CONNECTION_INFO_REC;


typedef struct cons_connection_info_record
  {
  LONG                            lNumOfConnections;
  ULONG                           ulConnectionInfoOffset;   // - CONS_SINGLE_CONNECTION_INFO_REC * 'lNumOfConnections'
  } CONS_CONNECTION_INFO_REC;

typedef CONS_CONNECTION_INFO_REC         *PCONS_CONNECTION_INFO_REC;



typedef struct cons_last_moves_entry
{
  SAPDB_UInt4                 MoveTime;    
  ULONG                       ulTaskIndex;
  SAPDB_UInt8                 TaskMaxRunnableMicroSeconds;
  ULONG                       ulSrcUKTIndex;
  SAPDB_UInt8                 SrcMaxRunnableMicroSeconds;
  ULONG                       ulDstUKTIndex;
  SAPDB_UInt8                 DstMaxRunnableMicroSeconds;
} CONS_LAST_MOVES_ENTRY;

typedef CONS_LAST_MOVES_ENTRY         *PCONS_LAST_MOVES_ENTRY;


typedef struct cons_single_ukt_move_info_record
  {
  ULONG                           ulUKTIndex;
  ULONG                           ulMoveTaskToUKTUKTIndex;
  SAPDB_UInt4                     NumOfMovableTasks;
  ULONG                           TaskMoveIntervalStartTime;
  SAPDB_UInt8                     UKTSleepTimeMicroSeconds;
  SAPDB_UInt8                     RunnableMicroSeconds;
  SAPDB_UInt8                     MaxRunnableMicroSeconds;

  } CONS_SINGLE_UKT_MOVE_INFO_REC;

typedef CONS_SINGLE_UKT_MOVE_INFO_REC         *PCONS_SINGLE_UKT_MOVE_INFO_REC;

typedef struct cons_task_move_info_record
  {
  SAPDB_Int4                      CheckIntervalTime;
  SAPDB_UInt4                     TaskMoveIntervalStartTime;
  SAPDB_UInt2                     TaskMoveMeasIntervals;
  SAPDB_UInt2                     MinimumDiff;
  SAPDB_UInt2                     ValueEquality;
  
  LONG                            lNumOfLastRecs;
  ULONG                           ulLastMovesInfoOffset;   // - CONS_LAST_MOVES_ENTRY * 'lNumOfLastRecs'

  LONG                            lNumOfUKTs;
  ULONG                           ulUKTInfoOffset;         // - CONS_SINGLE_UKT_MOVE_INFO_REC * 'lNumOfUKTs'

  } CONS_TASK_MOVE_INFO_REC;

typedef CONS_TASK_MOVE_INFO_REC         *PCONS_TASK_MOVE_INFO_REC;

typedef struct cons_check_interval_record
  {
  USHORT                          usReturnCode;
  SAPDB_Int4                      CheckIntervalTime;
  } CONS_CHECK_INTERVAL_RECORD;

typedef union console_data_record
  {
  CONS_TASK_INFO_REC              TaskInfo;
  CONS_IO_INFO_REC                IOInfo;
  CONS_AIO_INFO_REC               AIOInfo;
  CONS_SEM_INFO_REC               SemInfo;
  CONS_STORAGE_INFO_REC           StorageInfo;
  CONS_STATE_INFO_REC             StateInfo;
  CONS_RTE_INFO_REC               RTEInfo;
  CONS_CANCEL_TASK_REC            CancelTaskCtrl;
  CONS_KILL_TASK_REC              KillTaskCtrl;
  CONS_QUEUE_STAT_INFO_REC        QueInfo;
  CONS_TASK_COUNTS_INFO_REC       TaskCountsInfo;
  CONS_TOTAL_COUNTS_INFO_REC      TotalCountsInfo;
  CONS_TIME_MEASURE_REC           TimeMeasureCtrl;
  CONS_SET_DEBUGLEVEL_REC         SetDebugLevelCtrl;
  CONS_RESET_REC                  ResetCtrl;
  CONS_UKT_SLEEPTIME_REC          UKTSleepTimeInfo;
  CONS_SHOW_SUSPEND_REC           SuspendReasons;
  CONS_THREAD_TIMES_INFO_REC      ThreadTimeInfo;
  CONS_GETPARAM_REC               GetParamInfo;
  CONS_PUTPARAM_REC               PutParamInfo;
  CONS_CRASH_DUMP_REC             CrashDump;
  CONS_UKT_QUEUES_REC             UKTQueues;
  CONS_SET_UKT_AFFINITY_REC       UKTAffinity;
  CONS_FORCE_IO_ERROR_REC         ForceIOError;
  CONS_DEBUG_BREAK_REC            DebugBreak;
  CONS_SET_PROCESSOR_MASK_REC     SetProcessorMask;
  CONS_DEBUG_TASK_REC             DebugTask;
  CONS_AWE_INFO_REC               AWEInfo;
  CONS_CONNECTION_INFO_REC        ConnectionInfo;
  CONS_TASK_MOVE_INFO_REC         TaskMoveInfo;
  CONS_CHECK_INTERVAL_RECORD      SetCheckInterval;
  } CONSOLE_DATA_REC;

typedef CONSOLE_DATA_REC          *PCONSOLE_DATA_REC;

typedef struct console_record
  {
  BOOL                            fDataAvailable;
  CONSOLE_DATA_REC                Data;
  } CONSOLE_REC;

typedef CONSOLE_REC               *PCONSOLE_REC;


#endif  /* GOS00CN_H */
