/*!**************************************************************************

  module      : vos52k.c

  -------------------------------------------------------------------------

  responsible : RaymondR

  special area: DATE/TIME QUERY
  description : 


  last changed: 2001-09-21  13:41
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/



/*
 * INCLUDE FILES
 */
#include "gos00.h"
#include "heo00.h"
#include "heo46.h"
#include "geo007_1.h"
#include "gos00k.h"
#include "heo52.h"


/*
 *  DEFINES
 */
#define MOD__  "VOS52KC : "
#define MF__   MOD__"UNDEFINED"

/*
 *  MACROS
 */

/*
 *  LOCAL TYPE AND STRUCT DEFINITIONS
 */


/*
 *  EXPORTED VARIABLES
 */



/*
 * LOCAL VARIABLES
 */


/*
 * LOCAL FUNCTION PROTOTYPES
 */
static SAPDB_UInt4      globStartMilliSec;
static SAPDB_UInt8      globMilliSec;

typedef tsp_c8					tsp_date;
typedef tsp_c8					tsp_time;
 
/*
 * ========================== GLOBAL FUNCTIONS ================================
 */

VOID  vdattime (tsp_date pDate, tsp_time pTime )
  {
  #undef  MF__
  #define MF__ MOD__"vdattime"
  CHAR                                   *cp;
  #if defined(_WIN32)
    SYSTEMTIME                           SystemTime;
  #else
    DATETIME                             DateTime;
  #endif

  DBGPAS;

  /*
   * Return date and time as YYYYMMDD and HHHHMMSS
   */
  memset (pDate, '0', sizeof(tsp_date));
  memset (pTime, '0', sizeof(tsp_time));

  #if defined(_WIN32)
    /*
     * obtain current date and time
     */
    GetLocalTime(&SystemTime);

    cp = pDate;
    *cp++ += SystemTime.wYear   / 1000; SystemTime.wYear %= 1000;
    *cp++ += SystemTime.wYear   / 100;  SystemTime.wYear %= 100;
    *cp++ += SystemTime.wYear   / 10;
    *cp++ += SystemTime.wYear   % 10;
    *cp++ += SystemTime.wMonth  / 10;
    *cp++ += SystemTime.wMonth  % 10;
    *cp++ += SystemTime.wDay    / 10;
    *cp++ += SystemTime.wDay    % 10;
    cp = pTime + 2;
    *cp++ += SystemTime.wHour   / 10;
    *cp++ += SystemTime.wHour   % 10;
    *cp++ += SystemTime.wMinute / 10;
    *cp++ += SystemTime.wMinute % 10;
    *cp++ += SystemTime.wSecond / 10;
    *cp++ += SystemTime.wSecond % 10;
  #else
    /*
     * obtain current date and time
     */
    DosGetDateTime (&DateTime);

    cp             = pDate;
    *cp++         += (CHAR)(DateTime.year / 1000);
    DateTime.year  = DateTime.year        % 1000;
    *cp++         += (CHAR)(DateTime.year / 100);
    DateTime.year  = DateTime.year        % 100;
    *cp++         += (CHAR)(DateTime.year / 10);
    *cp++         += (CHAR)(DateTime.year % 10);
    *cp++         += DateTime.month       / 10;
    *cp++         += DateTime.month       % 10;
    *cp++         += DateTime.day         / 10;
    *cp++         += DateTime.day         % 10;

    cp             = pTime + 2;
    *cp++         += DateTime.hours       / 10;
    *cp++         += DateTime.hours       % 10;
    *cp++         += DateTime.minutes     / 10;
    *cp++         += DateTime.minutes     % 10;
    *cp++         += DateTime.seconds     / 10;
    *cp++         += DateTime.seconds     % 10;
  #endif
  }

/*------------------------------*/

/* vtimestamp now in veo52.c */

/*------------------------------*/


VOID  vclock (INT4  *Sec, INT4  *MicroSec )
  {
  #undef  MF__
  #define MF__ MOD__"vclock"

  static LONG         lExclusive;
  SAPDB_UInt4         CurrTickCount;
  SAPDB_UInt8         MilliSeconds;

  DBGPAS;

  WAIT_UNTIL_ALONE(&lExclusive);
  MilliSeconds = globMilliSec;   // --- get global millisecond value. Use 'interlock' because
                                 //     64-bit operation are not atomic on 32-bit platforms.
  CLEARLOCK( &lExclusive );

  CurrTickCount = GetTickCount();

  if ( (SAPDB_UInt4)MilliSeconds > CurrTickCount ) // overrun ?
    MilliSeconds += 0x100000000I64;

  MilliSeconds = ( MilliSeconds & ~0xFFFFFFFFI64 ) | CurrTickCount;


  WAIT_UNTIL_ALONE(&lExclusive);

  if ( MilliSeconds > globMilliSec ) // has another thread changed this value?
    globMilliSec = MilliSeconds;     // no - update global millisecond value
  else
    MilliSeconds = globMilliSec;

  CLEARLOCK( &lExclusive );

  *Sec      = (INT4) ((MilliSeconds - globStartMilliSec) / 1000);
  *MicroSec = (INT4)(((MilliSeconds - globStartMilliSec) % 1000) * 1000);
  }

/*------------------------------*/

VOID sql52k_vclock_init (VOID)
  {
  #undef  MF__
  #define MF__ MOD__"sql52k_vclock_init"

  DBGPAS;

  globStartMilliSec  = GetTickCount();
  }

/*------------------------------*/

tsp00_Longreal vTimeZoneDelta()
{
    SYSTEMTIME gmt;
    SYSTEMTIME our;

   /* Weekdays may differ due to timezone between localtime and GMT */
   /* Since weekday information [0-6] is a range, rather than an absolute */
   /* value, the difference 0 - 6 and 6 - 0 are special cases. To handle these */
   /* cases, the following array is used. Its index is the difference biased by 6 */
   /* to guarantee a non negativ number */
#define SAPDB_WEEKDAY_DIFF_BIAS 6
#define SAPDB_ILLEGAL_WEEKDAY_DIFF -2
    static int weekdayDifference[13] = {
     1, /* -6 = 0 - 6 ->  1 localtime sunday, GMT saturday */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /* -5 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /* -4 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /* -3 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /* -2 = illegal */
    -1, /* -1               localtime one day behind GMT */
     0, /*  0                same day */
     1, /*  1               localtime one day ahead of GMT */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /*  2 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /*  3 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /*  4 = illegal */
     SAPDB_ILLEGAL_WEEKDAY_DIFF, /*  5 = illegal */
    -1  /*  6 = 6 - 0 -> -1 localtime saturday, GMT sunday */
    };

    GetSystemTime(&gmt);
    GetLocalTime(&our);
    /*
      There could be a jitter in the minute if we have two different calls getting time just 
      when 59 seconds of a minute passed. Therefore the number of seconds reported are checked
      to be greater or equal. Otherwise another minute has begone already in our local time 
      and the gmt info must be adjusted by a minute.
      Adjustment only needs to adjust wMinute,wHour and wDayOfWeek...
     */
    if ( gmt.wSecond > our.wSecond )
    {
        gmt.wMinute++;
        if ( 60 == gmt.wMinute )
        {
           gmt.wHour++;
           if ( 24 == gmt.wHour )
           {
              gmt.wDayOfWeek++;
              if ( 7 == gmt.wDayOfWeek )
              {
                gmt.wDayOfWeek = 0;
              }
           }
        }
    }

    if ( weekdayDifference[SAPDB_WEEKDAY_DIFF_BIAS + our.wDayOfWeek - gmt.wDayOfWeek] == SAPDB_ILLEGAL_WEEKDAY_DIFF )
    {
        MSGD(( WRN_TIMEZONE_DELTA, (int)our.wDayOfWeek, (int)gmt.wDayOfWeek ));
        return 0.;
    }

   /* 
      Return difference (localtime - GMT) in hours:
      Weekday difference see above.
      diffMinute is added to gmt.wMinute and module 60 to get the correct value for comparison.
    */
   return (  ( our.wMinute - gmt.wMinute ) / 60.
           + ( our.wHour - gmt.wHour )
           + ( weekdayDifference[SAPDB_WEEKDAY_DIFF_BIAS + our.wDayOfWeek - gmt.wDayOfWeek] * 24. ) );
}

/*
 * ========================== LOCAL FUNCTIONS =================================
 */


/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
