/*!**************************************************************************

  module      : vos55k.c

  -------------------------------------------------------------------------

  responsible : RaymondR

  special area: Mutual Exclusion
  description : 


  last changed: 2001-09-21  13:44
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin  GPL
  Copyright (C) 2001 SAP AG

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ========== licence end

*****************************************************************************/



/*
 * INCLUDE FILES
 */
#include "gos00.h"
#include "heo00.h"
#include "heo46.h"
#include "geo007_1.h"
#include "gos00k.h"
#include "geo002.h"
#include "heo55k.h"
#include "gos74.h"


/*
 *  DEFINES
 */
#define MOD__  "VOS55KC : "
#define MF__   MOD__"UNDEFINED"

#define MAX_TAS_CNT   -1
// #define SLEEP_CNT     10
#define SLEEP_CNT     MAX_TAS_CNT
#define BUSY_WAIT    1
#define NO_BUSY_WAIT 0


#define RESCHEDULE { SLEEP ( 0 ); }

/*
 *  MACROS
 */
#if defined (USE_HUGE_MACROS)
 #define WAIT_LOCK( _ppUKT, _pTaskCtrl, _pSemCtrl, _plExclusive, _sid,       \
                                       _plExclTasCnt, _fBusyWait )          \
   {                                                                        \
   LONG                           lTasCount;                                \
                                                                            \
   if ( INTERLOCK( _plExclusive ) )                                         \
     {                                                                      \
     lTasCount = MAX_TAS_CNT;                                               \
                                                                            \
     DBG4 (( MF__, "[T:0x%03u] Waiting for region interlock %u",            \
             _pTaskCtrl->ulTaskIndex, _sid ));                              \
                                                                            \
     while ( INTERLOCK( _plExclusive ) )                                    \
       {                                                                    \
       lTasCount--;                                                         \
                                                                            \
       if ( !lTasCount )                                                    \
         {                                                                  \
         MSGD (( ERR_INDEF_LOOP, _sid ));                                   \
         ABORT();                                                           \
         }                                                                  \
                                                                            \
       if ( kgs.fBusyWaitingForbidden || !( lTasCount % SLEEP_CNT )) \
         {                                                                  \
         RESCHEDULE ;                                                       \
         }                                                                  \
       else                                                                 \
         if ( !_fBusyWait )                                                 \
         {                                                                  \
         /* --- reschedule myself via rex queue  */                         \
         _pTaskCtrl->SpecDLQElem2.ulReqType = REQ_RESCHEDULE;               \
         sql74k_FastRUN_enqu(*_ppUKT, &_pTaskCtrl->SpecDLQElem2, BASE_PRIO_REX ); \
                                                                            \
         GOTO_DISP(_ppUKT);                                                 \
         }                                                                  \
       }                                                                    \
       *_plExclTasCnt += MAX_TAS_CNT - lTasCount + 1;                       \
     }                                                                      \
   }
#else
 #define WAIT_LOCK( _ppUKT, _pTaskCtrl, _pSemCtrl, _plExclusive, _sid,      \
                                       _plExclTasCnt, _fBusyWait )          \
   if ( INTERLOCK( _plExclusive ) )                                         \
     {                                                                      \
     __sql55k_wait_lock( _ppUKT, _pTaskCtrl, _pSemCtrl, _plExclusive,       \
                         _sid, _plExclTasCnt, _fBusyWait );                 \
     }
#endif

#define DO_WHEN_DUMMY_REGION(_RegId,_Action) \
        { if ( _RegId == -1 ) _Action ; }

/*
 *  LOCAL TYPE AND STRUCT DEFINITIONS
 */


/*
 * EXTERNAL VARIABLES
 */


/*
 *  EXPORTED VARIABLES
 */


/*
 * LOCAL VARIABLES
 */


/*
 * LOCAL FUNCTION PROTOTYPES
 */
VOID         sql55k_init_semaliste  ( PSEM_CTRL_REC   pSemCtrl );
VOID         sql55k_remote_vendexcl ( PPTASK_CTRL_REC ppCTask,
                                      PTASK_CTRL_REC  pFromTask,
                                      PPDLQ_REC       ppCurrReq,
                                      ULONG           ulDummyParam );
// --- inline functions
_INLINE VOID __sql55k_wait_lock     ( PUKT_CTRL_REC   *ppUKT,
                                      PTASK_CTRL_REC  pTaskCtrl,
                                      PSEM_CTRL_REC   pSemCtrl,
                                      PLONG           plExclusive,
                                      tsp00_RegionId  sid ,
                                      ULONG           *_plExclTasCnt ,
                                      BOOLEAN         fBusyWait );

/*
 * ========================== GLOBAL FUNCTIONS ================================
 */

VOID  vsemaphid ( tsp00_Region    semaname,
                  tsp00_RegionId  *sid )
  {
  #undef  MF__
  #define MF__ MOD__"vsemaphid"
  ULONG                          ulSem;
  PSEM_CTRL_REC                  pSemCtrl;
  #ifdef DEBUG_RTE
   PTASK_CTRL_REC                pTaskCtrl = THIS_UKT_CTRL->pCTask;
  #endif

  DBGIN_T (pTaskCtrl->ulTaskIndex);

  DBG3 (( MF__, "[T:0x%03u] Region '%.8s'",
          pTaskCtrl->ulTaskIndex, semaname ));

  // --- Initialisation already done
  for ( ulSem = 0; ulSem < XPARAM(ulNoOfRegions); ulSem ++ )
    {
    if ( kgs.pSemaphores[ ulSem ].szSemaName[ 0 ] == '\0' )
      break;
    }

  if ( ulSem >= XPARAM(ulNoOfRegions) )
    {
    MSGD (( ERR_TOO_MANY_REGIONS_REQ ));
    DBG1 (( MF__, "[T:0x%03u] Wrong SID:%u", pTaskCtrl->ulTaskIndex, sid ));
    ABORT();
    }

  *sid                     = ulSem + 1;
  pSemCtrl                 = &kgs.pSemaphores[ ulSem ];
  pSemCtrl->pOwnerTaskCtrl = NULL;

  eo46PtoC ( pSemCtrl->szSemaName, semaname, sizeof(pSemCtrl->szSemaName) -1 );

  sql55k_init_semaliste ( pSemCtrl );

  DBGOUT_T (pTaskCtrl->ulTaskIndex);
  return;
  }

/*
 *=======================================================================
 */

VOID  vsemainfo ( tsp00_RegionId sid ,
                  tsp00_Region   semaname)

{
  if ( (ULONG ) sid <= XPARAM(ulNoOfRegions) )
    { eo46CtoP ( semaname,
                 kgs.pSemaphores[sid-1].szSemaName, sizeof(tsp00_Region) );
    }
}

/*------------------------------*/


BOOLEAN visexcl ( tsp00_TaskId    pid, 
                  tsp00_RegionId  sid )
  {
  #undef  MF__
  #define MF__ MOD__"visexcl"
  #ifdef DEVELOP_CHECKS
   PUKT_CTRL_REC                 pUKT      = THIS_UKT_CTRL;
   PTASK_CTRL_REC                pTaskCtrl = pUKT->pCTask;
  #else
   PTASK_CTRL_REC                pTaskCtrl = &kgs.pFirstTaskCtrl[pid - 1];
   PUKT_CTRL_REC                 pUKT      = pTaskCtrl->pUKT;
  #endif
  PSEM_CTRL_REC                  pSemCtrl;

  DBGIN_T (pTaskCtrl->ulTaskIndex);

  DO_WHEN_DUMMY_REGION( sid , return(TRUE) );

  if ( sid == -1 )
    return ( pTaskCtrl->lNestingExcl != 0 ) ? TRUE : FALSE;

  if ( sid < 1 || sid > (tsp00_RegionId)XPARAM(ulNoOfRegions) )
     {
     MSGD (( ERR_WRONG_SEMAPHORE_IDX ));
     ABORT();
     }

  pSemCtrl = &kgs.pSemaphores[ sid - 1 ];

  if ( pSemCtrl->pOwnerTaskCtrl == pTaskCtrl )
    {
    DBGOUT_T (pTaskCtrl->ulTaskIndex);
    return (TRUE);
    }

  DBGOUT_T (pTaskCtrl->ulTaskIndex);
  return (FALSE);
  }

/*------------------------------*/

VOID vbegexcl ( tsp00_TaskId    pid, 
                tsp00_RegionId  sid  )
  {
  #undef  MF__
  #define MF__ MOD__"vbegexcl"
  #ifdef DEVELOP_CHECKS
   PUKT_CTRL_REC                 pUKT      = THIS_UKT_CTRL;
   PTASK_CTRL_REC                pTaskCtrl = pUKT->pCTask;
  #else
   PTASK_CTRL_REC                pTaskCtrl = &kgs.pFirstTaskCtrl[pid - 1];
   PUKT_CTRL_REC                 pUKT      = pTaskCtrl->pUKT;
  #endif
  PSEM_CTRL_REC                  pSemCtrl;
  PSEM_QUEUE_HEAD_REC            pSemQueHead;
  ULONG                          lReservedLoopCnt;
  BOOLEAN                        fRegionFree,fLoopEnd;

  DBGIN_T (pTaskCtrl->ulTaskIndex);

  #ifdef DEVELOP_CHECKS
   if ( (ULONG)pid != pTaskCtrl->ulTaskIndex )
     {
     MSGD (( ERR_VXXXX_WRONG_TASK, "vbegexcl", pid ));
     ABORT();
     }

   if ( sid < 1 || sid > (tsp00_RegionId)XPARAM(ulNoOfRegions) )
     {
     MSGD (( ERR_WRONG_SEMAPHORE_IDX ));
     ABORT();
     }
  #endif

  DO_WHEN_DUMMY_REGION( sid , return );

  pTaskCtrl->TaskState       = TSK_VBEGEXCL;
  pTaskCtrl->Sid             = sid;
  pTaskCtrl->ulBegExclCalls++;
  pTaskCtrl->ulBegExclCnt++;
  pTaskCtrl->lNestingExcl++;

  pSemCtrl    = kgs.pSemaphores + sid - 1;
  pSemQueHead = &pSemCtrl->SemaList;
  pSemCtrl->lGetCount++;

  if ( pSemCtrl->pOwnerTaskCtrl == pTaskCtrl )
    {
    MSGD (( ERR_SEC_REGION_REQUEST, sid ));
    ABORT();
    }

  DBG3 (( MF__, "[T:0x%03u] Region '%s', sid: %d",
          pTaskCtrl->ulTaskIndex, pSemCtrl->szSemaName, sid ));

  if ( !XPARAM(fExclQueue) )
    {
      do
        { fRegionFree = pSemCtrl->pOwnerTaskCtrl == NULL; /* dirty read */
          if ( fRegionFree )
            { WAIT_LOCK ( &pUKT, pTaskCtrl, pSemCtrl, &pSemQueHead->lExclusive,
                          sid , & pSemCtrl->lBegExclTasCount, NO_BUSY_WAIT);
              if ( !(fRegionFree = pSemCtrl->pOwnerTaskCtrl == NULL) )
                CLEARLOCK( &pSemQueHead->lExclusive );
            }
          if ( !fRegionFree )
            { pSemCtrl->lCollisionCount++;
              pTaskCtrl->SpecDLQElem2.ulReqType = REQ_RESCHEDULE;
              sql74k_FastRUN_enqu ( pUKT, &pTaskCtrl->SpecDLQElem2, BASE_PRIO_REX  );
              GOTO_DISP(&pUKT);
            }
        }
      while ( !fRegionFree ) ;
    } 
  else
  if ( XPARAM(ulRegionCollisionLoop) &&
       ( !pTaskCtrl->PrioFlag || !XPARAM(fBegExclQueWhenPrio) )) 
    {
    lReservedLoopCnt = XPARAM(ulRegionCollisionLoop);
#ifndef ORG_STYLE
    do
      {
      fLoopEnd    = lReservedLoopCnt <= 0 ||
                    ( pTaskCtrl->PrioFlag && XPARAM(fBegExclQueWhenPrio) ) ;
      fRegionFree = !XPARAM(fBegExclDirtyRead) ||
                    pSemCtrl->pOwnerTaskCtrl == NULL;
      if ( fRegionFree || fLoopEnd )
        {
        WAIT_LOCK ( &pUKT, pTaskCtrl, pSemCtrl, &pSemQueHead->lExclusive,
                    sid , & pSemCtrl->lBegExclTasCount, NO_BUSY_WAIT);

        fRegionFree = pSemCtrl->pOwnerTaskCtrl == NULL;

        if ( fRegionFree )
          fLoopEnd = TRUE;
        else
          {
          if ( !fLoopEnd )
            CLEARLOCK( &pSemQueHead->lExclusive );
          }
        }
      if ( !fLoopEnd )
        {
        lReservedLoopCnt--;

        pTaskCtrl->SpecDLQElem2.ulReqType = REQ_RESCHEDULE;
        sql74k_FastRUN_enqu ( pUKT, &pTaskCtrl->SpecDLQElem2, BASE_PRIO_REX  );

        GOTO_DISP(&pUKT);
        }
      }
    while ( !fLoopEnd );
#else

    WAIT_LOCK ( &pUKT, pTaskCtrl, pSemCtrl, &pSemQueHead->lExclusive, sid ,
                &pSemCtrl->lBegExclTasCount, NO_BUSY_WAIT);

    // --- if we have more processors then UKTs, loop here
//  while ( lReservedLoopCnt         &&
//          pSemCtrl->pOwnerTaskCtrl &&
//          pSemCtrl->pOwnerTaskCtrl->pUKT != pUKT )
    while ( lReservedLoopCnt && pSemCtrl->pOwnerTaskCtrl &&
            ! pTaskCtrl->PrioFlag )
      {
      CLEARLOCK( &pSemQueHead->lExclusive );

      lReservedLoopCnt--;

      // --- reschedule myself via rex queue
      pTaskCtrl->SpecDLQElem2.ulReqType = REQ_RESCHEDULE;
      sql74k_FastRUN_enqu ( pUKT, &pTaskCtrl->SpecDLQElem2, BASE_PRIO_REX );

      GOTO_DISP(&pUKT);

      WAIT_LOCK ( &pUKT, pTaskCtrl, pSemCtrl, &pSemQueHead->lExclusive,
                  sid, & pSemCtrl->lBegExclTasCount, NO_BUSY_WAIT);
      }
#endif
    if (( (ULONG)lReservedLoopCnt != XPARAM(ulRegionCollisionLoop) ) &&
        ( !pSemCtrl->pOwnerTaskCtrl ))
      {
      // --- normaly 'vendexcl' detects collisions. In
      //     this situation it can't but we do
      pSemCtrl->lCollisionCount += XPARAM(ulRegionCollisionLoop) - lReservedLoopCnt;
      }
    }
  else
    { WAIT_LOCK ( &pUKT, pTaskCtrl, pSemCtrl, &pSemQueHead->lExclusive, sid ,
                  &pSemCtrl->lBegExclTasCount, pTaskCtrl->PrioFlag );
    }

  /*
   * we are alone now
   */
  if ( !pSemCtrl->pOwnerTaskCtrl )
    {
    pSemCtrl->pOwnerTaskCtrl = pTaskCtrl;

    CLEARLOCK( &pSemQueHead->lExclusive );
    }
  else
    {
    pSemQueHead->pInsertPos->pTaskCtrl  = pTaskCtrl;
    pSemQueHead->pInsertPos             = pSemQueHead->pInsertPos->pNextElem;

    CLEARLOCK( &pSemQueHead->lExclusive );
    pSemCtrl->lWaitCount++;
    pUKT->ulNumRegWait++;

    #ifdef DEVELOPMENT
     pTaskCtrl->ulWaitTime = kgs.ulCurrTime;
    #endif

    GOTO_DISP(&pUKT);
    }

  pTaskCtrl->TaskState = TSK_RUNNING;

  DBGOUT_T (pTaskCtrl->ulTaskIndex);
  return;
  }


/*------------------------------*/

VOID vendexcl ( tsp00_TaskId    pid, 
                tsp00_RegionId  sid )
  {
  #undef  MF__
  #define MF__ MOD__"vendexcl"
  #ifdef DEVELOP_CHECKS
   PUKT_CTRL_REC                 pUKT      = THIS_UKT_CTRL;
   PTASK_CTRL_REC                pTaskCtrl = pUKT->pCTask;
  #else
   PTASK_CTRL_REC                pTaskCtrl = &kgs.pFirstTaskCtrl[pid - 1];
   PUKT_CTRL_REC                 pUKT      = pTaskCtrl->pUKT;
  #endif
  PSEM_CTRL_REC                  pSemCtrl;
  PTASK_CTRL_REC                 pNextTaskCtrl;
  PSEM_QUEUE_HEAD_REC            pSemQueHead;

  DBGIN_T (pTaskCtrl->ulTaskIndex);

  #ifdef DEVELOP_CHECKS
   if ( (ULONG)pid != pTaskCtrl->ulTaskIndex )
     {
     MSGD (( ERR_VXXXX_WRONG_TASK, "vendexcl", pid ));
     ABORT();
     }
  #endif

  DO_WHEN_DUMMY_REGION( sid , return );
  pTaskCtrl->TaskState = TSK_VENDEXCL;

  pSemCtrl    = kgs.pSemaphores + sid - 1;
  pSemQueHead = &pSemCtrl->SemaList;

  DBG3 (( MF__, "[T:0x%03u] Region '%s'",
          pTaskCtrl->ulTaskIndex, pSemCtrl->szSemaName ));

  if ( XPARAM(fExclQueue ) )
    WAIT_LOCK ( &pUKT, pTaskCtrl, pSemCtrl, &pSemQueHead->lExclusive, sid ,
                &pSemCtrl->lEndExclTasCount, XPARAM(fEndExclBusyWait));

  /*
   * we are alone now
   */

   // --- check if i am the owner
  if (pSemCtrl->pOwnerTaskCtrl != pTaskCtrl)
    {
    if ( XPARAM(fExclQueue ) )
      CLEARLOCK( &pSemQueHead->lExclusive );
    MSGD (( ERR_ENDEXCL_WITHOUT_BEG, pTaskCtrl->ulTaskIndex ));
    ABORT();
    }

  #ifdef DEVELOPMENT
   pTaskCtrl->ulWaitTime = 0;
  #endif

  pTaskCtrl->lNestingExcl--;

  pNextTaskCtrl = pSemQueHead->pRemovePos->pTaskCtrl;

  if ( pNextTaskCtrl )
    {
    //
    // --- somebody else is waiting for 'bendexcl'
    //
    pSemCtrl->pOwnerTaskCtrl            = pNextTaskCtrl;
    pSemQueHead->pRemovePos->pTaskCtrl  = NULL;
    pSemQueHead->pRemovePos             = pSemQueHead->pRemovePos->pNextElem;

    CLEARLOCK( &pSemQueHead->lExclusive );
    pSemCtrl->lCollisionCount++;

    // ---  is the next tasks UKT eq. to the local UKT
    if ( pNextTaskCtrl->pUKT == pUKT )
      {
      pUKT->ulNumRegWait--;
      DBG4 (( MF__, "[T:0x%03u] vendexcl request for local task [T:0x%03u]",
              pTaskCtrl->ulTaskIndex, pNextTaskCtrl->ulTaskIndex ));

      pNextTaskCtrl->SpecDLQElem1.ulReqType = REQ_VENDEXCL;
      sql74k_RUN_enqu ( pUKT, pNextTaskCtrl, &pNextTaskCtrl->SpecDLQElem1, 
                        BASE_PRIO_RAV );

      // --- reschedule myself via rex queue
      pTaskCtrl->SpecDLQElem1.ulReqType     = REQ_RESCHEDULE;
      sql74k_FastRUN_enqu ( pUKT, &pTaskCtrl->SpecDLQElem1, BASE_PRIO_REX );

      GOTO_DISP(&pUKT);
      }
    else
      {
      DBG4 (( MF__, "[T:0x%03u] vendexcl request for remote task [T:0x%03u]",
              pTaskCtrl->ulTaskIndex, pNextTaskCtrl->ulTaskIndex ));

      sql74k_UToU_enqu_and_wake ( pUKT, pTaskCtrl, pNextTaskCtrl->pUKT,
                                  pNextTaskCtrl, sql55k_remote_vendexcl, 0 );

      if ( kgs.fBusyWaitingForbidden )
         {
         RESCHEDULE ;
         }
      }
    }
  else
    {
    /*
     * Queue is empty
     */
    pSemCtrl->pOwnerTaskCtrl = NULL;

    if ( XPARAM(fExclQueue) )
      CLEARLOCK( &pSemQueHead->lExclusive );

    if (( pTaskCtrl->ulBegExclCnt > XPARAM(ulRegLockSlice) ||
          pTaskCtrl->pUKT->ulImmediateDispatch ) &&
          !pTaskCtrl->lNestingExcl && !pTaskCtrl->PrioFlag )
      {
      // --- reschedule myself via rex queue
      if ( pTaskCtrl->pUKT->ulImmediateDispatch ) pTaskCtrl->pUKT->ulImmediateDispatch--;
      pTaskCtrl->SpecDLQElem1.ulReqType = REQ_RESCHEDULE;
      sql74k_FastRUN_enqu ( pUKT, &pTaskCtrl->SpecDLQElem1, BASE_PRIO_REX  );
      pTaskCtrl->ulSelfSuspCount++;
      pTaskCtrl->TaskState = TSK_RUNNABLE;

      GOTO_DISP(&pUKT);
      }
    }

  pTaskCtrl->TaskState = TSK_RUNNING;
  pTaskCtrl->Sid = 0;

  DBGOUT_T (pTaskCtrl->ulTaskIndex);
  return;
  }

/*------------------------------*/

VOID sql55k_free_regions ( PUKT_CTRL_REC   pUKT )
  {
  #undef  MF__
  #define MF__ MOD__"sql55k_free_regions"
  ULONG                          ulSem;
  PSEM_CTRL_REC                  pSemCtrl;
  PTASK_CTRL_REC                 pOwnerTaskCtrl;
  PSEM_QUEUE_REC                 pSemQueElem;

  DBGPAS;

  for ( ulSem = 0; ulSem < XPARAM(ulNoOfRegions); ulSem ++ )
    {
    pSemCtrl = kgs.pSemaphores + ulSem;


    WAIT_UNTIL_ALONE( &pSemCtrl->SemaList.lExclusive );

    /*
     * we are alone now
     */
    // --- save current owner
    pOwnerTaskCtrl = pSemCtrl->pOwnerTaskCtrl;

    // --- continue
    //      - if there is no owner
    //      - if i am not the owner and the owner isn't dead
    //
    if (( pOwnerTaskCtrl == NULL ) || (( pOwnerTaskCtrl->pUKT != pUKT ) &&
        ( pOwnerTaskCtrl->pUKT->ThrdCtrlHeader.ThrdState != UKT_TERMINATED )))
      {
      CLEARLOCK( &pSemCtrl->SemaList.lExclusive );
      continue;
      }

    // --- is the current owner not the tracewriter?
    if ( pOwnerTaskCtrl != kgs.pTW )
      {
      pSemCtrl->pOwnerTaskCtrl->TaskState = TSK_LOCKED;
      pSemCtrl->pOwnerTaskCtrl            = NULL;

      // --- try to make tracewriter to new owner
      for ( pSemQueElem =  pSemCtrl->SemaList.pRemovePos;
            pSemQueElem->pTaskCtrl;
            pSemQueElem =  pSemQueElem->pNextElem )
        {
        if ( pSemQueElem->pTaskCtrl == kgs.pTW )
          {
          // -- tracewriter is the new owner
          pSemCtrl->pOwnerTaskCtrl      = pSemQueElem->pTaskCtrl;
          pSemQueElem->pTaskCtrl        = NULL;
          pSemQueElem                   = pSemQueElem->pNextElem;
          }
        else
          {
          pSemQueElem->pTaskCtrl->TaskState = TSK_LOCKED;
          pSemQueElem->pTaskCtrl            = NULL;
          }
        }
      pSemCtrl->SemaList.pRemovePos = pSemQueElem;
      pSemCtrl->SemaList.pInsertPos = pSemQueElem;
      }

    // --- owner changed?
    if ( pOwnerTaskCtrl != pSemCtrl->pOwnerTaskCtrl )
      pOwnerTaskCtrl = pSemCtrl->pOwnerTaskCtrl;
    else
      pOwnerTaskCtrl = NULL;

    CLEARLOCK( &pSemCtrl->SemaList.lExclusive );


    // --- is there a new owner
    if ( pOwnerTaskCtrl )
      {
      // --- the enqueue functions and the following output
      //     needs the current-task-value for DEBUG purpose!
      if (pUKT->pCTask) pUKT->pCTask = pOwnerTaskCtrl;

      DBG4 (( MF__, "[T:0x%03u] Regions freed, wake up TW",
              pOwnerTaskCtrl->ulTaskIndex ));

      if ( pOwnerTaskCtrl->pUKT == pUKT )
        {
        // --- 'vendexcl' via rav queue
        pOwnerTaskCtrl->SpecDLQElem1.ulReqType = REQ_VENDEXCL;
        sql74k_RUN_enqu ( pUKT, pOwnerTaskCtrl, &pOwnerTaskCtrl->SpecDLQElem1,
                          BASE_PRIO_RAV );
        }
      else
        {
        sql74k_UToU_enqu_and_wake ( pUKT, pUKT->pCTask, pOwnerTaskCtrl->pUKT,
                                    pOwnerTaskCtrl, sql55k_remote_vendexcl, 0 );
        }
      }
    }
  return;
  }


/*
 * ========================== LOCAL FUNCTIONS =================================
 */

static VOID  sql55k_init_semaliste ( PSEM_CTRL_REC  pSemCtrl )
  {
  #undef  MF__
  #define MF__ MOD__"sql55k_init_semaliste"
  INT                            i;
  PSEM_QUEUE_REC                 pSemQueElem1;
  PSEM_QUEUE_REC                 pSemQueElem2;
  #ifdef DEBUG_RTE
   PTASK_CTRL_REC                pTaskCtrl = THIS_UKT_CTRL->pCTask;
  #endif

  DBGIN_T (pTaskCtrl->ulTaskIndex);

  pSemQueElem1 = &pSemCtrl->SemaList.pSEMs[0];

  for ( i = 0; (ULONG)i < kgs.ulNumOfTasks; i ++ )
    {
    pSemQueElem1->pTaskCtrl = NULL;
    pSemQueElem2            = pSemQueElem1 + 1;
    pSemQueElem1->pNextElem = pSemQueElem2;
    pSemQueElem1            = pSemQueElem2;
    }

  pSemQueElem1--;
  pSemQueElem2            = &pSemCtrl->SemaList.pSEMs[0];
  pSemQueElem1->pNextElem = pSemQueElem2;

  pSemCtrl->SemaList.pInsertPos = pSemQueElem2;
  pSemCtrl->SemaList.pRemovePos = pSemQueElem2;
  pSemCtrl->SemaList.lExclusive = 0;

  DBGOUT_T (pTaskCtrl->ulTaskIndex);
  return;
  }

/*------------------------------*/

#if defined (OS2)   // -- ignore unused parameter  'ulDummyParam'
# pragma info (nopar)
#endif

VOID sql55k_remote_vendexcl ( PPTASK_CTRL_REC ppCTask,
                                     PTASK_CTRL_REC pFromTask,
                                     PPDLQ_REC       ppCurrReq,
                                     ULONG           ulDummyParam )
  {
  #undef  MF__
  #define MF__ MOD__"sql55k_remote_vendexcl"
  DBGPAS_T ((*ppCTask)->ulTaskIndex);

  (*ppCTask)->pUKT->ulNumRegWait--;
  *ppCurrReq                         = &((*ppCTask)->SpecDLQElem1);
  (*ppCTask)->SpecDLQElem1.ulReqType = REQ_VENDEXCL;

  return;
  }

#if defined (OS2)
# pragma info (restore)
#endif

/*------------------------------*/

#if !defined (USE_HUGE_MACROS)
_INLINE VOID __sql55k_wait_lock ( PUKT_CTRL_REC   *ppUKT,
                                  PTASK_CTRL_REC  pTaskCtrl,
                                  PSEM_CTRL_REC   pSemCtrl,
                                  PLONG           plExclusive,
                                  tsp00_RegionId  sid ,
                                  ULONG           *plExclTasCnt,
                                  BOOLEAN         fBusyWait )
  {
  #undef  MF__
  #define MF__ MOD__"__sql55k_wait_lock"
  LONG                     lTasCount;

  DBGPAS_T (pTaskCtrl->ulTaskIndex);


  lTasCount = MAX_TAS_CNT;

  DBG4 (( MF__, "[T:0x%03u] Waiting for region interlock %u",
          pTaskCtrl->ulTaskIndex, sid ));

  while ( INTERLOCK( plExclusive ) )
    {
    lTasCount--;

    if ( !lTasCount )
      {
      MSGD (( ERR_INDEF_LOOP, sid ));
      ABORT();
      }

    if ( kgs.fBusyWaitingForbidden || !( lTasCount % SLEEP_CNT )) 
      {
      RESCHEDULE ;
      }
    else
      if ( !fBusyWait )
      {
      /* --- reschedule myself via rex queue  */
      pTaskCtrl->SpecDLQElem2.ulReqType = REQ_RESCHEDULE;
      sql74k_FastRUN_enqu(*ppUKT, &pTaskCtrl->SpecDLQElem2, BASE_PRIO_REX);
      GOTO_DISP(ppUKT);
      }
    }
  *plExclTasCnt += MAX_TAS_CNT - lTasCount + 1;

  return;
  }
#endif

/*
 * =============================== END ========================================
 */
