/*!**************************************************************************

  module      : vos90c.c

  -------------------------------------------------------------------------

  responsible : RaymondR

  special area: Start
  description : 


  last changed: 2001-09-21  15:38
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*
 * MODULE - XSTART
 */

/*
 * INCLUDE FILES
 */
#include "gos00.h"
#include "heo00.h"
#include "heo46.h"
#include "geo007_1.h"
#include "geo007_2.h"
#include "hsp100.h"

#if defined (CLUSTER_SERVER_VERSION)
# include "gos200.h"   /* nocheck */
# include "gos212.h"   /* nocheck */
#endif

#include           <stdarg.h>

/*
 *  DEFINES
 */
#define MOD__  "VOS90CC : "
#define MF__   MOD__"UNDEFINED"

#if defined(_WIN32)
 #define OPTION_STRING             "id:s:q:n:lt:"
#else
 #define OPTION_STRING             "d:s:q:a"
#endif

/*
 *  MACROS
 */


/*
 *  LOCAL TYPE AND STRUCT DEFINITIONS
 */


/*
 * EXTERNAL VARIABLES
 */
extern int        sql80_OptInd;
extern int        sql80_OptErr;
extern char*      sql80_OptArg;


/*
 *  EXPORTED VARIABLES
 */


/*
 * LOCAL VARIABLES
 */


/*
 * LOCAL FUNCTION PROTOTYPES
 */


/*
 * ========================== GLOBAL FUNCTIONS ================================
 */

INT main (INT     argc, CHAR   *argv[])
  {
  #undef  MF__
  #define MF__ MOD__"main"
  INT                       OptionChar;
  APIRET                    rc                    = NO_ERROR;
  PSZ                       pszServerDB           = NULL;
  PSZ                       pszNodeName           = NULL;
  BOOL                      fCommandError         = FALSE;
  PSZ                       pszDBRoot;
  ULONG                     ulKernelType          = KERNEL_TYPE_FAST;
  ULONG                     ulOptions             = 0;

  DBGIN;

  while ( (OptionChar = sql80_GetOpt ( argc, argv, OPTION_STRING )) != -1 )
    {
    switch ( OptionChar )
      {
      case 'd' :
              pszServerDB = sql80_OptArg;
              break;
      case 'n' :
              pszNodeName = sql80_OptArg;
              break;
      case 's' :
              if (!strcmp ( sql80_OptArg, "low" ) && ( pszNodeName == NULL ) )
                {
                ulKernelType = KERNEL_TYPE_SLOW;
                }
              else
                fCommandError = TRUE;
              break;
      case 'q' :
              if (!strcmp ( sql80_OptArg, "uick" ) && ( pszNodeName == NULL ) )
                {
                ulKernelType = KERNEL_TYPE_QUICK;
                }
              else
                fCommandError = TRUE;
              break;
      case 't' :
              if (!strcmp ( sql80_OptArg, "est" ) && ( pszNodeName == NULL ) )
                {
                ulKernelType = KERNEL_TYPE_TEST;
                }
              else
                fCommandError = TRUE;
              break;
      case 'a' :
              ulOptions  |= AUTOCLOSE;
              break;
      case 'l' :
              ulOptions  |= LOW_PROCESS_PRIORITY;
              break;
      case 'i' :
              ulOptions  |= IGNORE_AUTORESTART;
              break;
      default :
              fCommandError = TRUE;
      }
    }

  if ( (argc - sql80_OptInd) || (fCommandError == TRUE) )
    {
    #if defined(_WIN32)
     if ( sql02_get_platform_id() == VER_PLATFORM_WIN32_NT )
       printf (XSTART_USAGE_NT);
     else
       printf (XSTART_USAGE);
    #else
     printf (XSTART_USAGE);
    #endif
    exit(0);
    }


  if ( pszServerDB == NULL)
    {
    if ( sql01c_get_serverdb ( &pszServerDB  ) == FALSE )
      {
      printf (XSTART_USAGE);
      DBGOUT;
      exit(1);
      }
    }

  #if defined(_WIN32)
   if ( sql02_get_platform_id() == VER_PLATFORM_WIN32_NT )
     {
     rc = sql90_start_db_gw_service ( pszNodeName, pszServerDB,
                                      ulOptions, ulKernelType );

     DBGOUT;
     return( rc );
     }
  #endif

  // --- get DBROOT from Environment
  if ( sql01c_get_dbroot (&pszDBRoot) == FALSE )
    {
    MSGCD(( ERR_DBROOT_NOT_SET ));
    DBGOUT;
    exit(1);
    }

  rc = sql90_start_serverdb( pszServerDB, pszDBRoot,
                             ulOptions, ulKernelType );

  DBGOUT;
  return( rc );
  }

/*
 * ========================== LOCAL FUNCTIONS =================================
 */

/*
 * =============================== END ========================================
 */

