/****************************************************************************

  module      : vos92.c

  -------------------------------------------------------------------------

  responsible : RaymondR

  special area: cons (x_cons) ...
  description : NT main module ...


  last changed: 2000-12-05  10:25
  see also    : example.html ...

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1999



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "hsp100.h"
#include "heo00x.h"
#include "geo00_2.h"
#include "geo002.h"
#include "heo46.h"
#include "heo92.h"
#include "gos00k.h"
#include "gos00cn.h"
#include "geo007_1.h"
#include "geo007_2.h"
#include "gos92.h"
#include "gos41.h"


#ifdef  CTRLCOMP
#include "vsp004.h" /* nocheck */
#include "vsp009c.h" /* nocheck */
#include "heo05.h" /* nocheck */
#endif

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define MOD__  "VOS92.C : "
#define MF__   MOD__"UNDEFINED"

#define MAX_WAIT          30       // --- 30 sec.
#define MAX_ERRSTRING     200
#define ARGS(max, cnt)      ((max-cnt) >= 1)

#define COM_UNDEF           0
#define COM_CANCEL          1
#define COM_KILL            2
#define COM_SHOW_UPDATE     3
#define COM_HELP            4
#define COM_DROP            5
#define COM_TIME_MEASURE    6
#define COM_RESET           7
#define COM_GETPARAM        8
#define COM_PUTPARAM        9
#define COM_CRASH_DUMP     10
#define COM_SET_DEBUGLEVEL 11
#define COM_UKT_AFFINITY   12
#define COM_ERRIOR         13
#define COM_ERRIOW         14
#define COM_DEBUG_BREAK    15
#define COM_PROCESSOR_MASK 16
#define COM_DEBUG_TASK     17
#define COM_SET_LB_CHK     18


#define PARA_NONE         0x00
#define PARA_ENABLE       0x01
#define PARA_DISABLE      0x02

#define PARA_IO           0x01
#define PARA_STORAGE      0x02
#define PARA_TASKS        0x04
#define PARA_VERSIONS     0x08
#define PARA_REGIONS      0x10
#define PARA_STATE        0x20
#define PARA_RTE          0x40
#define PARA_TASK_STAT    0x100
#define PARA_TASK_COUNTS  0x200
#define PARA_QUEUE_STAT   0x400
#define PARA_TOTAL_COUNTS 0x800
#define PARA_ACTIVE       0x1000
#define PARA_UKT_SL_STAT  0x2000
#define PARA_SUSPENDS     0x4000
#define PARA_UKT_QUEUES   0x8000
#define PARA_RUNNABLE     0x10000
#define PARA_AIO          0x20000
#define PARA_THREAD_TIMES 0x40000
#define PARA_AWE          0x80000
#define PARA_CONNECTIONS  0x100000
#define PARA_TASK_MOVE    0x200000
#define PARA_MOVE_INFO    0x400000

#define PARA_SET          0x01
#define PARA_CLEAR        0x02


#define TASKS_INFO_P_LINE 2

#define PARA_DEFAULT     PARA_IO | PARA_TASKS | PARA_VERSIONS

#if defined ( ALL_WITH_AWE )
# define PARA_ALL          ( 0xFFFFFFFF - PARA_ACTIVE - PARA_RUNNABLE )
#else
# define PARA_ALL          ( 0xFFFFFFFF - PARA_ACTIVE - PARA_RUNNABLE - PARA_AWE )
#endif



/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/

#define STATE_TIME_AVAIL(state) ( state.ulSec || state.ulMicroSec )

#define TOT_TIME( State ) \
            ((float )State.ulSec + (float )State.ulMicroSec / 1000000.0 )

#define OFFSET_OFF(_strct,_elem)   ((size_t)&(((_strct *)0)->_elem))

/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/

typedef struct xcons_record
  {
  SQL_DBNAMEC                     szServerDB;
  PCONSOLE_REC                    pConsoleRec;
  HANDLE                          hConsoleRec;
  C40C                            RTEVersion;
  C40C                            KernelVersion;
  ULONG                           ulShmSize;
  LONG                            lCommand;
  LONG                            lParam;
  LONG                            lValue;
  PSZ                             pszParam;
  LONG                            lExtParam;
  LONG                            lInterval;
  LONG                            lRepeatCount;
  HANDLE                          hHandle;

  PID                             pidQueueOwner; // --- OS/2 only
  SECURITY_ATTRIBUTES             WorldSA;       // --- WIN32 only

  #if defined(_WIN32)
   IPC_BASE_OBJ_NAME              szShmName;
  #endif

  }  XCONS_REC;

typedef XCONS_REC                 *PXCONS_REC;

typedef struct thread_times
  {
    ULONG                         ulDays,
                                  ulHr,
                                  ulMin, 
                                  ulSec,
                                  ulMsec ;
  } XC_THREAD_TIMES ;



/*===========================================================================*
 *  GLOBAL VARIABLES                                                         *
 *===========================================================================*/


/*===========================================================================*
 *  LOCAL VARIABLES                                                          *
 *===========================================================================*/



/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
 *===========================================================================*/

LONG sql92c_get_num_param         ( PSZ                     pszParam,
                                    PLONG                   plNum  );
LONG sql92c_GetProcessorAffinityMask ( PSZ      pszParam,
                                       PLONG    ProcessAffinityMask  );
LONG sql92c_get_idx_param         ( PSZ                     pszParam,
                                    CHAR                    cPrefix,
                                    PLONG                   plNum  );
LONG sql92c_get_set_clear_param   ( PSZ                     pszParam );

LONG sql92c_get_time_param        ( PSZ                     pszParam );
LONG sql92c_get_show_res_param    ( PSZ                     pszParam,
                                    LONG                    lCommand );
LONG sql92c_init                  ( char const * const      pszServerDB,
                                    PXCONS_REC              pxcr,
                                    tsp00_Bool*             pDBIsOffline );
VOID sql92c_release               ( PXCONS_REC				      pxcr );
LONG sql92c_check_accesss         ( PXCONS_REC              pxcr );
LONG sql92c_send_req              ( PXCONS_REC              pxcr );
LONG sql92c_send                  ( PXCONS_REC              pxcr,
                                    ULONG                   ulRequest,
                                    tsp00_Bool*             pDBIsOffline );
BOOL sql92c_wait                  ( PXCONS_REC              pxcr,
                                    PBOOL                   pfFlag,
                                    ULONG                   ulSecWaitTime );
PSZ  sql92c_kt_state_name         ( ULONG                   ulState );
PSZ  sql92c_ukt_state_name        ( ULONG                   ulState );
PSZ  sql92c_req_type_name         ( ULONG                   ulReqType );
LONG sql92c_show_version_info     ( PXCONS_REC              pxcr );
LONG sql92c_show_task_info        ( PCONSOLE_REC            pConsoleRec, SAPDB_Bool fTaskMoveInfo );
LONG sql92c_show_io_info          ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_aio_info         ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_region_info      ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_storage_info     ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_state_info       ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_rte_info         ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_queue_info       ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_task_count_info  ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_total_count_info ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_ukt_sl_info      ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_ukt_queues       ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_UKT_queue        ( ULONG ulUKTQueueLen,
                                    PCONS_SINGLE_QUEUE_INFO_REC pUKTUKTQueue);
LONG sql92c_show_COM_queue        ( ULONG ulCOMQueueLen,
                                    PCONS_SINGLE_QUEUE_INFO_REC pUKTCOMQueue);
LONG sql92c_show_RUN_queue        ( ULONG ulCOMQueueLen,
                                    PCONS_SINGLE_QUEUE_INFO_REC pUKTCOMQueue);
LONG sql92c_show_suspends         ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_thread_times     ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_get_com_param         ( PSZ                     pszParam );

BOOL sql92c_kernel_is_alive       ( PXCONS_REC              pxcr );
LONG sql92c_show_param            ( PSZ                     pszParam,
                                    PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_awe_info         ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_connections_info ( PCONSOLE_REC            pConsoleRec );
LONG sql92c_show_task_move_info   ( PCONSOLE_REC            pConsoleRec );


/*===========================================================================*
 *  PUBLIC FUNCTIONS (CODE)                                                  *
 *===========================================================================*/

#if !defined (CTRLCOMP)
INT main (INT argc, CHAR *argv[])
  {
  #undef  MF__
  #define MF__ MOD__"main"
  XCONS_REC               xcr;
  INT                     ArgN          = 1;
  LONG                    rc            = NO_ERROR;
  LONG                    lCommand;
  LONG                    lParam;
  tsp00_Bool              fDummy;
  char const *            pszServerDB;

  DBGIN;

  memset ( &xcr, 0, sizeof(xcr) );
  xcr.lInterval     = 0;
  xcr.lRepeatCount  = 1;
  xcr.lCommand      = COM_UNDEF;
  xcr.lParam        = 0;
  xcr.lExtParam     = 0;

  if ( ARGS(argc, ArgN) )
    {
    if ((( argv [ ArgN ] [ 0 ] == '-' ) || ( argv [ ArgN ] [ 0 ] == '/' )) &&
        (( argv [ ArgN ] [ 1 ] == '?' ) || ( argv [ ArgN ] [ 1 ] == 'h' )) &&
        (  argv [ ArgN ] [ 2 ] == '\0' ))
      {
      xcr.lCommand = COM_HELP;
      }
    else
      {
      //
      // --- get command and server database name
      //
      xcr.lCommand = sql92c_get_com_param ( argv[ArgN] );

      if (( xcr.lCommand == COM_UNDEF ) || ( xcr.lCommand != COM_HELP ) || ( argc > 2 ))
        {
         pszServerDB = argv[ArgN++];

         if ( ARGS(argc, ArgN) )
           {
           xcr.lCommand = sql92c_get_com_param ( argv[ArgN++] );

           if ( xcr.lCommand == COM_UNDEF )
             rc = ! NO_ERROR;
           }
         else
           rc = ! NO_ERROR;
        }
      else if ( ARGS(argc, ++ArgN) )
        {
        lCommand = sql92c_get_com_param ( argv[ArgN] );

        if ( lCommand != COM_UNDEF ) // - is there another command?
          {
           pszServerDB = argv [ArgN - 1];
           ArgN++;
          }
        }

      if (( rc == NO_ERROR ) && ( xcr.lCommand != COM_HELP ))
        {
        switch ( xcr.lCommand )
          {
          case COM_UKT_AFFINITY:
            if ( ARGS(argc, ArgN) )
              rc = sql92c_get_idx_param ( argv[ArgN++], 'u', &xcr.lParam );
            else
              rc = ! NO_ERROR;
            break;

          // PTS 1107070
          case COM_PROCESSOR_MASK:
            if ( ARGS(argc, ArgN) )
            {
              rc = sql92c_GetProcessorAffinityMask ( argv[ArgN++], &xcr.lParam );
              
              if ( rc == NO_ERROR && ARGS(argc, ArgN) )
                rc = e92_get_tt_to_show (argv[ArgN], &xcr.lExtParam );
            }
            else
              rc = ! NO_ERROR;
            break;

          case COM_CANCEL:
            if ( ARGS(argc, ArgN) )
              rc = sql92c_get_idx_param ( argv[ArgN++], 't', &xcr.lParam );
            else
              rc = ! NO_ERROR;
            break;
            
          case COM_KILL  :
            if ( ARGS(argc, ArgN) )
              rc = sql92c_get_idx_param ( argv[ArgN++], 't', &xcr.lParam );
            else
              rc = ! NO_ERROR;
            break;

          case COM_DROP  :

            if ( !ARGS(argc, ArgN) ||
                 !e92_strcont( XCONS_CMD_TABLES, strupr(argv[ArgN++]),
                                  XCONS_MIN_CMD_LEN_TABLES ))
              {
              rc = ! NO_ERROR;
              }
            break;

          case COM_CRASH_DUMP:
            if ( !ARGS(argc, ArgN) ||
                 !e92_strcont( XCONS_CMD_DUMP, strupr(argv[ArgN++]),
                                  XCONS_MIN_CMD_LEN_DUMP ))
              {
              rc = ! NO_ERROR;
              }
            break;

          case COM_TIME_MEASURE :
            if ( ARGS(argc, ArgN) )
              {
              lParam = sql92c_get_time_param ( argv[ArgN++] );

              if ( lParam != PARA_NONE )
                xcr.lParam |= lParam;
              else
                rc = ! NO_ERROR;
              }
            else
              rc = ! NO_ERROR;
            break;

          case COM_RESET :
            if ( ARGS(argc, ArgN) )
              {
              lParam = sql92c_get_show_res_param( argv[ArgN++], xcr.lCommand );

              if ( lParam != PARA_NONE )
                xcr.lParam |= lParam;
              else
                rc = ! NO_ERROR;
              }
            else
              rc = ! NO_ERROR;
            break;

          case COM_ERRIOR :
          case COM_ERRIOW :
            if ( ARGS(argc, ArgN) )
              rc = sql92c_get_num_param ( argv[ArgN++], &xcr.lParam );
            else
              rc = ! NO_ERROR;
            break;

          case COM_SET_DEBUGLEVEL :
            if ( ARGS(argc, ArgN) )
              rc = sql92c_get_num_param ( argv[ArgN++], &xcr.lParam );
            else
              rc = ! NO_ERROR;
            break;

          case COM_GETPARAM :

            if ( ARGS(argc, ArgN) )
              xcr.pszParam = argv[ArgN++];
            else
              xcr.pszParam = "";

            break;

          case COM_PUTPARAM :

          case COM_SHOW_UPDATE :

            // --- command parameter found?
            if (( ARGS(argc, ArgN)) && ( !isdigit(argv[ArgN][0]) ))
              {
              xcr.lParam = 0;

              do
                {
                lParam = sql92c_get_show_res_param(argv[ArgN++], xcr.lCommand);

                if ( lParam != PARA_NONE )
                  {
                  BOOL fExtParamOk = FALSE ;
                  if (( lParam == PARA_TASK_COUNTS ) &&
                      ( ARGS(argc, ArgN)))
                    {
                    rc = sql92c_get_idx_param(argv[ArgN], 't', &xcr.lExtParam);

                    if ( ! (fExtParamOk = rc == NO_ERROR ) )
                      { rc = e92_get_tt_to_show (argv[ArgN], &xcr.lExtParam );
                        fExtParamOk = rc == NO_ERROR;
                      }
                    else
                      xcr.lExtParam = -xcr.lExtParam ;
                    }
                  else if (lParam == PARA_TASKS || lParam == PARA_ACTIVE || lParam == PARA_RUNNABLE  || lParam == PARA_TASK_MOVE)
                    {
                    xcr.lExtParam = SHOW_TT_ALL_EO92;
                    if ( ARGS(argc, ArgN) )
                      {
                      rc = e92_get_tt_to_show (argv[ArgN], &xcr.lExtParam );
                      fExtParamOk = rc == NO_ERROR;
                      }
                    }

                  xcr.lParam |= lParam;
                  if ( !fExtParamOk )
                    rc = NO_ERROR;
                  else
                    ArgN++;
                  }
                else
                  rc = ! NO_ERROR;
                }
              while (( rc == NO_ERROR )      &&
                     ( ARGS(argc, ArgN))     &&
                     ( !isdigit(argv[ArgN][0]) ));
              }
            else
              xcr.lParam = PARA_DEFAULT;

            // --- try to get interval and repeat count
            if (( rc == NO_ERROR ) && ARGS(argc, ArgN))
              {
              xcr.lRepeatCount = -1;

              // --- try to get interval
              rc = sql92c_get_num_param (argv[ArgN++], &xcr.lInterval);

              if ( rc == NO_ERROR )
                {
                if ( xcr.lInterval < 0 )
                  rc = ! NO_ERROR;
                else if ( ARGS(argc, ArgN))
                  {
                  // --- try to get repeat count
                  rc = sql92c_get_num_param (argv[ArgN++], &xcr.lRepeatCount);

                  if ( xcr.lRepeatCount < 1 )
                    rc = ! NO_ERROR;
                  }
                }
              }
            break;

          case COM_SET_LB_CHK:
            if ( ARGS(argc, ArgN) )
              rc = sql92c_get_num_param (argv[ArgN++], &xcr.lParam);
            else
              rc = ! NO_ERROR;
            break;

          case COM_DEBUG_TASK:

            if ( ARGS(argc, ArgN) )
              rc = sql92c_get_idx_param ( argv[ArgN++], 't', &xcr.lParam );
            else
              rc = ! NO_ERROR;
            break;

          case COM_DEBUG_BREAK:

            if ( ARGS(argc, ArgN) )
              {
              xcr.lParam = sql92c_get_set_clear_param ( argv[ArgN++] );

              if ( xcr.lParam == PARA_SET )
                {
                if ( ARGS(argc, ArgN) )
                  rc = sql92c_get_num_param (argv[ArgN++], &xcr.lValue);
                else
                  rc = ! NO_ERROR;
                }
              else if ( xcr.lParam != PARA_CLEAR )
                rc = ! NO_ERROR;
              }
            else
              rc = ! NO_ERROR;

            break;

          default :
            rc = ! NO_ERROR;
          }
        ArgN++;
        }
      }
    }
  else
    rc = ! NO_ERROR;

  //
  // --- display help info ?
  //
  if (( ARGS( argc, ArgN ) )     ||
      ( rc != NO_ERROR  )        ||
      ( xcr.lCommand == COM_HELP   ) ||
      ( xcr.lCommand == COM_UNDEF  ))
    {
    printf  (XCONS_USAGE);
    DBGOUT;
    exit(rc);
    }
  
  if ( pszServerDB == NULL )
     {
     rc = GETENV(SERVERDB_ENV_VAR, &pszServerDB);

     if (rc != NO_ERROR  ||  *pszServerDB == '\0')
       {
          printf  (XCONS_USAGE);
          DBGOUT;
          exit(1);
       }
     }


  if ( (rc = sql92c_init ( pszServerDB, &xcr, &fDummy )) == NO_ERROR )
    {
    rc = sql92c_send_req ( &xcr );

    sql92c_release( &xcr );
    }


  DBGOUT;
  exit(rc);

  return(rc);
  }
#endif

/*------------------------------*/

CONS_HDL sql92c_console_open ( char const * const  pszServerDB,
                               PCONSOLE_DATA_REC*  ppConsoleData,
                               tsp00_Uint4*        pMaxDataSize,
                               tsp00_Uint4*        pDBState )
{
  #undef  MF__
  #define MF__ MOD__"sql92c_console_open"

  LONG        rc;
  PXCONS_REC  pxcr;
  tsp00_Bool  fDBIsOffline;

  rc = ALLOC_MEM ((PPVOID)&pxcr, sizeof(XCONS_REC) );

  if( rc == NO_ERROR )
    rc = sql92c_init ( pszServerDB, pxcr, &fDBIsOffline );

  if( rc == NO_ERROR )
  {
    rc = sql92c_send ( pxcr, CON_REQ_STATE, &fDBIsOffline );

    if ( rc == NO_ERROR )
      {
      *pDBState       = pxcr->pConsoleRec->Data.StateInfo.DatabaseState;
      *ppConsoleData  = &pxcr->pConsoleRec->Data;
      *pMaxDataSize   = pxcr->ulShmSize;
       memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );
      }
  }

  if( rc != NO_ERROR )
  {
    FREE_MEM ( pxcr );
    *pDBState      = fDBIsOffline ? SERVER_OFFLINE : 0;
    *ppConsoleData = NULL;
    *pMaxDataSize  = 0;
    pxcr           = NULL;
    SetLastError( rc );
  }

  return (CONS_HDL)pxcr;
}

/*------------------------------*/

LONG sql92c_console_request ( CONS_HDL          Hdl,
                              tsp00_Uint4       Request,
                              tsp00_Bool        *pDBIsOffline )
{
  #undef  MF__
  #define MF__ MOD__"sql92c_console_request"

  PXCONS_REC  pxcr = (PXCONS_REC)Hdl;

  return sql92c_send ( pxcr, Request, pDBIsOffline );
}

/*------------------------------*/

void  sql92c_console_close ( CONS_HDL            Hdl )
{
  #undef  MF__
  #define MF__ MOD__"sql92c_console_close"

  PXCONS_REC  pxcr = (PXCONS_REC)Hdl;

  if ( pxcr != NULL )
  {
    sql92c_release( pxcr );
    FREE_MEM ( pxcr );
  }

  return;
}


/*===========================================================================*
 *  LOCAL/PRIVATE/PROTECTED FUNCTIONS (CODE)                                 *
 *===========================================================================*/

static LONG sql92c_init ( char const * const   pszServerDB,
                          PXCONS_REC           pxcr,
                          tsp00_Bool*          pDBIsOffline )

  {
  #undef  MF__
  #define MF__ MOD__"sql92c_init"
  LONG                    rc       = NO_ERROR;
  HANDLE                  hKSS     = 0;
  KERNEL_SHARED_SEG       *pKSS    = NULL;
  PSZ                     pszPath  = NULL;
  tsp00_Versionc          RTEVersionString;
  tsp100_VersionResult    Result;

  ULONG                   ulRetry  = 10;
  ACCESS_RIGHTS_REC       Access;

  DBGIN;

  strncpy ( pxcr->szServerDB, pszServerDB, sizeof(pxcr->szServerDB)-1 );
  pxcr->szServerDB[sizeof(pxcr->szServerDB)-1] = '\0';

  *pDBIsOffline = FALSE;

  //
  // --- allocate and initialize security attributes
  //
  pxcr->WorldSA.nLength             = sizeof(pxcr->WorldSA);
  pxcr->WorldSA.bInheritHandle      = FALSE;
  memset ( &Access, 0, sizeof (Access) );   // - this must be done first!

  Access.World.ulAccess = SPECIFIC_RIGHTS_ALL |
                          STANDARD_RIGHTS_ALL;
  rc = sql49c_alloc_and_init_SD( &Access, &pxcr->WorldSA.lpSecurityDescriptor );

  if ( rc != NO_ERROR )
    {
    DBGOUT;
    return ( rc );
    }
  
  //
  // --- open the request queue/mailslot
  //

  strupr ( pxcr->szServerDB );
  pszPath = MAIL_CONS;
  
  DBG3 ((MF__, "sql41c_open_queue_mailslot: %s", pxcr->szServerDB ));


  rc = sql41c_open_queue_mailslot ( &pxcr->pidQueueOwner,       // - OS/2 only
                                    &pxcr->hHandle,
                                    pszPath, pxcr->szServerDB );

 if ( rc == ERROR_FILE_NOT_FOUND )
   {
   FREE_MEM ( pxcr->WorldSA.lpSecurityDescriptor );
   MSGCD ((ERR_DB_OR_GW_NOT_STARTED, pxcr->szServerDB));
   *pDBIsOffline = TRUE;
   DBGOUT;
   return ( rc );
   }
 else if ( rc != NO_ERROR )
   {
   FREE_MEM ( pxcr->WorldSA.lpSecurityDescriptor );
   DBGOUT;
   return ( rc );
   }

  //
  // --- check database version
  //
  rc = sql41c_get_named_shrd_mem ( (PPVOID)&pKSS, SHM_KSS, pxcr->szServerDB,
                                   &hKSS );

  if ( rc == ERROR_INVALID_NAME )
    {
    FREE_MEM ( pxcr->WorldSA.lpSecurityDescriptor );
    sql41c_close_queue_mailslot ( pxcr->hHandle );

    MSGCD ((ERR_DB_OR_GW_NOT_STARTED, pxcr->szServerDB));
    DBGOUT;
    return ( rc );
    }
  else if (rc != NO_ERROR)
    {
    FREE_MEM ( pxcr->WorldSA.lpSecurityDescriptor );
    sql41c_close_queue_mailslot ( pxcr->hHandle );
    DBGOUT;
    return ( rc );
    }

  // --- set the RTE version string
  Result = sp100_CompareVersionIDs (&pKSS->RTEVersionID);


  if ( Result != IsEqual_esp100)
    {
    // --- set the RTE version string
    if ( sql02_get_platform_id() == VER_PLATFORM_WIN32_NT )
      sp100_GetVersionString(COMP_NAME_RTE_NT_SP100, s100buildnumber, RTEVersionString);
    else
      sp100_GetVersionString(COMP_NAME_RTE_WIN9X_SP100, s100buildnumber, RTEVersionString);

    MSGCD ((ERR_WRONG_DB_VERSION));

    printf (XCONS_WRONG_VERS_KERNEL, pKSS->RTEVersion);
    printf (XCONS_WRONG_VERS_XCONS , RTEVersionString);

    FREE_MEM ( pxcr->WorldSA.lpSecurityDescriptor );
    sql41c_close_queue_mailslot ( pxcr->hHandle );
    sql41c_free_shrd_mem ( pKSS, hKSS );

    DBGOUT;
    return ( 1 );
    }

  strcpy ( pxcr->RTEVersion,    pKSS->RTEVersion );
  strcpy ( pxcr->KernelVersion, pKSS->KernelVersion );

  pxcr->ulShmSize  = pKSS->ulConsoleShmSize;

  rc = sql41c_free_shrd_mem ( pKSS, hKSS );

  if ( rc != NO_ERROR )
    {
    FREE_MEM ( pxcr->WorldSA.lpSecurityDescriptor );
    sql41c_close_queue_mailslot ( pxcr->hHandle );

    DBGOUT;
    return ( rc );
    }

  //
  // --- allocate a shared memory segment
  //
  do
    {
    sql41c_build_unique_obj_name ( pxcr->szShmName );

    rc = sql41c_create_shrd_mem ( (PVOID*) &pxcr->pConsoleRec,
                                   pxcr->ulShmSize, SHM_CONSREC,
                                   pxcr->szShmName,
                                   &pxcr->WorldSA, &pxcr->hConsoleRec );

    // --- unique name already in use?
    if ( rc == ERROR_DUPLICATE_NAME )
      SLEEP ( 1 );
    }
  while (( rc == ERROR_DUPLICATE_NAME ) && (--ulRetry) );

  if ( rc != NO_ERROR )
    {
    FREE_MEM ( pxcr->WorldSA.lpSecurityDescriptor );
    sql41c_free_shrd_mem (pxcr->pConsoleRec, pxcr->hConsoleRec);
    sql41c_close_queue_mailslot ( pxcr->hHandle );
    DBGOUT;
    return ( rc );
    }

  DBGOUT;
  return ( rc );
  }

/*------------------------------*/

static VOID sql92c_release ( PXCONS_REC  pxcr )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_release"

  DBGPAS;

  FREE_MEM ( pxcr->WorldSA.lpSecurityDescriptor );
  sql41c_free_shrd_mem ( pxcr->pConsoleRec, pxcr->hConsoleRec );
  sql41c_close_queue_mailslot ( pxcr->hHandle );

  pxcr->WorldSA.lpSecurityDescriptor = NULL;
  pxcr->pConsoleRec				     = NULL;
  pxcr->hHandle                = INVALID_HANDLE_VALUE;
  pxcr->hConsoleRec            = INVALID_HANDLE_VALUE;
  
  return;
  }

/*------------------------------*/

static LONG sql92c_send ( PXCONS_REC     pxcr,
                          ULONG          ulRequest,
                          tsp00_Bool*    pDBIsOffline )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_send"
  LONG              rc = NO_ERROR;
  CONS_REQUEST_REC  Request;

  DBGIN;

  if (( ulRequest & SECURITY_CHECK_ACCESS_RIGHTS ) != 0 )
  {
    if ( sql92c_check_accesss ( pxcr ) != NO_ERROR )
      return ( ERROR_ACCESS_DENIED );
  }

  if ( pDBIsOffline != NULL )
     *pDBIsOffline = FALSE;

  strcpy ( Request.szClientObjName, pxcr->szShmName );
  Request.ulReqType       = ulRequest;
  Request.ulConsoleRecLen = pxcr->ulShmSize;

  rc = sql41c_write_queue_mailslot ( pxcr->hHandle,
                                     &Request,
                                     sizeof (CONS_REQUEST_REC),
                                     pxcr->ulShmSize,           // - OS/2 only
                                     pxcr->pConsoleRec );       // - OS/2 only

  if ( rc != NO_ERROR )
    {
    if ( rc == ERROR_HANDLE_EOF )
      MSGCD (( ERR_DB_OR_GW_NOT_STARTED, pxcr->szServerDB ));

    DBGOUT;
    return ( rc );
    }

  if (!sql92c_wait ( pxcr, &pxcr->pConsoleRec->fDataAvailable, MAX_WAIT ))
    {
    if ( pDBIsOffline != NULL )
      *pDBIsOffline = sql92c_kernel_is_alive(pxcr) == FALSE;

    DBGOUT;
    return ( 1 );
    }

  DBGOUT;
  return ( rc );
  }

/*------------------------------*/


static LONG sql92c_send_req ( PXCONS_REC              pxcr )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_send_req"
  LONG                    rc = NO_ERROR;

  DBGIN;


  switch ( pxcr->lCommand )
    {
    case COM_UKT_AFFINITY :
      memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

      pxcr->pConsoleRec->Data.UKTAffinity.ulUKTIndex = pxcr->lParam;

      rc = sql92c_send (pxcr, CON_REQ_SET_UKT_AFFINITY, NULL );

      if ( rc != NO_ERROR ) return ( rc );

      switch ( pxcr->pConsoleRec->Data.UKTAffinity.usReturnCode )
        {
        case CON_NO_ERROR        :
          break;
        case CON_WRONG_UKT :
          MSGCD (( ERR_XCONS_WRONG_UKT ));
          return ( 1 );
        case CON_ERROR          :
          MSGCD (( ERR_XCONS_SET_UKT_AFFINITY ));
          return ( 1 );
        case CON_NO_PROCESSOR   :
          MSGCD (( ERR_XCONS_AFFIN_NO_PROCESSOR ));
          return ( 1 );
        default:
          MSGCD (( ERR_UNKNOWN_RET_CODE ));
          return ( 1 );
        }

      break;

    case COM_PROCESSOR_MASK :
      memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

      pxcr->pConsoleRec->Data.SetProcessorMask.ProcessAffinityMask = pxcr->lParam;
      pxcr->pConsoleRec->Data.SetProcessorMask.TaskType            = (UINT1 )pxcr->lExtParam;

      rc = sql92c_send (pxcr, CON_REQ_SET_PROCESSOR_MASK, NULL );

      if ( rc != NO_ERROR ) return ( rc );

      switch ( pxcr->pConsoleRec->Data.SetProcessorMask.usReturnCode )
        {
        case CON_NO_ERROR        :
          break;
        case CON_ERROR          :
          MSGCD (( ERR_XCONS_SET_UKT_AFFINITY ));
          return ( 1 );
        default:
          MSGCD (( ERR_UNKNOWN_RET_CODE ));
          return ( 1 );
        }
      break;

    case COM_DEBUG_TASK:
      memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

      pxcr->pConsoleRec->Data.DebugTask.ulTask = pxcr->lParam;

      rc = sql92c_send (pxcr, CON_REQ_DEBUG_TASK, NULL );

      if ( rc != NO_ERROR ) return ( rc );

      switch ( pxcr->pConsoleRec->Data.DebugTask.usReturnCode )
        {
        case CON_NO_ERROR               :
          break;
        case CON_TASK_INACTIVE:
          MSGCD ((ERR_XCONS_TASK_INACTIVE ));
          return ( 1 );
        case CON_WRONG_TASK     :
          MSGCD ((ERR_XCONS_WRONG_TASK ));
          return ( 1 );
        default:
          MSGCD (( ERR_UNKNOWN_RET_CODE ));
          return ( 1 );
        }

      break;

    case COM_SET_LB_CHK:
      memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

      pxcr->pConsoleRec->Data.SetCheckInterval.CheckIntervalTime = pxcr->lParam;

      rc = sql92c_send (pxcr, CON_REQ_SET_CHECK_INTERVAL, NULL );

      if ( rc != NO_ERROR ) return ( rc );

      switch ( pxcr->pConsoleRec->Data.SetCheckInterval.usReturnCode )
        {
        case CON_NO_ERROR               :
          break;
        case CON_WRONG_VALUE:
          MSGCD (( ERR_XCONS_WRONG_VALUE ));
          return ( 1 );
        default:
          MSGCD (( ERR_UNKNOWN_RET_CODE ));
          return ( 1 );
        }
       break;

    case COM_DEBUG_BREAK:
      memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

      pxcr->pConsoleRec->Data.DebugBreak.ulBreakPosition = pxcr->lValue;

      if ( pxcr->lParam == PARA_CLEAR )
        pxcr->pConsoleRec->Data.DebugBreak.fClearPosition = TRUE;
      else
        pxcr->pConsoleRec->Data.DebugBreak.fClearPosition = FALSE;

      rc = sql92c_send (pxcr, CON_REQ_DEBUG_BREAK, NULL );

      if ( rc != NO_ERROR ) return ( rc );

      switch ( pxcr->pConsoleRec->Data.DebugBreak.usReturnCode )
        {
        case CON_NO_ERROR               :
          break;
        case CON_NO_INTERACT_WITH_DECTOP:
          MSGCD ((ERR_XCONS_INTERACT_WTH_DECTOP ));
          return ( 1 );
        default:
          MSGCD (( ERR_UNKNOWN_RET_CODE ));
          return ( 1 );
        }

      break;


    case COM_CANCEL:
      memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

      pxcr->pConsoleRec->Data.CancelTaskCtrl.ulTask = pxcr->lParam;

      rc = sql92c_send (pxcr, CON_REQ_CANCEL, NULL );

      if ( rc != NO_ERROR ) return ( rc );

      switch ( pxcr->pConsoleRec->Data.CancelTaskCtrl.usReturnCode )
        {
        case CON_NO_ERROR       :
          break;
        case CON_TASK_INACTIVE:
          MSGCD ((ERR_XCONS_TASK_INACTIVE ));
          return ( 1 );
        case CON_WRONG_TASK     :
          MSGCD ((ERR_XCONS_WRONG_TASK ));
          return ( 1 );
        case CON_WRONG_TASK_TYPE:
          MSGCD ((ERR_XCONS_WRONG_TASK_TYPE ));
          return ( 1 );
        default:
          MSGCD (( ERR_UNKNOWN_RET_CODE ));
          return ( 1 );
        }

      break;

    case COM_KILL:
      memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

      pxcr->pConsoleRec->Data.KillTaskCtrl.ulTask = pxcr->lParam;

      rc = sql92c_send (pxcr, CON_REQ_KILL, NULL );

      if ( rc != NO_ERROR ) return ( rc );

      switch ( pxcr->pConsoleRec->Data.KillTaskCtrl.usReturnCode )
        {
        case CON_NO_ERROR       :
          break;
        case CON_TASK_INACTIVE:
          MSGCD ((ERR_XCONS_TASK_INACTIVE ));
          return ( 1 );
        case CON_WRONG_TASK     :
          MSGCD ((ERR_XCONS_WRONG_TASK ));
          return ( 1 );
        case CON_WRONG_TASK_TYPE:
          MSGCD ((ERR_XCONS_WRONG_TASK_TYPE ));
          return ( 1 );
        default:
          MSGCD (( ERR_UNKNOWN_RET_CODE ));
          return ( 1 );
        }

      break;

    case COM_CRASH_DUMP:
      memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

      rc = sql92c_send (pxcr, CON_REQ_CRASH_DUMP, NULL );

      if ( rc != NO_ERROR ) return ( rc );

      switch ( pxcr->pConsoleRec->Data.CrashDump.usReturnCode )
        {
        case CON_NO_ERROR :
          break;

        default:
          MSGCD (( ERR_UNKNOWN_RET_CODE ));
          return ( 1 );
        }

      break;

    case COM_DROP:


      break;

    case COM_GETPARAM:
       memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

       rc = sql92c_send (pxcr, CON_REQ_GETPARAM, NULL );

       if ( rc != NO_ERROR )
         return ( rc );

       rc = sql92c_show_param ( pxcr->pszParam, pxcr->pConsoleRec );

       if ( rc != NO_ERROR )
         return ( rc );

       break;


     case COM_PUTPARAM      :
       rc = sql92c_send (pxcr, CON_REQ_PUTPARAM, NULL );
       break;

 
     case COM_ERRIOR:
     case COM_ERRIOW:
       memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

       pxcr->pConsoleRec->Data.ForceIOError.ulDevno    = pxcr->lParam ;
       pxcr->pConsoleRec->Data.ForceIOError.fErrOnRead = pxcr->lCommand == COM_ERRIOR ;

       rc = sql92c_send (pxcr, CON_REQ_FORCE_IO_ERROR, NULL );
       break ;

     case COM_SET_DEBUGLEVEL:
       memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );
       pxcr->pConsoleRec->Data.SetDebugLevelCtrl.ulDebugLevel = pxcr->lParam ;

       rc = sql92c_send (pxcr, CON_REQ_SET_DEBUGLEVEL, NULL );
       break;

    case COM_TIME_MEASURE  :
      memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );
      if ( pxcr->lParam & PARA_ENABLE )
        {
        rc = sql92c_send (pxcr, CON_REQ_ENABLE_TIME_MEASURE, NULL );
        }
      if ( pxcr->lParam & PARA_DISABLE )
        {
        rc = sql92c_send (pxcr, CON_REQ_DISABLE_TIME_MEASURE, NULL );
        }
      break;

    case COM_RESET:
      memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );
      if ( (pxcr->lParam & PARA_ALL) == PARA_ALL )
        rc = sql92c_send (pxcr, CON_REQ_RES_ALL, NULL );
      else if ( pxcr->lParam & PARA_IO )
        rc = sql92c_send (pxcr, CON_REQ_RES_IO, NULL );
      else if ( pxcr->lParam & PARA_REGIONS )
        rc = sql92c_send (pxcr, CON_REQ_RES_REGION, NULL );
      else if ( pxcr->lParam & PARA_QUEUE_STAT )
        rc = sql92c_send (pxcr, CON_REQ_RES_QUEUE_STAT, NULL );
      else if ( pxcr->lParam & PARA_TASK_COUNTS )
        rc = sql92c_send (pxcr, CON_REQ_RES_TASK_COUNTS, NULL );
      else if ( pxcr->lParam & PARA_UKT_SL_STAT )
        rc = sql92c_send (pxcr, CON_REQ_RES_UKT_SLEEPTIME_STAT, NULL );
      else if ( pxcr->lParam & PARA_SUSPENDS    )
        rc = sql92c_send (pxcr, CON_REQ_RES_SUSPENDS, NULL );
      break;

    case COM_SHOW_UPDATE:

      printf ( XCONS_SERVERDB, pxcr->szServerDB );
      
      while ( pxcr->lRepeatCount )
        {
        if ( pxcr->lParam & PARA_VERSIONS )
          {
          rc = sql92c_show_version_info ( pxcr );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_IO )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_IO, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_io_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_AIO )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_AIO, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_aio_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_REGIONS )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_REGION, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_region_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_RTE )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_RTE, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_rte_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_STORAGE )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_STORAGE, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_storage_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_TASKS     ||
             pxcr->lParam & PARA_ACTIVE    ||
             pxcr->lParam & PARA_TASK_MOVE ||
             pxcr->lParam & PARA_RUNNABLE)
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          if ( pxcr->lParam & PARA_TASKS )
            pxcr->pConsoleRec->Data.TaskInfo.StateToShow = SHOW_ALL_EO92;
          else if ( pxcr->lParam & PARA_ACTIVE )
            pxcr->pConsoleRec->Data.TaskInfo.StateToShow = SHOW_ACTIVE_EO92;
          else if ( pxcr->lParam & PARA_TASK_MOVE )
            pxcr->pConsoleRec->Data.TaskInfo.StateToShow = SHOW_ALL_EO92;
          else
            pxcr->pConsoleRec->Data.TaskInfo.StateToShow = SHOW_RUNNABLE_EO92;

          /* PTS 1106166 */
          pxcr->pConsoleRec->Data.TaskInfo.TypeToShow  = 
                pxcr->lParam == PARA_ALL ? SHOW_TT_ALL_EO92 : (UINT1 )pxcr->lExtParam;


          if ( pxcr->lParam & PARA_TASK_MOVE )
          {
              rc = sql92c_send (pxcr, CON_REQ_TASK, NULL );

              if ( rc != NO_ERROR ) return ( rc );

              rc = sql92c_show_task_info ( pxcr->pConsoleRec, true);

              if ( rc != NO_ERROR ) return ( rc );
          }

          if (pxcr->lParam != PARA_TASK_MOVE)
          {
              rc = sql92c_send (pxcr, CON_REQ_TASK, NULL );

              if ( rc != NO_ERROR ) return ( rc );

              rc = sql92c_show_task_info ( pxcr->pConsoleRec, false);

              if ( rc != NO_ERROR ) return ( rc );
          }
        }
        
        if ( pxcr->lParam & PARA_UKT_SL_STAT )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_UKT_SLEEPTIME_STAT, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_ukt_sl_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_THREAD_TIMES )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_THREAD_TIMES, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_thread_times ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_SUSPENDS )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_SHOW_SUSPENDS, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_suspends ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_UKT_QUEUES )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_UKT_QUEUES, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_ukt_queues ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

         if ( pxcr->lParam & PARA_STATE )
           {
           memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

           rc = sql92c_send ( pxcr, CON_REQ_STATE, NULL );

           if ( rc != NO_ERROR ) return ( rc );

           rc = sql92c_show_state_info ( pxcr->pConsoleRec );

           if ( rc != NO_ERROR ) return ( rc );
           }

        if ( pxcr->lParam & PARA_QUEUE_STAT )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_QUEUE_STAT, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_queue_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_TASK_COUNTS )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );
          pxcr->pConsoleRec->Data.TaskCountsInfo.lNumOfActiveTasks = pxcr->lExtParam;

          rc = sql92c_send (pxcr, CON_REQ_TASK_COUNTS, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_task_count_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_TOTAL_COUNTS )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_TOTAL_COUNTS, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_total_count_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_AWE )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_AWE, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_awe_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_MOVE_INFO )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_TASK_MOVE, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_task_move_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if ( pxcr->lParam & PARA_CONNECTIONS )
          {
          memset ( pxcr->pConsoleRec, 0, pxcr->ulShmSize );

          rc = sql92c_send (pxcr, CON_REQ_CONNECTIONS, NULL );

          if ( rc != NO_ERROR ) return ( rc );

          rc = sql92c_show_connections_info ( pxcr->pConsoleRec );

          if ( rc != NO_ERROR ) return ( rc );
          }

        if (pxcr->lRepeatCount != -1 ) // --- don't loop forever?
          pxcr->lRepeatCount--;

        if (( pxcr->lInterval ) && ( pxcr->lRepeatCount ))
          SLEEP( pxcr->lInterval * 1000 );
        }

      break;

    default:
      return ( 1 );
    }


  DBGOUT;
  return ( rc );
  }

/*------------------------------*/

static LONG sql92c_check_accesss ( PXCONS_REC  pxcr )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_check_accesss"
  LONG              rc           = NO_ERROR;
  #if defined (_WIN32)
   HEV              hevSemCoord  = 0;
  #endif

  DBGIN;

  #if defined (_WIN32)
   //
   // --- try to open coordinators semaphore to check access rights
   //
   rc = sql41c_open_event_sem ( &hevSemCoord, SEM_COORD, pxcr->szServerDB,
                                ERROR_ACCESS_DENIED);

   if (rc == NO_ERROR )
     sql41c_close_event_sem ( hevSemCoord, "COORD" );
   else if (rc == ERROR_ACCESS_DENIED)
     MSGCD(( ERR_ACCESS_DENIED ))
   else if (rc == ERROR_SEM_NOT_FOUND)
     MSGCD ((ERR_DB_OR_GW_NOT_STARTED, pxcr->szServerDB))
  #endif

  DBGOUT;
  return ( rc );
  }

/*------------------------------*/

static LONG sql92c_get_num_param  ( PSZ      pszParam,
                                    PLONG    plNum  )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_get_num_param"
  PSZ      pszTmp;
  DBGIN;

  // --- is the parameter valid?
  pszTmp = pszParam;
  while ( *pszTmp != '\0')
    {
    if (!isdigit ( *pszTmp ))
      return ( 1 );

    pszTmp++;
    }

  *plNum = atol (pszParam);

  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_get_idx_param  ( PSZ      pszParam,
                                    CHAR     cPrefix,
                                    PLONG    plNum  )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_get_idx_param"
  PSZ      pszTmp;
  PSZ      pszParamTmp;
  DBGIN;

  // --- is the parameter valid?
  if ( *pszParam == _toupper(cPrefix) || *pszParam == cPrefix )
    {
    pszTmp      = pszParam + 1;
    pszParamTmp = pszParam + 1;
    }
  else
    {
    pszTmp      = pszParam;
    pszParamTmp = pszParam;
    }

  while ( *pszTmp != '\0')
    {
    if (!isdigit ( *pszTmp ))
      return ( 1 );

    pszTmp++;
    }

  *plNum = atol (pszParamTmp);

  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_GetProcessorAffinityMask ( PSZ      pszParam,
                                              PLONG    ProcessAffinityMask  )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_GetProcessorAffinityMask"
  LONG     rc = NO_ERROR;
  int      i, ParamLen ;

  DBGIN;

  for ( i = ParamLen = (int)strlen(pszParam), *ProcessAffinityMask = 0 ; 
        i > 0 && rc == NO_ERROR; i-- )
	if  ( pszParam[i-1] == '1' ) 
      *ProcessAffinityMask |= ( 1 << ( ParamLen-i) ) ;
	else
	  if ( pszParam[i-1] != '0' )
        rc = !NO_ERROR ;

  return ( rc );
  }

/*------------------------------*/

static LONG sql92c_get_time_param  ( PSZ pszParam )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_get_time_param"
  DBGIN;

  strupr (pszParam);

  if ( e92_strcont( XCONS_CMD_ENABLE, pszParam,
                       XCONS_MIN_CMD_LEN_ENABLE ))
   return( PARA_ENABLE );

  if ( e92_strcont( XCONS_CMD_DISABLE,  pszParam,
                       XCONS_MIN_CMD_LEN_DISABLE ))
   return( PARA_DISABLE );

  return ( PARA_NONE );
  }

/*------------------------------*/

static LONG sql92c_get_set_clear_param  ( PSZ    pszParam )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_get_set_clear_param"
  DBGIN;

  strupr (pszParam);

  if ( e92_strcont( XCONS_CMD_SET, pszParam,
                    XCONS_MIN_CMD_LEN_SET ))

   return( PARA_SET );

  if ( e92_strcont( XCONS_CMD_CLEAR, pszParam,
                    XCONS_MIN_CMD_LEN_CLEAR ))
   return( PARA_CLEAR );


  return ( PARA_NONE );
  }

/*------------------------------*/

static LONG sql92c_get_show_res_param  ( PSZ    pszParam,
                                         LONG   lCommand )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_get_show_res_param"
  DBGIN;

  strupr (pszParam);

  if ( e92_strcont( XCONS_CMD_IO, pszParam,
                       XCONS_MIN_CMD_LEN_IO ))
   return( PARA_IO );

  if ( e92_strcont( XCONS_CMD_AIO, pszParam,
                       XCONS_MIN_CMD_LEN_AIO ))
   return( PARA_AIO );

  if ( e92_strcont( XCONS_CMD_MOVE_INFO, pszParam,
                       XCONS_MIN_CMD_LEN_MOVE_INFO ))
   return( PARA_MOVE_INFO );

  if ( e92_strcont( XCONS_CMD_TASK_MOVE, pszParam,
                       XCONS_MIN_CMD_LEN_TASK_MOVE ))
   return( PARA_TASK_MOVE );

  if ( e92_strcont( XCONS_CMD_AWE, pszParam,
                       XCONS_MIN_CMD_LEN_AWE ))
   return( PARA_AWE );

  if ( e92_strcont( XCONS_CMD_REGIONS, pszParam,
                       XCONS_MIN_CMD_LEN_REGIONS  ) )
   return( PARA_REGIONS );

  if ( e92_strcont( XCONS_CMD_UKT_SLEEP_STAT, pszParam,
                       XCONS_MIN_CMD_LEN_UKT_SLEEP_STAT ) )
   return( PARA_UKT_SL_STAT );

  if ( e92_strcont( XCONS_CMD_ALL, pszParam,
                       XCONS_MIN_CMD_LEN_ALL ))
   return( PARA_ALL );


  if ( e92_strcont( XCONS_CMD_THREAD_TIMES, pszParam,
                       XCONS_MIN_CMD_LEN_THREAD_TIMES ))
   return( PARA_THREAD_TIMES );

  if ( e92_strcont( XCONS_CMD_UKT_QUEUES, pszParam,
                       XCONS_MIN_CMD_LEN_UKT_QUEUES ) )
    return( PARA_UKT_QUEUES );

  if ( e92_strcont( XCONS_CMD_SUSPENDS, pszParam,
                        XCONS_MIN_CMD_LEN_SUSPENDS ) )
    return( PARA_SUSPENDS );

  if ( e92_strcont( XCONS_CMD_TASK_COUNTS, pszParam,
                        XCONS_MIN_CMD_LEN_TASK_COUNTS ) )
    return( PARA_TASK_COUNTS );

  if ( e92_strcont( XCONS_CMD_QUEUE_STAT, pszParam,
                        XCONS_MIN_CMD_LEN_QUEUE_STAT ))
    return( PARA_QUEUE_STAT );


 if ( lCommand == COM_SHOW_UPDATE )
   {
   if ( e92_strcont( XCONS_CMD_VERSIONS, pszParam,
                        XCONS_MIN_CMD_LEN_VERSIONS ))
   return( PARA_VERSIONS );

   if ( e92_strcont( XCONS_CMD_STORAGE,  pszParam,
                        XCONS_MIN_CMD_LEN_STORAGE ))
   return( PARA_STORAGE );

   if ( e92_strcont( XCONS_CMD_RTE, pszParam,
                        XCONS_MIN_CMD_LEN_RTE ))
    return( PARA_RTE );

   if ( e92_strcont( XCONS_CMD_TASKS, pszParam,
                        XCONS_MIN_CMD_LEN_TASKS ))
    return( PARA_TASKS );

   if ( e92_strcont( XCONS_CMD_ACTIVE_TASKS, pszParam,
                        XCONS_MIN_CMD_LEN_ACTIVE_TASKS ))
    return( PARA_ACTIVE );

   if ( e92_strcont( XCONS_CMD_RUNNABLE_TASKS, pszParam,
                        XCONS_MIN_CMD_LEN_RUNNABLE_TASKS ))
    return( PARA_RUNNABLE );

    
   if ( e92_strcont( XCONS_CMD_STATE, pszParam,
                          XCONS_MIN_CMD_LEN_STATE ))
    return( PARA_STATE );

   if ( e92_strcont( XCONS_CMD_TASK_STAT, pszParam,
                          XCONS_MIN_CMD_LEN_TASK_STAT ))
    return( PARA_TASK_STAT );

   if ( e92_strcont( XCONS_CMD_TOTAL_COUNTS, pszParam,
                          XCONS_MIN_CMD_LEN_TOTAL_COUNTS ))
    return( PARA_TOTAL_COUNTS );
    
   if ( e92_strcont( XCONS_CMD_CONNECTIONS, pszParam,
                          XCONS_MIN_CMD_LEN_CONNECTIONS ))
    return( PARA_CONNECTIONS );
    
    }

  return ( PARA_NONE );
  }

/*------------------------------*/

static LONG sql92c_get_com_param  ( PSZ    pszParam )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_get_com_param"
  DBGIN;

  strupr (pszParam);


  if ( e92_strcont( XCONS_CMD_HELP1, pszParam,
                       XCONS_MIN_CMD_LEN_HELP1 ))
   return( COM_HELP );

  if ( e92_strcont( XCONS_CMD_HELP2, pszParam,
                       XCONS_MIN_CMD_LEN_HELP2 ))
   return( COM_HELP );

   if ( e92_strcont( XCONS_CMD_ERRIOR, pszParam,
                        XCONS_MIN_CMD_LEN_ERRIO ))
     return( COM_ERRIOR );

   if ( e92_strcont( XCONS_CMD_ERRIOW, pszParam,
                        XCONS_MIN_CMD_LEN_ERRIO ))
     return( COM_ERRIOW );

   if ( e92_strcont( XCONS_CMD_CANCEL, pszParam,
                        XCONS_MIN_CMD_LEN_CANCEL ))
    return( COM_CANCEL );

   if ( e92_strcont( XCONS_CMD_DEBUG_TASK, pszParam,
                        XCONS_MIN_CMD_LEN_DEBUG_TASK ))
    return( COM_DEBUG_TASK );

   if ( e92_strcont( XCONS_CMD_SET_LB_CHK, pszParam,
                        XCONS_MIN_CMD_SET_LB_CHK_TASK ))
    return( COM_SET_LB_CHK );

   if ( e92_strcont( XCONS_CMD_DEBUG_BREAK, pszParam,
                        XCONS_MIN_CMD_LEN_DEBUG_BREAK ))
    return( COM_DEBUG_BREAK );

   if ( e92_strcont( XCONS_CMD_UKT_AFFINITY, pszParam,
                        XCONS_MIN_CMD_LEN_UKT_AFFINITY ))
    return( COM_UKT_AFFINITY );

   // PTS 1107070
   if ( e92_strcont( XCONS_CMD_PROCESSOR_MASK, pszParam,
                        XCONS_MIN_CMD_LEN_PROCESSOR_MASK ))
    return( COM_PROCESSOR_MASK );

   if ( e92_strcont( XCONS_CMD_KILL, pszParam,
                        XCONS_MIN_CMD_LEN_KILL ))
    return( COM_KILL );

   if ( e92_strcont( XCONS_CMD_SHOW, pszParam,
                        XCONS_MIN_CMD_LEN_SHOW ))
     return( COM_SHOW_UPDATE );

   if ( e92_strcont( XCONS_CMD_TIME_MEASURE,  pszParam,
                        XCONS_MIN_CMD_LEN_TIME_MEASURE ))
     return( COM_TIME_MEASURE );

   if ( e92_strcont( XCONS_CMD_RESET_CNT, pszParam,
                        XCONS_MIN_CMD_LEN_RESET_CNT ))
     return( COM_RESET );

   if ( e92_strcont( XCONS_CMD_GETPARAM, pszParam,
                        XCONS_MIN_CMD_LEN_GETPARAM ))
     return( COM_GETPARAM );

   if ( e92_strcont( XCONS_CMD_PUTPARAM , pszParam,
                        XCONS_MIN_CMD_LEN_PUTPARAM ))
     return( COM_PUTPARAM );

   if ( e92_strcont( XCONS_CMD_DEBUGLEVEL , pszParam,
                        XCONS_MIN_CMD_LEN_DEBUGLEVEL ))
     return( COM_SET_DEBUGLEVEL );

   if ( sql02_get_platform_id () == VER_PLATFORM_WIN32_NT )
     {
     if ( e92_strcont( XCONS_CMD_CRASH, pszParam,
                          XCONS_MIN_CMD_LEN_CRASH ))
       return( COM_CRASH_DUMP );
     }
  
  return ( COM_UNDEF );
  }

/*------------------------------*/

static BOOL  sql92c_wait ( PXCONS_REC           pxcr,
                           PBOOL                pfFlag,
                           ULONG                ulSecWaitTime )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_wait"
  ULONG ulTime   = ulSecWaitTime * 10;
  bool  fIsAlive = true;

  DBGPAS;

  while ( *pfFlag == FALSE && --ulTime && fIsAlive )
  {
    SLEEP( 100 );

    if ( ulTime % 10 == 0 ) // one check per second
      fIsAlive = sql92c_kernel_is_alive(pxcr);
  }

  if (ulTime == 0)
    MSGCD (( ERR_COMMAND_TIMEOUT ));

  return (ulTime != 0 && fIsAlive);
  }

/*------------------------------*/

static LONG sql92c_show_version_info ( PXCONS_REC pxcr )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_version_info"

  DBGPAS;

   printf ( XCONS_SHOW_VERS_KERNEL, pxcr->KernelVersion );
   printf ( XCONS_SHOW_VERS_RTE   , pxcr->RTEVersion    );

  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_show_task_info ( PCONSOLE_REC  pConsoleRec, SAPDB_Bool fTaskMoveInfo )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_task_info"
  INT                         i;
  PCONS_SINGLE_TASK_INFO_REC  pTaskInfos;
   CHAR                       szWaitingIndex[20];
   CHAR                       szWaitingItem[20];
   CHAR                       szApplPid[14];
   CHAR                       szTimeOut[20];
   CHAR                       szTid[12];
   ULONG                      ulUnconnectedTasks   = 0;
   ULONG                      ulUnusedServerTasks  = 0;
   ULONG                      ulTimeOut            = 0;
   static  BOOL               FirstCall = TRUE ;

  DBGPAS;

  pTaskInfos = (PCONS_SINGLE_TASK_INFO_REC)
                   ((PCHAR)&pConsoleRec->Data +
                           pConsoleRec->Data.TaskInfo.ulTaskInfoOffset);

   if ( FirstCall || SHOW_TT_ALL_EO92 == pConsoleRec->Data.TaskInfo.TypeToShow)
     {
       if ( fTaskMoveInfo )
         printf ( XCONS_SHOW_TASK_MOVE_HEADER );
       else
         printf ( XCONS_SHOW_TASK_HEADER );

       FirstCall = FALSE ;
     }
   else
      printf ("\n" ) ;

   if ( pConsoleRec->Data.TaskInfo.lNumOfActiveTasks == 0 )
     return ( NO_ERROR );

   for ( i = 0; i < pConsoleRec->Data.TaskInfo.lNumOfActiveTasks; i++)
     {
     if ( pTaskInfos[i].ApplPid != (PID)UNDEF)
       {
       _ultoa ( pTaskInfos[i].ApplPid,
                szApplPid, 10 );
       if ( pTaskInfos[i].szClientNode[0] != '\0' )
           {
           strcat(szApplPid, "*");
           }
       }
     else
      szApplPid[0] = '\0';

     /* PTS 1115172 */
     szWaitingIndex[0] = '\0';
     if ( pTaskInfos[i].TaskType == TT_US_EO00 
         && ( pTaskInfos[i].TaskState == TSK_VSUSPEND_EO00 || pTaskInfos[i].TaskState == TSK_VWAIT_EO00 ) )
     {
         if ( 0 < pTaskInfos[i].lWaitForTask )
         {
             _ultoa ( pTaskInfos[i].lWaitForTask, szWaitingIndex, 10 );
         }
     }
     else if ( pTaskInfos[i].TaskState == TSK_IO_READ_EO00 || pTaskInfos[i].TaskState == TSK_IO_WRITE_EO00 ) 
     {
         _ultoa ( pTaskInfos[i].lWaitForDevno, szWaitingIndex, 10 );
     }
     else
     {
         if ( 0 < pTaskInfos[i].Sid )
         {
           _ultoa ( pTaskInfos[i].Sid, szWaitingIndex, 10 );
         }
     }

     szWaitingItem[0] = 0;
     if ( pTaskInfos[i].TaskType == TT_US_EO00 && pTaskInfos[i].TaskState == TSK_VSUSPEND_EO00) /* PTS 1115172 */
     {
        if ( pTaskInfos[i].LockedPageNo >= 0 && pTaskInfos[i].LockedPageNo != MAX_INT4_SP00 )
        {
            _ultoa( pTaskInfos[i].LockedPageNo, szWaitingItem, 10 );
        }
        else if ( pTaskInfos[i].lWaitForRoot >= 0 && pTaskInfos[i].lWaitForRoot != MAX_INT4_SP00 )
        {
            _ultoa( pTaskInfos[i].lWaitForRoot, szWaitingItem, 10 );
        }
     }
     else
     {
         if ( pTaskInfos[i].WaitTime > 0 )
         {
             _ultoa( pTaskInfos[i].WaitTime, szWaitingItem, 10 );
         }
     }

     if ( pTaskInfos[i].TaskState == TSK_VRECEIVE_EO00 )
       {
       if ( pTaskInfos[i].ulTaskTimeOut == (tsp00_Longuint) UNDEF )
         strncpy (szTimeOut, XCONS_SHOW_TASK_NO_TIMEOUT, 10);
       else if (pTaskInfos[i].ulTaskTimeOut < (tsp00_Longuint)time(NULL))
         ulTimeOut = 0;
       else
         ulTimeOut = (ULONG)(pTaskInfos[i].ulTaskTimeOut - time (NULL));

       if ( pTaskInfos[i].ulTaskTimeOut != (tsp00_Longuint) UNDEF )
         _ultoa ( ulTimeOut, szTimeOut, 10 );
       else
         // strncpy (szTimeOut, XCONS_SHOW_TASK_NO_TIMEOUT, 10);

       szTimeOut[10] = '\0';
       }
     else
     {
         _ultoa ( pTaskInfos[i].ulDynPrio, szTimeOut, 10 );
     }

     if (( pTaskInfos[i].TaskState == TSK_INITIALIZED_EO00         ||
           pTaskInfos[i].TaskState == TSK_INACTIVE_EO00            ||
           pTaskInfos[i].TaskState == TSK_MARKED_FOR_STARTUP_EO00  ||
           pTaskInfos[i].TaskState == TSK_CONNECTWAIT_EO00       ) &&
           pTaskInfos[i].TaskType  != TT_UT_EO00 )
       {
       if ( pTaskInfos[i].TaskType == TT_SV_EO00 )
         ulUnusedServerTasks++;
       else
         ulUnconnectedTasks++;
       }
     else
       {
       szTid[0] = '0';
       szTid[1] = 'x';
       _ultoa ( pTaskInfos[i].Tid, szTid + 2, 16 );
       strupr ( szTid + 2 );

       if ( fTaskMoveInfo )
         {
         CHAR   szLock[12];

         if( pTaskInfos[i].LongTermMoveLock )
         {
           strcpy ( szLock, "LT:" );
           _ultoa ( pTaskInfos[i].LongTermMoveLock, szLock + 3, 10 );
         }
         else
           _ultoa ( pTaskInfos[i].TemporaryMoveLock, szLock, 10 );

         printf ( XCONS_SHOW_TASK_MOVE_INFO,
                  pTaskInfos[i].ulTaskIndex,
                  pTaskInfos[i].ulUktIndex,
                  eo92GetTaskTypeName ( pTaskInfos[i].TaskType ),
                  pTaskInfos[i].QueLocation == TASK_IS_IN_RUN_QUEUE_EO00 ? '*' : ' ' ,
                  eo92GetTaskStateName (pTaskInfos[i].TaskState,pTaskInfos[i].ulStateReason),
                  !pTaskInfos[i].OmsCallbackActive ? ' ' : '&',
                   pTaskInfos[i].MaxRunnableMicroSeconds / 1000000,
                   pTaskInfos[i].MaxRunnableMicroSeconds % 1000000,
                   pTaskInfos[i].RunnableMicroSeconds / 1000000,
                   pTaskInfos[i].RunnableMicroSeconds % 1000000,
                   szLock );
         }
       else
         {
         printf ( XCONS_SHOW_TASK_INFO,
                  pTaskInfos[i].ulTaskIndex,
                  pTaskInfos[i].ulUktIndex,
                  szTid,
                  eo92GetTaskTypeName ( pTaskInfos[i].TaskType ),
                  szApplPid,
                  pTaskInfos[i].QueLocation == TASK_IS_IN_RUN_QUEUE_EO00 ? '*' : ' ' ,
                  eo92GetTaskStateName (pTaskInfos[i].TaskState,pTaskInfos[i].ulStateReason) ,
                  ! pTaskInfos[i].OmsCallbackActive ? ' ' : '&', /* CR 1106969 */
                  ! pTaskInfos[i].CancelFlag ? ' ' : '!',        /* CR 1103476 */
                  ! pTaskInfos[i].PrioFlag   ? ' ' :
                           '0' + pTaskInfos[i].PrioFlag
                               + pTaskInfos[i].SavePrioFlag ,
                  szTimeOut,
                  pTaskInfos[i].ulBegExclCnt,
                  szWaitingIndex,
                  szWaitingItem,
                  pTaskInfos[i].UktSleepCount,
                  pTaskInfos[i].UktSleepingFlag ? 's' : 'r' );
         }
       }
     }

   if ( ulUnusedServerTasks )
     {
     printf ( XCONS_SHOW_SERVER_UNUSED, ulUnusedServerTasks );
     }

   if ( ulUnconnectedTasks )
     {
     printf ( XCONS_SHOW_TASK_CONN_INFO,
              ulUnconnectedTasks, XCONS_TSK_STATE_CONNECTWAIT );
     }

  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_show_io_info ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_io_info"
  INT                i;
  ULONG              ulDevspaceNo;
  PCONS_DEV_INFO_REC pDevspaceInfos;

   PATHNAME          szPathName;
   ULONG             ulReadCnt  = 0;
   ULONG             ulWriteCnt = 0;
   ULONG             ulTotal    = 0;

  DBGPAS;

  pDevspaceInfos = (PCONS_DEV_INFO_REC)
                     ((PCHAR)&pConsoleRec->Data +
                             pConsoleRec->Data.IOInfo.ulDevspaceInfoOffset);

   printf ( XCONS_SHOW_IO_HEADER );

  for ( i = 0; i < pConsoleRec->Data.IOInfo.lNumOfDevspaces; i++)
    {

    ulDevspaceNo = pDevspaceInfos[ i ].ulDevspaceNo;

     // --- The compress value (44) must be identical to the value
     //     defined in 'XCONS_SHOW_IO_INFO'!!!!!!!!!!!
     eo92_compress_path( szPathName, pDevspaceInfos[ i ].DevspaceName, 44 );

     printf ( XCONS_SHOW_IO_INFO,
              szPathName,
              ulDevspaceNo,
              pDevspaceInfos[ i ].ulReadCnt,
              pDevspaceInfos[ i ].ulWriteCnt,
              pDevspaceInfos[ i ].ulReadCnt +
              pDevspaceInfos[ i ].ulWriteCnt );

     ulReadCnt  += pDevspaceInfos[ i ].ulReadCnt;
     ulWriteCnt += pDevspaceInfos[ i ].ulWriteCnt;
     ulTotal    += pDevspaceInfos[ i ].ulReadCnt +
                   pDevspaceInfos[ i ].ulWriteCnt;

    }

   printf ( XCONS_SHOW_IO_INFO_TOTAL,
            XCONS_SHOW_IO_INFO_TOTAL_STR,
            ulReadCnt,
            ulWriteCnt,
            ulTotal );
  
  return ( NO_ERROR );
  }


/*------------------------------*/

static LONG sql92c_show_aio_info ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_aio_info"

  INT                        i;
  PCONS_ASYNCI_INFO_REC      pASYNCiInfos;
  CHAR                       szTid[12];
  PATHNAME                   szPathName;
  CHAR                       szMode[2];

  pASYNCiInfos    = (PCONS_ASYNCI_INFO_REC)
                      ((PCHAR)&pConsoleRec->Data +
                          pConsoleRec->Data.AIOInfo.ulASYNCiInfoOffset);


  printf ( XCONS_SHOW_ASYNCIO_HEADER );

  if ( pConsoleRec->Data.AIOInfo.lNumOfASYNCis )
   {
   for ( i = 0; i < pConsoleRec->Data.AIOInfo.lNumOfASYNCis; i++ )
     {
     // --- The compress value (20) must be identical to the value
     //     defined in 'XCONS_SHOW_ASYNCIO_DEV_INFO'!!!!!!!!!!!
     eo92_compress_path ( szPathName,
                          pASYNCiInfos[i].szLogDeviceName,
                          20 );

     szTid[0] = '0';
     szTid[1] = 'x';
     _ultoa ( pASYNCiInfos[i].Tid, szTid + 2, 16 );
     strupr ( szTid + 2 );

     if ( pASYNCiInfos[i].fForWriting )
       {
       szMode[0] = 'W';
       szMode[1] = '\0';
       }
     else
       {
       szMode[0] = 'R';
       szMode[1] = '\0';
       }

     printf ( XCONS_SHOW_ASYNCIO_INFO,
              szPathName,
              szTid,
              szMode,
              pASYNCiInfos[i].ulMaxBlockCnt,
              pASYNCiInfos[i].ulBlockSize,
              pASYNCiInfos[i].DevIOStat.DevRead.IOStat.ulCount,
              pASYNCiInfos[i].DevIOStat.DevWrite.IOStat.ulCount,
              pASYNCiInfos[i].DevIOStat.ulIOQueLen,
              pASYNCiInfos[i].DevIOStat.ulMaxIOQueLen );
     }
  }

  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_show_region_info ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_region_info"
  INT                        i;
  INT                        j;
  PCONS_SINGLE_SEM_INFO_REC  pSingleSemInfos;
  PULONG                     pulWaitTasks;
  float                      flPercent;
  CHAR                      szOwnerTaskIndex[20];

  DBGPAS;

  pSingleSemInfos = (PCONS_SINGLE_SEM_INFO_REC)
                        ((PCHAR)&pConsoleRec->Data +
                                pConsoleRec->Data.SemInfo.ulSingleSemInfoOffset);

  printf ( XCONS_SHOW_REGION_HEADER );

  for ( i = 0; i < pConsoleRec->Data.SemInfo.lNumOfSems; i ++ )
    {
    pulWaitTasks = (PULONG)((PCHAR)&pConsoleRec->Data +
                                pSingleSemInfos[i].ulWaitTaskOffset);

    if (( ! pSingleSemInfos[i].lGetCount ) ||
        ( ! pSingleSemInfos[i].lCollisionCount ))
      flPercent = 0.0f;
    else
      flPercent = (float) pSingleSemInfos[i].lCollisionCount * 100.0f /
                  (float) pSingleSemInfos[i].lGetCount;


     if (pSingleSemInfos[i].ulOwnerTaskIndex)
       {
       szOwnerTaskIndex[0] = 'T';
       _ltoa ( pSingleSemInfos[i].ulOwnerTaskIndex, szOwnerTaskIndex + 1, 10 );
       }
     else
       szOwnerTaskIndex[0] = '\0';

     printf ( XCONS_SHOW_REGION_INFO,
              pSingleSemInfos[i].ulSid,
              pSingleSemInfos[i].szSemaName,
              szOwnerTaskIndex,
              pSingleSemInfos[i].lGetCount,
              pSingleSemInfos[i].lBegExclTasCount +
              pSingleSemInfos[i].lEndExclTasCount,
              pSingleSemInfos[i].lCollisionCount,
              pSingleSemInfos[i].lWaitCount,
              pSingleSemInfos[i].lExclusive,
              flPercent );

     if ( pSingleSemInfos[i].lNumWaitTasks)
       {
       printf ( XCONS_SHOW_REGION_WAIT_HD );

       for ( j = 0; j < (INT)pSingleSemInfos[i].lNumWaitTasks; j++ )
         {
         if ( !(j % TASKS_INFO_P_LINE) && j)
           printf ( XCONS_SHOW_REGION_WAIT_ILF,
                    pulWaitTasks[j] );
         else
           printf ( XCONS_SHOW_REGION_WAIT_I,
                    pulWaitTasks[j] );
         }
       printf ("\n");
       }

    }

  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_show_storage_info ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_storage_info"
  PCONS_TASK_STORAGE_INFO_REC pTaskStorInfos;
  INT                        i;
  DBGPAS;

  pTaskStorInfos = (PCONS_TASK_STORAGE_INFO_REC)
                       ((PCHAR)&pConsoleRec->Data +
                               pConsoleRec->Data.StorageInfo.ulTaskStorInfoOffset);
/*
   printf ( XCONS_SHOW_STOR_DYN_POOL,
            (pConsoleRec->Data.StorageInfo.lDynPoolUsed +
             pConsoleRec->Data.StorageInfo.lDynPoolAvail  )      / 1024,
            pConsoleRec->Data.StorageInfo.lDynPoolUsed           / 1024,
            pConsoleRec->Data.StorageInfo.lDynPoolAvail          / 1024 );
   printf ( XCONS_SHOW_STOR_DYN_DATA,
            (pConsoleRec->Data.StorageInfo.lDynDatUsed  +
             pConsoleRec->Data.StorageInfo.lDynDatAvail   )      / 1024,
            pConsoleRec->Data.StorageInfo.lDynDatUsed            / 1024,
            pConsoleRec->Data.StorageInfo.lDynDatAvail           / 1024 );
*/
   printf ( XCONS_SHOW_STOR_ALLOC_STCK,
            pConsoleRec->Data.StorageInfo.lTaskStacks      / 1024 );
   printf ( XCONS_SHOW_STOR_KGS,
            pConsoleRec->Data.StorageInfo.ulKGSSize        / 1024 );
   printf ( XCONS_SHOW_STOR_KSS,
            pConsoleRec->Data.StorageInfo.ulKSSSize        / 1024 );

   printf ( XCONS_SHOW_STOR_TSK_HEADER );

   for ( i = 0; i < pConsoleRec->Data.StorageInfo.lNumOfTasks; i++)
     {
     printf ( XCONS_SHOW_STOR_TSK_INFO,
              pTaskStorInfos[i].ulTaskIndex,
              eo92GetTaskTypeName ( pTaskStorInfos[i].TaskType ),
              (float)pTaskStorInfos[i].ulStackUsed / 1024.0,
              (float)pTaskStorInfos[i].ulStackFree / 1024.0 );
     }

  return ( NO_ERROR );
  }

/*------------------------------*/

#define MAX_SECS_PER_LOW_WORD ((double )429.5)

static VOID sql92c_conv_thread_time ( FILETIME        *pThreadTime ,
                                      XC_THREAD_TIMES *pTime ,
                                      XC_THREAD_TIMES *pSumTime )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_conv_thread_time"

  double flHigh = (double ) ( pThreadTime->dwHighDateTime) ;
  double flLow  = (double ) ( pThreadTime->dwLowDateTime ) ;
  double flTotalSec  ;

  flTotalSec = flHigh * MAX_SECS_PER_LOW_WORD + flLow / 10000000.0 ;

  pTime->ulDays = ( ULONG ) floor ( flTotalSec / 3600.0 / 24.0 );
  flTotalSec   -= (double )(pTime->ulDays) * 3600.0 * 24.0 ;

  pTime->ulHr   = ( ULONG ) floor ( flTotalSec / 3600.0 );
  flTotalSec   -= (double )(pTime->ulHr) * 3600.0 ;

  pTime->ulMin = ( ULONG ) floor ( flTotalSec / 60.0 );
  flTotalSec   -= (double )(pTime->ulMin) * 60.0 ;

  pTime->ulSec  = ( ULONG ) floor ( flTotalSec  );
  pTime->ulMsec = ( pThreadTime->dwLowDateTime % 10000000 ) / 10000 ;

  pSumTime->ulMsec  += pTime->ulMsec ;
  pSumTime->ulSec   += pTime->ulSec + (pSumTime->ulMsec / 1000) ;
  pSumTime->ulMsec  %= 1000 ;
  pSumTime->ulMin   += pTime->ulMin + (pSumTime->ulSec  /   60) ;
  pSumTime->ulSec   %= 60 ;
  pSumTime->ulHr    += pTime->ulHr  + (pSumTime->ulMin  /   60) ;
  pSumTime->ulMin   %= 60 ;
  pSumTime->ulDays  += pTime->ulDays+ (pSumTime->ulHr   /   24) ;
  pSumTime->ulHr    %= 24 ;
  return ;
  }


/*------------------------------*/

static LONG sql92c_show_thread_times ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_thread_times"

  PCONS_SINGLE_THREAD_TIMES_INFO_REC pThreadTimesInfo ;
  LONG                               lActThrdIndex ;
  XC_THREAD_TIMES                    SysTime, UsrTime ;
  XC_THREAD_TIMES                    TotSysTime, TotUsrTime ;
  char                               szSysDays[5], szUsrDays[5] ;

  pThreadTimesInfo       = (PCONS_SINGLE_THREAD_TIMES_INFO_REC)
                           ((PCHAR)&pConsoleRec->Data +
                            pConsoleRec->Data.ThreadTimeInfo.ulThreadTimesOffset);


  memset ( &SysTime   , 0, sizeof (SysTime) ) ;
  memset ( &UsrTime   , 0, sizeof (UsrTime) ) ;
  memset ( &TotSysTime, 0, sizeof (TotSysTime) );
  memset ( &TotUsrTime, 0, sizeof (TotUsrTime) );

  printf ( XCONS_SHOW_THREAD_TIMES_HEADER );

  for ( lActThrdIndex = 0; lActThrdIndex < pConsoleRec->Data.ThreadTimeInfo.lNumOfUKTs; 
        lActThrdIndex++, pThreadTimesInfo++ )
    {
    sql92c_conv_thread_time ( &pThreadTimesInfo->SystemTime, &SysTime , &TotSysTime ) ;
    
    sql92c_conv_thread_time ( &pThreadTimesInfo->UserTime,   &UsrTime , &TotUsrTime ) ;

    printf ( XCONS_FORMAT_SHOW_THREAD_TIMES ,
             pThreadTimesInfo->szThreadName,
             pThreadTimesInfo->Tid,
             pThreadTimesInfo->szCluster, 
             SysTime.ulDays ? _ultoa ( SysTime.ulDays, szSysDays, 10 ) : "",
             SysTime.ulHr, SysTime.ulMin, SysTime.ulSec, SysTime.ulMsec ,
             UsrTime.ulDays ? _ultoa ( UsrTime.ulDays, szUsrDays, 10 ) : "",
             UsrTime.ulHr, UsrTime.ulMin, UsrTime.ulSec, UsrTime.ulMsec ) ; 
    }

  printf ( XCONS_FORMAT_SHOW_THREAD_TOT_TIMES ,
           TotSysTime.ulDays ? _ultoa ( TotSysTime.ulDays, szSysDays, 10 ) : "",
           TotSysTime.ulHr, TotSysTime.ulMin, TotSysTime.ulSec, TotSysTime.ulMsec ,
           TotUsrTime.ulDays ? _ultoa ( TotUsrTime.ulDays, szSysDays, 10 ) : "",
           TotUsrTime.ulHr, TotUsrTime.ulMin, TotUsrTime.ulSec, TotUsrTime.ulMsec ) ; 

  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_show_rte_info ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_rte_info"
  ULONG                       ulDevspaceNo;
  INT                         i;
  INT                         j;
  PCONS_UKT_INFO_REC          pUKTInfos;
  PCONS_DEV_INFO_REC          pDevspaceInfos;
  PCONS_DEVI_INFO_REC         pDEViInfos;
  PCONS_DEV_WORKER_INFO_REC   pDevWorkerInfos;
  BOOL                        fUKTAffinity = FALSE;
  CHAR                       szTid[12];
  PATHNAME                   szPathName;


  DBGPAS;

  pDevWorkerInfos = (PCONS_DEV_WORKER_INFO_REC)
                   ((PCHAR)&pConsoleRec->Data +
                       pConsoleRec->Data.RTEInfo.ulDevWorkerInfoOffset);

  pUKTInfos       = (PCONS_UKT_INFO_REC)
                      ((PCHAR)&pConsoleRec->Data +
                          pConsoleRec->Data.RTEInfo.ulUKTInfoOffset);


  pDEViInfos      = (PCONS_DEVI_INFO_REC)
                      ((PCHAR)&pConsoleRec->Data +
                          pConsoleRec->Data.RTEInfo.ulDEViInfoOffset);



   // --- show kernel thread info
   printf ( XCONS_SHOW_RTE_KT_HEADER );

   //
   // --- COORDINATOR
   //
   if ( pConsoleRec->Data.RTEInfo.CoordTid != (TID)UNDEF)
     {
     szTid[0] = '0';
     szTid[1] = 'x';
     _ultoa ( pConsoleRec->Data.RTEInfo.CoordTid, szTid + 2, 16 );
     strupr ( szTid + 2 );
     }
   else
     szTid[0] = '\0';

   printf ( XCONS_SHOW_RTE_KT_INFO, XCONS_SHOW_RTE_KT_COORD, szTid,
            sql92c_kt_state_name( pConsoleRec->Data.RTEInfo.CoordState ) );

   //
   // --- TIMER
   //
   if ( pConsoleRec->Data.RTEInfo.TimerTid != (TID)UNDEF)
     {
     szTid[0] = '0';
     szTid[1] = 'x';
     _ultoa ( pConsoleRec->Data.RTEInfo.TimerTid, szTid + 2, 16 );
     strupr ( szTid + 2 );
     }
   else
     szTid[0] = '\0';

   printf ( XCONS_SHOW_RTE_KT_INFO, XCONS_SHOW_RTE_KT_TIMER, szTid,
            sql92c_kt_state_name( pConsoleRec->Data.RTEInfo.TimerState ) );

   //
   // --- CLOCK
   //
   if ( pConsoleRec->Data.RTEInfo.ClockTid != (TID)UNDEF)
     {
     szTid[0] = '0';
     szTid[1] = 'x';
     _ultoa ( pConsoleRec->Data.RTEInfo.ClockTid, szTid + 2, 16 );
     strupr ( szTid + 2 );
     }
   else
     szTid[0] = '\0';

   printf ( XCONS_SHOW_RTE_KT_INFO, XCONS_SHOW_RTE_KT_CLOCK, szTid,
            sql92c_kt_state_name( pConsoleRec->Data.RTEInfo.ClockState ) );

   //
   // --- DEV0
   //
   if ( pConsoleRec->Data.RTEInfo.Dev0Tid != (TID)UNDEF)
     {
     szTid[0] = '0';
     szTid[1] = 'x';
     _ultoa ( pConsoleRec->Data.RTEInfo.Dev0Tid, szTid + 2, 16 );
     strupr ( szTid + 2 );
     }
   else
     szTid[0] = '\0';

   printf ( XCONS_SHOW_RTE_KT_INFO, XCONS_SHOW_RTE_KT_DEV0, szTid,
            sql92c_kt_state_name( pConsoleRec->Data.RTEInfo.Dev0State ) );

   //
   // --- ASYNC0
   //
   if ( pConsoleRec->Data.RTEInfo.Async0Tid != (TID)UNDEF)
     {
     szTid[0] = '0';
     szTid[1] = 'x';
     _ultoa ( pConsoleRec->Data.RTEInfo.Async0Tid, szTid + 2, 16 );
     strupr ( szTid + 2 );
     }
   else
     szTid[0] = '\0';

   printf ( XCONS_SHOW_RTE_KT_INFO, XCONS_SHOW_RTE_KT_ASYNC0, szTid,
            sql92c_kt_state_name( pConsoleRec->Data.RTEInfo.Async0State ) );

   //
   // --- CONSOLE
   //
   if ( pConsoleRec->Data.RTEInfo.ConsoleTid != (TID)UNDEF)
     {
     szTid[0] = '0';
     szTid[1] = 'x';
     _ultoa ( pConsoleRec->Data.RTEInfo.ConsoleTid, szTid + 2, 16 );
     strupr ( szTid + 2 );
     }
   else
     szTid[0] = '\0';

   printf ( XCONS_SHOW_RTE_KT_INFO, XCONS_SHOW_RTE_KT_CONSOLE, szTid,
            sql92c_kt_state_name( pConsoleRec->Data.RTEInfo.ConsoleState ) );

   //
   // --- REQUESTOR
   //
   if ( pConsoleRec->Data.RTEInfo.RequestorTid != (TID)UNDEF)
     {
     szTid[0] = '0';
     szTid[1] = 'x';
     _ultoa ( pConsoleRec->Data.RTEInfo.RequestorTid, szTid + 2, 16 );
     strupr ( szTid + 2 );
     }
   else
     szTid[0] = '\0';

   printf ( XCONS_SHOW_RTE_KT_INFO, XCONS_SHOW_RTE_KT_REQUESTOR, szTid,
            sql92c_kt_state_name( pConsoleRec->Data.RTEInfo.RequestorState ) );

   if ( pConsoleRec->Data.RTEInfo.lNumOfDevWorkers )
     {
     printf ( XCONS_SHOW_RTE_DEV_WORKER_HEADER );

     for ( i = 0; i < pConsoleRec->Data.RTEInfo.lNumOfDevWorkers; i++ )
       {
       szTid[0] = '0';
       szTid[1] = 'x';
       _ultoa ( pDevWorkerInfos[i].Tid, szTid + 2, 16 );
       strupr ( szTid + 2 );
       printf ( XCONS_SHOW_RTE_DEV_WORKER_INFO,
                i,
                szTid,
                sql92c_ukt_state_name( pDevWorkerInfos[i].ThrdState ),
                pDevWorkerInfos[i].ulIOCount );
       }
     printf ( XCONS_SHOW_RTE_CONCURR_DEV_WORKER,
              pConsoleRec->Data.RTEInfo.ulNumOfConcurrentDevWorkers );
     }

   printf ( XCONS_SHOW_RTE_UKT_HEADER );

   for ( i = 0; i < pConsoleRec->Data.RTEInfo.lNumOfUKTs; i++ )
     {
     szTid[0] = '0';
     szTid[1] = 'x';
     _ultoa ( pUKTInfos[i].Tid, szTid + 2, 16 );
     strupr ( szTid + 2 );

     printf ( XCONS_SHOW_RTE_UKT_INFO,
              pUKTInfos[i].ulUKTIndex,
              szTid,
              sql92c_ukt_state_name( pUKTInfos[i].ThrdState ),
              pUKTInfos[i].dDispatcherCalls,
              pUKTInfos[i].ulActiveTasks,
              pUKTInfos[i].ulTotalTasks,
              pUKTInfos[i].szCluster );


     if ( pUKTInfos[i].ulProcessor )
       fUKTAffinity = TRUE;
     }

   if ( fUKTAffinity )
     {
     printf ( XCONS_SHOW_RTE_UKT_AFFIN_HEADER );

     for ( i = 0; i < pConsoleRec->Data.RTEInfo.lNumOfUKTs; i++ )
       {
       if ( pUKTInfos[i].ulProcessor )
         {
         printf ( XCONS_SHOW_RTE_UKT_AFFIN_INFO,
                  pUKTInfos[i].ulUKTIndex,
                  pUKTInfos[i].ulProcessor );
         }
       }
     }

   printf ( XCONS_SHOW_RTE_IO_HEADER );

   for ( i = 0; i < pConsoleRec->Data.RTEInfo.lNumOfUKTs; i++ )
     {
     pDevspaceInfos = (PCONS_DEV_INFO_REC)
                        ((PCHAR)&pConsoleRec->Data +
                                pUKTInfos[i].ulDevspaceInfoOffset);

     for ( j = 0; j < pUKTInfos[i].lNumOfDevspaces; j++ )
       {
       ulDevspaceNo = pDevspaceInfos[j].ulDevspaceNo;

       // --- The compress value (20) must be identical to the value
       //     defined in 'XCONS_SHOW_RTE_IO_UKT_INFO'!!!!!!!!!!!
       eo92_compress_path( szPathName, pDevspaceInfos[j].DevspaceName, 20 );

       szTid[0] = '0';
       szTid[1] = 'x';
       _ultoa ( pUKTInfos[i].Tid, szTid + 2, 16 );
       strupr ( szTid + 2 );

       printf ( XCONS_SHOW_RTE_IO_UKT_INFO,
                pUKTInfos[i].ulUKTIndex,
                szTid,
                szPathName,
                ulDevspaceNo,
                pDevspaceInfos[j].ulReadCnt,
                pDevspaceInfos[j].ulWriteCnt );
       }
     }

   for ( i = 0; i < pConsoleRec->Data.RTEInfo.lNumOfDEVis; i++ )
     {
     ulDevspaceNo = pDEViInfos[i].ulDevspaceNo;

     // --- The compress value (20) must be identical to the value
     //     defined in 'XCONS_SHOW_RTE_IO_DEV_INFO'!!!!!!!!!!!
     eo92_compress_path ( szPathName, pDEViInfos[i].DevspaceName, 20 );

     szTid[0] = '0';
     szTid[1] = 'x';
     _ultoa ( pDEViInfos[i].Tid, szTid + 2, 16 );
     strupr ( szTid + 2 );

     printf ( XCONS_SHOW_RTE_IO_DEV_INFO,
              pDEViInfos[i].ulDEViIndex,
              szTid,
              szPathName,
              ulDevspaceNo,
              pDEViInfos[i].DevIOStat.DevRead.IOStat.ulCount,
              pDEViInfos[i].DevIOStat.DevWrite.IOStat.ulCount,
              pDEViInfos[i].DevIOStat.ulIOQueLen,
              pDEViInfos[i].DevIOStat.ulMaxIOQueLen );
     }

  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_show_total_count_info ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_total_count_info"

  CONS_TOTAL_COUNTS_INFO_REC      *TotCntInfo ;
  TotCntInfo = & pConsoleRec->Data.TotalCountsInfo ;

  printf ( XCONS_SHOW_TOTAL_STAT_HEADER ) ;

  printf ( XCONS_SHOW_TOTAL_DISP_COMMAND_CNT ,
               XCONS_SHOW_TOTAL_KT_DISP_CALL_CNT ,
               TotCntInfo->ulTotalDispCalls,
               XCONS_SHOW_TOTAL_KT_COMMAND_CNT ,
               TotCntInfo->ulTotalCommands);

  printf ( XCONS_SHOW_TOTAL_KT_STATE_VWAIT_VSUSP_CNT ,
               XCONS_SHOW_TOTAL_KT_STATE_VWAIT_CNT,
               TotCntInfo->ulTotalVwait,
               XCONS_SHOW_TOTAL_KT_STATE_VSUSP_CNT,
               TotCntInfo->ulTotalVsusp );

  printf ( XCONS_SHOW_TOTAL_KT_STATE_VSLEEP,
               XCONS_SHOW_TOTAL_KT_STATE_VSLEEP_CNT,
               TotCntInfo->ulTotalVsleep);

  printf ( XCONS_SHOW_TOTAL_SELFIO_DEVIO_CNT ,
               XCONS_SHOW_TOTAL_KT_SELFIO_CNT ,
               TotCntInfo->ulTotalSelfIO,
               XCONS_SHOW_TOTAL_KT_DEVIO_CNT,
               TotCntInfo->ulTotalDevIO);

  printf ( XCONS_SHOW_TOTAL_REG_ACC_BEGEXCL_COLL,
               XCONS_SHOW_TOTAL_KT_REG_ACCESS,
               TotCntInfo->ulTotalRegAccess,
               XCONS_SHOW_TOTAL_KT_BEGEXCL_COLL,
               TotCntInfo->ulTotalBegExclTasCount);

  printf ( XCONS_SHOW_TOTAL_REG_COLL_ENDEXCL_COLL,
               XCONS_SHOW_TOTAL_KT_REG_COLL,
               TotCntInfo->ulTotalRegColl,
               XCONS_SHOW_TOTAL_KT_ENDEXCL_COLL,
               TotCntInfo->ulTotalEndExclTasCount);

  printf ( XCONS_SHOW_TOTAL_KT_REG_WAIT ,
               XCONS_SHOW_TOTAL_KT_REG_WAIT_CNT ,
               TotCntInfo->ulTotalRegWait);

  if ( TotCntInfo->TotalPrioStat.ulTotalCount )
   { printf ( XCONS_SHOW_TOTAL_PRIO_CNT_AND_OTHER,
                  XCONS_SHOW_TOTAL_KT_PRIO_CNT,
                  TotCntInfo->TotalPrioStat.ulTotalCount,
                  XCONS_SHOW_TOTAL_KT_PRIO_OTHER ,
                  TotCntInfo->TotalPrioStat.ulOtherUkt);
   }

  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_show_queue_info ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_queue_info"
  INT                              i;
  PCONS_SINGLE_QUEUE_STAT_INFO_REC pQueStatInfo;

  pQueStatInfo  = (PCONS_SINGLE_QUEUE_STAT_INFO_REC)
                      ((PCHAR)&pConsoleRec->Data +
                              pConsoleRec->Data.QueInfo.ulQueStatInfoOffset);

  printf ( XCONS_SHOW_QUEUE_STAT_HEADER );

  for ( i = 0; i < pConsoleRec->Data.QueInfo.lNumOfActiveTasks; i++)
    {
    printf ( XCONS_SHOW_QUEUE_STAT_INFO ,
             pQueStatInfo[i].ulTaskIndex,
             pQueStatInfo[i].QueStat.ulUToUQueCount,
             pQueStatInfo[i].QueStat.ulUKTQueCount,
             pQueStatInfo[i].QueStat.ulCOMQueCount ) ;
    }

  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_show_ukt_sl_info ( PCONSOLE_REC   pConsoleRec )
{
  #undef  MF__
  #define MF__ MOD__"sql92c_show_ukt_sl_info"
  INT                i;
  char               IdleTimeBuf[20];             /* ddd:hh:mm:ss: */
  char               IOWaitTimeBuf[20];           /* ddd:hh:mm:ss: */
 
  PCONS_SINGLE_UKT_SL_STAT_INFO_REC pSleepStatInfo;

  if ( pConsoleRec->Data.UKTSleepTimeInfo.lNumOfActiveUKTs > 0 )
  {
    pSleepStatInfo = (PCONS_SINGLE_UKT_SL_STAT_INFO_REC)
                         ((PCHAR)&pConsoleRec->Data +
                         pConsoleRec->Data.UKTSleepTimeInfo.ulSleepStatInfoOffset);

    printf ( XCONS_SHOW_UKT_SLEEP_STAT_HEADER ) ;

    for ( i = 0; i < pConsoleRec->Data.UKTSleepTimeInfo.lNumOfActiveUKTs; i++)
    {
      /* CR 1000097 */
      eo92_SecToDDHHMMSS( (ULONG)pSleepStatInfo[i].UKTIdleStat.ulAbsMicroSeconds/1000000, IdleTimeBuf, sizeof(IdleTimeBuf) ) ;

      eo92_SecToDDHHMMSS( (ULONG)pSleepStatInfo[i].UKTIOWaitStat.ulAbsMicroSeconds/1000000, IOWaitTimeBuf, sizeof(IOWaitTimeBuf) ) ; 

      printf( XCONS_SHOW_UKT_SLEEP_STAT_INFO ,
              pSleepStatInfo[i].ulUKTIndex ,
              pSleepStatInfo[i].UKTIdleStat.ulCount,
              IdleTimeBuf,
              ((double) (tsp00_Int8)pSleepStatInfo[i].UKTIdleStat.ulAbsMicroSeconds) * 100. /
              (double) (tsp00_Int8)pSleepStatInfo[i].measuredMicroSeconds,
              pSleepStatInfo[i].UKTIOWaitStat.ulCount,
              IOWaitTimeBuf,
              ((double) (tsp00_Int8)pSleepStatInfo[i].UKTIOWaitStat.ulAbsMicroSeconds) * 100. /
              (double) (tsp00_Int8)pSleepStatInfo[i].measuredMicroSeconds,
              pSleepStatInfo[i].ulRUNQueueLen );
    }
  }

  return ( NO_ERROR );
}


/*------------------------------*/

static LONG sql92c_show_task_count_info ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_task_count_info"
  INT                                i;
  CHAR                               szApplPid[12];
  PCONS_SINGLE_TASK_COUNTS_INFO_REC  pTaskCountsInfo;
  teo52_TaskStateStatisticRec        *pTaskStateStat;
  teo52_TaskPrioStatistic            *pTaskPrioStat;
  BOOL                               fTimeMeasureActive;
  int                                j1;

  /* PTS 1104727 */
  if ( pConsoleRec->Data.TaskCountsInfo.usReturnCode == CON_WRONG_TASK )
  {
    MSGCD ((ERR_XCONS_WRONG_TASK ));
    return ( 1 );
  }

  pTaskCountsInfo = (PCONS_SINGLE_TASK_COUNTS_INFO_REC)
                        ((PCHAR)&pConsoleRec->Data +
                          pConsoleRec->Data.TaskCountsInfo.ulTaskCountsInfoOffset);

  fTimeMeasureActive = pConsoleRec->Data.TaskCountsInfo.fTimeMeasureActive;

  for ( i = 0;
        i < pConsoleRec->Data.TaskCountsInfo.lNumOfActiveTasks;
        i++, pTaskCountsInfo++ )
    {
    pTaskStateStat  = &pTaskCountsInfo->StateStat;
    pTaskPrioStat   = &pTaskCountsInfo->PrioStat;

    if ( pTaskCountsInfo->ApplPid != (PID)UNDEF )
      {
      _ultoa ( pTaskCountsInfo->ApplPid, szApplPid, 10 );
      }
    else
      {
      szApplPid[0] = '?';
      szApplPid[1] = '\0';
      }


    printf ( XCONS_SHOW_TASK_COUNT_HEADER,
                XCONS_INFO_LINE ,
                pTaskCountsInfo->ulTaskIndex,
                eo92GetTaskTypeName ( pTaskCountsInfo->TaskType ),
                szApplPid,
                XCONS_INFO_LINE ) ;

    if ( pTaskCountsInfo->szClientNode[0] )
      {
        printf ( XCONS_SHOW_TASK_COUNT_REM_NODE_AND_PID ,
                 XCONS_SHOW_TASK_COUNT_REM_NODE ,
                 pTaskCountsInfo->szClientNode ,
                 XCONS_SHOW_TASK_COUNT_REM_PID  ,
                 szApplPid );
      }

    eo92PrintTaskOMSStatistics( &pTaskCountsInfo->TaskOMSStat ) ;

    printf ( XCONS_SHOW_TASK_COUNT_DISP_COMMAND_CNT    ,
                XCONS_SHOW_TASK_COUNT_KT_DISP_CALL_CNT,
                pTaskCountsInfo->ulDispatcherCalls,
                XCONS_SHOW_TASK_COUNT_KT_COMMAND_CNT,
                pTaskCountsInfo->ulCommands );

    printf ( XCONS_SHOW_TASK_COUNT_EXCL_SELF_SUSP_CNT  ,
                XCONS_SHOW_TASK_COUNT_KT_EXCLUSIVE_CNT ,
                pTaskCountsInfo->ulBegExclCalls ,
                XCONS_SHOW_TASK_COUNT_KT_SELF_SUSP_CNT ,
                pTaskCountsInfo->ulSelfSuspCount );

    for ( j1 = 0; j1 < MAX_SUSPEND_LEVEL; j1++ )
    {
      if ( pTaskCountsInfo->totalResumerCount[j1] > 0 )
      {
          int j2;
          int j;
          printf ( XCONS_SHOW_TASK_COUNT_RESUME_CNT,
              j1, (int)pTaskCountsInfo->lResumeCount[j1], (int)pTaskCountsInfo->totalResumerCount[j1]);
          j2 = pTaskCountsInfo->totalResumerCount[j1];
          j = MAX_RESUMER_PIDS;
          while( (j2-- > 0) && (j-- > 0) )
          {
              printf ( " T%d", (int)pTaskCountsInfo->resumerPid[j2%MAX_RESUMER_PIDS][j1] );
          }
          printf ( " ]\n" );
      }
    }

	/* PTS 1106969 */
	if ( *pTaskCountsInfo->DevNameWaitingFor )
        printf ( XCONS_SHOW_TASK_COUNT_WAIT_FOR_DEV, pTaskCountsInfo->DevNameWaitingFor) ;

    eo92PrintTaskCounts( (tsp00_Int4 ) fTimeMeasureActive, pTaskStateStat ) ;

    if ( pTaskPrioStat->ulTotalCount )
       printf ( XCONS_SHOW_TASK_COUNT_PRIO_TOT_OTH,
                      XCONS_SHOW_TASK_COUNT_KT_PRIO_CNT ,
                      pTaskPrioStat->ulTotalCount   ,
                      XCONS_SHOW_TASK_COUNT_KT_PRIO_FROM_OTHER,
                      pTaskPrioStat->ulOtherUkt ) ;

    printf ( XCONS_SHOW_TASK_COUNT_TRAILER ) ;
    }
  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_show_ukt_queues ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_ukt_queues"
  PCONS_SINGLE_UKT_QUEUES_INFO_REC  pUKTQueues;
  ULONG                             ulUKT;


  if ( pConsoleRec->Data.UKTQueues.lNumOfActiveUKTs > 0 )
    {
    pUKTQueues = (PCONS_SINGLE_UKT_QUEUES_INFO_REC)
                         ((PCHAR)&pConsoleRec->Data +
                         pConsoleRec->Data.UKTQueues.ulUKTQueuesOffset);

    for ( ulUKT = 0; ulUKT < (ULONG)pConsoleRec->Data.UKTQueues.lNumOfActiveUKTs;
          ulUKT++, pUKTQueues++ )
      if ( pUKTQueues->ulUKTQueueLen ||
           pUKTQueues->ulCOMQueueLen ||
           pUKTQueues->ulRUNQueueLen )
      {
        printf ( XCONS_SHOW_QUEUES_HEAD,
                         pUKTQueues->ulUKTIndex ,
                         pUKTQueues->Tid ) ;

        sql92c_show_UKT_queue ( pUKTQueues->ulUKTQueueLen,
                               (PCONS_SINGLE_QUEUE_INFO_REC )
                                ((PCHAR)&pConsoleRec->Data +
                                      pUKTQueues->ulUKTQueueOffset)) ;

        sql92c_show_COM_queue ( pUKTQueues->ulCOMQueueLen,
                               (PCONS_SINGLE_QUEUE_INFO_REC )
                                ((PCHAR)&pConsoleRec->Data +
                                      pUKTQueues->ulCOMQueueOffset)) ;

        sql92c_show_RUN_queue ( pUKTQueues->ulRUNQueueLen,
                               (PCONS_SINGLE_QUEUE_INFO_REC )
                                ((PCHAR)&pConsoleRec->Data +
                                      pUKTQueues->ulRUNQueueOffset)) ;
      }
    }

  return ( NO_ERROR );
  }

/*------------------------------*/


static LONG sql92c_show_UKT_queue ( ULONG ulUKTQueueLen,
                                    PCONS_SINGLE_QUEUE_INFO_REC pUKTUKTQueue)
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_UKT_queue"

  ULONG                             ulQueuElem;

  if ( ulUKTQueueLen > 0 )
    { printf ( XCONS_SHOW_UKT_QUEUE_HEAD ) ;
      for ( ulQueuElem = 0;
            ulQueuElem < ulUKTQueueLen ;
            ulQueuElem++, pUKTUKTQueue++ )
               if ( pUKTUKTQueue->PrioFlag )
                 printf( XCONS_SHOW_UKT_QUEUE_PRIO_ELEMENT,
                          pUKTUKTQueue->ulTaskIndex ,
                          pUKTUKTQueue->PrioFlag ) ;
               else
                 printf( XCONS_SHOW_UKT_QUEUE_ELEMENT,
                          pUKTUKTQueue->ulTaskIndex ) ;
    }

  return ( NO_ERROR );
  }


/*------------------------------*/


static LONG sql92c_show_COM_queue ( ULONG ulCOMQueueLen,
                                    PCONS_SINGLE_QUEUE_INFO_REC pUKTCOMQueue)
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_COM_queue"

  ULONG                             ulQueuElem;

  if ( ulCOMQueueLen > 0 )
    { printf ( XCONS_SHOW_COM_QUEUE_HEAD ) ;
      for ( ulQueuElem = 0;
            ulQueuElem < ulCOMQueueLen ;
            ulQueuElem++, pUKTCOMQueue++ )
               if ( pUKTCOMQueue->PrioFlag )
                 printf( XCONS_SHOW_COM_QUEUE_PRIO_ELEMENT,
                            pUKTCOMQueue->ulTaskIndex ,
                            pUKTCOMQueue->PrioFlag ) ;
               else
                 printf( XCONS_SHOW_COM_QUEUE_ELEMENT,
                            pUKTCOMQueue->ulTaskIndex ) ;
    }

  return ( NO_ERROR );
  }


/*------------------------------*/


static LONG sql92c_show_RUN_queue ( ULONG ulRUNQueueLen,
                                    PCONS_SINGLE_QUEUE_INFO_REC pUKTRUNQueue)
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_RUN_queue"

  ULONG                             ulQueuElem;
  CHAR                              szFromTask[10] ;

  if ( ulRUNQueueLen > 0 )
    { printf ( XCONS_SHOW_RUN_QUEUE_HEAD ) ;
      for ( ulQueuElem = 0;
            ulQueuElem < ulRUNQueueLen ;
            ulQueuElem++, pUKTRUNQueue++ )
        {   if ( pUKTRUNQueue->ulFromTask != UNDEF )
              sprintf(szFromTask, "(T%d)", pUKTRUNQueue->ulFromTask ) ;
            else
              szFromTask[0] = '\0' ;
            if ( pUKTRUNQueue->ulReqType != UNDEF )
               printf( XCONS_SHOW_RUN_QUEUE_ELEMENT_REQ,
                          pUKTRUNQueue->ulTaskIndex ,
                          eo92GetTaskStateName ( pUKTRUNQueue->TaskState,0 ),
                          pUKTRUNQueue->ulDynPrio ,
                          sql92c_req_type_name ( pUKTRUNQueue->ulReqType ),
                          szFromTask) ;
           else
               printf( XCONS_SHOW_RUN_QUEUE_ELEMENT,
                          pUKTRUNQueue->ulTaskIndex ,
                          eo92GetTaskStateName ( pUKTRUNQueue->TaskState,0 ),
                          pUKTRUNQueue->ulDynPrio , szFromTask) ;
        }
    }

  return ( NO_ERROR );
  }


/*------------------------------*/

static LONG sql92c_show_suspends ( PCONSOLE_REC  pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_suspends"
  INT             i;
  ULONG           ulTotalSuspends = 0 ;

  printf ( XCONS_SHOW_SUSPEND_HEADER );

  for ( i=0; i<MAX_SUSPEND_REASONS ; i++ )
    ulTotalSuspends += pConsoleRec->Data.SuspendReasons.ulSuspends [ i ] ;

  printf ( XCONS_SHOW_SUSPEND_TOTAL , ulTotalSuspends ) ;

  for ( i = 0; i < MAX_SUSPEND_REASONS ; i++ )
    {
    if ( pConsoleRec->Data.SuspendReasons.ulSuspends [ i ] )
      {
      printf( XCONS_SHOW_SUSPEND_CNT_PERCENT,
                SuspendReasonText[i] . szShortReason ,
                pConsoleRec->Data.SuspendReasons.ulSuspends [ i ] ,
                (float ) pConsoleRec->Data.SuspendReasons.ulSuspends [ i ] * 100.0 /
                (float ) ulTotalSuspends ,
                SuspendReasonText[i] . szLongReason ) ;
      }
    }

  return ( NO_ERROR );
  }

/*------------------------------*/


static LONG sql92c_show_param ( PSZ pszParam , PCONSOLE_REC pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_param"

  e92_print_param ( &(pConsoleRec->Data.GetParamInfo.XParam) , pszParam );

  return ( NO_ERROR );
  }


/*------------------------------*/

static LONG sql92c_show_state_info ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_storage_info"
  PSZ      pszStateStr;
  DBGPAS;

  
   switch (pConsoleRec->Data.StateInfo.DatabaseState)
     {
     case SERVER_STARTING      : pszStateStr = XCONS_SHOW_STATE_STARTING ; break;
     case SERVER_COLD          : pszStateStr = XCONS_SHOW_STATE_COLD     ; break;
     case SERVER_WARM          : pszStateStr = XCONS_SHOW_STATE_WARM     ; break;
     case SERVER_SHUTDOWN      : pszStateStr = XCONS_SHOW_STATE_SHTDWN   ; break;
     case SERVER_SHUTDOWNREINIT: pszStateStr = XCONS_SHOW_STATE_SHTDWNREI; break;
     case SERVER_SHUTDOWNKILL  : pszStateStr = XCONS_SHOW_STATE_SHTDWNKIL; break;
     case SERVER_STOP          : pszStateStr = XCONS_SHOW_STATE_STOP     ; break;
     case SERVER_KILL          : pszStateStr = XCONS_SHOW_STATE_KILL     ; break;
     case SERVER_ABORT         : pszStateStr = XCONS_SHOW_STATE_ABORT    ; break;
     default                   : pszStateStr = XCONS_SHOW_STATE_UNKNOWN  ; break;
     }

   if ( pConsoleRec->Data.StateInfo.ulTerminationTimeOut != 0 )
     {
     printf ( XCONS_SHOW_STATE_INFO_TERM_TIMEOUT, pszStateStr,
              pConsoleRec->Data.StateInfo.ulTerminationTimeOut );
     }
   else
     printf ( XCONS_SHOW_STATE_INFO, pszStateStr );
   
  return ( NO_ERROR );
  }


/*------------------------------*/

static LONG sql92c_show_awe_info ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_awe_info"

  PCONS_AWE_PHYS_PAGE_INFO_REC   pAWEPhysPageInfos;
  PCONS_AWE_MAP_PAGE_INFO_REC    pAWEMapPageInfos;
  SAPDB_UInt4                    i;

  if ( pConsoleRec->Data.AWEInfo.AWEAvailable )
  {
    pAWEMapPageInfos  = (PCONS_AWE_MAP_PAGE_INFO_REC)
                             ((PCHAR)&pConsoleRec->Data +
                                 pConsoleRec->Data.AWEInfo.ulAWEMapPageInfoOffset );

    pAWEPhysPageInfos = (PCONS_AWE_PHYS_PAGE_INFO_REC)
                             ((PCHAR)&pConsoleRec->Data +
                                 pConsoleRec->Data.AWEInfo.ulAWEPhysPageInfoOffset );

    printf ( XCONS_SHOW_AWE_INFO, 
             pConsoleRec->Data.AWEInfo.NumOfPhysPages,
             pConsoleRec->Data.AWEInfo.NumOfFreePhysPages,
             pConsoleRec->Data.AWEInfo.NumOfMapAreaPages,
             pConsoleRec->Data.AWEInfo.NumOfFreeMapAreaPages,
             pConsoleRec->Data.AWEInfo.NumOfMaps,
             pConsoleRec->Data.AWEInfo.NumOfUnMaps );


    if ( pConsoleRec->Data.AWEInfo.NumOfAWEMapPageInfos )
    {
      printf ( XCONS_SHOW_AWE_MAP_PAGE_HEADER );

      for ( i = 0; i < pConsoleRec->Data.AWEInfo.NumOfAWEMapPageInfos; i++ )
      {
        if (( pAWEMapPageInfos[i].PhysPageNo ==  -1 ) ||
            ( pAWEMapPageInfos[i].Reserved   == false ))
          printf ( XCONS_SHOW_AWE_UNMAP_PAGE_INFO,
                   pAWEMapPageInfos[i].pMappingPageAddr,
                   pAWEMapPageInfos[i].Reserved ? "YES": "NO",
                   pAWEMapPageInfos[i].MapCount );
        else
          printf ( XCONS_SHOW_AWE_MAP_PAGE_INFO,
                   pAWEMapPageInfos[i].pMappingPageAddr,
                   pAWEMapPageInfos[i].PhysPageNo,
                   pAWEMapPageInfos[i].MapCount );
      }
    }

    if ( pConsoleRec->Data.AWEInfo.NumOfAWEPhysPageInfos )
    {
      printf ( XCONS_SHOW_AWE_PHYS_PAGE_HEADER );

      for ( i = 0; i < pConsoleRec->Data.AWEInfo.NumOfAWEPhysPageInfos; i++ )
      {
        printf ( XCONS_SHOW_AWE_PHYS_PAGE_INFO,
                 i,
                 pAWEPhysPageInfos[i].Reserved ? "YES" : "NO",
                 pAWEPhysPageInfos[i].MapCount );
      }
    }
  }

  return ( NO_ERROR );
  }

/*------------------------------*/

static LONG sql92c_show_task_move_info ( PCONSOLE_REC   pConsoleRec )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_show_task_move_info"
  INT                               i;
  PCONS_SINGLE_UKT_MOVE_INFO_REC    pUKTInfos;
  PCONS_LAST_MOVES_ENTRY            pLastMoves;
  char                              TimeBuf[20];             /* ddd:hh:mm:ss: */

  DBGPAS;

  pUKTInfos   = (PCONS_SINGLE_UKT_MOVE_INFO_REC)
                   ((PCHAR)&pConsoleRec->Data +
                    pConsoleRec->Data.TaskMoveInfo.ulUKTInfoOffset);
  pLastMoves = (PCONS_LAST_MOVES_ENTRY)
                  ((PCHAR)&pConsoleRec->Data +
                   pConsoleRec->Data.TaskMoveInfo.ulLastMovesInfoOffset);

  if ( pConsoleRec->Data.TaskMoveInfo.TaskMoveIntervalStartTime )
    eo92_SecToDDHHMMSS( time(NULL) - pConsoleRec->Data.TaskMoveInfo.TaskMoveIntervalStartTime, 
                        TimeBuf, sizeof(TimeBuf) ) ;
  else
    TimeBuf[0] = 0;

  printf ( XCONS_SHOW_MOVE_INFO,
           pConsoleRec->Data.TaskMoveInfo.CheckIntervalTime ? "ENABLED" : "DISABLED",
           pConsoleRec->Data.TaskMoveInfo.CheckIntervalTime,
           TimeBuf,
           pConsoleRec->Data.TaskMoveInfo.TaskMoveMeasIntervals,
           pConsoleRec->Data.TaskMoveInfo.MinimumDiff,
           pConsoleRec->Data.TaskMoveInfo.ValueEquality );
 

  if ( pConsoleRec->Data.TaskMoveInfo.lNumOfLastRecs )
    {
    printf ( XCONS_SHOW_LAST_TASK_MOVES_HEADER );

    for ( i = 0; i < pConsoleRec->Data.TaskMoveInfo.lNumOfLastRecs; i++ )
      {
       eo92_SecToDDHHMMSS( time(NULL) - pLastMoves[i].MoveTime, TimeBuf, sizeof(TimeBuf));
       printf ( XCONS_SHOW_LAST_TASK_MOVES_INFO,
                TimeBuf,
                pLastMoves[i].ulTaskIndex,
                pLastMoves[i].TaskMaxRunnableMicroSeconds / 1000000,
                pLastMoves[i].TaskMaxRunnableMicroSeconds % 1000000,
                pLastMoves[i].ulSrcUKTIndex,
                pLastMoves[i].SrcMaxRunnableMicroSeconds / 1000000,
                pLastMoves[i].SrcMaxRunnableMicroSeconds % 1000000,
                pLastMoves[i].ulDstUKTIndex,
                pLastMoves[i].DstMaxRunnableMicroSeconds / 1000000,
                pLastMoves[i].DstMaxRunnableMicroSeconds % 1000000 );
       }
     }

   printf ( XCONS_SHOW_MOVE_UKT_HEADER );

   for ( i = 0; i < pConsoleRec->Data.TaskMoveInfo.lNumOfUKTs; i++ )
     {
     if ( pConsoleRec->Data.TaskMoveInfo.TaskMoveIntervalStartTime )
       eo92_SecToDDHHMMSS( time(NULL) - pUKTInfos[i].TaskMoveIntervalStartTime, 
                           TimeBuf, sizeof(TimeBuf) ) ;
     else
       TimeBuf[0] = 0;

     printf ( XCONS_SHOW_MOVE_UKT_INFO,
              pUKTInfos[i].ulUKTIndex,
              pUKTInfos[i].NumOfMovableTasks,
              TimeBuf,
              pUKTInfos[i].MaxRunnableMicroSeconds / 1000000,
              pUKTInfos[i].MaxRunnableMicroSeconds % 1000000,
              pUKTInfos[i].RunnableMicroSeconds / 1000000,
              pUKTInfos[i].RunnableMicroSeconds % 1000000,
              pUKTInfos[i].UKTSleepTimeMicroSeconds / 1000000,
              pUKTInfos[i].UKTSleepTimeMicroSeconds % 1000000 );
     }

  return ( NO_ERROR );
  }

/*------------------------------*/

# define MAX_NAME_BUFFER_SIZE 128

static LONG sql92c_show_connections_info ( PCONSOLE_REC   pConsoleRec )
  {
# undef  MF__
# define MF__ MOD__"sql92c_show_connections_info"

    PCONS_SINGLE_CONNECTION_INFO_REC   pConnectionInfo;
    LONG                               lActConnectionIndex;
    char                               buffer[MAX_NAME_BUFFER_SIZE];

    pConnectionInfo       = (PCONS_SINGLE_CONNECTION_INFO_REC)
                            ((PCHAR)&pConsoleRec->Data +
                            pConsoleRec->Data.ConnectionInfo.ulConnectionInfoOffset);


    for ( lActConnectionIndex = 0; lActConnectionIndex < pConsoleRec->Data.ConnectionInfo.lNumOfConnections; 
          lActConnectionIndex++, pConnectionInfo++ )
    {
      eo92GetConnectStateName( pConnectionInfo->connState, buffer, MAX_NAME_BUFFER_SIZE );

      printf ( XCONS_FORMAT_SHOW_CONNECTION_TASK ,
               pConnectionInfo->taskIndex,
               pConnectionInfo->uktIndex,
               eo92GetTaskTypeName ( (tsp00_Uint1)pConnectionInfo->taskType ) );

      if ( pConnectionInfo->connectTime == 0 )
      {
        printf ( XCONS_FORMAT_SHOW_CONNECTION_STATE_NOT_CONNECTED, buffer );
      } 
      else 
      {
        struct tm *pTime = localtime( &pConnectionInfo->connectTime );

        if ( pTime )
        {
          printf ( XCONS_FORMAT_SHOW_CONNECTION_STATE_TIME,
                   buffer, asctime(pTime) );
        } 
        else 
        {
          printf ( XCONS_FORMAT_SHOW_CONNECTION_STATE_DUMPTIME,
                   buffer, pConnectionInfo->connectTime );
        }
      }

      if ( pConnectionInfo->clientNode[0] == 0 || pConnectionInfo->clientNode[0] == ' ' )
      {
        printf ( XCONS_FORMAT_SHOW_CONNECTION_LOCAL, pConnectionInfo->peerPid );
      } 
      else 
      {
        sprintf(buffer, "%d", pConnectionInfo->remotePid );

        printf ( XCONS_FORMAT_SHOW_CONNECTION_REMOTE ,
                 pConnectionInfo->peerPid,
                 buffer,
                 pConnectionInfo->clientNode );
      }
    }

    if ( !pConsoleRec->Data.ConnectionInfo.lNumOfConnections )
      printf ( XCONS_FORMAT_SHOW_CONNECTION_NO_TASK_CONNECTED );

    return ( NO_ERROR );
  }

/*------------------------------*/

static PSZ sql92c_kt_state_name ( ULONG ulState )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_kt_state_name"

  DBGPAS;

  switch ( ulState )
    {
    case KT_INITIALIZED:    return ( XCONS_KT_STATE_INITIALIZED );
    case KT_RUNNING    :    return ( XCONS_KT_STATE_RUNNING     );
    case KT_TERMINATING:    return ( XCONS_KT_STATE_TERMINATING );
    case KT_TERMINATED :    return ( XCONS_KT_STATE_TERMINATED  );
    case KT_SLEEPING   :    return ( XCONS_KT_STATE_SLEEPING    );
    case KT_DETACHED   :    return ( XCONS_KT_STATE_DETACHED    );
    case KT_ATTACHED   :    return ( XCONS_KT_STATE_ATTACHED    );
    }

  return ( XCONS_KT_STATE_UNKNOWN );
  }

/*------------------------------*/

static PSZ sql92c_ukt_state_name ( ULONG ulState )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_ukt_state_name"

  DBGPAS;

  switch ( ulState )
    {
    case UKT_INITIALIZED:    return ( XCONS_UKT_STATE_INITIALIZE );
    case UKT_RUNNING    :    return ( XCONS_UKT_STATE_RUNNING    );
    case UKT_TERMINATED :    return ( XCONS_UKT_STATE_TERMINATED );
    case UKT_SLEEPING   :    return ( XCONS_UKT_STATE_SLEEPING   );
    }

  return ( XCONS_UKT_STATE_UNKNOWN );
  }

/*------------------------------*/

static PSZ  sql92c_req_type_name ( ULONG ulReqType )
  {
  #undef  MF__
  #define MF__ MOD__"sql92c_req_type_name"

  DBGPAS;

  switch ( ulReqType )
    {
      case REQ_VATTACH                 : return ( XCONS_REQ_VATTACH ) ;
      case REQ_VBLOCKIO                : return ( XCONS_REQ_VBLOCKIO ) ;
      case REQ_VDETACH                 : return ( XCONS_REQ_VDETACH ) ;
      case REQ_INITOK                  : return ( XCONS_REQ_INITOK ) ;
      case REQ_VRESUME                 : return ( XCONS_REQ_VRESUME ) ;
      case REQ_VSIGNAL                 : return ( XCONS_REQ_VSIGNAL ) ;
      case REQ_CONNECT                 : return ( XCONS_REQ_CONNECT );
      case REQ_RELEASE                 : return ( XCONS_REQ_RELEASE ) ;
      case REQ_VRECEIVE                : return ( XCONS_REQ_VRECEIVE) ;
      case REQ_SHUTDOK                 : return ( XCONS_REQ_SHUTDOK ) ;
      case REQ_SENDER_JOB_FREE         : return ( XCONS_REQ_SENDER_JOB_FREE) ;
      case REQ_VENDEXCL                : return ( XCONS_REQ_VENDEXCL ) ;
      case REQ_RESCHEDULE              : return ( XCONS_REQ_RESCHEDULE ) ;
      case REQ_SHUTDOWN_RESUME_TW      : return ( XCONS_REQ_SHUTDOWN_RESUME_TW);
      case REQ_ASYNOPEN                : return ( XCONS_REQ_ASYNOPEN ) ;
      case REQ_ASYNCLOSE               : return ( XCONS_REQ_ASYNCLOSE ) ;
      case REQ_ASYNIO                  : return ( XCONS_REQ_ASYNIO ) ;
      case REQ_ASYNWAIT                : return ( XCONS_REQ_ASYNWAIT ) ;
      case REQ_VNSHUTDOWN_REPLY        : return ( XCONS_REQ_VNSHUTDOWN_REPLY) ;
      case REQ_OUTGOING_CONNECT        : return ( XCONS_REQ_OUTGOING_CONNECT) ;
      case REQ_OUTGOING_CONNECT_REPLY  : return ( XCONS_REQ_OUTGOING_CONNECT_REPLY) ;
      case REQ_OPEN_WAKEUP_SD          : return ( XCONS_REQ_OPEN_WAKEUP_SD ) ;
      case REQ_CONNECT_RC              : return ( XCONS_REQ_CONNECT_RC ) ;
      case REQ_RELEASE_RC              : return ( XCONS_REQ_RELEASE_RC ) ;
      case REQ_CLOSE_OUTGOING          : return ( XCONS_REQ_CLOSE_OUTGOING ) ;
      case REQ_CLOSE_OUTGOING_REPLY    : return ( XCONS_REQ_CLOSE_OUTGOING_REPLY) ;
      case REQ_FINISH_SV               : return ( XCONS_REQ_FINISH_SV ) ;
      case REQ_ASYNCNTL                : return ( XCONS_REQ_ASYNCNTL ) ;
   }


  return ( XCONS_REQ_TYPE_UNKNOWN );
  }

/*------------------------------*/

static BOOL sql92c_kernel_is_alive ( PXCONS_REC     pxcr )
{
  #undef  MF__
  #define MF__ MOD__"sql92c_kernel_is_alive"
  PSZ                     pszPath  = NULL;
  LONG                    rc       = NO_ERROR;

  DBGPAS;

  pszPath = MAIL_CONS;

  rc = sql41c_open_queue_mailslot ( &pxcr->pidQueueOwner,       // - OS/2 only
                                    &pxcr->hHandle,
                                    pszPath, pxcr->szServerDB );

  if ( rc == ERROR_FILE_NOT_FOUND )
    return FALSE;
  else if ( rc == NO_ERROR )
    sql41c_close_queue_mailslot ( pxcr->hHandle );

  return TRUE;
}


/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
