/*!**************************************************************************

  module      : vos94.c

  -------------------------------------------------------------------------

  responsible : RaymondR

  special area: NT Server Configuration
  description : 


  last changed: 2001-09-24  14:45
  see also    : 

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 2001

  ========== licence begin LGPL
  Copyright (C) 2001 SAP AG

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ========== licence end

*****************************************************************************/



/*
 * INCLUDE FILES
 */
#include "gos00.h"
#include "heo00.h"
#include "heo46.h"
#include "hsp100.h"
#include "heo11.h"
#include "heo05.h"
#include "geo007_1.h"
#include "gos94.h"
#include "gos44.h"


#if !defined ( USER )
# include "vsp004.h"            /* nocheck */
# include "gsp09.h"             /* nocheck */
# include "geo007_2.h"          /* nocheck */
#endif

#if defined ( CLUSTER_SERVER_VERSION )
# include          "gos200.h"   /* nocheck */
# include          "gos212.h"   /* nocheck */
#endif

#include           <stdarg.h>

/*
 *  DEFINES
 */
#define MOD__  "VOS94C : "
#define MF__   MOD__"UNDEFINED"



#define PARAM_KERNEL_VERSION     "KERNELVERSION"

/*
 *  MACROS
 */
#if defined (CTRLCOMP)
  #undef MSGCD
  #define MSGCD(_arg)
#else
  #undef MSG_ERR
  #define MSG_ERR(_arg)
#endif

/*
 *  LOCAL TYPE AND STRUCT DEFINITIONS
 */

typedef struct SERVERDB_PARAM_RECORD {
  BOOL                  fTapeCompressEnabled;
  ULONG                 ulWaitSec;  
  ULONG                 ulTermTimeout;
  DWORD                 ValUseIndepPath; 
  DWORD                 AWEReservedMemory; 
  PATHNAME              szDBRoot;
  PATHNAME              szAEDebug;
  tsp100_VersionID0     RTEVersionID; 
  BOOL                  fDumpEnabled;
  BOOL                  fAlwaysCheckConn;
  } SERVERDB_PARAM_REC;

/*
 * EXTERNAL VARIABLES
 */


/*
 *  EXPORTED VARIABLES
 */


/*
 * LOCAL VARIABLES
 */


/*
 * LOCAL FUNCTION PROTOTYPES
 */
static APIRET sql94_read_parameters  ( PSZ                pszNodeName,
                                       PSZ                pszServiceName,
                                       SERVERDB_PARAM_REC *Params );

static APIRET sql94_alter_parameter  ( SERVERDB_PARAM_REC *Params,
                                       PSZ                pszDBRoot,
                                       PSZ                pszParameter,
                                       PSZ                pszValue );

static APIRET sql94_update_parameters( PSZ                pszNodeName,
                                       PSZ                pszServiceName,
                                       PSZ                pszServerDB,
                                       SERVERDB_PARAM_REC *Params,
                                       tsp09_rte_xerror   *xerror);

static APIRET sql94_get_account_name ( PSZ                pszNodeName,
                                       PSZ                pszServiceName,
                                       PSZ                pszAccountName,
                                       ULONG              ulMaxAccountNameLen );

static PSZ  sql94_version_from_param ( PSZ         pszServerDB,
                                       tsp00_C64c  KernelVersion);

/*
 * ========================== GLOBAL FUNCTIONS ================================
 */

#if defined (CTRLCOMP)

global VOID sqlxregisterdb (tsp09_cstr           dbname,
                            tsp09_cstr           dbroot,
                            tsp09_pgm_kind       pgmKind,
                            tsp09_server_options options,
                            tsp09_rte_xerror     *xerror)
  {
  #undef MF__
  #define MF__ MOD__"sqlxregisterdb"
  BOOL         fAutoStart    = FALSE;
  ULONG        ulKernelType;

  DBGPAS;

  eo44initError (xerror);

  if ( dbname == NULL)
    {
    if ( sql01c_get_serverdb ( & (PSZ)dbname  ) == FALSE )
      {
      if (xerror == NULL)
        {
        printf (SERVICE_USAGE);
        DBGOUT;
        EXITPROCESS(2);
        }
      else
        {
        eo44anyError (xerror, "No dbname given");
        DBGOUT;
        return;
        }
      }
    }

  if ( dbroot == NULL )
    {
    /*
     * get DBROOT from Environment
     */
    if ( sql01c_get_dbroot (& (PSZ)dbroot) == FALSE )
      {
      MSG_ERR ((xerror, ERR_DBROOT_NOT_SET));
      MSGCD ((ERR_DBROOT_NOT_SET));
      DBGOUT;
      if (xerror == NULL)
        {
        EXITPROCESS(2);
        }
      else
        return;
      }
    }

  switch ( pgmKind )
    {
    case csp9_quick_pgm:
      ulKernelType = KERNEL_TYPE_QUICK; break;
    case csp9_slow_pgm:
      ulKernelType = KERNEL_TYPE_SLOW;  break;
    case csp9_test_pgm:
      ulKernelType = KERNEL_TYPE_TEST;  break;
    default:
      ulKernelType = KERNEL_TYPE_FAST;  break;
    }


  fAutoStart = (options & csp9_start_automatic) != 0;

  sql94_service_entry ( UPDATE_SAPDB_SERVICE, ulKernelType,
                        fAutoStart, NULL, (PSZ)dbroot,
                        (PSZ)dbname, NULL, NULL, NULL, NULL, NULL, xerror );


  return;
  }

/*------------------------------*/

global VOID sqlxunregisterdb (tsp09_cstr         dbname,
                              tsp09_pgm_kind     pgmKind,
                              tsp09_rte_xerror*  xerror )
  {
  #undef MF__
  #define MF__ MOD__"sqlxunregisterdb"
  tsp09_pgm_kind      ulKernelType [4];
  int                 count;
  int                 i;
  BOOLEAN             serviceRemoved = FALSE;
  DBGPAS;

  eo44initError (xerror);

  if (pgmKind == csp9_any_pgm)
    {
    count = 4;
    ulKernelType [0] = KERNEL_TYPE_FAST;
    ulKernelType [1] = KERNEL_TYPE_QUICK;
    ulKernelType [2] = KERNEL_TYPE_SLOW;
    ulKernelType [3] = KERNEL_TYPE_TEST;
    }
  else
    {
    count = 1;
    switch ( pgmKind )
      {
      case csp9_quick_pgm:
        ulKernelType[0] = KERNEL_TYPE_QUICK; break;
      case csp9_slow_pgm:
        ulKernelType[0] = KERNEL_TYPE_SLOW;  break;
      case csp9_test_pgm:
        ulKernelType[0] = KERNEL_TYPE_TEST;  break;
      default:
        ulKernelType[0] = KERNEL_TYPE_FAST;  break;
      }
    }

  for (i = 0; i < count; ++i)
    {
    xerror->xe_native_error = UNDEF;

    sql94_service_entry ( REMOVE_SAPDB_SERVICE, ulKernelType[i],
                          FALSE, NULL, "", (PSZ)dbname, NULL, NULL, NULL, NULL, NULL, xerror );

    if (xerror->xe_native_error == UNDEF)
      serviceRemoved = TRUE;
    }

  return;
  }

#endif

/*------------------------------*/

APIRET sql94_service_entry ( ULONG             ulRequest,
                             ULONG             ulKernelType,
                             BOOL              fAutoStart,
                             PSZ               pszNodeName,
                             PSZ               pszDBRoot,
                             PSZ               pszServerDB,
                             PSZ               pszAccountName,
                             PSZ               pszPassword,
                             PSZ               pszParameter,
                             PSZ               pszValue,
                             PSZ               pszPgmName,
                             tsp09_rte_xerror *xerror)
  {
  #undef  MF__
  #define MF__ MOD__"sql94_service_entry"
  CHAR                  szServiceDB[MX_DBNAME + 80];
  CHAR                  szServDisplayDB[MX_DBNAME + 80];
  CHAR                  szServTstDisplayDB[40];
  PSZ                   pszService;
  PSZ                   pszServDisplay;
  PSZ                   pszPgmNamePos;
  CHAR                  szDB[MX_DBNAME + 2];
  CHAR                  szAccountName[80];
  CHAR                  szCurrAccountName[80];
  PATHNAME              szPgmName;
  SC_HANDLE             hScM, hS;
  ULONG                 ulStartOption;
  C40C                  szEventSource;
  DWORD                 dwServiceType;
  tsp100_VersionResult  Result;
  ULONG                 ulTmp       = 0;
  LONG                  lAtt        = 0;
  SC_LOCK                ScLock      = NULL;
  APIRET                rc          = NO_ERROR;
  static STARTUPINFO    StartupInfo = { sizeof(STARTUPINFO) };
  PSECURITY_DESCRIPTOR  pSD         = NULL;
  SERVERDB_PARAM_REC    Params      = { TRUE,
                                        DEFAULT_TAPE_LOAD_RETRY_TIME,
                                        DEFAULT_TERMINATION_TIMEOUT,
										                    -1,
                                        0,
                                        "",
                                        DR_WATSON_EXENAME,
                                        FALSE };

  DBGIN;


  if ((pszAccountName != NULL ) && (strrchr ( pszAccountName, '\\' ) == NULL))
    {
    szAccountName[sizeof(szAccountName) - 1];
    strcpy (szAccountName, ".\\");
    strncat(szAccountName, pszAccountName, sizeof(szAccountName) - 3 );
    pszAccountName = szAccountName;
    }

  strncpy(szDB, pszServerDB, MX_DBNAME + 1);
  pszServerDB = szDB;
  CharUpperBuff(pszServerDB, (DWORD)strlen(pszServerDB));

  CharUpperBuff(pszDBRoot, (DWORD)strlen(pszDBRoot));
  lstrcpy(szPgmName, pszDBRoot);

  // --- create program name
  //
  if ( pszDBRoot[strlen(pszDBRoot) - 1] != '\\' )
    lstrcat(szPgmName, "\\");

  switch ( ulKernelType )
    {
    case KERNEL_TYPE_QUICK:
      strcat(szPgmName, DB_QUICK_KERN_FILE_NAME);
      break;
    case KERNEL_TYPE_TEST:
      if ( pszPgmName == NULL )
        strcat(szPgmName, DB_TEST_KERN_FILE_NAME);
      else
      {
        strcat(szPgmName, DB_KERN_PATH_NAME);
        strcat(szPgmName, pszPgmName );
      }
      break;
    case KERNEL_TYPE_SLOW:
      strcat(szPgmName, DB_SLOW_KERN_FILE_NAME);
      break;
    default:
      strcat(szPgmName, DB_FAST_KERN_FILE_NAME);
      break;
    }

  // --- create service names
  //
  strcpy(szServiceDB,     SERVICE_ID);
  strcat(szServiceDB,     pszServerDB);
  strcpy(szServDisplayDB, SERVICE_DISPLAY_STR);
  strcat(szServDisplayDB, pszServerDB);
  pszService     = szServiceDB;
  pszServDisplay = szServDisplayDB;

  switch ( ulKernelType )
    {
    case KERNEL_TYPE_QUICK:

      strcat(pszServDisplay, SERVICE_QUICK_EXT);
      strcat(pszService,     SERVICE_QUICK_EXT);
      break;

    case KERNEL_TYPE_SLOW:
      strcat(pszServDisplay, SERVICE_SLOW_EXT);
      strcat(pszService,     SERVICE_SLOW_EXT);
      break;

    case KERNEL_TYPE_TEST:
      pszPgmNamePos = strrchr ( szPgmName, '\\' );
      pszPgmNamePos = pszPgmNamePos != NULL ? pszPgmNamePos + 1 : szPgmName;
      sprintf ( szServTstDisplayDB, " (%.20s)", pszPgmNamePos );
      strcat(pszServDisplay, szServTstDisplayDB);
      strcat(pszService,     SERVICE_TEST_EXT);
      break;
    }

  if (( ulRequest == CREATE_SAPDB_SERVICE ) ||
      ( ulRequest == UPDATE_SAPDB_SERVICE ))
    {
    rc = sql44c_get_file_info ( szPgmName, &lAtt, &ulTmp, &ulTmp );

    if ( rc == ERROR_FILE_NOT_FOUND )
      {
      MSG_ERR ((xerror,ERR_FILE_NOT_FOUND, szPgmName));
      MSGCD((ERR_FILE_NOT_FOUND, szPgmName ));

      DBGOUT;
      return (rc);
      }
    else if ( rc == ERROR_ACCESS_DENIED )
      {
      MSG_ERR ((xerror,ERR_ACCESS_DENIED_ON_XX, szPgmName));
      MSGCD((ERR_ACCESS_DENIED_ON_XX, szPgmName ));

      DBGOUT;
      return (rc);
      }
    else
      rc = NO_ERROR;
    }

  if (hScM = OpenSCManager( pszNodeName, NULL, SC_MANAGER_ALL_ACCESS ))
    {
    // --- does the service exist!
    //
    if ( ulRequest == UPDATE_SAPDB_SERVICE )
      {
      if (hS = OpenService(hScM, pszService, SERVICE_START))
        CloseServiceHandle(hS);
      else if ( GetLastError() == ERROR_SERVICE_DOES_NOT_EXIST )
        ulRequest = CREATE_SAPDB_SERVICE;
      }

    //
    //
    // ---- UPDATE SERVICE 
    //
    //
    if ( ulRequest == UPDATE_SAPDB_SERVICE )
      {
	  DWORD UseIndepPath = eo01_IsEnvVarDoNotUseIndepPathSet() ? 0 : 1 ;
      sql94_read_parameters ( pszNodeName, pszService, &Params );

      sql94_get_account_name ( pszNodeName, pszService, szCurrAccountName, 
                               sizeof(szCurrAccountName) );

      Result = sp100_CompareVersionIDs(&Params.RTEVersionID);

      if ( ( pszAccountName && stricmp(szCurrAccountName, pszAccountName) ) ||
           ( stricmp(pszDBRoot, Params.szDBRoot) != 0 )                     ||
           ( pszParameter                        != NULL )                  ||
           ( Result                              != IsEqual_esp100 )        ||
		       ( Params.ValUseIndepPath              != UseIndepPath ))
        {
          if ( pszParameter == NULL )
            {
            MSG_ERR ((xerror, INFO_UPDATING_SERVICE, pszServDisplay ));
            MSGCD ((INFO_UPDATING_SERVICE, pszServDisplay ));
            }

        rc = sql94_alter_parameter ( &Params, pszDBRoot, pszParameter, pszValue );

        if ( rc == NO_ERROR )
          {
          rc = sql94_update_parameters( pszNodeName, pszService,
                                        pszServerDB, &Params,
                                        xerror );
          }
        
        if ( rc == NO_ERROR )
          {
          if ((ScLock = LockServiceDatabase (hScM)) == NULL)
            rc = GetLastError();
          else
            {
            if ( hS = OpenService(hScM, pszService, SERVICE_CHANGE_CONFIG) )
              {
              if (( pszAccountName == NULL ) || ( pszAccountName[0] == '\0' ))
                dwServiceType = SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS;
              else
                dwServiceType = SERVICE_NO_CHANGE;

              if (ChangeServiceConfig ( hS, dwServiceType,
                                        SERVICE_NO_CHANGE, SERVICE_NO_CHANGE,
                                        szPgmName, NULL, NULL, NULL, 
                                        pszAccountName, pszPassword,
                                        pszServDisplay ) == FALSE )
                {
                rc = GetLastError();
                }

              if ( !CloseServiceHandle(hS) && rc == NO_ERROR )
                rc = GetLastError();
              }
            UnlockServiceDatabase (ScLock);
            }
          }
                //
           // --- set log on as a service right
        //
        if (( rc             == NO_ERROR ) && 
            ( pszAccountName != NULL )     && 
            ( pszPassword    != NULL ))
          {
          rc = sql49c_set_service_logon_right ( pszNodeName, pszAccountName );

          if ( rc == ERROR_ALREADY_INITIALIZED )
            rc = NO_ERROR;
          else if ( rc == NO_ERROR )
            MSGCD ((INFO_LOGON_AS_SERV_RIGHT_GRNT, 
                    (pszAccountName[0] == '.' && pszAccountName[1] == '\\') ? 
                    pszAccountName + 2 : pszAccountName));
          }

        }
      }
    //
    //
    // ---- REMOVE SERVICE 
    //
    //
    else if ( ulRequest == REMOVE_SAPDB_SERVICE )
      {
      if ((ScLock = LockServiceDatabase (hScM)) == NULL)
        rc = GetLastError();
      else
        {
        if ( hS = OpenService(hScM, pszService, DELETE) )
          {
          if ( !DeleteService(hS) )
            rc = GetLastError();
          else
            {
            if ( ulKernelType == KERNEL_TYPE_FAST )
              {
              strcpy ( szEventSource, SERVERDB_EVENT_SOURCE_NAME );
              strcat( szEventSource, pszServerDB );

              sql60_remove_event_source ( pszNodeName,
                                          szEventSource );
              }
            }

          if ( !CloseServiceHandle(hS) && rc == NO_ERROR )
            rc = GetLastError();
          }
        else
          rc = GetLastError();
        }
      UnlockServiceDatabase (ScLock);

#     if defined CLUSTER_SERVER_VERSION
       if ( ulKernelType == KERNEL_TYPE_FAST )
         os212_RemoveSERVERDBClusterResource ( pszServerDB, 0 );
#     endif
      }

    //
    //
    // ---- CREATE SERVICE 
    //
    //
    if (( ulRequest == CREATE_SAPDB_SERVICE  ) && ( rc == NO_ERROR ))
      {
      if ((ScLock = LockServiceDatabase (hScM)) == NULL)
        rc = GetLastError();
      else
        {
        //
        // --- check if service already exists?
        //
         if (hS = OpenService(hScM, pszService, SERVICE_START ))
          {
          CloseServiceHandle(hS);
          rc = ERROR_SERVICE_EXISTS;
          }
        else if ( (rc = GetLastError()) == ERROR_SERVICE_DOES_NOT_EXIST )
          rc = NO_ERROR;
        }
   
      if ( rc == NO_ERROR )
        {
        if ( fAutoStart )
          ulStartOption = SERVICE_AUTO_START;
        else
          ulStartOption = SERVICE_DEMAND_START;


        if (( pszAccountName == NULL ) || ( pszAccountName[0] == '\0' ))
          dwServiceType = SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS;
        else
          dwServiceType = SERVICE_WIN32_OWN_PROCESS;

        if ((hS = CreateService(hScM, pszService, pszServDisplay,
                                SERVICE_ALL_ACCESS, dwServiceType,
                                ulStartOption, SERVICE_ERROR_NORMAL,
                                szPgmName, NULL, NULL, NULL, 
                                pszAccountName, pszPassword )) == NULL)
          {
          rc = GetLastError();
          }
        else
          {
          rc = sql49c_alloc_and_init_service_SD ( &pSD, hS );

          if ( (rc == NO_ERROR ) && ( pSD != NULL ))
            {
            if (!SetServiceObjectSecurity( hS, DACL_SECURITY_INFORMATION, pSD ))
              rc = GetLastError ();
            }

          if (!CloseServiceHandle(hS) && rc == NO_ERROR)
            rc = GetLastError();

          if ( rc == NO_ERROR )
            {
            rc = sql94_alter_parameter ( &Params, pszDBRoot, pszParameter, pszValue );

            if ( rc == NO_ERROR )
              {
              rc = sql94_update_parameters( pszNodeName, pszService,
                                            pszServerDB, &Params,
                                            xerror );
              }
            }

          //
          // --- set log on as a service right
          //
          if (( rc == NO_ERROR ) && ( pszAccountName != NULL ) && ( pszPassword != NULL ))
            { 
            rc = sql49c_set_service_logon_right ( pszNodeName, pszAccountName );

            if ( rc == ERROR_ALREADY_INITIALIZED )
              rc = NO_ERROR;
            else if ( rc == NO_ERROR )
              {
              MSGCD ((INFO_LOGON_AS_SERV_RIGHT_GRNT, 
                     (pszAccountName[0] == '.' && pszAccountName[1] == '\\') ? 
                       pszAccountName + 2 : pszAccountName));
              }
            }

          if (( rc != NO_ERROR ) && ( hS = OpenService(hScM, pszService, DELETE) ))
            {
            DeleteService(hS);
            CloseServiceHandle(hS);
            }
          }
        }
      
         if ( ScLock != NULL )
        UnlockServiceDatabase (ScLock);

#       if defined CLUSTER_SERVER_VERSION
         if ( ulKernelType == KERNEL_TYPE_FAST )
           os212_AddSERVERDBClusterResource ( pszServerDB, "service entry" );
#       endif
      }

    if (!CloseServiceHandle(hScM) && rc == NO_ERROR)
      rc = GetLastError();
    }
  else
    rc = GetLastError();

  switch (rc)
    {
    case ERROR_INVALID_PARAMETER:         DBGOUT;
                                          return (rc);
    case NO_ERROR:                        DBGOUT;
                                          return (rc);
    case ERROR_SERVICE_DATABASE_LOCKED:   MSG_ERR ((xerror,ERR_SERVICE_DB_LOCKED));
                                          MSGCD(( ERR_SERVICE_DB_LOCKED ));
                                          break;
    case ERROR_INVALID_SERVICE_ACCOUNT:   MSG_ERR ((xerror,ERR_SERVICE_ACCOUNT_INVALID));
                                          MSGCD(( ERR_SERVICE_ACCOUNT_INVALID ));
                                          break;
    case ERROR_SERVICE_EXISTS:            MSG_ERR ((xerror,INFO_SERVICE_ALREADY_INST, pszServDisplay));
                                          MSGCD((INFO_SERVICE_ALREADY_INST, pszServDisplay));
                                          break;
    case ERROR_FILE_NOT_FOUND:
    case ERROR_PATH_NOT_FOUND:            MSG_ERR ((xerror,INFO_SERVICE_NOT_INST_CORR));
                                          MSGCD((INFO_SERVICE_NOT_INST_CORR));
                                          break;
    case ERROR_SERVICE_DOES_NOT_EXIST:    MSG_ERR ((xerror,INFO_SERVICE_NOT_INSTALLED, pszServDisplay));
                                          MSGCD((INFO_SERVICE_NOT_INSTALLED, pszServDisplay));
                                          break;
    case ERROR_SERVICE_MARKED_FOR_DELETE: MSG_ERR ((xerror,INFO_SERVICE_DELETE_MARKED));
                                          MSGCD((INFO_SERVICE_DELETE_MARKED));
                                          break;
    case RPC_S_SERVER_UNAVAILABLE:        MSG_ERR ((xerror,ERR_SERVER_NODE_UNKNOWN, pszNodeName));
                                          MSGCD((ERR_SERVER_NODE_UNKNOWN, pszNodeName));
                                          break;
    case ERROR_ACCESS_DENIED:             MSG_ERR ((xerror,ERR_CREAT_SERVICE_NO_ACCESS));
                                          MSGCD((ERR_CREAT_SERVICE_NO_ACCESS));
                                          break;
    default:                              MSG_ERR ((xerror,ERR_SERVICE_ERROR, rc));
                                          MSGCD((ERR_SERVICE_ERROR, rc));
    }

  DBGOUT;
  return ( rc );
  }


/*
 * ========================== LOCAL FUNCTIONS =================================
 */

static APIRET sql94_get_account_name ( PSZ                pszNodeName,
                                       PSZ                pszServiceName,
                                       PSZ                pszAccountName,
                                         ULONG              ulMaxAccountNameLen )
  { 
  #undef  MF__
  #define MF__ MOD__"sql94_get_account_name"
  APIRET              rc  = NO_ERROR;
  PATHNAME            szKey;
  REG_ENTRY_REC       RegistryEntries[1];
  ULONG               ulValues  = 0;
  

  DBGIN;

  pszAccountName[0] ='\0';

  strcpy ( szKey, pszServiceName );
  
  RegistryEntries[ulValues].pszValueName = REG_VN_OBJECT_NAME;
  RegistryEntries[ulValues].pValue       = pszAccountName;
  RegistryEntries[ulValues].ulValueSize  = ulMaxAccountNameLen;
  RegistryEntries[ulValues].ulValueType  = REG_SZ;
  ulValues++;


  // --- check array size
  RTE_ASSERT( sizeof (RegistryEntries)/sizeof(RegistryEntries[0]) < ulValues,
             (ERR_ASSERT, MF__, __LINE__));

  rc = sql50_reg_get_service_values ( pszNodeName, szKey,
                                      ulValues, RegistryEntries );



  DBGOUT;
  return ( rc );
  }

/*------------------------------*/

static APIRET sql94_read_parameters ( PSZ                pszNodeName,
                                      PSZ                pszServiceName,
                                      SERVERDB_PARAM_REC *Params )
  {
  #undef  MF__
  #define MF__ MOD__"sql94_read_parameters"
  APIRET              rc  = NO_ERROR;
  PATHNAME            szKey;
  REG_ENTRY_REC       RegistryEntries[8];
  ULONG               ulValues  = 0;
  

  DBGIN;

  strcpy ( szKey, pszServiceName );
  strcat ( szKey, "\\"REG_SK_SERVICE_PARAM );

  RegistryEntries[ulValues].pszValueName = REG_VN_DBROOT;
  RegistryEntries[ulValues].pValue       = Params->szDBRoot;
  RegistryEntries[ulValues].ulValueSize  = sizeof(Params->szDBRoot);
  RegistryEntries[ulValues].ulValueType  = REG_SZ;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_VERSION_ID;
  RegistryEntries[ulValues].pValue       = &(Params->RTEVersionID);
  RegistryEntries[ulValues].ulValueSize  = sizeof(Params->RTEVersionID);
  RegistryEntries[ulValues].ulValueType  = REG_BINARY;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_TAPE_COMPRESS;
  RegistryEntries[ulValues].pValue       = &Params->fTapeCompressEnabled;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_TAPE_LOAD_RETRY_TIME;
  RegistryEntries[ulValues].pValue       = &Params->ulWaitSec;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_TERMINATION_TIMEOUT;
  RegistryEntries[ulValues].pValue       = &Params->ulTermTimeout;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_AEDEBUG;
  RegistryEntries[ulValues].pValue       = Params->szAEDebug;
  RegistryEntries[ulValues].ulValueSize  = sizeof(Params->szAEDebug);
  RegistryEntries[ulValues].ulValueType  = REG_SZ;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_DUMP_ENABLED;
  RegistryEntries[ulValues].pValue       = &Params->fDumpEnabled;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_ALWAYS_CHECK_CONN;
  RegistryEntries[ulValues].pValue       = &Params->fAlwaysCheckConn;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  // --- check array size
  RTE_ASSERT( sizeof (RegistryEntries)/sizeof(RegistryEntries[0]) < ulValues,
             (ERR_ASSERT, MF__, __LINE__));

  rc = sql50_reg_get_service_values ( pszNodeName, szKey,
                                      ulValues, RegistryEntries );

  
  // --- Optional values. Ignoring return code from 'sql50_reg_get_service_values'
  ulValues = 0;
  RegistryEntries[ulValues].pszValueName = REG_VN_USE_INDEP_PATH;
  RegistryEntries[ulValues].pValue       = &Params->ValUseIndepPath;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_AWE_RESERVED_MEMORY;
  RegistryEntries[ulValues].pValue       = &Params->AWEReservedMemory;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  sql50_reg_get_service_values ( pszNodeName, szKey, ulValues, RegistryEntries );

  DBGOUT;
  return ( rc );
  }

/*------------------------------*/

static APIRET sql94_alter_parameter ( SERVERDB_PARAM_REC *Params,
                                      PSZ                pszDBRoot,
                                      PSZ                pszParameter,
                                      PSZ                pszValue )
  {
  #undef  MF__
  #define MF__ MOD__"sql94_alter_parameter"
  APIRET              rc  = NO_ERROR;
  LONG                lValue = 0;
  CHAR                szValue[30];

  DBGIN;


  // --- set current DBROOT
  strcpy ( Params->szDBRoot, pszDBRoot);

  if ( pszParameter == NULL )
    {
    DBGOUT;
    return ( NO_ERROR );
    }

  
  // --- skip blanks
  for ( ;; pszValue++ ) 
    {
    if ( *pszValue != ' ' ) 
      break;
    }

  if ( !stricmp ( REG_VN_AEDEBUG, pszParameter ) )                   // --- Debugger 
    {
    if ( sizeof(Params->szAEDebug) <= strlen (pszValue) )
      {
      rc = ERROR_INVALID_PARAMETER;
      MSGCD ((ERR_VALUE_TOO_LONG, pszParameter ));
      }
    else
        strcpy ( Params->szAEDebug, pszValue );
    }
  else if ( !stricmp ( REG_VN_TAPE_COMPRESS, pszParameter ) )        // --- TapeCompression
    {
    lValue = atol( pszValue );

    if ( stricmp ( pszValue, _ltoa ( (LONG)(lValue & 0x1), szValue, 10)) ) 
      {
      rc = ERROR_INVALID_PARAMETER;
      MSGCD ((ERR_INVALID_VALUE, pszParameter, pszValue ));
      }
    else
      Params->fTapeCompressEnabled = (BOOL)lValue;
    }
  else if ( !stricmp ( REG_VN_DUMP_ENABLED, pszParameter ) )         // --- sqladump
    {
    lValue = atol( pszValue );

    if ( stricmp ( pszValue, _ltoa ( (LONG)(lValue & 0x1), szValue, 10)) ) 
      {
      rc = ERROR_INVALID_PARAMETER;
      MSGCD ((ERR_INVALID_VALUE, pszParameter, pszValue ));
      }
    else
      Params->fDumpEnabled = (BOOL)lValue;
    }
  else if ( !stricmp ( REG_VN_ALWAYS_CHECK_CONN, pszParameter ) )   // --- AlwaysCheckConn
    {
    lValue = atol( pszValue );

    if ( stricmp ( pszValue, _ltoa ( (LONG)(lValue & 0x1), szValue, 10)) ) 
      {
      rc = ERROR_INVALID_PARAMETER;
      MSGCD ((ERR_INVALID_VALUE, pszParameter, pszValue ));
      }
    else
      Params->fAlwaysCheckConn = (BOOL)lValue;
    }
  else if ( !stricmp ( REG_VN_TERMINATION_TIMEOUT, pszParameter ) )      // --- TerminationTimeout
    {
    lValue = atol( pszValue );

    if ( stricmp ( pszValue, _ltoa ( (LONG)(ULONG)lValue, szValue, 10)) ) 
      {
      rc = ERROR_INVALID_PARAMETER;
      MSGCD ((ERR_INVALID_VALUE, pszParameter, pszValue ));
      }
    else
      Params->ulTermTimeout = (ULONG)lValue;
    }
  else if ( !stricmp ( REG_VN_TAPE_LOAD_RETRY_TIME, pszParameter ) ) // --- TWTermTimeout
    {
    lValue = atol( pszValue );

    if ( stricmp ( pszValue, _ltoa ( (LONG)(ULONG)lValue, szValue, 10)) ) 
      {
      rc = ERROR_INVALID_PARAMETER;
      MSGCD ((ERR_INVALID_VALUE, pszParameter, pszValue ));
      }
    else
      Params->ulWaitSec = (ULONG)lValue;
    }
  else if ( !stricmp ( REG_VN_AWE_RESERVED_MEMORY, pszParameter ) ) // --- AWEReservedPhysMemory
    {
    lValue = atol( pszValue );

    if ( stricmp ( pszValue, _ltoa ( (LONG)(ULONG)lValue, szValue, 10)) ) 
      {
      rc = ERROR_INVALID_PARAMETER;
      MSGCD ((ERR_INVALID_VALUE, pszParameter, pszValue ));
      }
    else
      Params->AWEReservedMemory = (ULONG)lValue;
    }
  else
    {
    MSGCD ((ERR_INVALID_PARAMETER, pszParameter ));
    rc = ERROR_INVALID_PARAMETER;
    }



  DBGOUT;
  return ( rc );
  }

/*------------------------------*/

static APIRET sql94_update_parameters( PSZ                pszNodeName,
                                       PSZ                pszServiceName,
                                       PSZ                pszServerDB,
                                       SERVERDB_PARAM_REC *Params,
                                       tsp09_rte_xerror   *xerror)
  {
  #undef  MF__
  #define MF__ MOD__"sql94_update_parameters"
  APIRET              rc  = NO_ERROR;
  PATHNAME            szKey;
  REG_ENTRY_REC       RegistryEntries[12];
  DWORD               ValUseIndepPath;    
  ULONG               ulValues  = 0;
  tsp100_VersionID0   RTEVersionID;
  PSZ                 pszKernelVersion = NULL;
  tsp00_C64c          KernelVersion;
  C40C                szEventSource;
  PATHNAME            szMsgFile;
  PATHNAME            szCategoryFile;
  tsp00_Versionc      RTEVersionString;

  DBGIN;

  strcpy ( szKey, pszServiceName );
  strcat ( szKey, "\\"REG_SK_SERVICE_PARAM );

  // --- set the RTE version string
  if ( sql02_get_platform_id() == VER_PLATFORM_WIN32_NT )
    sp100_GetVersionString(COMP_NAME_RTE_NT_SP100, s100buildnumber, RTEVersionString);
  else
    sp100_GetVersionString(COMP_NAME_RTE_WIN9X_SP100, s100buildnumber, RTEVersionString );

  RegistryEntries[ulValues].pszValueName = REG_VN_VERSION;
  RegistryEntries[ulValues].pValue       = RTEVersionString;
  RegistryEntries[ulValues].ulValueSize  = (ULONG)strlen(RTEVersionString);
  RegistryEntries[ulValues].ulValueType  = REG_SZ;
  ulValues++;

  sp100_GetVersionID ( VersionIDType0_esp100, s100buildnumberEx, &RTEVersionID );
  RegistryEntries[ulValues].pszValueName = REG_VN_VERSION_ID;
  RegistryEntries[ulValues].pValue       = &RTEVersionID;
  RegistryEntries[ulValues].ulValueSize  = sizeof(RTEVersionID);
  RegistryEntries[ulValues].ulValueType  = REG_BINARY;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_DBROOT;
  RegistryEntries[ulValues].pValue       = Params->szDBRoot;
  RegistryEntries[ulValues].ulValueSize  = (ULONG)strlen(Params->szDBRoot);
  RegistryEntries[ulValues].ulValueType  = REG_SZ;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_TAPE_COMPRESS;
  RegistryEntries[ulValues].pValue       = &Params->fTapeCompressEnabled;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_TAPE_LOAD_RETRY_TIME;
  RegistryEntries[ulValues].pValue       = &Params->ulWaitSec;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_TERMINATION_TIMEOUT;
  RegistryEntries[ulValues].pValue       = &Params->ulTermTimeout;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_AEDEBUG;
  RegistryEntries[ulValues].pValue       = Params->szAEDebug;
  RegistryEntries[ulValues].ulValueSize  = (ULONG)strlen(Params->szAEDebug);
  RegistryEntries[ulValues].ulValueType  = REG_SZ;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_DUMP_ENABLED;
  RegistryEntries[ulValues].pValue       = &Params->fDumpEnabled;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  RegistryEntries[ulValues].pszValueName = REG_VN_ALWAYS_CHECK_CONN;
  RegistryEntries[ulValues].pValue       = &Params->fAlwaysCheckConn;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  ValUseIndepPath = eo01_IsEnvVarDoNotUseIndepPathSet() ? 0 : 1 ;
  RegistryEntries[ulValues].pszValueName = REG_VN_USE_INDEP_PATH;
  RegistryEntries[ulValues].pValue       = &ValUseIndepPath;
  RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
  RegistryEntries[ulValues].ulValueType  = REG_DWORD;
  ulValues++;

  if ( Params->AWEReservedMemory != 0 )
  {
    RegistryEntries[ulValues].pszValueName = REG_VN_AWE_RESERVED_MEMORY;
    RegistryEntries[ulValues].pValue       = &Params->AWEReservedMemory;
    RegistryEntries[ulValues].ulValueSize  = sizeof(DWORD);
    RegistryEntries[ulValues].ulValueType  = REG_DWORD;
    ulValues++;
  }

  pszKernelVersion = sql94_version_from_param ( pszServerDB, KernelVersion );

  if ( pszKernelVersion != NULL )
    {
    RegistryEntries[ulValues].pszValueName = REG_VN_KERNEL_VERSION;
    RegistryEntries[ulValues].pValue       = pszKernelVersion;
    RegistryEntries[ulValues].ulValueSize  = (ULONG)strlen(pszKernelVersion);
    RegistryEntries[ulValues].ulValueType  = REG_SZ;
    ulValues++;
    }
  else
    sql50_reg_del_service_value ( pszNodeName, szKey, REG_VN_KERNEL_VERSION );

  // --- check array size
  RTE_ASSERT( sizeof (RegistryEntries)/sizeof(RegistryEntries[0]) < ulValues,
             (ERR_ASSERT, MF__, __LINE__));

  rc = sql50_reg_put_service_values ( pszNodeName, szKey, ulValues,
                                      RegistryEntries );

  if ( rc == NO_ERROR )
    {
    strcpy ( szEventSource, SERVERDB_EVENT_SOURCE_NAME );
    strcat( szEventSource, pszServerDB );

    szMsgFile[0] = '\0';
    sql01c_build_pgm_path ( Params->szDBRoot, szMsgFile ) ;
    strcat ( szMsgFile, REG_MSG_FILE );

    szCategoryFile[0] = '\0';
    strcpy ( szCategoryFile,  Params->szDBRoot );
    strcat ( szCategoryFile, "\\" );
    strcat ( szCategoryFile, DB_FAST_KERN_FILE_NAME );


    rc = sql60_install_event_source ( pszNodeName, 
                                      szEventSource,
                                      szMsgFile,
                                      szCategoryFile,
                                      NTKERN_CATEGORY_LAST);
    }

  DBGOUT;
  return ( rc );
  }

/*------------------------------*/

static PSZ sql94_version_from_param ( PSZ         pszServerDB, 
                                      tsp00_C64c  KernelVersion )
  {
  #undef  MF__
  #define MF__ MOD__"sql94_version_from_param"
  tsp01_RteError          RteError ;

  DBGPAS;

  // --- display no error messages
  sql60_disable_message_output ();

  eo11GetParam (pszServerDB, PARAM_KERNEL_VERSION, KernelVersion, &RteError ) ;

  sql60_enable_message_output();

  return (RteError.RteErrCode == RTE_NO_ERROR_SP01 ? KernelVersion : NULL) ;
  }

/*
 * =============================== END ========================================
 */

