/****************************************************************************

  module      : vos968.c

  -------------------------------------------------------------------------

  responsible : FrankH

  special area: TCP/IP_DATABASE_SERVER
  description : description ...


  last changed: 1999-12-21  17:12
  see also    : example.html ...

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1998



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*****************************************************************************/



/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/
#define RTE_USE_SOCK

#include		   "gos96.h"
#include		   "gos960.h"
#include		   "gos962.h"
#include		   "gos963.h"
#include           "hsp100.h"
#include		   "gos969.h"
#include           "heo01.h"
#include           "geo007_1.h"
#include           "geo007_2.h"
#include           "geo43_0.h"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define MOD__  "VOS968  : "
#define MF__   MOD__"UNDEFINED"

#define TIMEOUT                   120
#define DIAGSIZE                  100                      // - 4KByte Pages

#define OPT_SHOW_USAGE            0x001
#define OPT_START_SERVER          0x002
#define OPT_STOP_SERVER           0x004
#define OPT_START_SERVICE         0x008
#define OPT_STOP_SERVICE          0x010
#define OPT_INSTALL_SERVICE       0x020
#define OPT_REMOVE_SERVICE        0x040
#define OPT_RUN                   0x080
#define OPT_UPDATE_SERVICE        0x100
#define OPT_SHOW_VERSION          0x200


#define OPTION_STRING            "XsirD:RkSn:huVN:"

#define ICON_SERV                 130
#define SM_TRAY_CALLBACK          WM_TIMER  // - do nothing


/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/



/*===========================================================================*
 *  LOCAL CLASSES, STRUCTURES, TYPES, UNIONS ...                             *
 *===========================================================================*/

static int os968Main ( int argc, char  *argv[] ) ;
static void os968_ShowEnviron(char **envp);

/*===========================================================================*
 *  GLOBAL VARIABLES                                                         *
 *===========================================================================*/

extern int                       sql80_OptInd;
extern int                       sql80_OptErr;
extern char*                     sql80_OptArg;
extern HMODULE                   hModule;


/*===========================================================================*
 *  LOCAL VARIABLES                                                          *
 *===========================================================================*/

GLOBAL_XSERVER_REC      gxr;

/*===========================================================================*
 *  STATIC/INLINE FUNCTIONS (PROTOTYPES)                                     *
 *===========================================================================*/

BOOL __stdcall SetServiceBits( IN SERVICE_STATUS_HANDLE hServiceStatus,
                               IN DWORD                 dwServiceBits,
                               IN BOOL                  bSetBitsOn,
                               IN BOOL                  bUpdateImmediately );

static VOID _System os968Finish              ( ULONG  termcode );
static INT          os968Coordinator         ( VOID );
static INT          os968Server              ( VOID );

static BOOL WINAPI  os968ConsoleCtrlHandler  ( DWORD                  dwCtrlType );
static VOID WINAPI  os968ServiceCtrlHandler  ( DWORD                  dwCtrlCode );
static VOID         os968Service             ( VOID );
static BOOL         os968DebuggerIsDrWatson  ( VOID );
static void         os968UpdateVersion       ( VOID );
static LONG         os968_OpenDiagfile       ( PSECURITY_ATTRIBUTES     pSA,
											   DWORD                 dwDiagFileSize );
static teo00_Bool   os968_SetSymbolPath      ();

static int          os968_SupportOldConnections = 1;

/*===========================================================================*
 *  PUBLIC FUNCTIONS (CODE)                                                  *
 *===========================================================================*/
int main ( int argc, char  *argv[] )
{
  int rc ;

#ifdef DBMSRV
  os961InitDdmServerSpecification();
#else
  os960InitXserverSpecification();
#endif
  rc = os968Main(argc, argv);
  return(rc);
}


/*------------------------------*/

VOID  sql96c_abort ( VOID )
  {
  #undef  MF__
  #define MF__ MOD__"sql96c_abort"

  DBGPAS;

  /*
   * activide exception handler
   */
  RaiseException(XCPT_ABORT_EXCEPTION, 0, 0, NULL);

  // ---  should not come here
  EXITPROCESS( 1 );
  }

 /*------------------------------*/

 VOID sql96c_NewSrvState ( DWORD dwState,  ULONG ulExitCode )
   {
   #undef  MF__
   #define MF__ MOD__"sql96c_NewSrvState"
   LONG                 rc = NO_ERROR;

   DBGPAS;

   switch( dwState )
     {
     case XSERVER_INITIALIZE:
       gxr.ssStatus.dwCurrentState            = SERVICE_START_PENDING;
       gxr.ssStatus.dwServiceType             = SERVICE_WIN32_OWN_PROCESS;
       gxr.ssStatus.dwWin32ExitCode           = NO_ERROR;
       gxr.ssStatus.dwServiceSpecificExitCode = NO_ERROR;
       gxr.ssStatus.dwWaitHint                = 10000;
       gxr.ssStatus.dwCheckPoint              = 0;
       gxr.ssStatus.dwControlsAccepted        = SERVICE_ACCEPT_SHUTDOWN;
       return;

     case XSERVER_STARTING:
       gxr.ssStatus.dwCurrentState            = SERVICE_START_PENDING;
       gxr.ssStatus.dwWaitHint                = 30000;
       gxr.ssStatus.dwCheckPoint++;
       break;

     case XSERVER_RUNNING:
       gxr.ssStatus.dwControlsAccepted        = SERVICE_ACCEPT_STOP           |
                                                SERVICE_ACCEPT_SHUTDOWN;

       #if defined ( PAUSE_SERVICE_ALLOWED )
        gxr.ssStatus.dwControlsAccepted      |= SERVICE_ACCEPT_PAUSE_CONTINUE;
       #endif

       if ( gxr.fServicePaused )
         gxr.ssStatus.dwCurrentState          = SERVICE_PAUSED;
       else
         gxr.ssStatus.dwCurrentState          = SERVICE_RUNNING;

       gxr.ssStatus.dwWin32ExitCode           = NO_ERROR;
       gxr.ssStatus.dwServiceSpecificExitCode = NO_ERROR;
       gxr.ssStatus.dwWaitHint                = 0;
       gxr.ssStatus.dwCheckPoint              = 0;
       break;

     case XSERVER_STOPPED:
     case XSERVER_ABORT:
       gxr.ssStatus.dwControlsAccepted        = SERVICE_ACCEPT_STOP |
                                                SERVICE_ACCEPT_SHUTDOWN;
       gxr.ssStatus.dwCurrentState            = SERVICE_STOPPED;
       gxr.ssStatus.dwWaitHint                = 0;
       gxr.ssStatus.dwCheckPoint              = 0;


       if ( ulExitCode != NO_ERROR )
         {
         gxr.ssStatus.dwWin32ExitCode           = ERROR_SERVICE_SPECIFIC_ERROR;
         gxr.ssStatus.dwServiceSpecificExitCode = ulExitCode;
         }
        else
         {
         gxr.ssStatus.dwWin32ExitCode           = NO_ERROR;
         gxr.ssStatus.dwServiceSpecificExitCode = NO_ERROR;
         }
      break;

     default:
      gxr.ssStatus.dwCheckPoint++;
      break;
     }


   // Report the status of the service to the service control manager.
   if (!SetServiceStatus ( gxr.sshStatusHandle, &gxr.ssStatus ))
     {
     rc = GetLastError();
     MSGALL (( ERR_SET_SERVICE_STATUS, rc ));
     }

   return;
   }

/*------------------------------*/

DWORD os968ExceptionHandler (LPEXCEPTION_POINTERS lpExcPtrs )
  {
  #undef  MF__
  #define MF__ MOD__"os968ExceptionHandler"
  INT                    i;
  PSZ                    pszPhysDiagFileName = NULL;
  PEXCEPTION_RECORD      pExceptionRecord    = lpExcPtrs->ExceptionRecord;

  DBGPAS;

  sql60_reset_crit_section();

  switch(pExceptionRecord->ExceptionCode)
    {
    case STATUS_IN_PAGE_ERROR            :
    case STATUS_NONCONTINUABLE_EXCEPTION :
    case STATUS_INVALID_DISPOSITION      :
    case STATUS_ACCESS_VIOLATION         :
    case STATUS_INTEGER_DIVIDE_BY_ZERO   :
    case STATUS_FLOAT_DIVIDE_BY_ZERO     :
    case STATUS_FLOAT_INVALID_OPERATION  :
    case STATUS_ILLEGAL_INSTRUCTION      :
    case STATUS_PRIVILEGED_INSTRUCTION   :
    case STATUS_INTEGER_OVERFLOW         :
    case STATUS_FLOAT_OVERFLOW           :
    case STATUS_FLOAT_UNDERFLOW          :
    case STATUS_FLOAT_DENORMAL_OPERAND   :
    case STATUS_FLOAT_INEXACT_RESULT     :
    case STATUS_FLOAT_STACK_CHECK        :
    case STATUS_DATATYPE_MISALIGNMENT    :
    case STATUS_ARRAY_BOUNDS_EXCEEDED    :
    case STATUS_STACK_OVERFLOW           :

      if ( gxr.dwState != XSERVER_ABORT )
        {
        for (i = pExceptionRecord->NumberParameters; i < EXCEPTION_MAXIMUM_PARAMETERS; i++)
          pExceptionRecord->ExceptionInformation[i] = 0;

        MSGCD ((ERR_XSER_EXCEPTION, pExceptionRecord->ExceptionCode, pExceptionRecord->ExceptionAddress,
                                    pExceptionRecord->ExceptionInformation[0], 
                                    pExceptionRecord->ExceptionInformation[1],
                                    pExceptionRecord->ExceptionInformation[2], 
                                    pExceptionRecord->ExceptionInformation[3]));
        }
      break;

    case XCPT_ABORT_EXCEPTION:
      break;

    case STATUS_CONTROL_C_EXIT:
       switch(pExceptionRecord->ExceptionInformation[0])
         {
         case CTRL_LOGOFF_EVENT   :
         case CTRL_CLOSE_EVENT    :
           if ( gxr.fNTService )
             {
             DBG3 ((MF__, "Exception Ignored!"));
             return(EXCEPTION_CONTINUE_EXECUTION);
             }
           MSGD (( INFO_XSER_EXCEP_SIG_KILL ));
           break;

         case CTRL_SHUTDOWN_EVENT :
           MSGD (( INFO_XSER_EXCEP_SIG_KILL ));
           break;
        case CTRL_C_EVENT       :
        case CTRL_BREAK_EVENT   :
         default                :
           if ( gxr.fNTService )
             {
             DBG3 ((MF__, "Exception Ignored!"));
             return(EXCEPTION_CONTINUE_EXECUTION);
             }
           break;
         }
      break;

    default:
      if ( pExceptionRecord->ExceptionFlags & EXCEPTION_NONCONTINUABLE )
        {
        for (i = pExceptionRecord->NumberParameters; i < EXCEPTION_MAXIMUM_PARAMETERS; i++)
          pExceptionRecord->ExceptionInformation[i] = 0;

        MSGCD ((ERR_XSER_EXCEPTION, pExceptionRecord->ExceptionCode, pExceptionRecord->ExceptionAddress,
                                    pExceptionRecord->ExceptionInformation[0], 
                                    pExceptionRecord->ExceptionInformation[1],
                                    pExceptionRecord->ExceptionInformation[2], 
                                    pExceptionRecord->ExceptionInformation[3]));
        break;
        }
      return(EXCEPTION_CONTINUE_SEARCH);
    }

  if ( pExceptionRecord->ExceptionCode == STATUS_CONTROL_C_EXIT ) // - termination by signal?
    return(EXCEPTION_CONTINUE_SEARCH);

  if ( gxr.dwState != XSERVER_ABORT )  // --- don't display abort
    {                                  //     information twice
    gxr.dwState  = XSERVER_ABORT;

    sql60_get_diag_filename ( &pszPhysDiagFileName );

    if ( pszPhysDiagFileName != NULL)
      {
      MSGCEL (( INFO_XSER_ABORTS_DIAG_INFO, pszPhysDiagFileName ));
      MSGD   (( INFO_XSER_ABORTS ));
      }
    else
      MSGALL (( INFO_XSER_ABORTS ));

    if ( gxr.fNTService )
      sql96c_NewSrvState( gxr.dwState, gxr.ulCrashRc );
    }

  os968Finish (0);

  if ( gxr.fDebuggerIsDrWatson )
    {
    return(EXCEPTION_CONTINUE_SEARCH);  // - DrWatson will write a dump file!
    }

  EXITPROCESS( 1 );
  return(EXCEPTION_CONTINUE_SEARCH);
  }

/*===========================================================================*
 *  LOCAL/PRIVATE/PROTECTED FUNCTIONS (CODE)                                 *
 *===========================================================================*/

static int os968Main ( int argc, char  *argv[] )
  {
  LONG                    rc                = NO_ERROR;
  BOOL                    fCommandError     = FALSE;
  ULONG                   ulStrtOpt;
  INT                     OptionChar;
  CHAR                    szCmdLine[512];
  INT                     i;
  INT                     newDebugLevel = -1;

  PCHAR                  pszNodeName       = NULL;
  SERVICE_TABLE_ENTRY    dispatchTable[]   = { { "INIT",
                                               (LPSERVICE_MAIN_FUNCTION)os968Service },
                                               { NULL, NULL } };

  if ( sql02_get_platform_id() == VER_PLATFORM_WIN32_NT )
    sp100_GetVersionString(COMP_NAME_RTE_NT_SP100, s100buildnumber, gxr.RTEVersionString);
  else
    sp100_GetVersionString(COMP_NAME_RTE_WIN9X_SP100, s100buildnumber, gxr.RTEVersionString);

  dispatchTable[0].lpServiceName = gxr.ServerSpecification.ServerName ;

  gxr.fNTService       = FALSE;
  gxr.OldTCPIPCoord.sd = INVALID_SOCKET;
  gxr.TCPIPCoord.sd    = INVALID_SOCKET;

  ulStrtOpt            = OPT_START_SERVER;


   if ( sql02_get_platform_id() == VER_PLATFORM_WIN32_NT )
     {
     gxr.fNTService = TRUE;
     ulStrtOpt      = OPT_RUN;
     }

  while ( (OptionChar = sql80_GetOpt ( argc, argv, OPTION_STRING )) != -1 )
    {
    switch ( OptionChar )
      {
       case 'X' :  // Suppress support for old 6.1 connections
               os968_SupportOldConnections = 0;
               break;
       case 's' :  // --- hidden parameter
               if ( gxr.fNTService )
                 ulStrtOpt = OPT_START_SERVICE;
               break;
       case 'i' :  // --- hidden parameter
               if ( gxr.fNTService )
                 ulStrtOpt = OPT_INSTALL_SERVICE;
               else
                 fCommandError = TRUE;
               break;
       case 'u' :  // --- hidden parameter
               if ( gxr.fNTService )
                 ulStrtOpt = OPT_UPDATE_SERVICE;
               else
                 fCommandError = TRUE;
               break;

       case 'r' :
               if ( gxr.fNTService )
                 ulStrtOpt = OPT_REMOVE_SERVICE;
               else
                 fCommandError = TRUE;
               break;
       case 'D' :
               if (( ulStrtOpt   != OPT_INSTALL_SERVICE ) &&
                   ( ulStrtOpt   != OPT_REMOVE_SERVICE  ) &&
                   ( pszNodeName == NULL  ))
                 {
                 gxr.fNTService   = FALSE;
                 gxr.usDebugLevel = sql80_OptArg[ 0 ] - '0';
                 }
               else
                 fCommandError = TRUE;
               break;
       case 'N' : /* PTS 1111208 */
               if (( ulStrtOpt   == OPT_INSTALL_SERVICE ) ||
                   ( ulStrtOpt   == OPT_REMOVE_SERVICE  ) ||
                   ( ulStrtOpt   == OPT_UPDATE_SERVICE  ) ) {
                 fCommandError = TRUE;
			   }
			   else
			   {
                 newDebugLevel = sql80_OptArg[ 0 ] - '0';
			   }
               break;
       case 'R' :  // --- hidden parameter
               if ( pszNodeName  == NULL  )
                 {
                 gxr.fNTService = FALSE;
                 ulStrtOpt      = OPT_RUN;
                 }
               else
                 fCommandError = TRUE;

               break;
       case 'n' :
               if ( gxr.fNTService )
                 {
                 pszNodeName  = sql80_OptArg;

                 if ( ulStrtOpt  == OPT_RUN )     // - this option is not
                   ulStrtOpt = OPT_START_SERVICE; //   allowed, assuming start
                 }                                //   request.
               else
                 fCommandError = TRUE;
               break;
       case 'k' :
               if ( gxr.fNTService )
                 ulStrtOpt = OPT_STOP_SERVICE;
               else
                 ulStrtOpt = OPT_STOP_SERVER;
               break;
      case 'h' :
              ulStrtOpt = OPT_SHOW_USAGE;
              break;

      case 'S' :  // --- hidden parameter
              gxr.fSuppressXserverActiveMsg = TRUE;
              break;
      case 'V' :  // --- CR 1104459
              ulStrtOpt = OPT_SHOW_VERSION ;
              break;
      default :
              fCommandError = TRUE;
      }
    }

  if ( (argc - sql80_OptInd) || (fCommandError == TRUE) )
    {
    printf ( XSERVER_NO_VALID_OPTION );

    DBGOUT;
    return ( 1 );
    }

  if ( ulStrtOpt == OPT_SHOW_USAGE )
    {
     if ( sql02_get_platform_id() == VER_PLATFORM_WIN32_NT )
       printf ( XSERVER_USAGE_NT );
     else
       printf ( XSERVER_USAGE_WIN95 );

    DBGOUT;
    return ( 0 );
    }

  // --- CR 1104459
  if ( ulStrtOpt == OPT_SHOW_VERSION )
  {
    printf("%s\n",gxr.RTEVersionString);
    return ( 0 );
  }

  if ( newDebugLevel >= 0 )
  { /* PTS 1111208 */
	  tsp00_ErrTextc errTextC;
      (void)sql43_tcpip_init ();
	  if ( !pszNodeName ) pszNodeName = "localhost";
	  rc = os23NewDebugLevel(pszNodeName, newDebugLevel, errTextC);
	  if ( NO_ERROR == rc )
	  {
		  printf("New debug level %d\n", newDebugLevel );
	  }
	  else
	  {
		  printf("Failed to set new debug level:%s\n", &errTextC[0]);
	  }
	  DBGOUT;
	  return ( rc );
  }
   //
   // --- NT Service
   //
   if ( gxr.fNTService )
     {
     if (( ulStrtOpt == OPT_INSTALL_SERVICE ) ||
         ( ulStrtOpt == OPT_UPDATE_SERVICE ))
       {
       if ( ulStrtOpt == OPT_INSTALL_SERVICE )
         rc = os969InstallAnyXserverService ( pszNodeName, gxr.ServerSpecification.ServerName,TRUE );
       else
         rc = os969UpdateAnyServerService ( pszNodeName, gxr.ServerSpecification.ServerName, TRUE );
       }
     else if ( ulStrtOpt == OPT_START_SERVICE )
       {
       rc = os969StartAnyServerService ( pszNodeName, gxr.ServerSpecification.ServerName );

       if ( rc == NO_ERROR )
         MSGCD (( INFO_XSER_STARTED ));
         if ( 0 == os968_SupportOldConnections )
         {
           MSGD (( INFO_XSER_NEW_ONLY ));
         }
       }
     else if ( ulStrtOpt == OPT_STOP_SERVICE )
       rc = os969StopAnyServerService ( pszNodeName, gxr.ServerSpecification.ServerName );
     else if ( ulStrtOpt == OPT_REMOVE_SERVICE )
       rc = os969RemoveAnyXserverService ( pszNodeName, gxr.ServerSpecification.ServerName );
     else
       {
       sql60_open_event_log ( gxr.ServerSpecification.ServerName, EVENT_SOURCE_NAME );

       if (!StartServiceCtrlDispatcher(dispatchTable))
         {
         rc = GetLastError();

         MSGALL (( ERR_START_SERVICE, gxr.ServerSpecification.ServerName, rc ));

         if ( rc == ERROR_FAILED_SERVICE_CONTROLLER_CONNECT )
           {
           gxr.fNTService = FALSE;

           rc = os968Server ();
           }
         }
       }

     DBGOUT;
     return ( rc );
     }

  // --- Console application (SERVER)
  //
  if ( ulStrtOpt == OPT_START_SERVER )
    {
    szCmdLine[0] = '\0';

    for ( i = 1; i < argc; i++ )
      {
      strcat(szCmdLine, " ");
      strcat(szCmdLine, argv[i]);
      }

    rc = os969StartAnyServer ( szCmdLine, gxr.ServerSpecification.ServerName, gxr.ServerSpecification.ServerSemaName, gxr.fSuppressXserverActiveMsg );

    if ( rc == NO_ERROR )
      MSGCD (( INFO_XSER_STARTED ));
    }
  else if ( ulStrtOpt == OPT_STOP_SERVER )
    rc = os969StopAnyServer ( gxr.ServerSpecification.ServerName, gxr.ServerSpecification.ServerSemaName );
  else
    rc = os968Server ();

  DBGOUT;
  return ( rc );
  }

 /*------------------------------*/

 static HICON sql96c_get_icon_from_resource( UINT id )
   {
   #undef  MF__
   #define MF__ MOD__"sql96c_get_icon_from_resource"
   HICON                 hIcon = 0;
   HINSTANCE             hExe;           // handle to loaded .EXE file
   HRSRC                 hResource;      // handle for FindResource
   HRSRC                 hMem;           // handle for LoadResource
   BYTE                  *lpResource;    // address of resource data
   int                   nID;            // ID of resource that best fits current screen

   DBGIN;

   if ( (hExe = GetModuleHandle ( NULL ))                                   &&
       (hResource  = FindResource(hExe,MAKEINTRESOURCE(id),RT_GROUP_ICON)) &&
       (hMem       = LoadResource(hExe, hResource))                        &&
       (lpResource = (BYTE *)LockResource(hMem))                           &&
       (nID        = LookupIconIdFromDirectoryEx((PBYTE) lpResource, TRUE,
                                                 0, 0, LR_DEFAULTCOLOR))   &&
       (hResource  = FindResource(hExe, MAKEINTRESOURCE(nID),
                                  MAKEINTRESOURCE(RT_ICON)))               &&
       (hMem       = LoadResource(hExe, hResource))                        &&
       (lpResource = (BYTE *)LockResource(hMem))                                   &&
       (hIcon      = CreateIconFromResourceEx((PBYTE) lpResource,
                                               SizeofResource(hExe,hResource),
                                               TRUE, 0x00030000,
                                               0, 0, LR_DEFAULTCOLOR)))
    {
    DBGOUT
    return (hIcon);
    }


  DBGOUT
  return (0);
  }

 /*------------------------------*/

 static VOID sql96c_tray_message( HWND hWnd, UINT message)
   {
   #undef  MF__
   #define MF__ MOD__"sql96c_tray_message"
   NOTIFYICONDATA        tnd;
   static HICON          hIcon = 0;

   DBGIN;

   if ( hIcon == 0)
     hIcon = sql96c_get_icon_from_resource( ICON_SERV );

   wsprintf(tnd.szTip, XSERVER_ACTIVE );

   switch (message)
     {
     case NIM_ADD:
       tnd.uFlags = NIF_MESSAGE|NIF_ICON|NIF_TIP;
       break;

     case NIM_MODIFY:
       tnd.uFlags = NIF_TIP|NIF_ICON;
       break;

     case NIM_DELETE:
       tnd.uFlags = 0;
       break;
     }

   if ( hIcon )
     {
     tnd.uID              = (UINT)ICON_SERV;
     tnd.cbSize           = sizeof(NOTIFYICONDATA);
     tnd.hWnd             = hWnd;
     tnd.uCallbackMessage = SM_TRAY_CALLBACK;
     tnd.hIcon            = hIcon;
     Shell_NotifyIcon(message, &tnd);
     }

   DBGOUT
   return;
   }

 /*------------------------------*/

 static VOID os968Service ( VOID )
   {
   #undef  MF__
   #define MF__ MOD__"os968Service"
   LONG                    rc = NO_ERROR;
   PSZ                     pszUserName;
   BOOL                    fOk;
   SECURITY_ATTRIBUTES     SA;
   ACCESS_RIGHTS_REC       Access;
   REG_ENTRY_REC           RegistryEntries[1];
   DWORD                   dwValUseIndepPath = 0;
   PATHNAME                szKey;
   DWORD                   dwDebugLevel;
   DWORD                   dwDiagFileSize  = DIAGSIZE;

   DBGIN;

   __try
   {
    sql60_disable_default_device ();

    // --- initialize service control
    gxr.dwState  = XSERVER_INITIALIZE;
    sql96c_NewSrvState( gxr.dwState, NO_ERROR );

    gxr.sshStatusHandle = RegisterServiceCtrlHandler( gxr.ServerSpecification.ServerName, os968ServiceCtrlHandler );

    if ( gxr.sshStatusHandle == 0 )
      {
      rc = GetLastError();
      MSGALL (( ERR_SET_SERVICE_STATUS, rc ));
      }
    else
      {
      SetServiceBits ( gxr.sshStatusHandle,
                       SV_TYPE_SAPDB, TRUE, TRUE );

      gxr.dwState  = XSERVER_STARTING;
      sql96c_NewSrvState( gxr.dwState, NO_ERROR );

      gxr.fDebuggerIsDrWatson = os968DebuggerIsDrWatson();
 
//    PTS 1107043
      strcpy ( szKey, gxr.ServerSpecification.ServerName );
      strcat ( szKey, "\\"REG_SK_SERVICE_PARAM );
      RegistryEntries[0].pszValueName        = REG_VN_USE_INDEP_PATH;
      RegistryEntries[0].pValue              = &dwValUseIndepPath;
      RegistryEntries[0].ulValueSize         = sizeof(DWORD);
      RegistryEntries[0].ulValueType         = REG_DWORD;

      if (sql50_reg_get_service_values ( NULL, szKey, 1, RegistryEntries ) == NO_ERROR )
        {
		    if ( !dwValUseIndepPath ) 
          eo01_PutEnvVarDoNotUseIndepPath();
        }

      strcpy ( szKey, gxr.ServerSpecification.ServerName );
      strcat ( szKey, "\\"REG_SK_SERVICE_PARAM );
      RegistryEntries[0].pszValueName        = REG_VN_DEBUG_LEVEL;
      RegistryEntries[0].pValue              = &dwDebugLevel;
      RegistryEntries[0].ulValueSize         = sizeof(DWORD);
      RegistryEntries[0].ulValueType         = REG_DWORD;

      if (sql50_reg_get_service_values ( NULL, szKey, 1, RegistryEntries ) == NO_ERROR)
        {
  	    gxr.usDebugLevel = (SHORT)dwDebugLevel;
        }

      strcpy ( szKey, gxr.ServerSpecification.ServerName );
      strcat ( szKey, "\\"REG_SK_SERVICE_PARAM );
      RegistryEntries[0].pszValueName        = REG_VN_DIAGFILE_SIZE;
      RegistryEntries[0].pValue              = &dwDiagFileSize;
      RegistryEntries[0].ulValueSize         = sizeof(DWORD);
      RegistryEntries[0].ulValueType         = REG_DWORD;
      
      if (sql50_reg_get_service_values ( NULL, szKey, 1, RegistryEntries ) != NO_ERROR )
        dwDiagFileSize = DIAGSIZE * (gxr.usDebugLevel + 1);

      if ( !os968_SetSymbolPath() ) /* PTS 1104558 */
      {
        MSGALL (( WRN_XSER_CANT_SET_SYMBOL_PATH ));
      }

      rc = sql49c_get_user_info ( &pszUserName, NULL );

      if ( rc != NO_ERROR )
        pszUserName = "";

      sql49c_user_privilege_check ( &fOk );
 
      if ( !fOk )
        {
        MSGALL (( ERR_XSER_USER_MISSING_PRIV, pszUserName ));
        rc = ERROR_ACCESS_DENIED;
        }

      if ( rc == NO_ERROR )
        {
        SA.nLength              = sizeof(SECURITY_ATTRIBUTES);
        SA.bInheritHandle       = FALSE;
        SA.lpSecurityDescriptor = NULL;

        memset ( &Access, 0, sizeof (Access) );  // - this must be done first!
        Access.fDescOwnerIsAdmin       = FALSE;
        Access.fAddDefaultDACL         = FALSE;
        Access.User.ulAccess           = 0;
        Access.Creator.ulAccess        = SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL;
        Access.Admin.ulAccess          = SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL;
        Access.System.ulAccess         = SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL;
        Access.SAPDBOp.ulAccess        = SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL;
        Access.DomSAPDBOp.ulAccess     = SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL;
        Access.World.ulAccess          = 0;
        Access.BackupOp.ulAccess       = GENERIC_READ;

        rc = sql49c_alloc_and_init_SD( &Access, &SA.lpSecurityDescriptor );
        }

      if ( rc == NO_ERROR )
        rc = os968_OpenDiagfile ( &SA, max( dwDiagFileSize, DIAGSIZE ) );

      if ( rc == NO_ERROR )
        rc = FREE_MEM(SA.lpSecurityDescriptor);

      if ( rc == NO_ERROR )
       rc = os968Coordinator ();
        

      gxr.dwState  = XSERVER_STOPPED;
      sql96c_NewSrvState( gxr.dwState, rc );
      }
  } 
  __except( os968ExceptionHandler(GetExceptionInformation()) ){;}

   DBGOUT;
   exit ( rc );
   return;
   }


/* PTS 1104558 */
/*------------------------------*/
static teo00_Bool os968_SetSymbolPath()
 {
  #undef  MF__
  #define MF__ MOD__"os968_SetSymbolPath"

  teo00_Bool              Ok = true;
  tsp01_RteError          RteError ;
  tsp00_Pathc             IndepProgramsPath;

  if ( !sqlGetIndependentProgramsPath(IndepProgramsPath, TERM_WITHOUT_DELIMITER_EO01, &RteError ) )
  {
    if ( GetModuleFileName(NULL, IndepProgramsPath, sizeof (IndepProgramsPath) ) )
    {
      // IndepProgramsPath should be ...\pgm\serv.exe
      // SymbolPath        should be ...\symbols
      // sql01c_set_symbol_path needs the path BEFORE symbol ( ...\) 

      char *Hptr = strrchr(IndepProgramsPath, '\\' ); 
      if ( Hptr )
      {
        *Hptr = '\0' ;
        Hptr = strrchr(IndepProgramsPath, '\\' ) ;
      }

      if ( Hptr )
        *Hptr = '\0' ;
      else
        Ok = false ;
    }
    else
    {
      Ok = false ;
    }
  }
 
  if ( Ok ) 
  {
    DBG1  (( MF__, "set SYMBOL environment path to %s", IndepProgramsPath ));
    // --- set SYMBOL environment variable
    sql01c_set_symbol_path ( IndepProgramsPath ) ;
  }
  return(Ok);
 }

/* PTS 1104558 */
/*------------------------------*/

 static LONG os968_OpenDiagfile( PSECURITY_ATTRIBUTES     pSA,
								                 DWORD                    dwDiagFileSize )
 {
  #undef  MF__
  #define MF__ MOD__"os968_OpenDiagfile"

  LONG                    rc = NO_ERROR;
  tsp00_Pathc             DiagnoseFile ;
  tsp01_RteError          RteError ;

  DiagnoseFile[0] = '\0' ;

  if ( !sqlGetIndependentServiceProtPath ( DiagnoseFile, TERM_WITH_DELIMITER_EO01, &RteError )) /* PTS 1108067 */
  {
    MSGALL (( WRN_XSER_NO_PATH_FOR_PROTFILE_FOUND, 
              gxr.ServerSpecification.ServerDiagfile));
  }

  strcat ( DiagnoseFile, gxr.ServerSpecification.ServerDiagfile );

  rc = sql60_open_diag_file ( DiagnoseFile, dwDiagFileSize, pSA);

  return ( rc ) ;

 }

 /*------------------------------*/

 static VOID WINAPI os968ServiceCtrlHandler ( DWORD dwCtrlCode )
   {
   #undef  MF__
   #define MF__ MOD__"os968ServiceCtrlHandler"
   LONG                    rc      = NO_ERROR;

   DBGIN;

   // Handle the requested control code.
   //
   switch(dwCtrlCode)
     {
     case SERVICE_CONTROL_STOP:         // --- Stop the service.
     case SERVICE_CONTROL_SHUTDOWN:
         gxr.ssStatus.dwCurrentState     = SERVICE_STOP_PENDING;
         gxr.ssStatus.dwWin32ExitCode    = NO_ERROR;
         gxr.ssStatus.dwWaitHint         = 30000;
         gxr.ssStatus.dwControlsAccepted = 0;
         gxr.ssStatus.dwCheckPoint       = 0;

         // Report the status of the service to the service control manager.
         if (!SetServiceStatus ( gxr.sshStatusHandle, &gxr.ssStatus ))
           {
           rc = GetLastError();
           MSGALL (( ERR_SET_SERVICE_STATUS, rc ));
           }

         os969StopAnyServer ( gxr.ServerSpecification.ServerName, gxr.ServerSpecification.ServerSemaName );

         return;

     case SERVICE_CONTROL_PAUSE:
         gxr.fServicePaused              = TRUE;
         gxr.ssStatus.dwCurrentState     = SERVICE_PAUSED;
         gxr.ssStatus.dwWin32ExitCode    = NO_ERROR;
         gxr.ssStatus.dwCheckPoint       = 0;
         gxr.ssStatus.dwWaitHint         = 0;
         break;

     case SERVICE_CONTROL_CONTINUE:
         gxr.fServicePaused              = FALSE;
         gxr.ssStatus.dwCurrentState     = SERVICE_RUNNING;
         gxr.ssStatus.dwWin32ExitCode    = NO_ERROR;
         gxr.ssStatus.dwCheckPoint       = 0;
         gxr.ssStatus.dwWaitHint         = 0;
         break;

     case SERVICE_CONTROL_INTERROGATE:  // --- Update the service status.
     default:
         break;
     }

   // Report the status of the service to the service control manager.
   if (!SetServiceStatus ( gxr.sshStatusHandle, &gxr.ssStatus ))
     {
     rc = GetLastError();
     MSGALL (( ERR_SET_SERVICE_STATUS, rc ));
     }

   DBGOUT;
   return;
   }

/*------------------------------*/

static INT os968Server ( VOID )
  {
  #undef  MF__
  #define MF__ MOD__"os968Server"
  LONG                    rc          = NO_ERROR;
  HEV                     hevSemXserv = 0;
  CHAR                    szOldTitle[100];
  CHAR                    szTitle[sizeof(szOldTitle)];
  CHAR                    szNewTitle[sizeof(szOldTitle)];
  ULONG                   ulTimeOut   = 1000;
  HWND                    hwndConsole = NULL;

  DBGIN;
  wsprintf( szNewTitle,"%s (%xh)", gxr.ServerSpecification.ServerName,
            GetCurrentProcessId() );

  GetConsoleTitle ( szOldTitle, sizeof(szOldTitle));
  SetConsoleTitle( szNewTitle );

  do
    {
    GetConsoleTitle ( szTitle, sizeof(szTitle));
    SLEEP(10);
    }
  while ( strcmp (szTitle, szNewTitle) );

  hwndConsole = FindWindow(NULL, szNewTitle);

  if ( hwndConsole )
    {
    if (( sql02_get_platform_id() != VER_PLATFORM_WIN32_NT ) &&
        ( !gxr.usDebugLevel ))
      {
      ShowWindow(hwndConsole, SW_HIDE);
      SendMessage(hwndConsole, WM_SYSCOMMAND, SC_MINIMIZE | HTCLIENT, 0L);
      }
    sql96c_tray_message(hwndConsole, NIM_ADD);
    }

  SetConsoleTitle( szOldTitle );

  rc = sql41c_open_event_sem ( &hevSemXserv, gxr.ServerSpecification.ServerSemaName, "", NO_ERROR);

  if (rc == NO_ERROR)
    {
    sql41c_close_event_sem ( hevSemXserv, gxr.ServerSpecification.ServerName );

    #if defined(_WIN32)
      if ( hwndConsole )
        sql96c_tray_message(hwndConsole, NIM_DELETE);
    #endif

    MSGCD (( ERR_XSER_ALREADY_RUNNING ));

    DBGOUT;
    return( 0 );
    }

  rc = os968_OpenDiagfile ( NULL, DIAGSIZE * (gxr.usDebugLevel + 1) );

  if ( rc != NO_ERROR )
    {
    if ( hwndConsole )
      sql96c_tray_message(hwndConsole, NIM_DELETE);

    DBGOUT;
    return ( rc );
    }

   // --- now running as server process, change the caption bar...
   SetConsoleTitle (gxr.ServerSpecification.ServerName);

   if (!SetConsoleCtrlHandler( os968ConsoleCtrlHandler, TRUE ))
     {
     rc = GetLastError ( );
     if ( hwndConsole )
       sql96c_tray_message(hwndConsole, NIM_DELETE);

     DBGOUT;
     return ( rc );
     }
  rc = os968Coordinator ();

   if ( hwndConsole )
     sql96c_tray_message(hwndConsole, NIM_DELETE);

  DBGOUT;
  return( rc );
  }

/*------------------------------*/

STATIC INT os968Coordinator ( VOID )
  {
  #undef  MF__
  #define MF__ MOD__"os968Coordinator"
  LONG                    rc        = NO_ERROR;
  ULONG                   ulResult;
  #if defined(_WIN32)
   ACCESS_RIGHTS_REC      Access;
   PSZ                    pszUserName;
  #endif

  DBGIN;

  GETPROCESSID(&gxr.pid);

  gxr.hevSemXserv = INVALID_HANDLE_VALUE;

   rc = sql49c_get_user_info ( &pszUserName, NULL );

   if ( rc != NO_ERROR )
     pszUserName = "";

   rc = sql49c_set_sync_access ();

   if ( rc != NO_ERROR )
     {
     DBGOUT;
     return( rc );
     }

   gxr.WorldSA.nLength         = sizeof(gxr.WorldSA);
   gxr.WorldSA.bInheritHandle  = FALSE;

   memset ( &Access, 0, sizeof (Access) );   // - this must be done first!
   Access.World.ulAccess       = SPECIFIC_RIGHTS_ALL |
                                 STANDARD_RIGHTS_ALL;

   rc = sql49c_alloc_and_init_SD( &Access, &gxr.WorldSA.lpSecurityDescriptor );

   if ( rc != NO_ERROR )
     {
     DBGOUT;
     return( rc );
     }

   gxr.LockSA.nLength         = sizeof(gxr.LockSA);
   gxr.LockSA.bInheritHandle  = FALSE;

   memset ( &Access, 0, sizeof (Access) );  // - this must be done first!
   rc = sql49c_alloc_and_init_SD( &Access, &gxr.LockSA.lpSecurityDescriptor );

   if ( rc != NO_ERROR )
     {
     DBGOUT;
     return( rc );
     }

   gxr.AdminSA.nLength         = sizeof(gxr.AdminSA);
   gxr.AdminSA.bInheritHandle  = FALSE;

   memset ( &Access, 0, sizeof (Access) );  // - this must be done first!
   Access.Creator.ulAccess        = SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL;
   Access.Admin.ulAccess          = SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL;
   Access.System.ulAccess         = SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL;
   Access.SAPDBOp.ulAccess        = SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL;
   Access.DomSAPDBOp.ulAccess     = SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL;

   rc = sql49c_alloc_and_init_SD( &Access, &gxr.AdminSA.lpSecurityDescriptor );

   if ( rc != NO_ERROR )
     {
     DBGOUT;
     return( rc );
     }

  rc = sql41c_create_event_sem ( &gxr.hevSemXserv, gxr.ServerSpecification.ServerSemaName, "",
                                 0, FALSE, &gxr.AdminSA );

  if (rc == ERROR_DUPLICATE_NAME)
    {
    if ( gxr.fSuppressXserverActiveMsg == FALSE )
      MSGALL(( ERR_XSER_ALREADY_RUNNING ));

    DBGOUT;
    return( 0 );
    }
  else if (rc != NO_ERROR)
    {
    DBGOUT;
    return( rc );
    }

  __try
  {
    os968UpdateVersion();

    hModule = GetModuleHandle(NULL);   // - used in 'vos40lc'
	ulResult = sql43_tcpip_init ();

    if ( ulResult != TCPIP_INIT_OK )
      {
      if ( ulResult == TCPIP_INIT_WRONG_VERSION )
        MSGALL (( ERR_WRONG_TCPIP_VERSION ))
      else
        MSGALL (( ERR_XSER_CANNOT_INIT_TCPIP ))

      DBGOUT;
      return ( 1 );
      }

    // --- Write a message to the event log (Version included)
    //     and start the diagnostic file write cycle
    MSGALL (( INFO_XSER_STARTED_VERSION, gxr.RTEVersionString ));

    os968_ShowEnviron(_environ);

    sql60_strt_msg_wrt_cycle ( );


    rc = os96StartTCPIPCoord ( &gxr.TCPIPCoord ) ;

    if ( rc == NO_ERROR )
      rc = os962StartNICoord ( &gxr.NICoord ) ;

    if ( 1 == os968_SupportOldConnections )
    {
        if ( rc == NO_ERROR )
          {
          gxr.OldTCPIPCoord.ThrdCtrlHeader.Tid         = (TID)UNDEF;
          gxr.OldTCPIPCoord.ThrdCtrlHeader.hThrd       = (HANDLE)INVALID_HANDLE_VALUE;
          gxr.OldTCPIPCoord.ThrdCtrlHeader.lThrdPrio   = SERVER_LOOP_THRD_PRIO;
          gxr.OldTCPIPCoord.ThrdCtrlHeader.ThrdState   = THRD_INITIALIZED;
          gxr.OldTCPIPCoord.ThrdCtrlHeader.ulStartTime = (ULONG)UNDEF;

          rc = CREATE_THREAD( &gxr.OldTCPIPCoord.ThrdCtrlHeader.Tid,
                              &gxr.OldTCPIPCoord.ThrdCtrlHeader.hThrd,
                              sql96o_TCPIP_coordinator,
                              0,
                              CREATE_THRD_SUSPENDED,
                              TCPIP_COORD_STACK_SIZE );

          if ( rc == NO_ERROR )
            {
            rc = RESUME_THREAD( gxr.OldTCPIPCoord.ThrdCtrlHeader.Tid,
                                gxr.OldTCPIPCoord.ThrdCtrlHeader.hThrd );

            if ( rc != NO_ERROR )
              MSGALL (( ERR_XSER_RESUMING_THREAD, "OLDCOORDINATOR", rc));
            }
          else
            MSGALL (( ERR_XSER_CREATING_THREAD, "OLDCOORDINATOR", rc));
        }
     }

     if (( gxr.fNTService ) && ( rc == NO_ERROR ))
       {
       gxr.dwState = XSERVER_RUNNING;
       sql96c_NewSrvState( gxr.dwState, NO_ERROR );
       }

    if ( rc == NO_ERROR )
      rc = sql41c_wait_event_sem ( gxr.hevSemXserv, (ULONG)UNDEF, gxr.ServerSpecification.ServerName );
    }
  __finally
     {
     os968Finish(0);
     }

    if ( gxr.fNTService )
      {
      gxr.dwState = XSERVER_STOPPED;
      sql96c_NewSrvState( gxr.dwState, NO_ERROR );
      }

  DBGOUT;
  return ( rc );
  }

/*------------------------------*/


STATIC BOOL WINAPI os968ConsoleCtrlHandler ( DWORD dwCtrlType )
  {
  #undef  MF__
  #define MF__ MOD__"os968ConsoleCtrlHandler"

  os969StopAnyServer ( gxr.ServerSpecification.ServerName, gxr.ServerSpecification.ServerSemaName );
  SLEEP ( 2000 );

  DBGPAS;
  exit(0);
  return (FALSE);
  }

/*------------------------------*/

STATIC VOID _System os968Finish (ULONG  termcode)
  {
  #undef  MF__
  #define MF__ MOD__"os968Finish"

  DBGPAS;

  if ( gxr.hevSemXserv != INVALID_HANDLE_VALUE )
    {
    sql41c_close_event_sem ( gxr.hevSemXserv, gxr.ServerSpecification.ServerName );
    gxr.hevSemXserv = INVALID_HANDLE_VALUE;
    }

  if ( gxr.NICoord.ThrdCtrlHeader.Tid != UNDEF )
    os962NIFinish ();
  sql96o_finish ();

  os960Finish (&gxr.TCPIPCoord);

  MSGALL (( INFO_XSER_STOPPED_VERSION, gxr.RTEVersionString ));
  sql60_close_diag_file ();

  return;
  }


/*------------------------------*/

static BOOL os968DebuggerIsDrWatson( VOID )
  {
  #undef  MF__
  #define MF__ MOD__"os968DebuggerIsDrWatson"
  LONG             rc = NO_ERROR;
  PATHNAME         szDebuggerEnty;
  REG_ENTRY_REC    RegistryEntries[1];

  DBGIN;

  if ( sql02_get_platform_id () == VER_PLATFORM_WIN32_NT )
    {
    RegistryEntries[0].pszValueName = REG_VN_AEDEBUG;
    RegistryEntries[0].pValue       = szDebuggerEnty;
    RegistryEntries[0].ulValueSize  = sizeof(szDebuggerEnty);
    RegistryEntries[0].ulValueType  = REG_SZ;

    rc = sql50_reg_get_values ( NULL, HKEY_LOCAL_MACHINE, REG_SK_AEDEBUG, 1, RegistryEntries );

    if (rc != NO_ERROR)
      {
      DBGOUT;
      return( FALSE );
      }

    strupr ( szDebuggerEnty );

    if ( strstr ( szDebuggerEnty, DR_WATSON_EXENAME ) != NULL )
      {
      DBGOUT;
      return( TRUE );
      }
    }

  DBGOUT;
  return( FALSE );
  }

/*------------------------------*/

static void os968UpdateVersion( VOID )
  {
  #undef  MF__
  #define MF__ MOD__"os968UpdateVersion"
  REG_ENTRY_REC           RegistryEntries[2];
  tsp100_VersionID0       RTEVersionID;
  tsp00_Versionc          RTEVersionString;
  PATHNAME                szKey;

  DBGIN;

  if ( sql02_get_platform_id () == VER_PLATFORM_WIN32_NT )
    {
    // ---- insert version string entry into registry database
    if ( sql02_get_platform_id() == VER_PLATFORM_WIN32_NT )
      sp100_GetVersionString(COMP_NAME_RTE_NT_SP100, s100buildnumber, RTEVersionString);
    else
      sp100_GetVersionString(COMP_NAME_RTE_WIN9X_SP100, s100buildnumber, RTEVersionString );
    
    sp100_GetVersionID ( VersionIDType0_esp100, s100buildnumberEx, &RTEVersionID );
    
    RegistryEntries[0].pszValueName = REG_VN_VERSION;
    RegistryEntries[0].pValue       = RTEVersionString;
    RegistryEntries[0].ulValueSize  = (ULONG)strlen(RTEVersionString);
    RegistryEntries[0].ulValueType  = REG_SZ;
    
    RegistryEntries[1].pszValueName = REG_VN_VERSION_ID;
    RegistryEntries[1].pValue       = &RTEVersionID;
    RegistryEntries[1].ulValueSize  = sizeof(RTEVersionID);
    RegistryEntries[1].ulValueType  = REG_BINARY;


    strcpy ( szKey, SERVICE_ID_XSERVER );
    strcat ( szKey, "\\"REG_SK_SERVICE_PARAM );
    
    sql50_reg_put_service_values ( NULL, szKey, 2, RegistryEntries );
    }

  DBGOUT;
  return;
  }

/*-------------------------------------------------*/

static void os968_ShowEnviron(char **envp)
{
    char **pEnv;
    MSGD(( INFO_ENVIRONMENT_DUMP_START ));
    for ( pEnv = envp; pEnv && *pEnv; ++pEnv )
    {
#define MAX_ENV_PART 80
	    char  EnvBuffer[MAX_ENV_PART+2];
	    char  *pEnvPart;
        int   len;

	    for ( len  = (int)strlen(*pEnv), pEnvPart = *pEnv;
              len  > MAX_ENV_PART;
              len -= MAX_ENV_PART,       pEnvPart += MAX_ENV_PART )
	    {
            memset(EnvBuffer, 0, MAX_ENV_PART+2);
	        if ( pEnvPart == *pEnv )
            {
   	            memcpy(EnvBuffer, pEnvPart, MAX_ENV_PART);
            }
            else
            {
                EnvBuffer[0] = ' '; /* add a space for each followup line */
       	        memcpy(&EnvBuffer[1], pEnvPart, MAX_ENV_PART);
            }
            MSGD(( INFO_ENVIRONMENT_VARIABLE, EnvBuffer ))
	    }

        if ( pEnvPart == *pEnv )
        {
            MSGD(( INFO_ENVIRONMENT_VARIABLE, pEnvPart ))
        }
        else
        {
            EnvBuffer[0] = ' '; /* add a space for each followup line */
       	    memcpy(&EnvBuffer[1], pEnvPart, len+1);
            MSGD(( INFO_ENVIRONMENT_VARIABLE, EnvBuffer ))
        }
    }
    MSGD(( INFO_ENVIRONMENT_DUMP_COMPLETED ));
}

/*===========================================================================*
 *  END OF CODE                                                              *
 *===========================================================================*/
