/*!================================================================

  module:       vpr09Config.h

 -------------------------------------------------------------------

  responsible:  BurkhardD

  special area: Handling Configuration (ini) file entrys

  description:                 

  see also:

  -------------------------------------------------------------------------

  copyright:    Copyright by SAP AG, 1998



    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 ===================================================================*/

/*==================================================================*
 *  INCLUDES                                                        *
 *==================================================================*/

#include <stdio.h>
#include "vpr09Config.h"

/*==================================================================*
 *  DECLARATIONS                                                    *
 *==================================================================*/

#define err_create_key_pr09Config    "Can't create registy key"
#define err_delete_key_pr09Config    "Can't delete registy key"
#define err_open_key_pr09Config      "Can't open registy key"
#define err_read_key_pr09Config      "Can't read registy key"
#define err_read_value_pr09Config    "Can't read registy value"
#define err_close_key_pr09Config     "Can't close registy key"
#define err_create_entry_pr09Config  "Can't create registry value"
#define err_delete_entry_pr09Config  "Can't delete registry value"

static char *pr09ConfigKey = KEY_SAP;

/*==================================================================*
 *  CODE                                                            *
 *==================================================================*/

char *pr09SetDefaultKey(char *szKey)
{
  char *p = pr09ConfigKey;
  pr09ConfigKey = szKey;
  return p;
}

ConfigResult_tpr09  pr09ConfigPutString (const char *szFile, const char *szSection,
		     const char *szEntry, const char *szString,
		     tsp00_ErrText ErrText)
{
#ifndef WIN32  
  unsigned char Ok;
  if (sqlPutConfigString (szFile, szSection, szEntry, szString, ErrText, &Ok))
    return ERR_OK_epr09;
  return ERR_NOT_OK_epr09;
#else
  HKEY hkResult;
  HKEY hkResult1;
  LONG ret = ERR_NOT_OK_epr09;
  char szRegKey[MAX_PATH];
  sprintf(szRegKey, "%s\\%s\\%s", pr09ConfigKey, szFile, szSection);
  if (RegCreateKey( HKEY_LOCAL_MACHINE, szRegKey, &hkResult ) == ERROR_SUCCESS ) {
    if (RegCreateKey( hkResult, szEntry, &hkResult1 ) == ERROR_SUCCESS ) {
      if (RegSetValueEx( hkResult1, "", 0, REG_SZ, 
			 (const char *)szString, (int)strlen(szString)) == ERROR_SUCCESS ) {
	ret = ERR_OK_epr09;
      }
      else {
	strcpy(ErrText, err_create_entry_pr09Config);
      }
      RegCloseKey(hkResult1);
    }
    else {
      strcpy(ErrText, err_create_key_pr09Config);
    }
    RegCloseKey(hkResult);
  }  
  else {
    strcpy(ErrText, err_create_key_pr09Config);
  }
  return ret;
#endif  
}

ConfigResult_tpr09 pr09ConfigPutStringEx (const char *szFile, const char *szSection,
					  const char *szEntry, const char *szString,
					  tsp00_ErrText ErrText)
{
#ifndef WIN32  
  unsigned char Ok;
  if (sqlPutConfigString (szFile, szSection, szEntry, szString, ErrText, &Ok))
    return ERR_OK_epr09;
  return ERR_NOT_OK_epr09;
#else
  HKEY hkResult;
  LONG ret = ERR_NOT_OK_epr09;
  char szRegKey[MAX_PATH];
  sprintf(szRegKey, "%s\\%s\\%s", pr09ConfigKey, szFile, szSection);
  if (RegCreateKey( HKEY_LOCAL_MACHINE, szRegKey, &hkResult ) == ERROR_SUCCESS ) {
    if (RegSetValueEx( hkResult, szEntry, 0, REG_SZ, 
		       (const char *)szString, (int)strlen(szString)) == ERROR_SUCCESS ) {
      ret = ERR_OK_epr09;
    }
    else {
      strcpy(ErrText, err_create_entry_pr09Config);
    }
    RegCloseKey(hkResult);
  }  
  else {
    strcpy(ErrText, err_create_key_pr09Config);
  }
  return ret;
#endif  
}

ConfigHandle_tpr09 pr09ConfigOpenSection (const char *szFile, const char *szSection,
    tsp00_ErrText ErrText)
{
#ifndef WIN32  
  unsigned char Ok;
  ConfigHandle_tpr09 hConfig = sqlOpenConfigEnum (szFile, szSection, ErrText, &Ok);
  if (Ok == config_error_ok_chen09)
    return hConfig;
  else
    return (ConfigHandle_tpr09)NULL;
#else
  ConfigHandle_tpr09 hConfig;
  char szRegKey[MAX_PATH];
  hConfig = malloc(sizeof(ConfigHandleStruct_tpr09));
  hConfig->Index = 0;
  sprintf(szRegKey, "%s\\%s\\%s", pr09ConfigKey, szFile, szSection);
  if ( RegOpenKeyEx(HKEY_LOCAL_MACHINE, szRegKey, 0, 
		    KEY_READ | KEY_ENUMERATE_SUB_KEYS, &hConfig->hkResult) == ERROR_SUCCESS ) {
    return hConfig;
  }
  strcpy(ErrText, err_open_key_pr09Config);        
  return NULL;
#endif
}

ConfigResult_tpr09 pr09ConfigCloseSection (ConfigHandle_tpr09 hConfig, tsp00_ErrText ErrText)
{
#ifndef WIN32  
  unsigned char Ok;
  if (sqlCloseConfigEnum(hConfig, ErrText, &Ok))
    return ERR_OK_epr09;
  return ERR_NOT_OK_epr09;
#else
  LONG ret = RegCloseKey(hConfig->hkResult);
  free(hConfig);
  if (ret == ERROR_SUCCESS) {
    return TRUE;
  }
  else {
    strcpy(ErrText, err_close_key_pr09Config);        
    return FALSE;
  }
#endif
}

ConfigResult_tpr09 pr09ConfigNextEntry (
    ConfigHandle_tpr09 hConfig,
    char *szEntry,
    const int MaxEntryLen,
    char *szString,
    const int MaxStringLen,
    tsp00_ErrText ErrText)
{
#ifndef WIN32

  ConfigResult_tpr09 ret = ERR_OK_epr09;
  unsigned char Ok;
  int rc = sqlNextConfig(hConfig, szEntry, MaxEntryLen, szString, MaxStringLen, ErrText, &Ok);
  if (!rc) {
    if (Ok == config_error_end_chen09) 
      ret = ERR_NO_MORE_DATA_epr09;
    else
      ret = ERR_NOT_OK_epr09;      
  }
  return ret;
#else
  LONG rc;
  HKEY hkSub;
  DWORD cbType;
  rc = RegEnumKey(hConfig->hkResult, hConfig->Index, szEntry, MaxEntryLen);
  if (rc == ERROR_SUCCESS) {
    rc = RegOpenKeyEx(hConfig->hkResult, szEntry, 0, KEY_READ, &hkSub);
    if ( rc == ERROR_SUCCESS ) {
      rc = RegQueryValueEx( hkSub, "", 0, &cbType, (char *)szString, (LPDWORD)&MaxStringLen );  
      if (rc == ERROR_SUCCESS
	  || rc == ERROR_NO_MORE_ITEMS ) {
	RegCloseKey(hkSub);
	if (rc == ERROR_SUCCESS) {
	  hConfig->Index++;
	  return ERR_OK_epr09;
	}
	else {
	  return ERR_NO_MORE_DATA_epr09;      
	}
      }
      else {
	strcpy(ErrText, err_read_value_pr09Config);
	return ERR_NOT_OK_epr09;
      }
    }
  }
  if (rc == ERROR_NO_MORE_ITEMS) {
    return ERR_NO_MORE_DATA_epr09;      
  }
  strcpy(ErrText, err_read_key_pr09Config);
  return ERR_NOT_OK_epr09;
#endif
}

ConfigResult_tpr09 pr09ConfigGetString (const char * szFile, 
		     const char * szSection,
		     const char * szEntry,
		     char       * szString,
		     const int    MaxStringLen,
		     tsp00_ErrText ErrText)
{
#ifndef WIN32
  unsigned char Ok;
  if (sqlGetConfigString(szFile, szSection, szEntry, szString, MaxStringLen,
			 ErrText, &Ok))    
    return ERR_OK_epr09;
  return ERR_NOT_OK_epr09;
#else
  char szRegKey[MAX_PATH];
  HKEY hkResult, hkSub;
  LONG rc;
  DWORD cbType;
  
  sprintf(szRegKey, "%s\\%s\\%s", pr09ConfigKey, szFile, szSection);
  rc = RegOpenKeyEx(HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_WRITE | KEY_READ, &hkResult);
  if ( rc == ERROR_SUCCESS ) {
    rc = RegOpenKeyEx(hkResult, szEntry, 0, KEY_READ, &hkSub);
    if (rc == ERROR_SUCCESS) {
      rc = RegQueryValueEx( hkSub, "", 0, &cbType, (char *)szString, (LPDWORD)&MaxStringLen );  
      RegCloseKey(hkSub);
    }
    RegCloseKey(hkResult);
  }
  if (rc != ERROR_SUCCESS) {
    strcpy(ErrText, err_read_key_pr09Config);
    return ERR_NOT_OK_epr09;
  }
  return ERR_OK_epr09;
#endif
}

ConfigResult_tpr09 pr09ConfigGetStringEx (const char * szFile, 
					  const char * szSection,
					  const char * szEntry,
					  char       * szString,
					  const int    MaxStringLen,
					  tsp00_ErrText ErrText)
{
#ifndef WIN32
  unsigned char Ok;
  if (sqlGetConfigString(szFile, szSection, szEntry, szString, MaxStringLen,
			 ErrText, &Ok))    
    return ERR_OK_epr09;
  return ERR_NOT_OK_epr09;
#else
  char szRegKey[MAX_PATH];
  HKEY hkResult;
  LONG rc;
  DWORD cbType;
  
  sprintf(szRegKey, "%s\\%s\\%s", pr09ConfigKey, szFile, szSection);
  rc = RegOpenKeyEx(HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_WRITE | KEY_READ, &hkResult);
  if ( rc == ERROR_SUCCESS ) {
    rc = RegQueryValueEx( hkResult, szEntry, 0, &cbType, (char *)szString, (LPDWORD)&MaxStringLen );  
  }
  RegCloseKey(hkResult);
  if (rc != ERROR_SUCCESS) {
    strcpy(ErrText, err_read_key_pr09Config);
    return ERR_NOT_OK_epr09;
  }
  return ERR_OK_epr09;
#endif
}

ConfigResult_tpr09 pr09ConfigRemoveEntry(
    const char * szFile,
    const char * szSection,
    const char * szEntry,
    tsp00_ErrText ErrText)
{
#ifndef WIN32
  unsigned char Ok;
  if (sqlRemoveConfigString (szFile, szSection, szEntry, ErrText, &Ok))
    return ERR_OK_epr09;
  return ERR_NOT_OK_epr09;
#else
  char szRegKey[MAX_PATH];
  HKEY hkResult;
  LONG rc;
  
  sprintf(szRegKey, "%s\\%s\\%s", pr09ConfigKey, szFile, szSection);
  rc = RegOpenKeyEx(HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_WRITE | KEY_READ, &hkResult);
  if ( rc == ERROR_SUCCESS ) {
    rc = RegDeleteKey(hkResult, szEntry);  	      
    RegCloseKey(hkResult);
  }
  if (rc != ERROR_SUCCESS) {
    strcpy(ErrText, err_delete_key_pr09Config);
    return ERR_NOT_OK_epr09;
  }
  return ERR_OK_epr09;
#endif  
}
