/*                                                             
      lcstream.h - definitions/declarations for standard processing
      of liveCache IO streams. This file extended the general 
      declaration from cpc.h of the ADABAS for R/3 C/C++ Precompiler

       Copyright (c) 1998, SAP AG. All rights reserved.


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

*/
#ifndef __gpi00lcstream_h__
#define __gpi00lcstream_h__

#include <stdlib.h>
#include <livecachetypes.h>

/* returncodes for callbackfunktions */
#define SQL_STREAM_OK 0
#define SQL_STREAM_ERROR (-1)
#define SQL_STREAM_EXCEPTION (-2)
#define SQL_STREAM_NO_MORE_DATA (100)

/* Appends the oracle hostvartypes. */
#define SQL_ORA_STREAM_HANDLE 301

/*
  Applications should bind this value to ORA_T and set ORA_L
  to the number of bytes passing with  SQL_LC_StreamParm. 
  ORA_V should point to an hostvarible of the SQLStreamDesc type.
*/

typedef struct _ABAPTab {
  void* nilPointer;
  OmsTypeABAPTabHandle hABAPTab;
} ABAPTabStruct;

typedef struct _Stream {
  void* nilPointer;
  OmsTypeStreamHandle hStream;
} StreamStruct;

typedef struct _SQL_LC_StreamParm {
  union
  {
    ABAPTabStruct ABAPTab;
    StreamStruct Stream;
  }
#ifndef __cplusplus
  C_1
#endif
;
} SQL_LC_StreamParm;

/* callbackfunctions for handling stream IO */
typedef int (SQLStreamReadProc)(SQL_LC_StreamParm *ABAPTabParm, void *rgbOutStream, int cbStreamMax, int *pcbStreamLen, int *pcbNoOfRowsRead);

typedef int (SQLStreamWriteProc)(SQL_LC_StreamParm *ABAPTabParm, void *rgbInStream, int cbNoOfRows);

typedef struct _SQLStreamDesc {
  SQLStreamReadProc  *ReadProc;
  SQLStreamWriteProc *WriteProc;
  SQL_LC_StreamParm  *StreamParam;   
} SQLStreamDesc;


#ifdef __cplusplus
template <class T, long MCOUNT> class CStreamDescriptor : public SQLStreamDesc {
private :
    int currMember;
    void describeMember (OmsStreamTypes memberType, short length) {
        if (currMember < MCOUNT) {
            StreamParam->Stream.hStream.colDesc[currMember].memberType = memberType;
            StreamParam->Stream.hStream.colDesc[currMember].length     = length;
            if (0 == currMember) {
              StreamParam->Stream.hStream.colDesc[currMember].offset = 0;
            }
            else {
              StreamParam->Stream.hStream.colDesc[currMember].offset =
                StreamParam->Stream.hStream.colDesc[currMember-1].offset +
                StreamParam->Stream.hStream.colDesc[currMember-1].length;
            }
            currMember++;
        }
    }
public :
    CStreamDescriptor(long streamId) {
        ReadProc    = 0;
        WriteProc   = 0;
        currMember  = 0;
	if (MCOUNT == 0) {
	  /* allocate the maximum if the membercount is not set */
	  StreamParam = (SQL_LC_StreamParm*) malloc(sizeof(SQL_LC_StreamParm) - sizeof (OmsTypeStreamMemberDesc) * 256 + sizeof (OmsTypeStreamMemberDesc) * 256);
	}
	else {
	  StreamParam = (SQL_LC_StreamParm*) malloc(sizeof(SQL_LC_StreamParm) - sizeof (OmsTypeStreamMemberDesc) * 256 + sizeof (OmsTypeStreamMemberDesc) * MCOUNT);
	}
        if (StreamParam) {
            StreamParam->Stream.nilPointer          = (void*) 0;
&if $BIT64
&else
            StreamParam->Stream.hStream.reserved    = 0;
&endif
            StreamParam->Stream.hStream.ABAPTabId   = streamId;
            StreamParam->Stream.hStream.size        = sizeof(T);
            StreamParam->Stream.hStream.memberCount = MCOUNT;
        }
    }


    void Describe(short x)                   { describeMember (STYPE_I2,  sizeof(x)); }
    void Describe(unsigned short x)          { describeMember (STYPE_UI2, sizeof(x)); }
    void Describe(long x)                    { describeMember (STYPE_I4,  sizeof(x)); }
    void Describe(unsigned long x)           { describeMember (STYPE_UI4, sizeof(x)); }
    void Describe(float x)                   { describeMember (STYPE_R4,  sizeof(x)); }
    void Describe(double x)                  { describeMember (STYPE_R8,  sizeof(x)); }
    void Describe(const char* x, short size) { describeMember (STYPE_CHAR, size); }

protected :
    ~CStreamDescriptor() {
        if (StreamParam)
            free (StreamParam);
    }
};

template <class T, long MCOUNT> class COStreamDescriptor : public CStreamDescriptor<T,MCOUNT> {
public :
    COStreamDescriptor(long streamId, SQLStreamWriteProc pWriteProc) : 
        CStreamDescriptor<T,MCOUNT>(streamId) 
    {
        CStreamDescriptor<T,MCOUNT>::WriteProc = pWriteProc;
    }
};

template <class T, long MCOUNT> class CIStreamDescriptor : public CStreamDescriptor<T,MCOUNT> {
public :
    CIStreamDescriptor(long streamId, SQLStreamReadProc pReadProc) : 
        CStreamDescriptor<T,MCOUNT>(streamId) 
    {
        CStreamDescriptor<T,MCOUNT>::ReadProc = pReadProc;
    }
};

#else
#endif
#endif

